package uk.ac.sanger.mig.abr;

import java.security.NoSuchAlgorithmException;
import ChartDirector.ArrayMath;
import ChartDirector.LineLayer;
import ChartDirector.XYChart;

/**
 * This class generate the XYChart for the frequency diagrams
 * dataX is the x values for all traces
 * dataY are the y values for all traces
 *
 * addLegend is used to determine whether a legend should be added on
 * levels are used for generate the legend
 * @author wl1
 *
 */
public class FrequencyChart {

	private double[] dataX;
	private double[][] dataYArray;
	private String frequencyTitle;
	private double xInterval;
	private boolean addLegend;
	private int[] levels;
	public static final int CHART_SIZE_X = 230;
	public static final int CHART_SIZE_Y = 550;
	public static final int PLOT_SIZE_X = 210;
	public static final int PLOT_SIZE_Y = 480;
	public static final int CHART_SIZE_X_ENLARGED = 430;
	public static final int CHART_SIZE_Y_ENLARGED = 670;
	public static final int PLOT_SIZE_X_ENLARGED = 380;
	public static final int PLOT_SIZE_Y_ENLARGED = 600;


	/**
	 * @param dataX is the x-axis values of all datapoints
	 * @param dataYArray the y-axis values for all waveform
	 * @param frequencyTitle the title of the chart
	 * @param addLegend true if its the last chart of the multichart
	 * @param levels  if addLegend, the labels of levels otherwise unempty array
	 */
	public FrequencyChart(double[] dataX, double[][] dataYArray, String frequencyTitle, boolean addLegend, int[] levels,double xInterval) {
		this.dataX = dataX;
		this.dataYArray = dataYArray;
		this.frequencyTitle = frequencyTitle;
		this.addLegend = addLegend;
		this.levels = levels;
		this.xInterval =  xInterval;
	}

	public XYChart createDefaultSingleChartForMultiple() throws NoSuchAlgorithmException{

		int defaultLength = (int)(MouseChart.DEFAULT_X_GRID/this.xInterval+1);
		double[] defaultDataX = new double[defaultLength];
		double[] defaultDataY = new double[defaultLength];
		System.arraycopy(this.dataX, 0, defaultDataX, 0,defaultLength);

		int chartT=CHART_SIZE_X;
		if(addLegend){
			chartT = CHART_SIZE_X+30;
		}

		// set drawing area, white background, grey axis lines.
		XYChart c = new XYChart(chartT,CHART_SIZE_Y);
		c.setPlotArea(5,20,PLOT_SIZE_X,PLOT_SIZE_Y, 0xffffff, -1, 0xbdbdbe, -1, 0xc0c0c0);
		c.addTitle(frequencyTitle);
		c.xAxis().setTitle("ms", "Arial", 8);

		c.xAxis().setWidth(1);
		c.yAxis().setWidth(1);
		c.xAxis().setColors(0x000000,0x9e0000,0X000000,0x9e0000);//x label color red, title color black
		c.yAxis().setColors(0x000000,0x9e0000,0X000000,0x9e0000);

		c.yAxis().setLabelFormat("");
		c.xAxis().setLinearScale(0,MouseChart.DEFAULT_X_GRID,2);
		c.yAxis().setAutoScale(0, 0, 0);

		LineLayer layer1 = c.addLineLayer();
		layer1.setLineWidth(1);

		layer1.setXData(defaultDataX);

		//add waveforms
		for(int l=0; l<dataYArray.length;l++){
			String s = ColourGenerator.hexColourCode("Splv"+l);
			System.arraycopy(dataYArray[l], 0, defaultDataY, 0,defaultLength);
			layer1.addDataSet(defaultDataY,Integer.parseInt(s, 16),Integer.toString(l));
		}

		//add legend and text label for the waveforms
		c.layout();
		if(addLegend){
			for(int i=0;i<dataYArray.length;i++){
				int y = c.getYCoor(new ArrayMath(dataYArray[i]).avg());
				int x = c.getXCoor(dataX[defaultLength-1]);
				String s = ColourGenerator.hexColourCode("Splv"+i);
				c.addLine(x+5,y,x+25,y,Integer.parseInt(s, 16),2);
				c.addText(x+25, y-5, Integer.toString(levels[i]),"",8,Integer.parseInt(s, 16));
			}
		}

		return c;
	}

	/**
	 * @return the XYChart of the enlarged freqency diagram
	 * @throws NoSuchAlgorithmException
	 */
	public XYChart createEnlargedChart() throws NoSuchAlgorithmException{

		// set drawing area, white background, grey axis lines.
		XYChart c = new XYChart(CHART_SIZE_X_ENLARGED,CHART_SIZE_Y_ENLARGED);
		c.setPlotArea(5,20,PLOT_SIZE_X_ENLARGED,PLOT_SIZE_Y_ENLARGED, 0xffffff, -1, 0xbdbdbe, -1, 0xc0c0c0);

		c.addTitle(frequencyTitle);
		c.xAxis().setTitle("ms", "Arial", 8);

		c.xAxis().setWidth(1);
		c.yAxis().setWidth(1);
		c.yAxis().setLabelFormat("");
		c.xAxis().setColors(0x000000,0x9e0000,0X000000,0x9e0000);//x label color red, title color black
		c.yAxis().setColors(0x000000,0x9e0000,0X000000,0x9e0000);
		c.xAxis().setLinearScale(0,MouseChart.FULL_X_GRID,2);
		c.yAxis().setAutoScale(0,0,0);

		LineLayer layer1 = c.addLineLayer();
		layer1.setLineWidth(1);
		layer1.setXData(dataX);

		//add waveforms
		for(int l=0; l<dataYArray.length;l++){
			String s = ColourGenerator.hexColourCode("Splv"+l);
			layer1.addDataSet(dataYArray[l],Integer.parseInt(s, 16),Integer.toString(l));
		}

		//add legend and text label for the waveforms
		c.layout();
		if(addLegend){
			for(int i=0;i<dataYArray.length;i++){
				int y = c.getYCoor(new ArrayMath(dataYArray[i]).avg());
				int x = c.getXCoor(dataX[dataX.length-1]);

				String s = ColourGenerator.hexColourCode("Splv"+i);
				c.addLine(x+5,y,x+25,y,Integer.parseInt(s, 16),2);
				c.addText(x+25, y-5, Integer.toString(levels[i]),"",8,Integer.parseInt(s, 16));
			}
		}
		return c;
	}


	public boolean isAddLegend() {
		return addLegend;
	}
	public void setAddLegend(boolean addLegend) {
		this.addLegend = addLegend;
	}
	public double[] getDataX() {
		return dataX;
	}
	public void setDataX(double[] dataX) {
		this.dataX = dataX;
	}
	public double[][] getDataYArray() {
		return dataYArray;
	}
	public void setDataYArray(double[][] dataYArray) {
		this.dataYArray = dataYArray;
	}
	public String getFrequencyTitle() {
		return frequencyTitle;
	}
	public void setFrequencyTitle(String frequencyTitle) {
		this.frequencyTitle = frequencyTitle;
	}
	public int[] getLevels() {
		return levels;
	}
	public void setLevels(int[] levels) {
		this.levels = levels;
	}


	public double getXInterval() {
		return xInterval;
	}


	public void setXInterval(double interval) {
		xInterval = interval;
	}
}