/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package uk.ac.sanger.mig.abr.graph;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.reporting.ChartDisplay;
import uk.ac.sanger.mig.reporting.services.ChartUtils;

public class AbrPipelineGraph{
	private String pipeline;
	private int size;
	private HashMap<Long, AbrMouseLine> control;
	private List<HashMap<Long, AbrMouseLine>> mutant;
	private List<double[]> controlError;
	private List<List<double[]>> mutantError;
	private List<Long> controlMiceOrder;
	private List<List<Long>> mutantMiceOrder;
	private List<String> labels;

	public AbrPipelineGraph(ChartDisplay data,String pipeLine) {
		this.pipeline = pipeLine;
		control = new HashMap<Long, AbrMouseLine>();
		mutant = new ArrayList<HashMap<Long, AbrMouseLine>>();
		controlError = new  ArrayList<double[]>();
		mutantError = new ArrayList<List<double[]>>();
		controlMiceOrder = new ArrayList<Long>() ;
		mutantMiceOrder = new ArrayList<List<Long>>() ;
		labels = new ArrayList<String>();
		initializeData(data);
	}

	private void initializeData(ChartDisplay data){

		HashMap<Long, AbrMouseLine> het = new HashMap<Long,AbrMouseLine>();
		HashMap<Long, AbrMouseLine> hom = new HashMap<Long,AbrMouseLine>();
		HashMap<Long, AbrMouseLine> wt = new HashMap<Long,AbrMouseLine>();
		List<Long> hetMiceOrder = new ArrayList<Long>();
		List<Long> homMiceOrder = new ArrayList<Long>();
		List<Long> wtMiceOrder = new ArrayList<Long>();

		Collection<AbrThresholdLine> dataT = new ArrayList<AbrThresholdLine>((Collection)data.getData());

		for(AbrThresholdLine line: dataT){
			if(wt.containsKey(line.getExperimentId())){
				wt.get(line.getExperimentId()).getValues().add(line.getValue());
				wt.get(line.getExperimentId()).getXLabels().add(line.getXLabel());
			}else if(het.containsKey(line.getExperimentId())){
				het.get(line.getExperimentId()).getValues().add(line.getValue());
				het.get(line.getExperimentId()).getXLabels().add(line.getXLabel());
			}else if(hom.containsKey(line.getExperimentId())){
				hom.get(line.getExperimentId()).getValues().add(line.getValue());
				hom.get(line.getExperimentId()).getXLabels().add(line.getXLabel());
			}else{
				AbrMouseLine mouseLine = new AbrMouseLine();
				mouseLine.setExperimentId(line.getExperimentId());
				mouseLine.setMouseName(line.getMouseName());
				mouseLine.setAgeInWeeks(line.getAgeInWeeks());
				mouseLine.setGenderId(line.getGenderId());
				mouseLine.setGenotype(line.getGenotype());
				mouseLine.setGenotypeId(line.getGenotypeId());
				mouseLine.setValues(new ArrayList<String>());
				mouseLine.getValues().add(line.getValue());
				mouseLine.setXLabels(new ArrayList<String>());
				mouseLine.getXLabels().add(line.getXLabel());
				if(line.getGenotypeId()==GenotypeDictionary.WILDTYPE){
					wt.put(line.getExperimentId(), mouseLine);
					wtMiceOrder.add(line.getExperimentId());
				}else if(line.getGenotypeId()==GenotypeDictionary.HETEROZYGOUS){
					het.put(line.getExperimentId(), mouseLine);
					hetMiceOrder.add(line.getExperimentId());
				}else if(line.getGenotypeId()==GenotypeDictionary.HOMOZYGOUS){
					hom.put(line.getExperimentId(), mouseLine);
					homMiceOrder.add(line.getExperimentId());
				}
			}
		}
		initilizeOthers(data,wt,het,hom,wtMiceOrder,hetMiceOrder,homMiceOrder);
	}

	private void initilizeOthers(ChartDisplay data, HashMap<Long, AbrMouseLine> wt, HashMap<Long, AbrMouseLine> het, HashMap<Long, AbrMouseLine> hom, List<Long> wtMiceOrder, List<Long> hetMiceOrder, List<Long> homMiceOrder) {
		ChartUtils utils = new ChartUtils(data);
		List<double[]> wtError =
			utils.getAbrStandardError(GenotypeDictionary.WILDTYPE,wt.values().size());
		List<double[]> hetError =
			utils.getAbrStandardError(GenotypeDictionary.HETEROZYGOUS,het.values().size());
		List<double[]> homError =
			utils.getAbrStandardError(GenotypeDictionary.HOMOZYGOUS,hom.values().size());

        this.size = 2;
		if(hom.keySet().size()>0 && het.keySet().size()>0 && wt.keySet().size()==0){
			this.control = het;
			this.mutant.add(hom);
			this.mutantMiceOrder.add(homMiceOrder);
			this.controlMiceOrder = hetMiceOrder;
			this.controlError = hetError;
			this.mutantError.add(homError);
		}else{
			this.control = wt;
			this.mutant.add(hom);
			this.mutant.add(het);
			this.mutantMiceOrder.add(homMiceOrder);
			this.mutantMiceOrder.add(hetMiceOrder);
			this.controlMiceOrder = wtMiceOrder;
			this.controlError = wtError;
			this.mutantError.add(homError);
			this.mutantError.add(hetError);

			int i=0;
			if(hom.keySet().size()>0) i++;
			if(het.keySet().size()>0) i++;
			if(wt.keySet().size()>0) i++;

			if(i==3){
				this.size = 3;
			}else if(i==1){
				throw new RuntimeException("Only one genotype presents, Control cohort or Mutant cohort is missing!");
			}
		}

		for(AbrMouseLine mouseLine: this.control.values()){
			for(String label :mouseLine.getXLabels()){
				if(Integer.parseInt(label)>100){
					label = new String(Integer.parseInt(label)/1000 + "kHz");
				}else{
					label = "click";
				}
				labels.add(label);
			}
			break;
		}
	}

	public List<double[]> getControlError() {
		return controlError;
	}

	public void setControlError(List<double[]> controlError) {
		this.controlError = controlError;
	}



	public HashMap<Long, AbrMouseLine> getControl() {
		return control;
	}

	public void setControl(HashMap<Long, AbrMouseLine> control) {
		this.control = control;
	}

	public List<HashMap<Long, AbrMouseLine>> getMutant() {
		return mutant;
	}

	public void setMutant(List<HashMap<Long, AbrMouseLine>> mutant) {
		this.mutant = mutant;
	}

	public List<List<double[]>> getMutantError() {
		return mutantError;
	}

	public void setMutantError(List<List<double[]>> mutantError) {
		this.mutantError = mutantError;
	}

	public List<Long> getControlMiceOrder() {
		return controlMiceOrder;
	}

	public void setControlMiceOrder(List<Long> controlMiceOrder) {
		this.controlMiceOrder = controlMiceOrder;
	}

	public List<List<Long>> getMutantMiceOrder() {
		return mutantMiceOrder;
	}

	public void setMutantMiceOrder(List<List<Long>> mutantMiceOrder) {
		this.mutantMiceOrder = mutantMiceOrder;
	}

	public String getPipeline() {
		return pipeline;
	}

	public void setPipeline(String pipeline) {
		this.pipeline = pipeline;
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public List<String> getLabels() {
		return labels;
	}

	public void setLabels(List<String> labels) {
		this.labels = labels;
	}
}
