package uk.ac.sanger.mig.abr.parser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

/*
 *****************************************************
 * Class name: Block
 * User: holroyd
 * Date: Aug 5, 2008
 * Time: 10:03:00 AM
 *****************************************************
 */

public class Block {

	private String									name;
	private BlockTraceInfo							traceInfo;
	private BlockTraceName							traceName;
	private boolean									clickBlock;
	private Map<String, Collection<BlockAttribute>>	blockAttributes;
	private Collection<String>						userComments;

	public static final String						EXPERIMENT_BLOCK	= "ExperimentInfo";
	public static final String						TRACE_BLOCK			= "Trace_";

	public static final String						CALIBRATION_TYPE	= "Cal";
	public static final String						ABR_TYPE			= "Abr";

	public static final String						TRACE_INFO			= "TraceInfo";
	public static final String						TRACE_NAME			= "TraceName";
	public static final String						TRACE_DATA			= "TraceData";
	public static final String						USER_COMMENT		= "User Comments";


	/**
	 * Constructs a new Block object with blockName as its name. The block name determines if this
	 * is a trace data block. If the name is "Trace_.*" then it is treated as a trace block.
	 *
	 * @param blockName -
	 *            String name of the block.
	 */
	public Block(String blockName) {
		this.blockAttributes = new LinkedHashMap<String, Collection<BlockAttribute>>();
		this.name = blockName;
		this.traceInfo = null;
		this.traceName = null;
		this.clickBlock = false;
		this.userComments = new ArrayList<String>();
	}


	/**
	 * Adds a BlockAttribute object to the list of block attributes being managed by this block. A
	 * block attribute represents a single key, value pair in an ABR file.
	 *
	 * @param ba -
	 *            a BlockAttribute object.
	 * @throws AttributeFormatException -
	 *             file format error (see extractXXX() methods).
	 */
	public void addBlockAttribute(BlockAttribute ba) throws AttributeFormatException {
		this.extractTraceName(ba);
		this.extractTraceInfo(ba);
		this.extractUserComments(ba);
		if (!this.blockAttributes.containsKey(ba.getAttributeName()))
			this.blockAttributes.put(ba.getAttributeName(), new ArrayList<BlockAttribute>());
		this.blockAttributes.get(ba.getAttributeName()).add(ba);
	}


	/**
	 * Extracts the mouse barcode and the type of block which can be 'Cal' for calibration or 'ABR'
	 * for an ABR test.
	 *
	 * @param blockAttribute -
	 *            a BlockAttribute object.
	 * @throws AttributeFormatException -
	 *             block attribute representing the type is incorrectly formatted (usually the wrong
	 *             number of comma separated values - min 2).
	 */
	protected void extractTraceName(BlockAttribute blockAttribute) throws AttributeFormatException {
		if (!blockAttribute.getAttributeName().equals(TRACE_NAME)) {
			return;
		}
		this.traceName = new BlockTraceName(this, blockAttribute);
	}


	/**
	 * Extracts the blocks trace info by passing the block attribute to a new instance of a
	 * BlockTraceInfo object. So it's actually that class that does the extracting and this method
	 * that wraps it around finding the correct block attribute to send to it. The correct block
	 * attribute has a value of "TraceInfo".
	 *
	 * @param blockAttribute -
	 *            a BlockAttribute object.
	 * @throws AttributeFormatException -
	 *             the trace info format is incorrect.
	 */
	public void extractTraceInfo(BlockAttribute blockAttribute) throws AttributeFormatException {
		if (!blockAttribute.getAttributeName().equals(TRACE_INFO)) {
			return;
		}
		this.traceInfo = new BlockTraceInfo(this, blockAttribute);
	}


	/**
	 * Stores all user comments.
	 *
	 * @param blockAttribute -
	 *            the block attribute that may be a user comment.
	 */
	public void extractUserComments(BlockAttribute blockAttribute) {
		if (blockAttribute.getAttributeName().equals(USER_COMMENT)) {
			this.userComments.add(blockAttribute.getAttributeValue());
		}
	}


	/**
	 * Finds all trace data block attributes and returns them in a collection.
	 *
	 * @return Collection<BlockAttribute> - collection of BlockAttributes which are trace data type
	 *         attributes.
	 */
	public Collection<BlockAttribute> getTraceAttributes() {
		return this.isAbrTraceBlock() && this.blockAttributes.containsKey(TRACE_DATA) ? this.blockAttributes
				.get(TRACE_DATA) : new ArrayList<BlockAttribute>();
	}


	/**
	 * Returns true if this is the header block, otherwise false.
	 *
	 * @return boolean - true if the header, otherwise false.
	 */
	public boolean isHeaderBlock() {
		return this.name.equals(EXPERIMENT_BLOCK);
	}


	/**
	 * Returns boolean true if this block contains trace data. This is determined by the block name,
	 * if it starts with "Trace_" and then a number then it is considered a trace block.
	 *
	 * @return true - this is a trace block, false - this is not a trace block.
	 */
	public boolean isTraceBlock() {
		return this.name.startsWith(TRACE_BLOCK);
	}


	/**
	 * Returns true if this block is a calibration block. This is determined by the TraceName
	 * BlockAttribute which is extracted by the extractBlockType() method when the
	 * addBlockAttribute() method is called.
	 *
	 * @return true - if block is a calibration block, false otherwise.
	 */
	public boolean isCalibrationTraceBlock() {
		return this.isTraceBlock() && this.getType().equals(CALIBRATION_TYPE);
	}


	/**
	 * Returns true if this block is an ABR test block. This is determined by the TraceName
	 * BlockAttribute which is extracted by the extractBlockType() method when the
	 * addBlockAttribute() method is called.
	 *
	 * @return true - if block is an ABR test block, false otherwise.
	 */
	public boolean isAbrTraceBlock() {
		return this.isTraceBlock() && this.getTraceName().isAbrTraceBlock();
	}


	/**
	 * Returns a collection of the data points found in this block, if it is an ABR test block.
	 *
	 * @return traceDataPoints - collection of double data points.
	 * @throws IllegalArgumentException -
	 *             this is not an ABR test block.
	 * @throws NumberFormatException -
	 *             data point can not be casted to a Double.
	 */
	public double[] getDataPoints() throws NumberFormatException {
		if (!this.isAbrTraceBlock() && !this.isCalibrationTraceBlock())
			throw new IllegalArgumentException("block is not an ABR test or calibration block");
		int numberDataPoints = this.getBlockTraceInfo().getNumberPoints();
		double[] dataPoints = new double[numberDataPoints];
		int index = 0;
		for (BlockAttribute blockAttribute : this.getBlockAttributes().get(TRACE_DATA)) {
			if (blockAttribute.isTraceDataAttribute()) {
				String[] strDataPoints = blockAttribute.getAttributeValue().split(",");
				for (int i = 0; i < strDataPoints.length; i++, index++) {
					dataPoints[index] = Double.parseDouble(strDataPoints[i]);
				}
			}
		}
		return dataPoints;
	}


	/**
	 * Returns the name of this block.
	 *
	 * @return String blockName
	 */
	public String getName() {
		return name;
	}


	/**
	 * Returns a raw collection of all this blocks attributes.
	 *
	 * @return Collection<BlockAttribute> blockAttributes
	 */
	public Map<String, Collection<BlockAttribute>> getBlockAttributes() {
		return blockAttributes;
	}


	/**
	 * Returns the BlockTraceInfo object that contains the information on the traces if this is an
	 * ABR trace block containing traces.
	 *
	 * @return blockTraceInfo - information on the ABR trace block, if this is one, otherwise null.
	 *         Use the isAbrTraceBlock() to verify the type of block before calling this method or
	 *         accept that it may return null.
	 */
	public BlockTraceInfo getBlockTraceInfo() {
		return traceInfo;
	}


	/**
	 * Returns the BlockTraceName object that contains the information on the trace name such as the
	 * mouse id, the block type, the frequency (if this is an ABR block), the frquency colour (if
	 * this is a calibration type block) and the level.
	 *
	 * @return traceName - the BlockTraceName object.
	 */
	public BlockTraceName getTraceName() {
		return traceName;
	}


	/**
	 * Wrapper method that returns the string type of this block.
	 *
	 * @return type - the string type of this block (either CAL or ABR).
	 * @see uk.ac.sanger.mig.abr.parser.BlockTraceName.getType()
	 */
	public String getType() {
		return this.getTraceName() == null ? null : this.getTraceName().getType();
	}


	/**
	 * If set to true then this is a click block, which is similar to a calibration block, but is a
	 * real test just performed at setup and end of experiments. Defaults to false - not a click
	 * block.
	 *
	 * @param clickBlock -
	 *            true if this is a click block setup or end of experiment marker, otherwise false
	 *            (default).
	 */
	public void setClickBlock(boolean clickBlock) {
		this.clickBlock = clickBlock;
	}


	/**
	 * Returns true if this is a click block. A click block is a start and/or end marker for the
	 * experiment as a whole. It is used to calibrate that the machine is working correctly under
	 * normal test conditions.
	 *
	 * @return true - if this is a click block.
	 */
	public boolean isClickBlock() {
		return clickBlock;
	}


	/**
	 * Returns the string of user comments found in this block.
	 *
	 * @return user comments as a string.
	 */
	public Collection<String> getUserComments() {
		return this.userComments;
	}

}
