package uk.ac.sanger.mig.abr.parser;

public class BlockTraceName {

	private Block			block;
	private BlockAttribute	blockAttribute;
	private String			mouseBarcode;
	private String			type;
	private String			frequencyColour;
	private int				frequency;
	private int				level;


	public BlockTraceName(Block block, BlockAttribute blockAttribute) throws AttributeFormatException {
		this.block = block;
		this.blockAttribute = blockAttribute;
		this.mouseBarcode = null;
		this.type = null;
		this.frequencyColour = null;
		this.frequency = 0;
		this.level = 0;
		this.extract();
	}


	/**
	 * Extracts the trace name by splitting it into comma separated values and setting the mouse id,
	 * block type, frequency and level.
	 *
	 * @throws AttributeFormatException -
	 *             the expected formatting is wrong.
	 */
	private void extract() throws AttributeFormatException {
		this.extractTraceName();
	}


	/**
	 * Extracts the trace name data by splitting the string value using a comma delimeter. Uses the
	 * second and third element of the resulting array as the frequency and the level. If this is an
	 * ABR block then the frequency is set as a double, if it is a calibration block then it's set
	 * as the frequency colour (not sure what this means). Calls the extractMouseIdAndBlockType()
	 * method after successfully splitting the attribute value.
	 *
	 * @throws AttributeFormatException -
	 *             the format of the attribute value is wrong.
	 */
	private void extractTraceName() throws AttributeFormatException {
		String[] b = this.blockAttribute.getAttributeValue().split(",");
		if (b.length < 3) {
			throw new AttributeFormatException("trace name attribute format for block with name '"
					+ this.block.getName() + "' is incorrectly formatted");
		}
		this.extractMouseIdAndBlockType(b[0].trim());
		if (this.isAbrTraceBlock()) {
			this.frequency = Integer.parseInt(b[1].trim());
		} else {
			this.frequencyColour = b[1].trim();
		}

		this.level = Integer.parseInt(b[2].trim());
	}


	/**
	 * Extract the mouse id and the block type. These values are stored in att and are separated by
	 * a space.
	 *
	 * @param att -
	 *            string mouse id, space, then the block type.
	 * @throws AttributeFormatException -
	 *             the format of the mouse id and block type is wrong.
	 */
	private void extractMouseIdAndBlockType(String att) throws AttributeFormatException {
		String[] c = att.split(" ");
		if (c.length < 2) {
			throw new AttributeFormatException("trace name animal id and type for block with name '" + block.getName()
					+ "' is incorrectly formatted");
		}
		this.mouseBarcode = c[0];
		this.type = c[1];
	}


	/**
	 * Returns true if this block is an ABR test block. This is determined by the TraceName
	 * BlockAttribute which is extracted by the extractBlockType() method when the
	 * addBlockAttribute() method is called.
	 *
	 * @return true - if block is an ABR test block, false otherwise.
	 */
	public boolean isAbrTraceBlock() {
		return this.getType().equals("ABR");
	}


	/**
	 * Returns the frequency of the data points in this block if it is an ABR test block, otherwise
	 * 0d.
	 *
	 * @return frequency - positive double value if this is an ABR test block, otherwise 0d.
	 */
	public int getFrequency() {
		return frequency;
	}


	/**
	 * Returns the frquency colour (for want of a better word) if this is a calibration block,
	 * otherwise returns null.
	 *
	 * @return frequencyColour - usually white, but not to sure what this means, null if not a
	 *         calibration block.
	 */
	public String getFrequencyColour() {
		return frequencyColour;
	}


	/**
	 * Returns the level.
	 *
	 * @return level - the level.
	 */
	public int getLevel() {
		return level;
	}


	/**
	 * Returns the mouse barcode if it has been extracted. It may not have been extracted if this is
	 * a calibration or header block in which case this method returns null.
	 *
	 * @return String mouseBarcode or null
	 */
	public String getMouseBarcode() {
		return mouseBarcode;
	}


	/**
	 * Returns the block type, which is either 'Cal' (calibration), 'ABR' (ABR test block) or null
	 * (no type determined - if not a trace block). Returns null if the block has no TraceName
	 * attribute, for instance if it where a calibration or header block.
	 *
	 * @return blockType - String representation of the block type, this is either 'Cal' for a
	 *         calibration block or 'ABR' for an ABR trace block.
	 */
	public String getType() {
		return type;
	}


	/**
	 * Returns the block associated with this name.
	 *
	 * @return block - the Block objects.
	 */
	public Block getBlock() {
		return block;
	}


	/**
	 * Returns the block attribute that this object represents.
	 *
	 * @return blockAttribute - the attribute in the block.
	 */
	public BlockAttribute getBlockAttribute() {
		return blockAttribute;
	}

}
