package uk.ac.sanger.mig.abr.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/*
 *****************************************************
 * Class name: FileParser
 * User: holroyd
 * Date: Jul 31, 2008
 * Time: 1:35:06 PM
 *****************************************************
 */

public class FileParser {

	private File						file;
	private transient BufferedReader	reader;
	private Collection<Block>			blocks;
	private HeaderBlock					headerBlock;
	private List<Block>					startClickBlocks;
	private List<Block>					endClickBlocks;
	private Pattern						lineStrippingPattern;

	public static final int				CLICK_BLOCK_FREQUENCY		= 100;
	public static final int				CLICK_BLOCK_LEVEL			= 70;
	public static final int				DEFAULT_NUMBER_DATAPOINTS	= 1953;

	static final Comparator<Frequency>	FREQUENCY_ASC_ORDER			= new Comparator<Frequency>() {
																		public int compare(Frequency f1, Frequency f2) {
																			if (f1.getFrequency() < f2.getFrequency()) {
																				return -1;
																			}
																			if (f1.getFrequency() > f2.getFrequency()) {
																				return 1;
																			}
																			return 0;
																		}
																	};


	public FileParser() {
		initialise();
	}


	public FileParser(final File file) {
		this.file = file;
		initialise();
	}


	/**
	 * Initialise object attributes.
	 *
	 */
	private void initialise() {
		this.lineStrippingPattern = Pattern.compile("(\\x2c+)$");
	}


	/**
	 * Starts the parsing of the File. Sets the file attribute to the passed
	 * File and calls the <code>parse()</code> method.
	 *
	 * @param file -
	 *            the file to parse.
	 * @throws IOException -
	 *             an IO exception has occurred whilst reading the file.
	 * @throws BlockAttributeAssociationException -
	 *             an attribute has been found before a block has been found,
	 *             this is a file format error.
	 * @throws AttributeFormatException -
	 *             an attribute within a block is incorrectly formatted, this
	 *             usually means the attribute does not have the expected number
	 *             of comma separated values.
	 */
	public void parse(final File file) throws IOException, BlockAttributeAssociationException, AttributeFormatException, ParseException,
			StartClickBlockMissingException, EndClickBlockMissingException, HeaderBlockMissingException {
		this.setFile(file);
		this.parse();
	}


	/**
	 * Starts the parsing of the File. Sets the file attribute to the passed
	 * File and calls the <code>parse()</code> method.
	 *
	 * @throws IOException -
	 *             an IO error ocurred whilst reading the file.
	 * @throws BlockAttributeAssociationException -
	 *             a block style (key, value pair) element has been found before
	 *             a block name (wrapped in square brackets) has been found.
	 *             Check the file format.
	 */
	public void parse() throws IOException, BlockAttributeAssociationException, AttributeFormatException, ParseException,
			StartClickBlockMissingException, EndClickBlockMissingException, HeaderBlockMissingException {
		this.openFile();
		this.parseAbrExperimentFile();
	}


	/**
	 * Starts the parsing of the input stream. Uses the input stream to read the
	 * ABR file and calls the parseAbrExperimentFile() method.
	 *
	 * @param inputStreamReader -
	 *            an InputStreamReader to use as the ABR file.
	 * @throws IOException -
	 *             an IO error ocurred whilst reading the file.
	 * @throws BlockAttributeAssociationException -
	 *             a block style (key, value pair) element has been found before
	 *             a block name (wrapped in square brackets) has been found.
	 *             Check the file format.
	 */
	public void parse(InputStreamReader inputStreamReader) throws IOException, BlockAttributeAssociationException,
			AttributeFormatException, ParseException, StartClickBlockMissingException, EndClickBlockMissingException,
			HeaderBlockMissingException {
		this.openFile(inputStreamReader);
		this.parseAbrExperimentFile();
	}


	/**
	 * Initialises object attributes and parses the file, assumes that the
	 * reader has been set using a call to a parse() method. Parses the ABR
	 * experiment file and extracts the click blocks.
	 *
	 * @throws IOException
	 * @throws BlockAttributeAssociationException
	 * @throws AttributeFormatException
	 */
	private void parseAbrExperimentFile() throws IOException, BlockAttributeAssociationException, AttributeFormatException, ParseException,
			StartClickBlockMissingException, EndClickBlockMissingException, HeaderBlockMissingException {
		this.blocks = new ArrayList<Block>();
		this.startClickBlocks = new ArrayList<Block>();
		this.endClickBlocks = new ArrayList<Block>();
		this.readBlocks();
		this.extractHeaderBlock();
		this.extractStartClickBlocks();
		this.extractEndClickBlocks();
	}


	/**
	 * Opens the file and stores the BufferedReader ready for reading.
	 *
	 * @throws FileNotFoundException
	 */
	private void openFile() throws FileNotFoundException {
		this.reader = new BufferedReader(new FileReader(this.getFile()));
	}


	/**
	 * Opens the file and stores the BufferedReader ready for reading.
	 *
	 * @throws FileNotFoundException
	 */
	private void openFile(InputStreamReader inputStreamReader) throws FileNotFoundException {
		this.reader = new BufferedReader(inputStreamReader);
	}


	/**
	 * Reads each line of the file searching for block names (strings wrapped in
	 * square brackets) and appends lines beneath them to
	 *
	 * @throws IOException
	 * @throws BlockAttributeAssociationException
	 *
	 */
	private void readBlocks() throws IOException, BlockAttributeAssociationException, AttributeFormatException {
		String line;
		String blockName = "";
		Block block = null;
		while ((line = this.reader.readLine()) != null) {
			line = line.trim();
			if (line.length() == 0) {
				continue;
			}
			line = stripLine(line);
			if (line.charAt(0) == '[') {
				blockName = this.stripBlockName(line);
				block = new Block(blockName);
				blocks.add(block);
			} else {
				if (blockName.equals("") || block == null) {
					throw new BlockAttributeAssociationException(
							"format error - no current block found - perhaps this is not an ABR experiment file?");
				}
				String[] keyValuePair = line.split("\\s+=\\s*");
				if (keyValuePair.length != 2)
					throw new AttributeFormatException("format error - line '" + line + "' is missing a key or a value");
				String key = keyValuePair[0].trim();
				String value = keyValuePair[1].trim();
				block.addBlockAttribute(new BlockAttribute(key, value));
			}
		}
		this.reader.close();
	}


	/**
	 * Strips lines that have been truncated with commas by removing trailing
	 * commas. Comma truncation appears to happen when a csv file is edited in
	 * Excel and saved.
	 *
	 * @param line -
	 *            the line to match and remove trailing commas from.
	 * @return line - the line with the trailing commas removed.
	 */
	private String stripLine(String line) {
		Matcher m = this.lineStrippingPattern.matcher(line);
		if (m.find()) {
			int start = m.start();
			line = line.substring(0, start);
		}
		return line;
	}


	/**
	 * Extracts the header block and stores it in a wrapper HeaderBlock object
	 * that knows how to parse it. Stores the header block in the object
	 * attribute.
	 *
	 * @throws AttributeFormatException -
	 *             an attribute is badly formatter.
	 * @throws ParseException -
	 *             parsing the experiment date failed.
	 * @throws HeaderBlockMissingException -
	 *             the header block can not be found.
	 *
	 */
	public void extractHeaderBlock() throws AttributeFormatException, ParseException, HeaderBlockMissingException {
		Block header = null;
		for (Block b : this.blocks) {
			if (b.isHeaderBlock()) {
				header = b;
				break;
			}
		}
		if (header == null)
			throw new HeaderBlockMissingException();
		this.headerBlock = new HeaderBlock(header);
	}


	/**
	 * Extracts the click blocks at the start of the file. These blocks are
	 * running calibration tests that will be used to compare against the real
	 * test data.
	 *
	 */
	private void extractStartClickBlocks() throws StartClickBlockMissingException {
		this.startClickBlocks = this.extractClickBlocks(this.getAbrTraceBlocks());
		if (this.startClickBlocks.isEmpty())
			throw new StartClickBlockMissingException();
	}


	/**
	 * Extracts the click blocks at the end of the file. These blocks are
	 * running calibration tests that will be used to compare against the real
	 * test data.
	 *
	 */
	private void extractEndClickBlocks() throws EndClickBlockMissingException {
		List<Block> blocks = new ArrayList<Block>(this.getAbrTraceBlocks());
		Collections.reverse(blocks);
		this.endClickBlocks = this.extractClickBlocks(blocks);
		if (this.endClickBlocks.isEmpty())
			throw new EndClickBlockMissingException();
		Collections.reverse(this.endClickBlocks);
	}


	/**
	 * Generic method for finding click blocks in the collection of passed
	 * blocks and setting their internal flag before returning them. Click
	 * blocks should only be found right at the start of the list of blocks,
	 * otherwise they're real tests. Once this method finds one or more click
	 * blocks it stops assuming all other blocks that match the click block
	 * criteria after these are real tests, not click blocks.
	 *
	 * @param blocks -
	 *            collection of blocks to search for click blocks.
	 * @return clickBlocks - list of click blocks found.
	 */
	private List<Block> extractClickBlocks(Collection<Block> blocks) {
		List<Block> clickBlocks = new ArrayList<Block>();
		for (Block block : blocks) {
			if (block.getTraceName().getFrequency() == FileParser.CLICK_BLOCK_FREQUENCY
					&& block.getTraceName().getLevel() == FileParser.CLICK_BLOCK_LEVEL) {
				block.setClickBlock(true);
				clickBlocks.add(block);
			} else {
				break;
			}
		}
		return clickBlocks;
	}


	/**
	 * Block names are surrounded with square brackets. This method removes them
	 * and returns the new string.
	 *
	 * @param blockName -
	 *            the name of the block.
	 * @return String
	 */
	private String stripBlockName(final String blockName) {
		return blockName.substring(1, (blockName.length() - 1));
	}


	/**
	 * Returns a collection of the blocks containing trace data including
	 * calibration trace data and header blocks. If you don't want the
	 * calibration trace data blocks use the getAllTraceBlocks() method instead.
	 *
	 * @return blocks - collection of all blocks that make up the file being
	 *         parsed.
	 */
	public Collection<Block> getBlocks() {
		return this.blocks;
	}


	/**
	 * Returns a collection of the blocks containing trace data and are of type
	 * 'ABR', excludes header and calibration blocks.
	 *
	 * @return ArrayList<Block> traceBlocks
	 */
	public List<Block> getTraceBlocks() {
		List<Block> traceBlocks = new ArrayList<Block>();
		for (Block block : this.blocks) {
			if (block.isTraceBlock()) {
				traceBlocks.add(block);
			}
		}
		return traceBlocks;
	}


	/**
	 * Returns a collection of the blocks containing trace data. This method
	 * does not return blocks other than trace blocks of type ABR, calibration
	 * and header blocks are not returned - use the getAllBlocks() method to get
	 * all blocks regardless of content and type. Click blocks are also ignored.
	 * Blocks that do not have the default number of data points are also
	 * ignored for now.
	 *
	 * @return traceBlocks - collection of ABR trace blocks.
	 */
	public List<Block> getAbrTraceBlocks() {
		List<Block> traceBlocks = new ArrayList<Block>();
		for (Block block : this.getTraceBlocks()) {
			if (block.isAbrTraceBlock() && !block.isCalibrationTraceBlock() && !block.isClickBlock()
					&& block.getBlockTraceInfo().getNumberPoints() == DEFAULT_NUMBER_DATAPOINTS) {
				traceBlocks.add(block);
			}
		}
		return traceBlocks;
	}


	/**
	 * Returns a collection of the blocks containing trace data with given
	 * frequency. This method does not return blocks other than trace blocks of
	 * type ABR, calibration and header blocks are not returned - use the
	 * getAllBlocks() method to get all blocks regardless of content and type.
	 *
	 * @param frequency -
	 *            the frequency value to find ABR test blocks for.
	 * @return traceBlocks - collection of ABR blocks for the passed frequency.
	 */
	public Collection<Block> getFreqAbrTraceBlocks(final int frequency) {
		Collection<Block> traceBlocks = new ArrayList<Block>();
		for (Block block : this.getAbrTraceBlocks()) {
			if (block.isTraceBlock() && !block.isCalibrationTraceBlock() && (block.getTraceName().getFrequency() == frequency)) {
				traceBlocks.add(block);
			}
		}
		return traceBlocks;
	}


	/**
	 * Finds the distinct frequencies and levels for each block and stores them
	 * in the following hierarchy:
	 *
	 * <pre>
	 *  Frequency
	 *  	|
	 *  	+----- Collection&lt;Level&gt; levels
	 *  				|		|		|
	 *  				|		|		+----- Collection&lt;Block&gt; blocks
	 *  				|		|
	 *  				|		+----- Collection&lt;Block&gt; blocks
	 *  				|
	 *  				+----- Collection&lt;Block&gt; blocks
	 * </pre>
	 *
	 * Frequencies and levels are sorted into ascending order.
	 *
	 * @return sortedFrequencies - sorted collection of Frequency objects.
	 */
	public Collection<Frequency> getAbrTraceBlocksByFrequencyLevel() {
		Map<Integer, Frequency> frequencies = new HashMap<Integer, Frequency>();
		for (Block block : this.getAbrTraceBlocks()) {
			Integer frequency = new Integer(block.getTraceName().getFrequency());
			Integer level = new Integer(block.getTraceName().getLevel());
			if (!frequencies.containsKey(frequency)) {
				frequencies.put(frequency, new Frequency(frequency.intValue()));
			}
			frequencies.get(frequency).addLevelBlock(level, block);
		}
		List<Frequency> sortedFrequencies = new ArrayList<Frequency>(frequencies.values());
		Collections.sort(sortedFrequencies, FREQUENCY_ASC_ORDER);
		return sortedFrequencies;
	}


	/**
	 * Returns a collection of Block objects representinig the calibration data.
	 *
	 * @return calibrationBlocks -
	 */
	public Collection<Block> getCalibrationBlocks() {
		Collection<Block> calibrationBlocks = new ArrayList<Block>();
		for (Block block : this.blocks) {
			if (block.isCalibrationTraceBlock()) {
				calibrationBlocks.add(block);
			}
		}
		return calibrationBlocks;
	}


	/**
	 * Returns the last trace block that is defined as a calibration block.
	 *
	 * @return Block calibrationBlock
	 */
	public Block getCalibrationTraceBlock() {
		List<Block> calibrationBlocks = new ArrayList<Block>(this.getCalibrationBlocks());
		if (calibrationBlocks.isEmpty()) {
			return null;
		}
		return calibrationBlocks.get(calibrationBlocks.size() - 1);
	}


	/**
	 * Finds the highest number of data points in from all of the frequencies
	 * levels blocks and returns it.
	 *
	 * @param frequencies -
	 *            collection of the frequencies to find the highest number of
	 *            data points in.
	 * @return highestNumberDataPoints - the largest number of data points
	 *         found.
	 */
	public static int findHighestNumberDataPoints(Collection<Frequency> frequencies) {
		int highestNumberDataPoints = 0;
		for (Frequency frequency : frequencies) {
			for (Level level : frequency.getLevels()) {
				for (Block block : level.getBlocks()) {
					int numberDataPoints = block.getBlockTraceInfo().getNumberPoints();
					if (numberDataPoints > highestNumberDataPoints) {
						highestNumberDataPoints = numberDataPoints;
					}
				}
			}
		}
		return highestNumberDataPoints;
	}


	/**
	 * Returns the File that will be passed when the <code>parse()</code>
	 * method is called.
	 *
	 * @return File
	 */
	public File getFile() {
		return file;
	}


	/**
	 * Sets the file to be parsed. It is necessary to call the parse() method
	 * after this method has been used to set the file.
	 *
	 * @param file
	 */
	public void setFile(final File file) {
		this.file = file;
	}


	/**
	 * Returns the header block wrapper around the header Block object.
	 *
	 * @return header - the header Block.
	 */
	public HeaderBlock getHeaderBlock() {
		return this.headerBlock;
	}


	/**
	 * Returns the list of blocks that represent the click blocks executed at
	 * the beginning of the experiment as a sort of calibration test. There may
	 * be more than one as the test may have been repeated, use
	 * getStartClickBlock() method to find *the* start click block.
	 *
	 * @return startClickBlocks - list of blocks at the start of the experiment.
	 */
	public List<Block> getStartClickBlocks() {
		return startClickBlocks;
	}


	/**
	 * Returns the list of blocks that represent the click blocks executed at
	 * the end of the experiment as a sort of calibration test. There may be
	 * more than one as the test may have been repeated, use getEndClickBlock()
	 * method to find *the* end click block.
	 *
	 * @return endClickBlocks - list of blocks at the end of the experiment.
	 */
	public List<Block> getEndClickBlocks() {
		return endClickBlocks;
	}


	/**
	 * Returns the single click block that should be used as *the* click block
	 * for the start of the experiment or null if no click blocks have been
	 * found at the beginning of the file.
	 *
	 * @return block or null if no click block found.
	 */
	public Block getStartClickBlock() {
		return this.startClickBlocks.isEmpty() ? null : this.startClickBlocks.get(this.startClickBlocks.size() - 1);
	}


	/**
	 * Returns the single click block that should be used as *the* click block
	 * for the end of the experiment or null if no click blocks have been found
	 * at the end of the file.
	 *
	 * @return block or null if no click block found.
	 */
	public Block getEndClickBlock() {
		return this.endClickBlocks.isEmpty() ? null : this.endClickBlocks.get(this.endClickBlocks.size() - 1);
	}


	/**
	 * Returns all of the user comments from the experiment file.
	 *
	 * @return user comments.
	 */
	public Collection<ExperimentComment> getUserComments() {
		Collection<ExperimentComment> comments = new ArrayList<ExperimentComment>();
		for (Block b : this.blocks) {
			Collection<String> blockExperimentComments = b.getUserComments();
			if (blockExperimentComments.size() > 0) {
				int frequency = b.getTraceName().getFrequency();
				int level = b.getTraceName().getLevel();
				for (String comment : blockExperimentComments)
					comments.add(new ExperimentComment(frequency, level, comment));
			}
		}
		return comments;
	}


	/**
	 * Extracts each frequencies levels into a list, one per frequency as they
	 * may use different step intervals. The levels are added in reverse order
	 * for display.
	 *
	 * @param frequencies -
	 *            list of frequencies to extract levels from.
	 * @return
	 */
	public static List<List<Integer>> extractFrequencyLevelsForDisplay(List<Frequency> frequencies) {
		List<List<Integer>> frequencyLevels = new ArrayList<List<Integer>>();
		for (Frequency frequency : frequencies) {
			List<Integer> l = new ArrayList<Integer>();
			int[] levelValues = frequency.getLevelValues();
			for (int i = levelValues.length - 1; i >= 0; i--) {
				l.add(levelValues[i]);
			}
			frequencyLevels.add(l);
		}
		return frequencyLevels;
	}
}