package uk.ac.sanger.mig.abr.parser;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class HeaderBlock {

	private Block									headerBlock;
	private Map<String, Collection<BlockAttribute>>	blockAttributes;
	private String									mouseId;
	private String									mouseWeight;
	private String									anaesthetic;
	private String									anaestheticDose;
	private String									anaestheticRoute;
	private String									dateOfBirth;
	private String									dateOfExperiment;
	private String									inductionTime;
	private String									experimentType;
	private String									experimentName;
	private String									pil;
	private String									ppl;
	private Date									experimentDateTime;
	private String									optionalLicence;

	public static final String						ANIMAL_ID				= "Animal ID";
	public static final String						ANIMAL_WEIGHT			= "Animal Weight (g)";
	public static final String						ANAESTHETIC				= "Anaesthetic";
	public static final String						ANAESTHETIC_DOSE		= "Anaesthetic Dose (ml)";
	public static final String						ANAESTHETIC_ROUTE		= "Anaesthetic Route";
	public static final String						DATE_OF_BIRTH			= "Date of Birth (dd/mm/yyyy)";
	public static final String						DATE_OF_EXPERIMENT		= "Date of Experiment (dd/mm/yyyy)";
	public static final String						INDUCTION_TIME			= "Induction Time (hr:min)";
	public static final String						EXPERIMENT_TYPE			= "Experiment Type";
	public static final String						EXPERIMENT_NAME			= "Experimenter Name";
	public static final String						PIL						= "Personal Licence Number (PIL)";
	public static final String						PPL						= "Project Licence Number (PPL)";
	public static final String						OPTIONAL_LICENCE		= "19b";
	public static final String						HARMFUL_PROTOCOL_CODE	= "3b";


	public HeaderBlock(Block block) throws AttributeFormatException, ParseException {
		this.headerBlock = block;
		this.blockAttributes = block.getBlockAttributes();
		extractAttributes();
	}


	/**
	 * Extracts the attributes of the header block into the object attributes. Elements are set as
	 * null if they do not exists - this is a feature of the getAttributeValue() method.
	 *
	 */
	public void extractAttributes() throws AttributeFormatException, ParseException {
		this.mouseId = this.getAttributeValue(ANIMAL_ID);
		this.mouseWeight = this.getAttributeValue(ANIMAL_WEIGHT);
		this.anaesthetic = this.getAttributeValue(ANAESTHETIC);
		this.anaestheticDose = this.getAttributeValue(ANAESTHETIC_DOSE);
		this.anaestheticRoute = this.getAttributeValue(ANAESTHETIC_ROUTE);
		this.dateOfBirth = this.getAttributeValue(DATE_OF_BIRTH);
		this.dateOfExperiment = this.getAttributeValue(DATE_OF_EXPERIMENT);
		this.inductionTime = this.getAttributeValue(INDUCTION_TIME);
		this.experimentType = this.getAttributeValue(EXPERIMENT_TYPE);
		this.experimentName = this.getAttributeValue(EXPERIMENT_NAME);
		this.pil = this.getAttributeValue(PIL);
		this.ppl = this.getAttributeValue(PPL);
		this.optionalLicence = this.getAttributeValue(OPTIONAL_LICENCE, false);
		this.experimentDateTime = this.extractExperimentDateTime(this.dateOfExperiment, this.inductionTime);
	}


	/**
	 * Extracts the date and time of the experiment into a Date object. Uses a loose pattern to
	 * capture the input and reconstruct the paramters to make the date and time.
	 *
	 * @param experimentDate -
	 *            the string date from the ABR experiment file.
	 * @param experimentTime -
	 *            the string time from the ABR experiment file.
	 * @return Date object representing the experiment date and time.
	 * @throws AttributeFormatException -
	 *             the date and/or time is incorrectly formatted.
	 * @throws ParseException -
	 *             parsing the date and time from the reconstructed string failed.
	 */
	private Date extractExperimentDateTime(String experimentDate, String experimentTime)
			throws AttributeFormatException, ParseException {
		if (experimentDate == null || experimentTime == null)
			throw new IllegalArgumentException("null arguments are not allowed");
		String dateToMatch = experimentDate + ", " + experimentTime;
		Pattern p = Pattern
				.compile("^.*\\s*(\\d\\d)\\s*,\\s*(\\d\\d)\\s*,\\s*(\\d\\d\\d\\d)\\s*,\\s*(\\d\\d)\\s*,\\s*(\\d\\d).*$");
		Matcher m = p.matcher(dateToMatch);
		if (!m.matches())
			throw new AttributeFormatException("experiment date and/or induction time format is incorrect");
		StringBuffer reconstructedDateTime = new StringBuffer();
		for (int i = 1; i <= m.groupCount(); i++) {
			String group = (i == m.groupCount() ? m.group(i) : m.group(i) + ", ");
			reconstructedDateTime.append(group);
		}
		SimpleDateFormat formatter = new SimpleDateFormat("dd, MM, yyyy, HH, mm");
		return (Date) formatter.parse(reconstructedDateTime.toString());
	}


	/**
	 * Returns the value of the last instance of the keys values. The key must exist otherwise an
	 * IllegalArgumentException will be thrown.
	 *
	 * @param key -
	 *            hash key into the list of attributes.
	 * @return the value of the last attribute.
	 */
	public String getAttributeValue(String key) {
		return this.getAttributeValue(key, true);
	}


	/**
	 * Looks up the list of block attributes using the passed key. Throws an exception if the key is
	 * not in the map, otherwise return the last BlockAttribute objects string value.
	 *
	 * @param key -
	 *            the key in to the blockAttributes map used to find the collection of
	 *            BlockAttribute objects.
	 * @return string block attribute value.
	 * @throws IllegalArgumentException -
	 *             if the key does not exist in the map. This is synonymous with the key not
	 *             existing in the parsed file.
	 */
	public String getAttributeValue(String key, boolean isRequired) {
		List<BlockAttribute> ba = new ArrayList<BlockAttribute>();
		if (!this.blockAttributes.containsKey(key)) {
			if (isRequired)
				throw new IllegalArgumentException("block attribute is not found in file: " + key);
		} else {
			ba.addAll(blockAttributes.get(key));
		}
		int index = ba.size() > 1 ? ba.size() - 1 : 0;
		return ba.isEmpty() ? null : ba.get(index).getAttributeValue();
	}


	/**
	 * @return the anaesthetic
	 */
	public String getAnaesthetic() {
		return anaesthetic;
	}


	/**
	 * @return the anaestheticDose
	 */
	public Double getAnaestheticDose() {
		return anaestheticDose == null ? null : Double.parseDouble(anaestheticDose);
	}


	/**
	 * @return the anaestheticRoute
	 */
	public String getAnaestheticRoute() {
		return anaestheticRoute;
	}


	/**
	 * @return the blockAttributes
	 */
	public Map<String, Collection<BlockAttribute>> getBlockAttributes() {
		return blockAttributes;
	}


	/**
	 * @return the dateOfBirth
	 */
	public String getDateOfBirth() {
		return dateOfBirth;
	}


	/**
	 * @return the dateOfExperiment
	 */
	public String getDateOfExperiment() {
		return dateOfExperiment;
	}


	/**
	 * @return the experimentName
	 */
	public String getExperimentName() {
		return experimentName;
	}


	/**
	 * @return the experimentType
	 */
	public String getExperimentType() {
		return experimentType;
	}


	/**
	 * @return the headerBlock
	 */
	public Block getHeaderBlock() {
		return headerBlock;
	}


	/**
	 * @return the inductionTime
	 */
	public String getInductionTime() {
		return inductionTime;
	}


	/**
	 * @return the mouseId
	 */
	public String getMouseId() {
		return mouseId;
	}


	/**
	 * @return the mouseWeight
	 */
	public Double getMouseWeight() {
		return mouseWeight == null ? null : Double.parseDouble(mouseWeight);
	}


	/**
	 * @return the pil
	 */
	public String getPil() {
		return pil;
	}


	/**
	 * @return the ppl
	 */
	public String getPpl() {
		return ppl;
	}


	/**
	 * Returns the date and the time that the experiment was carried out. This is taken from the
	 * dateOfExperiment and inductionTime attributes.
	 *
	 * @return experiment date and time.
	 */
	public Date getExperimentDateTime() {
		return experimentDateTime;
	}


	/**
	 * Returns the optional licence value if one has been set, otherwise null.
	 *
	 * @return optionalLicence - usually 3a or 3b.
	 */
	public String getOptionalLicence() {
		return optionalLicence;
	}


	/**
	 * Returns true if an optional licence was included in the header.
	 *
	 * @return true if optional licence exists, otherwise false.
	 */
	public boolean hasOptionalLicence() {
		return optionalLicence != null ? true : false;
	}


	/**
	 * Returns true if the optional licence is 3b or 3B as this is deemed as a harmful mutant mouse
	 * by observation, so requires a different licence.
	 *
	 * @return true if optional licence is 3a, otherwise false.
	 */
	public boolean isHarmfulMutant() {
		if (!hasOptionalLicence())
			return false;
		if (optionalLicence.toLowerCase().equals(HARMFUL_PROTOCOL_CODE))
			return true;
		return false;
	}
}
