/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.abr.services;

import java.security.NoSuchAlgorithmException;
import java.util.List;

import uk.ac.sanger.mig.abr.ControlChart;
import uk.ac.sanger.mig.abr.FrequencyChart;
import uk.ac.sanger.mig.abr.MouseChart;
import uk.ac.sanger.mig.abr.parser.FileParser;
import uk.ac.sanger.mig.abr.parser.Frequency;
import uk.ac.sanger.mig.abr.parser.NotAnAbrTestBlockException;

/**
 * This service contains all methods generating the ABR diagrams for the purpose of storing or
 * enlarge the individual diagram(frequency diagram, calibration diagram and click diagram)
 *
 * @author wl1
 *
 */

public interface ChartService {

	/**
	 * @param fp-
	 *            the file parser containing all datapoints
	 * @return the offset used to space out at Y axis between the traces(waveforms)of the frequency,
	 *         so that the traces wont overlap on each other.
	 * @throws NumberFormatException
	 * @throws NotAnAbrTestBlockException
	 */
	public double caculateFinalOffsetFromParser(FileParser fp) throws NumberFormatException, NotAnAbrTestBlockException;


	/**
	 * @param f-
	 *            the frequency object which contains all datapoints of its traces.
	 * @param recordY-
	 *            if the SPL level legend is required
	 * @param offset-
	 *            the Y axis offset between each traces
	 * @return the frequency chart
	 * @throws NumberFormatException
	 * @throws NotAnAbrTestBlockException
	 * @throws NoSuchAlgorithmException
	 */
	public FrequencyChart caculateFreqChartFromParser(Frequency f, boolean recordY, double offset)
			throws NumberFormatException, NotAnAbrTestBlockException, NoSuchAlgorithmException;


	/**
	 * @param frequencyBlocks-
	 *            the list of the frequency objects.Each frequency object contains all datapoints of
	 *            its traces.
	 * @param offset-
	 *            the Y axis offset between the traces
	 * @return a list of freqency chart
	 * @throws NumberFormatException
	 * @throws NoSuchAlgorithmException
	 * @throws NotAnAbrTestBlockException
	 */
	public List<FrequencyChart> caculateFreqChartListFromParser(List<Frequency> frequencyBlocks, double offset)
			throws NumberFormatException, NoSuchAlgorithmException, NotAnAbrTestBlockException;


	/**
	 * @param fp-
	 *            the file parser
	 * @return the control chart(can generate both calibration and control chart)
	 */
	public ControlChart caculateOtherChartFromParser(FileParser fp);


	/**
	 * This method is called when used double click on the frequency chart
	 *
	 * @param fp-
	 *            the file parser
	 * @param freqIndex:
	 *            the index in the list of the frequecy objects in the file parser
	 * @return The enlarged Frequency Chart
	 * @throws NumberFormatException
	 * @throws NotAnAbrTestBlockException
	 * @throws NoSuchAlgorithmException
	 */
	public FrequencyChart enlargeFreqChartFromParser(FileParser fp, int freqIndex) throws NumberFormatException,
			NotAnAbrTestBlockException, NoSuchAlgorithmException;


	/**
	 * @param fp-
	 *            the file parer
	 * @return the MouseChart object which can used to store the mouse chart, calibration chart and
	 *         click chart in the database
	 * @throws NumberFormatException
	 * @throws NotAnAbrTestBlockException
	 * @throws NoSuchAlgorithmException
	 */
	public MouseChart createMouseChart(FileParser fp) throws NumberFormatException, NotAnAbrTestBlockException,
			NoSuchAlgorithmException;


	/**
	 * @param fp-
	 *            the file parer
	 * @return the byte[] of the mouse multichart
	 * @throws NumberFormatException
	 * @throws NotAnAbrTestBlockException
	 * @throws NoSuchAlgorithmException
	 */
	public byte[] createDefualtMouseChart(FileParser fp) throws NumberFormatException, NotAnAbrTestBlockException, NoSuchAlgorithmException;

}
