/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.abr.services;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.util.Collection;

import uk.ac.sanger.mig.abr.AbrCohortSetListViewSearchBean;
import uk.ac.sanger.mig.abr.FreqExperiment;
import uk.ac.sanger.mig.abr.MouseAbrListViewSearchBean;
import uk.ac.sanger.mig.abr.forms.AbrFileUploadForm;
import uk.ac.sanger.mig.abr.forms.AbrMouseForm;
import uk.ac.sanger.mig.abr.parser.AttributeFormatException;
import uk.ac.sanger.mig.abr.parser.BlockAttributeAssociationException;
import uk.ac.sanger.mig.abr.parser.EndClickBlockMissingException;
import uk.ac.sanger.mig.abr.parser.HeaderBlockMissingException;
import uk.ac.sanger.mig.abr.parser.StartClickBlockMissingException;
import uk.ac.sanger.mig.abr.parser.FileParser;
import uk.ac.sanger.mig.abr.parser.NotAnAbrTestBlockException;
import uk.ac.sanger.mig.abr.views.AbrCohortSetListView;
import uk.ac.sanger.mig.abr.views.MouseAbrListView;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.reporting.ChartDataSearchBean;
import uk.ac.sanger.mig.reporting.ChartDisplay;

/**
 * @author wl1
 *
 */
public interface MouseAbrService {

	public static final Long	EXPERIMENT_FILE_CONTENT_TYPE_ID		= 1L;
	public static final Long	MULTIPLE_CHART_CONTENT_TYPE_ID		= 2L;
	public static final Long	CLICK_CHART_CONTENT_TYPE_ID			= 3L;
	public static final Long	CALIBRATION_CHART_CONTENT_TYPE_ID	= 4L;
	public static final Long	ABR_MOUSE_PROCEDURE_DICT_ID			= 25L;

	public static final Long ABR_SUBGROUP_ID = 14L;
	public static final Long ABR_GRAPH_METADATA_ID = 273L;

	public enum ObservationMapping {
		USER_COMMENTS, USER_INCLUDE;
	}

	/**
	 * Filtering the results set by the search cirtira defined by the users
	 *
	 * @param bean:
	 *            the search bean contains the search citerias
	 * @return the collection of the filtered ABR experiments
	 */
	public Collection<MouseAbrListView> search(MouseAbrListViewSearchBean bean);

	public Collection<AbrCohortSetListView> search(AbrCohortSetListViewSearchBean bean);


	/**
	 * Finds and returns an ABR mouse using the passed experiment id. An experiment is 1:1 with a
	 * mouse.
	 *
	 * @param expId -
	 *            the experiment id to search for.
	 * @return MouseAbrListView - the ABR mouse details or null if nothing is found.
	 */
	public MouseAbrListView fetchAbrMouseByExpId(Long expId);


	/**
	 * Returns all ABR experiments.
	 *
	 * @param requiredComponents -
	 *            to attach, FreqExperiment.Components.EXPERIMENT for instance.
	 * @return collection containing all ABR experiments.
	 */
	public Collection<Experiment> fetchAllAbrExperiments(ComponentMap requiredComponents);


	/**
	 * Stores the experiment, its file and charts along with the experiments frequencies to the
	 * database.
	 *
	 * @param requiredObservations -
	 *            the collection of required observations to create observations with.
	 * @param binaryFile -
	 *            the uploaded ABR results file.
	 * @param fileParser -
	 *            the fileParser that has already passed the ABR file above.
	 * @param user -
	 *            the person that is storing this experiment and charts.
	 */
	public Experiment storeExperimentFileAndCharts(AbrFileUploadForm abrUploadForm, Person user)
			throws NoSuchAlgorithmException, NotAnAbrTestBlockException;


	/**
	 * Fetches the ABR test file from the APP_BINARY_FILES table using the passed experiment id. The
	 * experiment id is stored in the APP_BINARY_FILES.foreign_key_id.
	 *
	 * @param experimentId -
	 *            the experiment id to find the ABR file.
	 * @return FileParser that has parsed the file if one was found, otherwise null.
	 * @throws IOException -
	 *             IO error.
	 * @throws BlockAttributeAssociationException -
	 *             file parsing failed due to a file formatting error.
	 * @throws ParseException -
	 *             thrown when parsing dates fails.
	 * @throws AttributeFormatException -
	 *             one or more block attributes in the ABR file are incorrectly formatted.
	 * @throws StartClickBlockMissingException -
	 *             a start or end click block has not been found.
	 * @throws EndClickBlockMissingException
	 * @throws HeaderBlockMissingException
	 */
	public FileParser fetchAbrFileByExperimentId(Long experimentId) throws IOException,
			BlockAttributeAssociationException, ParseException, AttributeFormatException,
			StartClickBlockMissingException, EndClickBlockMissingException, HeaderBlockMissingException;


	/**
	 * Fetches the ABR test file from the APP_BINARY_FILES table using the passed experiment id. The
	 * experiment id is stored in the APP_BINARY_FILES.foreign_key_id.
	 *
	 * @param experimentId -
	 *            the experiment id to find the ABR file.
	 * @param includeBinaryData -
	 *            includes the binary data associated with the file if true.
	 * @return binary file if the binary file exist, otherwise return null
	 * @throws IOException -
	 *             IO error.
	 * @throws BlockAttributeAssociationException -
	 *             file parsing failed due to a file formatting error.
	 * @throws ParseException -
	 *             thrown when parsing dates fails.
	 * @throws AttributeFormatException -
	 *             one or more block attributes in the ABR file are incorrectly formatted.
	 */
	public BinaryFile fetchAbrFileByExperimentId(Long experimentId, boolean includeBinaryData);


	/**
	 * Finds and returns the ABR image of the passed type associated with the passed experiment id
	 * as a binary file.
	 *
	 * @param experimentId -
	 *            the experiment id to search for.
	 * @parma imageTypeDictId - the type of the image to search for (one of the
	 *        XXX_CHART_CONTENT_TYPE_ID static vars declared in the interface).
	 * @return most recent uploaded binaryFile if the result set is not null, otherwise throw
	 *         exception
	 */
	public BinaryFile fetchAbrImageByExperimentId(Long experimentId, Long imageTypeDictId);


	/**
	 * Attempts to find a mouse using the passed identifier as the mouse name and then as the mouse
	 * barcode if searching by name returns nothing.
	 *
	 * @param identifier -
	 *            the string identifier, could be a mouse name or barcode.
	 * @param requiredComponents -
	 *            the components to attach to the returned mouse.
	 * @return mouse - the mouse or null if one isn't found.
	 */
	public Mouse fetchMouseByNameOrBarcode(String identifier, ComponentMap requiredComponents);


	/**
	 * Returns true if the passed ABR file exists in the APP_BINARY_FILES table, otherwise returns
	 * false.
	 *
	 * @param file -
	 *            the ABR experiment file.
	 * @return true - the file exists. false - the file does not exist in the database.
	 */
	public boolean doesAbrFileExist(BinaryFile file);


	/**
	 * Storing the thresholds, user comments and include observation when user submits the form on
	 * mouse ABR analysis page
	 *
	 * @param frequencyExperiments
	 * @param person
	 */
	public Long storeAnalysisData(AbrMouseForm abrMouseForm, Person person);


	/**
	 * When the user submit the form. Updating the thresholds in aud_freq_experiment as the
	 * FreqExperiments are alreay created when uploading the file.
	 *
	 * @param frequencyExperiments
	 * @param person
	 */
	public void storeFreqExperiments(Collection<FreqExperiment> frequencyExperiments, Person person);


	/**
	 * Storing the include observation of ABR in phn_observation.
	 *
	 * If the observation is already exist, update it with the new observation value otherwise
	 * create a new observation and store it.
	 *
	 * @param experimentId
	 * @param shouldIncludeExperiment:
	 *            true or false. true means the data are good and the experiment is included.
	 *            Otherwise exclude this experiment for the final analysis.
	 * @param person
	 */
	public void storeIncludeExperiment(Long experimentId, boolean shouldIncludeExperiment, Person person);


	/**
	 * Storing the user comments observation(on ABR Mouse Analyse Page) of ABR in the
	 * phn_observation.
	 *
	 * If the observation is already exist, update it with the new observation value otherwise
	 * create a new observation and store it
	 *
	 * @param experimentId:
	 *            the phn_experiment_id
	 * @param comments
	 * @param person
	 */
	public void storeComments(Long experimentId, String comments, Person person);


	/**
	 * Returns true if the personal licence and project licence match to allow the user to complete
	 * the passed procedure.
	 *
	 * @param pil -
	 *            the personal licence.
	 * @param ppl -
	 *            the project licence.
	 * @param sop -
	 *            the standard operating procedure.
	 * @return true - if the person can do this procedure and there are protocols associated with
	 *         it, otherwise false.
	 */
	public boolean isLicenced(PersonalLicence pil, ProjectLicence ppl, StdOperatingProcedure sop);

	public double[][]  fetchGraphAges(Long alleleId, Long pipelineId);

	public ChartDisplay fetchAbrChartData(ChartDataSearchBean bean);

}