/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.abr.services;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.abr.AbrCohortSetListViewSearchBean;
import uk.ac.sanger.mig.abr.FreqExperiment;
import uk.ac.sanger.mig.abr.MouseAbrListViewSearchBean;
import uk.ac.sanger.mig.abr.MouseChart;
import uk.ac.sanger.mig.abr.dao.AbrCohortSetViewDao;
import uk.ac.sanger.mig.abr.dao.MouseAbrViewDao;
import uk.ac.sanger.mig.abr.forms.AbrFileUploadForm;
import uk.ac.sanger.mig.abr.forms.AbrMouseForm;
import uk.ac.sanger.mig.abr.parser.AttributeFormatException;
import uk.ac.sanger.mig.abr.parser.BlockAttributeAssociationException;
import uk.ac.sanger.mig.abr.parser.EndClickBlockMissingException;
import uk.ac.sanger.mig.abr.parser.FileParser;
import uk.ac.sanger.mig.abr.parser.Frequency;
import uk.ac.sanger.mig.abr.parser.HeaderBlock;
import uk.ac.sanger.mig.abr.parser.HeaderBlockMissingException;
import uk.ac.sanger.mig.abr.parser.NotAnAbrTestBlockException;
import uk.ac.sanger.mig.abr.parser.StartClickBlockMissingException;
import uk.ac.sanger.mig.abr.views.AbrCohortSetListView;
import uk.ac.sanger.mig.abr.views.MouseAbrListView;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.UploadMapping;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ObservationService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.UploadMappingService;
import uk.ac.sanger.mig.reporting.ChartDataSearchBean;
import uk.ac.sanger.mig.reporting.ChartDisplay;

public class MouseAbrServiceImpl implements MouseAbrService {

	private MouseAbrViewDao			abrViewDao;
	private AbrCohortSetViewDao 	abrCohortSetDao;
	private BinaryFileService		binaryFileService;
	private ExperimentService		experimentService;
	private ChartService			chartService;
	private MouseService			mouseService;
	private MouseProcedureService	mouseProcedureService;
	private FreqExperimentService	freqExperimentService;
	private ObservationService		observationService;
	private ProjectLicenceService	projectLicenceService;
	private PersonalLicenceService	personalLicenceService;
	private UploadMappingService	uploadMappingService;



	/**
	 * Stores the experiment, its file and charts along with the experiments frequencies to the
	 * database.
	 *
	 * @param requiredObservations -
	 *            the collection of required observations to create observations with.
	 * @param binaryFile -
	 *            the uploaded ABR results file.
	 * @param fileParser -
	 *            the fileParser that has already passed the ABR file above.
	 * @param user -
	 *            the person that is storing this experiment and charts.
	 */
	@Transactional
	public Experiment storeExperimentFileAndCharts(AbrFileUploadForm abrUploadForm, Person user)
			throws NoSuchAlgorithmException, NotAnAbrTestBlockException {
		FileParser fileParser = abrUploadForm.getFileParser();
		HeaderBlock headerBlock = fileParser.getHeaderBlock();
		MouseChart mouseChart = chartService.createMouseChart(fileParser);
		Mouse mouse = fetchMouseByNameOrBarcode(headerBlock.getMouseId(), ComponentMap.of(Mouse.Component.GENOTYPE));
		ProjectLicence projectLicence = projectLicenceService.fetchByLicenceNumber(headerBlock.getPpl(), null);
		PersonalLicence personalLicence = personalLicenceService.fetchByPersonalLicenceNumber(headerBlock.getPil(),
				null);
		MouseProcedure mouseProcedure = createMouseProcedure(mouse, headerBlock.isHarmfulMutant(), headerBlock.getExperimentDateTime(),
				projectLicence, personalLicence, user);
		Experiment experiment = new Experiment(mouse.getId(), abrUploadForm.getSop());
		experiment.setMouse(mouse);
		experiment.setProcedure(mouseProcedure);
		storeExperimentWithResults(experiment, fileParser.getHeaderBlock(), abrUploadForm.getUploadMappings(), user);
		storeExperimentFrequencies(fileParser, experiment, user);
		storeExperimentFile(experiment, abrUploadForm.getBinaryFile(), user);
		storeClickChart(experiment, mouseChart, user);
		storeCalibrationChart(experiment, mouseChart, user);
		return experiment;
	}
	public double[][]  fetchGraphAges(Long alleleId, Long pipelineId){

		Collection<MouseAbrListView> list = abrViewDao.fetchAbrAges(alleleId,pipelineId);
		DescriptiveStatistics wtStats = new DescriptiveStatistics();
		DescriptiveStatistics hetStats = new DescriptiveStatistics();
		DescriptiveStatistics homStats = new DescriptiveStatistics();
		double[][] ages = new double[3][3];

		for(MouseAbrListView listView : list){
			if(listView.getGenotypeId() == GenotypeDictionary.WILDTYPE){
				wtStats.addValue(listView.getAgeInWeeks());
			}else if(listView.getGenotypeId() == GenotypeDictionary.HETEROZYGOUS){
				hetStats.addValue(listView.getAgeInWeeks());
			}else if(listView.getGenotypeId() == GenotypeDictionary.HOMOZYGOUS){
				homStats.addValue(listView.getAgeInWeeks());
			}
		}

		if(wtStats.getN()>0){
			ages[0][0] = wtStats.getMean();
			ages[0][1] = wtStats.getStandardDeviation();
			ages[0][2] = wtStats.getStandardDeviation()/Math.sqrt(wtStats.getN());
		}
		if(hetStats.getN()>0){
			ages[1][0] = hetStats.getMean();
			ages[1][1] = hetStats.getStandardDeviation();
			ages[1][2] = hetStats.getStandardDeviation()/Math.sqrt(hetStats.getN());
		}
		if(homStats.getN()>0){
			ages[2][0] = homStats.getMean();
			ages[2][1] = homStats.getStandardDeviation();
			ages[2][2] = homStats.getStandardDeviation()/Math.sqrt(homStats.getN());
		}

		return ages;
	}

	/**
	 * Returns true if the personal licence and project licence match to allow the user to complete
	 * the passed procedure.
	 *
	 * @param pil -
	 *            the personal licence.
	 * @param ppl -
	 *            the project licence.
	 * @param sop -
	 *            the standard operating procedure.
	 * @return true - if the person can do this procedure and there are protocols associated with
	 *         it, otherwise false.
	 */
	public boolean isLicenced(PersonalLicence pil, ProjectLicence ppl, StdOperatingProcedure sop) {
		return personalLicenceService.isLicenced(pil, ppl, sop);
	}


	/**
	 * Finds and returns an ABR mouse using the passed experiment id. An experiment is 1:1 with a
	 * mouse.
	 *
	 * @param expId -
	 *            the experiment id to search for.
	 * @return MouseAbrListView - the ABR mouse details or null if nothing is found.
	 */
	public MouseAbrListView fetchAbrMouseByExpId(Long expId) {
		return abrViewDao.fetchById(expId);
	}


	/**
	 * Fetches the ABR test file from the APP_BINARY_FILES table using the passed experiment id. The
	 * experiment id is stored in the APP_BINARY_FILES.foreign_key_id.
	 *
	 * @param experimentId -
	 *            the experiment id to find the ABR file.
	 * @return FileParser that has parsed the file if one was found, otherwise null.
	 * @throws IOException -
	 *             IO error.
	 * @throws BlockAttributeAssociationException -
	 *             file parsing failed due to a file formatting error.
	 * @throws ParseException -
	 *             thrown when parsing dates fails.
	 * @throws AttributeFormatException -
	 *             one or more block attributes in the ABR file are incorrectly formatted.
	 */
	public FileParser fetchAbrFileByExperimentId(Long experimentId) throws IOException,
			BlockAttributeAssociationException, ParseException, AttributeFormatException,
			StartClickBlockMissingException, EndClickBlockMissingException, HeaderBlockMissingException {
		BinaryFile binaryFile = this.fetchAbrFileByExperimentId(experimentId, true);
		FileParser fp = null;
		if (binaryFile != null) {
			fp = new FileParser();
			fp.parse(new InputStreamReader(new ByteArrayInputStream(binaryFile.getBytes())));
		}
		return fp;
	}


	/**
	 * Fetches the ABR test file from the APP_BINARY_FILES table using the passed experiment id. The
	 * experiment id is stored in the APP_BINARY_FILES.foreign_key_id.
	 *
	 * @param experimentId -
	 *            the experiment id to find the ABR file.
	 * @param includeBinaryData -
	 *            includes the binary data associated with the file if true.
	 * @return binary file if the binary file exist, otherwise return null
	 * @throws IOException -
	 *             IO error.
	 * @throws BlockAttributeAssociationException -
	 *             file parsing failed due to a file formatting error.
	 * @throws ParseException -
	 *             thrown when parsing dates fails.
	 * @throws AttributeFormatException -
	 *             one or more block attributes in the ABR file are incorrectly formatted.
	 */
	public BinaryFile fetchAbrFileByExperimentId(Long experimentId, boolean includeBinaryData) {
		BinaryFile binaryFile = null;
		List<BinaryFile> binaryFiles = new ArrayList<BinaryFile>(binaryFileService.fetchByTableRef(
				TableMetadataEnum.PHN_EXPERIMENT, experimentId, EXPERIMENT_FILE_CONTENT_TYPE_ID, includeBinaryData,
				null));
		if (!binaryFiles.isEmpty()) {
			binaryFile = binaryFiles.get(binaryFiles.size() - 1);
		}
		return binaryFile;
	}


	/**
	 * Returns true if the passed ABR file exists in the APP_BINARY_FILES table, otherwise returns
	 * false.
	 *
	 * @param file -
	 *            the ABR experiment file.
	 * @return true - the file exists. false - the file does not exist in the database.
	 */
	public boolean doesAbrFileExist(BinaryFile file) {
		Collection<BinaryFile> binaryFiles = binaryFileService.fetchByFilenameAndContentTypeDictId(file.getFileName(),
				EXPERIMENT_FILE_CONTENT_TYPE_ID, false, null);
		return binaryFiles.isEmpty() ? false : true;
	}


	/**
	 * Finds and returns the ABR image of the passed type associated with the passed experiment id
	 * as a binary file.
	 *
	 * @param experimentId -
	 *            the experiment id to search for.
	 * @parma imageTypeDictId - the type of the image to search for (one of the
	 *        XXX_CHART_CONTENT_TYPE_ID static vars declared in the interface).
	 * @return most recent uploaded binaryFile if the result set is not null, otherwise throw
	 *         exception
	 */
	public BinaryFile fetchAbrImageByExperimentId(Long experimentId, Long imageTypeDictId) {
		List<BinaryFile> binaryFiles = new ArrayList<BinaryFile>(binaryFileService.fetchByTableRef(
				TableMetadataEnum.PHN_EXPERIMENT, experimentId, imageTypeDictId, true, null));

		if (!binaryFiles.isEmpty()) {
			return binaryFiles.get(binaryFiles.size() - 1);
		} else {
			return null;
		}

	}


	/**
	 * Attempts to find a mouse using the passed identifier as the mouse name and then as the mouse
	 * barcode if searching by name returns nothing.
	 *
	 * @param identifier -
	 *            the string identifier, could be a mouse name or barcode.
	 * @param requiredComponents -
	 *            the components to attach to the returned mouse.
	 * @return mouse - the mouse or null if one isn't found.
	 */
	public Mouse fetchMouseByNameOrBarcode(String identifier, ComponentMap requiredComponents) {
		Mouse mouse = null;
		mouse = mouseService.fetchByName(identifier, requiredComponents);
		if (mouse == null) {
			mouse = mouseService.fetchByBarcode(identifier, requiredComponents);
		}
		return mouse;
	}


	/**
	 * Wrapper that returns all ABR experiments.
	 *
	 * @param requiredComponents -
	 *            components to attach.
	 * @return collection containing all ABR experiments.
	 */
	public Collection<Experiment> fetchAllAbrExperiments(ComponentMap requiredComponents) {
		return experimentService.fetchAllAbrExperiments(requiredComponents);
	}


	/**
	 * Storing the thresholds, user comments and include observation when user submits the form on
	 * mouse ABR analysis page
	 *
	 * @param frequencyExperiments
	 * @param person
	 */
	@Transactional
	public Long storeAnalysisData(AbrMouseForm abrMouseForm, Person person) {
		storeFreqExperiments(abrMouseForm.getFreqExperiments(), person);
		storeIncludeExperiment(abrMouseForm.getMouseAbrListView().getExperimentId(), abrMouseForm.getInclude(), person);
		storeComments(abrMouseForm.getMouseAbrListView().getExperimentId(), abrMouseForm.getUserComments(), person);

		return abrMouseForm.getMouseAbrListView().getExperimentId();
	}


	/**
	 * When the user submit the form. Updating the thresholds in aud_freq_experiment as the
	 * FreqExperiments are alreay created when uploading the file.
	 *
	 * @param frequencyExperiments
	 * @param person
	 */
	@Transactional
	public void storeFreqExperiments(Collection<FreqExperiment> frequencyExperiments, Person person) {
		for (FreqExperiment freqExperiment : frequencyExperiments) {
			freqExperimentService.store(freqExperiment, person);
		}
	}


	/**
	 * Storing the include observation of ABR in phn_observation.
	 *
	 * If the observation is already exist, update it with the new observation value otherwise
	 * create a new observation and store it.
	 *
	 * @param experimentId
	 * @param shouldIncludeExperiment:
	 *            true or false. true means the data are good and the experiment is included.
	 *            Otherwise exclude this experiment for the final analysis.
	 * @param person
	 */
	public void storeIncludeExperiment(Long experimentId, boolean shouldIncludeExperiment, Person person) {
		StdOperatingProcedure sop = uploadMappingService.fetchActiveSopForUploadType(
				UploadMappingService.Type.ABR_ANALYSIS, ComponentMap
						.of(StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS));
		Collection<UploadMapping> mappings = uploadMappingService.fetchBySopId(sop.getId());
		Long includeObervateionId = null;

		for (UploadMapping uploadMapping : mappings) {
			if (uploadMapping.getFieldName().equals(MouseAbrService.ObservationMapping.USER_INCLUDE.toString())) {
				includeObervateionId = uploadMapping.getRequiredObservationId();
			}
		}

		Observation includeExperimentObservation = observationService.fetchByExperimentIdAndRequiredObservationId(
				experimentId, includeObervateionId);
		if (includeExperimentObservation == null) {
			includeExperimentObservation = new Observation();
			includeExperimentObservation.setExperimentId(experimentId);
			includeExperimentObservation.setRequiredObservationId(includeObervateionId);

		}
		includeExperimentObservation.setValue(shouldIncludeExperiment ? "1" : "0");
		observationService.storeAuthenticated(includeExperimentObservation, person);
	}


	/**
	 * Storing the comments observation of ABR in the phn_observation.
	 *
	 * If the observation is already exist, update it with the new observation value otherwise
	 * create a new observation and store it
	 *
	 * @param experimentId:
	 *            the phn_experiment_id
	 * @param comments
	 * @param person
	 */
	public void storeComments(Long experimentId, String comments, Person person) {

		StdOperatingProcedure sop = uploadMappingService.fetchActiveSopForUploadType(
				UploadMappingService.Type.ABR_ANALYSIS, null);
		Collection<UploadMapping> mappings = uploadMappingService.fetchBySopId(sop.getId());
		Long commentsObervateionId = null;

		for (UploadMapping uploadMapping : mappings) {
			if (uploadMapping.getFieldName().equals(MouseAbrService.ObservationMapping.USER_COMMENTS.toString())) {
				commentsObervateionId = uploadMapping.getRequiredObservationId();
			}
		}

		Observation userComments = observationService.fetchByExperimentIdAndRequiredObservationId(experimentId,
				commentsObervateionId);
		if (userComments == null) {
			userComments = new Observation();
			userComments.setExperimentId(experimentId);
			userComments.setRequiredObservationId(commentsObervateionId);
		}
		userComments.setValue(comments);
		observationService.storeAuthenticated(userComments, person);
	}

	public ChartDisplay fetchAbrChartData(ChartDataSearchBean bean){
		return abrViewDao.fetchAbrChartData(bean);
	}

	// private and protected methods

	/**
	 * Creates a new ABR mouse procedure for the passed mouse and under the passed project licence.
	 *
	 * @param mouse -
	 *            the mouse that the procedure has been done on.
	 * @param harmfulMutant -
	 *            true if harmful mutant.
	 * @param projectLicence -
	 *            the project licence to create the procedure under.
	 * @param user -
	 *            the user creating the procedure.
	 * @return mouseProcedure - the newly created mouse procedure.
	 */
	private MouseProcedure createMouseProcedure(Mouse mouse, boolean harmfulMutant, Date startDate,
			ProjectLicence projectLicence, PersonalLicence personalLicence, Person user) {
		MouseProcedure mouseProcedure = mouseProcedureService.createMouseProcedure(ABR_MOUSE_PROCEDURE_DICT_ID,
				projectLicence.getId(), harmfulMutant, user.getId());
		mouseProcedure.setMouseId(mouse.getId());
		mouseProcedure.setStartDate(startDate);
		mouseProcedure.setPersonalLicenceId(personalLicence.getId());
		return mouseProcedure;
	}


	/**
	 * Uses the mappings required observation id to find the correct observation and return it.
	 *
	 * @param observations -
	 *            collection of observations to search through.
	 * @param um -
	 *            the upload mapping containing the required observation id mapping.
	 * @return requiredObservation - the matching observation or null if not found.
	 */
	private Observation findObservationFromMapping(Collection<Observation> observations, UploadMapping um) {
		Observation requiredObservation = null;
		for (Observation o : observations) {
			if (o.getRequiredObservationId().equals(um.getRequiredObservationId()))
				requiredObservation = o;
		}
		return requiredObservation;
	}


	/**
	 * Links the passed experiment with its mouse procedure by setting the mouse procedures foreign
	 * table to the PHN_EXPERIMENT table and the foreign key to the id of the experiment.
	 *
	 * @param experiment -
	 *            the experiment with the mouse procedure set in to it.
	 */
	private void linkExperimentToProcedure(Experiment experiment) {
		MouseProcedure mouseProcedure = experiment.getProcedure();
		if (mouseProcedure.getForeignTableName() == null) {
			mouseProcedure.setForeignTableName(TableMetadataEnum.PHN_EXPERIMENT);
			mouseProcedure.setForeignKeyId(experiment.getId());
		}
	}


	/**
	 * Creates new observations for each of the required observations and sets their values using
	 * the data held in the ABR file header block. Stores the experiment along with its result
	 * observations. Links the procedure to the experiment by setting the procedures foreign table
	 * name and key to PHN_EXPERIMENT and the experiments new id.
	 *
	 * @param experiment -
	 *            the experiment.
	 * @param headerBlock -
	 *            the ABR header block.
	 * @param requiredObservations -
	 *            collection of required observations to create observations from.
	 * @param user -
	 *            the user storing the experiment and procedure.
	 */
	protected void storeExperimentWithResults(Experiment experiment, HeaderBlock headerBlock,
			Collection<UploadMapping> uploadMappings, Person user) {
		setObservationValues(experiment, headerBlock, uploadMappings);
		experimentService.storeWithResults(experiment, user);
		linkExperimentToProcedure(experiment);
		mouseProcedureService.store(experiment.getProcedure(), user);
	}


	/**
	 * Creates a new entry in the AUD_FREQ_EXPERIMENT table for each of this experiments frequency
	 * levels so that the thresholds can be set for each of them.
	 *
	 * @param fileParser -
	 *            the parsed file.
	 * @param experiment -
	 *            the experiment to link to, that has already been stored.
	 * @param user -
	 *            the user storing the experiment frequencies.
	 */
	protected void storeExperimentFrequencies(FileParser fileParser, Experiment experiment, Person user) {
		Collection<Frequency> frequencies = fileParser.getAbrTraceBlocksByFrequencyLevel();
		for (Frequency frequency : frequencies) {
			int frequencyLevel = frequency.getFrequency();
			FreqExperiment freqExperiment = new FreqExperiment();
			freqExperiment.setPhnExperimentId(experiment.getId());
			freqExperiment.setFrequence(frequencyLevel);
			freqExperimentService.store(freqExperiment, user);
		}
	}


	/**
	 * Sets up the ABR experiment BinaryFile object with type and links to the experiment in the
	 * EXPERIMENT table and stores it in the APP_BINARY_FILES table.
	 *
	 * @param experiment -
	 *            the experiment.
	 * @param binaryFile -
	 *            the ABR binary file.
	 * @param user -
	 *            the user storing the file.
	 */
	protected void storeExperimentFile(Experiment experiment, BinaryFile binaryFile, Person user) {
		binaryFile.setContentTypeDictId(EXPERIMENT_FILE_CONTENT_TYPE_ID);
		binaryFile.setForeignTableName(TableMetadataEnum.PHN_EXPERIMENT);
		binaryFile.setForeignKeyId(experiment.getId());
		binaryFileService.store(binaryFile, user);
	}


	/**
	 * Creates and stores the click chart into the APP_BINARY_FILES table and sets the foreign table
	 * and key to point to the experiment table.
	 *
	 * @param experiment -
	 *            the experiment.
	 * @param mouseChart -
	 *            the mouse chart used to create the charts.
	 * @param user -
	 *            the user creating the charts.
	 * @throws NoSuchAlgorithmException -
	 *             thrown by chart director.
	 */
	protected void storeClickChart(Experiment experiment, MouseChart mouseChart, Person user)
			throws NoSuchAlgorithmException {
		BinaryFile binaryFile = new BinaryFile();
		binaryFile.setBytes(mouseChart.createClickChart());
		binaryFile.setFileDescription("ABR Mouse click chart");
		binaryFile.setContentType("image/jpeg");
		binaryFile.setContentTypeDictId(CLICK_CHART_CONTENT_TYPE_ID);
		binaryFile.setForeignTableName(TableMetadataEnum.PHN_EXPERIMENT);
		binaryFile.setForeignKeyId(experiment.getId());
		binaryFileService.store(binaryFile, user);
	}


	/**
	 * Creates and stores the calibration chart into the APP_BINARY_FILES table and sets the foreign
	 * table and key to point to the experiment table.
	 *
	 * @param experiment -
	 *            the experiment.
	 * @param mouseChart -
	 *            the mouse chart used to create the charts.
	 * @param user -
	 *            the user creating the charts.
	 * @throws NoSuchAlgorithmException -
	 *             exception thrown from chart director.
	 */
	protected void storeCalibrationChart(Experiment experiment, MouseChart mouseChart, Person user)
			throws NoSuchAlgorithmException {
		BinaryFile binaryFile = new BinaryFile();
		binaryFile.setBytes(mouseChart.createCalibrationChart());
		binaryFile.setFileDescription("ABR Mouse calibration chart");
		binaryFile.setContentType("image/jpeg");
		binaryFile.setContentTypeDictId(CALIBRATION_CHART_CONTENT_TYPE_ID);
		binaryFile.setForeignTableName(TableMetadataEnum.PHN_EXPERIMENT);
		binaryFile.setForeignKeyId(experiment.getId());
		binaryFileService.store(binaryFile, user);
	}


	/**
	 * Populates the mandatory observations with the required observation and the value from the ABR
	 * file header block. Sets the newly created observations into the experiment.
	 *
	 * @param experiment -
	 *            the experiment.
	 * @param headerBlock -
	 *            the ABR file header block containing observation values.
	 * @param experimentComments -
	 *            the comments held within the experiment file.
	 * @param requiredObservations -
	 *            the required observations.
	 */
	private void setObservationValues(Experiment experiment, HeaderBlock headerBlock,
			Collection<UploadMapping> uploadMappings) {
		for (UploadMapping um : uploadMappings) {
			Observation observation = findObservationFromMapping(experiment.getObservations(), um);
			observation.setValue(headerBlock.getAttributeValue(um.getFieldName()));
		}
	}


	// the setter and getter
	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}


	public void setExperimentService(ExperimentService experimentService) {
		this.experimentService = experimentService;
	}


	public void setChartService(ChartService chartService) {
		this.chartService = chartService;
	}


	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}


	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}


	public void setAbrViewDao(MouseAbrViewDao abrViewDao) {
		this.abrViewDao = abrViewDao;
	}


	public Collection<MouseAbrListView> search(MouseAbrListViewSearchBean bean) {
		return abrViewDao.search(bean);
	}


	public Collection<AbrCohortSetListView> search(AbrCohortSetListViewSearchBean bean) {
		return abrCohortSetDao.search(bean);
	}


	public void setFreqExperimentService(FreqExperimentService freqExperimentService) {
		this.freqExperimentService = freqExperimentService;
	}


	public void setObservationService(ObservationService observationService) {
		this.observationService = observationService;
	}


	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}


	public void setPersonalLicenceService(PersonalLicenceService personalLicenceService) {
		this.personalLicenceService = personalLicenceService;
	}


	public void setUploadMappingService(UploadMappingService uploadMappingService) {
		this.uploadMappingService = uploadMappingService;
	}


	public void setAbrCohortSetDao(AbrCohortSetViewDao abrCohortSetDao) {
		this.abrCohortSetDao = abrCohortSetDao;
	}
}
