package uk.ac.sanger.mig.abr.validators;

import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Map;

import org.springframework.validation.Errors;

import uk.ac.sanger.mig.abr.forms.AbrFileUploadForm;
import uk.ac.sanger.mig.abr.parser.Block;
import uk.ac.sanger.mig.abr.parser.BlockAttribute;
import uk.ac.sanger.mig.abr.parser.FileParser;
import uk.ac.sanger.mig.abr.parser.HeaderBlock;
import uk.ac.sanger.mig.abr.services.MouseAbrService;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.validators.FileUploadFormValidator;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.UploadMapping;

public class EditAbrFileValidator extends FileUploadFormValidator {

	private MouseAbrService			mouseAbrService;
	private PersonalLicenceService	personalLicenceService;
	private ProjectLicenceService	projectLicenceService;


	public void setMouseAbrService(MouseAbrService mouseAbrService) {
		this.mouseAbrService = mouseAbrService;
	}


	public PersonalLicenceService getPersonalLicenceService() {
		return personalLicenceService;
	}


	public void setPersonalLicenceService(PersonalLicenceService personalLicenceService) {
		this.personalLicenceService = personalLicenceService;
	}


	public ProjectLicenceService getProjectLicenceService() {
		return projectLicenceService;
	}


	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}


	/**
	 * Validates that the binary file is the correct type, the mouse barcode is present and that
	 * there is a 19b ABR licenced procedure associated with it and that the required parts of the
	 * header are present.
	 *
	 * @param commandForm -
	 *            the AbrFileUpload form backing object.
	 * @param errors -
	 *            the errors object populated with the errors, if found.
	 */
	@Override
	public void validate(Object commandForm, Errors errors) {
		AbrFileUploadForm form = (AbrFileUploadForm) commandForm;
		form.setMessage(null);
		if (!isFileTypeValid(form.getBinaryFile())) {
			errors.reject(null, "ABR Experiment files should be .csv format");
		} else if (hasFileAlreadyBeenStored(form.getBinaryFile())) {
			errors.reject(null, "ABR file has already been stored - please iterate the file name and try again");
		} else {
			FileParser fileParser = new FileParser();
			try {
				fileParser.parse(new InputStreamReader(new ByteArrayInputStream(form.getBinaryFile().getBytes())));
				HeaderBlock headerBlock = fileParser.getHeaderBlock();
				if (headerBlock == null) {
					errors.reject(null, "no header block found in ABR file");
				} else {
					validateCalibrationBlocks(fileParser, errors);
					validateNumberOfBlocks(fileParser, errors);
					validateMouse(headerBlock, form, errors);
					validateHeaderBlock(headerBlock, errors);
					validateLicences(headerBlock, form.getSop(), errors);
					validateRequiredObservations(form, headerBlock, errors);
				}
			} catch (Exception e) {
				errors.reject(null, "failed to parse file: " + e.getMessage());
			}
			form.setFileParser(fileParser);
		}
		super.validate(commandForm, errors);
	}


	/**
	 * Returns true is the binary file is a .csv file, otherwise returns false.
	 *
	 * @param file -
	 *            the binary file to test.
	 * @return boolean - true if a .csv file, otherwise false.
	 */
	private boolean isFileTypeValid(BinaryFile file) {
		return file.getFileName().endsWith(".csv");
	}


	/**
	 * Returns true if the binary file has already been stored (this is only based on file name),
	 * otherwise returns false.
	 *
	 * @param file -
	 *            the ABR experiment file.
	 * @return true - the file has already been stored. false - the file has not been stored.
	 */
	private boolean hasFileAlreadyBeenStored(BinaryFile file) {
		boolean hasFileAlreadyBeenStored = mouseAbrService.doesAbrFileExist(file);
		return hasFileAlreadyBeenStored;
	}


	/**
	 * Validates that the mouse barcode is present in the ABR file that has been uploaded and that
	 * the mouse exists in the database.
	 *
	 * @param headerBlock -
	 *            the header block containing the mouse barcode.
	 * @param form -
	 *            the form.
	 * @param errors -
	 *            the errors object.
	 */
	private void validateMouse(HeaderBlock headerBlock, AbrFileUploadForm form, Errors errors) {
		String mouseBarcode = headerBlock.getMouseId();
		if (mouseBarcode == null) {
			errors.reject(null, "mouse barcode is not present in ABR file header");
		} else if (mouseAbrService.fetchMouseByNameOrBarcode(mouseBarcode, null) == null) {
			errors.reject(null, "Mouse '" + mouseBarcode + "' does not exist");
		}
	}


	/**
	 * Verifies that the PIL and PPL are present in the experiment file and they can be found in the
	 * database.
	 *
	 * @param headerBlock -
	 *            the experiment file header.
	 * @param errors -
	 *            object store for validation errors.
	 */
	private void validateLicences(HeaderBlock headerBlock, StdOperatingProcedure sop, Errors errors) {
		String pil = headerBlock.getPil();
		String ppl = headerBlock.getPpl();
		if (pil != null && ppl != null) {
			PersonalLicence personalLicence = personalLicenceService.fetchByPersonalLicenceNumber(pil, null);
			ProjectLicence projectLicence = projectLicenceService.fetchByLicenceNumber(ppl, null);
			if (personalLicence == null)
				errors.reject(null, "personal licence (PIL) can not be found");
			if (projectLicence == null)
				errors.reject(null, "project licence (PPL) can not be found");
			if (!errors.hasErrors()) {
				if (!mouseAbrService.isLicenced(personalLicence, projectLicence, sop))
					errors.reject(null, "user with PIL '" + pil + "' and PPL '" + ppl
							+ "' is not licenced for ABR procedures");
			}
		}
	}


	/**
	 * Validate the header block contains a PIL and a PPL licences.
	 *
	 * @param headerBlock -
	 *            the header block that should contain the PIL and PPL.
	 * @param errors -
	 *            the errors object populated with error messages if the PIL or PPL are missing.
	 */
	private void validateHeaderBlock(HeaderBlock headerBlock, Errors errors) {
		Map<String, Collection<BlockAttribute>> bas = headerBlock.getBlockAttributes();
		for (String attributeNameKey : bas.keySet()) {
			for (BlockAttribute ba : bas.get(attributeNameKey)) {
				if (ba.getAttributeValue() == null)
					errors.reject(null, ba.getAttributeName() + " attribute has no value");
			}
		}

		if (headerBlock.getPil() == null)
			errors.reject(null, "PIL attribute in ABR file header block has no value");
		if (headerBlock.getPpl() == null)
			errors.reject(null, "PPL attribute in ABR file header block has no value");
	}


	/**
	 * Validate the required observations exist and that they appear in the head block. They are
	 * required as they are made into observations in the MouseAbrService. Ignores the experiment
	 * comment observation as this is gathered and inserted later.
	 *
	 * @param form -
	 *            the form.
	 * @param headerBlock -
	 *            the header block that should contain the required observations.
	 * @param errors -
	 *            the errors object.
	 */
	private void validateRequiredObservations(AbrFileUploadForm form, HeaderBlock headerBlock, Errors errors) {
		for (UploadMapping um : form.getUploadMappings()) {
			if (headerBlock.getAttributeValue(um.getFieldName()) == null) {
				errors.reject(null, "required header attribute is required: " + um.getFieldName());
			}
		}
	}


	/**
	 * Rejects the submission if the parsed file contains no blocks with the default number of data
	 * points.
	 *
	 * @param fileParser -
	 *            the parsed file.
	 * @param errors -
	 *            errors container.
	 */
	private void validateNumberOfBlocks(FileParser fileParser, Errors errors) {
		Collection<Block> blocks = fileParser.getAbrTraceBlocks();
		if (blocks.isEmpty())
			errors.reject(null, "file contains no blocks with the default number of data points - "
					+ FileParser.DEFAULT_NUMBER_DATAPOINTS);
	}


	/**
	 * Rejects the submission if the parsed file contains no calibration blocks.
	 *
	 * @param fileParser -
	 *            the parsed file.
	 * @param errors -
	 *            errors container.
	 */
	private void validateCalibrationBlocks(FileParser fileParser, Errors errors) {
		Collection<Block> calibrationBlocks = fileParser.getCalibrationBlocks();
		if (calibrationBlocks.isEmpty())
			errors.reject(null, "no calibration blocks found in experiment file");
	}


	@Override
	public boolean supports(Class clazz) {
		return AbrFileUploadForm.class.equals(clazz);
	}

}
