/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.allelerequest.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.CancellableFormController;

import uk.ac.sanger.mig.allelerequest.AlleleRequest;
import uk.ac.sanger.mig.allelerequest.forms.AlleleRequestForm;
import uk.ac.sanger.mig.allelerequest.services.AlleleRequestService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryDao;
import uk.ac.sanger.mig.designrequest.DesignRequest;
import uk.ac.sanger.mig.designrequest.services.DesignRequestService;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.havanarequest.HavanaRequest;
import uk.ac.sanger.mig.havanarequest.services.HavanaRequestService;
import uk.ac.sanger.mig.personnel.Person;

public class AlleleRequestFormController extends CancellableFormController {

	private static final Long DEFAULT_FORM_STATUS = 1l;

	private static final Long VEGA_ANNOTATION_REQUESTED = 2l;

	private static final Long DEFAULT_TARGET_ENDPOINT = 2l;

	private static final Long DEFAULT_REQUEST_SOURCE = 3l;

	private static final Long DEFAULT_MOD_TYPE = 1l;

	private static final Long DESIGN_REQUESTED = 21l;

	private DictionaryDao dictionaryDao;

	private AlleleRequestService alleleRequestService;

	private HavanaRequestService havanaRequestService;

	private DesignRequestService designRequestService;

	public void setDictionaryDao(DictionaryDao dictionaryDao) {
		this.dictionaryDao = dictionaryDao;
	}

	public void setAlleleRequestService(AlleleRequestService alleleRequestService) {
		this.alleleRequestService = alleleRequestService;
	}

	public void setHavanaRequestservice(HavanaRequestService havanaRequestService) {
		this.havanaRequestService = havanaRequestService;
	}

	public void setHavanaRequestService(HavanaRequestService havanaRequestService) {
		this.havanaRequestService = havanaRequestService;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException arg3) throws Exception {	
		AlleleRequestForm form = (AlleleRequestForm) command;
		AlleleRequest alleleRequest = form.getAlleleRequest();
		boolean isNewRequest = (alleleRequest.getId() == null);    

		// Set the form submitter as the requestor for new forms
		if(alleleRequest.getId() == null || alleleRequest.getId() == 0) {
			alleleRequest.setRequestOwner(form.getSubmitter());
		}

		if(alleleRequest.getSource().getId().longValue() != form.getInitialSourceId()){
			alleleRequest.setSubmitToQueueDate(new Date());  
		}

		Person person = RequestUtils.getUser(request);
		Long requestId = alleleRequestService.store(alleleRequest, person);


		// if the status is "Vega Annotation Requested", add a new havana request
		// (if none exists)
		if(alleleRequest.getStatus().getId().equals(VEGA_ANNOTATION_REQUESTED)) {

			if(!havanaRequestService.alreadyExists(alleleRequest)) {
				HavanaRequest havanaRequest = new HavanaRequest();
				havanaRequest.setAlleleRequest(alleleRequest);
				havanaRequest.setAlleleRequestId(alleleRequest.getId());
				havanaRequest.setCreatorId(person.getId());
				havanaRequest.setCreatedDate(new Date());
				havanaRequest.setStatus(new DictionaryEntry(VEGA_ANNOTATION_REQUESTED));
				havanaRequest.setCreator(person);

				havanaRequestService.store(havanaRequest, person);

				havanaRequestService.sendNewRequestNotification(havanaRequest);

			}
		}		

		if(alleleRequest.getStatus().getId().equals(DESIGN_REQUESTED)) {

			DesignRequest designRequest = new DesignRequest();
			designRequest.setAlleleRequest(alleleRequest);
			designRequest.setCreatorId(person.getId());
			designRequest.setCreatedDate(new Date());
			designRequest.setStatusDictId(DESIGN_REQUESTED);

			/*
			 * Not plausible to get OTTMUSG id's for all genes which don't have a
			 * vega build hence setting the vegaId from what the annotators have
			 * set. Not explained clearly
			 */

			if (havanaRequestService.alreadyExists(alleleRequest)) {
				ComponentMap havanaRequestComponentMap = new ComponentMap();
				havanaRequestComponentMap.put(HavanaRequest.Component.ALL_COMPONENTS, null);
				HavanaRequest havanaRequest = havanaRequestService.fetchByAlleleRequestId(
						alleleRequest.getId(), havanaRequestComponentMap);
				designRequest.setVegaId(havanaRequest.getVegaId());
			}
			// When all the genes have a link to gene_build_gene with OTTMUS names
			// following
			// commented code can be used to get OTTMUSG id

			// if(alleleRequest.getGene().getVegaPrimaryGeneBuildGene()!=null){
			// designRequest.setVegaId(alleleRequest.getGene().getVegaPrimaryGeneBuildGene().getPrimaryName());
			// }

			if(alleleRequest.getMigGene().getPrimaryEnsemblBuildGeneId() != null){
				designRequest.setEnsemblId(alleleRequest.getMigGene().getPrimaryEnsemblBuildGene().getPrimaryName());
			}
			designRequest.setStatus(new DictionaryEntry(DESIGN_REQUESTED));
			designRequestService.store(designRequest, person);

			// TO DO Implementation of DesignRequestService object
			// designRequestService.sendNewRequestNotification(havanaRequest);

		}   

		String queryString = "?currentRecordId=" + requestId;

		// for new records filter by status and sort by descending ID
		if(isNewRequest) {
			queryString += "&bypassForm=true&newQuery=true&orderBy=id&orderDesc=true";
		}
		// for existing records, return to the last list page the user was on
		else {
			queryString += "&bypassForm=true&newQuery=false&retrieveFromSession=true";
		}

		return new ModelAndView(getSuccessView() + queryString);
	}  

	protected Map referenceData(HttpServletRequest request) throws Exception {
		Map<String, Object> referenceData = new HashMap<String, Object>();

		// Target Endpoints
		Collection targetEndpoints = dictionaryDao.fetchDictionaryByTableName(
				"ARQ_TARGET_ENDPOINT_DICT", false);
		referenceData.put("targetEndpoints", targetEndpoints);

		// Request Status
		Collection statuses = dictionaryDao.fetchDictionaryByTableName(
				"ARQ_ALLELE_REQUEST_STATUS_DICT", false);
		referenceData.put("statuses", statuses);

		// Request Source
		Collection sources = dictionaryDao.fetchDictionaryByTableName(
				"ARQ_ALLELE_REQUEST_SOURCE_DICT", false);
		referenceData.put("sources", sources);

		// Yes/No list
		Map<Boolean, String> yesNo = new HashMap<Boolean, String>();
		yesNo.put(false, "No");
		yesNo.put(true, "Yes");
		referenceData.put("yesNo", yesNo);

		// Does Knockout Exist
		Collection<String> knockoutExists = new ArrayList<String>();
		knockoutExists.add("Unknown");
		knockoutExists.add("Yes");
		knockoutExists.add("No");
		referenceData.put("knockoutExists", knockoutExists);

		// Nature of Allele
		Collection modTypes = dictionaryDao.fetchDictionaryByTableName(
				"ARQ_ALLELE_MOD_TYPE_DICT", false);
		referenceData.put("modTypes", modTypes);

		return referenceData;
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request)
	throws Exception {
		AlleleRequestForm form = (AlleleRequestForm) createCommand();
		AlleleRequest alleleRequest = null;

		// load alleleRequest from database if "id" parameter is available
		Long alleleRequestId = ServletRequestUtils.getLongParameter(request, "id");
		if (alleleRequestId != null) {

			ComponentMap componentMap =
				ComponentMap.putAll(
						new ComponentMap(AlleleRequest.Component.GENE, ComponentMap.of(MigGene.Component.PRIMARY_BUILD_GENE, MigGene.Component.PRIMARY_ENSEMBL_BUILD_GENE)),
						new ComponentMap(AlleleRequest.Component.DESIGNS, null),
						new	ComponentMap(AlleleRequest.Component.HAVANA_REQUEST, null));

			alleleRequest = alleleRequestService.fetchById(alleleRequestId, componentMap);

			// set the geneLookupTerm
			form.setGeneLookupTerm(alleleRequest.getMigGene().getPrimaryBuildGene().getPrimaryName());			
		}

		// if it's a new form, set the default values
		if (alleleRequest == null) {
			alleleRequest = buildNewAlleleRequest();
		}

		form.setSubmitter(RequestUtils.getUser(request));
		form.setInitialSourceId(alleleRequest.getSource().getId().longValue());

		form.setAlleleRequest(alleleRequest);


		return form;
	}

	/**
	 * If it's a new form, return to the main menu, otherwise return to the list
	 * of allele requests
	 */
	@Override
	protected ModelAndView onCancel(Object command) throws Exception {
		AlleleRequestForm form = (AlleleRequestForm) command;
		AlleleRequest alleleRequest = form.getAlleleRequest();

		if (alleleRequest.getId() == null) {
			return new ModelAndView("redirect:view_main.do");
		} else {
			return new ModelAndView(getCancelView() + "?status.id="
					+ form.getAlleleRequest().getStatus().getId());
		}
	}

	/**
	 * Set up an 'empty' AlleleRequest for new request.
	 * 
	 * @return an AlleleRequest object
	 */
	private AlleleRequest buildNewAlleleRequest() {
		AlleleRequest alleleRequest = new AlleleRequest();

		// set new source
		DictionaryEntry souce = new DictionaryEntry(DEFAULT_REQUEST_SOURCE);
		alleleRequest.setSource(souce);

		DictionaryEntry modType = new DictionaryEntry(DEFAULT_MOD_TYPE);
		alleleRequest.setModType(modType);

		// set new status
		DictionaryEntry status = new DictionaryEntry(DEFAULT_FORM_STATUS);
		alleleRequest.setStatus(status);

		// set new targetEndpoint
		DictionaryEntry targetEndpoint = new DictionaryEntry(
				DEFAULT_TARGET_ENDPOINT);
		alleleRequest.setTargetEndpoint(targetEndpoint);

		// set new gene
		MigGene gene = new MigGene();
		alleleRequest.setMigGene(gene);

		// set new requestOwner
		Person requestOwner = new Person();
		alleleRequest.setRequestOwner(requestOwner);

		alleleRequest
		.setComments("e.g. Imprinted gene, unpublished alternative splicing, key Pfam domains.");

		alleleRequest.setMgpPhenotypingRequired(false);

		return alleleRequest;
	}

	public DesignRequestService getDesignRequestService() {
		return designRequestService;
	}

	public void setDesignRequestService(DesignRequestService designRequestService) {
		this.designRequestService = designRequestService;
	}

}
