/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.allelerequest.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import uk.ac.sanger.mig.allelerequest.AlleleRequest;
import uk.ac.sanger.mig.allelerequest.AlleleRequestSearchBean;
import uk.ac.sanger.mig.allelerequest.views.AlleleRequestListView;
import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.charting.DatabaseChartData;
import uk.ac.sanger.mig.common.OperationNotSupported;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.LongRowMapper;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.personnel.Person;

public class AlleleRequestDaoSpringImpl extends SimpleJdbcDaoSupport
implements AlleleRequestDao<AlleleRequest> {

	private AlleleRequestRowMapper alleleRequestRowMapper;
	private AlleleRequestListViewRowMapper alleleRequestListViewRowMapper;
	private PaginatedSearchUtils<AlleleRequestListView> paginator;

	public void setPaginator(PaginatedSearchUtils<AlleleRequestListView> paginator) {
		this.paginator = paginator;
	}

	public void setAlleleRequestRowMapper(
			AlleleRequestRowMapper alleleRequestRowMapper) {
		this.alleleRequestRowMapper = alleleRequestRowMapper;
	}

	public void setAlleleRequestListViewRowMapper(
			AlleleRequestListViewRowMapper alleleRequestListViewRowMapper) {
		this.alleleRequestListViewRowMapper = alleleRequestListViewRowMapper;
	}

	public Long store(AlleleRequest alleleRequest) {
		throw new OperationNotSupported("store not supported - use storeAuthenticated");
	}


	public AlleleRequest fetchById(Long id) {
		AlleleRequest alleleRequest = null;
		String sql = " select * from arq_allele_request where id = ? ";

		try {
			alleleRequest = getSimpleJdbcTemplate().queryForObject(sql, alleleRequestRowMapper, id);
		}
		catch(IncorrectResultSizeDataAccessException e) {
			logger.warn(e);
		}

		return alleleRequest;
	}

	public Collection<Long> fetchDesignIds(Long id){

		Collection<Long> designIdsList = new ArrayList<Long>();

		String sql = "select distinct(a2d.design_id) return_long from arq_design_request adr, arq_design_2_design_request a2d where " +
		"a2d.design_request_id = adr.id and " +
		"adr.allele_request_id = ?";

		designIdsList = getSimpleJdbcTemplate().query(sql, new LongRowMapper(), id);

		return (designIdsList == null) ? new ArrayList<Long>() : designIdsList;
	}

	public Collection<Long> fetchFulfillingRequestDesignIds(Long id){

		Collection<Long> designIdsList = new ArrayList<Long>();
		String sql = "select distinct(a2d.design_id) return_long from arq_design_request adr, arq_design_2_design_request a2d where " +
		"a2d.design_request_id = adr.id and a2d.fulfills_request=1 and " +
		"adr.allele_request_id = ?";
		designIdsList = getSimpleJdbcTemplate().query(sql, new LongRowMapper(), id);

		return(designIdsList == null) ? new ArrayList<Long>() : designIdsList;
	}


	public Collection<AlleleRequest> fetchAll() {
		throw new OperationNotSupported("fetchAll not supported");
	}

	public Long storeAuthenticated(AlleleRequest alleleRequest, Person currentUser) {
		alleleRequest.setEditedBy(currentUser.getUserName());
		alleleRequest.setEditDate(new java.util.Date());

		if(alleleRequest.getId() == null || alleleRequest.getId() == 0) {
			insert(alleleRequest);
		}

		else {
			update(alleleRequest);
		}

		return alleleRequest.getId();
	}

	private Long update(AlleleRequest alleleRequest) {
		String sqlString = "update arq_allele_request set requester_person_id=?, target_endpoint_dict_id=?, edit_date=?, comments=?, researcher_person_id=?, request_reasoning=?, mutation_plan=?, knockout_exists=?, knockout_exists_reference=?, allele_mod_type_dict_id=?, request_priority=?, gene_id=?, review_comments=?, researcher_name=?, status_dict_id=?, allele_mod_other_expln=?, edited_by=?, unexpected_phenotype_ok=?, mgp_phenotyping_required=?, request_source_dict_id=?, gene_publications=?, mouse_publications=?, submit_to_queue_date =?, see_also=? where id=?";

		ParameterizedSqlUpdate sql = new ParameterizedSqlUpdate(getJdbcTemplate());
		bindVariables(sql,alleleRequest);
		sql.execute(sqlString);

		return alleleRequest.getId();
	}

	private Long insert(AlleleRequest alleleRequest) {
		alleleRequest.setId(getPk());

		String sqlString = "insert into arq_allele_request (requester_person_id,target_endpoint_dict_id,edit_date,comments,researcher_person_id,request_reasoning,mutation_plan,knockout_exists,knockout_exists_reference,allele_mod_type_dict_id,request_priority,gene_id,review_comments,researcher_name,status_dict_id,allele_mod_other_expln,edited_by,unexpected_phenotype_ok,mgp_phenotyping_required,request_source_dict_id,gene_publications,mouse_publications, submit_to_queue_date, see_also, id) values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

		ParameterizedSqlUpdate sql = new ParameterizedSqlUpdate(getJdbcTemplate());
		bindVariables(sql,alleleRequest);
		sql.execute(sqlString);

		return alleleRequest.getId();
	}

	private Long getPk() {
		return getSimpleJdbcTemplate().queryForLong("select arq_allele_request_seq.nextval from dual");
	}

	private void bindVariables(ParameterizedSqlUpdate sql, AlleleRequest alleleRequest) {

		sql.addLongParam(alleleRequest.getRequestOwner().getId());
		sql.addLongParam(alleleRequest.getTargetEndpoint().getId());
		sql.addDateParam(alleleRequest.getEditDate());
		sql.addStringParam(alleleRequest.getComments());
		sql.addLongParam(alleleRequest.getResearcherId());
		sql.addStringParam(alleleRequest.getReasoning());
		sql.addStringParam(alleleRequest.getMutationPlan());
		sql.addStringParam(alleleRequest.getKnockoutAvailable());
		sql.addStringParam(alleleRequest.getKnockoutReference());
		sql.addLongParam(alleleRequest.getModType().getId());
		sql.addIntParam(alleleRequest.getPriority());
		sql.addLongParam(alleleRequest.getMigGeneId());
		sql.addStringParam(alleleRequest.getReviewComments());
		sql.addStringParam(alleleRequest.getResearcherName());
		sql.addLongParam(alleleRequest.getStatus().getId());
		sql.addStringParam(alleleRequest.getModTypeOtherExplanation());
		sql.addStringParam(alleleRequest.getEditedBy());
		sql.addBooleanParam(alleleRequest.isUnexpectedPhenotypeOk());
		sql.addBooleanParam(alleleRequest.isMgpPhenotypingRequired());
		sql.addLongParam(alleleRequest.getSource().getId());
		sql.addStringParam(alleleRequest.getGenePublications());
		sql.addStringParam(alleleRequest.getMousePublications());
		sql.addDateParam(alleleRequest.getSubmitToQueueDate());
		sql.addStringParam(alleleRequest.getSeeAlso());
		sql.addLongParam(alleleRequest.getId());
	}

	public ChartData fetchStatusChartData() {
		String sql = "select status label, status_dict_id extra_field, count(*) data0 from arq_allele_request_list_vw group by status, status_dict_id order by data0 ";

		DatabaseChartData data = new DatabaseChartData(1);
		getJdbcTemplate().query(sql, data);

		return data;
	}

	public Collection<Long> fetchAllIds(){

		Collection<Long> alleleRequestIds = new ArrayList<Long>();
		String sql = "select a.id return_long from arq_allele_request_list_vw a order by a.id";
		alleleRequestIds= getSimpleJdbcTemplate().query(sql, new LongRowMapper());

		return (alleleRequestIds ==null) ? new ArrayList<Long>() : alleleRequestIds;
	}

	public Collection<AlleleRequestListView> search(AlleleRequestSearchBean searchBean) {

		Collection<Object> bindVariables = new ArrayList<Object>();
		AlleleRequest alleleRequest =  searchBean.getAlleleRequest();
		StringBuffer sql = new StringBuffer();
		sql.append(" select * from arq_allele_request_list_vw a where 1=1 ");

		// search by status id
		if(alleleRequest.getStatus() != null && alleleRequest.getStatus().getId() != null && alleleRequest.getStatus().getId() != 0 ) {
			sql.append(" and a.status_dict_id = ? ");
			bindVariables.add(searchBean.getAlleleRequest().getStatus().getId());
		}


		// search by source
		if(alleleRequest.getSource() != null && alleleRequest.getSource().getId() != null) {

			Long id = alleleRequest.getSource().getId();

			if(id > 0 ) {
				sql.append(" and a.REQUEST_SOURCE_DICT_ID = ? ");
				bindVariables.add(id);
			} else if(id == -5L) {
				sql.append(" and a.REQUEST_SOURCE_DICT_ID != ? ");
				bindVariables.add(Math.abs(id));
			}
		}

		if(searchBean.getFilterBy()!=null){
			if(searchBean.getFilterValue()!=null && searchBean.getFilterValue().length()!=0 && !searchBean.getFilterValue().contains("Please enter")) {
				if(searchBean.getFilterBy().equals("Request Number")) {
					Long id=Long.parseLong(searchBean.getFilterValue());
					sql.append(" and a.ID = ? ");
					bindVariables.add(id);
				}else if(searchBean.getFilterBy().equals("Gene Name")) {
					String geneName= searchBean.getFilterValue();
					sql.append(" and UPPER(a.GENE_NAME) = ? OR exists " +
					"(select null from gnm_gene_name gn where gn.gene_id = a.gene_id and UPPER(gn.name) = ?) ");
					bindVariables.add(geneName.toUpperCase());
					bindVariables.add(geneName.toUpperCase());
				}else if(searchBean.getFilterBy().equals("Ensembl ID")) {
					String ensemblId = searchBean.getFilterValue();
					sql.append(" and UPPER(a.ENSEMBL_ID) = ? OR exists " +
					"(select null from gnm_gene_name gn where gn.gene_id = a.gene_id and UPPER(gn.name) = ?) ");
					bindVariables.add(ensemblId.toUpperCase());
					bindVariables.add(ensemblId.toUpperCase());
				}
				else if(searchBean.getFilterBy().equals("Submitter Name")) {
					sql.append(" and upper(user_name) like '%'||upper(?)||'%' ");
					bindVariables.add(searchBean.getFilterValue());
				}
				else if(searchBean.getFilterBy().equals("See Also")) {
					sql.append(" and upper(see_also) like '%'||upper(?)||'%' ");
					bindVariables.add(searchBean.getFilterValue());
				}
				else if(searchBean.getFilterBy().equals("Researcher Name")) {
					sql.append(" and upper(researcher_name) like '%'||upper(?)||'%' ");
					bindVariables.add(searchBean.getFilterValue());
				}
			}
		}

		// search by Gene
		if(alleleRequest.getMigGene() != null){
			Long id = alleleRequest.getMigGene().getId();

			if(id > 0 ) {
				sql.append(" and a.GENE_ID = ? ");
				bindVariables.add(id);
			}
		}

		// Order by column and direction as supplied in the search bean
		if(searchBean.getOrderBy() != null && !searchBean.getOrderBy().equals("null") && !searchBean.getOrderBy().equals("")) {
			sql.append(" order by " + searchBean.getOrderBy());
			if(searchBean.isOrderDesc()) {
				sql.append(" desc ");
			}
		}
		logger.debug(sql.toString());
		return paginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, alleleRequestListViewRowMapper);
	}

	public Collection<AlleleRequest> fetchByGene(MigGene migGene) {
		String sql = " select * from arq_allele_request where gene_id = ? ";

		Collection<AlleleRequest> alleleRequests = getSimpleJdbcTemplate().query(sql, alleleRequestRowMapper, migGene.getId());

		return (alleleRequests == null) ? new ArrayList<AlleleRequest>() : alleleRequests;
	}

	public Collection<AlleleRequest> fetchCurrentByEditDate(Date editedFrom, Date editedTo) {
		Collection<AlleleRequest> alleleRequests = null;
		String sql = " select * from arq_allele_request where edit_date >= ? AND edit_date < ? ";

		try {
			alleleRequests = getSimpleJdbcTemplate().query(sql, alleleRequestRowMapper, editedFrom, editedTo);
		} catch(IncorrectResultSizeDataAccessException e) {

			logger.warn("No current AlleleRequest records found in arq_allele_request table.");

		}

		return (alleleRequests == null) ? new ArrayList<AlleleRequest>() : alleleRequests;
	}

	public Collection<AlleleRequest> fetchHistoricalByEditDate(Date editedFrom, Date editedTo) {

		Collection<AlleleRequest> alleleRequests = null;
		String sql = " select * from mig_audit.arq_allele_request where edit_date >= ? AND edit_date < ? ";

		try {
			alleleRequests = getSimpleJdbcTemplate().query(sql, alleleRequestRowMapper, editedFrom, editedTo);
		} catch(IncorrectResultSizeDataAccessException e) {

			logger.warn("No historical AlleleRequest records found in mig_audit.arq_allele_request table.");

		}

		return (alleleRequests == null) ? new ArrayList<AlleleRequest>() : alleleRequests;
	}
}
