/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.allelerequest.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import uk.ac.sanger.mig.allelerequest.AlleleRequest;
import uk.ac.sanger.mig.allelerequest.AlleleRequestSearchBean;
import uk.ac.sanger.mig.allelerequest.dao.AlleleRequestDao;
import uk.ac.sanger.mig.allelerequest.views.AlleleRequestListView;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.MailHelper;
import uk.ac.sanger.mig.common.Statistic;
import uk.ac.sanger.mig.designrequest.Design;
import uk.ac.sanger.mig.designrequest.services.DesignService;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.services.MigGeneService;
import uk.ac.sanger.mig.havanarequest.HavanaRequest;
import uk.ac.sanger.mig.havanarequest.dao.HavanaRequestDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.dao.MailingListDao;

public class AlleleRequestServiceImpl implements AlleleRequestService {

	private MailHelper mailHelper;
	private AlleleRequestDao<AlleleRequest> alleleRequestDao;//fetch and build model in service tier
	private HavanaRequestDao havanaRequestDao ;
	private DesignService designService;
	private MigGeneService migGeneService;

	public void setHavanaRequestDao(HavanaRequestDao havanaRequestDao) {
		this.havanaRequestDao = havanaRequestDao;
	}

	public void setAlleleRequestDao(AlleleRequestDao<AlleleRequest> alleleRequestDao) {

		this.alleleRequestDao = alleleRequestDao;
	}

	public void setMailHelper(MailHelper mailHelper) {
		this.mailHelper = mailHelper;
	}

	public AlleleRequest fetchById(Long id,  ComponentMap requiredComponents){
		AlleleRequest alleleRequest = alleleRequestDao.fetchById(id);
		if(alleleRequest != null){
			attachComponents(alleleRequest, requiredComponents);
		}
		return alleleRequest;

	}

	protected void attachComponents(AlleleRequest alleleRequest, ComponentMap requiredComponents) {
		if(alleleRequest == null) {
			throw new RuntimeException("Cannot attach components to null designinstance");
		}
		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(AlleleRequest.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(AlleleRequest.Component.DESIGNS)) {
				alleleRequest.setDesigns(fetchDesigns(alleleRequest, requiredComponents.get(AlleleRequest.Component.DESIGNS)));
			}

			if(fetchAll || requiredComponents.containsKey(AlleleRequest.Component.HAVANA_REQUEST)){
				alleleRequest.setHavanaRequest(fetchHavanaRequest(alleleRequest, requiredComponents.get(AlleleRequest.Component.HAVANA_REQUEST)));
			}

			if(fetchAll || requiredComponents.containsKey(AlleleRequest.Component.GENE)){
				alleleRequest.setMigGene(fetchMigGene(alleleRequest, requiredComponents.get(AlleleRequest.Component.GENE)));
			}
		}
	}



	private HavanaRequest fetchHavanaRequest(AlleleRequest alleleRequest, ComponentMap map) {

		return havanaRequestDao.fetchbyAlleleRequestId(alleleRequest.getId());
	}

	private Collection<Design> fetchDesigns(AlleleRequest alleleRequest, ComponentMap map) {

		Collection<Long> designIds = alleleRequestDao.fetchFulfillingRequestDesignIds(alleleRequest.getId());
		Collection<Design> designs = new ArrayList<Design>();
		for(Long designId: designIds){
			designs.add(designService.fetchById(designId, map));
		}
		return designs;
	}

	private MigGene fetchMigGene(AlleleRequest alleleRequest, ComponentMap map) {

		return migGeneService.fetchById(alleleRequest.getMigGeneId(), map);
	}

	public boolean sendAlleleActivityReport(Date activityFromDate, Date activityToDate) {
		Map<String, Object>model = new HashMap<String, Object>();
		List<AlleleRequest> alleleRequests =
			(List<AlleleRequest>)getAlleleActivity(activityFromDate, activityToDate);
		Collections.sort(alleleRequests); //sort here vs db b/c we're sorting merged list
		model.put("alleleRequests", alleleRequests);  //we want our raw data
		Collection<Statistic> alleleSummaryStats = deriveSummaryStats(alleleRequests);
		List<Statistic> sortedAalleleSummaryStats = new ArrayList<Statistic>(alleleSummaryStats);
		Collections.sort( sortedAalleleSummaryStats ); //sort comparable stats
		model.put("alleleSummaryStats", sortedAalleleSummaryStats); //...and our summary stats.
		model.put("activityFromDate",activityFromDate);//...and Date range
		model.put("activityToDate",activityToDate);
		return mailHelper.sendMail("allele_activity", model, MailingListDao.ALLELE_ACTIVITY_REPORT_MAILING_LIST);
	}

	public boolean sendAlleleActivityReportWeekToDate() {
		long now = System.currentTimeMillis();
		long lastWeek = now - 604800000L;
		return sendAlleleActivityReport(new Date(lastWeek), new Date(now));
	}

	public Collection<AlleleRequest> getAlleleActivity(Date activityFromDate, Date activityToDate) {
		Collection<AlleleRequest> current =
			alleleRequestDao.fetchCurrentByEditDate(activityFromDate,activityToDate);
		Collection<AlleleRequest> historical =
			alleleRequestDao.fetchHistoricalByEditDate(activityFromDate,activityToDate);
		current.addAll(historical);
		return current;
	}

	public Collection<Statistic> deriveSummaryStats(Collection<AlleleRequest> alleleRequests) {
		HashMap<String,Statistic> stats = new HashMap<String,Statistic>();
		Iterator iter = alleleRequests.iterator();
		while(iter.hasNext()) {
			AlleleRequest alleleRequest = (AlleleRequest)iter.next();

			if(alleleRequest.getStatus() == null) continue;//handle f-ed up no data migd

			if(!stats.containsKey(alleleRequest.getStatus().getName())) {
				Statistic stat = new Statistic(alleleRequest.getStatus().getName(),new Integer(1));
				stats.put(alleleRequest.getStatus().getName(), stat);
			} else {
				Statistic stat = stats.get(alleleRequest.getStatus().getName());
				Integer value = (Integer)stat.getValue();
				stat.setValue(value + 1);
				stats.put(alleleRequest.getStatus().getName(), stat);
			}
		}
		return stats.values();
	}

	public void setDesignService(DesignService designService) {
		this.designService = designService;
	}

	public AlleleRequest fetchById(Long id) {
		return alleleRequestDao.fetchById(id);
	}

	public Long store(AlleleRequest alleleRequest, Person user) {
		return alleleRequestDao.storeAuthenticated(alleleRequest, user);
	}

	public Collection<AlleleRequestListView> search(AlleleRequestSearchBean searchBean){
		return alleleRequestDao.search(searchBean);
	}

	public void setMigGeneService(MigGeneService migGeneService) {
		this.migGeneService = migGeneService;
	}
}
