/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.allelerequest.validators;

import java.util.Collection;
import java.util.Iterator;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.allelerequest.AlleleRequest;
import uk.ac.sanger.mig.allelerequest.forms.AlleleRequestForm;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.MigGeneSearchBean;
import uk.ac.sanger.mig.gene.services.MigGeneService;
import uk.ac.sanger.mig.personnel.Person;

public class AlleleRequestFormValidator implements Validator {

	private static final Long ALLELE_TYPE_OTHER = 2L;

	private MigGeneService migGeneService;

	public void setMigGeneService(MigGeneService migGeneService) {
		this.migGeneService = migGeneService;
	}

	public boolean supports(Class clazz) {
		return AlleleRequestForm.class.equals(clazz);
	}

	public void validate(Object command, Errors errors) {
		AlleleRequestForm form = (AlleleRequestForm)command;
		AlleleRequest alleleRequest = form.getAlleleRequest();

		// Name should be available in the GeneDatabase as a single result.
		if(form.getGeneLookupTerm() == null || form.getGeneLookupTerm().length() == 0) {
			errors.rejectValue("geneLookupTerm", "requiredField", "* Required field *");
		} else {
			// Look up the supplied name and only accept it if its a single result.
			MigGeneSearchBean migGeneSearchBean = new MigGeneSearchBean();
			migGeneSearchBean.setQueryString(form.getGeneLookupTerm());
			migGeneSearchBean.setPaginated(false);

			Collection<MigGene> migGenes = migGeneService.fetch(migGeneSearchBean, new ComponentMap());

			if(migGenes.size() == 0) {
				errors.rejectValue("geneLookupTerm", "noMatchingGene", "* Gene Not Found *");
			} else if(migGenes.size() > 1) {
				errors.rejectValue("geneLookupTerm", "multipleMatchingGenes", "* Gene Name Finds Multiple Matches - Please try an Ensembl ID *");
			} else {
				// Success... we have a Gene collection with the matching Gene object in it.
				Iterator iterator = migGenes.iterator();
				MigGene migGene = (MigGene) iterator.next();

				// Keep it for later, performance optimisation.
				alleleRequest.setMigGeneId(migGene.getId());
			}
		}

		// If alleleRequest.alleleNature is "Other" then this must not be null.
		if(ALLELE_TYPE_OTHER.equals(alleleRequest.getModType().getId())) {
			ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.modTypeOtherExplanation", "requiredField", "* Required field for Other *");
		}

		// Should be from a pull-down list, should be checked against the dictionary. Typically this will be "ES Cell", "Mouse", "Vector" 
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.targetEndpoint.id", "requiredField", "* Required field *");

		// Should be 'Yes, No or Other' 
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.knockoutAvailable", "requiredField", "* Required field *");

		// Free text
		//ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.knockoutReference", "requiredField", "* Required field *");

		// Free text - should not match the pre-loaded string.
		if(alleleRequest.getComments() == null) {
			errors.rejectValue("alleleRequest.comments", "requiredField", "* Required field *");
		} else if(alleleRequest.getComments().length() == 0){
			errors.rejectValue("alleleRequest.comments", "requiredField", "* Required field *");
		} else if(alleleRequest.getComments().equals("e.g. Imprinted gene, unpublished alternative splicing, key Pfam domains.")){
			errors.rejectValue("alleleRequest.comments", "unchangedDefaultFieldValue", "* Please don\'t leave the default text in the comments field. *");
		}

		// Free text
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.researcherName", "requiredField", "* Required field *");

		// Free text
		//ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.genePublications", "requiredField", "* Required field *");
		if(alleleRequest.getGenePublications() != null && alleleRequest.getGenePublications().length() > 4000) {
			errors.rejectValue("alleleRequest.genePublications", null, "Text cannot exceed 4000 characters");
		}

		// free text
		if(alleleRequest.getComments() != null && alleleRequest.getComments().length() > 2048) {
			errors.rejectValue("alleleRequest.comments", null, "Text cannot exceed 2048 characters");
		}
		
        // free text
		if(alleleRequest.getSeeAlso() != null && alleleRequest.getSeeAlso().length() > 30) {
			errors.rejectValue("alleleRequest.seeAlso", null, "Text cannot exceed 30 characters");
		}


		// Free text
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.reasoning", "requiredField", "* Required field *");
		if(alleleRequest.getReasoning() != null && alleleRequest.getReasoning().length() > 4000) {
			errors.rejectValue("alleleRequest.reasoning", null, "Text cannot exceed 4000 characters");
		}		

		// Free text
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.mutationPlan", "requiredField", "* Required field *");
		if(alleleRequest.getMutationPlan() != null && alleleRequest.getMutationPlan().length() > 4000) {
			errors.rejectValue("alleleRequest.mutationPlan", null, "Text cannot exceed 4000 characters");
		}	

		// Free text must be supplied if the user (logged in) is not a WTSI user.
		Person p = form.getSubmitter();
		if(!"WTSI".equals(p.getAddress())) { 
			ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.mousePublications", "requiredField", "* Required field *");
		}	
		if(alleleRequest.getMousePublications() != null && alleleRequest.getMousePublications().length() > 4000) {
			errors.rejectValue("alleleRequest.mousePublications", null, "Text cannot exceed 4000 characters");
		}			

		// Boolean 'Yes' or 'No'
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.unexpectedPhenotypeOk", "requiredField", "* Required field *");

		// Pull down indicating request source (for use by Vega annotation list), Generated from a dictionary.
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.source.id", "requiredField", "* Required field *");

		//Pull down, Boolean value 'Yes' or 'No'
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "alleleRequest.mgpPhenotypingRequired", "requiredField", "* Required field *");
	}

}