/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.alleles;

import java.util.Collection;

import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.gene.Locus;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.personnel.Person;

public class Allele extends AuthenticatedDataObject implements LookupListElement {

	public enum Component implements ComponentEnum{
		ALL_COMPONENTS,
		ALLELE_MAKER,
		GENOTYPE_TESTS,
		GENE,
		LOCUS,
		GEL_IMAGES,
		ALLELE_SYNONYMS
	}

	private boolean multiGeneRegion;

	private String multiGeneRegionDescrip;

	private Long locusId;
	private Locus locus; //chromosome,start,end

	private Long alleleMakerId;
	private Person alleleMaker;
	private String externalMakerName;

	private Long mutationNatureDictId;
	private DictionaryEntry mutationNature;
	private Long mutationClassDictId;
	private DictionaryEntry mutationClass;

	private Long inheritanceModeDictId;
	private DictionaryEntry inheritanceMode;

	private GenotypeTest genotypeTest;
	private Long genotypeTestId;

	private Long migGeneId;
	//private String migGeneString;
	private MigGene migGene;

	private String internalName;

	private String officialNameSuperscript;
	private String shortName;
	private String alleleDescription;
	private String homozygousLethal;

	private String multipleGeneName;
	private String hasAssociatedPhenotype;
	private String phenotypeDescription;

	private String team87GeneSymbol;

	private Collection<BinaryFile> gelImages;
	private Collection<AlleleSynonym> alleleSynonyms;
	private String descriptionUri;


	/**
	 * Temporary holder for Gene Symbol provided by Team 87
	 * web service.  Used to update migGene primary name
	 * if the ours is different.
	 *
	 * @return
	 */
	public String getTeam87GeneSymbol() {
		return this.team87GeneSymbol;
	}

	public void setTeam87GeneSymbol(String team87GeneSymbol) {
		this.team87GeneSymbol = team87GeneSymbol;
	}

	public DictionaryEntry getMutationClass() {
		return mutationClass;
	}

	public void setMutationClass(DictionaryEntry mutationClass) {
		this.mutationClass = mutationClass;
	}

	public Long getMutationClassDictId() {
		return mutationClassDictId;
	}

	public boolean isHarmfulMutation() {
		return (AlleleService.HARMFUL_MUTATION.equals(getMutationClassDictId()));
	}

	public void setMutationClassDictId(Long mutationClassDictId) {
		this.mutationClassDictId = mutationClassDictId;
	}

	public String getDisplayText() {
		String geneName = (migGene != null) ? migGene.getPrimaryName() : "(No Gene)";
		return geneName + "<" + officialNameSuperscript + ">";
	}

	public String getHtmlAlleleSymbol() {
		String geneName = (migGene != null) ? migGene.getPrimaryName() : "(No Gene)";
		return "<i>" + geneName + "<sup>" + officialNameSuperscript + "</sup></i>";
	}

	public String getElementData() {
		String geneName = (migGene != null) ? migGene.getPrimaryName() : "";
		return LookupListUtil.format("alleleId", getId()) +
			LookupListUtil.format("officialNameSuperscript", officialNameSuperscript) +
			LookupListUtil.format("primaryName", geneName);
	}

	public MigGene getMigGene() {
		return migGene;
	}

	public void setMigGene(MigGene migGene) {
		this.migGene = migGene;
	}

	public Locus getLocus() {
		return locus;
	}
	public void setLocus(Locus locus) {
		this.locus = locus;
	}

	public String getMultiGeneRegionDescrip() {
		return multiGeneRegionDescrip;
	}

	public void setMultiGeneRegionDescrip(String multiGeneRegionDescrip) {
		this.multiGeneRegionDescrip = multiGeneRegionDescrip;
	}

	public String getAlleleDescription() {
		return alleleDescription;
	}
	public void setAlleleDescription(String description) {
		this.alleleDescription = description;
	}

	public String getHasAssociatedPhenotype() {
		return hasAssociatedPhenotype;
	}

	public void setHasAssociatedPhenotype(String associatedPhenotype) {
		this.hasAssociatedPhenotype = associatedPhenotype;
	}

	public String getWhetherHomozygousLethal() {
		return homozygousLethal;
	}

	public void setWhetherHomozygousLethal(String homozygousLethal) {
		this.homozygousLethal = homozygousLethal;
	}

	public DictionaryEntry getInheritanceMode() {
		return inheritanceMode;
	}

	public void setInheritanceMode(DictionaryEntry inheritanceMode) {
		this.inheritanceMode = inheritanceMode;
	}

	public String getInternalName() {
		return internalName;
	}

	public void setInternalName(String internalName) {
		this.internalName = internalName;
	}

	public DictionaryEntry getMutationNature() {
		return mutationNature;
	}

	public void setMutationNature(DictionaryEntry mutationNature) {
		this.mutationNature = mutationNature;
	}

	public String getOfficialNameSuperscript() {
		return officialNameSuperscript;
	}

	public void setOfficialNameSuperscript(String officialNameSuperscript) {
		this.officialNameSuperscript = officialNameSuperscript;
	}

	public String getPhenotypeDescription() {
		return phenotypeDescription;
	}

	public void setPhenotypeDescription(String phenotypeDescription) {
		this.phenotypeDescription = phenotypeDescription;
	}

	public boolean isMultiGeneRegion() {
		return multiGeneRegion;
	}

	public void setMultiGeneRegion(boolean multiGeneRegion) {
		this.multiGeneRegion = multiGeneRegion;
	}

	public Person getAlleleMaker() {
		return alleleMaker;
	}

	public void setAlleleMaker(Person alleleMaker) {
		this.alleleMaker = alleleMaker;
	}

	public String getShortAlleleSymbol() {
		return shortName;
	}

	public void setShortAlleleSymbol(String shortName) {
		this.shortName = shortName;
	}

	public void setDisplayText(String displayText) {
		// TODO Auto-generated method stub
	}

	public void setElementData(String elementData) {
		// TODO Auto-generated method stub
	}

	public String getMultipleGeneName() {
		return multipleGeneName;
	}

	public void setMultipleGeneName(String multipleGeneName) {
		this.multipleGeneName = multipleGeneName;
	}

	public Long getMigGeneId() {
		return migGeneId;
	}

	public void setMigGeneId(Long migGeneId) {
		this.migGeneId = migGeneId;
	}

	public Long getInheritanceModeDictId() {
		return inheritanceModeDictId;
	}

	public void setInheritanceModeDictId(Long inheritanceModeDictId) {
		this.inheritanceModeDictId = inheritanceModeDictId;
	}

	public Long getLocusId() {
		return locusId;
	}

	public void setLocusId(Long locusId) {
		this.locusId = locusId;
	}

	public Long getMutationNatureDictId() {
		return mutationNatureDictId;
	}

	public void setMutationNatureDictId(Long mutationNatureDictId) {
		this.mutationNatureDictId = mutationNatureDictId;
	}

	public Long getAlleleMakerId() {
		return alleleMakerId;
	}

	public void setAlleleMakerId(Long alleleMakerId) {
		this.alleleMakerId = alleleMakerId;
	}

	public GenotypeTest getGenotypeTest() {
		return genotypeTest;
	}

	public void setGenotypeTest(GenotypeTest genotypeTest) {
		this.genotypeTest = genotypeTest;
	}

	public Long getGenotypeTestId() {
		return genotypeTestId;
	}

	public void setGenotypeTestId(Long genotypeTestId) {
		this.genotypeTestId = genotypeTestId;
	}

	public String getExternalMakerName() {
		return externalMakerName;
	}

	public void setExternalMakerName(String externalMakerName) {
		this.externalMakerName = externalMakerName;
	}

	public Collection<BinaryFile> getGelImages() {
		return gelImages;
	}

	public void setGelImages(Collection<BinaryFile> gelImages) {
		this.gelImages = gelImages;
	}

	public Collection<AlleleSynonym> getAlleleSynonyms() {
		return alleleSynonyms;
	}

	public void setAlleleSynonyms(Collection<AlleleSynonym> alleleSynonyms) {
		this.alleleSynonyms = alleleSynonyms;
	}

	public String getDescriptionUri() {
		return this.descriptionUri;
	}

	public void setDescriptionUri(String descriptionUri) {
		this.descriptionUri = descriptionUri;
	}
}
