/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.alleles.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.AlleleSynonym;
import uk.ac.sanger.mig.alleles.forms.AlleleInfoForm;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.controllers.AbstractFormChangeController;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.gene.Build;
import uk.ac.sanger.mig.gene.services.BuildService;
import uk.ac.sanger.mig.personnel.Person;

public class AlleleInfoFormController extends AbstractFormChangeController {

	private AlleleService alleleService;
	private DictionaryService dictionaryService;
	private BuildService buildService;
	private String successViewToEditCellLine;
	private ComponentMapService componentMapService;
	private BinaryFileService binaryFileService;

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setAlleleService(AlleleService alleleService) {
		this.alleleService = alleleService;
	}

	public void setSuccessViewToEditCellLine(String successViewToEditCellLine) {
		this.successViewToEditCellLine = successViewToEditCellLine;
	}

	public void setBuildService(BuildService buildService) {
		this.buildService = buildService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		AlleleInfoForm alleleInfoForm = (AlleleInfoForm)command;
		String changeRequest = ServletRequestUtils.getRequiredStringParameter(request, "changeForm");
		String changeRequestValue = ServletRequestUtils.getRequiredStringParameter(request, "changeFormValue");

		if(changeRequest.equals("deleteFile")) {
			Person user = RequestUtils.getUser(request);
			deleteFile(Long.valueOf(changeRequestValue), user, alleleInfoForm);
		}

		if(changeRequest.equals("addSynonym")) {
			addSynonym(alleleInfoForm);
			alleleInfoForm.getAllele().getAlleleSynonyms();
			alleleInfoForm.setAlleleSynonym("");
		}

		if(changeRequest.equals("deleteSynonym")) {
			deleteSynonym(changeRequestValue, alleleInfoForm);
		}
	}

	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		AlleleInfoForm alleleInfoForm = (AlleleInfoForm)createCommand();

		boolean comingFromEditCellLine = ServletRequestUtils.getBooleanParameter(httpServletRequest, "edit_cell_line", false);
		alleleInfoForm.setComingFromEditCellLine( comingFromEditCellLine );

		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");
		Allele allele = null; //parent

		Person currentUser = (Person)httpServletRequest.getSession().getAttribute("user");

		if(id != null) {
			//ok, existing allele id provided, so fetch for edit
			ComponentMap componentMap = componentMapService.getComponentMap("editAllele");
			allele = alleleService.fetchById(id, componentMap);
		}
		else {//else make new allele
			allele = new Allele();
			allele.setMultiGeneRegion(false);
			allele.setAlleleMaker(currentUser);
			allele.setAlleleMakerId(currentUser.getId());
			allele.setAlleleSynonyms(new ArrayList<AlleleSynonym>());
		}

		alleleService.inflateAlleleComponents(allele);
		alleleInfoForm.setAllele(allele);

		if(alleleInfoForm.getBinaryFile() == null) {
			alleleInfoForm.setBinaryFile(new BinaryFile());
		}

		return alleleInfoForm;
	}

	@Override
	protected Map referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		AlleleInfoForm form = (AlleleInfoForm)command;
		Allele allele = form.getAllele();

		Map<String,Object> referenceData = new HashMap<String,Object>();

		Collection<DictionaryEntry> inheritanceModes = dictionaryService.fetchForEdit(DictionaryEnum.ALLELE_INHERITANCE_MODE_DICT,  allele.getInheritanceModeDictId());
		referenceData.put("inheritanceModes", inheritanceModes);

		Collection<DictionaryEntry> mutationNatures = dictionaryService.fetchForEdit(DictionaryEnum.ALLELE_MUTATION_NATURE_DICT, allele.getMutationNatureDictId());
		referenceData.put("mutationNatures", mutationNatures);

		Collection<DictionaryEntry> mutationClasses = dictionaryService.fetchForEdit(DictionaryEnum.ALLELE_MUTATION_CLASS_DICT, allele.getMutationClassDictId());
		referenceData.put("mutationClasses", mutationClasses);

		Collection<Build> allGeneBuilds = buildService.fetchAll(null);
		referenceData.put("allGeneBuilds", allGeneBuilds);

		//Yes/No list
		Map<Boolean, String> yesNo = new HashMap<Boolean, String>();
		yesNo.put(true,"Yes");
		yesNo.put(false,"No");
		referenceData.put("yesNo", yesNo);

		//Yes/No/Unknown list
		Map<String, String> yesNoUnknown = new HashMap<String, String>();
		yesNoUnknown.put("Yes","Yes");
		yesNoUnknown.put("No","No");
		yesNoUnknown.put("Unknown","Unknown");
		referenceData.put("yesNoUnknown", yesNoUnknown);

		String chromosomeList = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,X,Y,Unknown";
		referenceData.put("chromosomeList", chromosomeList);

		return referenceData;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response,
			Object commandObject, BindException errors) throws Exception {
		AlleleInfoForm alleleInfoForm = (AlleleInfoForm)commandObject;
		Allele allele = alleleInfoForm.getAllele();

		boolean isNewRequest = (allele.getId() == null);

		Long requestId = alleleService.storeComplete(alleleInfoForm, RequestUtils.getUser(request));

		//check if this action evolved from Cell Line page and redirect accordingly with
		//allele params in HTTP query string
		String currentSuccessView = getSuccessView();

		if(alleleInfoForm.isComingFromEditCellLine()){
			currentSuccessView = successViewToEditCellLine + "?alleleId=" + allele.getId();
			return new ModelAndView( currentSuccessView );
		}else{

			String queryString = "?currentRecordId=" + requestId;

			// for new records filter by status and sort by descending ID
			if(isNewRequest) {
				queryString += "&bypassForm=true&newQuery=true&orderBy=id&orderDesc=true";
			}
			// for existing records, return to the last list page the user was on
			else {
				queryString += "&bypassForm=true&newQuery=false&retrieveFromSession=true";
			}

			return new ModelAndView(getSuccessView() + queryString);
		}
	}

	private void deleteFile(Long fileId, Person user, AlleleInfoForm form) {
		BinaryFile file = binaryFileService.fetchById(fileId, false, null);
		binaryFileService.delete(file, user);
		form.getAllele().getGelImages().remove(file);
	}

	private void addSynonym(AlleleInfoForm alleleInfoForm){
		//////////////////////////////////////////
		//TODO NEED TO CHECK FOR DUPLICATE NAMES//
		//////////////////////////////////////////
		String alleleSynonymsString = alleleInfoForm.getAlleleSynonym();

		Collection<AlleleSynonym> alleleSynonyms = alleleInfoForm.getAllele().getAlleleSynonyms();

		if(alleleSynonymsString != null){

			boolean existsInList;

			StringTokenizer stringTokenizer = new StringTokenizer(alleleSynonymsString, ",");

			while (stringTokenizer.hasMoreTokens()){

				existsInList = false;

				AlleleSynonym newAlleleSynonym = new AlleleSynonym();
				newAlleleSynonym.setAlleleSynonymText(stringTokenizer.nextToken().trim());

				Iterator iter = alleleSynonyms.iterator();

				while(iter.hasNext()){
					AlleleSynonym alleleSynonym = (AlleleSynonym)iter.next();
					if(alleleSynonym.getAlleleSynonymText().equals(newAlleleSynonym.getAlleleSynonymText())){
						existsInList = true;
					}
				}
				if(!existsInList){
					alleleSynonyms.add(newAlleleSynonym);
				}
			}

			alleleInfoForm.getAllele().setAlleleSynonyms(alleleSynonyms);
		}
	}

	private void deleteSynonym(String alleleSynonymToBeRemoved, AlleleInfoForm alleleInfoForm){
		Iterator iter = alleleInfoForm.getAllele().getAlleleSynonyms().iterator();

		ArrayList<AlleleSynonym> newSynonymList = new ArrayList<AlleleSynonym>();

		while(iter.hasNext()){
			AlleleSynonym alleleSynonym = (AlleleSynonym)iter.next();
			if(!alleleSynonym.getAlleleSynonymText().equals(alleleSynonymToBeRemoved)){
				newSynonymList.add(alleleSynonym);
			}
		}

		alleleInfoForm.getAllele().setAlleleSynonyms(newSynonymList);
	}
}







