/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.alleles.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.AlleleSearchBean;
import uk.ac.sanger.mig.alleles.dao.rowmappers.AlleleListViewRowMapper;
import uk.ac.sanger.mig.alleles.view.AlleleListView;
import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.charting.DatabaseChartData;
import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;

public class AlleleDaoSpringImpl extends AbstractDaoSpringImpl<Allele> implements AlleleDao {

	private PaginatedSearchUtils<Allele> paginator;
	private PaginatedSearchUtils<AlleleListView> listViewPaginator;
	private AlleleListViewRowMapper alleleListViewRowMapper;

	public void setPaginator(PaginatedSearchUtils<Allele> paginator) {
		this.paginator = paginator;
	}

	public void setListViewPaginator(
			PaginatedSearchUtils<AlleleListView> listViewPaginator) {
		this.listViewPaginator = listViewPaginator;
	}

	public void setAlleleListViewRowMapper(
			AlleleListViewRowMapper alleleListViewRowMapper) {
		this.alleleListViewRowMapper = alleleListViewRowMapper;
	}

	public Collection<AlleleListView> search(AlleleSearchBean searchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		Allele allele = searchBean.getAllele();

		StringBuffer sql = new StringBuffer();
		sql.append(" select * from typ_modified_allele_list_vw a where 1=1 ");

		if(allele.getMutationNatureDictId() != null) {
			sql.append(" and a.mutation_nature_id = ? ");
			bindVariables.add(allele.getMutationNatureDictId());
		}

		if(allele.getInheritanceModeDictId() != null) {
			sql.append(" and a.inheritance_mode_id = ? ");
			bindVariables.add(allele.getInheritanceModeDictId());
		}



		if(searchBean.getFilterByOption() != null && !searchBean.getFilterByOption().equals("") &&
			searchBean.getFilterByValue() != null && !searchBean.getFilterByValue().equals("")) {

			AlleleSearchBean.FilterOptions filterOption = Enum.valueOf(AlleleSearchBean.FilterOptions.class, searchBean.getFilterByOption());

			switch (filterOption) {
			case ENTERED_BY:
				sql.append(" and upper(a.creator_name) like '%'||upper(?)||'%' ");
				bindVariables.add(searchBean.getFilterByValue());
				break;
			case GENE_ID:
				try {
					int geneId = Integer.parseInt(searchBean.getFilterByValue());
					sql.append(" and a.gene_id = ? ");
					bindVariables.add(geneId);
				}
				catch(NumberFormatException e) {
					sql.append(" and a.gene_id = 0 ");
				}
				break;
			case GENE_NAME:
				sql.append(" and (upper(a.gene_name) like '%'||upper(?)||'%' or upper(a.official_name_superscript) like '%'||upper(?)||'%')");
				bindVariables.add(searchBean.getFilterByValue());
				bindVariables.add(searchBean.getFilterByValue());
				break;
			case ALLELE_SYNONYM:
				sql.append(" and (upper(a.allele_synonyms) like '%'||upper(?)||'%')");
				bindVariables.add(searchBean.getFilterByValue());
				break;
			}

		}

		if(searchBean.getOrderBy() != null && searchBean.getOrderBy().length() > 0) {
			sql.append(" order by " + searchBean.getOrderBy());
			if(searchBean.isOrderDesc()) {
				sql.append(" desc ");
			}
		}
		else {
			sql.append(" order by a.id ");
		}

		return listViewPaginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, alleleListViewRowMapper);
	}

	public Allele fetchBySuperscriptAndGeneId(Long geneId, String officialName ) {
		//look for allele uniquely identified by combo of official name and gene
		StringBuffer sql = new StringBuffer();
		sql.append(" SELECT * FROM TYP_MODIFIED_ALLELE a where 1=1 ");
		sql.append(" and a.gene_id = ? and upper(a.official_name_superscript) = upper(?)");

		Allele allele = null;

		try {

			allele = getSimpleJdbcTemplate().queryForObject(sql.toString(), getRowMapper(),
					 new Object[] {geneId, officialName});

		} catch(IncorrectResultSizeDataAccessException e) {}

		return allele;

	}

	public Collection<Allele> fetch(AlleleSearchBean searchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		Allele allele = searchBean.getAllele();

		StringBuffer sql = new StringBuffer();
		sql.append(" SELECT * FROM TYP_MODIFIED_ALLELE a where 1=1 ");

		if(allele.getMutationNatureDictId() != null) {
			sql.append(" and a.mutation_nature_id = ? ");
			bindVariables.add(allele.getMutationNatureDictId());
		}

		if(allele.getInheritanceModeDictId() != null) {
			sql.append(" and a.inheritance_mode_id = ? ");
			bindVariables.add(allele.getInheritanceModeDictId());
		}

		if(searchBean.getOrderBy() != null && searchBean.getOrderBy().length() > 0) {
			sql.append(" order by " + searchBean.getOrderBy());
			if(searchBean.isOrderDesc()) {
				sql.append(" desc ");
			}
		}
		else {
			sql.append(" order by id ");
		}

		return paginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, getRowMapper());
	}

	public Collection<AlleleListView> lookup(String geneOrAlleleName) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		geneOrAlleleName= geneOrAlleleName.replace('*','%');
		geneOrAlleleName = geneOrAlleleName.toUpperCase();
		geneOrAlleleName += "%";

		String sql = "select a.* ";
		sql += " from ";
		sql += "   typ_modified_allele_list_vw a ";
		sql += " where ";
		sql += "   upper(a.official_name_superscript) like ? ";
		sql += "   or upper(a.internal_name) like ? ";
		sql += "   or exists (select null from gnm_gene_alias_mv g where g.gene_id = a.gene_id and g.uc_name like ?) ";
		sql += "   or exists (select null from gnm_gene g where g.id = a.gene_id and g.uc_primary_name like ?) ";
		sql += " order by a.short_allele_symbol ";

		bindVariables.add(geneOrAlleleName);
		bindVariables.add(geneOrAlleleName);
		bindVariables.add(geneOrAlleleName);
		bindVariables.add(geneOrAlleleName);

		Collection<AlleleListView> alleles = getSimpleJdbcTemplate().query(sql, alleleListViewRowMapper, bindVariables.toArray());
		return (alleles == null) ? new ArrayList<AlleleListView>() : alleles;
	}

	public ChartData fetchMutationNatureChartData() {
		String sql = "select b.NAME label, a.mutation_nature_id extra_field, count(*) data0 from TYP_MODIFIED_ALLELE a, TYP_MUTATION_NATURE_DICT b WHERE a.mutation_nature_id = b.ID group by b.NAME, a.mutation_nature_id order by data0 ";

		DatabaseChartData data = new DatabaseChartData(1);
		getJdbcTemplate().query(sql, data);

		return data;
	}

	public Collection<Allele> fetchAll() {
		String sql = "SELECT * FROM TYP_MODIFIED_ALLELE ORDER BY ID";
		Collection<Allele> alleles = getSimpleJdbcTemplate().query(sql, getRowMapper());
		return (alleles == null) ? new ArrayList<Allele>() : alleles;
	}

	/**
	 * Used within GraphReporting - mng.
	 */
	public Collection<Allele> fetchAllByGeneId(Long geneId) {
		String sql = "SELECT * FROM TYP_MODIFIED_ALLELE WHERE GENE_ID = ? ";
		Collection<Allele> alleles = getSimpleJdbcTemplate().query(sql, getRowMapper(),geneId);
		return (alleles == null) ? new ArrayList<Allele>() : alleles;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Allele allele) {

		sql.addStringParam(allele.getDescriptionUri(), "DESCRIPTION_URI");

		sql.addLongParam(allele.getLocusId(), "LOCUS_ID");
		sql.addLongParam(allele.getMutationNatureDictId(), "MUTATION_NATURE_ID");
		sql.addLongParam(allele.getMutationClassDictId(), "MUTATION_CLASS_DICT_ID");
		sql.addStringParam(allele.getInternalName(), "INTERNAL_NAME");
		sql.addStringParam(allele.getOfficialNameSuperscript(), "OFFICIAL_NAME_SUPERSCRIPT");
		sql.addBooleanParam(allele.isMultiGeneRegion(), "MULTI_GENE_REGION");
		sql.addStringParam(allele.getWhetherHomozygousLethal(), "IS_HOMOZYGOUS_LETHAL");
		sql.addLongParam(allele.getInheritanceModeDictId(), "INHERITANCE_MODE_ID");
		sql.addStringParam(allele.getHasAssociatedPhenotype(), "HAS_ASSOCIATED_PHENOTYPE");
		sql.addStringParam(allele.getPhenotypeDescription(), "PHENOTYPE_DESCRIPTION");
		sql.addStringParam(allele.getMultiGeneRegionDescrip(), "MULTI_GENE_REGION_DESCRIP");
		sql.addStringParam(allele.getAlleleDescription(), "ALLELE_DESCRIPTION");
		sql.addStringParam(allele.getShortAlleleSymbol(), "SHORT_ALLELE_SYMBOL");
		sql.addLongParam(allele.getMigGeneId(), "GENE_ID");
		sql.addStringParam(allele.getMultipleGeneName(), "MULTIPLE_GENE_REGION_NAME");
		sql.addLongParam(allele.getAlleleMakerId(), "ALLELE_MAKER_ID");
		sql.addStringParam(allele.getExternalMakerName(), "MAKER_NAME");
		sql.addLongParam(allele.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(allele.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(allele.getEditedBy(), "EDITED_BY");
		sql.addDateParam(allele.getEditDate(), "EDIT_DATE");
		sql.addLongParam(allele.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(allele.getId(), "ID");
	}
}
