/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.alleles.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.AlleleSearchBean;
import uk.ac.sanger.mig.alleles.AlleleSynonym;
import uk.ac.sanger.mig.alleles.GenotypeTest;
import uk.ac.sanger.mig.alleles.dao.AlleleDao;
import uk.ac.sanger.mig.alleles.forms.AlleleInfoForm;
import uk.ac.sanger.mig.alleles.view.AlleleListView;
import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.gene.Locus;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.services.LocusService;
import uk.ac.sanger.mig.gene.services.MigGeneService;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class AlleleServiceImpl extends AbstractServiceImpl<Allele> implements AlleleService {

	private AlleleDao alleleDao;
	private PersonService personService;
	private GenotypeTestService genotypeTestService;
	private MigGeneService migGeneService;
	private LocusService locusService;
	private BinaryFileService binaryFileService;
	private AlleleSynonymService alleleSynonymService;

	private Long defaultAlleleMakerId;//Bill Skarnes
	private Long defaultInheritanceModeDictId;//unknown
	private String defaultAlleleDescription;//
	private String defaultHomozygousLethal;//unknown
    private Long defaultMutationClassDictId; //genetically modified
    private Long defaultMutationNatureDictId; //Targeted
    private String defaultHasAssociatedPhenotype; //unknown

    public static final String SUPERSCRIPT_PATTERN = "(tm\\d+[a-z]).*";

    public Allele fetchBySuperscriptAndGeneId(Long geneId, String officialName, ComponentMap requiredComponents) {

    	Allele allele = alleleDao.fetchBySuperscriptAndGeneId(
    			geneId, officialName);

		if(allele != null)
			attachComponents(allele, requiredComponents);

		return allele;

    }

    public Allele deriveEpdAllele(Allele team87Allele) throws InvalidTeam87InfoException {

    	if( team87Allele == null )
    		throw new IllegalArgumentException("New Allele cannot be null.");

    	//first check if already in db by geneId and official name
    	Allele migAllele = fetchBySuperscriptAndGeneId(
				team87Allele.getMigGeneId(),
				team87Allele.getOfficialNameSuperscript(),
				ComponentMap.of(Allele.Component.GENE ));

    	Allele workingAllele = null;

    	//use mig allele instead of team87 allele if it exists
    	if( migAllele != null ) {

       		logger.info("EPD Auto-Create Found existing allele based on gene id/official name: "
    				+ migAllele.getMigGeneId() + "/" + migAllele.getOfficialNameSuperscript());

       		workingAllele = migAllele;

    	} else {//otherwise attach gene and populate rest of defaults for epd allele

    		if(team87Allele.getMigGeneId() != null) {
    			populateGene( team87Allele );
    		}

    		populateFields( team87Allele );
	    	workingAllele = team87Allele;
    	}

    	return workingAllele;
    }

    private void populateGene(Allele allele) throws InvalidTeam87InfoException {

    	MigGene gene = fetchGene(allele, null);

    	if( gene == null ) {
    		throw new InvalidTeam87InfoException("Gene ID could not be resolved to a MigGene: " + allele.getMigGeneId());
    	}

    	allele.setMigGene( gene  );
    }

    private void populateFields(Allele allele) {

    	defaultAlleleDescription = (defaultAlleleDescription == null) ? "" : defaultAlleleDescription;
		//appends allele descrip to default descrip
		allele.setAlleleDescription(
				defaultAlleleDescription + allele.getAlleleDescription() );

    	if( allele.getAlleleMakerId() == null ) {
    		allele.setAlleleMakerId( defaultAlleleMakerId );
    	}

    	if( allele.getInheritanceModeDictId() == null ) {

    		allele.setInheritanceModeDictId( defaultInheritanceModeDictId );
    	}

    	if( allele.getWhetherHomozygousLethal() == null ) {

    		allele.setWhetherHomozygousLethal( defaultHomozygousLethal );
    	}

    	if( allele.getMutationClassDictId() == null ) {

    		allele.setMutationClassDictId( defaultMutationClassDictId );
    	}

    	if( allele.getMutationNatureDictId() == null ) {

    		allele.setMutationNatureDictId( defaultMutationNatureDictId );
    	}

    	if( allele.getHasAssociatedPhenotype() == null ) {

    		allele.setHasAssociatedPhenotype( defaultHasAssociatedPhenotype );
    	}

    }

    public void setDefaultMutationClassDictId(Long defaultMutationClassDictId) {
		this.defaultMutationClassDictId = defaultMutationClassDictId;
	}

	public void setDefaultMutationNatureDictId(Long defaultMutationNatureDictId) {
		this.defaultMutationNatureDictId = defaultMutationNatureDictId;
	}

	public void setAlleleDao(AlleleDao alleleDao) {
		this.alleleDao = alleleDao;
	}

	public ChartData fetchMutationNatureChartData() {
		return alleleDao.fetchMutationNatureChartData();
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setGenotypeTestService(GenotypeTestService genotypeTestService) {
		this.genotypeTestService = genotypeTestService;
	}

	public void setMigGeneService(MigGeneService migGeneService) {
		this.migGeneService = migGeneService;
	}

	public void setLocusService(LocusService locusService) {
		this.locusService = locusService;
	}

	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}

	public Allele fetchById(Long alleleId, ComponentMap requiredComponents) {
		Allele allele = alleleDao.fetchById(alleleId);

		if(allele != null)
			attachComponents(allele, requiredComponents);

		return allele;
	}

	@Transactional
	public Collection<AlleleListView> search(AlleleSearchBean alleleSearchBean) {
		return alleleDao.search(alleleSearchBean);
	}

	@Transactional
	public Long store(Allele allele, Person user) {
		if(allele == null) {
			throw new RuntimeException("Cannot save a null allele");
		}

		return alleleDao.store(allele, user);
	}

	@Transactional
	public Long storeComplete(AlleleInfoForm alleleForm, Person user) {
		Allele allele = alleleForm.getAllele();

		if(allele == null) {
			throw new RuntimeException("Cannot save a null allele");
		}

		if(allele.isMultiGeneRegion()){
			allele.setMigGene(null);
			allele.setMigGeneId(null);
		}
		else {
			allele.setMultiGeneRegionDescrip(null);
			allele.setMultipleGeneName(null);
			allele.setLocus(null);
			allele.setLocusId(null);
		}

		if(allele.getLocus() != null  && allele.getMigGeneId() == null)
			allele.setLocusId(locusService.storeAuthenticated(allele.getLocus(), user));

		Long alleleId = alleleDao.store(allele, user);

		if(allele.getGenotypeTest() != null){
			allele.getGenotypeTest().setAlleleId(alleleId);
			genotypeTestService.storeComplete(allele.getGenotypeTest(), user);
		}

		if(alleleForm.getBinaryFile().getSize() > 0) {
			BinaryFile file = alleleForm.getBinaryFile();
			file.setForeignTableName(TableMetadataEnum.TYP_MODIFIED_ALLELE);
			file.setForeignKeyId(alleleId);
			binaryFileService.store(alleleForm.getBinaryFile(), user);
		}

		if(allele.getAlleleSynonyms() != null){
			Collection<AlleleSynonym> databaseAlleleSynonyms = alleleSynonymService.fetchByAlleleId(alleleId, new ComponentMap());
			Iterator databaseAlleleSynonymsIterator = databaseAlleleSynonyms.iterator();
			Collection<AlleleSynonym> newAlleleSynonyms = allele.getAlleleSynonyms();

			if(!databaseAlleleSynonyms.isEmpty()){
				//There are some allele synonyms already stored
				while(databaseAlleleSynonymsIterator.hasNext()){

					AlleleSynonym databaseAlleleSynonym = (AlleleSynonym)databaseAlleleSynonymsIterator.next();
					// Delete any allele synonyms in the database that the user has deleted on the JSP
					if(!newAlleleSynonyms.contains(databaseAlleleSynonym)){
						alleleSynonymService.deleteAlleleSynonym(databaseAlleleSynonym, user);
					}
				}
				//Now store what the synonyms on the JSP
				Iterator newAlleleSynonymsIterator = allele.getAlleleSynonyms().iterator();
				while (newAlleleSynonymsIterator.hasNext()){
					AlleleSynonym newAlleleSynonym = (AlleleSynonym)newAlleleSynonymsIterator.next();
					newAlleleSynonym.setAlleleId(alleleId);
					alleleSynonymService.storeAuthenticated(newAlleleSynonym, user);
				}
			}else{
				//First allele synonym(s) to be stored
				Iterator newAlleleSynonymsIterator = allele.getAlleleSynonyms().iterator();
				while (newAlleleSynonymsIterator.hasNext()){
					AlleleSynonym newAlleleSynonym = (AlleleSynonym)newAlleleSynonymsIterator.next();
					newAlleleSynonym.setAlleleId(alleleId);
					alleleSynonymService.storeAuthenticated(newAlleleSynonym, user);
				}
			}
		}
		return alleleId;
	}

	@Override
	protected void attachComponents(Allele allele, ComponentMap requiredComponents) {
		if(allele == null) {
			throw new RuntimeException("Cannot attach components to null allele");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Allele.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			if(fetchAll || requiredComponents.containsKey(Allele.Component.ALLELE_MAKER)) {
				allele.setAlleleMaker(fetchAlleleMaker(allele, requiredComponents.get(Allele.Component.ALLELE_MAKER)));
			}

			if(fetchAll || requiredComponents.containsKey(Allele.Component.GENOTYPE_TESTS)) {
				allele.setGenotypeTest(fetchGenotypeTest(allele, requiredComponents.get(Allele.Component.GENOTYPE_TESTS)));
			}

			if(fetchAll || requiredComponents.containsKey(Allele.Component.GENE)) {
				allele.setMigGene(fetchGene(allele, requiredComponents.get(Allele.Component.GENE)));
			}

			if(fetchAll || requiredComponents.containsKey(Allele.Component.LOCUS)) {
				allele.setLocus(fetchLocus(allele, requiredComponents.get(Allele.Component.LOCUS)));
			}

			if(fetchAll || requiredComponents.containsKey(Allele.Component.GEL_IMAGES)) {
				allele.setGelImages(fetchGelImages(allele, requiredComponents.get(Allele.Component.GEL_IMAGES)));
			}

			if(fetchAll || requiredComponents.containsKey(Allele.Component.ALLELE_SYNONYMS)) {
				allele.setAlleleSynonyms(fetchAlleleSynonyms(allele, requiredComponents.get(Allele.Component.ALLELE_SYNONYMS)));
			}
		}
	}

	private Collection<BinaryFile> fetchGelImages(Allele allele, ComponentMap requiredComponents){
		return binaryFileService.fetchByTableRef(TableMetadataEnum.TYP_MODIFIED_ALLELE, allele.getId(), false, requiredComponents);
	}

	private Person fetchAlleleMaker(Allele allele, ComponentMap requiredComponents){
		if(allele.getAlleleMakerId() != null) {
			return personService.fetchById(allele.getAlleleMakerId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	private GenotypeTest fetchGenotypeTest(Allele allele, ComponentMap requiredComponents){
		return genotypeTestService.fetchGenotypeTestByAlleleId(allele.getId(), requiredComponents);
	}


	private MigGene fetchGene(Allele allele, ComponentMap requiredComponents){
		if(allele.getMigGeneId() != null) {
			return migGeneService.fetchById(allele.getMigGeneId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	private Locus fetchLocus(Allele allele, ComponentMap requiredComponents){
		if(allele.getLocusId() != null) {
			return locusService.fetchById(allele.getLocusId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	public Collection<AlleleSynonym> fetchAlleleSynonyms(Allele allele, ComponentMap requiredComponents) {
		return alleleSynonymService.fetchByAlleleId(allele.getId(), requiredComponents);
	}

	public Collection<Allele> fetch(AlleleSearchBean alleleSearchBean, ComponentMap requiredComponents) {
		Collection<Allele> alleles = alleleDao.fetch(alleleSearchBean);
		if(alleles != null)
			attachComponents(alleles, requiredComponents);

		return alleles;
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateAlleleComponents(Allele allele) {
		if(allele == null) {
			throw new RuntimeException("Allele cannot be null when inflating components.");
		}

		// set genotype test
		if(allele.getGenotypeTest() == null) {
			GenotypeTest genotypeTest = new GenotypeTest();
			genotypeTestService.inflateGenotypeTestComponents(genotypeTest);
			allele.setGenotypeTest(genotypeTest);
		}

		// set person
		if(allele.getAlleleMaker() == null){
			Person person = new Person();
			personService.inflatePersonComponents(person);
			allele.setAlleleMaker(person);
		}

		// set inheritance mode
		if(allele.getInheritanceMode() == null)
			allele.setInheritanceMode(new DictionaryEntry());

		// set mutation nature
		if(allele.getMutationNature() == null)
			allele.setMutationNature(new DictionaryEntry());

		// set migGene
		if(allele.getMigGene() == null){
			MigGene migGene = new MigGene();
			migGeneService.inflateMigGeneComponents(migGene);
			allele.setMigGene(migGene);
		}

		// set locus
		if(allele.getLocus()== null){
			Locus locus = new Locus();
			locusService.inflateLocusComponents(locus);
			allele.setLocus(locus);
		}

		// set gel images
		if(allele.getGelImages() == null) {
			allele.setGelImages(new ArrayList<BinaryFile>());
		}
	}

	public Collection<AlleleListView> lookup(String geneOrAlleleName) {
		return alleleDao.lookup(geneOrAlleleName);
	}

	public void setAlleleSynonymService(AlleleSynonymService alleleSynonymService) {
		this.alleleSynonymService = alleleSynonymService;
	}

	public void setDefaultAlleleDescription(String defaultAlleleDescription) {
		this.defaultAlleleDescription = defaultAlleleDescription;
	}

	public void setDefaultAlleleMakerId(Long defaultAlleleMakerId) {
		this.defaultAlleleMakerId = defaultAlleleMakerId;
	}

	public void setDefaultHasAssociatedPhenotype(
			String defaultHasAssociatedPhenotype) {
		this.defaultHasAssociatedPhenotype = defaultHasAssociatedPhenotype;
	}

	public void setDefaultHomozygousLethal(String defaultHomozygousLethal) {
		this.defaultHomozygousLethal = defaultHomozygousLethal;
	}

	public void setDefaultInheritanceModeDictId(Long defaultInheritanceModeDictId) {
		this.defaultInheritanceModeDictId = defaultInheritanceModeDictId;
	}


	public Collection<Allele> fetchAllByGeneId(Long geneId) {
		return alleleDao.fetchAllByGeneId(geneId);
	}





}