/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.alleles.services;

import java.util.Collection;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.GenotypeTest;
import uk.ac.sanger.mig.alleles.PrimerPair;
import uk.ac.sanger.mig.alleles.dao.GenotypeTestDao;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.personnel.Person;

public class GenotypeTestServiceImpl extends AbstractServiceImpl<GenotypeTest> implements GenotypeTestService {

	private GenotypeTestDao genotypeTestDao;
	private AlleleService alleleService;
	private PrimerPairService primerPairService;
	private static final Long WILDTYPE_PRIMER_PAIR = 1L;
	private static final Long ALLELE_PRIMER_PAIR = 2L;


	public void setAlleleService(AlleleService alleleService) {
		this.alleleService = alleleService;
	}

	public void setPrimerPairService(PrimerPairService primerPairService) {
		this.primerPairService = primerPairService;
	}

	public void setGenotypeTestDao(GenotypeTestDao genotypeTestDao) {
		this.genotypeTestDao = genotypeTestDao;
	}

	public Long storeAuthenticated(GenotypeTest genotypeTest, Person user) {
		if(genotypeTest == null) {
			throw new RuntimeException("Cannot save a null genotypeTest");
		}

		return genotypeTestDao.store(genotypeTest, user);
	}

	public Long storeComplete(GenotypeTest genotypeTest, Person user) {
		if(genotypeTest == null) {
			throw new RuntimeException("Cannot save a null genotypeTest");
		}

		genotypeTest.setId(genotypeTestDao.store(genotypeTest, user));

		if(genotypeTest.getWildtypePrimerPair() != null){
			genotypeTest.getWildtypePrimerPair().setGenotypeTestId(genotypeTest.getId());
			genotypeTest.getWildtypePrimerPair().setPrimerPairTypeId(WILDTYPE_PRIMER_PAIR);
			primerPairService.storeAuthenticated(genotypeTest.getWildtypePrimerPair(), user);
		}

		if(genotypeTest.getAllelePrimerPair() != null){
			genotypeTest.getAllelePrimerPair().setGenotypeTestId(genotypeTest.getId());
			genotypeTest.getAllelePrimerPair().setPrimerPairTypeId(ALLELE_PRIMER_PAIR);
			primerPairService.storeAuthenticated(genotypeTest.getAllelePrimerPair(), user);
		}

		return genotypeTest.getId();
	}

	public GenotypeTest fetchById(Long genotypeTestId, ComponentMap requiredComponents) {
		GenotypeTest genotypeTest = genotypeTestDao.fetchById(genotypeTestId);

		if(genotypeTest != null)
			attachComponents(genotypeTest, requiredComponents);

		return genotypeTest;
	}

	@Override
	protected void attachComponents(GenotypeTest genotypeTest, ComponentMap requiredComponents) {
		if(genotypeTest == null) {
			throw new RuntimeException("Cannot attach components to null genotypeTest");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(GenotypeTest.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(GenotypeTest.Component.PRIMER_PAIRS)) {
				genotypeTest.setWildtypePrimerPair(fetchWildtypePrimerPair(genotypeTest, requiredComponents.get(GenotypeTest.Component.PRIMER_PAIRS)));
				genotypeTest.setAllelePrimerPair(fetchAllelePrimerPair(genotypeTest, requiredComponents.get(GenotypeTest.Component.PRIMER_PAIRS)));
			}		
		}
	}

	public Allele fetchAllele(GenotypeTest genotypeTest, ComponentMap requiredComponents){
		if(genotypeTest.getId() != null) {
			return alleleService.fetchById(genotypeTest.getAlleleId(), requiredComponents);
		}
		else {
			return null;	
		}
	}

	public PrimerPair fetchWildtypePrimerPair(GenotypeTest genotypeTest, ComponentMap requiredComponents){
		Collection<PrimerPair> primerPairs =  primerPairService.fetchPrimerPairsByGenotypeTestId(genotypeTest.getId(), requiredComponents);
		if(primerPairs != null){
			PrimerPair wildtypePrimerPair = null;
			for(PrimerPair primerPair : primerPairs){
				if(primerPair.getPrimerPairTypeId() == WILDTYPE_PRIMER_PAIR)
					wildtypePrimerPair = primerPair;
			}
			return wildtypePrimerPair;
		}else
			return null;
	}

	public PrimerPair fetchAllelePrimerPair(GenotypeTest genotypeTest, ComponentMap requiredComponents){
		Collection<PrimerPair> primerPairs =  primerPairService.fetchPrimerPairsByGenotypeTestId(genotypeTest.getId(), requiredComponents);
		if(primerPairs != null){
			PrimerPair allelePrimerPair = null;
			for(PrimerPair primerPair : primerPairs){
				if(primerPair.getPrimerPairTypeId() == ALLELE_PRIMER_PAIR)
					allelePrimerPair = primerPair;
			}
			return allelePrimerPair;
		}else
			return null;
	}

	public GenotypeTest fetchGenotypeTestByAlleleId(Long alleleId, ComponentMap requiredComponents) {
		GenotypeTest genotypeTest = genotypeTestDao.fetchGenotypeTestByAlleleId(alleleId);

		if(genotypeTest != null)
			attachComponents(genotypeTest, requiredComponents);

		return genotypeTest;
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateGenotypeTestComponents(GenotypeTest genotypeTest) {
		if(genotypeTest == null) {
			throw new RuntimeException("GenotypeTest cannot be null when inflating components.");
		}

		if(genotypeTest.getWildtypePrimerPair() == null)
			genotypeTest.setWildtypePrimerPair(new PrimerPair());

		if(genotypeTest.getAllelePrimerPair() == null)
			genotypeTest.setAllelePrimerPair(new PrimerPair());
	}
}
