/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.alleles.validators;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.forms.AlleleInfoForm;
import uk.ac.sanger.mig.gene.Locus;

public class AlleleInfoFormValidator implements Validator {

	public boolean supports(Class clazz) {
		return AlleleInfoForm.class.equals(clazz);
	}

	public void validate(Object commandForm, Errors errors) {

		AlleleInfoForm alleleInfoForm = (AlleleInfoForm)commandForm;
		Allele allele = alleleInfoForm.getAllele();

    ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.officialNameSuperscript", null, "Please specify Allele Name.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.mutationNatureDictId", null, "Please specify Nature of Mutation.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.mutationClassDictId", null, "Please specify Mutation Class.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.inheritanceModeDictId", null, "Please specify a Mode of Inheritance.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.whetherHomozygousLethal", null, "Please specify if allele is Homozygous Lethal.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.hasAssociatedPhenotype", null, "Please specify if allele has an Associated Phenotype.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.shortAlleleSymbol", null, "Please specify a Short Allele Symbol");

		validatePrimerSequence(errors, "allele.genotypeTest.wildtypePrimerPair.fivePrimeForward", "WT 5' Forward", allele.getGenotypeTest().getWildtypePrimerPair().getFivePrimeForward());
		validatePrimerSequence(errors, "allele.genotypeTest.wildtypePrimerPair.threePrimeReverse", "WT 3' Reverse", allele.getGenotypeTest().getWildtypePrimerPair().getThreePrimeReverse());
		validatePrimerSequence(errors, "allele.genotypeTest.allelePrimerPair.fivePrimeForward", "Allele 5' Forward", allele.getGenotypeTest().getAllelePrimerPair().getFivePrimeForward());
		validatePrimerSequence(errors, "allele.genotypeTest.allelePrimerPair.threePrimeReverse", "Allele 3' Reverse", allele.getGenotypeTest().getAllelePrimerPair().getThreePrimeReverse());

		if(allele.isMultiGeneRegion()){

			Locus locus = allele.getLocus();

			//check chromosome name is entered
			ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.locus.chromosomeName", null, "Please specify chromosome or indicate if unknown.");
			// if chromosome name is available, check other locus details
			if(!errors.hasFieldErrors("allele.locus.chromosomeName") && !locus.getChromosomeName().equals("Unknown")) {

				// check that both start and end coords are entered
				if( (locus.getChromosomeStart() != null && !locus.getChromosomeStart().equals("") ) ||
					(locus.getChromosomeEnd() != null && !locus.getChromosomeEnd().equals("") ) ){
					ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.locus.chromosomeStart", null, "Please specify start coodinate for locus.");
					ValidationUtils.rejectIfEmptyOrWhitespace(errors, "allele.locus.chromosomeEnd", null, "Please specify end coodinate for locus.");
				}

			}
		}

		// if file description is supplied, then file to upload must also be supplied
		if(alleleInfoForm.getBinaryFile() != null
				&& alleleInfoForm.getBinaryFile().getFileDescription() != null && !alleleInfoForm.getBinaryFile().getFileDescription().equals("")
				&& alleleInfoForm.getBinaryFile().getId() == null && alleleInfoForm.getUploadFile().getSize() == 0) {
			ValidationUtils.rejectIfEmptyOrWhitespace(errors, "binaryFile.fileName", null, "If File Description is supplied, File Name is also a required field.");

		}
	}

	private void validatePrimerSequence(Errors errors, String fieldName, String primerName, String primerValue) {
		if(primerValue != null) {
			Pattern pattern = Pattern.compile("[^agctAGCT]");
			Matcher matcher = pattern.matcher(primerValue);
			if(matcher.find()) {
				errors.rejectValue(fieldName, null, primerName + ": Please enter A,G,C,T nucleotides only.");
			}
		}
	}

}
