/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.animalrequest;

import java.util.Date;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.Nullable;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.miprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.miprocessing.EmbryoTransferProcedure;
import uk.ac.sanger.mig.mousetracking.Room;
import uk.ac.sanger.mig.personnel.Person;

public class AnimalRequest extends AuthenticatedDataObject implements Nullable {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		PROJECT_LICENCE,
		DESTINATION_ROOM,
		REQUEST_OWNER
	}
	
	public enum Group {
		
		MICRO_INJECTION_EMBRYO_DONORS(     1L ),
		MICRO_INJECTION_EMBRYO_RECIPIENTS( 2L ),
		CRYO_PRESERVATION_EMBRYO_DONORS(   3L );
		
		private Long id;
		
		private Group(Long id) {
			this.id = id;
		}

		public Long getId() {
			return this.id;
		}
	}
	
	public static final Long REQUESTED_STATUS = 1L;
	public static final Long COMPLETED_STATUS = 5L;
	public static final Long CANCELLED_STATUS = 6L;
	public static final int    DEFAULT_NUMBER_DONORS          = 15;
	public static final int    DEFAULT_NUMBER_RECIPIENTS      = 15;
	
	private static final Long   DEFAULT_ROOM_ID                = 12L;
	private static final int    DEFAULT_NUMBER_IN_CAGE         = 5;
	private static final String DEVELOPMENT_ENDPOINT_PRENATAL  = "embryoAge";
	private static final String DEVELOPMENT_ENDPOINT_POSTNATAL = "ageOrWeight";		
	
	private Long projectLicenceId;
	private ProjectLicence projectLicence;	
	private Long statusDictId;
	private DictionaryEntry status;
	private Long strainId;
	private DictionaryEntry strain;
	private Long groupDictId;
	private DictionaryEntry groupDict;
	private Long destinationRoomId;
	private Room destinationRoom;
	private Long requestOwnerId;
	private Person requestOwner;
	private String requestReason;
	private String ageOrWeight;
	private String developmentEndpoint;
	private String ageOrWeightUnits;
	private String embryoAge;
	private int numMalesNeeded;
	private int numFemalesNeeded;
	private Date dateRequired;	
	private Date endDate;
	private String standingOrderFrequency;
	private String standingOrderDay;
	private int numberInCage;
	private boolean rederivationReq;
	private String comments;
	private String fundingSource;
	private boolean pseudoPregnantFemale;

	/*
	 *  (non-Javadoc)
	 * @see uk.ac.sanger.mig.common.Nullable#isNull()
	 */
	public boolean isBlank() {
		return false;
	}

	/**
	 * Factory method for creating default AnimalRequests for varous AnimalRequest Groups
	 * (eg, MI Donors, MI Recipients, Cryo Donors, etc.).  If request Group is null,
	 * an AnimalRequest is returned with just dateRequired, requesterId, and pplId info.
	 * provided as arguments.
	 * 
	 * AnimalRequest numFemales/numMales both default to 0.
	 * 
	 * @param requestGroup
	 * @param dateRequired
	 * @param requester
	 * @param pplId
	 * @return
	 */
	public static AnimalRequest createDefaultGroupRequest(Group requestGroup, Date dateRequired,
						Person requester, Long pplId) {	
		
		AnimalRequest animalRequest = null;	
		switch(requestGroup) {		
			case MICRO_INJECTION_EMBRYO_DONORS:
				animalRequest = buildMicroInjectionDonorRequest();
				break;			
			case MICRO_INJECTION_EMBRYO_RECIPIENTS:
				animalRequest = buildMicroInjectionRecipientRequest();
				break;			
			case CRYO_PRESERVATION_EMBRYO_DONORS:	
				animalRequest = buildCryoDonorRequest();
				break;			
			default: 		
				animalRequest = new AnimalRequest();
		}	
	
		animalRequest.setDateRequired( dateRequired );
		animalRequest.setProjectLicenceId( pplId ); 
		animalRequest.setRequestOwnerId( requester.getId() );	
		return animalRequest;
	}


	
	
	public DictionaryEntry  getStatus() {		
		return status;
	}

	public void setStatus(DictionaryEntry  status) {
		this.status = status;
	}

	public String getFundingSource() {
		return fundingSource;
	}

	public void setFundingSource(String fundingSource) {
		this.fundingSource = fundingSource;
	}
	
	public String getDevelopmentEndpoint() {	
		return developmentEndpoint;
	}

	public void setDevelopmentEndpoint(String ageOrWeightVersusEmbryoAge) {
		this.developmentEndpoint = ageOrWeightVersusEmbryoAge;
	}

	public String getAgeOrWeight() {
		return ageOrWeight;
	}


	public void setAgeOrWeight(String ageOrWeight) {
		this.ageOrWeight = ageOrWeight;
	}


	public String getAgeOrWeightUnits() {
		return ageOrWeightUnits;
	}

	public void setAgeOrWeightUnits(String ageOrWeightUnits) {
		this.ageOrWeightUnits = ageOrWeightUnits;
	}

	public String getComments() {
		return comments;
	}

	public void setComments(String comments) {
		this.comments = comments;
	}

	public Date getDateRequired() {
		return dateRequired;
	}
	
	public void setDateRequired(Date dateRequired) {
		this.dateRequired = dateRequired;
	}
	
	public Room getDestinationRoom() {
		return destinationRoom;
	}

	public void setDestinationRoom(Room destination) {
		this.destinationRoom = destination;
	}
	
	public int getNumberInCage() {
		return numberInCage;
	}

	public void setNumberInCage(int numberInCage) {
		this.numberInCage = numberInCage;
	}

	public int getNumFemalesNeeded() {
		return numFemalesNeeded;
	}

	public int getNumMalesNeeded() {
		return numMalesNeeded;
	}

	public String getEmbryoAge() {
		return embryoAge;
	}

	public void setEmbryoAge(String embryoAge) {
		this.embryoAge = embryoAge;
	}

	public ProjectLicence getProjectLicence() {
		return projectLicence;
	}

	public void setProjectLicence(ProjectLicence ppl) {
		this.projectLicence = ppl;
	}

	public boolean getRederivationReq() {
		return rederivationReq;
	}

	public void setRederivationReq(boolean rederivationReq) {		
		this.rederivationReq = rederivationReq;
	}

	public DictionaryEntry getStrain() {
		return strain;
	}

	public void setStrain(DictionaryEntry strain) {
		this.strain = strain;
	}


	public void setNumFemalesNeeded(int numFemalesNeeded) {
		this.numFemalesNeeded = numFemalesNeeded;
	}

	public void setNumMalesNeeded(int numMalesNeeded) {
		this.numMalesNeeded = numMalesNeeded;
	}

	public String getRequestReason() {
		return requestReason;
	}

	public void setRequestReason(String requestReason) {
		this.requestReason = requestReason;
	}	
	
	public String getStandingOrderDay() {
		return standingOrderDay;
	}

	public void setStandingOrderDay(String standingOrderDay) {
		this.standingOrderDay = standingOrderDay;
	}

	public String getStandingOrderFrequency() {
		return standingOrderFrequency;
	}

	public void setStandingOrderFrequency(String standingOrderFrequency) {
		this.standingOrderFrequency = standingOrderFrequency;
	}

	public Person getRequestOwner() {
		return requestOwner;
	}

	public void setRequestOwner(Person requestOwner) {
		this.requestOwner = requestOwner;
	}

	public Long getDestinationRoomId() {
		return destinationRoomId;
	}

	public void setDestinationRoomId(Long destinationRoomId) {
		this.destinationRoomId = destinationRoomId;
	}

	public Long getProjectLicenceId() {
		return projectLicenceId;
	}

	public void setProjectLicenceId(Long projectLicenceId) {
		this.projectLicenceId = projectLicenceId;
	}

	public Long getStatusDictId() {
		return statusDictId;
	}

	public void setStatusDictId(Long statusId) {
		this.statusDictId = statusId;
	}

	public Long getStrainId() {
		return strainId;
	}

	public void setStrainId(Long strainId) {
		this.strainId = strainId;
	}

	public Long getRequestOwnerId() {
		return requestOwnerId;
	}

	public void setRequestOwnerId(Long requestOwnerId) {
		this.requestOwnerId = requestOwnerId;
	}

	public boolean isPseudoPregnantFemale() {
		return pseudoPregnantFemale;
	}

	public void setPseudoPregnantFemale(boolean pseudoPregnantFemale) {
		this.pseudoPregnantFemale = pseudoPregnantFemale;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public DictionaryEntry getGroupDict() {
		return this.groupDict;
	}

	public void setGroupDict(DictionaryEntry groupDict) {
		this.groupDict = groupDict;
	}

	public Long getGroupDictId() {
		return this.groupDictId;
	}

	public void setGroupDictId(Long groupDictId) {
		this.groupDictId = groupDictId;
	}

	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// privates //////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////	
	private static AnimalRequest buildMicroInjectionDonorRequest() {
		AnimalRequest animalRequest = new AnimalRequest();
		
		animalRequest.setDestinationRoomId( DEFAULT_ROOM_ID );
		animalRequest.setDevelopmentEndpoint( DEVELOPMENT_ENDPOINT_PRENATAL ) ;
		animalRequest.setFundingSource( "Non-specific MI Requester" );
		animalRequest.setNumberInCage( DEFAULT_NUMBER_IN_CAGE ); 
		animalRequest.setStandingOrderFrequency( "One-time order" );
		animalRequest.setStatusDictId( REQUESTED_STATUS );
		
		animalRequest.setNumFemalesNeeded( DEFAULT_NUMBER_DONORS );
		animalRequest.setGroupDictId( Group.MICRO_INJECTION_EMBRYO_DONORS.getId() );
		animalRequest.setAgeOrWeight( ">6" );
		animalRequest.setAgeOrWeightUnits( "weeks" );
		animalRequest.setEmbryoAge( EmbryoHarvest.getDefaultDonorEmbryoAge() );
		animalRequest.setRequestReason( "Donor Females for Microinjection/Embryo Harvest" );
		return animalRequest;
	}
	
	private static AnimalRequest buildMicroInjectionRecipientRequest() {
		AnimalRequest animalRequest = new AnimalRequest();

		animalRequest.setDestinationRoomId( DEFAULT_ROOM_ID );
		animalRequest.setDevelopmentEndpoint( DEVELOPMENT_ENDPOINT_PRENATAL ) ;
		animalRequest.setFundingSource( "Non-specific MI Requester" );
		animalRequest.setNumberInCage( DEFAULT_NUMBER_IN_CAGE ); 
		animalRequest.setStandingOrderFrequency( "One-time order" );
		animalRequest.setStatusDictId( REQUESTED_STATUS );
		
		animalRequest.setNumFemalesNeeded( DEFAULT_NUMBER_RECIPIENTS );
		animalRequest.setGroupDictId( Group.MICRO_INJECTION_EMBRYO_RECIPIENTS.getId() );
		animalRequest.setAgeOrWeight( "20-25" );
		animalRequest.setAgeOrWeightUnits( "grams" );
		animalRequest.setEmbryoAge( EmbryoTransferProcedure.getDefaultRecipientEmbryoAge() );
		animalRequest.setStrainId( EmbryoTransferProcedure.getDefaultRecipientStrainId() );
		animalRequest.setPseudoPregnantFemale( true );
		animalRequest.setRequestReason( "Recipient Pseudo-pregnant Females for Microinjection/Embryo Transfer" );
		return animalRequest;
	}
	
	/*
	 * temp stub for cryo
	 */
	private static AnimalRequest buildCryoDonorRequest() {

		return new AnimalRequest();
	}
	
}
