/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.animalrequest.controllers;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.animalrequest.AnimalRequest;
import uk.ac.sanger.mig.animalrequest.forms.AnimalRequestForm;
import uk.ac.sanger.mig.animalrequest.services.AnimalRequestService;
import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.Room;
import uk.ac.sanger.mig.mousetracking.services.RoomService;
import uk.ac.sanger.mig.personnel.Person;

public class AnimalRequestFormController extends AbstractCancellableFormController {



	private AnimalRequestService animalRequestService;
	private PersonalLicenceService personalLicenceService;
	private ProjectLicenceService projectLicenceService;
	private RoomService roomService;
	private DictionaryService dictionaryService;

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}

	public void setAnimalRequestService(AnimalRequestService animalRequestService) {
		this.animalRequestService = animalRequestService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setPersonalLicenceService(
			PersonalLicenceService personalLicenceService) {
		this.personalLicenceService = personalLicenceService;
	}

	public void setRoomService(RoomService roomService) {
		this.roomService = roomService;
	}

	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		AnimalRequestForm animalRequestForm = (AnimalRequestForm)createCommand();
		Person user = RequestUtils.getUser(httpServletRequest);

		AnimalRequest animalRequest = null;

		Long id = ServletRequestUtils.getLongParameter(httpServletRequest,"id");

		if(id != null) { //existing database record
			animalRequest = animalRequestService.fetchById(id, ComponentMap.of(AnimalRequest.Component.ALL_COMPONENTS));
		}
		else { // new animal request
			animalRequest = buildNewAnimalRequest(user);
		}

		animalRequestService.inflateAnimalRequestComponents(animalRequest);

		animalRequestForm.setAnimalRequest(animalRequest);
		return animalRequestForm;
	}



	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest, Object command, Errors errors) throws Exception {
		AnimalRequestForm form = (AnimalRequestForm)command;

		Map<String,Object> referenceData = new HashMap<String,Object>();

		//NOTE: PPLs will be those available to animalRequestOriginator -- ie, the user (Person) if new
		//request or the original form creator (Person) if editing an existing request
		Collection availablePpls = projectLicenceService.fetchByLicenceHolder(form.getAnimalRequest().getRequestOwner().getId(), ComponentMap.of(ProjectLicence.Component.RESPONSIBLE_PERSON));
		referenceData.put("availablePpls", availablePpls);

		Collection<DictionaryEntry> strains = dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_WILDTYPE_STRAIN, form.getAnimalRequest().getStrainId());
		referenceData.put("strains", strains);

		Collection<Room> rooms = roomService.fetchCurrent(null);
		referenceData.put("rooms", rooms);

		Collection<DictionaryEntry> statuses = dictionaryService.fetchForEdit(DictionaryEnum.WILDTYPE_REQUEST_STATUS, form.getAnimalRequest().getStatusDictId());
		referenceData.put("statuses", statuses);

		return referenceData;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		AnimalRequestForm form = (AnimalRequestForm)command;
		Person user = RequestUtils.getUser(request);

		AnimalRequest animalRequest = form.getAnimalRequest();
		boolean isNewRequest = (form.getAnimalRequest().getId() == null) ? true : false;
		Long requestId = animalRequestService.sendEmailOnStore(animalRequest, user);

		String queryString = "?currentRecordId=" + requestId;

		// for new records filter by status and sort by descending ID
		if(isNewRequest) {
			queryString += "&bypassForm=true&newQuery=true&orderBy=id&orderDesc=true&animalRequest.statusDictId=" + animalRequest.getStatusDictId();
		}
		// for existing records, return to the last list page the user was on
		else {
			queryString += "&bypassForm=true&newQuery=false&retrieveFromSession=true";
		}

		return new ModelAndView(getSuccessView() + queryString);
	}

	private AnimalRequest buildNewAnimalRequest(Person person) {
		AnimalRequest animalRequest = new AnimalRequest();

		PersonalLicence personalLicence = personalLicenceService.fetchByPersonId(person.getId(), null);
		if( personalLicence == null ) {
			personalLicence = new PersonalLicence();
			personalLicence.setLicenceNumber("No Licence");
		}
		person.setPersonalLicence(personalLicence);

		animalRequest.setRequestOwner(person);
		animalRequest.setRequestOwnerId(person.getId());

		animalRequest.setStatusDictId(AnimalRequest.REQUESTED_STATUS);

		animalRequest.setFundingSource("Sanger");
		animalRequest.setStandingOrderFrequency("One-time order");

		return animalRequest;
	}
}
