/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.animalrequest.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.animalrequest.AnimalRequest;
import uk.ac.sanger.mig.animalrequest.AnimalRequestSearchBean;
import uk.ac.sanger.mig.animalrequest.dao.AnimalRequestDao;
import uk.ac.sanger.mig.animalrequest.views.AnimalRequestListView;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.MailHelper;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.Room;
import uk.ac.sanger.mig.mousetracking.services.RoomService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.dao.MailingListDao;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class AnimalRequestServiceImpl extends AbstractServiceImpl<AnimalRequest> implements AnimalRequestService {
  
  private AnimalRequestDao animalRequestDao;
  private MailHelper mailHelper;
  private MailingListDao mailingListDao;
  private RoomService roomService;
  private ProjectLicenceService projectLicenceService;
  private PersonService personService;
  
  public AnimalRequest fetchById(Long id, ComponentMap requiredComponents) {
    AnimalRequest animalRequest = animalRequestDao.fetchById(id);
    
    if(animalRequest != null)
      attachComponents(animalRequest, requiredComponents);
    
    return animalRequest;
  }
  
  public Collection<AnimalRequest> fetch(AnimalRequestSearchBean searchBean, ComponentMap requiredComponents) {
    Collection<AnimalRequest> animalRequests = animalRequestDao.fetch(searchBean);
    
    attachComponents(animalRequests, requiredComponents);
    
    return animalRequests;
  }
  
  public Collection<AnimalRequestListView> search(AnimalRequestSearchBean searchbean) {
    return animalRequestDao.search(searchbean);
  }
  
  @Transactional
  public Long store(AnimalRequest animalRequest, Person user) {
     
    if(animalRequest == null) {
      throw new RuntimeException("Cannot save a null animalRequest");
    }
    
    return animalRequestDao.store(animalRequest, user);   
   
  }	
  
  @Transactional
  public Long sendEmailOnStore(AnimalRequest animalRequest, Person user) {

	  boolean isNewRequest = (animalRequest.getId() == null);
	  
	  store(animalRequest, user);	  
	  
	  if(isNewRequest && animalRequest.getStatusDictId().equals(AnimalRequest.REQUESTED_STATUS)) {
	     
		  sendNewRequestNotification(animalRequest, user);
	    
	  } else if(animalRequest.getStatusDictId().equals(AnimalRequest.CANCELLED_STATUS)) {
	  
		  sendCancelledNotification(animalRequest, user);
	   
	  } else if(animalRequest.getStatusDictId().equals(AnimalRequest.COMPLETED_STATUS)) {			
	        
		  sendCompletedNotification(animalRequest, animalRequest.getRequestOwner());      
	  }
	  
	  return animalRequest.getId();
  }

  @Override
  protected void attachComponents(AnimalRequest animalRequest, ComponentMap requiredComponents) {
    if(animalRequest == null) {
      throw new RuntimeException("Cannot attach components to null animalRequest");
    }		
    
    boolean fetchAll = false;
    
    if(requiredComponents != null) {
      
      if(requiredComponents.containsKey(AnimalRequest.Component.ALL_COMPONENTS)) {
        fetchAll = true;
      }
      
      if(fetchAll || requiredComponents.containsKey(AnimalRequest.Component.DESTINATION_ROOM)) {
        animalRequest.setDestinationRoom(fetchDestinationRoom(animalRequest, requiredComponents.get(AnimalRequest.Component.DESTINATION_ROOM)));
      }
      
      if(fetchAll || requiredComponents.containsKey(AnimalRequest.Component.PROJECT_LICENCE)) {
        animalRequest.setProjectLicence(fetchProjectLicence(animalRequest, requiredComponents.get(AnimalRequest.Component.PROJECT_LICENCE)));				
      }		
      
      if(fetchAll || requiredComponents.containsKey(AnimalRequest.Component.REQUEST_OWNER)) {
        animalRequest.setRequestOwner(fetchRequestOwner(animalRequest, requiredComponents.get(AnimalRequest.Component.REQUEST_OWNER)));
      }
    }		
  }
  

  
  ////////////////////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////// dependencies ///////////////////////////////////////////////
  ////////////////////////////////////////////////////////////////////////////////////////////////////
  public void setAnimalRequestDao(AnimalRequestDao animalRequestDao) {
	    this.animalRequestDao = animalRequestDao;
	  }
	  
	  public void setMailHelper(MailHelper mailHelper) {
	    this.mailHelper = mailHelper;
	  }
	  
	  public void setMailingListDao(MailingListDao mailingListDao) {
	    this.mailingListDao = mailingListDao;
	  }
	  
	  public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
	    this.projectLicenceService = projectLicenceService;
	  }
	  
	  public void setRoomService(RoomService roomService) {
	    this.roomService = roomService;
	  }
	  
	  public void setPersonService(PersonService personService) {
	    this.personService = personService;
	  }
  
	  ////////////////////////////////////////////////////////////////////////////////////////////////////
	  /////////////////////////////////////// privates ///////////////////////////////////////////////////
	  ////////////////////////////////////////////////////////////////////////////////////////////////////
	  private Person fetchRequestOwner(AnimalRequest animalRequest, ComponentMap requiredComponents) {
	    if(animalRequest.getRequestOwnerId() != null) {
	      if(requiredComponents == null) {
	        requiredComponents = new ComponentMap();
	      }
	      requiredComponents.put(Person.Component.ALL_COMPONENTS, null); //Always add full Team and PIL for request owner
	      return personService.fetchById(animalRequest.getRequestOwnerId(), requiredComponents);
	    }
	    else {
	      return null;
	    }
	  }
	  
	  private Room fetchDestinationRoom(AnimalRequest animalRequest, ComponentMap requiredComponents) {
	    if(animalRequest.getDestinationRoomId() != null) {
	      return roomService.fetchById(animalRequest.getDestinationRoomId(), requiredComponents);
	    }
	    else {
	      return null;
	    }
	  }
	  
	  private ProjectLicence fetchProjectLicence(AnimalRequest animalRequest, ComponentMap requiredComponents) {
	    if(animalRequest.getProjectLicenceId() != null) {
	      return projectLicenceService.fetchById(animalRequest.getProjectLicenceId(), requiredComponents);
	    }
	    else {
	      return null;
	    }		
	  }
	  
	  
	  private boolean sendNewRequestNotification(AnimalRequest animalRequest, Person requestor) {
	    
	    Map<String, Object>model = new HashMap<String, Object>();
	    model.put("animalRequest", animalRequest);
	    
	    Collection<String> recipients = mailingListDao.fetchRecipientEmailsByListId(MailingListDao.ANIMAL_REQUEST_MAILING_LIST);
	    recipients.add(requestor.getEmail());
	    
	    return mailHelper.sendMail("animal_request", model, recipients.toArray(new String[]{}));
	  }
	  
	  private boolean sendCancelledNotification(AnimalRequest animalRequest, Person requestor) {
	    
	    Map<String, Object>model = new HashMap<String, Object>();
	    model.put("animalRequest", animalRequest);
	    
	    Collection<String> recipients = mailingListDao.fetchRecipientEmailsByListId(MailingListDao.ANIMAL_REQUEST_CANCELED_MAILING_LIST);
	    recipients.add(requestor.getEmail());
	    
	    return mailHelper.sendMail("animal_request_cancelled", model, recipients.toArray(new String[]{}));
	  }
	  
	  private boolean sendCompletedNotification(AnimalRequest animalRequest, Person requestor) {
	    
	    Map<String, Object>model = new HashMap<String, Object>();
	    model.put("animalRequest", animalRequest);
	    
	    Collection<String> recipients = 
	    	mailingListDao.fetchRecipientEmailsByListId(MailingListDao.ANIMAL_REQUEST_COMPLETED_MAILING_LIST);
	    recipients.add(requestor.getEmail());
	    
	    return mailHelper.sendMail("animal_request_completed", model, recipients.toArray(new String[]{}));
	  }
	  
	  public void inflateAnimalRequestComponents(AnimalRequest animalRequest) {
	    if(animalRequest == null) {
	      throw new RuntimeException("AnimalRequest cannot be null when inflating components.");
	    }
	    
	    if(animalRequest.getRequestOwner() == null) {
	      animalRequest.setRequestOwner(new Person());
	    }
	    
	    if(animalRequest.getProjectLicence() == null) {
	      animalRequest.setProjectLicence(new ProjectLicence());
	    }		
	  }	
	  
}
