/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.annotation.services;

import java.util.Collection;
import java.util.HashSet;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.annotation.Annotation;
import uk.ac.sanger.mig.annotation.dao.AnnotationDao;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;




public class AnnotationServiceImpl extends AbstractServiceImpl2<Annotation,AnnotationDao> implements AnnotationService {

	private PersonService personService;
	private DictionaryService dictionaryService;
	
	@Transactional
	public void updateAll(Collection<Annotation> existing, 
			Collection<Annotation> selected, TableMetadataEnum table, Person user) {	
		//use hash to filter out dupes
		Collection<Annotation> keepsAndAdds = new HashSet<Annotation>();
		for(Annotation selectedAnnotation : selected) {
			if( selectedAnnotation.getTermId() == null       ||
			    selectedAnnotation.getTermName() == null     ||
			    selectedAnnotation.getForeignKeyId() == null ||
			    selectedAnnotation.getForeignTableName() == null ||
			    !selectedAnnotation.getForeignTableName().equals(table)) {
				throw new IllegalStateException("Collection<Annotation> must all have non-null termId, termName, foreignKeyId and correct foreignTableName");
			}
			//if new, populate OntologyDictId
			if( !existing.contains( selectedAnnotation )  ) {
				Long ontologyDictId = dictionaryService.fetchEntryIdByName(DictionaryEnum.ANN_ONTOLOGY_DICT,
						selectedAnnotation.getOntologyDict().getName(), false);
				if( ontologyDictId == null ) {
					throw new RuntimeException("Could not resolve Annotation ontology name to a dictionary entry ID.");
				}
				selectedAnnotation.setOntologyDictId( ontologyDictId );
			}
			keepsAndAdds.add( selectedAnnotation );
		}
		//store keepsAndAdds
		if( !keepsAndAdds.isEmpty() ) {
			storeAll( keepsAndAdds, user );
			existing.removeAll( keepsAndAdds ); //determine delete list
		}
		//delete unwanted
		if(!existing.isEmpty()) {
			deleteAll(existing, user);	
		}
	}
	


	public Collection<Annotation> fetchByTableAndForeignKey(TableMetadataEnum table, Long foreignKeyId, ComponentMap requiredComponents) {

		Collection<Annotation> annotations
			= getPrimaryDao().fetchByTableAndForeignKey(table, foreignKeyId);

		attachComponents(annotations, requiredComponents);

		return annotations;
	}

	@Override
	protected void attachComponents(Annotation annotation, ComponentMap requiredComponents) {

		if(annotation == null) {
			throw new RuntimeException("Cannot attach components to null annotation");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Annotation.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Annotation.Component.CREATOR)) {
				annotation.setCreator( fetchCreator(
						annotation.getCreatorId(),
						requiredComponents.get(Annotation.Component.CREATOR)));
			}

		}
	}

	private Person fetchCreator(Long id, ComponentMap requiredComponents) {
		return personService.fetchById(id, requiredComponents);
	}

	
	private int deleteAll(Collection<Annotation> annotations, Person user) {

		int objectsDeleted = 0;

		for(Annotation annotation : annotations) {

			objectsDeleted += getPrimaryDao().delete(annotation, user);
		}

		return objectsDeleted;
	}
	
	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}


	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}




}
