/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.charting;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;


public class DatabaseChartData implements ParameterizedRowMapper<Object>, ChartData {

/**
 * A simple implementation of ChartData. 
 * DatabaseChartData provides the ability to map a resultset to a ChartData object.
 * 
 * The SQL used to create a ChartData object requires a string column called "label"
 * Optional string columns "extra_field" and "extra_field2" can be used for additional data-point information
 * Numeric columns called "data0" to "dataX" will be mapped to the dataset arrays 0..X
 * The number of data columns required is specified by the number of datasets in the constructor.
 *  
 */

	/**
	 * This constructor requires the number of datasets to be defined.
	 * For pie-charts and one-line XY charts, a single dataset is typically required.
	 * For multiple line XY charts, more than one dataset will be required.
	 * @param numberOfDataSets
	 */
	public DatabaseChartData(int numberOfDataSets) {
		this.dataSets = new LinkedList<List<Double>>();
		for(int i=0; i<numberOfDataSets; i++) {
			dataSets.add(new ArrayList<Double>());
		}
		this.numberOfDataSets = numberOfDataSets;	
		this.labels = new ArrayList<String>();
		this.extraFields = new ArrayList<String>();
		this.extraFields2 = new ArrayList<String>();		
	}

	private int numberOfDataSets;
	private List<List<Double>> dataSets;
	private List<String> labels;
	private List<String> extraFields;
	private List<String> extraFields2;
	
	/**
	 * Add a data-point to the chart data
	 * @param label label for the data point
	 * @param extraField additional information for the data point
	 * @param extraField2 additional information for the data point
	 * @param data value(s) for the datapoint
	 */
	private void add(String label, String extraField, String extraField2, double... data) {
		this.labels.add(label);
		this.extraFields.add(extraField);
		this.extraFields2.add(extraField2);
		for(int i=0; i<numberOfDataSets; i++) {
			this.dataSets.get(i).add(data[i]);
		}
	}

	/**
	 * @see ChartData#getLabels()
	 */
	public String[] getLabels() {
		return labels.toArray(new String[labels.size()]);
	}
	
	/**
	 * @see ChartData#getData(int)
	 */
	public double[] getData(int dataSetIndex) {
		double[] doubleData = new double[dataSets.get(dataSetIndex).size()];
		int index = 0;
		for(Double d : dataSets.get(dataSetIndex)) {
			doubleData[index++] = d.doubleValue();
		}
		
		return doubleData;
	}
	
	/**
	 * @see ChartData#getExtraFields()
	 */
	public String[] getExtraFields() {
		return extraFields.toArray(new String[extraFields.size()]);
	}
	
	/**
	 * @see ChartData#getExtraFields2()
	 */
	public String[] getExtraFields2() {
		return extraFields2.toArray(new String[extraFields2.size()]);
	}	

	/**
	 * Add a data-point to the chart from a resultset row.
	 * 
	 * The underlying SQL must contain a varchar column called "label",
	 * and at least one numeric data column called "data0".
	 * Additional data columns can be added called "data1", "data2" etc.
	 * Optional varchar columns "extra_field" and "extra_field2" can be included.
	 */
	public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
		String label = rs.getString("label");
		double[] data = new double[numberOfDataSets];
		
		for(int i=0; i < numberOfDataSets; i++) {
			data[i] = rs.getDouble("data"+i);
		}
		
		String extraData = "";
		String extraData2 = "";		
		
		try {
			extraData = rs.getString("extra_field");
		}
		catch(SQLException e) {
			
		}		
		
		try {
			extraData2 = rs.getString("extra_field2");
		}
		catch(SQLException e) {
			
		}		
		this.add(label, extraData, extraData2, data);
		return null;
	}	
}
