/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony;

import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.licensing.ProjectLicence;

public class ColonyInfo extends AuthenticatedDataObject implements LookupListElement {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		ALLELES,
		STRAINS,
		COAT_COLOURS,
		CONTACTS,
		BEHAVIOURS,
		CROSSES,
		FILES,
		ROOMS,
		PROJECT_LICENCE;
	}

    private String colonyName;
    private String colonyPrefix;

	private Long cardColourDictId;
	private DictionaryEntry cardColour;

    private Long projectLicenceId;
	private ProjectLicence projectLicence;

	private String miscNotes;
    private Date dateOfRederivation;
    private Float breedingLifespanWeeks;
    private String assessingPhenotype;
    private String modeOfInheritance;
    private String welfareAssessment;
    private String rederivationMethod;
    private boolean welfareAssessmentPerformed;
    private String environmentalEnrichment;
    private boolean wtsiOrigin;
    private String otherAbnormalities;
    private String colonyOrigin;
    private Integer averageLitterSize;
    private String placeOfOrigin;
    private String diet;
    private String housingSystem;
    private Date dateOfImport;
    private String colonyManagement;
    private String immuneStatus;
    private String husbandryRecommendations;
    private boolean subjectToMta;
    private boolean poolable;
    private boolean autofillStrain;

	private Collection<ColonyBehaviour> behaviours;
	private Collection<ColonyAllele> alleles;
	private Collection<ColonyCross> crosses;
	private Collection<ColonyContact> contacts;
	private Collection<ColonyRoom> rooms;
	private Collection<ColonyStrain> strains;
	private Collection<ColonyCoatColour> coatColours;
	private Collection<BinaryFile> files;

	public Collection<BinaryFile> getFiles() {
		return files;
	}
	public void setFiles(Collection<BinaryFile> files) {
		this.files = files;
	}
	public Collection<ColonyAllele> getAlleles() {
		return alleles;
	}
	public void setAlleles(Collection<ColonyAllele> alleles) {
		this.alleles = alleles;
	}
	public String getAssessingPhenotype() {
		return assessingPhenotype;
	}
	public void setAssessingPhenotype(String assessingPhenotype) {
		this.assessingPhenotype = assessingPhenotype;
	}
	public Integer getAverageLitterSize() {
		return averageLitterSize;
	}
	public void setAverageLitterSize(Integer averageLitterSize) {
		this.averageLitterSize = averageLitterSize;
	}
	public Collection<ColonyBehaviour> getBehaviours() {
		return behaviours;
	}
	public void setBehaviours(Collection<ColonyBehaviour> behaviours) {
		this.behaviours = behaviours;
	}
	public Float getBreedingLifespanWeeks() {
		return breedingLifespanWeeks;
	}
	public void setBreedingLifespanWeeks(Float breedingLifespanWeeks) {
		this.breedingLifespanWeeks = breedingLifespanWeeks;
	}
	public Long getCardColourDictId() {
	      return cardColourDictId;
	}

	public void setCardColourDictId(Long cardColourDictId) {
	      this.cardColourDictId = cardColourDictId;
	}
	public Collection<ColonyCoatColour> getCoatColours() {
		return coatColours;
	}
	public void setCoatColours(Collection<ColonyCoatColour> coatColours) {
		this.coatColours = coatColours;
	}
	public String getColonyManagement() {
		return colonyManagement;
	}
	public void setColonyManagement(String colonyManagement) {
		this.colonyManagement = colonyManagement;
	}
	public String getColonyName() {
		return colonyName;
	}
	public void setColonyName(String colonyName) {
		this.colonyName = colonyName;
	}
	public String getColonyOrigin() {
		return colonyOrigin;
	}
	public void setColonyOrigin(String colonyOrigin) {
		this.colonyOrigin = colonyOrigin;
	}
	public String getColonyPrefix() {
		return colonyPrefix;
	}
	public void setColonyPrefix(String colonyPrefix) {
		this.colonyPrefix = colonyPrefix;
	}
	public Collection<ColonyContact> getContacts() {
		return contacts;
	}
	public void setContacts(Collection<ColonyContact> contacts) {
		this.contacts = contacts;
	}
	public Collection<ColonyCross> getCrosses() {
		return crosses;
	}
	public void setCrosses(Collection<ColonyCross> crosses) {
		this.crosses = crosses;
	}
	public Date getDateOfImport() {
		return dateOfImport;
	}
	public void setDateOfImport(Date dateOfImport) {
		this.dateOfImport = dateOfImport;
	}
	public Date getDateOfRederivation() {
		return dateOfRederivation;
	}
	public void setDateOfRederivation(Date dateOfRederivation) {
		this.dateOfRederivation = dateOfRederivation;
	}
	public Collection<ColonyRoom> getRooms() {
		return rooms;
	}
	public void setRooms(Collection<ColonyRoom> rooms) {
		this.rooms = rooms;
	}
	public String getDiet() {
		return diet;
	}
	public void setDiet(String diet) {
		this.diet = diet;
	}
	public String getEnvironmentalEnrichment() {
		return environmentalEnrichment;
	}
	public void setEnvironmentalEnrichment(String environmentalEnrichment) {
		this.environmentalEnrichment = environmentalEnrichment;
	}
	public String getHousingSystem() {
		return housingSystem;
	}
	public void setHousingSystem(String housingSystem) {
		this.housingSystem = housingSystem;
	}
	public String getHusbandryRecommendations() {
		return husbandryRecommendations;
	}
	public void setHusbandryRecommendations(String husbandryRecommendations) {
		this.husbandryRecommendations = husbandryRecommendations;
	}
	public String getImmuneStatus() {
		return immuneStatus;
	}
	public void setImmuneStatus(String immuneStatus) {
		this.immuneStatus = immuneStatus;
	}
	public String getMiscNotes() {
		return miscNotes;
	}
	public void setMiscNotes(String miscNotes) {
		this.miscNotes = miscNotes;
	}
	public String getModeOfInheritance() {
		return modeOfInheritance;
	}
	public void setModeOfInheritance(String modeOfInheritance) {
		this.modeOfInheritance = modeOfInheritance;
	}
	public String getOtherAbnormalities() {
		return otherAbnormalities;
	}
	public void setOtherAbnormalities(String otherAbnormalities) {
		this.otherAbnormalities = otherAbnormalities;
	}
	public String getPlaceOfOrigin() {
		return placeOfOrigin;
	}
	public void setPlaceOfOrigin(String placeOfOrigin) {
		this.placeOfOrigin = placeOfOrigin;
	}
	public ProjectLicence getProjectLicence() {
		return projectLicence;
	}
	public void setProjectLicence(ProjectLicence projectLicence) {
		this.projectLicence = projectLicence;
	}
	public Long getProjectLicenceId() {
		return projectLicenceId;
	}
	public void setProjectLicenceId(Long projectLicenceId) {
		this.projectLicenceId = projectLicenceId;
	}
	public String getRederivationMethod() {
		return rederivationMethod;
	}
	public void setRederivationMethod(String rederivationMethod) {
		this.rederivationMethod = rederivationMethod;
	}
	public Collection<ColonyStrain> getStrains() {
		return strains;
	}
	public void setStrains(Collection<ColonyStrain> strains) {
		this.strains = strains;
	}
	public String getWelfareAssessment() {
		return welfareAssessment;
	}
	public void setWelfareAssessment(String welfareAssessment) {
		this.welfareAssessment = welfareAssessment;
	}
	public boolean isWelfareAssessmentPerformed() {
		return welfareAssessmentPerformed;
	}
	public void setWelfareAssessmentPerformed(boolean welfareAssessmentPerformed) {
		this.welfareAssessmentPerformed = welfareAssessmentPerformed;
	}
	public boolean isWtsiOrigin() {
		return wtsiOrigin;
	}
	public void setWtsiOrigin(boolean wtsiOrigin) {
		this.wtsiOrigin = wtsiOrigin;
	}
	public DictionaryEntry getCardColour() {
		return cardColour;
	}
	public void setCardColour(DictionaryEntry cardColour) {
		this.cardColour = cardColour;
	}
	public boolean isSubjectToMta() {
		return subjectToMta;
	}
	public void setSubjectToMta(boolean subjectToMta) {
		this.subjectToMta = subjectToMta;
	}
	public String getDisplayText() {
		return getColonyName() + "(" + getColonyPrefix() + ")";
	}
	public String getElementData() {
		return
			LookupListUtil.format("colonyInfoId", getId()) +
			LookupListUtil.format("colonyName", getColonyName()) +
			LookupListUtil.format("colonyPrefix", getColonyPrefix());
	}
	public void setDisplayText(String displayText) {
		// TODO Auto-generated method stub

	}
	public void setElementData(String elementData) {
		// TODO Auto-generated method stub

	}
	public boolean isPoolable() {
		return poolable;
	}
	public void setPoolable(boolean poolable) {
		this.poolable = poolable;
	}
	public boolean isAutofillStrain() {
		return autofillStrain;
	}
	public void setAutofillStrain(boolean autofillStrain) {
		this.autofillStrain = autofillStrain;
	}
}
