/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony.controllers;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.CancellableFormController;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.forms.ColonyInfoForm;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.personnel.Person;

public class ColonyInfoFormController extends CancellableFormController {
	
	private static final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd-MMM-yyyy");
	
	private ColonyInfoService colonyInfoService;
	private DictionaryService dictionaryService;
	private ProjectLicenceService projectLicenceService;

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}

	@Override
	protected void initBinder(HttpServletRequest httpServletRequest, ServletRequestDataBinder binder) {	
		
		// Create and register Long binder for Long values which also allows null values
		CustomNumberEditor customLongEditor = new CustomNumberEditor(Long.class, true);
		binder.registerCustomEditor(Long.class, customLongEditor);
	
		// Create and register Float binder for Float values which also allows null values
		CustomNumberEditor customFloatEditor = new CustomNumberEditor(Float.class, true);
		binder.registerCustomEditor(Float.class, customFloatEditor);

		// Create and register Integer binder for Integer values which also allows null values
		CustomNumberEditor customIntegerEditor = new CustomNumberEditor(Integer.class, true);
		binder.registerCustomEditor(Integer.class, customIntegerEditor);
		
		//Create and register Date binder for dd-MMM-yyyy
		CustomDateEditor customDateEditor = new CustomDateEditor(simpleDateFormat, true);
		binder.registerCustomEditor(Date.class,customDateEditor);	
	} 
	
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		ColonyInfoForm colonyInfoForm = (ColonyInfoForm)createCommand();				    
		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");		
		
		ColonyInfo colonyInfo = null;		        
					        
		if(id != null) {
		colonyInfo = colonyInfoService.fetchById(id, ComponentMap.of(ColonyInfo.Component.ALL_COMPONENTS));
		} else {
			colonyInfo = buildNewColonyInfo();
		}
   
		colonyInfoForm.setColonyInfo(colonyInfo);	
		return colonyInfoForm;
	}
		
	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest) throws Exception {
		Map<String,Object> referenceData = new HashMap<String,Object>();				
			
		Collection<DictionaryEntry> cardColours = dictionaryService.fetch(DictionaryEnum.COLONY_CARD_COLOUR);
		referenceData.put("cardColours", cardColours);
		
		Person person = RequestUtils.getUser(httpServletRequest);		
		Collection<ProjectLicence> ppls = projectLicenceService.fetchByLicenceHolder(person.getId(),null);
		referenceData.put("ppls", ppls);
		
		//Yes/No list
		Map<Boolean, String> yesNo = new HashMap<Boolean, String>();
		yesNo.put(false,"No");	
		yesNo.put(true,"Yes");	
		referenceData.put("yesNo",yesNo);
		
		return referenceData;
	} 

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		ColonyInfoForm form = (ColonyInfoForm)command;
		
		ColonyInfo colonyInfo = form.getColonyInfo();
		Person person = RequestUtils.getUser(request);
		
		colonyInfoService.store(colonyInfo, person);
		
		return new ModelAndView(getSuccessView() + colonyInfo.getId());
	}

	@Override
	protected ModelAndView onCancel(HttpServletRequest request, HttpServletResponse response, Object command) throws Exception {
		ColonyInfoForm form = (ColonyInfoForm)command;		
		ColonyInfo colonyInfo = form.getColonyInfo();
		
		if(colonyInfo.getId() != null) {
			return new ModelAndView(getSuccessView() + colonyInfo.getId());
		}
		else {
			return new ModelAndView(getCancelView());
		}
	}	
	
	private ColonyInfo buildNewColonyInfo() {		
		ColonyInfo colonyInfo = new ColonyInfo();

		colonyInfo.setWtsiOrigin(true);
		colonyInfo.setHousingSystem("IVCs");
		return colonyInfo;
	}	
}