/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.ColonyInfoSearchBean;
import uk.ac.sanger.mig.colony.dao.rowmappers.ColonyInfoListViewRowMapper;
import uk.ac.sanger.mig.colony.views.ColonyInfoListView;
import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.database.StringRowMapper;

public class ColonyInfoDaoSpringImpl extends AbstractDaoSpringImpl<ColonyInfo> implements ColonyInfoDao {

	private ColonyInfoListViewRowMapper colonyInfoListViewRowMapper;
	private PaginatedSearchUtils<ColonyInfoListView> paginator;
	private StringRowMapper stringRowMapper;

	public void setPaginator(PaginatedSearchUtils<ColonyInfoListView> paginator) {
		this.paginator = paginator;
	}

	public void setColonyInfoListViewRowMapper(
			ColonyInfoListViewRowMapper colonyInfoListViewRowMapper) {
		this.colonyInfoListViewRowMapper = colonyInfoListViewRowMapper;
	}

	public Collection<ColonyInfo> fetchForLookup(boolean excludeEmpty) {

		String sql = " select * from "+getTableName()+" c where 1=1 ";
		if(excludeEmpty) {
			sql += " and exists (select null from mts_mouse m where m.colony_info_id = c.id) ";
		}
		sql +=" order by lower(colony_name)";

		return fetchCollection(sql);
	}

	public List<String> fetchUsedPrefixes(String from, String to) {

		Collection<String> bindVariables = new ArrayList<String>();

		String sql = " select c.colony_prefix return_string from " + getTableName();
		sql += " c where 1=1 ";

		if( from != null ) {

			sql+=" and c.colony_prefix >= ? ";
			bindVariables.add( from );
		}

		if( to != null ) {

			sql+=" and c.colony_prefix <= ? ";
			bindVariables.add( to );
		}

		sql +=" order by c.colony_prefix asc ";

		return getSimpleJdbcTemplate().query(sql, stringRowMapper, bindVariables.toArray());
	}

	public boolean colonyNameExists(Long colonyInfoId, String colonyName) {
		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = " select count(*) from "+getTableName()+" where lower(colony_name) = lower(?) ";
		bindVariables.add(colonyName);

		if(colonyInfoId != null && colonyInfoId > 0) {
			sql += " and id != ? ";
			bindVariables.add(colonyInfoId);
		}

		int count = getSimpleJdbcTemplate().queryForInt(sql, bindVariables.toArray());

		return (count == 0) ? false : true;
	}

	public boolean colonyPrefixExists(Long colonyInfoId, String colonyPrefix) {
		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = " select count(*) from "+getTableName()+" where lower(colony_prefix) = lower(?) ";
		bindVariables.add(colonyPrefix);

		if(colonyInfoId != null && colonyInfoId > 0) {
			sql += " and id != ? ";
			bindVariables.add(colonyInfoId);
		}

		int count = getSimpleJdbcTemplate().queryForInt(sql, bindVariables.toArray());

		return (count == 0) ? false : true;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, ColonyInfo colonyInfo) {
		sql.addStringParam(colonyInfo.getColonyName(), "COLONY_NAME");
		sql.addLongParam(colonyInfo.getProjectLicenceId(), "PROJECT_LICENCE_ID");
		sql.addLongParam(colonyInfo.getCardColourDictId(), "CARD_COLOUR_DICT_ID");
		sql.addStringParam(colonyInfo.getColonyManagement(), "COLONY_MANAGEMENT");
		sql.addIntParam(colonyInfo.getAverageLitterSize(), "AVERAGE_LITTER_SIZE");
		sql.addFloatParam(colonyInfo.getBreedingLifespanWeeks(), "BREEDING_LIFESPAN_WEEKS");
		sql.addStringParam(colonyInfo.getImmuneStatus(), "IMMUNE_STATUS");
		sql.addStringParam(colonyInfo.getOtherAbnormalities(), "OTHER_ABNORMALITIES");
		sql.addStringParam(colonyInfo.getWelfareAssessment(), "WELFARE_ASSESSMENT");
		sql.addStringParam(colonyInfo.getHusbandryRecommendations(), "HUSBANDRY_RECOMMENDATIONS");
		sql.addStringParam(colonyInfo.getDiet(), "DIET");
		sql.addStringParam(colonyInfo.getHousingSystem(), "HOUSING_SYSTEM");
		sql.addStringParam(colonyInfo.getEnvironmentalEnrichment(), "ENVIRONMENTAL_ENRICHMENT");
		sql.addStringParam(colonyInfo.getMiscNotes(), "MISC_NOTES");
		sql.addStringParam(colonyInfo.getColonyOrigin(), "COLONY_ORIGIN");
		sql.addStringParam(colonyInfo.getPlaceOfOrigin(), "PLACE_OF_ORIGIN");
		sql.addDateParam(colonyInfo.getDateOfImport(), "DATE_OF_IMPORT");
		sql.addDateParam(colonyInfo.getDateOfRederivation(), "DATE_OF_REDERIVATION");
		sql.addStringParam(colonyInfo.getRederivationMethod(), "REDERIVATION_METHOD");
		sql.addStringParam(colonyInfo.getEditedBy(), "EDITED_BY");
		sql.addDateParam(colonyInfo.getEditDate(), "EDIT_DATE");
		sql.addBooleanParam(colonyInfo.isWelfareAssessmentPerformed(), "WELFARE_ASSESSMENT_PERFORMED");
		sql.addBooleanParam(colonyInfo.isWtsiOrigin(), "WTSI_ORIGIN");
		sql.addStringParam(colonyInfo.getColonyPrefix(), "COLONY_PREFIX");
		sql.addStringParam(colonyInfo.getModeOfInheritance(), "MODE_OF_INHERITANCE");
		sql.addStringParam(colonyInfo.getAssessingPhenotype(), "ASSESSING_PHENOTYPE");
		sql.addDateParam(colonyInfo.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(colonyInfo.getCreatorId(), "CREATED_BY");
		sql.addBooleanParam(colonyInfo.isSubjectToMta(), "SUBJECT_TO_MTA");
		sql.addLongParam(colonyInfo.getCheckNumber(), "CHECK_NUMBER");
		sql.addBooleanParam(colonyInfo.isPoolable(), "IS_POOLABLE");
		sql.addLongParam(colonyInfo.getId(), "ID");
		sql.addBooleanParam(colonyInfo.isAutofillStrain(), "AUTOFILL_STRAIN");
	}

	public Collection<ColonyInfoListView> search(ColonyInfoSearchBean bean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		String sql = " select * from mts_colony_info_list_vw c where 1=1 ";

		if(bean.getColonyNameOrPrefix() != null && !bean.getColonyNameOrPrefix().equals("")) {
			String matchString = null;
			if(bean.getStringMatchMode().equalsIgnoreCase("starts with")) {
				matchString = "upper(?||'%')";
			}
			else if(bean.getStringMatchMode().equalsIgnoreCase("ends with")) {
				matchString = "upper('%'||?)";
			}
			else {
				matchString = "upper('%'||?||'%')";
			}
			sql += " and (upper(colony_name) like " + matchString + " or upper(colony_prefix) like " + matchString + ") ";
			bindVariables.add(bean.getColonyNameOrPrefix());
			bindVariables.add(bean.getColonyNameOrPrefix());
		}

		if(bean.getColonyInfo().getProjectLicenceId() != null) {
			sql += " and c.project_licence_id = ? ";
			bindVariables.add(bean.getColonyInfo().getProjectLicenceId());
		}

		if(bean.getContactId() != null) {
			sql += " and exists (select null from mts_colony_contact p where p.colony_id = c.id and p.person_id = ?) ";
			bindVariables.add(bean.getContactId());
		}

		String orderBySql = "";

		if(bean.getOrderBy() != null && bean.getOrderBy().length() > 0) {
			orderBySql += " order by " + bean.getOrderBy();
			if(bean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}

		sql = sql + orderBySql;

		logger.debug(sql);

		return paginator.fetchSearchResults(sql, bindVariables, bean, colonyInfoListViewRowMapper);
	}

	public void setStringRowMapper(StringRowMapper stringRowMapper) {
		this.stringRowMapper = stringRowMapper;
	}



}