/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony.services;

import java.util.Collection;

import uk.ac.sanger.mig.colony.ColonyAllele;
import uk.ac.sanger.mig.colony.ColonyBehaviour;
import uk.ac.sanger.mig.colony.ColonyCoatColour;
import uk.ac.sanger.mig.colony.ColonyContact;
import uk.ac.sanger.mig.colony.ColonyCross;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.ColonyInfoSearchBean;
import uk.ac.sanger.mig.colony.ColonyRoom;
import uk.ac.sanger.mig.colony.ColonyStrain;
import uk.ac.sanger.mig.colony.views.ColonyInfoListView;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.personnel.Person;


public interface ColonyInfoService {

	/**
	 * Creates a default ColonyInfo object based on CellLine fields,
	 * default service configuration, and default prefix derivation
	 * method.
	 *
	 * Colony prefix will be the first available in the sequence
	 * MAAA..MZZZ, where 'M' is the fixed first letter by default,
	 * and excluding prefixes containing 'I' and 'O'.
	 *
	 * @param cellLine
	 * @return
	 */
	public ColonyInfo deriveDefaultColonyInfo(CellLine cellLine);

	/**
	 * Returns the first available prefix in upper case within the given range
	 * or null if none is available. If prefixExclusionPredicate is defined,
	 * prefixes meeting prefixExclusionPredicate.exclude() == true will be
	 * exluded.
	 *
	 *
	 * 	String arguments must be of the same length.
	 * @param from
	 * @param to
	 * @param prefixExclusionPredicate
	 * @return
	 */
	public String getFirstUnusedPrefix(String from, String to,
			PrefixExclusionPredicate prefixExclusionPredicate);

	public String getDefaultPrefixRangeFrom();

	public String getDefaultPrefixRangeTo();

	/**
	 * Returns the first available prefix within the default
	 * configured range.
	 *
	 * @return
	 */
	public String getDefaultFirstUnusedPrefix();

	/**
	 * Returns a the String argument incremented by one or
	 * null if it equals its maximum value, e.g., ZZZZ or
	 * is an empty String.
	 *
	 * e.g., AAAZ -> AABA
	 *
	 * Order is case-insensitive and any non-null return value
	 * will be a String of all upper-case letters.
	 *
	 * @param arg
	 * @return
	 * @throws IllegalArgumentException if argument is null
	 * or has any non-letter characters.
	 */
	public String increment(String arg);

	public ColonyInfo fetchById(Long colonyInfoId, ComponentMap requiredComponents);

	public Long store(ColonyInfo colonyInfo, Person user);

	public Collection<ColonyInfoListView> search(ColonyInfoSearchBean colonyInfoSearchBean);

	public Collection<ColonyInfo> fetchForLookup(boolean excludeEmpty, ComponentMap requiredComponents);

	public boolean colonyNameExists(Long colonyInfoId, String colonyName);

	public boolean colonyPrefixExists(Long colonyInfoId, String colonyPrefix);

	public Long storeColonyAllele(ColonyAllele colonyAllele, Person person);

	public Long storeColonyBehaviour(ColonyBehaviour colonyBehaviour, Person person);

	public Long storeColonyCoatColour(ColonyCoatColour colonyCoatColour, Person person);

	public Long storeColonyContact(ColonyContact colonyContact, Person person);

	public Long storeColonyCross(ColonyCross colonyCross, Person person);

	public Long storeColonyRoom(ColonyRoom colonyRoom, Person person);

	public Long storeColonyStrain(ColonyStrain colonyStrain, Person person);

	public int deleteColonyAllele(ColonyAllele colonyAllele, Person person);

	public int deleteColonyBehaviour(ColonyBehaviour colonyBehaviour, Person person);

	public int deleteColonyCoatColour(ColonyCoatColour colonyCoatColour, Person person);

	public int deleteColonyContact(ColonyContact colonyContact, Person person);

	public int deleteColonyCross(ColonyCross colonyCross, Person person);

	public int deleteColonyFile(BinaryFile binaryFile, Person person);

	public int deleteColonyRoom(ColonyRoom colonyRoom, Person person);

	public int deleteColonyStrain(ColonyStrain colonyStrain, Person person);

	public ColonyAllele fetchColonyAllele(Long id, ComponentMap requiredComponents);

	public ColonyBehaviour fetchColonyBehaviour(Long id, ComponentMap requiredComponents);

	public ColonyCoatColour fetchColonyCoatColour(Long id, ComponentMap requiredComponents);

	public ColonyContact fetchColonyContact(Long id, ComponentMap requiredComponents);

	public ColonyCross fetchColonyCross(Long id, ComponentMap requiredComponents);

	public BinaryFile fetchColonyFile(Long id, ComponentMap requiredComponents);

	public ColonyRoom fetchColonyRoom(Long id, ComponentMap requiredComponents);

	public ColonyStrain fetchColonyStrain(Long id, ComponentMap requiredComponents);

	public Float fetchStrainTotalPercent(Long colonyInfoId);

	public Long copyColony(Long oldColonyId, String newPrefix, String newName, Person user);

	public DictionaryEntry fetchStrainForAutofill(Long colonyInfoId);
}