/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony.services;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.colony.ColonyAllele;
import uk.ac.sanger.mig.colony.ColonyBehaviour;
import uk.ac.sanger.mig.colony.ColonyCoatColour;
import uk.ac.sanger.mig.colony.ColonyContact;
import uk.ac.sanger.mig.colony.ColonyCross;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.ColonyInfoSearchBean;
import uk.ac.sanger.mig.colony.ColonyRoom;
import uk.ac.sanger.mig.colony.ColonyStrain;
import uk.ac.sanger.mig.colony.dao.ColonyInfoDao;
import uk.ac.sanger.mig.colony.views.ColonyInfoListView;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.personnel.Person;

public class ColonyInfoServiceImpl extends AbstractServiceImpl<ColonyInfo> implements ColonyInfoService {

	private ProjectLicenceService projectLicenceService;
	private ColonyInfoDao colonyInfoDao;
	private ColonyAlleleService colonyAlleleService;
	private ColonyBehaviourService colonyBehaviourService;
	private ColonyCoatColourService colonyCoatColourService;
	private ColonyContactService colonyContactService;
	private ColonyCrossService colonyCrossService;
	private ColonyRoomService colonyRoomService;
	private ColonyStrainService colonyStrainService;
	private BinaryFileService binaryFileService;

	private String defaultPrefixRangeFrom, defaultPrefixRangeTo; //MAAA..MZZZ
	private char[] defaultPrefixExclusions;
	private Long defaultPplId; //99
	private String defaultColonyOrigin;//WTSI
	private String defaultDiet;//normal chow
	private String defaultHousingSystem;//IVCs
	private String defaultMiscNotes;

	public ColonyInfo deriveDefaultColonyInfo(CellLine cellLine) {

		ColonyInfo colonyInfo = new ColonyInfo();

		colonyInfo.setProjectLicenceId( defaultPplId );
		colonyInfo.setColonyOrigin( defaultColonyOrigin );
		colonyInfo.setDiet( defaultDiet );
		colonyInfo.setHousingSystem( defaultHousingSystem );

		//set misc notes with cell line strain
		colonyInfo.setMiscNotes( defaultMiscNotes );

		if( cellLine.getCellLineStrain() != null
				&& !StringUtils.isBlank( cellLine.getCellLineStrain().getName() ) ) {

			colonyInfo.setMiscNotes("Auto-EPD-Generated Cell Line. Team 87 Well "
					+ cellLine.getCloneName() + " -- "
					+ cellLine.getCellLineStrain().getName()
					+ " Parental ES Cells used.");
		}

		//derive prefix
		String firstUnusedPrefix = getDefaultFirstUnusedPrefix();

		colonyInfo.setColonyPrefix( firstUnusedPrefix );

		//set colony name (cell line name + clone id)
		colonyInfo.setColonyName( cellLine.getCellLineName()
				+ " " + cellLine.getCloneName());

		return colonyInfo;
	}


	public String getDefaultFirstUnusedPrefix() {


		return getFirstUnusedPrefix(
				defaultPrefixRangeFrom,
				defaultPrefixRangeTo,
				new PrefixExclusionPredicate() {
					public boolean exclude(String prefix) {

						for(char c : defaultPrefixExclusions) {

							if(prefix.contains( Character.toString(c).toUpperCase() ) ) {

								return true;
							}
						}

						return false;
					}
				});
	}

	public String getFirstUnusedPrefix(String prefixRangeFrom,
			String prefixRangeTo, PrefixExclusionPredicate prefixExclusionPredicate) {

		if( prefixRangeFrom.compareTo( prefixRangeTo ) > 0)
			throw new IllegalArgumentException("String from is greater than String to.");

		if( prefixRangeFrom.length() != prefixRangeTo.length() )
			throw new IllegalArgumentException("String args must be same length.");

		Set<String> hash = new HashSet<String>();

		List<String> usedPrefixed =  colonyInfoDao.fetchUsedPrefixes( prefixRangeFrom , prefixRangeTo );

		//1. hash -- faster (constant time) than doing log n binary search of list on every iteration
		for(String usedPrefix : usedPrefixed) {
			hash.add( usedPrefix );
		}

		//2. select k in n time, instead of nlog(n)
		String searchPrefix = new String( prefixRangeFrom ); //make defensive copy, instead of intern
		String nextUnusedPrefix = null;

		while( nextUnusedPrefix == null
				&& searchPrefix != null ) {

			if( !hash.contains( searchPrefix ) ) {

				if( prefixExclusionPredicate != null
						&& prefixExclusionPredicate.exclude( searchPrefix )) {

						searchPrefix = increment( searchPrefix );

				} else {

					nextUnusedPrefix = searchPrefix;
				}

			} else {

				searchPrefix = increment( searchPrefix );
			}
		}

		return nextUnusedPrefix;

	}

	public String increment( String letters ) {


		if( !StringUtils.isAlpha( letters ) ) {

			throw new IllegalArgumentException("Argument cannot be null or contain not-letter characters..");
		}


		if( letters.length() == 0 )
			return null;

		char[] chars = new char[letters.length()];

		letters.getChars(0, chars.length, chars, 0);

		chars = increment( chars, 0 );

		return (chars != null) ?  new String( chars ).toUpperCase() : null;
	}

	public ColonyInfo fetchById(Long id, ComponentMap componentMap) {
		ColonyInfo colonyInfo = colonyInfoDao.fetchById(id);
		if(colonyInfo != null) {
			attachComponents(colonyInfo, componentMap);
		}

		return colonyInfo;
	}

	public Collection<ColonyInfoListView> search(ColonyInfoSearchBean colonyInfoSearchBean) {
		return colonyInfoDao.search(colonyInfoSearchBean);
	}

	public Collection<ColonyInfo> fetchForLookup(boolean excludeEmpty, ComponentMap requiredComponents) {
		Collection<ColonyInfo> colonies = colonyInfoDao.fetchForLookup(excludeEmpty);
		attachComponents(colonies, requiredComponents);
		return colonies;
	}

	public Long storeColonyAllele(ColonyAllele colonyAllele, Person person) {
		return colonyAlleleService.store(colonyAllele, person);
	}

	public Long storeColonyBehaviour(ColonyBehaviour colonyBehaviour, Person person) {
		return colonyBehaviourService.storeAuthenticated(colonyBehaviour, person);
	}

	public Long storeColonyCoatColour(ColonyCoatColour colonyCoatColour, Person person) {
		return colonyCoatColourService.storeAuthenticated(colonyCoatColour, person);
	}

	public Long storeColonyContact(ColonyContact colonyContact, Person person) {
		return colonyContactService.storeAuthenticated(colonyContact, person);
	}

	public Long storeColonyCross(ColonyCross colonyCross, Person person) {
		return colonyCrossService.storeAuthenticated(colonyCross, person);
	}

	public Long storeColonyRoom(ColonyRoom colonyRoom, Person person) {
		return colonyRoomService.storeAuthenticated(colonyRoom, person);
	}

	public Long storeColonyStrain(ColonyStrain colonyStrain, Person person) {
		return colonyStrainService.storeAuthenticated(colonyStrain, person);
	}

	public int deleteColonyAllele(ColonyAllele colonyAllele, Person person) {
		return colonyAlleleService.delete(colonyAllele, person);
	}

	public int deleteColonyBehaviour(ColonyBehaviour colonyBehaviour, Person person) {
		return colonyBehaviourService.delete(colonyBehaviour, person);
	}

	public int deleteColonyCoatColour(ColonyCoatColour colonyCoatColour, Person person) {
		return colonyCoatColourService.delete(colonyCoatColour, person);
	}

	public int deleteColonyContact(ColonyContact colonyContact, Person person) {
		return colonyContactService.delete(colonyContact, person);
	}

	public int deleteColonyCross(ColonyCross colonyCross, Person person) {
		return colonyCrossService.delete(colonyCross, person);
	}

	public int deleteColonyFile(BinaryFile binaryFile, Person person) {
		return binaryFileService.delete(binaryFile, person);
	}

	public int deleteColonyRoom(ColonyRoom colonyRoom, Person person) {
		return colonyRoomService.delete(colonyRoom, person);
	}

	public int deleteColonyStrain(ColonyStrain colonyStrain, Person person) {
		return colonyStrainService.delete(colonyStrain, person);
	}

	public ColonyAllele fetchColonyAllele(Long id, ComponentMap requiredComponents) {
		return colonyAlleleService.fetchById(id, requiredComponents);
	}

	public ColonyBehaviour fetchColonyBehaviour(Long id, ComponentMap requiredComponents) {
		return colonyBehaviourService.fetchById(id, requiredComponents);
	}

	public ColonyCoatColour fetchColonyCoatColour(Long id, ComponentMap requiredComponents) {
		return colonyCoatColourService.fetchById(id, requiredComponents);
	}

	public ColonyContact fetchColonyContact(Long id, ComponentMap requiredComponents) {
		return colonyContactService.fetchById(id, requiredComponents);
	}

	public ColonyCross fetchColonyCross(Long id, ComponentMap requiredComponents) {
		return colonyCrossService.fetchById(id, requiredComponents);
	}

	public BinaryFile fetchColonyFile(Long id, ComponentMap requiredComponents) {
		return binaryFileService.fetchById(id, false, requiredComponents);
	}

	public ColonyRoom fetchColonyRoom(Long id, ComponentMap requiredComponents) {
		return colonyRoomService.fetchById(id, requiredComponents);
	}

	public ColonyStrain fetchColonyStrain(Long id, ComponentMap requiredComponents) {
		return colonyStrainService.fetchById(id, requiredComponents);
	}

	public Float fetchStrainTotalPercent(Long colonyInfoId) {
		return colonyStrainService.fetchTotalPercent(colonyInfoId);
	}

	public boolean colonyNameExists(Long colonyInfoId, String colonyName) {
		return colonyInfoDao.colonyNameExists(colonyInfoId, colonyName);
	}

	public boolean colonyPrefixExists(Long colonyInfoId, String colonyPrefix) {
		return colonyInfoDao.colonyPrefixExists(colonyInfoId, colonyPrefix);
	}

	@Transactional
	public Long copyColony(Long oldColonyId, String newPrefix, String newName, Person user) {
		ColonyInfo colony = fetchById(oldColonyId, ComponentMap.of(ColonyInfo.Component.ALL_COMPONENTS));
		Long newColonyId = null;

		if(colony != null) {

			colony.setId(null);
			colony.setColonyPrefix(newPrefix);
			colony.setColonyName(newName);

			newColonyId = store(colony, user);

			for(ColonyAllele allele : colony.getAlleles()) {
				allele.setColonyId(newColonyId);
				allele.setId(null);
				colonyAlleleService.store(allele, user);
			}

			for(ColonyBehaviour behaviour : colony.getBehaviours()) {
				behaviour.setColonyId(newColonyId);
				behaviour.setId(null);
				colonyBehaviourService.storeAuthenticated(behaviour, user);
			}

			for(ColonyCoatColour coatColour : colony.getCoatColours()) {
				coatColour.setColonyId(newColonyId);
				coatColour.setId(null);
				colonyCoatColourService.storeAuthenticated(coatColour, user);
			}

			for(ColonyContact contact : colony.getContacts()) {
				contact.setColonyId(newColonyId);
				contact.setId(null);
				colonyContactService.storeAuthenticated(contact, user);
			}

			for(ColonyCross cross : colony.getCrosses()) {
				cross.setColonyId(newColonyId);
				cross.setId(null);
				colonyCrossService.storeAuthenticated(cross, user);
			}

			for(ColonyRoom room : colony.getRooms()) {
				room.setColonyId(newColonyId);
				room.setId(null);
				colonyRoomService.storeAuthenticated(room, user);
			}

			for(ColonyStrain strain : colony.getStrains()) {
				strain.setColonyId(newColonyId);
				strain.setId(null);
				colonyStrainService.storeAuthenticated(strain, user);
			}
		}

		return newColonyId;
	}

	@Override
	protected void attachComponents(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		if(colonyInfo == null) {
			throw new RuntimeException("Cannot attach components to null colonyInfo");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(ColonyInfo.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.ALLELES)) {
				colonyInfo.setAlleles(fetchAlleles(colonyInfo, requiredComponents.get(ColonyInfo.Component.ALLELES)));
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.STRAINS)) {
				colonyInfo.setStrains(fetchStrains(colonyInfo, requiredComponents.get(ColonyInfo.Component.STRAINS)));
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.COAT_COLOURS)) {
				colonyInfo.setCoatColours(fetchCoatColours(colonyInfo, requiredComponents.get(ColonyInfo.Component.COAT_COLOURS)));
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.CONTACTS)) {
				colonyInfo.setContacts(fetchContacts(colonyInfo, requiredComponents.get(ColonyInfo.Component.CONTACTS)));
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.BEHAVIOURS)) {
				colonyInfo.setBehaviours(fetchBehaviours(colonyInfo, requiredComponents.get(ColonyInfo.Component.BEHAVIOURS)));
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.CROSSES)) {
				colonyInfo.setCrosses(fetchCrosses(colonyInfo, requiredComponents.get(ColonyInfo.Component.CROSSES)));
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.ROOMS)) {
				colonyInfo.setRooms(fetchRooms(colonyInfo, requiredComponents.get(ColonyInfo.Component.ROOMS)));
			}

			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.PROJECT_LICENCE)) {
				colonyInfo.setProjectLicence(fetchProjectLicence(colonyInfo, requiredComponents.get(ColonyInfo.Component.PROJECT_LICENCE)));
			}
			if(fetchAll || requiredComponents.containsKey(ColonyInfo.Component.FILES)) {
				colonyInfo.setFiles(fetchFiles(colonyInfo, requiredComponents.get(ColonyInfo.Component.FILES)));
			}
		}
	}

	private Collection<BinaryFile> fetchFiles(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return binaryFileService.fetchByTableRef(TableMetadataEnum.MTS_COLONY_INFO, colonyInfo.getId(), false, requiredComponents);
	}

	private Collection<ColonyAllele> fetchAlleles(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return colonyAlleleService.fetchByColonyId(colonyInfo.getId(), requiredComponents);
	}

	private Collection<ColonyStrain> fetchStrains(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return colonyStrainService.fetchByColonyId(colonyInfo.getId(), requiredComponents);
	}

	private Collection<ColonyCoatColour> fetchCoatColours(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return colonyCoatColourService.fetchByColonyId(colonyInfo.getId(), requiredComponents);
	}

	private Collection<ColonyContact> fetchContacts(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return colonyContactService.fetchByColonyId(colonyInfo.getId(), requiredComponents);
	}

	private Collection<ColonyBehaviour> fetchBehaviours(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return colonyBehaviourService.fetchByColonyId(colonyInfo.getId(), requiredComponents);
	}

	private Collection<ColonyCross> fetchCrosses(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return colonyCrossService.fetchByColonyId(colonyInfo.getId(), requiredComponents);
	}

	private Collection<ColonyRoom> fetchRooms(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		return colonyRoomService.fetchByColonyId(colonyInfo.getId(), requiredComponents);
	}

	private ProjectLicence fetchProjectLicence(ColonyInfo colonyInfo, ComponentMap requiredComponents) {
		if (colonyInfo.getProjectLicenceId() != null) {
			return projectLicenceService.fetchById(colonyInfo.getProjectLicenceId(), requiredComponents);
		}
		else {
			return null;
		}
	}




	private char[] increment(char[] c, int index) {

		int d = Character.digit(c[ c.length - 1 - index], 36);

		if (d == Character.MAX_RADIX - 1) {

			c[ c.length - 1 - index] = Character.forDigit(10, Character.MAX_RADIX);

			if( index < c.length - 1 ) {

				return increment( c, ++index );

			} else {
			//maxed out!
				 return null;

			}

		} else {

			c[ c.length - 1 - index] = Character.forDigit(d + 1, Character.MAX_RADIX);

		}

		return c;
	}

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}

	public void setColonyInfoDao (ColonyInfoDao colonyInfoDao) {
		this.colonyInfoDao= colonyInfoDao;
	}

	public void setColonyAlleleService(ColonyAlleleService colonyAlleleService) {
		this.colonyAlleleService = colonyAlleleService;
	}

	public void setColonyBehaviourService(ColonyBehaviourService colonyBehaviourService) {
		this.colonyBehaviourService = colonyBehaviourService;
	}

	public void setColonyCoatColourService(ColonyCoatColourService colonyCoatColourService) {
		this.colonyCoatColourService = colonyCoatColourService;
	}

	public void setColonyContactService(ColonyContactService colonyContactService) {
		this.colonyContactService = colonyContactService;
	}

	public void setColonyCrossService(ColonyCrossService colonyCrossService) {
		this.colonyCrossService = colonyCrossService;
	}

	public void setColonyRoomService(ColonyRoomService colonyRoomService) {
		this.colonyRoomService = colonyRoomService;
	}

	public void setColonyStrainService(ColonyStrainService colonyStrainService) {
		this.colonyStrainService = colonyStrainService;
	}

	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}

	public Long store(ColonyInfo colonyInfo, Person person) {
		return colonyInfoDao.store( colonyInfo, person );
	}




	public void setDefaultPplId(Long defaultPplId) {
		this.defaultPplId = defaultPplId;
	}



	public void setDefaultColonyOrigin(String defaultColonyOrigin) {
		this.defaultColonyOrigin = defaultColonyOrigin;
	}



	public void setDefaultDiet(String defaultDiet) {
		this.defaultDiet = defaultDiet;
	}



	public void setDefaultHousingSystem(String defaultHousingSystem) {
		this.defaultHousingSystem = defaultHousingSystem;
	}



	public void setDefaultMiscNotes(String defaultMiscNotes) {
		this.defaultMiscNotes = defaultMiscNotes;
	}


	public void setDefaultPrefixRangeFrom(String defaultPrefixRangeFrom) {
		this.defaultPrefixRangeFrom = defaultPrefixRangeFrom;
	}


	public void setDefaultPrefixRangeTo(String defaultPrefixRangeTo) {
		this.defaultPrefixRangeTo = defaultPrefixRangeTo;
	}


	public String getDefaultPrefixRangeFrom() {
		return this.defaultPrefixRangeFrom;
	}


	public String getDefaultPrefixRangeTo() {
		return this.defaultPrefixRangeTo;
	}


	public void setDefaultPrefixExclusions(char[] defaultPrefixExclusions) {
		this.defaultPrefixExclusions = defaultPrefixExclusions;
	}

	public DictionaryEntry fetchStrainForAutofill(Long colonyInfoId) {
		ColonyInfo colony = fetchById(colonyInfoId, ComponentMap.of(ColonyInfo.Component.STRAINS));
		DictionaryEntry strain = null;
		if(colony != null && colony.isAutofillStrain() && colony.getStrains().size() == 1) {
			strain = ((ColonyStrain)colony.getStrains().iterator().next()).getStrain();
		}
		return strain;
	}
}