/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony.validators;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.forms.ColonyInfoForm;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;

public class ColonyInfoFormValidator implements Validator {

	private ColonyInfoService colonyInfoService;

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	public boolean supports(Class clazz) {
		return ColonyInfoForm.class.equals(clazz);
	}

	public void validate(Object commandForm, Errors errors) {

		ColonyInfoForm form = (ColonyInfoForm)commandForm;
		ColonyInfo colonyInfo = form.getColonyInfo();

		if(colonyInfo.getColonyPrefix() != null) {
			colonyInfo.setColonyPrefix(colonyInfo.getColonyPrefix().trim().toUpperCase());
		}

		if(StringUtils.isEmpty(colonyInfo.getColonyPrefix())) {
			errors.reject(null, "Colony Prefix is a required field.");
		}
		else {
			if (colonyInfoService.colonyPrefixExists(colonyInfo.getId(), colonyInfo.getColonyPrefix())) {
				errors.reject(null, "This prefix is already in use - please choose another.");
			}
			else {
				colonyInfo.setColonyPrefix(colonyInfo.getColonyPrefix().toUpperCase());
			}
		}

		if(StringUtils.isEmpty(colonyInfo.getColonyName())) {
			errors.reject(null, "Colony Name is a required field.");
		}
		else {
			if (colonyInfoService.colonyNameExists(colonyInfo.getId(), colonyInfo.getColonyName())) {
				errors.reject(null, "This colony name is already in use - please choose another.");
			}
			else if (colonyInfo.getColonyName().length() > 64) {
				errors.reject(null, "Colony name should be 64 characters or less.");
			}
		}

		if (colonyInfo.getPlaceOfOrigin() != null && colonyInfo.getPlaceOfOrigin().length() > 128) {
			errors.reject(null, "Place of origin should be 128 characters or less.");
		}

		if (colonyInfo.getColonyOrigin() != null && colonyInfo.getColonyOrigin().length() > 128) {
			errors.reject(null, "Colony origin should be 128 characters or less.");
		}

		if (colonyInfo.getColonyManagement() != null && colonyInfo.getColonyManagement().length() > 4000) {
			errors.reject(null, "Colony management description should be 4000 characters or less.");
		}

		if (colonyInfo.getImmuneStatus() != null && colonyInfo.getImmuneStatus().length() > 64) {
			errors.reject(null, "Immune status description should be 64 characters or less.");
		}

		if (colonyInfo.getOtherAbnormalities() != null && colonyInfo.getOtherAbnormalities().length() > 4000) {
			errors.reject(null, "Other abnormalities description should be 4000 characters or less.");
		}

		if (colonyInfo.getWelfareAssessment() != null &&colonyInfo.getWelfareAssessment().length() > 4000) {
			errors.reject(null, "* Welfare assessment description should be 4000 characters or less.");
		}

		if (colonyInfo.getHusbandryRecommendations() != null && colonyInfo.getHusbandryRecommendations().length() > 4000) {
			errors.reject(null, "Husbandry recommendations should be 4000 characters or less.");
		}

		if (colonyInfo.getDiet() != null && colonyInfo.getDiet().length() > 128) {
				errors.reject(null, "Diet description should be 128 characters or less.");
		}

		if (colonyInfo.getHousingSystem() != null && colonyInfo.getHousingSystem().length() > 128) {
			errors.reject(null, "Housing system description should be 128 characters or less.");
		}

		if (colonyInfo.getEnvironmentalEnrichment() != null && colonyInfo.getEnvironmentalEnrichment().length() > 128) {
			errors.reject(null, "Environmental enrichment description should be 128 characters or less.");
		}

		if (colonyInfo.getMiscNotes() != null && colonyInfo.getMiscNotes().length() > 4000) {
			errors.reject(null, "Miscellaneous notes should be 4000 characters or less.");
		}

		if (colonyInfo.getRederivationMethod() != null && colonyInfo.getRederivationMethod().length() > 128) {
			errors.reject(null, "Rederivation method description should be 128 characters or less.");
		}

		if (colonyInfo.getModeOfInheritance() != null && colonyInfo.getModeOfInheritance().length() > 128) {
			errors.reject(null, "Mode of inheritance should be 128 characters or less.");
		}

		if (colonyInfo.getAssessingPhenotype() != null && colonyInfo.getAssessingPhenotype().length() > 4000) {
			errors.rejectValue(null, "Assessing phenotype should be 4000 characters or less.");
		}

		// check date of rederivation later than date of import (from Java cookbook):
		if ((colonyInfo.getDateOfImport() != null) && (colonyInfo.getDateOfRederivation() != null)) {
			if (!colonyInfo.getDateOfRederivation().after(colonyInfo.getDateOfImport())) {
				errors.reject(null, "Date of import must be before date of rederivation.");
			}
		}

		if(StringUtils.isNotEmpty(colonyInfo.getColonyPrefix())) {
			// colony prefix may not contain numbers
			if (!colonyInfo.getColonyPrefix().matches("[a-zA-Z]{2,4}")) {
				errors.reject(null, "Prefix must consisit 2-4 of letters only.");
			}
			else if(colonyInfo.getColonyPrefix().toUpperCase().endsWith("I")) {
				errors.reject(null, "Prefix must not end with the letter 'I'.");
			}
		}
	}
}