/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.util.HashSet;
import java.util.Set;

import org.springframework.validation.Errors;
import org.springframework.validation.FieldError;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;


/**
 * Encapsulates and provides access to Set of AllowedFormField objects
 * @authors ae1 and dg6
 *
 */
public abstract class AbstractBindingValidator implements Validator {

	private static final int PLACEHOLDER_INDEX_ZERO = 0; //index in errorMessages.properties to set values dynamically
	private Set<AllowedFormField> formFields;

	public AbstractBindingValidator() {
		this.formFields = defineAllowedFields();
	}

	/**
	 * Returns the set of allowed form fields for this binding validator.
	 */
	protected Set<AllowedFormField> defineAllowedFields() {
		return formFields;
	}
	
	public abstract boolean supports(Class clazz);	

	/**
	 * Validates objects that have gone through binding validation.
	 */
	public final void validate(Object target, Errors errors) {
		 boolean hasBindingErrors = resolveBindingErrors( errors );
		 
		 validateRequiredFields(errors);
		 validate(target, errors, hasBindingErrors);
	}	
	
	private void validateRequiredFields(Errors errors) {
		if(formFields != null) {
			for(AllowedFormField field : formFields) {
				if(field.isRequired()) {
					FieldError fieldError = errors.getFieldError(field.getFieldName());
					if(fieldError == null || !fieldError.isBindingFailure()) {
						ValidationUtils.rejectIfEmptyOrWhitespace(errors, field.getFieldName(), "required", new Object[]{field.getUserReadableName()}, field.getUserReadableName() + " is a required field.");
					}
				}
			}
		}
	}
	
	/**
	 * Validates object logic 
	 * @param object
	 * @param errors
	 * @param hasBindingErrors
	 */
	protected abstract void validate(Object target, Errors errors, boolean hasBindingErrors);
	
	/**
	 * Returns user-readable name for the given field name.
	 * @param fieldName
	 * @return
	 */
	public String getUserReadableName(String fieldName) {

		if (formFields != null) {
			for (AllowedFormField formField : formFields) {
				if (formField.getFieldName().equals(fieldName))
					return formField.getUserReadableName();
			}
		}
		return null;
	}


	/**
	 * Returns the Set of allowed field String names.
	 * @return
	 */
	public Set<String> getAllowedFields() {

		return getAllowedFields(null);
	}

	/**
	 * Returns set of fully qualified String field names (ie, w/nestedPath pre-pended) 
	 * for all form fields in this binding validator.
	 * @param nestedPath
	 * @return
	 */
	public Set<String> getAllowedFields(String nestedPath) {

		if (formFields == null)
			return new HashSet<String>(); //return empty set
		else {
			Set<String> allowedFields = new HashSet<String>();
			for (AllowedFormField formField : formFields) {
				allowedFields.add((nestedPath == null || nestedPath.length() == 0
				? "" : nestedPath + ".") + formField.getFieldName());
			}
			return allowedFields;
		}
	}

	/**
	 * Returns the Set of required field String names.
	 * @return
	 */
	public Set<String> getRequiredFields() {
		return getRequiredFields(null);
	}

	/**
	 * Returns the Set of required field String names with nested path prepended to each.
	 * @return
	 */
	public Set<String> getRequiredFields(String nestedPath) {

		if (formFields == null)
			return new HashSet<String>(); //return empty set
		else {
			Set<String> requiredFields = new HashSet<String>();
			for (AllowedFormField formField : formFields) {
				if (formField.isRequired()) {
					requiredFields.add((nestedPath == null || nestedPath.length() == 0
					? "" : nestedPath + ".") +formField.getFieldName());
				}
			}
			return requiredFields;
		}
	}

	/**
	 * Resolves req'd and optional typeMismatch binding errs to human readable messages.
	 * Returns true if any binding errors detected.
	 * @param errors
	 * @return
	 */
	protected boolean resolveBindingErrors(Errors errors) {
	
		Set<String> allowedFields = getAllowedFields();
		boolean hasBindingErrors = false;
		for (String allowedField : allowedFields) {
			FieldError fieldError = errors.getFieldError(allowedField);
			if (fieldError != null && fieldError.isBindingFailure()) {
				hasBindingErrors = true;
				fieldError.getArguments()[PLACEHOLDER_INDEX_ZERO] = getUserReadableName(
						allowedField);
			}
		}
		return hasBindingErrors;
	}








}
