/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.propertyeditors.CustomBooleanEditor;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.StringTrimmerEditor;
import org.springframework.validation.Validator;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.mvc.CancellableFormController;


/**
 * Abstract class with all functionality of CancellableFormController that standardized
 * initBinder implementation that registers custom date and string trim editors as well
 * as includes automatic binding validation capabillity (on required and typeMismatch)
 * for validators that subclass AbstractBindingValidator.
 *
 * Also provides (protected) convenience access to SimpleDateFormat SDF as well
 * as ability to check if Controller intends to call Spring MVC onSubmit() methods
 *
 * @authors ae1 and dg6
 *
 */
public abstract class AbstractCancellableFormController extends CancellableFormController {

	/**
	 * Convenience field for formatting dates.
	 */
	protected static final SimpleDateFormat SIMPLE_DATE_FORMAT = new SimpleDateFormat("dd-MMM-yyyy");

	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {

		binder.registerCustomEditor( String.class, new StringTrimmerEditor( true ) );
		binder.registerCustomEditor( Date.class,   new CustomDateEditor( SIMPLE_DATE_FORMAT, true ) );
		binder.registerCustomEditor(Boolean.class, new CustomBooleanEditor(true));

		if( isBindingValidation(request) ) {

			AbstractBindingValidator bindingValidator = (AbstractBindingValidator) getValidator();

			Set<String> allowedFields = bindingValidator.getAllowedFields();
			Set<String> requiredFields = bindingValidator.getRequiredFields();

			if( !allowAllFields() )
				binder.setAllowedFields( allowedFields.toArray( new String[allowedFields.size()] ) );
			binder.setRequiredFields( requiredFields.toArray( new String[requiredFields.size()] ) );
		}

		super.initBinder(request, binder);
	}

	/**
	 * If true, ServletRequestDataBinder binder allows any field to be bound to form.
	 * Otherwise, calls binder.setAllowedFields() and sets only those fields defined as
	 * allowed in AbstractBindingValidator.  Default is false.
	 * @return
	 */
	protected boolean allowAllFields() {
		return false;
	}

	/**
	 * Returns true if this controller will try to initialize fields for binding
	 * validation from AllowedFields.
	 *
	 * Default behavior is to return true if it's a an actual form submission (ie, a POST)
	 * and not just a form change -- ie, Controller intends to call onSubmit().
	 *
	 * May be overridden eg, if on form change you want
	 * binding validation, return true.
	 * @param request
	 * @return
	 */
	protected boolean isBindingValidation(HttpServletRequest request) {
		return hasBindingValidator()       &&
		isFormSubmission( request ) &&
		!isFormChangeRequest( request );
	}

	private boolean hasBindingValidator() {
		Validator validator = getValidator();
		return (validator != null &&
				validator instanceof AbstractBindingValidator);
	}

}


