/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import uk.ac.sanger.mig.database.LongRowMapper;
import uk.ac.sanger.mig.database.OptimisticLockException;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.personnel.Person;

/**
 * <pre>
 * An abstract class that provides concrete implementations to methods to fetch single records and
 * save records to the database.
 *
 * The class is typed and requires a parameter the extends AuthenticatedDataObject.
 *
 * The local variables tableName, primaryKeyColumn and sequenceName must be set externally
 * (e.g. by Spring Framework dependency injection)
 * </pre>
 *
 * @param <T> T is any Class that extends AuthenticatedDataObject
 */
public abstract class AbstractDaoSpringImpl<T extends AuthenticatedDataObject> extends SimpleJdbcDaoSupport {

	protected SimpleDateFormat defaultDateFormat = new SimpleDateFormat("dd-MMM-yyyy");

	private String tableName;
	private String primaryKeyColumn;
	private String sequenceName;

	private ParameterizedRowMapper<T> rowMapper;

	public void setTableName(String tableName) {
		this.tableName = tableName;
	}

	public void setPrimaryKeyColumn(String primaryKeyColumn) {
		this.primaryKeyColumn = primaryKeyColumn;
	}

	public void setSequenceName(String sequenceName) {
		this.sequenceName = sequenceName;
	}

	/**
	 * Return the default row mapper for this implementation.
	 * This is dependecy injected by the Spring Framework
	 *
	 * @return ParameterizedRowMapper<T> for Class of type T
	 */
	protected ParameterizedRowMapper<T> getRowMapper() {
		return rowMapper;
	}

	public void setRowMapper(ParameterizedRowMapper<T> rowMapper) {
		this.rowMapper = rowMapper;
	}

	protected String getTableName() {
		return tableName;
	}

	/**
	 * Fetch a single record from the database using supplied SQL.
	 * If a record is not found, null is returned.
	 *
	 * @param sql the SQL query to execute
	 * @param the list (null permitted) of bind variables associated with the query.
	 * @return database record of type T or null if not found.
	 */
	public T fetchOne(String sql, Object... bindVariables) {
		T t = null;

		try {
			if(bindVariables != null) {
				t = getSimpleJdbcTemplate().queryForObject(sql, rowMapper, bindVariables);
			}
			else {
				t = getSimpleJdbcTemplate().queryForObject(sql, rowMapper);
			}
		}
		catch(IncorrectResultSizeDataAccessException e) {

		}

		return t;
	}

	/**
	 * Fetch a collection of records from the database using supplied SQL.
	 * If a record is not found, an empty ArrayList is returned.
	 *
	 * @param sql the SQL query to execute
	 * @param the list (null permitted) of bind variables associated with the query.
	 * @return Collection of type T or empty ArrayList if not found.
	 */
	public Collection<T> fetchCollection(String sql, Object... bindVariables) {
		Collection<T> collection = null;

		if(bindVariables == null) {
			collection = getSimpleJdbcTemplate().query(sql, getRowMapper());
		}
		else {
			collection = getSimpleJdbcTemplate().query(sql, getRowMapper(), bindVariables);
		}

		return (collection == null) ? new ArrayList<T>() : collection;
	}

	/**
	 * Fetch a single record from the database using the primary key value (ID).
	 * If a record is not found, null is returned.
	 *
	 * @param id primary key value of the object to be fetched
	 * @return database record or null if not found.
	 */
	public T fetchById(Long id){
		if(id == null) {
			return null;
		}

		T t = null;

		String sql = "select * from " + tableName + " where " + primaryKeyColumn + " = ?";

		try {
			t = getSimpleJdbcTemplate().queryForObject(sql, rowMapper, id);
		}
		catch(IncorrectResultSizeDataAccessException e) {
			logger.warn("No record found in " + tableName +" for ID=" + id + "; " + e);
		}

		return t;
	}

	public Collection<Long> fetchLastSelectionIds(String sessionId, String queryName) {
		String sql = " select t." + primaryKeyColumn + " return_long from " + tableName + " t, app_user_searches a where t.id = a.record_id and a.session_id = ? and a.query_name = ? and a.selected is not null order by a.record_number ";

		Collection<Long> ids = getSimpleJdbcTemplate().query(sql, new LongRowMapper(), sessionId, queryName);
		return (ids == null) ? new ArrayList<Long>() : ids;
	}

	public Collection<T> fetchLastSelection(String sessionId, String queryName) {
		String sql = " select * from " + tableName + " t, app_user_searches a where t.id = a.record_id and a.session_id = ? and a.query_name = ? and a.selected is not null order by a.record_number ";
		return fetchCollection(sql, sessionId, queryName);
	}

	/**
	 * Method to delete a record based on the value of the primary key column value.
	 *
	 * @param t Class of type T to be deleted
	 * @param person the current user performing the method call
	 * @return int number of rows affected by the call (should usually be 1)
	 */
	public int delete(T t, Person person) {
		if(t == null || t.getId() == null) {
			throw new RuntimeException("Cannot call delete on a null or new object");
		}
		else if(person == null) {
			throw new RuntimeException("Cannot call delete with a null Person");
		}

		//first store the record to adjust the edited by/date.
		//this will show in the audit table who deleted the record.
		//this will also ensure that the checknumber is correct.
		store(t, person);

		String sqlString = " delete " + tableName + " where " + primaryKeyColumn + " = ?";

		ParameterizedSqlUpdate delete = new ParameterizedSqlUpdate(getJdbcTemplate());
		delete.addLongParam(t.getId());

		return delete.execute(sqlString);
	}

	/**
	 * Method to update or insert a record to the database.
	 * If the database table contains a column called "CHECK_NUMBER" then optimistic locking will be enabled.
	 * If the database table contains columns called "EDITED_BY" and "EDIT_DATE" then the username and timestamp
	 *
	 * If the getId() method evaluates to 0 or null, a new record will be inserted otherwise an update will be performed.
	 *
	 * Optimistic Locking:
	 * On update, the object's checkNumber is compared with the database value and if they are the same,
	 * the update is performed and the checkNumber
	 *
	 * @param t Class of type T to be stored
	 * @param person the current user performing the method call
	 * @return Long value of the primary key for the record
	 * @throws RuntimeException if optimistic locking fails
	 */
	public Long store(T t, Person person) {
		if(t == null) {
			throw new RuntimeException("Cannot call storeAuthenticated on a null object");
		} else if(person == null) {
			throw new RuntimeException("Cannot call storeAuthenticated with a null Person");
		}

		t.setEditedBy(person.getUserName());
		t.setEditDate(new Date());

		// if ID is null or zero then perform an insert
		if(t.getId() == null || t.getId() == 0) {
			t.setCheckNumber(generateCheckNumber());
			t.setCreatedDate(new Date());
			t.setCreatorId(person.getId());
			insert(t);
		}
		// otherwise perform an update (with optimistic locking)
		else {
			T existingRecord = fetchById(t.getId());
			// don't store update-locked records
			if(!existingRecord.isLockedForUpdate() || !t.isLockedForUpdate()) {
				if((t.getCheckNumber() == null && existingRecord.getCheckNumber() == null)
						|| t.getCheckNumber().equals(existingRecord.getCheckNumber())) {
					t.setCheckNumber(generateCheckNumber());
					update(t);
				}
				else {
					throw new OptimisticLockException("Cannot save record with ID=" + existingRecord.getId() + " for class " + t.getClass().getName() + ": CheckNumbers do not match. Last successful edit by user '" + existingRecord.getEditedBy() +"'");
				}
			}
		}

		return t.getId();
	}

	/**
	 * Method to attach field vales from T to an SQL statement.
	 * Required for correct implementation of insert and update
	 * @param sql ParameterizedSqlUpdate containing the SQL to be bound to
	 * @param t Class of type T whose values will be bound to the SQL statement.
	 */
	protected abstract void bindVariables(ParameterizedSqlUpdate sql, T t);

	/**
	 * Method to update a database record for the Object T
	 * @param t Class of type T to be assoicated with the update statement
	 * @return Long value of the primary key for T
	 */
	protected Long update(T t) {
		ParameterizedSqlUpdate sql = new ParameterizedSqlUpdate(getJdbcTemplate(), primaryKeyColumn);
		bindVariables(sql,t);
		sql.executeUpdate(tableName);
		return t.getId();
	}

	/**
	 * Method to insert a database record for the Object T
	 * @param t Class of type T to be assoicated with the insert statement
	 * @return Long value of the primary key for T
	 */
	protected Long insert(T t) {
		t.setId(getPk());

		if(t instanceof BarcodedObject) {
			((BarcodedObject)t).generateBarcode();
		}

		ParameterizedSqlUpdate sql = new ParameterizedSqlUpdate(getJdbcTemplate());
		bindVariables(sql,t);
		sql.executeInsert(tableName);
		return t.getId();
	}

	/**
	 * Retrieve the next value from the primary key sequence for the Class.
	 * The sequence name is injected via the setSequenceName() method.
	 *
	 * @return Long value of the next primary key sequence for the Class
	 */
	protected Long getPk() {
		return getSimpleJdbcTemplate().queryForLong("select " + sequenceName + ".nextval from dual");
	}

	/**
	 * Retrieve the next value from the check number sequence in the database.
	 * @return Long value of the next check number sequence in the database.
	 */
	private Long generateCheckNumber() {
		return getSimpleJdbcTemplate().queryForLong("select app_check_number_seq.nextval from dual");
	}

	/**
	 * Method that checks for the existance of a record in the database with the same id as the input Oject
	 * @param t Class of type T to be validated
	 * @return true if the database contains a record for this Class with the same id.
	 */
	public boolean isValidDataObject(T t) {
		if(t == null || t.getId() < 1) return false;
		return (fetchById(t.getId()) != null);
	}
}
