/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;


public abstract class AbstractSearchFormController extends AbstractCancellableFormController {

	/**
	 * Overridden method to control if validation should be performed on the command object (searchBean)
	 *
	 * There are two request parameters that can control the suppression of validation.
	 * (a) "suppressValidation" boolean value that, if true, will suppress validation.
	 * (b) "bypassForm" boolean value that, if true, will suppress validation.
	 *
	 * If neither of these parameters are found, or true, the method will delegate to the superclass.
	 *
	 * @param request
	 * @return true if validation should not be performed on the command object (searchBean)
	 */
	@Override
	protected boolean suppressValidation(HttpServletRequest request, Object command) {
		boolean suppressValidation = ServletRequestUtils.getBooleanParameter(request, "suppressValidation", false);
		boolean bypassForm = ServletRequestUtils.getBooleanParameter(request, "bypassForm", false);

		if(suppressValidation || bypassForm) {
			logger.debug("Validation suppressed");
			return true;
		}
		else {
			logger.debug("Delegating to superclass suppressValidation method.");
			return super.suppressValidation(request, command);
		}
	}

	/**
	 * Overridden method to control if the request is a form submission, or a request to show a search form
	 * A form submission will result in a call to the onSearch method.
	 *
	 *  There are two request parameters that can control the submission of the form.
	 *  (a) "processForm" boolean value that if true will force form submission.
	 *  (b) "bypassForm" boolean value that if true will force form submission.
	 *
	 *  bypassForm takes precedence over processForm. If neither parameter is found, or true,
	 *  the method delegates to the superclass.
	 *
	 * @param request
	 * @return true if the controller should call the onSearch method, false if the controller should first display a search form
	 */
	@Override
	protected boolean isFormSubmission(HttpServletRequest request) {
		boolean bypassForm = ServletRequestUtils.getBooleanParameter(request, "bypassForm", false);
		Boolean processForm = null;

		// it's neccessary to use a try/catch here as the overridden method
		// isFormSubmission does not throw any exceptions
		try {
			processForm = ServletRequestUtils.getBooleanParameter(request, "processForm");
		}
		catch(Exception e) {

		}

		if(bypassForm) {
			logger.debug("AbstractSearchFormController says: Request parameter 'bypassForm' is true. This is a form submission.");
			return true;
		}
		else if(processForm != null && processForm.booleanValue()) {
			logger.debug("AbstractSearchFormController says: Request parameter 'processForm' is true. This is a form submission.");
			return true;
		}
		else {
			logger.debug("AbstractSearchFormController says: Delegating to superclass isFormSubmission method.");
			return super.isFormSubmission(request);
		}
	}

	protected ModelAndView processFormSubmission(
			HttpServletRequest request, HttpServletResponse response, Object command, BindException errors)
			throws Exception {

		boolean retrieveFromSession = ServletRequestUtils.getBooleanParameter(request, "retrieveFromSession", false);
		String sessionObjectName = ServletRequestUtils.getStringParameter(request, "sessionObjectName", null);
		// if requested, retrieve the results of a previous query held in the session
		// in addition, add any new request parameter values to the command object
		// for example, additional sorting parameters etc.
		if(retrieveFromSession && command != null) {
			Object sessionCommand = retrieveCommandFromSession(request, sessionObjectName);
			if(sessionCommand != null) {
				logger.debug("AbstractSearchFormController says: Command Object retrieved from session." + sessionCommand);
				command = sessionCommand;

		        ServletRequestDataBinder binder = new ServletRequestDataBinder(command);
		        initBinder(request, binder);
		        binder.bind(request);
			}
		}

		// set the session ID of the bean - this is required by the pagination process
		if(command != null && command instanceof PaginatedSearchBean) {
			PaginatedSearchBean searchBean = (PaginatedSearchBean)command;
			searchBean.setSessionId(RequestUtils.getSessionId(request));

			//	put the command object into the session away from Spring's reach
			// this can be retrieved later to re-run the exact query (e.g. for resorting the results)
			String attributeName = (sessionObjectName == null) ? command.getClass().getName() : sessionObjectName;
			request.getSession().setAttribute(attributeName, command);
			logger.debug("AbstractSearchFormController says: Command object stored in session as '" + attributeName + "' " + command);
		}

		return super.processFormSubmission(request, response, command, errors);
	}

	@Override
	protected final Object formBackingObject(HttpServletRequest request) throws Exception {
		Object command = null;

		boolean retrieveFromSession = ServletRequestUtils.getBooleanParameter(request, "retrieveFromSession", false);
		String sessionObjectName = ServletRequestUtils.getStringParameter(request, "sessionObjectName", null);
		if(retrieveFromSession) {
			Object sessionCommand = retrieveCommandFromSession(request, sessionObjectName);
			if(sessionCommand != null) {
				command = sessionCommand;

				// it's necessary to set the status of the newQuery flag to true,
				// otherwise the form will not cause a new query to be executed.
				if(command instanceof PaginatedSearchBean) {
					((PaginatedSearchBean)command).setNewQuery(true);
				}
			}
		}

		if(command == null) {
			logger.debug("AbstractSearchFormController says: Command object is null, delegating creation of formBackingObject to super class");
			command = super.formBackingObject(request);
		}

		tweakFormBackingObject(request, command);

		return command;
	}

	protected void tweakFormBackingObject( HttpServletRequest request, Object command) throws Exception {

	}

	protected abstract ModelAndView onSearch(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception;

	@Override
	@SuppressWarnings("unchecked")
	protected final ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {

		ModelAndView mv = onSearch(request, response, command, errors);
		Map model = mv.getModel();
		model.put("searchBean", command);

		return mv;
	}

	/**
	 * Override handleInvalidSubmit to detect if a previous searchBean can be found in the session
	 * The overridden method is identical to the super method with the exception that a call
	 * to retrieveCommandFromSession is first invoked.
	 */
	@Override
	protected ModelAndView handleInvalidSubmit(HttpServletRequest request, HttpServletResponse response) throws Exception {
		logger.debug("AbstractSearchFormController says: Handling invalid submit");

		Boolean retrieveFromSession = ServletRequestUtils.getBooleanParameter(request, "retrieveFromSession");
		String sessionObjectName = ServletRequestUtils.getStringParameter(request, "sessionObjectName", null);

		if(retrieveFromSession != null && !retrieveFromSession) {
			logger.debug("AbstractSearchFormController says:Request parameter retrieveFromSession is false, delegating to superclass.");
			return super.handleInvalidSubmit(request, response);
		}
		else {
			Object command = retrieveCommandFromSession(request, sessionObjectName);

			if(command == null) {
				logger.debug("AbstractSearchFormController says: Command Object not retrieved from session, calling formBackingObject.");
				command = formBackingObject(request);
			}
			else {
				logger.debug("AbstractSearchFormController says: Command Object retrieved from session." + command);
				// set selectedOnly to false, otherwise the incorrect results (from a previous search) will be displayed.
				if(command instanceof PaginatedSearchBean) {
					((PaginatedSearchBean)command).setSelectedOnly(false);
				}
			}

			ServletRequestDataBinder binder = bindAndValidate(request, command);
			BindException errors = new BindException(binder.getBindingResult());
			return processFormSubmission(request, response, command, errors);
		}
	}

	private Object retrieveCommandFromSession(HttpServletRequest request, String sessionObjectName) {
		logger.debug("AbstractSearchFormController says: Retrieving formBackingObject from session");
		Object sessionCommand = null;
		if(sessionObjectName != null) {
			logger.debug("AbstractSearchFormController says: looking for session object '" + sessionObjectName + "'");
			sessionCommand = request.getSession(true).getAttribute(sessionObjectName);
		}
		else {
			logger.debug("AbstractSearchFormController says: looking for session object '" + getCommandClass().getName() + "'");
			sessionCommand = request.getSession(true).getAttribute(getCommandClass().getName());
		}
		logger.debug("AbstractSearchFormController says: Returning " + sessionCommand);
		return sessionCommand;
	}
}
