/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * An Map implementation that can be used to specify
 * what sub-components of an object are required during a fetch.
 * The object allows hierarchies of sub-components to be specified by
 * adding components which in turn contain a specified componentMap.
 *
 */
public class ComponentMap {

	private Map<ComponentEnum, ComponentMap> subComponents;
	private boolean locked;

	/**
	 * Default constructor - initialises an empty subComponent map.
	 *
	 */
	public ComponentMap() {
		subComponents = new HashMap<ComponentEnum, ComponentMap>();
	}

	/**
	 * Convenience constructor - initialises a single key/value sub-component
	 * @param key Enum that implements the ComponentEnum interface
	 * @param value ComponentMap
	 */
	public ComponentMap(ComponentEnum key, ComponentMap value) {
		subComponents = new HashMap<ComponentEnum, ComponentMap>();
		this.put(key, value);
	}

	/**
	 * Add a new component to this ComponentMap.
	 * @param key Enum that implements the ComponentEnum interface
	 * @param value ComponentMap
	 * @see Map#put(K, V)
	 */
	public ComponentMap put(ComponentEnum key, ComponentMap value) {
		if(!locked)
			return subComponents.put(key, value);
		else {
			throw new RuntimeException("Cannot put into locked component map");
		}
	}

	/**
	 * Test for the existance of a key in the ComponentMap.
     * @param key key whose presence in this map is to be tested.
     * @return true if this map contains a mapping for the specified key.
	 * @see Map#containsKey(java.lang.Object)
	 */
	public boolean containsKey(ComponentEnum key) {
		return subComponents.containsKey(key);
	}

	/**
	 * Retrieve the subset of required components for this component key
	 * @param key
	 * @return ComponentMap containing details of the required sub-components for this component key
	 * @see Map#get(java.lang.Object)
	 */
	public ComponentMap get(ComponentEnum key) {
		return subComponents.get(key);
	}

	public Set<ComponentEnum> getKeys() {
		return subComponents.keySet();
	}

	/**
	 * Convenience method to allow a simple way of adding a flat set of required components.
	 * I.e components that don't contain further sub-components.
	 *
	 * @param requiredComponents ComponentEnum[] of required components
	 * @return a ComponentMap containing the componentEnum keys and null values
	 */
	public static ComponentMap of(ComponentEnum... requiredComponents) {
		ComponentMap components = new ComponentMap();

		for(ComponentEnum component : requiredComponents)
			components.put(component, null);

		return components;
	}

	/**
	 * Convenience method for filling a component tree. Multiple component maps can be added in one operation.
	 * <br>
	 * Example Usage:
	 * <pre>
		ComponentMap rackComponents =
			ComponentMap.putAll(
					new ComponentMap(
							Rack.Component.AIR_HANDLING_UNIT, null),
					new ComponentMap(
							Rack.Component.CAGES,
							ComponentMap.putAll(
									new ComponentMap(
											Cage.Component.CONTENTS,
											new ComponentMap(
													Mouse.Component.LOCATION, null)
											),
									new ComponentMap(
											Cage.Component.BUDGET_HOLDER,null
											)
										)
									),
					new ComponentMap(
							Rack.Component.ROOM, null)

			);

	   This statement will create the following component map:
	   	RACK(*)
		  |
          +- AHU
          |
          +- CAGE(*)
          |    |
          |    +- CONTENTS
          |    |     |
          |    |     +- LOCATION
          |    |
          |    +- BUDGET_HOLDER
          |
          +- ROOM

		(*) requires putAll method
	 * </pre>
	 * @param componentMaps
	 * @return
	 */
	public static ComponentMap putAll(ComponentMap... componentMaps) {
		ComponentMap components = new ComponentMap();

		for(ComponentMap component : componentMaps) {
			Set<ComponentEnum> keys = component.getKeys();
			for(ComponentEnum key : keys) {
				components.put(key, component.get(key));
			}
		}

		return components;
	}

	/**
	 * Output tree-view of complete component map.
	 *
	 * <pre>
	    Example Output:

	 	ComponentMap
		  |
		  |--AIR_HANDLING_UNIT (empty)
		  |
		  |--CAGES
		  |  |
		  |  |--CONTENTS
		  |  |  |
		  |  |  |--LOCATION (empty)
		  |  |
		  |  |--BUDGET_HOLDER (empty)
		  |
		  |--ROOM (empty)
	 *  </pre>
	 */
	public String toString() {
		int level = 1;
		return "ComponentMap\n"+ toString(level);
	}

	/**
	 * Private method for contructing tree view for toString() method
	 * @param level - level of nested subcomponent map
	 * @return string
	 */
	private String toString(int level) {
		String string = "";

		for(ComponentEnum key : getKeys()) {
			String padding = "";
			for(int i=0; i<level; i++) {
				padding += "  |";
			}
			string += padding + "\n" + padding.substring(0, padding.length()-1);
			string += "+--" + key.toString();

			ComponentMap value = get(key);
			if(value != null) {
				if(isLocked()) {
					string += "(locked)";
				}
				string += "\n";
				string += value.toString(level+1);
			}
			else {
				string += " (empty)";
				string += "\n";
			}
		}

		return string;
	}

	public boolean isLocked() {
		return locked;
	}

	public boolean isEmpty() {
		return (subComponents == null || subComponents.isEmpty());
	}

	public void setLocked(boolean locked) {
		this.locked = locked;
	}
}
