/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.util.Map;

import org.apache.commons.digester.Digester;
import org.apache.commons.digester.Rule;
import org.apache.commons.digester.RuleSetBase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;

public class ComponentMapRuleSet extends RuleSetBase {
	protected static Log logger = LogFactory.getLog(ComponentMapRuleSet.class);
	public void addRuleInstances(Digester digester) {
		digester.addRule("*/ComponentMap", new AddComponentMapRule());
		digester.addObjectCreate("*/Component","uk.ac.sanger.mig.common.ComponentMap");
		digester.addRule("*/Component", new AddComponentRule());
	}
}

/**
 * Push and pop the name of the component map into a stack.
 * The name will be used as the key to add the final componentMap to the to the mapStore object.
 *
 */
final class AddComponentMapRule extends Rule {
	public AddComponentMapRule() {
		super();
	}

	@Override
	public void begin(String namespace, String name, Attributes attributes) throws Exception {
		this.digester.push("mapStack", attributes.getValue("name"));
	}

	@Override
	public void end(String namespace, String name) {
		this.digester.pop("mapStack");
	}
}

/**
 *
 */
@SuppressWarnings("unchecked")
final class AddComponentRule extends Rule {

	public AddComponentRule() {
		super();
	}

	/**
	 * This gets called at the start of a "component" tag.
	 */
	@Override
	public void begin(String namespace, String name, Attributes attributes) throws Exception {
		ComponentMap thisComponentMap = (ComponentMap)this.digester.peek();
		String enumString = attributes.getValue("enum");

		Object parent = (Object)this.digester.peek(1);
		if(parent instanceof ComponentMap) {
			String parentClassName = (String)this.digester.peek("classStack");
			Class parentClass = null;

			try {
				parentClass = Class.forName(parentClassName + "$Component");
			}
			catch(ClassNotFoundException e) {
				throw new RuntimeException("No Component enum found in class " + parentClassName);
			}

			ComponentEnum key = null;
			try {
				key = (ComponentEnum)Enum.valueOf(parentClass, enumString);
			}
			catch(IllegalArgumentException e) {
				throw new RuntimeException("No Component called '" + enumString + "' in class " + parentClassName);
			}
			((ComponentMap)parent).put(key, (ComponentMap)thisComponentMap);
		}
		else {
			Map mapStore = (Map)digester.getRoot();
			String componentMapName = (String)this.digester.peek("mapStack");

			if(mapStore.containsKey(componentMapName)) {
				throw new RuntimeException("Duplicate Component Store key '" + componentMapName + "' found");
			}
			else {
				mapStore.put(componentMapName, thisComponentMap);
			}
		}

		boolean allSubcomponents = Boolean.parseBoolean(attributes.getValue("all"));
		if(allSubcomponents) {
			Class thisClass = Class.forName(attributes.getValue("class")+"$Component");
			((ComponentMap)thisComponentMap).put((ComponentEnum)Enum.valueOf(thisClass, "ALL_COMPONENTS"), null);
		}

		String className =  attributes.getValue("class");
/*		if(className == null || className.equals("")) {
			throw new RuntimeException("attribute 'class' is required in 'component' tag.");
		}*/
		this.digester.push("classStack", className);
	}

	/**
	 * This gets called at the end of a "component" tag.
	 */
	@Override
	public void end(String namsspace, String name) throws Exception {
		// remove the classname of this component from the classname stack
		this.digester.pop("classStack");

		// lock the component map
		ComponentMap thisComponentMap = (ComponentMap)this.digester.peek();
		thisComponentMap.setLocked(true);
	}
}

