/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *
 */
package uk.ac.sanger.mig.common;


import java.lang.reflect.Method;

/**
 * Abstract class to be sub-classed by POJOs that are mapped to
 * a record a primary key in a database.
 * eg, AlleleRequest, Room, Person, etc.
 */
public abstract class DataObject {

	//protected transient Log logger = LogFactory.getLog(this.getClass());

	protected Long id; //must be inheritable1

	/**
	 * Returns the id of this POJO corresponding to the primary key
	 * of the database record to which it maps
	 * @return
	 */
	public Long getId() {
		return id;
	}

	public boolean hasId() {
		return (id == null) ? false : true;
	}

	/**
	 * Sets the id of this POJO corresponding to the primary key
	 * of the database record to which it maps
	 * @return
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * Compares this object to the specified input object.
	 * The result is true if both objects are of the same class and either
	 * (a) both objects return the same value for getId()
	 * or (b) hasId() is false for either object and objectA.equals(objectB) as defined in super.equals()
	 * @param object the object to compare with.
	 * @return true if the objects are the same; otherwise false.
	 */
	@Override
	public boolean equals(Object object) {
		if(object == this) return true;
		if( object == null || !(object instanceof DataObject) ) return false;
		DataObject dataObject = (DataObject)object;
		if(!dataObject.hasId() || !this.hasId()) return super.equals(dataObject);
		return ((dataObject.getId().equals(this.getId())) && this.getClass() == dataObject.getClass() );
	}

	//keep Object contract w/equals()

	/**
	 * Returns the hashCode for this DataObject. If hasId() is false, return 0.
	 * Otherwise return the hashCode for Long value of the 'id' field.
	 * @return a hashCode for this DataObject.
	 */
	@Override
	public int hashCode() {
		if(!hasId()) return 0;
		return this.id.hashCode();  //use the Long::hashCode() method on the 'id' to hash this DataObject
	}

	/**
	 * Returns String representation of the class with the values of all public get() and is() methods.
	 * @return String representation of the class with the values of all public get() and is() methods
	 */
	@Override
	public String toString() {
		try {
			StringBuffer sb = new StringBuffer();
			sb.append("[");
			sb.append(this.getClass().getName());
			sb.append("]\n");
			Method[] methods = this.getClass().getMethods();
			for(int i=0; i<methods.length; i++) {
				Method m = methods[i];
				if( !m.getName().equals("getClass") &&
					 (m.getName().startsWith("get") || m.getName().startsWith("is"))
				) {
					try {
						sb.append(m.getName());
						sb.append("=");

						Object o = m.invoke(this,(Object[])null);

						if(o != null && o instanceof DataObject) {
							DataObject dataObject = (DataObject)o;
							sb.append("[ID:");
							sb.append(dataObject.getId());
							sb.append("]");
						}
						else {
							sb.append(o);
						}

						sb.append("\n");
					}
					catch (Exception e) { }
				}
			}
			return sb.toString();
		}
		catch(Exception e) {
			return("ERROR: " + e.getMessage());
		}
	}

}
