/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common; 

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Collection;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import uk.ac.sanger.mig.database.DaoException;
import uk.ac.sanger.mig.database.ParameterizedSqlQuery;

/**
 * Utility class providing methods for creating collections of LookupListElements for use
 * in populating BasicLookupList objects.<br>
 * Each row of the ResultSet generated from executing an SQL query is converted into a LookupList Element.<br> 
 * <br>
 * The first column of the ResultSet is used to populate the LookupListElement displayText field.<br>
 * Subsequent columns are used to populate the LookupListElementImpl elementData field.<br>
 * These columns are converted to a String with the name/value pairs separated by tags.<br>
 * E.g if a result set row has the following data:
 * <pre><code>
 * displayText | firstName | lastName | userName
 * ------------+-----------+----------+----------
 * D.Gannnon   |David      |Gannon    |GANNONDA
 * </code></pre>
 * The LookupListElementImpl would contain a displayText field equal to "D.Gannon"<br>
 * And an elementField equal to "firstName&lt;/fieldName&gt;David&lt;/fieldValue&gt;lastName&lt;/fieldName&gt;Gannon&lt;/fieldValue&gt;userName&lt;/fieldName&gt;GANNONDA&lt;/fieldValue&gt;"<br>
 * It is up to the calling code to process the data in the elementData field
 */
public class LookupListDao extends SimpleJdbcDaoSupport implements ParameterizedRowMapper<LookupListElement>{	
	
	/**
	 * Returns a Collection of LookupListElement objects created by processing the input ResultSet.<br>
	 * The first column of the ResultSet is used to populate the LookupListElement displayText field.<br>
	 * Subsequent columns are used to populate the LookupListElementImpl elementData field.<br>
	 * These columns are converted to a String with the name/value pairs separated by tags.<br>
	 * E.g if a result set row has the following data:
	 * <pre><code>
	 * displayText | firstName | lastName | userName
	 * ------------+-----------+----------+----------
	 * D.Gannnon   |David      |Gannon    |GANNONDA
	 * </code></pre>
	 * The LookupListElement would contain a displayText field equal to "D.Gannon"<br>
	 * And an elementField equal to "firstName&lt;/fieldName&gt;David&lt;/fieldValue&gt;lastName&lt;/fieldName&gt;Gannon&lt;/fieldValue&gt;userName&lt;/fieldName&gt;GANNONDA&lt;/fieldValue&gt;"<br>
	 * It is up to the calling code to process the data in the elementData field
	 * @return Collection of LookupListElement objects
	 */
	
	public LookupListElement mapRow(ResultSet rs, int rowNum) throws SQLException {
		
		LookupListElement element = new LookupListElementImpl();
		
		ResultSetMetaData metaData = rs.getMetaData();
		int colNumber = metaData.getColumnCount();
		
		// if there's less than 2 columns we can't produce any PopupElements
		if(colNumber < 2)
			return null;
		
		String[] colNames = new String[colNumber];
		
		//get the columnNames
		for(int i = 0; i < colNumber; i++)
			colNames[i] = metaData.getColumnName(i + 1);
		
		
		// the first column holds the text value for the PopupElement
		String displayText = rs.getString(1);
		StringBuffer elementData = new StringBuffer();
		
		// add each subsequent column's name and value to the collectionString
		// using the start and end tags as delimiters
		for(int i = 1; i <colNames.length; i++){

			String fieldName = toCamelCase(colNames[i], false);
			String fieldValue = rs.getString(i + 1);
			
			elementData.append(LookupListUtil.format(fieldName, fieldValue));
		}
		
		element.setDisplayText(displayText);
		element.setElementData(elementData.toString());
		
		return element;
	}
	
	/**
	 * Returns a Collection of LookupListElements based on the ResultSet created by the input SQL String (without bind variables)
	 * @param sql String containing the SQL to be executed
	 * @return a Collection of LookupListElements based on the ResultSet created by the input SQL String
	 * @throws DAOException if an error occurs
	 */
	public Collection<LookupListElement> fetchListElements(String sql) throws DaoException {
		return getSimpleJdbcTemplate().query(sql,this);
	}
	
	/**
	 * Returns a Collection of LookupListElements based on the ResultSet created by the input SQL String (with bind variables)
	 * 
	 * @param connection Database connection
	 * @param sql String containing the SQL to be executed
	 * @param variables BindVariableList containing any variables to be bound to the SQL query
	 * @return a Collection of LookupListElements based on the ResultSet created by the input SQL String
	 * @throws DAOException if an error occurs
	 */
	public Collection<LookupListElement> fetchListElements(String sql, ParameterizedSqlQuery variables) 
	throws DaoException {	
		return getSimpleJdbcTemplate().query(sql, this, variables.getParameters());		
	}

	public static String toCamelCase(String inputName, boolean startWithUppercase) {
		
		boolean uppercase = startWithUppercase;
				
		String camelName = "";
		
		for (int i=0; i<inputName.length(); i++) {
			if(inputName.charAt(i) == '_') {
				uppercase = true;
			}
			else if(inputName.charAt(i) == '#') {
				camelName += ".";
			}
			else {
				if(uppercase)
					camelName += (inputName.substring(i,i+1).toUpperCase());
				else
					camelName += (inputName.substring(i,i+1).toLowerCase());
				uppercase = false;
			}
		}//for
		
		return camelName;
	}
}


