/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.VelocityException;
import org.apache.velocity.tools.generic.DateTool;
import org.apache.velocity.tools.generic.NumberTool;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.ui.velocity.VelocityEngineUtils;

import uk.ac.sanger.mig.personnel.dao.MailingListDao;


public class MailHelper  {
	private static final Log logger = LogFactory.getLog(MailHelper.class);

	private JavaMailSender mailSender;
	private VelocityEngine velocityEngine;
	private MailingListDao mailingListDao;
	private String fromAddress;
	private ServerInfo serverInfo;



	public String getFromAddress() {
		return this.fromAddress;
	}

	public ServerInfo getServerInfo() {
		return this.serverInfo;
	}

	public void setServerInfo(ServerInfo serverInfo) {
		this.serverInfo = serverInfo;
	}

	public void setMailSender(JavaMailSender mailSender) {
		this.mailSender = mailSender;
	}

	public void setVelocityEngine(VelocityEngine velocityEngine) {
		this.velocityEngine = velocityEngine;
	}

	public void setMailingListDao(MailingListDao mailingListDao) {
		this.mailingListDao = mailingListDao;
	}

	public void setFromAddress(String fromAddress) {
		this.fromAddress = fromAddress;
	}

	public boolean sendMail(String velocityTemplate, Map<String,Object> model, String[] recipients) {

		MimeMessage mimeMessage = null;
		String messageSubject = null;
		String messageBody = null;

		if(model == null) return false;

		//check incoming recipients are non-null and contain @ (primitive check for valid email address)
		Collection<String> validRecipients = new ArrayList<String>();
		for(String recipient : recipients) {
			if(recipient != null && recipient.contains("@")) {
				validRecipients.add(recipient);
			}
		}

		//tack on server name for debugging/tracking
		model.put("hostName", serverInfo.getHostName());

		//optional tools for formatting
		model.put( "date", new DateTool() );
		model.put( "number", new NumberTool() );

		// populate the velocity template for the message body
		try {
			messageBody = VelocityEngineUtils.mergeTemplateIntoString(velocityEngine, velocityTemplate+"_body.vm", model);
			messageSubject = VelocityEngineUtils.mergeTemplateIntoString(velocityEngine, velocityTemplate+"_subject.vm", model);
			messageSubject = messageSubject.trim();//preceding whitespace will break Outlook html rendering
		}
		catch(VelocityException e) {
			logger.error(e);
			return false;
		}


		// Only attempt to send if recipients is not empty
		if(validRecipients != null && validRecipients.size() !=0) {

			// Create MimeMessage
			//
			try {
				/*
				 *Create a new JavaMail MimeMessage for the underlying JavaMail
				 *Session of this sender. Needs to be called to create MimeMessage
				 *instances that can be prepared by the client and passed to send(MimeMessage).
				 */
				mimeMessage = mailSender.createMimeMessage();

				/*
				 * Helper class for populating a MimeMessage.
				 * Offers support for HTML text content, inline elements such as images,
				 * and typical mail attachments.
				 */
				MimeMessageHelper helper = new MimeMessageHelper(mimeMessage);

				helper.setFrom(fromAddress);
				helper.setTo(validRecipients.toArray(new String[]{}));
				helper.setSubject(messageSubject);
				helper.setText(messageBody, true);

			}
			catch(MessagingException e) {
				logger.error(e);
				return false;
			}

			// Send mail
			//
			try {
				mailSender.send(mimeMessage);
				return true;
			}
			catch(Exception e) {
				logger.error(e);
				return false;
			}
		}
		else {
			return false;
		}
	}

	public boolean sendMail(String velocityTemplate, Map<String,Object> model, Long mailingListId) {

		// fetch the list of recipients
		//
		try {
			Collection<String> recipients = mailingListDao.fetchRecipientEmailsByListId(mailingListId);

			//call sendMail with Array of recipients
			return sendMail(velocityTemplate, model, recipients.toArray(new String[]{}));
		}
		catch(RuntimeException e) {
			logger.error(e);
			return false;
		}
	}

}
