/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;


/**
 * The PaginatedSearchBean is an abstract class that facilitates pagination of search results.
 * The bean works in conjuction with the PaginatedSearchUtils class to store the results of a
 * search to a database table and then retrieve subsets of the results. The bean also provides
 * functionality to select records from the search results.
 * <p>
 */
public class PaginatedSearchBean extends SortableSearchBean {

	public static final int DEFAULT_OFFSET = 1;
	public static final int DEFAULT_LENGTH = 25;

	/**
	 * The first record to display on the page of results
	 */
	private int recordSetOffset = DEFAULT_OFFSET;

	/**
	 * The number of records to display per page of results
	 */
	private int recordSetLength = DEFAULT_LENGTH;

	/**
	 * The total number of records in the result set
	 */
	private int recordSetTotal;

	/**
	 * If true, this should cause the query mechanism to fetch a new version of the result set
	 */
	private boolean newQuery = true;

	/**
	 * This string should uniquely identify a user within the application scope so their paginated
	 * results can be retrieved
	 */
	private String sessionId;

	/**
	 * This string should uniquely identify the search that has been performed (at a per user scope).
	 * In combination with sessionId, this provides a unique key for the search result set
	 */
	private String queryName;

	private String viewName;

	/**
	 * If false, the full result set for a search should be returned without pagination
	 * (search results are returned directly and not stored, selection of result set records is not available).<p>
	 * If true, the result set should be paginated showing {recordSetLength} records starting a {recordSetOffset} offset
	 */
	private boolean paginated = true;

	/**
	 * If true, the full result set for a search should be returned, however pagination functionality (selection of result set records) still is available.<p>
	 */
	private boolean allRecords = false;

	/**
	 * Use this to prevent the "ALL" option being displayed if the number of records is too high
	 */
	private Integer allRecordsLimit;

	/**
	 * If true, only selected records should be returned.
	 */
	private boolean selectedOnly = false;

	/**
	 * The number of records in the paginated search results that are currently selected
	 */
	private int selectedRecordCount = 0;

	private Long currentRecordId;

	public boolean isNewQuery() {
		return newQuery;
	}
	public void setNewQuery(boolean newQuery) {
		this.newQuery = newQuery;
	}
	public int getRecordSetLength() {
		return recordSetLength;
	}
	public void setRecordSetLength(int recordSetLength) {
		this.recordSetLength = recordSetLength;
	}
	public int getRecordSetOffset() {
		return recordSetOffset;
	}
	public void setRecordSetOffset(int recordSetOffset) {
		this.recordSetOffset = recordSetOffset;
	}
	public int getRecordSetTotal() {
		return recordSetTotal;
	}
	public void setRecordSetTotal(int recordSetTotal) {
		this.recordSetTotal = recordSetTotal;
	}
	public String getSessionId() {
		return sessionId;
	}
	public void setSessionId(String sessionId) {
		this.sessionId = sessionId;
	}
	public boolean isPaginated() {
		return paginated;
	}
	public void setPaginated(boolean paginated) {
		this.paginated = paginated;
	}
	public int getSelectedRecordCount() {
		return selectedRecordCount;
	}
	public void setSelectedRecordCount(int selectedRecordCount) {
		this.selectedRecordCount = selectedRecordCount;
	}
	public boolean isAllRecords() {
		return allRecords;
	}
	public void setAllRecords(boolean allRecords) {
		this.allRecords = allRecords;
	}
	public String getQueryName() {
		return queryName;
	}
	public void setQueryName(String queryName) {
		this.queryName = queryName;
	}
	public Long getCurrentRecordId() {
		return currentRecordId;
	}
	public void setCurrentRecordId(Long currentRecordId) {
		this.currentRecordId = currentRecordId;
	}
	public boolean isSelectedOnly() {
		return selectedOnly;
	}
	public void setSelectedOnly(boolean selectedOnly) {
		this.selectedOnly = selectedOnly;
	}
	public String getViewName() {
		return viewName;
	}
	public void setViewName(String viewName) {
		this.viewName = viewName;
	}
	public Integer getAllRecordsLimit() {
		return allRecordsLimit;
	}
	public void setAllRecordsLimit(Integer allRecordsLimit) {
		this.allRecordsLimit = allRecordsLimit;
	}
}
