/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.common;

import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;

import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.object.SqlUpdate;

import uk.ac.sanger.mig.database.LongRowMapper;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;

/**
 * The PaginatedSearchUtils class provides a mechanism for paginating the results of a database query.
 * If pagination is on, the results of the query are stored in the database as a list of primary key values,
 * record numbers and a unique key for the instance of the query. This allows subsets of data to be returned.
 * The class also provides functionality to selected/deselect records in the result set by updating a flag
 * in the result set table.
 * <p>
 * The class relies on the existence of a database table named APP_USER_SEARCHES with these columns:
 * <p>
 * <pre>
 * RECORD_NUMBER (INTEGER)  - the number of the record within the result set, starting at 1 and incrementing by 1
 * RECORD_ID (INTEGER) - the primary key for the records being queried
 * SESSION_ID (STRING) - the session id of the user performing
 * QUERY_TABLE (STRING) - the name of the primary table/view that the query is run against
 * QUERY_DATE (DATE) - the date the query was executed
 * SELECTED (STRING) - toggle to indicate if record is selected
 * QUERY_NAME (STRING) - name of the query being executed
 * </pre>
 *
 * @param <T> The type of object returned by the search.
 */
public class PaginatedSearchUtils<T>  extends SimpleJdbcDaoSupport {

	private String viewName;
	private LongRowMapper longRowMapper = new LongRowMapper();

	public void setViewName(String viewName) {
		this.viewName = viewName;
	}

	public String getViewName() {
		return viewName;
	}

	/**
	 * Retrieve the results of a query from the database
	 * @param sql
	 * @param bindVariables
	 * @param searchBean
	 * @return
	 */
	public Collection<T> fetchSearchResults (
			String sql,
			Collection<Object> bindVariables,
			PaginatedSearchBean searchBean) {
		return fetchSearchResults(sql, bindVariables, searchBean, null);
	}

	/**
	 * Retrieve the results of a query from the database
	 * @param sql
	 * @param bindVariables
	 * @param searchBean
	 * @param rowMapper
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Collection<T> fetchSearchResults (
			String sql,
			Collection<Object> bindVariables,
			PaginatedSearchBean searchBean,
			ParameterizedRowMapper<T> rowMapper) {

		// back populate the view name into the search bean
		if(viewName != null) {
			searchBean.setViewName(viewName);
		}

		Collection<T> results = null;

		logger.debug("fetchSearchResults SQL: " + sql);
		logger.debug("fetchSearchResults bind variables: " + bindVariables);
		logger.debug("fetchSearchResults searchBean: " + searchBean);

		if(!searchBean.isPaginated()) {
			logger.debug("PaginateUtils: query not paginated - calling simpleJdbcTemplate.query");
			searchBean.setRecordSetOffset(1);
			if(rowMapper == null) {
				return (Collection<T>)getSimpleJdbcTemplate().queryForList(sql, bindVariables.toArray());
			}
			else {
				return getSimpleJdbcTemplate().query(sql, rowMapper, bindVariables.toArray());
			}
		}
		else {
			if(searchBean == null) {
				throw new RuntimeException("searchBean is null - cannot paginate search results.");
			}
			if(searchBean.getSessionId() == null) {
				throw new RuntimeException("sessionId is null - cannot paginate search results.");
			}
			if(searchBean.getQueryName() == null) {
				throw new RuntimeException("queryName is null - cannot paginate search results.");
			}
			if(searchBean.getViewName() == null) {
				throw new RuntimeException("viewName is null - cannot paginate search results.");
			}
		}

		if(searchBean.isNewQuery()) {
			logger.debug("isNewQuery = true");
			searchBean.setNewQuery(false);
			searchBean.setRecordSetOffset(PaginatedSearchBean.DEFAULT_OFFSET);
			//searchBean.setRecordSetLength(PaginatedSearchBean.DEFAULT_LENGTH);

			deleteRecordSet(searchBean);

			String insertSql =
				" insert into app_user_searches (record_number, record_id, session_id, query_table, query_name)" +
				" select rownum, id, ?, ?, ? from (" + sql + ")";

			Object[] extendedBindVariables = null;

			// need to add 2 additional bind variables to the query
			if(bindVariables != null) {
				extendedBindVariables = new Object[3 + bindVariables.toArray().length];
				//copy the existing bind variables into the new array
				System.arraycopy(bindVariables.toArray(), 0, extendedBindVariables, 3, bindVariables.toArray().length);
			}
			else {
				extendedBindVariables = new Object[3];
			}

			extendedBindVariables[0] = searchBean.getSessionId();
			extendedBindVariables[1] = searchBean.getViewName();
			extendedBindVariables[2] = searchBean.getQueryName();

			searchBean.setRecordSetTotal(getJdbcTemplate().update(insertSql, extendedBindVariables));
		}

		results = fetchSubset(
				searchBean,
				rowMapper);

		if(results != null) {
			return results;
		}
		else {
			searchBean.setRecordSetTotal(0);
			return new ArrayList<T>();
		}
	}

	@SuppressWarnings("unchecked")
	private Collection<T> fetchSubset(PaginatedSearchBean searchBean, ParameterizedRowMapper<T> rowMapper) {

		String sql = " select a.*,u.* from " + searchBean.getViewName() + " a, app_user_searches u where u.session_id = ? and u.query_table = ? and u.query_name = ? and a.id = u.record_id ";
		Object[] parameters = null;

		if(searchBean.isAllRecords() || searchBean.isSelectedOnly()) {
			parameters = new Object[3];
			parameters[0] = searchBean.getSessionId();
			parameters[1] = searchBean.getViewName();
			parameters[2] = searchBean.getQueryName();

			if(searchBean.isSelectedOnly()) {
				sql += " and u.selected is not null ";
			}
		}
		else {
			parameters = new Object[5];
			parameters[0] = searchBean.getSessionId();
			parameters[1] = searchBean.getViewName();
			parameters[2] = searchBean.getQueryName();
			parameters[3] = searchBean.getRecordSetOffset();
			parameters[4] = searchBean.getRecordSetOffset() + (searchBean.getRecordSetLength()-1);
			sql += " and u.record_number between ? and ? ";
		}

		sql += " order by u.record_number ";

		Collection<T> results = null;

		if(rowMapper == null) {
			results = (Collection<T>)getSimpleJdbcTemplate().queryForList(sql, parameters);
		}
		else {
			results = getSimpleJdbcTemplate().query(
				sql,
				rowMapper,
				parameters);
		}

		int selectedRecords = getSimpleJdbcTemplate().queryForInt("select count(*) from app_user_searches where session_id = ? and query_table = ? and query_name = ? and selected is not null", searchBean.getSessionId(),searchBean.getViewName(), searchBean.getQueryName());
		searchBean.setSelectedRecordCount(selectedRecords);

		return (results != null) ? results : new ArrayList<T>();
	}

	private void deleteRecordSet(PaginatedSearchBean searchBean) {
		String sql = " delete app_user_searches where session_id = ? and query_table = ? and query_name = ? ";
		ParameterizedSqlUpdate delete = new ParameterizedSqlUpdate(getJdbcTemplate());
		delete.addStringParam(searchBean.getSessionId());
		delete.addStringParam(searchBean.getViewName());
		delete.addStringParam(searchBean.getQueryName());
		delete.execute(sql);
	}

	public void toggleRecord(PaginatedSearchBean searchBean, Long recordNumber, boolean selected) {
		String sql = " update app_user_searches set selected = ? where session_id = ? and query_table = ? and query_name = ? and record_number = ?";

		SqlUpdate sqlUpdate = new SqlUpdate(getDataSource(),sql);
		sqlUpdate.declareParameter(new SqlParameter(Types.VARCHAR));
		sqlUpdate.declareParameter(new SqlParameter(Types.VARCHAR));
		sqlUpdate.declareParameter(new SqlParameter(Types.VARCHAR));
		sqlUpdate.declareParameter(new SqlParameter(Types.VARCHAR));
		sqlUpdate.declareParameter(new SqlParameter(Types.NUMERIC));

		Object[] parameters = new Object[5];

		if (selected) {
			parameters[0] = "checked";
		}
		else {
			parameters[0] = null;
		}

		parameters[1] = searchBean.getSessionId();
		parameters[2] = searchBean.getViewName();
		parameters[3] = searchBean.getQueryName();
		parameters[4] = recordNumber;

		sqlUpdate.update(parameters);
	}

	public Collection<Long> fetchSelectedIds(String sessionId, String queryName) {
		String sql = " select record_id return_long from app_user_searches where session_id = ? and query_name = ? and selected='checked' ";

		return getSimpleJdbcTemplate().query(sql, longRowMapper, sessionId, queryName);
	}

	/**
	 * Returns the number of records selected in the result set. The sessionId and queryName provide
	 * a unique key for each instance of a query execution.
	 *
	 * @param sessionId
	 * @param queryName
	 * @return the number of records selected in the result set
	 */
	public int countSelectedRecords(String sessionId, String queryName) {
		String sql = " select count(*) from app_user_searches where selected='checked' and session_id = ? and query_name = ? ";

		return getSimpleJdbcTemplate().queryForInt(sql, sessionId, queryName);
	}
}