/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.common.services;

import java.awt.Dimension;
import java.io.File;
import java.io.IOException;

/**
 * Class to provide general image processing support.
 *
 * @author ae1
 *
 */
public abstract class ImageProcessingService  {

	private String remoteSshProcessor;
	private String viewableRootDir;
	private String urlDomain;

	/**
	 * Calculates scaled dimensions within maxWidthxmaxHeight keeping aspect ratio.
	 *
	 * @param inputDimension
	 * @param maxWidth
	 * @param maxHeight
	 * @return
	 */
	public Dimension calculateScaledDimension(Dimension inputDimension, int maxWidth, int maxHeight) {

		Dimension d = null;

		double widthExcess = inputDimension.getWidth() - maxWidth;
		double heightExcess = inputDimension.getHeight() - maxHeight;

		if( widthExcess > 0 ||  heightExcess > 0) {
			//down scale by larger of two
			if(  widthExcess >  heightExcess ) {

				double heightRedux = (Math.abs( widthExcess ) / inputDimension.getWidth()) * inputDimension.getHeight();
				double scaledHeight = inputDimension.getHeight() - heightRedux;
				d = new Dimension(maxWidth, (int)scaledHeight);

			} else {
				double widthRedux = (Math.abs( heightExcess ) / inputDimension.getHeight()) * inputDimension.getWidth();
				double scaledWidth = inputDimension.getWidth() - widthRedux;
				d = new Dimension((int)scaledWidth, maxHeight);
			}

		} else {
			//upscale smaller of two
			if( Math.abs( widthExcess ) < Math.abs( heightExcess ) ) {

				double heightIncr = (Math.abs( widthExcess ) / inputDimension.getWidth()) * inputDimension.getHeight();
				double scaledHeight = inputDimension.getHeight() + heightIncr;
				d = new Dimension(maxWidth, (int)scaledHeight);

			} else {
				double widthIncr = (Math.abs( heightExcess ) / inputDimension.getHeight()) * inputDimension.getWidth();
				double scaledWidth = inputDimension.getWidth() + widthIncr;
				d = new Dimension((int)scaledWidth, maxHeight);
			}
		}


		return d;
	}

	public String filePathToUrl(String fileSystemPath) {

		String[] dirs = viewableRootDir.split(File.separator);
		String mappingDir = dirs[dirs.length - 1];
		String s2 = fileSystemPath.split( mappingDir )[1];
		s2 = s2.replace("\\", "/");
		return urlDomain + s2;
	}

	/**
	 * Makes image publicly readable.
	 *
	 * @param path
	 * @throws IOException
	 * @throws InterruptedException
	 * @throws ImageProcessingException
	 */
	public void publish(String path) throws IOException, InterruptedException, ImageProcessingException {

		String command = "chmod -R 755 " + path;

		Process process = null;
		try {
			process = Runtime.getRuntime().exec( command );
			int exitCode = process.waitFor();
			if (exitCode != 0) {
				throw new ImageProcessingException("Bad Exit: " + exitCode + " --> " + command);
			}
		} finally { if (process != null) { process.destroy(); } }
	}

	public void convert(File src, File dest, int maxWidth, int maxHeight)
			throws IOException, InterruptedException, ImageProcessingException {

		String command = buildConvertCommand(src, dest, maxWidth, maxHeight);
		Process process = null;
		try {
			process = Runtime.getRuntime().exec( command );
			int exitCode = process.waitFor();
			if (exitCode != 0) {
				throw new ImageProcessingException("Bad Exit: " + exitCode + " --> " + command);
			}
		} finally { if (process != null) { process.destroy(); } }
	}

	public String getRemoteSshProcessor() {
		return this.remoteSshProcessor;
	}

	public void setRemoteSshProcessor(String remoteSshProcessor) {
		this.remoteSshProcessor = remoteSshProcessor;
	}


	public String getViewableRootDir() {
		return this.viewableRootDir;
	}

	public void setViewableRootDir(String viewableRootDir) {
		this.viewableRootDir = viewableRootDir;
	}

	protected abstract String buildConvertCommand(File src, File dest,
			int maxWidth, int maxHeight);

	public String getUrlDomain() {
		return this.urlDomain;
	}

	public void setUrlDomain(String urlDomain) {
		this.urlDomain = urlDomain;
	}


}
