/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryoprocessing;

import java.util.ArrayList;
import java.util.Date;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.cryoprocessing.services.ScheduleCryoPreservationJobService;
import uk.ac.sanger.mig.cryorequest.CryoRequest;


public class ScheduledCryoPreservationJob extends AuthenticatedDataObject  {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,  //add additional component enums here as needed for this AuthenticatedDataObject
		EMBRYO_HARVEST,
		CRYO_REQUEST
	}
    private Long cryoRequestId;  
    private CryoRequest cryoRequest;
    private DictionaryEntry jobStatusDict;       
    private Date nextScheduledDate;        
    private String cancelComments;             
    private String comments;
    private EmbryoHarvest embryoHarvest;
    private Long statusDictId;
    private String jobName;
    private Long cancelReasonDictId;
    private DictionaryEntry cancelReasonDict;
   
    
	public enum InternalStatus  {
		
		SCHEDULED( 1L ),
		FLUSH_COMPLETED( 2L ),
		ATTEMPT_COMPLETE( 3L ),
		CANCELLED( 4L );
		
		private Long dictionaryId;
		
		InternalStatus(Long dictId) {
			this.dictionaryId = dictId;
		}
			
		public Long getId() {
			return dictionaryId;
		}	
	}
	
	public enum SystemType {
		STRAW,
		VIAL,
		UNKNOW ;
	}
	
	public void setStatusAttemptCompleted() {
	    	this.statusDictId = InternalStatus.ATTEMPT_COMPLETE.getId();
	}
	
	public void setStatusFlushCompleted() {
	    	this.statusDictId = InternalStatus.FLUSH_COMPLETED.getId();
	}
	
	public void setStatusCancelled() {
    	this.statusDictId = InternalStatus.CANCELLED.getId();
	}

	public void setStatusScheduled() {
    	this.statusDictId = InternalStatus.SCHEDULED.getId();
	}
	
	public EmbryoHarvest getEmbryoHarvest() {
		return embryoHarvest;
	}

	public void setEmbryoHarvest(EmbryoHarvest embryoHarvest) {
		this.embryoHarvest = embryoHarvest;
	}

	public Long getCryoRequestId() {
        return cryoRequestId;
    }
    
    public void setCryoRequestId(Long cryoRequestId) {
        this.cryoRequestId = cryoRequestId;
    }                                                                                                                                            
                                                                                                                                                                                                                                                                         
    public DictionaryEntry getJobStatusDict() {
		return jobStatusDict;
	}

	public void setJobStatusDict(DictionaryEntry status) {
		this.jobStatusDict = status;
	}
    
    public void setComments(String comments) {
        this.comments = comments;
    }

	public Date getNextScheduledDate() {
		return nextScheduledDate;
	}

	public void setNextScheduledDate(Date nextScheduledDate) {
		this.nextScheduledDate = nextScheduledDate;
	}

	public boolean isScheduled() {
		return  (jobStatusDict.getId() == InternalStatus.SCHEDULED.getId());
			
	}
	public boolean isFlushComplete() {
		return  (jobStatusDict.getId() == InternalStatus.FLUSH_COMPLETED.getId());
			
	}

	public boolean isAttemptCompleted() {
		return  (jobStatusDict.getId() == InternalStatus.ATTEMPT_COMPLETE.getId());
	}
	
	public boolean isCancelled(){
		return (jobStatusDict.getId() == InternalStatus.CANCELLED.getId());
	}

	public String getComments() {
		return comments;
	}

	public Long getStatusDictId() {
		return statusDictId;
	}

	public void setStatusDictId(Long statusDictId) {
		this.statusDictId = statusDictId;
	}

	public CryoRequest getCryoRequest() {
		return cryoRequest;
	}

	public void setCryoRequest(CryoRequest cryoRequest) {
		this.cryoRequest = cryoRequest;
	}

	public String getJobName() {
		return jobName;
	}

	public void setJobName(String jobName) {
		this.jobName = jobName;
	}
	
	public int getTotalFrozenDownEmbryos() {
		
		int frozenDownEmbryos = 0;
		if(this.getEmbryoHarvest() != null 
				&& this.getEmbryoHarvest().getFreezeDownSamples() != null
				&& this.getEmbryoHarvest().getFreezeDownSamples().size() != 0) {
			for (FreezeDownSample freezeDownSample: this.getEmbryoHarvest().getFreezeDownSamples()) {
				if(freezeDownSample.getEmbryoCount() != null) {
					frozenDownEmbryos += freezeDownSample.getEmbryoCount().intValue();
				}
			}
		}
		return frozenDownEmbryos;
	}
	
	public int calculateTotalFlushedEmbryos() {
		int flushedEmbryos = 0;
		if(this.getEmbryoHarvest() != null) {
			flushedEmbryos = this.getEmbryoHarvest().calculateAvailableEmbryos();
		}
		return flushedEmbryos;
	}
	
	public SystemType getSystem() {
		
		ArrayList<FreezeDownSample> freezeDownSamples = new ArrayList<FreezeDownSample>(this.getEmbryoHarvest().getFreezeDownSamples());
		
		if(freezeDownSamples.isEmpty()) {
			return SystemType.UNKNOW;
		}else if(freezeDownSamples.get(0).getStrawLocation() != null) {
			return SystemType.STRAW;
		}else if(freezeDownSamples.get(0).getVialLocation() != null) {
			return SystemType.VIAL;
		}else {
			throw new RuntimeException("Can not determine sample's location!");
		}
	
	}
	
	public boolean isNewFreezeDown() {
		return (getSystem() == SystemType.UNKNOW);
	}
	
	public boolean isStrawSystem() {
		return (getSystem() == SystemType.STRAW);
	}
	
	public boolean isVialSystem() {
		return (getSystem() == SystemType.VIAL);
	}

	public String getCancelComments() {
		return cancelComments;
	}

	public void setCancelComments(String cancelComments) {
		this.cancelComments = cancelComments;
	}

	public Long getCancelReasonDictId() {
		return cancelReasonDictId;
	}

	public void setCancelReasonDictId(Long cancelReasonDictId) {
		this.cancelReasonDictId = cancelReasonDictId;
	}

	public DictionaryEntry getCancelReasonDict() {
		return cancelReasonDict;
	}

	public void setCancelReasonDict(DictionaryEntry cancelReasonDict) {
		this.cancelReasonDict = cancelReasonDict;
	}
	
	public boolean isWtStrainNeeded(){
		
		String rescheduledByWrongMice = ScheduleCryoPreservationJobService.RESCHEDULE_COMMENTS+ScheduleCryoPreservationJobService.CANCEL_JOB_REASON_WRONG_STRAIN_MICE_NAME;
		String rescheduledByMixMice = ScheduleCryoPreservationJobService.RESCHEDULE_COMMENTS+ScheduleCryoPreservationJobService.CANCEL_JOB_REASON_MIXED_STRAIN_MICE_NAME;
		return(rescheduledByWrongMice.equalsIgnoreCase(this.getComments())
				||rescheduledByMixMice.equalsIgnoreCase(this.getComments()));
				
	}
	
	public void setRescheduleComments(String cancelReason){
		this.setComments(ScheduleCryoPreservationJobService.RESCHEDULE_COMMENTS+cancelReason);
	}
	
}