/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryoprocessing.controllers;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.cryoprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationJob;
import uk.ac.sanger.mig.cryoprocessing.forms.EmbryoHarvestForm;
import uk.ac.sanger.mig.cryoprocessing.services.EmbryoHarvestService;
import uk.ac.sanger.mig.cryoprocessing.services.ScheduleCryoPreservationJobService;
import uk.ac.sanger.mig.cryorequest.CryoRequest;
import uk.ac.sanger.mig.cryorequest.services.CryoRequestService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;


public class EmbryoHarvestFormController extends AbstractCancellableFormController {

	private EmbryoHarvestService embryoHarvestService;
	private CryoRequestService cryoRequestService;
	private DictionaryService dictionaryService;
	private String successFilterView;
	private ScheduleCryoPreservationJobService scheduleCryoPreservationJobService;

	public void setScheduleCryoPreservationJobService(
			ScheduleCryoPreservationJobService scheduleCryoPreservationJobService) {
		this.scheduleCryoPreservationJobService = scheduleCryoPreservationJobService;
	}
	public void setSuccessFilterView(String successFilterView) {
		this.successFilterView = successFilterView;
	}
	public String getSuccessFilterView() {
		return successFilterView;
	}

	private static final Integer SUPEROVULATED_COUNT_CIELING = 30;


	public void setEmbryoHarvestService(EmbryoHarvestService embryoHarvestService) {
		this.embryoHarvestService = embryoHarvestService;
	}

	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {

		EmbryoHarvestForm form = (EmbryoHarvestForm)createCommand();	

		boolean isFilterView = ServletRequestUtils.getBooleanParameter(httpServletRequest, "isFilterList",false);
		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");
		Long scpId = ServletRequestUtils.getLongParameter(httpServletRequest, "jobId");

		ScheduledCryoPreservationJob scpJob = scheduleCryoPreservationJobService.fetchWithAllComponentsById(scpId);
		EmbryoHarvest embryoHarvest = null; 

		if(id != null && id!=0) {
			//ok, existing embryoHarvest id provided, so fetch for edit. NOTE: ALL_COMPONENTS not obligatory
			//fetch component map that you need... 
			embryoHarvest = embryoHarvestService.fetchById(
					id,ComponentMap.of(EmbryoHarvest.Component.ALL_COMPONENTS));
			form.setEmbryoHarvest(embryoHarvest);	

		} else {//else make new embryoHarvest	
			embryoHarvest = embryoHarvestService.buildDefaultEmbryoHarvest(RequestUtils.getUser(httpServletRequest),scpJob);

			embryoHarvest.setScheduledJobId(scpId);
			embryoHarvest.setFlushDate(scpJob.getNextScheduledDate());
			form.setEmbryoHarvest(embryoHarvest);	

			if(scpJob.getCryoRequest() != null) {
				ComponentMap requiredComponents = cryoRequestService.getComponentMapForHeaderFile();
				CryoRequest cryoRequest = cryoRequestService.fetchById(scpJob.getCryoRequest().getId(), requiredComponents);
				form.getEmbryoHarvest().setFemaleStrainDictId(cryoRequest.getFemaleStrainDictId());
			}

		}

		form.setScpJob(scpJob);
		form.setFilterView(isFilterView);
		return form;
	}	

	@Override
	protected void onBindOnNewForm(HttpServletRequest request, Object command) throws Exception {

		super.onBindOnNewForm(request, command);
	}



	//called after databinding, before showForm(), merges Map into ModelAndView
	//to populate our form with form "metadata" -- eg, drop-down select lists, etc.

	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest) throws Exception {
		Map<String,Object> metaData = new HashMap<String,Object>();	
		EmbryoHarvestForm form = (EmbryoHarvestForm)getCommand(httpServletRequest);
		ScheduledCryoPreservationJob scpJob = form.getScpJob();
		CryoRequest cryoRequest = null;

		int[] superOvulatedRange = new int[ SUPEROVULATED_COUNT_CIELING+1 ];
		for(int x = 0; x < SUPEROVULATED_COUNT_CIELING+1; x++) {
			superOvulatedRange[x] = x;
		}
		metaData.put("superOvulatedRange", superOvulatedRange);
		//fetch header file information
		if(scpJob.getCryoRequest() != null) {
			ComponentMap requiredComponents = cryoRequestService.getComponentMapForHeaderFile();
			cryoRequest = cryoRequestService.fetchById(scpJob.getCryoRequest().getId(), requiredComponents);
		}
		metaData.put("cryoRequest", cryoRequest);


		//put header file info and superovulated range in meta data
		
		Collection<DictionaryEntry> wildTypeFemaleStrains = dictionaryService.fetch(DictionaryEnum.CRQ_CRYO_WILDTYPE_STRAIN_DICT);
		Collection<DictionaryEntry> femaleRequestStrains = dictionaryService.fetch(DictionaryEnum.CRQ_CRYO_REQUEST_STRAIN_DICT);

		if(scpJob.getCryoRequest() == null) {
			metaData.put("femaleStrains", wildTypeFemaleStrains);
		}else{
			metaData.put("femaleStrains", femaleRequestStrains);
		}

		return metaData;
	} 

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, 
			Object command, BindException errors) throws Exception {
		EmbryoHarvestForm form = (EmbryoHarvestForm) command;
		EmbryoHarvest embryoHarvest = form.getEmbryoHarvest();

		//service method that updates the scheduled job's status, and
		// if required the status of cryo request
		embryoHarvestService.storeAndUpdateJob(embryoHarvest, RequestUtils.getUser(request));

		if(form.isFilterView()){
			return new ModelAndView(getSuccessFilterView());
		}else{
			return new ModelAndView(getSuccessView());
		}// show list view
	}

	public void setCryoRequestService(CryoRequestService cryoRequestService) {
		this.cryoRequestService = cryoRequestService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

}