/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryoprocessing.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationJob;
import uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationSearchBean;
import uk.ac.sanger.mig.cryoprocessing.dao.rowmappers.ScheduledCryoPreservationListViewRowMapper;
import uk.ac.sanger.mig.cryoprocessing.forms.ScheduleCryoRequestForm;
import uk.ac.sanger.mig.cryoprocessing.views.ScheduledCryoPreservationJobListView;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;

public class ScheduleCryoPreservationJobDaoSpringImpl extends AbstractDaoSpringImpl<ScheduledCryoPreservationJob> implements ScheduleCryoPreservationJobDao {

	private ScheduledCryoPreservationListViewRowMapper scheduledCryoPreservationListViewRowMapper;
	private PaginatedSearchUtils<ScheduledCryoPreservationJobListView> paginator;

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, ScheduledCryoPreservationJob scheduleCryoPreservationJob) {
		sql.addLongParam(scheduleCryoPreservationJob.getCryoRequestId(), "CRYO_REQUEST_ID");
		sql.addLongParam(scheduleCryoPreservationJob.getStatusDictId(), "CRYO_JOB_STATUS_DICT_ID");
		sql.addDateParam(scheduleCryoPreservationJob.getNextScheduledDate(), "TARGET_SCHEDULED_DATE");
		sql.addStringParam(scheduleCryoPreservationJob.getCancelComments(), "CANCEL_COMMENTS");
		sql.addDateParam(scheduleCryoPreservationJob.getEditDate(), "EDIT_DATE");
		sql.addStringParam(scheduleCryoPreservationJob.getEditedBy(), "EDITED_BY");
		sql.addDateParam(scheduleCryoPreservationJob.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(scheduleCryoPreservationJob.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(scheduleCryoPreservationJob.getCheckNumber(), "CHECK_NUMBER");
		sql.addStringParam(scheduleCryoPreservationJob.getComments(), "COMMENTS");
		sql.addStringParam(scheduleCryoPreservationJob.getJobName(),"JOB_NAME");
		sql.addLongParam(scheduleCryoPreservationJob.getId(), "ID");
	}

	public Collection<ScheduledCryoPreservationJobListView> fetchAllScheduledJobsByCryoRequestId(Long cryoRequestId, ScheduleCryoRequestForm searchBean){
		StringBuffer sql = new StringBuffer();
  		sql.append("select * from crq_scheduled_job_list_vw cjv " +
  		"where cjv.cryo_request_id = ?");

  		if(searchBean.getOrderBy() != null && !searchBean.getOrderBy().equals("")) {
  			sql.append(" order by " + searchBean.getOrderBy());
  			if(searchBean.isOrderDesc()) {
  				sql.append(" desc ");
  			}
	  } else {
		  sql.append(" order by target_scheduled_date desc, job_name desc");
	  }

	  Collection<ScheduledCryoPreservationJobListView> results = getSimpleJdbcTemplate().query(sql.toString(), scheduledCryoPreservationListViewRowMapper,cryoRequestId);

	  return (results == null) ? new ArrayList<ScheduledCryoPreservationJobListView>() :  results;
	}

	public Collection<ScheduledCryoPreservationJobListView> fetchByTargetDate(Date date){
		String sql = "select * from crq_scheduled_job_list_vw cjv " +
	      "where cjv.target_scheduled_date = ? order by cjv.job_name desc";

		Collection<ScheduledCryoPreservationJobListView> results = getSimpleJdbcTemplate().query(sql, scheduledCryoPreservationListViewRowMapper,defaultDateFormat.format(date));
		return (results == null) ? new ArrayList<ScheduledCryoPreservationJobListView>() :  results;
	}

	public Collection<ScheduledCryoPreservationJobListView> fetchActiveJobs(Date date){
		String sql = "select * from crq_scheduled_job_list_vw cjv " +
	      "where cjv.target_scheduled_date = ? and cjv.status_id!="+ScheduledCryoPreservationJob.InternalStatus.CANCELLED.getId();

		Collection<ScheduledCryoPreservationJobListView> results = getSimpleJdbcTemplate().query(sql, scheduledCryoPreservationListViewRowMapper,defaultDateFormat.format(date));
		return (results == null) ? new ArrayList<ScheduledCryoPreservationJobListView>() :  results;

	}

	public Collection<ScheduledCryoPreservationJobListView> fetchOverdueJobs(Date date){
		String sql = "select * from crq_scheduled_job_list_vw cjv " +
			"where cjv.target_scheduled_date < ? " +
			"and (cjv.job_status is null or (cjv.status_id not like ? and cjv.status_id not like ?)) order by target_scheduled_date desc";
		Object[] args = new Object[3];
		args[0] = defaultDateFormat.format(date);
		args[1] = ScheduledCryoPreservationJob.InternalStatus.CANCELLED.getId();
		args[2] = ScheduledCryoPreservationJob.InternalStatus.ATTEMPT_COMPLETE.getId();

		Collection<ScheduledCryoPreservationJobListView> results = getSimpleJdbcTemplate().query(sql, scheduledCryoPreservationListViewRowMapper,args);
		return (results == null) ? new ArrayList<ScheduledCryoPreservationJobListView>() :  results;
	}

	public void setScheduledCryoPreservationListViewRowMapper(
		ScheduledCryoPreservationListViewRowMapper scheduledCryoPreservationListViewRowMapper) {
		this.scheduledCryoPreservationListViewRowMapper = scheduledCryoPreservationListViewRowMapper;
	}

	public Collection<ScheduledCryoPreservationJob> fetchWildTypeScheduledJobByDate(Date date) {
		String sql = "select * from crq_scheduled_cryo_prv sj where " +
				"sj.target_scheduled_date = ? and sj.cryo_request_id is null and sj.cryo_job_status_dict_id !="+ScheduledCryoPreservationJob.InternalStatus.CANCELLED.getId();

		Collection<ScheduledCryoPreservationJob> wildTypeJob = getSimpleJdbcTemplate().query(sql, getRowMapper(),defaultDateFormat.format(date));
		return wildTypeJob;
	}

	public Collection<ScheduledCryoPreservationJob> fetchActiveByCryoRequestId(Long requestId) {
		String sql = "select * from crq_scheduled_cryo_prv where cryo_request_id = ? and cryo_job_status_dict_id !="+ScheduledCryoPreservationJob.InternalStatus.CANCELLED.getId();
		Collection<ScheduledCryoPreservationJob> jobs = getSimpleJdbcTemplate().query(sql, getRowMapper(), requestId);
		return (jobs == null)? new ArrayList<ScheduledCryoPreservationJob>() : jobs;

	}
	public Collection<ScheduledCryoPreservationJobListView> fetch(ScheduledCryoPreservationSearchBean searchBean) {
		StringBuffer sql = new StringBuffer();
			sql.append("select * from crq_scheduled_job_list_vw cjv where 1=1 ");


		Collection<ScheduledCryoPreservationJobListView> results = new ArrayList<ScheduledCryoPreservationJobListView>();

		Collection<Object> bindVariables = new ArrayList<Object>();

		if(searchBean.getStartTargetDate() != null){
			sql.append("and cjv.target_scheduled_date >= ? ");
			bindVariables.add(defaultDateFormat.format(searchBean.getStartTargetDate()));
		}
		if(searchBean.getEndTargetDate() != null ){
			sql.append("and cjv.target_scheduled_date <= ?  ");
			bindVariables.add(defaultDateFormat.format(searchBean.getEndTargetDate()));
		}

		if(searchBean.getStatusDictId() != null ) {
			sql.append(" and cjv.status_id = ? ");
			bindVariables.add(searchBean.getStatusDictId());
		}

		if(searchBean.getId() != null ) {
			sql.append(" and cjv.id = ? ");
			bindVariables.add(searchBean.getId());
		}

		if(searchBean.getCryoRequestId() != null) {
			sql.append(" and cjv.cryo_request_id = ? ");
			bindVariables.add(searchBean.getCryoRequestId());
		}


		if(searchBean.getCancelComments() != null){
			sql.append(" and cjv.cancel_comments = ? ");
			bindVariables.add(searchBean.getCancelComments());
		}

		if(searchBean.getOrderBy() != null && !searchBean.getOrderBy().equals("") && !searchBean.getOrderBy().equalsIgnoreCase("null")) {
			sql.append(" order by " + searchBean.getOrderBy());
			if(searchBean.isOrderDesc()) {
				sql.append(" desc ");
			}
		}else {
			sql.append("order by cryo_request_id desc, job_name desc");
		}

		results = paginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, scheduledCryoPreservationListViewRowMapper);

		return (results == null) ? new ArrayList<ScheduledCryoPreservationJobListView>() :  results;
	}

	public void setPaginator(
			PaginatedSearchUtils<ScheduledCryoPreservationJobListView> paginator) {
		this.paginator = paginator;
	}

	public int fetchNumberOfJobs(Long cryoRequestId) {

		if(cryoRequestId != null) {
			String sql = "select count(*) from crq_scheduled_cryo_prv where cryo_request_id = ?";

			int count = getSimpleJdbcTemplate().queryForInt(sql,cryoRequestId);
			return count;
		}else{
			throw new IllegalArgumentException("cryoRequestId can not be null!");
		}
	}

	public int fetchFrozenNumbersById(Long id) {
		String sql = "select total_frozen_count from crq_scheduled_job_list_vw sjl where sjl.id = ?";
		Integer result = null;
		try{
			result = getSimpleJdbcTemplate().queryForInt(sql,id);
		}catch(IncorrectResultSizeDataAccessException e){

		}
		return (result == null)? 0: result.intValue();
	}


	/*
	public ScheduledCryoPreservationJobListView fetchListViewById(Long id) {
		String sql = "select * from crq_scheduled_job_list_vw sjl where sjl.id = ?";

		ScheduledCryoPreservationJobListView  jobListView = null;
		try{
			jobListView = getSimpleJdbcTemplate().queryForObject(sql, scheduledCryoPreservationListViewRowMapper,id);
		}catch(IncorrectResultSizeDataAccessException exception){
              throw new RuntimeException("the scheduled cryo job can not be found in the database.");
		}
		return jobListView;
	}
	*/
}