/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryoprocessing.services;

import java.util.Date;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.cryoprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationJob;
import uk.ac.sanger.mig.cryoprocessing.dao.EmbryoHarvestDao;
import uk.ac.sanger.mig.cryorequest.CryoRequest;
import uk.ac.sanger.mig.cryorequest.services.CryoRequestService;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class EmbryoHarvestServiceImpl extends AbstractServiceImpl<EmbryoHarvest> implements EmbryoHarvestService {
	
	private EmbryoHarvestDao embryoHarvestDao;
	private FreezeDownSampleService freezeDownSampleService;
	private DictionaryService dictionaryService;
	private PersonService personService;
	private ScheduleCryoPreservationJobService scheduledCryoPreservationJobService;
	private CryoRequestService cryoRequestService;
	private CohortService cohortService;
	
	public void setEmbryoHarvestDao(EmbryoHarvestDao embryoHarvestDao) {
		this.embryoHarvestDao = embryoHarvestDao;
	}

	@Transactional
	public Long storeAndUpdateJob(EmbryoHarvest embryoHarvest, Person user) {
		if(embryoHarvest == null) {
			throw new RuntimeException("Cannot save a null embryoHarvest");
		}
		
		updateScheduledJob(scheduledCryoPreservationJobService.fetchById(
				embryoHarvest.getScheduledJobId(), null), user);
		
		updateRequestIfStatusIsScheduled(embryoHarvest, user);
		return embryoHarvestDao.store(embryoHarvest, user);
	}
	
	private void updateScheduledJob(ScheduledCryoPreservationJob job, Person user) {
		if(!job.isAttemptCompleted()) {
			job.setStatusFlushCompleted();
			scheduledCryoPreservationJobService.storeAuthenticated(job, user);
		}
	}
	
	private void updateRequestIfStatusIsScheduled(EmbryoHarvest embryoHarvest,Person user){
		
		ScheduledCryoPreservationJob scheduledCryoPreservationJob = 
			scheduledCryoPreservationJobService.fetchById(embryoHarvest.getScheduledJobId(), null);
		
		CryoRequest request= cryoRequestService.fetchById(scheduledCryoPreservationJob.getCryoRequestId(), null);
		
		//if request is null, means its a wildtype flush, no request related to it
		if(request != null) {
			//if the request still has status scheduled, then update it to freezing down
			if(request.isScheduled()){
				request.setStatusInProcess();
				cryoRequestService.storeAuthenticated(request, user);
			}
		}
	}

	public EmbryoHarvest fetchById(Long embryoHarvestId, ComponentMap requiredComponents) {
		EmbryoHarvest embryoHarvest = embryoHarvestDao.fetchById(embryoHarvestId);
		
		if(embryoHarvest != null)
			attachComponents(embryoHarvest, requiredComponents);
		
		return embryoHarvest;
	}

	@Override
	protected void attachComponents(EmbryoHarvest embryoHarvest, ComponentMap requiredComponents) {
		if(embryoHarvest == null) {
			throw new RuntimeException("Cannot attach components to null embryoHarvest");
		}		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(EmbryoHarvest.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			
			if(fetchAll || requiredComponents.containsKey(EmbryoHarvest.Component.FREEZE_DOWN_SAMPLES)){
				embryoHarvest.setFreezeDownSamples(freezeDownSampleService.fetchActiveByEmbryoHarvestId(
						embryoHarvest.getId(), requiredComponents.get(EmbryoHarvest.Component.FREEZE_DOWN_SAMPLES)));
			}
			
			if(fetchAll || requiredComponents.containsKey(EmbryoHarvest.Component.TECHNICIAN)) {
				embryoHarvest.setTechnician(fetchTechnician(
						embryoHarvest.getTechnicianId(), 
						requiredComponents.get(EmbryoHarvest.Component.TECHNICIAN)));
			}
			if(fetchAll || requiredComponents.containsKey(EmbryoHarvest.Component.COHORT)) {
				embryoHarvest.setCohortName(fetchCohortName(
						embryoHarvest.getCohortId(),
						requiredComponents.get(EmbryoHarvest.Component.COHORT)));
			}
			
		}
	}
	
	private Person fetchTechnician(Long personId, ComponentMap requiredComponents) {
		return personService.fetchById( personId, requiredComponents );
	}
	
	private String fetchCohortName(Long cohortId, ComponentMap requiredComponents) {
		Cohort cohort =  cohortService.fetchById(cohortId, requiredComponents);
		return cohort == null ? "" : cohort.getCohortName();
	}


	public EmbryoHarvest fetchByScheduledJobId(Long id, ComponentMap requiredComponents) {
		EmbryoHarvest embryoHarvest = embryoHarvestDao.fetchByScheduledJobId(id);
		if(embryoHarvest != null)
			attachComponents(embryoHarvest, requiredComponents);
		
		return embryoHarvest;
	}
	
	public EmbryoHarvest buildDefaultEmbryoHarvest(Person user, ScheduledCryoPreservationJob scpJob){
		EmbryoHarvest embryoHarvest = new EmbryoHarvest();
		
		embryoHarvest.setTechnicianId(user.getId());
		embryoHarvest.setTechnician(user);
		embryoHarvest.setCellCount1(0);
		embryoHarvest.setCellCount2(0);
		embryoHarvest.setCellCount4(0);
		embryoHarvest.setCellCount8(0);
		embryoHarvest.setCellCount16(0);
		embryoHarvest.setCreatedDate(new Date());
		embryoHarvest.setDeadCount(0);
		embryoHarvest.setFragmentedCount(0);
		embryoHarvest.setOtherEmbryoTypeCount(0);
		embryoHarvest.setFlushDate(new Date());
		
		//set a default value to wildType female strain
		/*
		if(scpJob.getCryoRequestId()==null ){
			embryoHarvest.setFemaleStrain(dictionaryService.fetchEntry(
					DictionaryEnum.CRQ_CRYO_WILDTYPE_STRAIN_DICT, EmbryoHarvestService.WILD_TYPE_FEMALE_STRAIN));	
			embryoHarvest.setFemaleStrainDictId(EmbryoHarvestService.WILD_TYPE_FEMALE_STRAIN);
		}
		*/
		return embryoHarvest;
	}

	//setter methods for dependency injection

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}


	public DictionaryService getDictionaryService() {
		return dictionaryService;
	}


	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setScheduledCryoPreservationJobService(
			ScheduleCryoPreservationJobService scheduledCryoPreservationJobService) {
		this.scheduledCryoPreservationJobService = scheduledCryoPreservationJobService;
	}

	public FreezeDownSampleService getFreezeDownSampleService() {
		return freezeDownSampleService;
	}

	public void setFreezeDownSampleService(
			FreezeDownSampleService freezeDownSampleService) {
		this.freezeDownSampleService = freezeDownSampleService;
	}

	public void setCryoRequestService(CryoRequestService cryoRequestService) {
		this.cryoRequestService = cryoRequestService;
	}

	public void setCohortService(CohortService cohortService) {
		this.cohortService = cohortService;
	}

	
}
								