/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryoprocessing.services;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.time.DateUtils;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.cryoprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.cryoprocessing.FreezeDownSample;
import uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationJob;
import uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationSearchBean;
import uk.ac.sanger.mig.cryoprocessing.dao.ScheduleCryoPreservationJobDao;
import uk.ac.sanger.mig.cryoprocessing.forms.CancelScheduledCryoProcessingJobForm;
import uk.ac.sanger.mig.cryoprocessing.forms.ScheduleCryoRequestForm;
import uk.ac.sanger.mig.cryoprocessing.views.ScheduledCryoPreservationJobListView;
import uk.ac.sanger.mig.cryorequest.CryoRequest;
import uk.ac.sanger.mig.cryorequest.services.CryoRequestService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.personnel.Person;

public class ScheduleCryoPreservationJobServiceImpl extends AbstractServiceImpl<ScheduledCryoPreservationJob> implements ScheduleCryoPreservationJobService {

	private ScheduleCryoPreservationJobDao scheduleCryoPreservationJobDao;
	private EmbryoHarvestService embryoHarvestService;
	private CryoRequestService cryoRequestService;
	private StrawService strawService;
	private VialService vialService;
	private DictionaryService dictionaryService;

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setVialService(VialService vialService) {
		this.vialService = vialService;
	}

	public void setStrawService(StrawService strawService) {
		this.strawService = strawService;
	}

	public void setEmbryoHarvestService(EmbryoHarvestService embryoHarvestService) {
		this.embryoHarvestService = embryoHarvestService;
	}

	public void setCryoRequestService(CryoRequestService cryoRequestService) {
		this.cryoRequestService = cryoRequestService;
	}

	public void setScheduleCryoPreservationJobDao(ScheduleCryoPreservationJobDao cryoSchedulePreservationDao) {
		this.scheduleCryoPreservationJobDao = cryoSchedulePreservationDao;
	}


	public Long storeAuthenticated(ScheduledCryoPreservationJob cryoSchedulePreservationJob, Person user) {
		if(cryoSchedulePreservationJob == null) {
			throw new RuntimeException("Cannot save a null cryoSchedulePreservation");
		}

		return scheduleCryoPreservationJobDao.store(cryoSchedulePreservationJob, user);
	}

	public ScheduledCryoPreservationJob fetchById(Long cryoSchedulePreservationId, ComponentMap requiredComponents) {
		ScheduledCryoPreservationJob cryoSchedulePreservationJob = scheduleCryoPreservationJobDao.fetchById(cryoSchedulePreservationId);

		if(cryoSchedulePreservationJob != null)
			attachComponents(cryoSchedulePreservationJob, requiredComponents);

		return cryoSchedulePreservationJob;
	}

	@Override
	protected void attachComponents(ScheduledCryoPreservationJob cryoSchedulePreservationJob, ComponentMap requiredComponents) {
		if(cryoSchedulePreservationJob == null) {
			throw new RuntimeException("Cannot attach components to null cryoSchedulePreservation");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(ScheduledCryoPreservationJob.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledCryoPreservationJob.Component.EMBRYO_HARVEST)){
				cryoSchedulePreservationJob.setEmbryoHarvest(embryoHarvestService.fetchByScheduledJobId(
						cryoSchedulePreservationJob.getId(),requiredComponents.get(ScheduledCryoPreservationJob.Component.EMBRYO_HARVEST)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledCryoPreservationJob.Component.CRYO_REQUEST)){
				cryoSchedulePreservationJob.setCryoRequest(cryoRequestService.fetchById(
						cryoSchedulePreservationJob.getCryoRequestId(), requiredComponents.get(ScheduledCryoPreservationJob.Component.CRYO_REQUEST)));
			}
		}
	}

	//private helper methods--eg, fetchSomeSubComponent (REM: use service injection, not Dao!)

	public Collection<ScheduledCryoPreservationJobListView> fetchAllScheduledJobsbyCryoRequestId(Long cryoRequestId, ScheduleCryoRequestForm searchForm){

		return scheduleCryoPreservationJobDao.fetchAllScheduledJobsByCryoRequestId(cryoRequestId, searchForm);

	}

	public Collection<ScheduledCryoPreservationJobListView> fetchByTargetDate(Date date){

		return scheduleCryoPreservationJobDao.fetchByTargetDate(date);

	}

	public Collection<ScheduledCryoPreservationJobListView> fetchOverdueJobs(Date date){
		return scheduleCryoPreservationJobDao.fetchOverdueJobs(date);
	}


	public Collection<ScheduledCryoPreservationJob> fetchWildTypeScheduledJobByDate(Date date) {
		return scheduleCryoPreservationJobDao.fetchWildTypeScheduledJobByDate(date);

	}

	public Collection<ScheduledCryoPreservationJob> fetchActiveByCryoRequestId(Long requestId, ComponentMap requiredComponent) {
		Collection<ScheduledCryoPreservationJob> jobs = scheduleCryoPreservationJobDao.fetchActiveByCryoRequestId(requestId);
		attachComponents(jobs, requiredComponent);
		return jobs;
	}

	public Collection<ScheduledCryoPreservationJobListView> fetch(ScheduledCryoPreservationSearchBean searchBean) {
		return scheduleCryoPreservationJobDao.fetch(searchBean);
	}

	public ScheduledCryoPreservationJob fetchWithAllComponentsById(Long scheduledCryoPreservationJobId) {

		ComponentMap embryoHarvestComponentMap = new ComponentMap();

		embryoHarvestComponentMap =
			ComponentMap.putAll(
					new ComponentMap(
							EmbryoHarvest.Component.FREEZE_DOWN_SAMPLES,
							ComponentMap.putAll(
									new ComponentMap(FreezeDownSample.Component.VIAL_LOCATION, null),
									new ComponentMap(FreezeDownSample.Component.STRAW_LOCATION, null),
									new ComponentMap(FreezeDownSample.Component.TECHNICIAN, null))),
									new ComponentMap(
											EmbryoHarvest.Component.TECHNICIAN, null)
			);

		ComponentMap cryoRequestComponentMap = new ComponentMap();
		cryoRequestComponentMap = cryoRequestService.getComponentMapForHeaderFile();

		ComponentMap scheduledCryoJobComponentMap = new ComponentMap();
		scheduledCryoJobComponentMap.put(ScheduledCryoPreservationJob.Component.EMBRYO_HARVEST, embryoHarvestComponentMap);
		scheduledCryoJobComponentMap.put(ScheduledCryoPreservationJob.Component.CRYO_REQUEST, cryoRequestComponentMap);


		return this.fetchById(scheduledCryoPreservationJobId, scheduledCryoJobComponentMap);
	}

	@Transactional
	public void cancelJob(CancelScheduledCryoProcessingJobForm form, Person user) {
		ScheduledCryoPreservationJob scpJob = form.getScheduledCryoPreservationJob();
		DictionaryEntry cancelReasonDict =
			dictionaryService.fetchEntry(DictionaryEnum.CRQ_CANCEL_JOB_REASON_DICT,form.getCancelReasonDictId());
		Collection<ScheduledCryoPreservationJob> sJobs = null;
		CryoRequest request=null;




		//If its not a WT job, change the status of the request to reshcheule needed if cancel the last job and no reshceulling needed
		if(scpJob.getCryoRequestId()!=null){

			sJobs = scheduleCryoPreservationJobDao.fetchActiveByCryoRequestId(scpJob.getCryoRequestId());
			request= cryoRequestService.fetchById(scpJob.getCryoRequestId(), null);

			if(sJobs!=null){
				List<ScheduledCryoPreservationJob> jobs = new ArrayList<ScheduledCryoPreservationJob>(sJobs);
				//if scpJob is the only job and no reschedule needed make the request status reshchedue needed
				if(jobs.size() == 1 && jobs.get(0).getId().equals(scpJob.getId())){
					if(!form.isNeedReschedule()){
						request.setStatusRescheduleNeeded();
						cryoRequestService.storeAuthenticated(request, user);
					}
				}else if(jobs.size()>1){
					//not the only job, only the request with status freeze in process can possibly has its status changed to scheduled.
					//if other active jobs all scheduled => request SCHEDULED, otherwise some of them been  EHed=> IN PROCESS
					if(request.isInProcess()){
						boolean inProcess = false;
						for(ScheduledCryoPreservationJob job:jobs){
							if(!job.getId().equals(scpJob.getId())){
								//if there is a job completed or flush complete, keep the request In Process status
								if(job.isAttemptCompleted()||job.isFlushComplete()){
									inProcess = true;
									break;
								}
							}
						}
						if(inProcess == false){
							request.setStatusScheduled();
							cryoRequestService.storeAuthenticated(request, user);
						}
					}
				}else{
					throw new RuntimeException("No active jobs for this request!");
				}
			}
		}
//		update the status of cryo Job to be cancelled
		scpJob.setStatusDictId(
				ScheduledCryoPreservationJob.InternalStatus.CANCELLED.getId());
		scpJob.setCancelReasonDictId(form.getCancelReasonDictId());
		if(form.getCancelComments()!=null && !form.getCancelComments().equals("")){
			scpJob.setCancelComments( cancelReasonDict.getName()+":"+form.getCancelComments() );
		}else{
			scpJob.setCancelComments(cancelReasonDict.getName());
		}
		storeAuthenticated( scpJob, user);

		//all the samples related to this job should has its related straws or vials in use set to false
		if(scpJob.getEmbryoHarvest()!=null){
			for(FreezeDownSample sample:scpJob.getEmbryoHarvest().getFreezeDownSamples()) {
				if(sample.getStrawLocation() != null) {
					strawService.cancelStrawById(sample.getStrawLocation().getStrawId(), user, "The scheduled job cancelled.");
				}else if(sample.getVialLocation() != null) {
					vialService.cancelVialById(sample.getVialLocation().getVialId(), user, "The scheduled job cancelled.");
				}
			}
		}
		//if user choose reschedule needed as the cancel reason

		//if its normal scp job create a new job for today, in case there is no WT job, create one wt job for today as well
		if(form.isNeedReschedule()){
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(new Date());
			Date scheduledDate = DateUtils.truncate(calendar,Calendar.DATE).getTime();
			if(request!=null){
				cryoRequestService.storeAuthenticated(request, user);
				createRescheduledJob(scheduledDate, user, form.getScheduledCryoPreservationJob().getCryoRequestId(),cancelReasonDict.getName());
				createWildTypeJob(scheduledDate, user,cancelReasonDict.getName());
			}else{
				if(fetchActiveJobs(new Date()).size()>0){
				createWildTypeJob(scheduledDate, user,cancelReasonDict.getName());
				}
			}
		}
	}

	public Collection<ScheduledCryoPreservationJobListView> fetchActiveJobs(Date date){
		return scheduleCryoPreservationJobDao.fetchActiveJobs(date);
	}

	public void cancelChildJob(ScheduledCryoPreservationJob scpJob, Person person) {

		scpJob.setStatusDictId(
				ScheduledCryoPreservationJob.InternalStatus.CANCELLED.getId());
		scpJob.setCancelReasonDictId(4L);
		scpJob.setCancelComments("Cryo Request was cancelled");
		storeAuthenticated( scpJob, person);

		//all the samples related to this job should has its related straws or vials in use set to false
		if(scpJob.getEmbryoHarvest()!=null){
			for(FreezeDownSample sample:scpJob.getEmbryoHarvest().getFreezeDownSamples()) {
				if(sample.getStrawLocation() != null) {
					strawService.cancelStrawById(sample.getStrawLocation().getStrawId(), person, "The scheduled job cancelled.");
				}else if(sample.getVialLocation() != null) {
					vialService.cancelVialById(sample.getVialLocation().getVialId(), person, "The scheduled job cancelled.");
				}
			}
		}
	}


	/*
	 * Method to get the count of how many jobs pertain to the cryo request.
	 * @see uk.ac.sanger.mig.cryoprocessing.services.ScheduleCryoPreservationJobService#getNextJobName(java.lang.Long)
	 */
	public String getNextJobName(Long cryoRequestId) {

		int numberOfJobs = scheduleCryoPreservationJobDao.fetchNumberOfJobs(cryoRequestId);

		DecimalFormat reqName = new DecimalFormat("#####");
		reqName.setMinimumIntegerDigits(5); // coz. cryo request could go to 99,999 (could go beond then have to change)
		reqName.setMinimumFractionDigits(0);

		DecimalFormat nFJobName = new DecimalFormat("###");

		nFJobName.setMinimumIntegerDigits(3); //well hardcoded here, thinking there may not be more than 999 jobs for a request
		nFJobName.setMinimumFractionDigits(0);

		return "CRQ_REQ_" + reqName.format(cryoRequestId).toString() + "_JOB_" + nFJobName.format(numberOfJobs +1).toString();

	}

	/*
	 * @see uk.ac.sanger.mig.cryoprocessing.services.ScheduleCryoPreservationJobService#completeAttempt(uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationJob)
	 */
	public Long completeAttempt(ScheduledCryoPreservationJob scpJob, Person user) {
		if(scpJob != null) {
			scpJob.setStatusAttemptCompleted();
			scheduleCryoPreservationJobDao.store(scpJob, user);
			/*
			 * if(scpJob.getCryoRequest() != null && scpJob.getCryoRequest().isInProcess()) {
				 scpJob.getCryoRequest().setStatusAwaitingThawTest();
				 cryoRequestService.storeAuthenticated(scpJob.getCryoRequest(), user);
			}
			 */
		}
		else{
			throw new IllegalArgumentException("The scheduled cryo preservation job argument is null!");
		}
		return null;
	}

	public void updateCryoRequestOnSchedule(CryoRequest cryoRequest, Person user){
		CryoRequest request= cryoRequestService.fetchById(cryoRequest.getId(), null);

		if(request != null) {
			//if the request still has status cryo requested, then update it to in process
			if(request.isRequest() || request.isRescheduleNeeded()){
				request.setStatusScheduled();
				cryoRequestService.storeAuthenticated(request, user);
			}
		}
	}

	@Transactional
	public void createWildTypeJob(Date scheduledDate, Person person, String cancelReason) {
		if(fetchWildTypeScheduledJobByDate(scheduledDate).size()== 0){
			ScheduledCryoPreservationJob wildTypeScheduledCryoPreservationJob = new ScheduledCryoPreservationJob();
			wildTypeScheduledCryoPreservationJob.setComments("WildType FreezeDown Job");
			wildTypeScheduledCryoPreservationJob.setStatusScheduled();
			wildTypeScheduledCryoPreservationJob.setNextScheduledDate(scheduledDate);
			if(cancelReason!=null){
				wildTypeScheduledCryoPreservationJob.setComments("Rescheduled caused by "+cancelReason);
			}
			storeAuthenticated(wildTypeScheduledCryoPreservationJob, person);
		}

	}

	@Transactional
	public void createRescheduledJob(Date date, Person user, Long cryoRequestId, String cancelReaosn){
		ScheduledCryoPreservationJob newJob = new ScheduledCryoPreservationJob();
		newJob.setRescheduleComments(cancelReaosn);
		newJob.setCryoRequestId(cryoRequestId);
		newJob.setStatusScheduled();
		newJob.setNextScheduledDate(date);
		newJob.setJobName(getNextJobName(cryoRequestId));
		storeAuthenticated(newJob, user);
	}

	public int fetchFrozenCountById(Long id) {

		return scheduleCryoPreservationJobDao.fetchFrozenNumbersById(id);
	}
}