/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryoprocessing.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.cryoprocessing.Straw;
import uk.ac.sanger.mig.cryoprocessing.Wedge;
import uk.ac.sanger.mig.cryoprocessing.dao.WedgeDao;


public class WedgeServiceImpl extends AbstractServiceImpl<Wedge> implements WedgeService {
	
	private WedgeDao wedgeDao;
	private StrawService strawService;
	private FreezeDownSampleService freezeDownSampleService;
	
	public void setFreezeDownSampleService(
			FreezeDownSampleService freezeDownSampleService) {
		this.freezeDownSampleService = freezeDownSampleService;
	}

	public void setWedgeDao(WedgeDao wedgeDao) {
		this.wedgeDao = wedgeDao;
	}
	
	public void setStrawService(StrawService strawService) {
		this.strawService = strawService;
	}
	
	public Wedge fetchById(Long wedgeId, ComponentMap requiredComponents) {
		Wedge wedge = wedgeDao.fetchById(wedgeId);
		
		if(wedge != null)
			attachComponents(wedge, requiredComponents);
		
		return wedge;
	}
	
	public Collection<Wedge> fetchByGobletId(Long gobletId, ComponentMap componentMap) {
		
		Collection<Wedge> wedges = wedgeDao.fetchByGobletId(gobletId);
		attachComponents(wedges, componentMap);
		return wedges;
	}
	
	@Override
	protected void attachComponents(Wedge wedge, ComponentMap requiredComponents) {
		if(wedge == null) {
			throw new RuntimeException("Cannot attach components to null wedge");
		}		
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(Wedge.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			if(fetchAll || requiredComponents.containsKey(Wedge.Component.STRAWS)) {
				wedge.setStraws(fetchStraws(wedge, requiredComponents.get(Wedge.Component.STRAWS)));
			}
			
		}
	}
	
	public Set<String> fetchWedgeStrainNames(Wedge wedge) {
		
		Set<String> strainNames =  new HashSet<String>();
		
		if(wedge == null){
			throw new IllegalArgumentException("argument wedge is null!");
		}else {
			if(wedge.getStraws() != null && wedge.getStraws().size() >0) {
				ArrayList<Straw> straws = new ArrayList<Straw>(wedge.getStraws());
				
					for(Straw straw:straws) {
						if(straw.getFreezeDownSample() != null && straw.getFreezeDownSample().getId() != null){
							if(freezeDownSampleService.fetchSampleColonyNameById(straw.getFreezeDownSample().getId()) != null) {
							strainNames.add(freezeDownSampleService.fetchSampleColonyNameById(straw.getFreezeDownSample().getId()));
							}else{
								strainNames.add(freezeDownSampleService.fetchWildTypeNameById(straw.getFreezeDownSample().getId()));
							}
						}
					}
				
			}
		}
		
		return strainNames;
	}
	
		public String fetchWedgeColonyName(Wedge wedge) {
		
		String colonyName =  null;
		
		if(wedge == null){
			throw new IllegalArgumentException("argument wedge is null!");
		}else {
			if(wedge.getStraws() != null && wedge.getStraws().size() >0) {
				ArrayList<Straw> straws = new ArrayList<Straw>(wedge.getStraws());
				
					
					for(Straw straw:straws) {
						if(straw.getFreezeDownSample() != null && straw.getFreezeDownSample().getId() != null){
							colonyName = freezeDownSampleService.fetchSampleColonyNameById(straw.getFreezeDownSample().getId());
						}
						
						if(colonyName != null)
							break;
					}
				
			}
		}
		
		return colonyName;
	}
	
	private Collection<Straw> fetchStraws(Wedge wedge, ComponentMap componentMap){
		if(wedge.getId() != null) {
			return strawService.fetchActiveByWedgeId(wedge.getId(), componentMap);
			
		}else
		{
			return null;
		}
	}

}
								