/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryoprocessing.validators;

import java.util.HashSet;
import java.util.Set;

import org.springframework.validation.Errors;

import uk.ac.sanger.mig.common.AbstractBindingValidator;
import uk.ac.sanger.mig.common.AllowedFormField;
import uk.ac.sanger.mig.cryoprocessing.EmbryoHarvest;

public class EmbryoHarvestValidator extends AbstractBindingValidator {

	@Override
	public Set<AllowedFormField> defineAllowedFields() {

		Set<AllowedFormField> formFields = new HashSet<AllowedFormField>();
		formFields.add(new AllowedFormField("flushDate", "Flush Date", true) );
		formFields.add(new AllowedFormField("numberSuperovulated", "SuperOvulated Count", true) );
		formFields.add(new AllowedFormField("numberPlugged", "Number Plugged Count", true) );

		formFields.add(new AllowedFormField("cellCount16", "16 Cell count", true) );
		formFields.add(new AllowedFormField("cellCount8", "8 Cell Count", true) );
		formFields.add(new AllowedFormField("cellCount4", "4 cell Count", true) );
		formFields.add(new AllowedFormField("cellCount2", "2 cell Count", true) );
		formFields.add(new AllowedFormField("cellCount1", "1 Cell Count", true) );
		formFields.add(new AllowedFormField("deadCount", "Dead cell Count", true) );
		formFields.add(new AllowedFormField("fragmentedCount", "Fragmented Count", true) );
		formFields.add(new AllowedFormField("otherEmbryoTypeCount", "Other type count", true) );
		formFields.add(new AllowedFormField("technicianId", "Technician", true) );
		formFields.add(new AllowedFormField("femaleStrainDictId", "Female strain", false) );


		//allowed, but optional
		formFields.add(new AllowedFormField("use16Cell", "Use 16 Cell", false) );
		formFields.add(new AllowedFormField("use8Cell", "Use 8 Cell", false) );
		formFields.add(new AllowedFormField("use4Cell", "Use 4 Cell", false) );
		formFields.add(new AllowedFormField("use2Cell", "Use 2 Cell", false) );
		formFields.add(new AllowedFormField("comments", "Comments", false) );
		formFields.add(new AllowedFormField("femaleStrainComments", "Comments", false) );
		formFields.add(new AllowedFormField("technician.displayText", "Technician name", false));
		formFields.add(new AllowedFormField("cohortId", "Cohort Name", false) );

		return formFields;
	}

	@Override
	protected void validate(Object object, Errors errors, boolean hasBindingErrors) {

		if( !hasBindingErrors ) {

			EmbryoHarvest embryoHarvest = (EmbryoHarvest)object;

			if(embryoHarvest.getNumberSuperovulated() < embryoHarvest.getNumberPlugged()) {
				errors.reject(null, "The number of superovulated donors must be greater than or equal to the number plugged.");
			}

			if(embryoHarvest.isUse16Cell() && embryoHarvest.getCellCount16().equals(0)){
				errors.reject(null, "The 16 cell count must be greater than 0 when it is ticked.");
			}

			if(!embryoHarvest.isUse16Cell() && embryoHarvest.getCellCount16().intValue()>0) {
				errors.reject(null, "Please tick the box to the left of the 16 cell count.");
			}

			if(embryoHarvest.isUse8Cell() && embryoHarvest.getCellCount8().equals(0)){
				errors.reject(null, "The 8 cell count must be greater than 0 when it is ticked.");
			}

			if(!embryoHarvest.isUse8Cell() && embryoHarvest.getCellCount8().intValue()>0) {
				errors.reject(null, "Please tick the box to the left of the 8 cell count.");
			}

			if(embryoHarvest.isUse4Cell() && embryoHarvest.getCellCount4().equals(0)){
				errors.reject(null, "The 4 cell count must be greater than 0 when it is ticked.");
			}

			if(!embryoHarvest.isUse4Cell() && embryoHarvest.getCellCount4().intValue()>0) {
				errors.reject(null, "Please tick the box to the left of the 4 cell count.");
			}
			if(embryoHarvest.isUse2Cell() && embryoHarvest.getCellCount2().equals(0)){
				errors.reject(null, "The 2 cell count must be greater than 0 when it is ticked.");
			}

			if(!embryoHarvest.isUse2Cell() && embryoHarvest.getCellCount2().intValue()>0) {
				errors.reject(null, "Please tick the box to the left of the 2 cell count.");
			}

		}

	}

	public boolean supports(Class clazz) {
		return EmbryoHarvest.class.equals(clazz);
	}

}
