/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryorequest;

import java.util.Collection;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.DictionaryEntryEnum;
import uk.ac.sanger.mig.cryoprocessing.ThawTest;
import uk.ac.sanger.mig.cryoprocessing.views.SampleListView;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.personnel.Person;


public class CryoRequest extends AuthenticatedDataObject  {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		REQUESTER,
		PROJECT_LICENCE,
		PROTOCOL,
		COLONY_INFO,
		FREEZE_DOWN_SAMPLES,
		THAW_TESTS,
		STATUS
	}

    private Long requesterId;
    private Person requester;

    private Long projectLicenceId;
	private ProjectLicence projectLicence;
    private Long colonyInfoId;
    private ColonyInfo colonyInfo;

    private Protocol protocol;
    private Long femaleStrainDictId;
    private DictionaryEntry femaleStrain;

    private Long fundingSourceDictId;
    private DictionaryEntry fundingSourceDict;

    private Long statusDictId;
    private DictionaryEntry statusDict;

    private Collection<SampleListView> samples;

    private Integer targetNumber;
    private Integer totalFrozen;
    private String numberThawed;
    private String cancelReason;
    private Collection<ThawTest> thawTests;

    public enum RequestStatusEnum implements DictionaryEntryEnum {
		REQUESTED(11L ),  //system driven
		RESCHEULE_NEEDED(14L ), //system driven
		FREEZING_IN_PROCESS(13L ),  //system driven
		COMPLETED(15L ), //user driven
		CANCELLED(16L ), //user driven
		SCHEDULED(12L), //system driven
		FREEZING_IN_PROCESS_AWAITING_THAW_TEST(17L), //system driven
		FREEZING_COMPLETE_AWAITING_THAW_TEST(18L),
		FREEZING_IN_PROCESS_AWAITING_BIRTHS(19L), //system driven
		FREEZING_COMPLETE_AWAITING_BIRTHS(20L),
		FREEZING_IN_PROCESS_THAW_COMPLETE_AWAITING_GENOTYPING(21L),
		FREEZING_COMPLETE_THAW_COMPLETE_AWAITING_GENOTYPING(22L);

		private Long dictionaryId;

		RequestStatusEnum(Long dictId) {
			this.dictionaryId = dictId;
		}

		public Long getId() {
			return dictionaryId;
		}
	}


    public String getNumberThawed() {
		return numberThawed;
	}

	public void setNumberThawed(String numberThawed) {
		this.numberThawed = numberThawed;
	}

	public Integer getTargetNumber() {
		return targetNumber;
	}

	public void setTargetNumber(Integer targetNumber) {
		this.targetNumber = targetNumber;
	}

	public Long getRequesterId() {
        return requesterId;
    }

    public void setRequesterId(Long personId) {
        this.requesterId = personId;
    }

	public Person getRequester() {
		return requester;
	}

	public void setRequester(Person person) {
		this.requester = person;
	}

    public Long getProjectLicenceId() {
        return projectLicenceId;
    }

    public void setProjectLicenceId(Long projectLicenceId) {
        this.projectLicenceId = projectLicenceId;
    }

	public ProjectLicence getProjectLicence() {
		return projectLicence;
	}

	public void setProjectLicence(ProjectLicence projectLicence) {
		this.projectLicence = projectLicence;
	}

    public Long getColonyInfoId() {
        return colonyInfoId;
    }

    public void setColonyInfoId(Long colonyId) {
        this.colonyInfoId = colonyId;
    }

	public ColonyInfo getColonyInfo() {
		return colonyInfo;
	}

	public void setColonyInfo(ColonyInfo colonyInfo) {
		this.colonyInfo = colonyInfo;
	}

    public Long getFemaleStrainDictId() {
        return femaleStrainDictId;
    }

    public void setFemaleStrainDictId(Long femaleStrainDictId) {
        this.femaleStrainDictId = femaleStrainDictId;
    }

	public DictionaryEntry getFemaleStrain() {
		return femaleStrain;
	}

	public void setFemaleStrain(DictionaryEntry strain) {
		this.femaleStrain = strain;
	}

    public Long getStatusDictId() {
        return statusDictId;
    }

    public void setStatusDictId(Long statusDictId) {
        this.statusDictId = statusDictId;
    }

	public DictionaryEntry getStatusDict() {
		return statusDict;
	}

	public void setStatusDict(DictionaryEntry status) {
		this.statusDict = status;
	}

	public Integer getTotalFrozen() {
		return totalFrozen;
	}

	public void setTotalFrozen(Integer totalFrozen) {
		this.totalFrozen = totalFrozen;
	}

	public void setStatusRequested() {
		this.statusDictId = RequestStatusEnum.REQUESTED.getId();
	}

	public void setStatusScheduled() {
		this.statusDictId = RequestStatusEnum.SCHEDULED.getId();
	}

	public void setStatusRescheduleNeeded() {
		this.statusDictId = RequestStatusEnum.RESCHEULE_NEEDED.getId();
	}

	public void setStatusInProcess() {
		this.statusDictId = RequestStatusEnum.FREEZING_IN_PROCESS.getId();
	}

	public void setStatusFreezingInProcessAwaitingThawTest() {
		this.statusDictId = RequestStatusEnum.FREEZING_IN_PROCESS_AWAITING_THAW_TEST.getId();
	}

	public void setStatusFreezingCompleteAwaitingThawTest() {
		this.statusDictId = RequestStatusEnum.FREEZING_COMPLETE_AWAITING_THAW_TEST.getId();
	}

	public void setStatusFreezingInProcessAwaitingBirths() {
		this.statusDictId = RequestStatusEnum.FREEZING_IN_PROCESS_AWAITING_BIRTHS.getId();
	}

	public void setStatusFreezingCompleteAwaitingBirths() {
		this.statusDictId = RequestStatusEnum.FREEZING_COMPLETE_AWAITING_BIRTHS.getId();
	}


	public void setStatusFreezingInProcessThawCompleteAwaitingGenotyping() {
		this.statusDictId = RequestStatusEnum.FREEZING_IN_PROCESS_THAW_COMPLETE_AWAITING_GENOTYPING.getId();
	}

	public void setStatusFreezingCompleteThawCompleteAwaitingGenotyping() {
		this.statusDictId = RequestStatusEnum.FREEZING_COMPLETE_THAW_COMPLETE_AWAITING_GENOTYPING.getId();
	}
	public boolean isRequest() {
		return  (statusDict.getId() == RequestStatusEnum.REQUESTED.getId());
	}

	public boolean isInProcess() {
		return  (statusDict.getId() == RequestStatusEnum.FREEZING_IN_PROCESS.getId());
	}

	public boolean isScheduled() {
		return  (statusDict.getId() == RequestStatusEnum.SCHEDULED.getId());
	}

	public boolean isRescheduleNeeded() {
		return  (statusDict.getId() == RequestStatusEnum.RESCHEULE_NEEDED.getId());
	}

	public boolean isCompleted(){
		return  (statusDict.getId() == RequestStatusEnum.COMPLETED.getId());
	}

	public boolean isCancelled(){
		return  (statusDict.getId() == RequestStatusEnum.CANCELLED.getId());
	}

	public boolean isFreezeCompleteThawCompleteAwaitingGenotyping(){
		return  (statusDict.getId() == RequestStatusEnum.FREEZING_COMPLETE_THAW_COMPLETE_AWAITING_GENOTYPING.getId());
	}

//	public boolean isLegacyData() {
//		return(!(isCancelled()||isCompleted()||
//				isInProcess()||isRequest()||
//				isRescheduleNeeded()||isScheduled()||
//				isAwaitingThawTest()||isThawTestCompleted()));
//	}

	public Protocol getProtocol() {
		return protocol;
	}

	public void setProtocol(Protocol protocol) {
		this.protocol = protocol;
	}

	public void changeStatus(DictionaryEntry newStatus) {
		this.statusDict = newStatus;
		this.statusDictId=newStatus.getId();

	}

	public String getCancelReason() {
		return cancelReason;
	}

	public void setCancelReason(String cancelReason) {
		this.cancelReason = cancelReason;
	}

	public Collection<SampleListView> getSamples() {
		return samples;
	}

	public void setSamples(Collection<SampleListView> samples) {
		this.samples = samples;
	}

	public Collection<ThawTest> getThawTests() {
		return thawTests;
	}

	public void setThawTests(Collection<ThawTest> thawTests) {
		this.thawTests = thawTests;
	}

	public DictionaryEntry getFundingSourceDict() {
		return fundingSourceDict;
	}

	public void setFundingSourceDict(DictionaryEntry fundingSourceDict) {
		this.fundingSourceDict = fundingSourceDict;
	}

	public Long getFundingSourceDictId() {
		return fundingSourceDictId;
	}

	public void setFundingSourceDictId(Long fundingSourceDictId) {
		this.fundingSourceDictId = fundingSourceDictId;
	}
}