/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryorequest.controllers;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.CancellableFormController;

import uk.ac.sanger.mig.colony.ColonyAllele;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.cryorequest.CryoRequest;
import uk.ac.sanger.mig.cryorequest.forms.CryoRequestForm;
import uk.ac.sanger.mig.cryorequest.services.CryoRequestService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.personnel.Person;

public class CryoRequestFormController extends CancellableFormController {

	private CryoRequestService cryoRequestService;
	private DictionaryService dictionaryService;
	private ProjectLicenceService projectLicenceService;
	private ColonyInfoService colonyInfoService;

	private static final Long DEFAULT_REQUEST_STATUS = 1L;

	public void setCryoRequestService(CryoRequestService cryoRequestService) {
		this.cryoRequestService = cryoRequestService;
	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}

	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		CryoRequestForm cryoRequestForm = (CryoRequestForm)createCommand();
		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");

		CryoRequest cryoRequest = null;

		if(id != null) {
			cryoRequest = cryoRequestService.fetchById(id, ComponentMap.of(CryoRequest.Component.ALL_COMPONENTS));

		} else {//else make new cryoRequest
			cryoRequest = new CryoRequest();
			Person person = RequestUtils.getUser(httpServletRequest);
			cryoRequest.setRequester(person);
			cryoRequest.setRequesterId(person.getId());
			cryoRequest.setStatusDictId(DEFAULT_REQUEST_STATUS);

			//Get the number of project licenses this person belongs to, and if it is
			//just one then just straight away display that
			Collection<ProjectLicence> projectLicences = projectLicenceService.fetchByLicenceHolder(person.getId(), null);
			if(projectLicences != null && projectLicences.size() == 1){
				ProjectLicence projectLicence = projectLicences.iterator().next();
				cryoRequest.setProjectLicenceId(projectLicence.getId());
				cryoRequestForm.setProjectLicenceDisplayText(projectLicence.getDisplayText());
			}
		}

		cryoRequestService.inflateCryoRequestComponents(cryoRequest);
		cryoRequestForm.setCryoRequest(cryoRequest);
		return cryoRequestForm;
	}


	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest) throws Exception {
		Map<String,Object> referenceData = new HashMap<String,Object>();

		Collection<DictionaryEntry> statuses = dictionaryService.fetch(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT);
		referenceData.put("statuses", statuses);

		Collection<DictionaryEntry> femaleStrains = dictionaryService.fetch(DictionaryEnum.CRQ_CRYO_REQUEST_STRAIN_DICT);
		referenceData.put("femaleStrains", femaleStrains);

		Collection<DictionaryEntry> fundingResources = dictionaryService.fetch(DictionaryEnum.APP_FUNDING_SOURCE);
		referenceData.put("fundingSource", fundingResources);

		return referenceData;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		CryoRequestForm cryoRequestForm = (CryoRequestForm) command;
		CryoRequest cryoRequest = cryoRequestForm.getCryoRequest();

		boolean isNewRequest = (cryoRequest.getId() == null);


		if(cryoRequestForm.getCryoRequest().getStatusDictId().equals(CryoRequest.RequestStatusEnum.CANCELLED.getId())){
			cryoRequestService.cancelChildrenAndStore(cryoRequest, RequestUtils.getUser(request));
		}else{
			if(isNewRequest) {
				cryoRequest.setStatusRequested();
			}
			if(cryoRequestForm.isAddAllele()){
				ColonyAllele colonyAllele = new ColonyAllele();
				colonyAllele.setModifiedAlleleId(cryoRequestForm.getModifiedAlleleId());
				colonyAllele.setColonyId(cryoRequestForm.getCryoRequest().getColonyInfoId());
				Person person = RequestUtils.getUser(request);

				colonyInfoService.storeColonyAllele(colonyAllele, person);
			}
			cryoRequestService.storeAuthenticated(cryoRequest, RequestUtils.getUser(request));
		}


		String queryString = "?currentRecordId=" + cryoRequest.getId();

		// for new records filter by status and sort by descending ID
		if(isNewRequest) {
			queryString += "&bypassForm=true&newQuery=true&orderBy=id&orderDesc=true&cryoRequest.statusDictId=" + cryoRequest.getStatusDictId();
		}
		// for existing records, return to the last list page the user was on
		else {
			queryString += "&bypassForm=true&newQuery=false&retrieveFromSession=true";
		}

		return new ModelAndView(getSuccessView() + queryString);
	}

	@Override
	protected ModelAndView onCancel(HttpServletRequest arg0, HttpServletResponse arg1, Object command) throws Exception {
		CryoRequestForm cryoRequestForm = (CryoRequestForm) command;
		CryoRequest cryoRequest = cryoRequestForm.getCryoRequest();

		boolean isNewRequest = (cryoRequest.getId() == null);
		String queryString = "";

		if(isNewRequest){
			queryString+="";
		}else{
			queryString="?currentRecordId=" + cryoRequest.getId();
		}

		if(isNewRequest) {
			queryString += "?bypassForm=true&newQuery=true&orderBy=id&orderDesc=true";
		}
		// for existing records, return to the last list page the user was on
		else {
			queryString += "&bypassForm=true&newQuery=false&retrieveFromSession=true";
		}
		return new ModelAndView(getCancelView() + queryString);
	}


	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		String changeRequest = null;

		try{
			changeRequest = ServletRequestUtils.getStringParameter(request, "changeForm");
		}
		catch(ServletRequestBindingException e) {

		}

		if(changeRequest != null && changeRequest.length() >0) {
			return true;
		}
		else {
			return false;
		}
	}


	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command) throws Exception {
		CryoRequestForm form = (CryoRequestForm)command;
		String changeRequest = ServletRequestUtils.getRequiredStringParameter(request, "changeForm");

		if(changeRequest.equals("addAllele")
				&& form.getColonyName()!=null && form.getColonyName().length()>0) {
			ColonyInfo colonyInfo = colonyInfoService.fetchById(form.getCryoRequest().getColonyInfoId(),
					ComponentMap.of(ColonyInfo.Component.ALLELES));
			if(colonyInfo.getAlleles()==null||colonyInfo.getAlleles().size()==0){
				form.setAddAllele(true);
			}
		}

}


}