/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.cryorequest.services;


import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.colony.ColonyAllele;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.cryoprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.cryoprocessing.FreezeDownSample;
import uk.ac.sanger.mig.cryoprocessing.ScheduledCryoPreservationJob;
import uk.ac.sanger.mig.cryoprocessing.services.FreezeDownSampleService;
import uk.ac.sanger.mig.cryoprocessing.services.ScheduleCryoPreservationJobService;
import uk.ac.sanger.mig.cryoprocessing.views.SampleListView;
import uk.ac.sanger.mig.cryorequest.CryoRequest;
import uk.ac.sanger.mig.cryorequest.CryoRequestSearchBean;
import uk.ac.sanger.mig.cryorequest.dao.CryoRequestDao;
import uk.ac.sanger.mig.cryorequest.dao.CryoRequestListDao;
import uk.ac.sanger.mig.cryorequest.views.CryoRequestListView;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.licensing.services.ProtocolService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class CryoRequestServiceImpl extends AbstractServiceImpl<CryoRequest> implements CryoRequestService {

	private CryoRequestDao cryoRequestDao;
	private CryoRequestListDao cryoRequestListDao;
	private PersonService personService;
	private ProjectLicenceService projectLicenceService;
	private ColonyInfoService colonyInfoService;
	private DictionaryService dictionaryService;
	private ProtocolService protocolService;
	private ScheduleCryoPreservationJobService scheduleCryoPreservationJobService;
	private FreezeDownSampleService freezeDownSampleService;

	public void setFreezeDownSampleService(
			FreezeDownSampleService freezeDownSampleService) {
		this.freezeDownSampleService = freezeDownSampleService;
	}

	public void setCryoRequestDao(CryoRequestDao cryoRequestDao) {
		this.cryoRequestDao = cryoRequestDao;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	public Long storeAuthenticated(CryoRequest cryoRequest, Person user) {
		if(cryoRequest == null) {
			throw new RuntimeException("Cannot save a null cryoRequest");
		}

		return cryoRequestDao.store(cryoRequest, user);
	}

	public CryoRequest fetchById(Long cryoRequestId, ComponentMap requiredComponents) {
		CryoRequest cryoRequest = cryoRequestDao.fetchById(cryoRequestId);

		if(cryoRequest != null)
			attachComponents(cryoRequest, requiredComponents);

		return cryoRequest;
	}

	@Override
	protected void attachComponents(CryoRequest cryoRequest, ComponentMap requiredComponents) {
		if(cryoRequest == null) {
			throw new RuntimeException("Cannot attach components to null cryoRequest");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(CryoRequest.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			if(fetchAll || requiredComponents.containsKey(CryoRequest.Component.REQUESTER)) {
				cryoRequest.setRequester(fetchPerson(cryoRequest, requiredComponents.get(CryoRequest.Component.REQUESTER)));
			}

			if(fetchAll || requiredComponents.containsKey(CryoRequest.Component.PROJECT_LICENCE)) {
				cryoRequest.setProjectLicence(fetchProjectLicence(cryoRequest, requiredComponents.get(CryoRequest.Component.PROJECT_LICENCE)));
			}

			if(fetchAll || requiredComponents.containsKey(CryoRequest.Component.PROTOCOL)) {
				cryoRequest.setProtocol(fetchProtocol(cryoRequest, requiredComponents.get(CryoRequest.Component.PROTOCOL)));
			}

			if(fetchAll || requiredComponents.containsKey(CryoRequest.Component.COLONY_INFO)) {
				cryoRequest.setColonyInfo(fetchColonyInfo(cryoRequest, requiredComponents.get(CryoRequest.Component.COLONY_INFO)));
			}

			if(fetchAll || requiredComponents.containsKey(CryoRequest.Component.STATUS)){
				cryoRequest.setStatusDict(dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT, cryoRequest.getStatusDictId()));
			}
			if(fetchAll || requiredComponents.containsKey(CryoRequest.Component.FREEZE_DOWN_SAMPLES)){
				cryoRequest.setSamples(fetchSamples(cryoRequest, requiredComponents.get(CryoRequest.Component.FREEZE_DOWN_SAMPLES)));
			}
		}
	}


	private Collection<SampleListView> fetchSamples(CryoRequest cryoRequest, ComponentMap map) {
		if(cryoRequest.getId() != null) {
			return freezeDownSampleService.fetchSamplesByCryoRequestId(cryoRequest.getId(), null);
		}
		else {
			return null;
		}
	}

	public Person fetchPerson(CryoRequest cryoRequest, ComponentMap requiredComponents) {
		if(cryoRequest.getId() != null) {
			return personService.fetchById(cryoRequest.getRequesterId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	private ProjectLicence fetchProjectLicence(CryoRequest cryoRequest, ComponentMap requiredComponents) {
		if(cryoRequest.getProjectLicenceId() != null) {
			return projectLicenceService.fetchById(cryoRequest.getProjectLicenceId(), requiredComponents);
		}
		else {
			return null;
		}
	}


	private Protocol fetchProtocol(CryoRequest cryoRequest, ComponentMap requiredComponents) {
		if(cryoRequest.getProjectLicenceId() != null) {
			return protocolService.fetchByProcedureAndProject(MouseProcedureService.SUPEROVULATION, cryoRequest.getProjectLicenceId(), false, requiredComponents);
		}
		else {
			return null;
		}
	}

	public ColonyInfo fetchColonyInfo(CryoRequest cryoRequest, ComponentMap requiredComponents) {
		if(cryoRequest.getId() != null) {
			return colonyInfoService.fetchById(cryoRequest.getColonyInfoId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	public Collection<CryoRequest> fetch(CryoRequestSearchBean cryoRequestSearchBean, ComponentMap requiredComponents) {
		Collection<CryoRequest> cryoRequests = cryoRequestDao.fetch(cryoRequestSearchBean);
		if(cryoRequests != null)
			attachComponents(cryoRequests, requiredComponents);

		return cryoRequests;
		//return cryoRequestDao.fetch(cryoRequestSearchBean);
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateCryoRequestComponents(CryoRequest cryoRequest) {
		if(cryoRequest == null) {
			throw new RuntimeException("CryoRequest cannot be null when inflating components.");
		}

		// set colony
		if(cryoRequest.getColonyInfo() == null) {
			cryoRequest.setColonyInfo(new ColonyInfo());
		}

		// set project licence
		if(cryoRequest.getProjectLicence() == null)
			cryoRequest.setProjectLicence(new ProjectLicence());

		// set strain
		if(cryoRequest.getFemaleStrain() == null)
			cryoRequest.setFemaleStrain(new DictionaryEntry());

		// set status
		if(cryoRequest.getStatusDict() == null)
			cryoRequest.setStatusDict(new DictionaryEntry());

	}

	public Collection<ScheduledCryoPreservationJob> fetchAllScheduledJobs(Long cryoRequestId) {
		// TODO Auto-generated method stub
		return null;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public Collection<CryoRequestListView> fetchList(CryoRequestSearchBean searchBean) {
		return cryoRequestListDao.fetch(searchBean);

	}

	public void setCryoRequestListDao(CryoRequestListDao cryoRequestListDao) {
		this.cryoRequestListDao = cryoRequestListDao;
	}

	public ComponentMap getComponentMapForHeaderFile() {
		ComponentMap requiredComponent = new ComponentMap();
		requiredComponent.put(CryoRequest.Component.ALL_COMPONENTS,null);
		ComponentMap projectLicenceComponent = new ComponentMap();
		projectLicenceComponent.put(ProjectLicence.Component.ALL_COMPONENTS, null);

		ComponentMap modifiedAlleleComponent = new ComponentMap();
		modifiedAlleleComponent.put(Allele.Component.GENE, null);

		ComponentMap allelecomponent = new ComponentMap();
		allelecomponent.put(ColonyAllele.Component.MODIFIED_ALLELE, modifiedAlleleComponent);


		ComponentMap colonyInfoComponent = new ComponentMap();

		colonyInfoComponent.put(ColonyInfo.Component.STRAINS, null);
		colonyInfoComponent.put(ColonyInfo.Component.ALLELES, allelecomponent);


		requiredComponent.put(CryoRequest.Component.PROJECT_LICENCE,projectLicenceComponent);
		requiredComponent.put(CryoRequest.Component.COLONY_INFO,colonyInfoComponent);
		return requiredComponent;
	}

	public Long getMaxActiveStatusinJobs(Long cryoRequestId) {
		return cryoRequestDao.getMaxActiveStatusinJobs(cryoRequestId);


	}

	public Integer getTotalFrozen(Long cryoRequestId) {
		return cryoRequestDao.getTotalFrozen(cryoRequestId);
	}

	public Collection<String> fetchAllSampleLocations(Long cryoRequestId) {
		return cryoRequestDao.fetchAllSampleLocations(cryoRequestId);
	}

	public void setProtocolService(ProtocolService protocolService) {
		this.protocolService = protocolService;
	}
	@Transactional
	public void cancelChildrenAndStore(CryoRequest cryoRequest, Person person) {


			ComponentMap jobComponent = new ComponentMap();
			jobComponent.put(ScheduledCryoPreservationJob.Component.ALL_COMPONENTS, null);
			ComponentMap embryoharvestComponent = new ComponentMap();
			embryoharvestComponent.put(EmbryoHarvest.Component.ALL_COMPONENTS, null);
			ComponentMap freezeDownSampleComponents = new ComponentMap();
			freezeDownSampleComponents.put(FreezeDownSample.Component.ALL_COMPONENTS, null);

			embryoharvestComponent.put(EmbryoHarvest.Component.FREEZE_DOWN_SAMPLES, freezeDownSampleComponents);
			jobComponent.put(ScheduledCryoPreservationJob.Component.EMBRYO_HARVEST, embryoharvestComponent);
			Collection<ScheduledCryoPreservationJob> scheduledJobs =
				scheduleCryoPreservationJobService.fetchActiveByCryoRequestId(cryoRequest.getId(), jobComponent);

			for(ScheduledCryoPreservationJob job : scheduledJobs){
				scheduleCryoPreservationJobService.cancelChildJob(job, person);
			}
			storeAuthenticated(cryoRequest, person);
	}

	public void setScheduleCryoPreservationJobService(
			ScheduleCryoPreservationJobService scheduleCryoPreservationJobService) {
		this.scheduleCryoPreservationJobService = scheduleCryoPreservationJobService;
	}

	public void setStatus(CryoRequest cryoRequest, String newStatus, Person user) {
		switch( Enum.valueOf(CryoRequest.RequestStatusEnum.class, newStatus) ) {

		case CANCELLED:
			cryoRequest.changeStatus(
				dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
						CryoRequest.RequestStatusEnum.CANCELLED.getId()));
			cancelChildrenAndStore(cryoRequest, user);

		break;

		case COMPLETED:
			cryoRequest.changeStatus(
					dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
							CryoRequest.RequestStatusEnum.COMPLETED.getId()));
		break;

		case FREEZING_IN_PROCESS_AWAITING_THAW_TEST:
			cryoRequest.changeStatus(
					dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
							CryoRequest.RequestStatusEnum.FREEZING_IN_PROCESS_AWAITING_THAW_TEST.getId()));
		break;

		case FREEZING_COMPLETE_AWAITING_THAW_TEST:
			cryoRequest.changeStatus(
					dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
							CryoRequest.RequestStatusEnum.FREEZING_COMPLETE_AWAITING_THAW_TEST.getId()));
		break;

		case FREEZING_IN_PROCESS_AWAITING_BIRTHS:
			cryoRequest.changeStatus(
					dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
							CryoRequest.RequestStatusEnum.FREEZING_IN_PROCESS_AWAITING_BIRTHS.getId()));
		break;

		case FREEZING_COMPLETE_AWAITING_BIRTHS:
			cryoRequest.changeStatus(
					dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
							CryoRequest.RequestStatusEnum.FREEZING_COMPLETE_AWAITING_BIRTHS.getId()));
		break;

		case FREEZING_IN_PROCESS_THAW_COMPLETE_AWAITING_GENOTYPING:
			cryoRequest.changeStatus(
					dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
							CryoRequest.RequestStatusEnum.FREEZING_IN_PROCESS_THAW_COMPLETE_AWAITING_GENOTYPING.getId()));
		break;

		case FREEZING_COMPLETE_THAW_COMPLETE_AWAITING_GENOTYPING:
			cryoRequest.changeStatus(
					dictionaryService.fetchEntry(DictionaryEnum.CRQ_CRYO_REQUEST_STATUS_DICT,
							CryoRequest.RequestStatusEnum.FREEZING_COMPLETE_THAW_COMPLETE_AWAITING_GENOTYPING.getId()));
		break;

	}
		if(!newStatus.equals("CANCELLED"))   //because the call to updateChildrenAndStore has already saved the object
			storeAuthenticated(cryoRequest, user);

	}

	public boolean isCompleteOK(Long id) {
		Collection<ScheduledCryoPreservationJob> jobs = scheduleCryoPreservationJobService.fetchActiveByCryoRequestId(id, null);
		boolean complete = true;

		for(ScheduledCryoPreservationJob job:jobs){
			if(!job.isAttemptCompleted()){
				complete = false;
				break;
			}
		}
		return complete;
	}

	public boolean isAwaitingOK(Long id) {
		Collection<ScheduledCryoPreservationJob> jobs = scheduleCryoPreservationJobService.fetchActiveByCryoRequestId(id, null);
		boolean awaiting = false;

		for(ScheduledCryoPreservationJob job:jobs){
			if(job.isAttemptCompleted()){
				awaiting = true;
				break;
			}
		}
		return awaiting;
	}

}