package uk.ac.sanger.mig.customquery.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractSearchFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.customquery.CustomQuery;
import uk.ac.sanger.mig.customquery.CustomQueryColumn;
import uk.ac.sanger.mig.customquery.CustomQueryColumnValidValue;
import uk.ac.sanger.mig.customquery.ObjectFieldComparator;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFilterEnum;
import uk.ac.sanger.mig.customquery.forms.CustomQueryForm;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFormFilter;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFormSort;
import uk.ac.sanger.mig.customquery.services.CustomQueryColumnService;
import uk.ac.sanger.mig.customquery.services.CustomQueryService;
import uk.ac.sanger.mig.personnel.Person;

public class CustomQueryController extends AbstractSearchFormController {

	private FormFilterPropertyEditor formFilterPropertyEditor;
	private CustomQueryService customQueryService;
	private CustomQueryColumnService customQueryColumnService;
	private String errorView;

	public void setErrorView(String errorPage) {
		this.errorView = errorPage;
	}

	public void setCustomQueryService(CustomQueryService customQueryService) {
		this.customQueryService = customQueryService;
	}

	public void setCustomQueryColumnService(
			CustomQueryColumnService customQueryColumnService) {
		this.customQueryColumnService = customQueryColumnService;
	}

	public void setFormFilterPropertyEditor(FormFilterPropertyEditor formFilterPropertyEditor) {
		this.formFilterPropertyEditor = formFilterPropertyEditor;
	}

	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		super.initBinder(request, binder);

		binder.registerCustomEditor(CustomQueryFormFilter.class, formFilterPropertyEditor);
	}

	@Override
	protected Map referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		CustomQueryForm form = (CustomQueryForm)command;
		Map<String, Object> refData = new HashMap<String, Object>();

		Person user = RequestUtils.getUser(request);
		form.setPersonId(user.getId());

		ComponentMap columnMap = ComponentMap.of(CustomQueryColumn.Component.ALL_COMPONENTS);
		ComponentMap queryMap = new ComponentMap();
		queryMap.put(CustomQuery.Components.ALL_COMPONENTS, null);
		queryMap.put(CustomQuery.Components.QUERY_COLUMNS, columnMap);
		queryMap.put(CustomQuery.Components.DISPLAY_COLUMNS, columnMap);

		CustomQuery customQuery = null;
		if(form.getCustomQueryId() == null) {
			customQuery = customQueryService.fetchByUniqueName(form.getUniqueName(), form.getPersonId(), queryMap);
		}
		else {
			customQuery = customQueryService.fetchById(form.getCustomQueryId(), form.getPersonId(), queryMap);
		}
		refData.put("customQuery", customQuery);
		form.setCustomQueryId(customQuery.getId());
		form.setUniqueName(customQuery.getUniqueName());

		Collection<CustomQueryColumn> filterFields = customQuery.getQueryColumns();
		refData.put("filterFields", filterFields);

		Collection<CustomQueryColumn> displayFields = new ArrayList<CustomQueryColumn>(customQuery.getDisplayColumns());
		Collections.sort((List<CustomQueryColumn>)displayFields, new ObjectFieldComparator("columnTitle"));
		refData.put("displayFields", displayFields);

		Map<String, Collection<CustomQueryColumnValidValue>> validValues = new HashMap<String, Collection<CustomQueryColumnValidValue>>();
		for(CustomQueryColumn field : filterFields) {
			if(field.getValidValues() != null && !field.getValidValues().isEmpty()) {
				validValues.put(field.getColumnName(), field.getValidValues());
			}
		}
		refData.put("validValues", validValues);


		Map<String, Boolean> qualifiers = new LinkedHashMap<String, Boolean>();
		qualifiers.put("is", false);
		qualifiers.put("is not", true);
		refData.put("qualifiers", qualifiers);

		Map<CustomQueryFilterEnum, String> comparators = new LinkedHashMap<CustomQueryFilterEnum, String>();
		for(CustomQueryFilterEnum filterEnum : CustomQueryFilterEnum.values()) {
			comparators.put(filterEnum, filterEnum.getText());
		}
		refData.put("comparators", comparators);

		Map<String, Boolean> sortOrder = new LinkedHashMap<String, Boolean>();
		sortOrder.put("ascending", false);
		sortOrder.put("descending", true);
		refData.put("sortOrder", sortOrder);

		Map<String, Boolean> nullOrder = new LinkedHashMap<String, Boolean>();
		nullOrder.put("blanks last", false);
		nullOrder.put("blanks first", true);
		refData.put("nullOrder", nullOrder);

		return refData;
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		String changeRequest = ServletRequestUtils.getStringParameter(request, "changeForm", "");

		return (StringUtils.isNotEmpty(changeRequest));
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command) {
		CustomQueryForm form = (CustomQueryForm)command;

		if(form.getChangeForm().equalsIgnoreCase("addFilter")) {
			String fieldName = form.getChangeFormValue();
			if(StringUtils.isNotEmpty(fieldName)) {
				CustomQueryColumn column = customQueryColumnService.fetchByName(form.getCustomQueryId(), fieldName, ComponentMap.of(CustomQueryColumn.Component.ALL_COMPONENTS));

				if(column != null) {
					CustomQueryFormFilter filter = new CustomQueryFormFilter(column);
					form.addFilter(filter);
				}
			}
		}
		else if(form.getChangeForm().indexOf("removeFilter") == 0) {
			String indexString = form.getChangeForm().replace("removeFilter_","");
			String[] indexes = indexString.split("_");
			int fieldIndex = Integer.parseInt(indexes[0],10);
			int conditionIndex = Integer.parseInt(indexes[1], 10);
			form.removeFilter(fieldIndex, conditionIndex);
		}
		else if(form.getChangeForm().equalsIgnoreCase("addSort")) {
			String fieldName = form.getChangeFormValue();
			if(StringUtils.isNotEmpty(fieldName)) {
				form.addSort(new CustomQueryFormSort(fieldName));
			}
		}
		else if(form.getChangeForm().indexOf("removeSort") == 0) {
			int index = Integer.parseInt(form.getChangeForm().replace("removeSort_",""));
			form.removeSort(index);
		}
		else if(form.getChangeForm().indexOf("retrieveFromSession") == 0) {
			retrieveFromSession(request, form);
		}

		// reset form change attributes
		form.setChangeForm(null);
		form.setChangeFormValue(null);
	}

	private void retrieveFromSession(HttpServletRequest request, CustomQueryForm form) {
		String sessionFormName = "stored_session_customQuery_" + form.getUniqueName();
		CustomQueryForm sessionForm = (CustomQueryForm)request.getSession().getAttribute(sessionFormName);
		if(sessionForm != null) {
			form = sessionForm;
		}
		logger.debug("Attempted to retrieve customQuery session object named " + sessionFormName + ". Object=" + sessionForm);
	}

	@Override
	protected ModelAndView onSearch(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		CustomQueryForm form = (CustomQueryForm)command;

		CustomQuery tempCustomQuery = null;
		if(StringUtils.isNotEmpty(form.getUniqueName())) {
			tempCustomQuery = customQueryService.fetchByUniqueName(form.getUniqueName(), null, null);
		}
		else {
			tempCustomQuery = customQueryService.fetchById(form.getCustomQueryId(), null, null);
		}
		form.setCustomQueryId(tempCustomQuery.getId());
		form.setUniqueName(tempCustomQuery.getUniqueName());

		String queryName = customQueryService.fetchQueryName(form.getCustomQueryId());
		form.setQueryName(queryName);
		form.setAllRecordsLimit(1000);
		form.setRecordSetLength(50);

		Person user = RequestUtils.getUser(request);
		form.setPersonId(user.getId());

		Map<String, Object> model = null;
		try {
			model = customQueryService.fetchCustomQueryModel(form);
		}
		catch(Exception e) {
			model = new HashMap<String, Object>();
			model.put("error", e);
			e.printStackTrace();
			return new ModelAndView(errorView, model);
		}

		String viewName = getSuccessView();
		if(form.isExcel()) {
			viewName += "_excel";
			return new ModelAndView(viewName, model);
		}

		return new ModelAndView(getSuccessView(), model);
	}
}
