package uk.ac.sanger.mig.customquery.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.customquery.CustomQuery;
import uk.ac.sanger.mig.customquery.CustomQueryColumn;
import uk.ac.sanger.mig.customquery.CustomQueryAction;
import uk.ac.sanger.mig.customquery.SelectedRecordAction;
import uk.ac.sanger.mig.customquery.dao.CustomQueryDao;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFieldCondition;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFilterEnum;
import uk.ac.sanger.mig.customquery.forms.CustomQueryForm;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFormFilter;

public class CustomQueryServiceImpl extends AbstractServiceImpl<CustomQuery> implements CustomQueryService {

	private CustomQueryDao customQueryDao;
	private CustomQueryColumnService customQueryColumnService;
	private SelectedRecordActionService selectedRecordActionService;
	private CustomQueryActionService customQueryActionService;

	public void setCustomQueryActionService(CustomQueryActionService customQueryActionService) {
		this.customQueryActionService = customQueryActionService;
	}

	public void setCustomQueryDao(CustomQueryDao customQueryDao) {
		this.customQueryDao = customQueryDao;
	}

	public void setCustomQueryColumnService(
			CustomQueryColumnService customQueryColumnService) {
		this.customQueryColumnService = customQueryColumnService;
	}

	public void setSelectedRecordActionService(
			SelectedRecordActionService selectedRecordActionService) {
		this.selectedRecordActionService = selectedRecordActionService;
	}

	public CustomQuery fetchById(Long queryId) {
		return customQueryDao.fetchById(queryId);
	}

	public Collection<CustomQuery> fetchCustomQueries(ComponentMap requiredComponents) {
		Collection<CustomQuery> customQueries = customQueryDao.fetchCustomQueries();
		attachComponents(customQueries, requiredComponents);
		return customQueries;
	}

	public CustomQueryFormFilter constructFormFilter(Long queryId, String columnName, boolean negativeQualifier, CustomQueryFilterEnum comparator, Object testValue) {
		CustomQueryFormFilter filter = new CustomQueryFormFilter();

		CustomQueryFieldCondition fieldCondition = new CustomQueryFieldCondition(negativeQualifier, comparator, testValue);
		CustomQueryColumn column = customQueryColumnService.fetchByName(queryId, columnName, null);
		filter.setColumn(column);
		filter.getConditions().add(fieldCondition);

		return filter;
	}

	public CustomQuery fetchById(Long queryId, Long personId, ComponentMap requiredComponents) {
		CustomQuery customQuery = customQueryDao.fetchById(queryId);

		if(customQuery != null) {
			customQuery.setPersonId(personId);
			attachComponents(customQuery, requiredComponents);
		}

		return customQuery;
	}

	public CustomQuery fetchByUniqueName(String uniqueName, Long personId,  ComponentMap requiredComponents) {
		CustomQuery customQuery = customQueryDao.fetchByUniqueName(uniqueName);

		if(customQuery != null) {
			customQuery.setPersonId(personId);
			attachComponents(customQuery, requiredComponents);
		}

		return customQuery;
	}

	public Map<String, Object> fetchCustomQueryModel(CustomQueryForm form) {
		Map<String, Object> model = new HashMap<String, Object>();

		validateFormState(form);

		CustomQuery customQuery = fetchById(form.getCustomQueryId(), form.getPersonId(), ComponentMap.of(CustomQuery.Components.ALL_COMPONENTS));

		Pair<String, Collection<Object>> sql = customQueryDao.generateSQL(form);

		String sqlString = sql.getObjectA();
		Collection<Object> bindVariables = sql.getObjectB();

		// need to set the view name in the form to allow pagination
		form.setViewName(customQuery.getQueryViewName());

		Collection<Map<String, Object>> results = customQueryDao.executeCustomQuery(sqlString, bindVariables, form);
		customQuery.setResults(results);

		String modelKey = (form.getModelKey() == null) ? "customQuery" : form.getModelKey();
		model.put(modelKey, customQuery);

		return model;
	}

	@Override
	protected void attachComponents(CustomQuery customQuery, ComponentMap requiredComponents) {
		if(customQuery == null) {
			throw new RuntimeException("Cannot attach components to null customQuery");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(CustomQuery.Components.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(CustomQuery.Components.QUERY_COLUMNS)) {
				customQuery.setQueryColumns(fetchQueryColumns(customQuery, requiredComponents.get(CustomQuery.Components.QUERY_COLUMNS)));
			}

			if(fetchAll || requiredComponents.containsKey(CustomQuery.Components.DISPLAY_COLUMNS)) {
				customQuery.setDisplayColumns(fetchDisplayColumns(customQuery, requiredComponents.get(CustomQuery.Components.DISPLAY_COLUMNS)));
			}

			if(fetchAll || requiredComponents.containsKey(CustomQuery.Components.SELECTED_RECORD_ACTIONS)) {
				customQuery.setSelectedRecordActions(fetchSelectedRecordActions(customQuery, requiredComponents.get(CustomQuery.Components.SELECTED_RECORD_ACTIONS)));
			}

			if(fetchAll || requiredComponents.containsKey(CustomQuery.Components.ACTIONS)) {
				customQuery.setActions(fetchActions(customQuery, requiredComponents.get(CustomQuery.Components.ACTIONS)));
			}
		}
	}

	private Collection<CustomQueryAction> fetchActions(CustomQuery customQuery, ComponentMap requiredComponents) {
		return customQueryActionService.fetchByCustomQueryId(customQuery.getId(), requiredComponents);
	}

	private Collection<SelectedRecordAction> fetchSelectedRecordActions(CustomQuery customQuery, ComponentMap requiredComponents) {
		return selectedRecordActionService.fetchByCustomQueryId(customQuery.getId(), requiredComponents);
	}

	private Collection<CustomQueryColumn> fetchQueryColumns(CustomQuery customQuery, ComponentMap requiredComponents) {
		return customQueryColumnService.fetchFilterable(customQuery.getId(), requiredComponents);
	}

	private Collection<CustomQueryColumn> fetchDisplayColumns(CustomQuery customQuery, ComponentMap requiredComponents) {
		return customQueryColumnService.fetchColumnPrefsByQueryId(customQuery.getId(), customQuery.getPersonId(), requiredComponents);
	}

	public Collection<Long> fetchLastSelection(Long customQueryId, String sessionId) {
		return customQueryDao.fetchLastSelectionIds(sessionId, fetchQueryName(customQueryId));
	}

	public String fetchQueryName(Long customQueryId) {
		CustomQuery customQuery = customQueryDao.fetchById(customQueryId);
		return "customQuery" + customQuery.getUniqueName();
	}

	private void validateFormState(CustomQueryForm form) {
		if(form == null) {
			throw new IllegalStateException("CustomQueryForm cannot be null");
		}
		else if(form.getCustomQueryId() == null) {
			throw new IllegalStateException("customQueryId cannot be null");
		}
		else if(form.getPersonId() == null) {
			throw new IllegalStateException("personId cannot be null");
		}
	}
}
