/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.database;

import java.util.Collection;

import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.DictionaryEntryEnum;

public interface DictionaryService {


	/**
	 * This method can be used to fetch all active dictionary records
	 * plus one other record (which may be inactive).
	 *
	 * This functionality is required when editing a record whose dictionary entry may have been
	 * inactivated. It is necessary on the edit screen to only display current dictionary entries,
	 * but also to display the existing dictionary value.
	 *
	 * @param dictionaryEnum the dictionary to retrieve
	 * @param entryId the value that may or may not be active - this entry is included in the returned collection.
	 * @return
	 */
	public Collection<DictionaryEntry> fetchForEdit(DictionaryEnum dictionaryEnum, Long entryId);

	/**
	 * Returns a collection of Dictionary Entries.
	 * If activeOnly is true, only current dictionary entries are returned, otherwise all
	 * records are returned.
	 *
	 * @param dictionary DictionaryEnum of dictionary being queried
	 * @param activeOnly is true, only current dictionary entires are returned
	 * @return Collection of DictionaryEntry
	 */
	public Collection<DictionaryEntry> fetch(DictionaryEnum dictionary, boolean activeOnly);

	/**
	 * Return a collection of all active Dictionary Entries for a given dictionary.
	 *
	 * @param dictionary DictionaryEnum of dictionary being queried
	 * @return Collection of DictionaryEntry
	 */
	public Collection<DictionaryEntry> fetch(DictionaryEnum dictionary);

	/**
	 * Return a DictionaryEntry from a dictionary based on primary key.
	 *
	 * @param dictionary DictionaryEnum of dictionary being queried
	 * @param entryId primary key of dictionary entry being retrieved
	 * @return DictionaryEntry
	 */
	public DictionaryEntry fetchEntry(DictionaryEnum dictionary, Long entryId);

	/**
	 * Return a DictionaryEntry from a dictionary based on name.
	 *
	 * @param dictionary DictionaryEnum of dictionary being queried
	 * @param name of dictionary entry being retrieved
	 * @param whether query should ignore case
	 * @return DictionaryEntry
	 */
	public DictionaryEntry fetchEntry(DictionaryEnum dictionary, String name,
			boolean caseInsensitive);


	/**
	 * Returns ID of DictionaryEntry from a dictionary based on name or null
	 * if it cannot be found.
	 *
	 * @param dictionary DictionaryEnum of dictionary being queried
	 * @param name of dictionary entry being retrieved
	 * @param whether query should ignore case
	 * @return Long
	 */
	public Long fetchEntryIdByName(DictionaryEnum dictionary, String name, boolean caseInsensitive);

	/**
	 *  Return a DictionaryEntry from a dictionary given object
	 *  implementing the DictionaryEntryEnum interface.
	 * @param dictionary
	 * @param entryEnum
	 * @return
	 */
	public DictionaryEntry fetchEntry(DictionaryEnum dictionary, DictionaryEntryEnum entryEnum);

	/**
	 * Return a filtered collection of Dictionary Entries for a given dictionary
	 * dependant on an id matching on the specified column
	 *
	 * @param dictionary DictionaryEnum of dictionary being queried
	 * @param entryId primary key of dictionary dependant entry being retrieved
	 * @param dependantColumnName column name to filter on using entryId
	 * @return Collection of DictionaryEntry
	 */
	public Collection<DictionaryEntry> fetchDependant(DictionaryEnum dictionary, Long entryId, String dependantColumnName);

	/**
	 * Returns an array Dictionary Entries
	 * If activeOnly is true, only current dictionary entries are returned, otherwise all
	 * records are returned.
	 *
	 * @param dictionary DictionaryEnum of dictionary being queried
	 * @param activeOnly is true, only current dictionary entires are returned
	 * @return Map of DictionaryEntry
	 */
	public DictionaryEntry[] fetchArray(DictionaryEnum dictionary, boolean activeOnly);

}
