/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.database;

import java.io.File;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.support.SqlLobValue;
import org.springframework.jdbc.object.SqlUpdate;
import org.springframework.jdbc.support.lob.OracleLobHandler;

import uk.ac.sanger.mig.common.Pair;

public class ParameterizedSqlUpdate {

	private static Log logger = LogFactory.getLog(ParameterizedSqlUpdate.class);

	private Map<String, Pair<SqlParameter, Object>> bindVariables = new LinkedHashMap<String, Pair<SqlParameter,Object>>();
	private JdbcTemplate jdbcTemplate;
	private String primaryKeyColumn;
	private int anonymousColumnId = 0;

	public ParameterizedSqlUpdate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public ParameterizedSqlUpdate(JdbcTemplate jdbcTemplate, String primaryKeyColumn) {
		this.jdbcTemplate = jdbcTemplate;
		this.primaryKeyColumn = primaryKeyColumn;
	}

	private Pair<SqlParameter, Object> makePair(SqlParameter paramType, Object paramValue) {
		return new Pair<SqlParameter, Object>(paramType, paramValue);
	}

	public void addEnumParam(Enum param, String columnName) {
		Object value = (param == null) ? null : param.name();
		bindVariables.put(columnName, makePair(new SqlParameter(Types.VARCHAR), value));
	}

	public void addDoubleParam(Double param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.NUMERIC), param));
	}
	@Deprecated
	public void addStringParam(String param) {
		bindVariables.put(String.valueOf(anonymousColumnId++), makePair(new SqlParameter(Types.VARCHAR), param));
	}

	public void addStringParam(String param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.VARCHAR), param));
	}

	@Deprecated
	public void addIntParam(Integer param) {
		bindVariables.put(String.valueOf(anonymousColumnId++), makePair(new SqlParameter(Types.NUMERIC), param));
	}

	public void addIntParam(Integer param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.NUMERIC), param));
	}

	public void addFloatParam(Float param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.NUMERIC), param));
	}

	@Deprecated
	public void addBooleanParam(Boolean param) {
		bindVariables.put(String.valueOf(anonymousColumnId++), makePair(new SqlParameter(Types.NUMERIC), param));
	}

	public void addBooleanParam(Boolean param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.NUMERIC), param));
	}
	@Deprecated
	public void addLongParam(Long param) {
		bindVariables.put(String.valueOf(anonymousColumnId++), makePair(new SqlParameter(Types.NUMERIC), param));
	}

	public void addLongParam(Long param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.NUMERIC), param));
	}

	@Deprecated
	public void addDateParam(Date param) {
		bindVariables.put(String.valueOf(anonymousColumnId++), makePair(new SqlParameter(Types.TIMESTAMP), param));
	}

	public void addDateParam(Date param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.TIMESTAMP), param));
	}

	public void addClobParam(String param, String columnName) {
		bindVariables.put(columnName, makePair(new SqlParameter(Types.VARCHAR), param));
	}

	public void addFilePathParam(File param, String columnName) {
		String paramValue = (param == null) ? "" : param.toString();
		bindVariables.put(columnName, makePair(new SqlParameter(Types.VARCHAR), paramValue));
	}

	/**
	 * Use includeIfNull to determine what happens to the blob field in the event of a null byte array.
	 * If true, the blob will be made null in the event of a null or empty input byte array.
	 * If false, the blob column will be ignored, allowing updates of the other columns without impacting the blob.
	 * Typically false would be the required behaviour.
	 *
	 * @param bytes
	 * @param columnName
	 * @param includeIfNull - set true if you want the blob to be made null if the input byte array is null.
	 *
	 *
	 *
	 */
	public void addBlobParam(byte[] bytes, String columnName, boolean includeIfNull) {
		if(includeIfNull || (bytes != null && bytes.length > 0)) {
			OracleLobHandler olh = new OracleLobHandler();
			olh.setNativeJdbcExtractor(jdbcTemplate.getNativeJdbcExtractor());
			bindVariables.put(columnName, makePair(new SqlParameter(Types.BLOB), new SqlLobValue(bytes, olh)));
		}
	}

	public int execute(String sql) {
		return execute(sql, bindVariables.values());
	}

	private int execute(String sql, Collection<Pair<SqlParameter, Object>> paramPairs) {
		SqlUpdate sqlUpdate = new SqlUpdate(jdbcTemplate.getDataSource(), sql);

		List<Object> bindValues = new ArrayList<Object>();

		for(Pair<SqlParameter, Object> paramPair : paramPairs) {
			sqlUpdate.declareParameter(paramPair.getObjectA());
			bindValues.add(paramPair.getObjectB());
		}

		return sqlUpdate.update(bindValues.toArray());
	}

	// does not work with anonymous bind variables
	public int executeUpdate(String tableName) {
		String sql = " update " + tableName + " set ";
		List<Pair<SqlParameter, Object>> paramPairs = new ArrayList<Pair<SqlParameter,Object>>();

		if (!bindVariables.containsKey(primaryKeyColumn)) {
			throw new IllegalArgumentException("This method requires a primary key value, but we don't have one bound.");
		}

		for(String columnName : bindVariables.keySet()) {
			// exclude pk from SET clause
			if(!columnName.equals(primaryKeyColumn)) {
				sql += columnName + "=?, ";
				paramPairs.add(bindVariables.get(columnName));
			}
		}

		// remove trailing comma
		sql = StringUtils.removeEnd(sql, ", ");

		// add the PK column as the WHERE clause
		sql += " where " + primaryKeyColumn + "=?";
		paramPairs.add(bindVariables.get(primaryKeyColumn));

		logger.debug("Execute Update: SQL="+sql);
		logger.debug("Execute Update: Bind Variables=" + paramPairs);

		return execute(sql, paramPairs);
	}

	public int executeInsert(String tableName) {

		String sql = " insert into " + tableName + "(";
		List<Pair<SqlParameter, Object>> paramPairs = new ArrayList<Pair<SqlParameter,Object>>();

		String bindVariableString = "";

		for(String columnName : bindVariables.keySet()) {

			sql += columnName + ", ";
			bindVariableString += "?, ";
			paramPairs.add(bindVariables.get(columnName));
		}

		// remove trailing commas
		sql = StringUtils.removeEnd(sql, ", ");
		bindVariableString = StringUtils.removeEnd(bindVariableString, ", ");

		sql += ") values (" + bindVariableString + ")";


		logger.debug("Execute Insert: SQL="+sql);
		logger.debug("Execute Insert: Bind Variables="+paramPairs);

		return execute(sql, paramPairs);
	}
}
