package uk.ac.sanger.mig.europhenome;


import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

import org.europhenome.Parameter;
import org.europhenome.Procedure;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;

public class ProcedureDefinition extends AuthenticatedDataObject  {

  public enum Component implements ComponentEnum {
    ALL_COMPONENTS,  //add additional component enums here as needed for this AuthenticatedDataObject
    PARAMETER_DEFINITIONS
  }

  private String name;
  private String empressId;

  private Collection<ParameterDefinition> parameterDefinitions;

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }
  public String getEmpressId() {
    return empressId;
  }

  public void setEmpressId(String empressId) {
    this.empressId = empressId;
  }

  public Collection<ParameterDefinition> getParameterDefinitions() {
    return parameterDefinitions;
  }

  public void setParameterDefinitions(Collection<ParameterDefinition> parameterDefinitions) {
    this.parameterDefinitions = parameterDefinitions;
  }

  /**
   * Find the mapping required for an observation
   *
   * Parameter definitions and mappings must be attached, or an exception will be thrown
   *
   * In one case we have to report the same piece of information twice, therefore it gets two mapping entries
   *
   * @param o
   * @return the mapping, or null if there isn't one
   */
  public Collection<ParameterMapping> getMappingForObservation(Observation o) {
    Collection<ParameterMapping> mappings = new LinkedList<ParameterMapping>();

  	if (o == null) {
      throw new IllegalArgumentException("Observation cannot be null");
    }
    if (parameterDefinitions == null) {
      throw new IllegalStateException("Cannot find mapping: definitions have not been attached");
    }

    Long requiredObservationId = o.getRequiredObservationId();


    for (ParameterDefinition paramDef : parameterDefinitions) {

      if (paramDef.getParameterMappings() == null) {
        throw new IllegalStateException("Cannot find mapping: mappings have not been attached");
      }

      for (ParameterMapping mapping : paramDef.getParameterMappings()) {
        boolean match = mapping.getRequiredObservationId().equals(requiredObservationId);

        if (match) mappings.add(mapping);
      }
    }

    return mappings;
  }

  @SuppressWarnings("unchecked")
  public Pair<Boolean,Collection<String>> validate(Procedure procedure) {
    boolean valid = false;
    Collection<String> msgs = new LinkedList<String>();

    if (procedure.getEMPReSSlimID() == null || !procedure.getEMPReSSlimID().equals(empressId)) {
      valid = false;
      msgs.add("Bad empress id");
    }

    // what is absolutely essential ?
    Set<String> requiredParams = new HashSet<String>();
    for (ParameterDefinition def : parameterDefinitions) {
      if (def.isRequired()) requiredParams.add(def.getEmpressId());
    }

    for (Parameter param : (Collection<Parameter>)procedure.getParameters()) {
      ParameterDefinition paramDef = getParameterDefinitionByEmpressID(param.getParameterId());

      // is this parameter in the definition for this procedure?
      if (paramDef == null) {
        valid = false;
        msgs.add("No parameter definition found - "+param.getParameterId());
      }

      if (param.getValue() != null && (paramDef.getMinBound() != null || paramDef.getMaxBound() != null)) {
        // should be numeric
        try {
          double value = Double.valueOf(param.getValue());

          // too small
          if (paramDef.getMinBound() != null &&
              value < paramDef.getMinBound()) {
            valid = false;
            msgs.add("Value too small: "+param.getParameterId());
          }

          // too big
          if (paramDef.getMaxBound() != null &&
              value > paramDef.getMaxBound()) {
            valid = false;
            msgs.add("Value too big: "+param.getParameterId());
          }

        }
        catch (NumberFormatException e) {
          // not a number
          valid = false;
          msgs.add("Value is not numeric: "+param.getParameterId());
        }
      }

      // parameter types don't match
      if (paramDef.getParameterType().getEurophenomeId() != param.getMeasurementType()) {
        valid = false;
        msgs.add("Parameter types don't match: "+param.getParameterId());
      }


      // check off a requirement
      if (param.getValue() != null ||
          param.getParameterStatus() != null ||
          (paramDef.isDataSeries() &&
              param.getIncrements() != null &&
              param.getIncrements().size() > 0)){
        requiredParams.remove(param.getParameterId());
      }
    }


    // do we have all the essentials?
    if (requiredParams.size() > 0) {
      valid = false;
      msgs.add("Missing required parameters: "+requiredParams);
    }

    return new Pair(valid,msgs);
  }

  public ParameterDefinition getParameterDefinitionByEmpressID(String empressId) {
    if (parameterDefinitions == null) throw new IllegalStateException("Attach paramter definitions");
    ParameterDefinition desiredParamDef = null;

    for (ParameterDefinition paramDef : parameterDefinitions) {
      boolean match = (paramDef.getEmpressId().equals(empressId));

      if (match && desiredParamDef != null) {
        throw new RuntimeException("Multiple parameter definition matches for empress ID "+empressId);
      }

      if (match) desiredParamDef = paramDef;
    }

    return desiredParamDef;
  }

}