/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.europhenome.dao.rowmappers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Vector;

import org.europhenome.Parameter;
import org.europhenome.Procedure;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;


public abstract class AbstractBloodsExportRowMapper implements ParameterizedRowMapper<Procedure> {

	private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd-MMM-yyyy");

	public Procedure mapRow(ResultSet rs, int row) throws SQLException {
		Procedure p = new Procedure();
		Vector<Parameter> parameters = new Vector<Parameter>();
		p.setParameters(parameters);
		p.setEMPReSSlimID(getProcedureEmpressId());

		// base procedure setup
		String mouseBarcode = rs.getString("MOUSE_BARCODE");
		Long runId = rs.getLong("RUN_ID");
		Date runDate = rs.getDate("RUN_DATE");

		Vector<String> mouse= new Vector<String>();
		mouse.add(mouseBarcode);
		p.setMice(mouse);
		p.setStartTime(runDate);
		p.setExternalProcedureId("BR"+runId); // Use BR to denote bloods run, rather than EXP for experiment

		// standard collection notes
		Date bcDate = rs.getDate("BLOOD_COLLECTION_DATE");
		Parameter bcParam = new Parameter();
		bcParam.setParameterId(this.getCollectionDateEmpressId());
		if (bcDate == null)
			this.addDataNotAvailableStatus(bcParam);
		else
			bcParam.setValue(DATE_FORMAT.format(bcDate));
		bcParam.setMeasurementType(Parameter.PROCEDURE_METADATA);
		parameters.add(bcParam);


		String btText = rs.getString("BLEED_TYPE");
		Parameter btParam = new Parameter();
		btParam.setParameterId(this.getBleedTypeEmpressId());
		if (btText == null)
			this.addDataNotAvailableStatus(btParam);
		else
			btParam.setValue(btText);
		btParam.setMeasurementType(Parameter.PROCEDURE_METADATA);
		parameters.add(btParam);

		String anText = rs.getString("ANAESTHESIA");
		Parameter anParam = new Parameter();
		anParam.setParameterId(this.getAnaesthesiaEmpressId());
		if (anText == null)
			this.addDataNotAvailableStatus(anParam);
		else
			anParam.setValue(anText);
		anParam.setMeasurementType(Parameter.PROCEDURE_METADATA);
		parameters.add(anParam);

		// callback to get parameters specific to this implementation
		parameters.addAll(this.getParameters(rs));

		return p;
	}

	public void addDataNotAvailableStatus(Parameter p) {
		p.setParameterStatus("ESLIM_PARAMSC_005:No data");
	}

	public Parameter buildFloatParam(ResultSet rs, String column, String paramId) throws SQLException {
		Float val = rs.getFloat(column);
		Parameter param = buildParam(paramId);
		if (rs.wasNull())
			this.addDataNotAvailableStatus(param);
		else
			param.setValue(val.toString());

		return param;
	}

	public Parameter buildParam(String paramId) {
		Parameter p = new Parameter();
		p.setParameterId(paramId);
		return p;
	}

	public Parameter buildStringParam(String paramId, String value) {
		Parameter p = buildParam(paramId);
		p.setValue(value);
		return p;
	}

	abstract String getProcedureEmpressId();
	abstract String getCollectionDateEmpressId();
	abstract String getBleedTypeEmpressId();
	abstract String getAnaesthesiaEmpressId();

	abstract Collection<Parameter> getParameters(ResultSet rs) throws SQLException;
}
