/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.europhenome.services;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.europhenome.dao.CohortExportDao;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.MigGeneName;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.services.CellLineService;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.services.CohortServiceImpl;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class CohortExportServiceImpl extends CohortServiceImpl implements CohortExportService {
	private CohortExportDao cohortExportDao;
	private MouseExportedService mouseExportedService;
	private CellLineService cellLineService;
	private ColonyInfoService colonyInfoService;
	private Map<String,String> geneticBackgroundTranslation;

	public CohortExportServiceImpl() {
		super();
		geneticBackgroundTranslation = new HashMap<String, String>();
		geneticBackgroundTranslation.put("129P2 x CBA", "129P2/OlaHsd & CBA");
		geneticBackgroundTranslation.put("129P2/OldHsd x 129S5", "129P2/OlaHsd & 129S5");
		geneticBackgroundTranslation.put("129S5 & C57BL6c-c-", "129S5 & C57BL/6J-Tyrc-brd");
		geneticBackgroundTranslation.put("C57BL/6N Taconic Denmark & C57BL/6J c-c-", "C57BL/6NTac & C57BL/6J-Tyrc-brd");
		geneticBackgroundTranslation.put("C57BL/6N Taconic Denmark (prefix NTAC)", "C57BL/6NTac");
		geneticBackgroundTranslation.put("Unknown (MAMG)", "Unknown");
		geneticBackgroundTranslation.put("Unknown (MBTS)", "Unknown");
	}

	public void setMouseExportedService(MouseExportedService mouseExportedService) {
		this.mouseExportedService = mouseExportedService;
	}

	public void setCellLineService(CellLineService cellLineService) {
		this.cellLineService = cellLineService;
	}

	public void setCohortExportDao(CohortExportDao cohortExportDao) {
		this.cohortExportDao = cohortExportDao;
	}

	public Collection<Cohort> fetchCohortsToExport(ComponentMap requiredComponents) {
		Collection<Cohort> cohorts = cohortExportDao.fetchCohortsToExport();
		attachComponents(cohorts, requiredComponents);
		return cohorts;
	}

	@Transactional
	public Vector<org.europhenome.Cohort> exportCohorts(Person person) {
		ComponentMap cm = new ComponentMap(
				Cohort.Component.MICE,
				ComponentMap.putAll(
						new ComponentMap(Mouse.Component.ALLELES,ComponentMap.of(Allele.Component.GENE)),
						ComponentMap.of(Mouse.Component.CHIMERIC_DATA))
		);

		Collection<Cohort> ourCohorts = fetchCohortsToExport(cm);

		Vector<org.europhenome.Cohort> euCohorts = new Vector<org.europhenome.Cohort>();

		for(Cohort ourCohort : ourCohorts) {
			euCohorts.addAll(convertCohort(ourCohort));
			mouseExportedService.recordCohortExport(ourCohort, person);
		}
		return euCohorts;
	}

	@SuppressWarnings("unchecked")
	private Collection<org.europhenome.Cohort> convertCohort(Cohort ourCohort) {
		if (ourCohort  == null) throw new IllegalArgumentException("Method is useless without source cohort");
		if (ourCohort.getMice() == null) throw new IllegalArgumentException("Cannot build cohort from a null collection of mice");

		// remove mice that can't be exported
		filterMice(ourCohort);
		// add the colony info, it's needed for the next step
		fillInColonyInfo(ourCohort);

		// group the remaining mice
		Map<MouseGroupingCriteria,Collection<Mouse>> groupedMice = groupMice(ourCohort);

		// create one EUMODIC cohort per grouping
		Collection<org.europhenome.Cohort> euCohorts = new ArrayList<org.europhenome.Cohort>(groupedMice.size());

		for (Entry<MouseGroupingCriteria,Collection<Mouse>> entry : groupedMice.entrySet()) {
			org.europhenome.Cohort euCohort = buildEuCohort(ourCohort, entry.getKey(),entry.getValue());
			euCohorts.add(euCohort);
		}

		return euCohorts;
	}

	private org.europhenome.Cohort buildEuCohort(
			Cohort ourCohort,
			MouseGroupingCriteria criteria,
			Collection<Mouse> mice) {
		org.europhenome.Cohort euCohort = new org.europhenome.Cohort();

		euCohort.setCohortId(""+ourCohort.getId()+"_"+criteria.hashCode());
		euCohort.setStrain(translateGeneticBackground(criteria.getGeneticBackground()));

		fillIInBasicInfo(criteria, euCohort);
		fillInAlleleAndGene(euCohort, criteria.getAllele());
		fillInCellLine(criteria, euCohort);


		Vector<org.europhenome.Mouse> euMice = new Vector<org.europhenome.Mouse>();
		euCohort.setMice(euMice);
		for (Mouse m : mice) {
			boolean male = (m.getGenderDictId() == MouseService.MOUSE_GENDER_MALE);
			org.europhenome.Mouse euMouse = new org.europhenome.Mouse(male,m.getBarcode(),m.getBirthDate());
			euMice.add(euMouse);
		}

		return euCohort;
	}

	private String translateGeneticBackground(String internalGeneticBackground){
		if (geneticBackgroundTranslation.containsKey(internalGeneticBackground)){
			return geneticBackgroundTranslation.get(internalGeneticBackground);
		}
		else {
			return internalGeneticBackground.replaceAll("\\s\\(prefix \\S+\\)", "");
		}
	}

	private void fillInCellLine(MouseGroupingCriteria criteria, org.europhenome.Cohort euCohort) {
		if (!criteria.getGenotype().equals(GenotypeDictionary.WILDTYPE)
				&& criteria.colonyInfo != null){
			CellLine cellLine = null;
			Collection<CellLine> colonyCellLines = cellLineService.fetchCellLinesForColony(criteria.getColonyInfo().getId(),
					new ComponentMap(
							CellLine.Component.ALLELE,
							ComponentMap.of(Allele.Component.GENE)
					)
			);
			if (colonyCellLines.size() > 1) {
				throw new RuntimeException("Multiple cell lines found for cohort "+criteria.getCohortName());
			}
			if (!colonyCellLines.isEmpty()) {
				cellLine = colonyCellLines.iterator().next();
			}
			// set cell line information
			if (cellLine != null) {
				if (!cellLine.getModifiedAllele().equals(criteria.getAllele()))
					throw new RuntimeException(
							"Mismatch between cell line allele and mouse allele in cohort "
							+criteria.getCohortName()+" mouse had allele "
							+criteria.getAllele().getId()+", cell line has allele "
							+cellLine.getModifiedAllele().getId());

				euCohort.setCloneName(cellLine.getCloneName());

				String source = null;

				if (cellLine.getFundingDict() != null && !cellLine.getFundingDict().getName().equalsIgnoreCase("FACULTY")) {
					source = cellLine.getFundingDict().getName();
				}
				else if (cellLine.getCellLineSource().getName().toLowerCase().startsWith("team")){
					source = "WTSI "+cellLine.getCellLineSource().getName();
				}
				else {
					source = cellLine.getCellLineSource().getName();
				}

				euCohort.setSource(source);
			}
		}
	}

	private void fillInAlleleAndGene(org.europhenome.Cohort euCohort,
			Allele allele) {
		if (allele != null) {
			euCohort.setAlleleName(allele.getOfficialNameSuperscript());
			if (allele.getMutationNature() != null) {
				euCohort.setAlleleType(allele.getMutationNature().getName());
			}

			MigGene gene = allele.getMigGene();
			if (gene != null) {
				euCohort.setGeneName(gene.getPrimaryName());

				// set MGI gene symbol
				Set<MigGeneName> mgiSymbols = gene.getSources().get("MGI");
				if (mgiSymbols != null && !mgiSymbols.isEmpty()){
					euCohort.setGeneSymbol(mgiSymbols.iterator().next().getName());
				}

				String mgiId = gene.getMgiId();

				if (mgiId.length() > 1) {
					euCohort.setGeneId(mgiId);
				}
				else if (gene.getPrimaryBuildGene() != null){
					euCohort.setGeneId(gene.getPrimaryBuildGene().getPrimaryName());
				}
			}
		}
	}

	private void fillIInBasicInfo(MouseGroupingCriteria criteria, org.europhenome.Cohort euCohort) {
		euCohort.setPipelineId(criteria.getPipeline());

		if (criteria.getGenotype().equals(GenotypeDictionary.WILDTYPE)) {
			euCohort.setCloneName("background");
			euCohort.setSource("");
		}
		else if (criteria.getGenotype().equals(GenotypeDictionary.HEMIZYGOUS)){
			euCohort.setZygosity(org.europhenome.Cohort.HEMIZYGOUS);
		}
		else if (criteria.getGenotype().equals(GenotypeDictionary.HOMOZYGOUS)){
			euCohort.setZygosity(org.europhenome.Cohort.HOMOZYGOUS);
		}
		else if (criteria.getGenotype().equals(GenotypeDictionary.HETEROZYGOUS)){
			euCohort.setZygosity(org.europhenome.Cohort.HETROZYGOUS);
		}
		else {
			throw new RuntimeException("Unexpected genotype encountered in cohort "+criteria.getCohortName());
		}
	}

	private Map<MouseGroupingCriteria, Collection<Mouse>> groupMice(Cohort cohort) {
		Map<MouseGroupingCriteria, Collection<Mouse>> groupedMice = new HashMap<MouseGroupingCriteria, Collection<Mouse>>();

		for (Mouse m : cohort.getMice()) {
			MouseGroupingCriteria criteria = new MouseGroupingCriteria(m,cohort);
			if (!groupedMice.containsKey(criteria)) {
				groupedMice.put(criteria, new LinkedList<Mouse>());
			}
			groupedMice.get(criteria).add(m);
		}

		return groupedMice;
	}

	private void fillInColonyInfo(Cohort cohort) {
		Map<Long,ColonyInfo> cachedColonies = new HashMap<Long, ColonyInfo>();

		for (Mouse m : cohort.getMice()) {
			if (!cachedColonies.containsKey(m.getColonyInfoId())) {
				ColonyInfo ci = colonyInfoService.fetchById(m.getColonyInfoId(), null);
				cachedColonies.put(m.getColonyInfoId(),ci);
			}
			ColonyInfo ci = cachedColonies.get(m.getColonyInfoId());
			if (ci != null)
				m.setColonyInfo(ci);
		}
	}

	private void filterMice(Cohort cohort) {
		long[] validGenotypes = new long[] {
				GenotypeDictionary.WILDTYPE,
				GenotypeDictionary.HOMOZYGOUS,
				GenotypeDictionary.HETEROZYGOUS,
				GenotypeDictionary.HEMIZYGOUS
		};
		Arrays.sort(validGenotypes);

		Iterator<Mouse> i = cohort.getMice().iterator();

		while (i.hasNext()) {
			Mouse m = i.next();
			boolean remove = false;
			for (MouseAllele ma : m.getAlleles()) {
				if (ma.getGenotypeDictId().equals(GenotypeDictionary.FAILED)
						|| ma.getGenotypeDictId().equals(GenotypeDictionary.UNTESTED)) {
						remove = true;
				}
				else if (Arrays.binarySearch(validGenotypes, ma.getGenotypeDictId()) < 0) {
					throw new RuntimeException("Mouse cannot be exported, it's genotype isn't currently reportable: "+m.getBarcode());
				}
			}

			if (remove) i.remove();
		}
		if (cohort.getMice().isEmpty()) {
			throw new RuntimeException("Cohort did not contain any usable mice: "+cohort.getCohortName());
		}
	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	private class MouseGroupingCriteria {
		private Long cohortId;
		private int pipeline;
		private String geneticBackground;
		private Long genotype;
		private Allele allele;
		private ColonyInfo colonyInfo;
		private String cohortName;

		private int hashCode;

		private MouseGroupingCriteria (Mouse m, Cohort cohort) {
			this.cohortId = cohort.getId();
			this.cohortName = cohort.getCohortName();
			this.pipeline = cohort.getCohortPipelineDictId().intValue();
			this.geneticBackground = m.getStrain().getName();

			if (m.isMutant()) {
				if (m.getAlleles().size() != 1) throw new RuntimeException("Mouse with multiple alleles: "+m.getBarcode());
				MouseAllele ma = m.getAlleles().iterator().next();
				this.allele = ma.getAllele();
				this.genotype = ma.getGenotypeDictId();
				ColonyInfo ci = m.getColonyInfo();
				if (m.getName().startsWith(ci.getColonyPrefix()))
					this.colonyInfo = ci;
			}
			else {
				this.genotype = GenotypeDictionary.WILDTYPE;
			}
			// object is immutable, pre-calculate hash code
			hashCode = new HashCodeBuilder(99,43)
				.append(cohortId)
				.append(geneticBackground).append(genotype)
				.append(allele).append(colonyInfo)
				.toHashCode();

			if (cohort.getCohortPipelineDictId() != 1L && cohort.getCohortPipelineDictId() != 2)
				throw new RuntimeException("Bad pipeline for cohort "+cohortName);
		}

		public String getGeneticBackground() {return geneticBackground;}
		public Long getGenotype() {return genotype;}
		public Allele getAllele() {return allele;}
		public ColonyInfo getColonyInfo() {return colonyInfo;}
		public int hashCode() {return hashCode;}
		public Long getCohortId(){return cohortId;}
		public int getPipeline(){return pipeline;}
		public String getCohortName(){return cohortName;}

		public boolean equals(Object obj) {
			if (obj instanceof MouseGroupingCriteria == false) {
		     return false;
		   }
		   if (this == obj) {
		     return true;
		   }
		   MouseGroupingCriteria rhs = (MouseGroupingCriteria) obj;
		   return new EqualsBuilder()
			   .append(cohortId, rhs.cohortId)
			   .append(geneticBackground, rhs.geneticBackground)
			   .append(genotype, rhs.genotype)
			   .append(allele, rhs.allele)
			   .append(colonyInfo, rhs.colonyInfo)
			   .isEquals();
		}
	}

}
