/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.europhenome.services;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.Vector;
import java.util.logging.Logger;

import org.europhenome.Procedure;
import org.europhenome.ResultState;
import org.europhenome.XMLGenerator;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.europhenome.InvalidResultStateException;
import uk.ac.sanger.mig.personnel.Person;

public class EurophenomeExportServiceImpl implements EurophenomeExportService {

  private String centreIdentifier;
  private String tempDir = System.getProperty("java.io.tmpdir");
  private String xmlDir = System.getProperty("user.home") + '\\';
  private CohortExportService cohortExportService;
  private ExperimentExportService experimentExportService;
  private BloodsDataExportService bloodsDataExportService;

  private Logger logger = Logger.getLogger("EUMODIC Export Logger");

  public void setCohortExportService(CohortExportService cohortExportService) {
    this.cohortExportService = cohortExportService;
  }

  public void setExperimentExportService(ExperimentExportService experimentExportService) {
    this.experimentExportService = experimentExportService;
  }

  public void setCentreIdentifier(String centreIdentifier) {
    this.centreIdentifier = centreIdentifier;
  }

  public void setTempDir(String tempDir) {
    this.tempDir = tempDir;
  }

  public void setXmlDir(String xmlDir) {
    this.xmlDir = xmlDir;
  }


  @Transactional
  public void doExport(Person person){
    XMLGenerator xmlGenerator
    	= new XMLGenerator(xmlDir,tempDir,centreIdentifier,logger,logger);

    //get new cohorts
    Vector<org.europhenome.Cohort> cohorts = cohortExportService.exportCohorts(person);
    ResultState cohortResultState = xmlGenerator.addCohorts(cohorts);

    if (!cohortResultState.isValid()) {
      throw new InvalidResultStateException("Invalid Cohorts",cohortResultState);
    }
    //get death dates
    //Vector<Procedure> deathDates = mouseExportService.exportDeathDates(person);
    Collection<Procedure> deathDates = Collections.emptyList();

    // get sop based experiments for exported cohorts
    Collection<Procedure> sopDrivenProcedures = experimentExportService.exportExperiments(person);

    // get bloods experiments for exported cohorts
    Collection<Procedure> bloodsProcedures = bloodsDataExportService.exportBloodsData(person);

    // merge
    Vector<Procedure> allProcedures = new Vector<Procedure>(deathDates.size() + sopDrivenProcedures.size() + bloodsProcedures.size());
    allProcedures.addAll(deathDates);
    allProcedures.addAll(sopDrivenProcedures);
    allProcedures.addAll(bloodsProcedures);

    ResultState procedureResultState = xmlGenerator.addProcedures(allProcedures);
    if (!procedureResultState.isValid()) {
      throw new InvalidResultStateException("Invalid procedures",procedureResultState);
    }

    // generate output
    try {
      xmlGenerator.generateXML();
      xmlGenerator.generateZipfile();
    }
    catch (IOException e) {
      // rethrow this so that the transaction fails
      throw new RuntimeException(e);
    }
  }

	public void setBloodsDataExportService(BloodsDataExportService bloodsDataExportService) {
		this.bloodsDataExportService = bloodsDataExportService;
	}
}
