/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.europhenome.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Vector;

import org.europhenome.Parameter;
import org.europhenome.Procedure;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.europhenome.ParameterDefinition;
import uk.ac.sanger.mig.europhenome.ParameterMapping;
import uk.ac.sanger.mig.europhenome.ProcedureDefinition;
import uk.ac.sanger.mig.europhenome.dao.ExperimentExportDao;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.calorimetry.CalorimetryDataPoint;
import uk.ac.sanger.mig.phenotyping.calorimetry.services.CalorimetryDataPointService;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.UploadMapping;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentServiceImpl;
import uk.ac.sanger.mig.phenotyping.datacapture.services.RequiredObservationService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.UploadMappingService;

public class ExperimentExportServiceImpl extends ExperimentServiceImpl implements ExperimentExportService {
  private ExperimentExportDao experimentExportDao;
  private ProcedureDefinitionService procedureDefinitionService;
  private DataExportedService dataExportedService;
  private UploadMappingService uploadMappingService;
  private CalorimetryDataPointService calorimetryDataPointService;
  private RequiredObservationService requiredObservationService;

  public void setCalorimetryDataPointService(CalorimetryDataPointService calorimetryDataPointService) {
		this.calorimetryDataPointService = calorimetryDataPointService;
	}

	public void setUploadMappingService(UploadMappingService uploadMappingService) {
		this.uploadMappingService = uploadMappingService;
	}

	public void setProcedureDefinitionService(ProcedureDefinitionService procedureDefinitionService) {
    this.procedureDefinitionService = procedureDefinitionService;
  }

  public void setDataExportedService(DataExportedService dataExportedService) {
    this.dataExportedService = dataExportedService;
  }

  public void setExperimentExportDao(ExperimentExportDao experimentExportDao) {
    this.experimentExportDao = experimentExportDao;
  }

  public Collection<Experiment> fetchExperimentsForExport(ComponentMap requiredComponents) {
    Collection<Experiment> experiments = experimentExportDao.fetchExperimentsForExport();
    attachComponents(experiments, requiredComponents);
    return experiments;
  }

  @Transactional
  public Vector<Procedure> exportExperiments(Person person) {
    Vector<Procedure> procedures = new Vector<Procedure>();
    procedures.addAll(getProceduresForExport(person));
    return procedures;
  }

  private Collection<Procedure> getProceduresForExport(Person user) {
    Collection<Procedure> procedures = new LinkedList<Procedure>();
    Collection<Experiment> experiments = fetchExperimentsForExport(ComponentMap.putAll(
        new ComponentMap(
            Experiment.Component.PROCEDURE,
            ComponentMap.of(MouseProcedure.Component.PERSONAL_LICENCE)
        ),
        ComponentMap.of(
            Experiment.Component.MOUSE,
            Experiment.Component.OBSERVATIONS
        )
    ));

    Map<Long, Collection<ProcedureDefinition>> defCache = new HashMap<Long, Collection<ProcedureDefinition>>();
    Map<Long, RequiredObservation> reqCache = new HashMap<Long, RequiredObservation>();

    for (Experiment e : experiments) {

      if (!defCache.containsKey(e.getSopId())) {
        defCache.put(e.getSopId(),fetchProcedureDefinitions(e.getSopId()));
        Collection<RequiredObservation> ros = requiredObservationService.fetchBySopId(
        		e.getSopId(),
        		ComponentMap.of(RequiredObservation.Component.OBSERVATION_TYPE));

        for (RequiredObservation ro : ros) {
        	reqCache.put(ro.getId(), ro);
        }
      }

      Collection<ProcedureDefinition> definitions = defCache.get(e.getSopId());

      for (Observation o : e.getObservations()) {
      	o.setRequiredObservation(reqCache.get(o.getRequiredObservationId()));
      }

      procedures.addAll(createProcedures(e,definitions));
      dataExportedService.recordDataExported(e, user);
    }

    return procedures;
  }

  private Collection<ProcedureDefinition> fetchProcedureDefinitions(Long sopId) {
    Collection<ProcedureDefinition> procedureDefinitions = procedureDefinitionService.fetchBySopId(
        sopId,
        new ComponentMap(
            ProcedureDefinition.Component.PARAMETER_DEFINITIONS,
            new ComponentMap(
                ParameterDefinition.Component.PARAMETER_MAPPINGS,
                ComponentMap.putAll(
                  ComponentMap.of(
                      ParameterMapping.Component.OPTION_TRANSLATIONS,
                      ParameterMapping.Component.PARAMETER_DEFINITION
                  ),
                  new ComponentMap(
                      ParameterMapping.Component.REQUIRED_OBSERVATION,
                      ComponentMap.of(RequiredObservation.Component.OBSERVATION_TYPE)
                  )
                )
            )
        )
    );

    if (procedureDefinitions.size() < 1) {
      throw new RuntimeException("Could not find procedure definitions for mapping sop "+sopId);
    }

    return procedureDefinitions;
  }

	private Collection<Procedure> createProcedures(Experiment experiment, Collection<ProcedureDefinition> procedureDefinitions) {
    if (experiment == null) throw new IllegalArgumentException("Experiment cannot be null");
    if (experiment.getProcedure() == null) throw new IllegalArgumentException("MouseProcedure must be attached to experiment");
    if (experiment.getProcedure().getPersonalLicence() == null) throw new IllegalArgumentException("Personal licence must be attached to MouseProcedure");
    if (procedureDefinitions == null) throw new IllegalArgumentException("Procedure definitions cannot be null");
    if (procedureDefinitions.size() < 1) throw new IllegalArgumentException("Must provide some procedure definitions");

    Collection<Procedure> procedures = new LinkedList<Procedure>();

    for (ProcedureDefinition procedureDefinition : procedureDefinitions) {
    	Procedure procedure = convertExperimentToProcedure(experiment, procedureDefinition);
    	procedures.add(procedure);
    }

    return procedures;
  }

  @SuppressWarnings("unchecked")
	protected Procedure convertExperimentToProcedure(Experiment experiment, ProcedureDefinition procedureDefinition) {
    Procedure proc = createProcedure(experiment, procedureDefinition);

    proc.setParameters(createParameters(experiment, procedureDefinition));

    handleSpecialCases(proc);

    Pair<Boolean, Collection<String>> validationResult = procedureDefinition.validate(proc);

    if (!validationResult.getObjectA()) {
      for (String msg : validationResult.getObjectB()) {
        throw new RuntimeException("Validation error: "+msg+" for "+experiment.getId());
      }
    }

    //  TODO remove this, it's temporary while  the validator is wrong
  	//Parameter weightParam = findParameter("ESLIM_013_001_018", proc.getParameters());
  	//if (weightParam != null) proc.getParameters().remove(weightParam);

    return proc;
  }

	@SuppressWarnings("unchecked")
	private void handleSpecialCases(Procedure proc) {
		Collection<Parameter> params = (Collection<Parameter>)proc.getParameters();

		//  special handling for unmappable x-ray parameters
    if (proc.getEMPReSSlimID().equals("ESLIM_006_001")){
    	Parameter scanPerson = new Parameter();
    	scanPerson.setMeasurementType(Parameter.PROCEDURE_METADATA);
    	scanPerson.setParameterId("ESLIM_006_001_806");
    	scanPerson.setValue(proc.getExperimenterId());
    	params.add(scanPerson);

    	Parameter analysisPerson = new Parameter();
    	analysisPerson.setMeasurementType(Parameter.PROCEDURE_METADATA);
    	analysisPerson.setParameterId("ESLIM_006_001_807");
    	analysisPerson.setValue(proc.getExperimenterId());
    	params.add(analysisPerson);

    	Parameter images = new Parameter();
    	images.setMeasurementType(Parameter.MEDIA_SERIES);
    	images.setParameterStatus("ESLIM_PARAMSC_005:Image export not supported");
    	images.setParameterId("ESLIM_006_001_033");
    	params.add(images);

    	String[] requiredXrayParams = new String[]{
  			"ESLIM_006_001_001", "ESLIM_006_001_002", "ESLIM_006_001_003",
  			"ESLIM_006_001_004", "ESLIM_006_001_005", "ESLIM_006_001_006",
  			"ESLIM_006_001_007", "ESLIM_006_001_008", "ESLIM_006_001_009",
  			"ESLIM_006_001_010", "ESLIM_006_001_011", "ESLIM_006_001_012",
  			"ESLIM_006_001_013", "ESLIM_006_001_014", "ESLIM_006_001_015",
  			"ESLIM_006_001_016", "ESLIM_006_001_017", "ESLIM_006_001_018",
  			"ESLIM_006_001_019", "ESLIM_006_001_020", "ESLIM_006_001_021",
  			"ESLIM_006_001_022", "ESLIM_006_001_023", "ESLIM_006_001_024",
  			"ESLIM_006_001_025", "ESLIM_006_001_026", "ESLIM_006_001_027",
  			"ESLIM_006_001_028", "ESLIM_006_001_029", "ESLIM_006_001_030",
  			"ESLIM_006_001_031", "ESLIM_006_001_032", "ESLIM_006_001_034"
  		};
    	for (String empressId : requiredXrayParams) {
    		if (!hasParameter(empressId, params)){
    			Parameter p = new Parameter();
    			p.setParameterId(empressId);
    			p.setParameterStatus("ESLIM_PARAMSC_005:No data");
    			params.add(p);
    		}
    	}


    }

    // opthalmascope
    if (proc.getEMPReSSlimID().equals("ESLIM_013_001")){
    	if (!hasParameter("ESLIM_013_001_018", params)) {
    		Parameter noWeight = new Parameter();
    		noWeight.setParameterId("ESLIM_013_001_018");
    		noWeight.setParameterStatus("ESLIM_PARAMSC_007");
    		noWeight.setMeasurementType(Parameter.SIMPLE_PARAMETER);
    		params.add(noWeight);
    	}

    	Parameter images = new Parameter();
    	images.setParameterId("ESLIM_013_001_017");
    	images.setParameterStatus("ESLIM_PARAMSC_005:Image export not supported");
    	images.setMeasurementType(Parameter.MEDIA_SERIES);
    	params.add(images);
    }

    //slitlamp
    if (proc.getEMPReSSlimID().equals("ESLIM_014_001")){
    	Parameter images = new Parameter();
    	images.setParameterId("ESLIM_014_001_016");
    	images.setParameterStatus("ESLIM_PARAMSC_005:Image export not supported");
    	images.setMeasurementType(Parameter.MEDIA_SERIES);
    	params.add(images);
    }
	}

	private Parameter findParameter(String empressId,Collection<Parameter> params) {
		Parameter specifiedParam = null;

		for (Parameter p : params) {
			if( p.getParameterId().equals(empressId)) {
				specifiedParam = p;
				break;
			}
		}

		return specifiedParam;
	}

	private boolean hasParameter(String empressId, Collection<Parameter> params) {
		return findParameter(empressId, params) != null;
	}

  private Vector<Parameter> createParameters(Experiment experiment, ProcedureDefinition procedureDefinition) {
    Vector<Parameter> parameters = new Vector<Parameter>();

    for (Observation observation : experiment.getObservations()) {
      Collection<ParameterMapping> mappings = procedureDefinition.getMappingForObservation(observation);

      for (ParameterMapping mapping : mappings) {
        addParameterForObservation(observation, mapping, parameters);
      }
    }

    if (isCalorimetry(experiment)) {
    	parameters.addAll(getCalorimetryData(experiment, procedureDefinition));
    }

    return parameters;
  }

  private boolean isCalorimetry(Experiment e) {
  	boolean isCalorimetry = false;

  	Collection<UploadMapping> mappings = uploadMappingService.fetchBySopId(e.getSopId());
  	for (UploadMapping mapping : mappings){
  		if(mapping.getName().equals(UploadMappingService.Type.MGP_CALORIMETRY.name())) {
  			isCalorimetry = true;
  		}
  	}

  	return isCalorimetry;
  }

  private Collection<Parameter> getCalorimetryData(Experiment experiment, ProcedureDefinition procedureDefinition) {
  	Collection<ParameterDefinition> defs = procedureDefinition.getParameterDefinitions();

  	//series data
  	Parameter oxygenConsumption = createParameterFromTag("[VO2]",defs); // ml/h/animal
  	Parameter Co2production = createParameterFromTag("[VCO2]",defs); // ml/h/animal
  	Parameter metabolicRate = createParameterFromTag("[HP]",defs); // kcal/h/animal
  	Parameter xActivity = createParameterFromTag("[xamb]",defs); // count/hour
  	Parameter xActivityFine = createParameterFromTag("[xtot]",defs); // count/hour
  	Parameter yActivity = createParameterFromTag("[yamb]",defs); // count/hour
  	Parameter yActivityFine = createParameterFromTag("[ytot]",defs); // count/hour
  	Parameter cumalativeFoodIntake = createParameterFromTag("[food]",defs); // g

  	Collection<Parameter> parameters = new ArrayList<Parameter>(8);
  	parameters.add(oxygenConsumption);
  	parameters.add(Co2production);
  	parameters.add(metabolicRate);
  	parameters.add(xActivity);
  	parameters.add(xActivityFine);
  	parameters.add(yActivity);
  	parameters.add(yActivityFine);
  	parameters.add(cumalativeFoodIntake);

  	Collection<CalorimetryDataPoint> rawData = calorimetryDataPointService.fetchByExperimentId(experiment.getId(), null);

  	for (CalorimetryDataPoint dataPoint : rawData) {
  		Date timePoint = dataPoint.getSampleTime();

  		addIncrementValue(oxygenConsumption, dataPoint.getVo2Animal(), timePoint);
  		addIncrementValue(Co2production, dataPoint.getVco2Animal(), timePoint);
  		addIncrementValue(metabolicRate, dataPoint.getHeatAnimal(), timePoint);
  		addIncrementValue(xActivity, dataPoint.getRateXA(), timePoint);
  		addIncrementValue(xActivityFine, dataPoint.getRateXT(), timePoint);
  		addIncrementValue(yActivity, dataPoint.getRateYA(), timePoint);
  		addIncrementValue(yActivityFine, dataPoint.getRateYT(), timePoint);
  		addIncrementValue(cumalativeFoodIntake, dataPoint.getCumulativeFoodIntake(), timePoint);
  	}

  	return parameters;
  }

  private void addIncrementValue(Parameter p, Number value, Date timePoint) {
  	if (value != null){
	  	ParameterMapping fakeMapping = new ParameterMapping();
	  	fakeMapping.addIncrementValue(p, value.toString() ,timePoint);
  	}
  }

  /**
   * We can't use the usual mapping system for calorimetry data, as it isn't in the observation table
   * This will get a definition which has a tag in it's name column,
   *  and create a parameter object based upon it;
   * @param tag
   * @param defs
   * @return a new Parameter
   * @throws RuntimeException if it finds no matching tag, or too many
   */
  private Parameter createParameterFromTag(String tag, Collection<ParameterDefinition> defs){
  	ParameterDefinition definition = null;

  	for(ParameterDefinition d : defs ) {
  		if (d.getName().contains(tag)) {

	  		if (definition != null) {
	  			throw new RuntimeException("Too many definitions match with tag "+ tag);
	  		}

	  		definition = d;
  		}
  	}

  	if (definition == null) {
  		throw new RuntimeException("No definition found with tag "+tag);
  	}

  	Parameter p = new Parameter();
  	p.setParameterId(definition.getEmpressId());
  	p.setUnit(definition.getUnits());
  	p.setMeasurementType(definition.getParameterType().getEurophenomeId());

  	return p;
  }



  @SuppressWarnings("unchecked")
  private void addParameterForObservation(Observation observation, ParameterMapping mapping, Collection<Parameter> existingParameters) {
    if (observation.getValue() == null && !observation.getRequiredObservation().getObservationType().isFreeText()) {
      return;
    }

    Parameter parameter = null;
    ParameterDefinition paramDef = mapping.getParameterDefinition();

    // if this is series data there may already be parameter object to use
    if (paramDef.isDataSeries()) {
      parameter = findParameterByEmpressId(paramDef.getEmpressId(),existingParameters);
    }

    if (parameter == null) {
      parameter = new Parameter();
      existingParameters.add(parameter);

      parameter.setParameterId(mapping.getParameterDefinition().getEmpressId());

      int measurementType = mapping.getParameterDefinition().getParameterType().getEurophenomeId();
      parameter.setMeasurementType(measurementType);

      parameter.setUnit(mapping.getParameterDefinition().getUnits());

      if (paramDef.isDataSeries()) {
        parameter.setIncrements(new Vector());
        parameter.setIncrementUnit(mapping.getParameterDefinition().getSeriesUnit());
      }
    }

    String value = mapping.convertObservationForExport(observation);

    if (paramDef.isDataSeries()) {
    	mapping.addIncrementValue(parameter,value,observation.getObservationTime());
    }
    else if (observation.getValue() == null && observation.getRequiredObservation().getObservationType().isFreeText()) {
    	parameter.setValue("");
    }
    // TODO: should we handle parameter status codes differently?
    else if (value.startsWith("ESLIM_PARAMSC")) {
    	parameter.setParameterStatus(value);
    }
    else {
    	parameter.setValue(value);
    }
  }

  private Parameter findParameterByEmpressId(String empressId, Collection<Parameter> parameters) {
    Parameter param = null;

    for (Parameter p : parameters) {
      boolean match = p.getParameterId().equals(empressId);

      if (match && param != null) {
        throw new RuntimeException("Multiple parameters with same empress id: "+empressId);
      }

      if (match) param = p;
    }

    return param;
  }

  private Procedure createProcedure(Experiment experiment, ProcedureDefinition procedureDefinition) {
    Procedure p = new Procedure();

    //provide a reference to our db
    // use prefix as we will use ids from other tables
    p.setExternalProcedureId("EXP"+experiment.getId()+"_"+procedureDefinition.getId());

    // Use the mapping to EMPReSS provided
    p.setEMPReSSlimID(procedureDefinition.getEmpressId());

    // They allow for several mice, we allow for one
    Vector<String> mice = new Vector<String>();
    mice.add(experiment.getMouse().getBarcode());
    p.setMice(mice);

    // get start time & experimenter
    MouseProcedure mouseProcedure = experiment.getProcedure();
    Date startTime = mouseProcedure.getStartDate();
    String experimenter = mouseProcedure.getPersonalLicence().getPersonId().toString();

    p.setStartTime(startTime);
    p.setExperimenterId(experimenter);

    return p;
  }

	public void setRequiredObservationService(RequiredObservationService requiredObservationService) {
		this.requiredObservationService = requiredObservationService;
	}



}
