/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.filesupport.dao;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.io.IOUtils;
import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.dao.rowmappers.BinaryFileLazyRowMapper;

public class BinaryFileDaoSpringImpl extends AbstractDaoSpringImpl<BinaryFile> implements BinaryFileDao {

	private BinaryFileLazyRowMapper binaryFileLazyRowMapper;

	public void setBinaryFileLazyRowMapper(BinaryFileLazyRowMapper binaryFileLazyRowMapper) {
		this.binaryFileLazyRowMapper = binaryFileLazyRowMapper;
	}


	public BinaryFile fetchByPath(String path, String contentType) {

		BinaryFile binaryFile = new BinaryFile();
		String fileName = new File(path).getName();
		byte[] b = fetchBytesByPath( path );
		binaryFile.setContentType( contentType );
		binaryFile.setBytes( b );
		binaryFile.setFileDescription( path );
		binaryFile.setFileName(fileName);
		return binaryFile;
	}



	public Collection<BinaryFile> fetchByTableRef(TableMetadataEnum tableName, Long fkRef, boolean includeBinaryData) {
		String sql = " select * from app_binary_files where foreign_table_name = ? and foreign_key_id = ? order by file_description ";

		Collection<BinaryFile> files = null;

		if(includeBinaryData) {
			files = getSimpleJdbcTemplate().query(sql, getRowMapper(), tableName.name(), fkRef);
		} else {
			files = getSimpleJdbcTemplate().query(sql, binaryFileLazyRowMapper, tableName.name(), fkRef);
		}

		return (files == null) ? new ArrayList<BinaryFile>() : files;
	}

	public Collection<BinaryFile> fetchByTableRef(TableMetadataEnum tableName, Long fkRef, Long contentTypeDictId,
			boolean includeBinaryData) {
		String sql = " select * from app_binary_files where foreign_table_name = ? and foreign_key_id = ? and content_type_dict_id = ? order by file_description ";

		Collection<BinaryFile> files = null;
		if (includeBinaryData) {
			files = getSimpleJdbcTemplate().query(sql, getRowMapper(), tableName.name(), fkRef, contentTypeDictId);
		} else {
			files = getSimpleJdbcTemplate().query(sql, binaryFileLazyRowMapper, tableName.name(), fkRef,
					contentTypeDictId);
		}
		return (files == null) ? new ArrayList<BinaryFile>() : files;
	}


	/**
	 * Searches for files called filename and of the specified content type. Returns a collection of
	 * matching binary files.
	 *
	 * @param filename -
	 *            the filename to search for.
	 * @param contentTypeDictId -
	 *            the content type of the files to search for.
	 * @param includeBinaryData -
	 *            include the binary data.
	 * @return files - collection of matching binary files or an empty collection if nothing found.
	 */
	public Collection<BinaryFile> fetchByFilenameAndContentTypeDictId(String filename, Long contentTypeDictId,
			boolean includeBinaryData) {
		String sql = " select * from app_binary_files where file_name = ? and content_type_dict_id = ? order by created_date";
		Collection<BinaryFile> files = null;
		if (includeBinaryData) {
			files = getSimpleJdbcTemplate().query(sql, getRowMapper(), filename, contentTypeDictId);
		} else {
			files = getSimpleJdbcTemplate().query(sql, binaryFileLazyRowMapper, filename, contentTypeDictId);
		}

		return (files == null) ? new ArrayList<BinaryFile>() : files;
	}


	public BinaryFile fetchById(Long id, boolean includeBinaryData) {
		String sql = " select * from app_binary_files where id = ? ";

		BinaryFile file = null;
		try {
			if (includeBinaryData) {
				file = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), id);
			}
			else {
				file = getSimpleJdbcTemplate().queryForObject(sql, binaryFileLazyRowMapper, id);
			}
		}
		catch (IncorrectResultSizeDataAccessException e) {

		}

		return file;
	}


	protected void bindVariables(ParameterizedSqlUpdate sql, BinaryFile binaryFile) {
		sql.addBlobParam(binaryFile.getBytes(), "BINARY_DATA", false);
		sql.addStringParam(binaryFile.getFileName(), "FILE_NAME");
		sql.addStringParam(binaryFile.getContentType(), "CONTENT_TYPE");
		sql.addLongParam(binaryFile.getContentTypeDictId(), "CONTENT_TYPE_DICT_ID");
		sql.addStringParam(binaryFile.getFileDescription(), "FILE_DESCRIPTION");
		sql.addDateParam(binaryFile.getEditDate(), "EDIT_DATE");
		sql.addStringParam(binaryFile.getEditedBy(), "EDITED_BY");
		sql.addDateParam(binaryFile.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(binaryFile.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(binaryFile.getCheckNumber(), "CHECK_NUMBER");
		sql.addEnumParam(binaryFile.getForeignTableName(), "FOREIGN_TABLE_NAME");
		sql.addLongParam(binaryFile.getForeignKeyId(), "FOREIGN_KEY_ID");
		sql.addLongParam(binaryFile.getId(), "ID");
	}

	public BinaryFile fetchByName(String fileName, TableMetadataEnum tableName, Long fkRef) {
		String sql = " select * from app_binary_files where file_name = ? and foreign_table_name = ? and foreign_key_id = ?";

		BinaryFile file = null;
		try {
				file = getSimpleJdbcTemplate().queryForObject(sql, binaryFileLazyRowMapper, fileName, tableName.name(), fkRef);
		}
		catch (IncorrectResultSizeDataAccessException e) {

		}

		return (file != null) ? file : null;
	}

	private byte[] fetchBytesByPath(String path) {

		byte[] imageBytes = null;

		FileInputStream fis = null;

		try {

			fis = new FileInputStream( path );

			imageBytes = new byte[fis.available()];

			fis.read( imageBytes );

		} catch(IOException e) {

			logger.error("Failed to fetch image " + path.toString() + "\n" + e);

		} finally {

			IOUtils.closeQuietly( fis );
		}

		return imageBytes;

	}

}