/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.filesupport.services;

import java.io.File;
import java.io.IOException;

/**
 * @author ae1
 *
 */
public interface FileSystemService {

	/**
	 * Translates path into the BlueArc-mounted path for the
	 * current Win or *nix operating system.  Returns the translated path
	 * as a File object.
	 *
	 * If in Unix, does a case-insensitive match to see if path starts
	 * with winDirRoot and if so replaces it with unixDirRoot.  (This is
	 * because windows path is case-insensitive and can have many variants
	 * by case, whereas Unix is the opposite.)
	 *
	 * e.g, \\evs-users\MIG_data\file trasnslates to /nfs/mig_data/file
	 * @param fullPath full path to target file
	 * @param winDirRoot  wind dir root to swap in if win file sys
	 * @param unixDirRoot unix dir root to swap in if unix file sys
	 * @return
	 */
	public File translatePath(String fullPath, String winDirRoot, String unixDirRoot);

	/**
	 * Creates a subdirectory under temp cache directory.
	 *
	 * If subdirectory already exists, this method returns normally
	 * without doing anything.
	 *
	 * @param dirName Name of directory to create under temp cache directory.
	 * @return
	 */
	public File makeTempSubDirectory(String dirName);

	/**
	 * Generates a random hexadecimal number with width x 2 digits.
	 *
	 * @param input
	 * @param width
	 * @return
	 */
	public String generateRandomHex(String input, int width);

	/**
	 * Tries to empty the cache of all files and sub-directories.
	 *
	 * @throws IOException
	 */
	public void flushTempCache() throws IOException;

	/**
	 * Tries to empty the cache of all files and sub-directories starting with
	 * targetPrefix (case-sensitive).
	 *
	 * If String targetPrefix is an empty String, all sub-directories
	 * will be recursively deleted.
	 *
	 * @param targetDirPrefix - Prefix of the files and sub-directories to delete.
	 *
	 * @param targetPrefix
	 * @throws IOException
	 */
	public void flushTempCache(final String targetPrefix)
			throws IOException;

	public String getTempCache();

}