/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.filesupport.services;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.PrefixFileFilter;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author ae1
 *
 */
public class FileSystemServiceImpl implements FileSystemService {

	protected Log logger = LogFactory.getLog(this.getClass());


	private String tempCache;

	public File translatePath(String fullPath, String winDirRoot, String unixDirRoot)  {

		if( isUnixFileSys() &&
				fullPath.toLowerCase().startsWith(winDirRoot.toLowerCase()) ) {
			//winDirRoot requires 2 esc seqs for string literal followed by regex interpolation
			fullPath = fullPath.replaceFirst("(?i)" + winDirRoot.replace("\\", "\\\\"), unixDirRoot);
			fullPath = fullPath.replace("\\", "/");

		} else if( isWindowsFileSys() &&
				fullPath.startsWith(unixDirRoot) ) {

			fullPath = fullPath.replace(unixDirRoot, winDirRoot);
			fullPath = fullPath.replace("/", "\\");
		}

		return new File( fullPath );
	}

	public File makeTempSubDirectory(String dirName) {

		File tempSubDir = new File( getTempCacheAsFile() + File.separator + dirName);

		if (!tempSubDir.exists() && !tempSubDir.mkdir()) {

			throw new RuntimeException("Failed to create tempSubDir " + tempSubDir);
		}

		return tempSubDir;

	}


	public String generateRandomHex(String input, int width ) {

		if(width < 1) throw new RuntimeException("charWidth must be > 0");

		MessageDigest md = null;
		try {

			md = MessageDigest.getInstance("SHA");

		} catch(NoSuchAlgorithmException e) {}

	     md.update(input.getBytes());

	     byte[] b =  md.digest();

	     StringBuffer hexString = new StringBuffer();

	     for (int i = 0; i < width; i++) {
	         String plainText = Integer.toHexString(0xFF & b[i]);

	         if (plainText.length() < 2) {
	             plainText = "0" + plainText;
	         }

	         hexString.append(plainText);
	     }



		return hexString.toString();
	}


	public synchronized void flushTempCache() throws IOException {

		FileUtils.cleanDirectory( getTempCacheAsFile() );
	}


	public synchronized void flushTempCache(final String targetPrefix)  throws IOException {

		if( StringUtils.isBlank( targetPrefix ) )
			throw new IllegalArgumentException("String targetDirPrefix cannot be null, empty or just whitespace.");

		FilenameFilter filter = new PrefixFileFilter( targetPrefix );

		File[] files = getTempCacheAsFile().listFiles( filter );

		for( File file : files ) {

			if( file.isDirectory() )
				FileUtils.deleteDirectory( file );
			else
				file.delete();
		}

	}
	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////// privates ///////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	private File getTempCacheAsFile() {
		//ensure we always have a valid temp cache defined

		if( tempCache == null || tempCache.length() == 0)
			throw new IllegalStateException("tempImageCache not defined!");

		File tempImageCacheDirectory = new File( tempCache );

		if ( !tempImageCacheDirectory.canWrite() ) {
			throw new RuntimeException("Failed to find writable system directory "
					+ tempImageCacheDirectory);
		}

		return tempImageCacheDirectory;
	}

	public String getTempCache() {
		return this.tempCache;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// injections ///////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////


	public void setTempCache(String tempImageCache) {
		this.tempCache = tempImageCache;
	}

	private boolean isUnixFileSys() {
		return File.separator.equals("/");
	}

	private boolean isWindowsFileSys() {
		return File.separator.equals("\\");
	}

}
