/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.filters;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This filter activates for all requests ending in .do
 * When the filter is activated, it looks for an attribute in the session with the key value of "user"
 * If the attribute is not null, the request continues processing.
 * If the attribute is null, it is assumed that the current session is not correctly logged in, and the 
 * request is redirected to a time-out page.
 */
public class CheckUserSessionFilter implements Filter {
	
	private static final Log logger = LogFactory.getLog(CheckUserSessionFilter.class);
	
	String redirectURL = "";
	
	/**
	 * Empty method - init not required for this filter
	 */
	public void init(FilterConfig config) {		
		redirectURL = config.getInitParameter("redirectURL");		
	}
	
	/**
	 * Continue processing of request if non-null "user" attribute in session is found.
	 * Otherwise redirect to a timed-out page
	 * 
	 * @param _request incoming request
	 * @param _response outgoing response
	 * @param chain FilterChain
	 */
	public void doFilter(ServletRequest _request, ServletResponse _response,
			FilterChain chain) throws java.io.IOException,
			javax.servlet.ServletException {
		
		// cast the request and response to http
		//
		HttpServletRequest request = (HttpServletRequest) _request;
		HttpServletResponse response = (HttpServletResponse) _response;
		
		String contextPath = request.getContextPath();	
		
		Object object = request.getSession(true).getAttribute("user");
		
		// no object in the session so throw the user out of the system
		//
		if (object == null) {				
			try {
				RequestDispatcher dispatcher = request
				.getRequestDispatcher(redirectURL);
				
				dispatcher.forward(request, response);
			} catch (Exception e) {
				
				logger.error(
						" Could not not dispatch to '" + contextPath
						+ redirectURL + "' cause: " + e.getMessage());
				
				throw new ServletException(
						this.getClass().getName()
						+ " Could not not dispatch to '" + contextPath
						+ redirectURL + "' cause: " + e.getMessage());
			}
		} else {
			chain.doFilter(request, response);
		}		
	}
	
	/**
	 * Empty method - destroy not required
	 */
	public void destroy() {
		
	}	
}
