/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.geneevidence.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.geneevidence.FacultyVote;

public class FacultyVoteDaoSpringImpl extends AbstractDaoSpringImpl<FacultyVote> implements FacultyVoteDao {

	public Collection<FacultyVote> fetchAllFacultyVotesByMigGeneId(Long migGeneId) {

		String sql = " select * from GEV_FACULTY_VOTE where GENE_ID = ? ";

		Collection<FacultyVote> facultyVoteCollection = getSimpleJdbcTemplate().query(sql, getRowMapper(), migGeneId);

		return (facultyVoteCollection == null) ? new ArrayList<FacultyVote>() : facultyVoteCollection;
	}

	/**
	 * Get the current vote (the last vote they made in terms of time) for a given
	 * Person on a given Gene.
	 * 
	 * @param gene is the Gene for which we need the current vote value.
	 * @param person is the Person whos vote we are interested in.
	 * @return either the current vote placed by the supplied user on the supplied
	 *     gene or the string 'NEUTRAL' if no vote has been cast.
	 */
	public FacultyVote fetchCurrentFacultyVoteWithMigGeneIdAndPersonId(Long migGeneId, Long personId) {

		Collection<Object> bindVariables = new ArrayList<Object>();	

		FacultyVote facultyVote = null;

		String sql = "SELECT * FROM (SELECT VOTE FROM GEV_FACULTY_VOTE WHERE GENE_ID = ? AND CREATOR_ID = ? ORDER BY EDIT_DATE DESC) WHERE ROWNUM=1 ";

		bindVariables.add(migGeneId);
		bindVariables.add(personId);

		try{
			facultyVote = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), bindVariables.toArray());
		}
		catch(IncorrectResultSizeDataAccessException e) {
			logger.debug("Table name=GEV_FACULTY_VOTE;" + e);
		}		

		return facultyVote;

	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, FacultyVote facultyVote) {
		sql.addStringParam(facultyVote.getVote(), "VOTE");
		sql.addLongParam(facultyVote.getMigGeneId(), "GENE_ID");
		sql.addLongParam(facultyVote.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(facultyVote.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(facultyVote.getEditedBy(), "EDITED_BY");
		sql.addDateParam(facultyVote.getEditDate(), "EDIT_DATE");
		sql.addLongParam(facultyVote.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(facultyVote.getId(), "ID");
	}	
}
