/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.geneevidence.services;

import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.geneevidence.MigGeneSummaryScore;
import uk.ac.sanger.mig.geneevidence.dao.MigGeneSummaryScoreDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class MigGeneSummaryScoreServiceImpl extends AbstractServiceImpl<MigGeneSummaryScore> implements MigGeneSummaryScoreService {

	private MigGeneSummaryScoreDao migGeneSummaryScoreDao;
	private PersonService personService;

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	@Transactional
	public Long storeAuthenticated(MigGeneSummaryScore migGeneSummaryScore, Person person) {
		if(migGeneSummaryScore == null) {
			throw new RuntimeException("Cannot save a null migGeneSummaryScore");
		}

		return migGeneSummaryScoreDao.store(migGeneSummaryScore, person);
	}

	public MigGeneSummaryScore fetchById(Long migGeneSummaryScoreId, ComponentMap requiredComponents){
		MigGeneSummaryScore migGeneSummaryScore = migGeneSummaryScoreDao.fetchById(migGeneSummaryScoreId);
		
		if(migGeneSummaryScore != null)
			attachComponents(migGeneSummaryScore, requiredComponents);
		
		return migGeneSummaryScore;
	}

	/**
	 * Fetch all score update records from the database and render these into GeneSummaryScore objects.
	 * The collection is returned as an ArrayList of these records.
	 * 
	 * @param g is the Gene for which the records are required.
	 * @return an ArrayList of all the GeneSummaryScore objects associated with this Gene.
	 */
	public Collection<MigGeneSummaryScore> fetchAllMigGeneSummaryScoresByMigGeneId(Long migGeneId, ComponentMap requiredComponents){
		Collection<MigGeneSummaryScore> migGeneSummaryScores = migGeneSummaryScoreDao.fetchAllMigGeneSummaryScoresByMigGeneId(migGeneId);
		if(migGeneSummaryScores != null)
			for(MigGeneSummaryScore migGeneSummaryScore : migGeneSummaryScores)
				attachComponents(migGeneSummaryScore, requiredComponents);
		return migGeneSummaryScores;		
	}

	/**
	 * Fetch the current summary score for the supplied MigGene entity. The current score is
	 * taken to be the entry with the most recent date.
	 * 
	 * @param migGene is the MigGene object for which the current score is required.
	 * @return the integer value for the score most reciently associated with this gene.
	 */
	public int fetchCurrentScore(Long migGeneId){
		return migGeneSummaryScoreDao.fetchCurrentScore(migGeneId);
	}
	
	@Override
	protected void attachComponents(MigGeneSummaryScore migGeneSummaryScore, ComponentMap requiredComponents) {
		if(migGeneSummaryScore == null) {
			throw new RuntimeException("Cannot attach components to null migGeneSummaryScore");
		}		
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(MigGeneSummaryScore.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
						
			if(fetchAll || requiredComponents.containsKey(MigGeneSummaryScore.Component.CREATOR)) {
				migGeneSummaryScore.setCreator(fetchCreator(migGeneSummaryScore, requiredComponents.get(MigGeneSummaryScore.Component.CREATOR)));
			}
		}
	}
	
	private Person fetchCreator(MigGeneSummaryScore migGeneSummaryScore, ComponentMap components) {
		return personService.fetchById(migGeneSummaryScore.getCreatorId(), components);
	}

	public void inflateMigGeneSummaryScoreComponents(MigGeneSummaryScore migGeneSummaryScore) {
		if(migGeneSummaryScore == null) {
			throw new RuntimeException("MigGeneSummaryScore cannot be null when inflating components.");
		}
	}
	
	public void setMigGeneSummaryScoreDao(MigGeneSummaryScoreDao migGeneSummaryScoreDao) {
		this.migGeneSummaryScoreDao = migGeneSummaryScoreDao;
	}
}
