/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.geneset;

import java.util.Collection;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.personnel.Person;

public class MigGeneSet extends AuthenticatedDataObject {
	
	/**
	 *  Class used to maintain a name and set of notes associated with a collection of Genes. This
	 *  is typically used to store a related set of genes, for example, 'Core Gene List 2006' together
	 *  with a description of how the list was arrived at.
	 *  
	 * @author dgm
	 *
	 */
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		CONTENTS,
		CREATOR
	}

	private Long creatorId;
	private Person creator;
	
	private String name;
	private String description;
	//private int size = 0; // Used as an optimisation, could be determined from Collection.
	private Collection<MigGene> migGenes;
	private boolean isCoreSet;

	/**
	 * Get the name of the gene set, this is a user assigned String. This will typically be a user
	 * assigned meaningful name for the collection, e.g. 'Core Gene List 2006' 
	 * @return the name associated with the gene set.
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Attach a name to the collection of genes. This will typically be a user assigned meaningful name
	 * for the collection, e.g. 'Core Gene List 2006'
	 * @param name is the name to attach to this gene set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Get the user assigned description associated with the set of genes, for example how the set was generated.
	 * @return the String representing the description associated with the gene set.
	 */
	public String getDescription() {
		return description;
	}
	
	/**
	 * Set the user assigned description associated with the set of genes, for example how the set was generated.
	 * @param description the String representing the description associated with the gene set.
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	
//	/**
//	 * Return an iterator that will traverse the set of genes contained within this object.
//	 * @return an iterator that will visit (in no particular order) the set of genes stored in this
//	 *     object.
//	 */
//	public Iterator getElements(){
//		return migGenes.iterator();
//	}
	
	/**
	 * Add a gene to this gene set.
	 * @param migGene is the gene to be added.
	 */
	public void add(MigGene migGene) {
		migGenes.add(migGene);
		//size += 1;
	}
	
//	/**
//	 * Get the current count of genes in the set.
//	 * @return the number of genes currently stored in this set.
//	 */
//	public int getSize() {
//		return size;
//	}
//	
//	/**
//	 * Set the size of the set. This is pre-fetched since some operations rely upon knowing how big the set is
//	 * but dont actally need to load the actual collection og Genes.
//	 * 
//	 * @param size is the current size of the set (as observed by the Dao). This should only be used by the Dao.
//	 */
//	public void setSize(int size){
//		this.size = size;
//	}

	/**
	 * Set the flag to indicate that this set is a 'Core set'. This identifies a gene set that should be included
	 * when calculating the number of sets a specific gene appears in.
	 * 
	 * @param value should be set to true if the list is to be a 'public' list.
	 */
	public void setCoreSet(boolean value) {
		isCoreSet = value;
	}
	
	/**
	 * Determine if this gene set if considered to be a 'core set'. That is a set that should be counted
	 * when computing the number of sets a specific gene appears in.
	 * @return true if the set should form part of the computation of the number of sets a gene appears in.
	 */
	public boolean isCoreSet(){
		return isCoreSet;
	}

	public Person getCreator() {
		return creator;
	}

	public void setCreator(Person creator) {
		this.creator = creator;
	}

	public Long getCreatorId() {
		return creatorId;
	}

	public void setCreatorId(Long creatorId) {
		this.creatorId = creatorId;
	}

	public Collection<MigGene> getMigGenes() {
		return migGenes;
	}

	public void setMigGenes(Collection<MigGene> migGenes) {
		this.migGenes = migGenes;
	}
	
}