/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.geneset.services;

import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.services.MigGeneService;
import uk.ac.sanger.mig.geneset.MigGeneSet;
import uk.ac.sanger.mig.geneset.MigGeneSetSearchBean;
import uk.ac.sanger.mig.geneset.dao.MigGeneSetDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class MigGeneSetServiceImpl extends AbstractServiceImpl<MigGeneSet> implements MigGeneSetService {
	
	private MigGeneSetDao migGeneSetDao;
	private PersonService personService;
	private MigGeneService migGeneService;
	
	public void setMigGeneService(MigGeneService migGeneService) {
		this.migGeneService = migGeneService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}
	
	public void setMigGeneSetDao(MigGeneSetDao migGeneSetDao) {
		this.migGeneSetDao = migGeneSetDao;
	}
	
	@Transactional
	public Long storeMigGeneSet(MigGeneSet migGeneSet, Person user){
		if(migGeneSet == null) {
			throw new RuntimeException("Cannot save a null migGeneSet");
		}
		
		Long migGeneSetId = migGeneSetDao.store(migGeneSet, user);
		
		for(MigGene migGene : migGeneSet.getMigGenes()){
			migGeneService.linkToMigGeneSet(migGene.getId(), migGeneSetId);
		}
		
		return migGeneSetId;
	}
	
	public MigGeneSet fetchById(Long migGeneSetId, ComponentMap requiredComponents) {
		MigGeneSet migGeneSet = migGeneSetDao.fetchById(migGeneSetId);
		
		if(migGeneSet != null)
			attachComponents(migGeneSet, requiredComponents);
		
		return migGeneSet;
	}
	
	public Collection<MigGeneSet> fetchAllWithMigGeneId(MigGeneSetSearchBean migGeneSetSearchBean) {
		return migGeneSetDao.fetchAllWithMigGeneId(migGeneSetSearchBean);
	}
	
	public Collection<MigGeneSet> fetchAll(MigGeneSetSearchBean migGeneSetSearchBean) {
		return migGeneSetDao.fetchAll(migGeneSetSearchBean);
	}

//	public MigGeneSet createMigGeneSetFromNameList(Collection<String> targets){
//		
//		MigGeneSet migGeneSuperSet = new MigGeneSet();
//		
//		int itemCount = 0;
//		for(String targetName : targets) {
//			MigGeneSet migGeneSubSet = fetchMigGeneSetByMigGeneName(targetName);		
//			
//			if(migGeneSubSet != null) {
//				Iterator iter = migGeneSubSet.i.getElements();
//				while(iter.hasNext()) {		
//					migGeneSubSet.add((Gene)iter.next());
//					itemCount++;
//				}
//				
//			} else {
//				logger.debug("Gene Name: " + targetName + " not found for any genes in MIG!");
//				//figure out way to show user not found list
//			}			
//		}
//		migGeneSuperSet.setSize(itemCount);
//		return migGeneSuperSet;
//	}
	
	@Override
	protected void attachComponents(MigGeneSet migGeneSet, ComponentMap requiredComponents) {
		if(migGeneSet == null) {
			throw new RuntimeException("Cannot attach components to null migGeneSet");
		}		
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(MigGeneSet.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			
			if(fetchAll || requiredComponents.containsKey(MigGeneSet.Component.CREATOR)) {
				migGeneSet.setCreator(fetchCreator(migGeneSet, requiredComponents.get(MigGeneSet.Component.CREATOR)));
			}
			
			if(fetchAll || requiredComponents.containsKey(MigGeneSet.Component.CONTENTS)) {
				migGeneSet.setMigGenes(fetchContents(migGeneSet, requiredComponents.get(MigGeneSet.Component.CONTENTS)));
			}
		}
	}

	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.geneset.services.GeneSetService#search(uk.ac.sanger.mig.geneset.GeneSetSearchBean)
	 */
	public Collection<MigGeneSet> search(MigGeneSetSearchBean migGeneSetSearchBean) {
		return migGeneSetDao.search(migGeneSetSearchBean);
	}
	
	//private helper methods--eg, fetchSomeSubComponent (REM: use service injection, not Dao!)
	
	private Person fetchCreator(MigGeneSet migGeneSet, ComponentMap requiredComponents){
		return personService.fetchById(migGeneSet.getCreatorId(), requiredComponents);
	}
	
	private Collection<MigGene> fetchContents(MigGeneSet migGeneSet, ComponentMap requiredComponents) {		
		return migGeneService.fetchMigGenesForGeneSetId(migGeneSet.getId(), requiredComponents);
	}
}
								