/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.genotyping;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;

public class AssayPlate extends AuthenticatedDataObject implements BarcodedObject {

	public static final int DEFAULT_ROWS = 8;
	public static final int DEFAULT_COLUMNS = 12;

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		COHORT,
		WELL_ARRAY,
		ASSAYS
	}

    private Long cohortId;
    private String sheetBarcode;
    private String plateBarcode;
    private boolean autoPopulated;
    private int rowsOnPlate;
    private int columnsOnPlate;
    private Long sampleTypeDictId;
    private DictionaryEntry sampleType;
    private Long plateCategoryId;
    private DictionaryEntry plateCategory;
    private Boolean postMortem;

    private Cohort cohort;
    private Collection<Assay> assays;
    private AssayPlateWell[][] wellArray;

    private AssayPlate() {
    	//hidden default constructor
    }

    /**
     * Construct a new AssayPlate with an empty wellArray of rows x columns
     * @param rows number of rows in the plate wellArray
     * @param columns number columns in the plate wellArray
     */
    public AssayPlate(int rows, int columns) {
    	this.rowsOnPlate = rows;
    	this.columnsOnPlate = columns;
    	this.wellArray = new AssayPlateWell[rowsOnPlate][columnsOnPlate];
    }

    public void setWell(int row, int column, AssayPlateWell well) {
    	if(row < rowsOnPlate && column < columnsOnPlate)
    	this.wellArray[row][column] = well;
    }

    public Long getCohortId() {
        return cohortId;
    }

    public Collection<AssayPlateWell> getWells() {
    	Collection<AssayPlateWell> wells = new ArrayList<AssayPlateWell>();

    	for(AssayPlateWell[] rows : wellArray) {
    		wells.addAll(Arrays.asList(rows));
    	}

    	return wells;
    }

    public void setCohortId(Long cohortId) {
        this.cohortId = cohortId;
    }

    public Collection<Assay> getAssays() {
		return assays;
	}

	public void setAssays(Collection<Assay> assays) {
		this.assays = assays;
	}

	public String getSheetBarcode() {
        return sheetBarcode;
    }

    public void setSheetBarcode(String sheetBarcode) {
        this.sheetBarcode = sheetBarcode;
    }
    public String getPlateBarcode() {
        return plateBarcode;
    }

    public void setPlateBarcode(String plateBarcode) {
        this.plateBarcode = plateBarcode;
    }
    public boolean isAutoPopulated() {
        return autoPopulated;
    }

    public void setAutoPopulated(boolean isAutoPopulated) {
        this.autoPopulated = isAutoPopulated;
    }

	public Cohort getCohort() {
		return cohort;
	}

	public void setCohort(Cohort cohort) {
		this.cohort = cohort;
	}

	/**
	 * Return a 2-dimensional array of plate wells (rows X columns)
	 * representing the grid of wells on the plate
	 * @return 2-dimensional array of AssayPlateWells
	 */
	public AssayPlateWell[][] getWellArray() {
		return wellArray;
	}

	/**
	 * Returns the distinct set of alleles found on a row of the plate
	 * @param rowIndex row of interest where 0 is the first row
	 * @return distinct Set of alleles in the row
	 */
	public Set<Allele> getAllelesInRow(int rowIndex) {
		Set<Allele> alleles = new HashSet<Allele>();

		if(wellArray != null && rowIndex <= getRowCount()) {
			AssayPlateWell[] columns = wellArray[rowIndex];
			for(AssayPlateWell well : columns) {
				if(well != null) {
					Mouse mouse = well.getMouse();
					if(mouse != null) {
						Collection<MouseAllele> mouseAlleles = mouse.getAlleles();
						if(mouseAlleles != null) {
							for(MouseAllele mouseAllele : mouseAlleles) {
								if(mouseAllele.getAllele() != null) {
									alleles.add(mouseAllele.getAllele());
								}
							}
						}
					}
				}
			}
		}
		return alleles;
	}


	/**
	 * Returns the distinct set of alleles found on the plate
	 * @return distinct Set of alleles on the plate
	 */
	public Set<Allele> getAllelesInPlate() {
		Set<Allele> alleles = new HashSet<Allele>();

		if(wellArray != null) {
			for(AssayPlateWell[] columns : wellArray) {
				for(AssayPlateWell well : columns) {
					if(well != null) {
						Mouse mouse = well.getMouse();
						if(mouse != null) {
							Collection<MouseAllele> mouseAlleles = mouse.getAlleles();
							if(mouseAlleles != null) {
								for(MouseAllele mouseAllele : mouseAlleles) {
									if(mouseAllele.getAllele() != null) {
										alleles.add(mouseAllele.getAllele());
									}
								}
							}
						}
					}
				}
			}
		}
		return alleles;
	}

	public DictionaryEntry getSampleType() {
		return sampleType;
	}

	public void setSampleType(DictionaryEntry sampleType) {
		this.sampleType = sampleType;
	}

	public Long getSampleTypeDictId() {
		return sampleTypeDictId;
	}

	public void setSampleTypeDictId(Long sampleTypeDictId) {
		this.sampleTypeDictId = sampleTypeDictId;
	}


	/**
	 * Returns the collection of wells on a plate that contain a particular allele.
	 * The corresponding mouse can then be retrieved from the well if required.
	 *
	 * @param alleleId allele of interest, or null if all wells are required
	 * @return Collection of wells containing the allele of interest, or all wells if alleleId is null.
	 * An empty Collection is returned if the plate has no associated wellArray.
	 */
	public Collection<AssayPlateWell> getWellsByAlleleId(Long alleleId) {
		Collection<AssayPlateWell> wells = new ArrayList<AssayPlateWell>();

		if(wellArray != null) {
			for(AssayPlateWell[] columns : wellArray) {
				for(AssayPlateWell well :columns) {
					if(well != null && well.getMouse() != null && well.getMouse().getAlleles() != null) {
						for(MouseAllele mouseAllele : well.getMouse().getAlleles()) {
							if(alleleId == null || mouseAllele.getAlleleId().equals(alleleId)) {
								wells.add(well);
								break; // can break for multi-allele mice
							}
						}
					}
				}
			}
		}

		return wells;
	}

	/**
	 * Convenience method to return the letter (A..H) equivalent of a row index
	 * @param rowIndex row of interest where 0 is the first row
	 * @return
	 */
	public String getRowLetter(int rowIndex) {
		return String.valueOf((char)65+rowIndex);
	}

	/**
	 * Returns the length of the first dimension of the well array.
	 * This represents the number of rows on the plate.
	 * @return the number of rows on the plate.
	 */
	public int getRowCount() {
		return (wellArray == null) ? 0 : wellArray.length;
	}

	/**
	 * Returns the length of the second dimension of the well array.
	 * Specifically, returns the number of columns of the first row of the plate
	 * and assumes that all rows have the same number of columns.
	 * @return
	 */
	public int getColumnCount() {
		return (wellArray == null || wellArray.length == 0) ? 0 : wellArray[0].length;
	}

	public void generateBarcode() {
		if(getId() == null) {
			throw new IllegalStateException("Cannot generate barcode if id is null");
		}
		DecimalFormat df = new DecimalFormat("00000000");
		this.sheetBarcode = "P" + df.format(getId());
	}

	public String getBarcode() {
		return getSheetBarcode();
	}

	public String getBarcodeDescription() {
		return getSheetBarcode();
	}

	public BarcodeType getBarcodeType() {
		return BarcodeType.ASSAY_PLATE;
	}

	/**
	 * Insert an AssayPlateWell object into a position on the wellArray.
	 * @param well non null AssayPlateWell object
	 * @param row row to insert the well into
	 * @param column column to insert the well into
	 * @throws IllegalArgumentException if AssayPlateWell is null
	 * @throws IllegalStateException is wellArray is null
	 */
	public void populateWell(AssayPlateWell well, int row, int column) {
		if(well == null) {
			throw new IllegalArgumentException("AssayPlateWell cannot be null");
		}
		else if(wellArray == null) {
			throw new IllegalStateException("Plate wellArray cannot be null");
		}

		wellArray[row][column] = well;
	}

	public AssayPlateWell getWell(int row, int column) {
		AssayPlateWell well = null;

		if(wellArray != null) {
			well = wellArray[row][column];
		}

		return well;
	}

	/**
	 * Convenience method to query how many samples are on this plate.
	 * @return the number of samples on this plate.
	 */
	public int getSampleCount() {
		int totalSamples  = 0;
		if(wellArray != null) {
			for(AssayPlateWell[] columns : wellArray) {
				for(AssayPlateWell well :columns) {
					if(well != null) {
						if(Boolean.TRUE.equals(well.getSampleCollected())) {
							totalSamples++;
						}
					}
				}
			}
		}

		return totalSamples;
	}

	public int getColumnsOnPlate() {
		return columnsOnPlate;
	}

	public void setColumnsOnPlate(int columnsOnPlate) {
		this.columnsOnPlate = columnsOnPlate;
	}

	public int getRowsOnPlate() {
		return rowsOnPlate;
	}

	public void setRowsOnPlate(int rowsOnPlate) {
		this.rowsOnPlate = rowsOnPlate;
	}

	public DictionaryEntry getPlateCategory() {
		return plateCategory;
	}

	public void setPlateCategory(DictionaryEntry plateCategory) {
		this.plateCategory = plateCategory;
	}

	public Long getPlateCategoryId() {
		return plateCategoryId;
	}

	public void setPlateCategoryId(Long plateCategoryId) {
		this.plateCategoryId = plateCategoryId;
	}

	public Boolean getPostMortem() {
		return postMortem;
	}

	public void setPostMortem(Boolean postMortem) {
		this.postMortem = postMortem;
	}

}