package uk.ac.sanger.mig.genotyping.controllers;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.validation.BindException;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.genotyping.AssayPlate;
import uk.ac.sanger.mig.genotyping.AssayPlateSampleTypeDictionary;
import uk.ac.sanger.mig.genotyping.AssayPlateWell;
import uk.ac.sanger.mig.genotyping.AssayPlateWellTypeDictionary;
import uk.ac.sanger.mig.genotyping.forms.AddTissueForm;
import uk.ac.sanger.mig.genotyping.services.AssayPlateService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class AddTissueFormHelper {

	private ComponentMapService componentMapService;
	private AssayPlateService assayPlateService;
	private DictionaryService dictionaryService;
	private MouseProcedureService mouseProcedureService;

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setAssayPlateService(AssayPlateService assayPlateService) {
		this.assayPlateService = assayPlateService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	void initForm(AddTissueForm form, Long plateId) {
		form.setPlate(null);
		form.setPlateBarcode(null);
		form.setProcedureSubform(new ProcedureSubform());
		form.setSheetBarcode(null);

		if(plateId != null) {
			AssayPlate plate = assayPlateService.fetchById(plateId, null);
			if(plate != null) {
				form.setSheetBarcode(plate.getBarcode());
				scanSheetBarcode(form, new BindException(form, "form"));
			}
		}
	}

	Long processForm(AddTissueForm form, Person user) {
		if(!form.isValidated()) {
			throw new IllegalStateException("form is not valid");
		}
		// copy the plate barcode from the form to the plate.
		AssayPlate plate = form.getPlate();
		plate.setPlateBarcode(form.getPlateBarcode());

		Collection<Mouse> miceToStore = new ArrayList<Mouse>();
		Collection<MouseProcedure> proceduresToStore = new ArrayList<MouseProcedure>();

		ProcedureSubform genotypingProcDefaults = new ProcedureSubform();
		genotypingProcDefaults.setPersonalLicenceId(form.getProcedureSubform().getPersonalLicenceId());
		genotypingProcDefaults.setProcedureStartDate(form.getProcedureSubform().getProcedureStartDate());

		Long previousLitterId = null;
		MouseProcedure genotypingProcedure = null;

		// process the mice to be stored with the plate.
		for(AssayPlateWell well : plate.getWells()) {
			if(well.getMouse() != null) {
				// set the earclip where approprate
				if(!well.isLocked()) {
					Mouse mouse = well.getMouse();

					if(well.getEarmarkDictId() != null) {
						// only store valid earmark entries (not 'other' which is -1L)
						if(well.getEarmarkDictId() > 0) {
							mouse.setEarmarkDictId(well.getEarmarkDictId());

						}
						else {
							well.setEarmarkDictId(null); // don't store 'other'
						}

						// cull live mice who are having post-mortem tissue collected.
						if(Boolean.TRUE.equals(plate.getPostMortem()) && mouse.isAlive()) {
							mouse.setDeathDate(form.getProcedureSubform().getProcedureStartDate());
							mouse.setCauseOfDeathDictId(MouseService.CAUSE_OF_DEATH_CULLED);
						}

						// get the mouse current PPL, or if not present, the default from the form
						Long projectLicenceId = (mouse.getCurrentProjectLicenceId() == null) ? form.getProcedureSubform().getProjectLicenceId(): mouse.getCurrentProjectLicenceId();

						genotypingProcDefaults.setProjectLicenceId(projectLicenceId);

						/**
						 * As an optimization step, if this mouse is from the same litter as the previous one,
						 * there is no need to create a new earclip procedure, can reuse the details of the last one.
						 */
						if(previousLitterId == null || mouse.getLitterId() == null || mouse.getLitterId() != previousLitterId) {
							genotypingProcedure = mouseProcedureService.createGenotypingProcedure(mouse, genotypingProcDefaults, plate.getSampleTypeDictId(), null);
						}
						else {
							genotypingProcedure = genotypingProcedure.clone();
						}

						if(genotypingProcedure != null) {
							genotypingProcedure.setMouseId(mouse.getId());
							genotypingProcedure.setMouse(mouse);
							proceduresToStore.add(genotypingProcedure);
						}

						previousLitterId = mouse.getLitterId();
					}
					// if sample collected is True but there is no sample, then set it back
					// to null so it can be re-edited in the future.
					else if(Boolean.TRUE.equals(well.getSampleCollected())) {
						well.setSampleCollected(null);
					}
					miceToStore.add(mouse);
				}
			}
		}

		return assayPlateService.storeWithMice(plate, miceToStore, proceduresToStore, user);
	}

	/**
	 * Need to remove the "NM" earclip- as this means "no mark" and isn't an earclip.
	 * Need to add "Other" to indicate non-earclip tissue collection.
	 *
	 * @param form
	 * @return
	 */
	Collection<DictionaryEntry> getEarmarkDictionaryEntries(AddTissueForm form) {

		Collection<DictionaryEntry> earmarks = dictionaryService.fetch(DictionaryEnum.MOUSE_EARMARK_DICT, true);
		earmarks.add(new DictionaryEntry(AddTissueForm.EARCLIP_DROPDOWN_OTHER, "Other"));
		earmarks.remove(new DictionaryEntry(AddTissueForm.EARCLIP_DROPDOWN_NOMARK,"NM"));
		return earmarks;
	}


	/**
	 * Lock wells which already have an earclip. Default the 'sample collected' flag to true
	 * for samples that have not yet been collected.
	 *
	 * @param plate
	 */
	void setWellStatuses(AssayPlate plate) {
		AssayPlateWell[][] wells = plate.getWellArray();
		for(AssayPlateWell[] rows : wells){
			for(AssayPlateWell well : rows) {
				if(Boolean.TRUE.equals(well.getSampleCollected())) {
					well.setLocked(true);
				}
				if(well.getSampleCollected() == null && well.getMouseId() != null) {
					well.setSampleCollected(true);
				}
			}
		}
	}


	void scanPlateBarcode(AddTissueForm form, BindException errors) {
		String plateBarcode= form.getPlateBarcode();
		if(plateBarcode != null) {
			AssayPlate assayPlate = assayPlateService.fetchByBarcode(plateBarcode, null);

			if(assayPlate == null) {
				form.setPlateBarcode(plateBarcode);
			}else {
				errors.reject(null, "Barcode " + plateBarcode + " has already been assigned to a sheet or a plate. Please choose another.");
				form.setPlateBarcode(null);

			}
		}
	}


	void scanSheetBarcode(AddTissueForm form, BindException errors) {

		String sheetBarcode = form.getSheetBarcode();

		if(sheetBarcode != null) {
			ComponentMap plateComponents = componentMapService.getComponentMap("addTissueToPlate");

			AssayPlate assayPlate = assayPlateService.fetchByBarcode(sheetBarcode, plateComponents);

			if(assayPlate == null) {
				errors.reject(null, "No assay plate found corresponding to barcode '" + sheetBarcode + "'");
				form.setSheetBarcode(null);
			}
			else {
				form.setPlateBarcode(assayPlate.getPlateBarcode());
				form.setSheetBarcode(assayPlate.getSheetBarcode());

				setWellStatuses(assayPlate);
				form.setPlate(assayPlate);

				// default new plates to be earclip
				if(assayPlate.getSampleCount() == 0 && assayPlate.getSampleTypeDictId() == null) {
					assayPlate.setSampleTypeDictId(AssayPlateSampleTypeDictionary.EARCLIP);
				}

				checkIsProjectLicenceRequired(form);
			}
		}
	}

	void setEarclipProcedure(AddTissueForm form) {
		setEarclipIds(form, null);
	}

	void setOtherTissueProcedure(AddTissueForm form) {
		setEarclipIds(form,AddTissueForm.EARCLIP_DROPDOWN_OTHER);
	}

	private void setEarclipIds(AddTissueForm form, Long clipId) {
		for(int row = 0; row<form.getPlate().getRowsOnPlate();row++) {
			for(int col = 0; col<form.getPlate().getColumnsOnPlate(); col++) {
				AssayPlateWell well = form.getPlate().getWell(row, col);
				if(well.getMouse() != null && !well.isLocked()) {
					well.setEarmarkDictId(clipId);

					if(AddTissueForm.EARCLIP_DROPDOWN_OTHER.equals(clipId)) {
						well.setWellTypeDictId(AssayPlateWellTypeDictionary.OTHER_TISSUE);
					}
					else {
						well.setWellTypeDictId(AssayPlateWellTypeDictionary.EARCLIP);
					}

					form.getPlate().setWell(row, col, well);
				}
			}
		}
	}

	/**
	 * If one or more mice is missing a project licence, then it will have to be supplied in the form.
	 * This method sets the projectLicenceDefaultRequired flag to true.
	 *
	 * @param form
	 * @return true if any mice on the plate are missing project licence details.
	 */
	void checkIsProjectLicenceRequired(AddTissueForm form) {

		if(Boolean.TRUE.equals(form.getPlate().getPostMortem())) {
			form.setProjectLicenceDefaultRequired(false);
		}
		else {
			form.setProjectLicenceDefaultRequired(false);
			for(AssayPlateWell well : form.getPlate().getWells()) {
				if(well.getMouse() != null && well.getMouse().getCurrentProjectLicenceId() == null) {
					form.setProjectLicenceDefaultRequired(true);
					break;
				}
			}
		}
	}

	public Collection<DictionaryEntry> getSampleTypeDictionaryEntries(AddTissueForm form) {
		return dictionaryService.fetch(DictionaryEnum.GENOTYPING_SAMPLE_TYPE, true);
	}

	public Collection<DictionaryEntry> getGenderDictionaryEntries(AddTissueForm form) {
		return dictionaryService.fetch(DictionaryEnum.MOUSE_GENDER_DICT, true);
	}
}
