package uk.ac.sanger.mig.genotyping.controllers;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.web.multipart.MultipartFile;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.customquery.CustomQuery;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFilterEnum;
import uk.ac.sanger.mig.customquery.forms.CustomQueryForm;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFormFilter;
import uk.ac.sanger.mig.customquery.forms.CustomQueryFormSort;
import uk.ac.sanger.mig.customquery.services.CustomQueryService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.genotyping.Assay;
import uk.ac.sanger.mig.genotyping.AssayPlate;
import uk.ac.sanger.mig.genotyping.AssayPlateWell;
import uk.ac.sanger.mig.genotyping.AssayResult;
import uk.ac.sanger.mig.genotyping.AssaySearchBean;
import uk.ac.sanger.mig.genotyping.forms.EditAssayForm;
import uk.ac.sanger.mig.genotyping.forms.EditAssayPlateWellSubform;
import uk.ac.sanger.mig.genotyping.services.AssayPlateService;
import uk.ac.sanger.mig.genotyping.services.AssayService;
import uk.ac.sanger.mig.personnel.Person;

public class EditAssayFormHelper {

	private static final SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");
	private AssayService assayService;
	private AssayPlateService assayPlateService;
	private ComponentMapService componentMapService;
	private DictionaryService dictionaryService;
	private CustomQueryService customQueryService;
	private BinaryFileService binaryFileService;
	private AlleleService alleleService;

	public void setAssayPlateService(AssayPlateService assayPlateService) {
		this.assayPlateService = assayPlateService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setAssayService(AssayService assayService) {
		this.assayService = assayService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setCustomQueryService(CustomQueryService customQueryService) {
		this.customQueryService = customQueryService;
	}

	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}

	void initForm(EditAssayForm form, String barcode, String assayDate) {
		form.setBarcode(null);
		form.setAssayPlate(null);
		form.setAssay(null);
		form.setAssayDate(null);
		form.setSelectedAlleleId(null);
		form.setSelectedAssayTypeDictId(null);
		form.setSelectedAssay(null);
		form.setSelectedWells(null);

		if(StringUtils.isNotEmpty(barcode) && StringUtils.isNotEmpty(assayDate)) {
			form.setBarcode(barcode);
			scanBarcode(form, new BindException(form, "form"));

			try {
				form.setAssayDate(sdf.parse(assayDate));
				selectAssayDate(form);
			}
			catch(ParseException e) {
				form.setAssayDate(null);
			}
		}
	}

	/**
	 * Populate the AssayPlate object in the form based on the input barcode.
	 * The plate will be retrieved with an attached wellArray.
	 * Reset the other form fields as these will need to be re-selected
	 *
	 * A global error will be recorded if no plate is found for the corresponding barcode.
	 *
	 * @param form
	 * @param errors
	 */
	void scanBarcode(EditAssayForm form, BindException errors) {
		//reset all form fields and objects
		form.setAssayPlate(null);
		form.setAssay(null);
		form.setAssayDate(null);
		form.setSelectedAlleleId(null);
		form.setSelectedAssayTypeDictId(null);
		form.setSelectedAssay(null);
		form.setSelectedWells(null);

		if(form.getBarcode() != null) {
			ComponentMap plateComponents = componentMapService.getComponentMap("viewOneAssayPlate");

			AssayPlate assayPlate = assayPlateService.fetchByBarcode(form.getBarcode(), plateComponents);

			if(assayPlate == null) {
				errors.reject(null, "No assay plate found corresponding to barcode '" + form.getBarcode() + "'");
				form.setBarcode(null);
			}
			else {
				form.setAssayPlate(assayPlate);
				bindFilesToForm(assayPlate, form);
			}
		}
	}



	/**
	 * After selecting the assay date, any existing Assay and AssayResults are retreived
	 * from the database based on assayTypeDictId, plateId, alleleId and assayDate.
	 *
	 * AssayResults are attached individually to the corresponding AssayPlateWell in the form and
	 * any AssayPlateWells with no AssayResult have a new empty one attached.
	 *
	 * @param form
	 */
	void selectAssay(EditAssayForm form, BindException errors) {
		form.setSelectedWells(null);
		form.setAssay(null);

		if(form.getAssayPlate() == null || form.getAssayDate() == null) {
			errors.reject(null, "Form is in an inconsistent state. Cancel and try again.");
		}

		Long assayTypeDictId = null;
		Long alleleId = null;
		String selectedOption = form.getSelectedAssay();
		if(selectedOption !=null) {
			if(StringUtils.contains(selectedOption, "_")) {
				String[] longValues = selectedOption.split("_");
				assayTypeDictId = Long.parseLong(longValues[0]);
				alleleId = Long.parseLong(longValues[1]);
			}
			else {
				assayTypeDictId = Long.parseLong(selectedOption);
			}
		}
		form.setSelectedAssayTypeDictId(assayTypeDictId);
		form.setSelectedAlleleId(alleleId);

		if(form.getSelectedAssayTypeDictId() != null) {
			AssayPlate plate = form.getAssayPlate();

			ComponentMap assayComponents = componentMapService.getComponentMap("editAssay");

			AssaySearchBean bean = new AssaySearchBean();
			bean.setAssayPlateId(plate.getId());
			bean.setAssayTypeDictId(form.getSelectedAssayTypeDictId());
			bean.setAlleleId(form.getSelectedAlleleId());
			bean.setAssayDate(form.getAssayDate());

			Assay dbAssay = assayService.fetchByPlateAlleleTypeAndDate(bean, assayComponents);
			Collection<AssayPlateWell> wells = plate.getWellsByAlleleId(form.getSelectedAlleleId());


			Collection<EditAssayPlateWellSubform> selectedWells = new ArrayList<EditAssayPlateWellSubform>();
			for(AssayPlateWell well : wells) {
				if(Boolean.TRUE.equals(well.getSampleCollected())) {
					EditAssayPlateWellSubform wellForm = new EditAssayPlateWellSubform();
					wellForm.setAssayPlateWell(well);
					selectedWells.add(wellForm);
				}
			}

			if(dbAssay != null) {
				form.setAssay(dbAssay);
				bindAssayResultToWells(dbAssay.getAssayResults(), selectedWells);
				bindFilesToForm(dbAssay,form);
			}
			else {
				addNewAssay(form);
				bindAssayResultToWells(null, selectedWells);
				form.setAssayFiles(new LinkedList<BinaryFile>());
				form.setAssayFilesUpload(new LinkedList<MultipartFile>());
			}
			form.setSelectedWells(selectedWells);
		}
	}

	private void addNewAssay(EditAssayForm form) {
		form.setAssay(new Assay());

		if (form.getSelectedAlleleId() != null) {
			form.getAssay().setAllele( alleleService.fetchById(form.getSelectedAlleleId(), ComponentMap.of(Allele.Component.GENE))  );
		}
		form.getAssay().setAssayType(
			dictionaryService.fetchEntry(DictionaryEnum.GENOTYPING_ASSAY_TYPE, form.getSelectedAssayTypeDictId())
		);
		form.getAssay().setAssayDate(form.getAssayDate());
	}


	/**
	 * Find files associated with an assay and bind them to the form
	 *
	 * @param assay
	 * @param form
	 */
	private void bindFilesToForm(Assay assay, EditAssayForm form) {
		List<BinaryFile> files = new LinkedList<BinaryFile>(
				binaryFileService.fetchByTableRef(
						TableMetadataEnum.GTP_ASSAY,
						assay.getId(),
						false, null)
		);

		List<Boolean> toDelete = new ArrayList<Boolean>(files.size());
		while (toDelete.size() < files.size()) {
			toDelete.add(Boolean.FALSE);
		}
		form.setAssayFilesToDelete(toDelete);
		form.setAssayFiles(files);
		form.setAssayFilesUpload(new LinkedList<MultipartFile>());
	}

	/**
	 * Find files associated with a plate and bind them to the form
	 *
	 * @param assay
	 * @param form
	 */
	private void bindFilesToForm(AssayPlate plate, EditAssayForm form) {
		List<BinaryFile> files = new LinkedList<BinaryFile>(
				binaryFileService.fetchByTableRef(
						TableMetadataEnum.GTP_ASSAY_PLATE,
						plate.getId(),
						false, null)
		);

		List<Boolean> toDelete = new ArrayList<Boolean>(files.size());
		while (toDelete.size() < files.size()) {
			toDelete.add(Boolean.FALSE);
		}
		form.setPlateFilesToDelete(toDelete);
		form.setPlateFiles(files);
		form.setPlateFilesUpload(new LinkedList<MultipartFile>());
	}

	/**
	 * If a new assay date is selected, the assay and selected wells must be reset
	 *
	 * @param form
	 */
	void selectAssayDate(EditAssayForm form) {
		form.setSelectedWells(null);
		form.setAssay(null);
		form.setSelectedAssay(null);
		form.setSelectedAssayTypeDictId(null);
		form.setSelectedAlleleId(null);
	}

	/**
	 * Associates an individual AssayResult to a well in the list of wells on the form.
	 * AssayResult is linked to the well via the FK between well and result. Any wells
	 * with no existing result get a new AssayResult object.
	 *
	 * @param results
	 * @param wells
	 */
	private void bindAssayResultToWells(Collection<AssayResult> results, Collection<EditAssayPlateWellSubform> wells) {
		for(EditAssayPlateWellSubform well : wells) {
			AssayResult usedResult = null;
			boolean resultFound = false;
			if(results != null) {
				for(AssayResult result : results) {
					if(result.getAssayPlateWellId().equals(well.getAssayPlateWell().getId())) {
						usedResult = result;
						resultFound = true;
						break;
					}
				}
			}
			if(resultFound) {
				well.setAssayResult(usedResult);
				results.remove(usedResult);
				usedResult = null;
				resultFound = false;
			}
			else {
				well.setAssayResult(new AssayResult());
			}
		}
	}

	/**
	 * Return a Map for use in a select-list of assayType and allele combinations.
	 * E.g. LacZ, Tpm1(Mutant), Tpm1(Wildtype).
	 * The select-list value must contain both assayTypeDictId and alleleId.
	 * In the case of plate-scope assays, the alleleId is not required.
	 * The select option value will be in the format typeId_alleleId.
	 * E.g. 1_2
	 * For plate-scope assays the select option value will be just the typeId
	 * E.g. 1
	 * @param form EditAssayForm
	 * @return map of select options.
	 * @throws IllegalStateException if form or form.assayPlate is null
	 */
	Map<String, String> makeAssaySelectListOptions (EditAssayForm form) {
		if(form == null || form.getAssayPlate() == null) {
			throw new IllegalStateException("Invalid state: AssayPlate cannot be null on form");
		}

		Set<Allele> alleles = form.getAssayPlate().getAllelesInPlate();

		Map<String, String> assays = new LinkedHashMap<String, String>();

		Collection<DictionaryEntry> plateScopeTypes = assayService.fetchPlateAssayTypes();
		Collection<DictionaryEntry> alleleScopeTypes = assayService.fetchAlleleAssayTypes();

		for(DictionaryEntry plateScopeType : plateScopeTypes) {
			assays.put(String.valueOf(plateScopeType.getId()), plateScopeType.getName());
		}

		for(Allele allele : alleles) {
			for(DictionaryEntry alleleScopeType : alleleScopeTypes) {
				assays.put(alleleScopeType.getId() + "_" + allele.getId(), allele.getDisplayText() + "(" + alleleScopeType.getName() + ")");
			}
		}

		return assays;
	}

	Collection<DictionaryEntry> makeAssayResultSelectOptions() {
		return dictionaryService.fetch(DictionaryEnum.GENOTYPING_ASSAY_RESULT_TYPE);
	}

	/**
	 * This method gets the Assay ready for storing by binding values from the form
	 * to the assay and by associating the assay results from the form into the collection
	 * of assayResults in the assay object.
	 *
	 * @param form
	 * @param user
	 */
	Long processForm(EditAssayForm form, Person user) {
		if(!form.isValidated()) {
			throw new IllegalStateException("form is not valid");
		}

		Assay assay = form.getAssay();
		if(assay.isNewRecord()) {
			assay.setAssayDate(form.getAssayDate());
			assay.setAssayPlateId(form.getAssayPlate().getId());
			assay.setAssayTypeDictId(form.getSelectedAssayTypeDictId());
			assay.setAlleleId(form.getSelectedAlleleId());
		}

		Collection<AssayResult> results = new ArrayList<AssayResult>();
		if(form.getSelectedWells() != null) {
			for(EditAssayPlateWellSubform well : form.getSelectedWells()) {
				AssayResult result = well.getAssayResult();
				result.setAssayPlateWellId(well.getAssayPlateWell().getId());
				result.setAssayPlateWell(well.getAssayPlateWell());
				results.add(result);
			}
		}
		assay.setAssayResults(results);

		form.handleUploadedFiles();
		Collection<BinaryFile> assayFilesToAdd = getFilesToAdd(form.getAssayFiles(),form.getAssayFilesToDelete());
		Collection<BinaryFile> plateFilesToAdd = getFilesToAdd(form.getPlateFiles(),form.getPlateFilesToDelete());
		Collection<BinaryFile> filesToDelete = getFilesToDelete(form.getAssayFiles(), form.getAssayFilesToDelete());
		filesToDelete.addAll(getFilesToDelete(form.getPlateFiles(),form.getPlateFilesToDelete()));

		return assayService.storeCompleteWithFiles(assay, assayFilesToAdd, plateFilesToAdd, filesToDelete, user);
	}

	private Collection<BinaryFile> getFilesToDelete(List<BinaryFile> files, List<Boolean> deleteList){
		Collection<BinaryFile> toDelete = new LinkedList<BinaryFile>();

		ListIterator<BinaryFile> i = files.listIterator();
		while (i.hasNext()) {
			BinaryFile f = i.next();
			int index = i.previousIndex();
			if (f.getId() != null && deleteList.get(index).booleanValue()) {
				toDelete.add(f);
			}
		}

		return toDelete;
	}


	private Collection<BinaryFile> getFilesToAdd(List<BinaryFile> files, List<Boolean> deleteList) {
		Collection<BinaryFile> filesToAdd = new LinkedList<BinaryFile>();

		ListIterator<BinaryFile> i = files.listIterator();
		while (i.hasNext()) {
			BinaryFile f = i.next();
			int index = i.previousIndex();

			// not previously stored and not marked for deletion
			if (f.getId() == null && !deleteList.get(index).booleanValue()) {
				filesToAdd.add(f);
			}
		}

		return filesToAdd;
	}

	Map<String, Object> fetchAssayResultSummary(Person user, Long plateId, Date assayDate) {
		CustomQuery customQuery = customQueryService.fetchByUniqueName("assayPlateResultSummary", user.getId(), null);

		Long queryId = customQuery.getId();

		CustomQueryForm queryForm = new CustomQueryForm(queryId, user.getId(), "assayPlateResultSummary");
		CustomQueryFormFilter plateIdFilter = customQueryService.constructFormFilter(queryId, "plate_id", false, CustomQueryFilterEnum.EQUALS, plateId);
		CustomQueryFormFilter assayDateFilter = customQueryService.constructFormFilter(queryId, "assay_date", false, CustomQueryFilterEnum.EQUALS, sdf.format(assayDate));

		queryForm.addFilter(plateIdFilter);
		queryForm.addFilter(assayDateFilter);

		CustomQueryFormSort alleleSort = new CustomQueryFormSort("allele", true, false);
		CustomQueryFormSort assaySort = new CustomQueryFormSort("assay", true, false);

		queryForm.addSort(alleleSort);
		queryForm.addSort(assaySort);

		return customQueryService.fetchCustomQueryModel(queryForm);
	}

	public void setAlleleService(AlleleService alleleService) {
		this.alleleService = alleleService;
	}
}
