/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.genotyping.controllers;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.ParameterizableViewController;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.genotyping.services.AssayResultService;
import uk.ac.sanger.mig.genotyping.views.AssayResultsView;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class ViewAssayHistoryController extends ParameterizableViewController {

	private MouseService mouseService;
	private AssayResultService assayResultService;

	public void setAssayResultService(AssayResultService assayResultService) {
		this.assayResultService = assayResultService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	@Override
	protected ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response) throws Exception{
		Long mouseId = ServletRequestUtils.getRequiredLongParameter(request, "id");

		ComponentMap mouseComponents = new ComponentMap(
				Mouse.Component.ALLELES,
				ComponentMap.of(MouseAllele.Component.GENOTYPE_HISTORIES)
		);
		mouseComponents.get(Mouse.Component.ALLELES).put(MouseAllele.Component.ALLELE, ComponentMap.of(Allele.Component.GENE));

		Mouse mouse = mouseService.fetchById(mouseId, mouseComponents);
		Collection<AssayResultsView> results = assayResultService.fetchResultsViewByMouseId(mouseId);

		Map<String,Object> model = new HashMap<String, Object>();
		model.put("mouse",mouse);
		model.put("alleleResults",assembleResults(results, mouse));

		return new ModelAndView(getViewName(), model);
	}

	private Map<MouseAllele,List<AssayResultsView>> assembleResults(Collection<AssayResultsView> results, Mouse mouse){
		Map<MouseAllele,List<AssayResultsView>> alleleAssayResults = new LinkedHashMap<MouseAllele, List<AssayResultsView>>();
		Map<Long,MouseAllele> alleles = new LinkedHashMap<Long,MouseAllele>();
		List<AssayResultsView> globalAssayResults = new LinkedList<AssayResultsView>();

		// make an id lookup for the alleles and init the grouped results
		for(MouseAllele ma : mouse.getAlleles()) {
			alleles.put(ma.getAlleleId(),ma);
			alleleAssayResults.put(ma, new LinkedList<AssayResultsView>());
		}

		// assign each result to an allele or the global group
		for (AssayResultsView assayResult : results) {
			if (assayResult.getAssayScope().equalsIgnoreCase("plate")) {
				globalAssayResults.add(assayResult);
			}
			else {
				MouseAllele allele = alleles.get(assayResult.getAlleleId());
				if (allele == null) {
					throw new IllegalStateException("Attempting to show genotyping assay result for an unexpected allele. Mouse "+assayResult.getMouseId()+" Allele "+assayResult.getAlleleId()+ " Well ID "+assayResult.getAssayPlateWellId());
				}

				alleleAssayResults.get(allele).add(assayResult);
			}
		}

		// add the global results to each allele and order the
		for (List<AssayResultsView> resultsPerAllele : alleleAssayResults.values()){
			resultsPerAllele.addAll(globalAssayResults);
			Collections.sort(resultsPerAllele);
		}

		return alleleAssayResults;
	}
}
