/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.genotyping.controllers;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.ParameterizableViewController;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.genotyping.Assay;
import uk.ac.sanger.mig.genotyping.AssayPlate;
import uk.ac.sanger.mig.genotyping.services.AssayService;

public class ViewAttachmentsController extends ParameterizableViewController {

	private AssayService assayService;
	private BinaryFileService binaryFileService;

	private static final String ALL_FILES_KEY = "allFiles";
	private static final String IMAGE_FILES_KEY = "imageFiles";

	public void setAssayService(AssayService assayService) {
		this.assayService = assayService;
	}

	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}

	@Override
	protected ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response) throws Exception{
		Long assayId = ServletRequestUtils.getRequiredLongParameter(request, "id");

		Map<String, Object> model = new HashMap<String, Object>();

		model.put("allFilesKey", ALL_FILES_KEY);
		model.put("imageFilesKey", IMAGE_FILES_KEY);

		Assay selectedAssay = assayService.fetchById(
				assayId,
				new ComponentMap(
						Assay.Component.ASSAY_PLATE,
						new ComponentMap(
								AssayPlate.Component.ASSAYS,
								new ComponentMap(
										Assay.Component.ALLELE,
										ComponentMap.of(Allele.Component.GENE)
								)
						)
				)
		);

		if (selectedAssay != null) {
			model.put("selectedAssay",selectedAssay);
			model.put("plate",selectedAssay.getAssayPlate());
			model.put("plateFiles", binaryFileService.fetchByTableRef(TableMetadataEnum.GTP_ASSAY_PLATE, selectedAssay.getAssayPlateId(), false, null));

			Map<DictionaryEntry,Collection<Assay>> globalAssays = new LinkedHashMap<DictionaryEntry, Collection<Assay>>();
			Map<Allele,Map<DictionaryEntry,Collection<Assay>>> alleleAssays = new LinkedHashMap<Allele,Map<DictionaryEntry,Collection<Assay>>>();
			Map<Assay,Collection<BinaryFile>> assayFiles = new HashMap<Assay, Collection<BinaryFile>>();

			model.put("globalAssays",globalAssays);
			model.put("alleleAssays",alleleAssays);
			model.put("assayFiles",assayFiles);


			for (Assay assay : selectedAssay.getAssayPlate().getAssays()) {

				if (assay.getAllele() != null) {
					addAlleleAssay(alleleAssays,assay,assayFiles);
				}
				else {
					addTypedAssay(globalAssays,assay,assayFiles);
				}

			}

		}

		return new ModelAndView(getViewName(), model);
	}

	private void addTypedAssay(Map<DictionaryEntry,Collection<Assay>> typedAssays, Assay assay, Map<Assay,Collection<BinaryFile>> assayFiles) {
		DictionaryEntry type = assay.getAssayType();
		if (!typedAssays.containsKey(type)) {
			typedAssays.put(type, new LinkedList<Assay>());
		}

		typedAssays.get(type).add(assay);

		assayFiles.put(
				assay,
				binaryFileService.fetchByTableRef(
						TableMetadataEnum.GTP_ASSAY,
						assay.getId(),false, null)
		);
	}

	private void addAlleleAssay(Map<Allele,Map<DictionaryEntry,Collection<Assay>>> alleleAssays, Assay assay, Map<Assay,Collection<BinaryFile>> assayFiles) {
		Allele allele = assay.getAllele();

		if (!alleleAssays.containsKey(allele)){
			alleleAssays.put(allele, new LinkedHashMap<DictionaryEntry,Collection<Assay>>());
		}

		Map<DictionaryEntry,Collection<Assay>> typeMap = alleleAssays.get(allele);
		addTypedAssay(typeMap, assay, assayFiles);
	}

}
