/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.genotyping.dao;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.DictionaryDao;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.genotyping.Assay;
import uk.ac.sanger.mig.genotyping.AssaySearchBean;

public class AssayDaoSpringImpl extends AbstractDaoSpringImpl<Assay> implements AssayDao {

	private DictionaryDao dictionaryDao;

	public void setDictionaryDao(DictionaryDao dictionaryDao) {
		this.dictionaryDao = dictionaryDao;
	}

	public Collection<DictionaryEntry> fetchPlateAssayTypes() {
		String sql = " select * from gtp_assay_type_dict where scope = 'Plate' and active = 1 order by order_by ";
		return dictionaryDao.fetchByCustomQuery(sql, new ArrayList<Object>());
	}

	public Collection<DictionaryEntry> fetchAlleleAssayTypes() {
		String sql = " select * from gtp_assay_type_dict where scope = 'Allele' and active = 1 order by order_by ";
		return dictionaryDao.fetchByCustomQuery(sql, new ArrayList<Object>());
	}

	public Collection<Assay> fetchByPlateId(Long plateId) {
		String sql = " select * from gtp_assay where assay_plate_id = ? order by assay_date, assay_type_dict_id, allele_id ";
		return fetchCollection(sql, plateId);
	}

	public Assay fetchByPlateAlleleTypeAndDate(AssaySearchBean bean) {
		if(bean == null) {
			throw new IllegalArgumentException("searchBean cannot be null");
		}

		String sql;
		if(bean.getAlleleId() == null) {
			sql = " select * from gtp_assay where assay_plate_id = ? and allele_id is null and assay_type_dict_id = ? and assay_date = ? ";
			return fetchOne(sql, bean.getAssayPlateId(), bean.getAssayTypeDictId(), bean.getAssayDate());
		}
		else {
			sql = " select * from gtp_assay where assay_plate_id = ? and allele_id = ? and assay_type_dict_id = ? and assay_date = ? ";
			return fetchOne(sql, bean.getAssayPlateId(), bean.getAlleleId(), bean.getAssayTypeDictId(), bean.getAssayDate());
		}
	}

	public Collection<Assay> fetchLatestByPlateAndAlleleId(Long plateId, Long alleleId) {
		String sql =
			" with latest_assay as ( " +
			" select " +
			" p.id plate_id, " +
			" a.allele_id, " +
			" a.assay_type_dict_id, " +
			" max(a.assay_date) assay_date " +
			" from  " +
			" gtp_assay a, " +
			" gtp_assay_plate p " +
			" where " +
			" a.assay_plate_id = p.id " +
			" group by " +
			" p.id, " +
			" a.allele_id, " +
			" a.assay_type_dict_id " +
			" ) " +
			" select a.* " +
			" from  " +
			" gtp_assay a, " +
			" gtp_assay_plate p, " +
			" latest_assay l " +
			" where " +
			" p.id = a.assay_plate_id and " +
			" nvl(a.allele_id,0) = nvl(l.allele_id,0) and " +
			" a.assay_type_dict_id = l.assay_type_dict_id and " +
			" a.assay_date = l.assay_date and " +
			" p.id = l.plate_id and " +
			" p.id = ? and (a.allele_id = ? or a.allele_id is null)" +
			" order by a.assay_type_dict_id ";
		return fetchCollection(sql, plateId, alleleId);
	}


	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Assay assay) {
		sql.addLongParam(assay.getAssayTypeDictId(), "ASSAY_TYPE_DICT_ID");
		sql.addDateParam(assay.getAssayDate(), "ASSAY_DATE");
		sql.addLongParam(assay.getAlleleId(), "ALLELE_ID");
		sql.addLongParam(assay.getAssayPlateId(), "ASSAY_PLATE_ID");
		sql.addLongParam(assay.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(assay.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(assay.getEditedBy(), "EDITED_BY");
		sql.addDateParam(assay.getEditDate(), "EDIT_DATE");
		sql.addLongParam(assay.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(assay.getId(), "ID");
		sql.addStringParam(assay.getComments(), "COMMENTS");
	}

}