/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.genotyping.dao;

import java.util.Collection;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.genotyping.AssayResult;
import uk.ac.sanger.mig.genotyping.views.AssayResultsView;

public class AssayResultDaoSpringImpl extends AbstractDaoSpringImpl<AssayResult> implements AssayResultDao {

	private ParameterizedRowMapper<AssayResultsView> viewRowMapper;

	public void setViewRowMapper(ParameterizedRowMapper<AssayResultsView> viewRowMapper) {
		this.viewRowMapper = viewRowMapper;
	}

	public Collection<AssayResult> fetchByAssayId(Long assayId) {
		String sql = " select * from gtp_assay_result where assay_id = ? order by id ";
		return fetchCollection(sql, assayId);
	}

	public AssayResult fetchByAssayPlateWellId(Long wellId) {
		String sql = " select * from gtp_assay_result where assay_plate_well_id = ? ";
		return fetchOne(sql, wellId);
	}

	public Collection<AssayResult> fetchLatestByMouseAlleleId(Long mouseAlleleId) {
		String sql =
			" with latest_assay as ( " +
			" select " +
			" w.mouse_id, " +
			" a.allele_id, " +
			" a.assay_type_dict_id, " +
			" max(a.assay_date) assay_date " +
			" from " +
			" gtp_assay a, " +
			" gtp_assay_plate_well w, " +
			" gtp_assay_result r " +
			" where " +
			" r.assay_plate_well_id = w.id and " +
			" r.assay_id = a.id " +
			" group by " +
			" w.mouse_id, " +
			" a.allele_id, " +
			" a.assay_type_dict_id " +
			" ) " +
			" select distinct r.* ,l.mouse_id " +
			" from " +
			" gtp_assay a, " +
			" gtp_assay_result r, " +
			" gtp_assay_plate_well w, " +
			" mts_mouse_allele m, " +
			" latest_assay l " +
			" where " +
			" a.id = r.assay_id and " +
			" r.assay_plate_well_id = w.id and " +
			" w.mouse_id = m.mouse_id and " +
			" m.mouse_id = l.mouse_id and " +
			" nvl(a.allele_id,0) = nvl(l.allele_id,0) and " +
			" a.assay_type_dict_id = l.assay_type_dict_id and " +
			" a.assay_date = l.assay_date and " +
			" m.id = ? ";

		return fetchCollection(sql, mouseAlleleId);
	}


	/**
	 * Returns a collection of <code>Assay</code> objects representing the
	 */
	public Collection<AssayResult> fetchByMouseAllele(Long mouseId, Long alleleId) {
		if(mouseId == null || alleleId == null)
			throw new IllegalArgumentException("mouse or allele id can not be null");
		String sql = "select " +
			"a.* " +
			"from " +
			"gtp_assay_plate_well apw, " +
			"gtp_assay_result ar, " +
			"gtp_assay a " +
			"where apw.mouse_id = ? " +
			"and APW.ID = AR.ASSAY_PLATE_WELL_ID " +
			"and AR.ASSAY_ID = A.ID " +
			"and A.ALLELE_ID = ?";
		return fetchCollection(sql, mouseId, alleleId);
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, AssayResult assayResult) {
		sql.addLongParam(assayResult.getAssayResultDictId(), "ASSAY_RESULT_DICT_ID");
		sql.addLongParam(assayResult.getAssayId(), "ASSAY_ID");
		sql.addLongParam(assayResult.getAssayPlateWellId(), "ASSAY_PLATE_WELL_ID");
		sql.addLongParam(assayResult.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(assayResult.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(assayResult.getEditedBy(), "EDITED_BY");
		sql.addDateParam(assayResult.getEditDate(), "EDIT_DATE");
		sql.addLongParam(assayResult.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(assayResult.getId(), "ID");
		sql.addStringParam(assayResult.getComments(), "COMMENTS");
	}

	public Collection<AssayResultsView> fetchResultsViewByMouseId(Long mouseId) {
		String sql = "select * from gtp_assay_results_vw where mouse_id = ? order by assay_date desc";
		return getSimpleJdbcTemplate().query(sql, viewRowMapper, mouseId);
	}

}