/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.genotyping.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import javax.swing.tree.TreeNode;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.genotyping.Assay;
import uk.ac.sanger.mig.genotyping.AssayPlate;
import uk.ac.sanger.mig.genotyping.AssayPlateWell;
import uk.ac.sanger.mig.genotyping.AssayPlateWellTypeDictionary;
import uk.ac.sanger.mig.genotyping.dao.AssayPlateDao;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class AssayPlateServiceImpl extends AbstractServiceImpl2<AssayPlate,AssayPlateDao> implements AssayPlateService {

	private CohortService cohortService;
	private AssayPlateWellService assayPlateWellService;
	private AssayService assayService;
	private MouseService mouseService;
	private MouseProcedureService mouseProcedureService;

	@Deprecated
	private static final int PLATE_SIZE = 96;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setAssayPlateWellService(AssayPlateWellService assayPlateWellService) {
		this.assayPlateWellService = assayPlateWellService;
	}

	public void setCohortService(CohortService cohortService) {
		this.cohortService = cohortService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	public Collection<AssayPlate> fetchByCohortId(Long cohortId, ComponentMap requiredComponents) {
		Collection<AssayPlate> plates = getPrimaryDao().fetchByCohortId(cohortId);
		attachComponents(plates, requiredComponents);
		return plates;
	}

	@Override
	protected void attachComponents(AssayPlate assayPlate, ComponentMap requiredComponents) {
		if(assayPlate == null) {
			throw new RuntimeException("Cannot attach components to null assayPlate");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(AssayPlate.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(AssayPlate.Component.COHORT)) {
				assayPlate.setCohort(fetchCohort(assayPlate.getCohortId(), requiredComponents.get(AssayPlate.Component.COHORT)));
			}

			if(fetchAll || requiredComponents.containsKey(AssayPlate.Component.WELL_ARRAY)) {
				Collection<AssayPlateWell> wells = fetchWells(assayPlate.getId(), requiredComponents.get(AssayPlate.Component.WELL_ARRAY));
				for(AssayPlateWell well : wells) {
					int row = well.getPlateRow();
					int col = well.getPlateColumn();
					assayPlate.setWell(row, col, well);
				}
			}

			if (fetchAll || requiredComponents.containsKey(AssayPlate.Component.ASSAYS)) {
				Collection<Assay> assays = assayService.fetchByPlateId(assayPlate.getId(), requiredComponents.get(AssayPlate.Component.ASSAYS));
				assayPlate.setAssays(assays);
			}
		}
	}

	private Cohort fetchCohort(Long cohortId, ComponentMap requiredComponents) {
		if(cohortId == null) {
			return null;
		}
		else {
			return cohortService.fetchById(cohortId, requiredComponents);
		}
	}

	private Collection<AssayPlateWell> fetchWells(Long plateId, ComponentMap requiredComponents) {
		return assayPlateWellService.fetchByPlateId(plateId, requiredComponents);
	}

	public AssayPlate fetchByBarcode(String barcode, ComponentMap requiredComponents) {
		AssayPlate assayPlate = getPrimaryDao().fetchByBarcode(barcode);
		if(assayPlate != null) {
			attachComponents(assayPlate, requiredComponents);
		}
		return assayPlate;
	}


	/**
	 * Method to store a complete plate including iterating through all the
	 * filled wells and storing those in turn.
	 *
	 * @param assayPlate the assayPlate object to be stored
	 * @param person
	 * @throws RuntimeException
	 */
	public Long storeComplete(AssayPlate assayPlate, Person user) {
		if(assayPlate == null) {
			throw new RuntimeException("Cannot call storeComplete on a null assayPlate");
		}
		// Store plate.
		Long plateId = store(assayPlate, user);
		// Iterate over all the valid wells and store.
		AssayPlateWell[][] wells = assayPlate.getWellArray();

		for(int row=0; row<assayPlate.getRowCount(); row++) {
			for(int col=0; col<assayPlate.getColumnCount(); col++) {
				if(wells[row][col] != null) {
					AssayPlateWell well = wells[row][col];

					well.setAssayPlateId(plateId);
					well.setPlateRow(row);
					well.setPlateColumn(col);
					assayPlateWellService.store(well, user);
				}
			}
		}
		return plateId;
	}

	@Transactional
	public Long storeWithMice(AssayPlate assayPlate, Collection<Mouse> mice, Collection<MouseProcedure> procedures, Person user) {

		for(Mouse mouse : mice) {
			mouseService.store(mouse, user);
		}

		for(MouseProcedure mouseProcedure : procedures) {
			mouseProcedureService.storeIfNotExists(mouseProcedure, user);
		}

		return storeComplete(assayPlate, user);


	}

	@Transactional
	public Set<Long> storeCollection(Collection<AssayPlate> assayPlates, Person user) {
		Set<Long> plateIds = new LinkedHashSet<Long>();

		if(assayPlates != null) {
			for(AssayPlate assayPlate : assayPlates) {

				if(assayPlate.isFlaggedForDelete()) {
					deleteComplete(assayPlate, user);
				}
				else {
					Long plateId = storeComplete(assayPlate, user);
					plateIds.add(plateId);
				}
			}
		}

		return plateIds;
	}

	@Transactional
	public void deleteComplete(AssayPlate assayPlate, Person user) {

		for(AssayPlateWell[] rows : assayPlate.getWellArray()) {
			for(AssayPlateWell well : rows) {
				assayPlateWellService.delete(well, user);
			}
		}

		getPrimaryDao().delete(assayPlate, user);
	}

	/**
	 * Generate a collection of AssayPlate objects with the respective AssayPlateWells
	 * populated.
	 * @param cohort cohort containing mice for arraying
	 * @throws IllegalArgumentException if Cohort is null
	 */
	public Collection<AssayPlate> fetchArrayedPlates(Long cohortId) {
		if(cohortId == null) {
			throw new IllegalArgumentException("CohortId cannot be null");
		}

		Cohort cohort = cohortService.fetchById(cohortId, ComponentMap.of(Cohort.Component.MICE));

		// Get the mice.
		Collection<Mouse> mice = cohort.getMice();

		ArrayList<MouseGroup> groupList = new ArrayList<MouseGroup>();
		Collection<AssayPlate> arrayedPlates = new ArrayList<AssayPlate>();

		// group into alleles.
		for(Mouse m : mice) {
			Long colonyInfoId = m.getColonyInfoId();  // use reflect on string method name...todo.
			insertKeyNodeByName(colonyInfoId,m,groupList);
		}

		// add controls to the groups.
		addControlWellsAndSort(groupList);

		// register the number of distinct groups.
		int freeNodeCount = groupList.size();

		// sort by number of samples per group.
		Collections.sort(groupList, new MouseGroupSortComparator());

		// Count the number of mice int he first group.
		int firstGroupMouseCount = groupList.get(0).getMice().size();
		// If we can make a whole plate out of all/some of it....
		while(firstGroupMouseCount >= PLATE_SIZE - 2){ // - 2 as a whole plate needs to have controls on the end.
			// get the group
			MouseGroup group = groupList.remove(0);
			Collection<Mouse> miceInGroup = group.getMice();
			// create a plate.
			AssayPlate wholePlate = createWholeAssayPlateFromGroup(miceInGroup,cohortId);
			arrayedPlates.add(wholePlate);
			// remove mice used in the plate from the group
			for(AssayPlateWell well  : wholePlate.getWells()) {
				miceInGroup.remove(well.getMouse());
			}
			// Set remaining mice back in the group.
			group.setMice(miceInGroup);
			groupList.add(group);
			// ReSort the groups again...
			Collections.sort(groupList, new MouseGroupSortComparator());
			// reset the largets group mouse count for the while loop.
			firstGroupMouseCount = groupList.get(0).getMice().size();
		}


		// pick the plates.
		createPlates(cohortId,groupList,arrayedPlates,freeNodeCount);

		return arrayedPlates;
	}


	private void addControlWellsAndSort(ArrayList<MouseGroup> groupList) {
		for(MouseGroup group : groupList) {
			// At this point we make 2 fake Mouse objects which will become the control wells.
			addControlMice(group);
			Collections.sort((List<Mouse>)group.getMice(), new CageSortComparator());

		}
	}

	/**
	 * Generate the plate.
	 * @param cohortId
	 */
	private AssayPlate createWholeAssayPlateFromGroup(Collection<Mouse> miceForPlate, Long cohortId) {
		int row = 1;
		int col = 1;

		AssayPlate plate = new AssayPlate(8,12);
		plate.setAutoPopulated(true);

		Mouse[] mice = (Mouse[])miceForPlate.toArray(new Mouse[0]);
		for(int i = 1; i<=PLATE_SIZE - 2; i++,col++) {
			AssayPlateWell well = new AssayPlateWell();
			Mouse mouse = mice[i-1];
			if(mouse !=null) {
				well.setMouse(mouse);
				well.setMouseId(mouse.getId());

				if(mouse.getName().equals("Control")) {
					well.setWellTypeDictId(2L); // Control ID
				}else{
					well.setWellTypeDictId(3L); // Earclip
				}
			}

			well.setPlateRow(row-1);
			well.setPlateColumn(col-1);
			plate.populateWell(well, row-1, col-1);

			if(col%12 == 0) {row++;col=0;}

		}
		// As we're making a whole plate, we need to add 2 controls to the end
		Mouse control_1 = new Mouse();
		control_1.setName("Control");
		control_1.setCageId(-1L);
		AssayPlateWell well = new AssayPlateWell();
		well.setMouse(control_1);
		well.setWellTypeDictId(2L);
		well.setPlateRow(row-1);
		well.setPlateColumn(col-1);
		plate.populateWell(well, row-1, col-1);
		col++;

		Mouse control_2 = new Mouse();
		control_2.setName("Control");
		control_2.setCageId(-1L);
		AssayPlateWell well2 = new AssayPlateWell();
		well2.setMouse(control_1);
		well2.setWellTypeDictId(2L);
		well2.setPlateRow(row-1);
		well2.setPlateColumn(col-1);
		plate.populateWell(well2, row-1, col-1);


		plate.setCohortId(cohortId);

		return plate;
	}

	/**
	 * method to iterate through the grouop lists creating plaates.
	 * groupList is sorted by number of samples.
	 *
	 * @param cohortId
	 * @param groupList
	 * @param arrayedPlates
	 * @param freeGroupCount
	 */
	private void createPlates(Long cohortId, ArrayList<MouseGroup> groupList, Collection<AssayPlate> arrayedPlates, int freeGroupCount) {

		// Loop while we have unassigned groups left.
		while(freeGroupCount > 0) {
			Node currentNode = null;
			Node parentNode = null;
			int listSize = groupList.size();
			int totalSamples = 0;
			// loop through the groups.
			for(int i=0; i<listSize; i++) {
				MouseGroup item = (MouseGroup)groupList.get(i);
				// test if group has been used.
				if(!item.isNodeUsed()) {
					String key = item.getKeyName();
					int samples = item.getNumSamples();
					int samplesRounded = item.getRowRoundupSamples();
					// First time round...
					if(parentNode == null) {
						currentNode = new Node(key);
						currentNode.setRealSampleCount(samples);
						groupList.get(i).setNodeUsed(true);
						parentNode = currentNode;
						totalSamples += samplesRounded;
					}else{
						totalSamples += samplesRounded;
						// If this node can be added to the plate...
						if(totalSamples <= PLATE_SIZE) {
							currentNode = new Node(key);
							currentNode.setRealSampleCount(samples);
							groupList.get(i).setNodeUsed(true);
							parentNode.add(currentNode);
							parentNode = currentNode;
						}else{
							continue;
						}
					}
				}
			}

			// Grab the bestNode and fetch the path (MouseGroup key names) from the nodes.
			ArrayList<String> plateContent = new ArrayList<String>();
			TreeNode[] branchPath = parentNode.getPath();
			for(int j =0; j<branchPath.length;j++) {
				if(branchPath[j].toString().equals("root")) {continue;}
				plateContent.add(branchPath[j].toString());
			}
			int row = 1;
			int col = 1;

			AssayPlate plate = new AssayPlate(8,12);
			plate.setAutoPopulated(true);

			// Array the samples onto a AssayPlate object.
			for(String key : plateContent) {
				// get all the mice for that key
				Mouse[] mice = fetchMice(key,groupList);
				for(int i = 1; i<=mice.length; i++,col++) {
					AssayPlateWell well = new AssayPlateWell();
					Mouse mouse = mice[i-1];
					if(mouse !=null) {
						well.setMouse(mouse);
						well.setMouseId(mouse.getId());

						if(mouse.getName().equals("Control")) {
							well.setWellTypeDictId(2L); // Control ID
						}else{
							well.setWellTypeDictId(3L); // Earclip
						}
					}

					well.setPlateRow(row-1);
					well.setPlateColumn(col-1);
					plate.populateWell(well, row-1, col-1);

					if(col%12 == 0 && i!= mice.length) {row++;col=0;}
				}
				row++;
				col = 1;
			}
			plate.setCohortId(cohortId);
			arrayedPlates.add(plate);
			// recalculate the free groups.
			freeGroupCount = getFreeGroupCount(plateContent,groupList, freeGroupCount);
		}
	}


	private int getFreeGroupCount(ArrayList<String> newPlate, ArrayList<MouseGroup> groupList, int freeNodeCount) {

		if(newPlate == null) {
			throw new IllegalArgumentException("Plate list (newPlate) cannont be null");
		}
		for(String s : newPlate) {
			for(int i = 0; i<groupList.size(); i++) {
				if(groupList.get(i).getKeyName().equals(s)) {
					// decrement our count of available nodes (MouseGroup)
					freeNodeCount--;
				}
			}
		}
		return freeNodeCount;
	}




	/**
	 * Method to insert a Mouse object into the correct MouseGroup.
	 *
	 * @param mouseGroupKey
	 * @param mouse
	 * @throws IllegalArgumentException if mouseGroupKey is null
	 * @throws IllegalArgumentException if mouse is null
	 */
	private void insertKeyNodeByName(Long mouseGroupKey, Mouse mouse, ArrayList<MouseGroup> groupList) {
		if(mouseGroupKey == null || mouseGroupKey.equals("")) {
			throw new IllegalArgumentException("mouseGroupKey cannot be null or empty.");
		}
		if(mouse == null) {
			throw new IllegalArgumentException("mouse cannot be null.");
		}
		if(!groupList.isEmpty()) {
			for(MouseGroup group : groupList) {
				if(group.getKeyName().equals(mouseGroupKey.toString())) {
					group.getMice().add(mouse);
					return;
				}
			}
		}

		MouseGroup newGroup = new MouseGroup();
		newGroup.getMice().add(mouse);
		newGroup.setKeyName(mouseGroupKey);
		groupList.add(newGroup);
	}

	/**
	 * Method to add 2 fake mouse objects to the group list.
	 * These will be mapped as Control wells on the AssayPlate
	 *
	 * @param newGroup
	 */
	private void addControlMice(MouseGroup newGroup) {
		Mouse control_1 = new Mouse();
		control_1.setName("Control");
		control_1.setCageId(-1L);
		Mouse control_2 = new Mouse();
		control_2.setName("Control");
		control_2.setCageId(-1L);
		newGroup.getMice().add(control_1);
		newGroup.getMice().add(control_2);
	}


	/**
	 * Method to fetch the Mouse[] of Mouse objects based on the key passed in.
	 *
	 * @param mouseGroupKey
	 * @throws IllegalArgumentException if mouseGroupKey is null
	 * @return An array of Mouse objects
	 */
	private Mouse[] fetchMice(String mouseGroupKey,ArrayList<MouseGroup> groupList) {
		if(mouseGroupKey == null || mouseGroupKey.equals("")) {
			throw new IllegalArgumentException("mouseGroupKey cannot be null or empty.");
		}
		for(MouseGroup group : groupList) {
			if(group.getKeyName().equals(mouseGroupKey)) {
				Collection<Mouse> mouseList = group.getMice();
				Mouse[] mice = (Mouse[])mouseList.toArray(new Mouse[0]);
				return mice;
			}
		}
		return null;
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateAssayPlateComponents(AssayPlate assayPlate) {
		if(assayPlate == null) {
			throw new RuntimeException("AssayPlate cannot be null when inflating components.");
		}

		if(assayPlate.getCohort() == null) {
			assayPlate.setCohort(new Cohort());
		}

		for(int row = 0; row < assayPlate.getRowsOnPlate(); row++) {
			for(int col = 0; col < assayPlate.getColumnsOnPlate(); col++) {
				if(assayPlate.getWell(row, col) == null) {
					AssayPlateWell blank = new AssayPlateWell();
					blank.setWellTypeDictId(AssayPlateWellTypeDictionary.BLANK);
					assayPlate.setWell(row, col, blank);
				}
			}
		}
	}

	public void setAssayService(AssayService assayService) {
		this.assayService = assayService;
	}
}
