/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.genotyping.services;

import java.util.Collection;

import uk.ac.sanger.mig.common.ComponentMap;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;

import uk.ac.sanger.mig.genotyping.Assay;
import uk.ac.sanger.mig.genotyping.AssayPlateWell;
import uk.ac.sanger.mig.genotyping.AssayResult;
import uk.ac.sanger.mig.genotyping.dao.AssayResultDao;
import uk.ac.sanger.mig.genotyping.views.AssayResultsView;
import uk.ac.sanger.mig.mousetracking.MouseAllele;

public class AssayResultServiceImpl extends AbstractServiceImpl2<AssayResult, AssayResultDao> implements
		AssayResultService {

	private AssayService			assayService;
	private AssayPlateWellService	assayPlateWellService;


	public void setAssayService(AssayService assayService) {
		this.assayService = assayService;
	}


	public void setAssayPlateWellService(AssayPlateWellService assayPlateWellService) {
		this.assayPlateWellService = assayPlateWellService;
	}


	public Collection<AssayResult> fetchByAssayId(Long assayId, ComponentMap requiredComponents) {
		Collection<AssayResult> assayResults = getPrimaryDao().fetchByAssayId(assayId);
		attachComponents(assayResults, requiredComponents);
		return assayResults;
	}


	public AssayResult fetchByAssayPlateWellId(Long wellId, ComponentMap requiredComponents) {
		AssayResult assayResult = getPrimaryDao().fetchByAssayPlateWellId(wellId);
		if (assayResult != null) {
			attachComponents(assayResult, requiredComponents);
		}
		return assayResult;
	}


	public Collection<AssayResult> fetchLatestByMouseAlleleId(Long mouseAlleleId, ComponentMap requiredComponents) {
		Collection<AssayResult> assayResults = getPrimaryDao().fetchLatestByMouseAlleleId(mouseAlleleId);
		attachComponents(assayResults, requiredComponents);
		return assayResults;
	}


	@Override
	protected void attachComponents(AssayResult assayResult, ComponentMap requiredComponents) {
		if (assayResult == null) {
			throw new RuntimeException("Cannot attach components to null assayResult");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(AssayResult.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(AssayResult.Component.ASSAY)) {
				assayResult.setAssay(fetchAssay(assayResult.getAssayId(), requiredComponents
						.get(AssayResult.Component.ASSAY)));
			}

			if (fetchAll || requiredComponents.containsKey(AssayResult.Component.ASSAY_PLATE_WELL)) {
				assayResult.setAssayPlateWell(fetchAssayPlateWell(assayResult.getAssayPlateWellId(), requiredComponents
						.get(AssayResult.Component.ASSAY_PLATE_WELL)));
			}
		}
	}


	private Assay fetchAssay(Long assayId, ComponentMap requiredComponents) {
		if (assayId == null) {
			return null;
		} else {
			return assayService.fetchById(assayId, requiredComponents);
		}
	}


	private AssayPlateWell fetchAssayPlateWell(Long assayPlateWellId, ComponentMap requiredComponents) {
		if (assayPlateWellId == null) {
			return null;
		} else {
			return assayPlateWellService.fetchById(assayPlateWellId, requiredComponents);
		}
	}


	public Collection<AssayResultsView> fetchResultsViewByMouseId(Long mouseId) {
		if (mouseId == null)
			throw new IllegalArgumentException("Mouse ID is required");

		return getPrimaryDao().fetchResultsViewByMouseId(mouseId);
	}


	/**
	 * Returns true if this mouse and allele combination has been used in one or more assays with
	 * results, otherwise returns false.
	 *
	 * @param mouseAllele -
	 *            the <code>MouseAllele</code> to check for.
	 * @return true if assays found, otherwise false.
	 */
	public boolean hasResults(MouseAllele mouseAllele) {
		if (mouseAllele == null)
			throw new IllegalArgumentException("mouse allele can not be null");
		Collection<AssayResult> assayResults = getPrimaryDao().fetchByMouseAllele(mouseAllele.getMouseId(),
				mouseAllele.getAlleleId());
		return assayResults.isEmpty() ? false : true;
	}
}
