/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.genotyping.services;

import java.text.MessageFormat;
import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.genotyping.Assay;
import uk.ac.sanger.mig.genotyping.AssayPlate;
import uk.ac.sanger.mig.genotyping.AssayPlateWell;
import uk.ac.sanger.mig.genotyping.AssayResult;
import uk.ac.sanger.mig.genotyping.AssaySearchBean;
import uk.ac.sanger.mig.genotyping.dao.AssayDao;
import uk.ac.sanger.mig.personnel.Person;

public class AssayServiceImpl extends AbstractServiceImpl2<Assay, AssayDao> implements AssayService {

	private AlleleService			alleleService;
	private AssayPlateService		assayPlateService;
	private AssayResultService		assayResultService;
	private DictionaryService		dictionaryService;
	private BinaryFileService		binaryFileService;
	private AssayPlateWellService	assayPlateWellService;


	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}


	public void setAlleleService(AlleleService alleleService) {
		this.alleleService = alleleService;
	}


	public void setAssayPlateService(AssayPlateService assayPlateService) {
		this.assayPlateService = assayPlateService;
	}


	public void setAssayResultService(AssayResultService assayResultService) {
		this.assayResultService = assayResultService;
	}


	public DictionaryService getDictionaryService() {
		return dictionaryService;
	}


	public void setAssayPlateWellService(AssayPlateWellService assayPlateWellService) {
		this.assayPlateWellService = assayPlateWellService;
	}


	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}


	public Collection<DictionaryEntry> fetchAlleleAssayTypes() {
		return getPrimaryDao().fetchAlleleAssayTypes();
	}


	public Collection<DictionaryEntry> fetchPlateAssayTypes() {
		return getPrimaryDao().fetchPlateAssayTypes();
	}


	@Override
	protected void attachComponents(Assay assay, ComponentMap requiredComponents) { // NOPMD by dg6
		// on 08/09/08
		// 16:28
		if (assay == null) {
			throw new RuntimeException("Cannot attach components to null assay");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(Assay.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(Assay.Component.ALLELE)) {
				assay.setAllele(fetchAllele(assay.getAlleleId(), requiredComponents.get(Assay.Component.ALLELE)));
			}

			if (fetchAll || requiredComponents.containsKey(Assay.Component.ASSAY_RESULTS)) {
				assay.setAssayResults(fetchAssayResults(assay.getId(), requiredComponents
						.get(Assay.Component.ASSAY_RESULTS)));

			}

			if (fetchAll || requiredComponents.containsKey(Assay.Component.ASSAY_PLATE)) {
				assay.setAssayPlate(fetchAssayPlate(assay.getAssayPlateId(), requiredComponents
						.get(Assay.Component.ASSAY_PLATE)));

			}
		}
	}


	private Allele fetchAllele(Long alleleId, ComponentMap requiredComponents) {
		return alleleService.fetchById(alleleId, requiredComponents);
	}


	private AssayPlate fetchAssayPlate(Long assayPlateId, ComponentMap requiredComponents) {
		return assayPlateService.fetchById(assayPlateId, requiredComponents);
	}


	private Collection<AssayResult> fetchAssayResults(Long assayId, ComponentMap requiredComponents) {
		return assayResultService.fetchByAssayId(assayId, requiredComponents);
	}


	public Assay fetchByPlateAlleleTypeAndDate(AssaySearchBean searchBean, ComponentMap requiredComponents) {
		Assay assay = getPrimaryDao().fetchByPlateAlleleTypeAndDate(searchBean);
		if (assay != null) {
			attachComponents(assay, requiredComponents);
		}
		return assay;
	}


	@Transactional
	public Long storeComplete(Assay assay, Person user) {
		Long assayId = store(assay, user);

		Collection<AssayResult> results = assay.getAssayResults();
		if (results != null) {
			for (AssayResult result : results) {
				if (result.getAssayResultDictId() != null) {
					result.setAssayId(assayId);
					assayResultService.store(result, user);

					AssayPlateWell well = result.getAssayPlateWell();
					well.setLocked(true);
					assayPlateWellService.store(well, user);
				}
			}
		}
		return assayId;
	}


	public Collection<Assay> fetchLatestByPlateAndAlleleId(Long plateId, Long alleleId, ComponentMap requiredComponents) {
		Collection<Assay> assays = getPrimaryDao().fetchLatestByPlateAndAlleleId(plateId, alleleId);
		attachComponents(assays, requiredComponents);
		return assays;
	}


	@Transactional
	public Long storeCompleteWithFiles(Assay assay, Collection<BinaryFile> assayFilesToAdd,
			Collection<BinaryFile> plateFilesToAdd, Collection<BinaryFile> filesToDelete, Person user) {
		Long assayId = storeComplete(assay, user);

		addFiles(assayFilesToAdd, assayId, TableMetadataEnum.GTP_ASSAY, user);
		addFiles(plateFilesToAdd, assay.getAssayPlateId(), TableMetadataEnum.GTP_ASSAY_PLATE, user);

		for (BinaryFile file : filesToDelete) {
			checkFileRelevance(assay, file);
			binaryFileService.delete(file, user);
		}

		return assayId;
	}


	private void addFiles(Collection<BinaryFile> files, Long fkId, TableMetadataEnum table, Person user) {
		for (BinaryFile f : files) {

			// check if file already exists with this name.
			BinaryFile file = binaryFileService.fetchByName(f.getFileName(), table, fkId);
			if (file != null) {
				file.setBytes(f.getBytes()); // substitute db data for recent file data.
				binaryFileService.store(file, user);
			} else {
				f.setForeignKeyId(fkId);
				f.setForeignTableName(table);
				binaryFileService.store(f, user);
			}
		}
	}


	private void checkFileRelevance(Assay assay, BinaryFile file) {
		boolean assayFileOk = (file.getForeignTableName().equals(TableMetadataEnum.GTP_ASSAY) && file.getForeignKeyId()
				.equals(assay.getId()));
		boolean plateFileOk = (file.getForeignTableName().equals(TableMetadataEnum.GTP_ASSAY_PLATE) && file
				.getForeignKeyId().equals(assay.getAssayPlateId()));

		if (!assayFileOk && !plateFileOk) {
			String errorMessage = MessageFormat
					.format(
							"Attempt to work on binary files that are not related to this assay. Assay id: {0}, Plate id: {1}, File fk id: {2} File foreign table: {3}",
							new Object[] { assay.getId(), assay.getAssayPlateId(), file.getForeignKeyId(),
									file.getForeignTableName().name() });

			throw new IllegalArgumentException(errorMessage);
		}
	}


	public Collection<Assay> fetchByPlateId(Long plateId, ComponentMap componentMap) {
		Collection<Assay> assays = getPrimaryDao().fetchByPlateId(plateId);
		attachComponents(assays, componentMap);
		return assays;
	}
}
