/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.havanarequest.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.allelerequest.AlleleRequest;
import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.havanarequest.HavanaRequest;
import uk.ac.sanger.mig.havanarequest.HavanaRequestSearchBean;
import uk.ac.sanger.mig.havanarequest.views.HavanaRequestListView;

public class HavanaRequestDaoSpringImpl extends AbstractDaoSpringImpl<HavanaRequest> implements HavanaRequestDao {

	private HavanaRequestListViewRowMapper havanaRequestListViewRowMapper;
	private PaginatedSearchUtils<HavanaRequestListView> paginator;

	
	public void setHavanaRequestListViewRowMapper(
			HavanaRequestListViewRowMapper havanaRequestListViewRowMapper) {
		this.havanaRequestListViewRowMapper = havanaRequestListViewRowMapper;
	}

	
	public void setPaginator(PaginatedSearchUtils<HavanaRequestListView> paginator) {
		this.paginator = paginator;
	}


	public boolean alreadyExists(AlleleRequest alleleRequest) {
		int recordCount = 
			getSimpleJdbcTemplate()
			.queryForInt("select count(*) from arq_havana_request where allele_request_id = ?", 
					alleleRequest.getId());
		if (recordCount > 0) { return true; } 
		else 				 {return false;}
	}

	public Collection<HavanaRequestListView> search(HavanaRequestSearchBean searchBean) {
		HavanaRequest havanaRequest = searchBean.getHavanaRequest();

		Collection<Object> bindVariables = new ArrayList<Object>();

		StringBuffer sql = new StringBuffer();		
		sql.append("select * from arq_havana_request_list_vw v where 1=1 ");

		if(searchBean.getSearchString()!=null && 
				!searchBean.getSearchString().equals("") && !searchBean.getSearchBy().equals("") && !searchBean.getSearchString().contains("Please enter")
		){
			if(searchBean.getSearchBy().equals("ID")){
				sql.append(" and v.id = ? ");
				bindVariables.add(new Long(searchBean.getSearchString()));
			}
			if(searchBean.getSearchBy().equals("VEGA_ID")){
				sql.append(" and upper(v.vega_id) = ? ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}
			if(searchBean.getSearchBy().equals("PRIMARY_NAME")){
				sql.append(" and upper(v.primary_name) =? ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}
			if(searchBean.getSearchBy().equals("ENSEMBL_ID")){
				sql.append(" and upper(v.ENSEMBL_ID) = ? ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}

		}
		else{
    	// search by status id
			if(havanaRequest.getStatusDictId() != null && havanaRequest.getStatusDictId() > 0) {
				sql.append(" and v.status_dict_id = ? ");
				bindVariables.add(havanaRequest.getStatusDictId());
			}

			// search by source id
			if(havanaRequest.getAlleleRequest().getSource() != null 
					&& havanaRequest.getAlleleRequest().getSource().getId() != null
					&& havanaRequest.getAlleleRequest().getSource().getId() > 0) {
				sql.append(" and v.request_source_dict_id = ? ");
				bindVariables.add(havanaRequest.getAlleleRequest().getSource().getId());
			}

			if(havanaRequest.getChromosomeName() != null && !havanaRequest.getChromosomeName().equals("")) {
				sql.append(" and v.chromosome_name = ? ");
				bindVariables.add(havanaRequest.getChromosomeName());
			}
			
			if(searchBean.getOrderBy() != null && !searchBean.getOrderBy().equals("")) {
				sql.append(" order by " + searchBean.getOrderBy());
				if(searchBean.isOrderDesc()) {
					sql.append(" desc ");
				}
			}
		}
		
		return paginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, havanaRequestListViewRowMapper); 
	}

	public HavanaRequest fetchbyAlleleRequestId(Long id){
		HavanaRequest havanaRequest = null;
		String sql = "select * from arq_havana_request where allele_request_id = ?";   
		try {
			havanaRequest = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), id);
		} catch(IncorrectResultSizeDataAccessException e) {
			logger.warn("fetchByAlleleRequestId: " + e);
		} 
		return havanaRequest;
	}


	public Collection<HavanaRequestListView> fetchCurrentByEditDate(Date editedFrom, Date editedTo) {
		
		Collection<HavanaRequestListView> havanaRequests = null;
		
		String sql = " select * from arq_havana_request_list_vw where edited_date >= ? AND edited_date < ? ";		
		
		try {
			havanaRequests = getSimpleJdbcTemplate().query(sql,havanaRequestListViewRowMapper,editedFrom,editedTo); 
		} catch(IncorrectResultSizeDataAccessException e) {
			
			logger.warn("No current Havana Request records found in arq_havana_request_list_vw table.");
		} 
		
		return (havanaRequests == null) ? new ArrayList<HavanaRequestListView>() : havanaRequests;
	}

	public Collection<HavanaRequestListView> fetchHistoricalByEditDate(Date editedFrom, Date editedTo) {
		Collection<HavanaRequestListView> havanaRequests = null;
		String sql = " select * from arq_havana_req_historical_vw where edited_date >= ? AND edited_date < ? ";		
		
		try {
			havanaRequests = getSimpleJdbcTemplate().query(sql,havanaRequestListViewRowMapper,editedFrom,editedTo);
		} catch(IncorrectResultSizeDataAccessException e) {

			logger.warn("No historical Havana Request records found in arq_havana_req_historical_vw table.");		
		} 
		
		return (havanaRequests == null) ? new ArrayList<HavanaRequestListView>() : havanaRequests;
	}	


	protected void bindVariables(ParameterizedSqlUpdate sql, HavanaRequest havanaRequest) {

		sql.addLongParam(havanaRequest.getStatus().getId(), "STATUS_DICT_ID");
		sql.addLongParam(havanaRequest.getAlleleRequestId(), "ALLELE_REQUEST_ID");
		sql.addStringParam(havanaRequest.getVegaId(), "VEGA_ID");
		sql.addStringParam(havanaRequest.getComments(), "COMMENTS");
		sql.addStringParam(havanaRequest.getUpdatedGeneName(),"UPDATED_GENE_NAME");
		sql.addBooleanParam(havanaRequest.isEndNotFound(), "END_NOT_FOUND");
		sql.addStringParam(havanaRequest.getUpdatedChromosomeName(), "UPDATED_CHROMOSOME_NAME");
		sql.addBooleanParam(havanaRequest.isStartNotFound(), "START_NOT_FOUND");
		sql.addLongParam(havanaRequest.getCreatorId(), "CREATED_BY");
		sql.addStringParam(havanaRequest.getCriticalExonId(), "CRITICAL_EXON_ID");
		sql.addStringParam(havanaRequest.getEditedBy(), "EDITED_BY");
		sql.addDateParam(havanaRequest.getCreatedDate(), "CREATED_DATE");
		sql.addDateParam(havanaRequest.getEditDate(), "EDITED_DATE");
		sql.addBooleanParam(havanaRequest.isPseudogene(), "PSEUDOGENE");
		//sql.addBooleanParam(havanaRequest.isAssemblyError(), "ASSEMBLY_ERROR");
		sql.addBooleanParam(havanaRequest.isSmallIntrons(), "SMALL_INTRONS");
		sql.addBooleanParam(havanaRequest.isMultipleCodingTrancripts(), "MULTIPLE_CODING_TRANSCRIPTS");
		sql.addBooleanParam(havanaRequest.isSymmetricalExons(), "SYMMETRICAL_EXON");
    //LARGE_CODING_EXON=?, LESS_THAN_THREE_CODING_EXONS=?, NO_NMD_INDUCED=?, POSSIBLE_REINITIATION=?, contains_hydrophobic_region=?, critical_exon_contains_ATG=?, has_transmembrane_domain =?, has_signal_peptide = ? where ID=?";
		sql.addBooleanParam(havanaRequest.isLargeCodingExon(), "LARGE_CODING_EXON");
		sql.addBooleanParam(havanaRequest.isLessThanThreeCodingExons(), "LESS_THAN_THREE_CODING_EXONS");
		sql.addBooleanParam(havanaRequest.isNoNMDInduced(), "NO_NMD_INDUCED");
		//sql.addBooleanParam(havanaRequest.isPossibleReinitiation(), "POSSIBLE_REINITIATION");
		//sql.addBooleanParam(havanaRequest.isContainsHydrophobicRegion(), "contains_hydrophobic_region");
		//sql.addBooleanParam(havanaRequest.isCriticalExonContainsATG(), "critical_exon_contains_ATG");
		sql.addBooleanParam(havanaRequest.isTransMembraneDomain(), "has_transmembrane_domain");
		sql.addBooleanParam(havanaRequest.isSignalPeptide(), "has_signal_peptide");
		sql.addBooleanParam(havanaRequest.isCriticalExonToo3(), "critical_exon_too_3_prime");
		sql.addBooleanParam(havanaRequest.isGeneCluster(), "GENE_CLUSTER");
		sql.addBooleanParam(havanaRequest.isOverlappingLoci(), "OVERLAPPING_LOCI");
		sql.addBooleanParam(havanaRequest.isRepeatRegion(), "REPEAT_REGION");
		sql.addLongParam(havanaRequest.getId(), "ID");

	}

	public HavanaRequest fetchbyVegaId(String vegaId) {
		HavanaRequest havanaRequest = null;
		String sql = "select * from arq_havana_request where vega_id = ?";   
		try {
			havanaRequest = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), vegaId);
		} catch(IncorrectResultSizeDataAccessException e) {
			logger.warn("fetchByAlleleRequestId: " + e);
		} 
		return havanaRequest;
	}

	public HavanaRequest fetchbyGeneName(String geneName) {
		HavanaRequest havanaRequest = null;
		String sql = "select * from arq_havana_request where allele_request_id = ?";   
		try {
			havanaRequest = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), geneName);
		} catch(IncorrectResultSizeDataAccessException e) {
			logger.warn("fetchByAlleleRequestId: " + e);
		} 
		return havanaRequest;
	}

	public HavanaRequest fetchbyEnsemblId(String ensemBlId) {
		HavanaRequest havanaRequest = null;
		String sql = "select * from arq_havana_request where allele_request_id = ?";   
		try {
			havanaRequest = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), ensemBlId);
		} catch(IncorrectResultSizeDataAccessException e) {
			logger.warn("fetchByAlleleRequestId: " + e);
		} 
		return havanaRequest;
	}	
}
