/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.havanarequest.services;

import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import uk.ac.sanger.mig.allelerequest.AlleleRequest;
import uk.ac.sanger.mig.allelerequest.services.AlleleRequestService;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.MailHelper;
import uk.ac.sanger.mig.common.Statistic;
import uk.ac.sanger.mig.havanarequest.HavanaRequest;
import uk.ac.sanger.mig.havanarequest.dao.HavanaRequestDao;
import uk.ac.sanger.mig.havanarequest.enums.HavanaRequestComponent;
import uk.ac.sanger.mig.havanarequest.views.HavanaRequestListView;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.dao.MailingListDao;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class HavanaRequestServiceImpl extends AbstractServiceImpl<HavanaRequest> implements HavanaRequestService {

	private MailHelper mailHelper;
	private HavanaRequestDao havanaRequestDao;
	private AlleleRequestService alleleRequestService;
	private PersonService personService;

	public void setHavanaRequestDao(HavanaRequestDao havanaRequestDao) {
		this.havanaRequestDao = havanaRequestDao;
	}

	public void setAlleleRequestService(AlleleRequestService alleleRequestService) {
		this.alleleRequestService = alleleRequestService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setMailHelper(MailHelper mailHelper) {
		this.mailHelper = mailHelper;
	}

	public Long store(HavanaRequest havanaRequest, Person user) {
		return havanaRequestDao.store(havanaRequest, user);
	}

	public HavanaRequest fetchById(Long requestId, ComponentMap requiredComponents) {
		HavanaRequest havanaRequest = havanaRequestDao.fetchById(requestId);

		if(havanaRequest != null) {
			attachComponents(havanaRequest, requiredComponents);
		}

		return havanaRequest;
	}

	public boolean sendNewRequestNotification(HavanaRequest havanaRequest) {	
		Map<String, Object>model = new HashMap<String, Object>();
		model.put("havanaRequest", havanaRequest);			
		return mailHelper.sendMail("havana_request", model, MailingListDao.HAVANA_REQUEST_MAILING_LIST);

	}

	public boolean sendHavanaActivityReport(Date activityFromDate, Date activityToDate) {	
		Map<String, Object>model = new HashMap<String, Object>();

		List<HavanaRequestListView> havanaActivity = 
			(List<HavanaRequestListView>)getHavanaActivity(activityFromDate, activityToDate);

		Collections.sort(havanaActivity);	
		model.put("havanaActivity", havanaActivity);	//we want our raw data
		Collection<Statistic> havanaSummaryStats = deriveSummaryStats(havanaActivity);
		model.put("havanaSummaryStats", havanaSummaryStats); //...and our summary stats.
		model.put("activityFromDate",activityFromDate);//...and Date range
		model.put("activityToDate",activityToDate);
		return mailHelper.sendMail("havana_activity", model, MailingListDao.HAVANA_ACTIVITY_REPORT_MAILING_LIST);	
	}

	public boolean sendHavanaActivityReportWeekToDate() {				
		long now = System.currentTimeMillis();
		long lastWeek = now - 604800000L;		
		return sendHavanaActivityReport(new Date(lastWeek), new Date(now));
	}

	public Collection<HavanaRequestListView> getHavanaActivity(Date activityFromDate, Date activityToDate) {
		
		Collection<HavanaRequestListView> current = 
			havanaRequestDao.fetchCurrentByEditDate(activityFromDate,activityToDate);		
		
		Collection<HavanaRequestListView> historical = 
			havanaRequestDao.fetchHistoricalByEditDate(activityFromDate,activityToDate);
		
		current.addAll(historical);
		
		return current;
	}

	public Collection<Statistic> deriveSummaryStats(Collection<HavanaRequestListView> havanaActivity) {	
		HashMap<String,Statistic> stats = new HashMap<String,Statistic>();
		Iterator iter = havanaActivity.iterator();
		while(iter.hasNext()) {
			HavanaRequestListView havanaRequestListView = (HavanaRequestListView)iter.next();
			if(havanaRequestListView.getStatus() == null || havanaRequestListView.getStatus().length()==0)
				continue;
			if(!stats.containsKey(havanaRequestListView.getStatus())) {
				Statistic stat = new Statistic(havanaRequestListView.getStatus(),new Integer(1));
				stats.put(havanaRequestListView.getStatus(), stat);
			} else {
				Statistic stat = stats.get(havanaRequestListView.getStatus());
				Integer value = (Integer)stat.getValue();
				stat.setValue(value + 1);
				stats.put(havanaRequestListView.getStatus(), stat);		
			}
		}
		return stats.values();
	}

	@Override
	protected void attachComponents(HavanaRequest havanaRequest, ComponentMap requiredComponents) {
		if(havanaRequest == null) {
			throw new RuntimeException("Cannot attach components to null havanaRequest");
		}		

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(HavanaRequestComponent.ALL_HAVANA_REQUEST_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(HavanaRequestComponent.ALLELE_REQUEST)) {
				havanaRequest.setAlleleRequest(fetchAlleleRequest(havanaRequest, requiredComponents.get(HavanaRequestComponent.ALLELE_REQUEST)));
			}

			if(fetchAll || requiredComponents.containsKey(HavanaRequestComponent.CREATOR)) {
				havanaRequest.setCreator(fetchCreator(havanaRequest, requiredComponents.get(HavanaRequestComponent.CREATOR)));
			}    
		}		
	}

	private AlleleRequest fetchAlleleRequest(HavanaRequest havanaRequest, ComponentMap requiredComponents) {
		if(havanaRequest.getAlleleRequestId() != null) {
			return alleleRequestService.fetchById(havanaRequest.getAlleleRequestId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	private Person fetchCreator(HavanaRequest havanaRequest, ComponentMap requiredComponents) {
		if(havanaRequest.getCreatorId() != null) {
			return personService.fetchById(havanaRequest.getAlleleRequestId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	public boolean alreadyExists(AlleleRequest alleleRequest) {

		return havanaRequestDao.alreadyExists(alleleRequest);
	}

	public HavanaRequest fetchByAlleleRequestId(Long alleleRequestId, ComponentMap requiredComponents) {

		HavanaRequest havanaRequest = havanaRequestDao.fetchbyAlleleRequestId(alleleRequestId);

		if(havanaRequest != null) {
			attachComponents(havanaRequest, requiredComponents);
		}

		return havanaRequest;
	}
}
