package uk.ac.sanger.mig.histology;

import java.awt.Rectangle;
import java.io.File;
import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.annotation.Annotation;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;


public final class Crop extends AuthenticatedDataObject implements Cloneable  {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		CROP_JOB,
		TISSUE,
		ANNOTATIONS,
		CREATOR
	}

	private Collection<Annotation> annotations;

    private int cropWidth;
    private int cropHeight;
    private int x;
    private int y;
    private File fullImagePath;
    private File resizedImagePath;

    private String comments;
    private Boolean stainingPresent;
    private boolean externalApproved;
    private Boolean qcPass;
    private String qcFailReason;
    private Long cropJobId;
    private CropJob cropJob;
    private Long tissueId;
    private Tissue tissue;

    private boolean flaggedForTiling;
    private int tileEdgeExponent;//allows us to resize tiles by powers of zoomRadix (no edge loss)
    private int zoomRadix;
    private boolean tiled;

    private boolean qcLocked;


    public Object clone() {

    	try {

    	    return  (Crop) super.clone();
    	} catch (CloneNotSupportedException e) {
    	    return null; // never invoked
    	}
    }

	public boolean isQcLocked() {
		return this.qcLocked;
	}

	public void setQcLocked(boolean qcActionLocked) {
		this.qcLocked = qcActionLocked;
	}

	public boolean isFlaggedForTiling() {
		return this.flaggedForTiling;
	}

	public void setFlaggedForTiling(boolean flaggedForTiling) {
		this.flaggedForTiling = flaggedForTiling;
	}


	public boolean isTiled() {
    	return tiled;
    }

	public void setTiled(boolean tiled) {
		this.tiled = tiled;
	}

	/**
	 * Returns the maximum tiling zoom depth. It equals number of times
	 * you would have to divide the Crop image by zoomRadix to fit it
	 * within a tile of dimensions zoomRadix^tileEdgeExponent.
	 *
	 * @return
	 * @throws IllegalStateException if cropWidth, cropHeight, tileEdgeExponent,
	 * or zoomRadix is not > 0
	 */
    public int getMaxZoomDepth() {

    	if( cropWidth < 1 || cropHeight < 1 || tileEdgeExponent < 1 || zoomRadix < 1)
    		throw new IllegalStateException("One of cropWidth, cropHeight, tileEdgeExponent, zoomRadix is not > 0!");

    	int largerOfWxH = (cropWidth > cropHeight) ? cropWidth : cropHeight;

    	double log = Math.log( largerOfWxH / (double) getTileEdge()) / Math.log( zoomRadix );

    	return (int)Math.ceil( log );
    }


    /**
     *
     * Returns tile edge defined for this Crop as a function
     * of zoomRadix^tileEdgeExponent.
     *
     * This guarantees no edge loss when resizing tiles by
     * powers of zoomRadix.
     *
     * An IllegalStateException will be thrown if either
     * tile exponent or zoom radix are not > 0.
     *
     * @return
     */
	public int getTileEdge() {

		if( getZoomRadix() < 1 || getTileEdgeExponent() < 1) {

			throw new IllegalStateException("Both tile exponent and zoom radix must be > 0");
		}

		return (int)Math.pow(getZoomRadix(), getTileEdgeExponent());
	}

	public int getTileEdgeExponent() {
		return this.tileEdgeExponent;
	}

	public void setTileEdgeExponent(int tileEdgeExponent) {
		this.tileEdgeExponent = tileEdgeExponent;
	}

	public int getZoomRadix() {
		return this.zoomRadix;
	}


	public void setZoomRadix(int zoomRadix) {
		this.zoomRadix = zoomRadix;
	}


	public Date getCopyright() {
    	return getEditDate();
    }

	public int getMinZoomDepth() {
    	return 0;
    }


	/**
     * Returns the size in MB of the full Crop image.
     *
     * Returns 0.0 if the image file cannot be found or
     * the file path is null.
     *
     * @return
     */
    public double getFullImageSize() {

    	if( fullImagePath == null ) {

    		return 0.0d;

    	} else {

    		return fullImagePath.length() / Math.pow(1024, 2);
    	}
    }

	public Boolean getQcPass() {
		return this.qcPass;
	}

	public void setQcPass(Boolean qcPass) {
		this.qcPass = qcPass;
	}

	public String getQcFailReason() {
		return this.qcFailReason;
	}

	public void setQcFailReason(String qcFailReason) {
		this.qcFailReason = qcFailReason;
	}


	/**
	 * Returns absolute file path to resized image or null
	 * if resizedImagePath is null.
	 *
	 * @return
	 */
    public File getResizedImagePath() {
    	return (resizedImagePath!= null) ? resizedImagePath.getAbsoluteFile() : null;
	}

	public void setResizedImagePath(File resizedFilePath) {
		this.resizedImagePath = resizedFilePath;
	}

	public CropJob getCropJob() {
		return this.cropJob;
	}

	public void setCropJob(CropJob cropJob) {
		this.cropJob = cropJob;
	}

	public Long getCropJobId() {
		return this.cropJobId;
	}

	public void setCropJobId(Long cropJobId) {
		this.cropJobId = cropJobId;
	}

    public Boolean getStainingPresent() {
		return this.stainingPresent;
	}

	public void setStainingPresent(Boolean stainingPresent) {
		this.stainingPresent = stainingPresent;
	}

	public int getCropHeight() {
		return this.cropHeight;
	}

	public void setCropHeight(int cropHeight) {
		this.cropHeight = cropHeight;
	}

	public int getCropWidth() {
		return this.cropWidth;
	}

	public void setCropWidth(int cropWidth) {
		this.cropWidth = cropWidth;
	}


    public int getX() {
		return this.x;
	}

	public void setX(int x) {
		this.x = x;
	}

	public int getY() {
		return this.y;
	}

	public void setY(int y) {
		this.y = y;
	}

	/**
	 * Returns absolute file path to the full image or null
	 * if fullImagePath is null.
	 *
	 * @return
	 */
	public File getFullImagePath() {
		return (fullImagePath!= null) ? fullImagePath.getAbsoluteFile() : null;
	}

	public void setFullImagePath(File filePath) {
		this.fullImagePath = filePath;
	}

	public String getComments() {
        return comments;
    }

    public void setComments(String comments) {
        this.comments = comments;
    }

	public boolean isExternalApproved() {
		return this.externalApproved;
	}

	public void setExternalApproved(boolean externalApproved) {
		this.externalApproved = externalApproved;
	}

	public Long getTissueId() {
		return this.tissueId;
	}

	public void setTissueId(Long tissueId) {
		this.tissueId = tissueId;
	}

	public Tissue getTissue() {
		return this.tissue;
	}

	public void setTissue(Tissue tissue) {
		this.tissue = tissue;
	}


	//use when comparing Crops from same Scan across different crop jobs
	//to ensure no dupes.
	public Rectangle getRectangle() {
		return new Rectangle(getX(), getY(), getCropWidth(), getCropHeight());
	}


	@Override
	public int hashCode() {
		//TODO add scan id as criteria
		return this.getRectangle().hashCode();
	}

	/**
	 * Returns true of x, y, width height are same.
	 *
	 */
	@Override
	public boolean equals(Object object) {

		if(object == this) return true;
		if( object == null || !(object instanceof Crop) ) return false;
		Crop crop = (Crop)object;

		return crop.getRectangle().equals(this.getRectangle());

	}

	public Collection<Annotation> getAnnotations() {
		return this.annotations;
	}

	public void setAnnotations(Collection<Annotation> annotations) {
		this.annotations = annotations;
	}



}