package uk.ac.sanger.mig.histology.applets.scancropper.controllers;

import java.applet.AudioClip;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.sanger.mig.histology.applets.scancropper.model.CropSet;
import uk.ac.sanger.mig.histology.applets.scancropper.services.HttpRequestService;
import uk.ac.sanger.mig.histology.applets.scancropper.services.ImageProcessingService;
import uk.ac.sanger.mig.histology.views.SerializableScanView;

/**
 *
 * Responds to user actions by delegating to the appropriate
 * service method(s).
 *
 * @author ae1
 *
 */
public class ActionController implements ActionListener,
										 ListSelectionListener,
										 ChangeListener {

	private Object[] currentSelectedScans;
	//services
	private HttpRequestService httpRequestService;
	private ImageProcessingService imageProcessingService;

	private Icon warningIcon, decideIcon;
	private AudioClip audioClip;

	public ActionController(Icon warningIcon, Icon decideIcon, AudioClip audioClip ) {
		this.warningIcon = warningIcon;
		this.decideIcon = decideIcon;
		this.audioClip = audioClip;
	}

	/**
	 * Identifies user button clicks.
	 * @author ae1
	 *
	 */
	public enum ButtonActionEnum {
		MAKE_CROPS,
		CLEAR_CROPS,
		SUBMIT_CROPS,
		REJECT_SCAN;
	}

	/**
	 * Identifies radio button selections
	 * @author ae1
	 *
	 */
	public enum RadioActionEnum {
		CROP_NOW,
		CROP_OVERNIGHT;
	}


	/**
	 * Responds to user changing threshold or dilation parameter inputs.
	 */
	public void stateChanged(ChangeEvent e) {

		Object source = e.getSource();

		if(source instanceof JSpinner) {

			JSpinner dialationSpinner = (JSpinner)source;
			Integer value = (Integer)dialationSpinner.getValue();
			imageProcessingService.setCurrentDilationCount( value );
		}
	}

	/**
	 * Responds to user button clicks by delegating
	 * to the appropriate service method(s) for either making,
	 * clearing or submitting crops remotely, as the case may be.
	 * Also, handles Scan rejection and crop now/overnight toggle.
	 */
	public void actionPerformed(ActionEvent e) {

		if( !imageProcessingService.isScanSelected() ) {
			Component parentComponent = ((Component)e.getSource()).getParent().getParent();
			JOptionPane.showMessageDialog(parentComponent,
					"You must first select a scan!",
					"Warning",
					JOptionPane.WARNING_MESSAGE, warningIcon );
			return;
		}

		if( e.getActionCommand().equals(ButtonActionEnum.MAKE_CROPS.toString())) {

			if( currentSelectedScans != null && currentSelectedScans.length > 0) {

				imageProcessingService.makeAllSelectedCrops( currentSelectedScans );

				if( audioClip != null ) audioClip.play();
			}



		} else 	if( e.getActionCommand().equals(ButtonActionEnum.CLEAR_CROPS.toString())) {

			imageProcessingService.clearCurrentCrops();

		} else 	if( e.getActionCommand().equals(ButtonActionEnum.SUBMIT_CROPS.toString())) {

			if( httpRequestService.isRequestInProgress() ) {

				Component parentComponent = ((Component)e.getSource()).getParent().getParent();
				JOptionPane.showMessageDialog(parentComponent,"Cannot submit, request in progress!",
						"Warning",
						JOptionPane.WARNING_MESSAGE, warningIcon );
				return;
			}

			CropSet cropSet = imageProcessingService.getCurrentCropSet();

			if( cropSet != null && !cropSet.isEmpty() ) {

				httpRequestService.scheduleCrops( cropSet );

			} else {

				Component parentComponent = ((Component)e.getSource()).getParent().getParent();
				JOptionPane.showMessageDialog(parentComponent,
						"You must make at least one crop for this Scan in order to submit.",
						"Warning",
						JOptionPane.WARNING_MESSAGE, warningIcon );

			}


		}  else 	if( e.getActionCommand().equals(ButtonActionEnum.REJECT_SCAN.toString())) {

			if( httpRequestService.isRequestInProgress() ) {

				Component parentComponent = ((Component)e.getSource()).getParent().getParent();
				JOptionPane.showMessageDialog(parentComponent,
						"Cannot reject, request in progress!",
						"Warning",
						JOptionPane.WARNING_MESSAGE, warningIcon );
				return;
			}

			Component parentComponent = ((Component)e.getSource()).getParent().getParent();

			String message = "Please indicate the reason for scan rejection: 'Bad Scan' or 'Bad Slide'." +
			"\nThe scan and any child crops will be flagged for deletion. " +
			"\nIf 'Bad Scan' is indicated, the original Slide will be placed in the re-scan bin. " +
			"\nIf 'Bad Slide' is indicated, original Slide will be place in the re-creation bin.";

			Object[] reasons = {"<choose reason>","Bad Scan", "Bad Slide"};

			String reason = null;

			while("<choose reason>".equals(reason = (String)JOptionPane.showInputDialog(
					parentComponent,
					message,
					"Indicate Reason for Scan Rejection",
					JOptionPane.PLAIN_MESSAGE,
					decideIcon, reasons,""))) {}

			if( reason != null ) {

				Long scanId = imageProcessingService.getCurrentScanId();
				httpRequestService.reject( scanId, reason.equals("Bad Slide")  );

			}

		} else if(e.getActionCommand().equals( RadioActionEnum.CROP_NOW.toString() ))  {

			imageProcessingService.setCropImmediately( true );

		} else if(e.getActionCommand().equals( RadioActionEnum.CROP_OVERNIGHT.toString() ))  {

			imageProcessingService.setCropImmediately( false );

		}


	}

	/**
	 * Responds to user barcode selection by loading remote thumbnail image
	 * for that particular slide.
	 */
	public void valueChanged(ListSelectionEvent event) {

		if( !(event.getSource() instanceof JList) )
			return;

		JList slideList = (JList)event.getSource();

		Object[] currentSelectedScans = slideList.getSelectedValues();

		//want first to compare old with new selected scans
		if(this.currentSelectedScans != null &&
				this.currentSelectedScans.length != currentSelectedScans.length) {

			this.currentSelectedScans = currentSelectedScans;

			slideList.addSelectionInterval(slideList.getMinSelectionIndex(),
					slideList.getMaxSelectionIndex());

			return;

		}

		this.currentSelectedScans = currentSelectedScans;


		if( !event.getValueIsAdjusting() ) {

			if( ! (slideList.getSelectedValue() instanceof SerializableScanView) )
				throw new RuntimeException("Selected element must be instance of SerializableScanView!");

			SerializableScanView scan = (SerializableScanView)slideList.getSelectedValue();

			if(scan != null)
				imageProcessingService.load( scan );
		}



	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////// dependencies /////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	public void setHttpRequestService(HttpRequestService httpRequestService) {
		this.httpRequestService = httpRequestService;
	}

	public void setImageProcessingService(
			ImageProcessingService imageProcessingService) {
		this.imageProcessingService = imageProcessingService;
	}


}
