package uk.ac.sanger.mig.histology.applets.scancropper.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.MemoryImageSource;

import javax.swing.JPanel;

import uk.ac.sanger.mig.histology.applets.scancropper.model.ConnectedComponentBounds;
import uk.ac.sanger.mig.histology.applets.scancropper.model.CropSet;
import uk.ac.sanger.mig.histology.applets.scancropper.model.ImageModel;

/**
 * Loads image and displays updated image data.
 * 
 * @author ae1
 *
 */
public class ImagePanel extends JPanel implements ImageUpdateListener, 
												  CropSetListener,
												  TissueLabelListener {
		
	private int displayedWidth, displayedHeight;
	private int xOffset, yOffset;
	
	private MemoryImageSource source;
	
	private Image displayedImage;
	private CropSet displayedCropSet;
	private Rectangle displayedNewCropRectangle;
	private String tissue;
	private Font tissueLabelFont;
	private Font tileFlagFont;
	private final int MAX_TISSUE_LABEL_CHARS = 41;
	private final int TISSUE_LABEL_FONT_SIZE = 13;
	
	public ImagePanel(Dimension preferredSize) {

		 setPreferredSize(preferredSize);
		 setBackground(Color.LIGHT_GRAY);
		 tissueLabelFont = new Font("dialog", Font.BOLD, TISSUE_LABEL_FONT_SIZE);
		 tileFlagFont = tissueLabelFont; //share same font
	}
	
	public void updateTissueLabel(String text) {
		
		if( text == null || text.length() == 0) {
		
			tissue = "Unidentified Tissue";
		
		} else if( text.length() > MAX_TISSUE_LABEL_CHARS ) {
			
			tissue = text.substring(0, MAX_TISSUE_LABEL_CHARS) + "...";
		
		} else {
			
			tissue = text;
		}
		
		repaint();
			
	}  
	
	private void paintTissueLabel(Graphics g) {
		Color c = g.getColor();
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, this.getWidth(), tissueLabelFont.getSize() + 6);//clears bg
		g.setFont( tissueLabelFont );
		g.setColor(Color.BLACK);
		g.drawString(tissue, 4, 15);
		g.setColor(Color.BLUE);
		g.drawRect(0, 0, this.getWidth() - 1 , tissueLabelFont.getSize() + 6);//border
		g.setColor(c);// restore color		
	}
	
	@Override
	public void paintComponent(Graphics g) {

		g.fillRect(0, 0, this.getWidth(), this.getHeight());//clears bg

		if( tissue != null ) {
			paintTissueLabel( g ); 
		}
		
		g.translate(xOffset, yOffset);
		
		if (displayedImage != null) {
			g.drawImage(displayedImage, 0,0, displayedWidth, displayedHeight, this);
		}
	}



	@Override
	protected void paintChildren(Graphics g) {
	//called after paintComponent...	
		Color c = g.getColor();

		//draw new rect for manual add
		if( displayedNewCropRectangle != null) {
						
			g.setColor(Color.RED);	
			g.drawRect(displayedNewCropRectangle.x, displayedNewCropRectangle.y, 
					displayedNewCropRectangle.width, displayedNewCropRectangle.height);
	
		}
			
		
		if ( displayedCropSet != null ) {
			
			for (ConnectedComponentBounds cc : displayedCropSet.getConnectedComponentBoundsSet()) {
	
				g.setClip(cc.minX, cc.minY, cc.maxX - cc.minX + 1, cc.maxY - cc.minY + 1);
				
				if( cc.isFlaggedForTiling() ) {
					g.setColor(Color.BLACK);
					g.setFont( tileFlagFont );
					g.drawString("T", cc.minX + 3, cc.minY + TISSUE_LABEL_FONT_SIZE);
				}
				
				g.setColor(Color.RED);
				g.drawRect(cc.minX, cc.minY, cc.maxX - cc.minX, cc.maxY - cc.minY);

			}		
			
		}
		
	
		g.setColor(c);// restore color
	}

	
	public void updateImage(ImageModel imageModel, long delay) {

		if(imageModel == null) {//image cleared or non existent
			
			source = null;
			displayedImage = null;

			
		} else {//image loading

			source = new MemoryImageSource(imageModel.getWidth(), imageModel.getHeight(), 
					imageModel.getPixels(), 0, imageModel.getWidth());
			
			source.setAnimated( true );
			
			this.displayedWidth = imageModel.getWidth();
			this.displayedHeight = imageModel.getHeight();
			
			displayedImage = createImage(source); // image to display
			// center the image...
			xOffset = (int) (((float) (getWidth() - this.displayedWidth)) / 2f);
			yOffset = (int) (((float) (getHeight() - this.displayedHeight)) / 2f) + tissueLabelFont.getSize();	
			
			if(delay > 0) {
				try {  Thread.sleep( delay ); } 
				catch (InterruptedException e) {}
			}
			
			source.newPixels(0, 0, displayedWidth, displayedHeight);	
			
		} 
		
		paintImmediately(0,0,this.getWidth(), this.getHeight());
	}

	public void updateCropSet(CropSet cropSet, Rectangle newCropRectangle, long delay) {
			
			this.displayedCropSet = cropSet;
			this.displayedNewCropRectangle = newCropRectangle;
			
			if(delay > 0) {
				try {  Thread.sleep( delay ); } 
				catch (InterruptedException e) {}
			}		
						
			paintImmediately(0,0,this.getWidth(), this.getHeight());	
	}


	public int getXOffset() {
		return this.xOffset;
	}


	public int getYOffset() {
		return this.yOffset;
	}

	

}
