package uk.ac.sanger.mig.histology.applets.scancropper.model;

import java.awt.Rectangle;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;


/**
 * Encapsulates a set of crop coordinates on a given slide--ie, x-y origin, width and height.

 * Also, provides convenience method for trasmitting this CropSet data as HTTP post.
 *
 * @author ae1
 */
public class CropSet {

	private Set<ConnectedComponentBounds> components;
	private Long scanId;
	private double widthScaleFactor;
	private double heightScaleFactor;
	private boolean cropImmediately;
	private boolean formSubmit;
	private int autoThresholdLevel;

	private int scaledWidthLimit; //to correct for rounding error when scaling up
	private int scaledHeightLimit; //to correct for rounding error when scaling up


	public int getAutoThresholdLevel() {
		return this.autoThresholdLevel;
	}

	public void setAutoThresholdLevel(int autoThresholdLevel) {
		this.autoThresholdLevel = autoThresholdLevel;
	}

	public long getScanId() {
		return scanId;
	}

	/**
	 * Clears crops from this CropSet.
	 *
	 */
	public void clear() {
		this.components = new HashSet<ConnectedComponentBounds>();
	}

	public boolean isFormSubmit() {
		return this.formSubmit;
	}



	public void setFormSubmit(boolean formSubmit) {
		this.formSubmit = formSubmit;
	}



	/**
	 * Constructs a CropSet object.
	 *
	 * NOTE: We use separate scale factor for width and height between thumbnail
	 * and original for precision, since they can differ slightly due to sub-sampling.
	 *
	 * Throws IllegalArgumentException if scanId null.
	 *
	 * @param slideScanId
	 * @param widthScaleFactor
	 * @param heightScaleFactor
	 * @param scaledWidthLimit
	 * @param scaledHeightLimit
	 */
	public CropSet(Long slideScanId, double widthScaleFactor, double heightScaleFactor,
			int scaledWidthLimit, int scaledHeightLimit) {

		if( slideScanId == null)
			throw new IllegalArgumentException("Slide Scan ID cannot be null!");

		this.widthScaleFactor = widthScaleFactor;
		this.heightScaleFactor = heightScaleFactor;
		this.scaledWidthLimit = scaledWidthLimit;
		this.scaledHeightLimit = scaledHeightLimit;
		this.scanId = slideScanId;
		this.components = new HashSet<ConnectedComponentBounds>();
	}



	public boolean isCropImmediately() {
		return this.cropImmediately;
	}
	public void setCropImmediately(boolean cropImmediately) {
		this.cropImmediately = cropImmediately;
	}



	public double getHeightScaleFactor() {
		return this.heightScaleFactor;
	}



	public double getWidthScaleFactor() {
		return this.widthScaleFactor;
	}



	public Set<ConnectedComponentBounds> getConnectedComponentBoundsSet() {
		return this.components;
	}


	public void addConnectedComponentBoundsList(Collection<ConnectedComponentBounds> ccBoundsList) {
		components.addAll(ccBoundsList);
	}



	public void addConnectedComponent(ConnectedComponentBounds cc) {
		components.add(cc);
	}

	public void remove(ConnectedComponentBounds cc) {
		components.remove( cc );
	}

	public void removeAll(Collection<ConnectedComponentBounds> ccs) {
		components.removeAll(ccs);
	}

	public int size() {
		return components.size();
	}

	public boolean isEmpty() {
		return components.isEmpty();
	}


	public boolean contains(ConnectedComponentBounds cc) {
		return components.contains(cc);
	}

	/**
	 * Returns data for this CropSet as NameValuePair[] array
	 *
	 * NOTE: Scaled Crop width and height may be trimmed slightly
	 * if rounding error causes a Crop to fall outside the
	 * bounds of the original DMI image.
	 *
	 * @return
	 */
	public PostData getPostData() {

		PostData postData = new PostData();

		postData.addKeyValuePair("scanId", scanId.toString());

		postData.addKeyValuePair( "cropImmediately", Boolean.toString(cropImmediately) );

		postData.addKeyValuePair( "formSubmit", Boolean.toString(true) );

		int cropIndex = 0;

		for(ConnectedComponentBounds cc : components) {

			Rectangle rect =  getScaledCrop( cc ) ;

			postData.addKeyValuePair( "crops["+cropIndex+"].x", Integer.toString(rect.x) );
			postData.addKeyValuePair( "crops["+cropIndex+"].y", Integer.toString(rect.y) );
			postData.addKeyValuePair( "crops["+cropIndex+"].cropWidth", Integer.toString(rect.width) );
			postData.addKeyValuePair( "crops["+cropIndex+"].cropHeight", Integer.toString(rect.height) );
			postData.addKeyValuePair( "crops["+cropIndex+"].scanId", Long.toString( scanId ));
			postData.addKeyValuePair( "crops["+cropIndex+"].flaggedForTiling", Boolean.toString( cc.isFlaggedForTiling() ));

			cropIndex++;
		}

		return postData;
	}


	@Override
	public String toString() {

		return getPostData().toString();

	}

	private Rectangle getScaledCrop(ConnectedComponentBounds cc) {

		Rectangle thumbRect = cc.getAsRectangle();

		int x = (int)Math.round(  (((double)thumbRect.x ) * getWidthScaleFactor() ) );
		int y = (int)Math.round(  (((double)thumbRect.y ) * getHeightScaleFactor() ) );
		int w = (int)Math.round(  (((double)thumbRect.width ) * getWidthScaleFactor() ) );
		int h = (int)Math.round(  (((double)thumbRect.height ) * getHeightScaleFactor() ) );

		//trim width/height if rounding error causes excess above max width of original dmi image
		if( x + w > scaledWidthLimit) {
			int overage = (x + w) - scaledWidthLimit;
			w = w - overage;
		}

		if( y + h > scaledHeightLimit) {
			int overage = (y + h) - scaledHeightLimit;
			h = h - overage;
		}

		return new Rectangle( x, y, w, h );
	}



}
