package uk.ac.sanger.mig.histology.applets.scancropper.services;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.histology.applets.scancropper.model.CropSet;
import uk.ac.sanger.mig.histology.applets.scancropper.model.PostData;
import uk.ac.sanger.mig.histology.views.SerializableScanView;

/**
 * Enables communication with server
 * 
 * @author ae1
 *
 */
public class HttpRequestService  {

	private MessageService messageService;
	private String baseUrl;
	private boolean requestInProgress;
	
	/**
	 * Indicates that an HTTP request is in progress. Used as a locking mechanism to
	 * prevent concurrent submissions to server. 
	 * @return
	 */
	public boolean isRequestInProgress() {
		return this.requestInProgress;
	}

	/**
	 * Locking mechanism to prevent concurrent submissions to server.
	 * @param requestInProgress
	 */
	public void setRequestInProgress(boolean requestInProgress) {
		this.requestInProgress = requestInProgress;
	}

	public void setMessageService(MessageService messageService) {
		this.messageService = messageService;
	}

	public HttpRequestService(String baseUrl) {
		this.baseUrl = baseUrl;
	}
		
	public void fetchScanList() {
	
		if( isRequestInProgress() )
			throw new RuntimeException("Cannot fetch scans when request in progress!");
		
		setRequestInProgress( true );
		
		Object scans = null;
		String queryString = baseUrl + "/list_crop_selection_scans.do";
		
		try {
			
			scans = doPost( queryString, null );		
					
		} catch(Exception e) {
			
			warnServerCommincationFailure( e );
			
		} finally {
			//so we don't freeze up controls on exception
			setRequestInProgress( false );
		}
		
		messageService.sendScansFetchedUpdate( (Collection<SerializableScanView>)scans );
	}
	
	
	public void scheduleCrops(CropSet cropSet) {
		
		if( isRequestInProgress() )
			throw new RuntimeException("Cannot schedule crops when request in progress!");
		
		setRequestInProgress( true );
		
		if( cropSet != null && !cropSet.isEmpty()) {
		
			Object response = null;
			String urlString = baseUrl + "/schedule_crop_job.do";
			Thread cropImmediatelyThread = null;	
			
			if( cropSet.isCropImmediately() ) {
				//want to free up AWT-Event thread to do IO, otherwise it blocks
				//until InputStream EOF received which is not the behaviour we want--ie
				//we want to see streaming text response.
				cropImmediatelyThread = new Thread( new SteamingResponseGobbler( urlString, cropSet ) );
				cropImmediatelyThread.start();			
				
			} else {
				//schedule job for later. do regular post
	
				try {
					
					response = doPost( urlString, cropSet.getPostData() );
					
					if( response == null )
						throw new Exception("Null response when scheduling Crop Job!");
					
				} catch(Exception e) {
					
					warnServerCommincationFailure( e );
				
				} finally {
				
					if( cropImmediatelyThread == null )
						setRequestInProgress( false );
				}
				
				messageService.sendTextareaUpdate( response.toString(), true );
				
				if( response.toString().contains("OK") ) {

					messageService.sendSubmittedUpdate( cropSet.getScanId() );
				    
				} else
					messageService.sendTextareaUpdate( "\nAborting local Scan list update!\n\n", true );				
					
			}
		} else {
			messageService.sendTextareaUpdate("\nNo crops to send! First, " +
					"load & crop a slide, then submit to crop job queue.", true);
		}	
	}

	public void reject(Long scanId, boolean badSlide) {

		if( isRequestInProgress() )
			throw new RuntimeException("Cannot reject when request in progress!");
		
		setRequestInProgress( true );
	
		Object response = null;
		String urlString = baseUrl + "/reject_scan.do";
		PostData postData = new PostData();
		postData.addKeyValuePair("id", scanId);
		postData.addKeyValuePair("badSlide", badSlide);
		
		try {

			response = doPost( urlString, postData );	
			
		} catch(Exception e) {
		
			warnServerCommincationFailure( e );
		
		} finally {
		
			setRequestInProgress( false );
		}
		
		messageService.sendTextareaUpdate( response.toString(), true );	
		
		if( response.toString().contains("OK") )
			messageService.sendRejectedUpdate( scanId );	
		else
			messageService.sendTextareaUpdate( "\nReject failed.  Aborting local Scan list update!\n\n", true );
			
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////// privates ///////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////
	private class SteamingResponseGobbler implements Runnable {
		
		private String urlString; 
		private CropSet cropSet;
			
		public SteamingResponseGobbler(String urlString, CropSet cropSet) {
			
			if( urlString == null || cropSet == null )
				throw new IllegalArgumentException("urlString and cropSet must not be null or empty!");
			
			this.urlString = urlString;
			this.cropSet = cropSet;
		}

		public void run() {
			
			boolean isJobSuccess = false;
			
			try {
				
				isJobSuccess = doStreamingPost( urlString, cropSet.getPostData() );
				
			} catch(Exception e) {
							
				warnServerCommincationFailure( e );
					
			} finally {
			
				setRequestInProgress( false );	
			}
	
			
			messageService.sendSubmittedUpdate( cropSet.getScanId() );
			
			if( !isJobSuccess )
				messageService.sendTextareaUpdate( "\nJob processing failed!\n\n", true );
		}
			
		private boolean doStreamingPost(String urlString, PostData postData) {
						
			boolean isJobSuccess = false;
			
			OutputStreamWriter writer = null;
			BufferedReader reader = null;
			
			try {
				
		        URL url = new URL(urlString);
		        URLConnection conn = url.openConnection();
		        conn.setDoOutput(true);
		        //write
		        writer = new OutputStreamWriter(conn.getOutputStream());
		        writer.write(postData.getData());
		        writer.flush();
				
				//read
		        InputStream inputStream = conn.getInputStream();
	 
				reader = new BufferedReader( new InputStreamReader( inputStream ) );
				
				String line = null;
				while((line = reader.readLine()) != null) {
					
					if(line.contains("OK")) {
												
						messageService.sendTextareaUpdate("\n" + line, true);
						
						isJobSuccess = true;
						
						Thread.sleep( 1000L );
						
					} else if(line.contains("***CropJob failed")) {
						//not exception, b/c it is specific to particular Job and
						//not failure of applet or applet/server communication.
						messageService.sendTextareaUpdate("\n" + line, true);
						isJobSuccess = false;
						break;
						
					} else if(line.contains( "<title>Session Timeout</title>" )) {
						
						throw new IOException("Session timed out.");
						
					} else {
						messageService.sendTextareaUpdate("\n" + line, true);
					}
				}	
				
			} catch (Exception e) {
				
				throw new RuntimeException(e);
			
			} finally {
				
				try {
					if(reader!= null) reader.close();	
				} catch(IOException e) {}
				try {
					if(writer!= null) writer.close();	
				} catch(IOException e) {}
			}
			
			return isJobSuccess;
		}			
		
	} //end Runnable

	private Object doPost(String urlString, PostData postData) {
		
		Object returnObject = null;
		ObjectInputStream reader = null;
		OutputStreamWriter writer = null;
		
		try {
	        URL url = new URL(urlString);
	        URLConnection conn = url.openConnection();
	        
	        if( postData != null ) {//write
		        conn.setDoOutput(true);
		        writer = new OutputStreamWriter(conn.getOutputStream());
		        writer.write( postData.getData() );
		        writer.flush();
	        }
	        
	        //read
			reader = new ObjectInputStream( conn.getInputStream() );	
			returnObject = reader.readObject();
	 			
		} catch (Exception e) {
		
			throw new RuntimeException(e);
		
		} finally {
			try {
				if(reader!= null) reader.close();	
			} catch(IOException e) {}
			try {
				if(writer != null) writer.close();
			} catch(IOException e) {}
		}
		return returnObject;
	}
	
	private void warnServerCommincationFailure(Exception e) {
		
		messageService.sendTextareaUpdate("\n\n***There's appears to be a problem communicating with server.\nCurrent Time: " +
				new Date() +
				"\nTry to login again, then retry. If that fails, contact team110dev@sanger.ac.uk", true);
		
		if( e != null ) {
			messageService.sendTextareaUpdate("\n\n******************Local Stack Trace******************\n" + e.getMessage(), true);
			StackTraceElement[] stack = e.getStackTrace();
			for(StackTraceElement elem : stack) {
				messageService.sendTextareaUpdate("\n" + elem, true);
			}
			
			e.printStackTrace();//so we have warning if applet doen't load
		}
	}
}
