package uk.ac.sanger.mig.histology.applets.scancropper.services;

import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.PixelGrabber;
import java.io.ByteArrayInputStream;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.imageio.ImageIO;

import uk.ac.sanger.mig.histology.applets.scancropper.model.ConnectedComponentBounds;
import uk.ac.sanger.mig.histology.applets.scancropper.model.CropSet;
import uk.ac.sanger.mig.histology.applets.scancropper.model.ImageModel;
import uk.ac.sanger.mig.histology.applets.scancropper.model.ImageModelCache;
import uk.ac.sanger.mig.histology.applets.scancropper.model.SegmentationConstants;
import uk.ac.sanger.mig.histology.applets.scancropper.model.ImageModel.SegmentationAlgorithmEnum;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.ConnectComponentsOperation;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.DilationOperation;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.GreyscaleOperation;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.MultipassOperation;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.NoiseFilterOperation;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.ThresholdDecision;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.ThresholdDecisionFactory;
import uk.ac.sanger.mig.histology.applets.scancropper.services.operations.ThresholdOperation;
import uk.ac.sanger.mig.histology.views.SerializableScanView;

/**
 * Performs all operaions on ImageModel
 *
 * @author ae1
 *
 */
public class ImageProcessingService implements HttpServiceListener {

	private MessageService messageService;
	private Collection<SerializableScanView> scanViews;
	private ImageModelCache imageModelCache = new ImageModelCache();
	private ImageModel currentImageModel;
	private long DEFAULT_DELAY = 50L;
	private boolean DEFAULT_TILING_FLAG = true;

	private static NumberFormat nf = NumberFormat.getNumberInstance();

	/**
	 * Attempts to auto-crop every scan currently selected in list.
	 * Any existing Crops for those scans will be cleared.
	 */
	public void makeAllSelectedCrops(Object[] currentSelectedScans) {

		if( currentSelectedScans == null )
			throw new IllegalArgumentException("Object[] currentSelectedScans cannot be null!");

		for(Object object : currentSelectedScans) {

			SerializableScanView scan = (SerializableScanView)object;

			load( scan );

			clearCurrentCrops();

			segment();

			filterMinCrops();

			setDefaultTilingMode();

			try {  Thread.sleep( DEFAULT_DELAY * 3 ); }
			catch (InterruptedException e) {}
		}
	}


	/**
	 * Loads the current ImageModel for given scan name from the ImageModelCache.
	 *
	 * @param scanId
	 */
	public void load(SerializableScanView scan)  {

		currentImageModel = imageModelCache.get( scan.getId() );

		if( currentImageModel == null )
			throw new RuntimeException("Could not find ImageModel in cache for Scan " + scan );

		messageService.sendImageUpdate( currentImageModel, 0 );
		messageService.sendCropSetUpdate( currentImageModel.getCropSet(), null, 0 );

		updateControls();

		messageService.sendTextareaUpdate("\n\n************************* Loading Scan *************************", true);

		messageService.sendTextareaUpdate("\nDMI File: " + scan.getFilePath(), true);

		messageService.sendTextareaUpdate("\nWidth x Height: " + scan.getAreaDescription()
				+ " pixels", true);

		nf.setMaximumFractionDigits( 1 );
		messageService.sendTextareaUpdate("\nBitmap Size: " + scan.getDisplayableBitmapSize(), true);

		messageService.sendTextareaUpdate("\nThumbnail: " + scan.getThumbnailAreaDescription()
				+ " pixels", true);

		messageService.sendTextareaUpdate("\nStatus: " + scan.getStatusDictName(), true);

		nf.setMaximumFractionDigits( 5 );
		messageService.sendTextareaUpdate("\nWidth Scale Factor: " +
				nf.format( currentImageModel.getCropSet().getWidthScaleFactor() ), true);

		messageService.sendTextareaUpdate("\nHeight Scale Factor: " +
					nf.format( currentImageModel.getCropSet().getHeightScaleFactor() ), true);

		if( !scan.isOrphan() ) {
			messageService.sendTextareaUpdate("\nStaining: " + scan.getStaining(), true);
			messageService.sendTextareaUpdate("\nBlock: " +  scan.getBlockDescription() + " (#" + scan.getBlockName()+ ")", true);
			messageService.sendTissueLabelUpdate( scan.getBlockDescription() );
			messageService.sendTextareaUpdate("\nMouse: " + scan.getMouseBarcode(), true);
			messageService.sendTextareaUpdate("\nGenotype: " + scan.getGenotype(), true);

		} else {
			messageService.sendTextareaUpdate("\nNO BARCODED SLIDE ASSOCIATED WITH THIS SCAN!", true);
		}

		messageService.sendTextareaUpdate("\n********************************************************************\n", true);

	}

	public boolean isScanSelected() {
		return ( currentImageModel != null );
	}

	public void updateControls() {
		messageService.sendCropNowUpdate( currentImageModel.getCropSet().isCropImmediately() );
		messageService.sendDialationUpdate( currentImageModel.getDilationCount() );
	}

	public void drawNewCropRectangle(Rectangle newCropRectangle) {
		messageService.sendCropSetUpdate( currentImageModel.getCropSet(), newCropRectangle, 0 );
	}

	public void commitManualCropRectangle(Rectangle newCropRectangle) {

		CropSet cropSet = currentImageModel.getCropSet();

		if( cropSet != null) {

			//trim
			newCropRectangle = newCropRectangle.createIntersection(currentImageModel.getAsRectangle()).getBounds();

			ConnectedComponentBounds cc = new ConnectedComponentBounds(-1);
			cc.minX = newCropRectangle.x;
			cc.minY = newCropRectangle.y;
			cc.maxX = newCropRectangle.x + newCropRectangle.width;
			cc.maxY = newCropRectangle.y + newCropRectangle.height;
			cc.flaggedForTiling = DEFAULT_TILING_FLAG; //default
			cropSet.addConnectedComponent(cc);

			messageService.sendCropSetUpdate( cropSet, null, 0 );
			messageService.sendTextareaUpdate("\nAdded new Crop " + cc, true);
			messageService.sendTextareaUpdate(" Total Crops: " + cropSet.size(), true);
		}
	}

	public void setCropImmediately( boolean cropImmediately ) {

		CropSet cropSet = currentImageModel.getCropSet();

		if( cropSet != null ) {

			cropSet.setCropImmediately( cropImmediately );
		}

	}


	public void toggleTilingFlags(Point point) {

		CropSet cropSet = currentImageModel.getCropSet();

		if(point.x < 0 || point.x > currentImageModel.getWidth() ||
		   point.y < 0 || point.y > currentImageModel.getHeight() )
			return;


		Set<ConnectedComponentBounds> ccBoundsSet = cropSet.getConnectedComponentBoundsSet();

		for(ConnectedComponentBounds ccBounds : ccBoundsSet) {

			if(ccBounds.contains( point )) {

				ccBounds.setFlaggedForTiling( !ccBounds.isFlaggedForTiling() );
			}
		}

		messageService.sendCropSetUpdate( cropSet, null, 0 );
	}


	public void removeCropsContainng(Point point) {

		CropSet cropSet = currentImageModel.getCropSet();

		if(point.x < 0 || point.x > currentImageModel.getWidth() ||
		   point.y < 0 || point.y > currentImageModel.getHeight() )
			return;

		Collection<ConnectedComponentBounds> removeList = new ArrayList<ConnectedComponentBounds>();

		Set<ConnectedComponentBounds> ccBoundsSet = cropSet.getConnectedComponentBoundsSet();

		for(ConnectedComponentBounds ccBounds : ccBoundsSet) {

			if(ccBounds.contains( point )) {
				//messageService.sendTextareaUpdate("\nRemoving Region " + ccBounds, true);
				removeList.add( ccBounds );
			}
		}

		ccBoundsSet.removeAll( removeList );

		messageService.sendCropSetUpdate( cropSet, null, 0 );
		//messageService.sendTextareaUpdate(", Total Regions: " + cropSet.size(), true);

	}

	public Long getCurrentScanId() {
		return (currentImageModel != null) ?  currentImageModel.getCropSet().getScanId() : null;
	}

	public CropSet getCurrentCropSet() {
		return (currentImageModel != null) ? currentImageModel.getCropSet() : null;

	}

	/**
	 * Caches scans as new ImageModels if they aren't already cached.
	 * @param scans
	 */
	public void cacheNewImageModels(Collection<SerializableScanView> scans) {

		if( scans == null )
			throw new IllegalArgumentException("scans canot be null!");

		for(SerializableScanView scan : scans) {

			if( scan.getThumbnail() == null )
				throw new RuntimeException("Scan must have a thumbnail BinaryFile object attached, so piss off!");

			if( !imageModelCache.isCached( scan.getId() )) {

				ImageModel imageModel = deriveImageModel( scan );
				imageModelCache.put(scan.getId(), imageModel);
			}
		}
	}

	public void clearCurrentCrops() {
		if( currentImageModel != null &&
				currentImageModel.getCropSet() != null &&
				!currentImageModel.getCropSet().isEmpty()) {

			currentImageModel.clearCrops();
			messageService.sendCropSetUpdate( currentImageModel.getCropSet(), null, 0 );
		}
	}

	public SegmentationAlgorithmEnum getCurrentAlgorithm() {

		return currentImageModel.getAlgorithm();

	}


	public Integer getCurrentDilationCount() {
		return currentImageModel.getDilationCount();
	}



	public void setCurrentDilationCount(Integer dilationCount) {
		if( currentImageModel != null )
			currentImageModel.setDilationCount(dilationCount);
	}



	public void setMessageService(MessageService messageService) {
		this.messageService = messageService;
	}

	public void notifySubmitted(Long scanId) {

		removeScan( scanId );

		updateScanList( scanViews, 0 );
	}

	public void notifyRejected(Long scanId) {

		removeScan( scanId );

		updateScanList( scanViews, 0 );

	}

	private void updateScanList(Collection<SerializableScanView> scans, int defaultSelectedIndex) {

		messageService.sendScanListChangeUpdate( scans, defaultSelectedIndex );

		Iterator<SerializableScanView> it = scans.iterator();

		if(!scans.isEmpty() && defaultSelectedIndex > -1) {

			int i = 0;

			SerializableScanView scan = it.next();

			while( i < defaultSelectedIndex ) {

				scan = it.next();
				i++;
			}

			load( scan );

		}

	}


	public void notifyScansFetched(Collection<SerializableScanView> scans) {

		setScans( scans );

		if( scans != null )
			cacheNewImageModels( scans );
		else
			messageService.sendTextareaUpdate("\nWARNING! Scan fetch returned null!", true);

		updateScanList( scans, -1 );
		int scanCount = ( scans != null ) ? scans.size() : 0;
		messageService.sendTextareaUpdate("\n" + scanCount + " Scans Loaded.", true);

	}


	/////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////// privates ///////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * Performs auto-crop on the currently loaded scan.
	 *
	 * @throws IllegalStateException if image is not loaded.
	 */
	private synchronized void segment() throws IllegalStateException {

		if( currentImageModel == null )
			throw new IllegalStateException("ImageModel cannot be null during segmentation.");

		filterNoise( 	   currentImageModel );
		threshold( 		   currentImageModel );
		dilate( 		   currentImageModel );
		connectComponents( currentImageModel );
		restorePixels();

		messageService.sendTextareaUpdate("\nScan successfully auto-cropped! ! !", true);
	}

	private void filterMinCrops() {

		CropSet cropSet = currentImageModel.getCropSet();

		double thumbArea = currentImageModel.getWidth() * currentImageModel.getHeight();

		Set<ConnectedComponentBounds> boundSet = cropSet.getConnectedComponentBoundsSet();

		Collection<ConnectedComponentBounds> removeBoundList
				= new ArrayList<ConnectedComponentBounds>();

		for( ConnectedComponentBounds bounds : boundSet ) {

			Rectangle rectangle = bounds.getAsRectangle();

			double cropArea = rectangle.getWidth() * rectangle.getHeight();

			if( (cropArea / thumbArea) <
						(SegmentationConstants.MINIMUM_AUTO_CROP_SIZE_FILTER.getPercentValue()/10.0d)) {

				removeBoundList.add( bounds );

				//System.out.println((cropArea / thumbArea));
			}


		}

		cropSet.removeAll( removeBoundList );

		messageService.sendCropSetUpdate( cropSet, null, DEFAULT_DELAY );

		messageService.sendTextareaUpdate("\n" + removeBoundList.size()
				+ " crops filtered out because less than " +
				SegmentationConstants.MINIMUM_AUTO_CROP_SIZE_FILTER.getIntValue() / 10.0d +
				"% of whole scan.", true);

	}

	private void setScans(Collection<SerializableScanView> scans) {
		this.scanViews = scans;
	}

	private void removeScan(Long scanId) {

		SerializableScanView removeScan = null;
		for(SerializableScanView scan : scanViews) {
			if( scanId.equals( scan.getId() ) ) {
				removeScan = scan;
				break;
			}
		}

		scanViews.remove( removeScan );
		imageModelCache.remove( scanId );
		messageService.sendTissueLabelUpdate("");
		currentImageModel = null;
		messageService.sendDialationUpdate( SegmentationConstants.DIALATION_COUNT_DEFAULT.getIntValue());
		messageService.sendCropNowUpdate( false );
		messageService.sendImageUpdate( null, 0 );
		messageService.sendCropSetUpdate( null, null, 0 );

	}

	private SerializableScanView getScan(Long scanId) {

		for(SerializableScanView scanView : scanViews) {

			if( scanView.getId().equals(scanId) )
				return scanView;
		}

		return null;
	}

	private void threshold(ImageModel model) {
		ThresholdOperation op = new ThresholdOperation();
		op.setBackgroundColour(SegmentationConstants.BACKGROUND_MASK.getIntValue());
		op.setForegroundColour(SegmentationConstants.FOREGROUND_MASK.getIntValue());

		SerializableScanView scanView
		   = getScan(  model.getCropSet().getScanId() );

		ThresholdDecision thresholdDecision
			= ThresholdDecisionFactory.createThresholdDecision(
					scanView.getStaining(), null );

		op.setThresholdDecision( thresholdDecision );

		messageService.sendTextareaUpdate("\nThresh Strategy: " +
				thresholdDecision.getName(), true);

		op.process( model );

		messageService.sendTextareaUpdate("\nThresh Found (0-255): " +
				model.getCropSet().getAutoThresholdLevel(), true);

		messageService.sendImageUpdate( model, DEFAULT_DELAY );
	}

	/*
	 * Eval BG pixels w/3x3 mask, if any surrounding FG pixels, set BG to FG.
	 *
	 */
	private void dilate(ImageModel model) {
		DilationOperation op = new DilationOperation();
		op.setBackgroundColour(SegmentationConstants.BACKGROUND_MASK.getIntValue());
		op.setForegroundColour(SegmentationConstants.FOREGROUND_MASK.getIntValue());

		MultipassOperation repOp = new MultipassOperation();
		repOp.setRepeatCount( getCurrentDilationCount() );
		repOp.setOperator(op);
		repOp.process( model );

		messageService.sendImageUpdate( model, DEFAULT_DELAY );
		//messageService.sendTextareaUpdate("\nDialation complete..." + "Dialations = "
		//		+ repOp.getRepeatCount(), true);

	}

	private void setDefaultTilingMode() {

		CropSet cropSet = currentImageModel.getCropSet();

		Set<ConnectedComponentBounds> ccBoundsSet
			= cropSet.getConnectedComponentBoundsSet();

		for(ConnectedComponentBounds cc : ccBoundsSet) {

			cc.flaggedForTiling = DEFAULT_TILING_FLAG;
		}

		messageService.sendCropSetUpdate( cropSet, null, DEFAULT_DELAY );
	}

	private void filterNoise(ImageModel imageModel) {
		NoiseFilterOperation op = new NoiseFilterOperation();

		for(int pass = 0; pass < SegmentationConstants.NOISE_FILTER_PASSES.getIntValue(); pass++) {
			op.process( imageModel );
		}

		messageService.sendImageUpdate( imageModel, DEFAULT_DELAY );
		//messageService.sendTextareaUpdate("\nMean filter complete...", true);
	}

	/*
	 * 8-pixel connectivity and Quick-Union algorithm (quadratic time?) to merge
	 * same-set regions
	 */
	private void connectComponents(ImageModel model) {

		CropSet cropSet = currentImageModel.getCropSet();

		ConnectComponentsOperation op = new ConnectComponentsOperation();
		op.setBackgroundColour(SegmentationConstants.BACKGROUND_MASK.getIntValue());
		op.setForegroundColour(SegmentationConstants.FOREGROUND_MASK.getIntValue());
		op.setDilationCount( currentImageModel.getDilationCount() );
		op.setLabelColourSeed(42L);

		Map<Integer, ConnectedComponentBounds> components
			= (Map<Integer, ConnectedComponentBounds>)op.connectPixels(currentImageModel);

		//hash component list b/c we want to guarantee unique set of crops in
		//externally exoposed CropSet object for any given S
		cropSet.addConnectedComponentBoundsList( components.values() );

		messageService.sendImageUpdate( model, DEFAULT_DELAY );

		messageService.sendCropSetUpdate( cropSet, null, DEFAULT_DELAY );

		messageService.sendTextareaUpdate("\nConnecting Components..."
				+ "Found " + cropSet.size(), true);

	}

	private void restorePixels() {
		currentImageModel.restorePixels();
		messageService.sendImageUpdate( currentImageModel, 0 );
	}

	private ImageModel deriveImageModel(SerializableScanView scan) {
		//from byte[] imageBytes to Image to ImageModel
		byte[] imageBytes = scan.getThumbnail();

		Image thumbnail = null;

		try {
			thumbnail = ImageIO.read ( new ByteArrayInputStream( imageBytes ) );
		} catch(Exception e) {throw new RuntimeException(e);}

		int thumbWidth = scan.getThumbWidth();
		int thumbHeight = scan.getThumbHeight();

		int pixelCount = (thumbWidth > 0 && thumbHeight > 0) ? thumbWidth * thumbHeight : 0;

		int[] pixels = new int[pixelCount];

		PixelGrabber pixelGrabber = new PixelGrabber(thumbnail, 0, 0, thumbWidth, thumbHeight, pixels, 0, thumbWidth); // orig

		try { pixelGrabber.grabPixels(); }
		catch (InterruptedException e) {throw new RuntimeException(e);}

		CropSet cropSet = new CropSet(
				scan.getId(),
				scan.getWidthScaleFactor(),
				scan.getHeightScaleFactor(),
				scan.getWidth(),
				scan.getHeight() );

		return new ImageModel(pixels, thumbWidth, thumbHeight, cropSet);

	}






}
