package uk.ac.sanger.mig.histology.applets.scancropper.services.operations;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import uk.ac.sanger.mig.histology.applets.scancropper.model.ConnectedComponentBounds;
import uk.ac.sanger.mig.histology.applets.scancropper.model.EquivalencyPair;
import uk.ac.sanger.mig.histology.applets.scancropper.model.ImageModel;
import uk.ac.sanger.mig.histology.applets.scancropper.model.SegmentationConstants;

public class ConnectComponentsOperation implements ImageOperation {

	private int foregroundColour;
	private int backgroundColour;
	private long labelColourSeed = 0;
	private int dilationCount;

	public void process(ImageModel imageModel) {
		this.connectPixels(imageModel);
	}

	/**
	 * Returns a ConnectedComponentMap representing a set of
	 * connected pixel regions.
	 *
	 * For algorithm @see http://homepages.inf.ed.ac.uk/rbf/HIPR2/label.htm
	 * @param imageModel
	 * @return
	 */
	public Map<Integer, ConnectedComponentBounds> connectPixels(ImageModel imageModel) {

		final int[] pixels = imageModel.getPixels();
		final int width = imageModel.getWidth();
		final int height = imageModel.getHeight();

		int label = 0;
		Set<EquivalencyPair> equivPairSet = new HashSet<EquivalencyPair>();

		final int DMETRIX_EDGE_STRIPS_CORRECTION = SegmentationConstants.DMETRIX_EDGE_STRIPS_CORRECTION.getIntValue();

		//need to ignore pixels further inward to offset dilation of error foreground
		int westEdgeCorrection = DMETRIX_EDGE_STRIPS_CORRECTION + dilationCount;
		int eastEdgeCorrection = width - (DMETRIX_EDGE_STRIPS_CORRECTION + dilationCount);
		int southEdgeCorrection = height - (DMETRIX_EDGE_STRIPS_CORRECTION + dilationCount);

		for (int y = 1; y < southEdgeCorrection; y++) {
			for (int x = westEdgeCorrection ; x < eastEdgeCorrection; x++) {

				if (pixels[y * width + x] == foregroundColour) {

					int[] scanner = new int[4];

					// these are all or neighbors we've already scanned and
					// labelled any that are foreground. ignore edges, assume BG and correct later
					scanner[0] = (x == westEdgeCorrection) ? backgroundColour : pixels[(y) * width + (x - 1)];
					scanner[1] =  (x == westEdgeCorrection || y == 1) ? backgroundColour : pixels[(y - 1) * width + (x - 1)];
					scanner[2] =  (y == 1) ? backgroundColour : pixels[(y - 1) * width + (x)];
					scanner[3] =  (y == 1) ? backgroundColour : pixels[(y - 1) * width + (x + 1)];

					boolean noNeighbors = true;
					for (int i = 0; i < scanner.length; i++) {

						if (scanner[i] != backgroundColour && scanner[i] != foregroundColour) {
							// concerned with previously foreground labeled pix
							// (note: x/y == 1 avoided to skirt null pointer)

							if (pixels[y * width + x] == foregroundColour) {

								pixels[y * width + x] = scanner[i];// if pix hasn't been set, set it

							} else if (pixels[y * width + x] != scanner[i]) {
								// record new equiv. pair found for any
								// neighbors w/diff. label---
								equivPairSet.add(new EquivalencyPair(pixels[y
										* width + x], scanner[i]));
							}

							noNeighbors = false;
						}
					}

					if (noNeighbors) { // otherwise it's a new FG element
						// requiring a new label
						pixels[y * width + x] = label++;
					}
				}
			}
		}

		// Merge connected components
		Map<Integer, ConnectedComponentBounds> componentMap
				= new HashMap<Integer, ConnectedComponentBounds>();

		//index is unique id for unioned component
		int[] connectedComponents = unionFind(equivPairSet, label);

		Random r = new Random(labelColourSeed);

		for (int i = 0; i < connectedComponents.length; i++) {
			// plop rand color in for root component labels for visualizing
			if (connectedComponents[i] == -1) {

				int colorLabel = ((int) r.nextInt() & 0xffffffff);

				componentMap.put(
						new Integer(i),
						new ConnectedComponentBounds(colorLabel));
			}
		}

		for (int y = 1; y < height; y++) {
			for (int x = 1; x < width - 1; x++) {

				// re-label same-set regions by "root" component class
				if (pixels[y * width + x] != backgroundColour
						&& pixels[y * width + x] != foregroundColour) {

					int componentId = root(connectedComponents, pixels[y* width + x]); // find
					// root
					ConnectedComponentBounds cc = componentMap.get(componentId);
					pixels[y * width + x] = cc.label; // label same component pixel to root label

					// ...and derive crop box while we're here
					if (x < cc.minX)
						cc.minX = x;
					if (y < cc.minY)
						cc.minY = y;

					if (x > cc.maxX)
						cc.maxX = x;
					if (y > cc.maxY)
						cc.maxY = y;
				}
			}
		}

		postProcess( componentMap, westEdgeCorrection, eastEdgeCorrection, southEdgeCorrection, width, height );

		return componentMap;
	}

	/*
	 * Compensate for edge strips when sections touching left edge of slide
	 */
	private void postProcess( Map<Integer, ConnectedComponentBounds> componentMap,
			int westEdgeCorrection, int eastEdgeCorrection, int southEdgeCorrection, int width, int height ) {

		Set<Integer> keys = componentMap.keySet();
		for(Integer key : keys) {
			ConnectedComponentBounds cc = componentMap.get(key);

			if(cc.minX < westEdgeCorrection + 1)
				cc.minX = 0;

			if(cc.maxX >= eastEdgeCorrection - 1)
				cc.maxX = width - 1;

			//correct for dialation kernel avoiding edges
			if(cc.minY < 2) //2 padding
				cc.minY = 0;

			if(cc.maxY >= southEdgeCorrection - 1)
				cc.maxY = height;

		}


	}

	private int[] unionFind(Set<EquivalencyPair> equivList, int equivListLength) {

		// initialize forest of singleton nodes with no parent (shown by -1)
		int[] id = new int[equivListLength];
		for (int i = 0; i < equivListLength; i++) {
			id[i] = -1;
		}

		// merge elem to elem in pair setting in equivalencyClasses
		for (EquivalencyPair p : equivList) {
			unite(id, p.a, p.b);
		}

		return id;
	}

	private int root(int id[], int x) {
		while (id[x] != -1)
			x = id[x];
		return x;
	}

	private void unite(int id[], int a, int b) {
		int p = root(id, a), q = root(id, b);
		if (p == q)
			return;//if already in same set, return
		id[p] = q; //make root of q the new parent of (root of) p
	}

	public int getBackgroundColour() {
		return backgroundColour;
	}

	public void setBackgroundColour(int backgroundColour) {
		this.backgroundColour = backgroundColour;
	}

	public int getForegroundColour() {
		return foregroundColour;
	}

	public void setForegroundColour(int foregroundColour) {
		this.foregroundColour = foregroundColour;
	}

	public long getLabelColourSeed() {
		return labelColourSeed;
	}

	public void setLabelColourSeed(long labelColourSeed) {
		this.labelColourSeed = labelColourSeed;
	}



	public void setDilationCount(int dilationCount) {
		this.dilationCount = dilationCount;
	}


}