package uk.ac.sanger.mig.histology.applets.scancropper.services.operations;

import uk.ac.sanger.mig.histology.applets.scancropper.model.ImageModel;

public class ThresholdOperation implements ImageOperation {

	private int foregroundColour;
	private int backgroundColour;
	private ThresholdDecision thresholdDecision;

	public void process(ImageModel imageModel) {

		int[] pixels = imageModel.getPixels();
		int width = imageModel.getWidth();
		int height = imageModel.getHeight();

		int maxInterclassVarianceThreshold = getMaxInterclassVarianceThreshold(imageModel);

		imageModel.getCropSet().setAutoThresholdLevel( maxInterclassVarianceThreshold );

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {

				boolean isFG = thresholdDecision.isForeground(
										pixels,
										( y * width + x ),
										maxInterclassVarianceThreshold );

				pixels[y * width + x] = ( isFG ) ? foregroundColour	 : backgroundColour;

			}
		}
	}

	/*
	 *
		In Otsu's method we exhaustively search for the threshold that minimizes the
		intra-class variance, defined as a weighted sum of variances of the two classes:

		Weights are the probabilities of the two classes separated by a threshold t and
		variances of these classes.

		Otsu shows that minimizing the intra-class variance is the same as maximizing
		inter-class variance, which is expressed in terms of class probabilities (p)
		and class means (m) which in turn can be updated iteratively.

		This idea yields an effective algorithm:

		1. Compute histogram and probabilities of each intensity level
		2. Set up initial p and m
		3. Step through all possible thresholds maximum intensity
			a. Update p and m
			b. Compute inter-class variance: p1(t)*p2(t)*[m1(t) - m2(t)]^2
		4. Desired threshold corresponds to the maximum
	 *
	 */
	public int getMaxInterclassVarianceThreshold(ImageModel imageModel) {

		int[] pixels = imageModel.getPixels();
		int width = imageModel.getWidth();
		int height = imageModel.getHeight();

		int[] hist = new int[256];

		//build instensity histogram
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {

				int pixel = pixels[y * width + x];
				int red = (pixel >> 16) & 0xff;
				int green = (pixel >> 8) & 0xff;
				int blue = (pixel) & 0xff;

				int intensity8bit =  (int)Math.round(((double)(red + green + blue))/(3.0d));

				hist[ intensity8bit ]++;
			}
		}

		int totalPixels = width * height;

		double maxInterClassVariance = 0.0d;
		int maxInterClassVarianceThresh = 0;

		for(int trialThreshold = 1; trialThreshold < hist.length; trialThreshold++) {

			double probC1 = 0.0, meanC1 = 0.0, probC2 = 0.0, meanC2 = 0.0;
			int totC1 = 0, totC2 = 0;

			int totBinWeightC1 = 0, totBinWeightC2 = 0;

			for(int c1 = 0; c1 < trialThreshold; c1++) {

				totC1 = totC1 + hist[c1];
				totBinWeightC1 = totBinWeightC1 + ( c1 * hist[ c1 ] );

			}

			probC1 = totC1 / (double)totalPixels;
			meanC1 = totBinWeightC1 / (double)totC1;

			for(int c2 = trialThreshold; c2 < hist.length; c2++) {

				totC2 = totC2 + hist[c2];
				totBinWeightC2 = totBinWeightC2 + ( c2 * hist[ c2 ] );
			}

			probC2 =  1 - probC1;
			meanC2 = totBinWeightC2 / (double)totC2;

			double currentInterClassVariance
				= probC1 * probC2 * (Math.pow( ( meanC1 - meanC2 )  , 2));

			if( currentInterClassVariance > maxInterClassVariance) {

				maxInterClassVariance = currentInterClassVariance;

				maxInterClassVarianceThresh = trialThreshold;
			}
		}

		return maxInterClassVarianceThresh;
	}

	/*
	 * 	K-Means:
	 *  - Choose the number of clusters, k.
		- Randomly generate k clusters and determine the cluster centers,
		  or directly generate k random points as cluster centers.
		- Assign each point to the nearest cluster center.
		- Recompute the new cluster centers.
		- Repeat the two previous steps until some convergence criterion is
		  met (usually that the assignment hasn't changed).

	public void processKmeans(ImageModel imageModel) {
		int[] pixels = imageModel.getPixels();

		int[] segmentedPixels = new int[pixels.length];

		double[] fgBgCentroids = initCentroids();
		boolean stillReassigning = false;

		do {



			stillReassigning = false;

			for (int i = 0; i < pixels.length; i++) {

					int pixel = pixels[i];
					int red = (pixel >> 16) & 0xff;
					int green = (pixel >> 8) & 0xff;
					int blue = (pixel) & 0xff;

					double distanceToFgCentroid = Math.sqrt( Math.pow(red - fgBgCentroids[0], 2)
							+ Math.pow(green - fgBgCentroids[1], 2) + Math.pow(blue - fgBgCentroids[2], 2) );

					double distanceToBgCentroid = Math.sqrt( Math.pow(red - fgBgCentroids[3], 2)
							+ Math.pow(green - fgBgCentroids[4], 2) + Math.pow(blue - fgBgCentroids[5], 2) );

					int startingPixelCluster = segmentedPixels[i]; //need to check if we reassign

					segmentedPixels[i] = ( distanceToFgCentroid < distanceToBgCentroid) ?
							foregroundColour : backgroundColour;

					//as long as just one reassignment/change form fb/bg, we keep iterating
					if( startingPixelCluster != segmentedPixels[i] ) {
						stillReassigning = true;

					}
			}

			if( stillReassigning )
				fgBgCentroids = calculateCentroids( imageModel, segmentedPixels );

		} while( stillReassigning );

	    imageModel.setPixels( segmentedPixels );
	}


	private double[] calculateCentroids(ImageModel imageModel, int[] segmentedPixels) {
		double[] rgbFgRgbBg = new double[6];
		long totFgR = 0, totFgG = 0, totFgB = 0, totBgR = 0, totBgG = 0, totBgB = 0;

		int fgPixCount = 0;

		int[] pixels = imageModel.getPixels();

		System.out.println("new calc");

		for (int i = 0; i < pixels.length; i++) {

			if( segmentedPixels[i] == foregroundColour ) {

				fgPixCount++;
				int pixel = pixels[i];
				totFgR = totFgR + ((pixel >> 16) & 0xff);
				totFgG = totFgG + ((pixel >> 8) & 0xff);
				totFgB = totFgB + ((pixel) & 0xff);

				if(i % 200 == 0)
					System.out.println(totFgR);

			} else {

				int pixel = pixels[i];
				totBgR = totBgR + ((pixel >> 16) & 0xff);
				totBgG = totBgG + ((pixel >> 8) & 0xff);
				totBgB = totBgB + ((pixel) & 0xff);
			}
		}

		rgbFgRgbBg[0] = ((double)totFgR) / ((double)fgPixCount);
		rgbFgRgbBg[1] = ((double)totFgG) / ((double)fgPixCount);
		rgbFgRgbBg[2] = ((double)totFgB) / ((double)fgPixCount);
		rgbFgRgbBg[3] = ((double)totBgR) / ((double)(pixels.length - fgPixCount));
		rgbFgRgbBg[4] = ((double)totBgG) / ((double)(pixels.length - fgPixCount));
		rgbFgRgbBg[5] = ((double)totBgB) / ((double)(pixels.length - fgPixCount));

		System.out.println("------centroids-----------");
		for(int i = 0; i < rgbFgRgbBg.length; i++)
			System.out.println(rgbFgRgbBg[i]);

		return rgbFgRgbBg;
	}
	 */

	/*
	 * Start with best guess what's FG vs. BG for initial centroid points

	private double[] initCentroids() {

		double[] rgbFgRgbBg = new double[6];

		rgbFgRgbBg[0] = 0;
		rgbFgRgbBg[1] = 0;
		rgbFgRgbBg[2] = 0;

		rgbFgRgbBg[3] = 255;
		rgbFgRgbBg[4] = 255;
		rgbFgRgbBg[5] = 255;

		return rgbFgRgbBg;
	}
	 */
	public int getBackgroundColour() {
		return backgroundColour;
	}

	public void setBackgroundColour(int backgroundColour) {
		this.backgroundColour = backgroundColour;
	}

	public int getForegroundColour() {
		return foregroundColour;
	}

	public void setForegroundColour(int foregroundColour) {
		this.foregroundColour = foregroundColour;
	}

	public void setThresholdDecision(ThresholdDecision thresholdDecision) {
		this.thresholdDecision = thresholdDecision;
	}




}
