package uk.ac.sanger.mig.histology.controllers;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DataObjectIdComparator;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.histology.forms.BatchProcessingForm;
import uk.ac.sanger.mig.histology.services.BlockService;
import uk.ac.sanger.mig.histology.services.BlockService.CassettePrinter;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.CohortService;

public class BatchProcessingFormController extends SimpleFormController {
  private CohortService cohortService;
  private BlockService blockService;
  private DictionaryService dictionaryService;
  private ComponentMapService componentMapService;

  @Override
  protected Object formBackingObject(HttpServletRequest request) throws Exception {
    BatchProcessingForm form = (BatchProcessingForm)createCommand();

    Long batchId =  ServletRequestUtils.getRequiredLongParameter(request, "id");
    String reportPrintingMessage = ServletRequestUtils.getStringParameter(request, "oraclePrinting");

    form.setReportPrintingMessage(reportPrintingMessage);


    // Should be cohort -> mouse -> blocks (-> block types) -> slides
    ComponentMap batchComponents = componentMapService.getComponentMap("miceToBlocksForCohort");

    Cohort batch = cohortService.fetchById(batchId, batchComponents);

    // order the mice by ID
    List<Mouse> mice = new ArrayList<Mouse>(batch.getMice());
    Collections.sort(mice, new DataObjectIdComparator());
    batch.setMice(mice);

    form.setBatch(batch);

    form.fillInCassetteTypesFromBlocks();
    form.setProgrammeTypes(dictionaryService.fetchArray(DictionaryEnum.HTL_PROGRAM_DICT, true));

    /* Page looks nicer if we have some cassette types,
     * but the fill in will provide any if the mice have blocks
     */
    if (form.getCassetteTypes().length < 1) {
      form.setCassetteTypes(dictionaryService.fetchArray(DictionaryEnum.HTL_CASSETTE_TYPE_DICT, true));
    }

    form.buildCheckboxes(batch.getMice(), form.getCassetteTypes());

    return form;
  }

  @Override
  protected Map referenceData(HttpServletRequest request) throws Exception {
  	Map<String,Object> model = new HashMap<String,Object>();

  	Set<CassettePrinter> printers = new TreeSet<CassettePrinter>();
	  for (CassettePrinter cp : CassettePrinter.values()) {
	  	printers.add(cp);
	  }

	  model.put("cassettePrinters",printers);

	  String printErrorMessage = ServletRequestUtils.getStringParameter(request, "printingMessage");
	  model.put("printMessage", (printErrorMessage != null)? printErrorMessage:" ");

	  return model;
  }
  @Override
  protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
    BatchProcessingForm form = (BatchProcessingForm)command;

    // form method gets the blocks suitable for update, and applies the changes in the form to them
    for(Block block:form.getBlocksOfCheckedMiceForUpdate()){
    	blockService.store(block,RequestUtils.getUser(request));
    }

    return new ModelAndView(getSuccessView()+form.getBatch().getId());
  }


  public void setBlockService(BlockService blockService) {
    this.blockService = blockService;
  }


  public void setCohortService(CohortService cohortService) {
    this.cohortService = cohortService;
  }


  public void setComponentMapService(ComponentMapService componentMapService) {
    this.componentMapService = componentMapService;
  }


  public void setDictionaryService(DictionaryService dictionaryService) {
    this.dictionaryService = dictionaryService;
  }

  @Override
  protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
    binder.registerCustomEditor(Long.class, new CustomNumberEditor(Long.class, true));
    binder.registerCustomEditor(Date.class, new CustomDateEditor(new SimpleDateFormat("dd-MMM-yyyy HH:mm"), true));
  }


}
