package uk.ac.sanger.mig.histology.controllers;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.histology.BlockType;
import uk.ac.sanger.mig.histology.forms.CassetteForm;
import uk.ac.sanger.mig.histology.services.BlockService;
import uk.ac.sanger.mig.histology.services.BlockTypeService;
import uk.ac.sanger.mig.histology.services.BlockService.CassettePrinter;
import uk.ac.sanger.mig.histology.views.BlockListView;
import uk.ac.sanger.mig.histology.views.BlockViewComparator;
import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class CassetteFormController extends AbstractCancellableFormController {

	private BlockService blockService;
	private MouseService mouseService;
	private String errorView;
	private String mainView;
	private BlockTypeService blockTypeService;
	private ComponentMapService componentMapService;

	public void setComponentMapService(ComponentMapService componentMapService) {
    this.componentMapService = componentMapService;
  }

  public void setBlockTypeService(BlockTypeService blockTypeService) {
		this.blockTypeService = blockTypeService;
	}

	public String getErrorView() {
		return errorView;
	}

	public void setErrorView(String errorView) {
		this.errorView = errorView;
	}

	public void setBlockService(BlockService blockService) {
		this.blockService = blockService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public String getMainView() {
		return mainView;
	}

	public void setMainView(String mainView) {
		this.mainView = mainView;
	}

	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		binder.registerCustomEditor(Long.class, new CustomNumberEditor(Long.class, true));
		binder.registerCustomEditor(Double.class, new CustomNumberEditor(Integer.class, true));
		binder.registerCustomEditor(Integer.class, new CustomNumberEditor(Double.class, true));
		binder.registerCustomEditor(Date.class, new CustomDateEditor(new SimpleDateFormat("dd-MMM-yyyy"), true));
	}
	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		CassetteForm cassetteForm = (CassetteForm)createCommand();
		Long mouseId = ServletRequestUtils.getLongParameter(httpServletRequest, "mouseId");
		Long cohortId = ServletRequestUtils.getLongParameter(httpServletRequest, "cohortId");

		//going to the cassettes display page without scan the barcode
		if(mouseId!=null){
			cassetteForm.setMouse(mouseService.fetchById(
			    mouseId,
			    componentMapService.getComponentMap("mouseToFixation")
			));
			cassetteForm.setMouseId(mouseId);
			cassetteForm.setErrorMessage("");//this is the error message for invalid barcode
			fetchExistingBlocksByMouseId(mouseId, cassetteForm);
		}
		cassetteForm.setCohortId(cohortId);
		return cassetteForm;

	}
	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		boolean formChange = ServletRequestUtils.getBooleanParameter(request, "formChanged", false);
		boolean newBlockRequested = ServletRequestUtils.getBooleanParameter(request, "newBlockRequested", false);
		boolean saveCassettes = ServletRequestUtils.getBooleanParameter(request, "saveCassettes", false);

		if( formChange || newBlockRequested || saveCassettes)
			return true;
		else
			return super.isFormChangeRequest( request );
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response,
			Object command, BindException bindEx) throws Exception {

		CassetteForm cassetteForm = (CassetteForm)command;
		if(cassetteForm.getDeleteComments() != null && cassetteForm.getDeleteExtraBlockTypeId() != null){
			deleteExtraCassettes(cassetteForm);
		}else if(cassetteForm.isFormChanged()){
			doValidationOnBarcode(cassetteForm);
		}else if(cassetteForm.isNewBlockRequested()){
			addNewBlock(cassetteForm);
		}else if(cassetteForm.isSaveCassettes()){
			saveCassettes(cassetteForm,  RequestUtils.getUser(request));
		}
		cassetteForm.setFormChanged(false);
		cassetteForm.setNewBlockRequested(false);
		cassetteForm.setSaveCassettes(false);
	}

	@Override
	protected Map referenceData(HttpServletRequest request, Object command, Errors arg2) throws Exception {
		Map<String,Object> formMetaData = new HashMap<String,Object>();

		String printErrorMessage = ServletRequestUtils.getStringParameter(request, "printErrorMessage");
		formMetaData.put("printErrorMessage",printErrorMessage);

		Collection<BlockType> blockTypeDropDown = blockTypeService.fetchActiveBlockTypes(false,null);
		formMetaData.put("tissueTypes", blockTypeDropDown);

		Set<CassettePrinter> cassettePrinters  = new TreeSet<CassettePrinter>();
		for (CassettePrinter cp : CassettePrinter.values()) {
			cassettePrinters.add(cp);
		}
		formMetaData.put("cassettePrinters", cassettePrinters);

		return formMetaData;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response,
			Object command, BindException errors) throws Exception {
		CassetteForm blockForm = (CassetteForm) command;
		Collection<BlockListView> blocksToPrint = new ArrayList<BlockListView>();
		Collection<BlockListView> selectableBlocks = new ArrayList<BlockListView>(blockForm.getStandardBlocks().size() + blockForm.getExtraBlocks().size());
		selectableBlocks.addAll(blockForm.getStandardBlocks());
		selectableBlocks.addAll(blockForm.getExtraExistingBlocks());

		CassettePrinter printer = blockForm.getCassettePrinter();

		for(BlockListView block : selectableBlocks) {
			if(block.isSelected()){
				blocksToPrint.add(block);
			}
		}
		if(!blocksToPrint.isEmpty()){
			String printErrorMessage = blockService.printCassettes(blocksToPrint,printer);

			if(printErrorMessage == null) {
				if(blockForm.getCohortId() != null){
					//come from the working page of the batch
					return new ModelAndView(getSuccessView()+"id="+blockForm.getCohortId()+"&oraclePrinting="
							+blockForm.getMouse().getBarcode()+" : Selected Cassette Printing job successly sent to the printer!");
				}else{
					String errorView = getErrorView()+"mouseId="+blockForm.getMouseId()+"&printErrorMessage="+"Job successfully sent!";
					return new ModelAndView(errorView);
				}
			}else {

				String errorView = getErrorView()+"mouseId="+blockForm.getMouseId()+"&printErrorMessage="+printErrorMessage;
				return new ModelAndView(errorView);
			}
		}else{
			String errorView = getErrorView()+"mouseId="+blockForm.getMouseId()+"&printErrorMessage="+"Please select the blocks.";
			return new ModelAndView(errorView);
		}
	}

	private void doValidationOnBarcode(CassetteForm cassetteForm) {
		CassetteForm form = cassetteForm;
		BarcodedObject barcodedObject;
		barcodedObject = mouseService.fetchByBarcode(form.getBarcode().trim(), null);
		if(barcodedObject == null) {
			barcodedObject = blockService.fetchByBarcode(form.getBarcode().trim(), ComponentMap.of(Block.Component.ALL_COMPONENTS));
		}

		if(barcodedObject!=null){
			if(barcodedObject instanceof Mouse) {
				cassetteForm.setMouseId(barcodedObject.getId());
			}else{
				BlockListView blockView = blockService.fetchBlockListViewById(barcodedObject.getId());
				cassetteForm.setMouseId(blockView.getMouseId());
				cassetteForm.setBlockId(blockView.getId());
			}

			if(fetchExistingBlocksByMouseId(cassetteForm.getMouseId(), cassetteForm)){
				cassetteForm.setMouse(mouseService.fetchById(cassetteForm.getMouseId(), null));
				cassetteForm.setErrorMessage("");
			}else{
				cassetteForm.setMouseId(null);
				cassetteForm.setErrorMessage("Can not find blocks for "+
						cassetteForm.getBarcode());
				cassetteForm.setBarcode("");
				cassetteForm.setMouse(null);
			}
		}else{
			cassetteForm.setMouseId(null);
			cassetteForm.setErrorMessage("Can not find mouse "+
					cassetteForm.getBarcode());
			cassetteForm.setBarcode("");
			cassetteForm.setMouse(null);
		}
	}

	private void deleteExtraCassettes(CassetteForm cassetteForm) {
		Block dBlock = new Block();
		for(Block block: cassetteForm.getExtraBlocks()) {
			if(block.getBlockTypeId().equals(cassetteForm.getDeleteExtraBlockTypeId())
					&& block.getComments().equals(cassetteForm.getDeleteComments())){
				dBlock = block;
				break;
			}
		}
		cassetteForm.getExtraBlocks().remove(dBlock);
		cassetteForm.setDeleteComments(null);
		cassetteForm.setDeleteExtraBlockTypeId(null);
	}
	private void addNewBlock(CassetteForm cassetteForm) {
		Collection<Block> extraBlocks = cassetteForm.getExtraBlocks();
		Block block = new Block();
		BlockType blockType = blockTypeService.fetchById(cassetteForm.getBlockTypeDictId(), null);
		Mouse mouse = mouseService.fetchById(
			    cassetteForm.getMouseId(),
			    componentMapService.getComponentMap("mouseToFixation"));
		block.setFixationId(mouse.getExperimentData().getNecropsy().getFixation().getId());
		block.setBlockTypeId(cassetteForm.getBlockTypeDictId());
		block.setBlockType(blockType);
		block.setComments(cassetteForm.getComments());
		extraBlocks.add(block);

		cassetteForm.setBlockTypeDictId(null);
		cassetteForm.setComments(null);

	}
	private void saveCassettes(CassetteForm cassetteForm, Person creator) {
		for(Block block:cassetteForm.getExtraBlocks()){
			blockService.store(block, creator);
		}

		Collection<BlockListView> extraExistingBlocks = blockService.fetchStandardByMouseId(cassetteForm.getMouseId(), false);
		cassetteForm.setExtraExistingBlocks(extraExistingBlocks);
		cassetteForm.getExtraBlocks().removeAll(cassetteForm.getExtraBlocks());
		cassetteForm.setSaveCassettes(false);

	}

	private boolean fetchExistingBlocksByMouseId(Long mouseId, CassetteForm cassetteForm) {

		Collection<BlockListView> standardBlocks = blockService.fetchStandardByMouseId(mouseId,true);
		Collection<BlockListView> extraBlocks = blockService.fetchStandardByMouseId(mouseId, false);

		if(standardBlocks != null && standardBlocks.size()>0){
			Collections.sort((List<BlockListView>)standardBlocks, new BlockViewComparator());
			cassetteForm.setStandardBlocks(standardBlocks);
		}else{
			return false;
		}
		if(extraBlocks != null && extraBlocks.size()>0){
			Collections.sort((List<BlockListView>)extraBlocks, new BlockViewComparator());
			cassetteForm.setExtraExistingBlocks(extraBlocks);
		}
		return true;
	}
}