package uk.ac.sanger.mig.histology.controllers;

import java.util.Date;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.springframework.validation.BindException;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.controllers.SerializedObjectView;
import uk.ac.sanger.mig.histology.forms.CropJobForm;
import uk.ac.sanger.mig.histology.services.CropJobService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.quartz.JobService;

public class CropJobController extends AbstractCancellableFormController {

	private CropJobService cropJobService;
	private JobService quartzJobService;
	private static final String CROP_JOB_NAME = "Crop Job Processor";
	private static final String CROP_JOB_GROUP = "Batch Processing";

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response,
				Object command, BindException errors) throws Exception {

		CropJobForm form = (CropJobForm)command;

		Long newJobId = cropJobService.create(form, true, RequestUtils.getUser(request));

		if( form.isCropImmediately() ) {//immediate crop

			ServletOutputStream out = response.getOutputStream();
			response.setContentType("text/plain");

			try {

				if( newJobId != null ) {

					out.println("Cropping Immediately...");

					Boolean result =
						runAndReportFailure( newJobId, true, RequestUtils.getUser(request) );

					out.println( (result) ? "*** OK *** Success" : "Failure" );

				} else {
					out.println("*** OK *** \nNo new Crop Job created since identical region(s) already exist for this Scan.");
				}

			}  finally {

				IOUtils.closeQuietly( out );
			}

			return null;

		}  else {//overnight crop


			String successMessage = "\n*** OK ***";

			if( newJobId == null ) {

				successMessage += "\nNo new Crop Job created since identical region(s) already exist for this Scan.\n";

			}

			successMessage += "\nNEXT BATCH RUN: " + quartzJobService.getNextFireTimeByJob(
					CROP_JOB_NAME,CROP_JOB_GROUP);

			return new ModelAndView(  new SerializedObjectView(), getSuccessView(), successMessage);
		}
	}

	/*
	 * TODO: Move to non-transactional service
	 * 
	 * NOTE: Called outside service layer to run() and storeFailedJob() are
	 * properly transactional.
	 *
	 * Runs a CropJob and reports CropJob failure if it does not succeed, storing
	 * a failed CropJob.  Returns true if success, false if failure;

	 * Successful running of a CropJob guarantees that readable crop images have been
	 * created on file server an references to them have been stored in the Crop table.
	 *
	 * If failure there is failure, any database changes to Crop table will roll back,
	 * however, any system files that may have been created are not guaranteed to be
	 * removed.
	 *
	 */
	private Boolean runAndReportFailure( Long cropJobId, boolean timeout, Person user ) {

		Boolean returnValue = null;

		Date startTime = new Date();

		try {
			//if run() fails, it should be rolled back to here and caught
			cropJobService.run( cropJobId, startTime, timeout, user );
			returnValue = Boolean.TRUE;

		} catch(Throwable e) {
			//storeFailedJob() is separate transaction (ie, not nested in any other transaction controlled method)

			logger.error("Storing failed job.", e);

			cropJobService.storeFailedJob(cropJobId, startTime, e, user);

			returnValue = Boolean.FALSE;
		}

		return returnValue;
	}


	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) 
								throws Exception {
		return createCommand();
	}

	@Override
	protected boolean isFormSubmission(HttpServletRequest request) {
		return true;
	}


	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// dependencies /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	public void setCropJobService(CropJobService cropJobService) {
		this.cropJobService = cropJobService;
	}

	public void setQuartzJobService(JobService quartzJobService) {
		this.quartzJobService = quartzJobService;
	}



}
