package uk.ac.sanger.mig.histology.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.TimeoutException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.histology.BlockType;
import uk.ac.sanger.mig.histology.Slide;
import uk.ac.sanger.mig.histology.forms.PrintSlideCheckbox;
import uk.ac.sanger.mig.histology.forms.SlideLabelsForm;
import uk.ac.sanger.mig.histology.services.BlockService;
import uk.ac.sanger.mig.histology.services.SlideService;
import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class GenerateSlideLabelsFormController extends AbstractCancellableFormController {
	private MouseService mouseService;
	private SlideService slideService;
	private BlockService blockService;
	private DictionaryService dictionaryService;
	private String errorView;
	private String mainView;
	private ComponentMapService componentMapService;

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}


	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}


	public String getErrorView() {
		return errorView;
	}

	public void setErrorView(String errorView) {
		this.errorView = errorView;
	}

	public void setBlockService(BlockService blockService) {
		this.blockService = blockService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setSlideService(SlideService slideService) {
		this.slideService = slideService;
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request, Object command) {
		SlideLabelsForm form = (SlideLabelsForm) command;

		if( form.isFormChanged()) 
			return true;
		else
			return super.isFormChangeRequest( request );		
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command) throws Exception {
		SlideLabelsForm slideLabelsForm = (SlideLabelsForm)command;

		BarcodedObject barcodedObject = doValidationOnBarcode(slideLabelsForm);

		if(barcodedObject==null){
			slideLabelsForm.setErrorMessage("Could not find mouse or block corresponding to "+ 
					slideLabelsForm.getBarcode());
			slideLabelsForm.setBarcode("");
			slideLabelsForm.setScannedObject(null);
			return;
		}else{
			slideLabelsForm.setErrorMessage("");
		}

		Mouse mouse = null;    
		slideLabelsForm.setScannedObject(barcodedObject);

		if(barcodedObject instanceof Mouse) {
			mouse = (Mouse) barcodedObject;
		}
		else if (barcodedObject instanceof Block) {
			Block block = (Block) barcodedObject;
			mouse = mouseService.fetchById(
					block.getFixation().getNecropsy().getMouseId(),
					ComponentMap.of(Mouse.Component.ALL_COMPONENTS));
		}
		else if (barcodedObject instanceof Slide) {
			Slide slide = (Slide) barcodedObject;
			mouse = mouseService.fetchById(
					slide.getBlock().getFixation().getNecropsy().getMouseId(),
					ComponentMap.of(Mouse.Component.ALL_COMPONENTS));
		}

		if (mouse != null) {
			slideLabelsForm.setMouse(mouse);
			fillInFormDetails(slideLabelsForm);
		}
	}

	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		SlideLabelsForm form = (SlideLabelsForm)createCommand();           
		Long mouseId = ServletRequestUtils.getLongParameter(httpServletRequest, "mouseId");
		String printErrorMessage = ServletRequestUtils.getStringParameter(httpServletRequest, "printErrorMessage");
		Long cohortId = ServletRequestUtils.getLongParameter(httpServletRequest, "cohortId");

		form.setCohortId(cohortId);
		form.setPrintingMessage(printErrorMessage);
		if(mouseId != null){
			Mouse mouse = mouseService.fetchById(mouseId, ComponentMap.of(Mouse.Component.ALL_COMPONENTS));
			form.setBarcode(mouse.getBarcode());
			form.setScannedObject(mouse);
			form.setMouse(mouse);
			fillInFormDetails(form);
		}
		return form;    
	}

	private void fillInFormDetails(SlideLabelsForm slideLabelsForm) {
		Collection<Block> blocks = new ArrayList<Block>();

		blocks = blockService.fetchByMouseId(slideLabelsForm.getMouse().getId(),
				ComponentMap.putAll(
						ComponentMap.of(Block.Component.SLIDES),
						new ComponentMap(
								Block.Component.BLOCK_TYPE,
								ComponentMap.of(BlockType.Component.CASSETTE_TYPE, BlockType.Component.TISSUES,BlockType.Component.DEFAULT_STAINS)
						)
				)
		);

		if (blocks.size() < 1) {
			slideLabelsForm.setErrorMessage("Could not find blocks for selected mouse");
		}

		DictionaryEntry[] stains = dictionaryService.fetchArray(DictionaryEnum.HTL_STAINING_TYPE_DICT, true);
		slideLabelsForm.setStains(stains);

		List<PrintSlideCheckbox> checkboxes = new LinkedList<PrintSlideCheckbox>();

		for (Block block : blocks) {
			for (DictionaryEntry stain : stains) {
				PrintSlideCheckbox sc = createCheckbox(block, stain, slideLabelsForm.getScannedObject());
				sc.setIndexInList(checkboxes.size());
				checkboxes.add(sc);
			}
		}

		slideLabelsForm.setSlideSelection(Collections.unmodifiableList(checkboxes));
		slideLabelsForm.setFormChanged(false);
	}

	private PrintSlideCheckbox createCheckbox(Block block, DictionaryEntry stain, BarcodedObject barcodedObject) {
		PrintSlideCheckbox checkbox = new PrintSlideCheckbox();

		checkbox.setBlock(block);
		checkbox.setBlockId(block.getId());
		checkbox.setStainId(stain.getId());

		/* Logic to determine if this slide should default to checked
		 * Current logic: 
		 *  If..
		 *    we don't have any slides for this stain and
		 *    this stain is required by default
		 *    
		 *  ..then check the box
		 */
		boolean slidesExist = false;
		boolean defaultToRequired = block.getBlockType().getDefaultStains().contains(stain);
		long currentMaxVersionNumber = 0l;

		for(Slide s: block.getSlides()){
			if(s.getStainingTypeDictId().equals(checkbox.getStainId())){
				slidesExist = true;
				if(s.getVersionNumber() > currentMaxVersionNumber){
					currentMaxVersionNumber = s.getVersionNumber();
				}
			}
		}
		checkbox.setChecked(!slidesExist && defaultToRequired);
		if(currentMaxVersionNumber>0l)
			checkbox.setDisplayText(currentMaxVersionNumber+" labels printed");
		else
			checkbox.setDisplayText("");
		return checkbox;
	}

	private BarcodedObject doValidationOnBarcode(SlideLabelsForm form) {
		BarcodedObject barcodedObject = null;

		if (form.getBarcode() != null) {      
			barcodedObject = mouseService.fetchByBarcode(
					form.getBarcode(),
					ComponentMap.of(Mouse.Component.ALL_COMPONENTS));

			if(barcodedObject == null) {
				barcodedObject = blockService.fetchByBarcode(
						form.getBarcode(),
						componentMapService.getComponentMap("blockToNecropsy"));
			}

			if(barcodedObject==null){
				barcodedObject = slideService.fetchByBarcode(
						form.getBarcode(), 
						componentMapService.getComponentMap("slideToNecropsy")
				);

			}
		}
		return barcodedObject==null || barcodedObject.getId() == null ? null: barcodedObject;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request,
			HttpServletResponse response, Object command, BindException errors) throws Exception {
		SlideLabelsForm form = (SlideLabelsForm)command;
 
		try{
			Collection<Slide> slides = slideService.createAll(form.getSlideSelection(), RequestUtils.getUser(request));
			

			
			if(form.getCohortId() != null) {
				if(slides.size()==0){
					String errorView = getErrorView()+"mouseId="+form.getMouse().getId()+"&printErrorMessage="+"No slide has been selected!";
					return new ModelAndView(errorView);
				}else{
					slideService.printSlides(slides);
					return new ModelAndView(getSuccessView()+"id="+form.getCohortId()+"&oraclePrinting="
						+form.getMouse().getBarcode()+" : Selected Slides Printing job successfully sent to the printer."); 
				}
			}else{
				return new ModelAndView(getMainView());
			}
		}catch(TimeoutException e){
			String errorView = getErrorView()+"mouseId="+form.getMouse().getId()+"&printErrorMessage="
			+form.getMouse().getBarcode()+" : Print Failed! The printing job has timed out, Please contact Mouse Informatics Group!";
			return new ModelAndView(errorView);
		}catch(InterruptedException e){
			String errorView = getErrorView()+"mouseId="+form.getMouse().getId()+"&printErrorMessage="
			+form.getMouse().getBarcode()+" : Print Failed! The printing job has been interrupted, Please contact Mouse Informatics Group!";
			return new ModelAndView(errorView);
		}catch(RuntimeException e){
			String errorView = getErrorView()+"mouseId="+form.getMouse().getId()+"&printErrorMessage="
			+form.getMouse().getBarcode()+" : Print Failed! Please make sure your slides are selected and the printer is on and working properly!";
			return new ModelAndView(errorView);
		}
	}


	public String getMainView() {
		return mainView;
	}


	public void setMainView(String mainView) {
		this.mainView = mainView;
	}

}
