/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.controllers;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.beans.propertyeditors.StringTrimmerEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

import uk.ac.sanger.mig.common.DataObjectIdComparator;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.histology.forms.ManageBatchForm;
import uk.ac.sanger.mig.histology.services.BatchService;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.CohortSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.mousetracking.views.CohortListView;
import uk.ac.sanger.mig.personnel.Person;

public class ManageBatchController extends SimpleFormController {

	private BatchService batchService;
	private CohortService cohortService;
	private MouseService mouseService;
	private String printView;
	private ComponentMapService componentMapService;

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setBatchService(BatchService batchService) {
		this.batchService = batchService;
	}

	public void setCohortService(CohortService cohortService) {
		this.cohortService = cohortService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setPrintView(String printViewName) {
		this.printView = printViewName;
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request, Object command) {
		ManageBatchForm form = (ManageBatchForm)command;

		boolean changeRequest = false;

		if (form.getChangeRequest() != null && form.getChangeRequest().length() > 0) {
			changeRequest = true;
		}

		return changeRequest;
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command) throws Exception {
		ManageBatchForm form = (ManageBatchForm)command;

		if(form.getChangeRequest().equalsIgnoreCase("cage")) {
			formChangeScanCage(form);
		}
		else if(form.getChangeRequest().equalsIgnoreCase("cohort")) {
			formChangeChooseCohort(form);
		}
		else if (form.getChangeRequest().equalsIgnoreCase("mouse")) {
			formChangeScanMouse(form);
		}
		else if (form.getChangeRequest().equalsIgnoreCase("update")) {
			form.update();
		}
		else {
			logger.warn("Form change received unexpected change request: "+form.getChangeRequest());
		}

		form.setChangeRequest(null);
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		ManageBatchForm form = (ManageBatchForm)createCommand();

		Long batchId = ServletRequestUtils.getLongParameter(request, "id");

		if (batchId == null) {
			form.setBatch(new Cohort());
			form.getBatch().setCohortTypeDictId(CohortService.COHORT_TYPE_HISTOLOGY);
			form.getBatch().setEndDate(new Date());
			form.getBatch().setMice(new ArrayList<Mouse>());
			form.getBatch().setCohortPipelineDictId(null);
		}
		else {
			form.setCohortIdForBatch(batchId);
			Cohort batch = cohortService.fetchById(
					batchId,
					componentMapService.getComponentMap("miceToBlocksForCohort")
			);
			batch.setCohortPipelineDictId(null);
			form.setBatch(batch);
			form.initMiceInBatch();
		}

		return form;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		ManageBatchForm form = (ManageBatchForm)command;

		Person user = RequestUtils.getUser(request);

		// apply the requested changes to the forms copy of the batch before storing
		form.update();

		batchService.manageBatch(form.getBatch(), form.getBatch().getMice(), user);

		return new ModelAndView(printView+"?id="+ form.getBatch().getId());
	}

	private void formChangeScanCage(ManageBatchForm form) {
		Collection<Mouse> mice = mouseService.fetchByCageId(form.getSelectorId(), componentMapService.getComponentMap("miceToBlocks"));
		setMiceToSelectFrom(mice,form);
	}

	private void formChangeChooseCohort(ManageBatchForm form) {
		Cohort cohort = cohortService.fetchById(
				form.getSelectorId(),
				componentMapService.getComponentMap("miceToBlocksForCohort"));
		setMiceToSelectFrom(cohort.getMice(),form);
	}

	private void formChangeScanMouse(ManageBatchForm form) {
		Mouse m = mouseService.fetchById(form.getSelectorId(), componentMapService.getComponentMap("miceToBlocks"));
		addMouseToSelectFrom(m, form);
	}

	private void addMouseToSelectFrom(Mouse m, ManageBatchForm form) {
		CohortSearchBean searchBean = new CohortSearchBean();
		searchBean.setMouse(m);
		searchBean.setCohortTypeDictId(CohortService.COHORT_TYPE_HISTOLOGY);
		searchBean.setPaginated(false);

		Collection<CohortListView> cohorts =  cohortService.search(searchBean);

		boolean inHistologyBatch = (cohorts.size() > 0);

		form.addMouseToSelectFrom(m, inHistologyBatch);

	}

	private void setMiceToSelectFrom(Collection<Mouse> mice, ManageBatchForm form) {
		form.clearMiceToSelectFrom();

		List<Mouse> sortedMice = new ArrayList<Mouse>(mice);
		Collections.sort(sortedMice, new DataObjectIdComparator());

		for (Mouse m : sortedMice) {
			addMouseToSelectFrom(m, form);
		}
	}


	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		binder.registerCustomEditor(Long.class, new CustomNumberEditor(Long.class, true));
		binder.registerCustomEditor(Double.class, new CustomNumberEditor(Double.class, true));
		binder.registerCustomEditor(Integer.class, new CustomNumberEditor(Integer.class, true));
		binder.registerCustomEditor(Date.class, new CustomDateEditor(new SimpleDateFormat("dd-MMM-yyyy"), true));
		binder.registerCustomEditor(String.class, new StringTrimmerEditor(true)); //empty strings convert to null
	}



}
