package uk.ac.sanger.mig.histology.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.LongRowMapper;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropSearchBean;
import uk.ac.sanger.mig.histology.dao.rowmappers.CropViewRowMapper;
import uk.ac.sanger.mig.histology.views.CropView;

public class CropDaoSpringImpl extends AbstractDaoSpringImpl<Crop> implements CropDao {

	private LongRowMapper longRowMapper;
	private CropViewRowMapper cropViewRowMapper;
	private PaginatedSearchUtils<CropView> paginator;


	public Collection<CropView> search(CropSearchBean searchBean) {

		if( searchBean == null ) {
			throw new IllegalArgumentException("searchBean cannot be null.");
		}

		StringBuffer sql = new StringBuffer();
		sql.append(" select * from htl_crop_vw c where 1=1 ");

		Collection<Object> bindVariables = new ArrayList<Object>();

		if( searchBean.getQcSearchCategory() != null ) {

			switch( searchBean.getQcSearchCategory() ) {

				case PASS:
					sql.append(" and c.qc_pass = 1 ");
					break;
				case FAIL:
					sql.append(" and c.qc_pass = 0 ");
					break;
				case COMPLETE:
					sql.append(" and c.qc_pass is not null ");
					break;
				case AWAIT_QC:
					sql.append(" and c.qc_pass is null ");
					break;
			}
		}

		if( searchBean.getGenotypeDictId() != null ) {

			sql.append(" and c.genotype_dict_id = ? ");
			bindVariables.add( searchBean.getGenotypeDictId() );
		}

		if( searchBean.getAlleleId() != null ) {

			sql.append(" and c.allele_id = ? ");
			bindVariables.add( searchBean.getAlleleId() );
		}

		if( searchBean.getGenderDictId() != null ) {

			sql.append(" and c.gender_dict_id = ? ");
			bindVariables.add(  searchBean.getGenderDictId() );
		}

		if( searchBean.getMouseId() != null ) {

			sql.append(" and c.mouse_id = ? ");
			bindVariables.add( searchBean.getMouseId() );
		}

		if( searchBean.getSourceBlockId() != null ) {

			sql.append(" and c.source_block_id = ? ");
			bindVariables.add( searchBean.getSourceBlockId() );
		}

		if( StringUtils.isNotBlank( searchBean.getMouseBarcode() ) ) {

			sql.append(" and (upper(c.mouse_barcode) like '%'||upper(?)||'%') ");
			bindVariables.add( searchBean.getMouseBarcode() );
		}

		if( StringUtils.isNotBlank( searchBean.getSlideBarcode() ) ) {

			sql.append(" and (upper(c.slide_barcode) like '%'||upper(?)||'%') ");
			bindVariables.add( searchBean.getSlideBarcode() );
		}

		if( !StringUtils.isBlank( searchBean.getPipeline() ) ) {

			sql.append(" and (upper(c.pipeline) like '%'||upper(?)||'%') ");
			bindVariables.add( searchBean.getPipeline() );
		}

		if( StringUtils.isNotBlank( searchBean.getAnnotations() ) ) {

			sql.append(" and (upper(c.annotations) like '%'||upper(?)||'%') ");
			bindVariables.add( searchBean.getAnnotations() );
		}

		if( StringUtils.isNotBlank( searchBean.getComments() ) ) {

			sql.append(" and (upper(c.comments) like '%'||upper(?)||'%') ");
			bindVariables.add( searchBean.getComments() );
		}


		if( StringUtils.isNotBlank( searchBean.getOrderBy() ) ) {
			sql.append(" order by ");
			sql.append(searchBean.getOrderBy());

			if(searchBean.isOrderDesc()) {
				sql.append(" desc ");
			}
		}



		return paginator.fetchSearchResults(sql.toString(), bindVariables,
				searchBean, cropViewRowMapper);


	}

	public List<Long> fetchCropIdsByMouse(Long mouseId) {

		String sql = "select c.id return_long from HTL_CROP c, HTL_CROP_JOB cj, " +
		"HTL_SCAN s, HTL_SLIDE sl, HTL_BLOCK b, HTL_FIXATION f, HTL_NECROPSY n, " +
		"mts_mouse m where c.crop_job_id = cj.id AND cj.scan_id = s.id AND " +
		"s.slide_id = sl.id AND sl.block_id = b.id AND b.fixation_id = f.id AND " +
		"f.necropsy_id = n.id AND n.mouse_id = m.id AND m.id = ? " +
		"order by sl.block_id, s.slide_id, cj.scan_id, c.y_location";

		List<Long> ids = getSimpleJdbcTemplate().query(sql, longRowMapper, mouseId);
		return (ids == null) ? new ArrayList<Long>() : ids;
	}

	public Collection<Crop> fetchCropsByCropJobId(Long cropJobId) {
		if( cropJobId == null )
			throw new IllegalArgumentException("cropJobId cannot be null!");

		String sql = " select c.* from " + getTableName() + " c where c.crop_job_id = ? ";
		Collection<Crop> crops = getSimpleJdbcTemplate().query(sql, getRowMapper(), cropJobId);
		return (crops == null) ? new ArrayList<Crop>() : crops;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Crop crop) {
		sql.addIntParam(crop.getCropWidth(), "WIDTH");
		sql.addIntParam(crop.getCropHeight(), "HEIGHT");
		sql.addLongParam(crop.getCropJobId(), "CROP_JOB_ID");
		sql.addIntParam(crop.getX(), "X_LOCATION");
		sql.addIntParam(crop.getY(), "Y_LOCATION");

		sql.addFilePathParam( crop.getFullImagePath(), "FULL_IMAGE_PATH");
		sql.addFilePathParam( crop.getResizedImagePath(), "RESIZED_IMAGE_PATH");

		sql.addIntParam( crop.getTileEdgeExponent(), "TILE_EDGE_EXPONENT");
		sql.addIntParam( crop.getZoomRadix(), "ZOOM_RADIX");
		sql.addBooleanParam( crop.isTiled(), "TILED");


		sql.addBooleanParam(crop.isFlaggedForTiling(), "IS_FLAGGED_FOR_TILING");

		sql.addStringParam(crop.getComments(), "COMMENTS");
		sql.addBooleanParam(crop.getStainingPresent(), "STAINING_PRESENT");
		sql.addDateParam(crop.getEditDate(), "EDIT_DATE");
		sql.addStringParam(crop.getEditedBy(), "EDITED_BY");
		sql.addDateParam(crop.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(crop.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(crop.getCheckNumber(), "CHECK_NUMBER");
		sql.addBooleanParam(crop.isFlaggedForDelete(), "IS_FLAGGED_FOR_DELETE");

		sql.addBooleanParam(crop.isQcLocked(), "QC_LOCKED");

		sql.addBooleanParam(crop.getQcPass(), "qc_pass");
		sql.addStringParam(crop.getQcFailReason(), "qc_fail_reason");
		sql.addLongParam(crop.getTissueId(), "TISSUE_ID");
		sql.addLongParam(crop.getId(), "ID");
	}

	public void setLongRowMapper(LongRowMapper longRowMapper) {
		this.longRowMapper = longRowMapper;
	}

	public void setCropViewRowMapper(CropViewRowMapper cropViewRowMapper) {
		this.cropViewRowMapper = cropViewRowMapper;
	}

	public void setPaginator(PaginatedSearchUtils<CropView> paginator) {
		this.paginator = paginator;
	}

}