/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.histology.dao;

import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.histology.Tile;
import uk.ac.sanger.mig.histology.dao.rowmappers.TileRowMapper;

public class TileDaoSpringImpl extends SimpleJdbcDaoSupport implements TileDao {


	private TileRowMapper tileRowMapper;
	private String tableName;


	public void deleteAllByCropId(Long cropId) {

		if(cropId == null) {
			throw new IllegalArgumentException("Cannot call delete on a null cropId");
		}

		String sqlString = " delete " + tableName + " where crop_id = ?";

		ParameterizedSqlUpdate delete = new ParameterizedSqlUpdate(getJdbcTemplate());
		delete.addLongParam(cropId);

		delete.execute(sqlString);

	}

	public Tile fetch(final Long cropId, final int zoomLevel, final int arrayIndex) {

		String sql = "select tile.* from " + tableName + " tile where "+
					 "tile.CROP_ID = ? AND tile.ZOOM_LEVEL = ? AND tile.ARRAY_INDEX = ?";

		Tile tile = null;

		try {
			tile = getSimpleJdbcTemplate().queryForObject(sql,
					tileRowMapper, new Object[] {cropId, zoomLevel, arrayIndex});
		}
		catch(IncorrectResultSizeDataAccessException e) {
			logger.warn("Incorrect result set size: " + tableName +" for tile with" +
					"cropId =" + cropId + ", zoomLevel =  " + zoomLevel + ", arrayIndex = "
					+ arrayIndex + ": " + e);
		}

		return tile;
	}


	public void storeAll(Collection<Tile> tiles) {

		for (Tile tile : tiles) {
			store( tile );
		}

	}

	public void store(Tile tile) {

		insert( tile );

	}

	protected void insert(Tile tile) {
		ParameterizedSqlUpdate sql = new ParameterizedSqlUpdate(getJdbcTemplate());
		bindVariables(sql,tile);
		sql.executeInsert(tableName);

	}


	protected void bindVariables(ParameterizedSqlUpdate sql, Tile tile) {
		sql.addLongParam(tile.getCropId(), "CROP_ID");
		sql.addIntParam(tile.getZoomLevel(), "ZOOM_LEVEL");
		sql.addIntParam(tile.getArrayIndex(), "ARRAY_INDEX");
		sql.addBlobParam(tile.getImage(), "IMAGE", false);
		sql.addStringParam(tile.getFormatExtension(), "FORMAT_EXTENSION");
	}


	public void setTableName(String tableName) {
		this.tableName = tableName;
	}


	public void setTileRowMapper(TileRowMapper tileRowMapper) {
		this.tileRowMapper = tileRowMapper;
	}



}