/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.forms;

import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.mousetracking.Mouse;

/**
 * Object to indicate whether a mouse has been selected, with additional
 * information about the processing performed on that mouse
 */
public class BatchProcessingCheckbox extends MouseCheckbox {
  
  private List<ProcessingSummary> processingSummaries = new LinkedList<ProcessingSummary>();
  
  public BatchProcessingCheckbox(Mouse m, boolean checked, DictionaryEntry[] cassetteTypes) {
    super(m,checked);
    
    for(DictionaryEntry cassetteType : cassetteTypes) {
      ProcessingSummary summary = new ProcessingSummary();
      summary.setCassetteType(cassetteType);
      processingSummaries.add(summary);
      
      List<Block> blocks = filterBlocksByCassetteType(cassetteType.getId());

      // assume that if there are more than one block, they all have the same details
      if (blocks.size() > 0) {
        Block b = blocks.get(0);
        
        summary.setFixationEnd(b.getFixationEndTime());
        summary.setProgrammeStart(b.getProgrammeStartTime());
        summary.setProgrammeType(b.getProgrammeType());
      }
    }
    
  }
  
  public List<Block> filterBlocksByCassetteType(Long cassetteTypeId) {
    List<Block> matchingBlocks = new LinkedList<Block>();
    Collection<Block> blocks = getMouse().getExperimentData().getNecropsy().getFixation().getBlocks();
    for (Block b : blocks) {
      if (b.getBlockType().getCassetteTypeDictId().equals(cassetteTypeId) ) {
        matchingBlocks.add(b);
      }
    }
    
    return matchingBlocks;
  }
  
  
  public boolean isCassettesExist() {
    Collection<Block> blocks = getMouse().getExperimentData().getNecropsy().getFixation().getBlocks();
    return (blocks.size() > 1);
  }

  public boolean isNonStandardCassettesExist() {
    boolean hasNonStandardCassettes = false;
    
    Collection<Block> blocks = getMouse().getExperimentData().getNecropsy().getFixation().getBlocks();
    
    for (Block b : blocks) {

      if (!b.getBlockType().isStandardBlock()) {
        hasNonStandardCassettes = true;
        break;
      }

    }
    
    
    return hasNonStandardCassettes;
  }
  
  public class ProcessingSummary {
    private Date fixationEnd;
    private Date programmeStart;
    private DictionaryEntry programmeType;
    private DictionaryEntry cassetteType;
    
    public void setCassetteType(DictionaryEntry cassetteType) {
      this.cassetteType = cassetteType;
    }
    public void setFixationEnd(Date fixationEnd) {
      this.fixationEnd = fixationEnd;
    }
    public void setProgrammeStart(Date programmeStart) {
      this.programmeStart = programmeStart;
    }
    public void setProgrammeType(DictionaryEntry programmeType) {
      this.programmeType = programmeType;
    }
    public DictionaryEntry getCassetteType() {
      return cassetteType;
    }
    public Date getFixationEnd() {
      return fixationEnd;
    }
    public Date getProgrammeStart() {
      return programmeStart;
    }
    public DictionaryEntry getProgrammeType() {
      return programmeType;
    }
  }

  public List<ProcessingSummary> getProcessingSummaries() {
    return processingSummaries;
  }
  

}
