/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.forms;

import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeSet;

import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.histology.services.BlockService.CassettePrinter;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.Mouse;

public class BatchProcessingForm {
  // which batch are we working on
  private Cohort batch;

  // Process information
  private Long cassetteTypeId = 0L;
  private Date fixationEnd;
  private Long programmeTypeId = 0L;
  private Date programmeStart;

  // reference information
  private DictionaryEntry[] cassetteTypes;
  private DictionaryEntry[] programmeTypes;

  // mice we will apply the process information to
  private List<BatchProcessingCheckbox> checkboxes;

  private String reportPrintingMessage;

  private String cassettePrinterName = CassettePrinter.TISSUE_TEK.toString();

	public String getReportPrintingMessage() {
		return reportPrintingMessage;
	}

	public void setReportPrintingMessage(String reportPrintingMessage) {
		this.reportPrintingMessage = reportPrintingMessage;
	}

	public Cohort getBatch() {
    return batch;
  }

  public void setBatch(Cohort batch) {
    this.batch = batch;
  }

  public Long getCassetteTypeId() {
    return cassetteTypeId;
  }

  public void setCassetteTypeId(Long cassetteTypeId) {
    this.cassetteTypeId = cassetteTypeId;
  }

  public DictionaryEntry[] getCassetteTypes() {
    return cassetteTypes;
  }

  public void setCassetteTypes(DictionaryEntry[] cassetteTypes) {
    this.cassetteTypes = cassetteTypes;
  }

  public List<BatchProcessingCheckbox> getCheckboxes() {
    return checkboxes;
  }

  public void setCheckboxes(List<BatchProcessingCheckbox> checkboxes) {
    this.checkboxes = checkboxes;
  }

  public Date getFixationEnd() {
    return fixationEnd;
  }

  public void setFixationEnd(Date fixationEnd) {
    this.fixationEnd = fixationEnd;
  }

  public Date getProgrammeStart() {
    return programmeStart;
  }

  public void setProgrammeStart(Date programmeStart) {
    this.programmeStart = programmeStart;
  }

  public Long getProgrammeTypeId() {
    return programmeTypeId;
  }

  public void setProgrammeTypeId(Long programmeTypeId) {
    this.programmeTypeId = programmeTypeId;
  }

  public DictionaryEntry[] getProgrammeTypes() {
    return programmeTypes;
  }

  public void setProgrammeTypes(DictionaryEntry[] programmeTypes) {
    this.programmeTypes = programmeTypes;
  }

  /**
   * Populate the checkboxes
   *
   * @param mice
   * @param cassetteTypes
   */
  public void buildCheckboxes(Collection<Mouse> mice, DictionaryEntry[] cassetteTypes) {
    checkboxes = new LinkedList<BatchProcessingCheckbox>();

    for (Mouse m : mice) {
      Collection<Block> blocks = m.getExperimentData().getNecropsy().getFixation().getBlocks();
      boolean checked = (blocks.size() > 0);
      BatchProcessingCheckbox checkbox = new BatchProcessingCheckbox(m,checked,cassetteTypes);
      checkbox.setChecked(false);
      checkboxes.add(checkbox);
    }
  }

  /**
   * Pull out blocks for checked mice that match the cassette type, and
   * apply the programme start, programme type and/or fixation end fields from the form
   *
   * Returns the blocks that have been changed, so that they can be passed to a store method
   *
   * @return
   */
  public Collection<Block> getBlocksOfCheckedMiceForUpdate() {
    Collection<Block> allBlocksForUpdate = new LinkedList<Block>();

    for (BatchProcessingCheckbox checkbox : checkboxes) {

      if (checkbox.isChecked()) {
        Collection<Block> blocks = checkbox.filterBlocksByCassetteType(cassetteTypeId);
        allBlocksForUpdate.addAll(blocks);

        for (Block b : blocks){

          if (programmeStart != null) {
            b.setProgrammeStartTime(programmeStart);
          }

          if (programmeTypeId != null && programmeTypeId > 0) {
            b.setProgrammeTypeId(programmeTypeId);
          }

          if (fixationEnd != null) {
            b.setFixationEndTime(fixationEnd);
          }
        }
      }
    }

    return allBlocksForUpdate;
  }
  /**
   * Are there any cassettes recorded for this batch of mice
   * @return
   */
  public boolean isAnyCassettesExist() {
    boolean someExist = false;

    if (batch != null && batch.getMice() != null) {
      for (Mouse m : batch.getMice()) {
        Collection<Block> blocks = m.getExperimentData().getNecropsy().getFixation().getBlocks();
        if (blocks.size() > 0) {
          someExist = true;
          break;
        }
      }
    }

    return someExist;
  }
  /**
   * Are there any non-standard cassettes recorded for this batch of mice?
   * @return
   */
  public boolean isAnyNonStandardCassettesExist() {
    boolean someExist = false;


    if (batch != null && batch.getMice() != null) {
      outer: for (Mouse m: batch.getMice()) {
        Collection<Block> blocks = m.getExperimentData().getNecropsy().getFixation().getBlocks();
          for (Block b : blocks) {
            if (b.getBlockType() != null && !b.getBlockType().isStandardBlock()) {
              someExist = true;
              break outer;
            }
          }
      }
    }

    return someExist;
  }

  /**
   * Populate the cassetteTypes using the cassette types for the blocks
   * in this batch, so that we only present a list of types that are relevant
   */
  public void fillInCassetteTypesFromBlocks() {
    TreeSet<DictionaryEntry> cassetteTypesFound = new TreeSet<DictionaryEntry>();

    if (batch != null && batch.getMice() != null) {
      for(Mouse m : batch.getMice()) {
        Collection<Block> blocks = m.getExperimentData().getNecropsy().getFixation().getBlocks();
        for (Block b : blocks) {
          if (b.getBlockType() != null &&
              b.getBlockType().getCassetteTypeDict() != null) {
            cassetteTypesFound.add(b.getBlockType().getCassetteTypeDict());
          }
        }
      }
    }

    /* the toArray method looks a bit goofy with generics,
     * the new array parameter is there to provide a type
     */
    cassetteTypes = cassetteTypesFound.toArray(new DictionaryEntry[0]);
  }

	public String getCassettePrinterName() {
		return cassettePrinterName;
	}

	public void setCassettePrinterName(String cassettePrinterName) {
		this.cassettePrinterName = cassettePrinterName;
	}

	public CassettePrinter getCassettePrinter() {
		return CassettePrinter.valueOf(cassettePrinterName);
	}




}
