/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.histology.services;

import java.util.Collection;
import java.util.Date;
import java.util.Random;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import uk.ac.sanger.mig.common.OracleReport;

public abstract class AbstractMicrowriterPrinterService<T> {
	private OracleReport oracleReport;
	private String reportName;
	private String reportOutputPath;
	private int timeOut = 90;
	private static final String SPACE= "%20";
  private static final String DOUBLE_QUOTE = "%22";

  protected Log logger = LogFactory.getLog(this.getClass());

	private final Random random = new Random();

	public String submitPrintJob(Collection<T> printables){
		String errorMessage = null;
		boolean mysteriousFailure = false;
		boolean timedOut = false;

		for (String mainReportChunk : generateIdentifierStrings(printables)) {
			ExecutorService executorService = Executors.newSingleThreadExecutor();
			String reportString = getReportString(mainReportChunk);
			try {
				Future<Boolean> future = executorService.submit(
						new RunnableOracleReport(reportString),
						Boolean.TRUE
				);
				future.get(timeOut, TimeUnit.SECONDS);
			} catch (ExecutionException e) {
				mysteriousFailure = true;
				logger.error("Print job failed: "+reportString,e);
			} catch (InterruptedException e) {
				mysteriousFailure = true;
				logger.error("Print job failed: "+reportString,e);
			} catch (TimeoutException e) {
				timedOut = true;
				logger.error("Print job timed out: "+reportString,e);
			}
			finally{
				executorService.shutdown();
			}
		}

		if (mysteriousFailure) {
			errorMessage = "Printing failed. ";
		}
		if (timedOut) {
			errorMessage = "Printing timed out. ";
		}

		return errorMessage;
	}

	private String getReportString(String mainReportChunk) {
		StringBuilder reportString = new StringBuilder(reportName);
		reportString.append('+');

		reportString.append(mainReportChunk);

		//  target file on migsrv. this seems like bad coupling
		reportString.append('+');

		reportString.append(getReportDestination());

		return reportString.toString();
	}
	/**
	 * Get the full path of the report destination, encoded to account for the possibility of spaces in the name
	 * @return
	 */
	private StringBuilder getReportDestination() {
		StringBuilder destination = new StringBuilder();
		destination.append(DOUBLE_QUOTE);
    destination.append(reportOutputPath.replace(" ", SPACE));

		// trying to create a unique file name without being able to view the dir
		Date now = new Date();
		destination.append(now.getTime());
		destination.append(random.nextInt());

		destination.append(".txt");
		destination.append(DOUBLE_QUOTE);

		return destination;
	}

	public abstract Collection<String> generateIdentifierStrings(Collection<T> printables);

	private class RunnableOracleReport implements Runnable {
		private String reportString;

		public RunnableOracleReport(String reportString) {
			this.reportString = reportString;
		}

		public void run() {
			String errorMessage = oracleReport.run(reportString);
			if(errorMessage != null){
				throw new RuntimeException("Report failed.");
			}

		}
	}

	public void setOracleReport(OracleReport oracleReport) {
		this.oracleReport = oracleReport;
	}

	public void setReportName(String reportName) {
		this.reportName = reportName;
	}

	public void setReportOutputPath(String reportOutputPath) {
		this.reportOutputPath = reportOutputPath;
	}

	public void setTimeOut(int timeOut) {
		this.timeOut = timeOut;
	}

}
