/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.OracleReport;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.histology.BlockType;
import uk.ac.sanger.mig.histology.Fixation;
import uk.ac.sanger.mig.histology.dao.BlockDao;
import uk.ac.sanger.mig.histology.views.BlockListView;
import uk.ac.sanger.mig.histology.views.BlockViewComparator;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.LabelPrinterService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class BlockServiceImpl extends AbstractServiceImpl<Block> implements BlockService {

	private BlockDao blockDao;
	private SlideService slideService;
	private BlockTypeService blockTypeService;
	private LabelPrinterService labelPrinterService;
	private OracleReport oracleReport;
	private MouseService mouseService;
	private FixationService fixationService;
	private CassettePrinterService cassettePrinterService;

	public void setFixationService(FixationService fixationService) {
    this.fixationService = fixationService;
  }

  public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setSlideService(SlideService slideService) {
		this.slideService = slideService;
	}

	public void setLabelPrinterService(LabelPrinterService labelPrinterService) {
		this.labelPrinterService = labelPrinterService;
	}

	public void setOracleReport(OracleReport oracleReport) {
		this.oracleReport = oracleReport;
	}

	public void setBlockDao(BlockDao blockDao) {
		this.blockDao = blockDao;
	}

	public void setBlockTypeService(BlockTypeService blockTypeService) {
		this.blockTypeService = blockTypeService;
	}

	@Override
	protected void attachComponents(Block block, ComponentMap requiredComponents) {
		if(block == null) {
			throw new RuntimeException("Cannot attach components to null block");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Block.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Block.Component.BLOCK_TYPE)){
				block.setBlockType(blockTypeService.fetchById(block.getBlockTypeId(), requiredComponents.get(Block.Component.BLOCK_TYPE)));
			}

			if(fetchAll || requiredComponents.containsKey(Block.Component.SLIDES)){
				block.setSlides(slideService.fetchAllByBlockId(block.getId(), requiredComponents.get(Block.Component.SLIDES)));
			}

			if(fetchAll || requiredComponents.containsKey(Block.Component.FIXATION)){
        block.setFixation(fixationService.fetchById(block.getFixationId(), requiredComponents.get(Block.Component.FIXATION)));
      }

		}
	}

	@Transactional
	public Long store(Block block, Person user) {
		if(block == null) {
			throw new RuntimeException("Cannot save a null block");
		}

		// name the block if it is new and isn't already named
		if (block.getId() == null && block.getName() == null) {
		  String name = fetchNextBlockName(block.getFixationId(), block.getBlockTypeId());
		  block.setName(name);
		}

		return blockDao.store(block, user);
	}

	public Block fetchById(Long blockId, ComponentMap requiredComponents) {
		Block block = blockDao.fetchById(blockId);

		if(block != null)
			attachComponents(block, requiredComponents);

		return block;
	}

	public Block fetchByBarcode(String barcode, ComponentMap map) {

		Block block = blockDao.fetchByBarcode(barcode);
		if(block !=null)
			attachComponents(block, map);

		return block;
	}

	public Collection<Block> fetchByFixationId(Long fixationId, ComponentMap map) {
		Collection<Block> blocks = blockDao.fetchByFixationId(fixationId);


		attachComponents(blocks, map);

		return blocks;
	}

	public Collection<Block> fetchByMouseId(Long mouseId, ComponentMap map) {
	    Collection<Block> blocks = blockDao.fetchByMouseId(mouseId);

	    attachComponents(blocks, map);

	    return blocks;
	}

	public Collection<BlockListView> fetchStandardByMouseId(Long mouseId,boolean standard) {

		return blockDao.fetchAllByMouseId(mouseId, standard);
	}

	public Collection<BlockListView> fetchListViewByMouseId(Long mouseId) {

		return blockDao.fetchListViewsByMouseId(mouseId);
	}

	public String printCassettes(Collection<BlockListView> cassettes, CassettePrinter printer) {
		if (printer == null) throw new IllegalArgumentException("Cassette printer cannot be null");
		if (cassettes == null) throw new IllegalArgumentException("Cassette list cannot be null");

		String message = null;

		List<BlockListView> sortedCassettes = new ArrayList<BlockListView>(cassettes);
		Collections.sort(sortedCassettes, new BlockViewComparator());

		if (CassettePrinter.MICROWRITER.equals(printer)) {
			message = cassettePrinterService.submitPrintJob(sortedCassettes);
		}
		else if (CassettePrinter.TISSUE_TEK.equals(printer)) {
			message = printToTissueTek(sortedCassettes);
		}

		return message;
	}

	private String printToTissueTek(Collection<BlockListView> cassettes) {
		Collection<BlockListView> softTissues = new ArrayList<BlockListView>();
		Collection<BlockListView> hardTissues = new ArrayList<BlockListView>();
		Collection<BlockListView> brainTissues = new ArrayList<BlockListView>();
		Collection<BlockListView> embryoTissues = new ArrayList<BlockListView>();

		for (BlockListView b : cassettes) {
			if (b.isSoftTissue()) {
				softTissues.add(b);
			}
			else if (b.isHardTissue()) {
				hardTissues.add(b);
			}
			else if (b.isBrainTissue()) {
				brainTissues.add(b);
			}
			else if (b.isEmbryoTissue()){
				embryoTissues.add(b);
			}
			else {
				throw new IllegalArgumentException("Unexpected cassette type for block "+b.getId());
			}
		}

		String softTissueReportString = "htl_block_cassette_label+" + labelPrinterService.fetchById(SOFT_TISSUE_PRINTER_ID, null).getSysName();
		String hardTissueReportString = "htl_block_cassette_label+" + labelPrinterService.fetchById(HARD_TISSUE_PRINTER_ID, null).getSysName();
		String brainTissueReportString = "htl_block_cassette_label+" + labelPrinterService.fetchById(BRAIN_TISSUE_PRINTER_ID, null).getSysName();
		String embryoTissueReportString = "htl_block_cassette_label+" + labelPrinterService.fetchById(EMBRYO_TISSUE_PRINTER_ID, null).getSysName();
		String errorMessage = "";

		if(softTissues != null && !softTissues.isEmpty()) {
			errorMessage = constructPrintString(softTissues,softTissueReportString,"Soft Tissue", errorMessage);
		}

		if(hardTissues != null && !hardTissues.isEmpty()) {
			errorMessage = constructPrintString(hardTissues,hardTissueReportString, "Hard Tissue",errorMessage);

		}
		if(brainTissues != null && !brainTissues.isEmpty()) {
			errorMessage = constructPrintString(brainTissues,brainTissueReportString,"Brain Tissue", errorMessage);
		}
		if(embryoTissues != null && !embryoTissues.isEmpty()) {
			errorMessage = constructPrintString(embryoTissues,embryoTissueReportString, "Embryo Tissue", errorMessage);
		}

		if(errorMessage.length() != 0) {
			errorMessage +="Please make sure the printer is on and running properly! ";
		}else{
			errorMessage = null;
		}


		return errorMessage;
	}

	/*
	 * The error message could be failed(RunTimeException been throwed by the Runnable object
	 * timed out(TimeOut or Interrupted exceptions throwed by the future object)
	 */
	private String constructPrintString(Collection<BlockListView> tissues,
			String reportString, String tissueType,  String errorMessage) {

		reportString +="+";
		for(BlockListView block:tissues) {
			reportString += block.getId()+",";
		}
		reportString = reportString.substring(0, reportString.length()-1);
		ExecutorService executorService = Executors.newSingleThreadExecutor();


		try {
			Future<Boolean> future = executorService.submit(new RunnableOracleReport(reportString),Boolean.TRUE);
			future.get(HTL_BLOCK_PRINTING_TIME_OUT, TimeUnit.SECONDS);
		}catch(RuntimeException e){
			errorMessage += tissueType+" cassettes printing failed.  ";
		}catch (Exception e) {
			errorMessage += tissueType+" cassettes printing timed out.  ";
		} finally{
			executorService.shutdown();
		}

		return errorMessage;
	}

	@Transactional
	public void createStandardBlocks(Fixation fixation, Person user) {
		Collection<BlockType> blockTypes = blockTypeService.fetchActiveBlockTypes(true, null);
		if (fixation.getBlocks() == null) {
		  fixation.setBlocks(new LinkedList<Block>());
		}
		Collection<Block> newBlocks = fillInBlocksFromTypeList(fixation, blockTypes);
		storeBlocks(newBlocks, user);
	}


	protected Collection<Block> fillInBlocksFromTypeList(Fixation fixation, Collection<BlockType> blockTypes) {
		Collection<Block> newBlocks = new LinkedList<Block>();
		Iterator<BlockType> typeIter = blockTypes.iterator();


		types: while (typeIter.hasNext()) {
			BlockType type = typeIter.next();

			for (Block b : fixation.getBlocks()) {
				if (b.getBlockTypeId().equals(type.getId())) {
					// we have a block of this type, we don't need another one;
					typeIter.remove();
					continue types;
				}
			}
		}

		// we've now removed all the block types that we already have from the collection
		// create block entries for the remainder
		for (BlockType type : blockTypes) {
			Block b = new Block();

			b.setBlockTypeId(type.getId());
			b.setFixationId(fixation.getId());

			newBlocks.add(b);
		}
		return newBlocks;
	}


	@Transactional
	public void storeBlocks(Collection<Block> blocks, Person user) {
		for (Block b : blocks) {
			store(b,user);
		}

	}

	/*
	 * return null if the report sent to the printer successfully
	 * return error messages failed/timed out
	 */

	public String printExtraCassettes(Long cohortId, boolean standard, CassettePrinter printer){
		Collection<Mouse> mice = mouseService.fetchByCohortId(cohortId, null);

		Collection<BlockListView> blocks = new ArrayList<BlockListView>();
		for(Mouse mouse:mice){
			blocks.addAll(fetchStandardByMouseId(mouse.getId(), standard));
		}

		return printCassettes(blocks,printer);
	}


	public BlockListView fetchBlockListViewById(Long id) {
		return blockDao.fetchBLockListViewById(id);
	}

	private class RunnableOracleReport implements Runnable {
		private String reportString;

		public RunnableOracleReport(String reportString) {
			this.reportString = reportString;
		}

		public void run() {
			String errorMessage = oracleReport.run(reportString);
			if(errorMessage != null){
				throw new RuntimeException("Report failed.");
			}

		}
	}

  public String fetchNextBlockName(Long fixationId, Long blockTypeId) {
    BlockType blockType = blockTypeService.fetchById(blockTypeId, null);

    if (blockType == null) {
      return null;
    }

    String typeName = blockType.getName();
    int nextNumber = blockDao.fetchNextNumberForBlockType(fixationId, blockTypeId);

    return typeName+'.'+nextNumber;
  }

	public void setCassettePrinterService(CassettePrinterService cassettePrinterService) {
		this.cassettePrinterService = cassettePrinterService;
	}

}
