package uk.ac.sanger.mig.histology.services;

import java.util.Collection;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletOutputStream;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.histology.CropJob;
import uk.ac.sanger.mig.histology.forms.CropJobForm;
import uk.ac.sanger.mig.personnel.Person;


public interface CropJobService {


	/**
	 * Takes a Collection of CrobJobs and returns it as a sorted List
	 * by CropJob creation date, oldest jobs first.
	 *
	 * Throws an IllegalArgumantException if cropJobs not an instance of type List.
	 *
	 * @param cropJobs
	 * @return
	 */
	public List<CropJob> ensureSortedByCreatedDate( Collection<CropJob> cropJobs );

	/**
	 * Stores a failed CropJob including reason for failure.
	 *
	 * @param cropJobId
	 * @param startTime
	 * @param e
	 * @param user
	 */
	public void storeFailedJob(Long cropJobId, Date startTime, Throwable e, Person user) ;

	/**
	 * Returns the maximum time a CropJob may run before timing out.
	 * @return
	 */
	public long getSecondsToTimeout();

	/**
	 * Returns a Collection of CropJobs awaiting for processing (i.e.,
	 * for crop extraction) ordered by the date they were created, oldest
	 * jobs first.
	 *
	 * A job 'awaiting processing' is defined as one that has not
	 * run successfully and actual attempts < maxAttempts, where
	 * maxAttempts is injected at startup.
	 *
	 * @param requiredComponents
	 * @return
	 */
	public Collection<CropJob> fetchAwaitProcessing(ComponentMap requiredComponents);



	/**
	 * Stores CropJob as record in database and returns the record's primary key
	 * @param cropJob
	 * @return
	 */
	public Long store(CropJob cropJob, Person person);

	/**
	 * Returns cropJob for a given id that maps to primary key
	 * of a record in a database table.
	 * @param id
	 * @return
	 */
	public CropJob fetchById(Long id, ComponentMap componentMap);


	public Collection<CropJob> fetchByScanId(Long scanId, ComponentMap componentMap);

	/**
	 * Creates a CropJob with a unique set of Crops for a given Scan for
	 * future processing.
	 *
	 * This method will check any sibling CropJobs on the same Scan for
	 * equivalent Crops that may already, testing equality by x origin, y origin,
	 * width, and height.  If it finds an identical that already exists Crop,
	 * it will discard it from creation in the new CropJob.
	 *
	 * Because the Scan has been selected for cropping, this
	 * method also sets it's QC status to THUMBNAIL_QC_PASS if
	 * passScanThumnail is true.
	 *
	 * Scan associated with this CropJob is automatically
	 * removed from HTL_CROP_SELCTION_BIN.
	 *
	 * Returns null if no new CropJob was created--e.g., if new Crops found
	 * for Scan.
	 *
	 * @param cropJobForm
	 * @param qcPassScanThumbnail Sets parent Scan status to qc thumbnail passed.
	 * @param user
	 * @return
	 */
	public Long create(CropJobForm cropJobForm, boolean qcPassScanThumbnail, Person user);

	/**
	 * Extracts, compresses and resizes a set of crop images from a Dmetrix slide scan.
	 * Crop images are stored on a file system in two formats: 1.) lossless
	 * LZW-compressed tif and 2.) resiized JPEG. Their absolute file paths
	 * are stored in corresponding Crop objects on the database.
	 *
	 * In addition, if the Crop is flagged for tiling, it will be tiled and its tiles
	 * stored on database.
	 *
	 * Successful extraction of all crops in a CropJob for a given Scan is treated as an
	 * atomic unit of work for transaction-control purposes. (This avoids the complexity
	 * of having to track partially cropped scans, and it avoids concurrent reads
	 * on the same scan.)
	 *
	 * If the method returns normally, *all* crops images are guaranteed to be stored
	 * and readable on the file system with their with corresponding Crop objects and
	 * file references stored on the database.
	 *
	 * IMPORTANT: 'Readable' does *not* mean the image format can necessarily be decoded,
	 * but simply that a file exists and can be read by this method.
	 *
	 * An IllegalArgumentException is thrown if Long cropJobId does not refer to
	 * a valid CropJob object on the database.
	 *
	 * @param cropJobId
	 * @param startTime
	 * @param timeout
	 * @param user
	 * @throws CropProcessingException if 1.) Failure of crop extraction,
	 * resizing, compression or tiling (optional) or 2.) Processing times
	 * out (if timeout set).
	 */
	public void run(Long cropJobId, Date startTime,
			boolean timeout, Person user) throws Exception;



}