package uk.ac.sanger.mig.histology.services;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.apache.commons.collections15.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.common.services.ImageProcessingException;
import uk.ac.sanger.mig.common.services.ImageProcessingService;
import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropJob;
import uk.ac.sanger.mig.histology.CropProcessingException;
import uk.ac.sanger.mig.histology.Scan;
import uk.ac.sanger.mig.histology.dao.CropJobDao;
import uk.ac.sanger.mig.histology.forms.CropJobForm;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class CropJobServiceImpl extends AbstractServiceImpl<CropJob> implements CropJobService {

	private CropJobDao cropJobDao;
	private ScanService scanService;
	private CropService cropService;
	private PersonService personService;
	private ComponentMapService componentMapService;
	private TileService tileService;
	private ImageProcessingService imageProcessingService;

	private String viewableRoot;
	private int maxResizeWidth;
	private int maxResizeHeight;
	private long secondsToTimeout; //seconds to timeout
	private String defaultFormatExtension;
	private int maxAttempts;

	public List<CropJob> ensureSortedByCreatedDate( Collection<CropJob> cropJobs ) {

		if(!(cropJobs instanceof List))
			throw new IllegalArgumentException("Collection<CropJob> cropJobs must be an instance of List.");

		List<CropJob> cropJobList = (List<CropJob>)cropJobs;
		Collections.sort( cropJobList, new Comparator<CropJob>() {
			public int compare(CropJob job1, CropJob job2) {
				if( job1.getCreatedDate().before( job2.getCreatedDate() ) ) {
					return -1;

				} else if( job1.getCreatedDate().after( job2.getCreatedDate() ) ) {
					return 1;

				} else {
					return 0;
				}
			} //end compare

			@Override
			public boolean equals(Object obj) {
				return super.equals(obj);
			}
		});
		return cropJobList;
	}

	public File getViewableRootDirectory() {
		File viewableRootDirectory = new File( viewableRoot );
		if ( !viewableRootDirectory.canWrite() ) {
			throw new RuntimeException("Failed to find system directory "
					+ viewableRootDirectory);
		}
		return viewableRootDirectory;
	}

	@Transactional
	public Long create(CropJobForm cropJobForm, boolean qcPassScanThumbnail, Person user) {
		//create new CropJob
		if( cropJobForm.getCrops().isEmpty() )
			throw new IllegalArgumentException("cropJobForm must have at least one Crop defined ");

		Long cropJobId = null;
		ComponentMap cropJobMap = new ComponentMap();
		cropJobMap.put(CropJob.Component.CROPS, null);
		ComponentMap scanMap = new ComponentMap();
		scanMap.put(Scan.Component.CROP_JOBS, cropJobMap);
		Scan scan = scanService.fetchById( cropJobForm.getScanId(), scanMap);
		Collection<Crop> newUniqueCrops
			= CollectionUtils.subtract(cropJobForm.getCrops(), scan.getCrops());
		logger.debug(newUniqueCrops.size() + " new unique Crops selected.");

		if( newUniqueCrops.size() != 0 ) {

			CropJob cropJob = new CropJob();
			cropJob.setScanId( cropJobForm.getScanId() );
			cropJob.setAttempts( 0 );
			cropJobId = store( cropJob, user );
			for(Crop crop : newUniqueCrops) {

				crop.setCropJobId( cropJobId );
				crop.setCreatorId( user.getId() );
				cropService.store( crop , user );
			}

		} else {

			logger.debug("No new unique Crops found, therefore no new CropJob created.");
		}

		scanService.removeFromCropSelectionBin( cropJobForm.getScanId() );
		if( qcPassScanThumbnail ) {
			scan.setStatusDictId(Scan.StatusDictEnum.THUMBNAIL_QC_PASS.getId());
			scanService.store(scan, user);
		}

		return cropJobId;
	}

	@Transactional
	public Long store(CropJob cropJob, Person user) {
		if(cropJob == null) {
			throw new RuntimeException("Cannot save a null cropJob");
		}
		return cropJobDao.store(cropJob, user);
	}

	public CropJob fetchById(Long cropJobId, ComponentMap requiredComponents) {
		CropJob cropJob = cropJobDao.fetchById(cropJobId);
		if(cropJob != null) {
			attachComponents(cropJob, requiredComponents);
		}
		return cropJob;
	}



	public Collection<CropJob> fetchAwaitProcessing(ComponentMap requiredComponents) {

		Collection<CropJob> cropJobs = cropJobDao.fetchAwaitProcessing( maxAttempts );
		if(!cropJobs.isEmpty()) {

			attachComponents( cropJobs, requiredComponents );
		}
		return cropJobs;
	}

	@Transactional
	public void run(Long cropJobId, Date startTime,
			boolean timeout, Person user) throws CropProcessingException   {

		//fetch parent hierarchy
		ComponentMap componentMap = componentMapService.getComponentMap("cropJobDetail");
		CropJob cropJob = this.fetchById( cropJobId, componentMap );

		if( cropJob == null )
			throw new IllegalArgumentException("CropJob must not be null!");

		if( cropJob.getCrops() == null)
			throw new IllegalArgumentException("CROPS component must be attached, Collection<Crop> cannot be empty or null!");

		cropJob.setStartTime( startTime );
		Boolean success = Boolean.FALSE;
		if( timeout ) {//case for single CropJob with timeout

			//we spawn thread to run synchronous thread just for time out ability
			ExecutorService  executorService = Executors.newSingleThreadExecutor();
			Future<Boolean> future = executorService.submit( new TimeoutWrapper( cropJob, user )  );
			sendOutput("CROP JOB #"+cropJob.getId()+": Starting...Job timeout in "
					+ ((float)getSecondsToTimeout())/60f + " minutes.");
			try {

				success = future.get( getSecondsToTimeout(), TimeUnit.SECONDS );

			}  catch (InterruptedException e) {
				throw new CropProcessingException( e );
			} catch (TimeoutException e) {
				throw new CropProcessingException( e );
			} catch (ExecutionException e) {
				throw new CropProcessingException( e );
			} finally {

				if( !executorService.isShutdown() ) {
					executorService.shutdownNow();
				}
			}

		} else {//case for multiple CropJobs.  Timeout handled outside this method in batch processor

			sendOutput("CROP JOB #"+cropJob.getId()+": Starting...");
			makeCrops( cropJob, user ); //run in current thread
			success = Boolean.TRUE;
		}


		cropJob.setEndTime(new Date());
		cropJob.setSuccess( success );
		if( Boolean.TRUE.equals( cropJob.getSuccess() ) )
			cropJob.setFailureReason( null );

		cropJob.incrementAttempts();
		store(cropJob, user);
		sendOutput("CROP JOB #"+cropJob.getId()+": Completed in " +
			 (	((float)( (cropJob.getEndTime().getTime() - cropJob.getStartTime().getTime() )) /1000.0f )/60.0f )	+ " minutes.");
	}

	/*
	 * NOTE: Made public (but not part of IF, to improve performance when it gets called by inner class)
	 */
	@Transactional
	public void makeCrops(CropJob cropJob, Person user) throws CropProcessingException  {

		//Build directory structure
		File scanDirectory = makeViewableScanDirectory( cropJob );
		///////////////////////////////////////////////////////////
		Collection<Crop> crops = cropJob.getCrops();
		int cropCount = 0;
		if( !crops.isEmpty() ) {

			for(Crop crop : crops) {

				cropCount++;
				String whichCrop = "Crop " + cropCount + " of " + crops.size();
				File cropDirectory = makeCropDirectory( scanDirectory, crop.getId() );
				File fullImagePath =  deriveFullImagePath( cropDirectory, crop.getId() );

				//1.) EXTRACT
				sendOutput("CROP JOB #"+cropJob.getId()+": Extracting " + whichCrop);
				extract( cropJob.getScan().getDmiFilePath(), fullImagePath, crop );
				crop.setFullImagePath( fullImagePath );

				//ensure state as if new un-QCed Crop
				crop.setQcPass( null );
				crop.setFlaggedForDelete( false );
				crop.setQcFailReason( null );
				crop.setTiled( false );
				crop.setQcLocked( false );

				/*--.) COMPRESS (make full lossless version)
				sendOutput( out, "CROP JOB #"+cropJob.getId()+": Compressing...");
				crop.setFullImagePath( fullImagePath );
				compress(tempFullCropPath, crop.getFullImagePath(),  out);
				*/

				//2.) RESIZE (make scaled web version)
				sendOutput("CROP JOB #"+cropJob.getId()+": Resizing " + whichCrop);
				File resizedImagePath =  deriveResizedImagePath( cropDirectory, crop.getId() );
				crop.setResizedImagePath( resizedImagePath );
				try {

					imageProcessingService.convert(fullImagePath,
							crop.getResizedImagePath(), getMaxResizeWidth(), getMaxResizeHeight());

				} catch(Exception e) {
					throw new CropProcessingException( e );
				}

				//4.) PUBLISH (makes files under viewable/.../cropDir/ exposed o unpriv read (eg, Apache)
				//publishViewableFiles( cropDirectory, out );

				//5.) TILE (optional tiling)
				if( crop.isFlaggedForTiling() ) {//tile the tiff

					sendOutput("CROP JOB #"+cropJob.getId()+": Tiling " + whichCrop);
					crop.setTileEdgeExponent(tileService.getDefaultEdgeExponent());
					crop.setZoomRadix(tileService.getDefaultZoomRadix());
					//defensive copy
					tileService.makeTilePyramid( (Crop) crop.clone(), user );
					crop.setTiled( true );
					crop.setFlaggedForTiling( false );

				}

				//6.) CLEAN UP (uncompressed full image file)
				sendOutput("CROP JOB #"+cropJob.getId()+": Deleting full image "+crop.getFullImagePath());
				if( crop.getFullImagePath().delete() ) {

					crop.setFullImagePath( null );
				}

				sendOutput("CROP JOB #"+cropJob.getId()+": Storing " + whichCrop);
				cropService.store( crop, user );
				sendOutput("CROP JOB #"+cropJob.getId()+": Completed " + whichCrop);
			}
		}
	}

	public Collection<CropJob> fetchByScanId(Long scanId, ComponentMap componentMap) {

		Collection<CropJob> cropJobs = cropJobDao.fetchByScanId( scanId );
		attachComponents(cropJobs, componentMap);
		return cropJobs;
	}


	@Transactional
	public void storeFailedJob(Long cropJobId, Date startTime, Throwable e, Person user) {

		CropJob cropJob = this.fetchById(cropJobId, null);
		cropJob.setSuccess( Boolean.FALSE );
		e.fillInStackTrace();//make sure we have stack trace
		cropJob.setFailureReason( e.toString() );
		cropJob.setStartTime( startTime );
		cropJob.setEndTime(new Date());
		cropJob.incrementAttempts();
		store(cropJob, user);

	}



	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// privates /////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////

	private void extract(File dmiFilePath, File tempFilePath, Crop crop)
	throws CropProcessingException {

		String command = buildCommand( dmiFilePath, tempFilePath, crop);
		Process process = null;
		BufferedReader bufferedReader = null;

		try {

			sendOutput("\n-Execute Extract Command: " + command);
			process = Runtime.getRuntime().exec( command );
			InputStreamReader inputStreamReader = new InputStreamReader( process.getInputStream() );
			bufferedReader = new BufferedReader( inputStreamReader );

			String line;
			while ((line = bufferedReader.readLine()) != null) {

					sendOutput(line );//
			}

			int exitCode = process.waitFor();
			if( exitCode != 0 ) {

				throw new CropProcessingException("\ndmiconvert process returned abnormally.\nExit code: " +  exitCode +
						"\n****Echo Command: " + command + "\n");
			}

		} catch(IOException e) {

			throw new CropProcessingException(e);

		} catch(InterruptedException e) {
			//process.waitFor() has been interrupted while blocking, eg by timeout;
			//NOTE: for now this terminates ssh, not necessarily dmiconvert!
			//need to test to see if dmiconvert getting kill signal...

			throw new CropProcessingException( e );

		}	finally {

			if( process != null ) process.destroy();
			IOUtils.closeQuietly( bufferedReader );
		}

	}

	/*
	private void compress(File src, File dest, ServletOutputStream out)
	 				throws CropProcessingException {


		String command = getRemoteSshImConvert() + " " + src + " -compress LZW " + dest;

		sendOutput( out,"\n-Execute Compress Command: " + command);

		Process process = null;

		try {

			process = Runtime.getRuntime().exec( command );

			int exitCode = process.waitFor();

			if( exitCode != 0 ) {
				//look at command for debugging
				throw new CropProcessingException("\nImageMagick convert returned abnormally.\nExit code:"
						+  exitCode + "\n****Echo Command: " + command  + "\n");
			}

		} catch(IOException e) {

			throw new CropProcessingException(e);

		} catch(InterruptedException e) {

			throw new CropProcessingException(e);

		} finally {

			if( process != null ) process.destroy();

		}

	}
*/
	/*
	private String buildCommand(File dmiFilePath, File tempFilePath, Crop crop, boolean reportProgress) {

		StringBuffer command = new StringBuffer();

		command.append( oldDmiConvert );
		command.append(" ");
		command.append("-d");
		command.append(" ");
		command.append( dmiFilePath.toString() );
		command.append(" ");
		command.append( "-m" );
		command.append(" ");
		command.append( "0" );

		command.append(" ");
		command.append("-o"); //add x,y
		command.append(" ");
		command.append(  crop.getX() + ":" + crop.getY()  );
		command.append(" ");
		command.append("-s");//width,height
		command.append(" ");
		command.append( crop.getCropWidth() + ":" + crop.getCropHeight() );
		command.append(" ");
		command.append("-c"); //output file
		command.append(" ");
		command.append( tempFilePath.toString() );
		command.append(" ");
		command.append("-p"); //stream crop progress
		command.append(" ");
		command.append( ( reportProgress )? "0" : "0" );

		return command.toString();

	}
*/
	/*
	private void publishViewableFiles(File cropDir, ServletOutputStream out) throws CropProcessingException {

		String command = getRemoteSshPublishFiles() + " " + cropDir + "/*";

		sendOutput( out, "\n-Execute Publish Command: " + command + "\n");

		Process process = null;

		try {

			process = Runtime.getRuntime().exec( command );

			int exitCode = process.waitFor();

			if( exitCode != 0 ) {
				//look at command for debugging
				throw new CropProcessingException("\nchmod returned abnormally.\nExit code:"
						+  exitCode + "\n****Echo Command: " + command + "\n");
			}

		} catch(IOException e) {

			throw new CropProcessingException(e);

		} catch(InterruptedException e) {

			throw new CropProcessingException(e);

		} finally {

			if( process != null ) process.destroy();

		}

	}*/


	private String buildCommand(File dmiFilePath, File tempFilePath,
			Crop crop) {

		StringBuilder command = new StringBuilder();

		command.append( scanService.getRemoteSshDmiconvert() );
		command.append(" ");
		command.append("-tif"); //add x,y
		command.append(" ");
		command.append("-offset"); //add x,y
		command.append(" ");
		command.append(  crop.getX() + ":" + crop.getY()  );
		command.append(" ");
		command.append("-size");//width,height
		command.append(" ");
		command.append( crop.getCropWidth() + ":" + crop.getCropHeight() );
		command.append(" ");
		command.append("-out"); //destination file
		command.append(" ");
		command.append( tempFilePath.toString() );
		command.append(" ");
		command.append( dmiFilePath.toString() ); //source file
		return command.toString();

	}


	private File makeViewableScanDirectory(CropJob  cropJob) {
		File scanDirectory = new File(
				      scanService.getIndexedSubDir( getViewableRootDirectory() ) +
				      File.separator +
					  cropJob.getScan().getSlide().getBarcode() +
					  File.separator +
					  "scan" + cropJob.getScan().getId());
		if( !scanDirectory.exists() &&  !scanDirectory.mkdirs()) {
			throw new RuntimeException("Failed to create scanDirectory " + scanDirectory);
		}
		return scanDirectory;
	}

	private File makeCropDirectory(File scanDirectory, Long cropId) {
		File cropDirectory = new File( scanDirectory + File.separator + "crop" + cropId);
		logger.debug("\n-Making Crop Directory: " + cropDirectory);
		if( !cropDirectory.exists() &&  !cropDirectory.mkdirs()) {
			throw new RuntimeException("Failed to create cropDirectory " + cropDirectory);
		}
		return cropDirectory;

	}

	/*
	private File deriveTempImagePath(File tempImageDir, Long cropId) {

		return new File(tempImageDir  +  File.separator + "temp" + getDefaultFormatExtension());
	}
	*/

	private File deriveResizedImagePath(File cropDirectory, Long cropId) {
		return new File(cropDirectory  +  File.separator + "c" + cropId + "resized.jpg");
	}

	private File deriveFullImagePath(File cropDirectory, Long cropId) {
		return new File(cropDirectory  +  File.separator + "c" + cropId
				+ "full.tif");
	}




	@Override
	protected void attachComponents(CropJob cropJob, ComponentMap requiredComponents) {
		if(cropJob == null) {
			throw new RuntimeException("Cannot attach components to null cropJob");
		}
		boolean fetchAll = false;
		if(requiredComponents != null) {

			if(requiredComponents.containsKey(CropJob.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(CropJob.Component.CROPS)) {
				cropJob.setCrops(fetchCropsByCropJobId(
						cropJob.getId(),
						requiredComponents.get(CropJob.Component.CROPS)));
			}

			if(fetchAll || requiredComponents.containsKey(CropJob.Component.CREATOR)) {
				cropJob.setCreator(fetchByCreatorId(
						cropJob.getCreatorId(),
						requiredComponents.get(CropJob.Component.CREATOR)));
			}

			if(fetchAll || requiredComponents.containsKey(CropJob.Component.SCAN)) {
				cropJob.setScan(fetchScanById(
						cropJob.getScanId(),
						requiredComponents.get(CropJob.Component.SCAN)));
			}
		}
	}

	private Scan fetchScanById(Long scanId, ComponentMap requiredComponents) {
		return scanService.fetchById(scanId, requiredComponents);
	}

	private Person fetchByCreatorId(Long creatorId, ComponentMap requiredComponents) {
		return personService.fetchById(creatorId, requiredComponents);
	}

	private Collection<Crop> fetchCropsByCropJobId(Long cropJobId, ComponentMap requiredComponents) {
		return cropService.fetchCropsByCropJobId(cropJobId, requiredComponents);
	}


	/*
	 * Inner class to process CropJob Job on one Scan in separate
	 * thread.
	 */
	private class TimeoutWrapper implements Callable<Boolean> {

		private CropJob cropJob;
		private Person user;

		public TimeoutWrapper(CropJob cropJob, Person user) {
			this.cropJob = cropJob;
			this.user = user;
		}

		public Boolean call()
		 	throws CropProcessingException {
				makeCrops( cropJob, user );
				return Boolean.TRUE;
		}

	}


	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// injections /////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////

	public void setCropJobDao(CropJobDao cropJobDao) {
		this.cropJobDao = cropJobDao;
	}

	public void setCropService(CropService cropService) {
		this.cropService = cropService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setViewableRoot(String viewableRoot) {
		this.viewableRoot = viewableRoot;
	}


	public void setScanService(ScanService scanService) {
		this.scanService = scanService;
	}

	public int getMaxResizeHeight() {
		return this.maxResizeHeight;
	}

	public void setMaxResizeHeight(int maxResizeHeight) {
		this.maxResizeHeight = maxResizeHeight;
	}

	public int getMaxResizeWidth() {
		return this.maxResizeWidth;
	}

	public void setMaxResizeWidth(int maxResizeWidth) {
		this.maxResizeWidth = maxResizeWidth;
	}


	public long getSecondsToTimeout() {
		return this.secondsToTimeout;
	}

	public void setSecondsToTimeout(long secondsToTimeout) {

		if( secondsToTimeout == 0 )
			throw new IllegalArgumentException("secondsToTimeout must be set > 0!");

		this.secondsToTimeout = secondsToTimeout;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setTileService(TileService tileService) {
		this.tileService = tileService;
	}


	public String getDefaultFormatExtension() {
		return this.defaultFormatExtension;
	}

	public void setDefaultFormatExtension(String defaultFormatExtension) {
		this.defaultFormatExtension = defaultFormatExtension;
	}

	public void setMaxAttempts(int maxAttempts) {
		this.maxAttempts = maxAttempts;
	}


	public void setImageProcessingService(
			ImageProcessingService imageProcessingService) {
		this.imageProcessingService = imageProcessingService;
	}

	private void sendOutput(String line) {
			logger.info( line );
	}


}


