package uk.ac.sanger.mig.histology.services;

import java.net.URL;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.xml.rpc.ServiceException;

import uk.ac.sanger.mig.annotation.Annotation;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropSearchBean;
import uk.ac.sanger.mig.histology.views.CropView;
import uk.ac.sanger.mig.personnel.Person;

public interface CropService {

	public enum QcFailAction {

		TRY_FAILING_AND_REORDER_SLIDE("Try Fail/Re-Order Slide",
				"Places an order for Slide to be re-sectioned with same staining from its originating Block. " +
				"Slide will be QC-failed only if it has no QC-passed Regions. Non-QC-ed Regions will be automatically failed"),

		TRY_FAILING_AND_REORDER_SCAN("Try Fail/Re-Order Scan",
				"Places an order for Slide to be re-scanned. Scan will be QC-failed only if it has no " +
				"QC-passed Regions. Non-QC-ed Regions will be automatically failed."),

		RESELECT_REGIONS("Re-Select Regions",
				"Places Scan back into Region selection bin. "),

		MARK_REDUNDANT("Redundant",
				"Marks region as being redundant -- i.e., another equivalent already exists.")

		;

		private String label;
		private String description;

		private QcFailAction(String label, String description) {this.label = label; this.description = description;}

		public String getLabel() {return this.label;}

		public String getDescription() { return this.description;}

		public String getEnumString() { return  this.name();}

	}

	/**
	 * Stores a Crop updated with selected Annotations.
	 *
	 * @param crop
	 * @param selectedAnnotations
	 * @param user
	 * @return
	 */
	public Long storeAnnotated(Crop crop, Collection<Annotation> selectedAnnotations, Person user);

	/**
	 * QC-passes a Crop region.
	 *
	 * Also, ensures Crop is not flagged for delete
	 * and clears its failure reason, if any.
	 *
	 * @param crop
	 * @param user
	 * @return id of the passed Crop
	 * @throws IllegalArgumentException if Crop is null.
	 */
	public Long qcPass(Crop crop, Person user);

	/**
	 * QC Fails a Crop, setting failure reason, ensuring
	 * it is *not* in an externally approved state.
	 *
	 * In addition, if failed Crop was previously flagged
	 * for tiling, it will be un-flagged.
	 *
	 * @param crop
	 * @param qcFailReason
	 * @param user
	 * @return id of the failed Crop
	 * @throws IllegalArgumentException if Crop is null.
	 */
	public Long qcFail(Crop crop, String qcFailReason, Person user) ;

	/**
	 * Sets Crop state to QC-locked. This prevents the user from
	 * creating an inconsistent state if he/she tries to QC-pass
	 * the child Crop of an already-failed Scan.
	 *
	 * @param crop
	 * @param user
	 * @return id of the locked Crop
	 * @throws IllegalArgumentException if Crop is null.
	 */
	public Long qcLock(Crop crop, Person user);

	/**
	 * Fails a Crop only if it has not been passed --ie, QcPass == null or false.
	 *
	   If successfully failed, the Crop, its image, and any tiles, will be deleted.
	 *
	 * Returns true of Crop is good--ie, failure attempt did not succeed.
	 *
	 * @param crop
	 * @param scanId
	 * @param user
	 * @return true of failing succeeds.
	 */
	public boolean gentlyFail(Crop crop, Long scanId, Person user);



	/**
	 * Initiates additional work in connection with the failed Crop region
	 * and all its siblings depending on whether the cause of failure
	 * indicated by the QcFailAction argument was due to a bad Slide,
	 * a bad Scan, a bad cropping or a redundant Crop.
	 *
	 * If a bad Slide or bad Scan, the Slide or Scan will be re-ordered and
	 * an attempt will be made to fail the Slide or Scan.
	 *
	 * If a bad cropping, the Scan will be scheduled for re-cropping.
	 *
	 * If a redundant Crop, the Crop comments field will be appended with
	 * the text 'Marked Redundant'.
	 *
	 * @param crop
	 * @param qcFailAction
	 * @param user
	 * @throws IllegalArgumentException if crop is null.
	 * @throws RuntimeException if Crop is QC-passed or locked.
	 */
	public void initiateQcFailAction(Crop crop, QcFailAction qcFailAction, Person user);

	public URL getViewableRootUrl();

	public Collection<CropView> search(CropSearchBean searchBean);

	/**
	 * Returns a BinaryFile object holding image data for
	 * a particular Crop. Returns null if no Crop exists
	 * or Crop exists but has no file path to the image.
	 *
	 * @param id
	 * @return
	 */
	public BinaryFile fetchResizedImageByCropId(Long id);

	/**
	 * Stores Crop as record in database and returns the record's primary key
	 * @param crop
	 * @return
	 */
	public Long store(Crop crop, Person person);

	/**
	 * Returns crop for a given id that maps to primary key
	 * of a record in a database table.
	 * @param id
	 * @return
	 */
	public Crop fetchById(Long id, ComponentMap componentMap);

	/**
	 * Fetches all Crops associated with s given CropJob.
	 * @param cropJobId
	 * @param map
	 * @return
	 */
	public Collection<Crop> fetchCropsByCropJobId(Long cropJobId, ComponentMap map);


	/**
	 * Returns List of all Crop ids for a given mouse ordered
	 * by block_id, slide_id, cj.scan_id, and y_location
	 *
	 *
	 * @param mouseId
	 * @return
	 */
	public List<Long> fetchCropIdsByMouse(Long mouseId);

}