package uk.ac.sanger.mig.histology.services;

import java.io.File;
import java.net.URL;
import java.util.Collection;
import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.annotation.Annotation;
import uk.ac.sanger.mig.annotation.services.AnnotationService;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.dao.BinaryFileDao;
import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropJob;
import uk.ac.sanger.mig.histology.CropSearchBean;
import uk.ac.sanger.mig.histology.Scan;
import uk.ac.sanger.mig.histology.dao.CropDao;
import uk.ac.sanger.mig.histology.views.CropView;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;


public class CropServiceImpl extends AbstractServiceImpl<Crop> implements
		CropService {

	private CropDao cropDao;
	private CropJobService cropJobService;
	private PersonService personService;
	private URL viewableRootUrl;
	private ScanService scanService;
	private SlideService slideService;
	private AnnotationService annotationService;
	private BinaryFileDao binaryFileDao;

	@Transactional
	public Long storeAnnotated(Crop crop, Collection<Annotation> selected, Person user) {

		annotationService.updateAll(crop.getAnnotations(), selected, 
				TableMetadataEnum.HTL_CROP, user);

		return store(crop, user);
	}

	@Transactional
	public void initiateQcFailAction(Crop crop, QcFailAction qcFailAction, Person user) {

		if( crop== null || qcFailAction == null)
			throw new IllegalArgumentException("Neither crop nor qcFailAction may be null.");

		if( Boolean.TRUE.equals ( crop.getQcPass() ) )
			throw new RuntimeException("Cannot initiate QC fail action on QC-passed Crop.");

		if( crop.isQcLocked() )
			throw new RuntimeException("Cannot initiate QC fail action on QC-locked Crop");

		switch( qcFailAction ) {

			case TRY_FAILING_AND_REORDER_SLIDE:

				slideService.gentlyFailAndReorder( crop.getCropJob().getScan().getSlideId(), user);

				break;

			case TRY_FAILING_AND_REORDER_SCAN:

				scanService.gentlyFailAndReorder( crop.getCropJob().getScanId(), user) ;

				break;

			case RESELECT_REGIONS:

				scanService.addToCropSelectionBin(crop.getCropJob().getScanId(), user,
						Scan.StatusDictEnum.FLAGGED_FOR_RESELECTION.getId() );

				break;

			case MARK_REDUNDANT:

				markRedundant( crop, user );

				break;

			default:
				throw new RuntimeException("Invalid QcFailAction.");
		}

	}



	@Transactional
	public Long qcLock(Crop crop, Person user) {

		if( crop == null )
			throw new IllegalArgumentException("Crop cannot be null.");

		crop.setQcLocked( true );
		return store(crop, user);
	}

	@Transactional
	public boolean gentlyFail(Crop crop, Long scanId, Person user) {

		boolean failedOK = true;

		if( Boolean.TRUE.equals( crop.getQcPass() ) ) {

			failedOK = false;

		} else {

			qcFail(crop, "Bad Scan/Slide", user);
		}

		return failedOK;
	}

	@Transactional
	public Long qcFail(Crop crop, String qcFailReason, Person user) {

		if( crop == null )
			throw new IllegalArgumentException("Crop cannot be null.");

		crop.setExternalApproved( false );
		crop.setQcPass( Boolean.FALSE );
		crop.setQcFailReason( qcFailReason );
		crop.setFlaggedForTiling( false );

		return store( crop, user );
	}

	@Transactional
	public Long qcPass(Crop crop, Person user) {

		if( crop == null )
			throw new IllegalArgumentException("Crop cannot be null.");

		crop.setFlaggedForDelete( false );
		crop.setQcPass( Boolean.TRUE );
		crop.setQcFailReason( null );

		return store( crop, user );
	}


	public Collection<CropView> search(CropSearchBean searchBean) {

		return cropDao.search( searchBean );
	}

	public BinaryFile fetchResizedImageByCropId(Long id) {

		Crop crop = cropDao.fetchById(id);

		if( crop == null || crop.getResizedImagePath() == null ) {
			return null;
		}

		File path = crop.getResizedImagePath();

		String contentType = "image/"+ path.getName().substring(
				path.getName().lastIndexOf("." ) + 1 ) ;

		BinaryFile binaryFile = binaryFileDao.fetchByPath(path.toString(), contentType);

		binaryFile.setFileDescription( path.toString() );

		return binaryFile;

	}


	public List<Long> fetchCropIdsByMouse(Long mouseId) {

		return cropDao.fetchCropIdsByMouse(mouseId);
	}

	public Crop fetchById(Long cropId, ComponentMap requiredComponents) {
		Crop crop = cropDao.fetchById(cropId);

		if (crop != null)
			attachComponents(crop, requiredComponents);

		return crop;
	}




	@Transactional
	public Long store(Crop crop, Person user) {

		if (crop == null) {
			throw new RuntimeException("Cannot save a null crop.");
		}

		return cropDao.store(crop, user);

	}

	public Collection<Crop> fetchCropsByCropJobId(Long cropJobId, ComponentMap requiredComponents) {

		Collection<Crop> crops = cropDao.fetchCropsByCropJobId(cropJobId);

		attachComponents(crops, requiredComponents);

		return crops;
	}

	@Override
	protected void attachComponents(Crop crop, ComponentMap requiredComponents) {
		if (crop == null) {
			throw new RuntimeException("Cannot attach components to null crop");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(Crop.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Crop.Component.CROP_JOB)) {
				crop.setCropJob( fetchCropJobById(
						crop.getCropJobId(),
						requiredComponents.get(Crop.Component.CROP_JOB)));
			}

			if(fetchAll || requiredComponents.containsKey(Crop.Component.CREATOR)) {
				crop.setCreator(fetchCreatorById(
						crop.getCreatorId(),
						requiredComponents.get(Crop.Component.CREATOR)));
			}

			if(fetchAll || requiredComponents.containsKey(Crop.Component.ANNOTATIONS)) {
				crop.setAnnotations(fetchAnnotationsByTableAndForeignKey(
						TableMetadataEnum.HTL_CROP,
						crop.getId(),
						requiredComponents.get(Crop.Component.ANNOTATIONS)));
			}

		}
	}

	private Collection<Annotation> fetchAnnotationsByTableAndForeignKey(TableMetadataEnum table, Long foreignKeyId, ComponentMap requiredComponents) {

		return annotationService.fetchByTableAndForeignKey(table, foreignKeyId, requiredComponents);

	}

	private Person fetchCreatorById(Long creatorId, ComponentMap requiredComponents) {
		return personService.fetchById(creatorId, requiredComponents);
	}

	private CropJob fetchCropJobById(Long cropJobId, ComponentMap requiredComponents) {
		return cropJobService.fetchById(cropJobId, requiredComponents);
	}

	private Long markRedundant(Crop crop, Person user) {

		String comments = ( crop.getComments() == null ) ? "" : crop.getComments();

		crop.setComments( comments + "***" +
				CropService.QcFailAction.MARK_REDUNDANT.getLabel());

		return store( crop, user );
	}
	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// dependencies /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	public void setCropDao(CropDao cropDao) {
		this.cropDao = cropDao;
	}

	public void setCropJobService(CropJobService cropJobService) {
		this.cropJobService = cropJobService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public URL getViewableRootUrl() {
		return this.viewableRootUrl;
	}

	public void setViewableRootUrl(URL viewableRootUrl) {
		this.viewableRootUrl = viewableRootUrl;
	}

	public void setScanService(ScanService scanService) {
		this.scanService = scanService;
	}



	public void setSlideService(SlideService slideService) {
		this.slideService = slideService;
	}

	public void setAnnotationService(AnnotationService annotationService) {
		this.annotationService = annotationService;
	}

	public void setBinaryFileDao(BinaryFileDao binaryFileDao) {
		this.binaryFileDao = binaryFileDao;
	}




}
