package uk.ac.sanger.mig.histology.services;

import java.io.File;
import java.util.Collection;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.histology.Scan;
import uk.ac.sanger.mig.histology.SweepScanException;
import uk.ac.sanger.mig.histology.views.SerializableScanView;
import uk.ac.sanger.mig.personnel.Person;

public interface ScanService {

	/**
	 * Adds a Scan to crop selection bin if it has no already been added.
	 *
	 * @param scanId
	 * @param user TODO
	 * @param statusId TODO
	 * @return id of the scan added to the Crop selection bin.
	 */
	public Long addToCropSelectionBin(Long scanId, Person user, Long statusId);

	/**
	 * Returns the max number of files allowed in an indexed directory.
	 *
	 * @return
	 */
	public int getDefaultIndexDirectoryCapacity();

	/**
	 * Returns the highest indexed directory with < 1000 files underneath
	 * parentDir. If the highest indexed directory is full (having >= 1000 files)
	 * a new empty index directory is created with incremented index.
	 *
	 * @param parentDir
	 * @return
	 */
	public File getIndexedSubDir(File parentDir);


	/**
	 * Removes Scan record from HTL_CROP_SELECTION_BIN
	 * @param scanId
	 * @return
	 */
	public Scan removeFromCropSelectionBin(Long scanId);


	/**
	 * QC-fails a Scan if it has no QC-passed child Crops, returning
	 * true if failure succeeded, false otherwise.
	 *
	 * If the Scan does fail, it will be guaranteed not to be
	 * scheduled for Crop Selection, its status will be set
	 * to Scan.StatusDictEnum cause, and all if its child crops
	 * will be QC-locked.
	 *
	 * If the Scan does not fail, its QC status will remain unchanged
	 * and it will neither be added nor removed from crop selection.
	 *
	 * All non-QC-passed child Crops are failed regardless
	 * of whether Scan succeeds in failing.
	 *
	 * @param id - Scan id.
	 * @param cause - The reason for the failure (e.g., bad scanning or bad sectioning)
	 * @param user
	 * @return
	 * @Throws IllegalArgumentException if cause or user is null.
	 * @Throws RuntimeException if Scan cannot be found for the given ID.
	 */
	public boolean gentlyFail(Long id, Scan.StatusDictEnum cause, Person user);


	/**
	 *
	 * Re-orders the scanning of a particular Slide and fails
	 * the Scan in question if it has no QC-passed Crops.
	 *
	 * The parent Slide will be added to the re-scan bin and
	 * its status set to AWAITING_SCAN.
	 *
	 * @param scanId
	 * @param user
	 * @return Returns true if failure succeeded.
	 * @throws IllegalArgumentException if scanId is null.
	 */
	public boolean gentlyFailAndReorder(Long scanId, Person user);

	/**
	 * Stores a simple SlideScan object.
	 * Throws IllegalArgumentException if slide is null.
	 * @param scan
	 * @param user
	 * @return
	 */
	public Long store(Scan scan, Person user);

	/**
	 * Returns list of all *.dmi extension files under a sweep directory.
	 * @return
	 */
	public String[] fetchSweepFiles();


	/**
	 * Extracts metadata & thumbnail from a Dmetrix scan image into a Scan object
	 * on the database.
	 *
	 * If the method returns normally, a Dmetrix scan image (*.dmi) will be stored
	 * and readable on a file system with a reference file path in the MGC
	 * database (in the Scan object).
	 *
	 * If called in a transaction and database updates get rolled back on transaction
	 * failure -- i.e., either an unchecked exception or a SweepScanException
	 * gets thrown -- file system artifacts are not guaranteed to be removed.

	 * A SweepScanException will cause this method to fail if any of the following occur:
	 *
	 * 1.) The Dmetrix image either does not exist or cannot be read
	 * 2.) A barcoded Slide cannot be linked to a Mouse on MGC database.
	 * 3.) Metadata and/or thumbnail extraction fails.
	 * 4.) The Dmetrix image is unsuccessfully re-located (renamed) on the file system.
	 *
	 * @param filName The file name of the Dmetrix image.
	 * @param user
	 * @return Returns true if file was successfully swept and false if file was ignored.
	 * @throws SweepScanException
	 */
	public void process(String fileName, Person user)
		throws SweepScanException;


	/**
	 * Returns SlideScan for a given id that maps to primary key
	 * of a record in a database table.
	 * @param id
	 * @return
	 */
	public Scan fetchById(Long id, ComponentMap componentMap);

	/**
	 * Returns File object representing path to the sweep directory holding unprocessed
	 * *.dmi image files (ie, whose metadata & thumnails have *not* been
	 * extracted and 'swept' into th MGC database) .
	 * @return
	 */
	public File getSweepDirectory();

	/**
	 * Returns File object representing path to the parent directory holding
	 * processed *.dmi files (ie, whose metadata & thumbnails have been
	 * successfully extracted and 'swept' into the MGC database)  .
	 * @return
	 */
	public File getDmiRootDirectory();

	/**
	 * Returns the file extension for Dmetrix images.
	 * @return
	 */
	public String getDmiImageExtension();

	/**
	 * Returns remote ssh dmiconvert command.
	 * @return
	 */
	public String getRemoteSshDmiconvert();

	/**
	 * Returns all Scans swept into database on a given date.
	 * @param sweepDate
	 * @param componentMap
	 * @return
	 */
	//public Collection<Scan> fetchBySweepDate(Date sweepDate, ComponentMap componentMap);

	/**
	 * Fetches Scans FLAGGED_FOR_CROP_SELECTION
	 * @return
	 */
	public Collection<SerializableScanView> fetchCropSelectionBinSerialized();


	/**
	 * Returns the Scan associated with the Crops in a given CropJob. (NOTE: All
	 * Crops in a CropJob always associate with a single Scan.)
	 * @param cropJobId
	 * @param componentMap
	 * @return
	 */
	public Scan fetchByCropJobId(Long cropJobId, ComponentMap componentMap);

	/**
	 * Returns all Scans associated with a given Slide.
	 * @param slideId
	 * @param componentMap
	 * @return
	 */
	public Collection<Scan> fetchAllBySlideId(Long slideId, ComponentMap componentMap);
}