
package uk.ac.sanger.mig.histology.services;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.apache.commons.collections15.BidiMap;
import org.apache.commons.collections15.bidimap.DualTreeBidiMap;
import org.apache.commons.io.IOUtils;
import org.apache.commons.io.filefilter.PrefixFileFilter;
import org.apache.commons.io.filefilter.SuffixFileFilter;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.DataBinder;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropJob;
import uk.ac.sanger.mig.histology.Scan;
import uk.ac.sanger.mig.histology.ScanSearchBean;
import uk.ac.sanger.mig.histology.Slide;
import uk.ac.sanger.mig.histology.SweepScanException;
import uk.ac.sanger.mig.histology.dao.ScanDao;
import uk.ac.sanger.mig.histology.views.SerializableScanView;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class ScanServiceImpl extends AbstractServiceImpl<Scan> implements ScanService {

	private ScanDao scanDao;
	private SlideService slideService;
	private BinaryFileService binaryFileService;
	private CropService cropService;
	private CropJobService cropJobService;
	private ComponentMapService componentMapService;
	private PersonService personService;

	private String dmiImageExtension;
	private String remoteSshDmiconvert;
	private String sweepDirectoryStr;
	private String dmiRootDirectoryStr;
	private String indexDirectoryPrefix;
	private int maxIndexDirectoryCapacity;
	private String dmiThumbExtension;
	private String dmiFieldSeparator;
	private SimpleDateFormat dmiDateFormat;

	//we want to guarantee keys mapped to unique values
	private final BidiMap<String, String> dmetrixToMigBeanFields
			= new DualTreeBidiMap<String, String>();

	public ScanServiceImpl() {

		super();

		//dmetrix does not comply w/JavaBean spec., so must map them to Scan
		dmetrixToMigBeanFields.put("DM::WhoScanned", "scanPersonName");
		dmetrixToMigBeanFields.put("DM::ScanTime", "scanDate");

		dmetrixToMigBeanFields.put("DM::ThumbnailWidth", "thumbWidth");
		dmetrixToMigBeanFields.put("DM::ThumbnailHeight", "thumbHeight");

		dmetrixToMigBeanFields.put("DM::ScanWidth", "width");
		dmetrixToMigBeanFields.put("DM::ScanHeight", "height");

		dmetrixToMigBeanFields.put("DM::PixelWidthMicrons", "pixelWidth");
		dmetrixToMigBeanFields.put("DM::PixelHeightMicrons", "pixelHeight");
	}


	@Transactional
	public boolean gentlyFail(Long id, Scan.StatusDictEnum cause, Person user) {

		if( cause == null || user == null )
			throw new IllegalArgumentException("Scan.StatusDictEnum cause and Person user both must be not null.");

		ComponentMap cropJobComponents = new ComponentMap();
		cropJobComponents.put(CropJob.Component.CROPS, null);
		ComponentMap scanComponents = new ComponentMap();
		scanComponents.put(Scan.Component.CROP_JOBS, cropJobComponents);

		Scan scan = fetchById(id, scanComponents);

		if( scan == null )
			throw new RuntimeException("Unable to find Scan to fail having ID: " + id);

		//fail offspring
		Collection<CropJob> cropJobs = scan.getCropJobs();

		boolean failedOK = true;

		for(CropJob cropJob : cropJobs) {

			for(Crop crop : cropJob.getCrops()) {

				//just need one that did not fail to block failure and lock-down
				if( !cropService.gentlyFail(crop, cropJob.getScanId(), user) ) {

					failedOK = false;
				}
			}
		}

		if( failedOK ) {//we can safely fail scan and lock down children

			qcFail(scan, cause, user);

			for(CropJob cropJob : cropJobs) {

				for(Crop crop : cropJob.getCrops()) {

					cropService.qcLock( crop, user );

				}
			}
		}

		return failedOK;
	}

	@Transactional
	public boolean gentlyFailAndReorder(Long scanId,  Person user) {

		if(scanId == null )
			throw new IllegalArgumentException("scanId cannnot be null.");

		Scan scan = fetchById(scanId, null);

		//parent slide should be await scan if we are re-using it
		Slide slide = slideService.fetchById(scan.getSlideId(), null);

		slide.setStatusDictId(Slide.StatusDictEnum.AWAITING_SCAN.getId());

		slideService.store(slide, user);

		slideService.addToScanningBin( scan.getSlideId() );

		return gentlyFail( scan.getId(), Scan.StatusDictEnum.THUMBNAIL_QC_FAIL_BAD_SCAN, user );
	}

	private void qcFail(Scan scan, Scan.StatusDictEnum cause, Person user) {

		removeFromCropSelectionBin( scan.getId() );

		scan.setStatusDictId( cause.getId() );

		store( scan, user );

	}

	public Collection<Scan> fetchAllBySlideId(Long slideId, ComponentMap componentMap) {

		ScanSearchBean searchBean = new ScanSearchBean();

		searchBean.setSlideId(slideId);

		Collection<Scan> scans = scanDao.fetch( searchBean );

		attachComponents( scans, componentMap);

		return scans;
	}

	public Scan fetchByCropJobId(Long cropJobId, ComponentMap requiredComponents) {

		Scan scan = scanDao.fetchByCropJobId(cropJobId);

		if( scan != null ) {
			attachComponents(scan, requiredComponents);
		}

		return scan;
	}

	@Transactional
	public Scan removeFromCropSelectionBin(Long scanId) {
		return scanDao.removeFromCropSelectionBin(scanId);
	}

	@Transactional
	public Long addToCropSelectionBin(Long scanId, Person user, Long statusId) {

		//parent scan should be THUMBNAIL_QC_PASS if we are re-using it
		Scan scan = fetchById( scanId, null);

		if( scan == null )
			throw new RuntimeException("Could not find Scan with ID: " + scanId);

		scan.setStatusDictId( statusId );

		Long id = store( scan, user );

		logger.debug("\n\nFlagging scan for (re)select: " + statusId);

		if( !scanDao.existsInCropSelectionBin(scanId) )
			scanDao.addToCropSelectionBin(scanId);

		return id;
	}

	public Collection<SerializableScanView> fetchCropSelectionBinSerialized() {

		return scanDao.fetchCropSelectionBinSerialized();

	}


	@Transactional
	public void process(String fileName, Person user)
			throws SweepScanException {

		File sweepFilePath = new File( getSweepDirectory() + File.separator  + fileName );

		if( !sweepFilePath.canWrite() ) {

			throw new SweepScanException("Cannot read/write to " + sweepFilePath.toString());
		}

		String barcode = parseBarcode( fileName ); //try to get parent Slide barcode from file name

		//need slideDetail ComponentMap so we can validate object tree
		//up through to Mouse
		Slide slide = slideService.fetchByBarcode( barcode,
				componentMapService.getComponentMap("slideToMouseValidation")  );

		//2 a.
		validate( slide );

		File dmiSlideSubDir = new File( getIndexedSubDir( getDmiRootDirectory() ) + File.separator + slide.getBarcode() );

		slide.setStatusDictId( Slide.StatusDictEnum.SCANNED.getId() );

		Long slideId = slideService.store(slide, user);

		//remove  slide with this id from slide_scanning_bin, b/c we know it's scanned **********************
		slideService.removeFromScanningBin( slideId );

		Scan scan = buildNewScan( slide );

		File tempThumbFilePath = new File( getSweepDirectory().getAbsolutePath()
				+ File.separator  + deriveThumbName( sweepFilePath.getName() ));

		//2 c/d.
		extractMetadata( scan, sweepFilePath, tempThumbFilePath );

		scan.getThumbnail().setFileName( tempThumbFilePath.getName() );

		//e. flag for crop selection
		Long scanId = storeWithThumbnail( scan, user );

		addToCropSelectionBin( scanId, user,
				Scan.StatusDictEnum.AWAITING_THUMBNAIL_QC.getId() );

		//f. cleanup
		File dmiScanDir = new File(dmiSlideSubDir + File.separator + "scan"+ scanId);

		File dmiScanPath = cleanUp( sweepFilePath , tempThumbFilePath, dmiScanDir);

		//update scan dmi file path
		scan = fetchById( scanId, null );

		//update with file path we've created after clean up move
		scan.setDmiFilePath( dmiScanPath );
		store( scan, user );

		/*
		 * final check of image can be read on file system after
		 * all database transactions have returned  normally.
		 *
		 * This will roll them back if image not found on file system.
		 */
		if( !dmiScanPath.canRead() )
			throw new SweepScanException("Dmi image unsuccessfully moved. Cannot read " + dmiScanPath.toString());

	}

	/*
	 * Returns the highest indexed directory with < 1000 files.
	 * If the highest indexed directory is full (having >= 1000 files)
	 * a new empty index directory is created with incremented index.
	 */
	public File getIndexedSubDir(File parentDir) {

		File indexDir = null;

		FilenameFilter filter = new PrefixFileFilter( indexDirectoryPrefix );

		File[] files = parentDir.listFiles( filter );

		if( files.length > 0 ) {

			List<File> fileList = Arrays.asList( files );

			File highestIndexDir = Collections.max( fileList, new Comparator<File>() {

				public int compare(File f1, File f2)  {

					int i1 = getDirectoryIndex( f1 );
					int i2 = getDirectoryIndex( f2 );

					if( i1 < i2 ) return -1;
					if( i1 > i2 ) return 1;

					return 0;

				}
				//don't need: compares two Comparators
				public boolean equals(Object o) { return false; }

			});

			//check we're not full
			if( highestIndexDir.list().length  < getDefaultIndexDirectoryCapacity() ) {

				indexDir = highestIndexDir;

			} else {//otherwise create new index dir incremented by one

				indexDir = new File( parentDir + File.separator
						+ indexDirectoryPrefix + ( getDirectoryIndex( highestIndexDir ) + 1) );
			}

		} else {//create the very first index file
			indexDir = new File( parentDir + File.separator + indexDirectoryPrefix + 0 );
		}

		if(!indexDir.exists() && !indexDir.mkdirs() ) {

			throw new RuntimeException("Failed to create indexDir " + indexDir);

		}

		return indexDir;
	}


	private int getDirectoryIndex(File f) {

		if( !f.isDirectory() || !f.getName().startsWith( indexDirectoryPrefix ))
			throw new IllegalArgumentException("File must be an index directory!");

		String indexStr = f.getName().split("_")[1];

		return Integer.parseInt( indexStr );
	}


	private void validate(Slide slide) throws  SweepScanException {

		if( slide == null )
			throw new SweepScanException("No Slide found for the Scan.");

		if( slide.getBlock() == null)
			throw new SweepScanException("Slide missing Block.");

		if( slide.getBlock().getFixation() == null)
			throw new SweepScanException("Block missing Fixation.");

		if( slide.getBlock().getFixation().getNecropsy() == null)
			throw new SweepScanException("Fixation missing Necropsy.");

		if( slide.getBlock().getFixation().getNecropsy().getMouse() == null)
			throw new SweepScanException("Necropsy missing a Mouse.");
	}

	private Long storeWithThumbnail(Scan slideScan, Person user) {

		if(slideScan.getThumbnail() == null)
			throw new IllegalArgumentException("Slide thumbnail cannot be null!");

		Long imageId = binaryFileService.store(slideScan.getThumbnail(), user);
		slideScan.setThumbnailImageId( imageId );
		return store( slideScan, user );
	}

	@Transactional
	public Long store(Scan scan, Person user) {

		if(scan == null) {
			throw new IllegalArgumentException("Cannot save a null slide");
		}

		return scanDao.store(scan, user);
	}


	public String[] fetchSweepFiles() {

		FilenameFilter fileNameFilter = new SuffixFileFilter( getDmiImageExtension() );

		return  getSweepDirectory().list( fileNameFilter );

	}


	public Scan fetchById(Long scanId, ComponentMap requiredComponents) {
		Scan scan = scanDao.fetchById(scanId);

		if(scan != null)
			attachComponents(scan, requiredComponents);

		return scan;
	}




	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////// protected & privates //////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
	@Override
	protected void attachComponents(Scan scan, ComponentMap requiredComponents) {
		if(scan == null) {
			throw new RuntimeException("Cannot attach components to null slide");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Scan.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Scan.Component.THUMBNAIL)) {
				scan.setThumbnail(fetchThumbnailById(
						scan.getThumbnailId(),
						requiredComponents.get(Scan.Component.THUMBNAIL)));
			}

			if(fetchAll || requiredComponents.containsKey(Scan.Component.SLIDE)) {
				scan.setSlide(fetchSlideById(
						scan.getSlideId(),
						requiredComponents.get(Scan.Component.SLIDE)));
			}

			if(fetchAll || requiredComponents.containsKey(Scan.Component.CREATOR)) {
				scan.setCreator(fetchCreatorById(
						scan.getCreatorId(),
						requiredComponents.get(Scan.Component.CREATOR)));
			}

			if(fetchAll || requiredComponents.containsKey(Scan.Component.CROP_JOBS)) {
				scan.setCropJobs(fetchCropJobsByScanId(
						scan.getId(),
						requiredComponents.get(Scan.Component.CROP_JOBS)));
			}

		}
	}

	private Collection<CropJob> fetchCropJobsByScanId(Long scanId, ComponentMap requiredComponents) {
		return cropJobService.fetchByScanId(scanId, requiredComponents);
	}

	private Person fetchCreatorById(Long creatorId, ComponentMap requiredComponents) {
		return personService.fetchById(creatorId, requiredComponents);
	}

	private Slide fetchSlideById(Long slideId, ComponentMap map) {
		return slideService.fetchById(slideId, map);
	}


	/*
	 * If slide is null, Scan.slideId is set to null and version number set to 0.
	 */
	private Scan buildNewScan(Slide slide) {

		Scan scan = new Scan();
		scan.setStatusDictId( Scan.StatusDictEnum.AWAITING_THUMBNAIL_QC.getId() );
		scan.setSlideId( (slide != null) ? slide.getId() : null );
		scan.setVersionNumber( (slide != null) ? slide.getScans().size() + 1 : 0  );
		scan.setSweepDate( new Date() );

		BinaryFile thumb = new BinaryFile();
		thumb.setContentType("image/" + dmiThumbExtension.substring(1));
		thumb.setFileDescription("Dmetrix Thumbnail");
		scan.setThumbnail( thumb );
		return scan;
	}


	public String parseBarcode(String dmiFileName) {
		//where 'S' followed by 9 consecutive digits anywhere in file name.
		return dmiFileName.replaceFirst(".*?(S\\d{9}).*", "$1");
	}


	private void extractMetadata(Scan scan, File sweepPath, File thumbFilePath)
		throws SweepScanException {

		Process process = null;
		InputStreamReader inputStreamReader = null;
		BufferedReader bufferedReader = null;
		String line = null;

		DataBinder propertyBinder = new DataBinder( scan );
		propertyBinder.setIgnoreUnknownFields( true );
		propertyBinder.registerCustomEditor( Date.class,
				new CustomDateEditor( dmiDateFormat, false ) );

		MutablePropertyValues propertyValues = new MutablePropertyValues();

		try {

			String execString = getRemoteSshDmiconvert() +
				" -metaonly -thumbout " + thumbFilePath + " " + sweepPath;

			process = Runtime.getRuntime().exec( execString );

			logger.debug("Executing " + execString);

			inputStreamReader = new InputStreamReader( process.getInputStream() );
			bufferedReader = new BufferedReader( inputStreamReader );

			while ((line = bufferedReader.readLine()) != null) {

				if( line.indexOf( dmiFieldSeparator ) != -1 ) {

					String[] s = line.split( dmiFieldSeparator, 2 );

					String dmiFieldName = s[0].trim();

					String migFieldName = dmetrixToMigBeanFields.get( dmiFieldName );

					String migFieldValue = s[1].trim();


					if( StringUtils.isNotBlank( migFieldName )) {

						System.out.println("migFieldName: " + migFieldName);

						System.out.println("migFieldValue: " + migFieldValue);

						propertyValues.addPropertyValue( migFieldName, migFieldValue );
					}
				}
			}

			int exitCode = process.waitFor();

			if( exitCode != 0)
					throw new SweepScanException("dmiconvert error, exit code not zero! " +
							"\nExit code: " + exitCode +
							"\nCommand: " + execString);

		} catch (Exception e1) {

			throw new SweepScanException(e1);

		} finally {
			if( process != null ) process.destroy();
			IOUtils.closeQuietly( bufferedReader );
		}

		propertyBinder.bind( propertyValues );

		byte[] bytes = readThumbnail( thumbFilePath );

		scan.getThumbnail().setBytes( bytes );

		validateExtraction( scan, propertyBinder );//propBinder may have errors w/out exception thrown

	}


	private byte[] readThumbnail(File thumbFilePath) {
		FileInputStream inputStream = null;
		byte[] bytes = null;
		try {

			inputStream = new FileInputStream( thumbFilePath );
			bytes = new byte[inputStream.available()];
			inputStream.read(bytes);
			inputStream.close();

		} catch(IOException e) {logger.error("Failed to read thumbnail " + thumbFilePath, e);}

		return bytes;
	}

	/*
	private byte[] readAndFlipThumbnail(File thumbFilePath, int width, int height)
		throws SweepScanException {

		byte[] bytes = null;
		ByteArrayOutputStream output = null;

		try {
		    int[] pixelsIn = new int[width * height];
		    int[] pixelsOut = new int[width * height];
			BufferedImage bufferedImage = ImageIO.read( thumbFilePath );
			pixelsIn = bufferedImage.getRGB(0,0, width, height, pixelsIn, 0, width);

		    for (int y = 0; y < height; y++) {
		        for (int x = 0; x < width; x++) {

		          pixelsOut[y * width + (width - x - 1)] = pixelsIn[y * width + x];

		        }
		    }

		    bufferedImage.setRGB(0,0, width, height, pixelsOut, 0, width);

		    output = new ByteArrayOutputStream();

		    ImageIO.write(bufferedImage, "jpg" , output);

		    bytes = output.toByteArray();

		} catch(IOException e) {

			throw new SweepScanException("IO problems flipping thumnail image: " + e );

		} finally {
			IOUtils.closeQuietly( output );
		}

		return bytes;
	}
	*/

	private void validateExtraction(Scan scan, DataBinder dataBinder)
			throws SweepScanException {

		boolean propertyBindingOK = !dataBinder.getBindingResult().hasErrors();

		boolean imageBytesOK = ( scan.getThumbnail().getBytes() != null );

		if( !propertyBindingOK ) {

			throw new SweepScanException("Property binding errors: " + dataBinder.getBindingResult().getErrorCount()
					+ " error(s) found!\n\tBindingResult: " + dataBinder.getBindingResult());
		}

		if( !imageBytesOK ) {
			throw new SweepScanException("Thumbnail binding error. Failed to bind image bytes!" );
		}

	}

	/*
	 * Deletes temp thumb and moves scan out of sweep to archive dir
	 */
	private File cleanUp(File sweepPath, File thumbFilePath, File dmiScanDir)
		throws SweepScanException {

		if( thumbFilePath.delete() )
			logger.debug(thumbFilePath + " successfully deleted.");
		else
			logger.error("Failed to delete " + thumbFilePath);

		try {

			if( !dmiScanDir.exists() &&  !dmiScanDir.mkdirs() ) {

				throw new SweepScanException("Failed to create " + dmiScanDir);
			}

		} catch(Exception e)  {e.printStackTrace();}

		File dmiScanPath = new File(dmiScanDir + File.separator  + sweepPath.getName());

		if( sweepPath.renameTo( dmiScanPath ))

			logger.debug("Successfully moved " + sweepPath + " to " + dmiScanPath);

		else {

			throw new SweepScanException("Failed to move " + sweepPath + " to " + dmiScanPath);
		}

		return dmiScanPath;
	}

	private String deriveThumbName( String dmiFileName ) {

		int startIndex = 0;
		int endIndex = dmiFileName.indexOf( getDmiImageExtension() );
		return dmiFileName.substring(startIndex, endIndex) + dmiThumbExtension;
	}

	private BinaryFile fetchThumbnailById(Long imageId, ComponentMap components) {
		return binaryFileService.fetchById(imageId, true, components);
	}

	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// dependencies //////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////


	public File getSweepDirectory() {

		File dmiSweepDirectory = new File( sweepDirectoryStr );

		if ( !dmiSweepDirectory.canWrite() ) {
			throw new RuntimeException("Failed to find writable system directory "
					+ dmiSweepDirectory);
		}

		return dmiSweepDirectory;
	}


	public File getDmiRootDirectory() {

		File dmiRootDirectory = new File( dmiRootDirectoryStr );

		if ( !dmiRootDirectory.canWrite() ) {
			throw new RuntimeException("Failed to find writable system directory "
					+ dmiRootDirectory);
		}

		return dmiRootDirectory;
	}

	public void setSweepDirectoryStr(String sweepDirectoryStr) {
		this.sweepDirectoryStr = sweepDirectoryStr;
	}

	public void setDmiRootDirectoryStr(String dmiRootDirectoryStr) {
		this.dmiRootDirectoryStr = dmiRootDirectoryStr;
	}

	public void setScanDao(ScanDao slideScanDao) {
		this.scanDao = slideScanDao;
	}

	public void setSlideService(SlideService slideService) {
		this.slideService = slideService;
	}

	public void setDmiThumbExtension(String thumbExtension) {
		this.dmiThumbExtension = thumbExtension;
	}

	public void setBinaryFileService(BinaryFileService binaryFileService) {
		this.binaryFileService = binaryFileService;
	}



	public void setDmiFieldSeparator(String dmiFieldSeparator) {
		this.dmiFieldSeparator = dmiFieldSeparator;
	}

	public void setDmiDateFormat(String dmiDateFormatStr) {
		dmiDateFormat = new SimpleDateFormat( dmiDateFormatStr );
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setCropService(CropService cropService) {
		this.cropService = cropService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setIndexDirectoryPrefix(String indexDirectoryPrefix) {
		this.indexDirectoryPrefix = indexDirectoryPrefix;
	}

	public void setMaxIndexDirectoryCapacity(int maxIndexDirectoryCapacity) {
		this.maxIndexDirectoryCapacity = maxIndexDirectoryCapacity;
	}



	public int getDefaultIndexDirectoryCapacity() {
		return this.maxIndexDirectoryCapacity;
	}

	public void setCropJobService(CropJobService cropJobService) {
		this.cropJobService = cropJobService;
	}


	public String getRemoteSshDmiconvert() {
		return this.remoteSshDmiconvert;
	}

	public void setRemoteSshDmiconvert(String remoteSshDmiconvert) {
		this.remoteSshDmiconvert = remoteSshDmiconvert;
	}

	public String getDmiImageExtension() {
		return this.dmiImageExtension;
	}

	public void setDmiImageExtension(String dmiImageExtension) {
		this.dmiImageExtension = dmiImageExtension;
	}



}
