package uk.ac.sanger.mig.histology.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.OracleReport;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.histology.BlockType;
import uk.ac.sanger.mig.histology.Scan;
import uk.ac.sanger.mig.histology.Slide;
import uk.ac.sanger.mig.histology.dao.SlideDao;
import uk.ac.sanger.mig.histology.forms.PrintSlideCheckbox;
import uk.ac.sanger.mig.mousetracking.services.LabelPrinterService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class SlideServiceImpl extends AbstractServiceImpl<Slide>
								implements SlideService {

	private SlideDao slideDao;
	private BlockService blockService;
	private ScanService scanService;
	private PersonService personService;
	private LabelPrinterService labelPrinterService;
	private OracleReport oracleReport;

	@Transactional
	public Long addToRecreationBin(Long slideId) {

		if( !slideDao.existsInRecreationBin(slideId) ) {

			slideDao.addToRecreationBin(slideId);
		}

		return slideId;

	}

	@Transactional
	public Long addToScanningBin(Long slideId)	{

		if( !slideDao.existsInScanningBin(slideId) ) {
			slideDao.addToScanningBin(slideId);
		}

		return slideId;
	}



	@Transactional
	public boolean gentlyFail(Long id, Person user) {

		if(id == null)
			throw new IllegalArgumentException("Slide ID cannot be null!");

		Slide slide = fetchById(id, ComponentMap.of(Slide.Component.SCANS));

		Collection<Scan> scans = slide.getScans();

		boolean childrenFailedOK = true;

		for( Scan scan : scans ) {

			 //just need one scan to avoid fail
			 if(!scanService.gentlyFail( scan.getId(),
								 Scan.StatusDictEnum.THUMBNAIL_QC_FAIL_BAD_SLIDE, user ) ) {

				 childrenFailedOK = false;

			 }
		}

		if( childrenFailedOK ) {

			qcFail(slide, user);

		}

		return childrenFailedOK;
	}

	@Transactional
	public boolean gentlyFailAndReorder(Long id, Person user) {

		addToRecreationBin( id );

		return gentlyFail(id, user);
	}

	private void qcFail(Slide slide, Person user) {

		slide.setStatusDictId( Slide.StatusDictEnum.QC_FAIL.getId() );
		slide.setFlaggedForDelete( true );
		store( slide, user );
	}

	public Collection<Slide> fetchScanningBin(ComponentMap componentMap) {
		 Collection<Slide> slides = slideDao.fetchScanningBin();

		 attachComponents(slides, componentMap);

		 return slides;
	}

	@Transactional
	public Long removeFromScanningBin(Long slideId) {

		slideDao.removeFromScanningBin(slideId);
		return slideId;

	}

	@Transactional
	public Long removeFromRecreationBin(Long slideId) {

		slideDao.removeFromRecreationBin(slideId);

		return slideId;
	}

	public Collection<Slide> fetchRecreationBin(ComponentMap componentMap) {

		 Collection<Slide> slides = slideDao.fetchRecreationBin();

		 attachComponents(slides, componentMap);

		 return slides;
	}





	//create the slides on database this will commit the data
	@Transactional
	public Collection<Slide> createAll(
			Collection<PrintSlideCheckbox> checkboxes, Person user) {
		Collection<Slide> slides = new LinkedList<Slide>();
		for (PrintSlideCheckbox cb : checkboxes) {
			if (cb.isChecked()) {
				slides.add(create(cb.getBlockId(), cb.getStainId(), user));
			}
		}
		return slides;
	}

	@Transactional
	public Collection<Slide> recreateAll(Collection<Long> originalSlideIds, Person user) {

		if( originalSlideIds == null )
			throw new IllegalArgumentException("Collection<Long> originalSlideIds cannot be null.");

		Collection<Slide> recreatedSlides = new ArrayList<Slide>();

		for(Long originalSlideId : originalSlideIds) {

			Slide slide = recreate( originalSlideId, user );

			recreatedSlides.add( slide );
		}

		return recreatedSlides;
	}

	@Transactional
	public Long store(Slide slide, Person user) {

		if (slide == null) {
			throw new IllegalArgumentException("Cannot save a null slide");
		}

		return slideDao.store(slide, user);
	}

	public Slide fetchById(Long slideId, ComponentMap requiredComponents) {
		Slide slide = slideDao.fetchById(slideId);

		if ( slide != null )
			attachComponents(slide, requiredComponents);

		return slide;
	}

	public Slide fetchByBarcode(String barcode, ComponentMap componentMap) {
		Slide slide = slideDao.fetchByBarcode(barcode);

		if ( slide != null )
			attachComponents(slide, componentMap);

		return slide;
	}

	@Override
	protected void attachComponents(Slide slide, ComponentMap requiredComponents) {
		if ( slide == null ) {
			throw new RuntimeException(
			"Cannot attach components to null or empty (blank) slide");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(Slide.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll
					|| requiredComponents.containsKey(Slide.Component.BLOCK)) {
				slide.setBlock(fetchBlockById(slide.getBlockId(),
						requiredComponents.get(Slide.Component.BLOCK)));
			}

			if(fetchAll || requiredComponents.containsKey(Slide.Component.CREATOR)) {
				slide.setCreator(fetchCreatorById(
						slide.getCreatorId(),
						requiredComponents.get(Slide.Component.CREATOR)));
			}

			if(fetchAll || requiredComponents.containsKey(Slide.Component.SCANS)) {
				slide.setScans(fetchScansBySlideId(
						slide.getId(),
						requiredComponents.get(Slide.Component.SCANS)));
			}

		}
	}


	public void setSlideDao(SlideDao slideDao) {
		this.slideDao = slideDao;
	}

	public Collection<Slide> fetchByMouseId(Long mouseId, ComponentMap componentMap) {

		Collection<Slide> slides = slideDao.fetchByMouseId(mouseId);

		if( !slides.isEmpty() )
			attachComponents(slides, componentMap);

		return slides;
	}

	public Collection<Slide> fetchAllByBlockId(Long blockId, ComponentMap map) {
		Collection<Slide> slides = slideDao.fetchAllByBlockId(blockId);

		if( !slides.isEmpty() )
			attachComponents(slides, map);

		return slides;
	}



	@Transactional
	public Collection<Slide> createDefaultSlides(Long mouseId, Person user) {

	  Collection<Block> blocks = new ArrayList<Block>();
	  Collection<Slide> slides = new ArrayList<Slide>();

	  blocks = blockService.fetchByMouseId(mouseId,
	      new ComponentMap(
	          Block.Component.BLOCK_TYPE,
	          ComponentMap.of(
	              BlockType.Component.DEFAULT_STAINS
	          )
	      )
    );

	  for (Block block : blocks) {
	    for (DictionaryEntry stain : block.getBlockType().getDefaultStains()) {
	      Slide slide = create(block.getId(),stain.getId(),user);
	      slides.add(slide);
	    }
	  }

		return slides;
	}

	public void printSlides(Collection<Slide> slides) throws InterruptedException, ExecutionException, TimeoutException{
		String	slideReportString = "htl_slide_label+" + labelPrinterService.fetchById(HTL_SLIDE_PRINTER_ID, null).getSysName();
		slideReportString = constructPrintString(slides,slideReportString);

		ExecutorService executorService = Executors.newSingleThreadExecutor();

		try{
			Future<Boolean> future = executorService.submit(new RunnableOracleReport(slideReportString), Boolean.TRUE );
			future.get(HTL_REPORT_TIME_OUT, TimeUnit.SECONDS);//waits unitl completion, exception, or timeout
		}catch(TimeoutException e){
			throw e;
		}catch(ExecutionException e){
			throw e;
		}catch(InterruptedException e){
			throw e;
		}finally{
			executorService.shutdown();
		}
	}




	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// privates     ////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	private Collection<Scan> fetchScansBySlideId(Long slideId, ComponentMap requiredComponents) {
		return scanService.fetchAllBySlideId(slideId, requiredComponents);
	}

	private Person fetchCreatorById(Long creatorId, ComponentMap requiredComponents) {
		return personService.fetchById(creatorId, requiredComponents);
	}

	private Block fetchBlockById(Long blockId, ComponentMap requiredComponents) {
		return blockService.fetchById(blockId, requiredComponents);
	}


	/*
	 * Recreates a Slide returning a new Slide with the same
	 * Block and Staining.
	 *
	 * If the original Slide has a status of QC_FAIL,
	 * its flaggedForDelete  field will be set to true.
	 */
	private Slide recreate(Long slideId, Person user) {

		if( slideId == null )
			throw new IllegalArgumentException("Long slideId cannot be null.");

		removeFromRecreationBin( slideId );

		Slide originalSlide = fetchById(slideId, null);

		if( Slide.StatusDictEnum.QC_FAIL.getId().equals(
				originalSlide.getStatusDictId() ) ) {

			originalSlide.setFlaggedForDelete( true );
		}

		store( originalSlide, user );

		return create( originalSlide.getBlockId(),
				originalSlide.getStainingTypeDictId(), user );
	}

	/*
	 * Creates a Slide object with initial status of AWAITING_QC and
	 * with the revision number increased by 1 if a Slide with same
	 * Block and stainId already exists in the database.
	 */
	private Slide create(Long blockId, Long stainId, Person user) {
		Slide slide = new Slide();
		Collection<Slide> oldSlides = fetchByBlockIdAndStainDictId(blockId,
				stainId);
		if (oldSlides != null) {
		    Integer currentMaxVersionNumber = getMaxVersionNumber( oldSlides );
			slide.setVersionNumber( ++currentMaxVersionNumber );
		} else {
			slide.setVersionNumber( 1 );
		}
		slide.setBlockId(blockId);
		slide.setStatusDictId(Slide.StatusDictEnum.AWAITING_SCAN.getId());
		slide.setStainingTypeDictId(stainId);
		store(slide, user);
		return slide;
	}

	private Integer getMaxVersionNumber(Collection<Slide> oldSlides) {
		Integer maxVersionNumber = new Integer( 0 );
		if (oldSlides != null) {
			for (Slide slide : oldSlides) {
				if (slide.getVersionNumber() > maxVersionNumber) {
					maxVersionNumber = slide.getVersionNumber();
				}
			}
		}
		return maxVersionNumber;
	}

	private String constructPrintString(Collection<Slide> slides,
			String reportString) {

		reportString +="+";
		for(Slide slide:slides) {
			reportString += slide.getId()+",";
		}
		reportString = reportString.substring(0, reportString.length()-1);
		return reportString;
	}

	private Collection<Slide> fetchByBlockIdAndStainDictId(Long blockId,
			Long stainId) {
		return slideDao.fetchByBlockIdAndStainDictId(blockId, stainId);
	}




	/////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// dependencies ////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////
	public void setScanService(ScanService scanService) {
		this.scanService = scanService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setOracleReport(OracleReport oracleReport) {
		this.oracleReport = oracleReport;
	}

	public void setLabelPrinterService(LabelPrinterService labelPrinterService) {
		this.labelPrinterService = labelPrinterService;
	}
	public void setBlockService(BlockService blockService) {
		this.blockService = blockService;
	}

	/*
	 * Inner class to process oracle printing job in separate
	 * thread.
	 */
	private class RunnableOracleReport implements Runnable {

        private String reportString;

		public RunnableOracleReport(String reportString) {
			this.reportString = reportString;
		}

		public void run() {

			String errorMessage = oracleReport.run(reportString);
			if(errorMessage != null){
				logger.error("Report failed: "+errorMessage);
				throw new RuntimeException("Report failed: "+errorMessage);
			}

		}

	}
}
