/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.histology.services;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.services.ImageProcessingService;
import uk.ac.sanger.mig.filesupport.services.FileSystemService;
import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropProcessingException;
import uk.ac.sanger.mig.histology.Tile;
import uk.ac.sanger.mig.histology.dao.TileDao;
import uk.ac.sanger.mig.personnel.Person;

public class TileServiceImpl implements TileService {

	private TileDao tileDao;
	private ImageProcessingService imageProcessingService;
	private FileSystemService fileSystemService;
	private String defaultFormatExtension;
	private int defaultEdgeExponent;
	private int defaultZoomRadix;
	//TODO inject default Tile format extension

	protected Log logger = LogFactory.getLog(this.getClass());

	public Tile fetch(final Long cropId, final int zoomLevel,
			final int arrayIndex) {

		return tileDao.fetch(cropId, zoomLevel, arrayIndex);
	}

	@Transactional
	public void deleteAllByCropId(Long cropId) {

		tileDao.deleteAllByCropId(cropId);
	}

	@Transactional
	public void makeTilePyramid(Crop crop, Person user)
			throws CropProcessingException {

		String tempTiledCropDirPrefix = "tiledCrop" +  crop.getId();

		try {


			File imageToTile = crop.getFullImagePath();//start with full image b/c can't rely on temp cached if called later

			File tempTiledCropDir = fileSystemService.makeTempSubDirectory( tempTiledCropDirPrefix );

			//temp cache bmp, we're just going to read it again,
			//so why compress just to decompresss again
			File tempResizedImage = deriveTempResizedImagePath( tempTiledCropDir, crop.getId() );

			int maxWidth = crop.getCropWidth();
			int maxHeight = crop.getCropHeight();

			for (int zoomLevel = crop.getMaxZoomDepth(); zoomLevel >= crop
					.getMinZoomDepth(); zoomLevel--) {

				sendOutput("\n-Building zoom level " + zoomLevel);

				File zoomDir = makeZoomDirectory(tempTiledCropDir, zoomLevel);

				if (zoomLevel < crop.getMaxZoomDepth()) {

					if ( tempResizedImage.exists() )
						imageToTile = tempResizedImage;
					else
						throw new CropProcessingException("Temp image not available to tile: " +tempResizedImage);
				}

				tile(imageToTile, zoomDir, crop.getTileEdge());

				//sweep into DB
				sweep( zoomLevel, zoomDir, crop.getId());

				maxWidth /= crop.getZoomRadix();
				maxHeight /= crop.getZoomRadix();

				if( zoomLevel > crop.getMinZoomDepth() ) //resize for the next iteration
					imageProcessingService.convert(imageToTile, tempResizedImage, maxWidth, maxHeight);

			}

		} catch(Exception e) {	
		} finally {

			//clean up
			try {

				fileSystemService.flushTempCache( tempTiledCropDirPrefix );

				sendOutput( "Temp tiles flush success!\n");

			} catch(IOException e) {

				logger.debug( e );
			}

		}

	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// privates /////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	private void sweep( int zoomLevel, File zoomDir, Long cropId ) {

		File[] files = zoomDir.listFiles();

		for( int i = 0; i < files.length; i++ ) {

			FileInputStream fis =null;
			byte[] bytes = null;

			try {

				fis = new FileInputStream( files[i] );
				bytes = new byte[ fis.available() ];
				fis.read( bytes );

			} catch(IOException e) {
				e.printStackTrace();
			} finally {
				IOUtils.closeQuietly( fis );
			}


			Tile tile = new Tile();
			tile.setArrayIndex( i );
			tile.setCropId( cropId );
			tile.setFormatExtension(".jpg");
			tile.setZoomLevel(zoomLevel);
			tile.setImage( bytes );

			tileDao.store( tile );

		}

		sendOutput("\n-Stored tiles: " + files.length);
	}


	private File deriveTempResizedImagePath(File tileDir, Long cropId) {

		return new File(tileDir  +  File.separator + "tempResized"+cropId+".bmp");

	}

	private void tile(File srcPath, File zoomDir, int tileEdge) throws CropProcessingException {

		String command = imageProcessingService.getRemoteSshProcessor() + " " + srcPath
				+ " -background black " + "-crop " + tileEdge + "x" + tileEdge
				+ " -extent " + tileEdge + "x" + tileEdge + " " + zoomDir
				+ "/tile_%d" + getDefaultFormatExtension();


		sendOutput( "\n-Execute Tile Command: " + command);

		Process process = null;

		try {

			process = Runtime.getRuntime().exec(command);

			int exitCode = process.waitFor();

			if (exitCode != 0) {
				//look at command for debugging
				throw new CropProcessingException(
						"\nImageMagick Tiling returned abnormally.\nExit code:"
								+ exitCode + "\n****Echo Command: " + command  + "\n");
			}

		} catch (IOException e) {

			throw new CropProcessingException(e);

		} catch (InterruptedException e) {

			throw new CropProcessingException(e);

		} finally {

			if (process != null) process.destroy();
		}

	}

	private File makeZoomDirectory(File tileDir, int zoomLevel) {

		File zoomDir = new File(tileDir + File.separator + Integer.toString(zoomLevel));

		if (!zoomDir.exists() && !zoomDir.mkdirs()) {

			throw new RuntimeException("Failed to create zoomDir " + zoomDir);
		}

		return zoomDir;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// injections ///////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////

	public int getDefaultZoomRadix() {
		return this.defaultZoomRadix;
	}

	public void setDefaultZoomRadix(int defaultZoomRadix) {
		this.defaultZoomRadix = defaultZoomRadix;
	}

	public void setTileDao(TileDao tileDao) {
		this.tileDao = tileDao;
	}


	public void setImageProcessingService(
			ImageProcessingService imageProcessingService) {
		this.imageProcessingService = imageProcessingService;
	}

	public int getDefaultEdgeExponent() {
		return this.defaultEdgeExponent;
	}

	public void setDefaultEdgeExponent(int defaultEdgeExponent) {
		this.defaultEdgeExponent = defaultEdgeExponent;
	}

	public String getDefaultFormatExtension() {
		return this.defaultFormatExtension;
	}

	public void setDefaultFormatExtension(String defaultFormatExtension) {
		this.defaultFormatExtension = defaultFormatExtension;
	}

	private void sendOutput(String line) {


		logger.info( line );


	}

	public void setFileSystemService(FileSystemService fileSystemService) {
		this.fileSystemService = fileSystemService;
	}

}
