/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.validators;

import java.util.Collection;
import java.util.List;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.histology.Block;
import uk.ac.sanger.mig.histology.Necropsy;
import uk.ac.sanger.mig.histology.forms.BatchCheckbox;
import uk.ac.sanger.mig.histology.forms.ManageBatchForm;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.CohortSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.CohortService;

public class ManageBatchValidator implements Validator{

  private CohortService cohortService;
  private ComponentMapService componentMapService;

  public void setComponentMapService(ComponentMapService componentMapService) {
    this.componentMapService = componentMapService;
  }

  public boolean supports(Class clazz) {
    return ManageBatchForm.class.equals(clazz);
  }

  public void validate(Object command, Errors errors) {
    ManageBatchForm form = (ManageBatchForm) command;
    
    ValidationUtils.rejectIfEmpty(errors, "batch.endDate", null, "Please specify a date for necropsy");
    
    if (form.getCohortIdForBatch() != null && form.getCohortIdForBatch() > 0) {
      Cohort referenceCopy = cohortService.fetchById(
          form.getCohortIdForBatch(),
          componentMapService.getComponentMap("miceToBlocksForCohort")
      );
      
      checkForModificationOfLockedBatch(referenceCopy, errors, form);
      
      checkNecropsyDateChange(referenceCopy, errors, form);
    }
    

    checkMiceForExistingBatch(errors, form);
  }

  private void checkForModificationOfLockedBatch(Cohort referenceBatch, Errors errors, ManageBatchForm form) {
    boolean batchIsLocked = false;
    boolean changesMade = false;
    
    // batch is locked if an embedding run has be run for any mouse in the batch
    outer: for (Mouse m : referenceBatch.getMice()) {
      Collection<Block> blocks = m.getExperimentData().getNecropsy().getFixation().getBlocks();
      
      if (blocks != null ) {
        for (Block b: blocks) {
          if (b.getProgrammeStartTime() != null || b.getProgrammeTypeId() != null) {
            batchIsLocked = true;
            break outer;
          }
        }
      }
    }

    for (BatchCheckbox bc : form.getMiceInBatch()) {
      if (!bc.isChecked()) {
        changesMade = true;
        break;
      }
    }
    for (BatchCheckbox bc : form.getMiceToSelectFrom()) {
      if (bc.isChecked()) {
        changesMade = true;
        break;
      }
    }
    
    
    if (batchIsLocked && changesMade) {
      errors.reject(null,"Cannot change contents of batch once an embedding run has been recorded.");
    }
  }

  private void checkNecropsyDateChange(Cohort referenceBatch, Errors errors, ManageBatchForm form) {
    boolean dateIsChanging = (
        form.getBatch().getEndDate() != null &&
        referenceBatch.getEndDate() != null &&
        !referenceBatch.getEndDate().equals(form.getBatch().getEndDate())
    );
    
    // date is locked when a necropsy has been recorded for the batch
    boolean dateIsLocked = false;
    
    for (Mouse m: referenceBatch.getMice()) {
      Necropsy necropsy = m.getExperimentData().getNecropsy(); 
      
      // should only be filled in when necropsy has been done
      if (necropsy.getTechnicianId() != null) {
        dateIsLocked = true;
        break;
      }
    }

    if (dateIsChanging && dateIsLocked) {
      errors.reject(null,"Cannot change the date of necropsy for the batch once necropsy has been carried out.");     
    }
  }

  private void checkMiceForExistingBatch(Errors errors, ManageBatchForm form) {
    List<BatchCheckbox> checkBoxes = form.getMiceToSelectFrom();
    for (int i = 0; i < checkBoxes.size(); i++) {

      if (checkBoxes.get(i).isChecked()) {
        CohortSearchBean csb = new CohortSearchBean();
        csb.setMouse(checkBoxes.get(i).getMouse());
        csb.setCohortTypeDictId(CohortService.COHORT_TYPE_HISTOLOGY);
        csb.setPaginated(false);
        csb.setQueryName("histologyBatchValidation");

        if(cohortService.search(csb).size() > 0) {
          errors.rejectValue("miceToSelectFrom["+i+"]", null, "Mouse is already in a batch");
        }
      }

    }
  }

  public void setCohortService(CohortService cohortService) {
    this.cohortService = cohortService;
  }

}
