/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.histology.Fixation;
import uk.ac.sanger.mig.histology.forms.BatchProcessingCheckbox;
import uk.ac.sanger.mig.histology.forms.BatchProcessingForm;

public class ProcessBatchValidator implements Validator {

  public boolean supports(Class clazz) {
    return BatchProcessingForm.class.equals(clazz);
  }

  public void validate(Object command, Errors errors) {
    BatchProcessingForm form = (BatchProcessingForm)command;

    validateForTissueType(errors, form);

    validateTimes(errors, form);

  }

  private void validateForTissueType(Errors errors, BatchProcessingForm form) {
    // tissue type must be selected in all cases
    if (form.getCassetteTypeId() == null ||  form.getCassetteTypeId() < 1) {
      errors.rejectValue("cassetteTypeId", null, "Please select a tissue type");
    }
  }

  private void validateTimes(Errors errors, BatchProcessingForm form) {

    for (int i = 0; i < form.getCheckboxes().size(); i++) {
      String rootPath = "checkboxes["+i+"]";
      BatchProcessingCheckbox cb = form.getCheckboxes().get(i);
      Fixation fixation = cb.getMouse().getExperimentData().getNecropsy().getFixation();
      
      // if set, fixation end should be after fixation start
      if (fixationEndBeforeFixationStart(form, cb, fixation)
      ) {
        errors.rejectValue(rootPath+".mouse",null,"Fixation end time cannot be before fixation start time");
      }

      // if set, programme start should be after fixation start
      if (programmeStartBeforeFixationStart(form, cb, fixation)
      ) {
        errors.rejectValue(rootPath+".mouse",null,"Programme start time cannot be before fixation start time");
      }

    }

  }

  private boolean programmeStartBeforeFixationStart(BatchProcessingForm form, BatchProcessingCheckbox cb, Fixation fixation) {
    return cb.isChecked() &&
        form.getProgrammeStart() != null &&
        fixation.getFixationStartTime() != null &&
        fixation.getFixationStartTime().getTime() >= form.getProgrammeStart().getTime();
  }

  private boolean fixationEndBeforeFixationStart(BatchProcessingForm form, BatchProcessingCheckbox cb, Fixation fixation) {
    return cb.isChecked() &&
        form.getFixationEnd() != null &&
        fixation.getFixationStartTime() != null &&
        fixation.getFixationStartTime().getTime() >= form.getFixationEnd().getTime();
  }

}


