/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.licensing.dao;

import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.licensing.PersonalLicence;

public class PersonalLicenceDaoSpringImpl extends AbstractDaoSpringImpl<PersonalLicence> implements PersonalLicenceDao {

	public PersonalLicence fetchByPersonId(Long personId) {
		String sql = " select * from lic_personal_licence where person_id = ? ";
		PersonalLicence personalLicence = null;

		try {
			personalLicence = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), personId);
		} catch (IncorrectResultSizeDataAccessException e) {
			logger.debug(e);
		}

		return personalLicence;
	}


	public Collection<PersonalLicence> fetchCurrent() {
		String sql = " select l.* from lic_personal_licence l, per_person p where p.person_id = l.person_id and p.active = 1 order by p.last_name, p.first_name";

		return getSimpleJdbcTemplate().query(sql, getRowMapper());
	}


	/**
	 * Finds the personal licence details using the string personal licence number.
	 *
	 * @param personalLicenceNumber -
	 *            string PIL i.e. '50/3052'.
	 * @return PersonalLicence - the personal licence object or null if nothing found.
	 */
	public PersonalLicence fetchByPersonalLicenceNumber(String personalLicenceNumber) {
		String sql = "select * from lic_personal_licence where pil_number = ?";
		return fetchOne(sql, personalLicenceNumber);
	}


	/**
	 * Returns true if the person with the personal licence (PIL) and the project licence (PPL) can
	 * work on the passed standard operating procedure.
	 *
	 * @param pil -
	 *            the persons personal licence id.
	 * @param ppl -
	 *            the project licence id.
	 * @param procedureDictId -
	 *            the procedure id they wish to carry out.
	 * @return true - if the pil and project licences are associated together as well as the
	 *         procedure, otherwise false.
	 */
	public boolean isLicenced(Long pil, Long ppl, Long procedureDictId) {
		String sql = "select" + " count(p19b.id)" + " from" + " lic_project_licence ppl,"
				+ " lic_project_2_personal_licence link1," + " lic_protocol_19b p19b,"
				+ " lic_procedure_2_protocol link2" + " where ppl.project_licence_id = ?"
				+ " and ppl.project_licence_id = link1.project_licence_id" + " and link1.personal_licence_id = ?"
				+ " and ppl.project_licence_id = p19b.project_licence_id" + " and p19b.id = link2.protocol_id"
				+ " and link2.procedure_dict_id = ?" + " order by ppl.licence_number";
		return getSimpleJdbcTemplate().queryForInt(sql, ppl, pil, procedureDictId) > 0 ? true : false;
	}


	public int linkToPpl(Long personalLicenceId, Long projectLicenceId) {
		String sql = " insert into lic_project_2_personal_licence(project_licence_id,personal_licence_id) select ?,? from dual where not exists (select null from lic_project_2_personal_licence where project_licence_id = ? and personal_licence_id =?)";

		return getSimpleJdbcTemplate().update(sql, projectLicenceId, personalLicenceId, projectLicenceId,
				personalLicenceId);

	}


	public int clearLinksToPpls(Long personalLicenceId) {
		String sql = " delete from lic_project_2_personal_licence where personal_licence_id = ?";

		return getSimpleJdbcTemplate().update(sql, personalLicenceId);
	}


	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, PersonalLicence personalLicence) {
		sql.addLongParam(personalLicence.getPersonId(), "PERSON_ID");
		sql.addStringParam(personalLicence.getLicenceNumber(), "PIL_NUMBER");
		sql.addStringParam(personalLicence.getPersonName(), "PERSON_NAME");
		sql.addLongParam(personalLicence.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(personalLicence.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(personalLicence.getEditedBy(), "EDITED_BY");
		sql.addDateParam(personalLicence.getEditDate(), "EDIT_DATE");
		sql.addLongParam(personalLicence.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(personalLicence.getId(), "PERSONAL_LICENCE_ID");
	}

}