/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.licensing.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.licensing.Protocol;

public class ProtocolDaoSpringImpl extends AbstractDaoSpringImpl<Protocol> implements ProtocolDao {

	public Collection<Protocol> fetchByProcedureAndProject(Long procedureId, Long projectLicenceId) {
		String sql = "select a.* from lic_protocol_19b a, lic_procedure_2_protocol b where a.id = b.protocol_id and b.procedure_dict_id = ? and a.project_licence_id = ? order by lower(a.protocol_code) ";
		Collection<Protocol> protocols = getSimpleJdbcTemplate().query(sql, getRowMapper(), procedureId, projectLicenceId);
		return (protocols != null) ? protocols : new ArrayList<Protocol>();
	}

	public Collection<Protocol> fetchByProjectLicenceId(Long projectLicenceId) {
		String sql = " select * from lic_protocol_19b where project_licence_id = ? order by lower(protocol_code)";
		Collection<Protocol> protocols = getSimpleJdbcTemplate().query(sql, getRowMapper(), projectLicenceId);
		return (protocols != null) ? protocols : new ArrayList<Protocol>();
	}

	public Collection<Protocol> fetchByProcedureId(Long procedureId) {
		String sql = " select prot.* from lic_protocol_19b prot, lic_project_licence ppl, lic_procedure_2_protocol linker where ppl.project_licence_id = prot.project_licence_id and prot.id = linker.protocol_id and linker.procedure_dict_id = ? order by ppl.licence_number";
		Collection<Protocol> protocols = getSimpleJdbcTemplate().query(sql, getRowMapper(), procedureId);
		return (protocols != null) ? protocols : new ArrayList<Protocol>();
	}

	public Protocol fetchByProcedureAndProject(Long procedureId, Long projectId, boolean isHarmfulMutant) {
		String sql = " select a.* from lic_protocol_19b a, lic_procedure_2_protocol b where a.id = b.protocol_id and b.procedure_dict_id = ? and a.project_licence_id = ? and a.harmful_mutant = ? ";
		Protocol protocol = null;

		try {
			protocol = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), procedureId, projectId, isHarmfulMutant);
		}
		catch(IncorrectResultSizeDataAccessException e1) {
			// if it's a harmful mutant and no protocol is found, try to find non-harmful mutant protocol
			if(isHarmfulMutant) {
				try {
					protocol = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), procedureId, projectId, !isHarmfulMutant);
				}
				catch(IncorrectResultSizeDataAccessException e2) {

				}
			}
		}

		return protocol;
	}

	public int linkToProcedure(Long procedureId, Long protocolId) {
		String sql = " insert into lic_procedure_2_protocol(procedure_dict_id, protocol_id) select ?,? from dual where not exists (select null from lic_procedure_2_protocol where procedure_dict_id = ? and protocol_id = ?) ";
		return getSimpleJdbcTemplate().update(sql, procedureId, protocolId, procedureId, protocolId);
	}

	public int unlinkFromProcedure(Long procedureId, Long protocolId) {
		String sql = " delete from lic_procedure_2_protocol where procedure_dict_id = ? and protocol_id = ? ";
		return getSimpleJdbcTemplate().update(sql, procedureId, protocolId);
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Protocol protocol) {
		sql.addLongParam(protocol.getProjectLicenceId(), "PROJECT_LICENCE_ID");
		sql.addStringParam(protocol.getProtocolCode(), "PROTOCOL_CODE");
		sql.addStringParam(protocol.getTitle(), "TITLE");
		sql.addLongParam(protocol.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(protocol.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(protocol.getEditedBy(), "EDITED_BY");
		sql.addDateParam(protocol.getEditDate(), "EDIT_DATE");
		sql.addLongParam(protocol.getCheckNumber(), "CHECK_NUMBER");
		sql.addBooleanParam(protocol.isHarmfulMutant(), "HARMFUL_MUTANT");
		sql.addLongParam(protocol.getId(), "ID");
	}
}