/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.licensing.services;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.dao.PersonalLicenceDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;

public class PersonalLicenceServiceImpl extends AbstractServiceImpl<PersonalLicence> implements PersonalLicenceService {

	private PersonalLicenceDao		personalLicenceDao;
	private PersonService			personService;
	private ProjectLicenceService	projectLicenceService;


	public void setPersonalLicenceDao(PersonalLicenceDao personalLicenceDao) {
		this.personalLicenceDao = personalLicenceDao;
	}


	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}


	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}


	@Transactional
	public Long store(PersonalLicence personalLicence, Person user) {
		if (personalLicence == null) {
			throw new RuntimeException("Cannot save a null personalLicence");
		}
		return personalLicenceDao.store(personalLicence, user);
	}


	@Transactional
	public int linkToPpl(Long personalLicenceId, Long projectLicenceId) {
		return personalLicenceDao.linkToPpl(personalLicenceId, projectLicenceId);
	}


	@Transactional
	public int clearLinksToPpls(Long personalLicenceId) {
		return personalLicenceDao.clearLinksToPpls(personalLicenceId);
	}


	public PersonalLicence fetchById(Long id, ComponentMap requiredComponents) {
		PersonalLicence personalLicence = personalLicenceDao.fetchById(id);

		if (personalLicence != null)
			attachComponents(personalLicence, requiredComponents);

		return personalLicence;
	}


	public PersonalLicence fetchByPersonId(Long personId, ComponentMap requiredComponents) {
		PersonalLicence personalLicence = personalLicenceDao.fetchByPersonId(personId);

		if (personalLicence != null)
			attachComponents(personalLicence, requiredComponents);

		return personalLicence;
	}


	public Collection<PersonalLicence> fetchCurrent(ComponentMap requiredComponents) {
		Collection<PersonalLicence> pils = personalLicenceDao.fetchCurrent();

		// force the addition of person to the personal licence (for display of person's name)
		if (requiredComponents == null) {
			requiredComponents = ComponentMap.of(PersonalLicence.Component.PERSON);
		}

		attachComponents(pils, requiredComponents);

		return (pils == null) ? new ArrayList<PersonalLicence>() : pils;
	}


	/**
	 * Returns the PersonalLicence based on the passed PIL number or null if none is found.
	 *
	 * @param personalLicenceNumber -
	 *            the string PIL number such as '50/3052'.
	 * @param requiredComponents -
	 *            any required components, see attachComponents method.
	 * @return personalLicence - the PersonalLicence or null if none found.
	 */
	public PersonalLicence fetchByPersonalLicenceNumber(String personalLicenceNumber, ComponentMap requiredComponents) {
		PersonalLicence personalLicence = personalLicenceDao.fetchByPersonalLicenceNumber(personalLicenceNumber);
		if (personalLicence != null)
			attachComponents(personalLicence, requiredComponents);
		return personalLicence;
	}


	@Override
	protected void attachComponents(PersonalLicence personalLicence, ComponentMap requiredComponents) {
		if (personalLicence == null) {
			throw new RuntimeException("Cannot attach components to null personalLicence");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(PersonalLicence.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(PersonalLicence.Component.PERSON)) {
				personalLicence.setPerson(fetchPerson(personalLicence, requiredComponents));
			}
		}
	}


	private Person fetchPerson(PersonalLicence personalLicence, ComponentMap requiredComponents) {
		if (personalLicence.getPersonId() != null) {
			return personService.fetchById(personalLicence.getPersonId(), requiredComponents);
		} else {
			return null;
		}
	}


	/**
	 * Returns true if the person with the personal licence (PIL) and the project licence (PPL) can
	 * work on the passed standard operating procedure.
	 *
	 * @param pil -
	 *            the persons personal licence.
	 * @param ppl -
	 *            the project licence.
	 * @param sop -
	 *            the standard operating procedure associated with the procedure.
	 * @return true - if the pil and project are licenced to carry out the procedure, otherwise
	 *         false.
	 */
	public boolean isLicenced(PersonalLicence pil, ProjectLicence ppl, StdOperatingProcedure sop) {
		return personalLicenceDao.isLicenced(pil.getId(), ppl.getId(), sop.getProcedureId());
	}
}
