/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.licensing.services;

import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.dao.ProjectLicenceDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class ProjectLicenceServiceImpl extends AbstractServiceImpl<ProjectLicence> implements ProjectLicenceService {

	private ProjectLicenceDao	projectLicenceDao;
	private ProtocolService		protocolService;
	private PersonService		personService;


	public Collection<ProjectLicence> fetchAll(ComponentMap requiredComponents) {
		Collection<ProjectLicence> licences = projectLicenceDao.fetchAll();
		attachComponents(licences, requiredComponents);
		return licences;
	}


	public Collection<ProjectLicence> fetchCurrent(ComponentMap requiredComponents) {
		Collection<ProjectLicence> licences = projectLicenceDao.fetchCurrent();
		attachComponents(licences, requiredComponents);
		return licences;
	}


	public ProjectLicence fetchById(Long id, ComponentMap requiredComponents) {
		ProjectLicence projectLicence = projectLicenceDao.fetchById(id);

		if (projectLicence != null)
			attachComponents(projectLicence, requiredComponents);

		return projectLicence;
	}


	/**
	 * Returns the project licence associated with the passed licence number or null if none found.
	 *
	 * @param licenceNumber -
	 *            the string licence number to find the project licence for.
	 * @param requiredComponents -
	 *            the required components - see attachComponents method.
	 * @return projectLicence - the ProjectLicence or null if nothing found.
	 */
	public ProjectLicence fetchByLicenceNumber(String licenceNumber, ComponentMap requiredComponents) {
		ProjectLicence projectLicence = projectLicenceDao.fetchByLicenceNumber(licenceNumber);
		if(projectLicence != null)
			attachComponents(projectLicence, requiredComponents);
		return projectLicence;
	}


	public Collection<ProjectLicence> fetchByPersonalLicenceId(Long personalLicenceId, ComponentMap requiredComponents) {
		Collection<ProjectLicence> licences = projectLicenceDao.fetchByPersonalLicenceId(personalLicenceId);
		attachComponents(licences, requiredComponents);
		return licences;
	}


	public Collection<ProjectLicence> fetchByLicenceHolder(Long personId, ComponentMap requiredComponents) {
		Collection<ProjectLicence> licences = projectLicenceDao.fetchByLicenceHolder(personId);
		attachComponents(licences, requiredComponents);
		return licences;
	}


	public Collection<ProjectLicence> fetchByPersonalLicenceAndProcedureId(Long personalLicenceId, Long procedureId,
			ComponentMap requiredComponents) {
		Collection<ProjectLicence> licences = projectLicenceDao.fetchByPersonalLicenceAndProcedureId(personalLicenceId,
				procedureId);
		if (!licences.isEmpty())
			attachComponents(licences, requiredComponents);

		return licences;
	}


	@Override
	protected void attachComponents(ProjectLicence projectLicence, ComponentMap requiredComponents) {
		if (projectLicence == null) {
			throw new RuntimeException("Cannot attach components to null projectLicence");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(ProjectLicence.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(ProjectLicence.Component.RESPONSIBLE_PERSON)) {
				projectLicence.setResponsiblePerson(fetchResponsiblePerson(projectLicence, requiredComponents
						.get(ProjectLicence.Component.RESPONSIBLE_PERSON)));
			}

			if (fetchAll || requiredComponents.containsKey(ProjectLicence.Component.PROTOCOLS)) {
				projectLicence.setProtocols(fetchProtocols(projectLicence, requiredComponents
						.get(ProjectLicence.Component.PROTOCOLS)));
			}
		}
	}


	private Person fetchResponsiblePerson(ProjectLicence projectLicence, ComponentMap requiredComponents) {
		if (projectLicence.getResponsiblePersonId() != null) {
			return personService.fetchById(projectLicence.getResponsiblePersonId(), requiredComponents);
		} else {
			return null;
		}
	}


	private Collection<Protocol> fetchProtocols(ProjectLicence projectLicence, ComponentMap requiredComponents) {
		return protocolService.fetchByProjectLicenceId(projectLicence.getId(), requiredComponents);
	}


	public Collection<ProjectLicence> fetchForAddToProcedure(Long procedureDictId, ComponentMap requiredComponents) {
		Collection<ProjectLicence> projectLicences = projectLicenceDao.fetchForAddToProcedure(procedureDictId);

		attachComponents(projectLicences, requiredComponents);

		return projectLicences;
	}


	// ///////////////////////////////////////////////////////////////////////////////////////////
	// //////////////////////////////// dependencies /////////////////////////////////////////////
	// ///////////////////////////////////////////////////////////////////////////////////////////
	public void setProjectLicenceDao(ProjectLicenceDao projectLicenceDao) {
		this.projectLicenceDao = projectLicenceDao;
	}


	public void setProtocolService(ProtocolService protocolService) {
		this.protocolService = protocolService;
	}


	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

}