/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.licensing.services;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.dao.ProtocolDao;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAge;
import uk.ac.sanger.mig.personnel.Person;

public class ProtocolServiceImpl extends AbstractServiceImpl<Protocol> implements
		ProtocolService {

	private ProtocolDao protocolDao;
	private ProjectLicenceService projectLicenceService;

	public void setProtocolDao(ProtocolDao protocolDao) {
		this.protocolDao = protocolDao;
	}

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}

	public Collection<Protocol> fetchByProjectLicenceId(Long projectLicenceId, ComponentMap requriedComponents) {
		Collection<Protocol> protocols = protocolDao.fetchByProjectLicenceId(projectLicenceId);
		attachComponents(protocols, requriedComponents);
		return protocols;
	}

	public Collection<Protocol> fetchByProcedureId(Long procedureId, ComponentMap requriedComponents) {
		Collection<Protocol> protocols = protocolDao.fetchByProcedureId(procedureId);
		attachComponents(protocols, requriedComponents);
		return protocols;
	}


	/**
	 * @see ProtocolService#is19bRequired(Mouse)
	 */
	public boolean is19bRequired(Mouse mouse) {
		boolean required = false;

		// if mouse is chimeric or has alleles, a 19b is required
		if(mouse.getChimericData() != null && mouse.getChimericData().getPercentChimeric() != null) {
			required = true;
		}
		else if(mouse.getAlleles() != null && mouse.getAlleles().size() > 0) {
			required = true;
		}

		// not required for <10.5 day old embryos
		MouseAge mouseAge = new MouseAge(mouse);
		if(mouseAge.isEmbryonic() && mouseAge.ageInDays() < 10.5) {
			required = false;
		}

		return required;
	}

	@Override
	protected void attachComponents(Protocol protocol, ComponentMap requiredComponents) {
		if(protocol == null) {
			throw new RuntimeException("Cannot attach components to null protocol");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Protocol.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Protocol.Component.PROJECT_LICENCE)) {
				protocol.setProjectLicence(fetchProjectLicence(protocol, requiredComponents.get(Protocol.Component.PROJECT_LICENCE)));
			}
		}
	}

	private ProjectLicence fetchProjectLicence(Protocol protocol, ComponentMap requiredComponents) {
		if(protocol.getProjectLicenceId() != null) {
			return projectLicenceService.fetchById(protocol.getProjectLicenceId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	public Protocol fetchById(Long id, ComponentMap requiredComponents) {
		Protocol protocol =  protocolDao.fetchById(id);
		if(protocol != null)
			attachComponents(protocol, requiredComponents);

		return protocol;
	}

	public Collection<Protocol> fetchByProcedureAndProject(Long procedureId, Long projectId, ComponentMap requiredComponents) {
		Collection<Protocol> protocols =  null;

		if(procedureId != null && projectId != null) {
			protocols = protocolDao.fetchByProcedureAndProject(procedureId, projectId);
		}
		attachComponents(protocols, requiredComponents);

		return (protocols == null) ? new ArrayList<Protocol>() : protocols;
	}

	public Protocol fetchByProcedureAndProject(Long procedureId, Long projectId, boolean isHarmfulMutant, ComponentMap requiredComponents) {
		Protocol protocol =  null;

		if(procedureId != null && projectId != null) {
			protocol = protocolDao.fetchByProcedureAndProject(procedureId, projectId, isHarmfulMutant);
		}

		if(protocol != null)
			attachComponents(protocol, requiredComponents);

		return protocol;
	}

	@Transactional
	public int linkToProcedure(Long procedureId, Long protocolId) {
		return protocolDao.linkToProcedure(procedureId, protocolId);
	}

	@Transactional
	public int unlinkFromProcedure(Long procedureId, Long protocolId) {
		return protocolDao.unlinkFromProcedure(procedureId, protocolId);
	}

	@Transactional
	public Long store(Protocol protocol, Person user) {
		return protocolDao.store(protocol, user);
	}
}
