/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.login;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.Security;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.novell.ldap.LDAPConnection;
import com.novell.ldap.LDAPException;
import com.novell.ldap.LDAPJSSESecureSocketFactory;

public class LdapAuthenticator {

	private String ldapServer;
	private String trustStore;
	private static Log logger = LogFactory.getLog(LdapAuthenticator.class);

	/**
	 * Create an instance of a LDAPAuthenticator that is associated with the supplied
	 * LDAP server together with the location of a trustStore that contains the CA root
	 * certificate if this is required for SSL validation.
	 *  
	 * @param ldapServer is the String containing the DNS name of the LDAP server.
	 * @param trustStore is a String containing the location of a file that contains the
	 *     CA root certificate. This needs to be in the format produced by the java keytool
	 *     utility. (e.g. run keytool -keystore sanger.ca -import -file cacert.crt, where the
	 *     cacert.crt is the file available from  https://ca.sanger.ac.uk/)
	 */
	public LdapAuthenticator(String ldapServer, String trustStore) {
		this.ldapServer = ldapServer;
		this.trustStore = trustStore;
	}

	/**
	 * Authenticate a user against the Sanger LDAP server. This connects to the server using
	 * SSL and determines if the user is valid based on whether they can successfully bind to
	 * an entry named "uid=user,ou=people,dc=sanger,dc=ac,dc=uk". If this successfuly binds then
	 * we have a valid password, otherwise we fail it.
	 * 
	 * @param server is the server name for the LDAP server
	 * @param caFile is the location of a keyStore file that contains the root certificate used to
	 *     sign the LDAP server certificate.
	 * @param user is the user name that needs to be authenticated
	 * @param passwd is the password for the user.
	 * @return true if the user name and password can be found in the LDAP database.
	 */
	public boolean authenticate(String user, String password)
	throws RuntimeException {

		// for debugging show where ldap authentication key should be located
		if(logger.isDebugEnabled()) {
			try {
				String currentWorkingDirectory = new File(".").getCanonicalPath();
				logger.debug("Looking for sanger.ca file in this directory: \"" + currentWorkingDirectory + "\"");
			}
			catch(IOException e) {
				e.printStackTrace();
			}
		}

		int ldapVersion = LDAPConnection.LDAP_V3;

		String objectDN = "uid=" + user + ",ou=people,dc=sanger,dc=ac,dc=uk";

		// Dynamically set Sun as the security provider
		Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

		// Dynamically set the property that JSSE uses to identify
		// the keystore that holds trusted root certificates

		System.setProperty("javax.net.ssl.trustStore", trustStore);

		LDAPJSSESecureSocketFactory ssf = new LDAPJSSESecureSocketFactory();
		LDAPConnection lc = new LDAPConnection(ssf);

		boolean passOK = false;

		try {
			// connect to the server as anonymous over an SSL connection
			lc.connect(ldapServer, LDAPConnection.DEFAULT_SSL_PORT);

			// Try and connect to the objectDN node
			lc.bind(ldapVersion, objectDN, password.getBytes("UTF8"));
			if (lc.isBound()) {
				passOK = true;
			}
		} catch (LDAPException e) {
			if (e.getResultCode() == LDAPException.INVALID_CREDENTIALS) {
				// NO-OP - the user password is probably just duff.
			} else {
				throw new RuntimeException(e);
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			try {
				lc.disconnect();
			} catch (LDAPException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return passOK;
	}	
}
