/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.microscopy.services;

import java.awt.Dimension;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.services.ImageProcessingException;
import uk.ac.sanger.mig.common.services.ImageProcessingService;
import uk.ac.sanger.mig.filesupport.services.FileSystemService;
import uk.ac.sanger.mig.microscopy.MetaData;
import uk.ac.sanger.mig.microscopy.ReadOnlyMetaData;
import uk.ac.sanger.mig.personnel.Person;

/**
 * @author ae1
 *
 */
public class ImageSweepServiceImpl implements ImageSweepService {

	//protected Log logger = LogFactory.getLog(this.getClass());

	private FileSystemService fileSystemService;
	private ImageProcessingService imageProcessingService;
	private MetaDataService metaDataService;
	private String microscopyDirectory;
	private int maxWidth, maxHeight;
	private String extension;
	private String defaultWinDirRoot, defaultUnixDirRoot;


	@Transactional
	public void sweep(ReadOnlyMetaData readOnlyMetaData, Person user)
			throws IOException, InterruptedException,
				    FileNotFoundException, ImageProcessingException {

		File translatedSourcePath = fileSystemService.translatePath(
				readOnlyMetaData.getImagePath(),
				defaultWinDirRoot, defaultUnixDirRoot);

		if( translatedSourcePath.canRead() ) {

			File destPath = createDestPath( translatedSourcePath.getName() );
			imageProcessingService.convert( translatedSourcePath, destPath, maxWidth, maxHeight );
			imageProcessingService.publish( destPath.getAbsolutePath() );
			MetaData metaData = new MetaData();
			metaData.setConvertedImagePath( destPath.getAbsolutePath() );
			metaData.setImageUrl(
					imageProcessingService.filePathToUrl(
							metaData.getConvertedImagePath() ) );

			Dimension scaledDim = imageProcessingService.calculateScaledDimension(
					new Dimension(readOnlyMetaData.getImageWidth(),
								  readOnlyMetaData.getImageHeight()), maxWidth, maxHeight);
			metaData.setImageWidth( (int)scaledDim.getWidth() );
			metaData.setImageHeight( (int)scaledDim.getHeight() );

			metaData.setReadOnlyMetaDataId( readOnlyMetaData.getId() );
			metaDataService.store( metaData, user );

		} else {
			throw new FileNotFoundException("Unable to read source path: " +
					translatedSourcePath.getAbsolutePath());
		}
	}

	/*
	 * TODO make work on windows for localhost testing
	 */
	private File createDestPath(String sourceFileName) throws FileNotFoundException {

		File destSubDir =  new File(
						imageProcessingService.getViewableRootDir() +
						File.separator +
						microscopyDirectory +
	    				File.separator +
	    				fileSystemService.generateRandomHex(sourceFileName,1) );

		String destFileName = sourceFileName.replaceFirst("\\.\\w+$", extension);

		if( !destSubDir.exists() && !destSubDir.mkdir() )
			throw new FileNotFoundException("Unable to create required subDir" + destSubDir);

		return new File(destSubDir +
						File.separator +
						destFileName);
	}

	public void setFileSystemService(FileSystemService fileSystemService) {
		this.fileSystemService = fileSystemService;
	}

	public void setImageProcessingService(
			ImageProcessingService imageProcessingService) {
		this.imageProcessingService = imageProcessingService;
	}

	public void setMetaDataService(MetaDataService metaDataService) {
		this.metaDataService = metaDataService;
	}



	public void setMaxHeight(int maxHeight) {
		this.maxHeight = maxHeight;
	}



	public void setMaxWidth(int maxWidth) {
		this.maxWidth = maxWidth;
	}


	public void setExtension(String extension) {
		this.extension = extension;
	}

	public void setMicroscopyDirectory(String microscopyDirectory) {
		this.microscopyDirectory = microscopyDirectory;
	}

	public void setDefaultUnixDirRoot(String defaultUnixDirRoot) {
		this.defaultUnixDirRoot = defaultUnixDirRoot;
	}

	public void setDefaultWinDirRoot(String defaultWinDirRoot) {
		this.defaultWinDirRoot = defaultWinDirRoot;
	}


}
