/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing;

import java.util.Date;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.personnel.Person;


public class EmbryoHarvest extends AuthenticatedDataObject  {
	
	public enum Component implements ComponentEnum {
		TECHNICIAN,
		ALL_COMPONENTS
	}
   
	private static final Integer DONOR_COUNT_CIELING 		= 60;
	private static final Long DEFAULT_MATERNAL_STRAIN       = 3L; //BL6
	private static final Long DEFAULT_MATERNAL_COAT_COLOR   = 9L;	//CALB - albino
	private static final Integer DEFAULT_EMBRYO_GRADE_COUNT = 0;	
	private static final Long TYP_STRAIN_C57BL6CC_CALB = 3L; //donor male variant
	private static final Long TYP_STRAIN_C57BL6J_CBLJ = 1L; //donor male variant
	private static final String DEFAULT_EMBRYO_AGE = "E3.5";
	
    private String comments;        
    private boolean overnightIncubation;
    private Date flushDate;    
    private Date plugCheckDate;    
    private Integer totalDonors;   
    private Long motherStrainId 		  = DEFAULT_MATERNAL_STRAIN;
    private DictionaryEntry motherStrain;
    private Long motherCoatColourDictId   = DEFAULT_MATERNAL_COAT_COLOR;
    private DictionaryEntry motherCoatColourDict;
    private Long fatherStrainId;
    private DictionaryEntry fatherStrain;
    private Long fatherCoatColourDictId;
    private DictionaryEntry fatherCoatColourDict; 
    private Person technician;
    private Long technicianId;
    
    //TODO make into separate object, Collection of EmbryoGrade objects
    private Integer blastocystCount 	  = DEFAULT_EMBRYO_GRADE_COUNT;        
    private Integer moralaeCount 		  = DEFAULT_EMBRYO_GRADE_COUNT;        
    private Integer eightCellCompCount 	  = DEFAULT_EMBRYO_GRADE_COUNT;        
    private Integer eightCellNonCompCount = DEFAULT_EMBRYO_GRADE_COUNT;        
    private Integer fourCellCount 		  = DEFAULT_EMBRYO_GRADE_COUNT;        
    private Integer hatchedCount 		  = DEFAULT_EMBRYO_GRADE_COUNT;        
    private Integer deadOrFragmentedCount = DEFAULT_EMBRYO_GRADE_COUNT; 
    private Integer otherEmbryoTypeCount  = DEFAULT_EMBRYO_GRADE_COUNT;   
    private boolean useBlastocysts;
    private boolean useMoralae;
    private boolean useEightCells;
    private boolean useEightCellNonComp;
    private boolean useFourCell;
    private boolean useHatched;
    private boolean useDeadOrFragmented;
    private boolean useOther;
    
    
    
    public static String getDefaultDonorEmbryoAge() {
		return DEFAULT_EMBRYO_AGE;
	}

	public static Long getPaternalDonorStrainForAlbino() {
    	return TYP_STRAIN_C57BL6CC_CALB;
    }
 
    public static Long getPaternalDonorStrainForBlack() {
    	return TYP_STRAIN_C57BL6J_CBLJ;
    }
    
    public Integer getInjectableHarvestCount() {
    	Integer total = 0;
    	total+= (useBlastocysts)      ? blastocystCount       : 0;
    	total+= (useMoralae)          ? moralaeCount          : 0;
    	total+= (useEightCells)       ? eightCellCompCount    : 0;
    	total+= (useEightCellNonComp) ? eightCellNonCompCount : 0;
    	total+= (useFourCell)     	  ? fourCellCount         : 0;
    	total+= (useHatched)     	  ? hatchedCount          : 0;
    	total+= (useDeadOrFragmented) ? deadOrFragmentedCount : 0;
    	total+= (useOther)     		  ? otherEmbryoTypeCount  : 0;
    	return total;
    }
    
	public Integer getTotalEmbryoCount() {
		
		return (otherEmbryoTypeCount != null ? otherEmbryoTypeCount : 0) + 
		       (blastocystCount  != null ? blastocystCount : 0) +
		       (moralaeCount  != null ? moralaeCount : 0) +
		       (eightCellCompCount != null ? eightCellCompCount : 0) + 
		       (eightCellNonCompCount != null ? eightCellNonCompCount : 0) + 
		       (fourCellCount != null ? fourCellCount : 0) + 
		       (hatchedCount != null ? hatchedCount : 0) + 
		       (deadOrFragmentedCount != null ? deadOrFragmentedCount : 0);  
	}

	public static Integer getDonorCountCeiling() {
		return DONOR_COUNT_CIELING;
	}
	
	//////////////////////////////////////////////////////////
	///////////////// bit bucket methods /////////////////////
	//////////////////////////////////////////////////////////	
	public boolean isUseDeadOrFragmented() {
		return useDeadOrFragmented;
	}

	public void setUseDeadOrFragmented(boolean deadOrFragmented) {
		this.useDeadOrFragmented = deadOrFragmented;
	}

	public boolean isUseOther() {
		return useOther;
	}

	public void setUseOther(boolean other) {
		this.useOther = other;
	}

	public boolean isUseBlastocysts() {
		return useBlastocysts;
	}

	public void setUseBlastocysts(boolean useBlastocysts) {
		this.useBlastocysts = useBlastocysts;
	}

	public boolean isUseEightCellNonComp() {
		return useEightCellNonComp;
	}

	public void setUseEightCellNonComp(boolean useEightCellNonComp) {
		this.useEightCellNonComp = useEightCellNonComp;
	}

	public boolean isUseEightCells() {
		return useEightCells;
	}

	public void setUseEightCells(boolean useEightCells) {
		this.useEightCells = useEightCells;
	}

	public boolean isUseFourCell() {
		return useFourCell;
	}

	public void setUseFourCell(boolean useFourCell) {
		this.useFourCell = useFourCell;
	}

	public boolean isUseHatched() {
		return useHatched;
	}

	public void setUseHatched(boolean useHatched) {
		this.useHatched = useHatched;
	}

	public boolean isUseMoralae() {
		return useMoralae;
	}

	public void setUseMoralae(boolean useMoralae) {
		this.useMoralae = useMoralae;
	}


    public Date getFlushDate() {
		return flushDate;
	}

	public void setFlushDate(Date flushDate) {
		this.flushDate = flushDate;
	}

	public String getComments() {
        return comments;
    }
    
    public void setComments(String comments) {
        this.comments = comments;
    }                                                                                                                                            
    public Integer getOtherEmbryoTypeCount() {
        return otherEmbryoTypeCount;
    }
    
    public void setOtherEmbryoTypeCount(Integer totalEmbryoCount) {
        this.otherEmbryoTypeCount = totalEmbryoCount;
    }                                                                                                                                            
    public Integer getBlastocystCount() {
        return blastocystCount;
    }
    
    public void setBlastocystCount(Integer blastocystCount) {
        this.blastocystCount = blastocystCount;
    }                                                                                                                                            
    public Integer getMoralaeCount() {
        return moralaeCount;
    }
    
    public void setMoralaeCount(Integer moralaeCount) {
        this.moralaeCount = moralaeCount;
    }                                                                                                                                            
    public Integer getEightCellCompCount() {
        return eightCellCompCount;
    }
    
    public void setEightCellCompCount(Integer eightCellCompCount) {
        this.eightCellCompCount = eightCellCompCount;
    }                                                                                                                                            
    public Integer getEightCellNonCompCount() {
        return eightCellNonCompCount;
    }
    
    public void setEightCellNonCompCount(Integer eightCellNonCompCount) {
        this.eightCellNonCompCount = eightCellNonCompCount;
    }                                                                                                                                            
    public Integer getFourCellCount() {
        return fourCellCount;
    }
    
    public void setFourCellCount(Integer fourCellCount) {
        this.fourCellCount = fourCellCount;
    }                                                                                                                                            
    public Integer getHatchedCount() {
        return hatchedCount;
    }
    public void setHatchedCount(Integer hatchedCount) {
        this.hatchedCount = hatchedCount;
    }                                                                                                                                            
    public Integer getDeadOrFragmentedCount() {
        return deadOrFragmentedCount;
    }
    public void setDeadOrFragmentedCount(Integer deadOrFragmentedCount) {
        this.deadOrFragmentedCount = deadOrFragmentedCount;
    }

	public boolean isOvernightIncubation() {
		return overnightIncubation;
	}

	public void setOvernightIncubation(boolean overNightIncubation) {
		this.overnightIncubation = overNightIncubation;
	}

	@Deprecated
	public DictionaryEntry getMotherCoatColourDict() {
		return motherCoatColourDict;
	}

	@Deprecated
	public void setMotherCoatColourDict(DictionaryEntry donorCoatColourDict) {
		this.motherCoatColourDict = donorCoatColourDict;
	}

	@Deprecated
	public Long getMotherCoatColourDictId() {
		return motherCoatColourDictId;
	}

	@Deprecated
	public void setMotherCoatColourDictId(Long donorCoatColourDictId) {
		this.motherCoatColourDictId = donorCoatColourDictId;
	}

	public DictionaryEntry getMotherStrain() {
		return motherStrain;
	}

	public void setMotherStrain(DictionaryEntry donorStrain) {
		this.motherStrain = donorStrain;
	}

	public Long getMotherStrainId() {
		return motherStrainId;
	}
	
	public void setMotherStrainId(Long donorStrainId) {
		this.motherStrainId = donorStrainId;
	}

	@Deprecated
	public DictionaryEntry getFatherCoatColourDict() {
		return fatherCoatColourDict;
	}

	@Deprecated
	public void setFatherCoatColourDict(DictionaryEntry fatherCoatColourDict) {
		this.fatherCoatColourDict = fatherCoatColourDict;
	}

	@Deprecated
	public Long getFatherCoatColourDictId() {
		return fatherCoatColourDictId;
	}
	
	@Deprecated
	public void setFatherCoatColourDictId(Long fatherCoatColourDictId) {
		this.fatherCoatColourDictId = fatherCoatColourDictId;
	}

	public DictionaryEntry getFatherStrain() {
		return fatherStrain;
	}

	public void setFatherStrain(DictionaryEntry fatherStrain) {
		this.fatherStrain = fatherStrain;
	}

	public Long getFatherStrainId() {
		return fatherStrainId;
	}

	public void setFatherStrainId(Long fatherStrainId) {
		this.fatherStrainId = fatherStrainId;
	}

	public Integer getTotalDonors() {
		return totalDonors;
	}

	public void setTotalDonors(Integer totalDonors) {
		this.totalDonors = totalDonors;
	}

	public Date getPlugCheckDate() {
		return this.plugCheckDate;
	}

	public void setPlugCheckDate(Date plugCheckDate) {
		this.plugCheckDate = plugCheckDate;
	}

	public Person getTechnician() {
		return technician;
	}

	public void setTechnician(Person technician) {
		this.technician = technician;
	}

	public Long getTechnicianId() {
		return technicianId;
	}

	public void setTechnicianId(Long technicianId) {
		this.technicianId = technicianId;
	}

                                                                                                                        	
}