/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing;

import java.util.Calendar;
import java.util.Date;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.cryoprocessing.ThawTest;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;

public class EmbryoTransferProcedure extends AuthenticatedDataObject{

	public enum Component implements ComponentEnum {
		SCHEDULED_MICRO_INJECTION,
		MOUSE_PROCEDURE,
		MOUSE,
		THAW_TEST,
		ALL_COMPONENTS
	}

	private final static Long DEFAULT_TRANSFER_TYPE_DICT_ID = 1L;//uterine
	private final static Long DEFAULT_ANAESTHETIC_DICT_ID = 2L;  //ul (10mg/ml Ketamine/ 0.1% Xylazine)
	private final static Long DEFAULT_ANALGESIA_DICT_ID = 1L;//Analgesia
	private final static String DEFAULT_EMBRYO_AGE = "2.5";
	private final static String OVERNIGHT_EMBRYO_AGE = "0.5";
	private final static Long DEFAULT_RECIPIENT_STRAIN_ID = 6L;
	private final static Double DOSE_ANALGESIA = 100D;

	private ScheduledMicroInjectionJob scheduledMicroInjection;
	private Long scheduledMicroInjectionId;
	private Long transferTypeDictId = DEFAULT_TRANSFER_TYPE_DICT_ID;
	private DictionaryEntry transferTypeDict;
    private Double recipientWeight;
    private Integer numberRight;
    private Integer numberLeft;
    private Long anaestheticDictId  = DEFAULT_ANAESTHETIC_DICT_ID;
    private DictionaryEntry anaestheticDict;
    private Double doseAnaesthetic;
    private Long analgesiaDictId    = DEFAULT_ANALGESIA_DICT_ID;
    private DictionaryEntry analgesiaDict;
    private Double doseAnalgesia = DOSE_ANALGESIA;
    private DictionaryEntry stufferStrainDict;
    private Long stufferStrainDictId;
    private Integer numberRightStuffers;
    private Integer numberLeftStuffers;
    private String comments;
    private MouseProcedure mouseProcedure;
    private Long mouseId;
    private Mouse mouse;
    private String daysFromPlug = DEFAULT_EMBRYO_AGE;
    private ThawTest thawTest;
    private Long thawTestId;


    public Date getDueDate() {

    	if( mouseProcedure == null )
    		throw new RuntimeException("MouseProcedure null! Required to be attaced to EmbryoTransfer!");

    	Calendar cal = Calendar.getInstance();

    	Date transferDate = mouseProcedure.getStartDate();
    	cal.setTime( transferDate );

    	if( OVERNIGHT_EMBRYO_AGE.equals(daysFromPlug) )
    		cal.add(Calendar.DAY_OF_YEAR, 19);
    	else if(DEFAULT_EMBRYO_AGE.equals(daysFromPlug))
    		cal.add(Calendar.DAY_OF_YEAR, 17);
    	else
    		cal.add(Calendar.DAY_OF_YEAR, 17);

    	return cal.getTime();
    }

    public Date getPlugDate() {
    	if( mouseProcedure == null )
    		throw new RuntimeException("MouseProcedure null! Required to be attaced to EmbryoTransfer!");

    	Calendar cal = Calendar.getInstance();

    	Date transferDate = mouseProcedure.getEndDate();
    	cal.setTime( transferDate );

    	if( OVERNIGHT_EMBRYO_AGE.equals( daysFromPlug ) )
    		cal.add(Calendar.DAY_OF_YEAR, -1);
    	else if( DEFAULT_EMBRYO_AGE.equals(daysFromPlug) )
    		cal.add(Calendar.DAY_OF_YEAR, -3);
    	else
    		cal.add(Calendar.DAY_OF_YEAR, -3);

    	return cal.getTime();

    }

    public static Long getDefaultRecipientStrainId() {
		return DEFAULT_RECIPIENT_STRAIN_ID;
	}

	public static String getDefaultRecipientEmbryoAge() {
		return DEFAULT_EMBRYO_AGE;
	}

	/**
     * Returns the MicroInjection for this EmbryoTrasnfer
     * @return
     */
    public MicroInjection getMicroInjection () {
    	return (scheduledMicroInjection != null ) ?
    			scheduledMicroInjection.getMicroInjection() : null;
    }

    /**
     * Returns the plug check date of the EmbryoHarvest from this
     * MicroInjection for this EmbryoTrasnfer
     * @return
     */
    public Date getHarvestPlugCheckDate() {
    	return (getMicroInjection () != null) ?
    			getMicroInjection().getHarvestPlugCheckDate() : null;
    }

    /**
     * Returns the earmark of the recipient Mouse used in this EmbryoTransfer
     * @return
     */
    public Long getEarmarkId() {
    	return getMouse().getEarmarkDictId();
    }

    /**
     * Returns the requester's person id for the associated MiRequest
     * @return
     */
    public Long getMiRequesterId() {
    	return getScheduledMicroInjection().getMiRequest().getRequesterId();
    }

    /**
     * Returns the Personal Licence holders Person Id
     * @return
     */
    public Long getLicenceHolderId() {
    	return getMouseProcedure().getPersonalLicence().getPersonId();
    }

    /**
     * Returns the totals embryos transfered
     * @return
     */
    public Integer getTotalEmbryosTransferred() {
    	return ( (numberRight != null) ? numberRight : 0 ) +
    	       ( (numberLeft != null) ? numberLeft : 0 );
    }

    /**
     * Returns the totals stuffer embryos transfered
     * @return
     */
	public Integer getTotalStuffers() {
    	return ( (numberRightStuffers != null) ? numberRightStuffers : 0 ) +
	       ( (numberLeftStuffers != null) ? numberLeftStuffers : 0 );
	}

	//////////////////////////////////////////////////////////
	///////////////// bit bucket methods /////////////////////
	//////////////////////////////////////////////////////////
	public Mouse getMouse() {
		return mouse;
	}
	public void setMouse(Mouse mouse) {
		this.mouse = mouse;
	}
	public Long getMouseId() {
		return mouseId;
	}
	public void setMouseId(Long mouseId) {
		this.mouseId = mouseId;
	}
	public String getComments() {
		return comments;
	}
	public void setComments(String comments) {
		this.comments = comments;
	}
	public DictionaryEntry getTransferTypeDict() {
		return transferTypeDict;
	}
	public void setTransferTypeDict(DictionaryEntry transferTypeDict) {
		this.transferTypeDict = transferTypeDict;
	}
	public Long getTransferTypeDictId() {
		return transferTypeDictId;
	}
	public void setTransferTypeDictId(Long transferTypeDictId) {
		this.transferTypeDictId = transferTypeDictId;
	}
	public DictionaryEntry getAnaestheticDict() {
		return anaestheticDict;
	}
	public void setAnaestheticDict(DictionaryEntry anaestheticDict) {
		this.anaestheticDict = anaestheticDict;
	}
	public Long getAnaestheticDictId() {
		return anaestheticDictId;
	}
	public void setAnaestheticDictId(Long anaestheticDictId) {
		this.anaestheticDictId = anaestheticDictId;
	}
	public DictionaryEntry getAnalgesiaDict() {
		return analgesiaDict;
	}
	public void setAnalgesiaDict(DictionaryEntry analgesiaDict) {
		this.analgesiaDict = analgesiaDict;
	}
	public Long getAnalgesiaDictId() {
		return analgesiaDictId;
	}
	public void setAnalgesiaDictId(Long analgesiaDictId) {
		this.analgesiaDictId = analgesiaDictId;
	}
	public Double getDoseAnaesthetic() {
		return doseAnaesthetic;
	}
	public void setDoseAnaesthetic(Double doseAnaesthetic) {
		this.doseAnaesthetic = doseAnaesthetic;
	}
	public Double getDoseAnalgesia() {
		return doseAnalgesia;
	}
	public void setDoseAnalgesia(Double doseAnalgesia) {
		this.doseAnalgesia = doseAnalgesia;
	}


	public Integer getNumberLeft() {
		return numberLeft;
	}
	public void setNumberLeft(Integer numberLeft) {
		this.numberLeft = numberLeft;
	}
	public Integer getNumberLeftStuffers() {
		return numberLeftStuffers;
	}
	public void setNumberLeftStuffers(Integer numberLeftStuffers) {
		this.numberLeftStuffers = numberLeftStuffers;
	}
	public Integer getNumberRight() {
		return numberRight;
	}
	public void setNumberRight(Integer numberRight) {
		this.numberRight = numberRight;
	}

	public Integer getNumberRightStuffers() {
		return numberRightStuffers;
	}

	public void setNumberRightStuffers(Integer numberRightStuffers) {
		this.numberRightStuffers = numberRightStuffers;
	}

	public DictionaryEntry getStufferStrainDict() {
		return stufferStrainDict;
	}
	public void setStufferStrainDict(DictionaryEntry stufferStrainDict) {
		this.stufferStrainDict = stufferStrainDict;
	}
	public Long getStufferStrainDictId() {
		return stufferStrainDictId;
	}
	public void setStufferStrainDictId(Long stufferStrainDictId) {
		this.stufferStrainDictId = stufferStrainDictId;
	}
	public Double getRecipientWeight() {
		return recipientWeight;
	}
	public void setRecipientWeight(Double recipientWeight) {
		this.recipientWeight = recipientWeight;
	}
	public ScheduledMicroInjectionJob getScheduledMicroInjection() {
		return scheduledMicroInjection;
	}
	public void setScheduledMicroInjection(
			ScheduledMicroInjectionJob scheduledMicroInjection) {
		this.scheduledMicroInjection = scheduledMicroInjection;
	}

	public Long getScheduledMicroInjectionId() {
		return scheduledMicroInjectionId;
	}

	public void setScheduledMicroInjectionId(Long scheduledMicroInjectionId) {
		this.scheduledMicroInjectionId = scheduledMicroInjectionId;
	}

	public MouseProcedure getMouseProcedure() {
		return mouseProcedure;
	}

	public void setMouseProcedure(MouseProcedure mouseProdedure) {
		this.mouseProcedure = mouseProdedure;
	}

	public String getDaysFromPlug() {
		return this.daysFromPlug;
	}

	public void setDaysFromPlug(String daysFromPlug) {
		this.daysFromPlug = daysFromPlug;
	}

	public ThawTest getThawTest() {
		return thawTest;
	}

	public void setThawTest(ThawTest thawTest) {
		this.thawTest = thawTest;
	}

	public Long getThawTestId() {
		return thawTestId;
	}

	public void setThawTestId(Long thawTestId) {
		this.thawTestId = thawTestId;
	}

	public boolean isFromThawTest() {
		return (thawTestId != null) ? true : false;
	}

	public boolean isFromMicroInjection() {
		return (scheduledMicroInjectionId != null) ? true : false;
	}

}