/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import uk.ac.sanger.mig.animalrequest.AnimalRequest;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.DictionaryEntryEnum;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.miprocessing.workflow.Job;
import uk.ac.sanger.mig.miprocessing.workflow.Task;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.MiRequest.EndpointEnum;
import uk.ac.sanger.mig.personnel.Person;


public class ScheduledMicroInjectionJob extends AuthenticatedDataObject 
		implements Job<ScheduledMicroInjectionJob.MilestoneEnum> {

	public enum Component implements ComponentEnum {
		
		MI_REQUEST,
		TASKS,
		MICRO_INJECTION,
		EMBRYO_TRANSFERS,
		DONOR_ANIMAL_REQUEST,
		RECIPIENT_ANIMAL_REQUEST,
		SUPPLEMENTAL_RECIPIENT_ANIMAL_REQUEST,
		ACTUAL_CELL_LINE_PROVIDER,
		ALL_COMPONENTS  
	}
     	
	public enum InternalStatus implements DictionaryEntryEnum  {
		
		FAILED( 9L ),     //user driven
		CANCELLED( 10L ),  //user or system driven
		COMPLETED( 13L ),  //user driven
		SCHEDULED( 1L ),   //system driven
		MICRO_INJECTION_COMPLETED( 3L ),  //system driven
		EMBRYO_TRANSFER_COMPLETED( 4L ),   //system driven
		F0_WEANED( 6L ),         //system driven
		F1_GENOTYPED( 12L ),  //system driven
		EMBRYONIC_CHIMERAS_FLUSHED( 5L );  //system driven
		
		private Long dictionaryId;
		
		InternalStatus(Long dictId) {
			this.dictionaryId = dictId;
		}
			
		public Long getId() {
			return dictionaryId;
		}	
	}
	
	public enum MilestoneEnum  { //enums implement Comparrable by default
		
		MICRO_INJECTION_TASK_COMPLETED("Microinjection Completed", 0),//enum order is order in which declared
		EMBRYO_TRANSFER_TASK_COMPLETED("Embryo Transfer Completed", 1),
		CHIMERA_EMBRYO_HARVEST_TASK_COMPLETED ("Chimera Embryo Harvest", 2), //2 for Chimera Embryo  endpoint
		F0_WEANED_TASK_COMPLETED("F0s Weaned", 2),
		F1_GENOTYPING_TASK_COMPLETED("F1s Genotyped", 3);
		
		private String taskName;
		private int sequence;
		
		MilestoneEnum(String taskName, int sequence) {
			this.taskName = taskName; 
		}

		public String getTaskName() {
			return taskName;
		}

		public int getSequence() {
			return sequence;
		}
		
		@Override
		public String toString() {
			return getTaskName();
		}	
	}
	
	//filter status
	public static final Long FILTER_STATUS_ACTIVE = 0L;
	public static final Long FILTER_STATUS_CANCEL_FAIL = 1L;
	public static final Long FILTER_STATUS_ENSDPOINT_REACHED = 2L;
	public static final Long FILTER_NEXT_TASK_TO_COMPLETE_MI = 0L;
	public static final Long FILTER_NEXT_TASK_TO_COMPLETE_ET = 1L;
	public static final Long FILTER_NEXT_TASK_TO_COMPLETE_F0_WEANED = 2L;
	public static final Long FILTER_NEXT_TASK_TO_COMPLETE_F1_GENOTYPED = 3L;
	
	//private static final String F0_BIRTH = "F0s Born"; //TODO refactor to use key
	
	private MicroInjection microInjection;
	private Collection<EmbryoTransferProcedure> embryoTransfers;	
	private List<Task> scheduledMiTasks;
	private DictionaryEntry scheduledMiStatusDict;
    private Long scheduledMiStatusDictId;        
    private Date anchorDate;          
    private MiRequest miRequest; //
    private Long miRequestId;      
    
    //consider consolidating fields into their own WT request related class
    private AnimalRequest donorAnimalRequest;
    private AnimalRequest recipientAnimalRequest;
    private AnimalRequest supplementalRecipientAnimalRequest;  
    private Long donorAnimalRequestId, recipientAnimalRequestId, supplementalRecipientAnimalRequestId;    
    private boolean donorRequestEmailSent, recipientRequestEmailSent;
    private DictionaryEntry requiredDonorStrain; //paternal
    private DictionaryEntry requiredDonorCoatColor; //paternal
    ///////////////////////////////////////////////////////////////
    
    private String comments;        
    private String cancelOrFailReasons;
    private Long cellLineProviderId;
    private Person cellLineProvider;
    private Integer availableEmbryos;
    private String jobId;

    /**
     * Returns thenumber of EmbryoTrasnfers performed
     * in this  ScheduledMicroInjectionJob  
     * @return
     */
    public Integer getEmbryoTransferCount() {
    	return ( embryoTransfers != null) ? embryoTransfers.size() : 0;
    }

    
    /**
     * Returns the paternal strain coat colour required for the EmbryoHarvest
     * used in this ScheduledMicroInjectionJob 
     * @return
     */
    public String getRequiredHarvestCoatColour() {
    	return (requiredDonorCoatColor != null) ? requiredDonorCoatColor.getName() : null;
    }
    
    /**
     * Returns the completion date of the Microinjection performed
     * in this  ScheduledMicroInjectionJob  
     * 
     * Throws UnattachedComponentException if microInjection is null.
     * 
     * @return
     */
    public Date getMicroInjectionDateCompleted() {
 
		if( microInjection == null )
			throw new UnattachedComponentException(ScheduledMicroInjectionJob.Component.MICRO_INJECTION);
    	
    	return microInjection.getDateCompleted();
    }

    /**
     * Returns the number of injected embryos in this job.
     * 
     * Throws UnattachedComponentException if microInjection is null.
     * 
     * @return
     */
    public Integer getInjectedEmbryoCount() {
    	
		if( microInjection == null )
			throw new UnattachedComponentException(ScheduledMicroInjectionJob.Component.MICRO_INJECTION);
    	
    	return (microInjection.getInjectedEmbryoCount() != null) ?
    		microInjection.getInjectedEmbryoCount() : 0;
    }
    
    /**
     * Returns the number of unused microinjected embryos
     * in this  ScheduledMicroInjectionJob.
     * 
     *   
     * Throws UnattachedComponentException if embryoTransfers or microInjection
     * 
     *   
     * @return
     */
	public Integer getUnusedMicroInjectedEmbryos() {
		
		if( embryoTransfers == null )
			throw new UnattachedComponentException(ScheduledMicroInjectionJob.Component.EMBRYO_TRANSFERS);
		
		Integer totalTrasnferedEmbryos = 0;
		
		for(EmbryoTransferProcedure embryoTransfer : embryoTransfers) {
			totalTrasnferedEmbryos += embryoTransfer.getTotalEmbryosTransferred();
		}
		
		return ( getInjectedEmbryoCount() - totalTrasnferedEmbryos );
	}
  
	public Task getNextMilestoneTaskToComplete() {
		for(Task milestone : this.getMilestones()) {
			if(milestone.getActualCompletionDate() == null)
				return milestone;
		}
		return new NullScheduledMiTask();
	} 

	
	public boolean isAborted() {	
		return (scheduledMiStatusDictId.equals( InternalStatus.CANCELLED.getId() )
				|| scheduledMiStatusDictId.equals( InternalStatus.FAILED.getId()) );
	}
 
	public boolean isCompleted() {
		return  (scheduledMiStatusDictId.equals( InternalStatus.COMPLETED.getId()) );
	}
	
	public boolean isActive() {
    	return !( scheduledMiStatusDictId.equals( InternalStatus.FAILED.getId() ) 
    			|| scheduledMiStatusDictId.equals( InternalStatus.CANCELLED.getId() )
    			|| scheduledMiStatusDictId.equals( InternalStatus.COMPLETED.getId() ) );
    	
    }	   
    
	/**
	 * Returns true if this Job has any active sibling jobs. Required components
	 * MiRequest and Collection of Jobs for that MiRequest.
	 * @return
	 */
	public boolean hasActiveSiblings() {
			
		if( getMiRequest() == null)
			throw new UnattachedComponentException( ScheduledMicroInjectionJob.Component.MI_REQUEST );
		
		boolean foundActiveSibling = false;
		
		for(Job siblingJob : getMiRequest().getActiveJobs() ) {
			if( siblingJob.equals( this )) continue; //skip if this job, only want sibs
			foundActiveSibling = siblingJob.isActive();
		}
		return foundActiveSibling;
	}
	
	/**
	 * Returns sub-list of tasks that are milestones.
	 */
    public List<Task> getMilestones() {
    	
    	if( scheduledMiTasks == null)
    		throw new UnattachedComponentException( ScheduledMicroInjectionJob.Component.TASKS );
    	
    	List<Task> milestones = new ArrayList<Task>();
    	for(Task milestoneTask : getTasks()) {
    		if(milestoneTask.isMilestone()) {
    			milestones.add(milestoneTask);
    		}	
    	}
    	return milestones;
    }
      
    /**
     * Looks up and returns a Task referenced by the milestoneEnum .
     * 
     * Returns a NullScheduledMiTask if no Task can be found or
     * if milestoneEnum is null.
     * 
     */
    public Task getMilestoneByEnum(MilestoneEnum milestoneEnum) {
    	
    	if( milestoneEnum != null ) {
	    	
    		List<Task> milestones = this.getMilestones();	
	    	
    		for(Task milestone: milestones) {
	    		
    			if(milestone.getName().equals(milestoneEnum.getTaskName())) {
	    			return milestone;			
    			}
	    	}
    	}
    	
    	return new NullScheduledMiTask();
	}

	public boolean isEndpointReached() {		
    		

		if(getEndpointId().equals( EndpointEnum.F1.getId() ) &&
				getMilestoneByEnum(MilestoneEnum.F1_GENOTYPING_TASK_COMPLETED).isComplete()) {
    		
    		return true;
    		
    	} else if(getEndpointId().equals( EndpointEnum.F0.getId() ) &&
    			getMilestoneByEnum(MilestoneEnum.F0_WEANED_TASK_COMPLETED).isComplete()) {
    		
    		return true;
    		
    	} else if(getEndpointId().equals( EndpointEnum.EMBRYONIC_CHIMERAS.getId() ) && 
    			getMilestoneByEnum(MilestoneEnum.EMBRYO_TRANSFER_TASK_COMPLETED).isComplete()) {
    			
    		return true;
    	} 
		
    	return false;
   
    } 

	/**
	 * Changes the status of this request (NOTE: both dict and dictId 
	 * changed for maintaining consistent object state!)
	 * @param newStatus
	 */
	public void changeStatus(DictionaryEntry newStatus) {
		scheduledMiStatusDict = newStatus;
		scheduledMiStatusDictId = newStatus.getId();
	}
  
	
	
	public void setTasks(List<Task> tasks) {
		
		this.scheduledMiTasks = tasks;
	}    	
    
	//////////////////////////////////////////////////////////
	///////////////// bit bucket methods /////////////////////
	//////////////////////////////////////////////////////////	
    public String getJobId() {
		return this.jobId;
	}

	public void setJobId(String jobId) {

		this.jobId = jobId;
	}
	
	public DictionaryEntry getRequiredDonorCoatColor() {
		return requiredDonorCoatColor;
	}

	public void setRequiredDonorCoatColor(DictionaryEntry requiredDonorCoatColor) {
		this.requiredDonorCoatColor = requiredDonorCoatColor;
	}


	public DictionaryEntry getRequiredDonorStrain() {
		return requiredDonorStrain;
	}


	public void setRequiredDonorStrain(DictionaryEntry requiredDonorStrain) {
		this.requiredDonorStrain = requiredDonorStrain;
	}


	public void setMiRequestId(Long miRequestId) {
		this.miRequestId = miRequestId;
	}
	
	public Person getCellLineProvider() {
		return cellLineProvider;
	}


	public void setCellLineProvider(Person cellLineProvider) {
		this.cellLineProvider = cellLineProvider;
	}

   	/**
   	 * Returns tasks sorted
   	 */
	public List<Task> getTasks() {
		
		if( scheduledMiTasks != null ) {
			
			Collections.sort(scheduledMiTasks);
		
		} 
		
		return scheduledMiTasks;
	}


    public Date getAnchorDate() {
        return anchorDate;
    }
    
    public void setAnchorDate(Date targetScheduleDate) {
        this.anchorDate = targetScheduleDate;
    }  
     
    public Long getEndpointId() {
    	return miRequest.getEndpointDictId();
    }
    
    public AnimalRequest getDonorAnimalRequest() {
		return donorAnimalRequest;
	}

	public void setDonorAnimalRequest(AnimalRequest donorAnimalRequest) {
		this.donorAnimalRequest = donorAnimalRequest;
	}

	public MiRequest getMiRequest() {
		return miRequest;
	}

	public void setMiRequest(MiRequest microInjectionRequest) {
		this.miRequest = microInjectionRequest;
	}

	public AnimalRequest getRecipientAnimalRequest() {
		return recipientAnimalRequest;
	}

	public void setRecipientAnimalRequest(AnimalRequest recipientAnimalRequest) {
		this.recipientAnimalRequest = recipientAnimalRequest;
	}

	public DictionaryEntry getScheduledMiStatusDict() {
		return scheduledMiStatusDict;
	}

	public void setScheduledMiStatusDict(DictionaryEntry scheduledMiStatusDict) {
		this.scheduledMiStatusDict = scheduledMiStatusDict;
	}

	public Long getScheduledMiStatusDictId() {
        return scheduledMiStatusDictId;
    }
    
    public void setScheduledMiStatusDictId(Long scheduledMiStatusDictId) {
        this.scheduledMiStatusDictId = scheduledMiStatusDictId;
    }                                                                                                                                                                                                                                                                                      
                                                                                                                                         
    public Long getMiRequestId() {
        return miRequestId;
    }
    
    public void setMicroInjectionRequestId(Long microInjectionRequestId) {
        this.miRequestId = microInjectionRequestId;
    }             
    
    public Long getDonorAnimalRequestId() {
        return donorAnimalRequestId;
    }
    
    public void setDonorAnimalRequestId(Long donorAnimalRequestId) {
        this.donorAnimalRequestId = donorAnimalRequestId;
    }                                                                                                                                            
    public Long getRecipientAnimalRequestId() {
        return recipientAnimalRequestId;
    }
    
    public void setRecipientAnimalRequestId(Long recipientAnimalRequestId) {
        this.recipientAnimalRequestId = recipientAnimalRequestId;
    }                                                                                                                                            
    public String getComments() {
        return comments;
    }
    
    public void setComments(String comments) {
        this.comments = comments;
    }


	public Collection<EmbryoTransferProcedure> getEmbryoTransfers() {
		return embryoTransfers;
	}

	public void setEmbryoTransfers(
			Collection<EmbryoTransferProcedure> embryoTransfers) {
		this.embryoTransfers = embryoTransfers;
	}

	public MicroInjection getMicroInjection() {
		return microInjection;
	}

	public void setMicroInjection(MicroInjection microInjection) {
		this.microInjection = microInjection;
	}

	public AnimalRequest getSupplementalRecipientAnimalRequest() {
		return supplementalRecipientAnimalRequest;
	}

	public void setSupplementalRecipientAnimalRequest(
			AnimalRequest supplementalRecipientAnimalRequest) {
		this.supplementalRecipientAnimalRequest = supplementalRecipientAnimalRequest;
	}

	public Long getSupplementalRecipientAnimalRequestId() {
		return supplementalRecipientAnimalRequestId;
	}

	public void setSupplementalRecipientAnimalRequestId(
			Long supplementalRecipientAnimalRequestId) {
		this.supplementalRecipientAnimalRequestId = supplementalRecipientAnimalRequestId;
	}

	public String getCancelOrFailReasons() {
		return cancelOrFailReasons;
	}

	public void setCancelOrFailReasons(String cancelOrFailReasons) {
		this.cancelOrFailReasons = cancelOrFailReasons;
	}

	public Long getCellLineProviderId() {
		return cellLineProviderId;
	}

	public void setCellLineProviderId(Long cellLineProviderId) {
		this.cellLineProviderId = cellLineProviderId;
	}

	public void setAvailableEmbryos(Integer availableEmbryos) {
		this.availableEmbryos = availableEmbryos;
	}

	public Integer getAvailableEmbryos() {
		return availableEmbryos;
	}


	public boolean isDonorRequestEmailSent() {
		return this.donorRequestEmailSent;
	}


	public void setDonorRequestEmailSent(boolean donorRequestEmailSent) {
		this.donorRequestEmailSent = donorRequestEmailSent;
	}


	public boolean isRecipientRequestEmailSent() {
		return this.recipientRequestEmailSent;
	}


	public void setRecipientRequestEmailSent(boolean recipientRequestEmailSent) {
		this.recipientRequestEmailSent = recipientRequestEmailSent;
	}

	
}