/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.controllers;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.miprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.miprocessing.forms.EmbryoHarvestForm;
import uk.ac.sanger.mig.miprocessing.services.EmbryoHarvestService;
import uk.ac.sanger.mig.personnel.Person;

public class EmbryoHarvestFormController extends AbstractCancellableFormController {

	private EmbryoHarvestService embryoHarvestService;
	private DictionaryService dictionaryService;
	private String successViewMiForm;

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {

		EmbryoHarvestForm embryoHarvestForm = (EmbryoHarvestForm)createCommand();

		EmbryoHarvest embryoHarvest = null;

		Long id = ServletRequestUtils.getLongParameter(request, "embryoHarvestId");

		if(id != null) {

			embryoHarvest = embryoHarvestService.fetchWithAllComponentsById( id );

			if( embryoHarvest.getTechnician() == null)
				embryoHarvest.setTechnician(new Person());//backward compat for old recs w/out technician

		} else {

			embryoHarvest = embryoHarvestService.buildDefaultEmbryoHarvest(
					RequestUtils.getUser(request) );
		}

		embryoHarvestForm.setEmbryoHarvest(embryoHarvest);

		return embryoHarvestForm;

	}

	/* (non-Javadoc)
	 * @see org.springframework.web.servlet.mvc.AbstractFormController#onBindOnNewForm(javax.servlet.http.HttpServletRequest, java.lang.Object)
	 */
	@Override
	protected void onBindOnNewForm(HttpServletRequest request, Object command) throws Exception {

		super.onBindOnNewForm(request, command);

		EmbryoHarvestForm embryoHarvestForm = (EmbryoHarvestForm)command;

		EmbryoHarvest embryoHarvest = embryoHarvestForm.getEmbryoHarvest();

		if( embryoHarvest.getId() == null) {

		    //set default flush/plug dates if it is a new embryo harvest
			//based on MI form from which this controller was called.
			//If embryo harvest is new, these dates are derived from
			//the first MI that uses it
			embryoHarvest.setFlushDate( (embryoHarvestForm.getDefaultMiDateCompleted() == null ) ?
					new Date() : embryoHarvestForm.getDefaultMiDateCompleted());

			embryoHarvest.setPlugCheckDate( embryoHarvestService.getDefaultPlugDate(
					embryoHarvest.getFlushDate() ) );

		}
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response,
			Object command, BindException errors) throws Exception {

		EmbryoHarvestForm embryoHarvestForm = (EmbryoHarvestForm) command;
		EmbryoHarvest embryoHarvest = embryoHarvestForm.getEmbryoHarvest();

		embryoHarvestService.store( embryoHarvest, RequestUtils.getUser(request) );

		//redir to MI form referring page or show list view by default
		if( "miForm".equals( embryoHarvestForm.getRedirect()) && embryoHarvestForm.getSmiId() != null  )

			return new ModelAndView( successViewMiForm +
					"?microInjection.scheduledMicroInjectionId=" + embryoHarvestForm.getSmiId() +
					"&redirectId=" + embryoHarvestForm.getRedirectId() +
					"&embryoHarvestId=" + embryoHarvest.getId());

		else if ( "embryoHarvestDetailList".equals( embryoHarvestForm.getRedirect() )){

			return new ModelAndView( getSuccessView() +
					"?retrieveFromSession=true&newQuery=false&bypassForm=true&currentRecordId=" +
					embryoHarvest.getId() ); // show list view cached

		} else {//default

			return new ModelAndView( getSuccessView() +
					"?bypassForm=true&paginated=true&orderBy=flush_date&orderDesc=true" );
		}
	}

	@Override
	protected Map referenceData(HttpServletRequest arg0) throws Exception {

		Map<String,Object> metaData = new HashMap<String,Object>();

		Collection<DictionaryEntry> strains
			= dictionaryService.fetch(DictionaryEnum.MOUSE_STRAIN, true);

		int[] donorCount = new int[ EmbryoHarvest.getDonorCountCeiling() ];

		for(int x = 0; x < donorCount.length; x++) {
			donorCount[x] = x + 1;
		}

		metaData.put("strains", strains);
		metaData.put("donorCount", donorCount);
		return metaData;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// dependencies /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	public void setEmbryoHarvestService(EmbryoHarvestService embryoHarvestService) {
		this.embryoHarvestService = embryoHarvestService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setSuccessViewMiForm(String successViewMiForm) {
		this.successViewMiForm = successViewMiForm;
	}




}
