/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.miprocessing.EmbryoTransferProcedure;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob.MilestoneEnum;
import uk.ac.sanger.mig.miprocessing.forms.EmbryoTransferProcedureForm;
import uk.ac.sanger.mig.miprocessing.services.EmbryoTransferProcedureService;
import uk.ac.sanger.mig.miprocessing.services.MirMouseTrackingBridgeService;
import uk.ac.sanger.mig.miprocessing.services.ScheduledMicroInjectionService;
import uk.ac.sanger.mig.personnel.Person;

public class EmbryoTransferProcedureFormController extends AbstractCancellableFormController {
		
	private EmbryoTransferProcedureService embryoTransferProcedureService;
	private ScheduledMicroInjectionService scheduledMicroInjectionService;	
	private DictionaryService dictionaryService;
	private MirMouseTrackingBridgeService mirMouseTrackingBridgeService;
	private PersonalLicenceService personalLicenceService;
	
	private String exitView; //Used on explicit exit. we use successView to reload page on update/delete/insert
	
	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		
		EmbryoTransferProcedureForm embryoTransferProcedureForm = (EmbryoTransferProcedureForm)createCommand();				    
		
		Long id = ServletRequestUtils.getLongParameter(request, "embryoTransferProcedure.id");		
		Long smiId = ServletRequestUtils.getRequiredLongParameter(request, 
				"embryoTransferProcedure.scheduledMicroInjectionId");
			
		//1.) fetch parent SMI for existing or new ET
		ScheduledMicroInjectionJob smi = scheduledMicroInjectionService.fetchWithAllComponentsById( smiId );		
		EmbryoTransferProcedure embryoTransferProcedure = null; 
		Integer avaiableEmbryos = 0;
		
		//2.) edit/create *individul* EmbryoTrasnfer objects one at a time
		if( id != null ) {
			//edit...
			embryoTransferProcedure = embryoTransferProcedureService.fetchWithAllComponentsById(id);		
			avaiableEmbryos = smi.getUnusedMicroInjectedEmbryos() 
			    			+ embryoTransferProcedure.getTotalEmbryosTransferred();
			embryoTransferProcedureForm.setEarmarkId(embryoTransferProcedure.getMouse().getEarmarkDictId());						

		} else {
			embryoTransferProcedure 
				= embryoTransferProcedureService.buildDefaultEmbryoTransfer(smiId, 
						RequestUtils.getUser(request) );
			avaiableEmbryos = smi.getUnusedMicroInjectedEmbryos();
		}
		
		smi.setAvailableEmbryos( avaiableEmbryos );
		embryoTransferProcedure.setScheduledMicroInjection( smi );
		Date defaultDate = smi.getMicroInjectionDateCompleted();	
		
		populateForm(embryoTransferProcedureForm, defaultDate, embryoTransferProcedure, request);
		return embryoTransferProcedureForm;
	}
		
	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest, Object command, Errors errors) throws Exception {
		
		EmbryoTransferProcedureForm form = (EmbryoTransferProcedureForm)command;
		ScheduledMicroInjectionJob smi = form.getScheduledMicroInjection();
		
		boolean completed = smi.getMilestoneByEnum(MilestoneEnum.EMBRYO_TRANSFER_TASK_COMPLETED).isComplete();
		
		Date dateCompleted = ( completed ) ? 
				smi.getMilestoneByEnum(MilestoneEnum.EMBRYO_TRANSFER_TASK_COMPLETED).getActualCompletionDate() : null;
		
		Collection<DictionaryEntry> stufferStrains = dictionaryService.fetch(DictionaryEnum.MOUSE_STRAIN, true);
		Collection<DictionaryEntry> anaesthetics = dictionaryService.fetch(DictionaryEnum.MIR_ANAESTHETIC_DICT, true);
		Collection<DictionaryEntry> analgesics = dictionaryService.fetch(DictionaryEnum.MIR_ANALGESIA_DICT, true);
		Collection<DictionaryEntry> transferTypes = dictionaryService.fetch(DictionaryEnum.MIR_TRANSFER_TYPE_DICT, true);
		
		Map<String,Object> metaData = new HashMap<String,Object>();	
		metaData.put("stufferStrains", stufferStrains);
		metaData.put("anaesthetics", anaesthetics);
		metaData.put("analgesics", analgesics);
		metaData.put("transferTypes", transferTypes);	
		metaData.put("daysFromPlugChoices", new String[] {"2.5", "0.5"});
		metaData.put("embryoRange", new int[] {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15});
		metaData.put("smi",smi);
		metaData.put("completed",completed);
		metaData.put("dateCompleted",dateCompleted);
		metaData.put("displayEmbryosRemaining", true);
		return metaData;
	} 
		
	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, 
				Object command, BindException errors) throws Exception {
		
		EmbryoTransferProcedureForm form = (EmbryoTransferProcedureForm) command;
		boolean exit = ServletRequestUtils.getBooleanParameter(request, "exit", false);	
		boolean complete = ServletRequestUtils.getBooleanParameter(request, "completeOnExit", false);
		Integer embryoTransferCount = ServletRequestUtils.getIntParameter(request, "embryoTransferCount");	
		
		//1.) we are exiting, 
		if(  exit ) {
			
			if( complete && embryoTransferCount != null && embryoTransferCount > 0 ) {
			//exit with completion...
					scheduledMicroInjectionService.completeEmbryoTransfer( 
							form.getScheduledMicroInjectionId(),
							RequestUtils.getUser(request));
			 }
			 
			//just exit, no completion...
			 return new ModelAndView(getExitView() + "?retrieveFromSession=true&newQuery=false&bypassForm=true&currentRecordId=" + 
					 form.getScheduledMicroInjectionId());	
		
		 } else { 
			 //2.) if not exiting, then we must be updating/inserting/deleting ET record

				dePopulateForm(form, RequestUtils.getUser(request));
				
				if("rack".equalsIgnoreCase(form.getEmbryoTransferBarcodeType())) {
					Long cageId =  mirMouseTrackingBridgeService.creatNewCageInRack(
							form.getEmbryoTransferCageOrRackId(),
							RequestUtils.getUser(request));
					
					form.setEmbryoTransferCageOrRackId( cageId );
				}
						
				//Persist domain object EmbroTransferProcedure (w/MouseProcedure and new Mouse recovery cage or rack)
				embryoTransferProcedureService.editEmbryoTransfer(
						form.getEmbryoTransferProcedure(), 
						form,
						RequestUtils.getUser(request));		
						
			   //return to ET form with updated list
			   return new ModelAndView(getSuccessView()+"?embryoTransferProcedure.scheduledMicroInjectionId=" +
					form.getScheduledMicroInjectionId());	
		}	
	}
	
	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
			
		boolean formChange = ServletRequestUtils.getBooleanParameter(request, "changeEarmarks", false);	
			if( formChange ) 
				return true;
			else
				return super.isFormChangeRequest( request );		
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, 
			Object command, BindException bindEx) throws Exception {
		
		EmbryoTransferProcedureForm embryoTransferProcedureForm = (EmbryoTransferProcedureForm)	command;
		
		Collection<DictionaryEntry> existingEarmarks = new ArrayList<DictionaryEntry>();
		Collection<DictionaryEntry> avaiableEarmarks = dictionaryService.fetch(DictionaryEnum.MOUSE_EARMARK_DICT, true);
		
		if("cage".equalsIgnoreCase(embryoTransferProcedureForm.getEmbryoTransferBarcodeType())) {
			
			existingEarmarks = mirMouseTrackingBridgeService.getEarmarksByCageId(
					embryoTransferProcedureForm.getEmbryoTransferCageOrRackId());
		
			if(existingEarmarks != null) {
				avaiableEarmarks.removeAll( existingEarmarks );
			}
		}
		
		embryoTransferProcedureForm.setEarmarks( avaiableEarmarks );
	
	}
	
	@Override
	protected void onBind(HttpServletRequest request, Object command)
			throws Exception {
		
		//lookup and bind PIL Holder to an actual PIL if there is one
		EmbryoTransferProcedureForm embryoTransferProcedureForm = (EmbryoTransferProcedureForm)	command;
		
		EmbryoTransferProcedure embryoTransferProcedure = embryoTransferProcedureForm.getEmbryoTransferProcedure();
		Long personId = embryoTransferProcedure.getMouseProcedure().getPersonalLicence().getPersonId();
		PersonalLicence pil = personalLicenceService.fetchByPersonId( personId, 
				ComponentMap.of(PersonalLicence.Component.PERSON) );	
		if( pil != null) {
			embryoTransferProcedure.getMouseProcedure().setPersonalLicenceId( pil.getId() );	
			embryoTransferProcedure.getMouseProcedure().setPersonalLicence( pil );	
		}
		
		super.onBind(request, command);
	}
	
	@Override
	protected boolean isFormSubmission(HttpServletRequest request) {
	
		boolean exit = ServletRequestUtils.getBooleanParameter(request, "exit", false);
		
		if(exit)
			return true;	
		
		return super.isFormSubmission(request);
	}
	
	@Override
	protected boolean suppressValidation(HttpServletRequest request, Object command) {

		boolean exit = ServletRequestUtils.getBooleanParameter(request, "exit", false);
		if(exit)
			return true;	
		return super.suppressValidation(request, command);
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// dependencies /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	
	public void setExitView(String exitView) {
		this.exitView = exitView;
	}
	
	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setScheduledMicroInjectionService(
			ScheduledMicroInjectionService scheduledMicroInjectionService) {
		this.scheduledMicroInjectionService = scheduledMicroInjectionService;
	}

	public void setEmbryoTransferProcedureService(EmbryoTransferProcedureService embryoTransferProcedureService) {
		this.embryoTransferProcedureService = embryoTransferProcedureService;
	}
	
	public void setMirMouseTrackingBridgeService(
			MirMouseTrackingBridgeService mirMouseTrackingBridgeService) {
		this.mirMouseTrackingBridgeService = mirMouseTrackingBridgeService;
	}
	
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////  privates ////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	private String getExitView() {
		return exitView;
	}
	
	private void dePopulateForm(EmbryoTransferProcedureForm form, Person person) {
		
		EmbryoTransferProcedure embryoTransferProcedure = form.getEmbryoTransferProcedure();
		embryoTransferProcedure.getMouseProcedure().setStartDate( form.getDateCompleted() );
		embryoTransferProcedure.getMouseProcedure().setEndDate( form.getDateCompleted() );	
		embryoTransferProcedure.getMouseProcedure().setMouseId( embryoTransferProcedure.getMouseId() );	
		embryoTransferProcedure.getMouseProcedure().setForeignTableName( TableMetadataEnum.MIR_EMBRYO_TRANSFER_PROCEDURE );	
		embryoTransferProcedure.setCreator(person);
		embryoTransferProcedure.setCreatorId(person.getId());
	}
	
	private void populateForm(EmbryoTransferProcedureForm form, Date defaultDate, 
			EmbryoTransferProcedure embryoTransferProcedure, HttpServletRequest request) {
		
		//at ET to form
		form.setEmbryoTransferProcedure(embryoTransferProcedure);	
		Collection<DictionaryEntry> avaiableEarmarks 
			= dictionaryService.fetch(DictionaryEnum.MOUSE_EARMARK_DICT, true);
	
		//prepopulate PPL info, derived from MiRequest requester's PPL
		if(embryoTransferProcedure.getId() == null) {
			
			form.setDateCompleted( defaultDate );
		
		} else {			
			form.setDateCompleted(
					embryoTransferProcedure.getMouseProcedure().getEndDate());
			
			form.setEarmarkId(embryoTransferProcedure.getMouse().getEarmarkDictId());
			form.setEmbryoTransferBarcode(embryoTransferProcedure.getMouse().getCage().getBarcode());
			form.setEmbryoTransferBarcodeDescription(embryoTransferProcedure.getMouse().getCage().getBarcodeDescription());
			form.setEmbryoTransferCageOrRackId(embryoTransferProcedure.getMouse().getCage().getId());
			form.setEmbryoTransferProcedure(embryoTransferProcedure);
			
			Collection<DictionaryEntry> existingEarmarks 
				= mirMouseTrackingBridgeService.getEarmarksByCageId(embryoTransferProcedure.getMouse().getCageId());
			existingEarmarks.remove(embryoTransferProcedure.getMouse().getEarmark());
			avaiableEarmarks.removeAll( existingEarmarks );
				
		}
		
		form.setEarmarks(avaiableEarmarks);
	}

	public void setPersonalLicenceService(
			PersonalLicenceService personalLicenceService) {
		this.personalLicenceService = personalLicenceService;
	}


}