/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.controllers;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.miprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.miprocessing.MicroInjection;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.forms.MicroInjectionForm;
import uk.ac.sanger.mig.miprocessing.services.EmbryoHarvestService;
import uk.ac.sanger.mig.miprocessing.services.MicroInjectionService;
import uk.ac.sanger.mig.miprocessing.services.ScheduledMicroInjectionService;

public class MicroInjectionFormController extends AbstractCancellableFormController {
	
	private MicroInjectionService microInjectionService;
	private ScheduledMicroInjectionService scheduledMicroInjectionService;
	private DictionaryService dictionaryService;
	private EmbryoHarvestService embryoHarvestService;
	
	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		
		MicroInjectionForm microInjectionForm = (MicroInjectionForm)createCommand();	

		Long redirectId = ServletRequestUtils.getLongParameter(httpServletRequest, "redirectId");//if returing from embryoHarvest
		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");
		id = (id == null && redirectId != null && redirectId > 0) ? redirectId : id;//check if coming back from embryo harvest form
		
		Long smiId = ServletRequestUtils.getRequiredLongParameter(httpServletRequest, "microInjection.scheduledMicroInjectionId");
		
		ScheduledMicroInjectionJob scheduledMicroInjectionJob 
		  		= scheduledMicroInjectionService.fetchWithAllComponentsById( smiId );
		
		MicroInjection microInjection = null; 
		
		if(id != null) {
			
			microInjection = microInjectionService.fetchById(id,
					ComponentMap.of( MicroInjection.Component.EMBRYO_HARVEST, 
									 MicroInjection.Component.TECHNICIAN      ));		
			microInjectionForm.setOldEmbryoHarvestCount(microInjection.getInjectedEmbryoCount());		
			
		} else {
			
			Long embryoHarvestId = ServletRequestUtils.getLongParameter(httpServletRequest, "embryoHarvestId");		
			Date defaultDate = ( embryoHarvestId != null ) ? 
				 embryoHarvestService.fetchById(embryoHarvestId, null).getFlushDate() : //returning from embryo harvest
					 scheduledMicroInjectionJob.getAnchorDate();//coming from SMI list				
			
				 microInjection = microInjectionService.buildDefaultMicroInjection( smiId, defaultDate, 
					RequestUtils.getUser(httpServletRequest) );
		}
		
		microInjectionForm.setMicroInjection( microInjection );		
		return microInjectionForm;
	}
	
	@Override
	protected Map referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		
		MicroInjectionForm microInjectionForm = (MicroInjectionForm)command;

		//fetch embryo harvest list available on given date 
		Date dateCompleted = microInjectionForm.getDateCompleted();	
		
		Long smiId = microInjectionForm.getScheduledMicroInjectionId();
		ScheduledMicroInjectionJob smi = 
			scheduledMicroInjectionService.fetchWithAllComponentsById( smiId );
		
		Collection<EmbryoHarvest> availableEmbryoHarvestList = null;
		if(dateCompleted != null) {
			availableEmbryoHarvestList = embryoHarvestService.fetchAvailableByDate(dateCompleted, true, 
					ComponentMap.of(EmbryoHarvest.Component.TECHNICIAN));
		} else if( isFormChangeRequest(request) ) {
			errors.reject(null, "Date Completed is a required field.");
		}
		
		Collection<DictionaryEntry> embryoStages 
			= dictionaryService.fetch( DictionaryEnum.MIR_EMBRYO_STAGE_DICT );	
		
		int[] esCellsPerEmbryo = {5,10,15,20,25,30,35,40};
		
		Map<String,Object> formMetaData = new HashMap<String,Object>();
		formMetaData.put("embryoStages",embryoStages);
		formMetaData.put("smi",smi);
		formMetaData.put("range",esCellsPerEmbryo);
		formMetaData.put("availableEmbryoHarvestList",availableEmbryoHarvestList);
		formMetaData.put("redirect", "miForm");
		formMetaData.put("smiId", smiId);
		formMetaData.put("redirectId",  
				(microInjectionForm.getMicroInjection().getId() == null) ? 0 :
					microInjectionForm.getMicroInjection().getId());		
		formMetaData.put("showEmbryoHarvestRadio", true);
		return formMetaData;
	}
	
	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		
		boolean formChange = ServletRequestUtils.getBooleanParameter(request, "updateEmbryoHarvestList", false);	
		if( formChange ) 
			return true;
		else
			return false;		
	}
	
	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, 
			Object command, BindException errors) throws Exception {
		
		MicroInjectionForm microInjectionForm = (MicroInjectionForm) command;
		
		scheduledMicroInjectionService.completeMicroInjection(
				microInjectionForm.getMicroInjection(),  
				RequestUtils.getUser(request));
		
		return new ModelAndView( getSuccessView() + "?retrieveFromSession=true&newQuery=false&bypassForm=true&currentRecordId=" + 
				microInjectionForm.getScheduledMicroInjectionId() ); // show list view

	}
	
	public void setEmbryoHarvestService(EmbryoHarvestService embryoHarvestService) {
		this.embryoHarvestService = embryoHarvestService;
	}
	
	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}
	
	public void setScheduledMicroInjectionService(
			ScheduledMicroInjectionService scheduledMicroInjectionService) {
		this.scheduledMicroInjectionService = scheduledMicroInjectionService;
	}
	
	public void setMicroInjectionService(MicroInjectionService microInjectionService) {
		this.microInjectionService = microInjectionService;
	}
	
	
	
}