/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.dao;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.DictionaryDao;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionSearchBean;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob.InternalStatus;
import uk.ac.sanger.mig.miprocessing.dao.rowmappers.ScheduledMicroInjectionListViewRowMapper;
import uk.ac.sanger.mig.miprocessing.views.ScheduledMicroInjectionListView;
import uk.ac.sanger.mig.mirequest.MiRequest.EndpointEnum;

public class ScheduledMicroInjectionDaoSpringImpl extends AbstractDaoSpringImpl<ScheduledMicroInjectionJob> implements ScheduledMicroInjectionDao {

	private PaginatedSearchUtils<ScheduledMicroInjectionListView> paginator;
	private ScheduledMicroInjectionListViewRowMapper scheduledMicroInjectionListViewRowMapper;
	private DictionaryDao dictionaryDao;
	
	public DictionaryEntry fetchRequiredDonorStrain(Long cellLineStrainDictId) {

		if( cellLineStrainDictId == null )
			throw new IllegalArgumentException("cellLineStrainDictId cannot be null.");
		
		String sql = " select m.* from " + DictionaryEnum.MOUSE_STRAIN.getTableName() +
					 " m, " + DictionaryEnum.MIR_CELL_LINE_STRAIN.getTableName() + 
					 " c where c.req_donor_strain_id = m.id and c.id = ? ";
		
		Collection<Object> bindVariables = new ArrayList<Object>();
		bindVariables.add( cellLineStrainDictId );

		return  dictionaryDao.fetchOneByCustomQuery(sql, bindVariables);
	}

	public Collection<ScheduledMicroInjectionJob> fetchAllActive() {
		String sql = " select * from " + getTableName() + " where NOT (internal_status_dict_id in(9,10,13)) order by target_schedule_date ";
		Collection<ScheduledMicroInjectionJob> smiList = getSimpleJdbcTemplate().query(sql, getRowMapper());
		return (smiList == null) ? new ArrayList<ScheduledMicroInjectionJob>() : smiList;

	}

	public Collection<ScheduledMicroInjectionJob> fetchAllByMiRequestId(Long miRequestId) {
		String sql = " select * from " + getTableName() + " where micro_injection_request_id = ?  order by target_schedule_date ";
		Collection<ScheduledMicroInjectionJob> smiList = getSimpleJdbcTemplate().query(sql, getRowMapper(), miRequestId);
		return (smiList == null) ? new ArrayList<ScheduledMicroInjectionJob>() : smiList;		
	}

	public Collection<ScheduledMicroInjectionListView> searchAllNonPaginated(ScheduledMicroInjectionSearchBean smiSearchBean) {

		String sql = " select * from " + paginator.getViewName();
		Collection<ScheduledMicroInjectionListView> smiViewList = getSimpleJdbcTemplate().query(sql, 
				scheduledMicroInjectionListViewRowMapper);
		
		return (smiViewList == null) ? new ArrayList<ScheduledMicroInjectionListView>() : smiViewList;	
		
	}
	
	
	public Collection<ScheduledMicroInjectionListView> search(ScheduledMicroInjectionSearchBean smiSearchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		String selectSql = " select smi.* ";
		String fromSql = " from " + paginator.getViewName() + " smi "; 
		String whereSql = " where 1=1 ";		

		//id
		if(smiSearchBean.getId() != null) {
			whereSql += " and smi.id = ? ";
			bindVariables.add(smiSearchBean.getId());			
		}	

		//internal status id
		if(smiSearchBean.getCustomFilterId() != null) {

			if(smiSearchBean.getCustomFilterId().equals( ScheduledMicroInjectionJob.FILTER_STATUS_ACTIVE )) {
				whereSql += " and smi.internal_status_id not in (?,?,?) ";
				bindVariables.add( InternalStatus.CANCELLED.getId() );
				bindVariables.add( InternalStatus.FAILED.getId() );
				bindVariables.add( InternalStatus.COMPLETED.getId() );
			} else if(smiSearchBean.getCustomFilterId().equals( ScheduledMicroInjectionJob.FILTER_STATUS_CANCEL_FAIL )) {
				whereSql += " and smi.internal_status_id in (?,?)";
				bindVariables.add( InternalStatus.CANCELLED.getId() );
				bindVariables.add( InternalStatus.FAILED.getId() );
			} else if(smiSearchBean.getCustomFilterId().equals( ScheduledMicroInjectionJob.FILTER_STATUS_ENSDPOINT_REACHED )) {
				whereSql += " and ((f0_actual_completion_date is not null and endpoint_id = 2) " +
				"OR (f1_actual_completion_date is not null and endpoint_id = 1)) ";
			} else {
				throw new IllegalArgumentException("Invalid SmiSearchBean argument. " +
						"If field filterStatusId != null, must pbe either FILTER_STATUS_ACTIVE, " +
						"FILTER_STATUS_CANCEL_FAIL, or FILTER_STATUS_ENSDPOINT_REACHED");
			}
				
		}

		//endpoint id
		if(smiSearchBean.getEndpointId() != null) {
			whereSql += " and smi.endpoint_id = ? ";
			bindVariables.add(smiSearchBean.getEndpointId());			
		}

		//target schedule date (from)
		if(smiSearchBean.getTargetScheduleDateFrom()!= null) {
			whereSql += " and smi.target_schedule_date >= ? ";
			bindVariables.add(defaultDateFormat.format(smiSearchBean.getTargetScheduleDateFrom()));			
		}

		//target schedule date (to)
		if(smiSearchBean.getTargetScheduleDateTo()!= null) {
			whereSql += " and smi.target_schedule_date <= ? ";
			bindVariables.add(smiSearchBean.getTargetScheduleDateTo());			
		}
		//next task to complete
		if(smiSearchBean.getNextTaskToCompleteId()!= null) {
			if(smiSearchBean.getNextTaskToCompleteId().equals(
					ScheduledMicroInjectionJob.FILTER_NEXT_TASK_TO_COMPLETE_MI)) {
				whereSql += " and smi.mi_actual_completion_date is null";
				
			} else if(smiSearchBean.getNextTaskToCompleteId().equals(
					ScheduledMicroInjectionJob.FILTER_NEXT_TASK_TO_COMPLETE_ET)) {
				whereSql += " and smi.mi_actual_completion_date is not null and smi.et_actual_completion_date is null";
				
			} else if(smiSearchBean.getNextTaskToCompleteId().equals(
					ScheduledMicroInjectionJob.FILTER_NEXT_TASK_TO_COMPLETE_F0_WEANED)) {
				whereSql += " and smi.mi_actual_completion_date is not null and " +
				"smi.et_actual_completion_date is not null and " +
				"smi.f0_actual_completion_date is null and " +
				"smi.endpoint_id != " + EndpointEnum.EMBRYONIC_CHIMERAS.getId();
				
				
			} else if(smiSearchBean.getNextTaskToCompleteId().equals(
					ScheduledMicroInjectionJob.FILTER_NEXT_TASK_TO_COMPLETE_F1_GENOTYPED))
				
				whereSql += " and smi.mi_actual_completion_date is not null and " +
				"smi.et_actual_completion_date is not null and " +
				"smi.f0_actual_completion_date is not null and " +
				"smi.f1_actual_completion_date is null and " +
				"smi.endpoint_id = " + EndpointEnum.F1.getId();
				
			
		}

		if(smiSearchBean.getMiRequestId() != null)  {
			whereSql += " and smi.MICRO_INJECTION_REQUEST_ID = ? ";
			bindVariables.add(smiSearchBean.getMiRequestId());				
		}

		String orderBySql = " order by ";

		if(smiSearchBean.getOrderBy() == null || smiSearchBean.getOrderBy().equals("")) {
			orderBySql += "	smi.id ";
		}
		else {
			orderBySql += smiSearchBean.getOrderBy();
		}

		if(smiSearchBean.isOrderDesc()) {
			orderBySql += " desc ";
		}

		String sql = selectSql + fromSql + whereSql + orderBySql ;

		logger.debug(sql);

		return paginator.fetchSearchResults(sql, bindVariables, smiSearchBean, scheduledMicroInjectionListViewRowMapper);
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, ScheduledMicroInjectionJob scheduledMicroInjection) {
		sql.addLongParam(scheduledMicroInjection.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(scheduledMicroInjection.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(scheduledMicroInjection.getEditedBy(), "EDITED_BY");
		sql.addDateParam(scheduledMicroInjection.getEditDate(), "EDITED_DATE");
		sql.addLongParam(scheduledMicroInjection.getCheckNumber(), "CHECK_NUMBER");
		sql.addDateParam(scheduledMicroInjection.getAnchorDate(), "TARGET_SCHEDULE_DATE");
		sql.addLongParam(scheduledMicroInjection.getMiRequestId(), "MICRO_INJECTION_REQUEST_ID");
		sql.addLongParam(scheduledMicroInjection.getDonorAnimalRequestId(), "DONOR_ANIMAL_REQUEST_ID");
		sql.addLongParam(scheduledMicroInjection.getRecipientAnimalRequestId(), "RECIPIENT_ANIMAL_REQUEST_ID");
		sql.addLongParam(scheduledMicroInjection.getSupplementalRecipientAnimalRequestId(), "SUPPLEMENTAL_RECIPIENT_REQ_ID");
		sql.addStringParam(scheduledMicroInjection.getComments(), "COMMENTS");
		sql.addStringParam(scheduledMicroInjection.getCancelOrFailReasons(), "CANCEL_OR_FAIL_REASONS");
		sql.addLongParam(scheduledMicroInjection.getScheduledMiStatusDictId(), "INTERNAL_STATUS_DICT_ID");
		sql.addLongParam(scheduledMicroInjection.getCellLineProviderId(), "CELL_LINE_PROVIDER_ID");
		sql.addStringParam(scheduledMicroInjection.getJobId(), "JOB_ID");
		sql.addLongParam(scheduledMicroInjection.getId(), "ID");
	}

	public void setScheduledMicroInjectionListViewRowMapper(
			ScheduledMicroInjectionListViewRowMapper scheduledMicroInjectionListViewRowMapper) {
		this.scheduledMicroInjectionListViewRowMapper = scheduledMicroInjectionListViewRowMapper;
	}

	public PaginatedSearchUtils<ScheduledMicroInjectionListView> getPaginator() {
		return paginator;
	}

	public void setPaginator(PaginatedSearchUtils<ScheduledMicroInjectionListView> paginator) {
		this.paginator = paginator;
	}

	public void setDictionaryDao(DictionaryDao dictionaryDao) {
		this.dictionaryDao = dictionaryDao;
	}

}