/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.miprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.miprocessing.EmbryoHarvestSearchBean;
import uk.ac.sanger.mig.miprocessing.dao.EmbryoHarvestDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class EmbryoHarvestServiceImpl extends AbstractServiceImpl<EmbryoHarvest> implements EmbryoHarvestService {

	private EmbryoHarvestDao embryoHarvestDao;
	private PersonService personService;

	public Collection<EmbryoHarvest> fetch(EmbryoHarvestSearchBean searchBean, ComponentMap requiredComponents) {
		Collection<EmbryoHarvest> embryoHarvest = embryoHarvestDao.fetchAll(searchBean);
		if( !embryoHarvest.isEmpty() )
			attachComponents(embryoHarvest, requiredComponents);
		return embryoHarvest;
	}

	public Collection<EmbryoHarvest> fetchAvailableByDate(Date date, boolean incubatedOvernight,
				ComponentMap requiredComponents) {

		Collection<EmbryoHarvest> embryoHarvest = embryoHarvestDao.fetchAvailableByDate( date, incubatedOvernight );
		if( !embryoHarvest.isEmpty() )
			attachComponents(embryoHarvest, requiredComponents);

		return embryoHarvest;
	}


	public EmbryoHarvest fetchWithAllComponentsById(Long id) {
		ComponentMap embryoHarvestComponents  = new ComponentMap(EmbryoHarvest.Component.TECHNICIAN, null);
	    ComponentMap technicianComponents = new ComponentMap(Person.Component.ALL_COMPONENTS, null);
		embryoHarvestComponents.put(EmbryoHarvest.Component.TECHNICIAN, technicianComponents);
		return fetchById(id, embryoHarvestComponents );
	}

	@Transactional
	public Long store(EmbryoHarvest embryoHarvest, Person user) {
		//Embryo harvest should not be stored alone but transactionally stored w/Mouse Reservations
		if(embryoHarvest == null) {
			throw new RuntimeException("Cannot save a null embryoHarvest");
		}

		return embryoHarvestDao.store(embryoHarvest, user);
	}

	public EmbryoHarvest fetchById(Long embryoHarvestId, ComponentMap requiredComponents) {
		EmbryoHarvest embryoHarvest = embryoHarvestDao.fetchById(embryoHarvestId);

		if(embryoHarvest != null)
			attachComponents(embryoHarvest, requiredComponents);

		return embryoHarvest;
	}



	public EmbryoHarvest buildDefaultEmbryoHarvest( Person user ) {

		EmbryoHarvest embryoHarvest = new EmbryoHarvest();
		embryoHarvest.setUseBlastocysts(true);
		embryoHarvest.setUseMoralae(true);
		embryoHarvest.setTechnician( user );
		embryoHarvest.setTechnicianId( user.getId() );

		return  embryoHarvest;
	}

	public Date getDefaultPlugDate(Date flushDate) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime( flushDate );
		calendar.add(Calendar.DAY_OF_YEAR, DEFAULT_PLUG_DAY_OFFSET);
		return calendar.getTime();
	}

	@Override
	protected void attachComponents(EmbryoHarvest embryoHarvest, ComponentMap requiredComponents) {
		if(embryoHarvest == null) {
			throw new RuntimeException("Cannot attach components to null embryoHarvest");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(EmbryoHarvest.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(EmbryoHarvest.Component.TECHNICIAN)) {
				embryoHarvest.setTechnician(fetchTechnician(
						embryoHarvest.getTechnicianId(),
						requiredComponents.get(EmbryoHarvest.Component.TECHNICIAN)));
			}
		}
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// dependencies /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setEmbryoHarvestDao(EmbryoHarvestDao embryoHarvestDao) {
		this.embryoHarvestDao = embryoHarvestDao;
	}


	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// privates /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	private Person fetchTechnician(Long personId, ComponentMap requiredComponents) {
		return personService.fetchById( personId, requiredComponents );
	}



}
