/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;

import java.util.Date;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.miprocessing.EmbryoHarvest;
import uk.ac.sanger.mig.miprocessing.MicroInjection;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.dao.MicroInjectionDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class MicroInjectionServiceImpl extends AbstractServiceImpl<MicroInjection> implements MicroInjectionService {
	
	private MicroInjectionDao microInjectionDao;
	private EmbryoHarvestService embryoHarvestService;
	private PersonService personService; 
	private ScheduledMicroInjectionService scheduledMicroInjectionService;

	public MicroInjection buildDefaultMicroInjection(Long simId, Date defaultDate, Person user) {		
		MicroInjection microInjection = new MicroInjection();
		microInjection.setDateCompleted( defaultDate );
		microInjection.setEmbryoHarvest( new EmbryoHarvest() );
		microInjection.setScheduledMicroInjectionId( simId );
		microInjection.setCreator( user );
		microInjection.setTechnician(user);
		microInjection.setTechnicianId(user.getId());
		return microInjection;
	}
	
	@Transactional
	public Long store(MicroInjection microInjection, Person user) {
		if(microInjection == null) {
			throw new RuntimeException("Cannot save a null microInjection");
		}
		
		return microInjectionDao.store(microInjection, user);
	}
	
	public MicroInjection fetchBySmiJobId(Long smiId, ComponentMap componentMap) {
		MicroInjection microInjection = microInjectionDao.fetchBySmiJobId(smiId);
		if(microInjection != null)
			this.attachComponents(microInjection, componentMap);
		return microInjection;
	}
	
	
	
	public MicroInjection fetchById(Long microInjectionId, ComponentMap requiredComponents) {
		MicroInjection microInjection = microInjectionDao.fetchById(microInjectionId);
		
		if(microInjection != null)
			attachComponents(microInjection, requiredComponents);
		
		return microInjection;
	}
	  
	@Override
	protected void attachComponents(MicroInjection microInjection, ComponentMap requiredComponents) {
		if(microInjection == null) {
			throw new RuntimeException("Cannot attach components to null MicroInjection.");
		}		
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(MicroInjection.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			
			if(fetchAll || requiredComponents.containsKey(MicroInjection.Component.EMBRYO_HARVEST)) {
				microInjection.setEmbryoHarvest(fetchEmbryoHarvestById(
						microInjection.getEmbryoHarvestId(), 
						requiredComponents.get(MicroInjection.Component.EMBRYO_HARVEST)));
			}
		
			if(fetchAll || requiredComponents.containsKey(MicroInjection.Component.TECHNICIAN)) {
				microInjection.setTechnician(fetchTechnicianById(
						microInjection.getTechnicianId(), 
						requiredComponents.get(MicroInjection.Component.TECHNICIAN)));
			}

			if(fetchAll || requiredComponents.containsKey(MicroInjection.Component.SCHEDULED_MICRO_INJECTION)) {
				microInjection.setScheduledMicroInjection(fetchSmiJobById(
						microInjection.getScheduledMicroInjectionId(), 
						requiredComponents.get(MicroInjection.Component.SCHEDULED_MICRO_INJECTION)));
			}
			
		}
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// dependencies /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////

	public void setScheduledMicroInjectionService(
			ScheduledMicroInjectionService scheduledMicroInjectionService) {
		this.scheduledMicroInjectionService = scheduledMicroInjectionService;
	}


	public void setEmbryoHarvestService(EmbryoHarvestService embryoHarvestService) {
		this.embryoHarvestService = embryoHarvestService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}


	public void setMicroInjectionDao(MicroInjectionDao microInjectionDao) {
		this.microInjectionDao = microInjectionDao;
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////// privates /////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	private ScheduledMicroInjectionJob fetchSmiJobById(Long smiJobId, ComponentMap requiredComponents) {
		if( smiJobId != null )
			return scheduledMicroInjectionService.fetchById(smiJobId, requiredComponents);
		return null;
	}
	
	private Person fetchTechnicianById(Long personId, ComponentMap requiredComponents) {
		if( personId != null )
			return personService.fetchById(personId, requiredComponents);
		return null;
	}
	
	private EmbryoHarvest fetchEmbryoHarvestById(Long embryoHarvestId, ComponentMap requiredComponents) {
		if(embryoHarvestId != null)
			return embryoHarvestService.fetchById(embryoHarvestId, requiredComponents);
		return null;
	}
	
	
}
								