/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.animalrequest.AnimalRequest;
import uk.ac.sanger.mig.animalrequest.services.AnimalRequestService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.miprocessing.EmbryoTransferProcedure;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.MouseWeight;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.mousetracking.services.CageService;
import uk.ac.sanger.mig.mousetracking.services.LitterService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.mousetracking.services.MouseWeightService;
import uk.ac.sanger.mig.personnel.Person;

/**
 * @see MirMouseTrackingBridgeService
 */
public class MirMouseTrackingBridgeServiceImpl implements MirMouseTrackingBridgeService {

	private AnimalRequestService animalRequestService;
	private MouseService mouseService;
	private LitterService litterService;
	private BreedingService breedingService;
	private CageService cageService;
	private MouseProcedureService mouseProcedureService;
	private EmbryoTransferProcedureService embryoTransferProcedureService;
	private MouseWeightService mouseWeightService;
	private ComponentMapService componentMapService;

	public Mouse fetchMouseById(Long mouseId, ComponentMap components) {
		return mouseService.fetchById(mouseId, components);
	}

	public MouseProcedure fetchMouseProcedureByMouseId(Long mouseId) {

		return mouseProcedureService.fetchCurrentEmbryoTransferedByMouseId(mouseId, null);
	}
	public EmbryoTransferProcedure fetchEmbryoTransferById(Long embryoTransferId, ComponentMap requiredComponents) {
		return embryoTransferProcedureService.fetchById(embryoTransferId, requiredComponents);
	}

	public void setMouseWeightService(MouseWeightService mouseWeightService) {
		this.mouseWeightService = mouseWeightService;
	}

	@Transactional
	public void moveToExistingCage(Mouse mouse, Long cageId, Person user) {
		mouseService.moveToExistingCage(mouse, cageId, user);
	}

	@Transactional
	public Long moveToNewCage(Mouse mouse, Long rackId, Person user) {

		return mouseService.moveToNewCage(mouse, rackId, user);
	}

	@Transactional
	public Long createBreedingAndLitter(Long embryoTransferProcedureId, Person user) {

		ComponentMap requiredComponents = componentMapService.getComponentMap("embryoTransferProcedureForCreateBreedingAndLitter");

		EmbryoTransferProcedure embryoTransferProcedure =
			embryoTransferProcedureService.fetchById(embryoTransferProcedureId, requiredComponents);

		// create breeding
		Breeding breeding = new Breeding();
		breeding.setOffspringGeneration("F0");
		breeding.setMaleId(MouseService.WT_STERILE_MALE_ID);
		breeding.setFemaleId(embryoTransferProcedure.getMouseId());
		breeding.setPseudoBreeding(true);
		breeding.setEmbryoTransferId(embryoTransferProcedure.getId());
		if(embryoTransferProcedure.getScheduledMicroInjection() != null){
			breeding.setOffspringColonyId(embryoTransferProcedure.getScheduledMicroInjection().getMiRequest().getColonyInfoId());
		}else if(embryoTransferProcedure.getThawTest()!=null){
			breeding.setOffspringColonyId(embryoTransferProcedure.getThawTest().getCryoRequest().getColonyInfoId());
		}else{
			throw new RuntimeException("No request associate with the embryo transfer!");
		}

		Calendar startDate = Calendar.getInstance();
		startDate.setTime( embryoTransferProcedure.getPlugDate()  );
		startDate.add(Calendar.DAY_OF_YEAR, -1); //breeding start 1 day before plug

		breeding.setStartDate( startDate.getTime() );

		breeding.setEndDate(null);

		Long breedingId = breedingService.store(breeding, user);

		// create litter
		Litter litter = new Litter();
		litter.setBreedingId(breedingId);
		litter.setPlugDate( embryoTransferProcedure.getPlugDate() );
		litter.setDueDate( embryoTransferProcedure.getDueDate() );
		litterService.makeLitterName(litter);
		litterService.store(litter, user);

		return breedingId;
	}

	@Transactional
	public Long storeMouseWeight(MouseWeight mouseWeight, Person user) {
		return mouseWeightService.store(mouseWeight, user);
	}

	public Collection<DictionaryEntry> getEarmarksByCageId(Long cageId) {
		Collection<Mouse> mice = mouseService.fetchByCageId(cageId,null);
		Collection<DictionaryEntry> earmarks = new ArrayList<DictionaryEntry>();

		for(Mouse mouse:mice) {
			if(mouse.getEarmark() != null) {
				earmarks.add(mouse.getEarmark());
			}
		}

		return earmarks;
	}

	@Transactional
	public int deleteMouseProcedureByTableRef(TableMetadataEnum tableName, Long fkRef, Person user) {
		return mouseProcedureService.deleteByTableRef(tableName, fkRef, user );
	}

	@Transactional
	public Long storeMouse(Mouse mouse, Person person) {
		return mouseService.store(mouse, person);
	}

	@Transactional
	public Long creatNewCageInRack(Long rackId, Person user) {
		return cageService.createInRack(rackId, user);
	}

	public Cage fetchCage(Long cageId) {
		return cageService.fetchById(cageId, null);
	}

	@Transactional
	public Long storeMouseProcedure(MouseProcedure mouseProcedure, Person user) {
		if(mouseProcedure == null) {
			throw new RuntimeException("Cannot save a null mouseProcedure");
		}

		return mouseProcedureService.store(mouseProcedure, user);
	}

	@Transactional
	public Long storeAnimalRequest( AnimalRequest animalRequest, Person user ) {

		return animalRequestService.store( animalRequest, user );
	}


	////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// dependencies ///////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////
	public void setAnimalRequestService(AnimalRequestService animalRequestService) {
		this.animalRequestService = animalRequestService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}


	public void setLitterService(LitterService litterService) {
		this.litterService = litterService;
	}

	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	public void setEmbryoTransferProcedureService(
			EmbryoTransferProcedureService embryoTransferProcedureService) {
		this.embryoTransferProcedureService = embryoTransferProcedureService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}
}
