/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;

import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.DictionaryEntryEnum;
import uk.ac.sanger.mig.miprocessing.MicroInjection;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionSearchBean;
import uk.ac.sanger.mig.miprocessing.views.ScheduledMicroInjectionListView;
import uk.ac.sanger.mig.miprocessing.workflow.Job;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.personnel.Person;

public interface ScheduledMicroInjectionService {
	//need to refactor in to enum and possibly relocate for better encapsulation,
	//or refactor into table w./logic mapping cell line strain to paternal donor strain/coat
	public static final Long CELL_LINE_STRAIN_E14 = 2L;
	public static final Long CELL_LINE_STRAIN_E14TG2a = 10L;
	public static final Long CELL_LINE_STRAIN_129S5 = 6L;
	public static final Long CELL_LINE_STRAIN_AB1 = 7L;
	public static final Long CELL_LINE_STRAIN_AB1_241 = 8L;
	public static final Long CELL_LINE_STRAIN_AB2_2 = 9L;
	public static final Long CELL_LINE_STRAIN_AB2_2_AB1 = 3L;
	public static final Long CELL_LINE_STRAIN_SI6_002_C21 = 12L;
	public static final Long CELL_LINE_STRAIN_JM8_PARENTAL = 11L;
	public static final Long CELL_LINE_STRAIN_C57BL6J = 1L;
	public static final Long CELL_LINE_STRAIN_JM8_F6 = 13L;
	public static final Long CELL_LINE_STRAIN_JM8_N4 = 14L;
	public static final Long CELL_LINE_STRAIN_C2_P13_NAGGY = 15L;
	public static final Long CELL_LINE_STRAIN_JM8_AF6 = 16L;


	//	need to refactor in to enum and possibly relocate for better encapsulation
	public static final Long CHIMERA_MATE_STRAIN_C57BL6CC_CALB = 3L;
	public static final Long CHIMERA_MATE_STRAIN_CBA_C57BL6J_F1= 2L;
	public static final Long CHIMERA_MATE_STRAIN_129_S5= 1L;

	/*
	 * TODO refactor to combinf with same method in miReq service
	 */
	public DictionaryEntry getNewStatusDict(DictionaryEntryEnum dictionaryEntryEnum);

	/**
	 * Returns logically named unique identifier for each ScheduledMicroInjectionJob.
	 * Prefix refers to the parent MiRequest. The suffix refers to the Scheduled MI Job
	 * working on that request.
	 * 
	 * Throws IllegalArgumentException if  miRequest is null.
	 * 
	 * Throws UnattachedComponentException if
	 * MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS not attached.
	 * 
	 * @param miRequest
	 * @return
	 */
	public String deriveJobId(MiRequest miRequest);

	/**
	 * Fetches deep copy of a ScheduledMicroInjectionJob by id
	 * @param id
	 * @return
	 */
	public ScheduledMicroInjectionJob fetchWithAllComponentsById(Long id);

	/**
	 * Returns a Collection of smi List view records base on search criteria of search bean
	 * @param scheduledMicroInjectionSearchBean
	 * @return
	 */
	public Collection<ScheduledMicroInjectionListView> search(ScheduledMicroInjectionSearchBean
			scheduledMicroInjectionSearchBean);

	/**
	 * Stores ScheduledMicroInjection as record in database and returns the record's primary key
	 * @param scheduledMicroInjection
	 * @return
	 */
	public Long store(ScheduledMicroInjectionJob scheduledMicroInjection, Person person);

	/**
	 * Returns scheduledMicroInjection for a given id that maps to primary key
	 * of a record in a database table.
	 * @param id
	 * @return
	 */
	public ScheduledMicroInjectionJob fetchById(Long id, ComponentMap componentMap);


	/**
	 * Returns Collection of all SMIs for a given MiRequest
	 * sorted by target date.
	 *
	 * Returns empty Collection if none ever scheduled;
	 * @param miRequestId
	 * @param componentMap
	 * @return
	 */
	public Collection<ScheduledMicroInjectionJob> fetchAllByMiRequestId(Long miRequestId, ComponentMap componentMap);


	/**
	 * Creates new ScheduledMicroInjectionJob
	 * 1.) Creates new SMI record w/Target anchor date and Cell Line provider
	 * 2.) Populates SMI Task list with anchor date at t0 offset.
	 * 3.) Sets SMI internal status to 'Scheduled'.
	 * @param smiJob
	 * @param user
	 * @param numberDonors
	 * @param numberRecipients
	 * @return
	 */
	public Long schedule(ScheduledMicroInjectionJob smiJob, Person user, int numberDonors, int numberRecipients);

	/**
	 * Cancels ScheduledMicroInjectionJob
	 * @param smiId
	 * @param user
	 * @param cancelReason
	 * @return
	 */
	public Long cancelJob(Long smiId, Person user, String cancelReason);

	/**
	 * Fails ScheduledMicroInjectionJob
	 * @param smiId
	 * @param user
	 * @param failReason
	 * @return
	 */
	public Long failJob(Long smiId, Person user, String failReason);

	/**
	 * Completes this ScheduledMicroInjectionJob.
	 * @param scheduledMicroInjectionJob
	 * @param user
	 * @return
	 */
	public Long completeJob( Job job, Person user  );

	/**
	 * Completes MicroInjection stage of this ScheduledMicroInjectionJob
	 * @param microInjection
	 * @param user
	 * @return
	 */
	public Long completeMicroInjection(MicroInjection microInjection, Person user );

	/**
	 * Completes Embryo Transfer stage of this ScheduledMicroInjectionJob
	 * @param smiId
	 * @param user
	 * @return
	 */
	public Long completeEmbryoTransfer( Long smiId, Person user );

	/**
	 * Completes Embryo Chimera Harvest stage of this ScheduledMicroInjectionJob
	 * @param scheduledMicroInjectionId
	 * @param user
	 */
	public void completeF0HarvestTask( Long scheduledMicroInjectionId, Person user );

	/**
	 * Indicates F0 Weaned stage reached of the ScheduledMicroInjectionJob associated
	 * with this embryoTransferId. NOTE: There may be multiple embryoTransferIds associated
	 * with particular ScheduledMicroInjectionJob, but we complete this milestone as soon as
	 * the first F0s are Weaned from a given Embryo Transfer.
	 * @param smiId
	 * @param actualCompletedDate
	 * @param user
	 */
	public void reachF0Milestone(Long smiId, Date actualCompletedDate, Person user);

	/**
	 * Indicates F1 Genotyping stage reached of this ScheduledMicroInjectionJob associated
	 * with this embryoTransferId. NOTE: There may be multiple embryoTransferIds associated
	 * with particular ScheduledMicroInjectionJob, but we complete this milestone as soon as
	 * the first F1s are genotyped from a given Embryo Transfer.
	 * @param smiId
	 * @param actualCompletedDate
	 * @param user
	 */
	public void reachF1Milestone(Long embryoTransferId, Date actualCompletedDate,  Person user);

	/**
	 * Given a particular ES cell line strain, returns the paternal
	 * Mouse strain required for future coat color testing when 
	 * generating blastocysts to be harvested for micro-injection.
	 * 
	 * Returns null if RequiredDonorStrain could not be found for a 
	 * given cell line strain.
	 * 
	 * Throws IllegalArgumentException if cellLineStrainDictId is null.
	 * 
	 * @param cellLineStrainDictId
	 * @return
	 */
	public DictionaryEntry fetchRequiredDonorStrain(Long cellLineStrainDictId);


}