/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;


import java.text.NumberFormat;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.animalrequest.AnimalRequest;
import uk.ac.sanger.mig.animalrequest.services.AnimalRequestService;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.DictionaryEntryEnum;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.miprocessing.EmbryoTransferProcedure;
import uk.ac.sanger.mig.miprocessing.MicroInjection;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionSearchBean;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob.MilestoneEnum;
import uk.ac.sanger.mig.miprocessing.dao.ScheduledMicroInjectionDao;
import uk.ac.sanger.mig.miprocessing.views.ScheduledMicroInjectionListView;
import uk.ac.sanger.mig.miprocessing.workflow.Job;
import uk.ac.sanger.mig.miprocessing.workflow.Task;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.services.MiRequestService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class ScheduledMicroInjectionServiceImpl extends AbstractServiceImpl<ScheduledMicroInjectionJob>
						implements ScheduledMicroInjectionService {

	private ScheduledMicroInjectionDao scheduledMicroInjectionDao;
	private MicroInjectionService microInjectionService;
	private EmbryoTransferProcedureService embryoTransferProcedureService;
	private ScheduledMiTaskService scheduledMiTaskService;
	private AnimalRequestService  animalRequestService;
	private MiRequestService miRequestService;
	private SchedulerService schedulerService;
	private PersonService personService;
	private DictionaryService dictionaryService;

		
	public DictionaryEntry fetchRequiredDonorStrain(Long cellLineStrainDictId) {
		
		if( cellLineStrainDictId == null )
			throw new IllegalArgumentException("cellLineStrainDictId cannot be null.");
		
		return scheduledMicroInjectionDao.fetchRequiredDonorStrain(cellLineStrainDictId);
	}


	
	/*
	 * TODO refactor to combinf with same method in miReq service
	 */
	public DictionaryEntry getNewStatusDict(DictionaryEntryEnum dictionaryEntryEnum) {
		return dictionaryService.fetchEntry(DictionaryEnum.MIR_EXTERNAL_STATUS_DICT, dictionaryEntryEnum);
	}

	public String deriveJobId(MiRequest miRequest) {
		
		if( miRequest == null )
			throw new IllegalArgumentException("miRequest cannot be null.");
		
		if( miRequest.getScheduledMicroInjections() == null)
			throw new UnattachedComponentException(MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS);
		
		int smiCount = miRequest.getScheduledMicroInjections().size() + 1;

		NumberFormat nf = NumberFormat.getInstance();
		nf.setMinimumIntegerDigits(2);
		String smiCountFmt = nf.format(smiCount) ;
		return "MIR_" + miRequest.getId() + ".JOB_" + smiCountFmt;
	}

	public Collection<ScheduledMicroInjectionListView> search(ScheduledMicroInjectionSearchBean
			searchBean) {

		if( !searchBean.isPaginated() && searchBean.isAllRecords())
			return scheduledMicroInjectionDao.searchAllNonPaginated( searchBean );
		else
			return scheduledMicroInjectionDao.search( searchBean );

	}

	@Transactional
	public Long store(ScheduledMicroInjectionJob scheduledMicroInjection, Person user) {
		if(scheduledMicroInjection == null) {
			throw new RuntimeException("Cannot save a null scheduledMicroInjection");
		}
		return scheduledMicroInjectionDao.store(scheduledMicroInjection, user);
	}

	public Collection<ScheduledMicroInjectionJob> fetchAllByMiRequestId(Long miRequestId,
			ComponentMap requiredComponents) {

		Collection<ScheduledMicroInjectionJob> smiList
			= scheduledMicroInjectionDao.fetchAllByMiRequestId( miRequestId );

		attachComponents(smiList, requiredComponents);

		return smiList;
	}

	public ScheduledMicroInjectionJob fetchById(Long scheduledMicroInjectionId, ComponentMap requiredComponents) {

		ScheduledMicroInjectionJob scheduledMicroInjection = scheduledMicroInjectionDao.fetchById(scheduledMicroInjectionId);

		if(scheduledMicroInjection != null)
			attachComponents(scheduledMicroInjection, requiredComponents);

		return scheduledMicroInjection;
	}

	public ScheduledMicroInjectionJob fetchWithAllComponentsById(Long id) {

		ComponentMap requiredSmiComponents = new ComponentMap();
		requiredSmiComponents.put(ScheduledMicroInjectionJob.Component.MICRO_INJECTION,
				ComponentMap.of(MicroInjection.Component.EMBRYO_HARVEST,
								MicroInjection.Component.TECHNICIAN));

		requiredSmiComponents.put(ScheduledMicroInjectionJob.Component.MI_REQUEST,
				ComponentMap.of(MiRequest.Component.CELL_LINE,
						MiRequest.Component.REQUESTER, MiRequest.Component.PROJECT_LICENCE));
		requiredSmiComponents.put(ScheduledMicroInjectionJob.Component.ACTUAL_CELL_LINE_PROVIDER,null);
		requiredSmiComponents.put(ScheduledMicroInjectionJob.Component.TASKS, null);

		ScheduledMicroInjectionJob scheduledMicroInjectionJob = 
			fetchById(id, requiredSmiComponents);

		if( scheduledMicroInjectionJob != null ) {

			Collection<EmbryoTransferProcedure> embryoTransfers
			 = embryoTransferProcedureService.fetchWithAllComponentsByScheduledMicroinjection( id );
	
			scheduledMicroInjectionJob.setEmbryoTransfers(embryoTransfers);
			assignRequiredDonorStrain( scheduledMicroInjectionJob );
			assignRequiredDonorCoatColor( scheduledMicroInjectionJob );
		}

		return scheduledMicroInjectionJob;
	}

	@Transactional
	public Long schedule(ScheduledMicroInjectionJob newSmiJob, Person user, int numberDonors, int numberRecipients) {
		miRequestService.processRequest(newSmiJob.getMiRequest(), user);
		return schedulerService.schedule(newSmiJob, user, numberDonors, numberRecipients);
	}

	@Transactional
	public Long completeJob(Job job, Person user) {
		job.changeStatus( getNewStatusDict( ScheduledMicroInjectionJob.InternalStatus.COMPLETED ) );
		ScheduledMicroInjectionJob smiJob = (ScheduledMicroInjectionJob)job;
		return store( smiJob, user );
	}

	@Transactional
	public Long cancelJob(Long smiId, Person user, String cancelReason) {

		ComponentMap smiComponents = new ComponentMap();
		ComponentMap miRequestComponents = new ComponentMap();
		miRequestComponents.put(MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS, null);
		smiComponents.put( ScheduledMicroInjectionJob.Component.MI_REQUEST, miRequestComponents);

		ScheduledMicroInjectionJob smiJob = fetchById(smiId, smiComponents);

		smiJob.changeStatus( getNewStatusDict( ScheduledMicroInjectionJob.InternalStatus.CANCELLED) );

		smiJob.setCancelOrFailReasons( cancelReason );
		//if no active jobs outstanding, set Request status back to Requested/Sched Needed
		if( !smiJob.hasActiveSiblings() ) {
			miRequestService.reRequest( smiJob.getMiRequest(), user );
		}

		return store( smiJob, user );
	}


	@Transactional
	public Long failJob(Long smiId, Person user, String failReason) {

		ComponentMap smiComponents = new ComponentMap();
		ComponentMap miRequestComponents = new ComponentMap();
		miRequestComponents.put(MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS, null);
		smiComponents.put( ScheduledMicroInjectionJob.Component.MI_REQUEST, miRequestComponents);

		ScheduledMicroInjectionJob smiJob = fetchById(smiId, smiComponents);

		smiJob.changeStatus( getNewStatusDict( ScheduledMicroInjectionJob.InternalStatus.FAILED ) );
		smiJob.setCancelOrFailReasons( failReason );
		//if no active jobs outstanding, set Request status back to Requested/Sched Needed
		if( !smiJob.hasActiveSiblings() ) {
			miRequestService.reRequest( smiJob.getMiRequest(), user );
		}

		return this.store( smiJob, user );
	}

	@Transactional
	public void completeF0HarvestTask(Long scheduledMicroInjectionId, Person user) {
		// TODO Auto-generated method stub
	}


	@Transactional
	public Long completeMicroInjection(MicroInjection microInjection, Person user){

		ScheduledMicroInjectionJob job
				= fetchById(microInjection.getScheduledMicroInjectionId(),
	    				ComponentMap.of( ScheduledMicroInjectionJob.Component.MI_REQUEST,
	    								 ScheduledMicroInjectionJob.Component.TASKS ));

		if( job.getScheduledMiStatusDictId().equals(
				ScheduledMicroInjectionJob.InternalStatus.SCHEDULED.getId() )  ) {

			job.changeStatus( getNewStatusDict( ScheduledMicroInjectionJob.InternalStatus.MICRO_INJECTION_COMPLETED ) );
			store( job, user );

		}

		Date completedDate = microInjection.getDateCompleted();

		scheduledMiTaskService.updateTaskCompletion(
				job.getId(),
				MilestoneEnum.MICRO_INJECTION_TASK_COMPLETED,
				completedDate, user );

   		microInjectionService.store( microInjection, user );

		return job.getId();
	}



	@Transactional
	public Long completeEmbryoTransfer(Long smiId, Person user) {

		ScheduledMicroInjectionJob job
				= this.fetchById( smiId,
						ComponentMap.of(
								ScheduledMicroInjectionJob.Component.MI_REQUEST,
								ScheduledMicroInjectionJob.Component.TASKS,
								ScheduledMicroInjectionJob.Component.EMBRYO_TRANSFERS ) );

		//ensure we have at least on EmbryoTransfer in Collection
		if( job.getEmbryoTransfers() == null ||
				job.getEmbryoTransfers().isEmpty() )
			throw new RuntimeException("Cannot update Embryo Transfer phase without at least one Embryo Transfer!");

		//change smi  status If completing ET and MI is last completed task...
		if( job.getScheduledMiStatusDictId().equals(
				ScheduledMicroInjectionJob.InternalStatus.MICRO_INJECTION_COMPLETED.getId())  ) {

			job.changeStatus( getNewStatusDict( ScheduledMicroInjectionJob.InternalStatus.EMBRYO_TRANSFER_COMPLETED ) );

			store( job, user );

		}

		Date completedDate = getLastEmbryoTransferCompletedDate( job );

		scheduledMiTaskService.updateTaskCompletion(
				job.getId(),
				MilestoneEnum.EMBRYO_TRANSFER_TASK_COMPLETED,
				completedDate, user );

		return job.getId();
	}

	@Transactional
	public void reachF0Milestone(Long smiId, Date actualCompletedDate, Person user) {

		ScheduledMicroInjectionJob job = fetchById(smiId,
				ComponentMap.of( ScheduledMicroInjectionJob.Component.MI_REQUEST,
								 ScheduledMicroInjectionJob.Component.TASKS ));

		//never move back
		if( !job.getScheduledMiStatusDictId().equals(
				ScheduledMicroInjectionJob.InternalStatus.F1_GENOTYPED.getId())  ) {

			job.changeStatus( getNewStatusDict( ScheduledMicroInjectionJob.InternalStatus.F0_WEANED ) );
			store( job, user );

		}

		scheduledMiTaskService.updateTaskCompletion(job.getId(),
					MilestoneEnum.F0_WEANED_TASK_COMPLETED,
					actualCompletedDate,
					user);

	}

	@Transactional
	public void reachF1Milestone(Long smiId, Date actualCompletedDate,  Person user) {

		ScheduledMicroInjectionJob job = fetchById(smiId,
				ComponentMap.of( ScheduledMicroInjectionJob.Component.MI_REQUEST,
								 ScheduledMicroInjectionJob.Component.TASKS ));

		job.changeStatus( getNewStatusDict( ScheduledMicroInjectionJob.InternalStatus.F1_GENOTYPED ) );
		store( job, user );

		scheduledMiTaskService.updateTaskCompletion(job.getId(),
					MilestoneEnum.F1_GENOTYPING_TASK_COMPLETED,
					actualCompletedDate,
					user);

	}


	@Override
	protected void attachComponents(ScheduledMicroInjectionJob scheduledMicroInjection, ComponentMap requiredComponents) {
		if(scheduledMicroInjection == null) {
			throw new RuntimeException("Cannot attach components to null scheduledMicroInjection");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.TASKS)) {
				scheduledMicroInjection.setTasks(fetchTasksBySmiJobId(
						scheduledMicroInjection.getId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.TASKS)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.MICRO_INJECTION)) {
				scheduledMicroInjection.setMicroInjection(fetchMicroInjectionBySmiJobId(
						 scheduledMicroInjection.getId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.MICRO_INJECTION)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.MI_REQUEST)) {
				scheduledMicroInjection.setMiRequest(fetchMiRequestById(
						scheduledMicroInjection.getMiRequestId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.MI_REQUEST)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.EMBRYO_TRANSFERS)) {
				scheduledMicroInjection.setEmbryoTransfers(fetchEmbryoTransfersBySmiJobId(
						scheduledMicroInjection.getId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.EMBRYO_TRANSFERS)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.DONOR_ANIMAL_REQUEST)) {
				scheduledMicroInjection.setDonorAnimalRequest(fetchAnimalRequestById(
						scheduledMicroInjection.getDonorAnimalRequestId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.DONOR_ANIMAL_REQUEST)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.RECIPIENT_ANIMAL_REQUEST)) {
				scheduledMicroInjection.setRecipientAnimalRequest(fetchAnimalRequestById(
						scheduledMicroInjection.getRecipientAnimalRequestId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.RECIPIENT_ANIMAL_REQUEST)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.SUPPLEMENTAL_RECIPIENT_ANIMAL_REQUEST)) {
				scheduledMicroInjection.setSupplementalRecipientAnimalRequest(fetchAnimalRequestById(
						scheduledMicroInjection.getSupplementalRecipientAnimalRequestId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.SUPPLEMENTAL_RECIPIENT_ANIMAL_REQUEST)));
			}

			if(fetchAll || requiredComponents.containsKey(ScheduledMicroInjectionJob.Component.ACTUAL_CELL_LINE_PROVIDER)) {
				scheduledMicroInjection.setCellLineProvider(fetchCellLineProviderById(
						scheduledMicroInjection.getCellLineProviderId(),
						requiredComponents.get(ScheduledMicroInjectionJob.Component.ACTUAL_CELL_LINE_PROVIDER)));
			}

		}
	}


	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// dependencies //////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setAnimalRequestService(AnimalRequestService animalRequestService) {
		this.animalRequestService = animalRequestService;
	}

	public void setEmbryoTransferProcedureService(
			EmbryoTransferProcedureService embryoTransferProcedureService) {
		this.embryoTransferProcedureService = embryoTransferProcedureService;
	}

	public void setMicroInjectionService(MicroInjectionService microInjectionService) {
		this.microInjectionService = microInjectionService;
	}


	public void setScheduledMiTaskService(
			ScheduledMiTaskService scheduledMiTaskService) {
		this.scheduledMiTaskService = scheduledMiTaskService;
	}

	public void setMiRequestService(MiRequestService miRequestService) {
		this.miRequestService = miRequestService;
	}

	public void setSchedulerService(SchedulerService schedulerService) {
		this.schedulerService = schedulerService;
	}

	public void setScheduledMicroInjectionDao(ScheduledMicroInjectionDao scheduledMicroInjectionDao) {
		this.scheduledMicroInjectionDao = scheduledMicroInjectionDao;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// privates //////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
	private Person fetchCellLineProviderById(Long personId, ComponentMap requiredComponents) {
		return personService.fetchById(personId, requiredComponents);
	}

	private Collection<EmbryoTransferProcedure> fetchEmbryoTransfersBySmiJobId(Long smiId,
			ComponentMap requiredComponents) {
		return embryoTransferProcedureService.fetchBySmiJobId(smiId,
				requiredComponents);
	}

	private List<Task> fetchTasksBySmiJobId(Long smiId, ComponentMap requiredComponents) {
		return scheduledMiTaskService.fetchAllBySmiJobId(smiId);
	}

	private MiRequest fetchMiRequestById(Long miId,  ComponentMap requiredComponents) {
		return miRequestService.fetchById( miId, requiredComponents );
	}

	private MicroInjection fetchMicroInjectionBySmiJobId(Long smiId, ComponentMap requiredComponents) {
		if(smiId != null )
			return  microInjectionService.fetchBySmiJobId(smiId, requiredComponents);
		return null;
	}

	private AnimalRequest fetchAnimalRequestById(Long animalRequestId, ComponentMap requiredComponents) {
		if( animalRequestId != null)
			return animalRequestService.fetchById(animalRequestId, requiredComponents);
		return null;
	}

	private Date getLastEmbryoTransferCompletedDate(ScheduledMicroInjectionJob scheduledMicroInjection) {
		return ((EmbryoTransferProcedure)(scheduledMicroInjection.
				getEmbryoTransfers().toArray())
				[ scheduledMicroInjection.getEmbryoTransfers().size() -1 ]).
				getCreatedDate();
	}

	//should refactor into MiRequest object sicve this is where CellLine located
	private void assignRequiredDonorStrain(ScheduledMicroInjectionJob smiJob) {

		Long cellLineStrainDictId = smiJob.getMiRequest().getCellLineStrainId();
		DictionaryEntry requiredDonorStrain  = fetchRequiredDonorStrain( cellLineStrainDictId );
		smiJob.setRequiredDonorStrain( requiredDonorStrain );
	}

	//	should refactor into MiRequest object sinve this is where CellLine located
	private void assignRequiredDonorCoatColor(ScheduledMicroInjectionJob smiJob) {
		
		Long requiredDonorStrainId = smiJob.getRequiredDonorStrain().getId();
		DictionaryEntry donorCoatColor = fetchCoatColorByStrainId( 
				requiredDonorStrainId );
		smiJob.setRequiredDonorCoatColor(donorCoatColor);
	
	}

	private DictionaryEntry fetchCoatColorByStrainId(Long strainId) {
		
		DictionaryEntry coatColour = null;

		Collection<DictionaryEntry> entries = dictionaryService.fetchDependant(
				DictionaryEnum.MOUSE_COLOUR_WITH_STRAIN, strainId, "strain_id");

		if(entries.size() == 1) {
			coatColour = entries.iterator().next();
		}

		return coatColour;
	}
}
