/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;

import java.util.Date;
import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.animalrequest.AnimalRequest;
import uk.ac.sanger.mig.animalrequest.AnimalRequest.Group;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.miprocessing.ScheduledMiTask;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.workflow.Task;
import uk.ac.sanger.mig.miprocessing.workflow.TaskService;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.services.MiRequestService;
import uk.ac.sanger.mig.personnel.Person;

/**
 *
 * Encapsulates schedule method into a single class at package level
 * (based on [Fowler,Beck] Move Method Into Class refactoring pattern)
 * @author ae1
 *
 */
public class SchedulerServiceImpl implements SchedulerService {

	private ScheduledMicroInjectionService scheduledMicroInjectionService;
	private MiRequestService miRequestService;
	private ScheduledMiTaskService scheduledMiTaskService;
	private TaskService taskService;
	private MirMouseTrackingBridgeService mirMouseTrackingBridgeService;

	@Transactional
	public Long schedule(final ScheduledMicroInjectionJob newSmiJob, final Person user,
			final int numberDonors, final int numberRecipients) {

		validate( newSmiJob, user, numberDonors, numberRecipients );
			
		MiRequest miRequest = miRequestService.fetchById(newSmiJob.getMiRequestId(),
				ComponentMap.of( MiRequest.Component.CELL_LINE ));	
		
		if( miRequest == null )
			throw new RuntimeException("Could not find MiRequest for newSmiJob: "+newSmiJob);
		
		final Date anchorDate = newSmiJob.getAnchorDate();
		
		//make wt reqs and attached to smi, if needed
		if( numberDonors > 0 ) {
			
			Long pplId = miRequest.getProjectLicenceId();
			
			DictionaryEntry requiredDonorStrain 
				= scheduledMicroInjectionService.fetchRequiredDonorStrain( miRequest.getCellLineStrainId() );
			
			Long animalRequestId = requestDonors(
					anchorDate, 
					numberDonors,
					pplId,
					requiredDonorStrain, 
					user);
			
			newSmiJob.setDonorAnimalRequestId( animalRequestId );
		}
		
		if( numberRecipients > 0 ) {
					
			Long pplId = miRequest.getProjectLicenceId();
			
			Long animalRequestId = requestRecipients(
					anchorDate, 
					numberRecipients,
					pplId,
					user);
			
			newSmiJob.setRecipientAnimalRequestId( animalRequestId );
		}

		//set smi status to scheduled
		newSmiJob.changeStatus( scheduledMicroInjectionService.getNewStatusDict(
				ScheduledMicroInjectionJob.InternalStatus.SCHEDULED ) );
		
		Long smiJobId = scheduledMicroInjectionService.store( newSmiJob, user );
		
		List<Task> tasks = taskService.generateTaskList(anchorDate, miRequest);

		attachTasksToScheduledMicroInjection( smiJobId, tasks, user );

		return newSmiJob.getId();
	}

	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// dependencies //////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
	public void setMiRequestService(MiRequestService miRequestService) {
		this.miRequestService = miRequestService;
	}

	public void setMirMouseTrackingBridgeService(
			MirMouseTrackingBridgeService mirMouseTrackingBridgeService) {
		this.mirMouseTrackingBridgeService = mirMouseTrackingBridgeService;
	}

	public void setScheduledMicroInjectionService(
			ScheduledMicroInjectionService scheduledMicroInjectionService) {
		this.scheduledMicroInjectionService = scheduledMicroInjectionService;
	}

	public void setScheduledMiTaskService(
			ScheduledMiTaskService scheduledMiTaskService) {
		this.scheduledMiTaskService = scheduledMiTaskService;
	}

	public void setTaskService(TaskService taskService) {
		this.taskService = taskService;
	}
	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// privates //////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
	private void attachTasksToScheduledMicroInjection(Long smiJobId,
			List<Task> tasks, Person user ) {
		
		for(Task smiTask : tasks) {
			ScheduledMiTask task = (ScheduledMiTask)smiTask;
			task.setParentJobId( smiJobId );
			scheduledMiTaskService.store(task,user);
		}
	}

	
	private Long requestDonors(Date dateRequired, Integer numberDonors, 
			Long pplId, DictionaryEntry requiredDonorStrain, Person user) {
	
		AnimalRequest animalRequest
			= AnimalRequest.createDefaultGroupRequest(Group.MICRO_INJECTION_EMBRYO_DONORS,
					dateRequired,
					user,
					pplId);
	
		animalRequest.setStrainId( requiredDonorStrain.getId() );	 //donor strain derived from MI Request
		animalRequest.setNumFemalesNeeded( numberDonors );
	
		return mirMouseTrackingBridgeService.storeAnimalRequest( animalRequest, user );
			
	}
	
	private Long requestRecipients(Date dateRequired, Integer numberRecipients, 
			Long pplId, Person user) {

		AnimalRequest animalRequest
		    	= AnimalRequest.createDefaultGroupRequest(Group.MICRO_INJECTION_EMBRYO_RECIPIENTS,
		    			dateRequired,
		    			user,
		    			pplId );

		animalRequest.setNumFemalesNeeded( numberRecipients );

		return mirMouseTrackingBridgeService.storeAnimalRequest( animalRequest, user );
	
	}
	
	private void validate(ScheduledMicroInjectionJob newSmiJob, Person user,
			int numberDonors, int numberRecipients) {
		
		if( newSmiJob == null )
			throw new IllegalArgumentException("newSmiJob cannot be null.");
	
		if( user == null )
			throw new IllegalArgumentException("user cannot be null.");
		
		if( numberDonors < 0 )
			throw new IllegalArgumentException("numberDonors cannot be < 0.");

		if( numberRecipients < 0 )
			throw new IllegalArgumentException("numberRecipients cannot be < 0.");	
		
		if( newSmiJob.getAnchorDate() == null)
			throw new IllegalStateException("SMI's Target (anchor) date canno be null.");
	
		if( newSmiJob.getMiRequestId() == null)
			throw new IllegalStateException("SMI's MiRequest ID cannot be null.");
		
	}


}


