/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.services.ProtocolService;
import uk.ac.sanger.mig.miprocessing.EmbryoTransferProcedure;
import uk.ac.sanger.mig.miprocessing.forms.EmbryoTransferProcedureForm;

public class EmbryoTransferProcedureFormValidator implements Validator {

	private ProtocolService protocolService;

	public static final Long NO_ANALGESIA = 2L;
	private final static Long DEFAULT_ANALGESIA_DICT_ID = 1L;//Analgesia Vetergesic

	public boolean supports(Class clazz) {
		return EmbryoTransferProcedureForm.class.equals(clazz);
	}

	public void validate(Object commandForm, Errors errors) {

		EmbryoTransferProcedureForm embryoTransferProcedureForm = (EmbryoTransferProcedureForm)commandForm;
		EmbryoTransferProcedure embryoTransferProcedure = embryoTransferProcedureForm.getEmbryoTransferProcedure();

		/*
		 * RULES:
		 --* 1. Personal Licence Holder (Person) is required and must have a valid PIL
		 --* 2. Project Licence must not be empty and must be valid--use to derive protocol
		 --* 3. Mouse is required field and must be female.
		 --* 4. embryo transfer left or right reqired fields
		 --* 5. if stuffers specified, must choose stuffer strain
		 --* 6. mouse weight is a required field
		 --* 7. anaesthetic and dose are required fields
		 --* 8. if analgesic specified must specify analgesia dose
		 --* 9. date completed is required
		 --* 10. must have cage/rack Id
		 --* 12. cannot transfer into female already used in transfer- this is postpond, need to revisit after the drop down list
		 --* 13. earmark id must not be empty
		 --* 14. The total embryo number can not be greater than the embryo microinjected
		     15. Transfer Type mandatory
		 */

		//1 PIL is populated here on validation
		Long personId = embryoTransferProcedure.getMouseProcedure().getPersonalLicence().getPersonId();
		Long personalLicenceId = embryoTransferProcedure.getMouseProcedure().getPersonalLicence().getId();

		if(personId == null) {

			errors.reject(null,"Personal License holder is required.");

		} else if (personalLicenceId == null) {

			errors.reject(null, "The selected PIL holder does not have a valid licence to perform this procedure.");
		}

		////2 Protocol is populated here on validation
		if( embryoTransferProcedure.getMouseProcedure().getProtocol().getProjectLicenceId() == null ) {
			errors.reject( null,"Project License (PPL) is required.");
			embryoTransferProcedure.getMouseProcedure().getProtocol().getProjectLicence().setDisplayText("");
		} else {

			//need protocol derived prom PPL/Procedure/PIL
			Protocol protocol = protocolService.fetchByProcedureAndProject(
					embryoTransferProcedure.getMouseProcedure().getProcedureDictId(),
					embryoTransferProcedure.getMouseProcedure().getProtocol().getProjectLicenceId(),
					false,
					null);

			if( protocol == null )
				errors.reject( null,"Could not find Protocol for Mouse Procedure ID " +
						embryoTransferProcedure.getMouseProcedure().getProcedureDictId() +
						", Project Licence ID " +
						embryoTransferProcedure.getMouseProcedure().getProtocol().getProjectLicenceId());
			else
				embryoTransferProcedure.getMouseProcedure().setProtocolId(protocol.getId());
		}

		//5
		if( embryoTransferProcedure.getTotalStuffers() > 0) {
			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"embryoTransferProcedure.stufferStrainDictId",
					null,"Stuffer Strain is required.");
		}

		//6
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"embryoTransferProcedure.recipientWeight",
				null,"Recipient Female Weight is required.");
		if(embryoTransferProcedure.getRecipientWeight() != null
				&& embryoTransferProcedure.getRecipientWeight().doubleValue()<=0)
			errors.reject(null,"Recipient Female Weight must be a positive value");

		//7 anaesthetic
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"embryoTransferProcedure.anaestheticDictId",
				null,"Anaesthetic Used is required.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"embryoTransferProcedure.doseAnaesthetic",
				null,"Anaesthetic Dose is required.");
		if(embryoTransferProcedure.getDoseAnaesthetic() != null
				&& embryoTransferProcedure.getDoseAnaesthetic().doubleValue()<=0)

			errors.reject(null,"Anaesthetic Dose must be a positive value");


		//8
		if(embryoTransferProcedure.getAnalgesiaDictId()!=null
				&& embryoTransferProcedure.getAnalgesiaDictId().equals(DEFAULT_ANALGESIA_DICT_ID)){

			if(embryoTransferProcedure.getDoseAnalgesia() == null
					|| embryoTransferProcedure.getDoseAnalgesia().equals(0D)){
				errors.reject(null,"Analgesia Dose is required.");
			}else{
				if(embryoTransferProcedure.getDoseAnalgesia().doubleValue()<=0){
					errors.reject(null,"Analgesia Dose must be a positive value");
				}
			}
		}

		//9
		if(embryoTransferProcedureForm.getDateCompleted() == null) {
			errors.reject(null, "Date Completed is required.");
		}

		//10
		if (embryoTransferProcedureForm.getEmbryoTransferCageOrRackId() ==  null) {
			errors.reject(null, "Recovery Rack or Cage Barcode is required. ");
		}

		//13
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "embryoTransferProcedure.mouse.earmarkDictId",
				null, "Earmark is required.");

		Integer availableEmbryos = embryoTransferProcedure.getScheduledMicroInjection().getAvailableEmbryos();


		//14
		Integer totalEmbryosTrasferred = embryoTransferProcedure.getTotalEmbryosTransferred();
		if(totalEmbryosTrasferred > availableEmbryos) {
			errors.reject(null, "Total embryos transfered " +
					totalEmbryosTrasferred + ", is greater than the number available, " +
					availableEmbryos +
					".  Please check the number of embryos transfered or the embryo count of the corresponding microinjection.");

		} else if(totalEmbryosTrasferred == 0) {
			errors.reject(null, "Please specify the number embryos transfer into left and/or right horns.");
		}
		
		//15
		if( embryoTransferProcedure.getTransferTypeDictId() == null ) {
			errors.reject(null, "Transfer Type is a required field.");
		}
	}

	///////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////// dependencies //////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////

	public void setProtocolService(ProtocolService protocolService) {
		this.protocolService = protocolService;
	}

}