/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.validators;

import java.util.HashSet;
import java.util.Set;

import org.springframework.validation.Errors;

import uk.ac.sanger.mig.common.AbstractBindingValidator;
import uk.ac.sanger.mig.common.AllowedFormField;
import uk.ac.sanger.mig.miprocessing.MicroInjection;
import uk.ac.sanger.mig.miprocessing.services.MicroInjectionService;

public class MicroInjectionValidator extends AbstractBindingValidator {
	
	private MicroInjectionService microInjectionService;
	
	@Override
	public Set<AllowedFormField> defineAllowedFields() {
		Set<AllowedFormField> formFields = new HashSet<AllowedFormField>();			
		//required fields
		formFields.add(new AllowedFormField("embryoStageDictId", "Embryo Stage", true) );
		formFields.add(new AllowedFormField("injectedEmbryoCount", "Embryos Injected", true) );
		formFields.add(new AllowedFormField("embryoHarvestId", "Embryo Harvest", true) );
		formFields.add(new AllowedFormField("dateCompleted", "Date Complete", true) );
		formFields.add(new AllowedFormField("esCellsPerEmbryoFrom", "ES Cells Per Embryo From", true) );
		formFields.add(new AllowedFormField("esCellsPerEmbryoTo", "ES Cells Per Embryo To", true) );		
		formFields.add(new AllowedFormField("technicianId", "Technician", true) );	
		formFields.add(new AllowedFormField("technician.displayText", "Technician Name", true) );	
		//allowed, but optional	
		formFields.add(new AllowedFormField("comments", "Comments", false) );	
		return formFields;
	}

	public void validate(Object object, Errors errors, boolean hasBindingErrors) {	
		
		if(hasBindingErrors) return; //do not validate logic unless our form is complete
		
        //validate logic...		
		MicroInjection microInjection = (MicroInjection)object;
			
		if((microInjection.getEsCellsPerEmbryoFrom() != null &&
				microInjection.getEsCellsPerEmbryoTo() != null) &&
				microInjection.getEsCellsPerEmbryoFrom() > microInjection.getEsCellsPerEmbryoTo()) {
			errors.reject(null, "ES Cells/Embryo 'From' field must be less than or equal to 'To' field."); 
		}
    
		//if new MI, make sure there's not already an MI with given SMI
		//This might happen if user clones browser and tries to
		//complete MI 2x for same SMI
		
		if( microInjection.getId() == null ) {
		
			MicroInjection existingMi 
				= microInjectionService.fetchBySmiJobId( 
						microInjection.getScheduledMicroInjectionId(), null);
		
			if( existingMi != null)
				errors.reject(null, "Unable to complete. A Microinjection already has been completed for the above Scheduled Job.");
		}
	
		
		
	}

	public boolean supports(Class clazz) {
		return MicroInjection.class.equals(clazz);
	}

	public void setMicroInjectionService(MicroInjectionService microInjectionService) {
		this.microInjectionService = microInjectionService;
	}



	
}