/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.workflow;

import java.util.Date;
import java.util.List;

import uk.ac.sanger.mig.common.DictionaryEntry;


public interface Job<T extends Enum> {
	
	/**
	 * Returns an ordered List of all Task objects.
	 * Ordering is determined by implementation of 
	 * the Comparable interface which Task extends,
	 * typically done using Task.getSequenceNumber().
	 * @return  
	 */
	public abstract List<Task> getTasks();

	
	/**
	 * Sets the List of all Task objects. User should
	 * use this method, and this method only,
	 * to set Task List to ensure proper ordering
	 * of Tasks.
	 * @param tasks
	 */
	public abstract void setTasks(List<Task> tasks);    
    
    /**
     * An arbitrary sub-set of the Job task list.
     * Ordering in the task list is preserved according
     * to Task implementation of Comparable interface.
     * @return
     */
	public abstract List<Task> getMilestones();
 
	/**
	 * Returns the next  milestone  task that needs to be completed in the task list
	 * @return
	 */
	public Task getNextMilestoneTaskToComplete();
		
	/**
	 * Returns a Task object whose milestone attribute is true
	 * given Enum object
	 * @param milestoneEnum
	 * @return
	 */
	public abstract Task getMilestoneByEnum(T milestoneEnum);
	
    /**
     * Returns the critical date to which the timing of
     * all other tasks are anchored for this Job.
     * @return
     */
	public abstract Date getAnchorDate();
    
	/**
	 * Sets the critical date to which the timing of
     * all other tasks are anchored for this Job.
	 * @param targetDate
	 */
    public abstract void setAnchorDate(Date targetDate);
    
    public abstract boolean isCompleted();
    
    public abstract boolean isAborted();
    
	public boolean isActive();  
    

	public boolean isEndpointReached();

	public void changeStatus(DictionaryEntry newStatus);
    
}
