/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.workflow;

import java.util.List;

import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.mirequest.MiRequest;

public class TaskDaoSpringImpl extends SimpleJdbcDaoSupport implements TaskDao {
	
	private TaskRowMapper taskRowMapper;
	private String taskTemplateTable;
	private String f1WorkflowRuleTable;
	
	public List<TaskTemplate> fetchTaskTemplates(MiRequest miRequest) {
		
		validate( miRequest );
		
		List<TaskTemplate> taskTemplates = null;
		
		if( miRequest.isEndpointF1() ) {
			
			taskTemplates = searchTemplatesByF1WorkflowRule( miRequest );
			
		}
		
		if( taskTemplates == null || taskTemplates.isEmpty() ) {
			
			taskTemplates = fetchDefaultTemplatesByEndpoint( miRequest.getEndpointDictId() );
			
		}
		
		if( taskTemplates.isEmpty() )
			throw new RuntimeException(
					"Unable to find task group for Endpoint ID: "+ miRequest.getEndpointDictId());
		
		return taskTemplates;
	}
	
	
	private  List<TaskTemplate> searchTemplatesByF1WorkflowRule(MiRequest miRequest) {

		String f1Sql = "select t.*, g.name from " + taskTemplateTable +
		 " t, " + DictionaryEnum.MIR_TASK_GROUP_DICT.getTableName() +
		 " g, " + f1WorkflowRuleTable + " w where 1=1" +
        " and w.cell_line_strain_id = ? and w.chimera_mate_strain_id = ? " +
        " and w.task_group_id = g.id and g.id = t.task_group_id";
	
		List<TaskTemplate> taskTemplates = getSimpleJdbcTemplate().query(
				f1Sql, 
				taskRowMapper, 
				miRequest.getCellLineStrainId(),
				miRequest.getChimeraMateStrainDictId());	
		
		return taskTemplates;
		
	}
	
	private  List<TaskTemplate> fetchDefaultTemplatesByEndpoint(Long endpointId) {

		String sql = "select t.* from mir_task_template t, mir_task_group_dict g, " +
		" mir_target_endpoint_dict e where e.id = ? and e.default_task_group_id = g.id " +
		" and t.task_group_id = g.id";
	
		List<TaskTemplate> taskTemplates = getSimpleJdbcTemplate().query(
				sql, 
				taskRowMapper, 
				endpointId);	
		
		return taskTemplates;
		
	}
	
	private void validate(MiRequest miRequest) {
		
		if( miRequest == null )
			throw new IllegalArgumentException("MiRequest cannot be null.");			
		
		if( miRequest.getEndpointDictId() == null )
			throw new RuntimeException("miRequest endpoint id cannot be null.");
		
		if( miRequest.isEndpointF1() &&
				(miRequest.getChimeraMateStrainDictId() == null ||
						miRequest.getCellLineStrainId() == null )) {	
			throw new RuntimeException("miRequest cell line strain and chimera mate strain " +
					"cannot be null if the miRequest endpoint is F1.");
		}
		
	}


	public void setTaskRowMapper(TaskRowMapper taskRowMapper) {
		this.taskRowMapper = taskRowMapper;
	}

	public void setTaskTemplateTable(String taskTemplateTable) {
		this.taskTemplateTable = taskTemplateTable;
	}



	public void setF1WorkflowRuleTable(String workflowRuleTable) {
		this.f1WorkflowRuleTable = workflowRuleTable;
	}
	


	
	
}
