/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.DictionaryEntryEnum;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.personnel.Person;

public class MiRequest extends AuthenticatedDataObject {
	
	public enum Component implements ComponentEnum {	
		ALL_MIREQUEST_COMPONENTS,
		REQUESTER,
		CELL_LINE,
		PROJECT_LICENCE,
		SCHEDULED_MICRO_INJECTION_JOBS,
		COLONY
	}
	
	public enum ExternalStatusEnum implements DictionaryEntryEnum {	
		REQUESTED( 			   1L ),  //system and user driven
		AWAITING_MAP_TEST(    14L ), //user driven
		IN_PROCESS( 	      17L ),  //system driven
		COMPLETED(            20L ), //user driven
		CANCELLED(            13L ), //user driven
		DELAYED( 			   3L ); //user driven
		
		private Long dictionaryId;
		
		ExternalStatusEnum(Long dictId) {
			this.dictionaryId = dictId;
		}

		public Long getId() {
			return dictionaryId;
		}		
	}
	
	public enum EndpointEnum implements DictionaryEntryEnum  {	
		F1(                 1L ),
		F0(                 2L ),
		EMBRYONIC_CHIMERAS( 3L );
		
		private Long dictionaryId;
		
		EndpointEnum(Long dictId) {
			this.dictionaryId = dictId;
		}
			
		public Long getId() {
			return dictionaryId;
		}	
	}

	private Long donorPlugCount;        
	private boolean genotypingRequired;  	
	private Long chimeraMateStrainDictId; 
	private DictionaryEntry chimeraMateStrainDict;
	private Long donorEmbryoStageDictId;        
	private DictionaryEntry donorEmbryoStageDict;
	private Long endpointDictId;   
	private DictionaryEntry endpointDict;	
	private Long workflowPipelineDictId; 
	private DictionaryEntry workflowPipelineDict;
	private Long cellLineId;  
	private CellLine cellLine;
	private Long projectLicenceId;  
	private ProjectLicence projectLicence;
	private Long requesterId; 
	private Person requester;
	private Long externalStatusDictId; 
	private DictionaryEntry externalStatusDict; 
	private String statusComments;
	private String fundingSource;
	private Collection<ScheduledMicroInjectionJob> scheduledMicroInjections;
	private boolean requesterProvidedGenotyping;//if user has genotype test already in place
	private Long colonyInfoId;
	private ColonyInfo colonyInfo;
	
	
	public boolean isTargetEndpointLocked() {
		
		if( id == null  ) return false;
		
		if(id != null && scheduledMicroInjections == null )
			throw new RuntimeException("Collection<ScheduledMicroInjectionJob> cannot be null!  Need to attach this component if existing MI Request.");
		return !scheduledMicroInjections.isEmpty();
	}
	
	/**
	 * Returns true if this request's status is Awaiting Map Test
	 * @return
	 */
	public boolean isAwaitingMapTest() {
		return externalStatusDictId.equals( ExternalStatusEnum.AWAITING_MAP_TEST.getId() );
	}
	
	/**
	 * Returns true if this request's status is Delayed
	 * @return
	 */
	public boolean isDelayed() {
		return externalStatusDictId.equals( ExternalStatusEnum.DELAYED.getId() );
	}
	
	/**
	 * Returns true if this request's status is Requested
	 * @return
	 */
	public boolean isRequested() {
		return externalStatusDictId.equals( ExternalStatusEnum.REQUESTED.getId() );
	}
	
	/**
	 * Returns true there are one or more active jobs outstanding *and*
	 * they have all reached their endpoint--ie, there are no active jobs 
	 * still working towards reaching their endpoint. 
	 * @return
	 */
	public boolean isReadyToComplete() {
		if(  getActiveJobs().isEmpty() ) return false;
		for(ScheduledMicroInjectionJob activeJob :  getActiveJobs() ) {
			if( !activeJob.isEndpointReached() )
				return false;		
		}
		return true;
	}
	
	/**
	 * Returns all currently active SMIs under this request
	 * 
	 * Collection<cheduledMicroInjection> must be attached
	 * 
	 * @return
	 */
	public Collection<ScheduledMicroInjectionJob> getActiveJobs() {
		
		if( scheduledMicroInjections == null )
			throw new UnattachedComponentException( MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS );
		
		Collection<ScheduledMicroInjectionJob> activeJobs 
			= new ArrayList<ScheduledMicroInjectionJob> ();
		
		for(ScheduledMicroInjectionJob job : scheduledMicroInjections) {
			if(job.isActive()) {
				activeJobs.add(job);
			}
		}
		
		return activeJobs;
	}
		 
	/**
	 * Returns true if this Request is active--ie, not complete and not cancelled.
	 * @return
	 */
	public boolean isActive() {
    	return (
    			!(
    				externalStatusDictId.equals( ExternalStatusEnum.COMPLETED.getId() )
    				||  externalStatusDictId.equals( ExternalStatusEnum.CANCELLED.getId() )
    			)
    		);		
	}
		
	public Long getCellLineStrainId() {	
		return ( getCellLine() != null ) ? getCellLine().getCellLineStrainDictId(): null;
	}
	
	/**
	 * Changes the status of this request (NOTE: both dict and dictId 
	 * changed for maintaining consistent object state!)
	 * @param newStatus
	 */
	public void changeStatus(DictionaryEntry newStatus) {
		this.externalStatusDict = newStatus;
		this.externalStatusDictId = newStatus.getId();
	}
	
	public boolean isEndpointF1() {
		return endpointDictId.equals(EndpointEnum.F1.getId());
	}
	


	//////////////////////////////////////////////////////////
	///////////////// bit bucket methods /////////////////////
	//////////////////////////////////////////////////////////		
	
	public String getStatusComments() {
		return statusComments;
	}

	public void setStatusComments(String statusComments) {
		this.statusComments = statusComments;
	}
	
	public DictionaryEntry getDonorCellLineStrainDict() {
		return (cellLine != null) ? cellLine.getCellLineStrain() : null;
	}
	
	public DictionaryEntry getChimeraMateStrainDict() {
		return chimeraMateStrainDict;
	}


	public void setChimeraMateStrainDict(DictionaryEntry chimeraMateStrainDict) {
		this.chimeraMateStrainDict = chimeraMateStrainDict;
	}


	public Long getChimeraMateStrainDictId() {
		return chimeraMateStrainDictId;
	}


	public void setChimeraMateStrainDictId(Long chimeraMateStrainDictId) {
		this.chimeraMateStrainDictId = chimeraMateStrainDictId;
	}
	
	public Long getCellLineId() {
		return cellLineId;
	}
	public void setCellLineId(Long cellLineId) {
		this.cellLineId = cellLineId;
	}
	
	public DictionaryEntry getDonorEmbryoStageDict() {
		return donorEmbryoStageDict;
	}
	public void setDonorEmbryoStageDict(DictionaryEntry donorEmbryoStageDict) {
		this.donorEmbryoStageDict = donorEmbryoStageDict;
	}
	public Long getDonorEmbryoStageDictId() {
		return donorEmbryoStageDictId;
	}
	public void setDonorEmbryoStageDictId(Long donorEmbryoStageDictId) {
		this.donorEmbryoStageDictId = donorEmbryoStageDictId;
	}
	public Long getDonorPlugCount() {
		return donorPlugCount;
	}
	public void setDonorPlugCount(Long donorPlugCount) {
		this.donorPlugCount = donorPlugCount;
	}
	
	public DictionaryEntry getEndpointDict() {
		return endpointDict;
	}
	public void setEndpointDict(DictionaryEntry endpointDict) {
		this.endpointDict = endpointDict;
	}
	public Long getEndpointDictId() {
		return endpointDictId;
	}
	public void setEndpointDictId(Long endpointDictId) {
		this.endpointDictId = endpointDictId;
	}
	
	public boolean isGenotypingRequired() {
		return genotypingRequired;
	}

	public void setGenotypingRequired(boolean genotypingRequired) {
		this.genotypingRequired = genotypingRequired;
	}
	public CellLine getCellLine() {
		return cellLine;
	}
	public void setCellLine(CellLine cellLine) {
		this.cellLine = cellLine;
	}
	public ProjectLicence getProjectLicence() {
		return projectLicence;
	}
	public void setProjectLicence(ProjectLicence projectLicence) {
		this.projectLicence = projectLicence;
	}
	public Long getProjectLicenceId() {
		return projectLicenceId;
	}
	public void setProjectLicenceId(Long projectLicenceId) {
		this.projectLicenceId = projectLicenceId;
	}
	public Person getRequester() {
		return requester;
	}
	public void setRequester(Person requester) {
		this.requester = requester;
	}
	public Long getRequesterId() {
		return requesterId;
	}
	public void setRequesterId(Long requesterId) {
		this.requesterId = requesterId;
	}


	public DictionaryEntry getWorkflowPipelineDict() {
		return workflowPipelineDict;
	}


	public void setWorkflowPipelineDict(DictionaryEntry workflowPipelineDict) {
		this.workflowPipelineDict = workflowPipelineDict;
	}


	public Long getWorkflowPipelineDictId() {
		return workflowPipelineDictId;
	}

	public void setWorkflowPipelineDictId(Long workflowPipelineDictId) {
		this.workflowPipelineDictId = workflowPipelineDictId;
	}

	public DictionaryEntry getExternalStatusDict() {
		return externalStatusDict;
	}

	public void setExternalStatusDict(DictionaryEntry externalStatusDict) {
		this.externalStatusDict = externalStatusDict;
	}

	public Long getExternalStatusDictId() {
		return externalStatusDictId;
	}

	public void setExternalStatusDictId(Long externalStatusDictId) {
		this.externalStatusDictId = externalStatusDictId;
	}

	public Collection<ScheduledMicroInjectionJob> getScheduledMicroInjections() {
		return scheduledMicroInjections;
	}

	public void setScheduledMicroInjections(
			Collection<ScheduledMicroInjectionJob> scheduledMicroInjections) {
		this.scheduledMicroInjections = scheduledMicroInjections;
	}

	public String getFundingSource() {
		return fundingSource;
	}

	public void setFundingSource(String fundingSource) {
		this.fundingSource = fundingSource;
	}

	public boolean isRequesterProvidedGenotyping() {
		return this.requesterProvidedGenotyping;
	}

	public void setRequesterProvidedGenotyping(boolean requesterProvidedGenotyping) {
		this.requesterProvidedGenotyping = requesterProvidedGenotyping;
	}

	public Long getColonyInfoId() {
		return colonyInfoId;
	}

	public void setColonyInfoId(Long colonyInfoId) {
		this.colonyInfoId = colonyInfoId;
	}

	public ColonyInfo getColonyInfo() {
		return colonyInfo;
	}

	public void setColonyInfo(ColonyInfo colonyInfo) {
		this.colonyInfo = colonyInfo;
	}
}
