/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.controllers;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.forms.CellLineForm;
import uk.ac.sanger.mig.mirequest.services.CellLineService;

public class CellLineFormController extends AbstractCancellableFormController {

	private CellLineService cellLineService;
	private DictionaryService dictionaryService;
	private AlleleService modifiedAlleleService;
	private String successViewComingFromMiRequest;
	private ComponentMapService componentMapService;

	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		CellLineForm cellLineForm = (CellLineForm)createCommand();

		boolean comingFromEditMiRequest = ServletRequestUtils.getBooleanParameter(httpServletRequest, "edit_mi_request", false);
		cellLineForm.setComingFromEditMiRequest( comingFromEditMiRequest );

		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");
		Long modifiedAlleleId = ServletRequestUtils.getLongParameter(httpServletRequest, "modifiedAlleleId");
		CellLine cellLine = null;
		if(id != null) {
			cellLine = cellLineService.fetchById(id, componentMapService.getComponentMap("editCellLine"));

		    cellLineForm.setCellLineProviderName(cellLine.getCellLineProvider().getDisplayText());
		    cellLineForm.setCellLineProviderId(cellLine.getCellLineProviderId());

		} else {//else make new cellLine, checking if we have modifiedAlleleId in our query string
				//if coming from Modified Allele input form
			cellLine = buildNewCellLine(modifiedAlleleId);
		}

		cellLineService.inflateCellLineComponents(cellLine);

		cellLineForm.setCellLine(cellLine);
		return cellLineForm;
	}

	//called after databinding, before showForm(), merges Map into ModelAndView
	//to populate our form with form "metadata" -- eg, drop-down select lists, etc.
	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest, Object command, Errors errors) throws Exception {
		CellLineForm cellLineForm = (CellLineForm)command;

		Map<String,Object> formMetaData = new HashMap<String,Object>();
		//..fetch and put any  dropdown list Collections, etc into Map
		Collection<DictionaryEntry> expectedCoatColors =  dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_COAT_COLOUR, cellLineForm.getCellLine().getId());
		Collection<DictionaryEntry> cellLineStrains =  dictionaryService.fetch(DictionaryEnum.MIR_CELL_LINE_STRAIN);
		Collection<DictionaryEntry> genotypes =  dictionaryService.fetch(DictionaryEnum.MOUSE_GENOTYPE);
		Collection<DictionaryEntry> cellLineSources =  dictionaryService.fetch(DictionaryEnum.MIR_CELL_LINE_SOURCE);
		Collection<DictionaryEntry> fundings =  dictionaryService.fetch(DictionaryEnum.APP_FUNDING_SOURCE);

		formMetaData.put("expectedCoatColors", expectedCoatColors);
		formMetaData.put("cellLineStrains", cellLineStrains);
		formMetaData.put("genotypes", genotypes);
		formMetaData.put("cellLineSources", cellLineSources);
		formMetaData.put("fundings", fundings);
		return formMetaData;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {

		CellLineForm cellLineForm = (CellLineForm)command;
		CellLine cellLine = cellLineForm.getCellLine();
		boolean isNewRequest = (cellLine.getId() == null);

		cellLine.setCellLineProviderId(cellLineForm.getCellLineProviderId());
		Long requestId = cellLineService.store( cellLine, RequestUtils.getUser(request) );//pass current user (not necessarily originator!)

		if(cellLineForm.isComingFromEditMiRequest()) {
			return new ModelAndView(successViewComingFromMiRequest + "?cellLineId=" + cellLine.getId());
		}
		else {
			String queryString = "?currentRecordId=" + requestId;

			// for new records filter by status and sort by descending ID
			if(isNewRequest) {
				queryString += "&bypassForm=true&newQuery=true&orderBy=id&orderDesc=true";
			}
			// for existing records, return to the last list page the user was on
			else {
				queryString += "&bypassForm=true&newQuery=false&retrieveFromSession=true";
			}

			return new ModelAndView(getSuccessView() + queryString);
		}
	}

	private CellLine buildNewCellLine(Long modifiedAlleleId) {
		CellLine cellLine = new CellLine();
    	Allele modifiedAllele = null;
    	if(modifiedAlleleId != null) {
    		modifiedAllele = modifiedAlleleService.fetchById(modifiedAlleleId, null);
    	}

    	cellLine.setModifiedAllele(modifiedAllele);
		return cellLine;
	}

	public void setSuccessViewComingFromMiRequest(
			String successViewComingFromMiRequest) {
		this.successViewComingFromMiRequest = successViewComingFromMiRequest;
	}

	public void setModifiedAlleleService(AlleleService modifiedAlleleService) {
		this.modifiedAlleleService = modifiedAlleleService;
	}

	public void setCellLineService(CellLineService cellLineService) {
		this.cellLineService = cellLineService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

}

