/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.controllers;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.oxm.XmlMappingException;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.alleles.GenotypeTest;
import uk.ac.sanger.mig.alleles.services.GenotypeTestService;
import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.forms.MiRequestForm;
import uk.ac.sanger.mig.mirequest.services.MiRequestService;
import uk.ac.sanger.mig.personnel.Person;


public class MiRequestFormController extends AbstractCancellableFormController {

	private MiRequestService miRequestService;
	private GenotypeTestService genotypeTestService;
	private DictionaryService dictionaryService;
	private ProjectLicenceService projectLicenceService;

	private Validator epdFormChangeValidator;

	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {

		MiRequestForm miRequestForm = (MiRequestForm)createCommand();
		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");
		Long cellLineId = ServletRequestUtils.getLongParameter(httpServletRequest, "cellLineId",0L);
		MiRequest miRequest = null;

		if(id != null) {

			miRequest = miRequestService.fetchById(id, ComponentMap.of(MiRequest.Component.ALL_MIREQUEST_COMPONENTS));

		} else {

			miRequest = miRequestService.buildNewMiRequest(cellLineId, RequestUtils.getUser(httpServletRequest));
		}


		Long alleleId = miRequest.getCellLine().getModifiedAlleleId();

		if( alleleId != null) {
			GenotypeTest genoTypeTest = genotypeTestService.fetchGenotypeTestByAlleleId(
				miRequest.getCellLine().getModifiedAlleleId(), null );
			miRequestForm.setGenotypeTestExists( (genoTypeTest != null) );
		}

		miRequestService.inflateMiRequestComponents(miRequest);
		miRequestForm.setMiRequest(miRequest);
		return miRequestForm;
	}

	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {

		MiRequestForm miRequestForm = (MiRequestForm)command;
		MiRequest miRequest = miRequestForm.getMiRequest();
		boolean isNewRequest = (miRequest.getId() == null) ? true : false;

		if(Boolean.FALSE.equals(miRequestForm.getGenotypeTestExists()))
			miRequest.setRequesterProvidedGenotyping(false);

		Long requestId = null;

		if(miRequestForm.isAutomatedEpdMode()) {

			requestId = miRequestService.storeComplex( miRequest, RequestUtils.getUser(request) );

		} else {

			requestId = miRequestService.storeWithColonyAlleleLinked( miRequest, RequestUtils.getUser(request) );
		}

		String queryString = "?currentRecordId=" + requestId;

		// for new records filter by status and sort by descending ID
		if(isNewRequest) {
			queryString += "&bypassForm=true&newQuery=true&orderBy=id&orderDesc=true";
		}
		// for existing records, return to the last list page the user was on
		else {
			queryString += "&bypassForm=true&newQuery=false&retrieveFromSession=true";
		}

		return new ModelAndView(getSuccessView() + queryString);

	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response,
			Object command, BindException errors) throws Exception {

		MiRequestForm miRequestForm = (MiRequestForm)command;
		MiRequest miRequest = miRequestForm.getMiRequest();

		if( miRequestForm.isAutomatedEpdMode() ) {

			epdFormChangeValidator.validate( miRequestForm, errors );

			if( !errors.hasErrors() ) {

				try {

					miRequestService.buildComplex( miRequest, miRequestForm.getEpdId() );
					miRequestForm.setValidEpdModelDerived( true );
				}
				  catch(IOException e) {                handleException(miRequestForm, errors, request, e); }
			      catch(XmlMappingException e) {        handleException(miRequestForm, errors, request, e); }
				  catch(InvalidTeam87InfoException e) { handleException(miRequestForm, errors, request, e); }

			} else {

				miRequestForm.setValidEpdModelDerived( false );
			}
		}
	}

	private void handleException(MiRequestForm miRequestForm, Errors errors,
			HttpServletRequest request, Exception e) {

		miRequestForm.setValidEpdModelDerived( false );//validation failed
	    logger.error( e );
		errors.reject(null, "Unable to find complete EPD information from the Team 87 Web Service for "
				+ miRequestForm.getEpdId()
				+ ". Please contact mig-help@sanger.ac.uk.  In the meantime, you may try another EPD ID.");
	    miRequestService.emailTeam87WebServiceError(
	    		miRequestForm.getEpdId(),
	    		e, errors, RequestUtils.getUser(request));
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {

		return ServletRequestUtils.getBooleanParameter(request, "formChange", false);
	}

	@Override
	protected Map referenceData(HttpServletRequest arg0, Object commandObject, Errors arg2) throws Exception {

		MiRequest miRequest = ((MiRequestForm)commandObject).getMiRequest();
		Person requester = miRequest.getRequester();

		Collection<DictionaryEntry> mirStages =
			dictionaryService.fetch(DictionaryEnum.MIR_EMBRYO_STAGE_DICT);

		Collection<DictionaryEntry> mirEndpoints =
			dictionaryService.fetch(DictionaryEnum.MIR_TARGET_ENDPOINT_DICT);

		Collection<DictionaryEntry> chimeraMateStrains =
			dictionaryService.fetch(DictionaryEnum.MIR_CHIMERA_MATE_STRAIN_DICT);

		Collection<DictionaryEntry> mirWorkflowPipelines =
			dictionaryService.fetch(DictionaryEnum.APP_WORKFLOW_PIPELINE_DICT);

		Collection availablePpls = projectLicenceService.fetchByLicenceHolder(requester.getId(),
				ComponentMap.of(ProjectLicence.Component.RESPONSIBLE_PERSON));

		Map<String,Object> model = new HashMap<String,Object>();
		model.put("availablePpls", availablePpls);
		model.put("mirStages",mirStages);
		model.put("mirEndpoints",mirEndpoints);
		model.put("chimeraMateStrains", chimeraMateStrains);
		model.put("mirWorkflowPipelines",mirWorkflowPipelines);


		return model;
	}


	////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// dependencies ///////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////
	public void setMiRequestService(MiRequestService miRequestService) {
		this.miRequestService = miRequestService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}


	public void setGenotypeTestService(GenotypeTestService genotypeTestService) {
		this.genotypeTestService = genotypeTestService;
	}

	public void setFormChangeValidator(Validator customValidator) {
		this.epdFormChangeValidator = customValidator;
	}

}