/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.charting.DatabaseChartData;
import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.LookupListDao;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.DictionaryDao;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.ParameterizedSqlQuery;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.database.StringRowMapper;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.CellLineSearchBean;
import uk.ac.sanger.mig.mirequest.dao.rowmappers.CellLineListViewRowMapper;
import uk.ac.sanger.mig.mirequest.view.CellLineListViewBean;

public class CellLineDaoSpringImpl extends AbstractDaoSpringImpl<CellLine> implements CellLineDao {

	private CellLineListViewRowMapper cellLineListViewRowMapper;
	private StringRowMapper stringRowMapper;
	private LookupListDao lookupListDao;
	private DictionaryDao dictionaryDao;
	private PaginatedSearchUtils<CellLineListViewBean> paginator;
	private String eucommVectorWellTable;

	public void setLookupListDao(LookupListDao lookupListDao) {
		this.lookupListDao = lookupListDao;
	}

	public void setCellLineListViewRowMapper(
			CellLineListViewRowMapper cellLineListViewRowMapper) {
		this.cellLineListViewRowMapper = cellLineListViewRowMapper;
	}

	public void setDictionaryDao(DictionaryDao dictionaryDao) {
		this.dictionaryDao = dictionaryDao;
	}

	public DictionaryEntry fetchStrainByTeam87Name(String team87Name) {

		Collection<Object> bindVariables = new ArrayList<Object>();

		String sql = "select d.* from  MIR_TEAM87_STRAIN_NAME_MAP a, "+
				      DictionaryEnum.MIR_CELL_LINE_STRAIN.getTableName()+" d " +
					 "where a.cell_line_strain_dict_id = d.id " +
					 "and a.team87_name = ?";

		bindVariables.add(team87Name);

		return dictionaryDao.fetchOneByCustomQuery(sql, bindVariables );

	}


	public boolean isValidEpdId(String epdId) {

		String sql = "select 1 from dual where exists ";
		sql+="(select null from " + eucommVectorWellTable + " where ";
		sql+="eucomm_vector.well.well_name = ?)";

		int count = 0;

		try {

			count = getSimpleJdbcTemplate().queryForInt(sql, epdId);

		} catch(EmptyResultDataAccessException e) {}

		return (count == 1);
	}

	public CellLine fetchByEpdId(String epdId) {

		CellLine cellLine = null;

		String sql = " select * from " + getTableName() + " where clone_name = ? ";

		try {

			cellLine = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(),epdId);

		} catch(IncorrectResultSizeDataAccessException e) {}

		return cellLine;
	}

	public DictionaryEntry fetchMouseStrainByCellLineDictId(Long cellLineDictId) {
		String sql = " select m.* from typ_strain m, mir_cell_line_strain_dict c where c.mouse_strain_dict_id = m.id and c.id = ? ";
		Collection<Object> bindVariables = new ArrayList<Object>();
		bindVariables.add(cellLineDictId);

		return dictionaryDao.fetchOneByCustomQuery(sql, bindVariables);
	}

	public boolean existsByCellLineAndCloneName(String cellLineName, String cloneName) {
		String sql = "select 1 from mir_cell_line where  lower(CELL_LINE_NAME) = lower(?) and lower(CLONE_NAME) = lower(?) and rownum = 1";

		int recordCount = 0;

		try {

			recordCount = getSimpleJdbcTemplate().queryForInt(sql, cellLineName,cloneName);

		} catch(IncorrectResultSizeDataAccessException e) {}

		return (recordCount == 0) ? false : true;
	}

	public ChartData fetchStrainChartData() {
		String sql = "select STRAIN_NAME label, strain_id extra_field, count(*) data0 from mir_cell_line_list_vw group by STRAIN_NAME, strain_id order by data0 ";

		DatabaseChartData data = new DatabaseChartData(1);
		getJdbcTemplate().query(sql, data);

		return data;
	}

	public Collection<CellLineListViewBean> search(CellLineSearchBean searchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		StringBuffer sql = new StringBuffer();
		sql.append(" select * from mir_cell_line_list_vw c where 1=1 ");

		if(searchBean.getSearchString()!=null && !searchBean.getSearchString().equals("") &&
		   !searchBean.getSearchBy().equals("") && !searchBean.getSearchString().contains("Please enter")
		) {
			if(searchBean.getSearchBy().equals("GENE_NAME")){
				sql.append(" and upper(c.gene_name) = ? ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}
			if(searchBean.getSearchBy().equals("ALLELE_NAME")){
				sql.append(" and upper(c.allele_name) = ? ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}
			if(searchBean.getSearchBy().equals("CELL_LINE_NAME")){
				sql.append(" and upper(c.CELL_LINE_NAME) =? ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}

			if(searchBean.getSearchBy().equals("CLONE_NAME")){
				sql.append(" and upper(c.CLONE_NAME) =? ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}
			if(searchBean.getSearchBy().equals("CELL_LINE_PROVIDER_NAME_NO")){
				sql.append(" and upper(c.CELL_LINE_PROVIDER_NAME_NO) like '%'||?||'%' ");
				bindVariables.add(searchBean.getSearchString().toUpperCase());
			}

		}
		else {
			// search by cell line strain id
			if( searchBean.getCellLine().getCellLineStrainDictId() != null) {
				sql.append(" and c.strain_id = ? ");
				bindVariables.add(searchBean.getCellLine().getCellLineStrainDictId());
			}

			// search by coat color
			if( searchBean.getCellLine().getExpectedCoatColorDictId() != null) {
				sql.append(" and c.coat_color_id = ? ");
				bindVariables.add(searchBean.getCellLine().getExpectedCoatColorDictId());

			}

			// search by  genotype
			if( searchBean.getCellLine().getGenotypeDictId() != null) {
				sql.append(" and c.genotype_id = ? ");
				bindVariables.add(searchBean.getCellLine().getGenotypeDictId());
			}

			// search by cell line source
			if( searchBean.getCellLine().getCellLineSourceDictId()!= null) {
				sql.append(" and c.cell_line_source_id = ? ");
				bindVariables.add(searchBean.getCellLine().getCellLineSourceDictId());
			}

			// search by funding
			if( searchBean.getCellLine().getFundingDictId()!= null) {
				sql.append(" and c.funding_id = ? ");
				bindVariables.add(searchBean.getCellLine().getFundingDictId());
			}

		}

		// Order by column and direction as supplied in the search bean
		sql.append(" order by ");
		if(searchBean.getOrderBy() == null || searchBean.getOrderBy().equals("")) {
			sql.append(" id ");
		}
		else {
			sql.append(searchBean.getOrderBy());
		}

		if(searchBean.isOrderDesc()) {
			sql.append(" desc ");
		}

		return paginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, cellLineListViewRowMapper);

	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, CellLine cellLine) {
		sql.addStringParam(cellLine.getCellLineName(), "CELL_LINE_NAME");
		sql.addLongParam(cellLine.getCellLineSourceDictId(), "CELL_LINE_SOURCE_DICT_ID");
		sql.addLongParam(cellLine.getFundingDictId(), "CELL_LINE_FUNDING_DICT_ID");
		sql.addLongParam(cellLine.getCellLineStrainDictId(), "CELL_LINE_STRAIN_DICT_ID");
		sql.addIntParam(cellLine.getTeam87RefId(), "TEAM_87_REF_ID");
		sql.addStringParam(cellLine.getCloneName(), "CLONE_NAME");
		sql.addStringParam(cellLine.getComments(), "COMMENTS");
		sql.addLongParam(cellLine.getModifiedAlleleId(), "MODIFIED_ALLELE_ID");
		sql.addDateParam(cellLine.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(cellLine.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(cellLine.getCheckNumber(), "CHECK_NUMBER");
		sql.addDateParam(cellLine.getEditDate(), "EDIT_DATE");
		sql.addStringParam(cellLine.getEditedBy(), "EDITED_BY");
		sql.addLongParam(cellLine.getExpectedCoatColorDictId(), "EXPECTED_COAT_COLOR_DICT_ID");
		sql.addLongParam(cellLine.getGenotypeDictId(), "GENOTYPE_DICT_ID");
		sql.addLongParam(cellLine.getCellLineProviderId(), "CELL_LINE_PROVIDER_ID");
		sql.addLongParam(cellLine.getId(), "ID");
	}

	public Collection<LookupListElement> lookup(String nameOrCloneId) {
		ParameterizedSqlQuery variables = new ParameterizedSqlQuery();
		nameOrCloneId= nameOrCloneId.replace('*','%');
		nameOrCloneId = nameOrCloneId.toUpperCase();

		String sql = "select c.cell_line_name||', '||c.clone_name display_text, c.id cell_line_id, c.*, genotype_test.genotype_test_id, decode(genotype_test.genotype_test_id,null,'false','true') genotype_test_exists";
		sql += " from  mir_cell_line c,";
		sql += " (select g.id genotype_test_id, g.allele_id allele_id  from typ_modified_allele a, typ_genotype_test g  where a.id = g.allele_id(+) ) genotype_test";
		sql += " where 1=1";
		sql += " and c.modified_allele_id = genotype_test.allele_id(+)";

		if(nameOrCloneId != null && !nameOrCloneId.equals("")) {

			sql += " and (upper(c.cell_line_name) like ? OR upper(c.clone_name) like ?)"; ;
			variables.addStringParam(nameOrCloneId);
			variables.addStringParam(nameOrCloneId);

		}
		sql += " order by lower(c.cell_line_name||', '||c.clone_name) ";

		return lookupListDao.fetchListElements(sql, variables);

	}

	public void setPaginator(PaginatedSearchUtils<CellLineListViewBean> paginator) {
		this.paginator = paginator;
	}

  public Collection<CellLine> fetchCellLinesInMouseFamilyTree(Long mouseId) {
    if (mouseId == null) {
      throw new IllegalArgumentException("Cannot query with a null mouse ID");
    }

    String sql =
      "with heritage as ( "+
      "  select m.id, b.MALE_ID, b.FEMALE_ID, b.EMBRYO_TRANSFER_ID from mts_mouse m "+
      "  join MTS_LITTER l on m.LITTER_ID = l.ID "+
      "  join MTS_BREEDING b on l.BREEDING_ID = b.ID "+
      "  start with m.id = ? "+
      "  connect by m.id = PRIOR b.male_id or m.id = PRIOR b.female_id "+
      ") "+
      "select distinct c.* from MIR_EMBRYO_TRANSFER_PROCEDURE p "+
      "join heritage h on p.ID = h.embryo_transfer_id "+
      "join MIR_SCHEDULED_MICRO_INJECTION s on p.SCHEDULED_MICRO_INJECTION_ID = s.ID "+
      "join MIR_MICRO_INJECTION_REQUEST r on s.MICRO_INJECTION_REQUEST_ID = r.ID " +
      "join MIR_CELL_LINE c on r.CELL_LINE_ID = c.ID";

    return getSimpleJdbcTemplate().query(sql, getRowMapper(), mouseId);
  }

	public Collection<CellLine> fetchCellLinesForColony(Long colonyInfoId) {
		String sql = "select distinct cl.* from MIR_CELL_LINE cl" +
			" join MIR_MICRO_INJECTION_REQUEST mir on cl.id = mir.cell_line_id" +
			" where mir.colony_info_id = ?";

		return super.fetchCollection(sql, colonyInfoId);
	}

	public void setEucommVectorWellTable(String eucommVectorWellTable) {
		this.eucommVectorWellTable = eucommVectorWellTable;
	}

	public void setStringRowMapper(StringRowMapper stringRowMapper) {
		this.stringRowMapper = stringRowMapper;
	}




}