/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.dao;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.charting.DatabaseChartData;
import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.MiRequestSearchBean;
import uk.ac.sanger.mig.mirequest.dao.rowmappers.MiRequestListViewRowMapper;
import uk.ac.sanger.mig.mirequest.view.MiRequestListViewBean;


public class MiRequestDaoSpringImpl extends AbstractDaoSpringImpl<MiRequest> implements MiRequestDao {

	private MiRequestListViewRowMapper miRequestListViewRowMapper;
	private PaginatedSearchUtils<MiRequestListViewBean> paginator;

	public ChartData fetchStatusChartData(Object... statusIds) {

		List<String> list = Arrays.asList( new String[statusIds.length] ) ;

		Collections.fill(list, "?");

		String sql = "select external_status_name label, external_status_dict_id extra_field, count(*) data0 " +
		"from mir_request_list_vw where external_status_dict_id in (" + StringUtils.join(list.iterator(), ",") + ") " +
		" group by external_status_name, external_status_dict_id order by data0";

		DatabaseChartData data = new DatabaseChartData(1);
		getSimpleJdbcTemplate().query(sql, data, statusIds);
		return data;
	}

	public Collection<MiRequestListViewBean> search(MiRequestSearchBean searchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		StringBuffer sql = new StringBuffer();
		sql.append(" select * from mir_request_list_vw c where 1=1 ");

		//search by endpoint
		if( searchBean.getMiRequest().getEndpointDictId() != null ) {
			sql.append(" and c.endpoint_id = ? ");
			bindVariables.add(searchBean.getMiRequest().getEndpointDictId());
		}

		//search by donor cell line strain
		if( searchBean.getDonorCellLineStrainDict() != null && searchBean.getDonorCellLineStrainDict().getId() != null
				&& searchBean.getDonorCellLineStrainDict().getId() > 0 ) {
			sql.append(" and c.strain_id = ? ");
			bindVariables.add(searchBean.getDonorCellLineStrainDict().getId());
		}

		//search by chimera mate strain
		if( searchBean.getMiRequest().getChimeraMateStrainDictId() != null) {
			sql.append(" and c.chimera_mate_strain_id = ? ");
			bindVariables.add(searchBean.getMiRequest().getChimeraMateStrainDictId());
		}

		//search by cell line name
		if( searchBean.getCellLineName()!= null) {
			sql.append(" and upper(c.cell_line_name) = upper(?) ");
			bindVariables.add(searchBean.getCellLineName());
		}

		//search by gene name
		if( searchBean.getGeneName()!= null) {
			sql.append(" and upper(c.gene_name) = upper(?) ");
			bindVariables.add(searchBean.getGeneName());
		}

		//search by stage
		if( searchBean.getMiRequest().getDonorEmbryoStageDictId() != null) {
			sql.append(" and c.embryo_stage_id = ? ");
			bindVariables.add(searchBean.getMiRequest().getDonorEmbryoStageDictId());
		}


		// search by new external request status
		if( searchBean.getMiRequest().getExternalStatusDictId() != null) {
			sql.append(" and c.external_status_dict_id = ? ");
			bindVariables.add(searchBean.getMiRequest().getExternalStatusDictId());

		}

		//	search by workflow
		if( searchBean.getMiRequest().getWorkflowPipelineDictId() != null ) {
			sql.append(" and c.workflow_pipeline_id = ?");
			bindVariables.add(searchBean.getMiRequest().getWorkflowPipelineDictId());
		}

		//search by requester
		if( searchBean.getMiRequest().getRequesterId() != null
				&& searchBean.getMiRequest().getRequesterId() > 0 ) {
			sql.append(" and c.requester_id = ?");
			bindVariables.add(searchBean.getMiRequest().getRequesterId());
		}

		sql.append(" order by ");
		if(searchBean.getOrderBy() == null || searchBean.getOrderBy().equals("")) {
			sql.append(" id ");
		} else {
			sql.append(searchBean.getOrderBy());
		}

		if(searchBean.isOrderDesc()) {
			sql.append(" desc ");
		}

		return paginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, miRequestListViewRowMapper);

	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, MiRequest mirRequest) {
		sql.addLongParam(mirRequest.getCellLineId(), "CELL_LINE_ID");
		sql.addLongParam(mirRequest.getEndpointDictId(), "ENDPOINT_DICT_ID");
		sql.addDateParam(mirRequest.getEditDate(), "EDIT_DATE");
		sql.addStringParam(mirRequest.getEditedBy(), "EDITED_BY");
		sql.addDateParam(mirRequest.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(mirRequest.getRequesterId(), "REQUESTER_ID");
		sql.addLongParam(mirRequest.getProjectLicenceId(), "PROJECT_LICENCE_ID");
		sql.addLongParam(mirRequest.getDonorPlugCount(), "DONOR_PLUG_COUNT");
		sql.addLongParam(mirRequest.getChimeraMateStrainDictId(), "CHIMERA_MATE_STRAIN_DICT_ID");
		sql.addBooleanParam(mirRequest.isGenotypingRequired(), "GENOTYPING_REQUIRED");
		sql.addLongParam(mirRequest.getDonorEmbryoStageDictId(), "DONOR_EMBRYO_STAGE_DICT_ID");
		sql.addLongParam(mirRequest.getWorkflowPipelineDictId(), "WORKFLOW_PIPELINE_DICT_ID");
		sql.addLongParam(mirRequest.getExternalStatusDictId(), "EXTERNAL_STATUS_DICT_ID");
		sql.addStringParam(mirRequest.getStatusComments(), "STATUS_COMMENTS");
		sql.addLongParam(mirRequest.getCheckNumber(), "CHECK_NUMBER");
		sql.addStringParam(mirRequest.getFundingSource(), "FUNDING_SOURCE");
		sql.addBooleanParam(mirRequest.isRequesterProvidedGenotyping(), "REQUESTER_PROVIDED_GENOTYPING");
		sql.addLongParam(mirRequest.getColonyInfoId(), "COLONY_INFO_ID");
		sql.addLongParam(mirRequest.getId(), "ID");
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////// dependencies ///////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////////
	public void setPaginator(PaginatedSearchUtils<MiRequestListViewBean> paginator) {
		this.paginator = paginator;
	}

	public void setMiRequestListViewRowMapper(
			MiRequestListViewRowMapper miRequestListViewRowMapper) {
		this.miRequestListViewRowMapper = miRequestListViewRowMapper;
	}

}
