/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.services;

import java.io.IOException;
import java.util.Collection;

import org.springframework.oxm.XmlMappingException;

import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.CellLineSearchBean;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.mirequest.view.CellLineListViewBean;
import uk.ac.sanger.mig.personnel.Person;

public interface CellLineService {




	/**
	 * Tests whether the given EPD identifier exists in storage.
	 *
	 * @param epdId
	 * @return
	 */
	public boolean isValidEpdId(String epdId);

	/**
	 * Stores a CellLine object along with its Allele component,
	 * setting the CellLine with the Allele's ID.
	 *
	 * @param celLine
	 * @param person
	 * @return CellLine id
	 * @throws IllegalArgumentException if CellLine is null.
	 * @throws UnattachedComponentException if Allele is null -- i.e., not "attached"
	 * to the CellLine.
	 *
	 */
	public Long storeComplex(CellLine celLine, Person person);

	/**
	 * Returns the Cell Line (with Allele and MigGene attached) based
	 * on an EPD identifier provided to Team 87's web service.
	 *
	 * If an XmlMappingException is thrown, the Team 87 response will
	 * be logged for corrective analysis of the externally malfomed XML.
	 *
	 * Cell Line name will be set to its short allele symbol followed
	 * by defaultCellLineNameType (e.g., 'Knockout'), followed by the
	 * design instance number if it can be derived from the official
	 * allele name (e.g., 2 in the case of 'tm2aWtsi')
	 *
	 * @param epdId
	 * @return
	 * @throws IOException if connection problem fetching Team 87 EPD info.
	 * @throws XmlMappingException if XML-to-object mapping problem
	 * @throws InvalidTeam87InfoException if Team87 data missing or invalid
	 */
	public CellLine deriveEpdCellLine(String epdId)
				throws IOException, XmlMappingException, InvalidTeam87InfoException;

	/**
	 * Stores CellLine as record in database and returns the
	 * record's primary key
	 *
	 * @param cellLine
	 * @return
	 */
	public Long store(CellLine cellLine, Person person);

	/**
	 * Returns cellLine for a given id that maps to primary key of a record in a
	 * database table.
	 *
	 * @param id
	 * @return
	 */
	public CellLine fetchById(Long id, ComponentMap componentMap);

	public boolean existsByCellLineAndCloneName(String cellLineName, String cloneName);

	/**
	 *
	 * @param cellLineSearchBean
	 * @return
	 */
	public Collection<CellLineListViewBean> search(CellLineSearchBean cellLineSearchBean);
	public Collection<LookupListElement> lookup(String name);

	public DictionaryEntry fetchMouseStrainByCellLineDictId(Long cellLineDictId);

	public ChartData fetchStrainChartData();

	public void inflateCellLineComponents(CellLine cellLine);

	/**
	 * Fetch all Cell Lines that are known for the ancestors of a mouse
	 *
	 * (i.e. where a mouse has ancestors that were produced by microinjection, this will find the cell lines)
	 *
	 * @param mouseId
	 * @param componentMap
	 * @return
	 */
	public Collection<CellLine> fetchCellLinesInMouseFamilyTree(Long mouseId, ComponentMap componentMap);


	/**
	 * Fetch all cell lines that can be linked to a colony, via a microinjection request
	 *
	 * @param colonyInfoId
	 * @param componentMap
	 * @return
	 */
	public Collection<CellLine> fetchCellLinesForColony(Long colonyInfoId, ComponentMap componentMap);

}