/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.services;
import java.io.IOException;
import java.net.URL;
import java.util.Collection;

import javax.xml.transform.stream.StreamSource;

import org.apache.commons.lang.StringUtils;
import org.springframework.oxm.Unmarshaller;
import org.springframework.oxm.XmlMappingException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ws.client.core.WebServiceTemplate;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.alleles.services.AlleleServiceImpl;
import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.services.MigGeneService;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.CellLineSearchBean;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.mirequest.dao.CellLineDao;
import uk.ac.sanger.mig.mirequest.view.CellLineListViewBean;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class CellLineServiceImpl extends AbstractServiceImpl<CellLine> implements CellLineService {


	private CellLineDao cellLineDao;
	private MigGeneService migGeneService;
	private PersonService personService;
	private DictionaryService dictionaryService;
	private AlleleService modifiedAlleleService;
	private WebServiceTemplate webServiceTemplate;
	private ComponentMapService componentMapService;
    private Long defaultCellLineSourceDictId;//T87
    private Long defaultGenotypeDictId; //undetermined
    private Long defaultCellLineProviderId; //skarnes
    private String defaultCellLineNameType;//Knockout
    private String defaultComments;

	public boolean isValidEpdId(String epdId) {

		return cellLineDao.isValidEpdId( epdId );
	}

    @Transactional
	public Long storeComplex(CellLine cellLine, Person person) {

		if( cellLine == null ) {
			throw new IllegalArgumentException("CellLine cannot be null.");
		}

		Allele allele = cellLine.getModifiedAllele() ;
		if( allele == null ) {
			throw new UnattachedComponentException(CellLine.Component.ALLELE);
		}

		MigGene gene = allele.getMigGene();

		//store gene if we have (NOTE: gene can be null)
		Long migGeneId = null;
		if( gene != null ) {
			migGeneId = migGeneService.store(gene, person);
		}

		//store allele
		allele.setMigGeneId(migGeneId);
		Long alleleId = modifiedAlleleService.store(allele, person);

		//store CellLine
		cellLine.setModifiedAlleleId( alleleId );
		return store(cellLine, person);
	}


	public CellLine deriveEpdCellLine(String epdId)
				throws IOException, XmlMappingException, InvalidTeam87InfoException  {

		CellLine team87CellLine        = fetchTeam87CellLine( epdId );
		String team87GeneSymbol 	   = team87CellLine.getModifiedAllele().getTeam87GeneSymbol();
		String team87AlleleSuperscript = team87CellLine.getModifiedAllele().getOfficialNameSuperscript();

		CellLine migCellLine = fetchByEpdId(epdId, componentMapService.getComponentMap("editCellLine"));

		CellLine workingCellLine  = null;

		if(migCellLine != null ) {

			logger.debug("Found existing EPD CellLine in MIG: " + epdId);
			workingCellLine = migCellLine;

		} else {

			logger.debug("Using Team87 info. to create new EPD CellLine: " + epdId);
			populateAllele(       team87CellLine );
			populateDictionaries( team87CellLine );
			populateFields(       team87CellLine );
			workingCellLine   =   team87CellLine;
		}

		refreshTeam87Naming(workingCellLine, team87GeneSymbol, team87AlleleSuperscript);

		return workingCellLine;
	}

	public void setWebServiceTemplate(WebServiceTemplate webServiceTemplate) {
		this.webServiceTemplate = webServiceTemplate;
	}

	public void setCellLineDao(CellLineDao cellLineDao) {
		this.cellLineDao = cellLineDao;
	}

	public ChartData fetchStrainChartData() {
		return cellLineDao.fetchStrainChartData();
	}

	public void setModifiedAlleleService(AlleleService modifiedAlleleService) {
		this.modifiedAlleleService = modifiedAlleleService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public Long store(CellLine cellLine, Person person) {
		return cellLineDao.store(cellLine, person);
	}

	public CellLine fetchById(Long id, ComponentMap componentMap) {
		CellLine cellLine = cellLineDao.fetchById(id);

		if(cellLine != null) {
			attachComponents(cellLine, componentMap);
		}

		return cellLine;
	}

	public boolean existsByCellLineAndCloneName(String cellLineName, String cloneName) {
		return cellLineDao.existsByCellLineAndCloneName(cellLineName,cloneName);
	}
	public Collection<CellLineListViewBean> search(CellLineSearchBean cellLineSearchBean) {
		return cellLineDao.search(cellLineSearchBean);
	}

	public Collection<LookupListElement> lookup(String name) {

		return cellLineDao.lookup(name);
	}



	@Override
	protected void attachComponents(CellLine cellLine,
			ComponentMap requiredComponents) {
		if (cellLine == null) {
			throw new RuntimeException("Cannot attach components to null cellLine");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(CellLine.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(CellLine.Component.ALLELE)) {
				cellLine.setModifiedAllele(fetchModifiedAllele(cellLine, requiredComponents.get(CellLine.Component.ALLELE)));
			}

			if (fetchAll || requiredComponents.containsKey(CellLine.Component.PERSON)) {
				cellLine.setCreator(fetchCreator(cellLine, requiredComponents.get(CellLine.Component.PERSON)));
			}

			if(fetchAll || requiredComponents.containsKey(CellLine.Component.DEFAULT_CELL_LINE_PROVIDER)) {
				cellLine.setCellLineProvider(fetchCellLineProvider(cellLine,requiredComponents.get(CellLine.Component.DEFAULT_CELL_LINE_PROVIDER)));
			}
		}

	}

	private CellLine fetchByEpdId(String epdId, ComponentMap requiredComponents) {

		CellLine cellLine = cellLineDao.fetchByEpdId(epdId);

		if(cellLine != null) {
			attachComponents(cellLine, requiredComponents);
		}

		return cellLine;

	}

    private CellLine fetchTeam87CellLine(String epdId)
    	throws IOException, XmlMappingException, InvalidTeam87InfoException {

    	CellLine cellLine = null;

		String urlString = webServiceTemplate.getDefaultUri() + epdId;;
		//webServiceTemplate.getDefaultUri() + epdId;
		//use http://localhost:8080/trunk/xsd/mig-mir-info.xml for testing
		URL url = new URL( urlString );
		StreamSource streamSource = new StreamSource( url.openStream() );
		Unmarshaller unmarshaller = webServiceTemplate.getUnmarshaller();

		synchronized ( unmarshaller ) {
			//should sync on unmarshaller for thread safety
			cellLine = (CellLine)(unmarshaller.unmarshal( streamSource ));
		}

		checkRequiredFields(cellLine, epdId);
		return cellLine;
    }

    private void checkRequiredFields(CellLine cellLine, String epdId)
    		throws InvalidTeam87InfoException{

    	if(  cellLine == null ) {

			throw new InvalidTeam87InfoException("Team 87 EPD: "
					+epdId+" - A CellLine could not be unmarshalled for this EDP." );
		}

		if(  cellLine.getModifiedAllele() == null ) {

			throw new InvalidTeam87InfoException("Team 87 EPD: "
				+epdId+" - An Allele could not be unmarshalled for this EDP." );

		}

		String alleleSuperscript = cellLine.getModifiedAllele().getOfficialNameSuperscript();

		if( StringUtils.isBlank( alleleSuperscript )
				|| alleleSuperscript.equalsIgnoreCase("not available yet")) {

			throw new InvalidTeam87InfoException("Team 87 EPD: "
					+epdId+" - Required Allele superscript not provided for this EDP." );
		}

		if( StringUtils.isBlank( cellLine.getModifiedAllele().getTeam87GeneSymbol() )) {

			throw new InvalidTeam87InfoException(
					"Team 87 EPD: "+epdId+" - Required Gene Symbol not provided for this EDP." );
		}
    }

	private Person fetchCellLineProvider(CellLine cellLine, ComponentMap requiredComponents) {
		if(cellLine.getCellLineProviderId() != null) {
			return personService.fetchById(cellLine.getCellLineProviderId(), requiredComponents);
		}

		return null;
	}

	private Person fetchCreator(CellLine cellLine, ComponentMap requiredComponents) {
		if (cellLine.getCreatorId() != null) {
			return personService.fetchById(cellLine.getCreatorId(),requiredComponents);
		} else {
			return null;
		}

	}

	private void refreshTeam87Naming(CellLine cellLine, String team87GeneSymbol,
			String team87AlleleSuperscript) {

		Allele allele = cellLine.getModifiedAllele();
		MigGene gene  = allele.getMigGene();

		if( gene != null ) {
			refreshGeneName(gene, team87GeneSymbol );
		}

		refreshAlleleName(allele, team87GeneSymbol, team87AlleleSuperscript);

		refreshCellLineName(cellLine, team87GeneSymbol, team87AlleleSuperscript);
	}

	private void refreshCellLineName(CellLine cellLine, String team87GeneSymbol,
			String team87AlleleSuperscript) {

		String shortSymbol = cellLine.getModifiedAllele().getShortAlleleSymbol();

		cellLine.setCellLineName( shortSymbol + " " + defaultCellLineNameType);

		if( team87AlleleSuperscript.matches( AlleleServiceImpl.SUPERSCRIPT_PATTERN ) ) {

			String designCountStr = team87AlleleSuperscript.replaceFirst(".*(\\d+).*", "$1");

			if( StringUtils.isNumeric(designCountStr) ) {

				int designCount = Integer.parseInt( designCountStr );

				cellLine.setCellLineName( cellLine.getCellLineName() + " " +  designCount);
			}
		}

	}

	private void refreshGeneName(MigGene gene, String team87GeneSymbol)  {

		if( team87GeneSymbol != null ) {

			gene.setPrimaryName( team87GeneSymbol );
			gene.setPrimaryNameSource( "Team 87" );
		}
	}

	private void refreshAlleleName(Allele allele, String team87GeneSymbol,
			String team87AlleleSuperscript) {

		allele.setTeam87GeneSymbol( team87GeneSymbol );
		allele.setShortAlleleSymbol( team87GeneSymbol );
		allele.setOfficialNameSuperscript(team87AlleleSuperscript);

		String truncatedSuperscript = "";

		if( team87AlleleSuperscript.matches( AlleleServiceImpl.SUPERSCRIPT_PATTERN ) ) {

			truncatedSuperscript = team87AlleleSuperscript.replaceFirst(
					AlleleServiceImpl.SUPERSCRIPT_PATTERN, "$1");
		}

		allele.setInternalName( allele.getShortAlleleSymbol() +  truncatedSuperscript);
	}

	private void populateAllele(CellLine cellLine) throws InvalidTeam87InfoException   {

		Allele workingAllele
			= modifiedAlleleService.deriveEpdAllele( cellLine.getModifiedAllele() );

		cellLine.setModifiedAllele( workingAllele );

	}

	private void populateDictionaries(CellLine cellLine) throws InvalidTeam87InfoException {
		//resolve dictionary names to PKs
		DictionaryEntry fundingDict = dictionaryService.fetchEntry(
				DictionaryEnum.APP_FUNDING_SOURCE,
				cellLine.getFundingDict().getName(), true);

		if( fundingDict != null ) {
			cellLine.setFundingDict(fundingDict);
			cellLine.setFundingDictId( fundingDict.getId() );
		}

		//hack until Team 87 strains synced
		String strainName = cellLine.getCellLineStrain().getName();
		DictionaryEntry strainDict = null;

		if( strainName != null ) {

			strainDict = cellLineDao.fetchStrainByTeam87Name( strainName );

			if( strainDict != null ) {
				cellLine.setCellLineStrain( strainDict );
				cellLine.setCellLineStrainDictId( strainDict.getId() );
			}

		}

		if( cellLine.getCellLineStrainDictId() == null ) {
			throw new InvalidTeam87InfoException("Team 87 EPD: "
					+cellLine.getCloneName()+" - A MIG strain could not be found for Team 87 strain name " + strainName );
		}

		DictionaryEntry coatDict = dictionaryService.fetchEntry(
				DictionaryEnum.MOUSE_COAT_COLOUR,
				cellLine.getExpectedCoatColor().getName(), true);

		if( coatDict != null ) {
			cellLine.setExpectedCoatColor( coatDict );
			cellLine.setExpectedCoatColorDictId( coatDict.getId() );
		}
	}

    private void populateFields(CellLine cellLine) {

		String comments = ( cellLine.getComments() == null ) ? "" : cellLine.getComments();

		cellLine.setComments( comments + " " + defaultComments );

		//set genotype
		if( cellLine.getGenotypeDictId() == null ) {

			cellLine.setGenotypeDictId( defaultGenotypeDictId );
		}

		//set source
		if( cellLine.getCellLineSourceDictId() == null ) {

			cellLine.setCellLineSourceDictId( defaultCellLineSourceDictId );
		}

		//set provider
		if( cellLine.getCellLineProviderId() == null ) {

			cellLine.setCellLineProviderId( defaultCellLineProviderId );
		}


    }

	private Allele fetchModifiedAllele(CellLine cellLine, ComponentMap requiredComponents) {
		if (cellLine.getModifiedAlleleId() != null) {
			return modifiedAlleleService.fetchById(cellLine.getModifiedAlleleId(), requiredComponents);
		} else
			return null;
	}

	public DictionaryEntry fetchMouseStrainByCellLineDictId(Long cellLineDictId) {
		return cellLineDao.fetchMouseStrainByCellLineDictId(cellLineDictId);
	}

	public void inflateCellLineComponents(CellLine cellLine) {
		if (cellLine == null) {
			throw new RuntimeException("CellLine cannot be null when inflating components.");
		}

		if(cellLine.getCellLineStrain() == null) {
	    	cellLine.setCellLineStrain(new DictionaryEntry());
	    }

	    if(cellLine.getGenotype() == null) {
	    	cellLine.setGenotype(new DictionaryEntry());
	    }

	    if(cellLine.getExpectedCoatColor() == null) {
	    	cellLine.setExpectedCoatColor(new DictionaryEntry());
	    }

	    if(cellLine.getCellLineSource() == null) {
	    	cellLine.setCellLineSource(new DictionaryEntry());
	    }

	    if(cellLine.getModifiedAllele() == null) {
	    	Allele modifiedAllele = new Allele();
	    	modifiedAllele.setMigGene(new MigGene());
	    	cellLine.setModifiedAllele(modifiedAllele);

	    }

	    if(cellLine.getModifiedAllele() != null && cellLine.getModifiedAllele().getMigGene() == null) {
	    	cellLine.getModifiedAllele().setMigGene(new MigGene());
	    }

	    if(cellLine.getCellLineProvider() == null) {
	    	cellLine.setCellLineProvider(new Person());
	    }

	}

	public Collection<CellLine> fetchCellLinesInMouseFamilyTree(Long mouseId, ComponentMap requiredComponents) {
	  Collection<CellLine> cellLines = cellLineDao.fetchCellLinesInMouseFamilyTree(mouseId);
	  attachComponents(cellLines, requiredComponents);
	  return cellLines;
	}

	public void setDefaultCellLineProviderId(Long defaultCellLineProviderId) {
		this.defaultCellLineProviderId = defaultCellLineProviderId;
	}

	public void setDefaultCellLineSourceDictId(Long defaultCellLineSourceDictId) {
		this.defaultCellLineSourceDictId = defaultCellLineSourceDictId;
	}

	public void setDefaultGenotypeDictId(Long defaultGenotypeDictId) {
		this.defaultGenotypeDictId = defaultGenotypeDictId;
	}


	public void setDefaultCellLineNameType(String defaultCellLineNameType) {
		this.defaultCellLineNameType = defaultCellLineNameType;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setDefaultComments(String defaultComments) {
		this.defaultComments = defaultComments;
	}


	public Collection<CellLine> fetchCellLinesForColony(Long colonyInfoId, ComponentMap componentMap) {
		Collection<CellLine> cellLines = cellLineDao.fetchCellLinesForColony(colonyInfoId);
		attachComponents(cellLines, componentMap);
		return cellLines;
	}

	public void setMigGeneService(MigGeneService migGeneService) {
		this.migGeneService = migGeneService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	/*
	private void handleXmlMappingException(XmlMappingException e, URL url, boolean rethrow)
		throws IOException {

		StringWriter  webServiceResponse = new StringWriter();
		IOUtils.copy( url.openStream(), webServiceResponse );
		logger.error("XmlMappingException -- Team 87 Web Service Response: " + webServiceResponse);
		logger.error( e );

		if( rethrow ) {
			throw e;
		}
	} */


}
