/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.services;

import java.io.IOException;
import java.util.Collection;

import org.springframework.oxm.XmlMappingException;
import org.springframework.validation.Errors;

import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.MiRequestSearchBean;
import uk.ac.sanger.mig.mirequest.view.MiRequestListViewBean;
import uk.ac.sanger.mig.personnel.Person;

public interface MiRequestService {

	/**
	 * Biz logic: Subset of WT Strains available to mate with F0 chimeras
	 *
	 */
	public enum ChimeraMateStrain {

		GPAA(5L),
		CBLJ(1L),
		CALB(3L);

		private Long dictId;

		private ChimeraMateStrain(Long dictId) {
			this.dictId = dictId;
		}

		public Long getDictId() {
			return this.dictId;
		}
	}


	/**
	 * Stores an MiRequest and links its ColonyInfo and CellLine Allele
	 * components via ColonyAllele.
	 *
	 * @param miRequest
	 * @param user
	 * @return
	 * @throws UnattachedComponentException if ColonyInfo or CellLine not attached.
	 */
	public Long storeWithColonyAlleleLinked(MiRequest miRequest, Person user);

	public boolean emailTeam87WebServiceError(String epdId, Exception exception,
			Errors errors, Person user);

	/**
	 * Populates and attaches to the MiRequest both CellLine (with Allele)
	 * and ColonyInfo objects based on Team 87 EPD information and default
	 * assumptions injected via Spring XML at start up.
	 *
	 * @param miRequest
	 * @param epdId
	 * @throws IOException if connection problem fetching Team 87 EPD information
	 * @throws XmlMappingException if XML-to-object mapping problem fetching
	 * Team 87 EPD information
	 * @throws InvalidTeam87InfoException if Team 87 EPD information is properly mapped but
	 * is invalid--i.e., if allele is blank, gene id is invalid, or official allele
	 * name is blank.
	 */
	public void buildComplex(MiRequest miRequest, String epdId)
		throws IOException, XmlMappingException, InvalidTeam87InfoException;

	/**
	 * Stores a MiRequest object including CellLine
	 * and ColonyInfo sub-components.
	 *
	 * In addition, creates and stores a CollonyAllele object
	 * linking the CellLine alleleId with ColonyInfo id.
	 *
	 * @param miRequest
	 * @param person
	 * @return MiRequest id
	 * @throws UnattachedComponentException if either CellLine
	 * or ColonyInfo is null -- i.e., not "attached".
	 */
	public Long storeComplex(MiRequest miRequest, Person person);

	/**
	 * Returns a new empty MiRequest object for a particular CellLine. If CellLine
	 * ID does not exist, an empty CellLine object is attached to the MiRequest,
	 * otherwise the relevant CellLine object is retrieved from the database and
	 * attached.  Default finding source is "Sanger", status is initialized
	 * to "Requested", and Requester is set to the user.
	 * @param cellLineId
	 * @param user
	 * @return
	 */
	public MiRequest buildNewMiRequest(Long cellLineId, Person user);

	/**
	 * Fetched active (and non-legacy status) status counts in MI processing
	 * @return
	 */
	public ChartData fetchNonCancelledStatusChartData();

	/**
	 * Fetches full MiRequest object hierarchy.
	 * @param id
	 * @return
	 */
	public MiRequest fetchWithAllComponentsById(Long id);

	/**
	 * Stores MiRequest and returns its id.
	 * @param miRequest
	 * @return
	 */
	public Long store(MiRequest miRequest, Person person);

	/**
	 * Returns a MiRequest object for a given id.
	 * @param id
	 * @return
	 */
	public MiRequest fetchById(Long id, ComponentMap componentMap);


	public Collection<MiRequestListViewBean> search(MiRequestSearchBean miRequestSearchBean);

	/**
	 * Cancels a Request provided it has no active (non-cancelled) jobs.
	 *
	 * If Request has any active jobs, the jobs will be cancelled and
	 * the Request re-requested, not cancelled.  This mechanism allows
	 * user to cancel undesired jobs from the request and re-schedule
	 * new jobs afresh without having to create a new request.
	 *
	 * @param miRequest
	 * @param user
	 * @return
	 */
	public Long cancelRequest(MiRequest miRequest, Person user);

	public Long completeRequest(MiRequest miRequest, Person user);

	public Long processRequest(MiRequest miRequest, Person user);

	public Long awaitMapTest(MiRequest miRequest, Person user);

	public Long delayRequest(MiRequest miRequest, Person user);

	public Long reRequest(MiRequest miRequest, Person user);

	public void inflateMiRequestComponents(MiRequest miRequest);
}