/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.services;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.oxm.XmlMappingException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.Errors;

import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyAlleleService;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.DictionaryEntryEnum;
import uk.ac.sanger.mig.common.MailHelper;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.services.ScheduledMicroInjectionService;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.MiRequestSearchBean;
import uk.ac.sanger.mig.mirequest.MiRequest.ExternalStatusEnum;
import uk.ac.sanger.mig.mirequest.dao.MiRequestDao;
import uk.ac.sanger.mig.mirequest.view.MiRequestListViewBean;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.dao.MailingListDao;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class MiRequestServiceImpl extends AbstractServiceImpl<MiRequest> implements MiRequestService {

	private MiRequestDao miRequestDao;
	private PersonService personService ;
	private ProjectLicenceService projectLicenceService ;
	private CellLineService cellLineService;
	private ScheduledMicroInjectionService  scheduledMicroInjectionService;
	private DictionaryService dictionaryService;
	private ColonyInfoService colonyInfoService;
	private ColonyAlleleService colonyAlleleService;


	private MailHelper mailHelper;
	private MailingListDao mailingListDao;

	@Transactional
	public Long storeWithColonyAlleleLinked(MiRequest miRequest, Person user) {

		Long id = store(miRequest, user);

		CellLine cellLine = fetchCellLine(miRequest, null);

		if (cellLine.getModifiedAlleleId() != null) {
			colonyAlleleService.linkColonyAllele(miRequest.getColonyInfoId(),
					cellLine.getModifiedAlleleId(), user);
		}

		return id;
	}

	public boolean emailTeam87WebServiceError(String epdId, Exception exception, Errors errors, Person user) {

		    Map<String, Object>model = new HashMap<String, Object>();
		    model.put("exception", exception);
		    model.put("errors", errors);
		    model.put("user", user);
		    model.put("epdId", epdId);
		    model.put("server", mailHelper.getServerInfo().getHostName());

		    Collection<String> recipients = mailingListDao.fetchRecipientEmailsByListId(MailingListDao.TEAM87_WEB_SERVICE_ERROR);
		    //Collection<String> recipients = new ArrayList<String>();//mailingListDao.fetchRecipientEmailsByListId(MailingListDao.ANIMAL_REQUEST_MAILING_LIST);
		    recipients.add("ae1@sanger.ac.uk");

		    return mailHelper.sendMail("team87_web_service_error", model, recipients.toArray(new String[]{}));
	}

	public void buildComplex(MiRequest miRequest, String epdId)
					throws IOException, XmlMappingException, InvalidTeam87InfoException {

		CellLine   cellLine   = cellLineService.deriveEpdCellLine(         epdId    );
		ColonyInfo colonyInfo = colonyInfoService.deriveDefaultColonyInfo( cellLine );

		miRequest.setCellLine(   cellLine   );
		miRequest.setColonyInfo( colonyInfo );
	}



	@Transactional
	public Long storeComplex(MiRequest miRequest, Person person) {

		if( miRequest == null ) {
			throw new IllegalArgumentException("MiRequest cannot be null.");
		}

		CellLine cellLine = miRequest.getCellLine();
		ColonyInfo colonyInfo = miRequest.getColonyInfo();

		if( cellLine == null ) {
			throw new UnattachedComponentException(MiRequest.Component.CELL_LINE);
		}

		if( colonyInfo == null ) {
			throw new UnattachedComponentException(MiRequest.Component.COLONY);
		}

		//store CellLine (and Allele via indirection),
		//   ColonyInfo and, finally, the MiRequest itself
		Long cellLineId = cellLineService.storeComplex( cellLine, person );

		miRequest.setCellLineId( cellLineId );

		//Colony PPL defaults to MiRequester selected PPL
		colonyInfo.setProjectLicenceId( miRequest.getProjectLicenceId() );

		Long colonyInfoId = colonyInfoService.store( colonyInfo, person );

		miRequest.setColonyInfoId( colonyInfoId );

		//need to store mir, update ColonyInfo with RederivationMethod based on id
		Long id =  store( miRequest, person );

		colonyInfo.setRederivationMethod("MIR " + id);

		colonyInfoService.store( colonyInfo, person );

		//link ColonyInfo to Allele via ColonyAllele and store
		colonyAlleleService.linkColonyAllele(colonyInfoId,
				cellLine.getModifiedAlleleId(), person);

		return id;
	}

	public MiRequest buildNewMiRequest(Long cellLineId, Person user) {

		MiRequest miRequest = new MiRequest();

		CellLine cellLine = cellLineService.fetchById(cellLineId,
					ComponentMap.of(CellLine.Component.ALL_COMPONENTS));

		cellLine = (cellLine == null) ? new CellLine() : cellLine;

		miRequest.setCellLineId( cellLineId );
		miRequest.setCellLine( cellLine ) ;

		miRequest.setExternalStatusDictId(ExternalStatusEnum.REQUESTED.getId());   //new always requested
		miRequest.setRequester(user);
		miRequest.setRequesterId(user.getId());
		miRequest.setFundingSource("Sanger"); //default
		return miRequest;

	}

	public ChartData fetchNonCancelledStatusChartData() {
		Object[] statusIds = new Long[] {
				MiRequest.ExternalStatusEnum.REQUESTED.getId(),
				MiRequest.ExternalStatusEnum.COMPLETED.getId(),
				MiRequest.ExternalStatusEnum.AWAITING_MAP_TEST.getId(),
				MiRequest.ExternalStatusEnum.DELAYED.getId(),
				MiRequest.ExternalStatusEnum.IN_PROCESS.getId()
		};

		return miRequestDao.fetchStatusChartData( statusIds );
	}

	public Long reRequest(MiRequest miRequest, Person user) {
		miRequest.changeStatus( getNewStatusDict( MiRequest.ExternalStatusEnum.REQUESTED ) );
		return store(miRequest, user);
	}

	public Long cancelRequest(MiRequest miRequest, Person user) {

		Collection<ScheduledMicroInjectionJob> activeJobs = miRequest.getActiveJobs();

		for(ScheduledMicroInjectionJob job : activeJobs) {
			scheduledMicroInjectionService.cancelJob(job.getId(),user,
					"Parent MI Request #" + miRequest.getId() + " cancelled " + new Date());
		}

		miRequest.changeStatus( getNewStatusDict( MiRequest.ExternalStatusEnum.CANCELLED ) );
		return store(miRequest, user);
	}

	public Long completeRequest(MiRequest miRequest, Person user) {
		miRequest.changeStatus( getNewStatusDict( MiRequest.ExternalStatusEnum.COMPLETED ) );
		return store(miRequest, user);
	}

	public Long processRequest(MiRequest miRequest, Person user) {
		miRequest.changeStatus( getNewStatusDict( MiRequest.ExternalStatusEnum.IN_PROCESS ) );
		return store(miRequest, user);
	}

	public Long awaitMapTest(MiRequest miRequest, Person user) {
		miRequest.changeStatus( getNewStatusDict( MiRequest.ExternalStatusEnum.AWAITING_MAP_TEST ) );
		return store(miRequest, user);
	}

	public Long delayRequest(MiRequest miRequest, Person user) {
		miRequest.changeStatus( getNewStatusDict( MiRequest.ExternalStatusEnum.DELAYED ) );
		return store(miRequest, user);
	}

	public MiRequest fetchWithAllComponentsById(Long id) {
		return fetchById(id, getFullCompoenentMap());
	}

	public MiRequest fetchById(Long id, ComponentMap componentMap) {
		MiRequest miRequest = miRequestDao.fetchById(id);
		if(miRequest != null)
			attachComponents(miRequest, componentMap);
		return miRequest;
	}

	@Override
	protected void attachComponents(MiRequest miRequest, ComponentMap requiredComponents) {

		if(miRequest == null) {
			throw new RuntimeException("Cannot attach components to null MI Request!");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(MiRequest.Component.ALL_MIREQUEST_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(MiRequest.Component.CELL_LINE)) {
				miRequest.setCellLine(fetchCellLine(miRequest, requiredComponents.get(MiRequest.Component.CELL_LINE)));
			}

			if(fetchAll || requiredComponents.containsKey(MiRequest.Component.PROJECT_LICENCE)) {
				miRequest.setProjectLicence(fetchProjectLicence(miRequest, requiredComponents.get(MiRequest.Component.PROJECT_LICENCE)));
			}

			if(fetchAll || requiredComponents.containsKey(MiRequest.Component.REQUESTER)) {
				miRequest.setRequester(fetchRequester(miRequest, requiredComponents.get(MiRequest.Component.REQUESTER)));
			}

			if(fetchAll || requiredComponents.containsKey(MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS)) {
				miRequest.setScheduledMicroInjections(fetchScheduledMicroInjections(miRequest,
						requiredComponents.get(MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS)));
			}
			if(fetchAll || requiredComponents.containsKey(MiRequest.Component.COLONY)) {
				miRequest.setColonyInfo(fetchColonyInfo(miRequest, requiredComponents.get(MiRequest.Component.COLONY)));
			}
		}
	}

	public ColonyInfo fetchColonyInfo(MiRequest miRequest, ComponentMap requiredComponents) {
		if(miRequest.getColonyInfoId() != null) {
			return colonyInfoService.fetchById(miRequest.getColonyInfoId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	public Collection<ScheduledMicroInjectionJob> fetchScheduledMicroInjections(MiRequest mirequest, ComponentMap requiredComponents) {
			return scheduledMicroInjectionService.fetchAllByMiRequestId(mirequest.getId(), requiredComponents);
	}

	private Person fetchRequester(MiRequest mirequest, ComponentMap requiredComponents) {
		if(mirequest.getRequesterId() != null) {
			return personService.fetchById(mirequest.getRequesterId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	private ProjectLicence fetchProjectLicence(MiRequest mirequest, ComponentMap requiredComponents) {
		if(mirequest.getProjectLicenceId() != null) {
			return projectLicenceService.fetchById(mirequest.getProjectLicenceId(),requiredComponents);
		}
		else {
			return null;
		}
	}

	private CellLine fetchCellLine(MiRequest mirequest, ComponentMap requiredComponents) {
		if(mirequest.getCellLineId() != null) {
			return cellLineService.fetchById(mirequest.getCellLineId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	public void setMiRequestDao (MiRequestDao miRequestDao) {
		this.miRequestDao= miRequestDao;
	}

	@Transactional
	public Long store(MiRequest miRequest, Person person) {
		return miRequestDao.store( miRequest, person );
	}

	public Collection<MiRequestListViewBean> search(MiRequestSearchBean miRequestSearchBean) {
		return miRequestDao.search(miRequestSearchBean);
	}

	public Collection<MiRequestListViewBean> getActiveMiRequests() {
		return null;
	}

	public void inflateMiRequestComponents(MiRequest miRequest) {
		if(miRequest == null) {
			throw new RuntimeException("MiRequest cannot be null when inflating components.");
		}

		if(miRequest.getCellLine() == null) {
			miRequest.setCellLine(new CellLine());
		}

		if(miRequest.getColonyInfo() == null) {
			miRequest.setColonyInfo(new ColonyInfo());
		}

		if(miRequest.getRequester() == null) {
			miRequest.setRequester(new Person());
		}

		if(miRequest.getProjectLicence() == null) {
			miRequest.setProjectLicence(new ProjectLicence());
		}

		if(miRequest.getScheduledMicroInjections() == null) {
			miRequest.setScheduledMicroInjections(new ArrayList<ScheduledMicroInjectionJob>());
		}
	}


	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////  dependencies //////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	public void setCellLineService(CellLineService cellLineService) {
		this.cellLineService = cellLineService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
		this.projectLicenceService = projectLicenceService;
	}

	public void setScheduledMicroInjectionService(
			ScheduledMicroInjectionService scheduledMicroInjectionService) {
		this.scheduledMicroInjectionService = scheduledMicroInjectionService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////  privates //////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////
	private ComponentMap getFullCompoenentMap() {
		ComponentMap requiredComponents = new ComponentMap();
		requiredComponents.put(MiRequest.Component.ALL_MIREQUEST_COMPONENTS,null);
		requiredComponents.put(MiRequest.Component.CELL_LINE,ComponentMap.of(
				CellLine.Component.DEFAULT_CELL_LINE_PROVIDER));
		requiredComponents.put(MiRequest.Component.REQUESTER,null);
		requiredComponents.put(MiRequest.Component.PROJECT_LICENCE,null);
		requiredComponents.put(MiRequest.Component.SCHEDULED_MICRO_INJECTION_JOBS,
				ComponentMap.of(ScheduledMicroInjectionJob.Component.ACTUAL_CELL_LINE_PROVIDER,
								ScheduledMicroInjectionJob.Component.TASKS,
								ScheduledMicroInjectionJob.Component.MI_REQUEST,
								ScheduledMicroInjectionJob.Component.MICRO_INJECTION,
								ScheduledMicroInjectionJob.Component.EMBRYO_TRANSFERS ));
		return requiredComponents;
	}

	//we change dictId *and* dict for maintaining a consistent object state!
	private DictionaryEntry getNewStatusDict(DictionaryEntryEnum dictionaryEntryEnum) {
		return dictionaryService.fetchEntry(DictionaryEnum.MIR_EXTERNAL_STATUS_DICT, dictionaryEntryEnum);
	}

	public void setMailHelper(MailHelper mailHelper) {
		this.mailHelper = mailHelper;
	}

	public void setMailingListDao(MailingListDao mailingListDao) {
		this.mailingListDao = mailingListDao;
	}



	public void setColonyAlleleService(ColonyAlleleService colonyAlleleService) {
		this.colonyAlleleService = colonyAlleleService;
	}



}