/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.forms.CellLineForm;
import uk.ac.sanger.mig.mirequest.services.CellLineService;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;

public class CellLineFormValidator implements Validator {

	private CellLineService cellLineService;

	public boolean supports(Class clazz) {
		return CellLineForm.class.equals(clazz);
	}

	public void validate(Object commandForm, Errors errors) {

		CellLineForm cellLineForm = (CellLineForm)commandForm;
		CellLine cellLine = cellLineForm.getCellLine();

		if(cellLine.getGenotypeDictId() == null || cellLine.getGenotypeDictId() < 1L) {
			errors.reject(null,"Genotype is a required field.");
		}
		else {
			if (cellLine.getGenotypeDictId().equals(GenotypeDictionary.WILDTYPE)){
				if (cellLine.getModifiedAlleleId() != null){
					errors.rejectValue("cellLine.modifiedAlleleId", "noAlleleForWt", "Wildtype cell lines should not have an allele");
				}
			}
			else { //not wildtype, so ensure we have a modified allele
				ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLine.modifiedAlleleId", "needAllele", "Non-wildtype genotype requires an allele.");
			}
		}

		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLine.cellLineName", "needCellLine", "Cell Line Name is a required field.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLine.cloneName", "needCellLine", "Clone Name is a required field.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLine.expectedCoatColorDictId", "needCoatColor", "Coat colour is a required field.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLine.cellLineStrainDictId", null, "Strain is a required field.");
		//ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLine.cellLineSourceDictId", null, "Cell line source is a required field.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLineProviderId", null, "Cell Line Provider is a required field.");
		//ValidationUtils.rejectIfEmptyOrWhitespace(errors,"cellLine.fundingDictId", "needFunding", "Funding is a required field.");

		// for new records, check that the cell line/clone name combination doesn't already exist
		if(cellLine.getId() == null && cellLine.getCellLineName() != null && cellLine.getCloneName() != null) {

			boolean existingCellLine =
				cellLineService.existsByCellLineAndCloneName(
						cellLine.getCellLineName(),
						cellLine.getCloneName()
				);

			if(existingCellLine) {
				errors.reject(null,"This combination cell line/clone name '"+cellLine.getCellLineName()+"/"+cellLine.getCloneName()+ "' already exists in the system.");
			}
		}

	}

	public void setCellLineService(CellLineService cellLineService) {
		this.cellLineService = cellLineService;
	}

}
