/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.validators;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.MiRequest.EndpointEnum;
import uk.ac.sanger.mig.mirequest.forms.MiRequestForm;

public class MiRequestFormValidator implements Validator {

	private ColonyInfoService colonyInfoService;

	public boolean supports(Class clazz) {
		return MiRequestForm.class.equals(clazz);
	}

	public void validate(Object commandForm, Errors errors) {

		MiRequestForm miRequestForm = (MiRequestForm)commandForm;
		MiRequest miRequest = miRequestForm.getMiRequest();

		if( !miRequestForm.isAutomatedEpdMode() ) {

			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.colonyInfoId", null, "Colony is a required field.");
			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.cellLineId", null, "Cell Line is a required field.");

		} else {

			if( !miRequestForm.isValidEpdModelDerived()  ) {

				errors.reject(null, "You must enter a valid EPD ID and click 'Auto-Create'");

			} else {

				if( StringUtils.isBlank( miRequest.getColonyInfo().getColonyPrefix() )
						|| !StringUtils.isAlpha( miRequest.getColonyInfo().getColonyPrefix() )
						|| miRequest.getColonyInfo().getColonyPrefix().matches(".*(I|O).*")
						|| miRequest.getColonyInfo().getColonyPrefix().length() != 4) {

					//this code should never be reached, since colonyservice auto-creates prefix
					//that the user for now cannot modify.  Uncomment if allowing user to modify
					//prefix
					//errors.reject(null, "Colony Prefix must be 4 letters and not contain the letters 'I' or 'O'.");

				} else {

					//Concurrency check: make sure we still have colony prefix available,
					//since second user could have submitted same before we submitted
					boolean prefixNoLongerAvailable = colonyInfoService.colonyPrefixExists(
							miRequest.getColonyInfoId(), miRequest.getColonyInfo().getColonyPrefix());

					if( prefixNoLongerAvailable ) {

						String newPrefix = colonyInfoService.getDefaultFirstUnusedPrefix();

						errors.reject(null,  miRequest.getColonyInfo().getColonyPrefix() + " was just taken by another user." +
								" Please submit again with the newly suggested Colony Prefix " + newPrefix + " or enter your own.");

					}
				}
			}
		}

		if(miRequest.getEndpointDictId() == null) {
			errors.rejectValue("miRequest.endpointDictId", "endpointId", "Endpoint is a required field.");
		} else if(miRequest.getEndpointDictId().equals( EndpointEnum.EMBRYONIC_CHIMERAS.getId() ) ){
			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.donorEmbryoStageDictId", null, "Embryo stage is a required field.");
		} else if(miRequest.getEndpointDictId().equals(  EndpointEnum.F1.getId() )){
			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.chimeraMateStrainDictId", null, "Chimera mate strain is a required field.");
		}

		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.projectLicenceId", null, "Project licence is a required field.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.workflowPipelineDictId", null, "Workflow Pipeline is a required field.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.genotypingRequired", null, "Genotyping required is a required field.");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors,"miRequest.fundingSource", null, "Funding Source is a required field.");

	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}


}
