/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.miprocessing.EmbryoTransferProcedure;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.mousetracking.services.BreedingType;

public class Breeding extends AuthenticatedDataObject {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		MALE,
		FEMALE,
		LITTERS,
		OFFSPRING_COLONY,
		EMBRYO_TRANSFER,
		PARENTS
	}

	private String comments;
	private Date startDate;
	private Date endDate;
	private boolean pseudoBreeding;
	private boolean timed;
	private Float timedEmbryoAge;
	private String namePrefix;
	private int breedingNumber;
	private Long breedingCategoryDictId;
	private DictionaryEntry breedingCategory;

	private Long embryoTransferId;
	private EmbryoTransferProcedure embryoTransfer;

	private Mouse male;
	private Long maleId;

	private Mouse female;
	private Long femaleId;

	private BreedingType breedingType;

	private Long offspringColonyId;
	private ColonyInfo offspringColony;
	private String offspringGeneration;

	private Collection<Litter> litters;

	public Mouse getFemale() {
		return female;
	}
	public void setFemale(Mouse female) {
		this.female = female;
	}
	public Collection<Litter> getLitters() {
		return litters;
	}
	public void setLitters(Collection<Litter> litters) {
		this.litters = litters;
	}
	public Mouse getMale() {
		return male;
	}
	public void setMale(Mouse male) {
		this.male = male;
	}
	public Date getEndDate() {
		return endDate;
	}
	public void setEndDate(Date eventEnd) {
		this.endDate = eventEnd;
	}
	public Date getStartDate() {
		return startDate;
	}
	public void setStartDate(Date eventStart) {
		this.startDate = eventStart;
	}
	public Long getFemaleId() {
		return femaleId;
	}
	public void setFemaleId(Long femaleId) {
		this.femaleId = femaleId;
	}
	public Long getMaleId() {
		return maleId;
	}
	public void setMaleId(Long maleId) {
		this.maleId = maleId;
	}
	public String getComments() {
		return comments;
	}
	public void setComments(String comments) {
		this.comments = comments;
	}
	public String getStatus() {
		if(endDate != null) {
			return BreedingService.BREEDING_STATUS_COMPLETE;
		}
		else if(male != null && female != null && male.getCageId() != null && male.getCageId().equals(female.getCageId()) ) {
			return BreedingService.BREEDING_STATUS_BREEDING;
		}
		else {
			return BreedingService.BREEDING_STATUS_SEPARATED;
		}
	}

	public String getBreedingName() {
		return namePrefix + breedingNumber;
	}
	public ColonyInfo getOffspringColony() {
		return offspringColony;
	}
	public void setOffspringColony(ColonyInfo offspringColony) {
		this.offspringColony = offspringColony;
	}
	public Long getOffspringColonyId() {
		return offspringColonyId;
	}
	public void setOffspringColonyId(Long offspringColonyId) {
		this.offspringColonyId = offspringColonyId;
	}
	public boolean isPseudoBreeding() {
		return pseudoBreeding;
	}
	public void setPseudoBreeding(boolean pseudoBreeding) {
		this.pseudoBreeding = pseudoBreeding;
	}
	public Long getEmbryoTransferId() {
		return embryoTransferId;
	}
	public void setEmbryoTransferId(Long embryoTransferId) {
		this.embryoTransferId = embryoTransferId;
	}
	public EmbryoTransferProcedure getEmbryoTransfer() {
		return embryoTransfer;
	}
	public void setEmbryoTransfer(EmbryoTransferProcedure embryoTransfer) {
		this.embryoTransfer = embryoTransfer;
	}
	public void setBreedingNumber(int breedingNumber) {
		this.breedingNumber = breedingNumber;
	}
	public void setNamePrefix(String namePrefix) {
		this.namePrefix = namePrefix;
	}
	public int getBreedingNumber() {
		return breedingNumber;
	}
	public String getNamePrefix() {
		return namePrefix;
	}
	public BreedingType getBreedingType() {
		return breedingType;
	}
	public void setBreedingType(BreedingType breedingType) {
		this.breedingType = breedingType;
	}
	public String getOffspringGeneration() {
		return offspringGeneration;
	}
	public void setOffspringGeneration(String offspringGeneration) {
		this.offspringGeneration = offspringGeneration;
	}
	public boolean isFromEmbryoTransfer() {
		return (embryoTransferId == null) ? false : true;
	}
	public boolean isFromMicroInjection() {
		return !(embryoTransfer == null || embryoTransfer.getScheduledMicroInjectionId() == null);
	}
	public boolean isFromCryo() {
		return !(embryoTransfer == null || embryoTransfer.getThawTestId() == null);
	}
	public boolean isTimed() {
		return timed;
	}
	public void setTimed(boolean timed) {
		this.timed = timed;
	}
	public Float getTimedEmbryoAge() {
		return timedEmbryoAge;
	}
	public void setTimedEmbryoAge(Float timedEmbryoAge) {
		this.timedEmbryoAge = timedEmbryoAge;
	}
	public Date getLatestPlug() {
		if(litters == null || litters.isEmpty()) {
			return null;
		}
		else {
			Litter[] litterArray = litters.toArray(new Litter[0]);
			return litterArray[litterArray.length-1].getPlugDate();
		}
	}
	public Date getTimedHarvestDate() {
		if(timed && timedEmbryoAge != null && getLatestPlug() != null) {
			Calendar cal = Calendar.getInstance();
			cal.setTime(getLatestPlug());
			cal.add(Calendar.DAY_OF_MONTH, timedEmbryoAge.intValue());
			return cal.getTime();
		}
		else {
			return null;
		}
	}
	public DictionaryEntry getBreedingCategory() {
		return breedingCategory;
	}
	public void setBreedingCategory(DictionaryEntry breedingCategory) {
		this.breedingCategory = breedingCategory;
	}
	public Long getBreedingCategoryDictId() {
		return breedingCategoryDictId;
	}
	public void setBreedingCategoryDictId(Long breedingCategoryDict) {
		this.breedingCategoryDictId = breedingCategoryDict;
	}
}
