/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking;

import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.BarcodeUtils;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.mousetracking.services.CageType;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.mousetracking.views.CageHeaderView;
import uk.ac.sanger.mig.mousetracking.views.CageProcedureList;
import uk.ac.sanger.mig.personnel.Person;

public class Cage extends AuthenticatedDataObject implements BarcodedObject {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		CONTENTS,
		HISTORIC_CONTENTS,
		RACK,
		BUDGET_HOLDER,
		CURRENT_BREEDINGS,
		STUD_BREEDINGS,
		PREVIOUS_CAGES,
		CAGE_HEADER,
		CAGE_TYPE,
		PROCEDURES,
		CAGE_TOTALS;
	}

	private String location;
	private String barcode;
	private String barcodeDescription;
	private String cageTotals;

	private String comments;

	private Collection<Mouse> contents;
	private Collection<Mouse> historicContents;
	private Collection<Breeding> currentBreedings;
	private Collection<StudBreeding> studBreedings;
	private Collection<Cage> previousCages;
	private Collection<CageProcedureList> procedures;

	private Rack rack;
	private Long rackId;

	private CageHeaderView cageHeader;
	private CageType cageType;

	private Person budgetHolder;
	private Long budgetHolderId;

	private Date lastCleanedDate;

	public Date getLastCleanedDate() {
		return lastCleanedDate;
	}
	public void setLastCleanedDate(Date lastCleanedDate) {
		this.lastCleanedDate = lastCleanedDate;
	}
	public Person getBudgetHolder() {
		return budgetHolder;
	}
	public void setBudgetHolder(Person budgetHolder) {
		this.budgetHolder = budgetHolder;
	}
	public Collection<Mouse> getContents() {
		return contents;
	}
	public void setContents(Collection<Mouse> contents) {
		this.contents = contents;
	}
	public String getLocation() {
		return location;
	}
	public void setLocation(String location) {
		this.location = location;
	}
	public Rack getRack() {
		return rack;
	}
	public void setRack(Rack rack) {
		this.rack = rack;
	}
	public Long getRackId() {
		return rackId;
	}
	public void setRackId(Long rackId) {
		this.rackId = rackId;
	}
	public Long getBudgetHolderId() {
		return budgetHolderId;
	}
	public void setBudgetHolderId(Long budgetHolderId) {
		this.budgetHolderId = budgetHolderId;
	}
	public String getBarcode() {
		return barcode;
	}
	public void setBarcode(String barcode) {
		this.barcode = barcode;
	}
	public String getComments() {
		return comments;
	}
	public void setComments(String comments) {
		this.comments = comments;
	}
	public String getBarcodeDescription() {
		if(barcodeDescription == null) {
			barcodeDescription = "Cage " + getId();

			if(rack != null && rack.getName() != null) {
				barcodeDescription += " Rack " + rack.getName();
			}
		}
		return barcodeDescription;
	}
	public void setBarcodeDescription(String barcodeDescription) {
		this.barcodeDescription = barcodeDescription;
	}

	public BarcodedObject.BarcodeType getBarcodeType() {
		return BarcodedObject.BarcodeType.CAGE;
	}
	public Collection<Mouse> getHistoricContents() {
		return historicContents;
	}
	public void setHistoricContents(Collection<Mouse> historicContents) {
		this.historicContents = historicContents;
	}
	public Collection<Breeding> getCurrentBreedings() {
		return currentBreedings;
	}
	public void setCurrentBreedings(Collection<Breeding> currentBreedings) {
		this.currentBreedings = currentBreedings;
	}
	public Collection<Cage> getPreviousCages() {
		return previousCages;
	}
	public void setPreviousCages(Collection<Cage> previousCages) {
		this.previousCages = previousCages;
	}
	public CageHeaderView getCageHeader() {
		return cageHeader;
	}
	public void setCageHeader(CageHeaderView cageHeader) {
		this.cageHeader = cageHeader;
	}
	public CageType getCageType() {
		return cageType;
	}
	public void setCageType(CageType cageType) {
		this.cageType = cageType;
	}
	public Collection<CageProcedureList> getProcedures() {
		return procedures;
	}
	public void setProcedures(Collection<CageProcedureList> procedures) {
		this.procedures = procedures;
	}

	public boolean isPopulated() {
		return (contents == null || contents.isEmpty()) ? false : true;
	}

	/**
	 * Only returns true if the contents of the cage are from the same litter and have an undefined gender.
	 * @return
	 */
	public boolean isReadyToSplit() {
		boolean readyToSplit = true;

		if(isPopulated()) {
			Long litterId = null;
			for(Mouse mouse : contents) {
				if(mouse.getLitterId() == null || !MouseService.MOUSE_GENDER_UNDEFINED.equals(mouse.getGenderDictId())){
					readyToSplit = false;
					break;
				}
				else if(litterId == null) {
					litterId = mouse.getLitterId();
				}
				else if(litterId != null && !litterId.equals(mouse.getLitterId())) {
					readyToSplit = false;
					break;
				}
			}
		}
		else {
			readyToSplit = false;
		}
		return readyToSplit;
	}


	public void generateBarcode() {
		this.barcode = BarcodeUtils.generateBarcode(getBarcodeType(), getId());
	}
	public Collection<StudBreeding> getStudBreedings() {
		return studBreedings;
	}
	public void setStudBreedings(Collection<StudBreeding> studBreedings) {
		this.studBreedings = studBreedings;
	}
	public String getCageTotals() {
		return cageTotals;
	}
	public void setCageTotals(String cageTotals) {
		this.cageTotals = cageTotals;
	}
}
