/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking;

import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.CalendarUtils;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class Litter extends AuthenticatedDataObject implements BarcodedObject {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		PUPS,
		BREEDING
	}

	private Date birthDate;
	private Date plugDate;
	private Date dueDate;
	private Integer numberBorn;
	private Integer numberHarvested;
	private String comments;
	private String namePrefix;
	private int litterNumber;
	private boolean notPregnant;

	private Breeding breeding;
    private Long breedingId;

	private Collection<Mouse> pups;

	public Date getBirthDate() {
		return birthDate;
	}
	public void setBirthDate(Date birthDate) {
		this.birthDate = birthDate;
	}
	public Breeding getBreeding() {
		return breeding;
	}
	public void setBreeding(Breeding breeding) {
		this.breeding = breeding;
	}
	public Collection<Mouse> getPups() {
		return pups;
	}
	public void setPups(Collection<Mouse> pups) {
		this.pups = pups;
	}
	public Long getBreedingId() {
		return breedingId;
	}
	public void setBreedingId(Long breedingId) {
		this.breedingId = breedingId;
	}
	public Date getPlugDate() {
		return plugDate;
	}
	public void setPlugDate(Date plugCheckDate) {
		this.plugDate = plugCheckDate;
	}
	public Integer getNumberBorn() {
		return numberBorn;
	}
	public int getNumberBornInt() {
		return (numberBorn == null) ? 0 : numberBorn;
	}
	public void setNumberBorn(Integer numberBorn) {
		this.numberBorn = numberBorn;
	}
	public String getComments() {
		return comments;
	}
	public void setComments(String comments) {
		this.comments = comments;
	}
	public int getNumberAliveInt() {
		int numberAlive = 0;
		if(pups != null) {
			for(Mouse pup : pups) {
				if(pup.isAlive()) {
					numberAlive++;
				}
			}
		}
		return numberAlive;
	}
	public String getLitterName() {
		return namePrefix + litterNumber;
	}
	public Integer getNumberHarvested() {
		return numberHarvested;
	}
	public int getNumberHarvestedInt() {
		return (numberHarvested == null) ? 0 : numberHarvested;
	}
	public void setNumberHarvested(Integer numberHarvested) {
		this.numberHarvested = numberHarvested;
	}
	public String getEmbryoAge() {
		if(birthDate == null) {
			if(plugDate == null) {
				return "No Plug Date";
			}
			// try to calculate based on plug date
			else {
				return CalendarUtils.diffInDays(plugDate, new Date()) + ".5E";
			}
		}
		// don't return embryo age if pups are already born
		else {
			return null;
		}
	}

	/**
	 * Returns true if any of the females in the litter has pooled flag set to true.
	 * @return
	 */
	public boolean isPooled() {
		boolean pooled = false;

		if(pups != null) {
			for(Mouse pup : pups) {
				if(pup.isPooled()) {
					pooled = true;
					break;
				}
			}
		}

		return pooled;
	}

	/**
	 * Returns true only if the entire live portion of the litter has not been weaned yet.
	 * @return
	 */
	public boolean isWeanable() {
		boolean weanable = true;
		int deadPups = 0;
		if(pups == null) {
			weanable = false;
		}
		else {
			for(Mouse pup : pups) {
				if(pup.isDead()) {
					deadPups++;
				}
				else if(pup.isAlive() && !pup.isWeanable()) {
					weanable = false;
					break;
				}
			}
			// if all the pups are dead, return false
			if(deadPups == pups.size()) {
				weanable = false;
			}
		}
		return weanable;
	}
	/**
	 * Litter can be harvested for embryos only if there is a plug date and no pups born
	 * and female is pregnant
	 * @return
	 */
	public boolean isHarvestable() {
		boolean harvestable = true;

		if(plugDate == null || isNotPregnant() || birthDate != null || (pups != null && !pups.isEmpty())) {
			harvestable = false;
		}

		return harvestable;
	}
	/**
	 * A litter will not be deletable if it contains ANY of the following:
	 * plug date, not pregnant flag, birth date, comments, pups, is from embryo transfer
	 * @return
	 */
	public boolean isDeletable() {
		boolean deletable = true;

		if (
				isFromEmbryoTransfer() ||
				plugDate != null ||
				isNotPregnant() ||
				birthDate != null ||
				comments != null ||
				(pups != null && !pups.isEmpty())
				) {
			deletable = false;
		}

		return deletable;
	}
	/**
	 * @return true if all live pups are in the same cage and are unsexed and weaned.
	 */
	public boolean isSplittable() {
		boolean splittable = true;
		Set<Long> cageIds = new HashSet<Long>();

		if(pups == null) {
			splittable = false;
		}
		else {
			for(Mouse pup : pups) {
				// only evaluate on live mice
				if(pup.isAlive()) {
					// fail for mice that aren't weaned or are male or female
					if(pup.isMale() || pup.isFemale() || !pup.isWeaned()) {
						splittable = false;
						break; // can exit early because one pup failing is enough
					}
					cageIds.add(pup.getCageId());
				}
			}
		}

		return (splittable && cageIds.size() == 1);
	}

	public boolean isFromEmbryoTransfer() {
		return !(breeding == null || breeding.getEmbryoTransferId() == null);
	}

	public boolean isFromMicroInjection() {
		return !(breeding == null || breeding.getEmbryoTransfer() == null || breeding.getEmbryoTransfer().getScheduledMicroInjection() == null);
	}

	public boolean isFromCryoPreservation() {
		return !(breeding == null || breeding.getEmbryoTransfer() == null || breeding.getEmbryoTransfer().getThawTest() == null);
	}

	public boolean isLitterUnsexed() {
		boolean unsexed = true;
		int deadPups = 0;
		if(pups == null) {
			unsexed = false; // if pups is null return false (assume they have been sexed)
		}
		else {
			for(Mouse pup : pups) {
				if(pup.isDead()) {
					deadPups++;
				}
				//if ANY pup is male or female return false
				if(!MouseService.MOUSE_GENDER_UNDEFINED.equals(pup.getGenderDictId())) {
					unsexed = false;
					break;
				}
			}
			// if all the pups are dead, return false
			if(deadPups == pups.size()) {
				unsexed = false;
			}
		}
		return unsexed;
	}
	/**
	 * return the number of animals of the given gender, optionally weaned only animals
	 * @param genderDictId
	 * @param weanedOnly
	 * @return
	 */
	public Integer getGenderCount(Long genderDictId, boolean weanedOnly) {
		Integer genderCount = null;
		if(pups != null && !pups.isEmpty()) {
			genderCount = 0;
			for(Mouse pup : pups) {
				if(genderDictId.equals(pup.getGenderDictId())) {
					if(!weanedOnly || (weanedOnly && pup.getWeanDate() != null)) {
						genderCount++;
					}
				}
			}
		}
		return genderCount;
	}
	public Integer getMaleCount() {
		return getGenderCount(MouseService.MOUSE_GENDER_MALE, false);
	}
	public Integer getFemaleCount() {
		return getGenderCount(MouseService.MOUSE_GENDER_FEMALE, false);
	}
	public Integer getUndefinedGenderCount() {
		return getGenderCount(MouseService.MOUSE_GENDER_UNDEFINED, false);
	}
	public Integer getWeanedMaleCount() {
		return getGenderCount(MouseService.MOUSE_GENDER_MALE, true);
	}
	public Integer getWeanedFemaleCount() {
		return getGenderCount(MouseService.MOUSE_GENDER_FEMALE, true);
	}
	public Integer getWeanedUndefinedGenderCount() {
		return getGenderCount(MouseService.MOUSE_GENDER_UNDEFINED, true);
	}
	public Integer getPreweanDeaths() {
		Integer preweanDeaths = null;
		if(pups != null && !pups.isEmpty()) {
			preweanDeaths = 0;
			for(Mouse pup : pups) {
				if(pup.isDead() && !pup.isWeaned()) {
					preweanDeaths++;
				}
			}
		}
		return preweanDeaths;
	}
	public Date getDueDate() {
		return dueDate;
	}
	public void setDueDate(Date dueDate) {
		this.dueDate = dueDate;
	}
	public int getLitterNumber() {
		return litterNumber;
	}
	public void setLitterNumber(int litterNumber) {
		this.litterNumber = litterNumber;
	}
	public String getNamePrefix() {
		return namePrefix;
	}
	public void setNamePrefix(String namePrefix) {
		this.namePrefix = namePrefix;
	}
	public boolean isNotPregnant() {
		return notPregnant;
	}
	public void setNotPregnant(boolean notPregnant) {
		this.notPregnant = notPregnant;
	}

	/**
	 *
	 * @return the cageId for prewean pups in a litter or null if this doesn't resolve to a single cage
	 */
	public Long getCageId() {
		Long cageId = null;

		if(pups != null) {
			Set<Long> cageIds = new HashSet<Long>();
			for(Mouse pup : pups) {
				if(pup.getWeanDate() == null && pup.getCageId() != null) {
					cageIds.add(pup.getCageId());
				}
			}
			if(cageIds.size() == 1) {
				cageId = cageIds.iterator().next();
			}
		}

		if(cageId == null && getBreeding() != null && getBreeding().getFemale() != null && getBreeding().getFemale().getCageId() != null) {
			cageId = getBreeding().getFemale().getCageId();
		}

		return cageId;
	}
	public void generateBarcode() {
		// not implemented
	}
	public String getBarcode() {
		return getLitterName();
	}
	public String getBarcodeDescription() {
		return getLitterName();
	}
	public BarcodeType getBarcodeType() {
		return BarcodeType.LITTER;
	}
}
