/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking;

import static uk.ac.sanger.mig.mousetracking.services.MouseService.MOUSE_GENDER_FEMALE;
import static uk.ac.sanger.mig.mousetracking.services.MouseService.MOUSE_GENDER_MALE;
import static uk.ac.sanger.mig.mousetracking.services.MouseService.MOUSE_GENDER_UNDEFINED;

import java.util.Collection;
import java.util.Date;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.lang.StringUtils;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.BarcodeUtils;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.mousetracking.views.MouseLocation;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.reservations.MouseReservation;

/**
 * The Mouse class represents the data for a single animal. <p> Mouse contains complex
 * sub-components, some of which must be present to correctly infer the value of certain attributes
 * (e.g. genotype). </p>
 *
 * <table border="1"> <caption>Sub-Components</caption> <tr> <th>Component</th> <th>Description</th>
 * </tr> <tr> <td>ALLELES</td> <td>Collection of {@see MouseAllele} objects, one per allele.
 * Wildtype mice may have a zero or null collection.</td> </tr> <tr> <td>CAGE</td> <td>{@link
 * Cage} object representing the current location of the mouse. Dead mice should have a null
 * cage.</td> </tr> <tr> <td>CHIMERIC_DATA</td> <td>{@link ChimericMouseData} object representing
 * the coat colour score for a chimeric mouse. Non-chimerics should have null chimeric data.</td>
 * </tr> <tr> <td>COLONY</td> <td>{@link ColonyInfo} object representing the details of the mouse's
 * current colony. Note that mice can be transferred from one colony to another over time.</td>
 * </tr> <tr> <td>CURRENT_OWNER</td> <td>{@link Person} object representing the current user
 * designated as the owner of the animal. Typically this should be someone who holds a personal
 * licence (PIL).</td> </tr> <tr> <td>LOCATION</td> <td>{@link MouseLocation} object representing
 * the current location of the mouse. This differes from Cage as it holds a full demormalised
 * description of the location including rack and room. Location is indended to be read-only,
 * therefore altering the contents of this object should not result in any update to the underlying
 * database records.</td> </tr> <tr> <td>LITTER</td> <td>{@link Litter} object representing the
 * litter of origin of the mouse. Mice may not have a litter of origin if they are founder
 * animals.</td> </tr> <tr> <td>CURRENT_LICENSED_PROCEDURE</td> <td>{@link MouseProcedure} object
 * representing the latest UK Government Home Office Licenced Procedure that the mouse was subject
 * to. This can be null if the mouse is wildtype and has not been used for a licenced
 * procedure.</td> </tr> <tr> <td>CURRENT_WEIGHT</td> <td>{@link MouseWeight} object representing
 * the latest weight (by sample date) for the mouse. Will be null if no weight record exists for the
 * mouse.</td> </tr> <tr> <td>LATEST_BREEDING</td> <td>{@link Breeding} object representing the
 * latest mating a mouse was involved in. This can be null if the mouse has never mated.</td> </tr>
 * <tr> <td>EXPERIMENT_DATA</td> <td>{@link ExperimentData} object representing a loose collection
 * of experimental data results for the mouse. This can be null if the mouse has not been
 * experimented upon.</td> </tr> <tr> <td>GENOTYPE</td> <td>This is a convenience component for
 * attaching ALLELES and CHIMERIC_DATA for a mouse in one operation. Both these sub-components are
 * required to correctly establish the genotype of a mouse.</td> </tr> </table>
 */
public class Mouse extends AuthenticatedDataObject
        implements BarcodedObject, FamilyTreeMember, LookupListElement {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		ALLELES,
		CAGE,
		CHIMERIC_DATA,
		COLONY,
		CURRENT_OWNER,
		LOCATION,
		LITTER,
		CURRENT_LICENSED_PROCEDURE,
		CURRENT_WEIGHT,
		LATEST_BREEDING,
		LATEST_STUD_BREEDING,
		EXPERIMENT_DATA,
		GENOTYPE,
        MOUSE_RESERVATION;
    }

    private String name;
    private String rfidTag;
    private String barcode;
    private String barcodeDescription;
    private String comments;
    private String generation;

    private boolean nonBreeder;
    private boolean sentinel;
    private boolean sterile;

    private Cage cage;
    private Long cageId;
    private MouseLocation location;

    private Person currentOwner;
    private Long currentOwnerId;

    private Litter litter;
    private Long litterId;

    private ColonyInfo colonyInfo;
    private Long colonyInfoId;

    private Date weanDate;
    private Date birthDate;
    private Date deathDate;
    private Date arrivalDate;

    private DictionaryEntry gender;
    private Long genderDictId;
    private DictionaryEntry coatColour;
    private Long coatColourDictId;
    private DictionaryEntry eyeColour;
    private Long eyeColourDictId;
    private DictionaryEntry earmark;
    private Long earmarkDictId;
    private DictionaryEntry causeOfDeath;
    private Long causeOfDeathDictId;
    private DictionaryEntry strain;
    private Long strainDictId;
    private boolean pooled;
    private boolean stud;
    private boolean provenStud;
    private MouseReservation mouseReservation;

    private Long familyTreeChild;
    private int familyTreeGeneration;
    private int familyTreeRowIndex;

    private Collection<MouseAllele> alleles;
    private ChimericMouseData chimericData;

    private MouseProcedure currentLicencedProcedure;
	private StudBreeding latestStudBreeding;

    private Breeding latestBreeding;

    private MouseWeight currentWeight;

    private ExperimentData experimentData;


    public MouseReservation getMouseReservation() {
        return mouseReservation;
    }


    public void setMouseReservation(MouseReservation mouseReservation) {
        this.mouseReservation = mouseReservation;
    }


    /**
     * This is the EXPERIMENT_DATA sub-component of the mouse
     */
    public ExperimentData getExperimentData() {
        return experimentData;
    }


    /**
     * This is the EXPERIMENT_DATA sub-component of the mouse
     */
    public void setExperimentData(ExperimentData experimentData) {
        this.experimentData = experimentData;
    }


    /**
     * This is the ALLELES sub-component of the mouse
     */
    public Collection<MouseAllele> getAlleles() {
        return alleles;
    }


    /**
     * This is the ALLELES sub-component of the mouse
     */
    public void setAlleles(Collection<MouseAllele> alleles) {
        this.alleles = alleles;
    }


    /**
     * This is the CAGE sub-component of the mouse
     */
    public Cage getCage() {
        return cage;
    }


    /**
     * This is the CAGE sub-component of the mouse
     */
    public void setCage(Cage cage) {
        this.cage = cage;
    }


    public DictionaryEntry getCoatColour() {
        return coatColour;
    }


    public void setCoatColour(DictionaryEntry coatColour) {
        this.coatColour = coatColour;
    }


    /**
     * This is the CURRENT_OWNER sub-component of the mouse
     */
    public Person getCurrentOwner() {
        return currentOwner;
    }


    /**
     * This is the CURRENT_OWNER sub-component of the mouse
     */
    public void setCurrentOwner(Person currentOwner) {
        this.currentOwner = currentOwner;
    }


    public DictionaryEntry getEarmark() {
        return earmark;
    }


    public void setEarmark(DictionaryEntry earclip) {
        this.earmark = earclip;
    }


    public Date getWeanDate() {
        return weanDate;
    }


    public void setWeanDate(Date weaningDate) {
        this.weanDate = weaningDate;
    }


    public Date getBirthDate() {
        return birthDate;
    }


    public Date getArrivalDate() {
        return arrivalDate;
    }


    public void setArrivalDate(Date arrivalDate) {
        this.arrivalDate = arrivalDate;
    }


    public void setBirthDate(Date birthDate) {
        this.birthDate = birthDate;
    }


    public DictionaryEntry getCauseOfDeath() {
        return causeOfDeath;
    }


    public void setCauseOfDeath(DictionaryEntry causeOfDeath) {
        this.causeOfDeath = causeOfDeath;
    }


    public Date getDeathDate() {
        return deathDate;
    }


    public void setDeathDate(Date deathDate) {
        this.deathDate = deathDate;
    }


    public DictionaryEntry getEyeColour() {
        return eyeColour;
    }


    public void setEyeColour(DictionaryEntry eyeColour) {
        this.eyeColour = eyeColour;
    }


    public String getName() {
        return name;
    }


    public void setName(String name) {
        this.name = name;
    }


    public String getRfidTag() {
        return rfidTag;
    }


    public void setRfidTag(String rfidTag) {
        this.rfidTag = rfidTag;
    }


    /**
     * This is the LITTER sub-component of the mouse
     */
    public Litter getLitter() {
        return litter;
    }


    /**
     * This is the LITTER sub-component of the mouse
     */
    public void setLitter(Litter litter) {
        this.litter = litter;
    }


    public Long getCageId() {
        return cageId;
    }


    public void setCageId(Long cageId) {
        this.cageId = cageId;
    }


    public Long getLitterId() {
        return litterId;
    }


    public void setLitterId(Long litterId) {
        this.litterId = litterId;
    }


    public Long getCurrentOwnerId() {
        return currentOwnerId;
    }


    public void setCurrentOwnerId(Long currentOwnerId) {
        this.currentOwnerId = currentOwnerId;
    }


    public DictionaryEntry getGender() {
        return gender;
    }


    public void setGender(DictionaryEntry gender) {
        this.gender = gender;
    }


    public Long getCauseOfDeathDictId() {
        return causeOfDeathDictId;
    }


    public void setCauseOfDeathDictId(Long causeOfDeathDictId) {
        this.causeOfDeathDictId = causeOfDeathDictId;
    }


    public Long getCoatColourDictId() {
        return coatColourDictId;
    }


    public void setCoatColourDictId(Long coatColourDictId) {
        this.coatColourDictId = coatColourDictId;
    }


    public Long getEarmarkDictId() {
        return earmarkDictId;
    }


    public void setEarmarkDictId(Long earmarkDictId) {
        this.earmarkDictId = earmarkDictId;
    }


    public Long getEyeColourDictId() {
        return eyeColourDictId;
    }


    public void setEyeColourDictId(Long eyeColourDictId) {
        this.eyeColourDictId = eyeColourDictId;
    }


    public Long getGenderDictId() {
        return genderDictId;
    }


    public void setGenderDictId(Long genderDictId) {
        this.genderDictId = genderDictId;
    }


    public DictionaryEntry getStrain() {
        return strain;
    }


    public void setStrain(DictionaryEntry strain) {
        this.strain = strain;
    }


    public Long getStrainDictId() {
        return strainDictId;
    }


    public void setStrainDictId(Long strainDictId) {
        this.strainDictId = strainDictId;
    }


    public String getBarcode() {
        return barcode;
    }


    public void setBarcode(String barcode) {
        this.barcode = barcode;
    }


    /**
     * This is the LOCATION sub-component of the mouse
     */
    public MouseLocation getLocation() {
        return location;
    }


    /**
     * This is the LOCATION sub-component of the mouse
     */
    public void setLocation(MouseLocation location) {
        this.location = location;
    }


    /**
     * Returns a human-readable description of mouse barcode.
     *
     * <p> The barcode description consists white-space separated concatenated non-null values of <ul>
     * <li>name (where name != barcode) <li>gender name <li>earmark name <li>coatColour name </ul> If
     * any of the underlying DictionaryEntry objects are themselves null, then the String "error" will
     * be returned. </p>
     *
     * <table border="1"> <caption>Examples</caption> <tr> <th>Barcode</th><th>Name</th><th>Gender</th><th>Earmark</th><th>Coat
     * Colour</th><th>Barcode Description</th> <tr> <tr> <td>M00000001</td><td>MAQZ1.1b</td><td>Male</td><td>NM</td><td>Agouti</td><td>MAQZ1.1b
     * Male NM Agouti</td> </tr> <tr> <td>M00000002</td><td>M00000002</td><td>Male</td><td>NM</td><td>Agouti</td><td>Male
     * NM Agouti</td> </tr> <tr> <td>M00000003</td><td>MAQZ1.1c</td><td>&nbsp;</td><td>NM</td><td>&nbsp;</td><td>MAQZ1.1c
     * NM</td> </tr> <tr> <td>M00000004</td><td>MAQZ1.1d</td><td>[null DictionaryEntry]</td><td>NM</td><td>Agouti</td><td>error</td>
     * </tr> </table>
     *
     * @return human-readable description of mouse barcode.
     */
    public String getBarcodeDescription() {
        if (barcodeDescription == null) {
            try {
                String description = "";
                if (getName() != null && !getName().equals("") && !getName().equals(getBarcode())) {
                    description += getName() + " ";
                }

                if (getGender() != null) {
                    description += getGender().getName() + " ";
                }
                if (getEarmark() != null) {
                    description += getEarmark().getName() + " ";
                }
                if (getCoatColour() != null) {
                    description += getCoatColour().getName() + " ";
                }

                return description;
            }
            catch (NullPointerException e) {
                return "error";
            }
        } else {
            return barcodeDescription;
        }
    }


    public void setBarcodeDescription(String barcodeDescription) {
        this.barcodeDescription = barcodeDescription;
    }


    public String getComments() {
        return comments;
    }


    public void setComments(String comments) {
        this.comments = comments;
    }


    public boolean isNonBreeder() {
        return nonBreeder;
    }


    public void setNonBreeder(boolean nonBreeder) {
        this.nonBreeder = nonBreeder;
    }


    public boolean isSentinel() {
        return sentinel;
    }


    public void setSentinel(boolean sentinel) {
        this.sentinel = sentinel;
    }


    public String getGeneration() {
        return generation;
    }


    public void setGeneration(String generation) {
        this.generation = generation;
    }


    /**
     * This is the COLONY sub-component of the mouse
     */
    public ColonyInfo getColonyInfo() {
        return colonyInfo;
    }


    /**
     * This is the COLONY sub-component of the mouse
     */
    public void setColonyInfo(ColonyInfo colonyInfo) {
        this.colonyInfo = colonyInfo;
    }


    public Long getColonyInfoId() {
        return colonyInfoId;
    }


    public void setColonyInfoId(Long colonyInfoId) {
        this.colonyInfoId = colonyInfoId;
    }


    public Long getFamilyTreeChild() {
        return familyTreeChild;
    }


    public void setFamilyTreeChild(Long familyTreeChild) {
        this.familyTreeChild = familyTreeChild;
    }


    public int getFamilyTreeGeneration() {
        return familyTreeGeneration;
    }


    public void setFamilyTreeGeneration(int familyTreeGeneration) {
        this.familyTreeGeneration = familyTreeGeneration;
    }


    public int getFamilyTreeRowIndex() {
        return familyTreeRowIndex;
    }


    public void setFamilyTreeRowIndex(int familyTreeRowIndex) {
        this.familyTreeRowIndex = familyTreeRowIndex;
    }


    public boolean isSterile() {
        return sterile;
    }


    public void setSterile(boolean sterile) {
        this.sterile = sterile;
    }


    public String getDisplayText() {
        return
                getBarcodeDescription();
    }


    public boolean isPooled() {
        return pooled;
    }


    public void setPooled(boolean pooled) {
        this.pooled = pooled;
    }


    public String getElementData() {
        return
                LookupListUtil.format("mouseId", getId()) +
                        LookupListUtil.format("mouseBarcode", getBarcode()) +
                        LookupListUtil.format("mouseBarcodeDescription", getBarcodeDescription());
    }


    /**
     * Returns the mutant status of the mouse.
     * A mouse is mutant if it has modified alleles that have not been genoyped as wildtype,
     * or if it is a chimera. <p> A prior attempt must have been made to attach the
     * sub-components ALLELES and CHIMERIC_DATA (or GENOTYPE) for this method to return a valid result.
     * If these components have not been attached, but do exist, then Wildtype will be returned
     * erroniously. </p>
     * <p>Alleles that are untested or failed genotyping will be considered to be mutant.</p>
     *
     * @return true if mouse has alleles with a or chimeric data, otherwise false.
     */
    public boolean isMutant() {
        if (alleles == null && chimericData == null) {
            return false;
        } else if (isChimera()) {
            return true;
        } else if (isZeroPercentChimera()) {
            return false;
        } else {
        	boolean isMutant = false;
        	for (MouseAllele ma : alleles) {
        		if (!ma.getGenotypeDictId().equals(GenotypeDictionary.WILDTYPE)){
        			isMutant = true;
        		}
        	}
        	return isMutant;
        }
    }




    /**
     * Returns a concatenated String of all allele genotypes for the mouse.
     *
     * @return a concatenated String of genotype descriptions for the mouse.
     */
    public String getGenotype() {
        String genotype = "";
        Set<String> genotypeSymbols = new TreeSet<String>();
        if(alleles != null) {
        	boolean hasMultipleAlleles = (alleles.size() > 1);
        		for (MouseAllele mouseAllele : alleles) {
          		genotypeSymbols.add(mouseAllele.getGenotypeSymbol(hasMultipleAlleles));
          	}
        }
        if(genotypeSymbols.isEmpty()) {
        	genotype = "+/+";
        }
        else {
        	String symbolString = genotypeSymbols.toString();
        	// Using a collection toString puts square brackets round the text
        	genotype += symbolString.substring(1, symbolString.length() - 1);
        }

        if(isChimera()) {
        	genotype += (StringUtils.isEmpty(genotype)) ? "" : " ";
        	genotype += chimericData.getPercentChimeric() + "%";
        }

        return genotype;
    }


    /**
     * This is the CHIMERIC_DATA sub-component of the mouse
     */
    public ChimericMouseData getChimericData() {
        return chimericData;
    }


    /**
     * This is the CHIMERIC_DATA sub-component of the mouse
     */
    public void setChimericData(ChimericMouseData chimericData) {
        this.chimericData = chimericData;
    }


    /**
     * Returns true only if the CHIMERIC_DATA sub-component is non-null and has a non-null
     * value for percentChimeric.
     *
     * <p> A prior attempt must have been made to attach the sub-component CHIMERIC_DATA for this
     * method to return a valid result. If this component has not been attached, but does exist, then
     * false may be returned erroniously. </p>
     *
     * @return true if mouse is a chimera, otherwise false
     */
    public boolean isChimera() {
        return (chimericData == null || chimericData.getPercentChimeric() == null)
                ? false : true;
    }


    /**
     * Returns true only if CHIMERIC_DATA has been attached and contains zero (0) in the
     * percentChimeric field.
     *
     * <p> This method is used to indicate that a potential chimera has been scored with zero (0) for
     * percent chimerism to differentiate from mice who have no chimeric data. </p>
     *
     * <p> A prior attempt must have been made to attach the sub-component CHIMERIC_DATA for this
     * method to return a valid result. If this component has not been attached, but does exist, then
     * false may be returned erroniously. </p>
     *
     * <p> A zero percent chimera is considered to be wildtype (+/+). </p>
     *
     * @return true if mouse is zero percent chimeric, otherwise false.
     */
    public boolean isZeroPercentChimera() {
        return (chimericData != null && chimericData.getPercentChimeric() != null &&
                chimericData.getPercentChimeric().equals(0));
    }


    /**
     * @return true if mouse has a non-null wean date, otherwise false.
     */
    public boolean isWeaned() {
        return (weanDate == null) ? false : true;
    }


    /**
     * @return true if mouse has a non-null birth date, otherwise false.
     */
    public boolean isBorn() {
        return (birthDate == null) ? false : true;
    }


    /**
     * A mouse is considered alive if it has a null death date.
     *
     * The antonym isDead(), returns the opposite boolean value.
     *
     * @return true if mouse alive.
     */
    public boolean isAlive() {
        return (deathDate == null) ? true : false;
    }


    /**
     * A mouse is considered dead if it has a non-null death date.
     *
     * The antonym isAlive(), returns the opposite boolean value.
     *
     * @return true if mouse is dead.
     */
    public boolean isDead() {
        return !isAlive();
    }


    /**
     * A mouse is weanable if it is alive and not already weaned and has come from a litter
     *
     * @return true is mouse can be weaned, otherwise false.
     */
    public boolean isWeanable() {
    	if(litterId == null) {
    		return false;
    	}
    	else {
    		return !(isDead() || isWeaned());
    	}
    }


    /**
     * Non-implemented method
     */
    public void setDisplayText(String displayText) {
        // NOT REQUIRED
    }


    /**
     * Non-implemented method
     */
    public void setElementData(String elementData) {
        // NOT REQUIRED
    }


    /**
     * Returns the BarcodeType for this Object.
     *
     * @return BarcodedObject.BarcodeType.MOUSE
     *
     * @see BarcodedObject.BarcodeType
     */
    public BarcodedObject.BarcodeType getBarcodeType() {
        return BarcodedObject.BarcodeType.MOUSE;
    }


    /**
     * This is the CURRENT_WEIGHT sub-component of the mouse
     */
    public MouseWeight getCurrentWeight() {
        return currentWeight;
    }


    /**
     * This is the CURRENT_WEIGHT sub-component of the mouse
     */
    public void setCurrentWeight(MouseWeight currentWeight) {
        this.currentWeight = currentWeight;
    }


    /**
     * Return String description of the mouse's age.
     *
     * @return String description of the mouse's age.
     *
     * @see MouseAge#toString()
     */
    public String getAgeString() {
        return new MouseAge(this).toString();
    }


    public MouseProcedure getCurrentLicencedProcedure() {
        return currentLicencedProcedure;
    }

    public ProjectLicence getCurrentProjectLicence() {
    	if(currentLicencedProcedure == null ||
    			currentLicencedProcedure.getProtocol() == null ||
    			currentLicencedProcedure.getProtocol().getProjectLicence() == null) {
    		return null;
    	}
    	else {
    		return currentLicencedProcedure.getProtocol().getProjectLicence();
    	}
    }

    public Long getCurrentProjectLicenceId() {
    	if(currentLicencedProcedure == null ||
    			currentLicencedProcedure.getProtocol() == null ||
    			currentLicencedProcedure.getProtocol().getProjectLicence() == null) {
    		return null;
    	}
    	else {
    		return currentLicencedProcedure.getProtocol().getProjectLicenceId();
    	}
    }

    public Long getCurrentProtocolId() {
    	if(currentLicencedProcedure == null ||
    			currentLicencedProcedure.getProtocolId() == null) {
    		return null;
    	}
    	else {
    		return currentLicencedProcedure.getProtocolId();
    	}
    }

    public void setCurrentLicencedProcedure(MouseProcedure currentLicencedProcedure) {
        this.currentLicencedProcedure = currentLicencedProcedure;
    }


    public boolean isMale() {
        return (MOUSE_GENDER_MALE.equals(genderDictId)) ? true : false;
    }


    public boolean isFemale() {
        return (MOUSE_GENDER_FEMALE.equals(genderDictId)) ? true : false;
    }


    public boolean isUnsexed() {
        return (MOUSE_GENDER_UNDEFINED.equals(genderDictId)) ? true : false;
    }


    /**
     * Returns true only if mouse contains at least one allele whose harmfulMutation field is true. <p>
     * A prior attempt must have been made to attach the sub-component ALLELES for this method to
     * return a valid result. If this component has not been attached, but does exist, then false may
     * be returned erroniously. </p>
     *
     * @return true if mouse contains a harmful allele, otherwise false.
     */
    public boolean isHarmfulMutant() {
        if (!isMutant() || alleles == null || alleles.isEmpty()) {
            return false;
        } else {
            for (MouseAllele mouseAllele : alleles) {
                if (mouseAllele.getAllele().isHarmfulMutation()) {
                    return true;
                }
            }
            return false;
        }
    }


    public Breeding getLatestBreeding() {
        return latestBreeding;
    }


    public void setLatestBreeding(Breeding latestBreeding) {
        this.latestBreeding = latestBreeding;
    }


    /**
     * This method sets the value of the barcode field for the mouse and is typically only called when
     * first storing a new mouse record in the database <p> The barcode for a mouse is a 9 digit String
     * starting with M and right-padded with zeros, ending with the mouse ID. </p> <p> E.g. for a mouse
     * ID 432 the barcode will be M00000432 </p>
     *
     * @throws IllegalArgumentException if id is null
     */
    public void generateBarcode() {
        this.barcode = BarcodeUtils.generateBarcode(getBarcodeType(), getId());
	}
	public boolean isStud() {
		return stud;
	}
	public void setStud(boolean stud) {
		this.stud = stud;
	}
	public boolean isProvenStud() {
		return provenStud;
	}
	public void setProvenStud(boolean provenStud) {
		this.provenStud = provenStud;
	}
	public StudBreeding getLatestStudBreeding() {
		return latestStudBreeding;
	}
	public void setLatestStudBreeding(StudBreeding latestStudBreeding) {
		this.latestStudBreeding = latestStudBreeding;
	}
}
