/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking;

import java.text.NumberFormat;
import java.util.Date;

import org.apache.commons.lang.time.DateUtils;

import uk.ac.sanger.mig.common.CalendarUtils;

/**
 * Mouse age is used to calculate and display the age of a mouse; embryonic, adult or deceased.
 */
public class MouseAge {

	private Date plugDate;
	private Date birthDate;
	private Date deathDate;
	private NumberFormat numberFormat;

	/**
	 * MouseAge requires a Mouse object to calculate the age of
	 * @param mouse
	 */
	public MouseAge(Mouse mouse) {
		plugDate = (mouse.getLitter() == null) ? null : mouse.getLitter().getPlugDate();
		birthDate = mouse.getBirthDate();
		deathDate = mouse.getDeathDate();
		numberFormat = NumberFormat.getInstance();
		numberFormat.setMaximumFractionDigits(1);
		numberFormat.setMinimumFractionDigits(1);
	}

	/**
	 * Method to indicate if the mouse is dead
	 * @return true if death date of mouse is not null
	 */
	public boolean isDead() {
		return (deathDate == null) ? false :true;
	}

	/**
	 * method to indicate if the mouse is embryonic
	 * @return true if the plug date is present and the birth date is null (i.e. mouse wasn't born)
	 */
	public boolean isEmbryonic() {
		if(plugDate == null) {
			// can't determine embryonic status if litter/plug date is null so assume it isn't.
			return false;
		}
		else {// if mouse hasn't been born it's an embryo
			return (birthDate == null);
		}
	}

	/**
	 * @return the age of the mouse in weeks to one decimal place
	 */
	public float ageInWeeks() {
		return ageInDays()/7f;
	}

	/**
	 * @return the age of the mouse in days always rounded up to .5  (e.g. 4.5 days)
	 */
	public float ageInDays() {
		float ageInDays = 0f;

		Date startDate = (isEmbryonic()) ? plugDate : birthDate;
		Date endDate = (deathDate == null) ? new Date() :deathDate;


		if(startDate != null && endDate != null) {
			ageInDays = .5f + CalendarUtils.diffInDays(startDate, endDate);
		}

		return ageInDays;
	}

	/**
	 * Returns a String representation of mouse age.
	 * <ul>
	 *  <li>If both plug and birth date are null, the age will be uncalculable and returned as "unknown".
	 *  <li>If the mouse is embryonic, the age will be returned in days, suffixed with "E", e.g. 10.5E.
	 *  <li>Otherwise the age will be returned in weeks suffixed with "w", e.g. 5.1w.
	 *  <li>If the mouse is dead the age will be suffixed with "(at death)", e.g. 10.5w (at death).
	 * </ul>
	 * @return String representation of mouse age
	 */
	@Override
	public String toString() {

		String ageString = "";

		if(isEmbryonic()) {
			ageString = numberFormat.format(ageInDays()) + "E";
		}
		else {
			if(birthDate == null) {
				ageString = "unknown";
			}
			else {
				ageString = numberFormat.format(ageInWeeks()) + "w";
			}
			if(isDead()) {
				ageString += " (at death)";
			}
		}

		return ageString;
	}

	public boolean isWithinLifetime(Date date) {
	  return (onOrAfterBirthDate(date) && onOrBeforeDeathDate(date));
	}

  private boolean onOrBeforeDeathDate(Date date) {
    if (deathDate == null) return true;

    if (date.before(deathDate))  return true;

    if (DateUtils.isSameDay(deathDate, date)) return true;

    return false;
  }

  private boolean onOrAfterBirthDate(Date date) {
    if (birthDate == null) return true;

    if (date.after(birthDate)) return true;

    if (DateUtils.isSameDay(birthDate, date)) return true;

    return false;
  }
}
