/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.genotyping.AssayResult;

public class MouseAllele extends AuthenticatedDataObject  {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		ALLELE,
		GENOTYPE_HISTORIES,
		LATEST_ASSAY_RESULTS
	}

    private Long mouseId;
    private Long alleleId;
    private Long genotypeDictId;
    private DictionaryEntry genotype;
    private Allele allele;
    private Collection<GenotypeHistory> genotypeHistories;
    private Collection<AssayResult> latestAssayResults;

    public Long getMouseId() {
        return mouseId;
    }

    public void setMouseId(Long mouseId) {
        this.mouseId = mouseId;
    }
    public Long getAlleleId() {
        return alleleId;
    }

    public void setAlleleId(Long alleleId) {
        this.alleleId = alleleId;
    }
    public Long getGenotypeDictId() {
        return genotypeDictId;
    }

    public void setGenotypeDictId(Long genotypeStatusDictId) {
        this.genotypeDictId = genotypeStatusDictId;
    }

	public DictionaryEntry getGenotype() {
		return genotype;
	}

	public void setGenotype(DictionaryEntry genotypeStatus) {
		this.genotype = genotypeStatus;
	}

	public Allele getAllele() {
		return allele;
	}

	public void setAllele(Allele allele) {
		this.allele = allele;
	}

	public Collection<GenotypeHistory> getGenotypeHistories() {
		return genotypeHistories;
	}

	public void setGenotypeHistories(Collection<GenotypeHistory> genotypeHistories) {
		this.genotypeHistories = genotypeHistories;
	}

	public Collection<AssayResult> getLatestAssayResults() {
		return latestAssayResults;
	}

	public void setLatestAssayResults(Collection<AssayResult> latestAssayResults) {
		this.latestAssayResults = latestAssayResults;
	}

	public String getGenotypeSymbolForMultiAlleleContext(){
		return getGenotypeSymbol(true);
	}

	public String getGenotypeSymbol() {
		return getGenotypeSymbol(false);
	}

	public String getGenotypeSymbol(boolean multiAlleleContext) {
		String genotypeString = "";
		String symbol = (StringUtils.isEmpty(allele.getShortAlleleSymbol())) ? "ERROR" : allele.getShortAlleleSymbol();

		String chromosome1 = "+";
		String chromosome2 = "+";
		boolean isRecessive = (AlleleService.RECESSIVE_ALLELE.equals(allele.getInheritanceModeDictId()));
		boolean isException = false;

		if(GenotypeDictionary.WILDTYPE.equals(genotypeDictId) && multiAlleleContext) {
			isException = true;
			genotypeString = "+/+";
		}
		else if(GenotypeDictionary.HETEROZYGOUS.equals(genotypeDictId)) {
			chromosome1 = symbol;
		}
		else if(GenotypeDictionary.HOMOZYGOUS.equals(genotypeDictId)) {
			chromosome1 = symbol;
			chromosome2 = symbol;
		}
		else if(GenotypeDictionary.HEMIZYGOUS.equals(genotypeDictId)) {
			chromosome1 = symbol;
			chromosome2 = "Y";
		}
		else if(GenotypeDictionary.UNTESTED.equals(genotypeDictId)) {
			isException = true;
			genotypeString = "untested";
		}
		else if(GenotypeDictionary.FAILED.equals(genotypeDictId)) {
			isException = true;
			genotypeString = "failed";
		}
		else if(GenotypeDictionary.CHIMERA.equals(genotypeDictId)) {
			isException = true;
			genotypeString = "chimera";
		}
		else if(GenotypeDictionary.CONDITIONAL_HET.equals(genotypeDictId)) {
			isException = true;
			genotypeString = "c/+";
		}
		else if(GenotypeDictionary.CONDITIONAL_HOM.equals(genotypeDictId)) {
			isException = true;
			genotypeString = "c/-";
		}
		else if(GenotypeDictionary.CONDITIONAL.equals(genotypeDictId)) {
			isException = true;
			genotypeString = "c/c";
		}

		if(isException) {
			genotypeString = symbol + ":" + genotypeString;
		}
		else {
			if(isRecessive) {
				genotypeString = chromosome2 + "/" + chromosome1;
			}
			else {
				genotypeString = chromosome1 + "/" + chromosome2;
			}
		}

		return genotypeString;
	}

	/**
	 * Get aproximations of the mouse allele for each change
	 *
	 * @return
	 */
	public Map<GenotypeHistory,MouseAllele> getHistoryAndAlles() {
		if (genotypeHistories == null) {
			throw new UnattachedComponentException(Component.GENOTYPE_HISTORIES);
		}
		Map<GenotypeHistory,MouseAllele> history = new LinkedHashMap<GenotypeHistory, MouseAllele>();

		for (GenotypeHistory gh : genotypeHistories) {
			MouseAllele oldAllele = new MouseAllele();
			oldAllele.setMouseId(mouseId);
			oldAllele.setAlleleId(alleleId);
			oldAllele.setAllele(allele);

			oldAllele.setGenotypeDictId(gh.getGenotypeDictId());
			oldAllele.setGenotype(gh.getGenotype());

			history.put(gh,oldAllele);
		}

		return history;
	}
}