/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking;

import java.util.Date;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.common.PaginatedSearchBean;
import uk.ac.sanger.mig.gene.MigGene;

public class MouseSearchBean extends PaginatedSearchBean {

	private Long id;
	private String name;
	private String rfidTag;
	private String status;
	private String barcode;
	private String barcodeDescription;
	private String comments;
	private String generation;
	private Integer earmarkLeft;
	private Integer earmarkRight;

	private Long genderDictId;
	private Long coatColourDictId;
	private Long eyeColourDictId;
	private Long causeOfDeathDictId;

	private Long litterId;
	private boolean litterIdPresent;

	private Long earmarkDictId;
	private boolean nullEarmark;

	private Boolean nonBreeder;
	private Boolean sentinel;

	private String colonyName;
	private Long colonyInfoId;

	private String cohortName;
	private Long cohortId;

	private String strainName;
	private Long strainId;

	private String projectLicenceDisplayText;
	private Long projectLicenceId;

	private String protocolDisplayText;
	private Long protocolId;

	private boolean nullCageId;
	private Long cageId;
	private String cageBarcode;
	private String cageBarcodeDescription;

	private Long rackId;
	private String rackBarcode;
	private String rackBarcodeDescription;

	private Long roomId;
	private String roomNumber;

	private String fatherBarcode;
	private Long fatherId;

	private String motherBarcode;
	private Long motherId;

	private Date birthDateFrom;
	private Date birthDateTo;
	private boolean nullBirthDate;

	private Date plugCheckDateFrom;
	private Date plugCheckDateTo;
	private boolean nullPlugCheckDate;

	private Date deathDateFrom;
	private Date deathDateTo;
	private boolean nullDeathDate;

	private Date weanDateFrom;
	private Date weanDateTo;
	private boolean nullWeanDate;

	private Long currentOwnerId;
	private String currentOwnerName;

	private Boolean hasCurrentHealthConcern;
	private Boolean mating;

	private Integer ageInMonths;
	private int ageRangeInWeeks; //specifically 'int' so there's always a value (even if its zero)

	private Allele allele;
	private Long alleleId;
	private Long genotypeDictId;
	private Long microInjectionRequestId;

	private Long reserverId;
	private String reserverName;
	private Long reservationTeamId;
	private String reservationTeamName;
	private Long reservationPurposeId;
	private String reservationPurposeName;
	private Long reservationCategoryId;

	private HistologyWorkProgress histologyProgress;

	public enum HistologyWorkProgress {

		SLIDES_EXIST(     	 		 "Slides Created"			 ),
		SCANS_EXIST(      	 		 "Scanned"        			 ),
		SCANS_QC_EXIST(   			 "Scans QC-ed"    			 ),
		CROPS_EXIST(      	 		 "Scan Regions Selected"     ),
		CROPS_QC_AWAIT(   	 		 "Await Scan Region QC"      ),
		CROPS_QC_EXIST(   	 		 "Scan Regions QC-ed"    	 );

		private String label;

		private HistologyWorkProgress(String label) {this.label = label;}

		public String getLabel() {return this.label;}

		public String getEnumString() { return  this.name();}
	}

	/**
	 * Returns search parameter for finding mice based on how far
	 * work has progressed in the histology pipeline.
	 *
	 * @return
	 */
	public HistologyWorkProgress getHistologyProgress() {
		return this.histologyProgress;
	}

	public void setHistologyProgress(HistologyWorkProgress histologyProgress) {
		this.histologyProgress = histologyProgress;
	}

	public MouseSearchBean() {
		allele = new Allele();
		allele.setMigGene(new MigGene());
	}

	public boolean isMinimumOneFieldSet() {

		return (//short-circuit most commonly queried --ie from simple search
				( barcode != null && barcode.length() > 0 )  ||
				cohortId != null ||
				colonyInfoId != null ||
				rfidTag !=null ||
				strainId != null           ||
				( status != null && status.length() > 0 )          ||
				//advance search strings
				( name != null && name.length() > 0 )              ||
				( comments != null && comments.length() > 0)       ||
				( generation != null && generation.length() > 0)   ||
				//advance search  ids
				genderDictId != null       ||
				coatColourDictId != null   ||
				earmarkDictId != null      ||
				earmarkLeft != null		   ||
				earmarkRight != null	   ||
				eyeColourDictId != null    ||
				causeOfDeathDictId != null ||
				litterId != null           ||
				projectLicenceId != null   ||
				protocolId != null         ||
				cageId != null             ||
				rackId != null             ||
				roomId != null 			   ||
				fatherId != null           ||
				motherId != null           ||
				currentOwnerId != null     ||
				ageInMonths != null		   ||
				alleleId != null		   ||
				genotypeDictId != null	   ||
				microInjectionRequestId != null ||
				//advance search  dates
				birthDateFrom != null      ||
				birthDateTo != null        ||
				deathDateFrom != null      ||
				deathDateTo != null        ||
				weanDateFrom != null       ||
				weanDateTo != null         ||
				//advance search  Booleans
				nonBreeder != null 		   ||
				sentinel != null		   ||
				hasCurrentHealthConcern != null ||
				nullBirthDate != false  ||
				nullDeathDate != false  ||
				nullWeanDate != false   ||
				//advanced search histology
				histologyProgress != null ||
				// reservations
				reservationCategoryId != null ||
				reservationPurposeId != null ||
				reservationTeamId != null ||
				reserverId != null
		);
	}

	public boolean isNullPlugCheckDate() {
		return this.nullPlugCheckDate;
	}

	public void setNullPlugCheckDate(boolean nullPlugCheckDate) {
		this.nullPlugCheckDate = nullPlugCheckDate;
	}

	public Date getPlugCheckDateFrom() {
		return this.plugCheckDateFrom;
	}

	public void setPlugCheckDateFrom(Date plugCheckDateFrom) {
		this.plugCheckDateFrom = plugCheckDateFrom;
	}

	public Date getPlugCheckDateTo() {
		return this.plugCheckDateTo;
	}

	public void setPlugCheckDateTo(Date plugCheckDateTo) {
		this.plugCheckDateTo = plugCheckDateTo;
	}

	public Integer getEarmarkLeft() {
		return earmarkLeft;
	}

	public void setEarmarkLeft(Integer earmarkLeft) {
		this.earmarkLeft = earmarkLeft;
	}

	public Integer getEarmarkRight() {
		return earmarkRight;
	}

	public void setEarmarkRight(Integer earmarkRight) {
		this.earmarkRight = earmarkRight;
	}

	public Integer getAgeInMonths() {
		return ageInMonths;
	}

	public void setAgeInMonths(Integer ageInMonths) {
		this.ageInMonths = ageInMonths;
	}

	public int getAgeRangeInWeeks() {
		return ageRangeInWeeks;
	}

	public void setAgeRangeInWeeks(int ageRangeInWeeks) {
		this.ageRangeInWeeks = ageRangeInWeeks;
	}

	public Boolean getHasCurrentHealthConcern() {
		return hasCurrentHealthConcern;
	}

	public void setHasCurrentHealthConcern(Boolean hasCurrentHealthConcern) {
		this.hasCurrentHealthConcern = hasCurrentHealthConcern;
	}

	public Long getColonyInfoId() {
		return colonyInfoId;
	}

	public void setColonyInfoId(Long colonyInfoId) {
		this.colonyInfoId = colonyInfoId;
	}

	public String getColonyName() {
		return colonyName;
	}

	public void setColonyName(String colonyInfoName) {
		this.colonyName = colonyInfoName;
	}

	public Long getStrainId() {
		return strainId;
	}

	public void setStrainId(Long strainId) {
		this.strainId = strainId;
	}

	public String getStrainName() {
		return strainName;
	}

	public void setStrainName(String strainName) {
		this.strainName = strainName;
	}

	public boolean isNullDeathDate() {
		return nullDeathDate;
	}

	public void setNullDeathDate(boolean nullDeathDate) {
		this.nullDeathDate = nullDeathDate;
	}

	public Date getBirthDateFrom() {
		return birthDateFrom;
	}

	public void setBirthDateFrom(Date birthDateFrom) {
		this.birthDateFrom = birthDateFrom;
	}

	public Date getBirthDateTo() {
		return birthDateTo;
	}

	public void setBirthDateTo(Date birthDateTo) {
		this.birthDateTo = birthDateTo;
	}

	public Date getDeathDateFrom() {
		return deathDateFrom;
	}

	public void setDeathDateFrom(Date deathDateFrom) {
		this.deathDateFrom = deathDateFrom;
	}

	public Date getDeathDateTo() {
		return deathDateTo;
	}

	public void setDeathDateTo(Date deathDateTo) {
		this.deathDateTo = deathDateTo;
	}

	public String getFatherBarcode() {
		return fatherBarcode;
	}

	public void setFatherBarcode(String fatherBarcode) {
		this.fatherBarcode = fatherBarcode;
	}

	public Long getFatherId() {
		return fatherId;
	}

	public void setFatherId(Long fatherId) {
		this.fatherId = fatherId;
	}

	public String getMotherBarcode() {
		return motherBarcode;
	}

	public void setMotherBarcode(String motherBarcode) {
		this.motherBarcode = motherBarcode;
	}

	public Long getMotherId() {
		return motherId;
	}

	public void setMotherId(Long motherId) {
		this.motherId = motherId;
	}

	public boolean isNullBirthDate() {
		return nullBirthDate;
	}

	public void setNullBirthDate(boolean nullBirthDate) {
		this.nullBirthDate = nullBirthDate;
	}

	public boolean isNullCageId() {
		return nullCageId;
	}

	public void setNullCageId(boolean nullCageId) {
		this.nullCageId = nullCageId;
	}

	public boolean isNullWeanDate() {
		return nullWeanDate;
	}

	public void setNullWeanDate(boolean nullWeanDate) {
		this.nullWeanDate = nullWeanDate;
	}

	public Date getWeanDateFrom() {
		return weanDateFrom;
	}

	public void setWeanDateFrom(Date weanDateFrom) {
		this.weanDateFrom = weanDateFrom;
	}

	public Date getWeanDateTo() {
		return weanDateTo;
	}

	public void setWeanDateTo(Date weanDateTo) {
		this.weanDateTo = weanDateTo;
	}

	public String getCageBarcode() {
		return cageBarcode;
	}

	public void setCageBarcode(String cageBarcode) {
		this.cageBarcode = cageBarcode;
	}

	public String getCageBarcodeDescription() {
		return cageBarcodeDescription;
	}

	public void setCageBarcodeDescription(String cageBarcodeDescription) {
		this.cageBarcodeDescription = cageBarcodeDescription;
	}

	public String getRackBarcode() {
		return rackBarcode;
	}

	public void setRackBarcode(String rackBarcode) {
		this.rackBarcode = rackBarcode;
	}

	public String getRackBarcodeDescription() {
		return rackBarcodeDescription;
	}

	public void setRackBarcodeDescription(String rackBarcodeDescription) {
		this.rackBarcodeDescription = rackBarcodeDescription;
	}

	public Long getRackId() {
		return rackId;
	}

	public void setRackId(Long rackId) {
		this.rackId = rackId;
	}

	public Long getRoomId() {
		return roomId;
	}

	public void setRoomId(Long roomId) {
		this.roomId = roomId;
	}

	public String getCurrentOwnerName() {
		return currentOwnerName;
	}

	public void setCurrentOwnerName(String currentOwnerName) {
		this.currentOwnerName = currentOwnerName;
	}

	public String getProjectLicenceDisplayText() {
		return projectLicenceDisplayText;
	}

	public void setProjectLicenceDisplayText(String projectLicenceDisplayText) {
		this.projectLicenceDisplayText = projectLicenceDisplayText;
	}

	public String getProtocolDisplayText() {
		return protocolDisplayText;
	}

	public void setProtocolDisplayText(String protocolDisplayText) {
		this.protocolDisplayText = protocolDisplayText;
	}

	public Long getProjectLicenceId() {
		return projectLicenceId;
	}

	public void setProjectLicenceId(Long projectLicenceId) {
		this.projectLicenceId = projectLicenceId;
	}

	public Long getProtocolId() {
		return protocolId;
	}

	public void setProtocolId(Long protocolId) {
		this.protocolId = protocolId;
	}

	public String getBarcode() {
		return barcode;
	}

	public void setBarcode(String barcode) {
		this.barcode = barcode;
	}

	public String getBarcodeDescription() {
		return barcodeDescription;
	}

	public void setBarcodeDescription(String barcodeDescription) {
		this.barcodeDescription = barcodeDescription;
	}

	public Long getCageId() {
		return cageId;
	}

	public void setCageId(Long cageId) {
		this.cageId = cageId;
	}

	public String getComments() {
		return comments;
	}

	public void setComments(String comments) {
		this.comments = comments;
	}

	public Long getCurrentOwnerId() {
		return currentOwnerId;
	}

	public void setCurrentOwnerId(Long currentOwnerId) {
		this.currentOwnerId = currentOwnerId;
	}

	public String getGeneration() {
		return generation;
	}

	public void setGeneration(String generation) {
		this.generation = generation;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Boolean getNonBreeder() {
		return nonBreeder;
	}

	public void setNonBreeder(Boolean nonBreeder) {
		this.nonBreeder = nonBreeder;
	}

	public String getRfidTag() {
		return rfidTag;
	}

	public void setRfidTag(String rfidTag) {
		this.rfidTag = rfidTag;
	}

	public Boolean getSentinel() {
		return sentinel;
	}

	public void setSentinel(Boolean sentinel) {
		this.sentinel = sentinel;
	}

	public String getStatus() {
		return status;
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public Long getCoatColourDictId() {
		return coatColourDictId;
	}

	public void setCoatColourDictId(Long coatColourDictId) {
		this.coatColourDictId = coatColourDictId;
	}

	public Long getEarmarkDictId() {
		return earmarkDictId;
	}

	public void setEarmarkDictId(Long earmarkDictId) {
		this.earmarkDictId = earmarkDictId;
	}

	public Long getEyeColourDictId() {
		return eyeColourDictId;
	}

	public void setEyeColourDictId(Long eyeColourDictId) {
		this.eyeColourDictId = eyeColourDictId;
	}

	public Long getGenderDictId() {
		return genderDictId;
	}

	public void setGenderDictId(Long genderDictId) {
		this.genderDictId = genderDictId;
	}

	public Long getCauseOfDeathDictId() {
		return causeOfDeathDictId;
	}

	public void setCauseOfDeathDictId(Long causeOfDeathDictId) {
		this.causeOfDeathDictId = causeOfDeathDictId;
	}

	public Long getLitterId() {
		return litterId;
	}

	public void setLitterId(Long litterId) {
		this.litterId = litterId;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Boolean getMating() {
		return mating;
	}

	public void setMating(Boolean mating) {
		this.mating = mating;
	}

	public Long getCohortId() {
		return cohortId;
	}

	public void setCohortId(Long cohortId) {
		this.cohortId = cohortId;
	}

	public String getCohortName() {
		return cohortName;
	}

	public void setCohortName(String cohortName) {
		this.cohortName = cohortName;
	}

	public Long getAlleleId() {
		return alleleId;
	}

	public void setAlleleId(Long alleleId) {
		this.alleleId = alleleId;
	}

	public Long getGenotypeDictId() {
		return genotypeDictId;
	}

	public void setGenotypeDictId(Long genotypeDictId) {
		this.genotypeDictId = genotypeDictId;
	}

	public Allele getAllele() {
		return allele;
	}

	public void setAllele(Allele allele) {
		this.allele = allele;
	}

	public Long getMicroInjectionRequestId() {
		return microInjectionRequestId;
	}

	public void setMicroInjectionRequestId(Long microInjectionRequestId) {
		this.microInjectionRequestId = microInjectionRequestId;
	}

	public String getRoomNumber() {
		return roomNumber;
	}

	public void setRoomNumber(String roomNumber) {
		this.roomNumber = roomNumber;
	}

	public Long getReservationCategoryId() {
		return reservationCategoryId;
	}

	public void setReservationCategoryId(Long reservationCategoryId) {
		this.reservationCategoryId = reservationCategoryId;
	}

	public Long getReservationPurposeId() {
		return reservationPurposeId;
	}

	public void setReservationPurposeId(Long reservationPurposeId) {
		this.reservationPurposeId = reservationPurposeId;
	}

	public String getReservationPurposeName() {
		return reservationPurposeName;
	}

	public void setReservationPurposeName(String reservationPurposeName) {
		this.reservationPurposeName = reservationPurposeName;
	}

	public Long getReservationTeamId() {
		return reservationTeamId;
	}

	public void setReservationTeamId(Long reservationTeamId) {
		this.reservationTeamId = reservationTeamId;
	}

	public String getReservationTeamName() {
		return reservationTeamName;
	}

	public void setReservationTeamName(String reservationTeamName) {
		this.reservationTeamName = reservationTeamName;
	}

	public Long getReserverId() {
		return reserverId;
	}

	public void setReserverId(Long reserverId) {
		this.reserverId = reserverId;
	}

	public String getReserverName() {
		return reserverName;
	}

	public void setReserverName(String reserverName) {
		this.reserverName = reserverName;
	}

	public boolean isNullEarmark() {
		return nullEarmark;
	}

	public void setNullEarmark(boolean nullEarmark) {
		this.nullEarmark = nullEarmark;
	}

	public boolean isLitterIdPresent() {
		return litterIdPresent;
	}

	public void setLitterIdPresent(boolean litterIdPresent) {
		this.litterIdPresent = litterIdPresent;
	}

}
