/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections15.FactoryUtils;
import org.apache.commons.collections15.ListUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.controllers.AbstractFormChangeController;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.forms.AddBreedingForm;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.personnel.Person;

public class AddBreedingFormController extends AbstractFormChangeController {

	private BreedingService breedingService;
	private DictionaryService dictionaryService;

	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {

		AddBreedingForm form = (AddBreedingForm)command;
		Person user = RequestUtils.getUser(request);

		Long cageId = breedingService.addManualBreeding(form, user);

		return new ModelAndView(getSuccessView() + "?id=" + cageId);
	}

	@Override
	protected Map referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		Map<String, Object> referenceData = new HashMap<String, Object>();
		AddBreedingForm form = (AddBreedingForm)command;


		// List of mating numbers
		Collection<Long> breedingNumbers = breedingService.fetchBreedingNumbersByColonyId(form.getOffspringColonyId());
		referenceData.put("breedingNumbers", breedingNumbers);

		// List of embryo ages for timed matings
		Map<Float, String> embryoAges = new LinkedHashMap<Float, String>();
		for(float i=0.5f; i < 21; i++) {
			embryoAges.put(i, i+"E");
		}
		referenceData.put("timedEmbryoAges", embryoAges);

		// Breeding Categories
		Collection<DictionaryEntry> categories = dictionaryService.fetch(DictionaryEnum.MOUSE_BREEDING_CATEGORY);
		referenceData.put("categories", categories);

		return referenceData;
	}

	@Override
	protected Object formBackingObject(HttpServletRequest arg0) throws Exception {
		AddBreedingForm form = (AddBreedingForm)createCommand();

		return form;
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		AddBreedingForm form = (AddBreedingForm)command;

		String changeForm = ServletRequestUtils.getStringParameter(request, "changeForm");

		if(changeForm.equals("fetchMatingNumbers")) {
			// reset the breeding number to be null so the user gets the next available one in the list
			form.setBreedingNumbers(ListUtils.lazyList(new ArrayList<Long>(), FactoryUtils.instantiateFactory(Long.class, new Class[]{long.class}, new Object[]{0})));

			// set the name prefix for each breeding to be the current colony prefix
			form.setNamePrefixes(ListUtils.lazyList(new ArrayList<String>(), FactoryUtils.instantiateFactory(String.class)));
			for(int i=0; i<form.getFemales().size(); i++) {
				form.getNamePrefixes().add(form.getOffspringColonyPrefix());
			}
		}
		else if(changeForm.equals("parentScanned")) {
			Long maleId = form.getMale().getId();
			Long femaleId = form.getFemales().iterator().next().getId();
			if(maleId != null && femaleId != null) {
				form.setOffspringGeneration(breedingService.nextGeneration(maleId, femaleId));
			}
		}
	}
}
