/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.forms.BarcodeForm;
import uk.ac.sanger.mig.mousetracking.forms.BreedingForm;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.personnel.Person;

public class EditBreedingFormController extends AbstractCancellableFormController {

	private BreedingService breedingService;
	private DictionaryService dictionaryService;
	private String redirectCageView;
	private String redirectBreedingView;

	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setRedirectBreedingView(String redirectBreedingView) {
		this.redirectBreedingView = redirectBreedingView;
	}

	public void setRedirectCageView(String redirectCageView) {
		this.redirectCageView = redirectCageView;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {

		BreedingForm form = (BreedingForm)command;
		Breeding breeding = form.getBreeding();

		Person user = RequestUtils.getUser(request);
		breedingService.editBreeding(form, user);

		if(form.getRedirectView() != null && form.getRedirectView().equalsIgnoreCase("cage")) {
			Breeding updatedBreeding = breedingService.fetchById(breeding.getId(), ComponentMap.of(Breeding.Component.FEMALE));
			if(updatedBreeding.getFemale().getCageId() == null) {
				return new ModelAndView(redirectBreedingView, "id", breeding.getId());
			}
			else {
				return new ModelAndView(redirectCageView, "id", updatedBreeding.getFemale().getCageId());
			}
		}
		else {
			return new ModelAndView(redirectBreedingView, "id", breeding.getId());
		}
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		BreedingForm breedingForm = (BreedingForm)createCommand();

		Long id = ServletRequestUtils.getRequiredLongParameter(request, "id");

		Breeding breeding = breedingService.fetchById(id, ComponentMap.of(Breeding.Component.ALL_COMPONENTS));
		breedingService.inflateBreedingComponents(breeding);

		breedingForm.setBreeding(breeding);
		breedingForm.setMaleLocation(new BarcodeForm());
		breedingForm.setFemaleLocation(new BarcodeForm());
		return breedingForm;
	}

	@Override
	protected Map<String, Object> referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		BreedingForm form = (BreedingForm)command;

		Map<String, Object> referenceData = new HashMap<String, Object>();

		Cage cage = breedingService.fetchForCageView(form.getBreeding().getId());
		referenceData.put("cage",cage);

		//Yes/No list
		Map<Boolean, String> yesNo = new HashMap<Boolean, String>();
		yesNo.put(false,"No");
		yesNo.put(true,"Yes");
		referenceData.put("yesNo",yesNo);


		// List of embryo ages for timed matings
		Map<Float, String> embryoAges = new LinkedHashMap<Float, String>();
		for(float i=0.5f; i < 21; i++) {
			embryoAges.put(i, i+"E");
		}
		referenceData.put("timedEmbryoAges", embryoAges);

		// Breeding Categories
		Collection<DictionaryEntry> categories = dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_BREEDING_CATEGORY, form.getBreeding().getBreedingCategoryDictId());
		referenceData.put("categories", categories);

		return referenceData;
	}
}
