/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.forms.BarcodeForm;
import uk.ac.sanger.mig.mousetracking.forms.LitterForm;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.forms.LitterForm.MODE;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.mousetracking.services.LitterService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class EditLitterFormController extends AbstractCancellableFormController {

	private MouseService mouseService;
	private LitterService litterService;
	private BreedingService breedingService;
	private DictionaryService dictionaryService;
	private ComponentMapService componentMapService;
	private MouseProcedureService mouseProcedureService;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setLitterService(LitterService litterService) {
		this.litterService = litterService;
	}

	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	private int pupsOnForm(LitterForm form) {
		return form.getLitter().getNumberBornInt() + form.getLitter().getNumberHarvestedInt();
	}

	private int pupsFlaggedForDelete(LitterForm form) {
		int numberFlaggedForDelete = 0;
		for(Mouse mouse : form.getLitter().getPups()) {
			if(mouse.isFlaggedForDelete()) {
				numberFlaggedForDelete++;
			}
		}
		return numberFlaggedForDelete;
	}

	private int pupsInCollection(LitterForm form) {
		return form.getLitter().getPups().size();
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		String changeRequest = ServletRequestUtils.getStringParameter(request, "changeForm", "");

		return (StringUtils.isNotEmpty(changeRequest));
	}

	@Override
	protected void onFormChange(HttpServletRequest request,	HttpServletResponse response, Object command, BindException errors) throws Exception {
		LitterForm form = (LitterForm)command;
		Litter litter = form.getLitter();
		Collection<Mouse> pups = litter.getPups();
		Mouse defaultPup = form.getDefaultPup();
		Person user = RequestUtils.getUser(request);

		if(form.getChangeForm().equalsIgnoreCase("setModeBorn")) {
			form.setMode(MODE.BORN);
		}
		else if(form.getChangeForm().equalsIgnoreCase("setModeHarvested")) {
			form.setMode(MODE.HARVESTED);
			form.setCullFemaleConfirmationRequired(true);
			setDefault19b(form, errors);
		}
		else if(form.getChangeForm().equals("editNumberPups")) {

			if(pupsOnForm(form) > (pupsInCollection(form) - pupsFlaggedForDelete(form))) {
				addPups(form, errors);
				setDefault19b(form, errors);
			}
			if(pupsOnForm(form) < pupsInCollection(form)) {
				removePups(form, errors);
			}
		}
		else if(form.getChangeForm().equals("copyCoatColour")) {
			for(Mouse pup : pups) {
				if(pup.getCoatColourDictId() == null) {
					pup.setCoatColourDictId(defaultPup.getCoatColourDictId());
				}
			}
		}
		else if(form.getChangeForm().equals("copyCauseOfDeath")) {
			for(Mouse pup : pups) {
				if(pup.getCauseOfDeathDictId() == null) {
					pup.setCauseOfDeathDictId(defaultPup.getCauseOfDeathDictId());
				}
			}
		}
		else if(form.getChangeForm().equals("copyDeathDate")) {
			for(Mouse pup : pups) {
				if(pup.getDeathDate() == null) {
					pup.setDeathDate(defaultPup.getDeathDate());
				}
			}
		}
		else if(form.getChangeForm().equals("copyWeanDate")) {
			for(Mouse pup : pups) {
				if(pup.getWeanDate() == null && pup.getDeathDate() == null) { //only wean live pups
					pup.setWeanDate(defaultPup.getWeanDate());
				}
			}
			if(litter.isFromMicroInjection() && litter.getBreeding().getFemale().isAlive() && defaultPup.getWeanDate() != null) {
				form.setCullFemaleConfirmationRequired(true);
			}
		}
		else if(form.getChangeForm().equals("setMaleCount")) {
			if(form.getMaleCountOnForm() == null) {
				for(Mouse pup : pups) {
					if(pup.isAlive()) {
						pup.setGenderDictId(MouseService.MOUSE_GENDER_UNDEFINED);
					}
				}
				form.setFemaleCountOnForm(null);
			}
			else {
				int maleCount = 0;
				int femaleCount = 0;
				for(Mouse pup : pups) {
					if(pup.isAlive() && maleCount < form.getMaleCountOnForm()) {
						maleCount++;
						pup.setGenderDictId(MouseService.MOUSE_GENDER_MALE);
					}
					else if(pup.isAlive()) {
						femaleCount++;
						pup.setGenderDictId(MouseService.MOUSE_GENDER_FEMALE);
					}
				}
				form.setMaleCountOnForm(new Long(maleCount));
				form.setFemaleCountOnForm(new Long(femaleCount));
			}
		}
		else if(form.getChangeForm().contains("addNewProcedure")) {
			addNewProcedure(form, user);
		}

		// reset form change attributes
		form.setChangeForm(null);
		form.setChangeFormValue(null);
	}

	private void addNewProcedure(LitterForm form, Person user) {
		form.setAddNewProcedure(true);
		if(form.getChangeForm().contains("EarclipForId")) {
			addEarclipForId(form, user);
		}
		else if(form.getChangeForm().contains("EarclipForGenotype")) {
			addEarclipForGenotyping(form, user);
		}
	}

	private void addEarclipForId(LitterForm form, Person user) {
		DictionaryEntry earclipProc = dictionaryService.fetchEntry(DictionaryEnum.MOUSE_PROCEDURE_DICT, MouseProcedureService.EARCLIP_FOR_ID);
		form.getProcedureSubform().setProcedureDisplayText(earclipProc.getDisplayText());
		form.getProcedureSubform().setProcedureId(earclipProc.getId());

		if(user.getPersonalLicence() != null) {
			form.getProcedureSubform().setPersonalLicenceId(user.getPersonalLicence().getId());
			form.getProcedureSubform().setPersonalLicenceDisplayText(user.getPersonalLicence().getDisplayText());
		}
	}

	private void addEarclipForGenotyping(LitterForm form, Person user) {
		DictionaryEntry earclipProc = null;
		if(form.getLitter().getBreeding().getOffspringGeneration() != null && form.getLitter().getBreeding().getOffspringGeneration().equals("F1")) {
			earclipProc = dictionaryService.fetchEntry(DictionaryEnum.MOUSE_PROCEDURE_DICT, MouseProcedureService.EARCLIP_FOR_GLT_GENOTYPING);
		}
		else {
			earclipProc = dictionaryService.fetchEntry(DictionaryEnum.MOUSE_PROCEDURE_DICT, MouseProcedureService.EARCLIP_FOR_GENOTYPING);
		}

		form.getProcedureSubform().setProcedureDisplayText(earclipProc.getDisplayText());
		form.getProcedureSubform().setProcedureId(earclipProc.getId());

		if(user.getPersonalLicence() != null) {
			form.getProcedureSubform().setPersonalLicenceId(user.getPersonalLicence().getId());
			form.getProcedureSubform().setPersonalLicenceDisplayText(user.getPersonalLicence().getDisplayText());
		}

		Litter litter = form.getLitter();

		Mouse mother = litter.getBreeding().getFemale();
		MouseProcedure motherProc = mother.getCurrentLicencedProcedure();

		if(motherProc != null) {
			Long projectLicenceId = motherProc.getProtocol().getProjectLicenceId();
			Long personId = mother.getCurrentOwnerId();

			MouseProcedure default19b = mouseProcedureService.createMouseProcedure(earclipProc.getId(), projectLicenceId, false, personId);

			form.getProcedureSubform().setProtocolId(default19b.getProtocolId());
			form.getProcedureSubform().setProtocolDisplayText(default19b.getProtocol().getDisplayText());

			form.getProcedureSubform().setProjectLicenceId(default19b.getProtocol().getProjectLicenceId());
			form.getProcedureSubform().setProjectLicenceDisplayText(default19b.getProtocol().getProjectLicence().getDisplayText());
		}
	}

	private void setDefault19b(LitterForm form, BindException errors) {
		String returnMessage = litterService.setDefault19b(form);
		if(returnMessage != null) {
			errors.reject(null, returnMessage);
		}

		// if the pups are transgenic, then a procedure is required
		if(form.getDefaultPup().isMutant() || form.getMode() == MODE.HARVESTED) {
			form.setProcedureRequired(true);
		}
	}

	private void addPups(LitterForm form, BindException errors) {
		if(form.getLitter().isNotPregnant()) {
			if(errors != null) {
				errors.reject(null, "Can't add pups - female was not pregnant.");
			}
		}
		else if(form.getLitter().getNumberBornInt() > 26 || form.getLitter().getNumberHarvestedInt() > 26) {
			if(errors != null) {
				errors.reject(null, "Too many pups in this litter!");
			}
			form.getLitter().setNumberBorn(form.getLitter().getPups().size());
		}
		else {
			while((pupsInCollection(form) - pupsFlaggedForDelete(form)) < pupsOnForm(form)) {
				if(pupsFlaggedForDelete(form) > 0) {
					for(Mouse mouse: form.getLitter().getPups()) {
						if(mouse.isFlaggedForDelete()) {
							mouse.setFlaggedForDelete(false);
						}
					}
				}
				else {
					Mouse pup = litterService.createPup(form);
					if(pup.isNewRecord()) {
						Mouse tempMouse = mouseService.fetchByName(pup.getName(), null);
						if(tempMouse != null && errors != null) {
							errors.reject(null, "A mouse named " + tempMouse.getName() + " already exists on the database. Cannot continue until this duplication is resolved.");
						}
					}
					form.getLitter().getPups().add(pup);
				}
			}
		}
	}
	/**
	 * Can't remove a mouse that has been identified by earmark or gender or has a wean/death date.
	 * @param mouse
	 * @return
	 */
	private boolean isRemovable(Mouse mouse) {
		if((mouse.getEarmarkDictId() != null && !MouseService.NO_EARMARK.equals(mouse.getEarmarkDictId())) ||
			!MouseService.MOUSE_GENDER_UNDEFINED.equals(mouse.getGenderDictId()) ||
			mouse.getDeathDate() != null ||
			mouse.getWeanDate() != null
			) {
			return false;
		}
		else {
			return true;
		}
	}

	private void removePups(LitterForm form, BindException errors) {
		Mouse[] pups = form.getLitter().getPups().toArray(new Mouse[0]);
		int lastPupIndex = pups.length-1;

		while((pupsInCollection(form) - pupsFlaggedForDelete(form)) > pupsOnForm(form)) {
			Mouse lastPup = pups[lastPupIndex];
			if(isRemovable(lastPup)) {
				if(lastPup.getId() == null) {
					lastPup.setId(-1L);
					form.getLitter().getPups().remove(lastPup);
					pups = form.getLitter().getPups().toArray(new Mouse[0]);
					lastPupIndex = pups.length-1;
				}
				else {
					lastPup.setFlaggedForDelete(true);
					lastPupIndex--;
				}
			}
			else {
				errors.reject(null, "Can't remove all pups - some are uniquely identified by gender or earmark or have been weaned.");
				form.getLitter().setNumberBorn(form.getLitter().getPups().size());
				break;
			}
		}
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		LitterForm form = (LitterForm)command;

		Litter litter = form.getLitter();
		Person user = RequestUtils.getUser(request);

		litterService.processLitterForm(form, user);

		return new ModelAndView(getSuccessView(), "id", litter.getId());
	}



	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		RequestUtils.setLastReferer(request);

		LitterForm form = (LitterForm)createCommand();

		Long id = ServletRequestUtils.getLongParameter(request, "id");
		Long breedingId = ServletRequestUtils.getLongParameter(request, "litter.breedingId");
		String splitMode = ServletRequestUtils.getStringParameter(request, "splitMode");

		Litter litter = new Litter();
		if(breedingId != null) {
			litter.setBreedingId(breedingId);
		}
		ComponentMap breedingComponents = componentMapService.getComponentMap("breedingForEditLitter");

		// for new litters, set the litter name
		if(id == null) {
			litterService.makeLitterName(litter);
		}
		// for existing litters, fetch from the database with all components
		else {
			ComponentMap litterComponents = componentMapService.getComponentMap("editLitter");
			litter = litterService.fetchById(id, litterComponents);
		}

		if(litter.getBreedingId() != null) {
			breedingId = litter.getBreedingId();
		}

		Breeding breeding = breedingService.fetchById(breedingId, breedingComponents);
		breedingService.inflateBreedingComponents(breeding);
		litter.setBreeding(breeding);

		litterService.inflateLitterComponents(litter);
		form.setLitter(litter);

		Mouse defaultPup = litterService.getDefaultPupValues(litter.getBreeding());

		form.setDefaultPup(defaultPup);

		// this is nasty and should be refactored!
		// Basically it's here to attempt to find a default genetic background (strain) if one
		// has already been set in the pups collection. This should be refactored so that for
		// matings with different parental strains, the offspring strain is stored in the mating
		// Without this, the user is repeatedly asked to enter the strain each time the litter is edited
		if(defaultPup.getStrainDictId() == null && litter.getPups() != null && litter.getPups().size() > 0) {
			Long defaultStrainDictId = litter.getPups().iterator().next().getStrainDictId();
			defaultPup.setStrainDictId(defaultStrainDictId);
		}
		// end of nastiness

		if(defaultPup.getColonyInfoId() == null) {
			form.setColonyRequired(true);
		}

		if(defaultPup.getStrainDictId() == null) {
			form.setStrainRequired(true);
		}

		if(defaultPup.getGeneration() == null) {
			form.setGenerationRequired(true);
		}

		// inflate pups
		for(Mouse pup : litter.getPups()) {
			mouseService.inflateMouseComponents(pup);
		}

		// inflate barcode objects
		form.setMaleLocation(new BarcodeForm());
		form.setFemaleLocation(new BarcodeForm());
		form.setUnsexedLocation(new BarcodeForm());

		// reset form change attributes
		form.setChangeForm(null);
		form.setChangeFormValue(null);

		// set procedure subform
		form.setProcedureSubform(new ProcedureSubform());

		// set display mode
		// litters from embryo-transfer don't get the option to harvest, it doesn't make sense.
		if(splitMode != null) {
			form.setMode(MODE.SPLIT);
		}
		else if(form.getLitter().getBirthDate() != null || form.getLitter().isFromEmbryoTransfer()) {
			form.setMode(MODE.BORN);
		}
		else if(form.getLitter().getNumberHarvested() != null) {
			form.setMode(MODE.HARVESTED);
		}
		else {
			form.setMode(MODE.NEW);
		}

		/*
		 * For legacy data there may be instances where there is a number for pups born
		 * but no actual mice records, this corrects the problem by adding the mice
		 * to the form at the start of editing. The mice will only be persisted if the form
		 * is submitted.
		 */
		if(litter.getNumberBornInt() > litter.getPups().size()) {
			addPups(form,null);
		}

		return form;
	}

	@Override
	protected Map<String, Object> referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		LitterForm form = (LitterForm)command;

		Map<String, Object> referenceData = new HashMap<String, Object>();

		Cage breedingCage = breedingService.fetchForCageView(form.getLitter().getBreedingId());
		referenceData.put("cage", breedingCage);

		referenceData.put("causeOfDeath", dictionaryService.fetch(DictionaryEnum.MOUSE_CAUSE_OF_DEATH));
		referenceData.put("coatColour", dictionaryService.fetch(DictionaryEnum.MOUSE_COAT_COLOUR));
		referenceData.put("earmark", dictionaryService.fetch(DictionaryEnum.MOUSE_EARMARK_DICT));
		referenceData.put("gender", dictionaryService.fetch(DictionaryEnum.MOUSE_GENDER_DICT));
		referenceData.put("genotype", dictionaryService.fetch(DictionaryEnum.MOUSE_GENOTYPE));

		referenceData.put("currentRecordId", form.getLitter().getId());

		//Yes/No list
		Map<Boolean, String> yesNo = new HashMap<Boolean, String>();
		yesNo.put(false,"No");
		yesNo.put(true,"Yes");
		referenceData.put("yesNo",yesNo);

		return referenceData;
	}

	@Override
	protected ModelAndView handleInvalidSubmit(HttpServletRequest request, HttpServletResponse response) throws Exception {
		return new ModelAndView("errors/handle_invalid_submit");
	}

}
