/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.forms.EditMiceForm;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.services.CageService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class EditMiceByCageController extends AbstractCancellableFormController {

	private ComponentMapService componentMapService;
	private CageService cageService;
	private DictionaryService dictionaryService;
	private MouseService mouseService;
	private MouseProcedureService mouseProcedureService;

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		String changeRequest = ServletRequestUtils.getStringParameter(request, "changeForm", "");

		return (StringUtils.isNotEmpty(changeRequest));
	}

	protected void doSubmit(EditMiceForm form, Person user) {
		cloneMouseProcedure(form);
		cloneMouseFields(form);
		mouseService.store(form.getMice(), user);
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		EditMiceForm form = (EditMiceForm)command;
		Person user = RequestUtils.getUser(request);
		doSubmit(form, user);
		return new ModelAndView(getSuccessView(), "id", form.getCage().getId());
	}

	@Override
	protected Map referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		Map<String, Object> referenceData = new HashMap<String, Object>();

		referenceData.put("causeOfDeath", dictionaryService.fetch(DictionaryEnum.MOUSE_CAUSE_OF_DEATH, false));
		referenceData.put("coatColour", dictionaryService.fetch(DictionaryEnum.MOUSE_COAT_COLOUR, false));
		referenceData.put("earmark", dictionaryService.fetch(DictionaryEnum.MOUSE_EARMARK_DICT, false));
		referenceData.put("genotype", dictionaryService.fetch(DictionaryEnum.MOUSE_GENOTYPE, false));

		return referenceData;
	}

	protected void setCageOrCohort(EditMiceForm form, Long cageOrCohortId) {
		ComponentMap cageComponents = componentMapService.getComponentMap("editMiceByCage");
		Cage cage = cageService.fetchById(cageOrCohortId, cageComponents);
		form.setCage(cage);
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		RequestUtils.setLastReferer(request);

		EditMiceForm form = (EditMiceForm)createCommand();

		Long cageOrCohortId = ServletRequestUtils.getRequiredLongParameter(request, "id");
		setCageOrCohort(form, cageOrCohortId);

		Mouse defaultValues = new Mouse();
		mouseService.inflateMouseComponents(defaultValues);

		for(Mouse mouse : form.getMice()) {
			if(mouse.isHarmfulMutant()) {
				form.setHarmfulMutant(true);
				break;
			}
		}

		form.setDefaultMouseValues(defaultValues);
		form.setProcedureSubform(new ProcedureSubform());

		return form;
	}

	private void cloneMouseProcedure(EditMiceForm form) {
		if(form.getProcedureSubform().getProcedureId() != null) {
			ProcedureSubform subform = form.getProcedureSubform();
			for(Mouse mouse : form.getMice()) {
				MouseProcedure mouseProcedure = mouseProcedureService.createFromProcedureSubform(subform);
				/**
				 * Only add the procedure if the mouse is alive, the procedure is legacy data or
				 * that the procedure is allowed after the mouse death date.
				 */
				if(mouse.isAlive() || (!mouse.isAlive() && (mouseProcedure.getStartDate().before(mouse.getDeathDate()) ||
						mouseProcedureService.isAllowablePostMortem(mouseProcedure.getProcedureDictId())))) {
						mouse.setCurrentLicencedProcedure(mouseProcedure);
				}
			}
		}
	}

	private void cloneMouseFields(EditMiceForm form) {
		Mouse defaultValues = form.getDefaultMouseValues();
		for(Mouse mouse : form.getMice()) {
			if(defaultValues.getCurrentOwnerId() != null) {
				mouse.setCurrentOwnerId(defaultValues.getCurrentOwnerId());
			}
		}
	}

	@Override
	protected ModelAndView handleInvalidSubmit(HttpServletRequest request, HttpServletResponse response) throws Exception {
		return new ModelAndView("errors/handle_invalid_submit");
	}
}
