/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections15.FactoryUtils;
import org.apache.commons.collections15.ListUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.mousetracking.ExperimentData;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.forms.MouseForm;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.services.ExperimentDataService;
import uk.ac.sanger.mig.mousetracking.services.MouseAlleleService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class EditMouseFormController extends AbstractCancellableFormController {

	private DictionaryService		dictionaryService;
	private MouseAlleleService		mouseAlleleService;
	private MouseService			mouseService;
	private String					multiMouseView;
	private MouseProcedureService	mouseProcedureService;
	private ExperimentDataService	experimentDataService;
	private ComponentMapService		componentMapService;

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setMouseAlleleService(MouseAlleleService mouseAlleleService) {
		this.mouseAlleleService = mouseAlleleService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setMultiMouseView(String multiMouseView) {
		this.multiMouseView = multiMouseView;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	public void setExperimentDataService(ExperimentDataService experimentDataService) {
		this.experimentDataService = experimentDataService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		super.initBinder(request, binder);

		binder.registerCustomEditor(Date.class, "mouse.currentWeight.sampleDate", new CustomDateEditor(
				new SimpleDateFormat("dd-MMM-yyyy HH:mm"), true));
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		String changeRequest = ServletRequestUtils.getStringParameter(request, "changeForm", "");

		return (StringUtils.isNotEmpty(changeRequest));
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command)
			throws Exception {
		MouseForm mouseForm = (MouseForm) command;
		String changeRequest = ServletRequestUtils.getRequiredStringParameter(request, "changeForm");
		String changeValue = ServletRequestUtils.getRequiredStringParameter(request, "changeFormValue");

		if (changeRequest.equals("addAllele")) {
			MouseAllele mouseAllele = new MouseAllele();
			mouseAllele.setGenotypeDictId(GenotypeDictionary.UNTESTED);
			mouseAlleleService.inflateMouseAlleleComponents(mouseAllele);
			mouseForm.getAlleles().add(mouseAllele);
		} else if (changeRequest.equals("removeAllele")) {
			int index = Integer.parseInt(changeValue);
			MouseAllele mouseAllele = mouseForm.getAlleles().get(index);
			if(mouseAllele.getId() == null)
				mouseForm.getAlleles().remove(mouseAllele);
			else
				mouseForm.getAlleles().get(index).setFlaggedForDelete(true);
		}else if (changeRequest.equals("reinstateAllele")) {
			int index = Integer.parseInt(changeValue);
			mouseForm.getAlleles().get(index).setFlaggedForDelete(false);
		} else if (changeRequest.equals("addNewProcedure")) {
			mouseForm.setAddNewProcedure(true);
			MouseProcedure mouseProcedure = new MouseProcedure();
			mouseProcedure.setLicencedProcedure(true);
			mouseProcedureService.inflateComponents(mouseProcedure);
			mouseForm.getMouse().setCurrentLicencedProcedure(mouseProcedure);
		} else if (changeRequest.equals("specifyMouseCount")) {
			if (mouseForm.getMouseNames() == null) {
				mouseForm.setMouseNames(new ArrayList<String>());

			}
			if (mouseForm.getEarmarks() == null) {
				mouseForm.setEarmarks(new ArrayList<Long>());
			}

			if (mouseForm.getMouseRfidTags() == null) {
				mouseForm.setMouseRfidTags(new ArrayList<String>());
			}

			if (mouseForm.getNumberOfMice() != null) {
				// names
				while (mouseForm.getNumberOfMice() > mouseForm.getMouseNames().size()) {
					mouseForm.getMouseNames().add("");
				}
				while (mouseForm.getNumberOfMice() < mouseForm.getMouseNames().size()) {
					List<String> names = (List<String>) mouseForm.getMouseNames();
					mouseForm.setMouseNames(names.subList(0, names.size() - 1));
				}
				// earmarks
				while (mouseForm.getNumberOfMice() > mouseForm.getEarmarks().size()) {
					mouseForm.getEarmarks().add(null);
				}
				while (mouseForm.getNumberOfMice() < mouseForm.getEarmarks().size()) {
					List<Long> earmarks = (List<Long>) mouseForm.getEarmarks();
					mouseForm.setEarmarks(earmarks.subList(0, earmarks.size() - 1));
				}

				// rfidtags
				while (mouseForm.getNumberOfMice() > mouseForm.getMouseRfidTags().size()) {
					mouseForm.getMouseRfidTags().add(null);
				}
				while (mouseForm.getNumberOfMice() < mouseForm.getMouseRfidTags().size()) {
					List<String> mouseRfidTags = (List<String>) mouseForm.getMouseRfidTags();
					mouseForm.setMouseRfidTags(mouseRfidTags.subList(0, mouseRfidTags.size() - 1));
				}
			}
		} else if (changeRequest.equals("toggleRfidInput")) {
			if (mouseForm.isRfidRequired()) {
				mouseForm.setRfidRequired(false);
				if (mouseForm.getMouse() != null && mouseForm.getMouse().getRfidTag() != null) {
					mouseForm.getMouse().setRfidTag(null);
				}
				if (mouseForm.getMouseRfidTags() != null) {
					for (int i = 0; i < mouseForm.getMouseRfidTags().size(); i++) {
						((List<String>) mouseForm.getMouseRfidTags()).set(i, null);
					}
				}
			} else {
				mouseForm.setRfidRequired(true);
			}
		}
		else if(changeRequest.equals("toggleNameInput")) {
			mouseForm.setNameFromBarcode(!mouseForm.isNameFromBarcode());
		}
	}

	private MouseForm retrieveSessionData(HttpServletRequest request) throws Exception {
		// fetch previous form from session
		MouseForm mouseForm = (MouseForm) request.getSession().getAttribute("persistentMouseForm");

		// explicitly remove ID values, as these should be unique for a mouse
		mouseForm.getMouse().setId(null);
		mouseForm.getMouse().setBarcode(null);
		mouseForm.getMouse().setRfidTag(null);
		mouseForm.getMouse().setName(null);
		// Also remove quantity of mice as this should be explicitly entered
		mouseForm.setNumberOfMice(null);

		// Also remove the litter ID
		mouseForm.getMouse().setLitterId(null);

		// Also remove the id from any alleles
		for (MouseAllele mouseAllele : mouseForm.getMouse().getAlleles()) {
			mouseAllele.setId(null);
			mouseAllele.setMouseId(null);

			// inflate the Allele as the migGene may be null (if copying from a
			// mouse that had a 'no
			// gene' allele)
			Allele allele = mouseAllele.getAllele();
			if (allele != null && allele.getMigGene() == null) {
				allele.setMigGene(new MigGene());
			}
		}

		// remove the id from Chimeric Data
		if (mouseForm.getMouse().getChimericData() != null) {
			mouseForm.getMouse().getChimericData().setId(null);
		}

		// remove the id from the Weight
		if (mouseForm.getMouse().getCurrentWeight() != null) {
			mouseForm.getMouse().getCurrentWeight().setWeightValue(null);
			mouseForm.getMouse().getCurrentWeight().setId(null); // don't
			// want to
			// overwrite
			// the
			// previous weight
			mouseForm.getMouse().getCurrentWeight().setSampleDate(null);
		}

		// remove the id from the CurrentLicencedProcedure
		if (mouseForm.getMouse().getCurrentLicencedProcedure() != null) {
			mouseForm.getMouse().getCurrentLicencedProcedure().setId(null);
		}

		// if the previous mouse had a procedure then allow it to be edited on
		// this form
		if (mouseForm.getMouse().getCurrentLicencedProcedure().getProcedureDictId() != null) {
			mouseForm.setAddNewProcedure(true);
		}
		return mouseForm;
	}

	private MouseForm buildNewForm(HttpServletRequest request) throws Exception {
		MouseForm mouseForm = (MouseForm) createCommand();

		Mouse mouse = new Mouse();
		Long id = ServletRequestUtils.getLongParameter(request, "id");

		if (id != null) {
			mouse = mouseService.fetchById(id, ComponentMap.of(Mouse.Component.ALL_COMPONENTS));

		}
		if (mouse.getRfidTag() == null) {
			mouseForm.setRfidRequired(false);
		} else {
			mouseForm.setRfidRequired(true);
		}
		// set the weight value and id to null
		// don't want to insert a new weight record unintentionally
		if (mouse.getCurrentWeight() != null) {
			mouse.getCurrentWeight().setWeightValue(null);
			mouse.getCurrentWeight().setId(null); // don't want to overwrite
			// the previous weight
			mouse.getCurrentWeight().setSampleDate(null);
		}

		mouseService.inflateMouseComponents(mouse);

		mouseForm.setMouse(mouse);

		mouseForm.setLocationBarcode(mouse.getCage().getBarcode());
		mouseForm.setLocationBarcodeDescription(mouse.getCage().getBarcodeDescription());
		mouseForm.setLocationBarcodeType("cage");
		mouseForm.setLocationId(mouse.getCageId());

		mouseForm.setProcedureSubform(new ProcedureSubform());

		if (mouse.getCurrentLicencedProcedure().getId() == null) {
			mouseForm.setAddNewProcedure(true);
		} else {
			mouseForm.setAddNewProcedure(false);
		}
		return mouseForm;
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		MouseForm mouseForm = null;

		RequestUtils.setLastReferer(request);

		boolean usePreviousForm = ServletRequestUtils.getBooleanParameter(request, "usePreviousForm", false);

		if (usePreviousForm && request.getSession().getAttribute("persistentMouseForm") != null) {
			mouseForm = retrieveSessionData(request);
		} else {
			mouseForm = buildNewForm(request);
		}

		// make collections lazy to avoid JSP crashes if the back button is used
		List<MouseAllele> lazyAlleles = ListUtils.lazyList((List<MouseAllele>) mouseForm.getMouse()
				.getAlleles(), FactoryUtils.instantiateFactory(MouseAllele.class));
		mouseForm.setAlleles(lazyAlleles);
		mouseForm.setMouseNames(ListUtils.lazyList(new ArrayList<String>(), FactoryUtils
				.instantiateFactory(String.class)));
		mouseForm.setEarmarks(ListUtils.lazyList(new ArrayList<Long>(), FactoryUtils.instantiateFactory(
				Long.class, new Class[] { long.class }, new Object[] { 0 })));
		return mouseForm;
	}

	@Override
	protected Map<String, Object> referenceData(HttpServletRequest request, Object command, Errors errors)
			throws Exception {
		MouseForm form = (MouseForm) command;
		Mouse mouse = form.getMouse();

		Map<String, Object> referenceData = new HashMap<String, Object>();

		referenceData.put("causeOfDeath", dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_CAUSE_OF_DEATH,
				mouse.getCauseOfDeathDictId()));
		referenceData.put("coatColour", dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_COAT_COLOUR,
				mouse.getCoatColourDictId()));
		referenceData.put("earmark", dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_EARMARK_DICT, mouse
				.getEarmarkDictId()));
		referenceData.put("eyeColour", dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_EYE_COLOUR_DICT,
				mouse.getEyeColourDictId()));
		referenceData.put("gender", dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_GENDER_DICT, mouse
				.getGenderDictId()));
		referenceData.put("genotype", dictionaryService.fetch(DictionaryEnum.MOUSE_GENOTYPE, false));

		// Yes/No list
		Map<Boolean, String> yesNo = new HashMap<Boolean, String>();
		yesNo.put(false, "No");
		yesNo.put(true, "Yes");
		referenceData.put("yesNo", yesNo);

		// Genotype status
		Set<String> genotypeStatus = new HashSet<String>();
		genotypeStatus.add("Confirmed");
		genotypeStatus.add("Uncertain");
		referenceData.put("genotypeStatus", genotypeStatus);

		ExperimentData experimentData = experimentDataService.fetchByMouseId(mouse.getId(),
				componentMapService.getComponentMap("viewOneMouseExperimentData"));
		referenceData.put("experimentData", experimentData);

		return referenceData;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command,
			BindException errors) throws Exception {
		MouseForm mouseForm = (MouseForm) command;

		// add the form to the session outside the Spring context, where it can
		// be reused.
		request.getSession().setAttribute("persistentMouseForm", mouseForm);

		Person user = RequestUtils.getUser(request);

		Map<String, Long> result = mouseService.createMice(mouseForm, user);

		// either return view of one mouse, or view cage of mice
		if (result.containsKey("mouseId")) {
			return new ModelAndView(getSuccessView(), "id", result.get("mouseId"));
		} else {
			return new ModelAndView(multiMouseView, "id", result.get("cageId"));
		}
	}

	@Override
	protected ModelAndView handleInvalidSubmit(HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		return new ModelAndView("errors/handle_invalid_submit");
	}
}