/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.controllers.AbstractFormChangeController;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseSearchBean;
import uk.ac.sanger.mig.mousetracking.Room;
import uk.ac.sanger.mig.mousetracking.BarcodedObject.BarcodeType;
import uk.ac.sanger.mig.mousetracking.forms.BarcodeForm;
import uk.ac.sanger.mig.mousetracking.forms.ManageCohortForm;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class ManageCohortFormController extends AbstractFormChangeController {

	private CohortService cohortService;
	private MouseService mouseService;
	private DictionaryService dictionaryService;
	private ComponentMapService componentMapService;

	private long defaultCohortTypeId = CohortService.COHORT_TYPE_PHENOTYPING;

	public void setDefaultCohortTypeId(long defaultCohortTypeId) {
		this.defaultCohortTypeId = defaultCohortTypeId;
	}

	public void setCohortService(CohortService cohortService) {
		this.cohortService = cohortService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	@Override
	/**
	 * Note: this is called by ManageGenotypingListFormController, so changing the behaviour might have interesting effects in genotyping
	 */
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws ServletRequestBindingException {
		ManageCohortForm form = (ManageCohortForm)command;
		String changeRequest = ServletRequestUtils.getRequiredStringParameter(request, "changeForm");
		String changeRequestValue = ServletRequestUtils.getRequiredStringParameter(request, "changeFormValue");

		if(changeRequest.equals("scanCage")) {
			formChangeScanBarcode(form);
		}
		else if(changeRequest.equals("validateCohort") && StringUtils.isNotEmpty(changeRequestValue)) {
			formChangeValidateCohort(form, changeRequestValue);
		}
		else if (changeRequest.equals("roomPrewean")) {
			formChangeRoomPicked(form);
		}
		else if (changeRequest.equals("pickerCohort")) {
			formChangeCohortPicked(form);
		}
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		RequestUtils.setLastReferer(request);
		ManageCohortForm form = (ManageCohortForm)createCommand();
		form.setBarcodeForm(new BarcodeForm());

		// has the user requested to add mice selected from a search?
		boolean addSelectedMice = ServletRequestUtils.getBooleanParameter(request, "addSelectedMice", false);

		initCohortOnForm(form, request);
		initMiceOnForm(form, addSelectedMice, request);

		return form;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		ManageCohortForm form = (ManageCohortForm)command;

		Person user = RequestUtils.getUser(request);

		form.updateMice();
		cohortService.manageCohort(form.getCohort(), form.getCohort().getMice(), user);

		return new ModelAndView(getSuccessView(), "id", form.getCohort().getId());
	}

	private void initCohortOnForm(ManageCohortForm form, HttpServletRequest request) throws Exception {
		Cohort cohort = null;

		Long cohortId = ServletRequestUtils.getLongParameter(request, "id");
		if(cohortId == null) {
			Long cohortTypeId = ServletRequestUtils.getLongParameter(request, "cohortTypeId", defaultCohortTypeId);
			DictionaryEntry cohortType = dictionaryService.fetchEntry(DictionaryEnum.MOUSE_COHORT_TYPE, cohortTypeId);

			cohort = new Cohort();
			cohort.setMice(new ArrayList<Mouse>());
			cohort.setCohortType(cohortType);
			cohort.setCohortTypeDictId(cohortTypeId);
			form.setNewCohort(true);
		}
		else {
			cohort = cohortService.fetchById(cohortId, componentMapService.getComponentMap("manageCohort"));
			form.setNewCohort(false);
		}

		if(cohort.getCohortTypeDictId().equals(CohortService.COHORT_TYPE_PHENOTYPING)) {
			form.setPipelineRequired(true);
		}
		else {
			form.setPipelineRequired(false);
		}

		if( cohort.getCohortTypeDictId().equals(CohortService.COHORT_TYPE_PHENOTYPING) ||
			cohort.getCohortTypeDictId().equals(CohortService.COHORT_TYPE_STUD_BANK)) {
			form.setDescriptionRequired(true);
		}
		else {
			form.setDescriptionRequired(false);
		}

		form.setCohort(cohort);
	}

	private void initMiceOnForm(ManageCohortForm form, boolean addSelectedMice, HttpServletRequest request) {

		if(addSelectedMice) {
			// try the session store from mouse list page
			String queryName = ServletRequestUtils.getStringParameter(request, "queryName", MouseService.MOUSE_SEARCH_QUERY);
			String sessionId = RequestUtils.getSessionId(request);

			Collection<Mouse> selectedMice = mouseService.fetchLastSelection(sessionId, queryName, componentMapService.getComponentMap("addToCohort"));
			form.addSelectableMice(selectedMice);
		}


	}

	private void formChangeScanBarcode(ManageCohortForm form) {
		BarcodeForm barcodeForm = form.getBarcodeForm();
		Collection<Mouse> miceToAdd = new ArrayList<Mouse>();

		if(BarcodeType.MOUSE.equals(barcodeForm.getBarcodeType())) {
			Mouse mouse = mouseService.fetchById(barcodeForm.getId(), getComponentMapForAddingMice());

			if(mouse.getCageId() == null) {
				miceToAdd.add(mouse);
			}
			else {
				miceToAdd.addAll(mouseService.fetchByCageId(mouse.getCageId(), getComponentMapForAddingMice()));
			}
		}
		else if(BarcodeType.CAGE.equals(barcodeForm.getBarcodeType())) {
			miceToAdd.addAll(mouseService.fetchByCageId(barcodeForm.getId(), getComponentMapForAddingMice()));
		}
		else if (BarcodeType.LITTER.equals(barcodeForm.getBarcodeType())) {
			miceToAdd.addAll(mouseService.fetchByLitterId(barcodeForm.getId() , getComponentMapForAddingMice()));
		}

		form.addSelectableMice(miceToAdd);

		// set the origin barcode on the form back to null so a new cage/mouse can be scanned
		form.setBarcodeForm(new BarcodeForm());
	}

	private void formChangeValidateCohort(ManageCohortForm form, String cohortName) {
		Cohort cohort = cohortService.fetchByName(cohortName, componentMapService.getComponentMap("manageCohort"));
		if(cohort == null) {
			cohort = new Cohort();
			cohort.setCohortName(cohortName);
			cohort.setCohortType(form.getCohort().getCohortType());
			cohort.setCohortTypeDictId(form.getCohort().getCohortTypeDictId());
			cohortService.inflateCohortComponents(cohort);
			form.setNewCohort(true);
		}
		else {
			form.setNewCohort(false);
		}
		form.setCohort(cohort);
	}

	/**
	 * Add the mice in the picked cohort to the list of addable mice
	 *
	 * @param form
	 */
	private void formChangeCohortPicked(ManageCohortForm form) {
		if (form.getChooserCohort().getId() != null) {
			Collection<Mouse> miceToAdd = mouseService.fetchByCohortId(
					form.getChooserCohort().getId(),
					getComponentMapForAddingMice()
			);

			form.addSelectableMice(miceToAdd);
			form.setChooserCohort(new Cohort());
		}
	}
	/**
	 * Add all the pups that
	 * <ul>
	 * <li>are in the specified room</li>
	 * <li>haven't had an earmark recorded</li>
	 * <li>haven't been weaned</li>
	 * <li>have a litter id (this excludes the virtual mice)</li>
	 * </ul>
	 * to the list of addable mice
	 * @param form
	 */
	private void formChangeRoomPicked(ManageCohortForm form) {
		if (form.getRoom().getId() != null) {
			MouseSearchBean msb = new MouseSearchBean();
			msb.setPaginated(false);
			msb.setRoomId(form.getRoom().getId());
			msb.setNullWeanDate(true);
			msb.setNullEarmark(true);
			msb.setLitterIdPresent(true);

			Collection<Mouse> miceToAdd = mouseService.fetch(msb, getComponentMapForAddingMice());

			form.addSelectableMice(miceToAdd);
		}
		form.setRoom(new Room());
	}

	@Override
	protected Map<String,Object> referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		ManageCohortForm form = (ManageCohortForm)command;
		Cohort cohort = form.getCohort();

		Map<String,Object> referenceData = new HashMap<String,Object>();

		referenceData.put("pipeline", dictionaryService.fetchForEdit(DictionaryEnum.MOUSE_COHORT_PIPELINE, cohort.getCohortPipelineDictId()));

		return referenceData;
	}

	@Override
	protected ModelAndView handleInvalidSubmit(HttpServletRequest request, HttpServletResponse response) throws Exception {
		return new ModelAndView("errors/handle_invalid_submit");
	}

	private ComponentMap getComponentMapForAddingMice() {
		return componentMapService.getComponentMap("addToCohort");
	}

}
