/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.forms.MoveMouseForm;
import uk.ac.sanger.mig.mousetracking.services.CageService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class MoveMouseFormController extends AbstractCancellableFormController {

	private CageService cageService;
	private MouseService mouseService;
	private ComponentMapService componentMapService;

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	@Override
	protected boolean isFormChangeRequest(HttpServletRequest request) {
		String changeRequest = ServletRequestUtils.getStringParameter(request, "changeForm", "");

		return (StringUtils.isNotEmpty(changeRequest));
	}

	@Override
	protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		MoveMouseForm form = (MoveMouseForm)command;
		String changeRequest = ServletRequestUtils.getRequiredStringParameter(request, "changeForm");
		String cageIdString = ServletRequestUtils.getRequiredStringParameter(request, "changeFormValue");

		if(changeRequest.equals("scanCage") && StringUtils.isNotEmpty(cageIdString)) {
			addCage(form, new Long(cageIdString),errors);
		}
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		MoveMouseForm form = (MoveMouseForm)command;

		Person user = RequestUtils.getUser(request);
		Set<Long> destinationCageIds = mouseService.moveMice(form, user);

		String cageIdString = "";
		for(Long cageId : destinationCageIds) {
			cageIdString +=cageId + ",";
		}

		return new ModelAndView(getSuccessView(), "cageIds", cageIdString);
	}

	private void addCage(MoveMouseForm form, Long cageId, BindException errors) {
		ComponentMap cageComponents = componentMapService.getComponentMap("moveMice");
		Cage cage = cageService.fetchById(cageId, cageComponents);

		if(!cage.isPopulated() && errors != null) {
			errors.reject(null, "Cage " + cage.getBarcode()+ " is empty");
		}
		else if(!form.getCages().contains(cage)) {
			form.getCages().add(cage);

			form.setTotalScannedMice(form.getTotalScannedMice() + cage.getContents().size());

			for(Mouse mouse : cage.getContents()) {
				if(mouse.getColonyInfo() != null && mouse.getColonyInfo().isPoolable()) {
					if(mouse.isMale()) {
						form.setDisplayStudField(true);
					}
					else if(mouse.isFemale()) {
						form.setDisplayFemalePool(true);
					}
				}
			}
		}

		// set the origin cage barcode on the form back to null so a new cage can be scanned
		form.setOriginCageBarcode(null);
		form.setOriginCageBarcodeDescription(null);
		form.setOriginCageId(null);
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		RequestUtils.setLastReferer(request);

		MoveMouseForm form = (MoveMouseForm)createCommand();

		Long cageId = ServletRequestUtils.getLongParameter(request, "cageId");

		if(cageId != null) {
			addCage(form, cageId, null);
		}

		return form;
	}

	@Override
	protected ModelAndView handleInvalidSubmit(HttpServletRequest request, HttpServletResponse response) throws Exception {
		return new ModelAndView("errors/handle_invalid_submit");
	}
}
