/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.mousetracking.LabelPrinter;
import uk.ac.sanger.mig.mousetracking.forms.PrintLabelForm;
import uk.ac.sanger.mig.mousetracking.services.LabelPrinterService;
import uk.ac.sanger.mig.mousetracking.services.PrintJobType;

public class PrintLabelsController extends SimpleFormController {

	private LabelPrinterService labelPrinterService;

	public void setLabelPrinterService(LabelPrinterService labelPrinterService) {
		this.labelPrinterService = labelPrinterService;
	}

	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		PrintLabelForm form = (PrintLabelForm)command;
		LabelPrinter printer = labelPrinterService.fetchById(form.getPrinterId(), null);
		form.setLabelPrinter(printer);

		Set<String> errorMessages = labelPrinterService.printLabels(form, RequestUtils.getUser(request));
		String errorMessage = "";

		if(errorMessages.size() == 1 && errorMessages.contains(null)) {
			errorMessage = "Labels have been sent to printer " + printer.getSysName();
			request.getSession().setAttribute("lastPrinterUsed", printer.getId());
		}
		else {
			for(String message : errorMessages) {
				errorMessage += message + "\n";
			}
		}

		return new ModelAndView(getSuccessView(), "message", errorMessage);
	}

	@Override
	protected Map referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		Map<String, Object> referenceData = new HashMap<String, Object>();

		String printJobName = ServletRequestUtils.getRequiredStringParameter(request, "printJobType");
		PrintJobType printJobType = PrintJobType.valueOf(printJobName);

		Collection<LabelPrinter> printers = labelPrinterService.fetchByMediaType(printJobType.getMediaType());

		referenceData.put("printerGroups", groupPrinters(printers));

		return referenceData;
	}

	@Override
	protected boolean isFormSubmission(HttpServletRequest request) {
		boolean bypassForm = ServletRequestUtils.getBooleanParameter(request, "bypassForm", false);

		if(bypassForm) {
			logger.debug("Request parameter 'bypassForm' is true. This is a form submission.");
			return true;
		}
		else {
			logger.debug("Delegating to superclass isFormSubmission method.");
			return super.isFormSubmission(request);
		}
	}


	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		PrintLabelForm form = (PrintLabelForm)createCommand();
		form.setCopies(1);

		if(request.getSession().getAttribute("lastPrinterUsed") != null) {
			form.setPrinterId((Long)request.getSession().getAttribute("lastPrinterUsed"));
		}

		return form;
	}

	private Map<String, Collection<LabelPrinter>> groupPrinters(Collection<LabelPrinter> printers) {
		Map<String, Collection<LabelPrinter>> printerGroups = new TreeMap<String, Collection<LabelPrinter>>();
		for(LabelPrinter printer : printers) {
			String corridor = printer.getCorridor();

			if(!printerGroups.containsKey(corridor)) {
				printerGroups.put(corridor, new ArrayList<LabelPrinter>());
			}
			printerGroups.get(corridor).add(printer);

		}
		return printerGroups;
	}

}
