/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.controllers;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.validation.Errors;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.CancellableFormController;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Rack;
import uk.ac.sanger.mig.mousetracking.forms.ValidateCageForm;
import uk.ac.sanger.mig.mousetracking.services.CageService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class ValidateCageDetailsController extends CancellableFormController {

	private final SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");

	private CageService cageService;
	private DictionaryService dictionaryService;

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	@Override
	protected void initBinder(HttpServletRequest httpServletRequest, ServletRequestDataBinder binder) {
		// Create and register Date binder for Date values which also allows null values
		CustomDateEditor customDateEditor = new CustomDateEditor(sdf, true);
		binder.registerCustomEditor(Date.class, customDateEditor);

		// Create and register Long binder for Long values which also allows null values
		CustomNumberEditor customLongEditor = new CustomNumberEditor(Long.class, true);
		binder.registerCustomEditor(Long.class, customLongEditor);
	}


	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		ValidateCageForm form = (ValidateCageForm)command;

		Person user = RequestUtils.getUser(request);
		Cage cage = form.getCage();

		cageService.clean(cage, user);

		return new ModelAndView(getSuccessView(), "id", cage.getId());
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		ValidateCageForm form = (ValidateCageForm)createCommand();

		Long cageId = ServletRequestUtils.getRequiredLongParameter(request, "id");

		ComponentMap rackComponents = new ComponentMap();
		rackComponents.put(Rack.Component.ROOM, null);

		ComponentMap cageComponents = new ComponentMap();
		cageComponents.put(Cage.Component.RACK, rackComponents);
		cageComponents.put(Cage.Component.ALL_COMPONENTS,null);

		Cage cage = cageService.fetchById(cageId, cageComponents);

		form.setCage(cage);

		return form;
	}


	@Override
	protected Map<String, Object> referenceData(HttpServletRequest request, Object command, Errors errors) throws Exception {
		Map<String, Object> referenceData = new HashMap<String, Object>();
		Long cageId = ServletRequestUtils.getRequiredLongParameter(request, "id");

		//Collection<DictionaryEntry> cullTypes = dictionaryService.fetch(DictionaryEnum.MOUSE_CAUSE_OF_DEATH, false);

		DictionaryEntry culled = dictionaryService.fetchEntry(DictionaryEnum.MOUSE_CAUSE_OF_DEATH, MouseService.CAUSE_OF_DEATH_CULLED);
		DictionaryEntry missing = dictionaryService.fetchEntry(DictionaryEnum.MOUSE_CAUSE_OF_DEATH, MouseService.CAUSE_OF_DEATH_MISSING);
		DictionaryEntry foundDead = dictionaryService.fetchEntry(DictionaryEnum.MOUSE_CAUSE_OF_DEATH, MouseService.CAUSE_OF_DEATH_FOUND_DEAD);


		/*Map<String, Long> cullTypesMap = new HashMap<String, Long>();
		Map<String, String> cullNameMap = new HashMap<String, String>();
		for(DictionaryEntry dictionaryEntry : cullTypes) {
			cullTypesMap.put(dictionaryEntry.getDescription(), dictionaryEntry.getId());
			cullNameMap.put(dictionaryEntry.getDescription(), dictionaryEntry.getName());
		}

		referenceData.put("cullTypes", cullTypesMap);
		referenceData.put("cullNames", cullNameMap);*/
		referenceData.put("culled",culled);
		referenceData.put("missing", missing);
		referenceData.put("foundDead",foundDead);

		Cage cageForView = cageService.fetchForViewById(cageId);
		referenceData.put("cage", cageForView);

		return referenceData;
	}


}
