/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.LongRowMapper;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.BreedingSearchBean;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.mousetracking.services.BreedingType;

public class BreedingDaoSpringImpl extends AbstractDaoSpringImpl<Breeding> implements BreedingDao {

	private PaginatedSearchUtils<Breeding> paginator;

	public void setPaginator(PaginatedSearchUtils<Breeding> paginator) {
		this.paginator = paginator;
	}

	public Collection<Breeding> fetchColonyBreedingsByBreedingId(Long breedingId) {
		String sql = " select c.* from mts_breeding c, mts_breeding b where b.id = ? and b.offspring_colony_id = c.offspring_colony_id order by c.breeding_number desc";
		return fetchCollection(sql, breedingId);
	}

	/**
	 * This query needs to fetch all ongoing matings in a cage
	 * OR, if there is a single female in the cage with a completed mating it should fetch that.
	 */
	public Collection<Breeding> fetchByCageId(Long cageId) {
		String
		sql = " select b.*";
		sql += " from ";
		sql += "  mts_breeding b, ";
		sql += "  mts_mouse m ";
		sql += " where ";
		sql += "  b.female_id = m.id ";
		sql += "  and ((select count(*) from mts_mouse x, mts_latest_breeding_vw l where x.cage_id = m.cage_id and x.id = l.mouse_id and x.id = m.id and l.breeding_id = b.id) = 1 ";
		sql += "   	  or b.end_date is null) ";
		sql += "  and m.cage_id = ? ";
		return fetchCollection(sql, cageId);
	}

	public BreedingType fetchBreedingType(Long breedingId) {
		String sql = " select decode(embryo_transfer_id,null,0,1) is_embryo_transfer from mts_breeding where id = ? ";

		int isEmbryoTransfer = getSimpleJdbcTemplate().queryForInt(sql, breedingId);

		return (isEmbryoTransfer == 0) ? BreedingType.MATING : BreedingType.EMBRYO_TRANSFER;
	}

	public Breeding fetchLatestForMouse(Long mouseId) {
		String sql = "select * from mts_breeding where id in (select breeding_id from mts_latest_breeding_vw where mouse_id = ?) ";
		return fetchOne(sql, mouseId);
	}

	public Collection<Breeding> fetchCurrentForMouse(Long mouseId) {
		String sql = " select * from mts_breeding b where b.end_date is null and (b.female_id = ? or b.male_id = ?) order by start_date desc ";
		return fetchCollection(sql, mouseId, mouseId);
	}

	/**
	 * Return the next breeding number for  the specified colony.
	 * Note this method is synchronized to prevent multiple users getting the same number
	 */
	public synchronized int nextBreedingNumber(Long colonyId) {

		Collection<Object> bindVariables = new ArrayList<Object>();

		String sql =
			" select nvl(max(breeding_number),0) + 1 next_breeding_number from mts_breeding where offspring_colony_id ";

		if(colonyId == null) {
			sql += " is null ";
		}
		else {
			sql += " = ? ";
			bindVariables.add(colonyId);
		}

		int nextNumber = getSimpleJdbcTemplate().queryForInt(sql, bindVariables.toArray());

		// check to see if there is an offset (for pre-existing colonies)
		if(nextNumber == 1 && colonyId != null) {
			sql = " select offset from mts_breeding_offset where colony_info_id = ? ";

			try {
				int offset = getSimpleJdbcTemplate().queryForInt(sql, bindVariables.toArray());
				nextNumber += offset;
			}
			catch(IncorrectResultSizeDataAccessException e) {
				// no offset record found - nothing to do here
			}
		}

		return nextNumber;
	}

	public Collection<Breeding> fetch(BreedingSearchBean searchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		String selectSql = " select distinct b.* ";
		String fromSql = " from mts_breeding b ";
		String whereSql = " where 1=1 ";

		//breeding name is should always be uppercase, so can safely upper the bind variable
		if(searchBean.getBreedingName() != null && !searchBean.getBreedingName().equals("")) {
			whereSql += " and lower(b.name_prefix||b.breeding_number) = lower(?) ";
			bindVariables.add(searchBean.getBreedingName());
		}

		if(searchBean.getBreedingCategoryDictId() != null) {
			whereSql += " and breeding_category_dict_id = ? ";
			bindVariables.add(searchBean.getBreedingCategoryDictId());
		}

		if(searchBean.getStartDateFrom() != null){
			whereSql += " and b.start_date >= ? ";
			bindVariables.add(searchBean.getStartDateFrom());
		}
		if(searchBean.getStartDateTo() != null){
			whereSql += " and b.start_date <= ? ";
			bindVariables.add(searchBean.getStartDateTo());
		}

		if(searchBean.getEndDateFrom() != null){
			whereSql += " and b.end_date >= ? ";
			bindVariables.add(searchBean.getEndDateFrom());
		}
		if(searchBean.getEndDateTo() != null){
			whereSql += " and b.end_date <= ? ";
			bindVariables.add(searchBean.getEndDateTo());
		}

		if(searchBean.isEndDateNull()) {
			whereSql += " and b.end_date is null ";
		}

		if(searchBean.getTimed() != null) {
			if(searchBean.getTimed()) {
				whereSql += " and b.timed_mating = 1 ";
			}
			else {
				whereSql += " and nvl(b.timed_mating,0) = 0 ";
			}
		}

		if(searchBean.getTimedHarvestDate() != null) {
			fromSql += ", mts_breeding_timed_harvest_vw h ";
			whereSql += " and h.breeding_id = b.id and h.timed_harvest_date = ? ";
			bindVariables.add(searchBean.getTimedHarvestDate());
		}

		if(searchBean.getParentId() != null) {
			whereSql += " and (b.male_id = ? or b.female_id = ?) ";
			bindVariables.add(searchBean.getParentId());
			bindVariables.add(searchBean.getParentId());
		}

		// add mts_mouse for parent animal if any of the following conditions are
		// present: parentColonyId, parentName, parentCage, room
		if(	   (searchBean.getParentName()!= null && !searchBean.getParentName().equals("")) ||
				searchBean.getParentCageId() != null ||
				searchBean.getRoomId() != null) {
			fromSql += ", mts_mouse parent ";
			whereSql += " and (b.male_id = parent.id or b.female_id = parent.id) ";
		}


		if(searchBean.getOffspringColonyId() != null) {
			whereSql += " and b.offspring_colony_id = ? ";
			bindVariables.add(searchBean.getOffspringColonyId());
		}


		if(searchBean.getParentName() != null && !searchBean.getParentName().equals("")) {
			whereSql += " and lower(parent.name) = lower(?) ";
			bindVariables.add(searchBean.getParentName());
		}


		if(searchBean.getParentCageId() != null) {
			whereSql += " and parent.cage_id = ? ";
			bindVariables.add(searchBean.getParentCageId());
		}

		if(searchBean.getFemaleCageId() != null) {
			fromSql += ", mts_mouse female ";
			whereSql += " and (b.female_id = female.id) and female.cage_id = ? ";
			bindVariables.add(searchBean.getFemaleCageId());
		}

		if(searchBean.getStatus() != null && !searchBean.getStatus().equals("")){
			if(searchBean.getStatus().equals(BreedingService.BREEDING_STATUS_COMPLETE)) {
				whereSql += " and b.embryo_transfer_id is null and b.end_date is not null ";
			}
			else if(searchBean.getStatus().equals(BreedingService.BREEDING_STATUS_BREEDING)) {
				fromSql += ", mts_mouse male_location, mts_mouse female_location ";
				whereSql += " and b.embryo_transfer_id is null and b.male_id = male_location.id and b.female_id = female_location.id and male_location.cage_id = female_location.cage_id ";
			}
			else if(searchBean.getStatus().equals(BreedingService.BREEDING_STATUS_SEPARATED)) {
				fromSql += ", mts_mouse male_location, mts_mouse female_location ";
				whereSql += " and b.end_date is null and b.embryo_transfer_id is null and b.male_id = male_location.id and b.female_id = female_location.id and male_location.cage_id != female_location.cage_id ";
			}
		}

		if(searchBean.getEmbryoTransferId() != null) {
			whereSql += " and b.embryo_transfer_id = ? ";
			bindVariables.add(searchBean.getEmbryoTransferId());
		}

		if(searchBean.getLitterNumber() != null) {
			String equality = "";
			if (searchBean.getLitterExpression().equals("Equal to")) { equality = "="; }
			if (searchBean.getLitterExpression().equals("More than")) { equality = ">"; }
			if (searchBean.getLitterExpression().equals("Less than")) { equality = "<"; }

			whereSql += " and (select count(*) from mts_litter l where l.breeding_id = b.id) " + equality + " ? ";
			bindVariables.add(searchBean.getLitterNumber());
		}

		if(searchBean.getMicroinjectionId() != null) {
			fromSql += ", mir_embryo_transfer_procedure e, mir_scheduled_micro_injection s ";
			whereSql += " and b.embryo_transfer_id = e.id and e.scheduled_micro_injection_id = s.id and s.micro_injection_request_id = ? ";
			bindVariables.add(searchBean.getMicroinjectionId());
		}

		// roomId of parent mice
		if(searchBean.getRoomId() != null) {
			fromSql += ", mts_cage_vw c ";
			whereSql += " and parent.cage_id = c.cage_id and c.room_id = ?" ;
			bindVariables.add(searchBean.getRoomId());
		}

		String orderBySql = " order by ";

		if(searchBean.getOrderBy() != null && searchBean.getOrderBy().equals("")) {
			orderBySql += "	b.id ";
		}
		else {
			orderBySql += searchBean.getOrderBy();
		}

		if(searchBean.isOrderDesc()) {
			orderBySql += " desc ";
		}

		String sql = selectSql + fromSql + whereSql + orderBySql ;

		logger.debug(sql);

		return paginator.fetchSearchResults(sql, bindVariables, searchBean, getRowMapper());
	}

	public Collection<Long> fetchBreedingNumbersByColonyId(Long colonyId) {
		String sql = " select column_value return_long from table(mts.mts_next_breeding_numbers(?)) ";
		Collection<Long> records = null;

		if(colonyId != null) {
			records = getSimpleJdbcTemplate().query(sql, new LongRowMapper(), colonyId);
		}

		return (records == null) ? new ArrayList<Long>() : records;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Breeding breeding) {
		sql.addDateParam(breeding.getStartDate(), "START_DATE");
		sql.addDateParam(breeding.getEndDate(), "END_DATE");
		sql.addLongParam(breeding.getCheckNumber(), "CHECK_NUMBER");
		sql.addStringParam(breeding.getEditedBy(), "EDITED_BY");
		sql.addDateParam(breeding.getEditDate(), "EDIT_DATE");
		sql.addStringParam(breeding.getComments(), "COMMENTS");
		sql.addLongParam(breeding.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(breeding.getCreatedDate(),"CREATED_DATE");
		sql.addLongParam(breeding.getMaleId(), "MALE_ID");
		sql.addLongParam(breeding.getFemaleId(), "FEMALE_ID");
		sql.addIntParam(breeding.getBreedingNumber(), "BREEDING_NUMBER");
		sql.addStringParam(breeding.getNamePrefix(), "NAME_PREFIX");
		sql.addLongParam(breeding.getOffspringColonyId(), "OFFSPRING_COLONY_ID");
		sql.addStringParam(breeding.getOffspringGeneration(), "OFFSPRING_GENERATION");
		sql.addBooleanParam(breeding.isPseudoBreeding(), "PSEUDO_BREEDING");
		sql.addLongParam(breeding.getEmbryoTransferId(), "EMBRYO_TRANSFER_ID");
		sql.addBooleanParam(breeding.isTimed(), "TIMED_MATING");
		sql.addFloatParam(breeding.getTimedEmbryoAge(), "TIMED_MATING_EMBRYO_AGE");
		sql.addLongParam(breeding.getBreedingCategoryDictId(), "BREEDING_CATEGORY_DICT_ID");
		sql.addLongParam(breeding.getId(), "ID");
	}
}
