/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.dao;

import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.database.StringRowMapper;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.services.CageType;


public class CageDaoSpringImpl extends AbstractDaoSpringImpl<Cage> implements CageDao {


	public Cage fetchPoolCage(Long colonyId, Date birthDate) {
		String sql = " select c.* from mts_cage c, mts_mouse m where m.cage_id = c.id and m.gender_dict_id = 1 and m.is_pooled = 1 and m.wean_date is not null and m.colony_info_id = :a and m.birth_date = :b and rownum = 1 ";
		return fetchOne(sql, colonyId, birthDate);
	}

	/**
	 * Returns Collection of Cage objects in use sorted by name (ie, varchar number)
	 */
	public Collection<Cage> fetchByRackId(Long rackId) {
		String sql = "select c.* from mts_cage c where c.rack_id = ? and exists (select null from mts_mouse m where m.cage_id = c.id) order by c.id ";
		return fetchCollection(sql, rackId);
	}

	public Collection<Cage> fetchByLitterId(Long litterId) {
		String sql = " select distinct ";
		sql += "  c.* ";
		sql += " from ";
		sql += "   mts_cage c, ";
		sql += "   mts_mouse m, ";
		sql += "   mts_litter l, ";
		sql += "   mts_breeding b, ";
		sql += "   mts_mouse f ";
		sql += " where ";
		sql += "   c.id = m.cage_id and ";
		sql += "   l.id = m.litter_id and ";
		sql += "   b.id = l.breeding_id and ";
		sql += "   f.id = b.female_id and ";
		sql += "   c.id != nvl(f.cage_id,0) and ";
		sql += "   l.id = ? ";
		sql += " order by ";
		sql += " c.barcode ";

		return fetchCollection(sql, litterId);
	}

	public Cage fetchByBarcode(String barcode) {
		String sql = "select * from mts_cage where barcode = upper(?) ";
		return fetchOne(sql, barcode);
	}

	public String fetchCageTotals(Long cageId) {
		String sql = " select mts_fetch_cage_totals(?) cage_totals from dual ";
		String x = getSimpleJdbcTemplate().queryForObject(sql, new StringRowMapper(), cageId);
		return x;
	}

	public CageType fetchCageType(Long cageId) {
		String sql = " select mts.fetch_cage_type(?) cage_type from dual ";
		String cageTypeString = getSimpleJdbcTemplate().queryForObject(sql, new StringRowMapper(), cageId);
		return CageType.valueOf(cageTypeString);
	}

	public boolean checkIfEmpty(Long cageId) {
		String sql = " select count(*) from mts_mouse m where m.cage_id = ? ";

		int count =  getSimpleJdbcTemplate().queryForInt(sql, cageId);

		return (count == 0) ? true : false;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Cage cage) {
		sql.addStringParam(cage.getBarcode(), "BARCODE");
		sql.addLongParam(cage.getRackId(), "RACK_ID");
		sql.addLongParam(cage.getCheckNumber(), "CHECK_NUMBER");
		sql.addStringParam(cage.getComments(), "COMMENTS");
		sql.addStringParam(cage.getEditedBy(), "EDITED_BY");
		sql.addDateParam(cage.getEditDate(), "EDIT_DATE");
		sql.addLongParam(cage.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(cage.getCreatedDate(), "CREATED_DATE");
		sql.addDateParam(cage.getLastCleanedDate(), "LAST_CLEANED_DATE");
		sql.addLongParam(cage.getId(), "ID");
	}

	public int getTotalCagesByRackId(Long rackId) {
		String sql = "select count(*) from mts_cage c where c.rack_id = ? and exists (select null from mts_mouse m where m.cage_id = c.id)";

		return getSimpleJdbcTemplate().queryForInt(sql, rackId);
	}

	public int getTotalCagesByRoomId(Long roomId) {

		String sql = "select count(*) from mts_cage c, mts_rack r  " +
		"where r.room_id = ? and r.status = 'In Use'" +
		"and c.rack_id = r.id " +
		"and exists (select null from mts_mouse m where m.cage_id = c.id) ";

		return getSimpleJdbcTemplate().queryForInt(sql, roomId);
	}

	public Collection<Cage> fetchPreviousCages(Long cageId) {
		String sql = " select c.* from mts_cage c where c.id in ( ";
		sql += " select h.cage_id ";
		sql += " from ";
		sql += " mts_mouse_cage_history h, ";
		sql += " mts_mouse m ";
		sql += " where ";
		sql += " m.id = h.mouse_id and ";
		sql += " m.cage_id = ? and ";
		sql += " trunc(h.end_date) = trunc(sysdate)";
		sql += " ) and exists (select null from mts_mouse m where m.cage_id = c.id) order by c.id ";

		return fetchCollection(sql, cageId);
	}
}
