/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.CohortSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.dao.rowmappers.CohortListViewRowMapper;
import uk.ac.sanger.mig.mousetracking.views.CohortListView;

public class CohortDaoSpringImpl extends AbstractDaoSpringImpl<Cohort> implements CohortDao {

	private PaginatedSearchUtils<CohortListView> listPaginator;
	private CohortListViewRowMapper cohortListViewRowMapper;

	public void setCohortListViewRowMapper(
			CohortListViewRowMapper cohortListViewRowMapper) {
		this.cohortListViewRowMapper = cohortListViewRowMapper;
	}

	public void setListPaginator(PaginatedSearchUtils<CohortListView> listPaginator) {
		this.listPaginator = listPaginator;
	}

	public Collection<Cohort> fetchForLookup(Long typeId) {
		String sql = "select * from mts_cohort where cohort_name is not null and cohort_type_id = ? order by lower(cohort_name) ";
		return fetchCollection(sql, typeId);
	}

	public Collection<Cohort> fetchForLookup(Long typeId, Long pipelineId) {
		String sql = "select * from mts_cohort where cohort_name is not null and cohort_type_id = ? and pipeline_dict_id = ? order by lower(cohort_name) ";
		return fetchCollection(sql, typeId, pipelineId);
	}
	public boolean isMouseInCohort(Long mouseId, Long cohortId) {
		String sql = " select count(*) from mts_cohort_mouse where mouse_id = ? and cohort_id = ? ";

		int count = getSimpleJdbcTemplate().queryForInt(sql, mouseId, cohortId);

		return (count == 0) ? false : true;
	}

	public Collection<Cohort> fetchByMouseId(Long mouseId) {
		String sql = " select c.* from mts_cohort c, mts_cohort_mouse m where c.id = m.cohort_id and m.mouse_id = ? order by lower(c.cohort_name) ";
		return fetchCollection(sql, mouseId);
	}

	public CohortListView fetchListViewById(Long id) {
		String sql = "select * from mts_cohort_list_vw where id = ?";
		CohortListView cohortListView = null;

		try{
			cohortListView = getSimpleJdbcTemplate().queryForObject(sql,cohortListViewRowMapper, id);
		}catch(IncorrectResultSizeDataAccessException e){

		}
		return cohortListView;
	}



	public Cohort fetchByName(String cohortName) {
		String sql = " select * from mts_cohort where lower(cohort_name) = lower(?) ";
		return fetchOne(sql, cohortName);
	}

	public Collection<CohortListView> search(CohortSearchBean bean) {
		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = constructSearchSql(bean, bindVariables, listPaginator.getViewName());
		return listPaginator.fetchSearchResults(sql, bindVariables, bean, cohortListViewRowMapper);
	}

	private String constructSearchSql(CohortSearchBean bean, Collection<Object> bindVariables, String tableName) {
		String selectSql  = " select c.* ";
		String fromSql = " from " + tableName + " c ";
		String whereSql = " where 1=1 ";

		if (bean.getCohortTypeDictId() != null) {
			whereSql += " and c.cohort_type_id = ?";
			bindVariables.add(bean.getCohortTypeDictId());
		}


		if(bean.getMouse() != null) {
			Mouse mouse = bean.getMouse();

			// mouse status
			if(bean.getMouseStatus() != null) {
				if(bean.getMouseStatus().equalsIgnoreCase("Alive")) {
					whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.death_date is null) ";
				}
				else if(bean.getMouseStatus().equalsIgnoreCase("Deceased")) {
					whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.death_date is not null) ";
				}
			}

			// mouse strain
			if(mouse.getStrainDictId() != null) {
				whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.mouse_strain_dict_id = ?) ";
				bindVariables.add(mouse.getStrainDictId());
			}

			// mouse id or similar
			if(mouse.getId() != null) {
				whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.id = ?) ";
				bindVariables.add(mouse.getId());
			}
			else if(mouse.getBarcode() != null && mouse.getBarcode().length() > 0) {
				whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.barcode = ?) ";
				bindVariables.add(mouse.getBarcode());
			}
			else if(mouse.getName() != null && mouse.getName().length() > 0) {
				whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.name = ?) ";
				bindVariables.add(mouse.getName());
			}
			else if (bean.getMouseIdentifier() != null && bean.getMouseIdentifier().length() > 0) {
				whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and ? in (m.name,m.barcode,to_char(m.id))) ";
				bindVariables.add(bean.getMouseIdentifier());
			}

			// cage id
			if(mouse.getCageId() != null) {
				whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.cage_id = ?) ";
				bindVariables.add(mouse.getCageId());
			}

			// colony id
			if(mouse.getColonyInfoId() != null) {
				whereSql += " and exists (select null from mts_mouse m, mts_cohort_mouse l where l.mouse_id = m.id and l.cohort_id = c.id and m.colony_info_id = ?) ";
				bindVariables.add(mouse.getColonyInfoId());
			}
		}

		if (bean.getCreatorId() != null) {
			whereSql += " and c.creator_id = ?";
			bindVariables.add(bean.getCreatorId());
		}

		if (bean.getFilterPeriodStart() != null) {
			whereSql += " and trunc(c.end_date) >= trunc(?)";
			bindVariables.add(bean.getFilterPeriodStart());
		}
		if (bean.getFilterPeriodEnd() != null) {
			whereSql += " and trunc(c.end_date) <= trunc(?)";
			bindVariables.add(bean.getFilterPeriodEnd());
		}



		String orderBySql = "";

		if(bean.getOrderBy() != null && bean.getOrderBy().length() > 0) {

			if (bean.getOrderBy().equalsIgnoreCase("total_mice")) {
				orderBySql = " order by nvl((select count(l.id) from mts_cohort_mouse l where l.cohort_id = c.id group by l.cohort_id),0)";
			}
			else if (bean.getOrderBy().equalsIgnoreCase("pipeline")) {
				orderBySql = " order by c.pipeline_dict_id";
			}
			else {
				orderBySql = " order by " + bean.getOrderBy();
			}

			if(bean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}

		String sql = selectSql + fromSql + whereSql + orderBySql;

		logger.debug(sql);

		return sql;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Cohort cohort) {
		sql.addLongParam(cohort.getCohortPipelineDictId(),"PIPELINE_DICT_ID");
		sql.addDateParam(cohort.getEndDate(),"END_DATE");
		sql.addLongParam(cohort.getCohortTypeDictId(), "COHORT_TYPE_ID");
		sql.addDateParam(cohort.getCreatedDate(),"CREATED_DATE");
		sql.addLongParam(cohort.getCreatorId(),"CREATOR_ID");
		sql.addStringParam(cohort.getEditedBy(),"EDITED_BY");
		sql.addDateParam(cohort.getEditDate(),"EDIT_DATE");
		sql.addLongParam(cohort.getCheckNumber(),"CHECK_NUMBER");
		sql.addStringParam(cohort.getComments(),"COMMENTS");
		sql.addStringParam(cohort.getReason(),"REASON");
		sql.addStringParam(cohort.getCohortName(), "COHORT_NAME");
		sql.addLongParam(cohort.getId(),"ID");
	}





}