/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.dao;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.LitterSearchBean;
import uk.ac.sanger.mig.mousetracking.dao.rowmappers.LitterListViewRowMapper;
import uk.ac.sanger.mig.mousetracking.services.LitterService;
import uk.ac.sanger.mig.mousetracking.views.LitterListView;

public class LitterDaoSpringImpl extends AbstractDaoSpringImpl<Litter> implements LitterDao {

	private PaginatedSearchUtils<LitterListView> paginator;
	private LitterListViewRowMapper litterListViewRowMapper;

	public void setLitterListViewRowMapper(
			LitterListViewRowMapper litterListViewRowMapper) {
		this.litterListViewRowMapper = litterListViewRowMapper;
	}

	public void setPaginator(PaginatedSearchUtils<LitterListView> paginator) {
		this.paginator = paginator;
	}

	public Collection<Litter> fetchByBreedingId(Long breedingId) {
		String sql = " select * from mts_litter where breeding_id = ? order by litter_number ";
		return fetchCollection(sql, breedingId);
	}

	public Litter getLastLitterForBreedingId(Long breedingId) {
		String sql = " select * from mts_litter where breeding_id = ? and edit_date in (select max(edit_date) from mts_litter where breeding_id = ?)";
		return fetchOne(sql, breedingId);
	}

	public Litter fetchByEmbryoTransferId(Long embryoTransferId) {
       String sql = "select l.* from mts_litter l, mts_breeding b where l.BREEDING_ID = b.id and b.EMBRYO_TRANSFER_ID=?";
       return fetchOne(sql, embryoTransferId);
	}

	public int nextLitterNumber(Long breedingId) {
		String sql = " select nvl(max(litter_number),0) + 1 next_litter from mts_litter where breeding_id = ? ";
		return getSimpleJdbcTemplate().queryForInt(sql, breedingId);
	}

	public Collection<LitterListView> search(LitterSearchBean searchBean) {

		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = " select l.* from mts_litter_list_vw l where 1=1 ";

		//litter name

		if(searchBean.getLitterName()!=null && !searchBean.getLitterName().equals("")){
			sql += " and lower(litter_name) = lower(?) ";
			bindVariables.add(searchBean.getLitterName());
		}

		//birth date range
		if(searchBean.getBirthDateFrom()!= null) {
			sql += " and birth_date >= ? ";
			bindVariables.add(defaultDateFormat.format(searchBean.getBirthDateFrom()));
		}

		if(searchBean.getBirthDateTo() != null) {
			sql += " and birth_date <= ? ";
			bindVariables.add(defaultDateFormat.format(searchBean.getBirthDateTo()));
		}

		//plug date range
		if(searchBean.getPlugDateFrom()!= null) {
			sql += " and plug_date >= ? ";
			bindVariables.add(defaultDateFormat.format(searchBean.getPlugDateFrom()));
		}

		if(searchBean.getPlugDateTo() != null) {
			sql += " and plug_date <= ? ";
			bindVariables.add(defaultDateFormat.format(searchBean.getPlugDateTo()));
		}

		if(searchBean.getParentId() != null) {
			sql += " and (mother_id = ? or father_id = ?) ";
			bindVariables.add(searchBean.getParentId());
			bindVariables.add(searchBean.getParentId());
		}


		if(searchBean.getStatus() != null) {

			if(searchBean.getStatus().equals(LitterService.LITTER_STATUS_UNBORN)) {
				sql += " and birth_date is null and nvl(number_embryos,0) = 0 and nvl(not_pregnant,0) != 1 ";
			}
			else if(searchBean.getStatus().equals(LitterService.LITTER_STATUS_HARVESTED)) {
				sql += " and number_embryos > 0 ";
			}
			else if(searchBean.getStatus().equals(LitterService.LITTER_STATUS_PREWEAN)) {
				sql += " and number_born > 0 and number_weaned = 0 and (number_used = 0 or number_used != number_pups) ";
			}
			else if(searchBean.getStatus().equals(LitterService.LITTER_STATUS_WEANED)) {
				sql += " and number_weaned > 0 ";
			}
			else if(searchBean.getStatus().equals(LitterService.LITTER_STATUS_NOT_PREGNANT)) {
				sql += " and not_pregnant = 1 ";
			}
		}

		if(searchBean.getColonyInfoId() != null) {
			sql += " and offspring_colony_id = ? ";
			bindVariables.add( searchBean.getColonyInfoId() );
		}

		if(searchBean.getRoomId() != null) {
			sql += " and exists (select null from mts_mouse m, mts_cage_vw c where l.mother_id = m.id and c.cage_id = m.cage_id and c.room_id = ?) ";
			bindVariables.add(searchBean.getRoomId());
		}

		sql += " order by ";
		if(searchBean.getOrderBy() == null || searchBean.getOrderBy().equals("") || searchBean.getOrderBy().equals("null")) {
			sql += " breeding_name_prefix,breeding_number,litter_number ";
		}
		// special case for sorting by name
		else if(searchBean.getOrderBy() != null && searchBean.getOrderBy().equals("name")) {
			if(searchBean.isOrderDesc()) {
				sql += " breeding_name_prefix desc,breeding_number desc,litter_number desc";
			}
			else {
				sql += " breeding_name_prefix,breeding_number,litter_number ";
			}
		}
		else {
			sql += searchBean.getOrderBy();

			if(searchBean.isOrderDesc()) {
				sql += " desc ";
			}
		}

		logger.debug(sql);

		return paginator.fetchSearchResults(sql, bindVariables, searchBean, litterListViewRowMapper);
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Litter litter) {
		sql.addLongParam(litter.getBreedingId(),"BREEDING_ID");
		sql.addDateParam(litter.getBirthDate(),"BIRTH_DATE");
		sql.addDateParam(litter.getDueDate(),"DUE_DATE");
		sql.addStringParam(litter.getComments(),"COMMENTS");
		sql.addIntParam(litter.getNumberBorn(),"NUMBER_BORN");
		sql.addIntParam(litter.getNumberHarvested(), "NUMBER_HARVESTED");
		sql.addLongParam(litter.getCheckNumber(),"CHECK_NUMBER");
		sql.addStringParam(litter.getEditedBy(),"EDITED_BY");
		sql.addDateParam(litter.getEditDate(),"EDIT_DATE");
		sql.addLongParam(litter.getCreatorId(),"CREATOR_ID");
		sql.addDateParam(litter.getCreatedDate(),"CREATED_DATE");
		sql.addDateParam(litter.getPlugDate(),"PLUG_CHECK_DATE");
		sql.addStringParam(litter.getNamePrefix(), "NAME_PREFIX");
		sql.addIntParam(litter.getLitterNumber(), "LITTER_NUMBER");
		sql.addBooleanParam(litter.isNotPregnant(), "NOT_PREGNANT");
		sql.addLongParam(litter.getId(),"ID");
	}

	public Litter fetchByName(String name) {
		String sql = "select * from mts_litter where name_prefix || litter_number = ?";
		return fetchOne(sql, name);
	}

}
