/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.dao;

import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.object.StoredProcedure;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseSearchBean;
import uk.ac.sanger.mig.mousetracking.dao.rowmappers.MouseFaxViewRowMapper;
import uk.ac.sanger.mig.mousetracking.dao.rowmappers.MouseListViewRowMapper;
import uk.ac.sanger.mig.mousetracking.views.MouseFaxView;
import uk.ac.sanger.mig.mousetracking.views.MouseListView;
import uk.ac.sanger.mig.personnel.Person;

public class MouseDaoSpringImpl extends AbstractDaoSpringImpl<Mouse> implements MouseDao {

	private MouseListViewRowMapper mouseListViewRowMapper;
	private MouseFaxViewRowMapper faxViewRowMapper;
	private PaginatedSearchUtils<Mouse> pojoPaginator;
	private PaginatedSearchUtils<MouseListView> listPaginator;

	public MouseListViewRowMapper getMouseListViewRowMapper() {
		return mouseListViewRowMapper;
	}

	public void setMouseListViewRowMapper(
			MouseListViewRowMapper mouseListViewRowMapper) {
		this.mouseListViewRowMapper = mouseListViewRowMapper;
	}

	public void setPojoPaginator(PaginatedSearchUtils<Mouse> paginator) {
		this.pojoPaginator = paginator;
	}

	public void setListPaginator(PaginatedSearchUtils<MouseListView> listPaginator) {
		this.listPaginator = listPaginator;
	}

	/**
	 * Fetch mouse by unique barcode or name if barcode not found
	 */
	public Mouse fetchByBarcode(String barcode) {
		Mouse mouse = null;

		String sql = "select * from " + getTableName() + " where barcode = upper(?) ";

		mouse = fetchOne(sql, barcode);

		if(mouse == null) {
			mouse = fetchByName(barcode);
		}

		if(mouse == null){
			Collection<Mouse> mice = fetchByRfidTag(barcode);
			if(mice.size() == 1){
				mouse = mice.iterator().next();
			}
		}

		return mouse;
	}

	public Collection<Mouse> fetchByRfidTag(String barcode) {
		String sql = "select * from mts_mouse where rfid_tag = ? and death_date is null";
		return fetchCollection(sql, barcode);
	}

	public boolean rfidTagInUse(Mouse mouse) {
		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = "select count(*) from mts_mouse where rfid_tag = ? and death_date is null ";
		bindVariables.add(mouse.getRfidTag());

		if(mouse.hasId()) {
			sql += " and id != ? ";
			bindVariables.add(mouse.getId());
		}

		int result = getSimpleJdbcTemplate().queryForInt(sql, bindVariables.toArray());

		return (result == 0) ? false : true;
	}


	/**
	 * Fetch mouse by unique name
	 * For this query to work effectively, a function-based index on lower(name) is required
	 */
	public Mouse fetchByName(String mouseName) {
		String sql = "select * from mts_mouse where lower(name) = lower(?)";
		return fetchOne(sql, mouseName);
	}

	/**
	 * Fetch collection of mouse objects by arbitrary search criteria
	 */
	public Collection<Mouse> fetch(MouseSearchBean bean) {
		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = constructSearchSql(bean, bindVariables, "mts_mouse");
		return pojoPaginator.fetchSearchResults(sql, bindVariables, bean, getRowMapper());
	}

	/**
	 * Fetch collection of mouseListView objects by arbitrary search criteria
	 */
	public Collection<MouseListView> search(MouseSearchBean bean) {
		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = constructSearchSql(bean, bindVariables, "mts_mouse_vw");
		return listPaginator.fetchSearchResults(sql, bindVariables, bean, mouseListViewRowMapper);
	}

	public MouseFaxView  fetchFaxMouseByBarcode(String barcode){

		String sql = " select * from fax_mouse_vw where barcode=?";


		MouseFaxView mouse = null;
		try {
			mouse = getSimpleJdbcTemplate().queryForObject(sql, getFaxViewRowMapper(), barcode);
		}
		catch(IncorrectResultSizeDataAccessException e){
			mouse = null;
		}

		return mouse;
	}

	/**
	 * Fetch collection of mice by unique cage id
	 */
	public Collection<Mouse> fetchByCageId(Long cageId) {
		String sql = " select * from mts_mouse where cage_id = ? order by name ";
		return fetchCollection(sql, cageId);
	}

	public Collection<Mouse> fetchHistoricByCageId(Long cageId) {
		String sql = " select m.* from mts_mouse m where exists (select null from mts_mouse_cage_history h where h.mouse_id = m.id and h.cage_id = ? and h.end_date is not null) and nvl(m.cage_id,-1) != ? order by name";
		return fetchCollection(sql, cageId, cageId);
	}

	/**
	 * Fetch collection of mice by unique litter id
	 */
	public Collection<Mouse> fetchByLitterId(Long litterId) {
		String sql = " select * from mts_mouse where litter_id = ? order by name";
		return fetchCollection(sql, litterId);
	}

	/**
	 * Fetch collection of mice by unique cohort id ordered by gender, location and name
	 */
	public Collection<Mouse> fetchByCohortId(Long cohortId) {
		String sql = " select m.* from mts_mouse m, mts_cohort_mouse c where c.cohort_id = ? and c.mouse_id = m.id order by m.gender_dict_id desc, m.cage_id, m.name ";
		return fetchCollection(sql, cohortId);
	}

	/**
	 *
	 */
	public Collection<Mouse> fetchParentsById(Long mouseId) {
		String sql = " select * from mts_mouse where id in (select parent_id from mts_parent_offspring_vw where offspring_id = ?)";
		Collection<Mouse> parents = getSimpleJdbcTemplate().query(sql, getRowMapper(), mouseId);
		return parents;
	}

	private String constructSearchSql(MouseSearchBean bean, Collection<Object> bindVariables, String tableName) {

		String selectSql  = " select m.* ";
		String fromSql = " from " + tableName + " m ";
		String whereSql = " where 1=1 ";

		// ID
		if(bean.getId() != null) {
			whereSql += " and m.id = ? ";
			bindVariables.add(bean.getId());
		}

		// Name (only search this if ID is null)
		if(bean.getName() != null && bean.getId() == null && !bean.getName().equals("")) {
			whereSql += " and lower(m.name) = lower(?) ";
			bindVariables.add(bean.getName());
		}

		// Barcode (only search this if ID is null)
		if(bean.getBarcode() != null && bean.getId() == null && !bean.getBarcode().equals("")) {
			whereSql += " and m.barcode = ? ";
			bindVariables.add(bean.getBarcode());
		}

		// Gender Dict ID
		if(bean.getGenderDictId() != null) {
			whereSql += " and m.gender_dict_id = ? ";
			bindVariables.add(bean.getGenderDictId());
		}

		// Colour Dict ID
		if(bean.getCoatColourDictId() != null) {
			whereSql += " and m.coat_colour_dict_id = ? ";
			bindVariables.add(bean.getCoatColourDictId());
		}

		// Earmark Dict ID
		if(bean.getEarmarkDictId() != null) {
			whereSql += " and m.earmark_dict_id = ? ";
			bindVariables.add(bean.getEarmarkDictId());
		}
		else if (bean.isNullEarmark()) {
			whereSql += " and m.earmark_dict_id is null";
		}

		// Eyecolour Dict ID
		if(bean.getEyeColourDictId() != null) {
			whereSql += " and m.eye_colour_dict_id = ? ";
			bindVariables.add(bean.getEyeColourDictId());
		}

		//rfid tag
		if(bean.getRfidTag() != null) {
			whereSql += " and m.rfid_tag = ?";
			bindVariables.add(bean.getRfidTag());
		}

		if(bean.getCohortId() != null) {
			whereSql += " and exists (select null from mts_cohort_mouse cm where cm.mouse_id = m.id and cm.cohort_id = ?) ";
			bindVariables.add(bean.getCohortId());
		}

		// ColonyInfo ID
		if(bean.getColonyInfoId() != null) {
			whereSql += " and m.colony_info_id = ? ";
			bindVariables.add(bean.getColonyInfoId());
		}

		// Strain
		if(bean.getStrainId() != null) {
			whereSql += " and m.mouse_strain_dict_id = ? ";
			bindVariables.add(bean.getStrainId());
		}

		// Generation
		if(bean.getGeneration() != null && !bean.getGeneration().equals("")) {
			whereSql += " and m.generation = ? ";
			bindVariables.add(bean.getGeneration());
		}

		// Comments
		if(bean.getComments() != null && !bean.getComments().equals("")) {
			whereSql += " and lower(m.comments) like lower(?||'%') ";
			bindVariables.add(bean.getComments());
		}

		// Litter ID
		if(bean.getLitterId() != null) {
			whereSql += " and m.litter_id = ? ";
			bindVariables.add(bean.getLitterId());
		}
		else if (bean.isLitterIdPresent()) {
			whereSql += " and m.litter_id is not null ";
		}

		// Mother ID
		if(bean.getMotherId() != null || bean.getFatherId() != null) {
			fromSql += ", mts_breeding b, mts_litter l ";
			whereSql += " and b.id = l.breeding_id and l.id = m.litter_id ";
		}

		if(bean.getMotherId() != null) {
			whereSql += " and b.female_id = ? ";
			bindVariables.add(bean.getMotherId());
		}

		// Father ID
		if(bean.getFatherId() != null) {
			whereSql += " and b.male_id = ? ";
			bindVariables.add(bean.getFatherId());
		}

		// Birth Date
		if(bean.isNullBirthDate()) {
			whereSql += " and m.birth_date is null ";
		}
		else {
			if(bean.getBirthDateFrom() != null) {
				whereSql += " and m.birth_date >= ? ";
				bindVariables.add(defaultDateFormat.format(bean.getBirthDateFrom()));
			}
			if(bean.getBirthDateTo() != null) {
				whereSql += " and m.birth_date <= ? ";
				bindVariables.add(defaultDateFormat.format(bean.getBirthDateTo()));
			}
		}

		// Wean Date
		if(bean.isNullWeanDate()) {
			whereSql += " and m.wean_date is null ";
		}
		else {
			if(bean.getWeanDateFrom() != null) {
				whereSql += " and m.wean_date >= ? ";
				bindVariables.add(defaultDateFormat.format(bean.getWeanDateFrom()));
			}
			if(bean.getWeanDateTo() != null) {
				whereSql += " and m.wean_date <= ? ";
				bindVariables.add(defaultDateFormat.format(bean.getWeanDateTo()));
			}
		}

		// Death Date
		if(bean.isNullDeathDate()) {
			whereSql += " and m.death_date is null ";
		}
		else {
			if(bean.getDeathDateFrom() != null) {
				whereSql += " and m.death_date >= ? ";
				bindVariables.add(defaultDateFormat.format(bean.getDeathDateFrom()));
			}
			if(bean.getDeathDateTo() != null) {
				whereSql += " and m.death_date <= ? ";
				bindVariables.add(defaultDateFormat.format(bean.getDeathDateTo()));
			}
		}

		// Cause of death
		if(bean.getCauseOfDeathDictId() != null) {
			if(bean.getCauseOfDeathDictId().equals(-1L)) {
				whereSql += " and m.cause_of_death_dict_id is not null";
			}
			else {
				whereSql += " and m.cause_of_death_dict_id = ? ";
				bindVariables.add(bean.getCauseOfDeathDictId());
			}
		}

		// Cage
		if(bean.getCageId() != null) {
			whereSql += " and m.cage_id = ? ";
			bindVariables.add(bean.getCageId());
		}

		// Rack
		if(bean.getRackId() != null) {
			whereSql += " and m.rack_id = ? ";
			bindVariables.add(bean.getRackId());
		}

		// Room
		if(bean.getRoomId() != null) {
			whereSql += " and exists (select null from mts_cage c, mts_rack r where m.cage_id = c.id and c.rack_id = r.id and r.room_id = ?) ";
			bindVariables.add(bean.getRoomId());
		}

		// Non Breeder
		if(bean.getNonBreeder() != null) {
			whereSql += " and m.is_non_breeder = ? ";
			bindVariables.add(bean.getNonBreeder() ? 1 : 0);
		}

		// Sentinel
		if(bean.getSentinel() != null) {
			whereSql += " and m.is_sentinel = ? ";
			bindVariables.add(bean.getSentinel() ? 1 : 0);

		}

		// Status
		if(bean.getStatus() != null && !bean.getStatus().equals("")) {
			whereSql += " and m.status = ? ";
			bindVariables.add(bean.getStatus());
		}

		//Owner
		if(bean.getCurrentOwnerId()!= null ) {
			whereSql += " and m.current_owner_id = ? ";
			bindVariables.add(bean.getCurrentOwnerId());
		}

		//PPL
		if(bean.getProjectLicenceId() != null ) {
			whereSql += " and m.current_project_licence_id = ? ";
			bindVariables.add(bean.getProjectLicenceId());
		}

		//Protocol
		if(bean.getProtocolId()!= null ) {
			whereSql += " and m.current_protocol_id = ? ";
			bindVariables.add(bean.getProtocolId());
		}

		// has current health concern
		if(bean.getHasCurrentHealthConcern() != null) {
			whereSql += " and ";
			if(bean.getHasCurrentHealthConcern() == false) {
				whereSql += " not ";
			}
			whereSql += " exists (select null from mts_mouse_health h where h.mouse_id = m.id and h.end_date is null ) ";
		}

		// age range
		if(bean.getAgeInMonths() != null) {
			whereSql += " and (m.birth_date between add_months((sysdate - (? * 7)), -?) ";
			bindVariables.add(bean.getAgeRangeInWeeks());
			bindVariables.add(bean.getAgeInMonths());
			whereSql += " and add_months((sysdate + (? * 7)), -? ))";
			bindVariables.add(bean.getAgeRangeInWeeks());
			bindVariables.add(bean.getAgeInMonths());
		}

		// breeding checks
		if(bean.getMating() != null) {
			whereSql += " and " + (bean.getMating().booleanValue() ? "" : "not")   + " exists (select null from mts_breeding b where b.end_date is null and (b.female_id = m.id or b.male_id = m.id)) ";
		}

		// allele
		if(bean.getAlleleId() != null) {
			fromSql += ",mts_mouse_allele a1 ";
			whereSql += " and m.id = a1.mouse_id ";
			whereSql += " and a1.allele_id = ? ";
			bindVariables.add(bean.getAlleleId());
		}

		// genotype status
		if(bean.getGenotypeDictId() != null) {

			// wildtype is special case
			if(GenotypeDictionary.WILDTYPE.equals(bean.getGenotypeDictId())) {
				whereSql += " and not exists (select null from mts_mouse_allele a where a.mouse_id = m.id and a.genotype_dict_id != 1) ";
			}
			else {
				whereSql += " and exists (select null from mts_mouse_allele a where a.mouse_id = m.id and a.genotype_dict_id = ?) ";
				bindVariables.add(bean.getGenotypeDictId());
			}
		}

		// find chimeras from MI Request number
		if(bean.getMicroInjectionRequestId() != null) {
			whereSql += " and exists (select null from mts_litter lt, mts_breeding bt, " +
			"mir_embryo_transfer_procedure et, mir_scheduled_micro_injection st " +
			"where m.litter_id = lt.id and lt.breeding_id = bt.id " +
			"and bt.embryo_transfer_id = et.id and et.scheduled_micro_injection_id = st.id " +
			"and st.micro_injection_request_id = ?) ";
			bindVariables.add(bean.getMicroInjectionRequestId());
		}

		// Reservation Category
		if (bean.getReservationCategoryId() != null) {
			whereSql += " and exists (select null" +
			" from res_mouse_reservation mr" +
			" join res_reservation r on mr.reservation_id = r.id" +
			" join res_purpose p on r.purpose_id = p.id" +
			" where mr.mouse_id = m.id and p.category_id = ?)";
			bindVariables.add(bean.getReservationCategoryId());
		}
		// Reservation Purpose
		if (bean.getReservationPurposeId() != null) {
			whereSql += " and exists (select null" +
			" from res_mouse_reservation mr" +
			" join res_reservation r on mr.reservation_id = r.id" +
			" where mr.mouse_id = m.id and r.purpose_id = ?)";
			bindVariables.add(bean.getReservationPurposeId());
		}
		// Reservation Owner
		if (bean.getReserverId() != null) {
			whereSql += " and exists (select null" +
			" from res_mouse_reservation mr" +
			" join res_reservation r on mr.reservation_id = r.id" +
			" where mr.mouse_id = m.id and r.reserver_id = ?)";
			bindVariables.add(bean.getReserverId());
		}

		// Reservation Team
		if (bean.getReservationTeamId() != null) {
			whereSql += " and exists (select null" +
			" from res_mouse_reservation mr" +
			" join res_reservation r on mr.reservation_id = r.id" +
			" where mr.mouse_id = m.id and r.team_id = ?)";
			bindVariables.add(bean.getReservationTeamId());
		}

		if( bean.getHistologyProgress() != null ) {

			switch(  bean.getHistologyProgress() ) {

			case SLIDES_EXIST:

				whereSql += " and exists (select null from htl_necropsy n, htl_fixation f, " +
				" htl_block b, htl_slide sl, htl_scan sc where m.id = n.mouse_id " +
				" AND n.id = f.necropsy_id AND f.id = b.fixation_id " +
				" AND b.id = sl.block_id ) ";
				break;

			case SCANS_EXIST:

				whereSql += " and exists (select null from htl_necropsy n, htl_fixation f, " +
				" htl_block b, htl_slide sl, htl_scan sc where m.id = n.mouse_id " +
				" AND n.id = f.necropsy_id AND f.id = b.fixation_id AND " +
				" b.id = sl.block_id AND sl.id = sc.slide_id) ";
				break;

			case SCANS_QC_EXIST:

				whereSql += " and exists (select null from htl_necropsy n, htl_fixation f, " +
				" htl_block b, htl_slide sl, htl_scan sc where m.id = n.mouse_id " +
				" AND n.id = f.necropsy_id AND f.id = b.fixation_id  AND " +
				" b.id = sl.block_id AND sl.id = sc.slide_id AND " +
				" sc.status_dict_id in(5,6,7)) ";
				break;


			case CROPS_EXIST:

				whereSql += " and exists (select null from htl_necropsy n, htl_fixation f, " +
				" htl_block b, htl_slide sl, htl_scan sc, htl_crop_job cj, " +
				" htl_crop cr where m.id = n.mouse_id AND n.id = f.necropsy_id " +
				" AND f.id = b.fixation_id AND b.id = sl.block_id AND " +
				" sl.id = sc.slide_id AND sc.status_dict_id = 5 AND " +
				" sc.id = cj.scan_id AND cj.id = cr.crop_job_id AND cr.full_image_path is not null) ";
				break;

			case CROPS_QC_AWAIT:

				whereSql += " and exists (select null from htl_necropsy n, htl_fixation f, " +
				" htl_block b, htl_slide sl, htl_scan sc, htl_crop_job cj, " +
				" htl_crop cr where m.id = n.mouse_id AND n.id = f.necropsy_id " +
				" AND f.id = b.fixation_id AND b.id = sl.block_id AND " +
				" sl.id = sc.slide_id AND sc.status_dict_id = 5 AND sc.id = cj.scan_id " +
				" AND cj.id = cr.crop_job_id AND cr.qc_pass is null) ";
				break;

			case CROPS_QC_EXIST:

				whereSql += " and exists (select null from htl_necropsy n, htl_fixation f, " +
				" htl_block b, htl_slide sl, htl_scan sc, htl_crop_job cj, " +
				" htl_crop cr where m.id = n.mouse_id AND n.id = f.necropsy_id " +
				" AND f.id = b.fixation_id AND b.id = sl.block_id AND " +
				" sl.id = sc.slide_id AND sc.status_dict_id = 5 AND sc.id = cj.scan_id " +
				" AND cj.id = cr.crop_job_id AND cr.qc_pass is not null) ";
				break;
			}
		}

		String orderBySql = "";

		if(bean.getOrderBy() != null && bean.getOrderBy().length() > 0) {
			orderBySql += " order by " + bean.getOrderBy();
			if(bean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}

		String sql = selectSql + fromSql + whereSql + orderBySql;

		logger.debug(sql);

		return sql;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Mouse mouse) {

		sql.addLongParam(mouse.getId(),"ID");
		sql.addStringParam(mouse.getBarcode(),"BARCODE");
		sql.addLongParam(mouse.getLitterId(),"LITTER_ID");
		sql.addStringParam(mouse.getName(),"NAME");
		sql.addDateParam(mouse.getEditDate(),"EDIT_DATE");
		sql.addStringParam(mouse.getRfidTag(),"RFID_TAG");
		sql.addLongParam(mouse.getCoatColourDictId(),"COAT_COLOUR_DICT_ID");
		sql.addLongParam(mouse.getEyeColourDictId(),"EYE_COLOUR_DICT_ID");
		sql.addLongParam(mouse.getEarmarkDictId(),"EARMARK_DICT_ID");
		sql.addDateParam(mouse.getBirthDate(),"BIRTH_DATE");
		sql.addDateParam(mouse.getDeathDate(),"DEATH_DATE");
		sql.addLongParam(mouse.getCauseOfDeathDictId(),"CAUSE_OF_DEATH_DICT_ID");
		sql.addStringParam(mouse.getEditedBy(),"EDITED_BY");
		sql.addDateParam(mouse.getWeanDate(),"WEAN_DATE");
		sql.addLongParam(mouse.getCageId(),"CAGE_ID");
		sql.addLongParam(mouse.getCurrentOwnerId(),"CURRENT_OWNER_ID");
		sql.addLongParam(mouse.getCheckNumber(),"CHECK_NUMBER");
		sql.addLongParam(mouse.getGenderDictId(),"GENDER_DICT_ID");
		sql.addLongParam(mouse.getStrainDictId(),"MOUSE_STRAIN_DICT_ID");
		sql.addStringParam(mouse.getComments(),"COMMENTS");
		sql.addBooleanParam(mouse.isNonBreeder(),"IS_NON_BREEDER");
		sql.addBooleanParam(mouse.isSentinel(),"IS_SENTINEL");
		sql.addStringParam(mouse.getGeneration(), "GENERATION");
		sql.addDateParam(mouse.getArrivalDate(), "ARRIVAL_DATE");
		sql.addLongParam(mouse.getColonyInfoId(), "COLONY_INFO_ID");
		sql.addLongParam(mouse.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(mouse.getCreatedDate(),"CREATED_DATE");
		sql.addBooleanParam(mouse.isSterile(), "IS_STERILE");
		sql.addBooleanParam(mouse.isPooled(), "IS_POOLED");
		sql.addBooleanParam(mouse.isStud(), "IS_STUD");
		sql.addBooleanParam(mouse.isProvenStud(), "IS_PROVEN_STUD");
	}

	public Mouse fetchMouseByTissueSampleId(Long tissueSampleId) {
		String sql = "select m.* from mts_mouse m, mts_tissue_sample mts where m.id = mts.mouse_id and mts.id = ?";
		return fetchOne(sql, tissueSampleId);
	}

	public void deleteMouse(Mouse mouse, Person user) {
		DeleteMouseStoredProcedure sp = new DeleteMouseStoredProcedure(getDataSource());
		sp.delete(mouse.getId());
	}

	public void setFaxViewRowMapper(MouseFaxViewRowMapper faxViewRowMapper) {
		this.faxViewRowMapper = faxViewRowMapper;
	}

	public MouseFaxViewRowMapper getFaxViewRowMapper() {
		return faxViewRowMapper;
	}
}

class DeleteMouseStoredProcedure extends StoredProcedure {

	public DeleteMouseStoredProcedure(DataSource dataSource) {
		setDataSource(dataSource);
		setSql("mts.mts_delete_mouse");

		declareParameter(new SqlParameter("in_mouse_id", Types.NUMERIC));

		compile();
	}

	public void delete(Long mouseId) {
		Map<String, Object> inputParameters = new HashMap<String, Object>();
		inputParameters.put("in_mouse_id", mouseId);
		execute(inputParameters);
	}
}
