/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.dao;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.personnel.Person;

public class MouseProcedureDaoSpringImpl extends AbstractDaoSpringImpl<MouseProcedure> implements MouseProcedureDao {

	public MouseProcedure fetchByTableRef(TableMetadataEnum tableName, Long fkRef) {
		String sql = " select * from mts_mouse_procedure where foreign_table_name = ? and foreign_key_id = ? ";
		return fetchOne(sql, tableName.name(), fkRef);
	}


	public MouseProcedure fetchCurrentLicencedByMouseId(Long mouseId) {
		String sql = " select * from (select * from mts_mouse_procedure where mouse_id = ? and protocol_id is not null order by start_date desc, id desc) where rownum=1 ";
		return fetchOne(sql, mouseId);
	}


	public boolean doesProcedureExist(MouseProcedure mouseProcedure) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		String sql = " select count(*) record_count from mts_mouse_procedure where mouse_id = ? and start_date = ? and procedure_dict_id = ? and personal_licence_id = ? ";
		bindVariables.add(mouseProcedure.getMouseId());
		bindVariables.add(mouseProcedure.getStartDate());
		bindVariables.add(mouseProcedure.getProcedureDictId());
		bindVariables.add(mouseProcedure.getPersonalLicenceId());

		if (mouseProcedure.getProtocolId() == null) {
			sql += " and protocol_id is null ";
		} else {
			sql += " and protocol_id = ? ";
			bindVariables.add(mouseProcedure.getProtocolId());
		}

		int recordCount = getSimpleJdbcTemplate().queryForInt(sql, bindVariables.toArray());

		return (recordCount == 0) ? false : true;
	}


	public MouseProcedure fetchByTableRefAndMouseId(TableMetadataEnum tableName, Long mouseId) {
		String sql = " select * from mts_mouse_procedure where foreign_table_name = ? and mouse_id = ? ";
		return fetchOne(sql, tableName.name(), mouseId);
	}


	/**
	 * Find all mouse procedures for a mouse with the passed procedure and returns them as a
	 * collection.
	 *
	 * @param procedureDictId -
	 *            the id of the procedure to lookup.
	 * @param mouseId -
	 *            the id of the mouse to lookup.
	 * @return collection of MouseProcedure objects.
	 */
	public Collection<MouseProcedure> fetchByProcedureDictIdAndMouseId(Long procedureDictId, Long mouseId) {
		String sql = " select * from mts_mouse_procedure where procedure_dict_id = ? and mouse_id = ? order by created_date";
		return fetchCollection(sql, procedureDictId, mouseId);
	}


	public Collection<MouseProcedure> fetchByMouseId(Long mouseId) {
		String sql = " select * from mts_mouse_procedure where mouse_id = ? and procedure_dict_id is not null order by start_date,id ";
		return fetchCollection(sql, mouseId);
	}


	public boolean isLicencedProcedure(Long procedureDictId) {
		String sql = " select is_licenced from mts_procedure_dict where id = ? ";

		int intBoolean = getSimpleJdbcTemplate().queryForInt(sql, procedureDictId);

		return (intBoolean == 0) ? false : true;
	}


	public boolean isTerminalProcedure(Long procedureDictId) {
		String sql = " select is_terminal from mts_procedure_dict where id = ? ";

		int intBoolean = getSimpleJdbcTemplate().queryForInt(sql, procedureDictId);

		return (intBoolean == 0) ? false : true;
	}

	public boolean isAllowablePostMortem(Long procedureDictId) {

		String sql = " select is_allowable_postmortem from mts_procedure_dict where id = ? ";

		int intBoolean = getSimpleJdbcTemplate().queryForInt(sql, procedureDictId);

		return (intBoolean == 0) ? false : true;

	}


	public int deleteByTableRef(TableMetadataEnum tableName, Long fkRef, Person user) {

		MouseProcedure mouseProcedure = fetchByTableRef(tableName, fkRef);

		if (mouseProcedure == null) {
			throw new RuntimeException("Cannot find MouseProcedure to delete w/tableRef provided");
		}
		return delete(mouseProcedure, user);
	}


	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, MouseProcedure mouseProcedure) {
		sql.addLongParam(mouseProcedure.getMouseId(), "MOUSE_ID");
		sql.addLongParam(mouseProcedure.getProtocolId(), "PROTOCOL_ID");
		sql.addLongParam(mouseProcedure.getPersonalLicenceId(), "PERSONAL_LICENCE_ID");
		sql.addDateParam(mouseProcedure.getStartDate(), "START_DATE");
		sql.addDateParam(mouseProcedure.getEndDate(), "END_DATE");
		sql.addEnumParam(mouseProcedure.getForeignTableName(), "FOREIGN_TABLE_NAME");
		sql.addLongParam(mouseProcedure.getForeignKeyId(), "FOREIGN_KEY_ID");
		sql.addDateParam(mouseProcedure.getEditDate(), "EDIT_DATE");
		sql.addStringParam(mouseProcedure.getEditedBy(), "EDITED_BY");
		sql.addDateParam(mouseProcedure.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(mouseProcedure.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(mouseProcedure.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(mouseProcedure.getProcedureDictId(), "PROCEDURE_DICT_ID");
		sql.addLongParam(mouseProcedure.getId(), "ID");
	}
}