/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.forms;

import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class LitterForm {

	public enum MODE {
		BORN,
		HARVESTED,
		NEW,
		SPLIT
	}

	private Litter litter;
	private String changeForm;
	private String changeFormValue;
	private Mouse defaultPup;
	private Long maleCountOnForm;
	private Long femaleCountOnForm;
	private Boolean cullFemaleRecipient;

	private boolean strainRequired;
	private boolean colonyRequired;
	private boolean generationRequired;
	private boolean locationRequired;
	private boolean cullFemaleConfirmationRequired;

	private BarcodedObject maleLocation;
	private BarcodedObject femaleLocation;
	private BarcodedObject unsexedLocation;

	private int malesInCage;
	private int femalesInCage;
	private int totalMalesPerCage = MouseService.MAX_MICE_PER_CAGE;
	private int totalFemalesPerCage = MouseService.MAX_MICE_PER_CAGE;

	private int unsexedInCage;
	private int totalUnsexedPerCage = MouseService.MAX_MICE_PER_CAGE;

	private ProcedureSubform procedureSubform;

	private boolean procedureRequired;
	private boolean addNewProcedure;

	private MODE mode;

	public Litter getLitter() {
		return litter;
	}

	public void setLitter(Litter litter) {
		this.litter = litter;
	}

	public String getChangeForm() {
		return changeForm;
	}

	public void setChangeForm(String changeForm) {
		this.changeForm = changeForm;
	}

	public String getChangeFormValue() {
		return changeFormValue;
	}

	public void setChangeFormValue(String changeFormValue) {
		this.changeFormValue = changeFormValue;
	}

	public Mouse getDefaultPup() {
		return defaultPup;
	}

	public void setDefaultPup(Mouse defaultPup) {
		this.defaultPup = defaultPup;
	}

	public Long getMaleCountOnForm() {
		return maleCountOnForm;
	}

	public void setMaleCountOnForm(Long maleCountOnForm) {
		this.maleCountOnForm = maleCountOnForm;
	}

	public Long getFemaleCountOnForm() {
		return femaleCountOnForm;
	}

	public void setFemaleCountOnForm(Long femaleCountOnForm) {
		this.femaleCountOnForm = femaleCountOnForm;
	}

	public Boolean getCullFemaleRecipient() {
		return cullFemaleRecipient;
	}

	public void setCullFemaleRecipient(Boolean cullFemaleRecipient) {
		this.cullFemaleRecipient = cullFemaleRecipient;
	}

	public boolean isStrainRequired() {
		return strainRequired;
	}

	public void setStrainRequired(boolean strainRequired) {
		this.strainRequired = strainRequired;
	}

	public boolean isColonyRequired() {
		return colonyRequired;
	}

	public void setColonyRequired(boolean colonyRequired) {
		this.colonyRequired = colonyRequired;
	}

	public boolean isGenerationRequired() {
		return generationRequired;
	}

	public void setGenerationRequired(boolean generationRequired) {
		this.generationRequired = generationRequired;
	}

	public MODE getMode() {
		return mode;
	}

	public void setMode(MODE displayMode) {
		this.mode = displayMode;
	}

	public boolean isCullFemaleConfirmationRequired() {
		return cullFemaleConfirmationRequired;
	}

	public void setCullFemaleConfirmationRequired(
			boolean cullFemaleConfirmationRequired) {
		this.cullFemaleConfirmationRequired = cullFemaleConfirmationRequired;
	}

	public boolean isAdditionalFieldsRequired() {
		return(colonyRequired || strainRequired || generationRequired || cullFemaleConfirmationRequired);
	}

	public boolean isLocationRequired() {
		return locationRequired;
	}

	public void setLocationRequired(boolean locationRequired) {
		this.locationRequired = locationRequired;
	}

	public BarcodedObject getMaleLocation() {
		return maleLocation;
	}

	public void setMaleLocation(BarcodedObject maleLocation) {
		this.maleLocation = maleLocation;
	}

	public BarcodedObject getFemaleLocation() {
		return femaleLocation;
	}

	public void setFemaleLocation(BarcodedObject femaleLocation) {
		this.femaleLocation = femaleLocation;
	}

	public BarcodedObject getUnsexedLocation() {
		return unsexedLocation;
	}

	public void setUnsexedLocation(BarcodedObject unsexedLocation) {
		this.unsexedLocation = unsexedLocation;
	}

	public boolean isAnyLocationRequired() {
		return (isMaleLocationRequired() || isFemaleLocationRequired() || isUnsexedLocationRequired());
	}

	public boolean isMaleLocationRequired() {
		return isLocationRequired(MouseService.MOUSE_GENDER_MALE);
	}

	public boolean isFemaleLocationRequired() {
		return isLocationRequired(MouseService.MOUSE_GENDER_FEMALE);
	}

	public boolean isUnsexedLocationRequired() {
		return isLocationRequired(MouseService.MOUSE_GENDER_UNDEFINED);
	}

	/**
	 * Location becomes a required field when any mice of the specified gender are alive
	 * and have a wean date
	 * and are in the same cage as mum
	 * @param genderDictId
	 * @return
	 */
	private boolean isLocationRequired(Long genderDictId) {
		boolean required = false;

		for(Mouse pup : litter.getPups()) {
			if(pup.isAlive() && pup.getWeanDate() != null && genderDictId.equals(pup.getGenderDictId())) {
				if(pup.getCageId() == null || pup.getCageId().equals(litter.getCageId())) {
					required = true;
					break;
				}
				else if(this.getMode() == MODE.SPLIT) {
					required = true;
					break;
				}
			}
		}

		return required;
	}

	public int getMalesInCage() {
		return malesInCage;
	}

	public void setMalesInCage(int malesInCage) {
		this.malesInCage = malesInCage;
	}

	public int getFemalesInCage() {
		return femalesInCage;
	}

	public void setFemalesInCage(int femalesInCage) {
		this.femalesInCage = femalesInCage;
	}

	public int getTotalMalesPerCage() {
		return totalMalesPerCage;
	}

	public void setTotalMalesPerCage(int totalMalesPerCage) {
		this.totalMalesPerCage = totalMalesPerCage;
	}

	public int getTotalFemalesPerCage() {
		return totalFemalesPerCage;
	}

	public void setTotalFemalesPerCage(int totalFemalesPerCage) {
		this.totalFemalesPerCage = totalFemalesPerCage;
	}

	public int getUnsexedInCage() {
		return unsexedInCage;
	}

	public void setUnsexedInCage(int unsexedInCage) {
		this.unsexedInCage = unsexedInCage;
	}

	public int getTotalUnsexedPerCage() {
		return totalUnsexedPerCage;
	}

	public void setTotalUnsexedPerCage(int totalUnsexedPerCage) {
		this.totalUnsexedPerCage = totalUnsexedPerCage;
	}

	public int getFemaleCount() {
		return (litter.getFemaleCount() == null) ? 0 : litter.getFemaleCount();
	}

	public int getMaleCount() {
		return (litter.getMaleCount() == null) ? 0 : litter.getMaleCount();
	}

	public int getWeaningCount(Long genderDictId, Long litterCageId) {
		int count = 0;
		if(litter.getPups() != null) {
			for(Mouse pup : litter.getPups()) {
				if(pup.isAlive() && genderDictId.equals(pup.getGenderDictId()) && pup.isWeaned() && (pup.getCageId() == null || pup.getCageId().equals(litterCageId))) {
					count++;
				}
			}
		}
		return count;
	}

	public int getUnsexedCount() {
		return (litter.getUndefinedGenderCount() == null) ? 0 : litter.getUndefinedGenderCount();
	}

	public ProcedureSubform getProcedureSubform() {
		return procedureSubform;
	}

	public void setProcedureSubform(ProcedureSubform procedureSubform) {
		this.procedureSubform = procedureSubform;
	}

	public boolean isProcedureRequired() {
		return procedureRequired;
	}

	public void setProcedureRequired(boolean procedureRequired) {
		this.procedureRequired = procedureRequired;
	}

	public boolean isAddNewProcedure() {
		return addNewProcedure;
	}

	public void setAddNewProcedure(boolean addNewProcedure) {
		this.addNewProcedure = addNewProcedure;
	}
}
