/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.Collection;
import java.util.Date;
import java.util.Set;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.CommonServices;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.BreedingSearchBean;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.forms.AddBreedingForm;
import uk.ac.sanger.mig.mousetracking.forms.BreedingForm;
import uk.ac.sanger.mig.personnel.Person;

public interface BreedingService extends CommonServices<Breeding>{

	public static final String BREEDING_STATUS_COMPLETE = "Complete";
	public static final String BREEDING_STATUS_SEPARATED = "Separated";
	public static final String BREEDING_STATUS_BREEDING = "Mating";
	public static final String BREEDING_STATUS_PSEUDO = "Pseudo";

	public Cage fetchForCageView(Long breedingId);

	public BreedingType fetchBreedingType(Long breedingId);

	public void editBreeding(BreedingForm breedingForm, Person user);

	public Collection<Breeding> search(BreedingSearchBean searchBean, ComponentMap requiredComponent);

    /**
     * Calculate what the generation field should be for an Mouse based on
     * the generation of it's parents.
     *
     * If the breeding is a pseudo-breeding (i.e sterile male) then
     * the generation will always be F0.
     *
     * If the mothers generation is greater than the father, then the mother's
     * generation + 1 is used, otherwise the father's + 1 is used.
     *
     * @param connection Database connection.
     * @param maleId Mouse record ID for the father.
     * @param femaleId Mouse record ID for the mother.
     * @return String containing the Mouse's generation.
     *
     */
	public String nextGeneration(Long maleId, Long femaleId);

	/**
	 * Create breedings from a manual breeding form.
	 *
	 * @param breedingForm
	 * @return
	 */
	public Long addManualBreeding(AddBreedingForm breedingForm, Person user);

	/**
	 * Persist Collection of Breeding objects and returns Set of ids of stored records
	 * sorted in ascending order
	 * @param breedings
	 * @param user
	 * @return
	 */
	public Set<Long> store(Collection<Breeding> breedings, Person user);

	/**
	 * Return the next breeding number for a colony
	 *
	 * @param offspringColonyId primary key of colony
	 * @return int value of the next number in the sequence of breedings for the colony
	 */
	public int nextBreedingNumber(Long offspringColonyId);

	/**
	 * Expands null child objects to contain empty new objects
	 * @param Breeding to be affected.
	 */
	public void inflateBreedingComponents(Breeding breeding);

	/**
	 * Attempt to delete a breeding record.
	 * Only breeding records with no litters should be permitted to be deleted.
	 *
	 * @param id Long primary key of breeding record to be deleted.
	 * @return true if delete succeeded, otherwise false.
	 */
	public boolean delete(Long breedingId, Person user);

	public void closeCurrentForMouse(Long mouseId, Date closeDate, Person user);

	/**
	 * Return a collection of breedings whose end date is empty for a given mouse (male or female)
	 * @param mouseId
	 * @param requiredComponents
	 * @return Collection of Breedings or an empty Collection if no breedings available
	 */
	public Collection<Breeding> fetchCurrentForMouse(Long mouseId, ComponentMap requiredComponents);

	/**
	 * Return a collection of breedings whose end date is empty for a given cage Id
	 * @param cageId
	 * @param requiredComponents
	 * @return Collection of Breedings or an empty Collection if no breedings available
	 */
	public Collection<Breeding> fetchCurrentForCage(Long cageId, ComponentMap requiredComponents);

	/**
	 * Return the latest (by date) breeding for a mouse (male or female)
	 * @param mouseId
	 * @param requiredComponents
	 * @return Breeding or null if no breedings available for the mouse
	 */
	public Breeding fetchLatestForMouse(Long mouseId, ComponentMap requiredComponents);

	/**
	 * Return the next 100 mating numbers for a colony including any "gaps" in the sequence
	 * @param colonyId
	 * @return
	 */
	public Collection<Long> fetchBreedingNumbersByColonyId(Long colonyId);

	/**
	 * Return all the existing matings for a given colony.
	 * @param offspring_colony_id
	 * @param requiredComponents
	 * @return
	 */
	public Collection<Breeding> fetchColonyBreedingsByBreedingId(Long breedingId, ComponentMap requiredComponents);
}
