/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Set;
import java.util.TreeSet;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.licensing.services.ProtocolService;
import uk.ac.sanger.mig.miprocessing.EmbryoTransferProcedure;
import uk.ac.sanger.mig.miprocessing.services.MirMouseTrackingBridgeService;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.BreedingSearchBean;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.dao.BreedingDao;
import uk.ac.sanger.mig.mousetracking.forms.AddBreedingForm;
import uk.ac.sanger.mig.mousetracking.forms.BreedingForm;
import uk.ac.sanger.mig.mousetracking.views.CageHeaderView;
import uk.ac.sanger.mig.personnel.Person;

public class BreedingServiceImpl extends AbstractServiceImpl2<Breeding, BreedingDao> implements BreedingService {

	private LitterService litterService;
	private MouseService mouseService;
	private ColonyInfoService colonyInfoService;
	private CageService cageService;
	private ProtocolService protocolService;
	private MirMouseTrackingBridgeService mirMouseTrackingBridgeService;
	private MouseProcedureService mouseProcedureService;
	private PersonalLicenceService personalLicenceService;
	private ComponentMapService componentMapService;

	public void setLitterService(LitterService litterService) {
		this.litterService = litterService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setProtocolService(ProtocolService protocolService) {
		this.protocolService = protocolService;
	}

	public void setMirMouseTrackingBridgeService(
			MirMouseTrackingBridgeService mirMouseTrackingBridgeService) {
		this.mirMouseTrackingBridgeService = mirMouseTrackingBridgeService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	public void setPersonalLicenceService(
			PersonalLicenceService personalLicenceService) {
		this.personalLicenceService = personalLicenceService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public Cage fetchForCageView(Long breedingId) {
		BreedingType breedingType = fetchBreedingType(breedingId);

		ComponentMap breedingComponents = null;

		switch (breedingType) {
		case MATING:
			breedingComponents = componentMapService.getComponentMap("viewMating");
			break;
		case EMBRYO_TRANSFER:
			breedingComponents = componentMapService.getComponentMap("viewEmbryoTransfer");
			break;
		default:
			break;
		}


		Breeding breeding = fetchById(breedingId, breedingComponents);
		if(breeding != null) {
			breeding.setBreedingType(breedingType);
		}

		Cage breedingCage = makeCageForView(breeding);
		cageService.inflateCageComponents(breedingCage);
		return breedingCage;
	}

	private CageHeaderView makeCageHeaderForView(Breeding breeding) {
		CageHeaderView cageHeader = new CageHeaderView();
		Mouse female = breeding.getFemale();

		MouseProcedure lastProcedure = null;
		switch (breeding.getBreedingType()) {
		case MATING:
			lastProcedure = female.getCurrentLicencedProcedure();
			break;
		case EMBRYO_TRANSFER:
			lastProcedure = breeding.getEmbryoTransfer().getMouseProcedure();
			break;
		default:
			break;
		}

		if(lastProcedure != null &&
				lastProcedure.getProtocol() != null &&
				lastProcedure.getProtocol().getProjectLicence() != null) {
			cageHeader.setLicenceNumber(lastProcedure.getProtocol().getProjectLicence().getLicenceNumber());
			cageHeader.setProtocolCode(lastProcedure.getProtocol().getProtocolCode());
			cageHeader.setOwnerName(female.getCurrentOwner().getFullName());
			cageHeader.setOwnerPhoneNumber(female.getCurrentOwner().getPhoneNumber());
		}
		return cageHeader;
	}

	private Cage makeCageForView(Breeding breeding) {
		Cage cage = null;

		if(breeding.getFemale().getCageId() == null) {
			cage = new Cage();
			CageHeaderView cageHeader = makeCageHeaderForView(breeding);
			cage.setCageHeader(cageHeader);
		}
		else {
			ComponentMap cageComponents = componentMapService.getComponentMap("cageLocation");
			cage = cageService.fetchById(breeding.getFemale().getCageId(), cageComponents);
		}

		Collection<Breeding> currentBreedings = new ArrayList<Breeding>();
		currentBreedings.add(breeding);

		CageType cageType = null;
		switch (breeding.getBreedingType()) {
		case MATING:
			cageType = CageType.MATING;
			break;
		case EMBRYO_TRANSFER:
			cageType = CageType.EMBRYO_TRANSFER;
			break;
		default:
			break;
		}
		cage.setCageType(cageType);
		cage.setCurrentBreedings(currentBreedings);
		cageService.inflateCageComponents(cage);

		return cage;
	}

	public BreedingType fetchBreedingType(Long breedingId) {
		return getPrimaryDao().fetchBreedingType(breedingId);
	}

	public Collection<Breeding> fetchCurrentForMouse(Long mouseId, ComponentMap requiredComponents) {
		Collection<Breeding> breedings = getPrimaryDao().fetchCurrentForMouse(mouseId);
		attachComponents(breedings, requiredComponents);
		return breedings;
	}

	public Collection<Breeding> fetchCurrentForCage(Long cageId, ComponentMap requiredComponents) {
		Collection<Breeding> breedings = getPrimaryDao().fetchByCageId(cageId);
		attachComponents(breedings, requiredComponents);
		return breedings;
	}

	public Breeding fetchLatestForMouse(Long mouseId, ComponentMap requiredComponents) {
		Breeding breeding = getPrimaryDao().fetchLatestForMouse(mouseId);

		if(breeding != null) {
			attachComponents(breeding, requiredComponents);
		}

		return breeding;
	}

	@Transactional
	public void editBreeding(BreedingForm form, Person user) {
		store(form.getBreeding(), user);

		if(form.getMaleLocation().getId() != null) {
			Mouse male = mouseService.fetchById(form.getBreeding().getMaleId(), null);
			mouseService.moveMouse(male, form.getMaleLocation().getBarcodeType(), form.getMaleLocation().getId(), user);
		}

		if(form.getFemaleLocation().getId() != null) {
			Mouse female = mouseService.fetchById(form.getBreeding().getFemaleId(), null);
			mouseService.moveMouse(female, form.getFemaleLocation().getBarcodeType(), form.getFemaleLocation().getId(), user);
		}
	}

	@Transactional
	public void closeCurrentForMouse(Long mouseId, Date closeDate, Person user) {
		BreedingSearchBean searchBean = new BreedingSearchBean();
		searchBean.setEndDateNull(true);
		searchBean.setParentId(mouseId);
		searchBean.setPaginated(false);

		Collection<Breeding> breedings = getPrimaryDao().fetch(searchBean);
		for(Breeding breeding : breedings) {
			breeding.setEndDate(closeDate);
			store(breeding, user);
		}
	}

	@Transactional
	public boolean delete(Long breedingId, Person user) {
		Breeding breeding = getPrimaryDao().fetchById(breedingId);
		boolean recordDeleted = false;

		if(breeding != null) {
			Collection<Litter> litters = litterService.fetchByBreedingId(breedingId, null);
			if(litters.isEmpty()) {
				getPrimaryDao().delete(breeding, user);
				recordDeleted = true;
			}
		}

		return recordDeleted;
	}

	@Transactional
	public Set<Long> store(Collection<Breeding> breedings, Person user) {
		Set<Long> idSet = new TreeSet<Long>();
		for(Breeding breeding : breedings) {
			Long id = store(breeding, user);
			idSet.add( id );
		}
		return idSet;
	}

	@Transactional
	@Override
	public Long store(Breeding breeding, Person person) {

		boolean isNewBreedingRecord = (breeding.getId() == null) ? true : false;
		storePreprocess(breeding);
		Long breedingId = getPrimaryDao().store(breeding, person);
		storePostprocess(breeding, person, isNewBreedingRecord);

		return breedingId;
	}

	@Transactional
	private void storePreprocess(Breeding breeding) {
		// For new records,
		// Generate the breeding name automatically prior to storing.
		if(breeding.getId() == null) {
			// prevent storage of a new breeding if the female is in an existing one.
			Collection<Breeding> femaleBreedings = fetchCurrentForMouse(breeding.getFemaleId(), null);
			if(!femaleBreedings.isEmpty()) {
				throw new RuntimeException("Cannot store breeding - female is already in mating " + femaleBreedings.iterator().next().getBreedingName());
			}

			Long colonyId = breeding.getOffspringColonyId();

			String colonyPrefix;
			if(colonyId == null) {
				colonyPrefix = "B";
			}
			else {
				ColonyInfo colony = colonyInfoService.fetchById(breeding.getOffspringColonyId(), null);
				colonyPrefix = colony.getColonyPrefix();
			}

			breeding.setNamePrefix(colonyPrefix);
			if(breeding.getBreedingNumber() == 0) {
				int breedingNumber = nextBreedingNumber(colonyId);
				breeding.setBreedingNumber(breedingNumber);
			}
		}

		// if male is vasectomised, then set this to be a pseudo-breeding
		Mouse male = mouseService.fetchById(breeding.getMaleId(), null);
		if(male != null && male.isSterile()) {
			breeding.setPseudoBreeding(true);
		}
	}

	@Transactional
	private void storePostprocess(Breeding breeding, Person person, boolean isNewBreedingRecord) {
		Mouse male = mouseService.fetchById(breeding.getMaleId(), ComponentMap.of(Mouse.Component.CURRENT_LICENSED_PROCEDURE));
		Mouse female = mouseService.fetchById(breeding.getFemaleId(), ComponentMap.of(Mouse.Component.CURRENT_LICENSED_PROCEDURE));

		//update male colony to offspring colony (only if different)
		if(male != null &&
				((male.getColonyInfoId() == null && breeding.getOffspringColonyId() != null) ||
						(male.getColonyInfoId() != null && !male.getColonyInfoId().equals(breeding.getOffspringColonyId()))
				)
		) {
			// move the male to the breeding offspring colony
			male.setColonyInfoId(breeding.getOffspringColonyId());
			mouseService.store(male, person);
		}
		//update female colony to offspring colony (only if different)
		if(female != null &&
				((female.getColonyInfoId() == null && breeding.getOffspringColonyId() != null) ||
						(female.getColonyInfoId() != null && !female.getColonyInfoId().equals(breeding.getOffspringColonyId()))
				)
		) {
			// move the female to the breeding offspring colony
			female.setColonyInfoId(breeding.getOffspringColonyId());
			mouseService.store(female, person);
		}

		// store new mouse procedure for male if male or female already under licence
		if(isNewBreedingRecord && (male.getCurrentLicencedProcedure() != null || female.getCurrentLicencedProcedure() != null)) {
			addBreedingProcedure(male, female, breeding.getStartDate(), person);
			addBreedingProcedure(female, male, breeding.getStartDate(), person);
		}
	}

	/**
	 * Add a new procedure for breeding of transgenic mice to the primary mouse if either mouse
	 * is under a licenced procedure
	 * @param primary the mouse getting a procedure added
	 * @param secondary the mate of the mouse which may be on a licenced procedure
	 * @param startDate start date of breeding
	 * @param user person performing the operation
	 */
	private void addBreedingProcedure(Mouse primary, Mouse secondary, Date startDate, Person user) {
		if(primary.getCurrentLicencedProcedure() != null || secondary.getCurrentLicencedProcedure() != null) {

			MouseProcedure procedure = new MouseProcedure();
			procedure.setStartDate(startDate);
			procedure.setMouseId(primary.getId());

			// if either mouse is F0 (Chimera) then choose breeding chimera procedure
			if(  (primary.getGeneration() != null && primary.getGeneration().equals("F0"))
			  || (secondary.getGeneration() != null && secondary.getGeneration().equals("F0"))) {
				procedure.setProcedureDictId(MouseProcedureService.BREEDING_CHIMERAS);
			}

			// if procedure is still null, then choose normal transgenic breeding
			if(procedure.getProcedureDictId() == null) {
				procedure.setProcedureDictId(MouseProcedureService.BREEDING_TRANSGENIC_MICE);
			}


			if(primary.getCurrentLicencedProcedure() == null) {
				Long projectLicenceId = secondary.getCurrentLicencedProcedure().getProtocol().getProjectLicenceId();
				Protocol breedingProtocol = protocolService.fetchByProcedureAndProject(procedure.getProcedureDictId(), projectLicenceId, secondary.isHarmfulMutant(), null);
				procedure.setProtocolId(breedingProtocol.getId());

				PersonalLicence pil = personalLicenceService.fetchByPersonId(secondary.getCurrentOwnerId(), null);
				if(pil != null) {
					procedure.setPersonalLicenceId(pil.getId());
				}

			}
			else {
				Long projectLicenceId = primary.getCurrentLicencedProcedure().getProtocol().getProjectLicenceId();
				Protocol breedingProtocol = protocolService.fetchByProcedureAndProject(procedure.getProcedureDictId(), projectLicenceId, primary.isHarmfulMutant(), null);
				procedure.setProtocolId(breedingProtocol.getId());

				PersonalLicence pil = personalLicenceService.fetchByPersonId(primary.getCurrentOwnerId(), null);
				if(pil != null) {
					procedure.setPersonalLicenceId(pil.getId());
				}
			}

			mouseProcedureService.storeIfNotExists(procedure, user);
		}
	}

	public Collection<Breeding> search(BreedingSearchBean searchBean, ComponentMap requiredComponent) {
		Collection<Breeding> breedings =  getPrimaryDao().fetch(searchBean);

		attachComponents(breedings, requiredComponent);

		return breedings;
	}

	/**
	 * @see BreedingService#nextGeneration(Long maleId, Long femaleId)
	 */
	public String nextGeneration(Long maleId, Long femaleId) {
		Mouse male = mouseService.fetchById(maleId, null);
		Mouse female = mouseService.fetchById(femaleId, null);

		if(male.isSterile()) {
			return "F0"; // pseudo-breeding
		}

		String generation = "F";

		int fatherGeneration = 0;
		try {
			fatherGeneration = Integer.parseInt(male.getGeneration().substring(1),10);
		}
		catch(Exception e) {
			fatherGeneration = 0;
		}

		int motherGeneration = 0;
		try {
			motherGeneration = Integer.parseInt(female.getGeneration().substring(1),10);
		}
		catch(Exception e) {
			motherGeneration = 0;
		}

		int newGeneration = 1 + ((motherGeneration >= fatherGeneration) ? motherGeneration : fatherGeneration);

		generation += newGeneration;

		return generation;
	}

	@Transactional
	public Long addManualBreeding(AddBreedingForm breedingForm, Person user) {

		Long maleId = breedingForm.getMale().getId();
		Mouse male = mouseService.fetchById(maleId, null);

		// if the offspring colony has not been set, take it from the male
		// (the validator should have enforced an offspring colony if the parental colonies were different)
		if(breedingForm.getOffspringColonyId() == null) {
			breedingForm.setOffspringColonyId(male.getColonyInfoId());
		}

		Long cageId = null;

		if("rack".equalsIgnoreCase(breedingForm.getBarcodeType())) {
			cageId =  cageService.createInRack(breedingForm.getCageOrRackId(), user);
		}
		else {
			cageId = breedingForm.getCageOrRackId();
		}

		// Move the male into the cage
		mouseService.moveToExistingCage(male, cageId, user);

		Mouse[] females = breedingForm.getFemales().toArray(new Mouse[0]);
		Long[] breedingNumbers = breedingForm.getBreedingNumbers().toArray(new Long[0]);

		for(int i=0; i<females.length; i++) {
			Mouse female = females[i];
			if(female.getId() != null) {
				female = mouseService.fetchById(female.getId(), null);
				Breeding breeding = new Breeding();

				breeding.setFemaleId(female.getId());
				breeding.setMaleId(maleId);
				breeding.setComments(breedingForm.getComments());
				breeding.setStartDate(breedingForm.getStartDate());
				breeding.setOffspringColonyId(breedingForm.getOffspringColonyId());
				breeding.setOffspringGeneration(breedingForm.getOffspringGeneration());
				breeding.setTimed(breedingForm.isTimed());
				breeding.setTimedEmbryoAge(breedingForm.getTimedEmbryoAge());
				breeding.setBreedingCategoryDictId(breedingForm.getBreedingCategoryDictId());

				// set the breeding number if available
				if(breedingNumbers.length == females.length && breedingNumbers[i] != null) {
					breeding.setBreedingNumber(breedingNumbers[i].intValue());
				}

				//if date is blank, use today's date.
				if(breedingForm.getStartDate() == null) {
					breeding.setStartDate(new Date());
				}

				// Move the female into the cage
				mouseService.moveToExistingCage(female, cageId, user);

				// store the new breeding record
				store(breeding, user);
			}
		}

		return cageId;
	}

	@Override
	protected void attachComponents(Breeding breeding, ComponentMap requiredComponents) {
		if(breeding == null) {
			throw new RuntimeException("Cannot attach components to null breeding");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Breeding.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(requiredComponents.containsKey(Breeding.Component.PARENTS)) {
				breeding.setMale(fetchMouse(breeding.getMaleId(), requiredComponents.get(Breeding.Component.PARENTS)));
				breeding.setFemale(fetchMouse(breeding.getFemaleId(), requiredComponents.get(Breeding.Component.PARENTS)));
			}
			else {
				if(fetchAll || requiredComponents.containsKey(Breeding.Component.MALE)) {
					breeding.setMale(fetchMouse(breeding.getMaleId(), requiredComponents.get(Breeding.Component.MALE)));
				}

				if(fetchAll || requiredComponents.containsKey(Breeding.Component.FEMALE)) {
					breeding.setFemale(fetchMouse(breeding.getFemaleId(), requiredComponents.get(Breeding.Component.FEMALE)));
				}
			}
			if(fetchAll || requiredComponents.containsKey(Breeding.Component.LITTERS)) {
				breeding.setLitters(fetchLitters(breeding.getId(),requiredComponents.get(Breeding.Component.LITTERS)));
			}

			if(fetchAll || requiredComponents.containsKey(Breeding.Component.OFFSPRING_COLONY)) {
				breeding.setOffspringColony(fetchOffspringColony(breeding, requiredComponents.get(Breeding.Component.OFFSPRING_COLONY)));
			}
			if(fetchAll || requiredComponents.containsKey(Breeding.Component.EMBRYO_TRANSFER)) {
				breeding.setEmbryoTransfer(fetchEmbryoTransfer(breeding, requiredComponents.get(Breeding.Component.EMBRYO_TRANSFER)));
			}
		}
	}

	private Mouse fetchMouse(Long mouseId, ComponentMap requiredComponents) {
		if(mouseId == null) {
			return null;
		}
		else {
			return mouseService.fetchById(mouseId, requiredComponents);
		}
	}

	private Collection<Litter> fetchLitters (Long breedingId, ComponentMap requiredComponents) {
		return litterService.fetchByBreedingId(breedingId, requiredComponents);
	}

	private ColonyInfo fetchOffspringColony(Breeding breeding, ComponentMap requiredComponents) {
		if(breeding.getOffspringColonyId() == null) {
			return null;
		}
		else {
			return colonyInfoService.fetchById(breeding.getOffspringColonyId(), requiredComponents);
		}
	}

	private EmbryoTransferProcedure fetchEmbryoTransfer(Breeding breeding, ComponentMap requiredComponents) {
		if(breeding.getEmbryoTransferId() == null) {
			return null;
		}
		else {
			return mirMouseTrackingBridgeService.fetchEmbryoTransferById(breeding.getEmbryoTransferId(), requiredComponents);
		}
	}

	public int nextBreedingNumber(Long offspringColonyId) {
		return getPrimaryDao().nextBreedingNumber(offspringColonyId);
	}

	public Collection<Long> fetchBreedingNumbersByColonyId(Long colonyId) {
		return getPrimaryDao().fetchBreedingNumbersByColonyId(colonyId);
	}

	public Collection<Breeding> fetchColonyBreedingsByBreedingId(Long breedingId, ComponentMap requiredComponents) {
		Collection<Breeding> breedings = getPrimaryDao().fetchColonyBreedingsByBreedingId(breedingId);

		attachComponents(breedings, requiredComponents);

		return breedings;
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateBreedingComponents(Breeding breeding) {
		if(breeding == null) {
			throw new RuntimeException("Breeding cannot be null when inflating components.");
		}

		// Litters
		if(breeding.getLitters() == null) {
			breeding.setLitters(new ArrayList<Litter>());
		}

		// Colony for offspring
		if(breeding.getOffspringColony() == null) {
			breeding.setOffspringColony(new ColonyInfo());
		}

		// Male (father)
		if(breeding.getMale() == null) {
			breeding.setMale(new Mouse());
		}

		// Female (mother or host)
		if(breeding.getFemale() == null) {
			breeding.setFemale(new Mouse());
		}
	}
}
