/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.Rack;
import uk.ac.sanger.mig.mousetracking.Room;
import uk.ac.sanger.mig.mousetracking.StudBreeding;
import uk.ac.sanger.mig.mousetracking.dao.CageDao;
import uk.ac.sanger.mig.mousetracking.forms.RegisterCagesForm;
import uk.ac.sanger.mig.mousetracking.views.CageHeaderView;
import uk.ac.sanger.mig.mousetracking.views.CageProcedureList;
import uk.ac.sanger.mig.personnel.Person;

public class CageServiceImpl extends AbstractServiceImpl2<Cage, CageDao> implements CageService {

	private MouseService mouseService;
	private RackService rackService;
	private BreedingService breedingService;
	private CageHeaderService cageHeaderService;
	private ComponentMapService componentMapService;
	private CageProcedureListService cageProcedureListService;
	private MouseProcedureService mouseProcedureService;
	private StudBreedingService studBreedingService;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setRackService(RackService rackService) {
		this.rackService = rackService;
	}

	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}

	public void setCageHeaderService(CageHeaderService cageHeaderService) {
		this.cageHeaderService = cageHeaderService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setCageProcedureListService(
			CageProcedureListService cageProcedureListService) {
		this.cageProcedureListService = cageProcedureListService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	public void setStudBreedingService(StudBreedingService studBreedingService) {
		this.studBreedingService = studBreedingService;
	}

	public CageType fetchCageType(Long cageId) {
		return getPrimaryDao().fetchCageType(cageId);
	}

	public String fetchCageTotals(Long cageId) {
		return getPrimaryDao().fetchCageTotals(cageId);
	}

	public Cage fetchByBarcode(String barcode, ComponentMap requiredComponents) {
		Cage cage = getPrimaryDao().fetchByBarcode(barcode);

		if(cage != null)
			attachComponents(cage, requiredComponents);

		return cage;
	}

	public Collection<Cage> fetchInUseByRackId(Long rackId, ComponentMap requiredComponents) {
		return fetchByRackId(rackId, requiredComponents);
	}

	public Cage fetchPoolCage(Long colonyId, Date birthDate, ComponentMap requiredComponents) {
		Cage cage = getPrimaryDao().fetchPoolCage(colonyId, birthDate);

		if(cage != null)
			attachComponents(cage, requiredComponents);

		return cage;
	}


	public Collection<Cage> fetchByRackId(Long rackId, ComponentMap requiredComponents) {
		Collection<Cage> cages = getPrimaryDao().fetchByRackId(rackId);

		attachComponents(cages, requiredComponents);

		return cages;
	}

	public Collection<Cage> fetchByLitterId(Long litterId, ComponentMap requiredComponents) {
		Collection<Cage> cages = getPrimaryDao().fetchByLitterId(litterId);

		attachComponents(cages, requiredComponents);

		return cages;
	}

	@Transactional
	public void moveCagesToAuditRack(Long roomId, Person user) {
		Rack auditRack = rackService.fetchAuditRack(roomId);

		if(auditRack != null) {
			Collection<Rack> racks = rackService.fetchByRoomId(roomId, null);

			for(Rack rack : racks) {
				Collection<Cage> cages = fetchByRackId(rack.getId(), ComponentMap.of(Cage.Component.CONTENTS));

				for(Cage cage: cages) {
					cage.setRackId(auditRack.getId());
					store(cage, user);
				}
			}
		}
	}

	public Collection<CageHeaderView> fetchHeadersByCageId(Long cageId, ComponentMap requiredComponents) {
		return cageHeaderService.fetchByCageId(cageId, requiredComponents);
	}

	@Transactional
	public void cleanCagesByRack(Long rackId, Person user) {
		Collection<Cage> cages = fetchByRackId(rackId, null);
		for(Cage cage : cages) {
			clean(cage, user);
		}
	}

	@Transactional
	public Long createInRack(Long rackId, Person user) {

		Cage cage = new Cage();
		cage.setRackId(rackId);

		return store(cage, user);
	}


	@Transactional
	public Map<String, Object> registerCagesInRack(RegisterCagesForm form, Person user) {
		String[] cageBarcodes = form.getListOfBarcodes().trim().split("\\s+");
		Long rackId = form.getRack().getId();
		Rack rack = rackService.fetchById(rackId, ComponentMap.of(Rack.Component.ROOM));

		Map<String, Object> results = new HashMap<String, Object>();

		Map<String, Cage> passedCages = new LinkedHashMap<String, Cage>();
		Map<String, Cage> failedCages = new LinkedHashMap<String, Cage>();

		for(String barcode : cageBarcodes) {
			Cage cage = fetchByBarcode(barcode, ComponentMap.of(Cage.Component.CONTENTS, Cage.Component.CAGE_TYPE));
			if(cage != null && cage.isPopulated() && rack != null) {
				cage.setRackId(rack.getId());
				store(cage, user);
				passedCages.put(cage.getBarcode(), cage);

				if(form.getNewOwner().getId() != null || form.getProcedureSubform().getProcedureId() != null) {
					Collection<Mouse> miceInCage = mouseService.fetchByCageId(cage.getId(), null);

					for(Mouse mouse : miceInCage) {
						if(form.getNewOwner().getId() != null) {
							mouse.setCurrentOwnerId(form.getNewOwner().getId());
						}

						if(form.getProcedureSubform().getProcedureId() != null) {
							MouseProcedure newProcedure = mouseProcedureService.createFromProcedureSubform(form.getProcedureSubform());
							mouse.setCurrentLicencedProcedure(newProcedure);
						}

						mouseService.storeComplete(mouse, user);
					}
				}
			}
			else {
				failedCages.put(barcode, cage);
			}
		}

		results.put("rack", rack);
		results.put("passedCages", passedCages);
		results.put("failedCages", failedCages);

		return results;
	}

	private Collection<Cage> fetchPreviousCages(Cage cage, ComponentMap requiredComponents) {
		Collection<Cage> cages = getPrimaryDao().fetchPreviousCages(cage.getId());

		attachComponents(cages, requiredComponents);

		return cages;
	}

	private CageHeaderView fetchCageHeader(Cage cage) {
		CageType cageType = null;

		if(cage.getCageType() == null) {
			cageType = fetchCageType(cage.getId());
			cage.setCageType(cageType);
		}
		else {
			cageType = cage.getCageType();
		}

		ComponentMap headerComponents = componentMapService.getComponentMap("view" + cageType.name() + "Cage");

		Collection<CageHeaderView> cageHeaders = fetchHeadersByCageId(cage.getId(), headerComponents);

		CageHeaderView cageHeader = null;
		if(!cageHeaders.isEmpty()) {
			cageHeader = cageHeaders.iterator().next();
		}
		return cageHeader;
	}

	@Transactional
	public int clean(Cage cage, Person user) {
		// store the cage with a new last cleaned date
		// this needs to come before the update of the mice
		// because updating the mice may result in the cage being
		// deleted
		cage.setLastCleanedDate(new Date());
		store(cage, user);

		Collection<Mouse> mice = cage.getContents();

		int miceCulled = 0;

		if(mice != null) {
			for(Mouse mouse: mice) {
				mouseService.store(mouse, user);
				if(mouse.getDeathDate() != null) {
					miceCulled++;
				}
			}
		}

		return miceCulled;
	}

	/**
	 * Attach optional associated objects to the input cage object.
	 *
	 * @param cage The Cage object to be modified
	 * @param requiredComponents ComponentMap of required CageComponents.
	 */
	protected void attachComponents(Cage cage, ComponentMap requiredComponents) {
		if(cage == null) {
			throw new RuntimeException("Cannot attach components to null cage");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Cage.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.CONTENTS)) {
				cage.setContents(fetchContents(cage, requiredComponents.get(Cage.Component.CONTENTS)));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.HISTORIC_CONTENTS)) {
				cage.setHistoricContents(fetchHistoricContents(cage, requiredComponents.get(Cage.Component.HISTORIC_CONTENTS)));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.RACK)) {
				cage.setRack(fetchRack(cage, requiredComponents.get(Cage.Component.RACK)));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.CURRENT_BREEDINGS)) {
				cage.setCurrentBreedings(fetchCurrentBreedings(cage, requiredComponents.get(Cage.Component.CURRENT_BREEDINGS)));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.STUD_BREEDINGS)) {
				cage.setStudBreedings(fetchStudBreedings(cage, requiredComponents.get(Cage.Component.STUD_BREEDINGS)));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.PREVIOUS_CAGES)) {
				cage.setPreviousCages(fetchPreviousCages(cage, requiredComponents.get(Cage.Component.PREVIOUS_CAGES)));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.CAGE_TYPE)) {
				cage.setCageType(fetchCageType(cage.getId()));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.CAGE_HEADER)) {
				cage.setCageHeader(fetchCageHeader(cage));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.CAGE_TOTALS)) {
				cage.setCageTotals(fetchCageTotals(cage.getId()));
			}

			if(fetchAll || requiredComponents.containsKey(Cage.Component.PROCEDURES)) {
				cage.setProcedures(fetchProcedures(cage));
			}

		}
	}

	private Collection<CageProcedureList> fetchProcedures(Cage cage) {
		return cageProcedureListService.fetchByCageId(cage.getId());
	}

	private Collection<StudBreeding> fetchStudBreedings(Cage cage, ComponentMap requiredComponents) {
		return studBreedingService.fetchByCageId(cage.getId(), requiredComponents);
	}

	private Collection<Breeding> fetchCurrentBreedings(Cage cage, ComponentMap requiredComponents) {
		return breedingService.fetchCurrentForCage(cage.getId(), requiredComponents);
	}

	private Collection<Mouse> fetchHistoricContents(Cage cage, ComponentMap requiredComponents) {
		return mouseService.fetchHistoricByCageId(cage.getId(), requiredComponents);
	}

	private Collection<Mouse> fetchContents(Cage cage, ComponentMap requiredComponents) {
		return mouseService.fetchByCageId(cage.getId(), requiredComponents);
	}

	private Rack fetchRack(Cage cage, ComponentMap requiredComponents) {
		if(cage.getRackId() == null) {
			return null;
		}
		else {
			return rackService.fetchById(cage.getRackId(), requiredComponents);
		}
	}

	public int getTotalCagesByRackId(Long rackId) {
		return getPrimaryDao().getTotalCagesByRackId(rackId);
	}


	public int getTotalCagesByRoomId(Long roomId) {
		return getPrimaryDao().getTotalCagesByRoomId(roomId);
	}

	public Cage fetchForViewById(Long cageId) {
		CageType cageType = fetchCageType(cageId);

		ComponentMap cageComponents = null;

		cageComponents = componentMapService.getComponentMap("view" + cageType.name() + "Cage");

		Cage cage = fetchById(cageId, cageComponents);
		if(cage != null) {
			cage.setCageType(cageType);
		}

		return cage;
	}

	public void inflateCageComponents(Cage cage) {
		if(cage.getRack() == null){
			cage.setRack(new Rack());
		}
		if(cage.getRack().getRoom() == null) {
			cage.getRack().setRoom(new Room());
		}
		if(cage.getBudgetHolder() == null) {
			cage.setBudgetHolder(new Person());
		}
		if(cage.getPreviousCages() == null) {
			cage.setPreviousCages(new ArrayList<Cage>());
		}
		if(cage.getCurrentBreedings() == null) {
			cage.setCurrentBreedings(new ArrayList<Breeding>());
		}
		if(cage.getContents() == null) {
			cage.setContents(new ArrayList<Mouse>());
		}
		if(cage.getHistoricContents() == null) {
			cage.setHistoricContents(new ArrayList<Mouse>());
		}
		if(cage.getCageHeader() == null) {
			cage.setCageHeader(new CageHeaderView());
		}
	}
}
