/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Set;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.CohortMouse;
import uk.ac.sanger.mig.mousetracking.CohortSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.dao.CohortDao;
import uk.ac.sanger.mig.mousetracking.views.CohortListView;
import uk.ac.sanger.mig.personnel.Person;

public class CohortServiceImpl extends AbstractServiceImpl2<Cohort, CohortDao> implements CohortService {

	private CohortMouseService cohortMouseService;
	private MouseService mouseService;

	public void setCohortMouseService(CohortMouseService cohortMouseService) {
		this.cohortMouseService = cohortMouseService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public Cohort fetchByName(String cohortName, ComponentMap requiredComponents) {
		Cohort cohort = getPrimaryDao().fetchByName(cohortName);

		if(cohort != null) {
			attachComponents(cohort, requiredComponents);
		}

		return cohort;
	}

	public Collection<Cohort> fetchByMouseId(Long mouseId, ComponentMap requiredComponents) {
		Collection<Cohort> cohorts = getPrimaryDao().fetchByMouseId(mouseId);

		attachComponents(cohorts, requiredComponents);

		return cohorts;
	}

	private void addMouseToCohort(Mouse mouse, Cohort cohort, Person user) {
		if(mouse == null || mouse.getId() == null) {
			throw new IllegalStateException("Cannot add null mouse to cohort");
		}
		if(cohort == null || cohort.getId() == null) {
			throw new IllegalStateException("Cannot add mouse to null cohort");
		}

		CohortMouse cohortMouse = cohortMouseService.fetchByMouseAndCohortId(mouse.getId(), cohort.getId(), null);

		if(cohortMouse == null) {
			cohortMouse = new CohortMouse();
			cohortMouse.setMouseId(mouse.getId());
			cohortMouse.setCohortId(cohort.getId());
			cohortMouseService.store(cohortMouse, user);
		}
	}

	public CohortListView fetchListViewById(Long id) {

		return getPrimaryDao().fetchListViewById(id);
	}
	private void removeMouseFromCohort(Mouse mouse, Cohort cohort, Person user) {
		if(mouse == null || mouse.getId() == null) {
			throw new IllegalStateException("Cannot remove null mouse from cohort");
		}
		if(cohort == null || cohort.getId() == null) {
			throw new IllegalStateException("Cannot remove mouse from null cohort");
		}

		CohortMouse cohortMouse = cohortMouseService.fetchByMouseAndCohortId(mouse.getId(), cohort.getId(), null);

		if(cohortMouse != null) {
			cohortMouseService.delete(cohortMouse, user);
		}
	}


	@Transactional
	public Pair<Set<Long>, Set<Long>> manageCohort(Cohort cohort, Collection<Mouse> newMice, Person user) {
		Set<Long> addedMiceIds = new LinkedHashSet<Long>();
		Set<Long> removedMiceIds = new LinkedHashSet<Long>();

		store(cohort,user);

		Collection<Mouse> currentMice = mouseService.fetchByCohortId(cohort.getId(), null);

		// remove mice in cohort that aren't in the latest list
		for(Mouse mouse : currentMice) {
			if(!newMice.contains(mouse)) {
				removeMouseFromCohort(mouse, cohort, user);
				removedMiceIds.add(mouse.getId());
			}
		}

		// add mice not currently in cohort
		for(Mouse mouse : newMice) {
			if(!currentMice.contains(mouse)) {
				addMouseToCohort(mouse, cohort, user);
				addedMiceIds.add(mouse.getId());
			}
		}

		return new Pair<Set<Long>, Set<Long>>(addedMiceIds, removedMiceIds);
	}


	@Override
	protected void attachComponents(Cohort cohort, ComponentMap requiredComponents) {
		if(cohort == null) {
			throw new RuntimeException("Cannot attach components to null reservation");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Cohort.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			if(fetchAll || requiredComponents.containsKey(Cohort.Component.MICE)) {
				cohort.setMice(fetchMice(cohort, requiredComponents.get(Cohort.Component.MICE)));
			}
		}
	}

	private Collection<Mouse> fetchMice(Cohort cohort, ComponentMap requiredComponents) {
		return mouseService.fetchByCohortId(cohort.getId(), requiredComponents);
	}

	public void inflateCohortComponents(Cohort cohort) {
		if(cohort == null) {
			throw new RuntimeException("Cohort cannot be null when inflating components.");
		}

		// Mice
		if(cohort.getMice() == null) {
			cohort.setMice(new ArrayList<Mouse>());
		}
	}

	public Collection<Cohort> fetchForLookup(Long typeId) {
		return getPrimaryDao().fetchForLookup(typeId);
	}

	public Collection<Cohort> fetchForLookup(Long typeId, Long pipelineId) {
		return getPrimaryDao().fetchForLookup(typeId, pipelineId);
	}

	public Collection<CohortListView> search(CohortSearchBean bean) {
		return getPrimaryDao().search(bean);
	}



}

