/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.mousetracking.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.TreeMap;
import org.springframework.transaction.annotation.Transactional;
import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.CohortSet;
import uk.ac.sanger.mig.mousetracking.CohortSetElement;
import uk.ac.sanger.mig.mousetracking.CohortSetListViewSearchBean;
import uk.ac.sanger.mig.mousetracking.CohortSetMouseListComparator;
import uk.ac.sanger.mig.mousetracking.CohortSetWTDictionary;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.MouseSearchBean;
import uk.ac.sanger.mig.mousetracking.dao.CohortSetDao;
import uk.ac.sanger.mig.mousetracking.views.CohortSetElementListView;
import uk.ac.sanger.mig.mousetracking.views.CohortSetListView;
import uk.ac.sanger.mig.mousetracking.views.MouseListView;
import uk.ac.sanger.mig.personnel.Person;

public class CohortSetServiceImpl extends AbstractServiceImpl2<CohortSet,CohortSetDao> implements CohortSetService {
	private AlleleService alleleService;
	private CohortSetElementService cohortSetElementService;
	private DictionaryService dictionaryService;
	private MouseService mouseService;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setCohortSetElementService(CohortSetElementService cohortSetElementService) {
		this.cohortSetElementService = cohortSetElementService;
	}

	public void setAlleleService(AlleleService alleleService) {
		this.alleleService = alleleService;
	}

	public Collection<CohortSet> fetchByAlleleId(Long alleleId,ComponentMap requiredComponents) {
		Collection<CohortSet> cohortSets = getPrimaryDao().fetchByAlleleId(alleleId);

		attachComponents(cohortSets, requiredComponents);

		return cohortSets;
	}

	@Override
	protected void attachComponents(CohortSet cohortSet, ComponentMap requiredComponents) {
		if(cohortSet == null) {
			throw new RuntimeException("Cannot attach components to null cohortSet");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(CohortSet.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			if(requiredComponents.containsKey(CohortSet.Component.ALLELE)||fetchAll){
				cohortSet.setAllele(
						fetchAllele(cohortSet, requiredComponents.get(CohortSet.Component.ALLELE)));
			}
			if(requiredComponents.containsKey(CohortSet.Component.COHORT_SET_ELEMENTS)||fetchAll){
				cohortSet.setCohortSetElements(
						fetchCohortSetElements(cohortSet));
			}
		}
	}

	private Collection<CohortSetElementListView> fetchCohortSetElements(CohortSet cohortSet) {
		Collection<CohortSetElementListView> cohortSetElement =
			cohortSetElementService.fetchByCohortSetId(cohortSet.getId());
		return cohortSetElement;
	}

	private Allele fetchAllele(CohortSet cohortSet, ComponentMap map) {
		return alleleService.fetchById(cohortSet.getAlleleId(), map);
	}


	@Transactional
	public Long editCohortSet(CohortSet cohortSet,Allele allele,Collection<CohortSetElement> deleteCohortElements,Collection<CohortSetElementListView> newCohortElements,Person user) {

		if(cohortSet.getId()==null){
			cohortSet.setAlleleId(allele.getId());
		}
		Long setId = getPrimaryDao().store(cohortSet, user);
		for(CohortSetElementListView cohort:newCohortElements){
			CohortSetElement cohortSetElement = new CohortSetElement();
			cohortSetElement.setSetId(setId);
			cohortSetElement.setCohortId(cohort.getCohortId());
			cohortSetElement.setWtDictId(cohort.getWtOnlyDictId());
			cohortSetElementService.store(cohortSetElement, user);
		}
		for(CohortSetElement cohortSetElement:deleteCohortElements){
			cohortSetElementService.delete(cohortSetElement, user);
		}

		return setId;
	}

	public Collection<CohortSetListView> search(CohortSetListViewSearchBean bean) {
		return getPrimaryDao().search(bean);
	}

	public HashMap<String, Collection<CohortSetElementListView>> getPipelineCohortsHash(Long cohortSetId) {

		Collection<CohortSetElementListView> cohorts = cohortSetElementService.fetchByCohortSetId(cohortSetId);
		HashMap<String,Collection<CohortSetElementListView>> pipeLineCohorts = new HashMap<String,Collection<CohortSetElementListView>>();

		Collection<DictionaryEntry> pipelines =
			dictionaryService.fetch(DictionaryEnum.MOUSE_COHORT_PIPELINE,true);

		for(DictionaryEntry pipeline : pipelines){
			pipeLineCohorts.put(pipeline.getName(),new ArrayList<CohortSetElementListView>());
		}
		for(CohortSetElementListView cohort: cohorts){
			pipeLineCohorts.get(cohort.getPipeline()).add(cohort);
		}
		return pipeLineCohorts;
	}

	public boolean checkAlleleAvaiable(Long alleleId) {
		Collection<Long> alleleIds = getPrimaryDao().fetchExsitingAlleleID();
		HashSet<Long> records = new HashSet<Long>(alleleIds);
		return records.add(alleleId);
	}

	public HashMap<String, Collection<MouseListView>> getMiceListView(Collection<CohortSetElementListView> cohorts, String sessionId) {
		HashMap<String, Collection<MouseListView>> miceHash = new HashMap<String, Collection<MouseListView>>();

		for(CohortSetElementListView c: cohorts){
			MouseSearchBean searchBean = new MouseSearchBean();
			searchBean.setSessionId(sessionId);

			searchBean.setQueryName("mouseListCohortSet");
			searchBean.setPaginated(false);
			searchBean.setCohortId(c.getCohortId());
			searchBean.setOrderBy("genotype,gender");

			if(c.getWtOnlyDictId()!= null){
				searchBean.setGenotypeDictId(GenotypeDictionary.WILDTYPE);
				if(c.getWtOnlyDictId().equals(CohortSetWTDictionary.MALE_ONLY)){
					searchBean.setGenderDictId(MouseService.MOUSE_GENDER_MALE);
				}else if(c.getWtOnlyDictId().equals(CohortSetWTDictionary.FEMALE_ONLY)){
					searchBean.setGenderDictId(MouseService.MOUSE_GENDER_FEMALE);
				}
			}
			Collection<MouseListView> cohortMice = mouseService.search(searchBean);

			if(miceHash.containsKey(c.getPipeline())){
				miceHash.get(c.getPipeline()).addAll(cohortMice);
			}else{
				miceHash.put(c.getPipeline(), cohortMice);
			}

		}

		for(String key: miceHash.keySet()){

			if(miceHash.get(key)!=null && miceHash.get(key).size()>0){
				Collections.sort((List<MouseListView>) miceHash.get(key),new CohortSetMouseListComparator());
			}
		}

		return miceHash;
	}
}
