/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.OracleReport;
import uk.ac.sanger.mig.mousetracking.LabelPrinter;
import uk.ac.sanger.mig.mousetracking.dao.LabelPrinterDao;
import uk.ac.sanger.mig.mousetracking.forms.PrintLabelForm;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentSetService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ObservationService;

public class LabelPrinterServiceImpl extends AbstractServiceImpl2<LabelPrinter,LabelPrinterDao> implements LabelPrinterService {

	private OracleReport oracleReport;
	private CageService cageService;
	private ObservationService observationService;
	private ExperimentSetService experimentSetService;

	public void setObservationService(ObservationService observationService) {
		this.observationService = observationService;
	}

	public void setExperimentSetService(ExperimentSetService experimentSetService) {
		this.experimentSetService = experimentSetService;
	}


	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setOracleReport(OracleReport oracleReport) {
		this.oracleReport = oracleReport;
	}

	public Collection<LabelPrinter> fetchAll() {
		return getPrimaryDao().fetchAll();
	}

	public Collection<LabelPrinter> fetchByMediaType(String mediaName) {
		return getPrimaryDao().fetchByMediaType(mediaName);
	}

	public Set<String> printLabels(PrintLabelForm form, Person user) {
		Collection<String> reportStrings = new ArrayList<String>();
		Long tempId = null;

		switch (form.getPrintJobType()) {
		case PRINT_CAGE:
			reportStrings.addAll(cageReportString(form));
			break;

		case PRINT_STRAW_LABEL:
			reportStrings.add(strawLabelReportString(form));
			break;

		case PRINT_STRAW_REPORT:
			reportStrings.add(strawReportString());
			break;

		case PRINT_BATCH_BLOOD_SAMPLE_LABELS:
			tempId = new Long(form.getArguments());
			experimentSetService.createBloodSampleBarcodesWhereRequired(tempId,user);
			reportStrings.add(batchBloodSampleReportString(form));
			break;

		case PRINT_BATCH_BLOOD_SAMPLE_LABELS_BRADY:
			tempId = new Long(form.getArguments());
			experimentSetService.createBloodSampleBarcodesWhereRequired(tempId,user);
			reportStrings.add(batchBradyBloodSampleReportString(form));
			break;

		case PRINT_ONE_BLOOD_SAMPLE_LABEL:
			Long bloodSampleId = new Long(form.getArguments());
			observationService.createBloodSampleBarcodeIfRequired(bloodSampleId, user);
			reportStrings.add(singleBloodSampleReportString(form));
			break;

		case PRINT_HISTOLOGY_POT_LABEL_LIST:
			reportStrings.add(potLabelsReportString(form));
			break;

		case PRINT_HISTOLOGY_POT_LABEL_SINGLE:
			reportStrings.add(potLabelReportString(form));
			break;

		case PRINT_TEST_CARD:
			reportStrings.add(testReportString(form));
			break;

		case PRINT_MATING_CARD:
			reportStrings.add(matingReportString());
			break;

		default:
			break;
		}

		Set<String> results = new HashSet<String>();
		for(String reportString : reportStrings) {
			results.add(oracleReport.run(reportString));
		}
		return results;
	}

	/**
	 * This method coalesces mulitple cages into a minimum number of calls to the report server for each cage type
	 * @param form
	 * @return
	 */
	private Collection<String> cageReportString(PrintLabelForm form) {
		Collection<String> reportStrings = new ArrayList<String>();
		Map<CageType, String> reportTypes = new HashMap<CageType, String>();
		for(String arg : form.getArguments().split(",")) {
			if(StringUtils.isNotEmpty(arg) && StringUtils.isNumeric(arg)) {
				Long cageId = new Long(arg);
				CageType cageType = cageService.fetchCageType(cageId);

				if(reportTypes.containsKey(cageType)) {
					String reportString = reportTypes.get(cageType);
					reportString += "," + cageId;
					reportTypes.put(cageType, reportString);
				}
				else {
					String reportString = cageType.getOracleReportName() + form.getLabelPrinter().getLoadedMedia()+ "+" + form.getLabelPrinter().getSysName() + "+" + cageId;
					reportTypes.put(cageType, reportString);
				}
			}
		}

		for(String reportString : reportTypes.values()) {
			// add 'copies' argument as the last argument of the report string
			reportString += "+" + form.getCopies();
			reportStrings.add(reportString);
		}
		return reportStrings;
	}

	//NOT IN USE
	private String strawLabelReportString(PrintLabelForm form) {
		return "cryo_straw_label+" + form.getLabelPrinter().getSysName() + "+" + form.getArguments();
	}

	//NOT IN USE
	private String strawReportString() {
		String today = new SimpleDateFormat("dd-MMM-yyyy").format(new Date());
		return "cryo_straw_report+" + today;
	}

	private String matingReportString() {
		return "";
	}

	private String batchBradyBloodSampleReportString(PrintLabelForm form) {
		return "phn_list_label_brady+" + form.getLabelPrinter().getSysName() + "+" + form.getArguments();
	}

	private String batchBloodSampleReportString(PrintLabelForm form) {
		return "phn_list_label+" + form.getLabelPrinter().getSysName() + "+" + form.getArguments();
	}

	private String singleBloodSampleReportString(PrintLabelForm form) {
		return "phn_single_label+" + form.getLabelPrinter().getSysName() + "+" + form.getArguments();
	}

	private String potLabelsReportString(PrintLabelForm form) {
		return "htl_pot_label_list+" + form.getLabelPrinter().getSysName() + "+" + form.getArguments();
	}

	private String potLabelReportString(PrintLabelForm form) {
		return "htl_single_pot_label+" + form.getLabelPrinter().getSysName() + "+" + form.getArguments();
	}

	private String testReportString(PrintLabelForm form) {
		return "test_labelA6L+"+form.getLabelPrinter().getSysName() + "+" + form.getCopies();
	}

	@Override
	protected void attachComponents(LabelPrinter labelPrinter, ComponentMap requiredComponents) {
		//no components defined
	}
}

