/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.colony.ColonyAllele;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.CalendarUtils;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.cryorequest.CryoRequest;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.services.CellLineService;
import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.ChimericMouseData;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.LitterSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.dao.LitterDao;
import uk.ac.sanger.mig.mousetracking.forms.LitterForm;
import uk.ac.sanger.mig.mousetracking.forms.LitterForm.MODE;
import uk.ac.sanger.mig.mousetracking.views.LitterListView;
import uk.ac.sanger.mig.personnel.Person;

public class LitterServiceImpl extends AbstractServiceImpl2<Litter,LitterDao> implements LitterService {

	private BreedingService breedingService;
	private MouseService mouseService;
	private MouseProcedureService mouseProcedureService;
	private CageService cageService;
	private CellLineService cellLineService;
	private MousePoolService mousePoolService;
	private ColonyInfoService colonyInfoService;

	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setCellLineService(CellLineService cellLineService) {
		this.cellLineService = cellLineService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	public void setMousePoolService(MousePoolService mousePoolService) {
		this.mousePoolService = mousePoolService;
	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	public Collection<LitterListView> search(LitterSearchBean searchBean) {
		return getPrimaryDao().search( searchBean );
	}

	@Transactional
	public boolean delete(Long litterId, Person user) {
		Litter litter = fetchById(litterId, null);
		boolean recordDeleted = false;

		if(litter != null) {
			Collection<Mouse> mice = mouseService.fetchByLitterId(litterId, null);
			if(mice.isEmpty()) {
				getPrimaryDao().delete(litter, user);
				recordDeleted = true;
			}
		}

		return recordDeleted;
	}

	@Transactional
	public Long processLitterForm(LitterForm form, Person person) {

		Litter litter = form.getLitter();

		Long maleCageId = null; //current cage Id for male pups
		Long femaleCageId = null; // current cage Id for female pups
		Long unsexedCageId = null; // current cage Id for unsexed pups

		setTotalsPerCage(form);	// define the number of animals allowed in a cage

		// for new litters we have to store twice: this first time is to assign the PK,
		// the next store is after the pups have been recorded so the litter status can be correctly assigned
		if(!litter.hasId()) {
			store(litter, person);
		}

		if(litter.getPups() != null) {
			for(Mouse pup : litter.getPups()) {
				if(pup.isFlaggedForDelete()) {
					mouseService.deleteMouse(pup, person);
				}
				else {
					boolean newProcedure = (form.isAddNewProcedure() || form.isProcedureRequired());
					boolean terminalProcedure = mouseProcedureService.isTerminalProcedure(form.getProcedureSubform().getProcedureId());

					copyRequiredDefaults(form, pup);

					if(pup.isAlive() && terminalProcedure) {
						pup.setDeathDate(form.getProcedureSubform().getProcedureStartDate());
						pup.setCauseOfDeathDictId(MouseService.CAUSE_OF_DEATH_CULLED);
					}

					// move pup as necessary
					// if pup is alive and in the original litter cage.
					if(canMovePup(pup, form)) {
						if(pup.isMale() && form.isMaleLocationRequired() && form.getMaleLocation().hasId()) {
							maleCageId = fetchCageIdForMalePup(maleCageId, form, person);
							pup.setCageId(maleCageId);
						}
						else if(pup.isFemale() && form.isFemaleLocationRequired() && form.getFemaleLocation().hasId()) {
							femaleCageId = fetchCageIdForFemalePup(femaleCageId, pup, form, person);
							pup.setCageId(femaleCageId);
						}
						else if(pup.isUnsexed() && form.isUnsexedLocationRequired() && form.getUnsexedLocation().hasId()) {
							unsexedCageId = fetchCageIdForUnsexedPup(unsexedCageId, form, person);
							pup.setCageId(unsexedCageId);
						}
					}

					mouseService.storeComplete(pup, person);
					if(newProcedure) {
						storeProcedureForPup(pup, form, person);
					}
				}
			}
		}


		if(form.isAdditionalFieldsRequired()) {
			/**
			 * Update the breeding record if mandatory fields have been supplied.
			 */
			boolean saveBreeding = false;
			Breeding breeding = litter.getBreeding();
			if(form.isColonyRequired()) {
				saveBreeding = true;
				breeding.setOffspringColonyId(form.getDefaultPup().getColonyInfoId());
			}
			if(form.isGenerationRequired()) {
				saveBreeding = true;
				breeding.setOffspringGeneration(form.getDefaultPup().getGeneration());
			}
			if(saveBreeding) {
				breedingService.store(breeding, person);
			}
		}

		/**
		 * Cull recipient female if necessary
		 */
		if(Boolean.TRUE.equals(form.getCullFemaleRecipient())) {
			cullFemaleRecipient(form, person);
		}

		return store(litter, person);
	}

	private boolean canMovePup(Mouse pup, LitterForm form) {
		Litter litter = form.getLitter();

		boolean canMove = false;

		if(pup.isWeaned() && (form.getMode().equals(LitterForm.MODE.SPLIT) || (pup.isAlive() && (pup.getCageId() == null || pup.getCageId().equals(litter.getCageId()))))) {
			canMove = true;
		}

		return canMove;
	}

	private void copyRequiredDefaults(LitterForm form, Mouse pup) {
		Litter litter = form.getLitter();
		Mouse defaultPup = form.getDefaultPup();

		pup.setLitterId(litter.getId());

		if(!pup.isPooled()) {
			pup.setBirthDate(litter.getBirthDate());
		}

		if(form.isColonyRequired()) {
			pup.setColonyInfoId(defaultPup.getColonyInfoId());
		}
		if(form.isGenerationRequired()) {
			pup.setGeneration(defaultPup.getGeneration());
		}
		if(form.isStrainRequired()) {
			pup.setStrainDictId(defaultPup.getStrainDictId());
		}
	}

	public void makeLitterName(Litter litter) {
		Long breedingId = litter.getBreedingId();

		int litterNumber = nextLitterNumber(breedingId);

		String litterPrefix;
		if(breedingId == null)  {
			litterPrefix = "L";
		}
		else {
			Breeding breeding = breedingService.fetchById(breedingId, null);
			litterPrefix = breeding.getBreedingName();
		}
		litter.setNamePrefix(litterPrefix  + ".");
		litter.setLitterNumber(litterNumber);
	}

	public Litter fetchByEmbryoTransferId(Long embryoTransferId, ComponentMap requiredComponents) {
        Litter litter = getPrimaryDao().fetchByEmbryoTransferId(embryoTransferId);

        if(litter != null)
        	attachComponents(litter, requiredComponents);
        return litter;
	}

	public Collection<Litter> fetchByBreedingId(Long breedingId, ComponentMap requiredComponents) {
		Collection<Litter> litters = getPrimaryDao().fetchByBreedingId(breedingId);
		attachComponents(litters, requiredComponents);
		return litters;
	}

	@Override
	protected void attachComponents(Litter litter, ComponentMap requiredComponents) {
		if(litter == null) {
			throw new RuntimeException("Cannot attach components to null litter");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Litter.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Litter.Component.BREEDING)) {
				litter.setBreeding(fetchBreeding(litter, requiredComponents.get(Litter.Component.BREEDING)));
			}

			if(fetchAll || requiredComponents.containsKey(Litter.Component.PUPS)) {
				litter.setPups(fetchPups(litter, requiredComponents.get(Litter.Component.PUPS)));
			}
		}
	}

	private Breeding fetchBreeding(Litter litter, ComponentMap requiredComponents) {
		if(litter.getBreedingId() == null) {
			return null;
		}
		else {
			return breedingService.fetchById(litter.getBreedingId(), requiredComponents);
		}

	}

	private Collection<Mouse> fetchPups(Litter litter, ComponentMap requiredComponents) {
		return mouseService.fetchByLitterId(litter.getId(), requiredComponents);
	}

	public void inflateLitterComponents(Litter litter) {
		if (litter == null) {
			throw new RuntimeException("Litter cannot be null when inflating components.");
		}

		// breeding
		if(litter.getBreeding() == null) {
			litter.setBreeding(new Breeding());
		}

		// pups
		if(litter.getPups() == null) {
			litter.setPups(new ArrayList<Mouse>());
		}
	}

	/**
	 * Input a cageId and create a new cage in the same rack, returning the new cageId
	 * @param cageId
	 * @param user
	 * @return id of the new cage in the same rack as the original one.
	 */
	@Transactional
	private Long createNextCage(Long cageId, Person user) {
		Cage cage = cageService.fetchById(cageId, null);
		Long newCageId = cageService.createInRack(cage.getRackId(), user);
		return newCageId;
	}

	@Transactional
	private Long fetchCageIdForMalePup(Long currentCageId, LitterForm form, Person user) {
		Long returnCageId = null;
		if(currentCageId == null) {
			if(BarcodedObject.BarcodeType.CAGE.equals(form.getMaleLocation().getBarcodeType())) {
				returnCageId = form.getMaleLocation().getId();
			}
			else if(BarcodedObject.BarcodeType.RACK.equals(form.getMaleLocation().getBarcodeType())) {
				returnCageId = cageService.createInRack(form.getMaleLocation().getId(), user);
			}
		}
		else if(form.getMalesInCage() >= form.getTotalMalesPerCage()) {
			form.setMalesInCage(0);
			returnCageId = createNextCage(currentCageId, user);
		}
		else {
			returnCageId = currentCageId;
		}

		form.setMalesInCage(form.getMalesInCage() + 1);

		return returnCageId;
	}

	private Long fetchCageIdForFemalePup(Long currentCageId, Mouse mouse, LitterForm form, Person user) {
		if(mouse.isPooled()) {
			currentCageId = mousePoolService.fetchPoolCageId(mouse, form.getFemaleLocation().getId(), user);
		}
		else if(currentCageId == null) {
			if(BarcodedObject.BarcodeType.CAGE.equals(form.getFemaleLocation().getBarcodeType())) {
				currentCageId = form.getFemaleLocation().getId();
			}
			else if(BarcodedObject.BarcodeType.RACK.equals(form.getFemaleLocation().getBarcodeType())) {
				currentCageId = cageService.createInRack(form.getFemaleLocation().getId(), user);
			}
		}
		else if(form.getFemalesInCage() >= form.getTotalFemalesPerCage()) {
			form.setFemalesInCage(0);
			currentCageId = createNextCage(currentCageId, user);
		}

		form.setFemalesInCage(form.getFemalesInCage() + 1);

		return currentCageId;
	}

	private Long fetchCageIdForUnsexedPup(Long currentCageId, LitterForm form, Person user) {
		if(currentCageId == null) {
			if(BarcodedObject.BarcodeType.CAGE.equals(form.getUnsexedLocation().getBarcodeType())) {
				currentCageId = form.getUnsexedLocation().getId();
			}
			else if(BarcodedObject.BarcodeType.RACK.equals(form.getUnsexedLocation().getBarcodeType())) {
				currentCageId = cageService.createInRack(form.getUnsexedLocation().getId(), user);
			}
		}
		else if(form.getUnsexedInCage() >= form.getTotalUnsexedPerCage()) {
			form.setUnsexedInCage(0);
			currentCageId = createNextCage(currentCageId, user);
		}

		form.setUnsexedInCage(form.getUnsexedInCage() + 1);

		return currentCageId;
	}

	private void setTotalsPerCage(LitterForm form) {
		form.setTotalUnsexedPerCage(1000); // group all unsexed pups into one cage by default

		Litter litter = form.getLitter();

		int malesToWean = form.getWeaningCount(MouseService.MOUSE_GENDER_MALE, litter.getCageId());
		int femalesToWean = form.getWeaningCount(MouseService.MOUSE_GENDER_FEMALE, litter.getCageId());


		if(malesToWean > MouseService.MAX_MICE_PER_CAGE) {
			double divisor = Math.ceil((double)malesToWean / (double)MouseService.MAX_MICE_PER_CAGE);
			double dMaxPerCage = Math.ceil((double)malesToWean / divisor);
			form.setTotalMalesPerCage((int)dMaxPerCage);
		}
		else {
			form.setTotalMalesPerCage(MouseService.MAX_MICE_PER_CAGE);
		}

		if(femalesToWean > MouseService.MAX_MICE_PER_CAGE) {
			double divisor = Math.ceil((double)femalesToWean / (double)MouseService.MAX_MICE_PER_CAGE);
			double dMaxPerCage = Math.ceil((double)femalesToWean / divisor);
			form.setTotalFemalesPerCage((int)dMaxPerCage);
		}
		else {
			form.setTotalFemalesPerCage(MouseService.MAX_MICE_PER_CAGE);
		}

	}

	public String setDefault19b(LitterForm form) {
		String returnMessage = null;
		Litter litter = form.getLitter();

		Mouse mother = litter.getBreeding().getFemale();
		MouseProcedure motherProc = mother.getCurrentLicencedProcedure();

		if(motherProc != null) {
			Long projectLicenceId = motherProc.getProtocol().getProjectLicenceId();
			Long personId = mother.getCurrentOwnerId();
			Long procedureDictId = null;

			if(litter.isFromMicroInjection()) {
				procedureDictId = MouseProcedureService.BREEDING_CHIMERAS;
			}
			else if(form.getMode() == MODE.HARVESTED) {
				if(litter.getPlugDate() != null && CalendarUtils.diffInDays(litter.getPlugDate(), new Date()) > 10) {
					procedureDictId = MouseProcedureService.EMBRYO_HARVEST_POST10_5;
				}
				else {
					procedureDictId = MouseProcedureService.EMBRYO_HARVEST_PRE10_5;
				}
			}
			else {
				procedureDictId = MouseProcedureService.BREEDING_TRANSGENIC_MICE;
			}

			MouseProcedure default19b = mouseProcedureService.createMouseProcedure(procedureDictId, projectLicenceId, false, personId);

			form.getProcedureSubform().setProcedureId(default19b.getProcedureDictId());
			form.getProcedureSubform().setProcedureDisplayText(default19b.getProcedure().getDisplayText());

			form.getProcedureSubform().setPersonalLicenceId(default19b.getPersonalLicenceId());
			form.getProcedureSubform().setPersonalLicenceDisplayText(default19b.getPersonalLicence().getDisplayText());

			form.getProcedureSubform().setProtocolId(default19b.getProtocolId());
			form.getProcedureSubform().setProtocolDisplayText(default19b.getProtocol().getDisplayText());

			form.getProcedureSubform().setProjectLicenceId(default19b.getProtocol().getProjectLicenceId());
			form.getProcedureSubform().setProjectLicenceDisplayText(default19b.getProtocol().getProjectLicence().getDisplayText());

			if(form.getMode() == MODE.HARVESTED) {
				form.getProcedureSubform().setPersonalLicenceDisplayText(null);
				form.getProcedureSubform().setPersonalLicenceId(null);
			}
			else {
				form.getProcedureSubform().setProcedureStartDate(litter.getBirthDate());
			}
		}
		return returnMessage;
	}

	public Mouse createPup(LitterForm form) {
		Mouse defaultValues = form.getDefaultPup();
		Mouse pup = new Mouse();
		// colony id
		pup.setColonyInfoId(defaultValues.getColonyInfoId());

		// genetic background
		pup.setStrainDictId(defaultValues.getStrainDictId());

		// colour
		pup.setCoatColourDictId(defaultValues.getCoatColourDictId());

		// owner
		pup.setCurrentOwnerId(defaultValues.getCurrentOwnerId());

		// generation
		pup.setGeneration(defaultValues.getGeneration());

		// wean date
		pup.setWeanDate(defaultValues.getWeanDate());

		// litter id
		pup.setLitterId(form.getLitter().getId());

		// birth date
		pup.setBirthDate(form.getLitter().getBirthDate());

		// death date
		pup.setDeathDate(defaultValues.getDeathDate());

		// cause of death
		pup.setCauseOfDeathDictId(defaultValues.getCauseOfDeathDictId());

		// name
		pup.setName(form.getLitter().getLitterName() + (char)(97 + form.getLitter().getPups().size()));

		// gender (undefined)
		pup.setGenderDictId(MouseService.MOUSE_GENDER_UNDEFINED);

		// alleles
		Collection<MouseAllele> alleles = new ArrayList<MouseAllele>();
		for(MouseAllele allele : form.getDefaultPup().getAlleles()) {
			MouseAllele newAllele = new MouseAllele();
			newAllele.setGenotype(allele.getGenotype());
			newAllele.setGenotypeDictId(allele.getGenotypeDictId());
			newAllele.setAllele(allele.getAllele());
			newAllele.setAlleleId(allele.getAlleleId());
			alleles.add(newAllele);
		}
		pup.setAlleles(alleles);

		// litter ID
		pup.setLitterId(form.getLitter().getId());

		// chimeric data (if from MI)
		if(form.getLitter().isFromMicroInjection()) {
			pup.setChimericData(new ChimericMouseData());
		}

		// location (default to mother's cage)
		pup.setCageId(form.getLitter().getBreeding().getFemale().getCageId());

		return pup;
	}

	private boolean storeProcedureForPup(Mouse mouse, LitterForm form, Person user) {
		// don't store if it's an earmark procedure and the mouse has no earmark
		if(mouseProcedureService.isEarmarkProcedure(form.getProcedureSubform().getProcedureId()) &&
				(mouse.getEarmarkDictId() == null || mouse.getEarmarkDictId().equals(MouseService.NO_EARMARK))
			) {
			return false;
		}
		else {
			MouseProcedure mouseProcedure = mouseProcedureService.createFromProcedureSubform(form.getProcedureSubform());

			mouseProcedure.setMouseId(mouse.getId());
			mouseProcedure.setMouse(mouse);

			mouseProcedureService.storeIfNotExists(mouseProcedure, user);

			return true;
		}
	}

	private void cullFemaleRecipient(LitterForm form, Person user) {
		Long femaleId = form.getLitter().getBreeding().getFemaleId();
		Date cullDate = null;

		if(form.getMode() == MODE.HARVESTED) {
			cullDate = form.getProcedureSubform().getProcedureStartDate();
		}
		else {
			//get first wean date from form and use this as the cull date.
			if(form.getLitter().getPups() != null) {
				for(Mouse pup : form.getLitter().getPups()) {
					if(pup.getWeanDate() != null) {
						cullDate = pup.getWeanDate();
						break;
					}
				}
			}
		}

		Mouse femaleRecipient = mouseService.fetchById(femaleId, null);
		if(femaleRecipient != null && femaleRecipient.isAlive() && cullDate != null) {
			femaleRecipient.setCauseOfDeathDictId(MouseService.CAUSE_OF_DEATH_CULLED);
			femaleRecipient.setDeathDate(cullDate);
			mouseService.store(femaleRecipient, user);
		}
	}

	public int nextLitterNumber(Long breedingId) {
		synchronized(this) {
			return getPrimaryDao().nextLitterNumber(breedingId);
		}
	}

	private Map<Long, MouseAllele> getNonWildTypeAlleles(Collection<MouseAllele> alleles) {
		Map<Long, MouseAllele> alleleMap = new HashMap<Long, MouseAllele>();

		if(alleles != null) {
			for(MouseAllele mouseAllele : alleles) {
				// only add non WT mice for inheritance
				if(!GenotypeDictionary.WILDTYPE.equals(mouseAllele.getGenotypeDictId())) {
					alleleMap.put(mouseAllele.getAlleleId(), mouseAllele);
				}
			}
		}

		return alleleMap;
	}

	/**
	 * Inherit alleles from male and female.
	 * If both parents have the same allele, only one will be added to the returned collection.
	 * The status of the copied alleles defaults to UNTESTED
	 * or HOMOZYGOUS if both parents are hom for that allele.
	 *
	 * @param father
	 * @param mother
	 * @return Collection of MouseAllele
	 */
	private Collection<MouseAllele> inheritParentAlleles(Mouse father, Mouse mother) {
		Map<Long, MouseAllele> fatherAlleles = getNonWildTypeAlleles(father.getAlleles());
		Map<Long, MouseAllele> motherAlleles = getNonWildTypeAlleles(mother.getAlleles());

		Map<Long, MouseAllele> parentalAlleles = new HashMap<Long, MouseAllele>();
		parentalAlleles.putAll(fatherAlleles);
		parentalAlleles.putAll(motherAlleles);

		Map<Long, MouseAllele> inheritedAlleles = new HashMap<Long, MouseAllele>();
		for(Long alleleId : parentalAlleles.keySet()) {

			Long genotypeId = null;
			MouseAllele fatherAllele = fatherAlleles.get(alleleId);
			MouseAllele motherAllele = motherAlleles.get(alleleId);

			// homozygous parents produce homozygous offspring
			if(fatherAllele != null && motherAllele != null
					&& GenotypeDictionary.HOMOZYGOUS.equals(fatherAllele.getGenotypeDictId())
					&& GenotypeDictionary.HOMOZYGOUS.equals(motherAllele.getGenotypeDictId())) {
				genotypeId = GenotypeDictionary.HOMOZYGOUS;
			}
			else {
				genotypeId = GenotypeDictionary.UNTESTED;
			}

			// if the inherited allels don't contain the allele id, add a new one of undefined genotype
			if(!inheritedAlleles.containsKey(alleleId)) {
				MouseAllele inheritedAllele = parentalAlleles.get(alleleId);

				MouseAllele newAllele = new MouseAllele();
				newAllele.setAlleleId(alleleId);
				newAllele.setGenotypeDictId(genotypeId);
				newAllele.setAllele(inheritedAllele.getAllele());
				inheritedAlleles.put(alleleId, newAllele);
			}
		}

		return inheritedAlleles.values();
	}

	/**
	 * Inherit alleles from microinjected cell line.
	 * The status of the copied alleles defaults to GENOTYPE_CHIMERIC.
	 *
	 * @param cellLine
	 * @return Collection of MouseAllele
	 */
	private Collection<MouseAllele> inheritCellLineAlleles(CellLine cellLine) {

		Collection<MouseAllele> cellLineAlleles = new ArrayList<MouseAllele>();

		if(cellLine != null && cellLine.getModifiedAllele() != null) {
			MouseAllele newAllele = new MouseAllele();
			Allele allele = cellLine.getModifiedAllele();
			newAllele.setAlleleId(allele.getId());
			newAllele.setGenotypeDictId(GenotypeDictionary.CHIMERA);
			newAllele.setAllele(allele);
			cellLineAlleles.add(newAllele);
		}

		return cellLineAlleles;
	}

	private Collection<MouseAllele> inheritColonyInfoAlleles(ColonyInfo colonyInfo) {

		Collection<MouseAllele> colonyInfoAlleles = new ArrayList<MouseAllele>();

		if(colonyInfo != null && colonyInfo.getAlleles() != null ) {

			for(ColonyAllele colonyAllele : colonyInfo.getAlleles()){

			MouseAllele newAllele = new MouseAllele();
			Allele allele = colonyAllele.getModifiedAllele();
			newAllele.setAlleleId(allele.getId());
			newAllele.setGenotypeDictId(GenotypeDictionary.UNTESTED);
			newAllele.setAllele(allele);
			colonyInfoAlleles.add(newAllele);
			}

		}

		return colonyInfoAlleles;
	}

	public Mouse getDefaultPupValues(Breeding breeding) {
		Mouse defaultValues = new Mouse();

		// if micro-injection is available get defaults from cell Line
		if(breeding.isFromMicroInjection()) {
			copyDefaultsFromMicroInjection(defaultValues, breeding);
		}
		// if cryo is avaiable get defaults from the colony in the request
		else if(breeding.isFromCryo()){
			copyDefaultsFromCryo(defaultValues,breeding);
		}
		// for standard breedings, set defaults from parent values
		else {
			copyDefaultsFromParents(defaultValues, breeding);
		}

		// set the colony from the breeding record
		defaultValues.setColonyInfoId(breeding.getOffspringColonyId());
		defaultValues.setColonyInfo(breeding.getOffspringColony());

		// if colony is auto-fill for strain, then override any previously set strain
		if(breeding.getOffspringColony().isAutofillStrain()) {
			DictionaryEntry colonyStrain = colonyInfoService.fetchStrainForAutofill(breeding.getOffspringColonyId());
			if(colonyStrain != null) {
				defaultValues.setStrain(colonyStrain);
				defaultValues.setStrainDictId(colonyStrain.getId());
			}
		}

		// set generation from breeding record
		defaultValues.setGeneration(breeding.getOffspringGeneration());

		// add stubs for any remaining null default components
		mouseService.inflateMouseComponents(defaultValues);

		return defaultValues;
	}

	private void copyDefaultsFromParents(Mouse defaultValues, Breeding breeding) {
		Mouse father = breeding.getMale();
		Mouse mother = breeding.getFemale();

		// inflate the father mouse to prevent null pointers
		mouseService.inflateMouseComponents(father);

		// if father and mother coat colours are defined and the same, store default colour for pups.
		if(father != null && father.getCoatColourDictId() != null && mother.getCoatColourDictId() != null
				&& father.getCoatColourDictId().equals(mother.getCoatColourDictId())) {
			defaultValues.setCoatColourDictId(father.getCoatColourDictId());
		}

		// set the strain if the parents are the same strain
		if(father.getStrainDictId() != null && father.getStrainDictId().equals(mother.getStrainDictId())) {
			defaultValues.setStrainDictId(father.getStrainDictId());
			defaultValues.setStrain(father.getStrain());
		}

		// set the current owner
		defaultValues.setCurrentOwnerId(father.getCurrentOwnerId());
		defaultValues.setCurrentOwner(father.getCurrentOwner());

		// default values for alleles from parents
		defaultValues.setAlleles(inheritParentAlleles(father, mother));
	}

	private void copyDefaultsFromMicroInjection(Mouse defaultValues, Breeding breeding) {

		if(breeding.getEmbryoTransfer() != null &&
				breeding.getEmbryoTransfer().getScheduledMicroInjection() != null &&
				breeding.getEmbryoTransfer().getScheduledMicroInjection().getMiRequest() != null &&
				breeding.getEmbryoTransfer().getScheduledMicroInjection().getMiRequest().getCellLine() != null) {

			MiRequest miRequest = breeding.getEmbryoTransfer().getScheduledMicroInjection().getMiRequest();
			CellLine cellLine = miRequest.getCellLine();

			//default allele values from cell line
			defaultValues.setAlleles(inheritCellLineAlleles(cellLine));

			DictionaryEntry cellLineMouseStrain = cellLineService.fetchMouseStrainByCellLineDictId(cellLine.getCellLineStrainDictId());
			if(cellLineMouseStrain != null) {
				defaultValues.setStrainDictId(cellLineMouseStrain.getId());
				defaultValues.setStrain(cellLineMouseStrain);
			}

			// set the current owner
			defaultValues.setCurrentOwnerId(miRequest.getRequesterId());
			defaultValues.setCurrentOwner(miRequest.getRequester());
		}
	}

	private void copyDefaultsFromCryo(Mouse defaultValues, Breeding breeding) {

		if(breeding.getEmbryoTransfer() != null &&
				breeding.getEmbryoTransfer().getThawTest() != null &&
				breeding.getEmbryoTransfer().getThawTest().getCryoRequest() != null &&
				breeding.getEmbryoTransfer().getThawTest().getCryoRequest().getColonyInfo() != null
				) {
			CryoRequest cryoRequest = breeding.getEmbryoTransfer().getThawTest().getCryoRequest();
			ColonyInfo colonyInfo = breeding.getEmbryoTransfer().getThawTest().getCryoRequest().getColonyInfo();

			//default allele values
			defaultValues.setAlleles(inheritColonyInfoAlleles(colonyInfo));
			defaultValues.setStrainDictId(null);
			defaultValues.setStrain(null);

			// set the current owner
			defaultValues.setCurrentOwnerId(cryoRequest.getRequesterId());
			defaultValues.setCurrentOwner(cryoRequest.getRequester());
		}
	}

	public Litter fetchByName(String litterName, ComponentMap requiredComponents) {
		Litter l = getPrimaryDao().fetchByName(litterName);
		if (l != null) {
			attachComponents(l, requiredComponents);
		}
		return l;
	}
}
