/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.genotyping.AssayResult;
import uk.ac.sanger.mig.genotyping.services.AssayResultService;
import uk.ac.sanger.mig.mousetracking.GenotypeHistory;
import uk.ac.sanger.mig.mousetracking.GenotypeHistorySourceDictionary;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.dao.MouseAlleleDao;
import uk.ac.sanger.mig.personnel.Person;

public class MouseAlleleServiceImpl extends AbstractServiceImpl2<MouseAllele, MouseAlleleDao> implements
		MouseAlleleService {

	private AlleleService			alleleService;
	private AssayResultService		assayResultService;
	private GenotypeHistoryService	genotypeHistoryService;


	public void setAlleleService(AlleleService alleleService) {
		this.alleleService = alleleService;
	}


	public void setAssayResultService(AssayResultService assayResultService) {
		this.assayResultService = assayResultService;
	}


	public void setGenotypeHistoryService(GenotypeHistoryService genotypeHistoryService) {
		this.genotypeHistoryService = genotypeHistoryService;
	}


	@Transactional
	public void storeForMouse(Long mouseId, Collection<MouseAllele> mouseAlleles, Long sourceDictId, Person user) {
		for (MouseAllele mouseAllele : mouseAlleles) {
			if (mouseAllele.getAlleleId() != null) {
				mouseAllele.setMouseId(mouseId);

				GenotypeHistory genotypeHistory = new GenotypeHistory();
				genotypeHistory.setSourceDictId(sourceDictId);

				storeWithHistory(mouseAllele, genotypeHistory, user);
			}
		}
	}


	private Long storeWithHistory(MouseAllele mouseAllele, GenotypeHistory genotypeHistory, Person user) {
		Long mouseAlleleId = null;

		if (mouseAllele.isFlaggedForDelete()) {
			if(mouseAllele.getId() != null)
				deleteMouseAllele(mouseAllele, user);
		}
		else {
			mouseAlleleId = store(mouseAllele, user);

			GenotypeHistory currentHistory = null;

			/**
			 * For manual genotype assignments, only store history where the genotype has changed.
			 */
			if (GenotypeHistorySourceDictionary.MANUAL.equals(genotypeHistory.getSourceDictId())) {
				currentHistory = genotypeHistoryService.fetchLatestByMouseAlleleId(mouseAlleleId, null);
			}

			// store a history record if there is none, or if the genotype is different
			if (currentHistory == null || !currentHistory.getGenotypeDictId().equals(mouseAllele.getGenotypeDictId())) {
				genotypeHistory.setAssignmentDate(mouseAllele.getEditDate());
				genotypeHistory.setMouseAlleleId(mouseAlleleId);
				genotypeHistory.setGenotypeDictId(mouseAllele.getGenotypeDictId());
				genotypeHistoryService.store(genotypeHistory, user);
			}

		}
		return mouseAlleleId;
	}


	@Transactional
	public void storeCollectionWithHistory(Collection<Pair<MouseAllele, GenotypeHistory>> mouseAlleles, Person user) {
		for (Pair<MouseAllele, GenotypeHistory> pair : mouseAlleles) {
			storeWithHistory(pair.getObjectA(), pair.getObjectB(), user);
		}
	}


	/**
	 * Deletes the passed mouse allele and cleans up the database.
	 *
	 * @param mouseAllele -
	 *            the <code>MouseAllele</code> to delete.
	 * @param user -
	 *            the user deleting the record.
	 */
	public void deleteMouseAllele(MouseAllele mouseAllele, Person user) {
		getPrimaryDao().deleteMouseAllele(mouseAllele, user);
	}


	@Override
	protected void attachComponents(MouseAllele mouseAllele, ComponentMap requiredComponents) {
		if (mouseAllele == null) {
			throw new RuntimeException("Cannot attach components to null mouseAllele");
		}

		// Note: this is an unusual behaviour - we ALWAYS fetch the corresponding ALLELE
		// regardless of the component map, as the MouseAllele with the Allele is useless)
		mouseAllele.setAllele(fetchAllele(mouseAllele, ComponentMap.of(Allele.Component.GENE)));

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(MouseAllele.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(MouseAllele.Component.GENOTYPE_HISTORIES)) {
				mouseAllele.setGenotypeHistories(fetchGenotypeHistories(mouseAllele.getId(), ComponentMap
						.of(MouseAllele.Component.GENOTYPE_HISTORIES)));
			}

			if (fetchAll || requiredComponents.containsKey(MouseAllele.Component.LATEST_ASSAY_RESULTS)) {
				mouseAllele.setLatestAssayResults(fetchLatestAssayResults(mouseAllele.getId(), ComponentMap
						.of(MouseAllele.Component.LATEST_ASSAY_RESULTS)));
			}
		}
	}


	private Allele fetchAllele(MouseAllele mouseAllele, ComponentMap requiredComponents) {
		if (mouseAllele.getAlleleId() == null) {
			return null;
		} else {
			return alleleService.fetchById(mouseAllele.getAlleleId(), requiredComponents);
		}
	}


	private Collection<GenotypeHistory> fetchGenotypeHistories(Long mouseAlleleId, ComponentMap requiredComponents) {
		return genotypeHistoryService.fetchByMouseAlleleId(mouseAlleleId, requiredComponents);
	}


	private Collection<AssayResult> fetchLatestAssayResults(Long mouseAlleleId, ComponentMap requiredComponents) {
		return assayResultService.fetchLatestByMouseAlleleId(mouseAlleleId, requiredComponents);
	}


	public Collection<MouseAllele> fetchByMouseId(Long mouseId, ComponentMap requiredComponents) {
		Collection<MouseAllele> alleles = getPrimaryDao().fetchByMouseId(mouseId);

		attachComponents(alleles, requiredComponents);

		return alleles;
	}


	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateMouseAlleleComponents(MouseAllele mouseAllele) {
		if (mouseAllele == null) {
			throw new RuntimeException("MouseAllele cannot be null when inflating components.");
		}

		// set allele
		if (mouseAllele.getAllele() == null) {
			Allele modifiedAllele = new Allele();
			modifiedAllele.setMigGene(new MigGene());
			mouseAllele.setAllele(modifiedAllele);
		}

		// set genotype
		if (mouseAllele.getGenotype() == null) {
			mouseAllele.setGenotype(new DictionaryEntry());
		}
	}

}
