/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.mousetracking.services;

import java.util.Date;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MousePool;
import uk.ac.sanger.mig.mousetracking.dao.MousePoolDao;
import uk.ac.sanger.mig.personnel.Person;

public class MousePoolServiceImpl extends AbstractServiceImpl2<MousePool, MousePoolDao> implements MousePoolService {

	private CageService cageService;
	private ColonyInfoService colonyInfoService;

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}

	public MousePool fetchByBarcode(String barcode, ComponentMap requiredComponents) {
		MousePool mousePool = getPrimaryDao().fetchByBarcode(barcode);

		if(mousePool != null)
			attachComponents(mousePool, requiredComponents);

		return mousePool;
	}

	public Long fetchPoolCageId(Mouse mouse, Long rackId, Person user) {
		if(mouse == null || rackId == null) {
			throw new IllegalArgumentException("mouse and RackId cannot be null.");
		}
		else if(mouse.getColonyInfoId() == null || mouse.getBirthDate() == null) {
			throw new IllegalArgumentException("mouse colonyId and birthDate cannot be null.");
		}

		MousePool pool = fetchPool(mouse.getColonyInfoId(), mouse.getBirthDate(), ComponentMap.of(MousePool.Component.ALL_COMPONENTS));

		if(pool == null) {
			pool = new MousePool();
			pool.setColonyId(mouse.getColonyInfoId());
			pool.setPoolDate(mouse.getBirthDate());
			pool.setCageId(cageService.createInRack(rackId, user));
			store(pool, user);
		}

		return pool.getCageId();
	}

	public MousePool fetchPool(Long colonyId, Date poolDate, ComponentMap requiredComponents) {
		MousePool mousePool = getPrimaryDao().fetchPool(colonyId, poolDate);
		if(mousePool != null) {
			attachComponents(mousePool, requiredComponents);
		}
		return mousePool;
	}

	public MousePool fetchByCageId(Long cageId, ComponentMap requiredComponents) {
		MousePool mousePool = getPrimaryDao().fetchByCageId(cageId);
		if(mousePool != null) {
			attachComponents(mousePool, requiredComponents);
		}
		return mousePool;
	}

	@Override
	protected void attachComponents(MousePool mousePool, ComponentMap requiredComponents) {
		if(mousePool == null) {
			throw new RuntimeException("Cannot attach components to null mousePool");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(MousePool.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(MousePool.Component.CAGE)) {
				mousePool.setCage(fetchCage(mousePool, requiredComponents.get(MousePool.Component.CAGE)));
			}

			if(fetchAll || requiredComponents.containsKey(MousePool.Component.COLONY)) {
				mousePool.setColony(fetchColony(mousePool, requiredComponents.get(MousePool.Component.COLONY)));
			}

		}
	}

	public boolean poolFemale(Mouse mouse, Date poolDate) {
		boolean pooled = false;

		if(mouse == null || poolDate == null) {
			throw new IllegalArgumentException("Mouse and pool date cannot be null when calling poolFemale method");
		}

		// only action for female animals
		if(MouseService.MOUSE_GENDER_FEMALE.equals(mouse.getGenderDictId())) {
			mouse.setBirthDate(poolDate);
			mouse.setName(mouse.getBarcode());
			mouse.setPooled(true);
			pooled = true;
		}

		return pooled;
	}

	private Cage fetchCage(MousePool mousePool, ComponentMap requiredComponents) {
		Cage cage = null;
		if(mousePool.getCageId() != null) {
			cage = cageService.fetchById(mousePool.getCageId(), requiredComponents);
		}
		return cage;
	}

	private ColonyInfo fetchColony(MousePool mousePool, ComponentMap requiredComponents) {
		ColonyInfo colonyInfo = null;
		if(mousePool.getCageId() != null) {
			colonyInfo = colonyInfoService.fetchById(mousePool.getColonyId(), requiredComponents);
		}
		return colonyInfo;
	}

}
