/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.Collection;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.CommonServices;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.MouseProcedureDictionaryEntry;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.personnel.Person;

public interface MouseProcedureService extends CommonServices<MouseProcedure> {

	public static final Long	EARCLIP_FOR_GLT_GENOTYPING	= 37L;
	public static final Long	EARCLIP_FOR_GENOTYPING		= 20L;
	public static final Long	EARCLIP_FOR_ID				= 18L;
	public static final Long	BREEDING_TRANSGENIC_MICE	= 21L;
	public static final Long	BREEDING_CHIMERAS			= 36L;
	public static final Long	EMBRYO_HARVEST_PRE10_5		= 22L;
	public static final Long	EMBRYO_HARVEST_POST10_5		= 33L;
	public static final Long	PREWEAN_USE					= 23L;
	public static final Long	BLOOD_SAMPLE_PHENOTYPING	= 32L;
	public static final Long	SUPEROVULATION				= 26L;
	public static final Long 	EARCLIP_POSTMORTEM 			= 47L;
	public static final Long	TAIL_SNIP_FOR_GENOTYPING	= 38L;
	public static final Long	TISSUE_FOR_GENOTYPING_POSTMORTEM = 48L;

	/**
	 * Construct a MouseProcedure object and subcomponents based on the values passed in a
	 * ProcedureSubform. The input subform must contain personal licence id if it contains a
	 * procedure id otherwise an exception will be thrown. The method assumes that validation has
	 * occurred so will not check if procedure requires 19b or not.
	 *
	 * Note: mouseId and Id are not populated.
	 *
	 * @param subform
	 * @return MouseProcedure object with populated fields, or null if input procedure Id is null.
	 * @throws IllegalArgumentException
	 *             if subform not correctly populated.
	 */
	public MouseProcedure createFromProcedureSubform(ProcedureSubform subform);


	/**
	 * Contruct a new MouseProcedure object for earclipping a mouse based on the values passed in a ProcedureSubform
	 * and the current status of the mouse.
	 * <pre>
	 * Mice with no earclip or null earclip should return a null result.
	 * Dead mice should return a non-19b procedure.
	 * The current 19b for the mouse should be used to find the appropriate earclip protoctol.
	 * Mice originating from Chimeric parents should attempt to retrieve the chimeric-offspring earclip procedure and
	 * failing that should revert to the standard earclip procedure.
	 *
	 * Components will be attached if specified in the requiredComponents argument.
	 *
	 * Note: the procedureID from the procedureSubform will be ignored and is therefore not required.
	 * Note: id and mouseId should not be set
	 * </pre>
	 * @param mouse
	 * @param procedureSubform - should contain procedure date, personal licence id and project licence id
	 * @param sampleTypeDictId - 1L=Earclip, 2L=Other Tissue, 3L=Tail
	 * @param requiredComponents - optional components to be linked to the returned MouseProcedure object
	 * @return
	 */
	public MouseProcedure createGenotypingProcedure(Mouse mouse, ProcedureSubform procedureSubform, Long sampleTypeDictId, ComponentMap requiredComponents);

	/**
	 * Store MouseProcedure as record in database and returns the record's primary key
	 * only if there isn't already a record for that protocol,procedure,person and start date
	 * I.E. this method can be used to prevent duplicate procedures being added for a mouse
	 * @param mouseProcedure
	 * @param person
	 * @return
	 */
	public Long storeIfNotExists(MouseProcedure mouseProcedure, Person person);

	/**
	 * Returns MouseProcedure whose foreign key id maps to a unique row (PK) in a procedure table
	 *
	 * @param tableId
	 * @param fkRef
	 * @param requiredComponents
	 * @return
	 */
	public MouseProcedure fetchByTableRef(TableMetadataEnum tableName, Long fkRef, ComponentMap requiredComponents);


	/**
	 * Deletes MouseProcedure record with table and fkRef param
	 *
	 * @param tableId
	 * @param fkRef
	 * @param user
	 * @return
	 */
	public int deleteByTableRef(TableMetadataEnum tableName, Long fkRef, Person user);


	/**
	 * Returns a collection of MouseProcedures for a given mouse id sorted by date
	 *
	 * @param mouseId
	 * @param requiredComponents
	 * @return
	 */
	public Collection<MouseProcedure> fetchByMouseId(Long mouseId, ComponentMap requiredComponents);


	/**
	 * Returns the latest licenced procedure for a given mouse id
	 *
	 * @param mouseId
	 * @param requiredComponents
	 * @return
	 */
	public MouseProcedure fetchCurrentLicencedByMouseId(Long mouseId, ComponentMap requiredComponents);


	/**
	 * Find all mouse procedures for a mouse with the passed procedure and returns them as a
	 * collection.
	 *
	 * @param procedureDictId -
	 *            the id of the procedure to lookup.
	 * @param mouseId -
	 *            the id of the mouse to lookup.
	 * @param requiredComponents -
	 *            the required component mappings to attach.
	 * @return collection of MouseProcedure objects.
	 */
	public Collection<MouseProcedure> fetchByProcedureDictIdAndMouseId(Long procedureId, Long mouseId,
			ComponentMap requiredComponents);


	public MouseProcedure fetchCurrentEmbryoTransferedByMouseId(Long mouseId, ComponentMap requiredComponents);


	public void inflateComponents(MouseProcedure mouseProcedure);


	public MouseProcedure createMouseProcedure(Long procedureDictId, Long projectLicence, boolean harmfulMutant,
			Long personId);


	public boolean isLicencedProcedure(Long procedureDictId);


	public boolean isTerminalProcedure(Long procedureDictId);


	public boolean isEarmarkProcedure(Long procedureDictId);


	public Collection<MouseProcedureDictionaryEntry> fetchDictionaryEntries(boolean activeOnly);


	public MouseProcedureDictionaryEntry fetchDictionaryEntryById(Long id);


	public Collection<MouseProcedureDictionaryEntry> fetchDictionaryEntryForEdit(Long entryId);

	/**
	 *
	 * @param procedureDictId
	 * @return boolean as to whether this procedure is allowed after mouse death.
	 */
	public boolean isAllowablePostMortem(Long procedureDictId);

}