/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import static uk.ac.sanger.mig.genotyping.AssayPlateSampleTypeDictionary.EARCLIP;
import static uk.ac.sanger.mig.genotyping.AssayPlateSampleTypeDictionary.TAIL;

import java.util.Collection;
import java.util.Date;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BeanPropertyBindingResult;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.licensing.services.ProtocolService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.MouseProcedureDictionaryEntry;
import uk.ac.sanger.mig.mousetracking.dao.MouseProcedureDao;
import uk.ac.sanger.mig.mousetracking.dao.MouseProcedureDictionaryEntryDao;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.validators.MouseProcedureValidator;
import uk.ac.sanger.mig.personnel.Person;

public class MouseProcedureServiceImpl extends AbstractServiceImpl2<MouseProcedure,MouseProcedureDao> implements MouseProcedureService {

	private MouseService mouseService;
	private ProtocolService  protocolService;
	private PersonalLicenceService personalLicenceService;
	private MouseProcedureValidator mouseProcedureValidator;
	private MouseProcedureDictionaryEntryDao mouseProcedureDictionaryEntryDao;

	public MouseProcedure createFromProcedureSubform(ProcedureSubform subform) {
		if(subform == null || !subform.isValidated()) {
			throw new IllegalArgumentException("Procedure subform is invalid");
		}
		MouseProcedure newProcedure = null;

		newProcedure = new MouseProcedure();

		//procedure
		newProcedure.setProcedureDictId(subform.getProcedureId());
		DictionaryEntry procedure = mouseProcedureDictionaryEntryDao.fetchById(subform.getProcedureId());
		newProcedure.setProcedure(procedure);

		//personal licence
		newProcedure.setPersonalLicenceId(subform.getPersonalLicenceId());
		PersonalLicence pil = personalLicenceService.fetchById(subform.getPersonalLicenceId(), ComponentMap.of(PersonalLicence.Component.PERSON));
		newProcedure.setPersonalLicence(pil);

		//protocol (and project licence)
		newProcedure.setProtocolId(subform.getProtocolId());
		if(subform.getProtocolId() != null) {
			Protocol protocol = protocolService.fetchById(subform.getProtocolId(), ComponentMap.of(Protocol.Component.PROJECT_LICENCE));
			newProcedure.setProtocol(protocol);
		}

		//start date
		newProcedure.setStartDate(subform.getProcedureStartDate());

		inflateComponents(newProcedure);

		return newProcedure;
	}

	public MouseProcedure createGenotypingProcedure(Mouse mouse, ProcedureSubform procedureSubform, Long sampleTypeDictId, ComponentMap requiredComponents) {
		if(procedureSubform.getPersonalLicenceId() == null || procedureSubform.getProcedureStartDate() == null) {
			throw new IllegalStateException("Personal licence and earclip date are required to create earclip procedure");
		}
		Date procedureDate = procedureSubform.getProcedureStartDate();
		Long personalLicenceId = procedureSubform.getPersonalLicenceId();
		Long projectLicenceId = procedureSubform.getProjectLicenceId();

		MouseProcedure genotypingProcedure = new MouseProcedure();

		//set start date and PIL
		genotypingProcedure.setStartDate(procedureDate);
		genotypingProcedure.setPersonalLicenceId(personalLicenceId);

		//if procedure is after death, this is an unlicened procedure
		if(mouse.isDead() && !procedureDate.before(mouse.getDeathDate())) {
			genotypingProcedure.setProcedureDictId(TISSUE_FOR_GENOTYPING_POSTMORTEM);
		}
		else {
			if(EARCLIP.equals(sampleTypeDictId)) {
				// a mouse with no earclip can't have an earclip procedure.
				if(mouse.getEarmarkDictId() == null || mouse.getEarmarkDictId().equals(MouseService.NO_EARMARK)) {
					return null;
				}
				else {
					//determine if the mouse has come from a chimera mating (if so for some PPLs the protocol differs from the normal one)
					boolean fromChimera = false;
					Collection<Mouse> parents = mouseService.fetchParents(mouse, 0, ComponentMap.of(Mouse.Component.GENOTYPE));
					if(parents != null) {
						for(Mouse parent : parents) {
							if(parent.isChimera()) {
								fromChimera = true;
								break;
							}
						}
					}
					if(fromChimera) {
						genotypingProcedure.setProcedureDictId(EARCLIP_FOR_GLT_GENOTYPING);
					}
					else {
						genotypingProcedure.setProcedureDictId(EARCLIP_FOR_GENOTYPING);
					}

					Protocol earclipProtocol = protocolService.fetchByProcedureAndProject(genotypingProcedure.getProcedureDictId(), projectLicenceId, false, null);

					// if nothing found and it's the F1 protocol, then revert to the standard protocol and try again
					if(earclipProtocol == null && genotypingProcedure.getProcedureDictId().equals(EARCLIP_FOR_GLT_GENOTYPING)) {
						genotypingProcedure.setProcedureDictId(EARCLIP_FOR_GENOTYPING);
						earclipProtocol = protocolService.fetchByProcedureAndProject(genotypingProcedure.getProcedureDictId(), projectLicenceId, false, null);
					}

					if(earclipProtocol != null) {
						genotypingProcedure.setProtocolId(earclipProtocol.getId());
					}
				}
			}
			else if(TAIL.equals(sampleTypeDictId)) {
				Protocol tailsnipProtocol = protocolService.fetchByProcedureAndProject(genotypingProcedure.getProcedureDictId(), projectLicenceId, false, null);
				genotypingProcedure.setProcedureDictId(TAIL_SNIP_FOR_GENOTYPING);
				tailsnipProtocol = protocolService.fetchByProcedureAndProject(genotypingProcedure.getProcedureDictId(), projectLicenceId, false, null);

				if(tailsnipProtocol != null) {
					genotypingProcedure.setProtocolId(tailsnipProtocol.getId());
				}
			}
		}

		if(requiredComponents != null) {
			attachComponents(genotypingProcedure, requiredComponents);
		}

		return genotypingProcedure;
	}

	public MouseProcedure createMouseProcedure(Long procedureDictId, Long projectLicenceId, boolean harmfulMutant, Long personId) {
		PersonalLicence pil = personalLicenceService.fetchByPersonId(personId, ComponentMap.of(PersonalLicence.Component.PERSON));
		DictionaryEntry procedure = mouseProcedureDictionaryEntryDao.fetchById(procedureDictId);
		Protocol protocol = protocolService.fetchByProcedureAndProject(procedureDictId, projectLicenceId, harmfulMutant, ComponentMap.of(Protocol.Component.PROJECT_LICENCE));

		MouseProcedure mouseProcedure = new MouseProcedure();
		mouseProcedure.setProcedure(procedure);
		mouseProcedure.setProcedureDictId(procedureDictId);
		mouseProcedure.setPersonalLicence(pil);
		if(pil != null) {
			mouseProcedure.setPersonalLicenceId(pil.getId());
		}
		mouseProcedure.setProtocol(protocol);
		if(protocol != null) {
			mouseProcedure.setProtocolId(protocol.getId());
		}
		inflateComponents(mouseProcedure);
		return mouseProcedure;
	}

	public boolean isLicencedProcedure(Long procedureDictId) {
		if(procedureDictId == null) {
			return false;
		}
		else {
			return getPrimaryDao().isLicencedProcedure(procedureDictId);
		}
	}

	public boolean isTerminalProcedure(Long procedureDictId) {
		if(procedureDictId == null) {
			return false;
		}
		else {
			return getPrimaryDao().isTerminalProcedure(procedureDictId);
		}
	}

	public Collection<MouseProcedure> fetchByMouseId(Long mouseId, ComponentMap requiredComponents) {
		Collection<MouseProcedure> procedures = getPrimaryDao().fetchByMouseId(mouseId);

		attachComponents(procedures, requiredComponents);

		return procedures;
	}

	public MouseProcedure fetchCurrentLicencedByMouseId(Long mouseId, ComponentMap requiredComponents) {
		MouseProcedure mouseProcedure = getPrimaryDao().fetchCurrentLicencedByMouseId(mouseId);

		if(mouseProcedure != null) {
			attachComponents(mouseProcedure, requiredComponents);
		}

		return mouseProcedure;
	}

	@Transactional
	@Override
	public Long store(MouseProcedure mouseProcedure, Person user) {
		if(mouseProcedure == null) {
			throw new IllegalArgumentException("Cannot save a null mouseProcedure");
		}

		Errors errors = new BeanPropertyBindingResult(mouseProcedure, "mouseProcedure");
		ValidationUtils.invokeValidator(mouseProcedureValidator, mouseProcedure, errors);

		if(errors.hasErrors()) {
			throw new RuntimeException("Cannot save invalid mouseProcedure." + errors.toString());
		}

		// don't save any records for the hard-coded sterile male
		// this can be removed once the producation suite (green corridor) is using the database
		if(mouseProcedure.getMouseId().equals(MouseService.WT_STERILE_MALE_ID)) {
			return null;
		}
		// if procedure is an earclip and the mouse has no earclip, then don't save for that mouse
		if(isEarmarkProcedure(mouseProcedure.getProcedureDictId())) {
			if(mouseProcedure.getMouse() == null) {
				throw new IllegalArgumentException("Mouse must be attached to MouseProcedure to store earclip procedures");
			}

			Mouse mouse = mouseProcedure.getMouse();
			if(mouse.getEarmarkDictId() == null || mouse.getEarmarkDictId().equals(MouseService.NO_EARMARK)) {
				return null;
			}
		}

		return getPrimaryDao().store(mouseProcedure, user);
	}

	@Transactional
	public Long storeIfNotExists(MouseProcedure mouseProcedure, Person user) {
		boolean recordExists = getPrimaryDao().doesProcedureExist(mouseProcedure);

		if(recordExists) {
			return null;
		} else {
			return store(mouseProcedure, user);
		}
	}



	public MouseProcedure fetchCurrentEmbryoTransferedByMouseId(Long mouseId, ComponentMap requiredComponents) {
		MouseProcedure mouseProcedure = getPrimaryDao().fetchByTableRefAndMouseId(TableMetadataEnum.MIR_EMBRYO_TRANSFER_PROCEDURE, mouseId);
		if(mouseProcedure != null)
			attachComponents(mouseProcedure, requiredComponents);
		return mouseProcedure;
	}
	public MouseProcedure fetchByTableRef(TableMetadataEnum tableName, Long fkRef, ComponentMap requiredComponents) {
		MouseProcedure mouseProcedure = getPrimaryDao().fetchByTableRef(tableName, fkRef);
		if(mouseProcedure != null)
			attachComponents(mouseProcedure, requiredComponents);
		return mouseProcedure;
	}

	@Transactional
	public int deleteByTableRef(TableMetadataEnum tableName, Long fkRef, Person user) {
		return getPrimaryDao().deleteByTableRef(tableName, fkRef, user);
	}

	public MouseProcedure fetchById(Long mouseProcedureId, ComponentMap requiredComponents) {
		MouseProcedure mouseProcedure = getPrimaryDao().fetchById(mouseProcedureId);

		if(mouseProcedure != null)
			attachComponents(mouseProcedure, requiredComponents);

		return mouseProcedure;
	}


	/**
	 * Find all mouse procedures for a mouse with the passed procedure and returns them as a
	 * collection.
	 *
	 * @param procedureDictId -
	 *            the id of the procedure to lookup.
	 * @param mouseId -
	 *            the id of the mouse to lookup.
	 * @param requiredComponents -
	 *            the required component mappings to attach.
	 * @return collection of MouseProcedure objects.
	 */
	public Collection<MouseProcedure> fetchByProcedureDictIdAndMouseId(Long procedureId, Long mouseId,
			ComponentMap requiredComponents) {
		Collection<MouseProcedure> mouseProcedures = getPrimaryDao().fetchByProcedureDictIdAndMouseId(procedureId,
				mouseId);
		if (!mouseProcedures.isEmpty())
			attachComponents(mouseProcedures, requiredComponents);
		return mouseProcedures;
	}


	@Override
	protected void attachComponents(MouseProcedure mouseProcedure, ComponentMap requiredComponents) {
		if(mouseProcedure == null) {
			throw new RuntimeException("Cannot attach components to null MouseProcedure");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(MouseProcedure.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(MouseProcedure.Component.MOUSE)) {
				mouseProcedure.setMouse(fetchMouseById(
						mouseProcedure.getMouseId(),
						requiredComponents.get(MouseProcedure.Component.MOUSE)));
			}

			if(fetchAll || requiredComponents.containsKey(MouseProcedure.Component.PERSONAL_LICENCE)) {
				mouseProcedure.setPersonalLicence(fetchPersonalLicenceById(
						mouseProcedure.getPersonalLicenceId(),
						requiredComponents.get(MouseProcedure.Component.PERSONAL_LICENCE)));
			}

			if(fetchAll || requiredComponents.containsKey(MouseProcedure.Component.PROTOCOL)) {
				mouseProcedure.setProtocol(fetchProtocolById(
						mouseProcedure.getProtocolId(),
						requiredComponents.get(MouseProcedure.Component.PROTOCOL)));
			}

		}
	}

	private Protocol fetchProtocolById(Long protoId, ComponentMap components) {
		return (protoId == null) ? null : protocolService.fetchById(protoId, components);
	}

	private PersonalLicence fetchPersonalLicenceById(Long pilId, ComponentMap components) {
		return (pilId == null) ? null : personalLicenceService.fetchById(pilId, components);
	}

	private Mouse fetchMouseById(Long mouseId, ComponentMap components) {
		return (mouseId == null) ? null : mouseService.fetchById(mouseId, components);
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setPersonalLicenceService(
			PersonalLicenceService personalLicenceService) {
		this.personalLicenceService = personalLicenceService;
	}

	public void setProtocolService(ProtocolService protocolService) {
		this.protocolService = protocolService;
	}

	public void setMouseProcedureValidator(
			MouseProcedureValidator mouseProcedureValidator) {
		this.mouseProcedureValidator = mouseProcedureValidator;
	}

	public void inflateComponents(MouseProcedure mouseProcedure) {
		if(mouseProcedure == null) {
			throw new RuntimeException("MouseProcedure cannot be null when inflating components.");
		}

		if(mouseProcedure.getPersonalLicence() == null) {
			PersonalLicence pil = new PersonalLicence();
			pil.setPerson(new Person());
			mouseProcedure.setPersonalLicence(pil);
		}

		if(mouseProcedure.getProtocol() == null) {
			Protocol protocol = new Protocol();
			protocol.setProjectLicence(new ProjectLicence());
			mouseProcedure.setProtocol(protocol);
		}

		if(mouseProcedure.getProcedure() == null) {
			mouseProcedure.setProcedure(new DictionaryEntry());
		}
	}

	public boolean isEarmarkProcedure(Long procedureDictId) {
		if(procedureDictId == null) {
			return false;
		}
		else {
			return( procedureDictId.equals(EARCLIP_FOR_GLT_GENOTYPING) ||
					procedureDictId.equals(EARCLIP_FOR_GENOTYPING) ||
					procedureDictId.equals(EARCLIP_FOR_ID));
		}
	}

	public Collection<MouseProcedureDictionaryEntry> fetchDictionaryEntries(boolean activeOnly) {
		return mouseProcedureDictionaryEntryDao.fetch(activeOnly);
	}

	public MouseProcedureDictionaryEntry fetchDictionaryEntryById(Long id) {
		return mouseProcedureDictionaryEntryDao.fetchById(id);
	}

	public Collection<MouseProcedureDictionaryEntry> fetchDictionaryEntryForEdit(Long entryId) {
		return mouseProcedureDictionaryEntryDao.fetchForEdit(entryId);
	}

	public void setMouseProcedureDictionaryEntryDao(MouseProcedureDictionaryEntryDao mouseProcedureDictionaryEntryDao) {
		this.mouseProcedureDictionaryEntryDao = mouseProcedureDictionaryEntryDao;
	}

	public boolean isAllowablePostMortem(Long procedureDictId) {
		return getPrimaryDao().isAllowablePostMortem(procedureDictId);
	}

}
