/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.Set;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.CommonServices;
import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseSearchBean;
import uk.ac.sanger.mig.mousetracking.forms.BulkUpdateMiceForm;
import uk.ac.sanger.mig.mousetracking.forms.MouseForm;
import uk.ac.sanger.mig.mousetracking.forms.MoveMouseForm;
import uk.ac.sanger.mig.mousetracking.views.MouseFaxView;
import uk.ac.sanger.mig.mousetracking.views.MouseListView;
import uk.ac.sanger.mig.personnel.Person;

public interface MouseService extends CommonServices<Mouse> {

	public static final Long MOUSE_GENDER_FEMALE = 1L;
	public static final Long MOUSE_GENDER_MALE = 2L;
	public static final Long MOUSE_GENDER_UNDEFINED = 3L;

	public static final String MOUSE_SEARCH_QUERY = "mouseSearch";

	public static final Long CAUSE_OF_DEATH_CULLED = 21L;
	public static final Long CAUSE_OF_DEATH_CULLED_SICK = 22L;
	public static final Long CAUSE_OF_DEATH_FOUND_DEAD = 23L;
	public static final Long CAUSE_OF_DEATH_MISSING = 24L;
	public static final Long CAUSE_OF_DEATH_EXPORTED = 26L;
	public static final Long CAUSE_OF_DEATH_TO_CULL = 27L;

	public static final Long NO_EARMARK = 1L;

	public static final Long WT_STOCK_MALE_ID = 99900001L;
	public static final Long WT_STERILE_MALE_ID = 99900002L;

	public static final Long MOUSE_COAT_COLOUR_MIXED = 5L;

	public static final int MAX_MICE_PER_CAGE = 5;

	public Mouse fetchByBarcode(String barcode, ComponentMap requiredComponents);

	public Collection<Mouse> fetchByCageId(Long cageId, ComponentMap requiredComponents);

	public Collection<Mouse> fetchHistoricByCageId(Long cageId, ComponentMap requiredComponents);

	public Collection<Mouse> fetchByLitterId(Long litterId, ComponentMap requiredComponents);

	public Collection<Mouse> fetchByCohortId(Long cohortId, ComponentMap requiredComponents);

	public Mouse fetchByName(String mouseName, ComponentMap requiredComponents);

	public Collection<Mouse> fetch(MouseSearchBean bean, ComponentMap requiredComponents);

	public Collection<MouseListView> search(MouseSearchBean bean);

	public MouseFaxView fetchFaxMouseByBarcode(String barcode);

	public void moveToExistingCage(Mouse mouse, Long cageId, Person user);

	public Long moveToNewCage(Mouse mouse, Long rackId, Person user);

	public Long moveMouse(Mouse mouse, BarcodedObject.BarcodeType objectType, Long objectId, Person user);

	/**
	 * Moves a set of mice from one or more cages to a new group cage, or to individual cages.
	 *
	 * The form must contain a valid destination (Cage or Rack) and a list of mice being moved.
	 *
	 * @param moveMouseForm a validated MouseMouseForm
	 * @param user Person performing the move operation
	 * @return set of cage IDs the mice were moved to
	 * @see MoveMouseForm
	 * @see MoveMouseFormValidator
	 */
	public Set<Long> moveMice(MoveMouseForm moveMouseForm, Person user);

	/**
	 * Stores a mouse with pre and post store processing.
	 *
	 * Prior to storing a mouse the death date is checked. If present, the mouse status is updated.
	 *
	 * After storing a mouse, the assocated cage status is updated.
	 *
	 * @param mouse
	 * @param user
	 * @return
	 */
	public Long store(Mouse mouse, Person user);

	public Long storeComplete(Mouse mouse, Person user);

	public Set<Long> store(Collection<Mouse> mice, Person user);

	public boolean cull(Mouse mouse, Long causeOfDeathDictId, Date cullDate, Person user);

	/**
	 * Set non-primative subcomponents to be not null.
	 *
	 * @param mouse the mouse object to be affected, not null.
	 */
	public void inflateMouseComponents(Mouse mouse);

	public Collection<Mouse> fetchParents(Mouse mouse, int generation, ComponentMap requiredComponents);

	public Collection<Mouse> fetchAncestors(Mouse mouse, int generations, ComponentMap requiredComponents);

	public int bulkUpdate(BulkUpdateMiceForm updateForm, Person user) throws Exception;

	/**
	 * <p>
	 * Create new mouse or mice or edit existing mouse using data provided in a MouseForm.
	 * This method should be transactional as multiple database objects may be created.
	 * </p>
	 *
	 * <p>
	 * The method should return a single value Map containing the key "mouseId"
	 * with value of mouse primary key if a single mouse is created/edited.
	 * Or a single value Map containing the key "cageId" with the value of the cage primary key
	 * if multiple mice were created.
	 * </p>
	 *
	 * <p>
	 * If the {@link MouseForm#getLocationBarcodeType()} equals "rack" then a new cage will be created to house the mice.
	 * Otherwise the value of {@link MouseForm#getLocationId()} will be used to set the Mouse.cageId
	 * </p>
	 *
	 * <p>
	 * A create action is triggered if the id of the MouseForm.Mouse is null otherwise an edit action is performed.
	 * Logically, an edit action should only affect a single mouse.
	 * </p>
	 *
	 * <p>
	 * Data from the {@link MouseForm#getMouse()} object will be copied to all newly created mice.
	 * Individual names, earmarks and rfids are copied from the corresponding arrays in the MouseForm for each new mouse.
	 * </p>
	 *
	 * @param mouseForm MouseForm containing data for mouse/mice
	 * @param user Person who is performing the operation
	 * @return Map containing the primary key of mouse or cage.
	 */
	public Map<String, Long> createMice(MouseForm mouseForm, Person user);

	public Mouse fetchMouseByTissueSampleId(Long tissueSampleId, ComponentMap requiredComponents);

	public void deleteMouse(Mouse mouse, Person user);

	public Collection<Mouse> fetchByRfidTag(String rfidTag);

	public boolean rfidTagInUse(Mouse mouse);

	/**
	 * <p>
	 * Reset the 'id' values for the mouse and its major subcomponents.
	 * (Alleles, CurrentLicencedProcedure, ChimericData, CurrentWeight - id and mouseId)
	 * This is essential when reusing a mouse object to create/copy to new mouse records.
	 * </p>
	 *
	 * <p>
	 * After calling this method the mouse and subcomponents will be in a ready state for storing as new records.
	 * </p>
	 *
	 * @param mouse whose id values are to be reset. mouse must not be null but subcomponents can be.
	 */
	public void resetIdFields(Mouse mouse);
}

