/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.mousetracking.BarcodedObject;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.ChimericMouseData;
import uk.ac.sanger.mig.mousetracking.FamilyTreeMember;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.GenotypeHistorySourceDictionary;
import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.MouseSearchBean;
import uk.ac.sanger.mig.mousetracking.MouseWeight;
import uk.ac.sanger.mig.mousetracking.Rack;
import uk.ac.sanger.mig.mousetracking.StudBreeding;
import uk.ac.sanger.mig.mousetracking.BarcodedObject.BarcodeType;
import uk.ac.sanger.mig.mousetracking.dao.MouseDao;
import uk.ac.sanger.mig.mousetracking.forms.BulkUpdateMiceForm;
import uk.ac.sanger.mig.mousetracking.forms.MouseForm;
import uk.ac.sanger.mig.mousetracking.forms.MoveMouseForm;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.views.MouseFaxView;
import uk.ac.sanger.mig.mousetracking.views.MouseListView;
import uk.ac.sanger.mig.mousetracking.views.MouseLocation;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.reservations.services.MouseReservationService;

public class MouseServiceImpl extends AbstractServiceImpl2<Mouse, MouseDao> implements MouseService {

	private CageService					cageService;
	private PersonService				personService;
	private MouseLocationService		mouseLocationService;
	private ColonyInfoService			colonyInfoService;
	private BreedingService				breedingService;
	private LitterService				litterService;
	private MouseAlleleService			mouseAlleleService;
	private ChimericMouseDataService	chimericMouseDataService;
	private MouseHealthService			mouseHealthService;
	private MouseProcedureService		mouseProcedureService;
	private MouseWeightService			mouseWeightService;
	private ExperimentDataService		experimentDataService;
	private StudBreedingService			studBreedingService;
	private MousePoolService			mousePoolService;
	private MouseReservationService		mouseReservationService;


	public void setExperimentDataService(ExperimentDataService experimentDataService) {
		this.experimentDataService = experimentDataService;
	}


	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}


	public void setMouseLocationService(MouseLocationService mouseLocationService) {
		this.mouseLocationService = mouseLocationService;
	}


	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}


	public void setColonyInfoService(ColonyInfoService colonyInfoService) {
		this.colonyInfoService = colonyInfoService;
	}


	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}


	public void setLitterService(LitterService litterService) {
		this.litterService = litterService;
	}


	public void setMouseAlleleService(MouseAlleleService mouseAlleleService) {
		this.mouseAlleleService = mouseAlleleService;
	}


	public void setChimericMouseDataService(ChimericMouseDataService chimericMouseDataService) {
		this.chimericMouseDataService = chimericMouseDataService;
	}


	public void setMouseHealthService(MouseHealthService mouseHealthService) {
		this.mouseHealthService = mouseHealthService;
	}


	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}


	public void setMouseWeightService(MouseWeightService mouseWeightService) {
		this.mouseWeightService = mouseWeightService;
	}


	public void setStudBreedingService(StudBreedingService studBreedingService) {
		this.studBreedingService = studBreedingService;
	}


	public void setMousePoolService(MousePoolService mousePoolService) {
		this.mousePoolService = mousePoolService;
	}


	public void setMouseReservationService(MouseReservationService mouseReservationService) {
		this.mouseReservationService = mouseReservationService;
	}


	public Mouse fetchByBarcode(String barcode, ComponentMap requiredComponents) {
		Mouse mouse = getPrimaryDao().fetchByBarcode(barcode);

		if (mouse != null)
			attachComponents(mouse, requiredComponents);

		return mouse;
	}


	public Mouse fetchByName(String mouseName, ComponentMap requiredComponents) {
		Mouse mouse = getPrimaryDao().fetchByName(mouseName);

		if (mouse != null)
			attachComponents(mouse, requiredComponents);

		return mouse;
	}


	public Collection<Mouse> fetchByLitterId(Long litterId, ComponentMap requiredComponents) {

		Collection<Mouse> mice = getPrimaryDao().fetchByLitterId(litterId);

		attachComponents(mice, requiredComponents);

		return mice;
	}


	public Collection<Mouse> fetchByCohortId(Long cohortId, ComponentMap requiredComponents) {

		Collection<Mouse> mice = getPrimaryDao().fetchByCohortId(cohortId);

		attachComponents(mice, requiredComponents);

		return mice;
	}


	public Collection<Mouse> fetchHistoricByCageId(Long cageId, ComponentMap requiredComponents) {

		Collection<Mouse> mice = getPrimaryDao().fetchHistoricByCageId(cageId);

		attachComponents(mice, requiredComponents);

		return mice;
	}


	public Collection<Mouse> fetchByCageId(Long cageId, ComponentMap requiredComponents) {

		Collection<Mouse> mice = getPrimaryDao().fetchByCageId(cageId);

		attachComponents(mice, requiredComponents);

		return mice;
	}


	public Collection<Mouse> fetch(MouseSearchBean searchBean, ComponentMap requiredComponents) {
		Collection<Mouse> mice = getPrimaryDao().fetch(searchBean);
		attachComponents(mice, requiredComponents);
		return mice;
	}


	public Collection<MouseListView> search(MouseSearchBean bean) {
		return getPrimaryDao().search(bean);
	}


	@Transactional
	public Set<Long> moveMice(MoveMouseForm form, Person user) {
		if (form == null || !form.isValidated()) {
			throw new IllegalArgumentException("Please supply a validated form.");
		}
		Set<Long> cageIds = new LinkedHashSet<Long>();

		Long cageId = null;

		// if only one cage has been scanned and all the mice are moving to a new rack
		// just move the cage rather than the mice.
		if (form.canMoveAsCage()) {
			Cage cage = form.getCages().iterator().next();
			cage.setRackId(form.getDestinationId());
			cageId = cageService.store(cage, user);
		} else {
			boolean createNewCage = (!form.isStud() && !form.isFemalePool() && form.getDestinationBarcodeType().equals(
					BarcodeType.RACK));

			if (createNewCage) {
				cageId = cageService.createInRack(form.getDestinationId(), user);
			} else {
				cageId = form.getDestinationId();
			}

			for (Long mouseId : form.getMovingMouseIds()) {
				Mouse mouse = fetchById(mouseId, null);
				if (form.isStud() && mouse.isMale()) {
					mouse.setStud(true);
				}
				if (form.isSingleHousing()) {
					cageId = moveToNewCage(mouse, form.getDestinationId(), user);
					cageIds.add(cageId);
				} else if (form.isFemalePool() && mouse.isFemale()) {
					mousePoolService.poolFemale(mouse, form.getFemalePoolDate());
					Long poolCageId = mousePoolService.fetchPoolCageId(mouse, form.getDestinationId(), user);
					moveMouse(mouse, BarcodeType.CAGE, poolCageId, user);
					cageIds.add(poolCageId);
				} else {
					moveToExistingCage(mouse, cageId, user);
					cageIds.add(cageId);
				}
			}
		}
		return cageIds;
	}


	@Transactional
	public Set<Long> store(Collection<Mouse> mice, Person user) {
		Set<Long> ids = new HashSet<Long>();

		if (mice != null) {
			for (Mouse mouse : mice) {
				Long mouseId = storeComplete(mouse, user);
				ids.add(mouseId);
			}
		}

		return ids;
	}


	/**
	 * Move a mouse to an existing cage and store mouse.
	 *
	 * @param mouse
	 *            Mouse to be moved
	 * @param cageId
	 *            Long primary key value for the cage that the mouse is moving to.
	 * @param user
	 *            person performing the move operation
	 */
	@Transactional
	public void moveToExistingCage(Mouse mouse, Long cageId, Person user) {
		Cage cage = cageService.fetchById(cageId, null);

		if (mouse != null && cage != null) {
			mouse.setCageId(cageId);
			store(mouse, user);
		}
	}


	/**
	 * Create a new cage in specified rack, move and store mouse.
	 *
	 * @param mouse
	 *            Mouse to be moved
	 * @param rackId
	 *            Long primary key value for the rack that the cage will be created in.
	 * @param user
	 *            person performing the move operation
	 * @return Long primary key of newly created cage
	 */
	@Transactional
	public Long moveToNewCage(Mouse mouse, Long rackId, Person user) {
		Long cageId = cageService.createInRack(rackId, user);

		if (mouse != null && cageId != null) {
			mouse.setCageId(cageId);
			store(mouse, user);
		}

		return cageId;
	}


	/**
	 * Store Mouse object to the database
	 *
	 * @param mouse
	 *            Mouse object to be stored
	 * @param user
	 *            person performing the create operation
	 * @return
	 * @throws RuntimeException
	 *             if mouse is null or id is not null or not zero
	 */
	@Transactional
	@Override
	public Long store(Mouse mouse, Person user) {
		if (mouse == null) {
			throw new RuntimeException("Cannot save a null mouse");
		}

		Mouse oldMouse = getPrimaryDao().fetchById(mouse.getId());

		storePreprocess(mouse);

		Long mouseId = getPrimaryDao().store(mouse, user);

		storePostprocess(mouse, oldMouse, user);

		return mouseId;
	}


	private void storePreprocess(Mouse newMouse) {

		// If mouse death date specified, need to remove
		// cage id & cage on store.
		if (newMouse.getDeathDate() != null) {
			newMouse.setCageId(null);
			newMouse.setCage(null);

			// if death date is before wean date, need to remove wean date
			if (newMouse.getWeanDate() != null && newMouse.getWeanDate().after(newMouse.getDeathDate())) {
				newMouse.setWeanDate(null);
			}

			// if fate date is provided and fate is "to cull" switch fate to "culled
			if (CAUSE_OF_DEATH_TO_CULL.equals(newMouse.getCauseOfDeathDictId())) {
				newMouse.setCauseOfDeathDictId(CAUSE_OF_DEATH_CULLED);
			}
		}
	}


	private void storePostprocess(Mouse newMouse, Mouse oldMouse, Person user) {

		boolean mouseCulled = false;

		// determine mouse is culled if there's a newly populated death date
		if (oldMouse != null && oldMouse.getDeathDate() == null && newMouse.getDeathDate() != null) {
			mouseCulled = true;
		}

		// if mouse has been culled, close any ongoing health concerns
		if (mouseCulled) {
			mouseHealthService.closeCurrentForMouse(newMouse.getId(), newMouse.getDeathDate(), user);
		}

		// if female mouse has been culled, close any ongoing breedings
		if (mouseCulled && newMouse.isFemale()) {
			breedingService.closeCurrentForMouse(newMouse.getId(), newMouse.getDeathDate(), user);
		}

		if (mouseCulled) {
			studBreedingService.closeCurrentForMouse(newMouse.getId(), newMouse.getDeathDate(), user);
		}
	}


	/**
	 * Find the mother and father records of an individual Animal. This method also sets up the
	 * position (generation and rowIndex) of the parents in a family tree grid.
	 *
	 * <pre>
	 * 	 0
	 * 	 /            \
	 * 	 0     1     2    3
	 * 	 / \   / \   / \  / \
	 * 	 0  1  2  3  4  5  6  7
	 *
	 * 	 The rowIndex of the parents of an animal will be 2x, 2x+1
	 * 	 where x is the rowIndex of the child.
	 *
	 *
	 * </pre>
	 *
	 * @param animal
	 *            Animal whose parents is being found.
	 * @param generation
	 *            the generation the parents belong to relative to the original Animal.
	 * @return Collection containing (at most) 2 Animal records.
	 * @throws Exception
	 */
	public Collection<Mouse> fetchParents(Mouse mouse, int generation, ComponentMap requiredComponents) {

		Collection<Mouse> parents = getPrimaryDao().fetchParentsById(mouse.getId());
		attachComponents(parents, requiredComponents);

		int rowIndexIncrement = 0;
		for (FamilyTreeMember parent : parents) {
			parent.setFamilyTreeGeneration(generation);
			parent.setFamilyTreeChild(mouse.getId());
			parent.setFamilyTreeRowIndex(2 * mouse.getFamilyTreeRowIndex() + rowIndexIncrement++);
		}

		return parents;
	}


	/**
	 * This method starts the process of finding ancestors based on a single animal it calls the
	 * method findAncestors(Connection, Collection, int) which then begins to look for the parents
	 * of the animal.
	 *
	 * @param animal
	 *            Animal whose ancestors are being found.
	 * @param generations
	 *            the number of generations of ancestors required 1 = parents, 2 = grandparents etc
	 * @return Collection of ancestor Animals (including the original animal) in the family tree
	 *
	 * @throws Exception
	 */
	public Collection<Mouse> fetchAncestors(Mouse mouse, int generations, ComponentMap requiredComponents) {
		Collection<Mouse> ancestors = new ArrayList<Mouse>();
		mouse.setFamilyTreeGeneration(generations + 1);
		ancestors.add(mouse);
		return fetchAncestors(ancestors, generations, requiredComponents);
	}


	/**
	 * This method finds the parent Animals for each Animal on the input Collection. The method is
	 * self-referening and decreases the generations counter with each iteration until zero.
	 *
	 * @param animals
	 *            Collection of Animals whose parents are to be found.
	 * @param generations
	 *            number of generations to find.
	 * @return Collection of Animal records containing all known ancestors (parents) of the original
	 *         animal.
	 *
	 * @throws Exception
	 */
	protected Collection<Mouse> fetchAncestors(Collection<Mouse> mice, int generations, ComponentMap requiredComponents) {
		// need to copy the incoming collection otherwise a ConcurrentModificationException is
		// thrown
		Collection<Mouse> copyOfMice = new ArrayList<Mouse>(mice);

		// generations gets decreased with every iteration so will eventually reach zero and
		// terminate the loop
		if (generations > 0) {
			for (Mouse mouse : copyOfMice) {
				// find the current animals parents
				Collection<Mouse> parents = fetchParents(mouse, generations, requiredComponents);

				// now find ancestors of the parents and add them to the outgoing collection
				Collection<Mouse> ancestors = fetchAncestors(parents, generations - 1, requiredComponents);
				mice.addAll(ancestors);
			}
		}

		attachComponents(mice, requiredComponents);

		return mice;
	}


	@Override
	protected void attachComponents(Mouse mouse, ComponentMap requiredComponents) {
		if (mouse == null) {
			throw new RuntimeException("Cannot attach components to null mouse");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(Mouse.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.CAGE)) {
				mouse.setCage(fetchCage(mouse, requiredComponents.get(Mouse.Component.CAGE)));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.COLONY)) {
				mouse.setColonyInfo(fetchColonyInfo(mouse, requiredComponents.get(Mouse.Component.COLONY)));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.CURRENT_OWNER)) {
				mouse.setCurrentOwner(fetchCurrentOwner(mouse, requiredComponents.get(Mouse.Component.CURRENT_OWNER)));
			}

			if (requiredComponents.containsKey(Mouse.Component.GENOTYPE)) {
				mouse.setAlleles(fetchAlleles(mouse, requiredComponents.get(Mouse.Component.GENOTYPE)));
				mouse.setChimericData(fetchChimericData(mouse, requiredComponents.get(Mouse.Component.GENOTYPE)));
			} else {
				if (fetchAll || requiredComponents.containsKey(Mouse.Component.ALLELES)) {
					mouse.setAlleles(fetchAlleles(mouse, requiredComponents.get(Mouse.Component.ALLELES)));
				}
				if (fetchAll || requiredComponents.containsKey(Mouse.Component.CHIMERIC_DATA)) {
					mouse.setChimericData(fetchChimericData(mouse, requiredComponents
							.get(Mouse.Component.CHIMERIC_DATA)));
				}
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.LITTER)) {
				mouse.setLitter(fetchLitter(mouse, requiredComponents.get(Mouse.Component.LITTER)));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.LOCATION)) {
				mouse.setLocation(fetchLocation(mouse, requiredComponents.get(Mouse.Component.LOCATION)));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.CURRENT_LICENSED_PROCEDURE)) {
				// add full nested path for current licenced procedure always
				// this is done as a local variable in order not to alter the incoming component map
				ComponentMap procedureComponents = ComponentMap.putAll(new ComponentMap(
						MouseProcedure.Component.PROTOCOL, new ComponentMap(Protocol.Component.PROJECT_LICENCE, null)),
						new ComponentMap(MouseProcedure.Component.PERSONAL_LICENCE, new ComponentMap(
								PersonalLicence.Component.PERSON, null)));

				mouse.setCurrentLicencedProcedure(fetchCurrentLicencedProcedure(mouse, procedureComponents));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.CURRENT_WEIGHT)) {
				mouse
						.setCurrentWeight(fetchCurrentWeight(mouse, requiredComponents
								.get(Mouse.Component.CURRENT_WEIGHT)));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.LATEST_BREEDING)) {
				mouse.setLatestBreeding(fetchLatestBreeding(mouse, requiredComponents
						.get(Mouse.Component.LATEST_BREEDING)));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.LATEST_STUD_BREEDING)) {
				mouse.setLatestStudBreeding(fetchLatestStudBreeding(mouse, requiredComponents
						.get(Mouse.Component.LATEST_STUD_BREEDING)));
			}

			// not using fetch all here, as experiment data will need its own components to be
			// explicitly loaded if it's to be useful
			if (requiredComponents.containsKey(Mouse.Component.EXPERIMENT_DATA)) {
				mouse.setExperimentData(experimentDataService.fetchByMouseId(mouse.getId(), requiredComponents
						.get(Mouse.Component.EXPERIMENT_DATA)));
			}

			if (fetchAll || requiredComponents.containsKey(Mouse.Component.MOUSE_RESERVATION)) {
				mouse.setMouseReservation(mouseReservationService.fetchForMouse(mouse.getId(), requiredComponents
						.get(Mouse.Component.MOUSE_RESERVATION)));
			}
		}
	}


	private Collection<MouseAllele> fetchAlleles(Mouse mouse, ComponentMap requiredComponents) {
		return mouseAlleleService.fetchByMouseId(mouse.getId(), requiredComponents);
	}


	private Breeding fetchLatestBreeding(Mouse mouse, ComponentMap requiredComponents) {
		return breedingService.fetchLatestForMouse(mouse.getId(), requiredComponents);
	}


	private StudBreeding fetchLatestStudBreeding(Mouse mouse, ComponentMap requiredComponents) {
		return studBreedingService.fetchLatestForMouse(mouse.getId(), requiredComponents);
	}


	private ChimericMouseData fetchChimericData(Mouse mouse, ComponentMap requiredComponents) {
		return chimericMouseDataService.fetchByMouseId(mouse.getId(), requiredComponents);
	}


	private Cage fetchCage(Mouse mouse, ComponentMap requiredComponents) {
		if (mouse.getCageId() == null) {
			return null;
		} else {
			return cageService.fetchById(mouse.getCageId(), requiredComponents);
		}
	}


	private ColonyInfo fetchColonyInfo(Mouse mouse, ComponentMap requiredComponents) {
		if (mouse.getColonyInfoId() == null) {
			return null;
		} else {
			return colonyInfoService.fetchById(mouse.getColonyInfoId(), requiredComponents);
		}
	}


	private Person fetchCurrentOwner(Mouse mouse, ComponentMap requiredComponents) {
		if (mouse.getCurrentOwnerId() == null) {
			return null;
		} else {
			return personService.fetchById(mouse.getCurrentOwnerId(), requiredComponents);
		}
	}


	private MouseLocation fetchLocation(Mouse mouse, ComponentMap requiredComponents) {
		return mouseLocationService.fetchById(mouse.getId(), requiredComponents);
	}


	private Litter fetchLitter(Mouse mouse, ComponentMap requiredComponents) {
		if (mouse.getLitterId() == null) {
			return null;
		} else {
			return litterService.fetchById(mouse.getLitterId(), requiredComponents);
		}
	}


	private MouseWeight fetchCurrentWeight(Mouse mouse, ComponentMap requiredComponents) {
		return mouseWeightService.fetchCurrentByMouseId(mouse.getId(), requiredComponents);
	}


	private MouseProcedure fetchCurrentLicencedProcedure(Mouse mouse, ComponentMap requiredComponents) {
		return mouseProcedureService.fetchCurrentLicencedByMouseId(mouse.getId(), requiredComponents);
	}


	@Transactional
	public Map<String, Long> createMice(MouseForm mouseForm, Person user) {

		Mouse mouse = mouseForm.getMouse();
		Long mouseId = null;

		// create a new cage if only a rack has been specified
		if ("rack".equalsIgnoreCase(mouseForm.getLocationBarcodeType())) {
			Long newCageId = cageService.createInRack(mouseForm.getLocationId(), user);
			mouse.setCageId(newCageId);
		} else {
			mouse.setCageId(mouseForm.getLocationId());
		}

		// store either 1 mice or multiple mice if specified
		int numberOfMice = (mouseForm.getNumberOfMice() == null) ? 1 : mouseForm.getNumberOfMice();

		for (int i = 0; i < numberOfMice; i++) {
			// for multiple mice use the same mouse object, just reset the id each time to null.
			if (i > 0) {
				resetIdFields(mouse);
			}

			if (mouseForm.isAddNewProcedure()) {
				ProcedureSubform subform = mouseForm.getProcedureSubform();
				if (subform.getProcedureId() != null) {
					MouseProcedure newProcedure = mouseProcedureService.createFromProcedureSubform(subform);
					if (newProcedure != null) {
						mouse.setCurrentLicencedProcedure(newProcedure);
					}
				}
			}

			if (mouseForm.getMouse().getId() == null && mouseForm.getMouseNames() != null) {
				mouse.setName(mouseForm.getMouseNames().toArray(new String[0])[i]);
			}

			if (mouseForm.getMouse().getId() == null && mouseForm.getEarmarks() != null) {
				mouse.setEarmarkDictId(mouseForm.getEarmarks().toArray(new Long[0])[i]);
			}

			if (mouseForm.getMouse().getId() == null && mouseForm.getMouseRfidTags() != null) {
				mouse.setRfidTag(mouseForm.getMouseRfidTags().toArray(new String[0])[i]);
			}

			mouseId = storeComplete(mouse, user);

			// this is a bit ugly, but we need to store the mouse again if its barcode should
			//be its name, as the barcode only gets assigned on storing.
			if(mouseForm.isNameFromBarcode()) {
				mouse.setName(mouse.getBarcode());
				store(mouse, user);
			}

			if(mouse.getCurrentWeight() != null && mouse.getCurrentWeight().getWeightValue() != null) {
				mouseWeightService.storeForMouse(mouseId, mouse.getCurrentWeight(), user);
			}
		}

		Map<String, Long> result = new HashMap<String, Long>();
		if (numberOfMice == 1) {
			result.put("mouseId", mouseId);
		} else {
			result.put("cageId", mouse.getCageId());
		}

		return result;
	}


	public void resetIdFields(Mouse mouse) {
		mouse.setId(null);

		// set the current licenced procedure id to null
		if (mouse.getCurrentLicencedProcedure() != null) {
			mouse.getCurrentLicencedProcedure().setId(null);
			mouse.getCurrentLicencedProcedure().setMouseId(null);
		}

		// set the allele ids to null
		if (mouse.getAlleles() != null) {
			for (MouseAllele allele : mouse.getAlleles()) {
				allele.setId(null);
				allele.setMouseId(null);
			}
		}

		// set the chimeric data id to null
		if (mouse.getChimericData() != null) {
			mouse.getChimericData().setId(null);
			mouse.getChimericData().setMouseId(null);
		}

		// set the current weight id to null
		if (mouse.getCurrentWeight() != null) {
			mouse.getCurrentWeight().setId(null);
			mouse.getCurrentWeight().setMouseId(null);
		}
	}


	@Transactional
	public boolean cull(Mouse mouse, Long causeOfDeathDictId, Date cullDate, Person user) {
		if (mouse == null || causeOfDeathDictId == null || cullDate == null || user == null) {
			throw new IllegalStateException("mouse, causeOfDeathDictId, cullDate and user must not be null.");
		}

		if (mouse.isDead()) {
			return false;
		} else {
			mouse.setDeathDate(cullDate);
			mouse.setCauseOfDeathDictId(causeOfDeathDictId);
			mouse.setCageId(null);
			store(mouse, user);
			return true;
		}
	}


	/**
	 * Stores a mouse object and also stores the mouse alleles and chimera data and current
	 * procedure
	 *
	 * @param mouse
	 * @param user
	 * @return
	 */
	@Transactional
	public Long storeComplete(Mouse mouse, Person user) {
		Long mouseId = store(mouse, user);

		if (mouse.getChimericData() != null) {
			chimericMouseDataService.storeForMouse(mouseId, mouse.getChimericData(), user);
		}
		if (mouse.getAlleles() != null) {
			if (mouse.isChimera()) {
				for (MouseAllele allele : mouse.getAlleles()) {
					allele.setGenotypeDictId(GenotypeDictionary.CHIMERA);
				}
			}
			mouseAlleleService.storeForMouse(mouseId, mouse.getAlleles(), GenotypeHistorySourceDictionary.MANUAL, user);
			this.synchronizeMouseAlleles(mouse);
		}
		if (mouse.getCurrentLicencedProcedure() != null && mouse.getCurrentLicencedProcedure().getId() == null
				&& mouse.getCurrentLicencedProcedure().getProcedureDictId() != null) {
			MouseProcedure mouseProcedure = mouse.getCurrentLicencedProcedure();
			mouseProcedure.setMouseId(mouseId);
			mouseProcedure.setMouse(mouse);
			mouseProcedureService.storeIfNotExists(mouseProcedure, user);
		}

		return mouseId;
	}


	/**
	 * Synchronises the passed mouse alleles by removing any allele that has been flagged for
	 * deletion. This method does not know if the mouse allele has actually been deleted, that is
	 * the responsibility of the caller.
	 *
	 * @param mouse -
	 *            the <code>Mouse</code> whos alleles are to be synchronised.
	 */
	private void synchronizeMouseAlleles(Mouse mouse) {
		Iterator<MouseAllele> i = mouse.getAlleles().iterator();
		while (i.hasNext()) {
			MouseAllele mouseAllele = i.next();
			if (mouseAllele.isFlaggedForDelete()) {
				i.remove();
			}
		}
	}


	@Transactional
	public int bulkUpdate(BulkUpdateMiceForm form, Person user) throws Exception {
		if (form == null || !form.isValidated()) {
			throw new IllegalArgumentException("form is invalid");
		}

		int recordsUpdated = 0;

		ProcedureSubform subform = form.getProcedureSubform();

		for (Long mouseId : form.getMouseIds()) {
			Mouse mouse = fetchById(mouseId, ComponentMap.of(Mouse.Component.ALLELES));

			if (mouse != null) {

				// colony
				if (form.getColonyInfoId() != null) {
					mouse.setColonyInfoId(form.getColonyInfoId());
				}

				// owner
				if (form.getCurrentOwnerId() != null) {
					mouse.setCurrentOwnerId(form.getCurrentOwnerId());
				}

				// comments
				if (form.getComments() != null) {
					if (form.isOverwriteComments() || mouse.getComments() == null) {
						mouse.setComments(form.getComments());
					} else {
						String comments = mouse.getComments();
						comments += "\n" + form.getComments();
						mouse.setComments(comments);
					}
				}

				// stud
				if (form.getStud() != null && mouse.isMale()) {
					mouse.setStud(form.getStud());
				}

				// proven stud
				if (form.getProvenStud() != null && mouse.isMale()) {
					mouse.setProvenStud(form.getProvenStud());
				}

				// sterile
				if (form.getSterile() != null) {
					mouse.setSterile(form.getSterile());
				}

				// procedure
				if (subform.getProcedureId() != null) {
					MouseProcedure newProcedure = mouseProcedureService.createFromProcedureSubform(subform);
					if (newProcedure != null) {
						mouse.setCurrentLicencedProcedure(newProcedure);
					}
				}

				// genotype lock
				if (form.isMouseAlleleQcLock() && mouse.getAlleles() != null) {
					for (MouseAllele mouseAllele : mouse.getAlleles()) {
						mouseAllele.setLockedForUpdate(true);
					}
				}

				storeComplete(mouse, user);
				recordsUpdated++;
			}
		}

		return recordsUpdated;
	}


	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateMouseComponents(Mouse mouse) {
		if (mouse == null) {
			throw new RuntimeException("Mouse cannot be null when inflating components.");
		}

		// set alleles
		if (mouse.getAlleles() == null) {
			mouse.setAlleles(new ArrayList<MouseAllele>());
		}

		// set cage and rack
		if (mouse.getCage() == null) {
			mouse.setCage(new Cage());
			mouse.getCage().setRack(new Rack());
		}

		// set colony
		if (mouse.getColonyInfo() == null) {
			mouse.setColonyInfo(new ColonyInfo());
		}

		// set owner
		if (mouse.getCurrentOwner() == null)
			mouse.setCurrentOwner(new Person());

		// set location
		if (mouse.getLocation() == null)
			mouse.setLocation(new MouseLocation());

		// set litter
		if (mouse.getLitter() == null)
			mouse.setLitter(new Litter());

		// set strain
		if (mouse.getStrain() == null)
			mouse.setStrain(new DictionaryEntry());

		// set current licenced procedure
		if (mouse.getCurrentLicencedProcedure() == null) {
			MouseProcedure mouseProcedure = new MouseProcedure();
			mouseProcedureService.inflateComponents(mouseProcedure);
			mouse.setCurrentLicencedProcedure(mouseProcedure);
		}

		// set chimericData
		if (mouse.getChimericData() == null)
			mouse.setChimericData(new ChimericMouseData());

		// set current Weight
		if (mouse.getCurrentWeight() == null) {
			mouse.setCurrentWeight(new MouseWeight());
		}

		// set latest breeding
		if (mouse.getLatestBreeding() == null) {
			mouse.setLatestBreeding(new Breeding());
		}

		// set latest stud breeding
		if (mouse.getLatestStudBreeding() == null) {
			mouse.setLatestStudBreeding(new StudBreeding());
		}
	}


	@Transactional
	public Long moveMouse(Mouse mouse, BarcodedObject.BarcodeType objectType, Long objectId, Person user) {
		Long cageId = null;
		if (objectType == BarcodedObject.BarcodeType.RACK) {
			cageId = moveToNewCage(mouse, objectId, user);
		} else if (objectType == BarcodedObject.BarcodeType.CAGE) {
			moveToExistingCage(mouse, objectId, user);
			cageId = objectId;
		}
		return cageId;
	}


	@Transactional
	public void deleteMouse(Mouse mouse, Person user) {
		getPrimaryDao().deleteMouse(mouse, user);
	}


	public Mouse fetchMouseByTissueSampleId(Long tissueSampleId, ComponentMap requiredComponents) {
		Mouse mouse = getPrimaryDao().fetchMouseByTissueSampleId(tissueSampleId);

		if (mouse != null)
			attachComponents(mouse, requiredComponents);

		return mouse;
	}


	public MouseFaxView fetchFaxMouseByBarcode(String barcode) {
		return getPrimaryDao().fetchFaxMouseByBarcode(barcode);
	}


	public Collection<Mouse> fetchByRfidTag(String rfidTag) {
		return getPrimaryDao().fetchByRfidTag(rfidTag);
	}


	public boolean rfidTagInUse(Mouse mouse) {
		if (mouse == null || mouse.getRfidTag() == null) {
			throw new IllegalStateException("Mouse or RFID cannot be null when checking RFID status");
		}
		return getPrimaryDao().rfidTagInUse(mouse);
	}
}
