/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Rack;
import uk.ac.sanger.mig.mousetracking.Room;
import uk.ac.sanger.mig.mousetracking.dao.RackDao;
import uk.ac.sanger.mig.tecniplast.AirHandlingUnit;
import uk.ac.sanger.mig.tecniplast.services.AirHandlingUnitService;

public class RackServiceImpl extends AbstractServiceImpl2<Rack,RackDao> implements RackService {

	private AirHandlingUnitService airHandlingUnitService;
	private CageService cageService;
	private RoomService roomService;

	public void setAirHandlingUnitService(
			AirHandlingUnitService airHandlingUnitService) {
		this.airHandlingUnitService = airHandlingUnitService;
	}

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setRoomService(RoomService roomService) {
		this.roomService = roomService;
	}

	public int getMaxCapacity(Long rackId) {
		return getPrimaryDao().getMaxCapacity(rackId);
	}

	public Rack fetchByBarcode(String barcode, ComponentMap requiredComponents) {
		Rack rack = getPrimaryDao().fetchByBarcode(barcode);

		if(rack != null)
			attachComponents(rack, requiredComponents);

		return rack;
	}

	public Rack fetchAuditRack(Long roomId) {
		return getPrimaryDao().fetchAuditRack(roomId);
	}

	public Collection<Rack> fetchByAHUId(Long ahuId, ComponentMap requiredComponents) {
		Collection<Rack> racks = getPrimaryDao().fetchByAHUId(ahuId);

		attachComponents(racks, requiredComponents);

		return racks;
	}

	public Collection<Rack> fetchByRoomId(Long roomId, ComponentMap requiredComponents) {
		Collection<Rack> racks = getPrimaryDao().fetchByRoomId(roomId);

		attachComponents(racks, requiredComponents);

		return racks;
	}

	/**
	 * Attach optional associated objects to the input rack object.
	 *
	 * @param rack The Rack object to be modified
	 * @param requiredComponents EnumSet of required RackComponents.
	 */
	@Override
	protected void attachComponents(Rack rack, ComponentMap requiredComponents) {
		if(rack == null) {
			throw new RuntimeException("Cannot attach components to null rack");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Rack.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Rack.Component.CAGES)) {
				rack.setCages(fetchCages(rack, requiredComponents.get(Rack.Component.CAGES)));
			}

			if(fetchAll || requiredComponents.containsKey(Rack.Component.ROOM)) {
				rack.setRoom(fetchRoom(rack, requiredComponents.get(Rack.Component.ROOM)));
			}

			if(fetchAll || requiredComponents.containsKey(Rack.Component.AIR_HANDLING_UNIT)) {
				rack.setAirHandlingUnit(fetchAirHandlingUnit(rack, requiredComponents.get(Rack.Component.AIR_HANDLING_UNIT)));
			}

			if(fetchAll || requiredComponents.containsKey(Rack.Component.CAGE_COUNT)) {
				rack.setCageCount(fetchCageCount(rack));
			}
		}
	}

	private Integer fetchCageCount(Rack rack) {
		return cageService.getTotalCagesByRackId(rack.getId());
	}

	private Room fetchRoom(Rack rack, ComponentMap requiredComponents) {

		if(rack.getRoomId() == null) {
			return null;
		}
		else {
			return roomService.fetchById(rack.getRoomId(), requiredComponents);
		}
	}

	private AirHandlingUnit fetchAirHandlingUnit(Rack rack, ComponentMap requiredComponents) {
		if(rack.getAirHandlingUnitId() == null) {
			return null;
		}
		else {
			return airHandlingUnitService.fetchById(rack.getAirHandlingUnitId(), requiredComponents);
		}
	}

	private Collection<Cage> fetchCages(Rack rack, ComponentMap requiredComponents) {
		if(rack.getId() == null) {
			return null;
		}
		else {
			return cageService.fetchByRackId(rack.getId(), requiredComponents);
		}
	}

	public int getTotalRacksByRoomId(Long roomId) {
		return getPrimaryDao().getTotalRacksByRoomId(roomId);
	}


}
