/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.mousetracking.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.Stack;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.mousetracking.Cage;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MousePool;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.StudBreeding;
import uk.ac.sanger.mig.mousetracking.StudBreedingSearchBean;
import uk.ac.sanger.mig.mousetracking.dao.StudBreedingDao;
import uk.ac.sanger.mig.mousetracking.forms.AddStudBreedingForm;
import uk.ac.sanger.mig.mousetracking.forms.SeparateStudBreedingsForm;
import uk.ac.sanger.mig.mousetracking.forms.StudBreedingForm;
import uk.ac.sanger.mig.mousetracking.forms.SeparateStudBreedingsForm.PlugStates;
import uk.ac.sanger.mig.mousetracking.views.CageHeaderView;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class StudBreedingServiceImpl extends AbstractServiceImpl2<StudBreeding,StudBreedingDao> implements StudBreedingService {

	private CageService cageService;
	private MouseService mouseService;
	private MousePoolService mousePoolService;
	private ComponentMapService componentMapService;
	private PersonService personService;

	public void setCageService(CageService cageService) {
		this.cageService = cageService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setMousePoolService(MousePoolService mousePoolService) {
		this.mousePoolService = mousePoolService;
	}

	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	@Transactional
	public Long editBreeding(StudBreedingForm form, Person user) {
		if(form == null || !form.isValidated()) {
			throw new IllegalArgumentException("form is invalid");
		}

		StudBreeding breeding = form.getBreeding();
		Mouse female = breeding.getFemale();
		Mouse male = breeding.getMale();
		store(breeding, user);

		switch (form.getMaleOption()) {
		case DO_NOTHING:
			break;
		case CULL_MALE:
			if(male.isAlive()) {
				male.setCauseOfDeathDictId(MouseService.CAUSE_OF_DEATH_CULLED);
				male.setDeathDate(breeding.getEndDate());
				mouseService.store(male, user);
			}
			break;
		default:
			break;
		}

		switch (form.getFemaleOption()) {
		case DO_NOTHING:
			break;
		case RETURN_TO_POOL:
			MousePool pool = mousePoolService.fetchPool(female.getColonyInfoId(), female.getBirthDate(), null);
			female.setCageId(pool.getCageId());
			mouseService.store(female, user);
			break;
		case CULL_FEMALE:
			if(female.isAlive()) {
				female.setCauseOfDeathDictId(MouseService.CAUSE_OF_DEATH_CULLED);
				female.setDeathDate(breeding.getEndDate());
				mouseService.store(female, user);
			}
			break;
		case MOVE_FEMALE:
			if(female.isAlive()) {
				mouseService.moveMouse(female, form.getFemaleLocation().getBarcodeType(), form.getFemaleLocation().getId(), user);
			}
			break;
		default:
			break;
		}
		return breeding.getId();
	}

	public Collection<StudBreeding> fetchByMouseId(Long mouseId, ComponentMap requiredComponents) {
		Collection<StudBreeding> breedings = getPrimaryDao().fetchByMouseId(mouseId);
		attachComponents(breedings, requiredComponents);
		return breedings;
	}

	public Collection<StudBreeding> fetchByCageId(Long cageId, ComponentMap requiredComponents) {
		Collection<StudBreeding> breedings = getPrimaryDao().fetchByCageId(cageId);
		attachComponents(breedings, requiredComponents);
		return breedings;
	}

	private Long returnToPool(StudBreeding breeding, Person user, Date endDate) {
		Mouse female = breeding.getFemale();
		MousePool pool = mousePoolService.fetchPool(female.getColonyInfoId(), female.getBirthDate(), null);
		female.setCageId(pool.getCageId());
		mouseService.store(female, user);

		breeding.setEndDate(endDate);
		breeding.setPlugDateTime(null);
		store(breeding, user);

		return pool.getCageId();
	}

	private void movePluggedFemale(StudBreeding breeding, Person user, Date endDate, Float timedEmbryoAge, Long cageId) {
		Mouse female = breeding.getFemale();

		female.setCageId(cageId);
		mouseService.store(female, user);

		breeding.setEndDate(endDate);
		breeding.setPlugDateTime(endDate);
		breeding.setTimedEmbryoAge(timedEmbryoAge);
		store(breeding, user);
	}

	@Transactional
	public Map<String, Object> separateBreedings(SeparateStudBreedingsForm form, Person user) {
		if(form == null || !form.isValidated()) {
			throw new IllegalArgumentException("form is invalid");
		}
		Map<String, Object> model = new HashMap<String, Object>();

		Set<Long> failedStudCageIds = new LinkedHashSet<Long>();
		Set<Long> studlyStudCageIds = new LinkedHashSet<Long>();
		Set<Long> poolCageIds = new LinkedHashSet<Long>();
		Set<Long> newStockCageIds = new LinkedHashSet<Long>();
		Integer failedStuds = 0;
		Integer studlyStuds = 0;
		Integer notpluggedFemales = 0;
		Integer culledFemales = 0;
		Integer pluggedFemales = 0;

		StudBreeding[] studBreedings = form.getStudBreedings().toArray(new StudBreeding[0]);
		PlugStates[] plugStates = form.getStudBreedingPlugs().toArray(new PlugStates[0]);

		int femalesInCage = 0;
		Long newCageId = null;

		for (int i=0 ; i < plugStates.length; i++) {
			StudBreeding breeding = studBreedings[i];
			breeding.setCheckPersonId(form.getCheckPerson().getId());
			PlugStates plugState = plugStates[i];

			if(plugState.equals(PlugStates.NOT_PLUGGED)) {
				failedStudCageIds.add(breeding.getMale().getCageId());
				failedStuds++;
				Long poolCageId = returnToPool(breeding, user, form.getCheckDateTime());
				poolCageIds.add(poolCageId);
				notpluggedFemales++;
			}
			else if(plugState.equals(PlugStates.PLUGGED)) {
				studlyStudCageIds.add(breeding.getMale().getCageId());
				studlyStuds++;
				if(femalesInCage == 0) {
					newCageId = cageService.createInRack(form.getDestinationRack().getId(), user);
				}
				femalesInCage++;
				if(femalesInCage == form.getFemalesPerCage()) {
					femalesInCage = 0;
				}
				movePluggedFemale(breeding, user, form.getCheckDateTime(), form.getTimedEmbryoAge(), newCageId);
				newStockCageIds.add(newCageId);
				pluggedFemales++;
			}
			else if(plugState.equals(PlugStates.CULL)) {
				failedStuds++;
				failedStudCageIds.add(breeding.getMale().getCageId());
				store(breeding, user);
				mouseService.cull(breeding.getFemale(), MouseService.CAUSE_OF_DEATH_CULLED, form.getCheckDateTime(), user);
				culledFemales++;
			}
		}

		model.put("failedStudCageIds", failedStudCageIds);
		model.put("pluggedStudCageIds", studlyStudCageIds);
		model.put("poolCageIds", poolCageIds);
		model.put("newStockCageIds", newStockCageIds);
		model.put("failedStudCount", failedStuds);
		model.put("pluggedStudCount", studlyStuds);
		model.put("pluggedFemaleCount", pluggedFemales);
		model.put("notpluggedFemaleCount", notpluggedFemales);
		model.put("culledFemaleCount", culledFemales);

		return model;
	}

	@Transactional
	public Set<Long> addBreedings(AddStudBreedingForm form, Person user) {
		if(form == null || !form.isValidated()) {
			throw new IllegalArgumentException("form is invalid");
		}

		Set<Long> studCageIds = new LinkedHashSet<Long>();

		//make list of pooled females, reverse and add to a stack, so they can be popped off.
		Stack<Mouse> females = new Stack<Mouse>();
		int poolArrayLength = form.getPools().size();

		//use a traditional loop as we need to access two different lists
		for(int i=0; i<poolArrayLength; i++) {
			long femalesToUse = form.getFemalesToUse().get(i);
			MousePool pool = form.getPools().get(i);
			for(Mouse mouse : pool.getCage().getContents()) {
				if(femalesToUse > 0) {
					females.add(mouse);
					femalesToUse--;
				}
			}
		}
		Collections.reverse(females);

		int studArrayLength = form.getStuds().size();

		for(int i=0; i<studArrayLength; i++) {

			if(form.getStudsToUse().get(i) != null) {

				StudBreeding studBreeding = new StudBreeding();
				Mouse male = form.getStuds().get(i);
				Mouse female = females.pop();

				studBreeding.setMaleId(male.getId());
				studBreeding.setFemaleId(female.getId());
				studBreeding.setStartDateTime(form.getStartDateTime());
				studBreeding.setPseudoBreeding(male.isSterile());
				studBreeding.setSetupPersonId(form.getSetupPerson().getId());

				store(studBreeding, user);

				mouseService.moveToExistingCage(female, male.getCageId(), user);

				studCageIds.add(male.getCageId());
			}
		}

		return studCageIds;
	}

	public Collection<StudBreeding> fetch(StudBreedingSearchBean searchBean, ComponentMap requiredComponents) {
		Collection<StudBreeding> breedings = getPrimaryDao().fetch(searchBean);
		attachComponents(breedings, requiredComponents);
		return breedings;
	}

	@Override
	protected void attachComponents(StudBreeding studBreeding, ComponentMap requiredComponents) {
		if(studBreeding == null) {
			throw new RuntimeException("Cannot attach components to null studBreeding");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(StudBreeding.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(StudBreeding.Component.MALE)) {
				studBreeding.setMale(fetchMouse(studBreeding.getMaleId(), requiredComponents.get(StudBreeding.Component.MALE)));
			}

			if(fetchAll || requiredComponents.containsKey(StudBreeding.Component.FEMALE)) {
				studBreeding.setFemale(fetchMouse(studBreeding.getFemaleId(), requiredComponents.get(StudBreeding.Component.FEMALE)));
			}

			if(fetchAll || requiredComponents.containsKey(StudBreeding.Component.SETUP_PERSON)) {
				studBreeding.setSetupPerson(fetchPerson(studBreeding.getSetupPersonId(), requiredComponents.get(StudBreeding.Component.SETUP_PERSON)));
			}

			if(fetchAll || requiredComponents.containsKey(StudBreeding.Component.CHECK_PERSON)) {
				studBreeding.setCheckPerson(fetchPerson(studBreeding.getCheckPersonId(), requiredComponents.get(StudBreeding.Component.CHECK_PERSON)));
			}
		}
	}

	private Mouse fetchMouse(Long mouseId, ComponentMap requiredComponents) {
		Mouse mouse = null;
		if(mouseId != null) {
			mouse = mouseService.fetchById(mouseId, requiredComponents);
		}
		return mouse;
	}

	private Person fetchPerson(Long personId, ComponentMap requiredComponents) {
		Person person = null;
		if(personId != null) {
			person = personService.fetchById(personId, requiredComponents);
		}
		return person;
	}

	@Transactional
	public void closeCurrentForMouse(Long mouseId, Date endDate, Person user) {
		Collection<StudBreeding> breedings = getPrimaryDao().fetchByMouseId(mouseId);
		for(StudBreeding breeding : breedings) {
			if(breeding.getEndDate() == null) {
				breeding.setEndDate(endDate);
				store(breeding, user);
			}
		}
	}

	public Cage fetchForCageView(Long breedingId) {
		ComponentMap breedingComponents = componentMapService.getComponentMap("viewStudBreeding");
		StudBreeding breeding = fetchById(breedingId, breedingComponents);
		Cage breedingCage = makeCageForView(breeding);
		cageService.inflateCageComponents(breedingCage);
		return breedingCage;
	}

	private CageHeaderView makeCageHeaderForView(StudBreeding breeding) {
		CageHeaderView cageHeader = new CageHeaderView();
		Mouse female = breeding.getFemale();

		MouseProcedure lastProcedure = female.getCurrentLicencedProcedure();

		if(lastProcedure != null &&
				lastProcedure.getProtocol() != null &&
				lastProcedure.getProtocol().getProjectLicence() != null) {
			cageHeader.setLicenceNumber(lastProcedure.getProtocol().getProjectLicence().getLicenceNumber());
			cageHeader.setProtocolCode(lastProcedure.getProtocol().getProtocolCode());
			cageHeader.setOwnerName(female.getCurrentOwner().getFullName());
			cageHeader.setOwnerPhoneNumber(female.getCurrentOwner().getPhoneNumber());
		}
		return cageHeader;
	}

	private Cage makeCageForView(StudBreeding breeding) {
		Cage cage = null;

		if(breeding.getFemale().getCageId() == null) {
			cage = new Cage();
			CageHeaderView cageHeader = makeCageHeaderForView(breeding);
			cage.setCageHeader(cageHeader);
		}
		else {
			ComponentMap cageComponents = componentMapService.getComponentMap("cageLocation");
			cage = cageService.fetchById(breeding.getMale().getCageId(), cageComponents);
		}

		Collection<StudBreeding> breedings = new ArrayList<StudBreeding>();
		breedings.add(breeding);
		cage.setStudBreedings(breedings);

		cage.setCageType(CageType.STUD_CAGE);
		cageService.inflateCageComponents(cage);

		return cage;
	}

	@Transactional
	public void storeCollection(Collection<StudBreeding> breedings, Person user) {
		if(breedings != null) {
			for(StudBreeding breeding : breedings) {
				store(breeding, user);
			}
		}
	}

	public StudBreeding fetchLatestForMouse(Long mouseId, ComponentMap requiredComponents) {
		StudBreeding breeding = getPrimaryDao().fetchLatestForMouse(mouseId);

		if(breeding != null) {
			attachComponents(breeding, requiredComponents);
		}

		return breeding;
	}

}
