/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.services;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Date;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.services.ProtocolService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.TissueSample;
import uk.ac.sanger.mig.mousetracking.dao.TissueSampleDao;
import uk.ac.sanger.mig.personnel.Person;

public class TissueSampleServiceImpl extends AbstractServiceImpl2<TissueSample,TissueSampleDao> implements TissueSampleService {

	private MouseProcedureService mouseProcedureService;
	private ProtocolService protocolService;
	private MouseService mouseService;

	public void setProtocolService(ProtocolService protocolService) {
		this.protocolService = protocolService;
	}

	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	@Transactional
	public void storeCollection(Collection<TissueSample> tissueSample, Person user) {
		if(tissueSample != null) {
			for(TissueSample sample : tissueSample) {
				if(sample.getBarcode() != null) {
					storeCollection(tissueSample, user);
				}
			}
		}
	}

	@Transactional
	public void storeWithProcedure(Collection<TissueSample> tissueSamples, Person user, Long projectLicenceId, Long personalLicenceId) {

		if(tissueSamples != null) {
			for(TissueSample sample : tissueSamples) {
				if(sample.getBarcode() != null) {
					Long sampleId = store(sample, user);
					// add 19b procedure
					Mouse mouse = mouseService.fetchById(sample.getMouseId(), ComponentMap.of(Mouse.Component.ALLELES));
					MouseProcedure procedure = new MouseProcedure();
					procedure.setMouseId(sample.getMouseId());
					procedure.setStartDate(new Date());
					procedure.setProcedureDictId(MouseProcedureService.BLOOD_SAMPLE_PHENOTYPING);
					procedure.setPersonalLicenceId(personalLicenceId);
					Protocol protocol = protocolService.fetchByProcedureAndProject(procedure.getProcedureDictId(), projectLicenceId, mouse.isHarmfulMutant(), null);
					procedure.setProtocolId(protocol.getId());
					procedure.setForeignKeyId(sampleId);
					procedure.setForeignTableName(TableMetadataEnum.MTS_TISSUE_SAMPLE);
					mouseProcedureService.store(procedure, user);
				}
			}
		}
	}


	@Transactional
	public void storeForMouse(Long mouseId, TissueSample tissueSample, Person user) {
		if(tissueSample == null) {
			throw new RuntimeException("Cannot save a null tissueSample");
		}

		if(tissueSample.getBarcode() != null ) {
			tissueSample.setMouseId(mouseId);
			store(tissueSample, user);
		}
	}

	public Collection<TissueSample> fetchByMouseId(Long mouseId, ComponentMap requiredComponents) {
		Collection<TissueSample> tissueSamples = getPrimaryDao().fetchByMouseId(mouseId);

		attachComponents(tissueSamples, requiredComponents);

		return tissueSamples;
	}

	@Override
	protected void attachComponents(TissueSample tissueSample, ComponentMap requiredComponents) {
		if(tissueSample == null) {
			throw new RuntimeException("Cannot attach components to null tissueSample");
		}
	}

	public TissueSample fetchByBarcode(String barcode, ComponentMap requiredComponents) {
		TissueSample sample = getPrimaryDao().fetchByBarcode(barcode);

		return sample;
	}

	public String generateBarcode() {
		NumberFormat fmt = new DecimalFormat("00000000");
		return fmt.format(getPrimaryDao().generateBarcode());
	}

}
