/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.validators;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.forms.AddBreedingForm;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class AddBreedingFormValidator implements Validator {

	private MouseService mouseService;
	private BreedingService breedingService;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}

	public boolean supports(Class clazz) {
		return AddBreedingForm.class.equals(clazz);
	}

	public void validate(Object command, Errors errors) {

		AddBreedingForm form = (AddBreedingForm)command;

		// cage or rack should be entered
		if(form.getCageOrRackId() == null) {
			errors.rejectValue("barcode", null, "Cage or Rack is required.");
		}

		// start date should be entered
		if(form.getStartDate() == null) {
			errors.rejectValue("startDate", null, "Date of pairing is required.");
		}

		// check male is present and is actually male and is alive
		Mouse male = mouseService.fetchById(form.getMale().getId(), null);
		if(male == null) {
			errors.rejectValue("male.barcode", null, "Male is required.");
		}
		else if(male.isDead()) {
			errors.rejectValue("male.barcode", null, "Male mouse is registered as deceased.");
		}
		else if(male.getGenderDictId() != MouseService.MOUSE_GENDER_MALE) {
			errors.rejectValue("male.barcode", null, "'Male' mouse is not male.");
		}
		else if(male.isNonBreeder()) {
			errors.rejectValue("male.barcode",null, "Male is designated as a non-breeder.");
		}

		// check at least one female is present and is actually female
		Set<Long> distinctFemaleIds = new HashSet<Long>();
		boolean femalePresent = false;
		int femaleIndex = 0;

		// loop through female records and validate
		for(Mouse temp : form.getFemales()) {
			Mouse female = mouseService.fetchById(temp.getId(), null);
			if(female != null) {
				femalePresent = true;

				Collection<Breeding> femaleBreedings = breedingService.fetchCurrentForMouse(female.getId(), null);

				//check that animal is female and alive
				if(female.getGenderDictId() != MouseService.MOUSE_GENDER_FEMALE) {
					errors.rejectValue("females["+(femaleIndex++)+"].barcode", null, "'Female' mouse " + female.getName() + " is not female.");
				}
				else if(female.isDead()) {
					errors.rejectValue("females["+(femaleIndex++)+"].barcode", null, "Female " + female.getName() + " is registered as deceased.");
				}
				else if(female.isNonBreeder()) {
					errors.rejectValue("females["+(femaleIndex++)+"].barcode", null, "Female " + female.getName() + " is designated as a non-breeder.");
				}
				else if(!femaleBreedings.isEmpty()) {
					errors.rejectValue("females["+(femaleIndex++)+"].barcode", null, "Female " + female.getName() + " is already in mating " + femaleBreedings.iterator().next().getBreedingName() + ".");
				}
				else if(female.isWeanable() && !female.isWeaned()) {
					errors.rejectValue("females["+(femaleIndex++)+"].barcode", null, "Female " + female.getName() + " has not yet been weaned.");
				}
				else {
					//check animal is not duplicated in form
					if(distinctFemaleIds.contains(female.getId())) {
						errors.rejectValue("females["+(femaleIndex++)+"].barcode", null, "Female " + female.getName() + " is duplicated on the form.");

					}
					else {
						distinctFemaleIds.add(female.getId());
					}
				}

				//check if colony is same as male, if not, offspring colony is required
				if(male != null && (female.getColonyInfoId() == null || !female.getColonyInfoId().equals(male.getColonyInfoId())) &&
						form.getOffspringColonyId() == null) {
					errors.rejectValue("offspringColonyId", null, "Please specify offspring colony");
				}
			}
		}
		// if no female present, alert the user.
		if(!femalePresent) {
			errors.rejectValue("females[0].barcode",null, "A minimum of one female is required.");
		}

		if(!form.isTimed() && form.getTimedEmbryoAge() != null) {
			errors.reject(null, "Please tick 'Timed Mating' or remove required embryo age");
		}

		checkDuplicateNumbers(form.getBreedingNumbers(), errors);
	}

	/**
	 * For multi-mouse form, check non-null earmarks are unique for the cage.
	 * @param earmarks
	 * @param errors
	 */
	private void checkDuplicateNumbers(Collection<Long> numbers, Errors errors) {
		if(numbers != null) {
			//check for duplicates in the collection
			Set<Long> uniqueEarmarks = new HashSet<Long>();
			Collection<Long> nonNullEarmarks = new ArrayList<Long>();
			for(Long number : numbers) {
				if(number != null) {
					uniqueEarmarks.add(number);
					nonNullEarmarks.add(number);
				}
			}
			if(nonNullEarmarks.size() != uniqueEarmarks.size()) {
				errors.reject(null, "Please correct duplicate mating numbers.");
			}
		}
	}
}
