package uk.ac.sanger.mig.mousetracking.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.mousetracking.MousePool;
import uk.ac.sanger.mig.mousetracking.forms.AddStudBreedingForm;

/**
 * <p>
 * Validates a {@link AddStudBreedingForm} prior to setting up stud breedings.
 * To be valid, the form must:
 * </p>
 * <ul>
 * <li> Contain at least one available pooled female
 * <li> Contain at least one available stud male
 * <li> Contain a start date
 * <li> Contain enough females to accommodate all studs
 * </ul>
 *
 */
public class AddStudBreedingFormValidator implements Validator {

	public boolean supports(Class clazz) {
		return AddStudBreedingForm.class.equals(clazz);
	}

	public void validate(Object command, Errors errors) {
		AddStudBreedingForm form = (AddStudBreedingForm)command;
		form.setValidated(false);

		if(form.getStartDateTime() == null) {
			errors.reject(null, "Date paired is a required field.");
		}

		if(form.getSetupPerson().getId() == null) {
			errors.reject(null, "Setup by is a required field.");
		}

		if(form.getTotalFemalesToUse() == 0) {
			errors.reject(null, "At least one female pool is a required.");
		}

		int i = 0;
		MousePool[] pools = form.getPools().toArray(new MousePool[0]);
		Long[] femalesToUse = form.getFemalesToUse().toArray(new Long[0]);
		for(i =0 ; i < form.getPools().size(); i++) {
			if(femalesToUse[i] != null && femalesToUse[i] > pools[i].getContentsSize()) {
				errors.reject(null, "More females chosen (" + femalesToUse[i] + ") than are available in pool(" + pools[i].getContentsSize() + ").");
			}
		}

		if(form.getStudsToUse().isEmpty()) {
			errors.reject(null, "At least one stud is required.");
		}
		else {
			boolean studIdFound = false;
			for(Long studId : form.getStudsToUse()) {
				if(studId != null) {
					studIdFound = true;
					break;
				}
			}
			if(!studIdFound) {
				errors.reject(null, "At least one stud is required");
			}
		}

		if(!form.getPools().isEmpty() && form.getTotalStudsToUse() > form.getTotalFemalesToUse()) {
			errors.reject(null, "Not enough females ("+form.getTotalFemalesToUse()+") selected for all studs (" +form.getStuds().size()+")" );
		}

		if(!errors.hasErrors()) {
			form.setValidated(true);
		}
	}
}
