/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.validators;

import java.util.Collection;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.forms.LitterForm;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.forms.LitterForm.MODE;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class LitterFormValidator implements Validator {

	private ProcedureSubformValidator procedureSubformValidator;
	private MouseService mouseService;

	public boolean supports(Class clazz) {
		return LitterForm.class.equals(clazz);
	}

	public void setProcedureSubformValidator(
			ProcedureSubformValidator procedureSubformValidator) {
		this.procedureSubformValidator = procedureSubformValidator;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void validate(Object target, Errors errors) {

		LitterForm form = (LitterForm)target;
		Litter litter = form.getLitter();
		Mouse defaultValues = form.getDefaultPup();
		Collection<Mouse> pups = litter.getPups();

		if(litter.isNotPregnant() && litter.getBirthDate() != null) {
			errors.reject(null, "Birth Date cannot be entered if Not Pregnant is ticked.");
		}

		if(litter.isNotPregnant() && litter.getNumberHarvestedInt() > 0) {

			errors.reject(null, "Number Harvested cannot be entered if Not Pregnant is ticked.");
		}

		if(form.getMode() != MODE.HARVESTED && litter.getNumberBorn() != null && litter.getBirthDate() == null) {
			errors.reject(null, "Birth Date must be specified for new-born mice.");
		}

		if(litter.getPlugDate() != null &&
		   litter.getBreeding().getStartDate() != null &&
		   litter.getPlugDate().before( litter.getBreeding().getStartDate() ) ) {
			errors.reject(null, "Plug Check Date must not be before Date Paired.");
		}

		if(litter.getBirthDate() != null &&
		   litter.getPlugDate() != null) {
			if(litter.getBirthDate().before( litter.getPlugDate()) ||
					litter.getPlugDate().after(litter.getBirthDate())) {
				errors.reject(null, "Birth Date must be after Plug Check Date.");
			}

		}

		if(litter.getBirthDate() != null &&
				litter.getBirthDate().before(litter.getBreeding().getStartDate())) {
			errors.reject(null, "Birth Date must not be before Date Paired.");
		}

		if(form.isProcedureRequired() || form.isAddNewProcedure()) {
			ProcedureSubform subform = form.getProcedureSubform();

			errors.pushNestedPath("procedureSubform");
			ValidationUtils.invokeValidator(procedureSubformValidator, subform, errors);
			errors.popNestedPath();

			if(litter.getPlugDate() != null && subform.getProcedureStartDate() != null && subform.getProcedureStartDate().before(litter.getPlugDate())) {
				errors.reject(null, "Procedure Date cannot be before Plug Date.");
			}
		}

		if(form.getLitter().isNotPregnant() || form.getLitter().getNumberHarvested() != null || form.getLitter().getNumberBorn() != null) {
			if(form.isCullFemaleConfirmationRequired() && form.getCullFemaleRecipient() == null) {
				errors.reject(null, "Please indicate if recipient should be culled on the database.");
			}
		}

		if(pups != null) {

			if(pups.size() >0) {
				if(form.isColonyRequired() && defaultValues.getColonyInfoId() == null) {
					errors.reject(null, "Colony is a required field.");
				}

				if(form.isStrainRequired() && defaultValues.getStrainDictId() == null) {
					errors.reject(null, "Genetic Background is a required field.");
				}
			}

			for(Mouse pup : pups) {
				if(pup.getDeathDate() != null && pup.getCauseOfDeathDictId() == null) {
					errors.reject(null, "Please indicate fate for pup " + pup.getName());
				}

				// if a fate date is not recorded, only "to cull" is permitted as a fate
				if(pup.getDeathDate() == null && pup.getCauseOfDeathDictId() != null && !pup.getCauseOfDeathDictId().equals(MouseService.CAUSE_OF_DEATH_TO_CULL)) {
					errors.reject(null, "A date of fate is required if fate is recorded. (except for 'To Cull')");
				}

				if(pup.getWeanDate() != null && litter.getBirthDate() != null && pup.getWeanDate().before(litter.getBirthDate())) {
					errors.reject(null, "Wean date cannot come before birth date for pup " + pup.getName());
				}

				if(pup.getWeanDate() != null && pup.getDeathDate() != null && pup.getWeanDate().after(pup.getDeathDate())) {
					errors.reject(null, "Wean date cannot come after date of death for pup " + pup.getName());
				}

				if(pup.isNewRecord()) {
					Mouse tempMouse = mouseService.fetchByName(pup.getName(), null);
					if(tempMouse != null) {
						errors.reject(null, "A mouse named " + tempMouse.getName() + " already exists on the database. Cannot continue until this duplication is resolved.");
					}
				}
			}
		}

		if(form.isMaleLocationRequired() && !form.getMaleLocation().hasId()) {
			errors.reject(null, "Please scan rack or cage barcode for male pups.");
		}

		if(form.isFemaleLocationRequired() && !form.getFemaleLocation().hasId()) {
			errors.reject(null, "Please scan rack or cage barcode for female pups.");
		}

		if(form.isUnsexedLocationRequired() && !form.getUnsexedLocation().hasId()) {
			errors.reject(null, "Please scan rack or cage barcode for unsexed pups.");
		}

	}
}
