/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.validators;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.genotyping.services.AssayResultService;
import uk.ac.sanger.mig.mousetracking.Breeding;
import uk.ac.sanger.mig.mousetracking.BreedingSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.MouseWeight;
import uk.ac.sanger.mig.mousetracking.forms.MouseForm;
import uk.ac.sanger.mig.mousetracking.forms.ProcedureSubform;
import uk.ac.sanger.mig.mousetracking.services.BreedingService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class MouseFormValidator implements Validator {

	private MouseNameValidator			mouseNameValidator;
	private MouseValidator				mouseValidator;
	private MouseService				mouseService;
	private ProcedureSubformValidator	procedureSubformValidator;
	private MouseProcedureService		mouseProcedureService;
	private AssayResultService			assayResultService;
	private BreedingService				breedingService;


	public void setBreedingService(BreedingService breedingService) {
		this.breedingService = breedingService;
	}


	public void setAssayResultService(AssayResultService assayResultService) {
		this.assayResultService = assayResultService;
	}


	public void setMouseValidator(MouseValidator mouseValidator) {
		this.mouseValidator = mouseValidator;
	}


	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}


	public void setMouseNameValidator(MouseNameValidator mouseNameValidator) {
		this.mouseNameValidator = mouseNameValidator;
	}


	public void setProcedureSubformValidator(ProcedureSubformValidator procedureSubformValidator) {
		this.procedureSubformValidator = procedureSubformValidator;
	}


	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}


	public boolean supports(Class clazz) {
		return MouseForm.class.equals(clazz);
	}


	public void validate(Object target, Errors errors) {

		MouseForm mouseForm = (MouseForm) target;
		Mouse mouse = mouseForm.getMouse();

		// for live mouse the location (cage or rack) must be specified
		if (mouseForm.getLocationId() == null && mouseForm.getMouse().getDeathDate() == null) {
			errors.reject(null, "A cage or rack must be specified for live mice.");
		}

		// for new mouse form, the number of mice must be specified
		if (mouseForm.getMouse().getId() == null) {
			if (mouseForm.getNumberOfMice() == null) {
				errors.reject(null, "When adding new mice, quantity must be present.");
			}

			if(mouseForm.getMouseNames() != null && !mouseForm.isNameFromBarcode()){
				checkMouseNames(mouseForm, errors);
			}

			if (mouseForm.getMouseRfidTags() != null) {
				checkDuplicateRfidTags(mouseForm.getMouseRfidTags(), errors);
			}
		} else if (mouseForm.getMouse().getName() == null) {
			errors.reject(null, "Mouse ID is a required field.");
		}

		// if weight is present, sample date/time must be present too.
		if (mouseForm.getMouse().getCurrentWeight() != null) {
			MouseWeight mouseWeight = mouseForm.getMouse().getCurrentWeight();
			if (mouseWeight.getWeightValue() != null && mouseWeight.getSampleDate() == null) {
				errors.reject(null, "If weight value is present, sample date/time is a required field.");
			}
		}

		// if a new procedure has been specified then validate it
		if (mouseForm.isAddNewProcedure()) {
			ProcedureSubform subform = mouseForm.getProcedureSubform();
			if (subform.getProcedureId() != null) {
				errors.pushNestedPath("procedureSubform");
				ValidationUtils.invokeValidator(procedureSubformValidator, subform, errors);
				errors.popNestedPath();
			}
		}

		// for new mutant mice, the 19b must be specified
		if (!mouse.getAlleles().isEmpty() && mouse.isNewRecord()
				&& mouseForm.getProcedureSubform().getProtocolId() == null) {
			errors.reject(null, "19b Protocol information is required for mutant mice.");
		}
		// for existing mutant mice, the 19b must pre-exist or be specified on
		// the form
		else if (!mouse.getAlleles().isEmpty() && !mouse.isNewRecord()) {
			MouseProcedure currentProcedure = mouseProcedureService.fetchCurrentLicencedByMouseId(mouse
					.getId(), null);
			if (currentProcedure == null && mouseForm.getProcedureSubform().getProtocolId() == null) {
				errors.reject(null, "19b Protocol information is required for mutant mice.");
			}
		}

		checkForDuplicateMouseAlleles(mouse, errors);
		checkAllelesInGenotyingAssays(mouse, errors);
		checkMouseInBreeding(mouse, errors);

		// validate the mouse object in the form
		try {
			errors.pushNestedPath("mouse");
			ValidationUtils.invokeValidator(mouseValidator, mouseForm.getMouse(), errors);
		} finally {
			errors.popNestedPath();
		}
	}


	/**
	 * Returns true if this mouse has alleles that have an id (are persistent)
	 * and are flagged for delete, otherwise false.
	 *
	 * @param mouse -
	 *            the <code>Mouse</code> to check.
	 * @return true if one or more alleles is flagged for delete and is
	 *         currently stored in the db, otherwise false.
	 */
	private boolean isRemovingPersistentAlleles(Mouse mouse) {
		boolean isRemovingPersistentAllele = false;
		for (MouseAllele mouseAllele : mouse.getAlleles()) {
			if (mouseAllele.getId() != null && mouseAllele.isFlaggedForDelete())
				isRemovingPersistentAllele = true;
		}
		return isRemovingPersistentAllele;
	}


	/**
	 * Checks if the mouse has been used in a breeding, if it has and one or
	 * more of its persistent alleles are flagged for delete, then rejects the
	 * submission.
	 *
	 * @param mouse -
	 *            the <code>Mouse</code> to check.
	 * @param errors -
	 *            <code>Errors</code> object that error is added to if
	 *            submission is rejected.
	 */
	private void checkMouseInBreeding(Mouse mouse, Errors errors) {
		if (isRemovingPersistentAlleles(mouse)) {
			BreedingSearchBean searchBean = new BreedingSearchBean();
			searchBean.setPaginated(false);
			searchBean.setParentId(mouse.getId());
			Collection<Breeding> breedings = breedingService.search(searchBean, null);
			if (!breedings.isEmpty()) {
				errors.reject(null, "Allele(s) can not be removed as this mouse has been used in a breeding.");
			}
		}
	}


	/**
	 * Checks if any of the mouse alleles that have been flagged for deletion
	 * have been used in a genotyping assay and have a result, if they do then
	 * the submission is rejected.
	 *
	 * @param mouse -
	 *            the <code>Mouse</code> object whos alleles are to be
	 *            checked.
	 * @param errors -
	 *            list of errors.
	 */
	private void checkAllelesInGenotyingAssays(Mouse mouse, Errors errors) {
		for (MouseAllele mouseAllele : mouse.getAlleles()) {
			if (mouseAllele.getId() != null && mouseAllele.isFlaggedForDelete()) {
				if (assayResultService.hasResults(mouseAllele)) {
					errors.reject(null, "Mouse allele '" + mouseAllele.getAllele().getDisplayText()
							+ "' has associated genotyping assay results so can not be removed.");
				}
			}
		}
	}


	/**
	 * Checks this mouses alleles for duplicates. Ignores duplicates that are
	 * already in the database, but prevents new duplicates being added.
	 * Duplicates are defined by having matching display text strings as the id
	 * of the allele may be null if it is being added.
	 *
	 * @param mouse -
	 *            the <code>Mouse</code> object whos alleles are checked for
	 *            duplicates.
	 * @param errors -
	 *            list of errors to add to if duplicates are found.
	 */
	private void checkForDuplicateMouseAlleles(Mouse mouse, Errors errors) {
		Map<Long, MouseAllele> checkedAlleles = new HashMap<Long, MouseAllele>();
		for (MouseAllele mouseAllele : mouse.getAlleles()) {
			Long alleleId = mouseAllele.getAlleleId();
			if (checkedAlleles.containsKey(alleleId)) {
				MouseAllele checkedMouseAllele = checkedAlleles.get(alleleId);
				if (checkedMouseAllele.isFlaggedForDelete() || mouseAllele.isFlaggedForDelete()) {
					errors
							.reject(
									null,
									"A duplicate allele exists, but has been flagged for deletion, please delete this allele before re-entering it.");
					if (checkedMouseAllele.isFlaggedForDelete())
						checkedMouseAllele.setFlaggedForDelete(false);
					else
						mouseAllele.setFlaggedForDelete(false);
				} else {
					errors.reject(null, "Allele has already been added.");
				}
			} else {
				checkedAlleles.put(alleleId, mouseAllele);
			}
		}
	}


	/**
	 * For multi-mouse form, check non-null names are unique and not already in
	 * use and valid.
	 *
	 * @param names
	 * @param errors
	 */
	private void checkMouseNames(MouseForm form, Errors errors) {
		Mouse defaultMouse = form.getMouse();
		Collection<String> names = form.getMouseNames();
		if (names != null) {
			int index = 1;
			// check for duplicates in the collection
			Set<String> uniqueCaseCorrectedNames = new HashSet<String>();
			Collection<String> nonNullNames = new ArrayList<String>();
			for (String name : names) {
				if (StringUtils.isNotEmpty(name)) {
					name = name.toUpperCase();
					// adjust name to be uppercase except the last letter
					if (name.length() > 1) {
						name = caseAdjustedName(name);
					}
					uniqueCaseCorrectedNames.add(name);
					nonNullNames.add(name);
				} else {
					errors.reject(null, "Mouse ID " + index + " is a required field.");
				}
				index++;
			}
			if (nonNullNames.size() != uniqueCaseCorrectedNames.size()) {
				errors.reject(null, "Please remove duplicate mouse names.");
			} else {
				for (String name : nonNullNames) {
					Mouse temp = mouseService.fetchByName(name, null);
					if (temp != null) {
						errors.reject(null, "Mouse name (" + temp.getName() + ") is already in use.");
					} else {
						// need to explicitly set litter and colony id to test
						// each mouse
						// as these values are stored on this form in a separate
						// default mouse
						// object
						temp = new Mouse();
						temp.setName(name);
						temp.setLitterId(defaultMouse.getLitterId());
						temp.setColonyInfoId(defaultMouse.getColonyInfoId());
						boolean isMouseNameValid = mouseNameValidator.validate(temp, true);

						if (!isMouseNameValid) {
							errors.reject(null, "Mouse name (" + temp.getName()
									+ ") does not conform to standard naming scheme.");
						}

					}
				}
			}

			// update names on form to be correct case if no errors have
			// occurred
			// if(!errors.hasErrors()) {
			String[] namesArray = form.getMouseNames().toArray(new String[0]);
			form.getMouseNames().removeAll(form.getMouseNames());
			for (int i = 0; i < namesArray.length; i++) {
				namesArray[i] = caseAdjustedName(namesArray[i]);
				form.getMouseNames().add(namesArray[i]);
			}
			// }
		}
	}


	private String caseAdjustedName(String name) {
		if (name == null)
			return null;
		// adjust name to be uppercase except the last letter
		if (name.length() > 1) {
			return name.substring(0, name.length() - 1).toUpperCase()
					+ name.substring(name.length() - 1).toLowerCase();
		} else {
			return name;
		}
	}


	/**
	 * For multi-mouse form, check non-null earmarks are unique for the cage.
	 *
	 * @param earmarks
	 * @param errors
	 */
	private void checkDuplicateRfidTags(Collection<String> rfids, Errors errors) {
		if (rfids != null) {
			// check for duplicates in the collection
			Set<String> uniqueRfids = new HashSet<String>();
			Collection<String> nonNullRfids = new ArrayList<String>();
			for (String rfid : rfids) {
				if (rfid != null) {
					uniqueRfids.add(rfid);
					nonNullRfids.add(rfid);

					Mouse tempMouse = new Mouse();
					tempMouse.setRfidTag(rfid);
					if (mouseService.rfidTagInUse(tempMouse)) {
						errors.reject(null, "Mouse RFID (" + tempMouse.getRfidTag()
								+ ") is already registered to another live mouse.");
					}
				}
			}
			if (nonNullRfids.size() != uniqueRfids.size()) {
				errors.reject(null, "Please correct duplicate RFID barcodes.");
			}
		}
	}
}
