package uk.ac.sanger.mig.mousetracking.validators;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Pattern;

import uk.ac.sanger.mig.mousetracking.Litter;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.LitterService;

public class MouseNameValidator {

	private final SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MMM-yyyy");
	private final String prefixPattern = "[A-Z]{2,4}";
	private final String litterPattern = "\\d+\\.\\d+";
	private final String suffixPattern = "[a-z]{1}";
	private final String barcodePattern = "M\\d{8}";

	private LitterService litterService;
	private Date cutoffDate;

	public void setLitterService(LitterService litterService) {
		this.litterService = litterService;
	}

	public void setCutoffDateString(String cutoffDateString) {
		try {
			cutoffDate = dateFormat.parse(cutoffDateString);
		}
		catch(ParseException e) {
			cutoffDate = new Date();
		}
	}


	/**
	 * Ensure mouse name conforms to the standard nomenclature.
	 *
	 * For the Sanger Institute, the standard format is a 2-4 letter colony prefix (uppercase)
	 * followed by a mating number, period, litter number and animal letter (lowercase)
	 *
	 * E.G. MAAA34.7a
	 *
	 * If a mouse belongs to a litter, it should conform to the details of the litter (prefix, mating + litter number should match)
	 * Otherwise, if a mouse only belongs to a colony it's prefix should match the colony prefix
	 *
	 * @param mouse
	 * @param strict if true, records created before Feb 2008 will be execpt from strict validation
	 * @return
	 */
	public boolean validate(Mouse mouse, boolean excludeOldRecords) {
		boolean isValid = true;

		if(excludeOldRecords && mouse.getCreatedDate() != null && mouse.getCreatedDate().before(cutoffDate)) {
			isValid = true;
		}
		else {
			String mouseName = mouse.getName();

			// if litter is present, name must match litter name plus a single lowercase letter
			if(mouse.getLitterId() != null) {
				Litter litter = litterService.fetchById(mouse.getLitterId(), null);
				isValid = Pattern.matches("^" + litter.getLitterName() + suffixPattern + "$", mouseName);
			}
			else {
				isValid = Pattern.matches("^" + prefixPattern + litterPattern + suffixPattern + "$", mouse.getName());
			}

			if(!isValid) {
				isValid = Pattern.matches(barcodePattern, mouse.getName());
			}
		}
		return isValid;
	}

}
