/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

public class MouseValidator implements Validator {

	private MouseService mouseService;
	private MouseNameValidator mouseNameValidator;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setMouseNameValidator(MouseNameValidator mouseNameValidator) {
		this.mouseNameValidator = mouseNameValidator;
	}

	public boolean supports(Class clazz) {
		return Mouse.class.equals(clazz);
	}

	public void validate(Object target, Errors errors) {
		Mouse mouse = (Mouse)target;

		// only validate non-null object
		if (mouse != null) {

			String displayName = "";

			// the name should be unique
			if(mouse.getName()!=null) {
				Mouse temp = mouseService.fetchByName(mouse.getName(), null);
				if(temp != null && !temp.getId().equals(mouse.getId())) {
					errors.reject(null, "Mouse name (" + temp.getName() + ") is already in use.");
				}
				else if(!mouseNameValidator.validate(mouse, true)) {
					errors.reject(null, "Mouse name (" + mouse.getName() + ") does not conform to standard naming scheme.");
				}
				else {
					displayName = mouse.getName()+": ";
				}
			}

			//rfid should not be registered to another live mouse
			if(mouse.getRfidTag() != null && mouse.isAlive()) {
				if(mouseService.rfidTagInUse(mouse)) {
					errors.reject(null, "Mouse RFID (" + mouse.getRfidTag() + ") is already registered to another live mouse.");
				}
			}

			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"genderDictId",null, displayName + "Gender is a required field.");
			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"strainDictId",null, displayName + "Genetic Background is a required field.");
			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"currentOwnerId", null, displayName + "Owner is a required field.");
			ValidationUtils.rejectIfEmptyOrWhitespace(errors,"colonyInfoId", null, displayName + "Colony is a required field.");

			// if date of death is recorded, cause of death is required
			if(mouse.getDeathDate() != null && mouse.getCauseOfDeathDictId() == null) {
				errors.reject(null, displayName + "Fate is required if date of fate is recorded.");
			}

			// if a fate date is not recorded, only "to cull" is permitted as a fate
			if(mouse.getDeathDate() == null && mouse.getCauseOfDeathDictId() != null && !mouse.getCauseOfDeathDictId().equals(MouseService.CAUSE_OF_DEATH_TO_CULL)) {
				errors.reject(null, displayName + "A date of fate is required if fate is recorded. (except for 'To Cull')");
			}

			if(mouse.getBirthDate() != null ) {
				// must be before death date.
				if(mouse.getDeathDate() != null ) {
					if(mouse.getBirthDate().after(mouse.getDeathDate())) {
						errors.reject(null, displayName + "Death date cannot be before birth date.");
					}
				}
				// must be before wean date.
				if(mouse.getWeanDate() != null ) {
					if(mouse.getBirthDate().after(mouse.getWeanDate())) {
						errors.reject(null, displayName + "Wean date cannot be before birth date.");
					}
				}
			}
		}
	}
}
