/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mousetracking.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.mousetracking.BarcodedObject.BarcodeType;
import uk.ac.sanger.mig.mousetracking.forms.MoveMouseForm;
import uk.ac.sanger.mig.mousetracking.services.MouseService;

/**
 * <p>
 * Validates a {@link MoveMouseForm} prior to moving mice.
 * To be valid, the form must:
 * </p>
 * <ul>
 * <li> Contain at least one origin cage
 * <li> Contain at least one mouse to move
 * <li> Contain a destination rack or cage
 * <li> Contain a destination rack if singlyHoused is true
 * <li> Contain a destination rack is femalePool is true
 * </ul>
 *
 * @see MouseService#moveMice(MoveMouseForm, uk.ac.sanger.mig.personnel.Person)
 */
public class MoveMouseFormValidator implements Validator {

	public boolean supports(Class clazz) {
		return MoveMouseForm.class.equals(clazz);
	}

	public void validate(Object target, Errors errors) {
		MoveMouseForm form = (MoveMouseForm)target;
		form.setValidated(false);

		if(form.getCages() == null || form.getCages().size() == 0) {
			errors.reject(null, "Please scan a cage to move mice from.");
		}
		else if(form.getMovingMouseIds() == null || form.getMovingMouseIds().length == 0) {
			errors.reject(null, "Please select some mice to move.");
		}

		if(form.getDestinationId() == null) {
			errors.reject(null, "Please scan a destination cage or rack to move mice to.");
		}
		else {
			if(form.isStud() && BarcodeType.CAGE.equals(form.getDestinationBarcodeType())) {
				errors.reject(null, "Please scan a destination rack to put mice in individual cages.");
			}
			if(form.isFemalePool() && BarcodeType.CAGE.equals(form.getDestinationBarcodeType())) {
				errors.reject(null, "Please scan a destination rack for pool cages.");
			}
			if(form.isFemalePool() && form.getFemalePoolDate() == null) {
				errors.reject(null, "Please specify pooled birth date for females.");
			}
		}

		if(!errors.hasErrors()) {
			form.setValidated(true);
		}
	}
}
