/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.personnel;

import java.util.HashMap;
import java.util.Map;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;
import uk.ac.sanger.mig.licensing.PersonalLicence;

public class Person extends AuthenticatedDataObject implements LookupListElement, Cloneable {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		PIL,
		TEAM,
		ROLES
	}
	
    private String firstName;        
    private String lastName;        
    private String passwordHash;        
    private String userName;         
    private String email;        
    private String address;        
    private String roomNumber;        
    private String phoneNumber; 
    private boolean hidden;
    private boolean active;
    private Long teamId; 
	private Map<String, Role> roles = null;
	private Team team;
	private PersonalLicence personalLicence;
	private String displayText;

	public Person clone() {
		Person clone = new Person();
		
		clone.setId(getId());
		clone.setFirstName(getFirstName());
		clone.setLastName(getLastName());
		clone.setPasswordHash(getPasswordHash());
		clone.setUserName(getUserName());
		clone.setEmail(getEmail());
		clone.setAddress(getAddress());
		clone.setRoomNumber(getRoomNumber());
		clone.setPhoneNumber(getPhoneNumber());
		clone.setHidden(isHidden());
		clone.setActive(isActive());
		clone.setRoles(new HashMap<String, Role>(roles));
		clone.setTeamId(getTeamId());
		if(getTeam() != null) {
			clone.setTeam(getTeam().clone());
		}
		if(getPersonalLicence() != null){
			clone.setPersonalLicence(getPersonalLicence().clone());
		}
		if(clone.getPersonalLicence() != null) {
			clone.getPersonalLicence().setPerson(clone);
		}
		clone.setDisplayText(getDisplayText());
		
		return clone;
	}
	
	public String getFullName() {
		String fullName = "";
		if(firstName != null && lastName != null) 
			fullName = firstName + " " + lastName;
		
		return fullName;
	}
	
	public String getFirstName() {
		return firstName;
	}
	
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}
	
	
	public String getLastName() {
		return lastName;
	}
	
	public void setLastName(String surName) {
		this.lastName = surName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}
	
	public String getUserName() {
		return userName;
	}
	
	/**
	 * Determine is this user is authorised to access a resource based on the supplied list of
	 * roles matched against their role permissions in the database.
	 * 
	 * @param roles is the set of roles associated with a resource that is being checked for
	 *     permission. 
	 * @return true if the user has permission to access the given resource and false otherwise.
	 */
	public boolean isAuthorised(String[] rolesList){
		
		if(roles == null){
			return false;
		}
		
		for(String roleName : rolesList){
			
			roleName = roleName.toLowerCase();
			
			if(roles.containsKey(roleName)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Set the Roles that this Person has.
	 * 
	 * @param rs A RoleSet containing a reference to all of the Roles that this person has.
	 */
	public void setRoles(Map<String, Role> roles){
		this.roles = roles;
	}
	

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getPhoneNumber() {
		return phoneNumber;
	}

	public void setPhoneNumber(String phoneNumber) {
		this.phoneNumber = phoneNumber;
	}

	public String getRoomNumber() {
		return roomNumber;
	}

	public void setRoomNumber(String roomNumber) {
		this.roomNumber = roomNumber;
	}


	public void setTeam(Team team) {
		this.team = team;
	}
	
	public Team getTeam() {
		return team;
	}

	public String getPasswordHash() {
		return passwordHash;
	}

	public void setPasswordHash(String passwordHash) {
		this.passwordHash = passwordHash;
	}

	public Long getTeamId() {
		return teamId;
	}

	public void setTeamId(Long teamId) {
		this.teamId = teamId;
	}

	public Map<String, Role> getRoles() {
		return roles;
	}

	public PersonalLicence getPersonalLicence() {
		return personalLicence;
	}

	public void setPersonalLicence(PersonalLicence personalLicence) {
		this.personalLicence = personalLicence;
	}

	public boolean isActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}

	public boolean isHidden() {
		return hidden;
	}

	public void setHidden(boolean hidden) {
		this.hidden = hidden;
	}

	public String getDisplayText() {	
		if(displayText != null) {
			return displayText;
		}
		else if(lastName != null && firstName != null) {
			return lastName + "," + firstName;
		}
		else {
			return "";
		
		}
	}

	public String getElementData() {
		return 
			LookupListUtil.format("personId", getId()) + 
			LookupListUtil.format("firstName", getFirstName()) +
			LookupListUtil.format("lastName", getLastName()) +
			LookupListUtil.format("userName", getUserName()) +
			LookupListUtil.format("personName", getFirstName() + " " + getLastName());
	}

	public void setDisplayText(String displayText) {
		this.displayText = displayText;		
	}

	public void setElementData(String elementData) {
		// TODO Auto-generated method stub		
	}

	
	
}
