/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.personnel.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.login.PasswordHasher;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.PersonSearchBean;

public class PersonDaoSpringImpl extends AbstractDaoSpringImpl<Person> implements PersonDao {

	private PasswordHasher passwordHasher = new PasswordHasher("SHA");
	
	public Collection<Person> fetch(PersonSearchBean bean) {
		Collection<Object> bindVariables = new ArrayList<Object>();
		String sql = "select * from per_person where 1=1 ";
		
		String orderBySql = "";
		
		if(bean.getHidden() != null) {
			sql += " and hidden = ? ";
			bindVariables.add(bean.getHidden());
		}
		
		if(bean.getActive() != null) {
			sql += " and active = ? ";
			bindVariables.add(bean.getActive());
		}
		
		if(bean.getOrderBy() != null && bean.getOrderBy().length() > 0) {
			orderBySql += " order by " + bean.getOrderBy();
			if(bean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}
		
		sql += orderBySql;
		
		Collection<Person> people = getSimpleJdbcTemplate().query(sql, getRowMapper(), bindVariables.toArray());
		
		return (people == null) ? new ArrayList<Person>() : people;
	}
	
	public Person fetchByUserName(String userName) {
		String sql = " select * from per_person where upper(user_name) = upper(?) ";
		Person person = null;
		
		try {
			person = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), userName);
		}
		catch(IncorrectResultSizeDataAccessException e) {
			logger.warn(e);
		}
		
		return person;		
	}

	public boolean authoriseAgainstDatabase(String username, String password) {
		String sql = " select count(*) from per_person where upper(user_name) = upper(?) and password_hash = ? and active = 1 ";
		
		String hashedPassword = passwordHasher.doHash(password);
		
		int personExists = getSimpleJdbcTemplate().queryForInt(sql, username, hashedPassword);
		
		return (personExists == 1) ? true : false;
	}

	public Person fetchPersonByFirstAndLastName(String firstName, String lastName) {
		Person person = null;		
		String sql = " select * from per_person where first_name = ? and last_name = ?";
		
		try { 
			 person = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), firstName, lastName);
		}
		catch(IncorrectResultSizeDataAccessException e) {
			logger.warn(e);
		}
		
		return person;	
	}
	
	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Person person) {
		sql.addStringParam(person.getFirstName(), "FIRST_NAME");
		sql.addStringParam(person.getLastName(), "LAST_NAME");
		sql.addStringParam(person.getPasswordHash(), "PASSWORD_HASH");
		sql.addStringParam(person.getUserName(), "USER_NAME");
		sql.addStringParam(person.getEmail(), "EMAIL");
		sql.addStringParam(person.getAddress(), "ADDRESS");
		sql.addStringParam(person.getRoomNumber(), "ROOM");
		sql.addStringParam(person.getPhoneNumber(), "PHONE_NUMBER");
		sql.addLongParam(person.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(person.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(person.getEditedBy(), "EDITED_BY");
		sql.addDateParam(person.getEditDate(), "EDIT_DATE");
		sql.addLongParam(person.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(person.getTeamId(), "TEAM_ID");
		sql.addBooleanParam(person.isHidden(), "HIDDEN");
		sql.addBooleanParam(person.isActive(), "ACTIVE");
		sql.addLongParam(person.getId(), "PERSON_ID");
	}
	
}
