/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.personnel.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.login.LdapAuthenticator;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.PersonSearchBean;
import uk.ac.sanger.mig.personnel.Role;
import uk.ac.sanger.mig.personnel.Team;
import uk.ac.sanger.mig.personnel.dao.PersonDao;
import uk.ac.sanger.mig.personnel.forms.PersonForm;

public class PersonServiceImpl extends AbstractServiceImpl<Person>
  implements PersonService {

	private PersonDao personDao;

	private TeamService teamService;
	private RoleService roleService;
	private PersonalLicenceService personalLicenceService;
	private boolean ldapEnabled;

	private LdapAuthenticator ldapAuthenticator;

	public void setPersonDao (PersonDao personDao) {
		this.personDao= personDao;
	}

	public void setRoleService(RoleService roleService) {
		this.roleService = roleService;
	}

	public void setTeamService(TeamService teamService) {
		this.teamService = teamService;
	}

	public void setPersonalLicenceService(
			PersonalLicenceService personalLicenceService) {
		this.personalLicenceService = personalLicenceService;
	}

	public void setLdapAuthenticator(LdapAuthenticator ldapAuthenticator) {
		this.ldapAuthenticator = ldapAuthenticator;
	}

	public void setLdapEnabled(boolean ldapEnabled) {
		this.ldapEnabled = ldapEnabled;
	}

  public boolean isLdapEnabled() {
    return ldapEnabled;
  }

	public Long store(Person person, Person user) {
		return personDao.store(person, user);
	}

	public Person fetchById(Long id, ComponentMap componentMap) {
		Person person = personDao.fetchById(id);

		if(person != null)
			attachComponents(person, componentMap);

		return person;
	}

	public Person fetchByUserName(String userName, ComponentMap requiredComponents) {
		Person person = personDao.fetchByUserName(userName);

		if(person != null)
			attachComponents(person, requiredComponents);

		return person;
	}

	public Person authenticate(String userName, String password) {
    if (userName == null || password == null) throw new IllegalArgumentException("authenticate method does not take null arguments");
		Person user = null;
		boolean isAuthorised = false;

		// verify that the user name exists
		user = personDao.fetchByUserName(userName);

		if (user != null) {
			// first try to authorise against LDAP

      RuntimeException ldapFailure = null;

			if(ldapEnabled) {
        try {
          isAuthorised = ldapAuthenticator.authenticate(userName, password);
			  }
        catch (RuntimeException e) {
          // swallow the exception for now, give the db authentication a chance
          ldapFailure = e;
        }
      }

			// if that fails, next try to authorise against our database
			if(!isAuthorised) {
				isAuthorised = personDao.authoriseAgainstDatabase(userName, password);
			}

			if(isAuthorised) {
				// if not found, load the "guest" account from the database
				if(!user.isActive()) {
					user = personDao.fetchByUserName("guest");
				}
			}

      /* throw the exception if authentication has failed,
      and ldap broke, and the user wasn't trying to authenticate against the db
      */
			if (!isAuthorised && ldapFailure != null && user.getPasswordHash() == null)
			  throw ldapFailure;

      if (!isAuthorised) {
				user = null;
			}

			if(user != null)
				attachComponents(user, ComponentMap.of(Person.Component.ALL_COMPONENTS));
		}

		return user;
	}

	@Override
	protected void attachComponents(Person person, ComponentMap requiredComponents) {
		if(person == null) {
			throw new RuntimeException("Cannot attach components to null person");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Person.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Person.Component.TEAM)) {
				person.setTeam(fetchTeam(person, requiredComponents.get(Person.Component.TEAM)));
			}

			if(fetchAll || requiredComponents.containsKey(Person.Component.ROLES)) {
				person.setRoles(fetchRoles(person, requiredComponents.get(Person.Component.ROLES)));
			}

			if(fetchAll || requiredComponents.containsKey(Person.Component.PIL)) {
				person.setPersonalLicence(fetchPersonalLicence(person, requiredComponents.get(Person.Component.PIL)));
			}

		}
	}

	public Collection<Person> fetch(PersonSearchBean searchBean, ComponentMap requiredComponents) {
		Collection<Person> people = personDao.fetch(searchBean);

		attachComponents(people, requiredComponents);

		return people;
	}

	private PersonalLicence fetchPersonalLicence(Person person, ComponentMap requiredComponents) {
		return personalLicenceService.fetchByPersonId(person.getId(), requiredComponents);
	}

	private Team fetchTeam(Person person, ComponentMap requiredComponents) {
		if(person.getTeamId() != null) {
			return teamService.fetchById(person.getTeamId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	private Map<String, Role> fetchRoles(Person person, ComponentMap requiredComponents) {
		if(person.getId() != null) {
			return roleService.fetchByPersonId(person.getId(), requiredComponents);
		}
		else {
			return null;
		}
	}

	@Transactional
	public Long store(PersonForm personForm, Person user) {
		Person person = personForm.getPerson();

		// store the person
		Long personId = store(person, user);

		// store the personal licence
		PersonalLicence personalLicence = person.getPersonalLicence();
		if(personalLicence.getLicenceNumber() != null && personalLicence.getLicenceNumber().length() > 0) {
			personalLicence.setPersonId(personId);
			Long personalLicenceId = personalLicenceService.store(personalLicence, user);

			// store links to project licences
			personalLicenceService.clearLinksToPpls(personalLicenceId);
			for(ProjectLicence ppl : personForm.getPpls()) {
				if(ppl.getId() != null) {
					personalLicenceService.linkToPpl(personalLicenceId, ppl.getId());
				}
			}
		}

		// store person roles
		roleService.clearLinksToPerson(personId);
		for(Role role : person.getRoles().values()) {
			roleService.linkToPerson(role.getId(), personId);
		}

		return personId;
	}

	public void inflatePersonComponents(Person person) {
		if(person == null) {
			throw new RuntimeException("Cannot inflate components of null person");
		}

		// team
		if(person.getTeam() == null) {
			person.setTeam(new Team());
		}

		// roles
		if(person.getRoles() == null) {
			person.setRoles(new HashMap<String, Role>());
		}

		// personal licence
		if(person.getPersonalLicence() == null) {
			person.setPersonalLicence(new PersonalLicence());
		}
	}
}