/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.abc.dao;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.phenotyping.abc.BloodsRun;
import uk.ac.sanger.mig.phenotyping.abc.BloodsRunSearchBean;

public class BloodsRunDaoSpringImpl extends AbstractDaoSpringImpl<BloodsRun> implements BloodsRunDao {

	private PaginatedSearchUtils<BloodsRun> paginator;	


	public void setPaginator(PaginatedSearchUtils<BloodsRun> paginator) {
		this.paginator = paginator;
	}

	/**
	 * 
	 * @return the paginator
	 */
	public PaginatedSearchUtils<BloodsRun> getPaginator() {
		return this.paginator;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, BloodsRun bloodsRun) {
		sql.addDateParam(bloodsRun.getRunDate(), "RUN_DATE");
		sql.addLongParam(bloodsRun.getMouseId(), "MOUSE_ID");
		sql.addLongParam(bloodsRun.getInstrumentId(), "INSTRUMENT_ID");
		sql.addLongParam(bloodsRun.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(bloodsRun.getCreatedBy(), "CREATED_BY");
		sql.addDateParam(bloodsRun.getCreatedDate(), "CREATED_DATE");
		sql.addStringParam(bloodsRun.getEditedBy(), "EDITED_BY");
		sql.addDateParam(bloodsRun.getEditDate(), "EDIT_DATE");
		sql.addStringParam(bloodsRun.getComments(), "COMMENTS");
		sql.addBooleanParam(bloodsRun.isFailed(), "FAILED");
		sql.addLongParam(bloodsRun.getRunTypeDictId(), "RUN_TYPE");
		sql.addBooleanParam(bloodsRun.isValidated(),"VALIDATED");
		sql.addLongParam(bloodsRun.getId(), "ID");

	}




	public Collection<BloodsRun> search(BloodsRunSearchBean bean) {

		Collection<Object> bindVariables = new ArrayList<Object>();	

		String selectSql  = " select p.* ";
		String fromSql = " from phn_bloods_run p, mts_mouse m ";
		String whereSql = " where 1=1 and p.mouse_id = m.id ";		

		// ID
		if(bean.getId() != null) {
			whereSql += " and p.id = ? ";
			bindVariables.add(bean.getId());
		}

		// mouse id
		if(bean.getMouseId() != null) {
			whereSql += " and p.mouse_id = ? ";
			bindVariables.add(bean.getMouseId());
		}

		if(bean.getCohortId() != null) {
			whereSql += " and exists (select null from mts_cohort_mouse cm where cm.mouse_id = m.id and cm.cohort_id = ?) ";
			bindVariables.add(bean.getCohortId());
		}

		// ColonyInfo ID
		if(bean.getColonyInfoId() != null) {
			whereSql += " and m.colony_info_id = ? ";
			bindVariables.add(bean.getColonyInfoId());
		}		

		// Strain
		if(bean.getStrainId() != null) {
			whereSql += " and m.mouse_strain_dict_id = ? ";
			bindVariables.add(bean.getStrainId());
		}


		// Start Date
		if(bean.getSampledDateFrom() != null) {
			whereSql += " and trunc(p.run_date) >= ? ";
			bindVariables.add(defaultDateFormat.format(bean.getSampledDateFrom()));
		}

		// End date
		if(bean.getSampledDateTo() != null) {
			whereSql += " and trunc(p.run_date) <= ? ";
			bindVariables.add(defaultDateFormat.format(bean.getSampledDateTo()));
		}

		// Failed Sample
		if(bean.isRejected()) {
			whereSql += " and p.failed = 1 ";
		}

		if(bean.getInstrumentId() != null) {
			// 0 == all machines.
			if(bean.getInstrumentId() == 0) {
				whereSql += " and p.instrument_id in (1,2) ";
			}else{
				whereSql += " and p.instrument_id = ? ";
				bindVariables.add(bean.getInstrumentId());
			}
		}


		String orderBySql = "";

		if(bean.getOrderBy() != null && bean.getOrderBy().length() > 0) {
			orderBySql += " order by " + bean.getOrderBy();
			if(bean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}

		String sql = selectSql + fromSql + whereSql + orderBySql;

		logger.debug(sql);

		return paginator.fetchSearchResults(sql, bindVariables, bean, getRowMapper());

	}





}

