/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.abc.services;

import java.util.Collection;

import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.abc.BloodsData;
import uk.ac.sanger.mig.phenotyping.abc.BloodsDataSearchBean;
import uk.ac.sanger.mig.phenotyping.abc.BloodsParamRange;
import uk.ac.sanger.mig.phenotyping.abc.dao.BloodsDataDao;

public class BloodsDataServiceImpl extends AbstractServiceImpl<BloodsData> implements BloodsDataService {

	private BloodsDataDao bloodsDataDao;
	private BloodsParamRangeService bloodsParamRangeService;


	public void setBloodsParamRangeService(
			BloodsParamRangeService bloodsParamRangeService) {
		this.bloodsParamRangeService = bloodsParamRangeService;
	}


	public void setBloodsDataDao(BloodsDataDao bloodsDataDao) {
		this.bloodsDataDao = bloodsDataDao;
	}


	public Long storeAuthenticated(BloodsData bloodsData, Person user) {
		if(bloodsData == null) {
			throw new RuntimeException("Cannot save a null bloodsData");
		}

		BloodsParamRange range = bloodsParamRangeService.fetchCurrentByParamId(bloodsData.getParameterId());
		if(range != null ) {
			bloodsData.setRangeId(range.getId());
		}

		return bloodsDataDao.store(bloodsData, user);
	}

	public BloodsData fetchById(Long bloodsDataId, ComponentMap requiredComponents) {
		BloodsData bloodsData = bloodsDataDao.fetchById(bloodsDataId);

		if(bloodsData != null)
			attachComponents(bloodsData, requiredComponents);

		return bloodsData;
	}

	public Collection<BloodsData> fetchByRunId(Long runId, ComponentMap requiredComponents) {

		Collection<BloodsData> data = bloodsDataDao.fetchByRunId(runId);
	
		return data;
	}

	public Collection<BloodsData> fetchByRunIdNoGraphs(Long runId) {

		Collection<BloodsData> data = bloodsDataDao.fetchByRunIdNoGraphs(runId);
		
		return data;
	}
	
	
	@Override
	protected void attachComponents(BloodsData bloodsData, ComponentMap requiredComponents) {
		if(bloodsData == null) {
			throw new RuntimeException("Cannot attach components to null bloodsData");
		}		

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(BloodsData.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}


		}
	}

	public Collection<BloodsData> search(BloodsDataSearchBean bloodsDataSearchBean) {
		// TODO Auto-generated method stub
		return null;
	}

	public Collection<BloodsData> fetchAllByParameterId(Long parameterId) {

		Collection<BloodsData> data = bloodsDataDao.fetchAllByParameterId(parameterId);

		return data;
	}


	public Double calculateMeanForDataSet(Collection<BloodsData> dataSet) {

		Double total = 0.0;

		for(BloodsData data : dataSet) {
			Double val = Double.parseDouble(data.getValue());
			total += val; 
		}

		Double mean = total / dataSet.size();

		return mean;

	}

	public Double calculateRangeForDataSet(Collection<BloodsData> dataSet) {

		Double minVal = 0.0;
		Double maxVal = 0.0;

		boolean firstMin = false;
		boolean firstMax = false;
		
		for(BloodsData data : dataSet) {
			Double val = Double.parseDouble(data.getValue());
			if(firstMin == false || minVal > val) {
				minVal = val;
				firstMin = true;
			}else if(firstMax == false || maxVal < val) {
				maxVal = val;
				firstMax = true;
			}
		}

		Double range = maxVal - minVal;

		return range;

	}

	public Double calculateStandardDeviationForDataSet(Collection<BloodsData> dataSet) {

		Double sumVal = 0.0;
		Double mean = calculateMeanForDataSet(dataSet);
		
		for(BloodsData data : dataSet) {
			Double val = Double.parseDouble(data.getValue());
			final double v = val - mean;
			sumVal += v * v;
		}
		
		return Math.sqrt(sumVal / dataSet.size());
		
	}

	public Double calcuateCoefficientOfVariation(Collection<BloodsData> dataSet) {
		
		Double m = calculateMeanForDataSet(dataSet);
		Double sd = calculateStandardDeviationForDataSet(dataSet);
		
		Double cv = (sd / m) * 100;
		
		return cv;
	}
	
	public ChartData fetchChartDataByParameterId(Long parameterId) {
		
		return bloodsDataDao.fetchChartDataByParameterId(parameterId);
		
	}


	public void deleteResultsByRunId(Long runId, Person user) {
		if(runId != null) {
			Collection<BloodsData> data = bloodsDataDao.fetchByRunId(runId);
			for(BloodsData dataObj : data) {
				bloodsDataDao.delete(dataObj, user);
			}
		}
	}
	
	public void deleteByTestId(Long testId, Person user) {
		
		BloodsData test = fetchById(testId, null);
		if(test != null) {
			bloodsDataDao.delete(test, user);
		}
	}
	
}
