/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.abc.views;

import uk.ac.sanger.mig.phenotyping.abc.ABCGraphData;
import ChartDirector.Chart;
import ChartDirector.LegendBox;
import ChartDirector.MultiChart;
import ChartDirector.XYChart;


public class AbcBloodDataChart {


	private MultiChart chart;
	private ABCGraphData plt;
	private ABCGraphData rbc;
	private ABCGraphData wbc;
	private double percentLYM;
	private double percentMON;

	public MultiChart getChart() {
		return chart;
	}

	public AbcBloodDataChart(ABCGraphData plt, ABCGraphData rbc, ABCGraphData wbc, double percentLYM, double percentMON) {

		this.plt = plt;
		this.rbc = rbc;
		this.wbc = wbc;

		this.percentLYM = percentLYM;
		this.percentMON = percentMON;

		// use MultiChart to make the one image.
		MultiChart m = new MultiChart(600, 100);

		
		// Create each chart seperately.
		if(plt != null) {
			XYChart pltChart = createPLTChart();
			m.addChart(0, 5, pltChart);
		}
		if(rbc != null) {
			XYChart rbcChart = createRBCChart();
			m.addChart(148, 5, rbcChart);
		}
		if(wbc != null) {
			XYChart wbcChart = createWBCChart();
			m.addChart(296, 5, wbcChart);
		}

		chart = m;
	}


	private XYChart createPLTChart() {

		XYChart pltChart = new XYChart(129, 95, 0xffffff, Chart.Transparent, 0);
		pltChart.setPlotArea(0, 15, 128, 60, 0xeeeeff);
		pltChart.addTitle("PLT","Arial", 8);

		pltChart.addAreaLayer(plt.getDataPoints(),0x000000);

		// 2 - 33 fl  128 channel. Add the axis labels.
		pltChart.xAxis().setLabelStyle("Serif", 7);
		float scalex = (float)128 / 33;
		double xcoord = 0;
		int[] coords = {2, 5, 10, 20, 30};
		for(Integer i : coords) {
			xcoord = (double)(i * scalex);
			pltChart.xAxis().addLabel(xcoord, i.toString());
		}
		return pltChart;
	}

	private XYChart createRBCChart() {

		XYChart rbcChart = new XYChart(129, 95, 0xffffff, Chart.Transparent, 0);
		rbcChart.setPlotArea(0, 15, 128, 60, 0xeeeeff);
		rbcChart.addTitle("RBC","Arial",8);

		rbcChart.addAreaLayer(rbc.getDataPoints(),0x000000);

		// 25 - 300 fl  256 channel. Add the axis labels.
		rbcChart.xAxis().setLabelStyle("Serif", 7);
		float scalex = (float)128 / 300;
		double xcoord = 0;
		int[] coords = {30, 100, 200};
		for(Integer i : coords) {
			xcoord = (double)(i * scalex);
			rbcChart.xAxis().addLabel(xcoord, i.toString());
		}
		return rbcChart;
	}

	private XYChart createWBCChart() {

		XYChart wbcChart = new XYChart(250, 95, 0xffffff, Chart.Transparent, 0);
		wbcChart.setPlotArea(0, 15, 128, 60, 0xeeeeff);
		wbcChart.addTitle("WBC","Arial", 8);

		wbcChart.addAreaLayer(wbc.getDataPoints(),0x000000);

		double total = totalPointCount();
		int lymPoint = getLymPoint(total); 
		int monPoint = getMonPoint(total); 


		if(lymPoint != 0 && monPoint != 0) {
			// Create the marks for seperating the zones.
			wbcChart.xAxis().addMark(lymPoint, wbcChart.dashLineColor(0xff0000, Chart.DotLine)).setDrawOnTop(false);
			wbcChart.xAxis().addMark(monPoint, wbcChart.dashLineColor(0xff0000, Chart.DotLine)).setDrawOnTop(false);

			// Create the shaded zones for.
			wbcChart.xAxis().addZone(0, lymPoint, 0xffe0e0);
			wbcChart.xAxis().addZone(lymPoint, monPoint, 0xe0ffe0);
			wbcChart.xAxis().addZone(monPoint, 128, 0xe0e0ff);
		}

		// Add a blank legend box
		LegendBox legendBox = wbcChart.addLegend(130, 15, false, "", 8);
		legendBox.setBackground(Chart.Transparent);
		// Add a key for each zone type.
		legendBox.addKey(" - Lymphocytes", 0xffe0e0);
		legendBox.addKey(" - Monocytes", 0xe0ffe0);
		legendBox.addKey(" - Granulocytes", 0xe0e0ff);

		// 30 - 460 fl  256 channel. Add the axis labels.
		wbcChart.xAxis().setLabelStyle("Serif", 7);
		float scalex = (float)128 / 460;
		double xcoord = 0;
		int[] coords = {50, 100, 200, 300, 400};
		for(Integer i : coords) {
			xcoord = (double)(i * scalex);
			wbcChart.xAxis().addLabel(xcoord, i.toString());
		}

		return wbcChart;
	}

	/** total all the point values*/
	private double totalPointCount() {
		double total = 0;
		for(int i =0; i<128; i++) {
			total += wbc.getDataPoints()[i];
		}
		return total;
	}

	private int getLymPoint(double total) {

		Double pt = (total/100) * percentLYM;
		double count = 0;

		for(int i =0; i<128; i++) {
			count += wbc.getDataPoints()[i];
			if(count>=pt) {
				return i;
			}
		}
		return 0;
	}

	private int getMonPoint(double total) {

		Double pt = (total/100) * (percentLYM + percentMON);
		int count = 0;
		for(int i =0; i<128; i++) {
			count += wbc.getDataPoints()[i];
			if(count>=pt) {
				return i;
			}
		}
		return 0;
	}

}


