/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS FloatERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.phenotyping.calorimetry;

import uk.ac.sanger.mig.common.ComponentEnum;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.Sample;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;

import java.util.Date;

import org.apache.commons.lang.time.DateUtils;

public class CalorimetryDataPoint extends AuthenticatedDataObject  {

  public enum Component implements ComponentEnum {
    ALL_COMPONENTS,
    EXPERIMENT
  }

  private Long experimentId;
  private Date sampleTime;
  private Float minutesElapsed;
  private Float co2;
  private Float o2;
  private Float refCo2;
  private Float refO2;
  private Float co2Difference;
  private Float o2Difference;
  private Float cumulativeFoodIntake;
  private Float flowRateIn;
  private Float sampleFlowRateOut;
  private Float temperature;
  private Float respiratoryExchangeRatio;
  private Integer vco2Bodyweight;
  private Integer vco2LeanMass;
  private Integer vco2Animal;
  private Integer vo2Bodyweight;
  private Integer vo2LeanMass;
  private Integer vo2Animal;
  private Float energyExpendBodyweight;
  private Float energyExpendLeanMass;
  private Float energyExpendAnimal;
  private Float heatBodyweight;
  private Float heatLeanMass;
  private Float heatAnimal;
  private Integer xaxisActivityCount;
  private Integer xaxisActivityCountTotal;
  private Integer yaxisActivityCount;
  private Integer yaxisActivityCountTotal;
  private Float rateXA;
  private Float rateXT;
  private Float rateYA;
  private Float rateYT;
  private Float rateFeed;
  private Float feedInInterval;

  private Experiment experiment;




  public CalorimetryDataPoint() {}

  /**
   * Constructor to populate this from the parser output and necesaary experiment details
   *
   * @param sample
   * @param experimentStart
   * @param experimentId
   */
  public CalorimetryDataPoint(Sample sample, Date experimentStart, Long experimentId) {
    this.experimentId = experimentId;

    this.sampleTime = sample.getDate();
    long timePointMillis = this.sampleTime.getTime() - experimentStart.getTime();
    this.minutesElapsed  = (float)timePointMillis / DateUtils.MILLIS_PER_MINUTE;

    this.co2 = sample.getCO2();
    this.o2 = sample.getO2();
    this.refCo2 = sample.getRefCO2();
    this.refO2 = sample.getRefO2();
    this.co2Difference = sample.getDO2();
    this.o2Difference = sample.getDCO2();
    this.cumulativeFoodIntake = sample.getFeed();
    this.flowRateIn = sample.getFlow();
    this.sampleFlowRateOut = sample.getFlowSamp();
    this.temperature = sample.getTemp();
    this.respiratoryExchangeRatio = sample.getRER();
    this.vco2Bodyweight = sample.getVCO2_1();
    this.vco2LeanMass = sample.getVCO2_2();
    this.vco2Animal = sample.getVCO2_3();
    this.vo2Bodyweight = sample.getVO2_1();
    this.vo2LeanMass = sample.getVO2_2();
    this.vo2Animal = sample.getVO2_3();
    this.energyExpendBodyweight = sample.getH_1();
    this.energyExpendLeanMass = sample.getH_2();
    this.energyExpendAnimal = sample.getH_3();
    this.heatBodyweight = sample.getHeatKcalPerHourPerKg();
    this.heatLeanMass = sample.getHeatKcalPerHourPerKgLeanBodyMass();
    this.heatAnimal = sample.getHeatKcalPerHour();
    this.xaxisActivityCount = sample.getXA();
    this.xaxisActivityCountTotal = sample.getXT();
    this.yaxisActivityCount = sample.getYA();
    this.yaxisActivityCountTotal = sample.getYT();
    this.rateXA = sample.getRateXA();
    this.rateXT = sample.getRateXT();
    this.rateYA = sample.getRateYA();
    this.rateYT = sample.getRateYT();
    this.rateFeed = sample.getRateFeed();
    this.feedInInterval = sample.getFeedInInterval();
  }

  public Float getCo2() {
    return co2;
  }

  public void setCo2(Float co2) {
    this.co2 = co2;
  }

  public Float getCo2Difference() {
    return co2Difference;
  }

  public void setCo2Difference(Float co2Difference) {
    this.co2Difference = co2Difference;
  }

  public Float getCumulativeFoodIntake() {
    return cumulativeFoodIntake;
  }

  public void setCumulativeFoodIntake(Float cumulativeFoodIntake) {
    this.cumulativeFoodIntake = cumulativeFoodIntake;
  }

  public Float getEnergyExpendAnimal() {
    return energyExpendAnimal;
  }

  public void setEnergyExpendAnimal(Float energyExpendAnimal) {
    this.energyExpendAnimal = energyExpendAnimal;
  }

  public Float getEnergyExpendBodyweight() {
    return energyExpendBodyweight;
  }

  public void setEnergyExpendBodyweight(Float energyExpendBodyweight) {
    this.energyExpendBodyweight = energyExpendBodyweight;
  }

  public Float getEnergyExpendLeanMass() {
    return energyExpendLeanMass;
  }

  public void setEnergyExpendLeanMass(Float energyExpendLeanMass) {
    this.energyExpendLeanMass = energyExpendLeanMass;
  }

  public Experiment getExperiment() {
    return experiment;
  }

  public void setExperiment(Experiment experiment) {
    this.experiment = experiment;
  }

  public Long getExperimentId() {
    return experimentId;
  }

  public void setExperimentId(Long experimentId) {
    this.experimentId = experimentId;
  }

  public Float getFlowRateIn() {
    return flowRateIn;
  }

  public void setFlowRateIn(Float flowRateIn) {
    this.flowRateIn = flowRateIn;
  }

  public Float getHeatAnimal() {
    return heatAnimal;
  }

  public void setHeatAnimal(Float heatAnimal) {
    this.heatAnimal = heatAnimal;
  }

  public Float getHeatBodyweight() {
    return heatBodyweight;
  }

  public void setHeatBodyweight(Float heatBodyweight) {
    this.heatBodyweight = heatBodyweight;
  }

  public Float getHeatLeanMass() {
    return heatLeanMass;
  }

  public void setHeatLeanMass(Float heatLeanMass) {
    this.heatLeanMass = heatLeanMass;
  }

  public Float getMinutesElapsed() {
    return minutesElapsed;
  }

  public void setMinutesElapsed(Float minutesElapsed) {
    this.minutesElapsed = minutesElapsed;
  }

  public Float getO2() {
    return o2;
  }

  public void setO2(Float o2) {
    this.o2 = o2;
  }

  public Float getO2Difference() {
    return o2Difference;
  }

  public void setO2Difference(Float difference) {
    o2Difference = difference;
  }

  public Float getRefCo2() {
    return refCo2;
  }

  public void setRefCo2(Float refCo2) {
    this.refCo2 = refCo2;
  }

  public Float getRefO2() {
    return refO2;
  }

  public void setRefO2(Float refO2) {
    this.refO2 = refO2;
  }

  public Float getRespiratoryExchangeRatio() {
    return respiratoryExchangeRatio;
  }

  public void setRespiratoryExchangeRatio(Float respiratoryExchangeRatio) {
    this.respiratoryExchangeRatio = respiratoryExchangeRatio;
  }

  public Float getSampleFlowRateOut() {
    return sampleFlowRateOut;
  }

  public void setSampleFlowRateOut(Float sampleFlowRateOut) {
    this.sampleFlowRateOut = sampleFlowRateOut;
  }

  public Date getSampleTime() {
    return sampleTime;
  }

  public void setSampleTime(Date sampleTime) {
    this.sampleTime = sampleTime;
  }

  public Float getTemperature() {
    return temperature;
  }

  public void setTemperature(Float temperature) {
    this.temperature = temperature;
  }

  public Integer getVco2Animal() {
    return vco2Animal;
  }

  public void setVco2Animal(Integer vco2Animal) {
    this.vco2Animal = vco2Animal;
  }

  public Integer getVco2Bodyweight() {
    return vco2Bodyweight;
  }

  public void setVco2Bodyweight(Integer vco2Bodyweight) {
    this.vco2Bodyweight = vco2Bodyweight;
  }

  public Integer getVco2LeanMass() {
    return vco2LeanMass;
  }

  public void setVco2LeanMass(Integer vco2LeanMass) {
    this.vco2LeanMass = vco2LeanMass;
  }

  public Integer getVo2Animal() {
    return vo2Animal;
  }

  public void setVo2Animal(Integer vo2Animal) {
    this.vo2Animal = vo2Animal;
  }

  public Integer getVo2Bodyweight() {
    return vo2Bodyweight;
  }

  public void setVo2Bodyweight(Integer vo2Bodyweight) {
    this.vo2Bodyweight = vo2Bodyweight;
  }

  public Integer getVo2LeanMass() {
    return vo2LeanMass;
  }

  public void setVo2LeanMass(Integer vo2LeanMass) {
    this.vo2LeanMass = vo2LeanMass;
  }

  public Integer getXaxisActivityCount() {
    return xaxisActivityCount;
  }

  public void setXaxisActivityCount(Integer xaxisActivityCount) {
    this.xaxisActivityCount = xaxisActivityCount;
  }

  public Integer getXaxisActivityCountTotal() {
    return xaxisActivityCountTotal;
  }

  public void setXaxisActivityCountTotal(Integer xaxisActivityCountTotal) {
    this.xaxisActivityCountTotal = xaxisActivityCountTotal;
  }

  public Integer getYaxisActivityCount() {
    return yaxisActivityCount;
  }

  public void setYaxisActivityCount(Integer yaxisActivityCount) {
    this.yaxisActivityCount = yaxisActivityCount;
  }

  public Integer getYaxisActivityCountTotal() {
    return yaxisActivityCountTotal;
  }

  public void setYaxisActivityCountTotal(Integer yaxisActivityCountTotal) {
    this.yaxisActivityCountTotal = yaxisActivityCountTotal;
  }

  public Float getFeedInInterval() {
    return feedInInterval;
  }

  public void setFeedInInterval(Float feedInInterval) {
    this.feedInInterval = feedInInterval;
  }

  public Float getRateFeed() {
    return rateFeed;
  }

  public void setRateFeed(Float rateFeed) {
    this.rateFeed = rateFeed;
  }

  public Float getRateXA() {
    return rateXA;
  }

  public void setRateXA(Float rateXA) {
    this.rateXA = rateXA;
  }

  public Float getRateXT() {
    return rateXT;
  }

  public void setRateXT(Float rateXT) {
    this.rateXT = rateXT;
  }

  public Float getRateYA() {
    return rateYA;
  }

  public void setRateYA(Float rateYA) {
    this.rateYA = rateYA;
  }

  public Float getRateYT() {
    return rateYT;
  }

  public void setRateYT(Float rateYT) {
    this.rateYT = rateYT;
  }
}