package uk.ac.sanger.mig.phenotyping.calorimetry.parser;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Scanner;

public class Parser {

	//Block 1 (single line): e.g.  Cohort31male129(2)301107;Exp001;TX001;TX002;TX003
    //Blank Line
	//Block 2 Box;Animal No.;Weight [g];Text1;Text2;Text3;
    //... ( may have lines of the form 6;1;1.0;Blank;Empty Run;Text03-6)
	//otherwise Animal number is required.
	//Blank line
	//Block 3 Calorimetry data proper.
	//Date;Time;Animal No.;Box;FlowSamp;Ref O2;Ref CO2;Flow;Temp;O2;CO2;dO2;dCO2;VO2(1);VO2(2);VO2(3);VCO2(1);VCO2(2);VCO2(3);RER;H(1);H(2);H(3);XT;XA;YT;YA;Feed;

	public CalorimetryData loadFromFile(String fileName) throws MalformedCalorimetryFileException, FileNotFoundException, ParseException {
		Scanner scanner = null;
    CalorimetryData data = null;

    try {
    scanner = new Scanner(new File(fileName));

		data = parse(scanner);
    }
    finally {
      if (scanner != null) scanner.close();
    }
		return data;
	}

  public CalorimetryData loadFromBytes(byte[] bytes) throws MalformedCalorimetryFileException, ParseException {
    ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes);

    Scanner scanner = null;
    CalorimetryData data = null;

    try {
      scanner = new Scanner(inputStream);
      data = parse(scanner);
    }
    finally {
      if (scanner != null) scanner.close();
    }

    return data;
  }


  public CalorimetryData parse(Scanner scanner) throws MalformedCalorimetryFileException, ParseException {
    CalorimetryData data = new CalorimetryData();

    parseFileHeader(data, scanner);

    parseBoxContentsBlock(data, scanner);

    parseCalorimetrySamples(data, scanner);

    data.interpolateTimepoints();

    return data;
  }

	/*
	 * Consume a line that is expected to be of the form
	 * Cohort31male129(2)301107;Exp001;TX001;TX002;TX003
	 * followed by a single blank line.
	 */
	private void parseFileHeader(CalorimetryData data, Scanner scanner){
		if(scanner.hasNextLine()){
			Scanner lineScanner = new Scanner(scanner.nextLine().trim());
			lineScanner.useDelimiter(";");

			data.setRunName(lineScanner.next());
			data.setExperimentId(lineScanner.next());
			data.setTextField1(lineScanner.next());
			data.setTextField2(lineScanner.next());
			data.setTextField3(lineScanner.next());

			scanner.nextLine(); // skip the blank line an leave the
			                // scanner ready for the next line.
		}
	}

	/*
	 * Consume a block of lines which are all of the form:
	 * 1;7757;28.1;NM;646.4b;Text03-1
	 * followed by a single blank line.
	 * NB. There is also a header line of the form:
	 * Box;Animal No.;Weight [g];Text1;Text2;Text3; which will
	 * be discarded. This implementation also assumes that the
	 * columns will not change in type or position.
	 */
	private void parseBoxContentsBlock(CalorimetryData data, Scanner scanner) throws MalformedCalorimetryFileException {

		Collection<BoxData> boxes = new ArrayList<BoxData>();

		int lineNo = 0;
		while(scanner.hasNextLine()){
			lineNo++;

			String line = scanner.nextLine().trim();

			if(line.length() ==0)
				break;

			// Skip the header line should be of the exact form:
			// Box;Animal No.;Weight [g];Text1;Text2;Text3;
			if(lineNo == 1){
				if(!line.equals("Box;Animal No.;Weight [g];Text1;Text2;Text3;")){
					throw new MalformedCalorimetryFileException("Box contents header absent");
				}
			} else {

				Scanner lineScanner = new Scanner(line);
				lineScanner.useDelimiter(";");

				int box = lineScanner.nextInt();
				long animalNumber = lineScanner.nextLong();
				float weight = lineScanner.nextFloat();

				String text1,text2,text3;

				if (lineScanner.hasNext()) text1 = lineScanner.next();
				else text1 = "";
				if (lineScanner.hasNext()) text2 = lineScanner.next();
				else text2 = "";
				if (lineScanner.hasNext()) text3 = lineScanner.next();
				else text3 = "";

				BoxData boxData = new BoxData();
				boxData.setBox(box);
				boxData.setAnimalNumber(animalNumber);
				boxData.setWeight(weight);
				boxData.setText1(text1);
				boxData.setText2(text2);
				boxData.setText3(text3);

				boxes.add(boxData);
			}
		}
		data.setBoxes(boxes);
	}

	private void parseCalorimetrySamples(CalorimetryData data, Scanner scanner) throws MalformedCalorimetryFileException, ParseException {

		Collection<Sample> samples = new ArrayList<Sample>();

		int lineNo = 0;
		while(scanner.hasNextLine()){
			lineNo++;

			String line = scanner.nextLine().trim();

			// Skip the header line should be of the exact form:
			// Date;Time;Animal No.;Box;FlowSamp;Ref O2;Ref CO2;Flow;Temp;O2;CO2;dO2;dCO2;VO2(1);VO2(2);VO2(3);VCO2(1);VCO2(2);VCO2(3);RER;H(1);H(2);H(3);XT;XA;YT;YA;Feed;
			if(lineNo == 1){
				if(!line.equals("Date;Time;Animal No.;Box;FlowSamp;Ref O2;Ref CO2;Flow;Temp;O2;CO2;dO2;dCO2;VO2(1);VO2(2);VO2(3);VCO2(1);VCO2(2);VCO2(3);RER;H(1);H(2);H(3);XT;XA;YT;YA;Feed;")){
					throw new MalformedCalorimetryFileException("Malformed sample header");
				}
			} else if(lineNo == 2) {
				// units line, we don't care what this is

			} else {
				Scanner lineScanner = new Scanner(line);
				lineScanner.useDelimiter(";");

				Sample sample = new Sample();

				String dateTime = lineScanner.next() + " " + lineScanner.next();
				SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yy hh:mm");
				sample.setDate(sdf.parse(dateTime));

				sample.setAnimalNo(lineScanner.nextLong());
				sample.setBox(lineScanner.nextInt());
				sample.setFlowSamp(lineScanner.nextFloat());
				sample.setRefO2(lineScanner.nextFloat());
				sample.setRefCO2(lineScanner.nextFloat());
				sample.setFlow(lineScanner.nextFloat());
				sample.setTemp(lineScanner.nextFloat());
				sample.setO2(lineScanner.nextFloat());
				sample.setCO2(lineScanner.nextFloat());
				sample.setDO2(lineScanner.nextFloat());
				sample.setDCO2(lineScanner.nextFloat());
				sample.setVO2_1(lineScanner.nextInt());
				sample.setVO2_2(lineScanner.nextInt());
				sample.setVO2_3(lineScanner.nextInt());
				sample.setVCO2_1(lineScanner.nextInt());
				sample.setVCO2_2(lineScanner.nextInt());
				sample.setVCO2_3(lineScanner.nextInt());
				sample.setRER(lineScanner.nextFloat());
				sample.setH_1(lineScanner.nextFloat());
				sample.setH_2(lineScanner.nextFloat());
				sample.setH_3(lineScanner.nextFloat());
				sample.setXT(lineScanner.nextInt());
				sample.setXA(lineScanner.nextInt());
				sample.setYT(lineScanner.nextInt());
				sample.setYA(lineScanner.nextInt());
				sample.setFeed(lineScanner.nextFloat());

				samples.add(sample);
			}
		}
		data.setSamples(samples);
	}

}
