package uk.ac.sanger.mig.phenotyping.calorimetry.parser;

import java.util.Date;

import org.apache.commons.lang.time.DateUtils;
/**
 * Represents a sample point from the metabolic cage system.
 * Descriptions of the fields from one of our scientists:
 *
 * <dl>
 * <dt>FlowSamp [l/min]</dt><dd>is the flowrate with which the sample is withdrawn from the cages</dd>
 * <dt>Ref O2  [%]</dt><dd>is the O2 levels we measure in room-air</dd>
 * <dt>Ref CO2 [%]</dt><dd>is the CO2 levels we measure in room-air</dd>
 * <dt>Flow  [l/min]</dt><dd>is the flowrate of air going into the cages, has to be greater than flowsamp so that there is some overflow from the cage</dd>
 * <dt>Temp  [C]</dt><dd>temperature in the cage</dd>
 * <dt>O2  [%]</dt><dd>is the O2 levels measured from the cage</dd>
 * <dt>CO2 [%]</dt><dd>is the O2 levels measured from the cage</dd>
 * <dt>dO2 [%]</dt><dd>delta O2 = RefO2-O2</dd>
 * <dt>dCO2  [%]</dt><dd>delta CO2 = CO2-RefCO2</dd>
 * <dt>VO2(1)  [ml/h/kg]</dt><dd>is VO2 relative to bodyweight VO2[ml/h/kg] = FlowML[ml/h] * (V1[%] + V2[%]) / (N2Ref[%] * Weight[kg] * 100[%])</dd>
 * <dt>VO2(2)  [ml/h/kg]</dt><dd>is VO2 relative to lean body mass</dd>
 * <dt>VO2(3)  [ml/h]</dt><dd>is VO2 per animal</dd>
 * <dt>VCO2(1) [ml/h/kg]</dt><dd>is VCO2 relative to bodyweight VCO2[ml/h/kg] = FlowML[ml/h] * dCO2[%] / (Weight[kg] * 100[%])</dd>
 * <dt>VCO2(2) [ml/h/kg]</dt><dd>is VCO2 relative to lean body mass</dd>
 * <dt>VCO2(3) [ml/h]</dt><dd>is VCO2 per animal</dd>
 * <dt>RER</dt><dd>respiratory exchange ratio (=VCO2/VO2) - This relates to which fuel the mouse is using (fat or carbohydrates)</dd>
 * <dt>H(1)  [kcal/h/kg]</dt><dd>energy expenditure (how much energy is the mouse using/kg) H1[kcal/h/kg] = (CVO2 * VO2[ml/h/kg] + CVCO2 * VCO2[ml/h/kg]) / 1000</dd>
 * <dt>H(2)  [kcal/h/kg]</dt><dd>energy expenditure/lean body mass</dd>
 * <dt>H(3)  [kcal/h]</dt><dd>energy expenditure/animal</dd>
 * <dt>XT  [Count]</dt><dd>number of activity counts on xaxis total (also includes fine movements=breaking the same beam repeatedly)</dd>
 * <dt>XA  [Count]</dt><dd>number of activity counts on xaxis ambulatory (when the mouse moves from breaking one beam to breaking the next)</dd>
 * <dt>YT  [Count]</dt><dd>same as for x activity but on y axis</dd>
 * <dt>YA  [Count]</dt><dd>same as for x total activity but on y axis</dd>
 * <dt>Feed [g]</dt><dd>accumulative food intake</dd>
 * </dl>
 * In addition, there are methods to get the 'heat' value. This is essentially the same as energy expenditure, but our scientists don't like the calculation used by the machine's software, so we compute values based on the calorifc value, which is in turn based on the RER
 *
 */
public class Sample implements Comparable<Sample>{
	private Date date; // Also holds the time
	private long animalNo;
	private int box;
	private float flowSamp;
	private float refO2;
	private float refCO2;
	private float flow;
	private float temp;
	private float O2;
	private float CO2;
	private float dO2;
	private float dCO2;
	private int VO2_1;
	private int VO2_2;
	private int VO2_3;
	private int VCO2_1;
	private int VCO2_2;
	private int VCO2_3;
	private float RER;
	private float H_1;
	private float H_2;
	private float H_3;
	private int XT;
	private int XA;
	private int YT;
	private int YA;
	private float feed;

  private Sample previousSample;

  public void setPreviousSample(Sample previousSample) {
    this.previousSample = previousSample;
  }

	public long getAnimalNo() {
		return animalNo;
	}
	public void setAnimalNo(long animalNo) {
		this.animalNo = animalNo;
	}
	public int getBox() {
		return box;
	}
	public void setBox(int box) {
		this.box = box;
	}
	public float getCO2() {
		return CO2;
	}
	public void setCO2(float co2) {
		CO2 = co2;
	}
	public Date getDate() {
		return  date;
	}
	public void setDate(Date date) {
		this.date = date;
	}
	public float getDCO2() {
		return dCO2;
	}
	public void setDCO2(float dco2) {
		dCO2 = dco2;
	}
	public float getDO2() {
		return dO2;
	}
	public void setDO2(float do2) {
		dO2 = do2;
	}
	public float getFeed() {
		return feed;
	}
	public void setFeed(float feed) {
		this.feed = feed;
	}
	public float getFlow() {
		return flow;
	}
	public void setFlow(float flow) {
		this.flow = flow;
	}
	public float getH_1() {
		return H_1;
	}
	public void setH_1(float h_1) {
		H_1 = h_1;
	}
	public float getH_2() {
		return H_2;
	}
	public void setH_2(float h_2) {
		H_2 = h_2;
	}
	public float getH_3() {
		return H_3;
	}
	public void setH_3(float h_3) {
		H_3 = h_3;
	}
	public float getFlowSamp() {
		return flowSamp;
	}
	public void setFlowSamp(float flowSamp) {
		this.flowSamp = flowSamp;
	}
	public float getO2() {
		return O2;
	}
	public void setO2(float o2) {
		O2 = o2;
	}
	public float getRefCO2() {
		return refCO2;
	}
	public void setRefCO2(float refCO2) {
		this.refCO2 = refCO2;
	}
	public float getRefO2() {
		return refO2;
	}
	public void setRefO2(float refO2) {
		this.refO2 = refO2;
	}
	public float getRER() {
		return RER;
	}
	public void setRER(float rer) {
		RER = rer;
	}
	public float getTemp() {
		return temp;
	}
	public void setTemp(float temp) {
		this.temp = temp;
	}
	public int getVCO2_1() {
		return VCO2_1;
	}
	public void setVCO2_1(int vco2_1) {
		VCO2_1 = vco2_1;
	}
	public int getVCO2_2() {
		return VCO2_2;
	}
	public void setVCO2_2(int vco2_2) {
		VCO2_2 = vco2_2;
	}
	public int getVCO2_3() {
		return VCO2_3;
	}
	public void setVCO2_3(int vco2_3) {
		VCO2_3 = vco2_3;
	}
	public int getVO2_1() {
		return VO2_1;
	}
	public void setVO2_1(int vo2_1) {
		VO2_1 = vo2_1;
	}
	public int getVO2_2() {
		return VO2_2;
	}
	public void setVO2_2(int vo2_2) {
		VO2_2 = vo2_2;
	}
	public int getVO2_3() {
		return VO2_3;
	}
	public void setVO2_3(int vo2_3) {
		VO2_3 = vo2_3;
	}
	public int getXA() {
		return XA;
	}
	public void setXA(int xa) {
		XA = xa;
	}
	public int getXT() {
		return XT;
	}
	public void setXT(int xt) {
		XT = xt;
	}
	public int getYA() {
		return YA;
	}
	public void setYA(int ya) {
		YA = ya;
	}
	public int getYT() {
		return YT;
	}
	public void setYT(int yt) {
		YT = yt;
	}

  public float getHeatKcalPerHour() {
    return (getCalorificValue() * getVO2_3()) / 1000;
  }

  public float getHeatKcalPerHourPerKg() {
    return (getCalorificValue() * getVO2_1()) / 1000;
  }

  public float getHeatKcalPerHourPerKgLeanBodyMass() {
    return (getCalorificValue() * getVO2_2()) / 1000;
  }

  public Float getRateXA() {
    return calculateRate(XA);
  }
  public Float getRateXT() {
    return calculateRate(XT);
  }
  public Float getRateYA() {
    return calculateRate(YA);
  }
  public Float getRateYT() {
    return calculateRate(YT);
  }

  public Float getRateFeed() {
    return calculateRate(getFeedInInterval());
  }

  public Float getFeedInInterval() {
    if (previousSample == null) {
      return feed;
    }
    else {
      return feed - previousSample.getFeed();
    }
  }
  /**
   * will be null if previous sample has not been set
   */
  protected Long getTimeElapsedSinceLastSample() {
    if (previousSample == null) {
      return null;
    }
    else {
      return date.getTime() - previousSample.getDate().getTime();
    }
  }

  protected Float getHoursElapsedSinceLastSample() {
    if (previousSample == null) {
      return null;
    }
    else {
      return getTimeElapsedSinceLastSample() / (float)DateUtils.MILLIS_PER_HOUR;
    }
  }

  protected Float calculateRate(Number measure) {
    if (previousSample == null) {
      return null;
    }
    else {
      return measure.floatValue() / getHoursElapsedSinceLastSample();
    }
  }

  /**
   * Explanation of the equation from one of our scientists:
   *
   * The 3.815 and 1.232 comes from a very old experiment (Lusk, 1928) where they analysed the oxidation of mixtures of carbohydrate and fat.
   * They are the constants of the line (y=3.815+1.232x) that you get when plotting RER (which correlates to which fuel is used) againts heat/liter
   *
   * You can see the reason for the shift in RER caused by using carbs or fat here; Oxidation of a molecule of Carbohydrate
   * 6 O2 + C6H12O6 = 6 CO2 + 6 H2O + 38 ATP
   * RER = VCO2/VO2 = 6 CO2/6 O2 = 1.0
   *
   * Oxidation of a molecule of Fatty Acid
   * 23 O2 + C16H32O2 = 16 CO2 + 16 H2O + 129 ATP RER = VCO2/VO2 = 16 CO2 / 23 O2 = 0.7
   *
   * Ref:
   * Lusk, G. The elements of the science of nutrition.
   * 4th ed, W.B. Saunders Company, Philadelphia, 1928.
   *
   * @return
   */

  public float getCalorificValue() {
    return 3.815f + (1.232f * getRER());
  }
  public int compareTo(Sample o) {
    int result = this.getDate().compareTo(o.getDate());

    if (result == 0) {
      result = ((Integer)this.getBox()).compareTo(o.getBox());
    }

    return result;
  }
}
