/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;

import org.apache.commons.collections15.FactoryUtils;
import org.apache.commons.collections15.list.LazyList;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;

/**
 * <p>This class represents an experiment performed on a mouse. Observations are made during an experiment,
 * as laid out in the standard operating procedure.</p>
 *
 * <p>It is completed once all the required observations (and any optional observations) have been made.</p>
 *
 * <p>An experiment is performed on a mouse, according to a standard operating procedure.
 * As a result observations are made.
 * It forms part of a set of experiments</p>
 *
 * @see StdOperatingProcedure
 * @see Observation
 * @see Mouse
 * @see ExperimentSet
 */
public class Experiment extends AuthenticatedDataObject {

  public enum Component implements ComponentEnum {
    ALL_COMPONENTS,
    MOUSE,
    OBSERVATIONS,
    SOP,
    PROCEDURE,
    EXPERIMENT_SET
    // add additional component enums here as needed for this
    // AuthenticatedDataObject
  }

  private Long sopId;
  private Long mouseId;
  private Long experimentSetId;
  private boolean complete;
  private StdOperatingProcedure sop;
  private Mouse mouse;
  private ExperimentSet experimentSet;
  private MouseProcedure procedure;

  /*
   * Use lazy list to make binding easier - Observations will be created
   * automatically whenever Spring tries to bind data to them - thus avoiding
   * null pointer errors
   */
  private Collection<Observation> observations = LazyList.decorate(new ArrayList<Observation>(), FactoryUtils
      .instantiateFactory(Observation.class));


  public Experiment() {}//default contructor

  /**
   * Convienience constructor, fills in details of mouse, sop
   * and creates observations based on the required observations
   * in the sop
   *
   * @param mouse
   * @param sop
   */
  public Experiment(Long mouseId, StdOperatingProcedure sop) {
    this.setMouseId(mouseId);


    this.setSopId(sop.getId());
    this.setSop(sop);

    if (sop.getRequiredObservations() == null) {
      throw new IllegalArgumentException("Cannot build experiment, we do not have the required observations");
    }

    for(RequiredObservation requiredObservation : sop.getRequiredObservations()) {
      if (requiredObservation.isActive()) {
        Observation observation = new Observation();

        observation.setRequiredObservationId(requiredObservation.getId());
        observation.setRequiredObservation(requiredObservation);

        // no experiment id until stored
        observation.setExperiment(this);

        this.getObservations().add(observation);
      }
    }
  }

  public Long getSopId() {
    return sopId;
  }

  public void setSopId(Long sopId) {
    this.sopId = sopId;
  }

  public Long getMouseId() {
    return mouseId;
  }

  public void setMouseId(Long mouseId) {
    this.mouseId = mouseId;
  }

  public boolean isComplete() {
    return complete;
  }

  public void setComplete(boolean complete) {
    this.complete = complete;
  }

  public Mouse getMouse() {
    return mouse;
  }

  public void setMouse(Mouse mouse) {
    this.mouse = mouse;
  }

  public Collection<Observation> getObservations() {
    return observations;
  }

  public void setObservations(Collection<Observation> observations) {
    this.observations = observations;
  }

  public StdOperatingProcedure getSop() {
    return sop;
  }

  public void setSop(StdOperatingProcedure sop) {
    this.sop = sop;
  }

  public ExperimentSet getExperimentSet() {
    return experimentSet;
  }

  public void setExperimentSet(ExperimentSet experimentSet) {
    this.experimentSet = experimentSet;
  }

  public Long getExperimentSetId() {
    return experimentSetId;
  }

  public void setExperimentSetId(Long experimentSetId) {
    this.experimentSetId = experimentSetId;
  }

  public MouseProcedure getProcedure() {
    return procedure;
  }

  public void setProcedure(MouseProcedure mouseProcedure) {
    this.procedure = mouseProcedure;
  }

  public Collection<Collection<Observation>> getGroupedObservations() {
    return getRequirementsAndObservations().values();
  }

  public Collection<RequiredObservation> getRequirements() {
    return getRequirementsAndObservations().keySet();
  }

  public Map<RequiredObservation, Collection<Observation>> getRequirementsAndObservations() {
    Map<RequiredObservation, Collection<Observation>> groupedObservations = new LinkedHashMap<RequiredObservation,Collection<Observation>>();

    if (observations == null)
      throw new UnattachedComponentException(Experiment.Component.OBSERVATIONS);

    for (Observation observation : observations) {
      if (observation.getRequiredObservation() == null)
        throw new UnattachedComponentException(Observation.Component.REQUIRED_OBSERVATION);

      if (!groupedObservations.containsKey(observation.getRequiredObservation())) {
        groupedObservations.put(observation.getRequiredObservation(), new LinkedList<Observation>());
      }

      groupedObservations.get(observation.getRequiredObservation()).add(observation);
    }

    return groupedObservations;
  }

}