/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture;

import java.io.Serializable;
import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.LinkedList;
import java.util.Map;
import java.util.TreeMap;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.mousetracking.Cage;

public class ExperimentSet extends AuthenticatedDataObject {

  public enum Component implements ComponentEnum {
    ALL_COMPONENTS,
    COHORT,
    EXPERIMENTS,
    PIL,
    PPL,
    SOP
  }

  private Long personalLicenceId;
  private Long projectLiceneceId;
  private Long sopId;
  private Date scheduledDate;
  private boolean complete;

  private Collection<Experiment> experiments;
  private StdOperatingProcedure sop;
  private PersonalLicence personalLicence;
  private ProjectLicence projectLicence;


  public int getBloodSampleBarcodesRequiredCount(){
    int count = 0;

    for (Experiment experiment : experiments) {
      for (Observation observation : experiment.getObservations()) {

        if (observation.getRequiredObservation().getObservationType().isBloodSampleBarcode()) {
          count++;
        }
      }
    }

    return count;
  }

  /**
   * This is a convienience method, it puts the objects into a group for display
   * There will be one group per cage.
   * If there isn't a cage, it's assumed that the mouse is dead, and a fake cage
   * is created, with an id of -1 and a barcode of 'Deceased'
   *
   * @return map
   */
  public Map<Cage, Collection<Experiment>> getExperimentsByCage() {
    Map<Cage, Collection<Experiment>> cageToExp = new TreeMap<Cage, Collection<Experiment>>(new CageBarcodeComparator());

    for (Experiment experiment : experiments) {

      Cage cage = experiment.getMouse().getCage();

      if (cage == null) {
        cage = new Cage();
        cage.setId(-1L);
        cage.setBarcode("Deceased");
        cage.setComments("Fake cage");
        experiment.getMouse().setCage(cage);
      }
      if (!cageToExp.containsKey(cage)) {
        cageToExp.put(cage, new LinkedList<Experiment>());
      }

      cageToExp.get(cage).add(experiment);
    }

    return cageToExp;
  }

  private static class CageBarcodeComparator implements Comparator<Cage>, Serializable {
    private static final long serialVersionUID = 2717038245186365979L;
    public int compare(Cage c1, Cage c2) {
      String b1 = c1.getBarcode();
      String b2 = c2.getBarcode();
      return b1.compareTo(b2);
    }
  }

  public int getExperimentCount() {
    return experiments.size();
  }

  public int getCompletedExperimentCount() {
    int count = 0;

    for (Experiment exp : experiments) {
      if (exp.isComplete()) count++;
    }

    return count;
  }

  public Collection<Experiment> getExperiments() {
    return experiments;
  }

  public StdOperatingProcedure getSop() {
    return sop;
  }

  public Long getSopId() {
    return sopId;
  }

  public boolean isComplete() {
    return complete;
  }

  public void setComplete(boolean complete) {
    this.complete = complete;
  }

  public void setExperiments(Collection<Experiment> experiments) {
    this.experiments = experiments;
  }

  public void setSop(StdOperatingProcedure sop) {
    this.sop = sop;
  }

  public void setSopId(Long sopId) {
    this.sopId = sopId;
  }

  public PersonalLicence getPersonalLicence() {
    return personalLicence;
  }

  public void setPersonalLicence(PersonalLicence personalLicence) {
    this.personalLicence = personalLicence;
  }

  public Long getPersonalLicenceId() {
    return personalLicenceId;
  }

  public void setPersonalLicenceId(Long personalLicenceId) {
    this.personalLicenceId = personalLicenceId;
  }

  public ProjectLicence getProjectLicence() {
    return projectLicence;
  }

  public void setProjectLicence(ProjectLicence projectLicence) {
    this.projectLicence = projectLicence;
  }

  public Long getProjectLicenceId() {
    return projectLiceneceId;
  }

  public void setProjectLiceneceId(Long projectLiceneceId) {
    this.projectLiceneceId = projectLiceneceId;
  }

  public Date getScheduledDate() {
    return scheduledDate;
  }

  public void setScheduledDate(Date scheduledDate) {
    this.scheduledDate = scheduledDate;
  }


}