/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.OperationNotSupported;

/**
 * Used to describe a type of data recorded in an experiment.
 *
 * The key component is the MetaType, which tells you if the data is text, or numeric, etc.
 * Validation code is written to deal with each meta type.
 *
 * Numeric types can have units, minimum values, and/or maximum values.
 *
 * Option list & radio button types contain a collection of RestrictedOptions -
 * these are the items from which a user will select a value.
 *
 * @see Observation
 * @see RestrictedOption
 */
public class ObservationType extends AuthenticatedDataObject {

	protected Log logger = LogFactory.getLog(this.getClass());

	public enum Component implements ComponentEnum {
    ACTIVE_RESTRICTED_OPTIONS,
    ALL_COMPONENTS,
    RESTRICTED_OPTIONS
	}

	/**
	 * These should match the dictionary entries for Observation Meta Type.
	 */
	public enum MetaType {
		FREE_TEXT(1L), INTEGER_NUMBER(2L), FLOATING_POINT_NUMBER(3L),
	    OPTION_LIST(4L), RADIO_BUTTON(5L), MOUSE_WEIGHT(6L),
	    DATE(7L), CLOCK_TIME(8L), DATE_TIME(9L),
	    EDTA_BLOOD_SAMPLE_BARCODE(10L), HEPARIN_BLOOD_SAMPLE_BARCODE(11L);

		private long id;

		private MetaType(long id) {
			this.id = id;
		}

		public long getId() {
			return id;
		}
	}

  private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd-MMM-yyyy");
  private static final SimpleDateFormat CLOCK_FORMAT = new SimpleDateFormat("HH:mm");
  private static final SimpleDateFormat DATE_TIME_FORMAT = new SimpleDateFormat("dd-MMM-yyyy HH:mm");

	private Long observationMetaTypeId;
	private DictionaryEntry observationMetaType;
	private String name;
	private String description;
	private Double rangeMin;
	private Double rangeMax;
	private String units;
	private boolean recordInstrument;
	private Collection<RestrictedOption> restrictedOptions;

	private boolean isSpecifiedMetaType(MetaType metaType) {
		return (observationMetaTypeId != null && observationMetaTypeId
				.equals(metaType.getId()));
	}

	public boolean isObservationValid(Observation observation) {
		boolean valid = true;

		String value = observation.getValue();
		boolean emptyObservation = (value == null || value.length() < 1);

    if (emptyObservation) {
      valid = !observation.getRequiredObservation().isMandatoryObservation();
    }
    else {
      valid = validateNonEmptyObservation(observation);
    }

		return valid;
	}

  private boolean validateNonEmptyObservation(Observation observation) {
    boolean valid = true;

    //  validate value based on it's type
    if (this.isFreeText()) {
      valid = validateTextValue(observation);
    }
    else if (this.isFloatingPointNumber()) {
      valid = validateFloatingPointValue(observation);
    }
    else if (this.isIntegerNumber()) {
      valid = validateIntegerValue(observation);
    }
    else if (this.isRestrictedOption()) {
      valid = validateOptionValue(observation);
    }
    else if (this.isDate()) {
      valid = validateDateValue(observation);
    }
    else if (this.isClockTime()) {
      valid = validateClockValue(observation);
    }
    else if (this.isDateTime()) {
      valid = validateDateTimeValue(observation);
    }
    else if (this.isBloodSampleBarcode()) {
      valid = validateBloodSampleBarcode(observation);
    }
    else {
      throw new OperationNotSupported("Attempted to validate observation with an unexpected type: " + observationMetaTypeId);
    }

    // validate other details
    if (this.isRecordInstrument() && observation.getInstrumentId() == null) {
      valid = false;
    }

    return valid;
  }


  protected boolean validateDateValue(Observation observation) {
    return validateDateFormatValue(observation,DATE_FORMAT);
  }

  protected boolean validateClockValue(Observation observation) {
    return validateDateFormatValue(observation,CLOCK_FORMAT);
  }

  protected boolean validateDateTimeValue(Observation observation) {
    return validateDateFormatValue(observation,DATE_TIME_FORMAT);
  }

  protected boolean validateDateFormatValue(Observation observation, DateFormat df) {
    boolean valid = false;
    String value = observation.getValue();

    try {
      Date date = df.parse(value);
      observation.setValue(df.format(date));
      valid = true;

    }
    catch (ParseException e) {
      valid = false;
    }

    return valid;
  }

	/**
	 * Validate an option value.
	 *
	 * Rules: value must be a valid Long integer. This integer must be the id of
	 * one of the options in restrictedOptions
	 *
	 * @param observation
	 * @return
	 */
	protected boolean validateOptionValue(Observation observation) {
		boolean valid = false;
		String value = observation.getValue();

		if (value != null) {
			try {
				Long optionId = Long.parseLong(value);
        observation.setValue(optionId.toString());

				valid = isOptionIdValid(optionId);

			} catch (NumberFormatException e) {
				valid = false;
			}
		}

		return valid;
	}

	/**
	 * Check that this observationType contains a RestrictedOption with the same
	 * id as the parameter optionId
	 *
	 * @param optionId
	 * @return
	 */
	protected boolean isOptionIdValid(long optionId) {
		boolean valid = false;

		for (RestrictedOption option : restrictedOptions) {
			if (optionId == option.getId().longValue()) {
				// found valid option, record and then exit loop
				valid = true;
				break;
			}
		}
		return valid;
	}

	/**
	 * Check that the value is a valid integer number.
	 *
	 * Rules: Valid number (uses Integer) Within bounds set by minimum and/or
	 * maximum (optional)
	 *
	 * @param observation
	 * @return
	 */
	protected boolean validateIntegerValue(Observation observation) {
		boolean valid = true;
		String value = observation.getValue();

		if (value == null) {
			valid = false;

		} else {

			try {
				Integer intValue = Integer.parseInt(value);
        observation.setValue(intValue.toString());

				if (rangeMin != null && intValue < rangeMin) {
					valid = false;
				}

				if (rangeMax != null && intValue > rangeMax) {
					valid = false;
				}

			} catch (NumberFormatException e) {
				valid = false;
			}
		}

		return valid;
	}

	/**
	 * Check that the value is a valid floating point number.
	 *
	 * Rules: Valid number (uses Double) Within bounds set by minimum and/or
	 * maximum (optional)
	 *
	 * @param observation
	 * @return
	 */
	protected boolean validateFloatingPointValue(Observation observation) {
		boolean valid = true;
    String value = observation.getValue();

		if (logger.isDebugEnabled()) {
			logger.debug("Checking "+value+" with min "+rangeMin+" and max "+rangeMax);
		}


		if (value == null) {
			valid = false;

		} else {
			try {
				Double numValue = Double.parseDouble(value);
        observation.setValue(numValue.toString());

				if (rangeMin != null && numValue < rangeMin) {
					valid = false;
				}

				if (rangeMax != null && numValue > rangeMax) {
					valid = false;
				}

			} catch (NumberFormatException e) {
				valid = false;
			}
		}

		return valid;
	}

  /**
   * Observation value should be an eight digit number, possibly including leading zeros
   * @param observation
   * @return
   */
  protected boolean validateBloodSampleBarcode(Observation observation) {
    boolean valid = true;
    final int validBarcodeLength = 8;
    String value = observation.getValue();

    if (value.length() != validBarcodeLength) {
      valid = false;
    }

    int i = 0;

    while (valid && i < value.length()) {
      String digit = value.substring(i, i+1);

      try {
        Integer.parseInt(digit);
      }
      catch (NumberFormatException e) {
        valid = false;
      }

      i++;
    }

    return valid;
  }


	/**
	 * Validate a text value
	 *
	 * Rules: Must be a non-null string. It can be empty.
	 *
	 * @param observation
	 * @return
	 */
	protected boolean validateTextValue(Observation observation) {
		return (observation.getValue() != null);
	}

	public boolean isFreeText() {
		return isSpecifiedMetaType(MetaType.FREE_TEXT);
	}

	public boolean isIntegerNumber() {
		return isSpecifiedMetaType(MetaType.INTEGER_NUMBER);
	}

	public boolean isFloatingPointNumber() {
		return isSpecifiedMetaType(MetaType.FLOATING_POINT_NUMBER) || isSpecifiedMetaType(MetaType.MOUSE_WEIGHT);
	}

  public boolean isDate() {
    return isSpecifiedMetaType(MetaType.DATE);
  }

  public boolean isDateTime() {
    return isSpecifiedMetaType(MetaType.DATE_TIME);
  }

  public boolean isClockTime() {
    return isSpecifiedMetaType(MetaType.CLOCK_TIME);
  }

	public boolean isOptionList() {
		return isSpecifiedMetaType(MetaType.OPTION_LIST);
	}

  public boolean isRadioButton() {
    return isSpecifiedMetaType(MetaType.RADIO_BUTTON);
  }

  public boolean isBloodSampleBarcode() {
    return isSpecifiedMetaType(MetaType.EDTA_BLOOD_SAMPLE_BARCODE) ||
    isSpecifiedMetaType(MetaType.HEPARIN_BLOOD_SAMPLE_BARCODE);
  }

  /**
   * Is this observation type based on restricted options?
   * @return
   */
  public boolean isRestrictedOption() {
    return (isOptionList() || isRadioButton());
  }

	public Long getObservationMetaTypeId() {
		return observationMetaTypeId;
	}

	public void setObservationMetaTypeId(Long observationMetaTypeId) {
		this.observationMetaTypeId = observationMetaTypeId;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Double getRangeMin() {
		return rangeMin;
	}

	public void setRangeMin(Double rangeMin) {
		this.rangeMin = rangeMin;
	}

	public Double getRangeMax() {
		return rangeMax;
	}

	public void setRangeMax(Double rangeMax) {
		this.rangeMax = rangeMax;
	}

	public String getUnits() {
		return units;
	}

	public void setUnits(String units) {
		this.units = units;
	}

	public boolean isRecordInstrument() {
		return recordInstrument;
	}

	public void setRecordInstrument(boolean recordInstrument) {
		this.recordInstrument = recordInstrument;
	}

	public void setObservationMetaType(DictionaryEntry observationMetaType) {
		this.observationMetaType = observationMetaType;
	}

	public DictionaryEntry getObservationMetaType() {
		return observationMetaType;
	}

	public Collection<RestrictedOption> getRestrictedOptions() {
		return restrictedOptions;
	}

	public void setRestrictedOptions(
			Collection<RestrictedOption> restrictedOptions) {
		this.restrictedOptions = restrictedOptions;
	}

  public int getNextOptionPosition() {
    int maxOrderPos = 0;

    if (restrictedOptions != null) {
      for (RestrictedOption option: restrictedOptions) {
        if (option.getOrderBy() > maxOrderPos) {
          maxOrderPos = option.getOrderBy();
        }
      }
    }

    return maxOrderPos + 1;
  }

}