/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.controllers;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.beans.propertyeditors.StringTrimmerEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSet;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.BeginPhenotypingForm;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.MouseProcedureDate;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentSetService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.StdOperatingProcedureService;

public class BeginPhenotypingController extends SimpleFormController{

  private String preferredLicenceNumber = "80/2076";
  private StdOperatingProcedureService sopService;
  private CohortService cohortService;
  private ProjectLicenceService projectLicenceService;
  private PersonalLicenceService personalLicenceService;
  private ExperimentSetService experimentSetService;
  private MouseService mouseService;

  @Override
  protected Object formBackingObject(HttpServletRequest request) throws Exception {
    BeginPhenotypingForm form = (BeginPhenotypingForm)createCommand();

    Person person = RequestUtils.getUser(request);

    if (person.getPersonalLicence() != null) {
      person.getPersonalLicence().setPerson(person);
      form.setPilId(person.getPersonalLicence().getId());
      form.setPilName(person.getPersonalLicence().getDisplayText());
      form.setPersonId(person.getId());
    }

    //Get the number of project licenses this person belongs to, and if it is
    //just one then just straight away display that. If there's more than one, use the MGP licence
    Collection<ProjectLicence> projectLicences = projectLicenceService.fetchByLicenceHolder(
        person.getId(),
        ComponentMap.of(ProjectLicence.Component.ALL_COMPONENTS)
    );

    if(projectLicences != null && projectLicences.size() == 1){
      ProjectLicence projectLicence = projectLicences.iterator().next();
      form.setPplId(projectLicence.getId());
      form.setPplName(projectLicence.getDisplayText());
    }
    else if (projectLicences != null) {

      for (ProjectLicence projectLicence : projectLicences) {
        if (projectLicence.getLicenceNumber().equals(preferredLicenceNumber)) {
          form.setPplId(projectLicence.getId());
          form.setPplName(projectLicence.getDisplayText());
        }
      }
    }

    return form;
  }

  @Override
  protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
    binder.registerCustomEditor(Long.class, new CustomNumberEditor(Long.class, true));
    binder.registerCustomEditor(Double.class, new CustomNumberEditor(Double.class, true));
    binder.registerCustomEditor(Integer.class, new CustomNumberEditor(Integer.class, true));
    binder.registerCustomEditor(Date.class, new CustomDateEditor(new SimpleDateFormat("dd-MMM-yyyy"), true));
    binder.registerCustomEditor(String.class, new StringTrimmerEditor(true)); //empty strings convert to null
  }

  @Override
  protected boolean isFormChangeRequest(HttpServletRequest request, Object command) {
    BeginPhenotypingForm form = (BeginPhenotypingForm)command;
    return form.isFormChange();
  }

  @Override
  protected void onBind(HttpServletRequest request, Object command) throws Exception {
    BeginPhenotypingForm form = (BeginPhenotypingForm)command;

    StdOperatingProcedure sop = sopService.fetchById(
        form.getSopId(),
        ComponentMap.putAll(
        new ComponentMap(
            StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS,
            ComponentMap.of(RequiredObservation.Component.OBSERVATION_TYPE)
        ),
        ComponentMap.of(StdOperatingProcedure.Component.PROCEDURE_ENTRY)
        )
    );
    form.setSop(sop);

    ProjectLicence ppl = projectLicenceService.fetchById(form.getPplId(), ComponentMap.of(ProjectLicence.Component.ALL_COMPONENTS));
    form.setProjectLicence(ppl);

    PersonalLicence pil = personalLicenceService.fetchById(form.getPilId(), ComponentMap.of(PersonalLicence.Component.ALL_COMPONENTS));
    form.setPersonalLicence(pil);

    form.setCohort(cohortService.fetchById(form.getCohortId(), null));

    for (MouseProcedureDate mpd : form.getMiceInCohort()) {
       Long mouseId = mpd.getMouse().getId();
      mpd.setMouse(mouseService.fetchById(mouseId, null));
    }
  }

  @Override
  protected void onFormChange(HttpServletRequest request, HttpServletResponse response, Object command) throws Exception {
    BeginPhenotypingForm form = (BeginPhenotypingForm)command;
    form.setFormChange(false);

    if (form.getCohortId() != null && form.getCohortId() > 0) {
      ComponentMap cohortMap = new ComponentMap();
      cohortMap.put(Cohort.Component.MICE, ComponentMap.of(Mouse.Component.ALL_COMPONENTS));

      Cohort cohort = cohortService.fetchById(form.getCohortId(), cohortMap);
      form.setCohort(cohort);
      form.createMouseProcedureDateEntries(cohort.getMice());
    }
  }



  @Override
  protected ModelAndView onSubmit(HttpServletRequest request,
      HttpServletResponse response, Object command, BindException errors)
      throws Exception {
    BeginPhenotypingForm form = (BeginPhenotypingForm)command;

    Person user = RequestUtils.getUser(request);

    ExperimentSet experimentSet = experimentSetService.createExperimentSet(form.getInitialisationInfo());
    experimentSetService.storeWithExperimentsAndObservations(experimentSet, user);

    // redirect to the interface for entering data
    return new ModelAndView(getSuccessView() + experimentSet.getId());
  }

  public void setCohortService(CohortService cohortService) {
    this.cohortService = cohortService;
  }


  public void setExperimentSetService(ExperimentSetService experimentSetService) {
    this.experimentSetService = experimentSetService;
  }

  public void setMouseService(MouseService mouseService) {
    this.mouseService = mouseService;
  }

  public void setPersonalLicenceService(PersonalLicenceService personalLicenceService) {
    this.personalLicenceService = personalLicenceService;
  }

  public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
    this.projectLicenceService = projectLicenceService;
  }

  public void setSopService(StdOperatingProcedureService sopService) {
    this.sopService = sopService;
  }



}