/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.controllers;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.beans.propertyeditors.StringTrimmerEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.StdOperatingProcedureForm;
import uk.ac.sanger.mig.phenotyping.datacapture.services.StdOperatingProcedureService;

public class StdOperatingProcedureFormController extends AbstractCancellableFormController {
	
	private StdOperatingProcedureService sopService;
	private DictionaryService dictionaryService;
  private BinaryFileService binaryFileService;
	
	public void setSopService(StdOperatingProcedureService sopService) {
		this.sopService = sopService;
	}
	
	public void setDictionaryService (DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setBinaryFileService(BinaryFileService binaryFileService) {
    this.binaryFileService = binaryFileService;
  }

  @Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		binder.registerCustomEditor(Long.class, new CustomNumberEditor(Long.class, true));
		binder.registerCustomEditor(Double.class, new CustomNumberEditor(Integer.class, true));
		binder.registerCustomEditor(Integer.class, new CustomNumberEditor(Double.class, true));
		binder.registerCustomEditor(Date.class, new CustomDateEditor(new SimpleDateFormat("dd-MMM-yyyy"), true));
		binder.registerCustomEditor(String.class, new StringTrimmerEditor(true));
	}

	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		StdOperatingProcedureForm form = (StdOperatingProcedureForm)createCommand();				    
		Long id = ServletRequestUtils.getLongParameter(httpServletRequest, "id");		
		
		StdOperatingProcedure sop = null; 
		        
		//REM always ensure our *full* object graph (all sub-objects) available for binding!
					        
		if(id != null) {
			//ok, existing stdOperatingProcedure id provided, so fetch for edit. NOTE: ALL_COMPONENTS not obligatory
			//fetch component map that you need... 
		  sop = sopService.fetchById(id,
		      ComponentMap.putAll(
		          new ComponentMap(
		              StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS_FOR_EDIT,
		              new ComponentMap(
		                  RequiredObservation.Component.OBSERVATION_TYPE,
		                  ComponentMap.of(ObservationType.Component.RESTRICTED_OPTIONS)
		              )
		          ),
		          ComponentMap.of(StdOperatingProcedure.Component.DOCUMENTATION)
		      ));
		
		} else {//else make new stdOperatingProcedure	
			sop = buildNewStdOperatingProcedure();
		}
   
		form.setStdOperatingProcedure(sop);
    
    if (sop.getId() != null) {
      Collection<BinaryFile> files = binaryFileService.fetchByTableRef(TableMetadataEnum.PHN_STD_OPERATING_PROCEDURE, sop.getId(), false, null);      
      
      if (files.size() > 0){
        form.setBinaryFile(files.iterator().next());
      }
    }
    
    if (form.getBinaryFile() == null) {
      BinaryFile file = new BinaryFile();
      form.setBinaryFile(file);
      
      file.setFileDescription("SOP Documentation");
      
      file.setForeignTableName(TableMetadataEnum.PHN_STD_OPERATING_PROCEDURE);
      file.setForeignKeyId(sop.getId());
    }

		return form;
	}
		
	//called after databinding, before showForm(), merges Map into ModelAndView
	//to populate our form with form "metadata" -- eg, drop-down select lists, etc.
		@Override
	protected Map referenceData(HttpServletRequest httpServletRequest) throws Exception {
		Map<String,Object> formMetaData = new HashMap<String,Object>();				
			
		formMetaData.put("procedures",dictionaryService.fetch(DictionaryEnum.MOUSE_PROCEDURE_DICT));
			
		return formMetaData;
	} 
	
	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, 
				Object command, BindException errors) throws Exception {
		StdOperatingProcedureForm stdOperatingProcedureForm = (StdOperatingProcedureForm) command;
		
		StdOperatingProcedure stdOperatingProcedure = stdOperatingProcedureForm.getStdOperatingProcedure();		
		sopService.storeWithRequiredObservations(stdOperatingProcedure, RequestUtils.getUser(request));
		
    if (stdOperatingProcedureForm.getBinaryFile().getSize() > 0 ) {
      stdOperatingProcedureForm.getBinaryFile().setForeignKeyId(stdOperatingProcedure.getId());
      binaryFileService.store(stdOperatingProcedureForm.getBinaryFile(), RequestUtils.getUser(request));
    }
    
		return new ModelAndView(getSuccessView());
	}
	
	private StdOperatingProcedure buildNewStdOperatingProcedure() {		
		StdOperatingProcedure stdOperatingProcedure = new StdOperatingProcedure();
		    
		stdOperatingProcedure.setProcedureId(0L);	
		    
		return stdOperatingProcedure;
	}	
}