/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.dao;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSearchBean;

public class ExperimentDaoSpringImpl extends AbstractDaoSpringImpl<Experiment> implements ExperimentDao {

	private PaginatedSearchUtils<Experiment>	paginator;


	public void setPaginator(PaginatedSearchUtils<Experiment> paginator) {
		this.paginator = paginator;
	}


	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Experiment experiment) {
		sql.addLongParam(experiment.getSopId(), "SOP_ID");
		sql.addLongParam(experiment.getExperimentSetId(), "EXPERIMENT_SET_ID");
		sql.addLongParam(experiment.getMouseId(), "MOUSE_ID");
		sql.addBooleanParam(experiment.isComplete(), "IS_COMPLETE");
		sql.addDateParam(experiment.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(experiment.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(experiment.getEditDate(), "EDIT_DATE");
		sql.addStringParam(experiment.getEditedBy(), "EDITED_BY");
		sql.addLongParam(experiment.getCheckNumber(), "CHECK_NUMBER");
		sql.addLongParam(experiment.getId(), "ID");
	}


	public Collection<Experiment> search(ExperimentSearchBean searchBean) {
		StringBuilder sql = new StringBuilder("select * from PHN_EXPERIMENT where 1=1 ");
		Collection<Object> bindVariables = new ArrayList<Object>();
		Experiment experiment = searchBean.getExperiment();

		if (experiment.getSopId() != null && experiment.getSopId() > 0) {
			bindVariables.add(experiment.getSopId());
			sql.append(" and sop_id = ?");
		}

		if (experiment.getMouseId() != null) {
			bindVariables.add(experiment.getMouseId());
			sql.append(" and mouse_id = ?");
		}

		if (searchBean.getComplete() != null) {
			bindVariables.add(searchBean.getComplete());
			sql.append(" and is_complete = ?");
		}

		if (searchBean.getOrderBy() != null && searchBean.getOrderBy().length() > 0) {
			sql.append(" order by " + searchBean.getOrderBy());
			if (searchBean.isOrderDesc()) {
				sql.append(" desc ");
			}
		} else {
			sql.append(" order by id");
		}
		return paginator.fetchSearchResults(sql.toString(), bindVariables, searchBean, getRowMapper());
	}


	public Collection<Experiment> fetchByExperimentSetId(Long experimentSetId) {
		String sql = "select e.* from phn_experiment e join mts_mouse m on e.mouse_id = m.id where e.experiment_set_id = ? order by m.name";

		Object[] bindVariables = { experimentSetId };

		Collection<Experiment> experiments = getSimpleJdbcTemplate().query(sql, getRowMapper(), bindVariables);

		return experiments;
	}


	/**
	 * Finds all experiments associated with the passed mouse id and sop id and orders them by
	 * creation date.
	 *
	 * @param mouseId -
	 *            the mouse id to search for.
	 * @param sopId -
	 *            the standard operation procedure id to search for.
	 * @return experiments - collection of experiments.
	 */
	public Collection<Experiment> fetchByMouseIdAndSopId(Long mouseId, Long sopId) {
		String sql = " select * from phn_experiment where mouse_id = ? and sop_id = ? order by created_date ";
		Collection<Experiment> experiments = getSimpleJdbcTemplate().query(sql, getRowMapper(), mouseId, sopId);
		return experiments.isEmpty() ? new ArrayList<Experiment>() : experiments;
	}


	/**
	 * Returns all of the ABR experiments. Finds the experiments using the FREQ_EXPERIMENT table.
	 *
	 * @return collection of all ABR experiments.
	 */
	public Collection<Experiment> fetchAllAbrExperiments() {
		String sql = "select e.* from phn_experiment e where e.id in ("
				+ "select distinct phn_experiment_id from aud_freq_experiment)" + " order by e.id";
		Collection<Experiment> experiments = fetchCollection(sql);
		return experiments.isEmpty() ? new ArrayList<Experiment>() : experiments;
	}
}