/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.dao;

import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.object.StoredProcedure;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSet;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSetSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.rowmappers.ExperimentSetListViewRowMapper;
import uk.ac.sanger.mig.phenotyping.datacapture.views.ExperimentSetListView;

public class ExperimentSetDaoSpringImpl extends AbstractDaoSpringImpl<ExperimentSet> implements ExperimentSetDao {

  private PaginatedSearchUtils<ExperimentSetListView> viewPaginator;
  private ExperimentSetListViewRowMapper experimentSetListViewRowMapper;

  public void setExperimentSetListViewRowMapper(ExperimentSetListViewRowMapper experimentSetListViewRowMapper) {
    this.experimentSetListViewRowMapper = experimentSetListViewRowMapper;
  }

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, ExperimentSet experimentSet) {
    sql.addDateParam(experimentSet.getScheduledDate(), "SCHEDULED_DATE");
	  sql.addLongParam(experimentSet.getSopId(),"SOP_ID");
		sql.addBooleanParam(experimentSet.isComplete(), "IS_COMPLETE");
		sql.addDateParam(experimentSet.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(experimentSet.getCreatorId(), "CREATOR_ID");
		sql.addDateParam(experimentSet.getEditDate(), "EDIT_DATE");
		sql.addStringParam(experimentSet.getEditedBy(), "EDITED_BY");
		sql.addLongParam(experimentSet.getCheckNumber(), "CHECK_NUMBER");
    sql.addLongParam(experimentSet.getPersonalLicenceId(), "PIL_ID");
    sql.addLongParam(experimentSet.getProjectLicenceId(), "PPL_ID");
		sql.addLongParam(experimentSet.getId(), "ID");
	}

  public Collection<ExperimentSetListView> search(ExperimentSetSearchBean searchBean) {
    Collection<Object> bindVariables = new ArrayList<Object>();
    ExperimentSet expSet = searchBean.getExperimentSet();

    StringBuilder sql = new StringBuilder("select * from PHN_EXPERIMENT_SET_VIEW where 1=1 ");

    if (expSet.getSop() != null &&
        expSet.getSop().getProcedureId() != null &&
        expSet.getSop().getProcedureId() > 0L) {
      sql.append("and procedure_id = ? ");
      bindVariables.add(expSet.getSop().getProcedureId());
    }

    if (expSet.getSopId() != null) {
      sql.append("and sop_id = ? ");
      bindVariables.add(expSet.getSopId());
    }

    if (searchBean.getCompleted() != null) {
      sql.append("and is_complete = ? ");
      int completed = searchBean.getCompleted().booleanValue() ? 1 : 0;
      bindVariables.add(completed);
    }

    if (searchBean.getCohortId() != null) {
      sql.append("and id in (select e.experiment_set_id ");
      sql.append("from phn_experiment e ");
      sql.append("join mts_cohort_mouse cm on cm.mouse_id = e.mouse_id ");
      sql.append("where cm.cohort_id = ?) ");
      bindVariables.add(searchBean.getCohortId());
    }

    if (expSet.getId() != null) {
    	sql.append("and id = ? ");
    	bindVariables.add(expSet.getId());
    }

    if (searchBean.getOrderBy() != null
        && searchBean.getOrderBy().length() > 0) {
      if (searchBean.getOrderBy().equals("COMPLETION_PCT")) {
        sql.append(" order by COMP_EXP_COUNT / EXP_COUNT");
      }
      else if (searchBean.getOrderBy().equals("COHORT_NAMES")) {
      	sql.append(" order by upper(trim(COHORT_NAMES))");
      }
      else {
        sql.append(" order by " + searchBean.getOrderBy());
      }

      if (searchBean.isOrderDesc()) {
        sql.append(" desc ");
      }
    } else {
      sql.append("order by id");
    }

    return viewPaginator.fetchSearchResults(sql.toString(), bindVariables,
        searchBean, experimentSetListViewRowMapper);
  }

  public void setViewPaginator(PaginatedSearchUtils<ExperimentSetListView> viewPaginator) {
    this.viewPaginator = viewPaginator;
  }

  public void fullDelete(ExperimentSet es, Person user) {
    DeleteExperimentSetStoredProcedure sp = new DeleteExperimentSetStoredProcedure(getDataSource());
    sp.delete(es.getId());
  }

   private static class DeleteExperimentSetStoredProcedure extends StoredProcedure {

    public DeleteExperimentSetStoredProcedure(DataSource dataSource) {
      setDataSource(dataSource);
      setSql("phn.phn_delete_experiment_set");
      declareParameter(new SqlParameter("in_set_id", Types.NUMERIC));
      compile();
    }

    public void delete(Long experimentSetId) {
      Map<String,Object> inputParameters = new HashMap<String,Object>();
      inputParameters.put("in_set_id", experimentSetId);
      execute(inputParameters);
    }
  }
}