/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.forms;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSet;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.UploadMapping;
import uk.ac.sanger.mig.phenotyping.datacapture.services.UploadMappingService.Fields;

/**
 *
 */
public class CalorimetryForm {

  public static class CalorimetryRow implements Comparable<CalorimetryRow>{
    private Experiment experiment;
    private Observation boxNumber;
    private Observation weightBefore;
    private Observation weightAfter;
    private Observation badFoodData;
    private Observation waterBottleWeightBefore;
    private Observation waterBottleWeightAfter;

    public CalorimetryRow(
    		Experiment e, Observation boxNumber, Observation weightBefore, Observation weightAfter,
    		Observation badFoodData, Observation waterBottleWeightBefore, Observation waterBottleWeightAfter) {
      this.experiment = e;
      this.boxNumber = boxNumber;
      this.weightBefore = weightBefore;
      this.weightAfter = weightAfter;
      this.badFoodData = badFoodData;
      this.waterBottleWeightBefore = waterBottleWeightBefore;
      this.waterBottleWeightAfter = waterBottleWeightAfter;
    }

    public Observation getBoxNumber() {
      return boxNumber;
    }
    public Mouse getMouse() {
      return experiment.getMouse();
    }
    public Observation getWeightAfter() {
      return weightAfter;
    }
    public Observation getWeightBefore() {
      return weightBefore;
    }

    public Experiment getExperiment() {
      return experiment;
    }

    public Observation getBadFoodData() {
      return badFoodData;
    }

    public Observation getWaterBottleWeightAfter() {
			return waterBottleWeightAfter;
		}

		public Observation getWaterBottleWeightBefore() {
			return waterBottleWeightBefore;
		}

    public int compareTo(CalorimetryRow o) {
      Integer thisBoxNumber = Integer.valueOf(this.getBoxNumber().getValue());
      Integer otherBoxNumber = Integer.valueOf(o.getBoxNumber().getValue());
      int result = thisBoxNumber.compareTo(otherBoxNumber);

      return result;
    }
  }

  private List<CalorimetryRow> rows = new LinkedList<CalorimetryRow>();
  private List<ExperimentSet> sets = new LinkedList<ExperimentSet>();

  public List<ExperimentSet> getSets() {
    return sets;
  }
  public List<CalorimetryRow> getRows() {
    return rows;
  }

  public CalorimetryForm(){}
  public CalorimetryForm(
      Collection<ExperimentSet> sets,
      Map<Long,Collection<UploadMapping>> mappings){

    this.sets.addAll(sets);

    // find the relevant information for each experiment
    for (ExperimentSet es : sets) {
      Long boxNumberId = null;
      Long weightBeforeId = null;
      Long weightAfterId = null;
      Long badFoodDataId = null;
      Long waterBottleBeforeId = null;
      Long waterBottleAfterId = null;

      Collection<UploadMapping> ums = mappings.get(es.getSopId());

      // find the mappings we need
      for (UploadMapping um : ums) {
        if (um.getFieldName().equals(Fields.BOX_NUMBER.toString())) {
          boxNumberId = um.getRequiredObservationId();
        }
        if (um.getFieldName().equals(Fields.WEIGHT_BEFORE.toString())) {
          weightBeforeId = um.getRequiredObservationId();
        }
        if (um.getFieldName().equals(Fields.WEIGHT_AFTER.toString())) {
          weightAfterId = um.getRequiredObservationId();
        }
        if (um.getFieldName().equals(Fields.BAD_FOOD_DATA.toString())) {
          badFoodDataId = um.getRequiredObservationId();
        }
        if (um.getFieldName().equals(Fields.WATER_BOTTLE_BEFORE.toString())) {
        	waterBottleBeforeId = um.getRequiredObservationId();
        }
        if (um.getFieldName().equals(Fields.WATER_BOTTLE_AFTER.toString())) {
        	waterBottleAfterId = um.getRequiredObservationId();
        }
      }


      for (Experiment e : es.getExperiments()) {
        Observation boxNumber = null;
        Observation weightAfter = null;
        Observation weightBefore = null;
        Observation badFoodData = null;
        Observation waterBottleBefore = null;
        Observation waterBottleAfter = null;

        for (Observation o : e.getObservations()) {
          Long roId = o.getRequiredObservationId();

            if (roId.equals(boxNumberId)) {
              boxNumber = o;
            }
            else if (roId.equals(weightBeforeId)) {
              weightBefore = o;
            }
            else if (roId.equals(weightAfterId)) {
              weightAfter = o;
            }
            else if (roId.equals(badFoodDataId)) {
              badFoodData = o;
            }
            else if (roId.equals(waterBottleBeforeId)) {
            	waterBottleBefore = o;
            }
            else if (roId.equals(waterBottleAfterId)) {
            	waterBottleAfter = o;
            }

        }

        CalorimetryRow row = new CalorimetryRow(
            e,
            boxNumber,
            weightBefore,
            weightAfter,
            badFoodData,
            waterBottleBefore,
            waterBottleAfter
        );

        rows.add(row);
      }
    }

    // sort the rows by box number
    Collections.sort(rows);
  }

  public Collection<Long> getCageIds() {
    Collection<Long> cageIds = new TreeSet<Long>();

    for (CalorimetryRow row : rows) {
      if (row.getMouse().getCageId() != null) {
        cageIds.add(row.getMouse().getCageId());
      }
    }

    return cageIds;
  }

}
