/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Collection;
import java.util.Date;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSearchBean;

public interface ExperimentService {

	/**
	 * Store experiments and all associated observations.
	 *
	 * Record a procedure when creating a new experiment.
	 *
	 * @param experiments
	 * @param user
	 */
	public void storeWithResults(Collection<Experiment> experiments, Person user);


	/**
	 * Stores the experiment and all associated observations.
	 *
	 * @param experiment -
	 *            the experiment.
	 * @param user -
	 *            the logged in user.
	 */
	public void storeWithResults(Experiment experiment, Person user);


	/**
	 * Stores Experiment as record in database and returns the record's primary key
	 *
	 * @param experiment
	 * @return
	 */
	public Long storeAuthenticated(Experiment experiment, Person person);


	/**
	 * Returns experiment for a given id that maps to primary key of a record in a database table.
	 *
	 * @param id
	 * @return
	 */
	public Experiment fetchById(Long id, ComponentMap componentMap);


	/**
	 * Fetch all experiments for an ExperimentSet
	 *
	 * @param experimentSetId
	 * @param componentMap
	 * @return
	 */
	public Collection<Experiment> fetchByExperimentSetId(Long experimentSetId, ComponentMap componentMap);


	/**
	 * Finds all experiments for the passed mouse id and sop id. Results are ordered by creation
	 * date.
	 *
	 * @param mouseId -
	 *            the mouse id to find.
	 * @param sopId -
	 *            the sop id to find.
	 * @param requiredComponents -
	 *            required components to be attached.
	 * @return experiments - collection of found experiments or null if none found.
	 */
	public Collection<Experiment> fetchByMouseIdAndSopId(Long mouseId, Long sopId, ComponentMap requiredComponents);


	/**
	 * Returns all of the ABR experiments.
	 *
	 * @param requiredComponents -
	 *            components to attach.
	 * @return collection of all ABR experiments.
	 */
	public Collection<Experiment> fetchAllAbrExperiments(ComponentMap requiredComponents);


	/**
	 * Takes search bean and returns a Collection of Experiment objects based on set fields in bean
	 * *and* (this is important) your implementation of search method in Dao layer -- eg, are you
	 * testing for equality? Greater than/less than, etc?
	 *
	 * @param experimentSearchBean
	 * @return
	 */
	public Collection<Experiment> search(ExperimentSearchBean experimentSearchBean, ComponentMap requiredComponents);


	/**
	 * Set whether or not an experiment has been completed
	 *
	 * @param experimentId
	 * @param completed
	 * @param user
	 * @return A pair containing the experiment and whether the requested change was successful
	 */
	public Pair<Experiment, Boolean> changeExperimentComplete(Long experimentId, boolean completed, Person user);


	/**
	 * Delete the experiment, but not the experiments it contains.
	 *
	 * @param e
	 * @param user
	 */
	public void delete(Experiment e, Person user);


	/**
	 * Change the start point for an existing experiment
	 *
	 * @param experimentId
	 * @param start
	 * @param user
	 * @return
	 */
	public Experiment changeExperimentStart(Long experimentId, Date start, Person user);


	/**
	 *
	 */
	public MouseProcedure recordProcedure(Experiment e, Date start, Date end, Person user);


	/**
	 * If an experiment hasn't been started, and the mouse is still alive, record a procedure for it
	 *
	 * @param experimentId
	 * @param user
	 * @return
	 */
	public Experiment startExperiment(Long experimentId, Person user);


	/**
	 * Remove the procedure record for this experiment and make sure the experiment is marked as not
	 * being completed
	 *
	 * @param experimentId
	 * @param user
	 * @return
	 */
	public Experiment removeProcedure(Long experimentId, Person user);
}