/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Collection;
import java.util.Date;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.licensing.Protocol;
import uk.ac.sanger.mig.licensing.services.ProtocolService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAge;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.ExperimentDao;

public class ExperimentServiceImpl extends AbstractServiceImpl<Experiment> implements ExperimentService {

	private ExperimentDao					experimentDao;
	private ProtocolService					protocolService;
	private MouseProcedureService			mouseProcedureService;
	private MouseService					mouseService;
	private ObservationService				observationService;
	private StdOperatingProcedureService	sopService;
	private ExperimentSetService			experimentSetService;


	public void setSopService(StdOperatingProcedureService sopService) {
		this.sopService = sopService;
	}


	public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
		this.mouseProcedureService = mouseProcedureService;
	}


	public void setProtocolService(ProtocolService protocolService) {
		this.protocolService = protocolService;
	}


	public void setObservationService(ObservationService observationService) {
		this.observationService = observationService;
	}


	// IMPORTANT: add other dependencies here: Should be other services *not*
	// dao typically
	/**
	 * eg private ExperimentDao experimentDao; private MouseService mouseService private
	 * PersonService personService;
	 */

	public void setExperimentDao(ExperimentDao experimentDao) {
		this.experimentDao = experimentDao;
	}


	@Transactional
	public Long storeAuthenticated(Experiment experiment, Person user) {
		if (experiment == null) {
			throw new RuntimeException("Cannot save a null experiment");
		}

		return experimentDao.store(experiment, user);
	}


	public Experiment fetchById(Long experimentId, ComponentMap requiredComponents) {
		Experiment experiment = experimentDao.fetchById(experimentId);

		if (experiment != null)
			attachComponents(experiment, requiredComponents);

		return experiment;
	}


	/**
	 * Finds all experiments for the passed mouse id and sop id. Results are ordered by creation
	 * date.
	 *
	 * @param mouseId -
	 *            the mouse id to find.
	 * @param sopId -
	 *            the sop id to find.
	 * @param requiredComponents -
	 *            required components to be attached.
	 * @return experiments - collection of found experiments or null if none found.
	 */
	public Collection<Experiment> fetchByMouseIdAndSopId(Long mouseId, Long sopId, ComponentMap requiredComponents) {
		Collection<Experiment> experiments = experimentDao.fetchByMouseIdAndSopId(mouseId, sopId);
		if (!experiments.isEmpty())
			attachComponents(experiments, requiredComponents);
		return experiments;
	}


	/**
	 * Returns all of the ABR experiments.
	 *
	 * @param requiredComponents -
	 *            components to attach.
	 * @return collection of all ABR experiments.
	 */
	public Collection<Experiment> fetchAllAbrExperiments(ComponentMap requiredComponents) {
		Collection<Experiment> experiments = experimentDao.fetchAllAbrExperiments();
		if (!experiments.isEmpty())
			attachComponents(experiments, requiredComponents);
		return experiments;
	}


	@Override
	protected void attachComponents(Experiment experiment, ComponentMap requiredComponents) {
		if (experiment == null) {
			throw new RuntimeException("Cannot attach components to null experiment");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(Experiment.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(Experiment.Component.MOUSE)) {
				experiment.setMouse(mouseService.fetchById(experiment.getMouseId(), requiredComponents
						.get(Experiment.Component.MOUSE)));
			}

			if (fetchAll || requiredComponents.containsKey(Experiment.Component.OBSERVATIONS)) {
				experiment.setObservations(observationService.fetchByExperimentId(experiment.getId(),
						requiredComponents.get(Experiment.Component.OBSERVATIONS)));
			}

			if (fetchAll || requiredComponents.containsKey(Experiment.Component.SOP)) {
				experiment.setSop(sopService.fetchById(experiment.getSopId(), requiredComponents
						.get(Experiment.Component.SOP)));
			}

			if (fetchAll || requiredComponents.containsKey(Experiment.Component.PROCEDURE)) {
				experiment.setProcedure(mouseProcedureService.fetchByTableRef(TableMetadataEnum.PHN_EXPERIMENT,
						experiment.getId(), requiredComponents.get(Experiment.Component.PROCEDURE)));
			}

			if (fetchAll || requiredComponents.containsKey(Experiment.Component.EXPERIMENT_SET)) {
				experiment.setExperimentSet(experimentSetService.fetchById(experiment.getExperimentSetId(),
						requiredComponents.get(Experiment.Component.EXPERIMENT_SET)));
			}
		}
	}


	public Collection<Experiment> search(ExperimentSearchBean experimentSearchBean, ComponentMap requiredComponents) {
		Collection<Experiment> experiments = experimentDao.search(experimentSearchBean);

		attachComponents(experiments, requiredComponents);

		return experiments;
	}


	@Transactional
	public void storeWithResults(Collection<Experiment> experiments, Person user) {
		for (Experiment experiment : experiments) {
			storeWithResults(experiment, user);
		}

	}


	@Transactional
	public void storeWithResults(Experiment experiment, Person user) {
		Long experimentId = storeAuthenticated(experiment, user);
		if (!experimentId.equals(experiment.getId()))
			throw new RuntimeException("Experiment id returned does not match experiment.id");
		storeObservations(experiment, user);
	}


	@Transactional
	public Experiment changeExperimentStart(Long experimentId, Date start, Person user) {

		if (experimentId == null || start == null || user == null) {
			throw new IllegalArgumentException("Cannot change experiment start with null parameter");
		}

		Experiment e = fetchById(experimentId, ComponentMap.of(Experiment.Component.MOUSE,
				Experiment.Component.PROCEDURE, Experiment.Component.SOP, Experiment.Component.EXPERIMENT_SET));

		if (e == null) {
			return null;
		}

		if (e.getProcedure() == null) {
			createNewProcedure(e);
		}

		MouseProcedure procedure = e.getProcedure();
		procedure.setStartDate(start);
		mouseProcedureService.store(procedure, user);

		return e;
	}


	private void createNewProcedure(Experiment e) {
		MouseProcedure proc = new MouseProcedure();
		e.setProcedure(proc);
		proc.setMouseId(e.getMouseId());
		proc.setForeignTableName(TableMetadataEnum.PHN_EXPERIMENT);

		Protocol protocol = protocolService.fetchByProcedureAndProject(e.getSop().getProcedureId(), e
				.getExperimentSet().getProjectLicenceId(), e.getMouse().isHarmfulMutant(), null);

		if (protocol == null) {
			throw new IllegalArgumentException("Could not retrieve protocol for experiment " + e.getId());
		}

		proc.setProtocolId(protocol.getId());
		proc.setPersonalLicenceId(e.getExperimentSet().getPersonalLicenceId());
		proc.setProcedureDictId(e.getSop().getProcedureId());
		proc.setForeignKeyId(e.getId());
		proc.setForeignTableName(TableMetadataEnum.PHN_EXPERIMENT);
	}


	private void storeObservations(Experiment experiment, Person user) {
		for (Observation observation : experiment.getObservations()) {
			observation.setExperimentId(experiment.getId());
			observationService.storeAuthenticated(observation, user);
		}
	}


	public Collection<Experiment> fetchByExperimentSetId(Long experimentSetId, ComponentMap componentMap) {
		Collection<Experiment> experiments = experimentDao.fetchByExperimentSetId(experimentSetId);
		attachComponents(experiments, componentMap);
		return experiments;
	}


	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}


	/**
	 * A request to set the complete status of an experiment
	 *
	 * Returns the experiment modified and a boolean to indicate if the request was fulfilled
	 *
	 * If the experiment is terminal and is being set to be complete, the mouse will be marked as
	 * deceased
	 *
	 * @param experimentId
	 * @param completed
	 * @param user
	 * @return
	 */
	@Transactional
	public Pair<Experiment, Boolean> changeExperimentComplete(Long experimentId, boolean completed, Person user) {
		ComponentMap componentMap = new ComponentMap();
		componentMap.put(Experiment.Component.OBSERVATIONS, new ComponentMap(
				Observation.Component.REQUIRED_OBSERVATION, new ComponentMap(
						RequiredObservation.Component.OBSERVATION_TYPE, ComponentMap
								.of(ObservationType.Component.ACTIVE_RESTRICTED_OPTIONS))));
		componentMap.put(Experiment.Component.PROCEDURE, null);
		componentMap.put(Experiment.Component.SOP, ComponentMap.of(StdOperatingProcedure.Component.PROCEDURE_ENTRY));
		componentMap.put(Experiment.Component.MOUSE, null);

		Experiment experiment = fetchById(experimentId, componentMap);

		boolean validRequest;

		if (completed) {
			// use defaults if no details have been sent
			for (Observation o : experiment.getObservations()) {
				if (o.getValue() == null && o.getRequiredObservation().getDefaultValue() != null) {
					o.setValue(o.getRequiredObservation().getDefaultValue());
					observationService.storeAuthenticated(o, user);
				}
			}

			if (isExperimentCompleteable(experiment)) { // experiment can be completed
				validRequest = true;
				experiment.setComplete(completed);
			} else {
				validRequest = false;
			}

		} else {
			validRequest = true; // always valid to say that it isn't complete
			experiment.setComplete(completed);
		}

		// record change
		if (validRequest) {
			storeAuthenticated(experiment, user);
		}

		// should the mouse be dead?
		Mouse m = experiment.getMouse();
		StdOperatingProcedure sop = experiment.getSop();
		if (validRequest && completed && sop.getProcedureEntry().isTerminal() && m.isAlive()) {

			m.setDeathDate(experiment.getProcedure().getStartDate());
			m.setCauseOfDeathDictId(MouseService.CAUSE_OF_DEATH_CULLED);

			mouseService.store(m, user);
		}

		return new Pair<Experiment, Boolean>(experiment, validRequest);
	}


	private boolean isExperimentCompleteable(Experiment experiment) {
		boolean completable = true;

		for (Map.Entry<RequiredObservation, Collection<Observation>> entry : experiment
				.getRequirementsAndObservations().entrySet()) {
			RequiredObservation reqObv = entry.getKey();
			int datapoints = 0;

			for (Observation observation : entry.getValue()) {
				if (!observation.isValid())
					completable = false;

				if (observation.getValue() != null && observation.getValue().length() > 0)
					datapoints++;
			}

			if (reqObv.isMandatoryObservation() && datapoints < 1) {
				completable = false;
			}

		}

		if (experiment.getProcedure() == null) {
			completable = false;
		}

		return completable;

	}


	public void delete(Experiment e, Person user) {
		experimentDao.delete(e, user);
	}


	public void setExperimentSetService(ExperimentSetService experimentSetService) {
		this.experimentSetService = experimentSetService;
	}


	public Experiment startExperiment(Long experimentId, Person user) {
		if (experimentId == null || user == null) {
			throw new IllegalArgumentException("Method does not take null paramerters");
		}

		Experiment exp = fetchById(experimentId, ComponentMap.of(Experiment.Component.MOUSE,
				Experiment.Component.PROCEDURE, Experiment.Component.SOP, Experiment.Component.EXPERIMENT_SET));

		if (exp == null) {
			return null;
		}

		// experiment has already started, nothing to do
		if (exp.getProcedure() != null) {
			return exp;
		}

		Date now = new Date();
		MouseAge mouseAge = new MouseAge(exp.getMouse());

		if (mouseAge.isWithinLifetime(now)) {
			createNewProcedure(exp);
			exp.getProcedure().setStartDate(now);
			mouseProcedureService.store(exp.getProcedure(), user);
		}

		return exp;
	}


	public Experiment removeProcedure(Long experimentId, Person user) {
		if (experimentId == null || user == null) {
			throw new IllegalArgumentException("Method does not take null paramerters");
		}

		Experiment exp = fetchById(experimentId, ComponentMap.of(Experiment.Component.PROCEDURE));

		if (exp == null) {
			return null;
		}

		mouseProcedureService.deleteByTableRef(TableMetadataEnum.PHN_EXPERIMENT, experimentId, user);

		if (exp.isComplete()) {
			exp.setComplete(false);
			storeAuthenticated(exp, user);
		}

		return exp;
	}


	public MouseProcedure recordProcedure(Experiment e, Date start, Date end, Person user) {
		createNewProcedure(e);
		e.getProcedure().setStartDate(start);
		e.getProcedure().setEndDate(end);

		mouseProcedureService.store(e.getProcedure(), user);
		return e.getProcedure();
	}

}
