/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSet;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSetSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.ExperimentSetDao;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.InitialisationInfo;
import uk.ac.sanger.mig.phenotyping.datacapture.views.ExperimentSetListView;

public class ExperimentSetServiceImpl extends AbstractServiceImpl<ExperimentSet> implements ExperimentSetService {

	private ExperimentSetDao experimentSetDao;
  private ExperimentService experimentService;
  private StdOperatingProcedureService sopService;
  private PersonalLicenceService personalLicenceService;
  private ProjectLicenceService projectLicenceService;
  private ObservationService observationService;

  private RequiredObservationService requiredObservationService;
  private ObservationTypeService observationTypeService;

	public void setObservationService(ObservationService observationService) {
    this.observationService = observationService;
  }

  public void setPersonalLicenceService(PersonalLicenceService personalLicenceService) {
    this.personalLicenceService = personalLicenceService;
  }

  public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
    this.projectLicenceService = projectLicenceService;
  }

  public void setExperimentService(ExperimentService experimentService) {
    this.experimentService = experimentService;
  }

  public void setSopService(StdOperatingProcedureService sopService) {
    this.sopService = sopService;
  }

	public void setExperimentSetDao(ExperimentSetDao experimentSetDao) {
		this.experimentSetDao = experimentSetDao;
	}


	public Long store(ExperimentSet experimentSet, Person user) {
		if(experimentSet == null) {
			throw new RuntimeException("Cannot save a null experimentSet");
		}

		return experimentSetDao.store(experimentSet, user);
	}

	public ExperimentSet fetchById(Long experimentSetId, ComponentMap requiredComponents) {
		ExperimentSet experimentSet = experimentSetDao.fetchById(experimentSetId);

		if(experimentSet != null)
			attachComponents(experimentSet, requiredComponents);

		return experimentSet;
	}

	@Override
	protected void attachComponents(ExperimentSet experimentSet, ComponentMap requiredComponents) {
		if(experimentSet == null) {
			throw new RuntimeException("Cannot attach components to null experimentSet");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(ExperimentSet.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(ExperimentSet.Component.EXPERIMENTS)) {
			  experimentSet.setExperiments(experimentService.fetchByExperimentSetId(
            experimentSet.getId(),
            requiredComponents.get(ExperimentSet.Component.EXPERIMENTS)));
      }

      if (fetchAll || requiredComponents.containsKey(ExperimentSet.Component.SOP)) {
        experimentSet.setSop(sopService.fetchById(
            experimentSet.getSopId(),
            requiredComponents.get(ExperimentSet.Component.SOP)));
      }

      if (fetchAll || requiredComponents.containsKey(ExperimentSet.Component.PIL)) {
        experimentSet.setPersonalLicence(personalLicenceService.fetchById(
            experimentSet.getPersonalLicenceId(),
            requiredComponents.get(ExperimentSet.Component.PIL)));
      }

      if (fetchAll || requiredComponents.containsKey(ExperimentSet.Component.PPL)) {
        experimentSet.setProjectLicence(projectLicenceService.fetchById(
            experimentSet.getProjectLicenceId(),
            requiredComponents.get(ExperimentSet.Component.PPL)));
      }

		}
	}


  public Collection<ExperimentSetListView> search(ExperimentSetSearchBean experimentSetSearchBean) {
    return experimentSetDao.search(experimentSetSearchBean);
  }

  @Transactional
  public Long storeWithExperimentsAndObservations(ExperimentSet experimentSet, Person user) {
    Long setId = store(experimentSet, user);
    experimentSet.setId(setId);

    for (Experiment experiment : experimentSet.getExperiments()) {
      if(experiment.getExperimentSetId() == null || experiment.getExperimentSetId() < 1L) {
        experiment.setExperimentSetId(setId);
      }
    }

    experimentService.storeWithResults(experimentSet.getExperiments(),user);

    return setId;
  }

  @Transactional
  public boolean fullDelete(Long experimentSetId, Person user) {
    boolean success = true;

    ExperimentSet es = fetchById(
        experimentSetId,
        new ComponentMap(
            ExperimentSet.Component.EXPERIMENTS,
            ComponentMap.of(Experiment.Component.OBSERVATIONS)
        )
    );

    if (es == null) {
      throw new IllegalArgumentException("Experiment Set "+experimentSetId+" could not be retrieved for deletion.");
    }

    if (es.isComplete()) {
      return false;
    }

    OUTER: for (Experiment e : es.getExperiments()) {
     if (e.isComplete()) {
       success = false;
       break OUTER;
     }

     for (Observation o : e.getObservations()) {
        boolean empty = (o.getValue() == null || o.getValue().length() < 1);
        if (!empty) {
          success = false;
          break OUTER;
        }
      }
    }

    if (success) {
      experimentSetDao.fullDelete(es, user);
    }

    return success;
  }

  @Transactional
  public void createBloodSampleBarcodesWhereRequired(Long experimentSetId, Person user) {
    ExperimentSet es = fetchById(experimentSetId,
        new ComponentMap(
            ExperimentSet.Component.EXPERIMENTS,
             ComponentMap.of(Experiment.Component.OBSERVATIONS)
        )
    );

    for (Experiment e : es.getExperiments()) {
      for (Observation o : e.getObservations()) {
        observationService.createBloodSampleBarcodeIfRequired(o.getId(), user);
      }
    }

  }

  @Transactional
  public boolean delete(ExperimentSet es, Person person) {
    int rowsAffected = experimentSetDao.delete(es, person);

    return (rowsAffected > 0 );
  }

  /**
   * Fetch method, using an attachment strategy that avoids
   * repeatedly requesting the same objects
   *
   * throws runtime errors if the caching strategy fails, which may be
   * an indication of data errors
   */
  public ExperimentSet fetchForPhenotyping(Long id) {

    ExperimentSet es = fetchById(id,
        ComponentMap.putAll(
            new ComponentMap(
                ExperimentSet.Component.EXPERIMENTS,
                ComponentMap.putAll(
                    ComponentMap.of(Experiment.Component.OBSERVATIONS),
                    new ComponentMap(
                        Experiment.Component.MOUSE,
                        ComponentMap.of(Mouse.Component.CAGE)
                    )
                )
            ),
            new ComponentMap(
                ExperimentSet.Component.SOP,
                ComponentMap.of(StdOperatingProcedure.Component.DOCUMENTATION)
            ),
            new ComponentMap(
                ExperimentSet.Component.COHORT,
                null),
                new ComponentMap(
                    ExperimentSet.Component.PIL,
                    ComponentMap.of(PersonalLicence.Component.PERSON)
                ),
                new ComponentMap(
                    ExperimentSet.Component.PPL,
                    null
                )
        )
    );

    if (es == null) return es;

    Map<Long,RequiredObservation> requirements = prefetchRequiredObservations(es.getSopId());

    for (Experiment e : es.getExperiments()) {
      for (Observation o : e.getObservations()) {

        Long roId = o.getRequiredObservationId();

        if (!requirements.containsKey(roId)) {
          throw new RuntimeException("Cache miss - cannot get required observation "+roId);
        }

        o.setRequiredObservation(requirements.get(roId));
      }
    }

    return es;
  }

  private Map<Long,RequiredObservation> prefetchRequiredObservations(Long sopId) {
    Map<Long,ObservationType> types = prefetchObservationTypes(sopId);
    Map<Long,RequiredObservation> requirements = new HashMap<Long,RequiredObservation>();

    for (RequiredObservation ro : requiredObservationService.fetchBySopId(sopId, false,null)) {
      requirements.put(ro.getId(), ro);

      if (!types.containsKey(ro.getObservationTypeId())) {
        throw new RuntimeException("Cache miss - cannot get type for required observation  "+ro.getId());
      }

      ro.setObservationType(types.get(ro.getObservationTypeId()));
    }

    return requirements;
  }

  private Map<Long,ObservationType> prefetchObservationTypes(Long sopId) {
    Map<Long,ObservationType> types = new HashMap<Long,ObservationType>();

    Collection<ObservationType> obsTypes = observationTypeService.fetchBySopId(sopId, ComponentMap.of(ObservationType.Component.RESTRICTED_OPTIONS));

    for (ObservationType ot : obsTypes) {
      types.put(ot.getId(), ot);
    }

    return types;
  }

  public void setRequiredObservationService(RequiredObservationService requiredObservationService) {
    this.requiredObservationService = requiredObservationService;
  }

  public void setObservationTypeService(ObservationTypeService observationTypeService) {
    this.observationTypeService = observationTypeService;
  }

  public ExperimentSet createExperimentSet(InitialisationInfo info) {
    ExperimentSet es = new ExperimentSet();

    es.setPersonalLicenceId(info.getPilId());
    es.setProjectLiceneceId(info.getPplId());
    es.setSopId(info.getSopId());
    es.setScheduledDate(info.getScheduledDate());

    es.setExperiments(new LinkedList<Experiment>());

    StdOperatingProcedure sop = sopService.fetchById(
        info.getSopId(),
        ComponentMap.putAll(
        new ComponentMap(
            StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS,
            ComponentMap.of(RequiredObservation.Component.OBSERVATION_TYPE)
        ),
        ComponentMap.of(StdOperatingProcedure.Component.PROCEDURE_ENTRY)
        )
    );

    if (sop == null) {
      throw new RuntimeException("Could not find an SOP for ID "+info.getSopId());
    }

    for (Mouse mouse : info.getMice()) {
      Experiment e = new Experiment(mouse.getId(),sop);

      e.setExperimentSet(es);
      e.setMouse(mouse);

      es.getExperiments().add(e);
    }

    return es;
  }

  @Transactional
  public Long[] storeWithExperimentsAndObservations(Collection<ExperimentSet> experimentSets, Person user) {
    Long[] ids = new Long[experimentSets.size()];

    int index = 0;

    for (ExperimentSet es : experimentSets) {
      Long id = this.storeWithExperimentsAndObservations(es, user);
      ids[index] = id;
      index++;
    }

    return ids;
  }

	public ExperimentSetListView fetchViewById(Long id) {
		ExperimentSetSearchBean essb = new ExperimentSetSearchBean();
		essb.setPaginated(false);
		essb.getExperimentSet().setId(id);

		Collection<ExperimentSetListView> views = experimentSetDao.search(essb);

		ExperimentSetListView view = null;

		if (views.size() > 1) {
			throw new IllegalStateException("Serious problem: returned multiple records when search by primary ID");
		}
		if (!views.isEmpty()) {
			view = views.iterator().next();
		}

		return view;
	}

}
