/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Collection;
import java.util.Date;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.mousetracking.TissueSample;
import uk.ac.sanger.mig.mousetracking.services.TissueSampleService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.ObservationDao;


public class ObservationServiceImpl extends AbstractServiceImpl<Observation> implements ObservationService {

	private ObservationDao observationDao;

  private RequiredObservationService requiredObservationService;
  private TissueSampleService tissueSampleService;
  private ExperimentService experimentService;

  public void setObservationDao(ObservationDao observationDao) {
	  this.observationDao = observationDao;
  }


  public void setRequiredObservationService(RequiredObservationService requiredObservationService) {
    this.requiredObservationService = requiredObservationService;
  }
  public void setTissueSampleService(TissueSampleService tissueSampleService) {
    this.tissueSampleService = tissueSampleService;
  }
  public void setExperimentService(ExperimentService experimentService) {
    this.experimentService = experimentService;
  }

  /**
   * Stores an observation
   *
   * Recording a blood sample barcode causes a tissue sample record to be created, using the
   * observation's value as a barcode
   *
   * RequiredObservation (including its ObservationType), and the Experiment fields must be
   * populated
   */
  @Transactional
	public Long storeAuthenticated(Observation observation, Person user) {
		if(observation == null) {
			throw new RuntimeException("Cannot save a null observation");
		}

		return observationDao.store(observation, user);
	}

  /**
   * If we are recording a blood sample barcode, find a new barcode and
   * create the matching tissue sample record
   */
  @Transactional
  private void createBloodSampleBarcode(Observation o, Person user) {
    String barcode = tissueSampleService.generateBarcode();

    o.setValue(barcode);
    Long mouseId = o.getExperiment().getMouseId();
    Date sampleDate = o.getObservationTime();

    TissueSample bloodSample = new TissueSample();
    bloodSample.setMouseId(mouseId);
    bloodSample.setBarcode(barcode);
    bloodSample.setSampleDate(sampleDate);
    bloodSample.setSampleType("Blood");

    tissueSampleService.store(bloodSample, user);
  }

	public Observation fetchById(Long observationId, ComponentMap requiredComponents) {
		Observation observation = observationDao.fetchById(observationId);

		if(observation != null)
			attachComponents(observation, requiredComponents);

		return observation;
	}

	@Override
	protected void attachComponents(Observation observation, ComponentMap requiredComponents) {
		if(observation == null) {
			throw new RuntimeException("Cannot attach components to null observation");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Observation.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Observation.Component.REQUIRED_OBSERVATION)) {
			  observation.setRequiredObservation(
				requiredObservationService.fetchById(observation.getRequiredObservationId(),
                requiredComponents.get(Observation.Component.REQUIRED_OBSERVATION)));
			}

			if (fetchAll || requiredComponents.containsKey(Observation.Component.EXPERIMENT)) {
				observation.setExperiment(
						experimentService.fetchById(observation.getExperimentId(),
								requiredComponents.get(Observation.Component.EXPERIMENT)
						));

			}

			if(fetchAll || requiredComponents.containsKey(Observation.Component.REQUIRED_OBSERVATION)) {
			  observation.setRequiredObservation(
            requiredObservationService.fetchById(observation.getRequiredObservationId(),
                requiredComponents.get(Observation.Component.REQUIRED_OBSERVATION)));
			}

		}
	}


	public Collection<Observation> search(ObservationSearchBean observationSearchBean) {
		Collection<Observation> observations = observationDao.search(observationSearchBean);
		return observations;
	}


  public Collection<Observation> fetchByExperimentId(Long experimentId, ComponentMap requiredComponents) {
    Collection<Observation> observations = observationDao.fetchByExperimentId(experimentId);
    attachComponents(observations, requiredComponents);
    return observations;
  }

  public Collection<Observation> fetchByObservationType(Long observationTypeId, ComponentMap componentMap) {
    Collection<Observation> observations = observationDao.fetchByObservationTypeId(observationTypeId);
    attachComponents(observations, componentMap);
    return observations;
  }

  @Transactional
  public void createBloodSampleBarcodeIfRequired(Long observationId, Person user) {
    Observation o = fetchById(
        observationId,
        ComponentMap.putAll(
            ComponentMap.of(Observation.Component.EXPERIMENT),
            new ComponentMap(
                Observation.Component.REQUIRED_OBSERVATION,
                ComponentMap.of(RequiredObservation.Component.OBSERVATION_TYPE)
            )
        )
    );
    if (o == null) {
      return;
    }

    boolean bloodSample = o.getRequiredObservation().getObservationType().isBloodSampleBarcode();
    boolean empty = (o.getValue() == null || o.getValue().length() < 1);

    if (bloodSample && empty) {
      createBloodSampleBarcode(o, user);
      storeAuthenticated(o,user);
    }
  }


  public void delete(Observation o, Person user) {
    observationDao.delete(o, user);

  }

  public Observation fetchByExperimentIdAndRequiredObservationId(Long experimentId, Long requiredObservationId) {

	return observationDao.fetchByExperimentIdAndRequiredObservationId(experimentId, requiredObservationId);
}

}
