/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationTypeSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.RestrictedOption;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.ObservationTypeDao;

public class ObservationTypeServiceImpl extends AbstractServiceImpl<ObservationType> implements ObservationTypeService {
	
	private ObservationTypeDao observationTypeDao;
	
	private RestrictedOptionService restrictedOptionService;
	
	public void setRestrictedOptionService(
			RestrictedOptionService restrictedOptionService) {
		this.restrictedOptionService = restrictedOptionService;
	}
  
	//IMPORTANT: add other dependencies here: Should be other services *not* dao typically
	/**
	 * eg 	private ObservationtypeDao observationTypeDao;
	 *    private MouseService mouseService
	 *    private PersonService personService; 
	 */
	
	public void setObservationTypeDao(ObservationTypeDao observationTypeDao) {
		this.observationTypeDao = observationTypeDao;
	}

  @Transactional
	public Long storeAuthenticated(ObservationType observationType, Person user) {
		if(observationType == null) {
			throw new RuntimeException("Cannot save a null observationType");
		}
		
		return observationTypeDao.store(observationType, user);
	}
  
  @Transactional
  public Long storeWithRestrictedOptions(ObservationType observationType, Person person) {
    Long id = storeAuthenticated(observationType, person);
    
    if (observationType.getRestrictedOptions() != null) {
      for (RestrictedOption option : observationType.getRestrictedOptions()) {
        restrictedOptionService.storeAuthenticated(option, person);
      }
    }
    
    return id;
  }
	
	public ObservationType fetchById(Long observationTypeId, ComponentMap requiredComponents) {
		ObservationType observationType = observationTypeDao.fetchById(observationTypeId);
		
		if(observationType != null)
			attachComponents(observationType, requiredComponents);
		
		return observationType;
	}

	@Override
	protected void attachComponents(ObservationType observationType, ComponentMap requiredComponents) {
		if(observationType == null) {
			throw new RuntimeException("Cannot attach components to null observationType");
		}		
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(ObservationType.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			
      if (fetchAll || requiredComponents.containsKey(ObservationType.Component.RESTRICTED_OPTIONS)) {
        Collection<RestrictedOption> options = restrictedOptionService.fetchByObservationType(observationType, requiredComponents.get(ObservationType.Component.RESTRICTED_OPTIONS));
        observationType.setRestrictedOptions(options);
      }
      
      if (requiredComponents.containsKey(ObservationType.Component.ACTIVE_RESTRICTED_OPTIONS)) {
        Collection<RestrictedOption> options = restrictedOptionService.fetchActiveByObservationType(observationType, requiredComponents.get(ObservationType.Component.ACTIVE_RESTRICTED_OPTIONS));
        observationType.setRestrictedOptions(options);
      }
		}
	}


	public Collection<ObservationType> search(ObservationTypeSearchBean observationTypeSearchBean) {
		return observationTypeDao.search(observationTypeSearchBean);
	}


	public Collection<ObservationType> fetchAll(ComponentMap componentMap) {
		
		Collection<ObservationType> types = observationTypeDao.fetchAll();
    attachComponents(types, componentMap);
		
		return types;
	}


	public Collection<ObservationType> fetchByMetaType(ObservationType.MetaType metaType, ComponentMap componentMap) {		
    Collection<ObservationType> types = observationTypeDao.fetchByMetaType(metaType);
    attachComponents(types, componentMap);
    return types;    
	}

	public Collection<ObservationType> fetchBySopId(Long sopId, ComponentMap requiredComponents) {
    Collection<ObservationType> observationTypes = observationTypeDao.fetchBySopId(sopId);
    
    if(observationTypes != null)
      attachComponents(observationTypes, requiredComponents);
    
    return observationTypes;
  }

	
	
}
								