/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.RestrictedOption;
import uk.ac.sanger.mig.phenotyping.datacapture.RestrictedOptionSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.RestrictedOptionDao;

public class RestrictedOptionServiceImpl extends AbstractServiceImpl<RestrictedOption> implements RestrictedOptionService {
	
	private RestrictedOptionDao restrictedOptionDao;
	private ObservationTypeService observationTypeService;
	
	//IMPORTANT: add other dependencies here: Should be other services *not* dao typically
	/**
	 * eg 	private RestrictedOptionDao restrictedOptionDao;
	 *    private MouseService mouseService
	 *    private PersonService personService; 
	 */
	
	public void setRestrictedOptionDao(RestrictedOptionDao restrictedOptionDao) {
		this.restrictedOptionDao = restrictedOptionDao;
	}
	
	public void setObservationTypeService(ObservationTypeService observationTypeService) {
		this.observationTypeService = observationTypeService;	}

  @Transactional
	public Long storeAuthenticated(RestrictedOption restrictedOption, Person user) {
		if(restrictedOption == null) {
			throw new RuntimeException("Cannot save a null restrictedOption");
		}
		
		return restrictedOptionDao.store(restrictedOption, user);
	}
	
	public RestrictedOption fetchById(Long restrictedOptionId, ComponentMap requiredComponents) {
		RestrictedOption restrictedOption = restrictedOptionDao.fetchById(restrictedOptionId);
		
		if(restrictedOption != null)
			attachComponents(restrictedOption, requiredComponents);
		
		return restrictedOption;
	}

	@Override
	protected void attachComponents(RestrictedOption restrictedOption, ComponentMap requiredComponents) {
		if(restrictedOption == null) {
			throw new RuntimeException("Cannot attach components to null restrictedOption");
		}		
		
		boolean fetchAll = false;

		if(requiredComponents != null) {
			if(requiredComponents.containsKey(RestrictedOption.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			
			if(fetchAll || requiredComponents.containsKey(RestrictedOption.Component.OBSERVATION_TYPE)) {

				ObservationType observationType = observationTypeService.fetchById(
						restrictedOption.getObservationTypeId(),
						null);
				restrictedOption.setObservationType(observationType);
			}
			
						
		}
	}


	public Collection<RestrictedOption> search(RestrictedOptionSearchBean restrictedOptionSearchBean) {
		Collection<RestrictedOption> options = restrictedOptionDao.search(restrictedOptionSearchBean);
		ComponentMap requiredComponents = new ComponentMap();
		requiredComponents.put(RestrictedOption.Component.ALL_COMPONENTS,null);

		attachComponents(options, requiredComponents);


		return options;
	}


	public Collection<RestrictedOption> fetchByObservationType(ObservationType observationType, ComponentMap componentMap) {
    Collection<RestrictedOption> options = restrictedOptionDao.fetchByObservationType(observationType);    
    attachComponents(options, componentMap);    
		return convertFlatCollectionToOptionTree(options);
	}
  
  public Collection<RestrictedOption> fetchActiveByObservationType(ObservationType observationType, ComponentMap componentMap) {   
    Collection<RestrictedOption> options = restrictedOptionDao.fetchActiveByObservationType(observationType);
    attachComponents(options, componentMap);
    return convertFlatCollectionToOptionTree(options);
  }
	
	/**
	 * RestrictedOption objects have fields for parent/child relationships, but these are not filled in
	 * by the dao.
	 * 
	 * This method gives a collection of the top level options. Traverse down from these to reach the children
	 * 
	 * @param flatOptions 
	 * @return collection of the top level RestrictedOptions
	 */
	private Collection<RestrictedOption> convertFlatCollectionToOptionTree(Collection<RestrictedOption> flatOptions) {
		Collection<RestrictedOption> optionTree = new ArrayList<RestrictedOption>();		
		
		for (RestrictedOption option : flatOptions) {
			if (option.isRootNode()) {
				optionTree.add(option);
			}
			else {
				RestrictedOption parentOption = extractParentOption(option, flatOptions);
				if (parentOption != null) {
					option.setParentOption(parentOption);
					parentOption.addChildOption(option);
				}
			}
		}
		
		return optionTree;
	}
	/**
	 * Find the parent for a child option.
	 * Logs errors if the parent can't be found, or if the observation types don't match.
	 * 
	 * @param childOption
	 * @param parentSource
	 * @return the parent option, or null if a parent could not be found
	 */
	private RestrictedOption extractParentOption(RestrictedOption childOption, Collection<RestrictedOption> parentSource) {
		RestrictedOption parentOption = null;
		
		for (RestrictedOption option : parentSource) {
			if (option.getId().equals( childOption.getParentOptionId() )) {
				parentOption = option;
				break;
			}
		}
		
		if (parentOption == null) {
			logger.error("Could not find parent option in collection provided. Child option: "+childOption+" Collection: "+parentSource);
		}
		
		if (parentOption != null
				&& !parentOption.getObservationTypeId().equals( childOption.getObservationTypeId() )) {
			logger.error("Parent and child options have a different observation type. Parent: "+parentOption+" Child: "+childOption);
		}
		
		return parentOption;
	}


	
	
	
}
								