/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.filesupport.services.BinaryFileService;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedureSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.StdOperatingProcedureDao;

public class StdOperatingProcedureServiceImpl extends AbstractServiceImpl<StdOperatingProcedure> implements StdOperatingProcedureService {
	
	private StdOperatingProcedureDao stdOperatingProcedureDao;
	private RequiredObservationService requiredObservationService;
  private BinaryFileService binaryFileService;
  private MouseProcedureService mouseProcedureService;
	
	//IMPORTANT: add other dependencies here: Should be other services *not* dao typically
	/**
	 * eg 	private StdOperatingProcedureDao stdOperatingProcedureDao;
	 *    private MouseService mouseService
	 *    private PersonService personService; 
	 */
	
	public void setStdOperatingProcedureDao(StdOperatingProcedureDao stdOperatingProcedureDao) {
		this.stdOperatingProcedureDao = stdOperatingProcedureDao;
	}

  public void setBinaryFileService(BinaryFileService binaryFileService) {
    this.binaryFileService = binaryFileService;
  }

  @Transactional
	public Long storeAuthenticated(StdOperatingProcedure stdOperatingProcedure, Person user) {
		if(stdOperatingProcedure == null) {
			throw new RuntimeException("Cannot save a null stdOperatingProcedure");
		}
		
		return stdOperatingProcedureDao.store(stdOperatingProcedure, user);
	}
  
  @Transactional
  public Long storeWithRequiredObservations(StdOperatingProcedure sop, Person user) {
    Long id = storeAuthenticated(sop, user);
    
    if (sop.getRequiredObservations() != null) {
      for (RequiredObservation reqObv : sop.getRequiredObservations()) {
        requiredObservationService.storeAuthenticated(reqObv, user);
      }
    }
    
    return id;
  }
	
	public StdOperatingProcedure fetchById(Long stdOperatingProcedureId, ComponentMap requiredComponents) {
		StdOperatingProcedure stdOperatingProcedure = stdOperatingProcedureDao.fetchById(stdOperatingProcedureId);
		
		if(stdOperatingProcedure != null)
			attachComponents(stdOperatingProcedure, requiredComponents);
		
		return stdOperatingProcedure;
	}
	
	@Override
	protected void attachComponents(StdOperatingProcedure sop, ComponentMap requiredComponents) {
		if(sop == null) {
			throw new RuntimeException("Cannot attach components to null stdOperatingProcedure");
		}		
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(StdOperatingProcedure.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			
			if (fetchAll || requiredComponents.containsKey(StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS)){
				sop.setRequiredObservations( requiredObservationService.fetchBySopId(sop.getId(), requiredComponents.get(StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS)));
			}
      
      if (fetchAll || requiredComponents.containsKey(StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS_FOR_EDIT)){
        sop.setRequiredObservations( requiredObservationService.fetchForEditBySopId(sop.getId(), requiredComponents.get(StdOperatingProcedure.Component.REQUIRED_OBSERVATIONS_FOR_EDIT)));
      }
      
      if (fetchAll || requiredComponents.containsKey(StdOperatingProcedure.Component.DOCUMENTATION)){
        // should only be one
        Collection<BinaryFile> files =  binaryFileService.fetchByTableRef(
            TableMetadataEnum.PHN_STD_OPERATING_PROCEDURE,
            sop.getId(), false, requiredComponents);
        
        if (files.size() > 0) {
          sop.setDocumentation(files.iterator().next());
        }
      }
      
      if (fetchAll || requiredComponents.containsKey(StdOperatingProcedure.Component.PROCEDURE_ENTRY)) {
        sop.setProcedureEntry(
            mouseProcedureService.fetchDictionaryEntryById(sop.getProcedureId())
        );
      }
		}
	}


	public void setRequiredObservationService(
			RequiredObservationService requiredObservationService) {
		this.requiredObservationService = requiredObservationService;
	}


	public Collection<StdOperatingProcedure> fetchAll() {
		Collection<StdOperatingProcedure> sops = stdOperatingProcedureDao.fetchAll();
		attachComponents(sops, ComponentMap.of(StdOperatingProcedure.Component.ALL_COMPONENTS));
		return sops;
	}
  
  public Collection<StdOperatingProcedure> fetchAllActive() {
    Collection<StdOperatingProcedure> sops = stdOperatingProcedureDao.fetchAllActive();
    attachComponents(sops, ComponentMap.of(StdOperatingProcedure.Component.ALL_COMPONENTS));
    return sops;
  }


	public Collection<StdOperatingProcedure> search(StdOperatingProcedureSearchBean searchBean) {
		Collection<StdOperatingProcedure> sops = stdOperatingProcedureDao.search(searchBean);
		attachComponents(sops, ComponentMap.of(StdOperatingProcedure.Component.ALL_COMPONENTS));
		return sops;
	}


  public Collection<StdOperatingProcedure> fetchAllAvailableToUser(Person user) {
    Collection<StdOperatingProcedure> sops = stdOperatingProcedureDao.fetchAllAvailableToUser(user);
    attachComponents(sops, ComponentMap.of(StdOperatingProcedure.Component.ALL_COMPONENTS));
    return sops;
  }

  public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
    this.mouseProcedureService = mouseProcedureService;
  }
  
  
	
	
}
								