/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang.time.DateUtils;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.Pair;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.calorimetry.CalorimetryDataPoint;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.BoxData;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.CalorimetryData;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.Sample;
import uk.ac.sanger.mig.phenotyping.calorimetry.services.CalorimetryDataPointService;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSet;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.UploadMapping;
import uk.ac.sanger.mig.phenotyping.datacapture.dao.UploadMappingDao;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.CalorimetryUploadForm;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.InitialisationInfo;

/**
 *
 */
public class UploadMappingServiceImpl implements UploadMappingService {

	private ExperimentSetService			experimentSetService;
	private ExperimentService				experimentService;
	private StdOperatingProcedureService	sopService;
	private UploadMappingDao				uploadMappingDao;
	private CohortService					cohortService;
	private CalorimetryDataPointService		calorimetryDataPointService;
	private MouseService					mouseService;


	@Transactional
	public ExperimentSet uploadCalorimetryData(CalorimetryUploadForm form, Person user) {
		InitialisationInfo info = createInitialisationInfo(form);

		ExperimentSet es = experimentSetService.createExperimentSet(info);

		// Which required observations map to calorimetry fields?
		Map<String, Long> fieldMappings = getFieldMappings(form.getSopId());

		// index Experiments for ease of access
		Map<Long, Experiment> experiments = indexExprimentsByMouseId(es);

		// get box data
		for (BoxData b : form.getCalorimetryData().getBoxes()) {
			Experiment e = experiments.get(b.getAnimalNumber());
			// box data doesn't always have a real mouse id
			if (e != null) {
				recordBoxData(e, b, fieldMappings);
			}
		}

		// fill in any default values
		for (Experiment e : experiments.values()) {
			useDefaults(e);
		}

		// store all results
		experimentSetService.storeWithExperimentsAndObservations(es, user);

		Pair<Date, Date> dateRange = findDateRange(form.getCalorimetryData().getSamples());
		Date earliestDate = dateRange.getObjectA();
		Date latestDate = dateRange.getObjectB();

		// add procedure records (requires experiments to have ids)
		recordProcedures(earliestDate, latestDate, experiments, user);

		// get data from samples
		for (Sample s : form.getCalorimetryData().getSamples()) {
			Experiment e = experiments.get(s.getAnimalNo());
			if (e == null)
				continue;
			recordCalorimetryData(e, s, earliestDate, user);
		}

		return es;
	}


	/**
	 * Checks each Observation in the experiment. If it does not have a Value set and there is a
	 * default value, it will set the Observation to use the default value
	 *
	 * Used because experiment metadata (e.g. machine model) is stored as the default value for the
	 * required observation
	 *
	 * @param e
	 */
	private void useDefaults(Experiment e) {
		for (Observation o : e.getObservations()) {
			RequiredObservation ro = o.getRequiredObservation();

			if (o.getValue() == null && ro.getDefaultValue() != null) {
				o.setValue(ro.getDefaultValue());
			}
		}
	}


	private void recordCalorimetryData(Experiment e, Sample s, Date earliestDate, Person user) {
		CalorimetryDataPoint dataPoint = new CalorimetryDataPoint(s, earliestDate, e.getId());

		calorimetryDataPointService.store(dataPoint, user);
	}


	private void recordProcedures(Date earliestDate, Date latestDate, Map<Long, Experiment> experiments, Person user) {
		for (Experiment e : experiments.values()) {
			experimentService.recordProcedure(e, earliestDate, latestDate, user);
		}
	}


	private Pair<Date, Date> findDateRange(Collection<Sample> samples) {
		Date earliestDate = null;
		Date latestDate = null;

		for (Sample s : samples) {
			if (earliestDate == null || s.getDate().before(earliestDate)) {
				earliestDate = s.getDate();
			}
			if (latestDate == null || s.getDate().after(latestDate)) {
				latestDate = s.getDate();
			}
		}
		return new Pair<Date, Date>(earliestDate, latestDate);
	}


	private void recordBoxData(Experiment e, BoxData b, Map<String, Long> fieldMappings) {
		Map<String, Number> data = new HashMap<String, Number>();

		data.put(Fields.BOX_NUMBER.toString(), b.getBox());
		data.put(Fields.WEIGHT_BEFORE.toString(), b.getWeight());

		for (Entry<String, Number> entry : data.entrySet()) {
			String fieldName = entry.getKey();

			if (fieldMappings.containsKey(fieldName)) {
				Long requiredObservationId = fieldMappings.get(fieldName);
				Observation o = getExistingObservation(e, requiredObservationId);

				o.setValue(entry.getValue().toString());

				if (!o.isValid()) {
					throw new RuntimeException("Validation failed for calorimetry field " + fieldName + " with value "
							+ entry.getValue() + " for required observation " + requiredObservationId);
				}
			}
		}

	}


	private Observation getExistingObservation(Experiment e, Long requiredObservationId) {
		Observation existingObservation = null;
		for (Observation o : e.getObservations()) {
			if (o.getRequiredObservationId().equals(requiredObservationId)) {
				existingObservation = o;
				break;
			}
		}
		return existingObservation;
	}


	private Map<String, Long> getFieldMappings(Long sopId) {
		Collection<UploadMapping> uploadMappings = uploadMappingDao.fetchBySopId(sopId);
		Map<String, Long> fieldMappings = new HashMap<String, Long>();

		for (UploadMapping um : uploadMappings) {
			fieldMappings.put(um.getFieldName(), um.getRequiredObservationId());
		}

		return fieldMappings;
	}


	private Map<Long, Experiment> indexExprimentsByMouseId(ExperimentSet... experimentSets) {
		Map<Long, Experiment> indexedExperiments = new HashMap<Long, Experiment>();

		for (ExperimentSet es : experimentSets) {
			for (Experiment e : es.getExperiments()) {
				indexedExperiments.put(e.getMouseId(), e);
			}
		}

		return indexedExperiments;
	}


	private InitialisationInfo createInitialisationInfo(CalorimetryUploadForm form) {
		CalorimetryData data = form.getCalorimetryData();

		// get the mice
		Collection<Long> mouseIds = new LinkedList<Long>();
		for (BoxData box : data.getBoxes()) {
			Long mouseId = box.getAnimalNumber();
			if (mouseId == null || mouseId < 2)
				continue;
			mouseIds.add(mouseId);
		}

		// get the earliest recording
		Date procedureDate = null;
		for (Sample sample : data.getSamples()) {
			if (procedureDate == null || sample.getDate().before(procedureDate)) {
				procedureDate = sample.getDate();
			}
		}
		// round off the hours, minutes, etc
		procedureDate = DateUtils.truncate(procedureDate, Calendar.DATE);

		Collection<Mouse> mice = new LinkedList<Mouse>();

		for (Long mouseId : mouseIds) {
			Mouse m = mouseService.fetchById(mouseId, null);
			mice.add(m);
		}

		InitialisationInfo info = new InitialisationInfo(form.getSopId(), form.getPplId(), form.getPilId(), mice,
				procedureDate);

		return info;
	}


	public StdOperatingProcedure fetchActiveSopForUploadType(UploadMappingService.Type type, ComponentMap componentMap) {
		StdOperatingProcedure sop = null;

		Long sopId = uploadMappingDao.fetchActiveSopIdForUploadType(type);
		if (sopId != null) {
			sop = sopService.fetchById(sopId, componentMap);
		}

		return sop;
	}


	public Map<Cohort, Collection<Long>> groupMiceByCohorts(Collection<Long> mouseIds) {
		Map<Cohort, Collection<Long>> candidateCohorts = getCandidateCohorts(mouseIds);

		Map<Cohort, Collection<Long>> bestFit = getBestFit(candidateCohorts);

		return bestFit;
	}


	private Map<Cohort, Collection<Long>> getCandidateCohorts(Collection<Long> mouseIds) {
		Map<Cohort, Collection<Long>> candidateCohorts = new HashMap<Cohort, Collection<Long>>();

		// get all the cohorts that we can use
		for (Long mouseId : mouseIds) {

			Collection<Cohort> cohorts = cohortService.fetchByMouseId(mouseId, null);

			if (cohorts.isEmpty())
				throw new IllegalArgumentException("All mice must be in at least one cohort: " + mouseId);

			for (Cohort c : cohorts) {
				// make sure it's the right sort of cohort
				if (c.getCohortTypeDictId().equals(CohortService.COHORT_TYPE_PHENOTYPING)) {
					if (!candidateCohorts.containsKey(c)) {
						candidateCohorts.put(c, new HashSet<Long>());
					}
					candidateCohorts.get(c).add(mouseId);
				}
			}
		}
		return candidateCohorts;
	}


	private Cohort selectLargest(Map<Cohort, Collection<Long>> candidateCohorts) {
		Cohort largestSoFar = null;
		int bestCountSoFar = 0;

		for (Map.Entry<Cohort, Collection<Long>> entry : candidateCohorts.entrySet()) {
			if (entry.getValue().size() > bestCountSoFar) {
				largestSoFar = entry.getKey();
				bestCountSoFar = entry.getValue().size();
			}
		}

		return largestSoFar;
	}


	private void removeCohortAndMice(Cohort c, Map<Cohort, Collection<Long>> candidateCohorts) {
		Collection<Long> mouseIds = candidateCohorts.get(c);
		candidateCohorts.remove(c);

		Iterator<Map.Entry<Cohort, Collection<Long>>> i = candidateCohorts.entrySet().iterator();
		while (i.hasNext()) {
			Map.Entry<Cohort, Collection<Long>> entry = i.next();
			// remove the mice that have already been taken care of
			entry.getValue().removeAll(mouseIds);

			// no mice left here that need matching, remove the cohort from consideration
			if (entry.getValue().size() < 1) {
				i.remove();
			}
		}
	}


	private Map<Cohort, Collection<Long>> getBestFit(Map<Cohort, Collection<Long>> candidateCohorts) {
		Map<Cohort, Collection<Long>> cohortsToGo = new HashMap<Cohort, Collection<Long>>();

		while (!candidateCohorts.isEmpty()) {
			Cohort currentBest = selectLargest(candidateCohorts);
			Collection<Long> mouseIds = candidateCohorts.get(currentBest);

			cohortsToGo.put(currentBest, mouseIds);

			removeCohortAndMice(currentBest, candidateCohorts);
		}

		return cohortsToGo;
	}


	public void setCohortService(CohortService cohortService) {
		this.cohortService = cohortService;
	}


	public void setExperimentSetService(ExperimentSetService experimentSetService) {
		this.experimentSetService = experimentSetService;
	}


	public void setSopService(StdOperatingProcedureService sopService) {
		this.sopService = sopService;
	}


	public void setUploadMappingDao(UploadMappingDao uploadMappingDao) {
		this.uploadMappingDao = uploadMappingDao;
	}


	public void setExperimentService(ExperimentService experimentService) {
		this.experimentService = experimentService;
	}


	public Collection<UploadMapping> fetchBySopId(Long sopId) {
		return uploadMappingDao.fetchBySopId(sopId);
	}


	public Collection<UploadMapping> fetchBySopIdAndName(Long sopId, UploadMappingService.Type uploadType) {
		return uploadMappingDao.fetchBySopIdAndName(sopId, uploadType);
	}


	public void setCalorimetryDataPointService(CalorimetryDataPointService calorimetryDataPointService) {
		this.calorimetryDataPointService = calorimetryDataPointService;
	}


	public Collection<Pair<Long, Type>> fetchSopsToUploadType() {
		return uploadMappingDao.fetchSopsToUploadType();
	}


	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

}
