/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.validators;

import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAge;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.BeginPhenotypingForm;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.MouseProcedureDate;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentService;

public class BeginPhenotypingFormValidator implements Validator {

  private ProjectLicenceService projectLicenceService;
  private ExperimentService experimentService;

  @SuppressWarnings("unchecked")
  public boolean supports(Class clazz) {
    return BeginPhenotypingForm.class.equals(clazz);
  }

  public void validate(Object command, Errors errors) {
    BeginPhenotypingForm form = (BeginPhenotypingForm) command;

    ValidationUtils.rejectIfEmpty(errors, "sop", null, "Please select a SOP");
    ValidationUtils.rejectIfEmpty(errors, "cohort", null, "Please select a cohort");
    ValidationUtils.rejectIfEmpty(errors, "projectLicence", null, "Please select a project licence");
    ValidationUtils.rejectIfEmpty(errors, "personalLicence", null, "Please select a personal licence");
    ValidationUtils.rejectIfEmpty(errors, "procedureDate", null, "Please select a date");

    if (form.getMiceIncludedInProcedure().size() < 1) {
      errors.reject(null, "No mice selected for procedure");
    }

    if (form.getSop() != null &&
        form.getSop().getProcedureEntry().isTerminal()) {
      List<MouseProcedureDate> mpds = form.getMiceInCohort();
      for (int i = 0; i < mpds.size(); i++) {
        if (mpds.get(i).isUseInProcedure()) {
          ExperimentSearchBean esb = new ExperimentSearchBean();
          esb.setPaginated(false);
          esb.setExperiment(new Experiment());
          esb.getExperiment().setMouseId(mpds.get(i).getMouse().getId());
          esb.getExperiment().setSopId(form.getSop().getId());
          Collection<Experiment> exps = experimentService.search(esb, null);

          if (!exps.isEmpty()) {
            String path = "miceInCohort["+i+"].useInProcedure";
            errors.rejectValue(path, null, "Procedure already recorded for mouse "+mpds.get(i).getMouse().getName());
          }

        }
      }
    }

    if (form.getProcedureDate() != null) {
      //  Procedures can not be recorded too far into the future
      int dayLimit = 7;
      Date now = new Date();
      Date limit = new Date((1000L*60*60*24*dayLimit) + now.getTime());

      if (form.getProcedureDate().after(limit)){
        errors.rejectValue("procedureDate", null, "Procedure cannot start more than "+dayLimit+" days into the future");
      }

      if (form.getProjectLicence() != null) {
        // Procedure can not be outside valid dates
        if (form.getProjectLicence().getStartDate() != null &&
            form.getProjectLicence().getStartDate() != null &&
            form.getProcedureDate().before(form.getProjectLicence().getStartDate())) {
          errors.rejectValue("procedureDate", null, "Procedure cannot occur before PPL start date");
        }

        if (form.getProjectLicence().getStartDate() != null &&
            form.getProjectLicence().getEndDate() != null &&
            form.getProcedureDate().after(form.getProjectLicence().getEndDate())) {
          errors.rejectValue("procedureDate", null, "Procedure cannot occur after PPL end date");
        }
      }

      List<MouseProcedureDate> mpds = form.getMiceInCohort();
      for (int i = 0; i < mpds.size(); i++) {
        MouseProcedureDate mpd = mpds.get(i);

        if (mpd.isUseInProcedure()) {
          mpd.setProcedureDate(form.getProcedureDate());
          checkMouseProcedureDate(form.getProcedureDate(), mpd.getMouse(), errors, i);
        }
      }
    }

    if (form.getSop() != null && form.getPersonalLicence() != null && form.getProjectLicence() != null) {
      Long personalLicenceId = form.getPersonalLicence().getId();
      Long procedureId = form.getSop().getProcedureId();

      Collection<ProjectLicence> ppls = projectLicenceService.fetchByPersonalLicenceAndProcedureId(
          personalLicenceId,
          procedureId,
          ComponentMap.of(ProjectLicence.Component.PROTOCOLS));

      if (ppls.size() < 1) {
        errors.reject(null, "This personal licence does not seem to have a project licence with this procedure");
      } else if (!ppls.contains(form.getProjectLicence())) {
        errors.reject(null, "This project licence does not seem to include this procedure");
      }
    }

  }

  private void checkMouseProcedureDate(Date procedureDate, Mouse mouse, Errors errors, int index) {
    MouseAge ma = new MouseAge(mouse);

    // Procedures can only happen while the mouse is alive
    if (!ma.isWithinLifetime(procedureDate)) {
      String path = "miceInCohort["+index+"].procedureDate";
      errors.rejectValue(path, null, "Procedure must occur within the lifetime of the mouse");
    }
  }

  public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
    this.projectLicenceService = projectLicenceService;
  }
  public void setExperimentService(ExperimentService experimentService) {
    this.experimentService = experimentService;
  }

}
