/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.validators;

import java.text.ParseException;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DataObjectUtils;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.CohortSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.mousetracking.views.CohortListView;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.BoxData;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.CalorimetryData;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.MalformedCalorimetryFileException;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.Parser;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.Sample;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.CalorimetryUploadForm;
import uk.ac.sanger.mig.phenotyping.datacapture.services.UploadMappingService;

public class CalorimetryUploadFormValidator implements Validator {

  protected Log logger = LogFactory.getLog(this.getClass());
  private MouseService mouseService;
  private ProjectLicenceService projectLicenceService;
  private CohortService cohortService;
  private UploadMappingService uploadMappingService;

  public boolean supports(Class clazz) {
    return CalorimetryUploadForm.class.equals(clazz);
  }


  public void validate(Object command, Errors errors) {
    CalorimetryUploadForm form = (CalorimetryUploadForm) command;

    ValidationUtils.rejectIfEmpty(errors, "pplId", null, "Please select a project licence");
    ValidationUtils.rejectIfEmpty(errors, "pilId", null, "Please select a personal licence");
    ValidationUtils.rejectIfEmpty(errors, "uploadFile", null, "Please select a data file");

    if (form.getUploadFile() != null){
      validateFile(form,errors);
    }

    if (form.getCalorimetryData() != null) {
      validateMice(form,errors);
      validateSamples(form,errors);
    }
    if (form.getPilId() != null && form.getPplId() != null) {
      validateLicence(form,errors);
    }
  }

  /**
   * We expect there to be readings for every mouse listed in the header section
   * Listing a mouse in the headers (boxData), but not having any data for it is an error.
   * The readings also contain the mouse ID - if it doesn't match the ID listed in the headers there is a problem
   *
   * errors will be added to uploadFile if either of these problems occur
   *
   */
  private void validateSamples(CalorimetryUploadForm form, Errors errors) {
  	Map<Integer,Long> mouseIdByBoxId = new HashMap<Integer, Long>();
  	Map<Integer,Collection<Sample>> samplesByBoxId = new HashMap<Integer, Collection<Sample>>();
  	Set<Integer> boxesWithIdMismatches = new TreeSet<Integer>();

  	for (BoxData boxData : form.getCalorimetryData().getBoxes()) {
  		if (boxData.getAnimalNumber() > 1) {
  			mouseIdByBoxId.put(boxData.getBox(), boxData.getAnimalNumber());
  		}
  	}

  	for (Sample sample : form.getCalorimetryData().getSamples()) {
  		Integer boxId = sample.getBox();
  		if (!samplesByBoxId.containsKey(boxId)){
  			samplesByBoxId.put(boxId, new LinkedList<Sample>());
  		}

  		samplesByBoxId.get(boxId).add(sample);

  		Long expectedAnimalId = mouseIdByBoxId.get(boxId);
  		if (expectedAnimalId != null) {
	  		if (!expectedAnimalId.equals(sample.getAnimalNo())) {
	  			boxesWithIdMismatches.add(boxId);
	  		}
  		}
  	}

  	for (Integer boxId : mouseIdByBoxId.keySet()){
  		Collection<Sample> samples = samplesByBoxId.get(boxId);
  		if (samples == null || samples.isEmpty()) {
  			errors.rejectValue("uploadFile", null, "Data was expected for box "+boxId+" but none was found");
  		}
  	}

  	for (Integer boxId  : boxesWithIdMismatches) {
  		errors.rejectValue("uploadFile", null, "The mouse ID given for box "+boxId+" does not match the ID given for some of the readings");
  	}

	}


	// are these mice known, and are they thought to be alive at the point this work was done
  protected void validateMice(CalorimetryUploadForm form, Errors errors) {
    Collection<Mouse> mice = new LinkedList<Mouse>();

    for (BoxData boxData : form.getCalorimetryData().getBoxes()) {
      Long mouseId = boxData.getAnimalNumber();

      boolean empty = mouseId < 2;

      if (!empty) {
        Mouse mouse = mouseService.fetchById(mouseId,null);

        if (mouse == null) {
          errors.rejectValue("uploadFile",null,"Could not find mouse in box "+boxData.getBox()+" with ID "+mouseId);
        }
        else {
          CohortSearchBean csb = new CohortSearchBean();
          csb.setMouse(mouse);
          csb.setCohortTypeDictId(CohortService.COHORT_TYPE_PHENOTYPING);
          csb.setPaginated(false);
          Collection<CohortListView> cohorts = cohortService.search(csb);

          if(cohorts.isEmpty()) {
            errors.rejectValue("uploadFile",null,"The mouse in box "+boxData.getBox()+" with ID "+mouseId+" is not in a cohort");
          }
          else {
            mice.add(mouse);
          }
        }
      }
    }
  }

  // can we parse the file?
  protected void validateFile(CalorimetryUploadForm form, Errors errors) {
    CalorimetryData data = null;

    try {
      if (form.getBinaryFile() != null &&
          form.getBinaryFile().getSize() > 0) {
        Parser parser = new Parser();
        data = parser.loadFromBytes(form.getBinaryFile().getBytes());
      }
    }
    catch (MalformedCalorimetryFileException e) {
      logger.warn("Could not parse calorimetry data file", e);
    }
    catch (ParseException e) {
      logger.warn("Could not parse calorimetry data file", e);
    }
    catch(NoSuchElementException e) {
      logger.warn("Could not parse calorimetry data file", e);
    }

    if (data == null) {
      errors.rejectValue("uploadFile",null,"Please provide a valid data file.");
    }
    else {
      form.setCalorimetryData(data);
    }
  }

  // is this person permitted to perform this work on these mice?
  protected void validateLicence(CalorimetryUploadForm form, Errors errors) {
    Long personalLicenceId = form.getPilId();
    Long projectLicenceId = form.getPplId();

    StdOperatingProcedure sop = uploadMappingService.fetchActiveSopForUploadType(UploadMappingService.Type.MGP_CALORIMETRY, null);

    if (sop == null) {
      errors.rejectValue("uploadFile", null, "Cannot find SOP details, please contact Team 110");
    }
    else {
      Long procedureId = sop.getProcedureId();
      form.setSopId(sop.getId());

      Collection<ProjectLicence> ppls =
        projectLicenceService.fetchByPersonalLicenceAndProcedureId(
          personalLicenceId,
          procedureId,
          ComponentMap.of(ProjectLicence.Component.PROTOCOLS)
        );

      if (ppls.size() < 1) {
        errors.reject("pilId", null, "This personal licence does not seem to have a project licence with this procedure");
      } else if (!DataObjectUtils.collectionContains(ppls, projectLicenceId)) {
        errors.reject("pplId", null, "This project licence does not seem to include this procedure");
      }
    }
  }

  public void setMouseService(MouseService mouseService) {
    this.mouseService = mouseService;
  }


  public void setProjectLicenceService(ProjectLicenceService projectLicenceService) {
    this.projectLicenceService = projectLicenceService;
  }


  public void setCohortService(CohortService cohortService) {
    this.cohortService = cohortService;
  }

  public void setUploadMappingService(UploadMappingService uploadMappingService) {
    this.uploadMappingService = uploadMappingService;
  }


}
