/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.validators;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.RestrictedOption;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.ObservationTypeForm;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ObservationService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ObservationTypeService;

public class ObservationTypeFormValidator implements Validator {
	
  private ObservationService observationService;
  private ObservationTypeService observationTypeService;
  
  public void setObservationService(ObservationService observationService) {
    this.observationService = observationService;
  }
  
  public void setObservationTypeService(ObservationTypeService observationTypeService) {
    this.observationTypeService = observationTypeService;
  }

  public boolean supports(Class clazz) {
		return ObservationTypeForm.class.equals(clazz);
	}

	public void validate(Object commandForm, Errors errors) {	

		ObservationTypeForm observationTypeForm = (ObservationTypeForm)commandForm;
		ObservationType observationType = observationTypeForm.getObservationType();				
		
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "observationType.name", null, "Please name this observation type");
		
		if (observationType.getObservationMetaTypeId() < 1) {
			errors.rejectValue("observationType.observationMetaTypeId",null, "Please choose an underlying type");
		}
    
    if (observationType.getRestrictedOptions() != null) {
      checkNestedOptions(observationType.getRestrictedOptions(), errors);
    }
    
    if (observationType.getRangeMin() != null && 
        observationType.getRangeMax() != null &&
        observationType.getRangeMin() >= observationType.getRangeMax()) {
      errors.rejectValue("observationType.rangeMin",null,"Minumum value needs to be less than the maximum");
    }
    
    if (observationType.getId() != null && observationType.getId() > 0){
      
      ObservationType oldVersion = observationTypeService.fetchById(observationType.getId(), null);

      if (incompatibleMetaTypeChange(observationType, oldVersion)) {
        errors.rejectValue("observationType.observationMetaTypeId",null,"This is not a compatible change");
      }
      
      if (badChangeToUnits(observationType, oldVersion)) {
        errors.rejectValue("observationType.units",null,"Cannot change - observations have been made with these units");
      }
    }
    
		
	}

  private void checkNestedOptions(Collection<RestrictedOption> restrictedOptions, Errors errors) {
    List<RestrictedOption> options = new ArrayList<RestrictedOption>();
    options.addAll(restrictedOptions);
    
    for(int i = 0; i < options.size(); i++) {
      RestrictedOption option = options.get(i);
      
      String rootPath = "observationType.restrictedOptions[" + i + "]";
      
      ValidationUtils.rejectIfEmptyOrWhitespace(errors, rootPath+".name", null, "Please name this item");
      
      if(duplicatePosition(options, option.getOrderBy())) {
        errors.rejectValue(rootPath+".orderBy", "duplicatePosition", "Duplicate position");
      }
    }
    
    
  }
  
  private boolean duplicatePosition(Collection<RestrictedOption> options, int orderBy) {
    int matchCount = 0;
    
    for (RestrictedOption option : options) {
      if (option.getOrderBy() == orderBy) {
        matchCount++;
      }
    }   
    
    return (matchCount > 1);
  }
  
  private boolean incompatibleMetaTypeChange(ObservationType newVersion, ObservationType oldVersion) {
    boolean badChange = false;
    
    boolean differentTypes = !newVersion.getObservationMetaTypeId().equals(oldVersion.getObservationMetaTypeId());    
    boolean bothUseOptions = (newVersion.isRestrictedOption() && oldVersion.isRestrictedOption());
    
    if (differentTypes && !bothUseOptions && observationsExist(newVersion.getId())) {
     badChange = true;
    }
    
    return badChange;    
  }
  
  private boolean badChangeToUnits(ObservationType newVersion, ObservationType oldVersion) {
    boolean badChange = false;
    
    String oldUnits = getUnitsSafely(oldVersion);
    String newUnits = getUnitsSafely(newVersion);
    
    if (!oldUnits.equals(newUnits) && observationsExist(newVersion.getId())) {
      badChange = true;
    }
    
    return badChange;
  }
  
  private boolean observationsExist(Long id) {
    int count = observationService.fetchByObservationType(id, null).size();
    return (count > 0);
  }

  private String getUnitsSafely(ObservationType ot) {
    if (ot.getUnits() == null) {
      return "";
    }
    else {
      return ot.getUnits();
    }
  }
  

}