/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.validators;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.mousetracking.TissueSample;
import uk.ac.sanger.mig.mousetracking.services.TissueSampleService;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ObservationService;


public class ObservationValidator implements Validator {

  private ObservationService observationService;
  private TissueSampleService tissueSampleService;
  
  public void validate(Object observation, Errors errors) {
    Observation obs = (Observation)observation;
    
    if (!obs.isValid()) {
      errors.rejectValue("value", null, "Observation is not valid");
    }
    
    boolean bloodSampleBarcode = obs.getRequiredObservation().getObservationType().isBloodSampleBarcode();
    
    if (bloodSampleBarcode && isBadBarcodeChange(obs)) {
      errors.rejectValue("value", null, "Cannot change barcode once it has been recorded");
    }
      
    if (bloodSampleBarcode && isAttemptedBarcodeReuse(obs)) {
      errors.rejectValue("value", null, "This barcode has already been used");
    }
  }

  private boolean isAttemptedBarcodeReuse(Observation observation) {
    boolean reuse = false;
    
    String barcode = observation.getValue();
    
    if (barcode != null && barcode.length() > 0) {
      TissueSample tissueSample = tissueSampleService.fetchByBarcode(barcode, null);
      reuse = (tissueSample != null);
    }
    return reuse;
  }

  private boolean isBadBarcodeChange(Observation observation) {
    boolean badChange = false;
    
    if (observation.getId() != null &&
        observation.getId() > 0) {
      
      Observation stored= observationService.fetchById(observation.getId(), null);
      if (stored.getValue() != null && stored.getValue().length() > 0) {
        // value had been recorded
        
        if (observation.getValue() == null || !observation.getValue().equals(stored.getValue())) {
          badChange = true;
        }
      }
    }
    
    return badChange;
  }
  
  public boolean supports(Class clazz) {
		return Observation.class.equals(clazz);
	}

  public void setObservationService(ObservationService observationService) {
    this.observationService = observationService;
  }

  public void setTissueSampleService(TissueSampleService tissueSampleService) {
    this.tissueSampleService = tissueSampleService;
  }

}