/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.validators;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAge;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.RecordExperimentStartForm;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentService;

public class RecordExperimentStartFormValidator implements Validator{

  private ExperimentService experimentService;

  public void setExperimentService(ExperimentService experimentService) {
    this.experimentService = experimentService;
  }

  @SuppressWarnings("unchecked")
  public boolean supports(Class clazz) {
    return RecordExperimentStartForm.class.equals(clazz);
  }

  public void validate(Object command, Errors errors) {
    RecordExperimentStartForm form = (RecordExperimentStartForm)command;

    ValidationUtils.rejectIfEmpty(errors,"experimentId",null,"An experiment ID is required");
    ValidationUtils.rejectIfEmpty(errors,"startTime",null,"A start time is required");

    Experiment e = experimentService.fetchById(
        form.getExperimentId(),
        ComponentMap.of(Experiment.Component.MOUSE)
    );

    if (e == null) {
      errors.rejectValue("experimentId",null,null,"Could not retrieve experiment "+form.getExperimentId());
    }
    else if (isOutsideMouseLifetime(e.getMouse(),form.getStartTime())) {
      String errorMsg = buildDateErrorMessage(e.getMouse());

      errors.rejectValue("startTime",null,null,errorMsg);
    }

    if (form.getStartTime() != null && isTooFarIntoTheFuture(form.getStartTime())) {
      errors.rejectValue("startTime",null,null,"Specified time is too far into the future");
    }
  }

  private boolean isTooFarIntoTheFuture(Date d) {
    boolean tooFarAhead = false;
    Date now = new Date();

    if (d == null || d.before(now)) {
      return tooFarAhead;
    }

    // get the number of milliseconds difference
    long timeDifference = Math.abs(d.getTime() - now.getTime());

    long milllisInAnHour = 1000 * 60 * 60;
    long numOfHours = 2;
    long limit = milllisInAnHour * numOfHours;

    if (timeDifference > limit) {
      tooFarAhead = true;
    }

    return tooFarAhead;
  }

  private boolean isOutsideMouseLifetime(Mouse m, Date d) {
    boolean outsideLifetime = false;

    if (d != null) {
      MouseAge mouseAge = new MouseAge(m);
      outsideLifetime = !mouseAge.isWithinLifetime(d);
    }

    return outsideLifetime;
  }

  private String buildDateErrorMessage(Mouse m) {
    SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MMM-yyyy");

    StringBuilder errorMsg = new StringBuilder();
    errorMsg.append("This time is not within the mouse lifetime (");
    if (m.getBirthDate() != null) {
      errorMsg.append("Born: ");
      errorMsg.append(dateFormat.format(m.getBirthDate()));
    }

    if (m.getBirthDate() != null &&
        m.getDeathDate()!= null) {
      errorMsg.append(", ");
    }

    if (m.getDeathDate() != null) {
      errorMsg.append("Died: ");
      errorMsg.append(dateFormat.format(m.getDeathDate()));
    }

    errorMsg.append(")");
    return errorMsg.toString();
  }

}
