/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.validators;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.StdOperatingProcedureForm;

public class StdOperatingProcedureFormValidator implements Validator {

  public boolean supports(Class clazz) {
    return StdOperatingProcedureForm.class.equals(clazz);
  }

  public void validate(Object commandForm, Errors errors) {    
    StdOperatingProcedureForm form = (StdOperatingProcedureForm) commandForm;
    StdOperatingProcedure sop = form.getStdOperatingProcedure();

    ValidationUtils.rejectIfEmptyOrWhitespace(errors, "stdOperatingProcedure.name", null, "Please name this SOP");
    
    if (sop.getProcedureId() == null || sop.getProcedureId() < 1L) {
      errors.rejectValue("stdOperatingProcedure.procedureId",null, "Please select a procedure");
    }
    
    if (sop.getRequiredObservations() != null) {
      List<RequiredObservation> requiredObservations = new ArrayList<RequiredObservation>();
      requiredObservations.addAll(sop.getRequiredObservations());

      checkRequiredObservations(requiredObservations, errors);
    }
    
    

  }

  private void checkRequiredObservations(List<RequiredObservation> requiredObservations, Errors errors) {
    for (int i = 0; i < requiredObservations.size(); i++) {
      RequiredObservation reqObv = requiredObservations.get(i);

      String rootPath = "stdOperatingProcedure.requiredObservations[" + i + "]";
      ValidationUtils.rejectIfEmptyOrWhitespace(errors, rootPath + ".displayName", null, "Please name this item");
      
      if (reqObv.getDefaultValue() != null) {
        Observation fakeObservation = new Observation();
        fakeObservation.setValue(reqObv.getDefaultValue());
        fakeObservation.setRequiredObservation(reqObv);
        
        if(!fakeObservation.isValid()) {
          errors.rejectValue(rootPath+".defaultValue","invalidDefaultValue","Please use a valid default");
        }
      }
      
      if (duplicatePosition(requiredObservations, reqObv.getOrderBy())) {
        errors.rejectValue(rootPath+".orderBy", null, "Duplicate position");
      }
      
      if (duplicateName(requiredObservations, reqObv)) {
        errors.rejectValue(rootPath+".displayName", null, "Duplicate name");
      }
    }

  }
  
  private boolean duplicatePosition(Collection<RequiredObservation> requiredObservations, int orderBy) {
    int matchCount = 0;
    
    for (RequiredObservation reqObv : requiredObservations) {
      if (reqObv.getOrderBy() == orderBy) {
        matchCount++;
      }
    }
    
    return (matchCount > 1);
  }
  
  private boolean duplicateName(Collection<RequiredObservation> requiredObservations, RequiredObservation roToTest)  {
    boolean duplicate = false;
    
    for (RequiredObservation ro : requiredObservations) {
      if (!roToTest.equals(ro) &&
          roToTest.getDisplayName().equalsIgnoreCase(ro.getDisplayName())) {
        duplicate = true;
        break;
      }
    }
    
    return duplicate;
  }
}