/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.faxitron.services;

/**
 *  Image Reader class. 2 methods taken from imageJ. One to read the 16-bit image
 *  and the other to work out sizes/buffers etc...
 * 
 */


import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.MemoryImageSource;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.imageio.ImageIO;

import uk.ac.sanger.mig.phenotyping.faxitron.DicomFilteredImage;
import uk.ac.sanger.mig.phenotyping.faxitron.DicomImage;
import uk.ac.sanger.mig.phenotyping.faxitron.DicomLevel;


public class DicomImageProcessingServiceImpl implements DicomImageProcessingService {

	private int bufferSize, byteCount, nPixels;
	private int width, height;
	private long skipCount = 0;
	private static int BYTES_PER_PIXEL = 2;
	BufferedInputStream in = null;


	private int thumbWidth;
	private int thumbHeight;


	public void setThumbHeight(int thumbHeight) {
		this.thumbHeight = thumbHeight;
	}

	public void setThumbWidth(int thumbWidth) {
		this.thumbWidth = thumbWidth;
	}


	/** Reads a 16-bit grey unsigned image.  */
	public short[] read16bitImage(BufferedInputStream in, int width, int height) throws IOException {
		

		byteCount = width*height*BYTES_PER_PIXEL;

		nPixels = width*height;
		bufferSize = byteCount/25;
		if (bufferSize<8192)
			bufferSize = 8192;
		else
			bufferSize = (bufferSize/8192)*8192;
			
		int pixelsRead;
		byte[] buffer = new byte[bufferSize];
		short[] pixels = new short[nPixels];
		int totalRead = 0;
		int base = 0;
		int count;
		int bufferCount;

		while (totalRead<byteCount) {
			if ((totalRead+bufferSize)>byteCount)
				bufferSize = byteCount-totalRead;
			bufferCount = 0;
			while (bufferCount<bufferSize) { // fill the buffer
				count = in.read(buffer, bufferCount, bufferSize-bufferCount);
				if (count==-1) {
					// EOF error.
					if (false)
						for (int i=base; i<pixels.length; i++)
							pixels[i] = (short)32768;
					return pixels;
				}
				bufferCount += count;
			}
			totalRead += bufferSize;

			pixelsRead = bufferSize/BYTES_PER_PIXEL;

			for (int i=base,j=0; i<(base+pixelsRead); i++,j+=2)
				pixels[i] = (short)(((buffer[j+1]&0xff)<<8) | (buffer[j]&0xff));

			base += pixelsRead;
		}
		
		in.close();
		
		return pixels;
	}

	


	public Image createFilteredImage(DicomImage dicomImage, DicomFilteredImage dicomFilteredImage, File destDir, short[] pixels) {

		
		int width = dicomImage.getImageWidth();
		int height = dicomImage.getImageHeight();
		int min = dicomFilteredImage.getLevel().getMin();
		int max = dicomFilteredImage.getLevel().getMax();
		
		// scale from 16-bits to 8-bits
		int size = width*height;

		byte[] pixels8 = new byte[size];
		int value;
		double scale = 256.0/(max-min+1);
		for (int i=0; i<size; i++) {
			value = (pixels[i]&0xffff)-min;
			if (value<0) value = 0;
			value = (int)(value*scale);
			if (value>255) value = 255;
			pixels8[i] = (byte)value;

		}


		IndexColorModel cm = invertLutModel();

		MemoryImageSource src = new MemoryImageSource(width, height, cm ,pixels8, 0, width);
		src.setAnimated(true);
		src.setFullBufferUpdates(true);
		
		Image im = Toolkit.getDefaultToolkit().createImage(src);

		
		return im;
	
	}

	// non-inverted LUT.
	public  IndexColorModel makeDefaultColorModel() {
		byte[] rLUT = new byte[256];
		byte[] gLUT = new byte[256];
		byte[] bLUT = new byte[256];
		for(int i=0; i<256; i++) {
			rLUT[i]=(byte)i;
			gLUT[i]=(byte)i;
			bLUT[i]=(byte)i;
		}
		return new IndexColorModel(8, 256, rLUT, gLUT, bLUT);
	}

	// these dicoms are 16bit indexed and have an inverted LUT.
	public  IndexColorModel invertLutModel() {

		IndexColorModel icm = makeDefaultColorModel();
		int mapSize = icm.getMapSize();
		byte[] reds = new byte[mapSize];
		byte[] greens = new byte[mapSize];
		byte[] blues = new byte[mapSize];	
		byte[] reds2 = new byte[mapSize];
		byte[] greens2 = new byte[mapSize];
		byte[] blues2 = new byte[mapSize];	
		icm.getReds(reds); 
		icm.getGreens(greens); 
		icm.getBlues(blues);
		for (int i=0; i<mapSize; i++) {
			reds2[i] = (byte)(reds[mapSize-i-1]&255);
			greens2[i] = (byte)(greens[mapSize-i-1]&255);
			blues2[i] = (byte)(blues[mapSize-i-1]&255);
		}
		IndexColorModel cm = new IndexColorModel(8, mapSize, reds2, greens2, blues2); 
		return cm;
	}


	public String  writeFilteredImage(File destDir, String dicomName, DicomLevel level, Image im) {

		BufferedOutputStream out = null;
		
		// remove the .dcm from the end.
		String dicomPrefix = dicomName.substring(0, dicomName.indexOf("."));
		
		String fName = dicomPrefix+ "_filtered_" +level.getMin()+"-"+level.getMax()+".jpg";
		
			try {
				out = new BufferedOutputStream(new FileOutputStream(destDir.getAbsolutePath()+"\\"+fName));

				BufferedImage buff  = createBufferedImage(im, BufferedImage.TYPE_BYTE_GRAY);
				
				ImageIO.write(buff, "jpg", out);

				out.close();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			return fName;
	}

	public byte[] writeFilteredThumbnailImage(Image im) {

		byte[] thumbBytes = null;
		try {
			thumbBytes = createBufferedThumbnail(im, BufferedImage.TYPE_BYTE_GRAY);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		return thumbBytes;
	
		
	}
	
	
	public BufferedImage createBufferedImage(Image imageIn, int imageType) {

		BufferedImage bufferedImageOut = new BufferedImage(imageIn
				.getWidth(null), imageIn.getHeight(null), imageType);
		Graphics g = bufferedImageOut.getGraphics();
		g.drawImage(imageIn, 0, 0, null);

		return bufferedImageOut;
	}


	public byte[] createBufferedThumbnail(Image imageIn, int imageType) throws IOException {

		BufferedImage bufferedImageOut = new BufferedImage(thumbWidth, thumbHeight, imageType);
		Graphics2D g = (Graphics2D)bufferedImageOut.getGraphics();
		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		g.drawImage(imageIn, 0, 0, thumbWidth, thumbHeight, null);

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ImageIO.write(bufferedImageOut, "jpg", out);
		 
		return out.toByteArray();
		
	}



}
