/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.faxitron.services;

import java.awt.Image;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.phenotyping.faxitron.DicomFilteredImage;
import uk.ac.sanger.mig.phenotyping.faxitron.DicomImage;
import uk.ac.sanger.mig.phenotyping.faxitron.DicomImageSearchBean;
import uk.ac.sanger.mig.phenotyping.faxitron.DicomLevel;
import uk.ac.sanger.mig.phenotyping.faxitron.dao.DicomImageDao;

/**
 * Service for DicomImage - information about the original dicom file.
 * 
 * @author mng
 *
 */
public class DicomImageServiceImpl extends AbstractServiceImpl<DicomImage> implements DicomImageService {

	private DicomImageDao dicomImageDao;
	private MouseService mouseService;
	private PersonService personService;
	private DicomLevelService dicomLevelService;
	private DicomParseService dicomParseService;
	private DicomImageProcessingService dicomImageProcessingService;
	private DicomFilteredImageService dicomFilteredImageService;
	
	private File dataDirectory;
	private File dicomDirectory;
	private File filteredDirectory;
	private DicomFileNameFilter dicomFileNameFilter;
	
	
	public void setDicomFilteredImageService(
			DicomFilteredImageService dicomFilteredImageService) {
		this.dicomFilteredImageService = dicomFilteredImageService;
	}

	public void setDicomImageProcessingService(DicomImageProcessingService dicomImageProcessingService) {
		this.dicomImageProcessingService = dicomImageProcessingService;
	}

	public void setDicomParseService(DicomParseService dicomParseService) {
		this.dicomParseService = dicomParseService;
	}

	public void setDicomLevelService(DicomLevelService dicomLevelService) {
		this.dicomLevelService = dicomLevelService;
	}

	public void setDicomImageDao(DicomImageDao dicomImageDao) {
		this.dicomImageDao = dicomImageDao;
	}

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	/**
	 * Method run as a quartz job. 
	 * 
	 */
	public void processDicomFiles() {

		File dataDir = getDataDirectory();     // Data to process
		File dicomDir = getDicomDirectory();   // where to put the anonymized dicoms
		File filteredDir = getFilteredDirectory();  // where to put the filtered jpegs.
		
		short[] pixels = null; // the image data.
		
		// Fetch a list of files from dataDir ending in .dcm
		for(String dicomName : fetchDicomFileNames()) {

			File dicomFile = new File(dataDir.getAbsolutePath()+"\\"+dicomName);
			File anonimyzedFile = new File(dicomDir.getAbsolutePath()+"\\"+dicomName);
			
			
			DicomImage dicomImage = null;
			try {
				// parse the dicom for information.
				dicomImage = dicomParseService.parseDicom(dicomFile);
			} catch (IOException e) {
				logger.warn("Unable to parse dicom file "+dicomFile);
				e.printStackTrace();
			} 
			
			if(dicomImage != null) {
				
				if(dicomImage.getOperatorId() != null) {

					// who took the x-ray
					dicomImage.setDicomName(dicomName);
					
					// location of the original dicom that we read 
					dicomImage.setDicomPathOriginal(dataDir.getAbsolutePath());
					
					// Read the dicom image, setting the pixel data. file is closed in this method.	
					try {
						pixels = dicomImageProcessingService.read16bitImage(dicomImage.getDicomStream(), dicomImage.getImageWidth(), dicomImage.getImageHeight());
						if(pixels.length == 0) {
							logger.warn("Failed to read pixels for dicom "+dicomFile);
							continue;
						}
					} catch (IOException e) {
						logger.warn("Unable to read dicom image for dicom file "+dicomFile);
						e.printStackTrace();
						continue;
					}
					
					// anonymize the dicom file and write a new one out. This will be the 
					// externally available version.
					dicomParseService.anonymizeDicom(dicomFile, anonimyzedFile);
					
					dicomImage.setDicomPath(dicomDir.getAbsolutePath());
					
					// store info about dicom file.
					Long dicomId = storeAuthenticated(dicomImage, dicomImage.getOperator());
					
					
					
					// fetch all filtering levels for the images.	
					Collection<DicomLevel> levels = dicomLevelService.fetchAll();
					
					if(levels !=null) {	
						for(DicomLevel level : levels) {

							DicomFilteredImage filteredImage = new DicomFilteredImage();
							filteredImage.setDicomId(dicomId);
							
							// set the level used for this image.
							filteredImage.setLevelId(level.getId());
							filteredImage.setLevel(level);

							// Create the filtered image.
							Image image = dicomImageProcessingService.createFilteredImage(dicomImage, filteredImage, filteredDir, pixels);

							// set the filtered image information.
							filteredImage.setFilteredDicomPath(filteredDir.getAbsolutePath());
							// save the filtered jpeg, store the name...
							try {
								filteredImage.setFilteredDicomName(dicomImageProcessingService.writeFilteredImage(filteredDir, dicomImage.getDicomName(), level, image));
							} catch (FileNotFoundException e) {
								logger.warn("Unable to wrote filtered jpeg file for "+dicomFile+" with levels "+level.getMin()+" - "+level.getMax());
								e.printStackTrace();
								continue;
							}
							
							// create thumbnail image and return the byte[]
							try {
								filteredImage.setFilteredThumbnail(dicomImageProcessingService.writeFilteredThumbnailImage(image));
							} catch (FileNotFoundException e) {
								logger.warn("Unable to wrote filtered thumbnail for "+dicomFile+" with levels "+level.getMin()+" - "+level.getMax());
								e.printStackTrace();
								continue;
							}
							
							// store the info.
							dicomFilteredImageService.storeAuthenticated(filteredImage, dicomImage.getOperator());
						}
					}
				
				}else{

					logger.warn("Skipping file "+dicomFile.getName()+" as no valid operator found.");
				}	
			}
		}

		
	}
	
	
	
	public void setDataDirectory(File dataDirectory) {

		if (dataDirectory.canRead()) {

			this.dataDirectory = dataDirectory;
			logger.info("Found required writable system directory: " + dataDirectory);

		} else {
			logger.warn("Failed to locate required system directory " + dataDirectory);
		}
	}

	public void setDicomDirectory(File dicomDirectory) {

		if (dicomDirectory.canWrite()) {

			this.dicomDirectory = dicomDirectory;
			logger.info("Found required writable system directory: " + dicomDirectory);

		} else {
			logger.warn("Failed to locate required system directory " + dicomDirectory);
		}
	}
	
	public void setFilteredDirectory(File filteredDirectory) {

		if (filteredDirectory.canWrite()) {

			this.filteredDirectory = filteredDirectory;
			logger.info("Found required writable system directory: " + filteredDirectory);

		} else {
			logger.warn("Failed to locate required system directory " + filteredDirectory);
		}
	}

	public void setDicomFileNameFilter(DicomFileNameFilter dicomFileNameFilter) {
		this.dicomFileNameFilter = dicomFileNameFilter;
	}

	public String[] fetchDicomFileNames() {	
		
		String[] fileNames = dataDirectory.list( dicomFileNameFilter );
			
		logger.debug("Found " + fileNames.length + " file(s) in " + dataDirectory);
		
		return fileNames;
	}
	
	public Long storeAuthenticated(DicomImage dicomImage, Person user) {
		if(dicomImage == null) {
			throw new RuntimeException("Cannot save a null dicomInfo");
		}

		return dicomImageDao.store(dicomImage, user);
	}

	public DicomImage fetchById(Long dicomImageId, ComponentMap requiredComponents) {
		DicomImage dicomImage = dicomImageDao.fetchById(dicomImageId);

		if(dicomImage != null)
			attachComponents(dicomImage, requiredComponents);

		return dicomImage;
	}

	@Override
	protected void attachComponents(DicomImage dicomImage, ComponentMap requiredComponents) {
		if(dicomImage == null) {
			throw new RuntimeException("Cannot attach components to null dicomImage");
		}		

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(DicomImage.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(DicomImage.Component.MOUSE)) {
				dicomImage.setMouse(fetchMouse(dicomImage, null));
			}

			if(fetchAll || requiredComponents.containsKey(DicomImage.Component.PERSON)) {
				dicomImage.setOperator(fetchPerson(dicomImage, requiredComponents.get(Person.Component.ALL_COMPONENTS)));
			}

			
		}
	}

	
	private Mouse fetchMouse(DicomImage info, ComponentMap requiredComponents) {
		if(info.getMouseId() != null) {
			return mouseService.fetchById(info.getMouseId(), requiredComponents);
		}
		else {
			return null;
		}
	}		

	private Person fetchPerson(DicomImage info, ComponentMap requiredComponents) {
		if(info.getOperatorId() != null) {
			return personService.fetchById(info.getOperatorId(), requiredComponents);
		}
		else {
			return null;
		}
	}		

	public Collection<DicomImage> fetchDicomImagesByMouseId(Long mouseId) {
		return dicomImageDao.fetchDicomImagesByMouseId(mouseId);
	}

	public File getDataDirectory() {
		return dataDirectory;
	}

	
	public File getDicomDirectory() {
		return dicomDirectory;
	}

	public File getFilteredDirectory() {
		return filteredDirectory;
	}

	public Collection<DicomImage> search(DicomImageSearchBean dicomImageSearchBean) {
		// TODO Auto-generated method stub
		return null;
	}

	
}
