/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.faxitron.services;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.phenotyping.faxitron.DicomImage;

/**
 * Class to parse a Dicom image and write out and anonymized version.
 * Originally taken from ImageJ and Anonymizer plugin code.
 * 
 * @author mng
 *
 */
class DicomParseServiceImpl implements DicomParseService {


	private static final int TRANSFER_SYNTAX_UID = 0x00020010;
	private static final int ICON_IMAGE_SEQUENCE = 0x00880200;
	private static final int ITEM = 0xFFFEE000;
	private static final int ITEM_DELIMINATION = 0xFFFEE00D;
	private static final int SEQUENCE_DELIMINATION = 0xFFFEE0DD;
	private static final int PIXEL_DATA = 0x7FE00010;

	// Fields needed.
	private static final int RUN_DATE = 0x00080020;
	private static final int RUN_TIME = 0x00080030;
	private static final int OPERATOR_NAME = 0x00081070;
	private static final int PATIENT_NAME = 0x00100010;
	private static final int PATIENT_ID = 0x00100020;
	private static final int PATIENT_COMMENTS = 0x00104000;
	private static final int IMAGE_WIDTH = 0x00280011;
	private static final int IMAGE_HEIGHT = 0x00280010;
	private static final int DOSE = 0x30040012;
	private static final int IMAGE_ID = 0x00540400;
	private static final int PIXEL_SPACING = 0x00280030;


	private static final int AE=0x4145, AS=0x4153, AT=0x4154, CS=0x4353, DA=0x4441, DS=0x4453, DT=0x4454,
	FD=0x4644, FL=0x464C, IS=0x4953, LO=0x4C4F, LT=0x4C54, PN=0x504E, SH=0x5348, SL=0x534C, 
	SS=0x5353, ST=0x5354, TM=0x544D, UI=0x5549, UL=0x554C, US=0x5553, UT=0x5554,
	OB=0x4F42, OW=0x4F57, SQ=0x5351, UN=0x554E, QQ=0x3F3F;

	public static final char[] hexDigits = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

	private static Properties dictionary;

	private String directory;
	private File fileName;
	private static final int ID_OFFSET = 128;  //location of "DICM"
	private static final String DICM = "DICM";

	private int location = 0;
	private boolean littleEndian = true;

	private int elementLength;
	private int vr;  // Value Representation
	private static final int IMPLICIT_VR = 0x2D2D; // '--' 

	private int previousGroup;
	private String previousInfo;

	private boolean dicmFound; // "DICM" found at offset 128
	private boolean oddLocations;  // one or more tags at odd locations
	private boolean bigEndianTransferSyntax = false;

	boolean inSequence;
	private BufferedInputStream f = null;

	DicomImage info = null;

	boolean log = false;

	int operator_len = 0, pname_len = 0, pid_len = 0, pcomm_len = 0;
	int operator_loc = 0, pname_loc = 0, pid_loc = 0, pcomm_loc = 0;

	double pixelWidth = 0, pixelHeight = 0;


	private PersonService personService;
	private MouseService mouseService;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd:HHmmss");

	public DicomParseServiceImpl() {

		if (dictionary==null) {
			DicomTagDictionary d = new DicomTagDictionary();
			dictionary = d.getDictionary();
		}

	}

	String getString(int length) throws IOException {
		byte[] buf = new byte[length];
		int pos = 0;
		while (pos<length) {
			int count = f.read(buf, pos, length-pos);
			pos += count;
		}
		location += length;
		return new String(buf);
	}

	int getByte() throws IOException {
		int b = f.read();
		if (b ==-1) throw new IOException("unexpected EOF");
		++location;
		return b;
	}

	int getShort() throws IOException {
		int b0 = getByte();
		int b1 = getByte();
		if (littleEndian)
			return ((b1 << 8) + b0);
		else
			return ((b0 << 8) + b1);
	}

	final int getInt() throws IOException {
		int b0 = getByte();
		int b1 = getByte();
		int b2 = getByte();
		int b3 = getByte();
		if (littleEndian)
			return ((b3<<24) + (b2<<16) + (b1<<8) + b0);
		else
			return ((b0<<24) + (b1<<16) + (b2<<8) + b3);
	}



	int getLength() throws IOException {
		int b0 = getByte();
		int b1 = getByte();
		int b2 = getByte();
		int b3 = getByte();

		// We cannot know whether the VR is implicit or explicit
		// without the full DICOM Data Dictionary for public and
		// private groups.

		// We will assume the VR is explicit if the two bytes
		// match the known codes. It is possible that these two
		// bytes are part of a 32-bit length for an implicit VR.

		vr = (b0<<8) + b1;

		switch (vr) {
		case OB: case OW: case SQ: case UN:
			// Explicit VR with 32-bit length if other two bytes are zero
			if ( (b2 == 0) || (b3 == 0) ) return getInt();
			// Implicit VR with 32-bit length
			vr = IMPLICIT_VR;
			if (littleEndian)
				return ((b3<<24) + (b2<<16) + (b1<<8) + b0);
			else
				return ((b0<<24) + (b1<<16) + (b2<<8) + b3);		
		case AE: case AS: case AT: case CS: case DA: case DS: case DT:  case FD:
		case FL: case IS: case LO: case LT: case PN: case SH: case SL: case SS:
		case ST: case TM:case UI: case UL: case US: case UT: case QQ:
			// Explicit vr with 16-bit length
			if (littleEndian)
				return ((b3<<8) + b2);
			else
				return ((b2<<8) + b3);
		default:
			// Implicit VR with 32-bit length...
			vr = IMPLICIT_VR;
		if (littleEndian)
			return ((b3<<24) + (b2<<16) + (b1<<8) + b0);
		else
			return ((b0<<24) + (b1<<16) + (b2<<8) + b3);
		}
	}

	int getNextTag() throws IOException {
		int groupWord = getShort();
		if (groupWord==0x0800 && bigEndianTransferSyntax) {
			littleEndian = false;
			groupWord = 0x0008;
		}
		int elementWord = getShort();
		int tag = groupWord<<16 | elementWord;
		elementLength = getLength();

		// hack needed to read some GE files
		// The element length must be even!
		if (elementLength==13 && !oddLocations) elementLength = 10; 

		// "Undefined" element length.
		// This is a sort of bracket that encloses a sequence of elements.
		if (elementLength==-1) {
			elementLength = 0;
			inSequence = true;
		}
		if(log) { System.out.println("getNextTag: "+tag+" "+elementLength); }
		return tag;
	}

	/**
	 *  Main Method for parsing.
	 */
	public DicomImage parseDicom(File dicomFile) throws IOException {

		// reset all the position. 
		location = 0;

		try {
			this.f = new BufferedInputStream(new FileInputStream(dicomFile));
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
			return null;
		}

		this.info = new DicomImage();


		long skipCount = (long)ID_OFFSET;
		while (skipCount > 0) skipCount -= f.skip( skipCount );
		location += ID_OFFSET;

		if (!getString(4).equals(DICM)) {
			f.close();
			f = new BufferedInputStream(f);
			location = 0;
			if (log) System.out.println(DICM + " not found at offset "+ID_OFFSET+"; reseting to offset 0");
		} else {
			dicmFound = true;
			if (log) System.out.println(DICM + " found at offset " + ID_OFFSET);
		}

		/**
		 * Read through the dicom, find the endian'ness.
		 * Other fields are the ones we want to remove when we write
		 * out the anonymized version OR information we want to store 
		 * in the database.
		 */
		boolean inSequence = true;
		boolean decodingTags = true;
		String runDate = null;
		while (decodingTags) {
			int tag = getNextTag();
			if ((location&1)!=0) // DICOM tags must be at even locations
				oddLocations = true;
			String s;
			switch (tag) {
			// Work out whether this is big or little endian.
			case TRANSFER_SYNTAX_UID:
				s = getString(elementLength);
				if (s.indexOf("1.2.4")>-1||s.indexOf("1.2.5")>-1) {
					f.close();
					String msg = "cannot open compressed DICOM images.\n \n";
					msg += "Transfer Syntax UID = "+s;
					throw new IOException(msg);
				}
				if (s.indexOf("1.2.840.10008.1.2.2")>=0)
					bigEndianTransferSyntax = true;
				break;

			case RUN_DATE:
				runDate = getString(elementLength);

				break;
			case RUN_TIME:
				if(runDate != null) {
					// add the time string to the date string and create the 
					// date object.
					String time = getString(elementLength);

					runDate = runDate.concat(":"+time);
					Date date = null;
					try {
						date = format.parse(runDate);
					} catch (ParseException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					info.setImageDate(date);
				}
				break;
			case OPERATOR_NAME:
				String opName = getString(elementLength).trim();

				Person person = personService.fetchByUserName(opName, null);
				if(person != null) {
					info.setOperatorId(person.getId());
					info.setOperator(person); //  save this as needed for store authenticated.
				}
				operator_len = elementLength;
				operator_loc = location - elementLength;
				break;
			case PATIENT_NAME: 
				String patientName = getString(elementLength);
				// faxitron enforces first, middle and last names. not needed but blank'able
				pname_len = elementLength;
				pname_loc = location - elementLength;
				break;
			case PATIENT_ID: 
				// get the mouse id. This is prefixed by ":  " so skip that. try a few
				// methods to get the mouse.
				String patientId = getString(elementLength).substring(3);
				//Mouse mouse = getMouseByPatientId(patientId);
				Mouse mouse = mouseService.fetchByBarcode(patientId, null);
				if(mouse != null) {
					info.setMouseId(mouse.getId());
				}
				pid_len = elementLength;
				pid_loc = location - elementLength;
				break;
			case PATIENT_COMMENTS:
				String comment = getString(elementLength);
				info.setComments(comment);
				pcomm_len = elementLength;
				pcomm_loc = location - elementLength;
				break;
			case IMAGE_HEIGHT:
				int height = getShort();

				info.setImageHeight(height);
				break;
			case IMAGE_WIDTH:
				int width = getShort();

				info.setImageWidth(width);
				break;
			case DOSE:
				String dose = getString(elementLength);

				info.setDose(dose);		
				break;
			case IMAGE_ID:
				String imageId = getString(elementLength);

				info.setDicomName(imageId);	
				break;
			case PIXEL_SPACING:
				String spacing = getString(elementLength);
				setSpatialScale(spacing);
				break;
			case PIXEL_DATA:
				// Start of image data...
				if (elementLength!=0) {	
					decodingTags = false;
				}
				break;

			default:
				// Not used, skip over it...
				addInfo(tag, null);

			} // end switch
		} // while(decodingTags)

		// set the resolution
		info.setResolutionPerMM((int)info.getImageWidth() / (int)(info.getImageWidth() * pixelWidth));

		info.setDicomStream(f);


		return info;

	}

	private Mouse getMouseByPatientId(String patientId) {
		Mouse mouse = null;
		// try barcode
		mouse =  mouseService.fetchByBarcode(patientId, null);
		if(mouse == null) {
			// then try name...
			mouse = mouseService.fetchByName(patientId, null);
		}
		return mouse;  
	}

	void addInfo(int tag, String value) throws IOException {
		String info = getHeaderInfo(tag, value);
		if (inSequence && info!=null && vr!=SQ) info = ">" + info;
		if (info!=null &&  tag!=ITEM) {
			int group = tag>>>16;

			previousGroup = group;
			previousInfo = info;

		}
	}

	void addInfo(int tag, int value) throws IOException {
		addInfo(tag, Integer.toString(value));
	}

	String getHeaderInfo(int tag, String value) throws IOException {
		if (tag==ITEM_DELIMINATION || tag==SEQUENCE_DELIMINATION) {
			inSequence = false;
			if (!log) return null;
		}
		String key = i2hex(tag);
		String id = (String)dictionary.get(key);
		if (id!=null) {
			if (vr==IMPLICIT_VR && id!=null)
				vr = (id.charAt(0)<<8) + id.charAt(1);
			id = id.substring(2);
		}
		if (tag==ITEM)
			return id!=null?id+":":null;
		if (value!=null)
			return id+": "+value;
		switch (vr) {
		case AE: case AS: case AT: case CS: case DA: case DS: case DT:  case IS: case LO: 
		case LT: case PN: case SH: case ST: case TM: case UI:
			value = getString(elementLength);
			break;
		case US:
			if (elementLength==2)
				value = Integer.toString(getShort());
			else {
				value = "";
				int n = elementLength/2;
				for (int i=0; i<n; i++)
					value += Integer.toString(getShort())+" ";
			}
			break;
		case IMPLICIT_VR:
			value = getString(elementLength);
			if (elementLength<=4 || elementLength>44) value=null;
			break;
		case SQ:
			value = "";
			boolean privateTag = ((tag>>16)&1)!=0;
			if (tag!=ICON_IMAGE_SEQUENCE && !privateTag)
				break;
			// else fall through and skip icon image sequence or private sequence
		default:
			long skipCount = (long)elementLength;
		while (skipCount > 0) skipCount -= f.skip(skipCount);
		location += elementLength;
		value = "";
		}
		if (value!=null && id==null && !value.equals("")) {
			return "---: "+value;
		}else if (id==null) {
			return null;
		}else {

			return id+": "+value;
		}
	}

	static char[] buf8 = new char[8];

	/** Converts an int to an 8 byte hex string. */
	String i2hex(int i) {
		for (int pos=7; pos>=0; pos--) {
			buf8[pos] = hexDigits[i&0xf];
			i >>>= 4;
		}
		return new String(buf8);
	}


	boolean dicmFound() {
		return dicmFound;
	}


	double s2d(String s) {
		Double d;
		try {d = new Double(s);}
		catch (NumberFormatException e) {d = null;}
		if (d!=null)
			return(d.doubleValue());
		else
			return(0.0);
	}

	void setSpatialScale(String scale) {
		double xscale=0, yscale=0;
		int i = scale.indexOf('\\');
		if (i>0) {
			yscale = s2d(scale.substring(0, i));
			xscale = s2d(scale.substring(i+1));
		}
		if (xscale!=0.0 && yscale!=0.0) {
			pixelWidth = xscale;
			pixelHeight = yscale;
		}
	}

	public void anonymizeDicom(File dicomFile, File destination) {


		// Read entire dicom into byte array.
		try {
			BufferedInputStream f = new BufferedInputStream(new FileInputStream(dicomFile));
			int totalFileLen = f.available();
			byte[] fBufCopy = new byte[totalFileLen];
			f.read(fBufCopy, 0, totalFileLen);
			f.close();

			// add a sanger into the 128byte header.
			String generator = "Generated by the Sanger Institute";
			byte[] head = generator.getBytes();
			for (int i=0; i< head.length; i++)
				fBufCopy[i] = head[i];
			
			// zero all the sensitive data. these fields are found within the parseDicom
			// method.
			if(operator_len > 0)
				for (int i=0; i< operator_len; i++)    fBufCopy[i+operator_loc] = 0;

			if(pname_len > 0)
				for (int i=0; i< pname_len; i++)    fBufCopy[i+pname_loc] = 0;

			if(pid_len > 0)
				for (int i=0; i< pid_len; i++)  fBufCopy[i+pid_loc] = 0;

			if(pcomm_len > 0)
				for (int i=0; i< pcomm_len; i++)  fBufCopy[i+pcomm_loc] = 0;

			// write back into same filename but in destination directory.
			BufferedOutputStream bos = 
				new BufferedOutputStream(new FileOutputStream(destination));
			bos.write(fBufCopy, 0, totalFileLen); 
			bos.close();

		} 
		catch (Exception e) {
			System.out.println("Error opening "+destination.getPath()+"\n \n\""+e.getMessage()+"\"");
		}
	}


}                    



