/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package uk.ac.sanger.mig.phenotyping.olympus.controllers;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.controllers.AbstractFormChangeController;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.phenotyping.abc.BloodsData;
import uk.ac.sanger.mig.phenotyping.olympus.SerumTest;
import uk.ac.sanger.mig.phenotyping.olympus.forms.SerumChemistryForm;
import uk.ac.sanger.mig.phenotyping.olympus.services.SerumTestService;

public class NewSerumChemistryFormController extends AbstractFormChangeController {

	private MouseService mouseService;
	private DictionaryService dictionaryService;
	private SerumTestService serumTestService;

	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setSerumTestService(SerumTestService serumTestService) {
		this.serumTestService = serumTestService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	@Override
	protected void onFormChange(HttpServletRequest request,
			HttpServletResponse response, Object command, BindException errors) throws Exception {

		SerumChemistryForm form = (SerumChemistryForm)command;
		String changeRequest = ServletRequestUtils.getRequiredStringParameter(request, "changeForm");

		if(changeRequest.equals("addTest")) {
			addTest(form);
		}else if(changeRequest.equals("removeTest")) {
			String removeBarcode = ServletRequestUtils.getRequiredStringParameter(request, "removeBarcode");
			removeTest(form,removeBarcode);
		}

		form.setBarcodeDescription(null);
		form.setBloodBarcode(null);
		form.setHitachiBarcode(null);
		form.setTissueId(null);

		// Empty form values.
		form.setChangeForm(null);
		form.setChangeFormValue(null);

	}

	private void addTest(SerumChemistryForm form) {

		SerumTest test = new SerumTest();
		test.setBarcode(form.getHitachiBarcode());
		// Get the mouse id from the tissue (from the blood tube barcode)
		Mouse mouse = mouseService.fetchMouseByTissueSampleId(form.getTissueId(), null);
		if(mouse != null) {
			test.setMouseId(mouse.getId());
		}
		test.setTestCodes(form.getSerumTestCodes());
		test.setBloodBarcode(form.getBloodBarcode());
		test.setProfileName(form.getProfileName());
		form.getSerumTests().add(test);
		form.incrTestCount();

	}

	private void removeTest(SerumChemistryForm form, String removeBarcode) {

		Collection<SerumTest> tests = form.getSerumTests();
		for(SerumTest test : tests) {
			if(test.getBarcode().equals(removeBarcode)) {
				tests.remove(test);
				form.getSerumTests().remove(tests);
				form.decrTestCount();
				break;
			}
		}

	}


	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors) throws Exception {
		SerumChemistryForm form = (SerumChemistryForm)command;

		Person user = RequestUtils.getUser(request);

		Collection<SerumTest> tests = form.getSerumTests();

		for(SerumTest test : tests) {
			// store each test.
			serumTestService.storeAuthenticated(test, user);
		}


		return new ModelAndView(getSuccessView());
	}

	@Override
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		SerumChemistryForm serumForm = (SerumChemistryForm)createCommand();

		Collection<BloodsData> testCodes = new ArrayList<BloodsData>();


		// temporary.
		Collection<DictionaryEntry> entries = dictionaryService.fetchDependant(DictionaryEnum.PHENOTYPING_PARAMS, 1L, "ACTIVE");

		for(DictionaryEntry entry : entries) {
			BloodsData code = new BloodsData();
			code.setParameterId(Long.valueOf(entry.getDescription()));
			code.setValue(entry.getName());
			testCodes.add(code);
		}
		serumForm.setTestCodes(testCodes);

		Collection<SerumTest> presets = serumTestService.fetchTestPresets();
		serumForm.setPresetTests(presets);

		return serumForm;
	}

	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest) throws Exception {
		Map<String,Object> formMetaData = new HashMap<String,Object>();

		HashMap<Integer,String> methods = new HashMap<Integer, String>();
		methods.put(0,"Normal");
		methods.put(1,"Dilution");
		methods.put(2,"Condense");

		formMetaData.put("methods", methods);


		SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");
		String today = sdf.format(new Date());

		formMetaData.put("today", today);

		return formMetaData;
	}
}