/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.olympus.views;

import java.awt.Color;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import uk.ac.sanger.mig.charting.ChartData;
import ChartDirector.Chart;
import ChartDirector.LineLayer;
import ChartDirector.MultiChart;
import ChartDirector.XYChart;


public class SerumQCChart {

	private int width;
	private int height;
	private ChartData data;
	private MultiChart chart;
	private Double mean;
	private Double standardDeviation;
	private Double range;

	public MultiChart getChart() {
		return chart;
	}

	public SerumQCChart(int width, int height, ChartData data, Double mean, Double standardDeviation, Double range) {

		this.width = width;
		this.height = height;
		this.data = data;
		this.mean = mean;
		this.standardDeviation = standardDeviation;
		this.range = range;

		XYChart deviationChart = createDeviationChart();
		XYChart rangeChart = createRangeChart();

		MultiChart m = new MultiChart(600, 370);
		m.setBorder(Chart.CColor(Color.BLACK));

		m.addChart(0, 0, deviationChart);

		m.addChart(0, 215, rangeChart);

		chart = m;
	}

	private XYChart createRangeChart() {

		XYChart rangeChart = new XYChart(width, height, 0xeeeeff, Chart.Transparent, 0);
		rangeChart.setPlotArea(40, 5, 500, 100, rangeChart.linearGradientColor(0, 55, 0, 335, 0xf9fcff, 0xaaccff),-1, -1, 0xcccccc, 0xcccccc);

		rangeChart.yAxis().setColors(0x000000, 0x000000, 0x000000);

		String[] st = {"0","1R","2R","3R"};
		rangeChart.yAxis().setLabels(st);

		// Strings for labels, Format the calculations to 2 decimal places.
		DecimalFormat df2 = new DecimalFormat( "#########0.00" );
		Double[] vXaxis = {
				new Double(df2.format(range)).doubleValue(),
				new Double(df2.format(range * 2)).doubleValue(),
				new Double(df2.format(range * 3)).doubleValue(),
		};
		String[] x2 = new String[4];
		x2[0] = "0";
		int i = 1;
		for(Double d : vXaxis) {
			x2[i] = d.toString();
			i++;
		}

		//String[] x2 = {new String("0"),Double.toString(range), Double.toString(range*2), Double.toString(range*3)};

		rangeChart.yAxis2().setColors(0x000000, 0x000000, 0x000000);
		rangeChart.setClipping();
		rangeChart.yAxis2().setLinearScale2(0, range*3, x2);


//		To use MultiFormat, the axis must be a date, parse the String[] to Date[]
		Date[] newLabels = parseLabelToDate(data.getLabels());
		rangeChart.xAxis().setLabels(newLabels);
		rangeChart.xAxis().setLabelStyle().setFontAngle(45.0); // angled on 45 degrees
		if(data.getLabels().length > 500) {	
			rangeChart.xAxis().setMultiFormat(Chart.StartOfMonthFilter(1,0.01), "<*font=bold*>{value|mmm}");
		}else if(data.getLabels().length > 120) {
			rangeChart.xAxis().setMultiFormat(Chart.StartOfDayFilter(1,0.01), "<*font=bold*>{value|mmm-dd}");
		}else{
			rangeChart.xAxis().setMultiFormat(Chart.StartOfDayFilter(1,0.01), "<*font=bold*>{value|mmm-dd}", Chart.AllPassFilter(), "<*font=small*>{value|hh:nn}");
		}
//		work out the tick spacing.
		int tickSpace = data.getData(0).length / 50;
		rangeChart.xAxis().setLabelStep(tickSpace);

		LineLayer layer1 = rangeChart.addLineLayer();
		layer1.addDataSet(data.getData(0), 0x000000, "Value").setDataSymbol(Chart.SquareSymbol,2);
		layer1.setLineWidth(1);
		layer1.setUseYAxis2();

		return rangeChart;
	}

	private XYChart createDeviationChart() {

		XYChart deviation = new XYChart(width, height, 0xeeeeff, Chart.Transparent, 0);
		deviation.setPlotArea(40, 10, 500, 180, deviation.linearGradientColor(0, 55, 0, 335, 0xf9fcff, 0xaaccff),-1, -1, 0xcccccc, 0xcccccc);

		deviation.yAxis().setColors(0x000000, 0x000000, 0x000000);

		String[] st = {"\\-3SD","\\-2SD","\\-1SD","Mean","+1SD","+2SD","+3SD"};
		deviation.yAxis().setLabels(st);

		// Strings for the labels, escape - for negative values otherwise they wont show up.
		// Format the calculations to 2 decimal places.
		DecimalFormat df2 = new DecimalFormat( "#########0.00" );
		Double[] vXaxis = {
				new Double(df2.format(mean - (3*standardDeviation))).doubleValue(),
				new Double(df2.format(mean - (2*standardDeviation))).doubleValue(),
				new Double(df2.format(mean - standardDeviation)).doubleValue(),
				new Double(df2.format(mean)).doubleValue(),
				new Double(df2.format(mean + standardDeviation)).doubleValue(),
				new Double(df2.format(mean + (2*standardDeviation))).doubleValue(),
				new Double(df2.format(mean + (3*standardDeviation))).doubleValue()
		};
		String[] x2 = new String[7];
		int i = 0;
		for(Double d : vXaxis) {
			if(d < 0) {
				x2[i] = "\\" + d.toString();
			}else{
				x2[i] = d.toString();
			}
			i++;
		}

		// Draw a dashed red line to indicate where zero is (if displayed)
		deviation.yAxis2().addMark(0, deviation.dashLineColor(0xff0000, Chart.DotLine)).setDrawOnTop(false);
		
		deviation.yAxis2().setColors(0x000000, 0x000000, 0x000000);
		deviation.setClipping();
		deviation.yAxis2().setLinearScale2(mean - (3*standardDeviation), mean + (3*standardDeviation),x2);

		String[] newLabels = parseLabelToBlank(data.getLabels());
		deviation.xAxis().setLabels(newLabels);

//		work out the tick spacing.
		int tickSpace = data.getData(0).length / 50;
		deviation.xAxis().setLabelStep(tickSpace);

		LineLayer layer1 = deviation.addLineLayer();
		layer1.addDataSet(data.getData(0), 0x000000, "Value").setDataSymbol(Chart.SquareSymbol,2);
		layer1.setLineWidth(1);
		layer1.setUseYAxis2();

		return deviation;
	}




	private Date[] parseLabelToDate(String[] labels) {
		Date[] newLabels = new Date[labels.length];

		int i = 0;
		for(String label : labels) {
			try {
				newLabels[i++] = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").parse(label);
			} catch (ParseException e) {
//				TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return newLabels;
	}

	private String[] parseLabelToBlank(String[] labels) {
		String[] newLabels = new String[labels.length];

		int i = 0;
		for(String label : labels) {

			newLabels[i++] = new String(" ");

		}

		return newLabels;
	}

}


