/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.pubmedsorter;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.pubmedsorter.services.ArticleService;

public class PubmedSearchEngine {
	
	private List<String> pubmedIds;
	private Map<String,Article> articles;
	private ArticleService articleService;
	private Long migGeneId;
		
	public void setArticleService(ArticleService articleService) {
		this.articleService = articleService;
	}
	
	public PubmedSearchEngine(){
		pubmedIds = new ArrayList<String>();
		articles =  new HashMap<String,Article>();
		Properties systemSettings = System.getProperties();
		systemSettings.put("proxySet", "true");
		systemSettings.put("proxyHost", "wwwcache.sanger.ac.uk");
		systemSettings.put("proxyPort", "3128");
		System.setProperties(systemSettings);
	}
	

	
	public void fetchPubmedIds(String keywords){
		
		if (keywords != null) {
			try{
				//Search PubMed to get the results list of pubmedIds and save the top 50 results in an array 
				String eUtilsResultsURLName = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term="+keywords+"&retmax=500";
				URL resultsURL = new URL(eUtilsResultsURLName);
				DocumentBuilderFactory resultsFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder resultsBuilder = resultsFactory.newDocumentBuilder();
				Document resultsDoc = resultsBuilder.parse(resultsURL.toString());
				
				Element eSearchResultsRoot = resultsDoc.getDocumentElement();
				NodeList resultNodes =  eSearchResultsRoot.getElementsByTagName("Id");
				for (int i = 0; i < resultNodes.getLength(); i++)
				{
					Element aNode = (Element) resultNodes.item(i);
					pubmedIds.add(i, returnElementData(aNode));
					
				}
			}catch(SAXException se){
				se.printStackTrace();
			}catch(IOException ioe){
				ioe.printStackTrace();
			}catch(ParserConfigurationException e) {
				e.printStackTrace();
			}
		}
	}
	
	public Article fetchArticle(String pubmedId, Person user){
		
		Article article = null;
		
		//first check if article is loaded into the memory cache
		if(articles.get(pubmedId) == null) {
			//if not loaded, then we need to load it.
			//first try fetching from our database
			
			ComponentMap requiredComponents = new ComponentMap(Article.Component.ALL_COMPONENTS, null);
						
			article = articleService.fetchByPubmedId(pubmedId, requiredComponents);
			
			if(article != null) {
				//if article is successfully fetched from database, add to cache
				articles.put(article.getPubmedId(), article);
				//articleList.add(article);
			} else {
				//if article failed to fetch from database, then fetch from pubmed
				fetchArticleFromPubmed(pubmedId, user);
			}
		}
		return articles.get(pubmedId);
	}
	
	
	private void fetchArticleFromPubmed(String pubmedId, Person user){
		
		Article article = new Article();
		try{
			
			
			//Fetch the xml page relating to the pubmed ID from pubmed and puts it into a document object  
			
			String articleURLName = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&id="+pubmedId+"&retmode=xml";
			URL articleURL = new URL(articleURLName);
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			//Parse the content of the given URL as an XML document and return a new DOM Document object.
			Document doc = builder.parse(articleURL.toString());
			Element root = doc.getDocumentElement();
			Element pubmedArticleElement = getElement(root, "PubmedArticle");
			
			if(getElement(pubmedArticleElement, "MedlineCitation") != null){
				
				Element medlineCitationElement = getElement(pubmedArticleElement, "MedlineCitation");

				if(getElement(medlineCitationElement, "PMID") != null){
					Element pubmedIdElement = getElement(medlineCitationElement, "PMID");
					String pmidString = returnElementData(pubmedIdElement);
					article.setPubmedId(pmidString);		
				}
				
				Element pubDateElement = (Element)medlineCitationElement.getElementsByTagName("PubDate").item(0);
				String yearString = null,monthString = null,dayString = null;
				if(pubDateElement != null){
					Element yearElement = getElement(pubDateElement, "Year");
					if(yearElement != null){
						yearString = returnElementData(yearElement);
						int year = Integer.parseInt(yearString);
						article.setYear(year);
					}
					Element monthElement = getElement(pubDateElement, "Month");
					if(monthElement != null){
						monthString = returnElementData(monthElement);
						article.setMonth(monthString);
					}
					Element dayElement = getElement(pubDateElement, "Day");
					if(dayElement != null){
						dayString = returnElementData(dayElement);
						int day = Integer.parseInt(dayString);
						article.setDay(day);
					}
				}
				
				Element journalTitleElement = (Element)medlineCitationElement.getElementsByTagName("Title").item(0);
				if(journalTitleElement == null){
					journalTitleElement = (Element)medlineCitationElement.getElementsByTagName("MedlineTA").item(0);
				}
				if(journalTitleElement != null){
				String journalString = returnElementData(journalTitleElement);
				article.setJournalTitle(journalString);
				}
					
				
				Element articleTitleElement = (Element)medlineCitationElement.getElementsByTagName("ArticleTitle").item(0);
				if(articleTitleElement != null){
					String articleTitleString = returnElementData(articleTitleElement);
					article.setArticleTitle(articleTitleString);
				}
				
				Element volumeElement = (Element)medlineCitationElement.getElementsByTagName("Volume").item(0);
				if(volumeElement != null){
					String volumeString = returnElementData(volumeElement);
					if(volumeString!=null){
						article.setVolume(volumeString);
					}
				}
				
				Element issueElement = (Element)medlineCitationElement.getElementsByTagName("Issue").item(0);
				if(issueElement != null){
					String issueString = returnElementData(issueElement);
					article.setIssue(issueString);
				}
				
				Element pageElement = (Element)medlineCitationElement.getElementsByTagName("MedlinePgn").item(0);
				if(pageElement != null){
					String pageString = returnElementData(pageElement);
					article.setPages(pageString);
				}
				
				Element affiliationElement = (Element)medlineCitationElement.getElementsByTagName("Affiliation").item(0);
				if(affiliationElement != null){
					String affiliationString = returnElementData(affiliationElement);
					article.setAffiliation(affiliationString);
				}
				
				Element abstractElement = (Element)medlineCitationElement.getElementsByTagName("AbstractText").item(0);
				if(abstractElement != null){
					String abstractString = returnElementData(abstractElement);
					article.setAbstractText(abstractString);
				}
				
				NodeList authorNodes =  medlineCitationElement.getElementsByTagName("Author");
				ArrayList<Author> authorList = new ArrayList<Author>();
				for (int i = 0; i<authorNodes.getLength();i++)
				{
					Author author = new Author();
					String foreNameString = null,lastNameString = null,initialsString = null;
					Element aNode = (Element) authorNodes.item(i);
					//get the last name of the author
					Element lastNameElement = getElement(aNode, "LastName");
					if(lastNameElement != null){
						lastNameString = returnElementData(lastNameElement);
					}
					//get the firstname/forename of the author
					Element foreNameElement = getElement(aNode, "FirstName");
					if(foreNameElement == null){
						foreNameElement = getElement(aNode, "ForeName");
					}
					if(foreNameElement != null){
						foreNameString = returnElementData(foreNameElement);
					}
					//get the initials
					Element initialsElement = getElement(aNode, "Initials");
					if(initialsElement != null){
						initialsString = returnElementData(initialsElement);
					}
					
					author.setForename(foreNameString);
					author.setInitials(initialsString);    
					author.setLastname(lastNameString);
					authorList.add(i,author);
					
				}
				
				article.setAuthors(authorList);
			}
			
		}catch(SAXException se){
			se.printStackTrace();
		}catch(IOException ioe){
			ioe.printStackTrace();
		}catch(ParserConfigurationException e) {
			e.printStackTrace();
		}
		
		articles.put(article.getPubmedId(), article);
		articleService.storeComplete(article, user);

	}
	
	
	/**
	 * Processes elements of an XML document to return child elements with a specified tag name
	 * @param element, tagName
	 * @return childElement
	 */
	private Element getElement(Element element, String tagName){
		
		NodeList elementChildren = element.getChildNodes();
		
		Element childElement = null;
		for (int i = 0; i < elementChildren.getLength(); i++) {
			
			Node childNode = elementChildren.item(i);
			
			if(childNode instanceof Element){
				
				childElement = (Element)childNode;
				
				if (childElement.getTagName().equals(tagName)){
					return childElement;
				}else{
					childElement = null;
				}
			}
		}
		return childElement;
	}
	
	/**
	 * Processes elements of an XML document to obtain the data from a text node and enter it into a hash map
	 * @param element, key name
	 * @return 
	 */
	private String returnElementData(Element element){
		
		String data = null;
		Text textNode = (Text)element.getFirstChild();
		if(textNode != null)
		{
			data = textNode.getData();
		}
		
		return data;
		
	}

	public List<String> getPubmedIds() {
		return pubmedIds;
	}

	public void setPubmedIds(ArrayList<String> pubmedIds) {
		this.pubmedIds = pubmedIds;
	}
	

	public List<Article> getArticles() {
		List<Article> articleList =  new ArrayList<Article>(articles.values());
		return articleList;
	}
	
	public Long getMigGeneId() {
		return migGeneId;
	}

	public void setMigGeneId(Long migGeneId) {
		this.migGeneId = migGeneId;
	}

	public Article getArticle(String pubmedId){
		Article article = articles.get(pubmedId);
		return article;
	}
	
}