/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.pubmedsorter.services;

import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.pubmedsorter.Article;
import uk.ac.sanger.mig.pubmedsorter.ArticleAbstract;
import uk.ac.sanger.mig.pubmedsorter.Author;
import uk.ac.sanger.mig.pubmedsorter.dao.ArticleDao;

public class ArticleServiceImpl extends AbstractServiceImpl<Article> implements ArticleService {
	
	private ArticleDao articleDao;
	private AuthorService authorService;
	private ArticleAbstractService articleAbstractService;
	
	public void setArticleDao(ArticleDao articleDao) {
		this.articleDao = articleDao;
	}
	
	public void setAuthorService(AuthorService authorService) {
		this.authorService = authorService;
	}

	public void setArticleAbstractService(
			ArticleAbstractService articleAbstractService) {
		this.articleAbstractService = articleAbstractService;
	}
	
	@Override
	protected void attachComponents(Article article, ComponentMap requiredComponents) {
		if(article == null) {
			throw new RuntimeException("Cannot attach components to null article");
		}
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(Article.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			if(fetchAll || requiredComponents.containsKey(Article.Component.ABSTRACT)) {
				article.setArticleAbstract(fetchArticleAbstract(article.getAbstractId(), requiredComponents.get(Article.Component.ABSTRACT)));
			}
			if(fetchAll || requiredComponents.containsKey(Article.Component.AUTHORS)) {
				article.setAuthors(fetchAuthors(article.getId(), requiredComponents.get(Article.Component.AUTHORS)));
			}
		}
	}
	
	
	private ArticleAbstract fetchArticleAbstract(Long abstractId, ComponentMap requiredComponents){
		
		return articleAbstractService.fetchById(abstractId, requiredComponents);
		
	}
	
	private Collection<Author> fetchAuthors(Long articleId, ComponentMap requiredComponents){
		
		return authorService.fetchAuthorsByArticleId(articleId, requiredComponents);
		
	}
	
	public void inflateArticleComponents(Article article, ComponentMap requiredComponents) {
		if(article == null) {
			throw new RuntimeException("Article cannot be null when inflating components.");
		}
	}

	public Article fetchById(Long id, ComponentMap requiredComponents) {
		Article article = articleDao.fetchById(id);
		
		if(article != null){
			attachComponents(article, requiredComponents);
		}

		return article;
	}

	public Article fetchByPubmedId(String pubmedId, ComponentMap requiredComponents) {
		Article article = articleDao.fetchByPubmedId(pubmedId);
		
		if(article != null){
			attachComponents(article, requiredComponents);
		}
		
		return article;
	}

	@Transactional
	public Long storeAuthenticated(Article article, Person user) {
		
		if(article == null) {
			throw new RuntimeException("Cannot save a null article");
		}

		return articleDao.store(article, user);
		
	}

	@Transactional
	public Long storeComplete(Article article, Person user) {
		
		if(article == null) {
			throw new RuntimeException("Cannot save a null article");
		}
		
		if(article.getAbstractText() != null){
			ArticleAbstract articleAbstract = new ArticleAbstract();
			articleAbstract.setAbstractText(article.getAbstractText());
			article.setAbstractId(articleAbstractService.storeAuthenticated(articleAbstract, user));
		}
		
		Long articleId = articleDao.store(article, user);
		
		if(article.getAuthors() != null)
			authorService.clearLinksToArticle(articleId);
			for(Author author:article.getAuthors()){
				//should check if the author is in the database or not
				if (!((author.getForename() == null) & (author.getInitials() == null) & (author.getLastname() == null))){
					
					Long authorId = null;
					
					if(authorService.checkAuthor(author) != null)
							authorId = author.getId();
					else
						authorId = authorService.storeAuthenticated(author, user);
					
					authorService.linkToArticle(articleId, authorId);
				}
			}		
		return articleId;
	}

}