/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.quartz;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.UnableToInterruptJobException;

import uk.ac.sanger.mig.quartz.StatefulTrigger.StateEnum;


/**
 * Simple wrapper around Quartz Scheduler
 * 
 * @author ae1
 *
 */
public class JobService {

	protected Log logger = LogFactory.getLog(this.getClass());
	
	private Scheduler scheduler; 

	/*
	 * need static ref for Jobs to use this service (otherwise Spring gets into infinite loop on start-up) 
	 * should be ok since we're a singleton
	*/

	private static JobService myService;  
	
	public JobService() {
		myService = this;
	}

	public static JobService getQuartzJobService() {
		return myService;
	}
		
	public void setScheduler(Scheduler scheduler) {
		this.scheduler = scheduler;
	}
	
	public SchedulerInfo getSchedulerInfo() throws SchedulerException {
		
		SchedulerInfo schedulerInfo = new SchedulerInfo();
		schedulerInfo.setMetaData( scheduler.getMetaData() );
		return schedulerInfo;
	}

	/**
	 * Returns earliest fire time of a given job.   If Job has no triggers or 
	 * or none of the trigger(s) have next fire time, this method will return null.
	 * 
	 * @param jobName
	 * @param groupName
	 * @return
	 * @throws SchedulerException
	 */
	public Date getNextFireTimeByJob(String jobName, 
			String groupName) throws SchedulerException  {
		
		Collection<StatefulTrigger> beans = getStatefulTriggersByJob(jobName, groupName);
		
		Date earliestFireTime = null;
		
		for(StatefulTrigger bean : beans) {
			
			Date nextFireTime = bean.getTrigger().getNextFireTime();
			
			if( nextFireTime != null ) {
				
				if( earliestFireTime != null ) {
					
					if(earliestFireTime.after( nextFireTime ))
						
						earliestFireTime = nextFireTime;
					
				} else {
					
					earliestFireTime = nextFireTime;	
				
				}		
			} 
		}
		
		return earliestFireTime;
	}
	
	public Collection<StatefulTrigger> getStatefulTriggersByJob(String jobName, 
			String groupName) throws SchedulerException {

		Collection<StatefulTrigger> statefulTriggers = new ArrayList<StatefulTrigger>();		
		
		Trigger[] trigs = scheduler.getTriggersOfJob(jobName, groupName);

		for(Trigger trig : trigs ) {//flatten
					
			StatefulTrigger statefulTriggerBean = new StatefulTrigger();
			
			statefulTriggerBean.setTrigger( trig );
			
			int state = scheduler.getTriggerState(trig.getName(), trig.getGroup()); 

			statefulTriggerBean.setState( state );		
			
			statefulTriggers.add( statefulTriggerBean );
		}
		
		return statefulTriggers;
		
	}
	
	/**
	 * Returns StatefulTriggerBean for all scheduled Jobs including state info. 
	 * @return
	 */
	public Collection<StatefulTrigger> getAllStatefulTriggers() throws Exception {
		
		Collection<StatefulTrigger> statefulTriggers = new ArrayList<StatefulTrigger>();
		
		String[] groupNames = scheduler.getJobGroupNames();
		
		for(String groupName: groupNames) {
		
			String[] jobNames = scheduler.getJobNames(groupName);
			
			for(String jobName : jobNames) {
			
				Trigger[] trigs = scheduler.getTriggersOfJob(jobName, groupName);
				for(Trigger trig : trigs ) {//flatten
					
					StatefulTrigger statefulTriggerBean = new StatefulTrigger();
					statefulTriggerBean.setTrigger( trig );
					int state = 
						scheduler.getTriggerState(trig.getName(), trig.getGroup()); 
					//add state
					statefulTriggerBean.setState( state );
					
					statefulTriggers.add( statefulTriggerBean );
				}	
			}
		}
		
		return statefulTriggers;
	}
	
	/**
	 * Returns true if one or more instances of the Job is running.
	 * 
	 * @param jobName
	 * @param groupName
	 * @return
	 * @throws SchedulerException
	 */
	public boolean isJobRuning(String jobName, String groupName) throws SchedulerException {
		
		boolean running = false;
				
		JobDetail jobDetail = scheduler.getJobDetail( jobName, groupName);
			
		List<JobExecutionContext> jobContexts = scheduler.getCurrentlyExecutingJobs();
			
		for(JobExecutionContext jobContext : jobContexts) {
			
			if( jobContext.getJobDetail().equals( jobDetail )  ) {
				running = true;
				break;
			}
		}
			
		return running;
	}
	
	/**
		Request the interruption of all currently executing instances of the 
		identified Job, which must be an implementor of the InterruptableJob 
		interface. 

		If more than one instance of the identified job is currently executing, 
		the InterruptableJob#interrupt() method will be called on each instance. 
		However, there is a limitation that in the case that interrupt() on one 
		instances throws an exception, all remaining instances (that have not yet 
		been interrupted) will not have their interrupt() method called. 

		If you wish to interrupt a specific instance of a job (when more than one 
		is executing) you can do so by calling getCurrentlyExecutingJobs() to obtain 
		a handle to the job instance, and then invoke interrupt() on it yourself. 

	 * @param jobName
	 * @param groupName
	
	public void interrupt(String jobName, String groupName) throws UnableToInterruptJobException {
	
		scheduler.interrupt(jobName, groupName);
	}
	 */
	
	/**
	 * Pause the JobDetail with the given name by pausing all of its current Triggers
	 * 
	 * @param jobName
	 * @param groupName
	 * @throws Exception
	 */
	public void pauseJob(String jobName, String groupName) throws SchedulerException {

		scheduler.pauseJob(jobName, groupName);
	}
	
	/**
	 * Resume (un-pause) the JobDetail with the given name.
	 * 
	 * @param jobName
	 * @param groupName
	 * @throws Exception
	 */
	public void resumeJob(String jobName, String groupName) throws SchedulerException {

		scheduler.resumeJob(jobName, groupName);
	}

	/**
	 * Trigger the identified JobDetail (execute it now). The generated 
	 * trigger will be volatile.
	 * 
	 * @param jobName
	 * @param groupName
	 * @throws Exception
	 */
	public void fireJobNow(String jobName, String groupName) throws SchedulerException {
		scheduler.triggerJobWithVolatileTrigger(jobName, groupName);
	}
	
	/**
	 * Remove (delete) the Trigger with the given name, and store the new given one, 
	 * which must be associated with the same job.  However, the new trigger need 
	 * not have the same name as the old trigger. 
	 * 
	 * @param triggerName
	 * @param groupName
	 * @param newTrigger
	 * @param jobName
	 * @param jobGroup
	 * @throws SchedulerException
	 
	public void rescheduleJob(String triggerName, String groupName, Trigger newTrigger, 
			String jobName, String jobGroup) throws SchedulerException {

		newTrigger.setJobName(jobName);
		
		newTrigger.setJobGroup(jobGroup);
		
		scheduler.rescheduleJob(triggerName, groupName, newTrigger);
	}
	*/
}
