/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.quartz.jobs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerException;

import uk.ac.sanger.mig.histology.CropJob;
import uk.ac.sanger.mig.histology.services.CropJobService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.quartz.JobService;

/**
 *
 * Executes a batch of CropJobs ordered by when each was created.
 *
 * Attempts to complete as many CropJobs as possible within an
 * allotted timeout period.
 *
 *
 *
 * @author ae1
 *
 */
public class BatchCropJob extends ScheduledJob {

	private CropJobService cropJobService;
	private int threadPoolSize;
	private final long SECS_PADDING_TO_NEXT_RUN = 1800L;//0.5 hour default to allow possible existing batch job to terminate

	@Override
	public void executeJob(JobExecutionContext context, Person quartzUser)
			throws JobExecutionException {

		Date batchProcessStartTime = new Date();

		Date nextFireTime = getNextFireTime( context );

		logger.info("Batch Crop Job starting. Next fire time: " + nextFireTime);

		// * 0. fetch unprocessed CropJobs
		//   1. ensure sorted by created date
		//	 2. Determine timeout
		//   3. Create thread pool
		 //* 4. process each in turn, ExecutorService will handle queueing and timeout
		Collection<CropJob> cropJobs = cropJobService.fetchAwaitProcessing( null );

		List<CropJob> sortedCropJobs = cropJobService.ensureSortedByCreatedDate( cropJobs );

		if( !sortedCropJobs.isEmpty() ) {

			logger.info("Batch Crop Job Found " + sortedCropJobs.size() + " CropJobs for processing");

			long timeout = calculateTimeout( nextFireTime, batchProcessStartTime, sortedCropJobs.size() );

			/*
			 * Creates a thread pool that reuses a fixed set of threads operating off a
			 * shared unbounded queue. If any thread terminates due to a failure during
			 * execution prior to shutdown, a new one will take its place if needed to execute
			 * subsequent tasks.
			 */
			ExecutorService pool = Executors.newFixedThreadPool( threadPoolSize );

			Collection<Callable<Boolean>> callableTasks = new ArrayList<Callable<Boolean>>();

			for(CropJob cropJob: sortedCropJobs) {

				//logger.info("Batch Crop Job Adding CropJob "+ cropJob.getId()+" to queue, job created on " + cropJob.getCreatedDate());

				callableTasks.add(  new CallableCropJob(cropJob.getId(), quartzUser)  );

			}

			List<Future<Boolean>> futureTasks = null;

			try {

				logger.info("Batch Crop Job Submitting Batch, Timeout in " + timeout/60.0f + " minutes.");

				futureTasks = pool.invokeAll( callableTasks, timeout, TimeUnit.SECONDS );

			} catch(InterruptedException e) {
				//thrown when thread waiting/sleeping (which pool is doing
				//while its Callable tasks complete) and another interrupts it
				logger.error("Thread Pool execution interrupted!\n", e );

				pool.shutdownNow();

				// Preserve interrupt status
			    Thread.currentThread().interrupt();
			}

			String report = "\n------------------ Batch Crop Job Report -----------------" +
					    "\nJob Count: " + callableTasks.size() +
					    "\nStarted: " + batchProcessStartTime +
					    "\nEnded:   " + new Date() +
					    "\nThread Pool: " + threadPoolSize;

			int i = 0;

			for(Future futureTask : futureTasks) {

				try {

					String result = null;
					//isDone() Returns true if this task completed.
					//Completion may be due to normal termination, an exception, or cancellation
					if( futureTask.isDone() )  {

						result = ( futureTask.get().equals(Boolean.TRUE) ) ?
						" Completed Successfully." : " Completed and Failed.";

					} else {

						result = " Did Not Complete.";
					}

					i++;

					report += "\nCrop Job " + i + " of " + callableTasks.size() + " " + result;

				} catch(Exception e) { logger.error( e ); }

			}

			report += "\n---------------------------------------------------------\n\n";

			logger.info( report );

		} else {
			logger.info("\n-------------------- BatchCropJob Report --------------------" +
					    "\n*** BatchCropJob found no CropJobs flagged for procssing ***" +
					    "\n-------------------------------------------------------------");
		}



	}

	/*
	 * Returns number of seconds until next timeout. SECS_PADDING_TO_NEXT_RUN is to give this batch run time to
	 * terminate before next batch run starts up when isDisallowConcurrentJobs() is true.
	 */
	private long calculateTimeout(Date nextFireTime, Date startTime, int numberTasks ) {

		long timeoutPerJob = cropJobService.getSecondsToTimeout();

		if( nextFireTime != null && isDisallowConcurrentJobs() ) {

			return Math.min( ( (long)((nextFireTime.getTime() - startTime.getTime())/1000f) ) -  SECS_PADDING_TO_NEXT_RUN,
				timeoutPerJob *  numberTasks );

		} else
			return ( timeoutPerJob *  numberTasks );

	}

	private Date getNextFireTime(JobExecutionContext context) {

		try {

			return JobService.getQuartzJobService().getNextFireTimeByJob(
					context.getJobDetail().getName(), context.getJobDetail().getGroup()) ;

		} catch(SchedulerException e) {
			//failure querying triggers for next for time. This should be fatal b/c unable to determine
			//when/if another job will run and no reasonable recovery alternative.
			throw new RuntimeException("Scheduler failed to return job triggers to query next fire time.");
		}
	}

	public void setCropJobService(CropJobService cropJobService) {
		this.cropJobService = cropJobService;

	}

	public void setThreadPoolSize(int threadPoolSize) {
		this.threadPoolSize = threadPoolSize;
	}

	private class CallableCropJob implements Callable<Boolean> {

		private Long cropJobId;
		private Person user;

		public CallableCropJob (Long cropJobId, Person user) {
			this.cropJobId = cropJobId;
			this.user = user;
		}

		public Boolean call() {

			return runAndReportFailure(cropJobId, false, user);

		}

		/*
		 * TODO: Move to non-trans service
		 *
		 * NOTE: Called outside service layer so run() and storeFailedJob() are
		 * properly transactional.
		 *
		 * Runs a CropJob and reports CropJob failure if it does not succeed, storing
		 * a failed CropJob.  Returns true if success, false if failure;

		 * Successful running of a CropJob guarantees that readable crop images have been
		 * created on file server and references to them have been stored in the Crop table.
		 *
		 * If failure, changes to Crop table will roll back. However, any system files
		 * that may have been created are not guaranteed to be removed.
		 *
		 */
		private Boolean runAndReportFailure( Long cropJobId, boolean timeout, Person user ) {

			Date startTime = new Date();

			Boolean returnValue = null;

			try {
				//if run() fails, it should be rolled back to here and caught
				cropJobService.run( cropJobId, startTime, timeout, user );
				returnValue = Boolean.TRUE;

			} catch(Throwable  e) {
				//storeFailedJob() is separate transaction (ie, not nested in any other transaction controlled method)

				logger.error("Storing failed job.", e);

				cropJobService.storeFailedJob(cropJobId, startTime, e, user);

				returnValue = Boolean.FALSE;
			}

			return returnValue;
		}

	}


}
