/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.quartz.jobs;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.scheduling.quartz.QuartzJobBean;

import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

/**
 * Superclass of Scheduled Quartz Jobs.
 *
 * @author ae1
 *
 */
public abstract class ScheduledJob extends QuartzJobBean {

	protected Log logger = LogFactory.getLog(this.getClass());
	private PersonService personService;
	//prevents a second job from executing concurrently. Default is true
	protected boolean disallowConcurrentJobs = true;

	@Override
	protected void executeInternal(JobExecutionContext context)
			throws JobExecutionException {
		if( disallowConcurrentJobs &&
				getJobInstanceCount( context ) > 1) {
			throw new JobExecutionException(context.getJobDetail().getName() + " is already running and disallowConcurrentJobs == true!");
		}
		Person user = personService.fetchByUserName("quartzScheduler", null);
		executeJob( context, user );
		logger.info("Finished Job: " + context.getJobDetail().getName());
	}

	public abstract void executeJob( JobExecutionContext context, Person quartzUser )
		throws JobExecutionException;

	/**
	 * Returns the number of Job instances Job executing in the JobExecutionContext.
	 *
	 * @param context
	 * @return
	 */
	private int getJobInstanceCount(JobExecutionContext context) {
		int count = 0;
		try {//first, get all JobExecutionContexts
			List<JobExecutionContext> jobContexts = context.getScheduler().getCurrentlyExecutingJobs();
			String jobName = context.getJobDetail().getName();
			String groupName = context.getJobDetail().getGroup();
			//then count the number with our name/group
			for(JobExecutionContext jobContext : jobContexts) {
				if( jobContext.getJobDetail().getName().equals( jobName ) &&
						jobContext.getJobDetail().getGroup().equals( groupName )  ) {
					count++;
				}
			}
		} catch(Exception e) {
			throw new RuntimeException( e );
		}
		return count;
	}

	public Log getLogger() {
		return this.logger;
	}

	public boolean isDisallowConcurrentJobs() {
		return this.disallowConcurrentJobs;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setDisallowConcurrentJobs(boolean disallowConcurrentJobs) {
		this.disallowConcurrentJobs = disallowConcurrentJobs;
	}
}
