/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.reporting.controllers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.ParameterizableViewController;

import ChartDirector.*;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.reporting.ChartDataTableSortComparator;
import uk.ac.sanger.mig.reporting.DrawableChart;
import uk.ac.sanger.mig.reporting.GraphMetaData;
import uk.ac.sanger.mig.reporting.Range;
import uk.ac.sanger.mig.reporting.RepGraph;
import uk.ac.sanger.mig.reporting.ChartDisplay;
import uk.ac.sanger.mig.reporting.ChartDataSearchBean;
import uk.ac.sanger.mig.reporting.Scatter;
import uk.ac.sanger.mig.reporting.services.BarRangeService;
import uk.ac.sanger.mig.reporting.services.GraphMetaDataService;
import uk.ac.sanger.mig.reporting.services.LineRangeService;
import uk.ac.sanger.mig.reporting.services.ScatterRangeService;
import uk.ac.sanger.mig.reporting.services.RepGraphService;
import uk.ac.sanger.mig.reporting.services.ChartDataService;
import uk.ac.sanger.mig.reporting.views.BarChart;
import uk.ac.sanger.mig.reporting.views.LineChart;
import uk.ac.sanger.mig.reporting.views.ScatterChart;


public class ViewChartDataController extends ParameterizableViewController {

	private ChartDataService chartDataService;
	private GraphMetaDataService graphMetaDataService;
	private ScatterRangeService scatterRangeService;
	private BarRangeService barRangeService;
	private RepGraphService graphService;
	private LineRangeService lineRangeService;
	private MouseService mouseService;


	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}

	public void setLineRangeService(LineRangeService lineRangeService) {
		this.lineRangeService = lineRangeService;
	}

	public void setBarRangeService(BarRangeService barRangeService) {
		this.barRangeService = barRangeService;
	}

	public void setGraphService(RepGraphService graphService) {
		this.graphService = graphService;
	}

	public void setChartDataService(ChartDataService chartDataService) {
		this.chartDataService = chartDataService;
	}

	public void setGraphMetaDataService(GraphMetaDataService graphMetaDataService) {
		this.graphMetaDataService = graphMetaDataService;
	}

	public void setScatterRangeService(ScatterRangeService scatterRangeService) {
		this.scatterRangeService = scatterRangeService;
	}



	@Override
	protected ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response) throws Exception {

		Long pipelineId = ServletRequestUtils.getRequiredLongParameter(request, "pipelineId");
		Long alleleId = ServletRequestUtils.getRequiredLongParameter(request, "alleleId");
		Long graphId = ServletRequestUtils.getRequiredLongParameter(request, "graphId");

		Map<String, Object> model = new HashMap<String, Object>();

		ChartDataSearchBean searchBean = new ChartDataSearchBean();
		searchBean.setPaginated(false);
		searchBean.setQueryName(ChartDataService.CHART_DATA_SEARCH_QUERY);
		searchBean.setAlleleId(alleleId);
		searchBean.setPipelineId(pipelineId);

		RepGraph graph = graphService.fetchById(graphId, ComponentMap.of(RepGraph.Component.META_DATA));
		GraphMetaData g = graphMetaDataService.fetchById(graph.getGraphMetadataId(),null);


		searchBean.setViewName(g.getViewName());
		searchBean.setPlotColumn(g.getPlotColumn());
		searchBean.setGraphTypeId(g.getGraphTypeId());

		if(g.getGraphTypeId() ==  GraphMetaDataService.LINE_GRAPH) {
			searchBean.setXAxisColumn(g.getXAxisColumn());
		}

		ChartDisplay data = chartDataService.fetchChartData(searchBean);

		if(data != null && data.getData().size() != 0) {
			Long strainId = getStrainIdFromChartData(data.getData());

			if(strainId != null) {

			Collection<? extends Range> ranges = null;

			/**
			 * Fetch the reference ranges, for scatter this is the green backgrounds,
			 * for bar this is the count for each data type.
			 */
			if(g.getGraphTypeId() == GraphMetaDataService.SCATTER_GRAPH) {
				ranges = scatterRangeService.fetchByMetaDataStrainPipelineId(g.getId(),strainId, pipelineId);
			}else if(g.getGraphTypeId() == GraphMetaDataService.BAR_GRAPH) {
				ranges = barRangeService.fetchByMetaDataStrainPipelineId(g.getId(),strainId, pipelineId);
			}else if(g.getGraphTypeId() == GraphMetaDataService.LINE_GRAPH) {
				ranges = lineRangeService.fetchByMetaDataStrainPipelineId(g.getId(),strainId, pipelineId);
			}

			data.setRefRange(ranges);
			data.setStatBackgroundId(g.getStatBackgroundId());

			// following info retrieved from metadata service.
			data.setTitle(g.getTitle());
			data.setXAxisTitle(g.getXaxisTitle());
			data.setYAxisTitle(g.getYaxisTitle());

			// make the multichart. pass the context as we use this to pick up
			// the sanger dna image from images. (see setSearchPath3)
			BaseChart bc = null;
			if(g.getGraphTypeId() == GraphMetaDataService.SCATTER_GRAPH) {
				// Create and fetch the chart.
				ScatterChart chart = new ScatterChart(getServletContext(),data);
				bc = chart.getChart();
			}else if(g.getGraphTypeId() == GraphMetaDataService.BAR_GRAPH) {
				// Create and fetch the chart.
				BarChart chart = new BarChart(getServletContext(),data);
				bc = chart.getChart();
			}else if(g.getGraphTypeId() == GraphMetaDataService.LINE_GRAPH) {
				// Create and fetch the chart.
				LineChart chart = new LineChart(getServletContext(),data,g);
				bc = chart.getChart();
			}

			// make the image.
			bc.makeSession(request, "scatterchart_"+graphId);

			if(g.getGraphTypeId() == GraphMetaDataService.SCATTER_GRAPH ||
					g.getGraphTypeId() == GraphMetaDataService.LINE_GRAPH) {
				//Include tool tip for the chart
				String imageMap1 = bc.getHTMLImageMap("", "", "title='{field0}  [{value}] '");
				model.put("imageMap", imageMap1);
			}

			model.put("graphId",graphId);
			model.put("maleData", getGenderDataForTable(data,MouseService.MOUSE_GENDER_MALE));
			model.put("femaleData", getGenderDataForTable(data,MouseService.MOUSE_GENDER_FEMALE));

			}
		}else{
			model.put("errors", "No Data Available.");
		}

		return new ModelAndView(getViewName(), model);
	}


	private Collection<DrawableChart> getGenderDataForTable(ChartDisplay chartData, Long requiredGender) {

		Collection<DrawableChart> genderData =  new ArrayList<DrawableChart>();

		for(DrawableChart cData : chartData.getData()){
			if(cData.getGenderId() ==  requiredGender) {
				Long genotypeId = cData.getGenotypeId();
				// Anything that has 'Unknown' genotype is not drawn in the chart but appears in the data set.
				// Only add those which are wt, het and hom.
				if(genotypeId ==  GenotypeDictionary.WILDTYPE){
					cData.setGenotype("WT");
					genderData.add(cData);
				}else if(genotypeId ==  GenotypeDictionary.HETEROZYGOUS) {
					cData.setGenotype("Het");
					genderData.add(cData);
				}else if(genotypeId ==  GenotypeDictionary.HOMOZYGOUS) {
					cData.setGenotype("Hom");
					genderData.add(cData);
				}
			}
		}

		Collections.sort((List)genderData,new ChartDataTableSortComparator());

		return genderData;
	}

	private Long getStrainIdFromChartData(Collection<DrawableChart> data) {

		for(DrawableChart s : data) {
			return s.getStrainId();
		}
		return null;
	}





}
