/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package uk.ac.sanger.mig.reporting.services;


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.collections15.Factory;

import ChartDirector.Chart;

import uk.ac.sanger.mig.abr.graph.AbrThresholdLine;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.reporting.Bar;
import uk.ac.sanger.mig.reporting.BarRange;
import uk.ac.sanger.mig.reporting.DrawableChart;
import uk.ac.sanger.mig.reporting.Line;
import uk.ac.sanger.mig.reporting.LineRange;
import uk.ac.sanger.mig.reporting.Range;
import uk.ac.sanger.mig.reporting.ScatterRange;
import uk.ac.sanger.mig.reporting.Scatter;
import uk.ac.sanger.mig.reporting.ChartDisplay;

/**
 * Utility class for fetching and munging  data.
 *
 * @author mng
 *
 */
public class ChartUtils {

	private ChartDisplay data;
	private Collection<DrawableChart> plotData;
	private Collection<? extends Range> ranges;
	private HashMap<String, Integer> colourMap;
	private double maxY;
	private double minY;


	Factory<Double> doubleFactory = new Factory<Double>() {
		public Double create() {
			return new Double(0.0);
		}
	};

	/**
	 *
	 * @param data
	 */
	public ChartUtils(ChartDisplay data) {
		this.data = data;
		plotData = data.getData();
		ranges = data.getRefRange();

	}

	/**
	 * DEV -  currently the MGWG want all line charts to have the same Y-axis scale.
	 * this method is only called from the LineChart class to work it out.
	 *
	 */
	public void setMinMaxYForDataSet() {
		maxY = getMaxYForDataSet();
		minY = getMinYForDataSet();

	}


	/**
	 *
	 * @param genotypeId
	 * @param genderId
	 * @return a double array containing the data
	 */
	public double[] fetchScatterData(Long genotypeId, Long genderId) {

		Collection<Scatter> preData =  new ArrayList<Scatter>();
		Collection<Scatter> postData =  new ArrayList<Scatter>();

		// pull out the objects with the correct genotype.
		for(DrawableChart s : plotData) {
			if(s.getGenotypeId() == genotypeId) {
				preData.add((Scatter)s);
			}
		}

		// if splitting on gender, parse the genotype list for the correct gender.
		if(genderId != null) {
			for(DrawableChart s : preData) {
				if(s.getGenderId() == genderId) {
					postData.add((Scatter)s);
				}
			}
		}

		// return a double[] containing the plot values.
		return (genderId != null) ? toPrimitive(postData) : toPrimitive(preData);
	}

	/**
	 *
	 * @param genotypeId
	 * @param genderId
	 * @return Collection<Scatter>
	 */
	public ArrayList<Scatter> fetchScatterData2(Long genotypeId, Long genderId) {

		ArrayList<Scatter> preData =  new ArrayList<Scatter>();
		ArrayList<Scatter> postData =  new ArrayList<Scatter>();

		// pull out the objects with the correct genotype.
		for(DrawableChart s : plotData) {
			if(s.getGenotypeId() == genotypeId) {
				preData.add((Scatter)s);
			}
		}

		// if splitting on gender, parse the genotype list for the correct gender.
		if(genderId != null) {
			for(DrawableChart s : preData) {
				if(s.getGenderId() == genderId) {
					postData.add((Scatter)s);
				}
			}
		}

		return (genderId != null) ? postData : preData;
	}


	public HashMap<String, double[]> fetchBarData(Long genderId) {

		HashMap<String, double[]> barData = new HashMap<String, double[]>();

		// unique list of columns for this gender. Columns names will actually become the legend.
		ArrayList<String> columns = getUniqueColumnTypesByGender(genderId);

		int maxLen = 4; // Wt,Het,Hom & base.
		for(String name : columns) {
			if(!barData.containsKey(name)
					&& !name.equals("No data")
					&& !name.equals("0")
					&& !name.equals("No information")) {
				double[] keyCounts = new double[maxLen];
				/**
				 * Iterate over these...
				 * GENOTYPE_HOMOZYGOUS = 3L;
				 * GENOTYPE_HETEROZYGOUS = 2L;
				 * GENOTYPE_WILDTYPE = 1L;
				 */

				for(int genotypeId = 1; genotypeId<4; genotypeId++ ) {
					double valueCount = 0;
					for(DrawableChart s : plotData) {
						if(s.getGenotypeId() == genotypeId && s.getGenderId() == genderId) {
							Bar b = (Bar)s;
							String val = b.getValue();
							if(val != null && b.getValue().equals(name)) {
								valueCount++;
							}
						}
					}
					// use the array position as genotypeId - 1;
					// Ignore 0 data.

					keyCounts[genotypeId-1] = (valueCount > 0) ? valueCount : Chart.NoValue;

					// add the base value from refrange for this name
					keyCounts[3] = getBarRangeCount(name,genderId);
				}
				barData.put(name, keyCounts);
			}
		}

		addAllRemainingRangeTypesForGender(barData,genderId);

		return barData;
	}

	/**
	 * Method to add all remaining names (column values) from the ranges.... should cover all.
	 */
	private void addAllRemainingRangeTypesForGender(HashMap<String, double[]> barData, Long genderId) {

		for(Range r : ranges) {
			BarRange b = (BarRange)r;
			if(b.getGenderId() == genderId) {
				String key = b.getColumnParameter();
				// If not seen already or doesn't equal 'No data' (to skip: na5 - 26/6/08)
				if(!barData.containsKey(key)
						&& !key.equals("No data")
						&& !key.equals("0")
						&& !key.equals("No information")) {
					barData.put(key, new double[] {Chart.NoValue,Chart.NoValue,Chart.NoValue,b.getParameterCount()});
				}
			}
		}

	}



	/**
	 * method to return the count value for all WT data having
	 * the same name (column value)
	 *
	 * @param name
	 * @return count for that column
	 */
	private double getBarRangeCount(String name, Long genderId) {

		for(Range r : ranges) {
			BarRange b = (BarRange)r;
			if(b.getColumnParameter().equals(name) && b.getGenderId() == genderId) {
				if(b.getParameterCount() > 0) {
					return b.getParameterCount();
				}else{
					return Chart.NoValue;
				}
			}
		}
		return Chart.NoValue;
	}


	private ArrayList<String> getUniqueColumnTypesByGender(Long genderId) {

		ArrayList<String> colNames = new ArrayList<String>();

		for(DrawableChart s : plotData) {
			if(s.getGenderId() == genderId) {
				Bar b = (Bar)s;
				String value = b.getValue();
				if(!colNames.contains(value)) {
					colNames.add(value);
				}
			}
		}
		return colNames;
	}


	public double[] getRefRangeByGenderId(Long genderId) {

		if(genderId != null) {
			for(Range r : ranges) {
				ScatterRange g = (ScatterRange)r;

				if(g.getGenderId() == genderId) {
					if(data.getStatBackgroundId() == 2) { // Standard Deviation
						// 5 ranges.
						double[] ret = new double[5];
						ret[0] = (g.getMinus2Sd() != null) ? g.getMinus2Sd() : 0;
						ret[1] = (g.getMinus1Sd()!= null) ? g.getMinus1Sd() : 0;
						ret[2] = (g.getMean()!= null) ? g.getMean() : 0;
						ret[3] = (g.getPlus1Sd()!= null) ? g.getPlus1Sd() : 0;
						ret[4] = (g.getPlus2Sd()!= null) ? g.getPlus2Sd() : 0;
						return ret;
					}else if(data.getStatBackgroundId() == 1) { // Percentiles
						double[] ret = new double[3];
						ret[0] = (g.getPercentile25() != null)? g.getPercentile25() : 0;
						ret[1] = (g.getMean() != null)? g.getMean() : 0;
						ret[2] = (g.getPercentile975() != null) ? g.getPercentile975() : 0;
						return ret;
					}
				}
			}
		}
		return null;

	}


	/**
	 *
	 * @param data
	 * @return  double[]
	 */
	private double[] toPrimitive(Collection<Scatter> data) {
		int len = data.size();
		double[] ret = new double[len];
		int i = 0;
		for(Scatter d : data) {
			ret[i++] = Double.valueOf(d.getValue());
		}
		return ret;
	}

	public void generateBarColourMap() {
		int[] colours = {0x8abbee,
				0xc3001d,
				0xb0e3c0,
				0xcbdced,
				0x057a52,
				0xff7200,
				0x004869,
				0x846d53,
				0x7c004f,
				0xccb376,
				0x000000,
				0xff0000,
				0x00ff00,
				0x0000ff,
				0xc3001d,
				0xb0e3c0,
				0xcbdced,
				0x057a52,
				0xff7200,
				0x004869,
				0x846d53,
				0x7c004f,
				0xccb376,
				0x000000,
				0xff0000,
				0x00ff00,
				0x0000ff};
		int cIndex = 0;
		colourMap = new HashMap<String, Integer>();
		for(Range r : ranges) {
			BarRange b = (BarRange)r;
			String type = b.getColumnParameter();
			if(!colourMap.containsKey(type)) {
				colourMap.put(type, colours[cIndex++]);
			}
		}
		for( DrawableChart d : plotData) {
			Bar b = (Bar)d;
			String type = b.getValue();
			if(!colourMap.containsKey(type)) {
				colourMap.put(type, colours[cIndex++]);
			}
		}

	}

	public int getColourByType(String type) {
		int colour = 0;
		if(colourMap.containsKey(type)) {
			colour =  colourMap.get(type);
		}
		return colour;
	}

	public int getColourMapSize() {

		return colourMap.size();

	}


	// Data comes in sorted from the DB.
	public String[] getUniqueXLabels(boolean timeAxis) {

		ArrayList<String> xLabels = new ArrayList<String>();

		for(DrawableChart s : plotData) {
			Line l = (Line)s;
			String label = l.getXLabel();
			if(timeAxis) {
				// If timeAxis, only use the wildtype labels so that it automatically trims the het/hom data points back
				// to the same length.
				//if(l.getGenotypeId().equals(GenotypeDictionary.WILDTYPE)) {
					if(!xLabels.contains(label)) {
						xLabels.add(label);
					}
				//}
			}else{
				if(!xLabels.contains(label)) {
					xLabels.add(label);
				}
			}
		}

		return xLabels.toArray(new String[0]);
	}


	/**
	 * Line specific stuff starts here.
	 */


	/**
	 * Method to work out the number of unique mice to be displayed
	 *
	 * @param genotypeId
	 * @param genderId
	 * @return count of unique data sets (mice) from the data.
	 */
	public String dataSetCount(Long genotypeId, Long genderId) {

		ArrayList<String> countList=  new ArrayList<String>();
		Integer numData = 0;

		// pull out the objects with the correct genotype.
		for(DrawableChart s : plotData) {
			if(s.getGenotypeId() == genotypeId && s.getGenderId() == genderId) {
				String mouse = s.getMouseName();
				if(!countList.contains(mouse)) {
					numData++;
					countList.add(mouse);
				}
			}
		}
		return numData.toString();
	}


	/**
	 * Method to map the multiple rows for each mouse from data into a single double[] describing
	 * the line data for each mouse. Data points have to be sorted and occur in the same order as the
	 * xAxis labels.
	 * Also, this deals with plotting either raw (mouse) data or averaged data depending on the onlyAverageDataLine
	 * flag. Its calculated regardless but only returned if needed.
	 *
	 * @param isTimeAxis
	 * @param genotypeId
	 * @param genderId
	 * @param onlyAverageDataLine
	 * @return HashMap<String,double[]> - Mouse and corresponding double[] for line data.
	 */
	public HashMap<String,double[]> fetchLineData(boolean isTimeAxis, Long genotypeId, Long genderId, boolean onlyAverageDataLine) {


		String[] labels = getUniqueXLabels(isTimeAxis);
		int numLabels = labels.length;

		ArrayList<Line> postData =  new ArrayList<Line>();
		HashMap<String,double[]> rawData = new HashMap<String,double[]>();
		HashMap<String,double[]> avgData = new HashMap<String,double[]>();
		double[] averages = new double[numLabels];

		// pull out the objects with the correct genotype and gender.
		for(DrawableChart s : plotData) {
			if(s.getGenotypeId() == genotypeId && s.getGenderId() == genderId) {
				postData.add((Line)s);
			}
		}
		// proceed if we have data to plot.
		if(postData.size() > 0) {
			/**
			 *  Iterate round labels (which is sorted for the xAxis). This ensures that as we find
			 *  objects with that label, then the data point for the line gets stored at the correct point
			 *  in the double array.
			 */

			// need a count of number of samples per plot point.
			int[] pointCounts = new int[labels.length];

			for(int i = 0; i<labels.length; i++) {
				pointCounts[i] = 0;
				for(Line l : postData) {
					if(l.getXLabel().equals(labels[i])) {
						String mouseName = l.getMouseName();
						if(!rawData.containsKey(mouseName)) {
							double[] mouseLine  = new double[numLabels];
							rawData.put(mouseName, mouseLine);
						}
						double[] points =  rawData.get(mouseName);
						double val = Double.parseDouble(l.getValue());
						if(val > 0) {
							points[i] = Double.parseDouble(l.getValue());
						}else{
							// remove a zero value for this point.
							points[i] = Chart.NoValue;
						}
						if(points[i] != Chart.NoValue) {
							averages[i] += points[i]; //Add up totals for average here. Ignore NoData, avg calculated below.
						}
						rawData.put(mouseName, points);  // add raw data if needed.
						pointCounts[i]++;  // count the data points for a given xAxis point.
					}
				}
			}

			// generate new data set that is the average of all the raw data.
			if(onlyAverageDataLine) {
				// Calculate averages now that we have a count for each point on the xAxis.
				for(int j=0; j<pointCounts.length; j++) {
					double avg = averages[j] / pointCounts[j];
					averages[j] = avg;
				}
				avgData.put(getTagString(genotypeId), averages); // set average data if needed.
			}
		}
		return (onlyAverageDataLine) ? emptyData(avgData) : emptyData(rawData);
	}

	/**
	 * Method to remove 0 data points from the data array. This is to get around the fact that
	 * IF there are more datapoints for a mouse OR whether some data start later than the rest
	 * the data array will be as long as the X values and pads entries with 0.0, these need to be removed.
	 *
	 * e.g   M1 =  [2,3,4,5]    => [2,3,4,5,0]
	 * 		 M2 =  [2,3,4,5,6]  => [2,3,4,5,6]
	 * OR
	 *  	 M1 =  [2,3,4,5]    => [0,2,3,4,5]
	 *  	 M2 =  [1,2,3,4,5]	=> [1,2,3,4,5]
	 *
	 *
	 */
	private HashMap<String,double[]> emptyData(HashMap<String,double[]> data) {

		for(String key : data.keySet()) {
			double[] values = data.get(key);
			// blat end points.
			for(int i= values.length-1; i>0; i--) {
				if(values[i] <= 0) {
					values[i] = Chart.NoValue;
				}
			}
			// blat start points.
			for(int i= 0; i<values.length; i++) {
				if(values[i] <= 0) {
					values[i] = Chart.NoValue;
				}
			}
			data.put(key, values);
		}
		return data;
	}

	/**
	 * Method to generate the corresponding  tag name for the image map on
	 * average line plot.
	 *
	 * @param genotypeId
	 * @return String for the mouse over image map for the average lines.
	 */
	private String getTagString(Long genotypeId) {

		String tag = "";
		if(genotypeId ==  GenotypeDictionary.WILDTYPE) {
			tag = "WT average";
		}
		else if(genotypeId == GenotypeDictionary.HETEROZYGOUS){
			tag = "Het average";
		}
		else if(genotypeId == GenotypeDictionary.HOMOZYGOUS) {
			tag = "Hom average";
		}
		else {
			tag = "Unknown";
		}
		return tag;
	}

	/**
	 * Method to calculate SEM (standard error of mean) for an average line plot, for a given genotype.
	 * Similar to fetchLineData but has additional calculations, could save values as this class is used once
	 * for the 4 line graphs drawn. Unsure as to whether values could get overwritten?? ... have to check...
	 * Used to draw whiskers on average line plot. Again, values have to be int he same order as the xAxis labels.
	 *
	 * @param isTimeAxis
	 * @param genotypeId
	 * @param genderId
	 * @return ArrayList<double[]> - 2 double arrays, one whisker SEM high and one SEM low.
	 */
	public ArrayList<double[]> getLinePointStandardError(boolean isTimeAxis, Long genotypeId, Long genderId) {
		String[] labels = getUniqueXLabels(isTimeAxis);

		int numLabels = labels.length;

		ArrayList<Line> postData =  new ArrayList<Line>();
		HashMap<String,double[]> rawData = new HashMap<String,double[]>();
		double[] averages = new double[numLabels];
		double[] squareSums = new double[numLabels];
		ArrayList<double[]> deviationErrors =  new ArrayList<double[]>();


		// pull out the objects with the correct genotype and gender.
		for(DrawableChart s : plotData) {
			if(s.getGenotypeId() == genotypeId && s.getGenderId() == genderId) {
				postData.add((Line)s);
			}
		}
		// proceed if we have data to plot.
		if(postData.size() > 0) {

			//need a count of number of samples per plot point.
			int[] pointCounts = new int[labels.length];

			for(int i = 0; i<labels.length; i++) {
				for(Line l : postData) {
					if(l.getXLabel().equals(labels[i])) {
						String mouseName = l.getMouseName();
						if(!rawData.containsKey(mouseName)) {
							double[] mouseLine  = new double[numLabels];
							rawData.put(mouseName, mouseLine);
						}
						double[] points =  rawData.get(mouseName);

						points[i] = Double.parseDouble(l.getValue());
						squareSums[i] += points[i] * points[i];
						averages[i] += points[i] ;/// dataSetSize; //calculate average here.
						rawData.put(mouseName, points);  // add raw data if needed.

						pointCounts[i]++;
					}
				}
			}


			// Calculate averages now that we have a count for each point on the xAxis.
			for(int j=0; j<pointCounts.length; j++) {
				double avg = averages[j] / pointCounts[j];
				averages[j] = avg;
			}


			deviationErrors = calculateDeviationErrors(numLabels, pointCounts, averages, squareSums);

		}
		return deviationErrors;


	}

	/**
	 * Method to work out SEM.   SD / square_root(data set size). Arrays contain values
	 * for +-1 SEM around the mean.
	 *
	 * @param numLabels
	 * @param pointCounts
	 * @param averages
	 * @param squareSums
	 * @return ArrayList<double[]> - SEM arrays
	 */
	private ArrayList<double[]> calculateDeviationErrors(int numLabels, int[] pointCounts, double[] averages, double[] squareSums) {
		double[] errorHigh = new double[numLabels];
		double[] errorLow = new double[numLabels];
		ArrayList<double[]> deviationErrors =  new ArrayList<double[]>();

		for(int j=0; j<numLabels; j++) {
			double mean = averages[j];
			double dev = Math.sqrt( (squareSums[j]/pointCounts[j]) - (mean*mean));
			//double err = dev / Math.sqrt(dataSetSize); // SEM

			errorHigh[j] = mean + dev;
			errorLow[j] = mean - dev;
		}
		deviationErrors.add(errorHigh);
		deviationErrors.add(errorLow);

		return deviationErrors;
	}

	/**
	 * Method that maps the range data calculated in the REP_LINE_RANGE table
	 * into corresponding double arrays ..... in the same order as the xAxis...
	 *
	 * @param isTimeAxis
	 * @param genderId
	 * @return ArrayList<double[]>
	 */
	public ArrayList<double[]> fetchLineRangeData(boolean isTimeAxis, Long genderId) {

		// sorted strings.
		String[] labels = getUniqueXLabels(isTimeAxis);
		int size = labels.length;
		// refrange lines
		double[] rrm2sd = new double[size];
		double[] rrm1sd = new double[size];
		double[] rrmean = new double[size];
		double[] rrp1sd = new double[size];
		double[] rrp2sd = new double[size];

		if(genderId != null) {
			for(int i = 0; i<labels.length; i++) {
				for(Range r : ranges) {
					LineRange g = (LineRange)r;
					if(g.getXLabel().equals(labels[i]) && g.getGenderId() == genderId) {
						if(data.getStatBackgroundId() == 2) { // Standard Deviation
							rrm2sd[i] = (g.getMinus2Sd() != null) ? g.getMinus2Sd() : Chart.NoValue;
							rrm1sd[i] = (g.getMinus1Sd()!= null) ? g.getMinus1Sd() : Chart.NoValue;
							rrmean[i] = (g.getMean()!= null) ? g.getMean() : Chart.NoValue;
							rrp1sd[i] = (g.getPlus1Sd()!= null) ? g.getPlus1Sd() : Chart.NoValue;
							rrp2sd[i] = (g.getPlus2Sd()!= null) ? g.getPlus2Sd() : Chart.NoValue;
						}else if(data.getStatBackgroundId() == 1) { // Percentiles
							// No implemented ...yet.. need to check.
						}
					}
				}
			}
		}

		ArrayList<double[]> rangeData = new ArrayList<double[]>();
		rangeData.add(rrm2sd);
		rangeData.add(rrm1sd);
		rangeData.add(rrmean);
		rangeData.add(rrp1sd);
		rangeData.add(rrp2sd);

		return rangeData;
	}


	/**
	 * Method specific to line graphs to work out the max Y value sot that i can
	 * set the YAxis the same for all 4 graphs.
	 *
	 */
	private double getMaxYForDataSet() {

		double maxYValue = 0;
		for(DrawableChart s : plotData) {
			double val = Double.parseDouble(((Line)s).getValue());
			maxYValue = checkMax(maxYValue, val);
		}
		for(Range r : ranges) {
			LineRange lRange = (LineRange)r;
			if(lRange.getMinus2Sd() != null) {
				maxYValue = checkMax(maxYValue, lRange.getMinus2Sd() );
			}
			if(lRange.getMinus1Sd() != null) {
				maxYValue = checkMax(maxYValue, lRange.getMinus1Sd() );
			}
			if(lRange.getMean() != null) {
				maxYValue = checkMax(maxYValue, lRange.getMean() );
			}
			if(lRange.getPlus1Sd() != null) {
				maxYValue = checkMax(maxYValue, lRange.getPlus1Sd() );
			}
			if(lRange.getPlus2Sd() != null) {
				maxYValue = checkMax(maxYValue, lRange.getPlus2Sd() );
			}
		}
		return maxYValue;
	}


	private double checkMax(double max, double val) {
		if(val > max) {
			max = val;
		}
		return max;
	}

	// limit for dropping below zero.
	private double checkMin(double min, double val) {
		if(val < min && val > 0) {
			min = val;
		}
		return min;
	}

	/**
	 * Method specific to line graphs to work out the min Y value sot that i can
	 * set the YAxis the same for all 4 graphs.
	 *
	 */
	private double getMinYForDataSet() {
		double minYValue = Chart.NoValue; // NoValue is a huge number :-)
		for(DrawableChart s : plotData) {
			double val = Double.parseDouble(((Line)s).getValue());
			minYValue = checkMin(minYValue, val);
		}
		for(Range r : ranges) {
			LineRange lRange = (LineRange)r;
			if(lRange.getMinus2Sd() != null) {
				minYValue = checkMin(minYValue, lRange.getMinus2Sd());
			}
			if(lRange.getMinus1Sd() != null) {
				minYValue = checkMin(minYValue, lRange.getMinus1Sd());
			}
			if(lRange.getMean() != null) {
				minYValue = checkMin(minYValue, lRange.getMean());
			}
			if(lRange.getPlus1Sd() != null) {
				minYValue = checkMin(minYValue, lRange.getPlus1Sd());
			}
			if(lRange.getPlus2Sd() != null) {
				minYValue = checkMin(minYValue, lRange.getPlus2Sd());
			}
		}
		return minYValue;
	}

	public double getMaxY() {
		return maxY;
	}

	public double getMinY() {
		return minY;
	}


//	ABR related methods
	public String abrDataSetCount(Long genotypeId) {

		ArrayList<Long> countList=  new ArrayList<Long>();
		Integer numData = 0;

		// pull out the objects with the correct genotype.
		for(DrawableChart s : plotData) {
			if(s.getGenotypeId() == genotypeId) {
				AbrThresholdLine mouse = (AbrThresholdLine)s;
				if(!countList.contains(mouse.getExperimentId())) {
					numData++;
					countList.add(mouse.getExperimentId());
				}
			}
		}
		return numData.toString();
	}

	public HashMap<String,double[]> fetchAbrLineData(Long genotypeId, boolean onlyAverageDataLine) {


		String[] labels = getUniqueXLabels(false);
		int numLabels = labels.length;

		ArrayList<AbrThresholdLine> postData =  new ArrayList<AbrThresholdLine>();
		HashMap<String,double[]> rawData = new HashMap<String,double[]>();
		HashMap<String,double[]> avgData = new HashMap<String,double[]>();
		double[] averages = new double[numLabels];

		// pull out the objects with the correct genotype and gender.
		for(DrawableChart s : plotData) {
			if(s.getGenotypeId() == genotypeId) {
				postData.add((AbrThresholdLine)s);
			}
		}
		// proceed if we have data to plot.
		if(postData.size() > 0) {
			/**
			 *  Iterate round labels (which is sorted for the xAxis). This ensures that as we find
			 *  objects with that label, then the data point for the line gets stored at the correct point
			 *  in the double array.
			 */

			// need a count of number of samples per plot point.
			int[] pointCounts = new int[labels.length];

			for(int i = 0; i<labels.length; i++) {
				pointCounts[i] = 0;
				for(AbrThresholdLine l : postData) {
					if(l.getXLabel().equals(labels[i])) {
						String mouseName = l.getExperimentId().toString();
						if(!rawData.containsKey(mouseName)) {
							double[] mouseLine  = new double[numLabels];
							rawData.put(mouseName, mouseLine);
						}
						double[] points =  rawData.get(mouseName);
						double val = Double.parseDouble(l.getValue());
						if(val > 0) {
							points[i] = Double.parseDouble(l.getValue());
						}else{
							// remove a zero value for this point.
							points[i] = Chart.NoValue;
						}
						if(points[i] != Chart.NoValue) {
							averages[i] += points[i]; //Add up totals for average here. Ignore NoData, avg calculated below.
						}
						rawData.put(mouseName, points);  // add raw data if needed.
						pointCounts[i]++;  // count the data points for a given xAxis point.
					}
				}
			}

			// generate new data set that is the average of all the raw data.
			if(onlyAverageDataLine) {
				// Calculate averages now that we have a count for each point on the xAxis.
				for(int j=0; j<pointCounts.length; j++) {
					double avg = averages[j] / pointCounts[j];
					averages[j] = avg;
				}
				avgData.put(getTagString(genotypeId), averages); // set average data if needed.
			}
		}
		return (onlyAverageDataLine) ? emptyData(avgData) : emptyData(rawData);
	}

	public ArrayList<double[]> fetchAbrLineRangeData() {
		// sorted strings.
		String[] labels = getUniqueXLabels(false);
		int size = labels.length;
		// refrange lines
		double[] rrm2sd = new double[size];
		double[] rrm1sd = new double[size];
		double[] rrmean = new double[size];
		double[] rrp1sd = new double[size];
		double[] rrp2sd = new double[size];


		for(int i = 0; i<labels.length; i++) {
			for(Range r : ranges) {
				LineRange g = (LineRange)r;
				if(g.getXLabel().equals(labels[i])) {
					if(g.getGenderId() == null) { // Standard Deviation
						rrm2sd[i] = (g.getMinus2Sd() != null) ? g.getMinus2Sd() : Chart.NoValue;
						rrm1sd[i] = (g.getMinus1Sd()!= null) ? g.getMinus1Sd() : Chart.NoValue;
						rrmean[i] = (g.getMean()!= null) ? g.getMean() : Chart.NoValue;
						rrp1sd[i] = (g.getPlus1Sd()!= null) ? g.getPlus1Sd() : Chart.NoValue;
						rrp2sd[i] = (g.getPlus2Sd()!= null) ? g.getPlus2Sd() : Chart.NoValue;
					}
				}
			}
		}

		ArrayList<double[]> rangeData = new ArrayList<double[]>();
		rangeData.add(rrm2sd);
		rangeData.add(rrm1sd);
		rangeData.add(rrmean);
		rangeData.add(rrp1sd);
		rangeData.add(rrp2sd);

		return rangeData;
	}

	public ArrayList<double[]> getAbrStandardError(Long genotypeId, int dataSetSize) {

		if(dataSetSize >0){
			String[] labels = getUniqueXLabels(false);

			int numLabels = labels.length;


			List<AbrThresholdLine> postData =  new ArrayList<AbrThresholdLine>();
			HashMap<Long,double[]> rawData = new HashMap<Long,double[]>();
			double[] averages = new double[numLabels];
			double[] squareSums = new double[numLabels];
			ArrayList<double[]> deviationErrors =  new ArrayList<double[]>();


			// pull out the objects with the correct genotype and gender.
			for(DrawableChart s : plotData) {

				if(s.getGenotypeId() == genotypeId){
					postData.add((AbrThresholdLine)s);
				}

			}
			// proceed if we have data to plot.
			if(postData.size() > 0) {
				for(int i = 0; i<labels.length; i++) {
					for(AbrThresholdLine l : postData) {
						if(l.getXLabel().equals(labels[i])) {
							Long experimentId = l.getExperimentId();
							if(!rawData.containsKey(experimentId)) {
								double[] mouseLine  = new double[numLabels];
								rawData.put(experimentId, mouseLine);
							}
							double[] points =  rawData.get(experimentId);
							points[i] = Double.parseDouble(l.getValue());
							squareSums[i] += points[i] * points[i];
							averages[i] += points[i] / dataSetSize; //calculate average here.
							rawData.put(experimentId, points);  // add raw data if needed.
						}
					}
				}

				deviationErrors = calculateABRStatis(numLabels, dataSetSize, averages, squareSums);
			}
			return deviationErrors;
		}else{
			return new ArrayList<double[]>();
		}


	}

	private ArrayList<double[]> calculateABRStatis(int numLabels, int dataSetSize, double[] averages, double[] squareSums) {
		double[] errorHigh = new double[numLabels];
		double[] errorLow = new double[numLabels];
		ArrayList<double[]> deviationErrors =  new ArrayList<double[]>();
		double[] sd = new double[numLabels];
		double[] se = new double[numLabels];

		for(int j=0; j<numLabels; j++) {
			double mean = averages[j];
			double dev = Math.sqrt( squareSums[j]/dataSetSize - mean*mean);
			double err = dev / Math.sqrt(dataSetSize);
			errorHigh[j] = mean + err;
			errorLow[j] = mean - err;
			sd[j] = dev;
			se[j] = err;
		}
		deviationErrors.add(errorHigh);
		deviationErrors.add(errorLow);
		deviationErrors.add(averages);
		deviationErrors.add(sd);
		deviationErrors.add(se);

		return deviationErrors;
	}

}
