/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.reporting.services;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.personnel.Person;

import java.util.Collection;
import java.util.HashMap;

import uk.ac.sanger.mig.reporting.RepPageOverview;
import uk.ac.sanger.mig.reporting.RepPageOverviewSearchBean;
import uk.ac.sanger.mig.reporting.RepPipelineTests;
import uk.ac.sanger.mig.reporting.dao.RepPageOverviewDao;

public class RepPageOverviewServiceImpl extends AbstractServiceImpl2<RepPageOverview,RepPageOverviewDao> implements RepPageOverviewService {


	private AlleleService alleleService;
	private RepPipelineTestsService repPipelineTestsService;
	private DictionaryService dictionaryService;

	public void setRepPipelineTestsService(
			RepPipelineTestsService repPipelineTestsService) {
		this.repPipelineTestsService = repPipelineTestsService;
	}

	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}

	public void setAlleleService(AlleleService alleleService) {
		this.alleleService = alleleService;
	}


	public Long store(RepPageOverview repPageOverview, Person user) {
		if(repPageOverview == null) {
			throw new RuntimeException("Cannot save a null repPageOverview");
		}

		return getPrimaryDao().store(repPageOverview, user);
	}

	public RepPageOverview fetchById(Long repPageOverviewId, ComponentMap requiredComponents) {
		RepPageOverview repPageOverview = getPrimaryDao().fetchById(repPageOverviewId);

		if(repPageOverview != null)
			attachComponents(repPageOverview, requiredComponents);

		return repPageOverview;
	}

	@Override
	protected void attachComponents(RepPageOverview repPageOverview, ComponentMap requiredComponents) {
		if(repPageOverview == null) {
			throw new RuntimeException("Cannot attach components to null repPageOverview");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(RepPageOverview.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(RepPageOverview.Component.ALLELES)) {
				repPageOverview.setAlleles(fetchAllelesByGeneId(repPageOverview.getGeneId()));
			}

			if(fetchAll || requiredComponents.containsKey(RepPageOverview.Component.PIPELINE_TESTS)) {
				repPageOverview.setPipelineTests(fetchAllPipelineTests());
			}
		}
	}

	/**
	 * Method for fetching all alleles for this gene Id.
	 *
	 * @param geneId
	 * @return Collection<Allele>
	 */
	private Collection<Allele> fetchAllelesByGeneId(Long geneId) {
		return alleleService.fetchAllByGeneId(geneId);
	}


	private HashMap<String,Collection<RepPipelineTests>> fetchAllPipelineTests() {
		return repPipelineTestsService.fetchAllPipelinesWithTests();
	}


	public HashMap<String,String> fetchPipelinesByAllele(Long allele) {

		// Get all at the moment but will be refactored to only get the pipelines
		// for which there is test data.
		HashMap<String, String> pipelineData = new HashMap<String, String>();
		Collection<DictionaryEntry> pipelines = dictionaryService.fetch(DictionaryEnum.MOUSE_COHORT_PIPELINE, true);
		for(DictionaryEntry entry : pipelines) {
			pipelineData.put(entry.getId()+"", entry.getName());
		}

		return pipelineData;
	}

	public HashMap<String,String> fetchPhenotypingTestsByPipeline(Long pipelineId) {

		HashMap<String, String> pipelineTests = new HashMap<String, String>();

		Collection<RepPipelineTests> tests = repPipelineTestsService.fetchAllTestsByPipelineId(pipelineId);
		for(RepPipelineTests test : tests) {
			pipelineTests.put(test.getPageTypeId()+"", test.getDescription());
		}
		return pipelineTests;
	}



	public RepPageOverview fetchByGeneId(Long geneId, ComponentMap componentMap) {


		RepPageOverview overview =  getPrimaryDao().fetchByGeneId(geneId);

		if(overview != null)
			attachComponents(overview, componentMap);

		return overview;
	}


	public Collection<RepPageOverview> search(
			RepPageOverviewSearchBean repPageOverviewSearchBean) {
		// TODO Auto-generated method stub
		return null;
	}

}
