/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.reservations.controllers;

import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import uk.ac.sanger.mig.common.AbstractCancellableFormController;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.RequestUtils;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryEnum;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.Team;
import uk.ac.sanger.mig.reservations.MouseReservation;
import uk.ac.sanger.mig.reservations.Purpose;
import uk.ac.sanger.mig.reservations.Reservation;
import uk.ac.sanger.mig.reservations.forms.ReservationForm;
import uk.ac.sanger.mig.reservations.services.ReservationService;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.*;

public class ReservationFormController extends AbstractCancellableFormController {

	private ReservationService	reservationService;

	private MouseService		mouseService;
	private DictionaryService	dictionaryService;
	private ComponentMapService	componentMapService;


	/**
	 * Sets the mouse service used to lookup from a mouse to a reservation.
	 *
	 * @param mouseService -
	 *            mouse service interface to MouseServiceImpl.
	 */
	public void setMouseService(MouseService mouseService) {
		this.mouseService = mouseService;
	}


	/**
	 * Sets the dictionary service for dictionary lookups on the reservations purpose category.
	 *
	 * @param dictionaryService -
	 *            dictionary service interface to DictionaryServiceImpl.
	 */
	public void setDictionaryService(DictionaryService dictionaryService) {
		this.dictionaryService = dictionaryService;
	}


	/**
	 * Sets the reservation service for for finding reservations for a mouse id.
	 *
	 * @param reservationService -
	 *            reservation service interface to ReservationServiceImpl.
	 */
	public void setReservationService(ReservationService reservationService) {
		this.reservationService = reservationService;
	}


	/**
	 * Sets the component map service for retrieving XML component map definitions for the
	 * reservation project held in maps/reservations.xml.
	 *
	 * @param componentMapService -
	 *            component map service interface to ComponentMapServiceImpl.
	 */
	public void setComponentMapService(ComponentMapService componentMapService) {
		this.componentMapService = componentMapService;
	}


	/**
	 * Initialises any binders for custom date fields and string trimming.
	 *
	 * @param request -
	 *            the request object.
	 * @param binder -
	 *            the binder object.
	 * @throws Exception -
	 *             general exception.
	 */
	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		binder.registerCustomEditor(Long.class, new CustomNumberEditor(Long.class, true));
		binder.registerCustomEditor(Double.class, new CustomNumberEditor(Integer.class, true));
		binder.registerCustomEditor(Integer.class, new CustomNumberEditor(Double.class, true));
		binder.registerCustomEditor(Date.class, new CustomDateEditor(new SimpleDateFormat("dd-MMM-yyyy"), true));
	}


	/**
	 * Creates the form backing object. There are several entrance points to this controller: <p/>
	 * View one mouse - mouse id passed in URL. Find if mouse has a reservation, if not build one
	 * and page is adding, otherwise it has one and page is editing it. If found, find the other
	 * mice in the reservation and display them as they will be affected by the edit. <p/> Selection
	 * (advanced mouse search) - a group of mice have been selected. No one reservation can be
	 * assumed as being edited, so this is an adding function.
	 *
	 * @param httpServletRequest -
	 *            the request object.
	 * @return reservationForm - the newly created and populated reservation form.
	 * @throws Exception -
	 *             cover all exception.
	 */
	@Override
	protected Object formBackingObject(HttpServletRequest httpServletRequest) throws Exception {
		ReservationForm reservationForm = (ReservationForm) createCommand();
		Long mouseId = ServletRequestUtils.getLongParameter(httpServletRequest, "mouseId");
		Reservation reservation;
		String sessionId = RequestUtils.getSessionId(httpServletRequest);
		List<Mouse> mouseList = new LinkedList<Mouse>();
		if (mouseId != null) {
			// Find mouse reservation and all mice associated with that reservation using
			// mouse id from the URL.
			reservation = reservationService.fetchByMouseId(mouseId, componentMapService
					.getComponentMap("reservationMice"));
			if (reservation == null) {
				reservation = createNewReservation(RequestUtils.getUser(httpServletRequest));
				mouseList.add(mouseService.fetchById(mouseId, ComponentMap.of(Mouse.Component.GENOTYPE)));
			} else {
				Collection<MouseReservation> mouseReservations = reservation.getMouseReservations();
				mouseList = extractMice(reservation, mouseReservations);
				if (mouseList.size() > 1)
					reservationForm.setHighlightMouseId(mouseId);
				expandExistingReservation(reservation);
			}
		} else {
			// try the session store from mouse list page
			String queryName = ServletRequestUtils.getStringParameter(httpServletRequest, "queryName",
					MouseService.MOUSE_SEARCH_QUERY);
			Collection<Mouse> selectedMouseIds = mouseService.fetchLastSelection(sessionId, queryName,
					componentMapService.getComponentMap("miceReservation"));
			mouseList.addAll(selectedMouseIds);

			// Make a new reservation.
			Person user = RequestUtils.getUser(httpServletRequest);
			reservation = createNewReservation(user);
		}
		reservationForm.setMouseList(mouseList);
		initSelectedMouseIds(reservationForm);
		reservationForm.setReservation(reservation);
		return reservationForm;
	}


	/**
	 * Creates a new Reservation object and populates it with the known details. This is used to
	 * build a resevation for a group of mice that do not current belong to a reservation.
	 *
	 * @param user -
	 *            the current user to set as the reserver.
	 * @return reservation - the newly created reservation.
	 */
	private Reservation createNewReservation(Person user) {
		Reservation reservation = new Reservation();
		Team team = user.getTeam();
		reservation.setReserver(user);
		reservation.setReserverId(user.getId());
		reservation.setReserverName(user.getFullName());
		if (team != null) {
			reservation.setTeamName(team.getTeamNumberAndName());
			reservation.setTeamId(team.getId());
		}
		reservation.setNotify(true);
		return reservation;
	}


	/**
	 * Expands the passed existing reservation. Sets the reserver name, team and associated id
	 * numbers. Also sets the category and purpose read for this reservation to be edited.
	 *
	 * @param reservation -
	 *            the reservation to expand.
	 */
	private void expandExistingReservation(Reservation reservation) {
		Person reserver = reservation.getReserver();
		Team team = reservation.getTeam();
		Purpose purpose = reservation.getPurpose();
		reservation.setReserverName(reserver.getFullName());
		reservation.setReserverId(reserver.getId());
		reservation.setTeamName(team.getTeamNumberAndName());
		reservation.setTeamId(team.getId());
		reservation.setCategoryId(purpose.getCategoryId());
		reservation.setPurposeId(purpose.getId());
		reservation.setPurposeName(purpose.getName());
	}


	/**
	 * Initialises the list of selected mouse ids so that it has the same number of elements as the
	 * list of mice being worked on.
	 *
	 * @param rf -
	 *            the reservation backing form object.
	 */
	private void initSelectedMouseIds(ReservationForm rf) {
		List<Mouse> mice = rf.getMouseList();
		for (Mouse m : mice) {
			rf.getSelectedMouseIds().add(m.getId());
		}
	}


	/**
	 * Extracts the Mouse objects from the passed MouseReservation collection. Views are built using
	 * Mouse objects.
	 *
	 * @param reservation -
	 *            the Reservation object pojo.
	 * @param mouseList -
	 *            collection of MouseReservation objects.
	 * @return mice - list of extracted Mouse objects.
	 */
	private List<Mouse> extractMice(Reservation reservation, Collection<MouseReservation> mouseList) {
		List<Mouse> mice = new LinkedList<Mouse>();
		for (MouseReservation mr : mouseList) {
			Mouse m = mr.getMouse();
			mr.setReservation(reservation);
			m.setMouseReservation(mr);
			mice.add(m);
		}
		return mice;
	}


	/**
	 * Called after databinding, before showForm(), merges Map into ModelAndView to populate our
	 * form with form "metadata" -- eg, drop-down select lists, etc.
	 *
	 * @param httpServletRequest -
	 *            the request object.
	 * @return formMetaData - hash map containing meta data.
	 * @throws Exception -
	 *             cover all exception.
	 */
	@Override
	protected Map referenceData(HttpServletRequest httpServletRequest) throws Exception {
		Map<String, Object> formMetaData = new HashMap<String, Object>();
		// Reservation category meta data for drop down.
		formMetaData.put("reservationCategory", dictionaryService.fetch(DictionaryEnum.RESERVATION_CATEGORY));
		return formMetaData;
	}


	/**
	 * Calls the reservation services reserve mice method that will manage any current reservation
	 * the selected mice are in and add or delete reservations as required.
	 *
	 * @param request -
	 *            the request object.
	 * @param response -
	 *            the response object
	 * @param command -
	 *            the backing object form.
	 * @param errors -
	 *            errors during form binding and/or vaidation.
	 * @return new model and view containing the success view.
	 * @throws Exception -
	 *             cover all exception.
	 */
	@Override
	protected ModelAndView onSubmit(HttpServletRequest request, HttpServletResponse response, Object command,
			BindException errors) throws Exception {
		ReservationForm reservationForm = (ReservationForm) command;
		Reservation reservation = reservationForm.getReservation();
		reservationService.reserveMice(reservation, reservationForm, RequestUtils.getUser(request));
		Collection<Long> mouseIds = getMouseIds(reservationForm.getMouseList());
		Map<String, Object> model = new HashMap<String, Object>();
		model.put("mouseIds", mouseIds);
		model.put("reservationId", reservation.getId());
		logger.debug("Reservation '" + reservation + "' successfully stored.");
		return new ModelAndView(getSuccessView(), model);// show list view
	}


	/**
	 * Retrieves the mouse ids from each of the passed Mouse objects and places them in a
	 * collection.
	 *
	 * @param mice -
	 *            list of Mouse objects to retrieve the ids from.
	 * @return mouseIds - collection of long mouse id numbers.
	 */
	private Collection<Long> getMouseIds(List<Mouse> mice) {
		Collection<Long> mouseIds = new LinkedList<Long>();
		for (Mouse m : mice) {
			mouseIds.add(m.getId());
		}
		return mouseIds;
	}

}