/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.reservations.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.MailHelper;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;

import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.reservations.MouseReservation;
import uk.ac.sanger.mig.reservations.Reservation;
import uk.ac.sanger.mig.reservations.dao.MouseReservationDao;

public class MouseReservationServiceImpl extends AbstractServiceImpl2<MouseReservation,MouseReservationDao> implements MouseReservationService {

  private MailHelper mailHelper;
  private MouseService mouseService;
  private ReservationService reservationService;

	@Override
	protected void attachComponents(MouseReservation mouseReservation, ComponentMap requiredComponents) {
		if(mouseReservation == null) {
			throw new IllegalArgumentException("Cannot attach components to null mouseReservation");
		}

    if(requiredComponents != null) {
      boolean fetchAll = requiredComponents.containsKey(MouseReservation.Component.ALL_COMPONENTS);

      if (fetchAll || requiredComponents.containsKey(MouseReservation.Component.MOUSE)) {
        mouseReservation.setMouse(
            mouseService.fetchById(
                mouseReservation.getMouseId(),
                requiredComponents.get(MouseReservation.Component.MOUSE)
        ));
      }

      if (fetchAll || requiredComponents.containsKey(MouseReservation.Component.RESERVATION)) {
        mouseReservation.setReservation(
            reservationService.fetchById(
                mouseReservation.getReservationId(),
                requiredComponents.get(MouseReservation.Component.RESERVATION)
        ));
      }
    }
	}

  public Collection<MouseReservation> fetchByReservation(Long reservationId,ComponentMap componentMap) {
    Collection<MouseReservation> mrs = getPrimaryDao().fetchByReservation(reservationId);
    attachComponents(mrs, componentMap);
    return mrs;
  }

  public MouseReservation fetchForMouse(Long mouseId, ComponentMap componentMap) {
    MouseReservation mr = getPrimaryDao().fetchForMouse(mouseId);
    if (mr != null) attachComponents(mr, componentMap);
    return mr;
  }

  @Transactional
  public void delete(MouseReservation mouseReservation, Person user) {
    if (mouseReservation == null || user == null) throw new IllegalArgumentException("Method does not take null arguments");
    getPrimaryDao().delete(mouseReservation, user);
    reservationService.deleteReservationIfEmpty(mouseReservation.getReservationId(), user);
  }

  @Transactional
  public void deleteReservationForMice(Collection<Long> mouseIds, Person user) {
    Collection<Long> reservationIds = new HashSet<Long>();

    for(Long id : mouseIds) {
      MouseReservation mr =  fetchForMouse(id,null);
      if (mr != null) {
        reservationIds.add(mr.getReservationId());
        getPrimaryDao().delete(mr,user);
      }
    }
    for(Long reservationId : reservationIds) {
      reservationService.deleteReservationIfEmpty(reservationId, user);
    }
  }


  @Transactional
  public void sendAlertEmails(Person user) {
    // group the dead mice for one e-mail per reservation
    Map<Long,Collection<MouseReservation>> miceByReservation = new HashMap<Long, Collection<MouseReservation>>();

    for (MouseReservation mr : getPrimaryDao().fetchForDeathNotification()) {

      if (!miceByReservation.containsKey(mr.getReservationId())) {
        miceByReservation.put(mr.getReservationId(), new LinkedList<MouseReservation>());
      }

      miceByReservation.get(mr.getReservationId()).add(mr);
    }


    // send the e-mails & record that it happened
    for (Map.Entry<Long,Collection<MouseReservation>> entry : miceByReservation.entrySet()) {
      boolean success = sendAlertEmail(entry.getKey(), entry.getValue());
      recordEmailSent(entry.getValue(), success, user);
    }


  }
  // Gather the data & send one e-mail
  private boolean sendAlertEmail(Long reservationId, Collection<MouseReservation> mice) {
    Reservation r = reservationService.fetchById(
        reservationId,
        ComponentMap.of(
            Reservation.Component.PURPOSE,
            Reservation.Component.TEAM,
            Reservation.Component.RESERVER)
    );

    attachComponents(mice, ComponentMap.of(MouseReservation.Component.MOUSE));

    Map<String, Object> model = new HashMap<String, Object>();
    model.put("reservation", r);
    model.put("mouseReservations",mice);

    return mailHelper.sendMail("reservation_alert", model, new String[]{r.getReserver().getEmail()});
  }

  private void recordEmailSent(Collection<MouseReservation> mice, boolean notificationSent, Person user) {
    for (MouseReservation mr : mice) {
      mr.setNotificationSent(notificationSent);
      store(mr,user);
    }
  }

  public void setMailHelper(MailHelper mailHelper) {
    this.mailHelper = mailHelper;
  }

  public void setMouseService(MouseService mouseService) {
    this.mouseService = mouseService;
  }

  public void setReservationService(ReservationService reservationService) {
    this.reservationService = reservationService;
  }
}
