/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.reservations.services;

import java.util.Collection;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl2;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.personnel.services.TeamService;
import uk.ac.sanger.mig.reservations.MouseReservation;
import uk.ac.sanger.mig.reservations.Reservation;
import uk.ac.sanger.mig.reservations.dao.ReservationDao;
import uk.ac.sanger.mig.reservations.forms.ReservationForm;

public class ReservationServiceImpl extends AbstractServiceImpl2<Reservation, ReservationDao> implements
		ReservationService {

	private PurposeService			purposeService;
	private PersonService			personService;
	private TeamService				teamService;
	private MouseReservationService	mouseReservationService;


	/**
	 * Sets the mouse reservation service required by this class to create a new mouse reservation.
	 *
	 * @param mouseReservationService -
	 *            the MouseReservationService interface to MouseReservationServiceImpl.
	 */
	public void setMouseReservationService(MouseReservationService mouseReservationService) {
		this.mouseReservationService = mouseReservationService;
	}


	/**
	 * Sets the person service required by this class to find the reservers details in the attachComponents() method.
	 *
	 * @param personService -
	 *            the PersonService interface to PersonServiceImpl.
	 */
	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}


	/**
	 * Sets the purpose service required by this class to find the purpose of a reservation in the attachComponents()
	 * method.
	 *
	 * @param purposeService -
	 *            the PurposeService interface to PurposeServiceImpl.
	 */
	public void setPurposeService(PurposeService purposeService) {
		this.purposeService = purposeService;
	}


	/**
	 * Sets the team service required by this class to find the team a reservation was assigned to in the
	 * attachComponents() method.
	 *
	 * @param teamService -
	 *            the TeamService interface to TeamServiceImpl.
	 */
	public void setTeamService(TeamService teamService) {
		this.teamService = teamService;
	}


	/**
	 * Normal project attachment method for this service. Attaches the following:
	 *
	 * PURPOSE - the purpose the mouse has been reserved for (always defined), contains the category via a dictionarry.
	 *
	 * RESERVER - the Person object representing the user that created the reservation.
	 *
	 * TEAM - the team that the reserver and the reservation are assigned to (this is not the same as the reservers
	 * default team).
	 *
	 * MOUSE_RESERVATIONS - a collection of associated mouse reservation for this reservation.
	 *
	 * @param reservation -
	 * @param requiredComponents
	 */
	@Override
	protected void attachComponents(Reservation reservation, ComponentMap requiredComponents) {
		if (reservation == null) {
			throw new IllegalArgumentException("Cannot attach components to null reservation");
		}

		boolean fetchAll = false;

		if (requiredComponents != null) {

			if (requiredComponents.containsKey(Reservation.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if (fetchAll || requiredComponents.containsKey(Reservation.Component.PURPOSE)) {
				reservation.setPurpose(purposeService.fetchById(reservation.getPurposeId(), requiredComponents
						.get(Reservation.Component.PURPOSE)));
			}

			if (fetchAll || requiredComponents.containsKey(Reservation.Component.RESERVER)) {
				reservation.setReserver(personService.fetchById(reservation.getReserverId(), requiredComponents
						.get(Reservation.Component.RESERVER)));
			}

			if (fetchAll || requiredComponents.containsKey(Reservation.Component.TEAM)) {
				reservation.setTeam(teamService.fetchById(reservation.getTeamId(), requiredComponents
						.get(Reservation.Component.TEAM)));
			}

			if (fetchAll || requiredComponents.containsKey(Reservation.Component.MOUSE_RESERVATIONS)) {
				reservation.setMouseReservations(mouseReservationService.fetchByReservation(reservation.getId(),
						requiredComponents.get(Reservation.Component.MOUSE_RESERVATIONS)));
			}

		}
	}


	/**
	 * Reserves the mice selected. Uses the following rules:
	 *
	 * 1. If a mouse is already part of the reservation and selected then ignore it.
	 *
	 * 2. If a mouse is already part of the reservation and not selected then delete it from the reservation.
	 *
	 * 3. If a mouse is not part of this reservation and is selected, but is part of another reservation then delete the
	 * mouse from its current reservation and add the new one.
	 *
	 * 4. If a mouse is not part of this reservation and is not selected then ignore.
	 *
	 * 5. If a mouse is not part of this reservation and is selected and is not part of another reservation then add it
	 * to this reservation.
	 *
	 * @param reservationInfo -
	 *            the reservation pojo.
	 * @param form -
	 *            the form backing object.
	 * @param user -
	 *            the logged in user.
	 * @throws IllegalArgumentException -
	 *             a parameter is null.
	 * @see uk.ac.sanger.mig.reservations.services.ReservationService#reserveMice(uk.ac.sanger.mig.reservations.Reservation,
	 *      uk.ac.sanger.mig.reservations.forms.ReservationForm, uk.ac.sanger.mig.personnel.Person)
	 */
	@Transactional
	public void reserveMice(Reservation reservationInfo, ReservationForm form, Person user) {
		if (reservationInfo == null || form == null || user == null)
			throw new IllegalArgumentException("Method does not accept null parameters");
		Long reservationId = store(reservationInfo, user);
		int numberOfMice = form.getMouseList().size();
		for (int i = 0; i < numberOfMice; i++) {
			Mouse mouse = form.getMouseList().get(i);
			if (!inSameReservation(reservationInfo, mouse)) { // if true, ignore
				if (form.getSelectedMouseIds().get(i) != null) { // is mouse selected
					if (hasReservation(mouse)) { // delete existing reservation
						mouseReservationService.delete(mouse.getMouseReservation(), user);
					}
					MouseReservation reservedMouse = new MouseReservation();
					reservedMouse.setMouseId(mouse.getId());
					reservedMouse.setReservationId(reservationId);
					mouseReservationService.store(reservedMouse, user);
				}
			} else if (hasReservation(mouse) && form.getSelectedMouseIds().get(i) == null) {
				// Mouse not selected but belongs to the same reservation id, so delete
				// the mouse reservation.
				mouseReservationService.delete(mouse.getMouseReservation(), user);
			}
		}
	}


	/**
	 * Returns true if the mouse has a mouse reservation, otherwise false.
	 *
	 * @param mouse -
	 *            the mouse to test.
	 * @return boolean - true, is part of a reservation, otherwise false.
	 */
	private boolean hasReservation(Mouse mouse) {
		return mouse.getMouseReservation() != null;
	}


	/**
	 * Tests if mouse is already part of a reservation and if it is the passed reservation.
	 *
	 * @param reservation -
	 *            the reservation object.
	 * @param mouse -
	 *            the mouse to test.
	 * @return boolean - true, in same reservation, otherwise false.
	 */
	private boolean inSameReservation(Reservation reservation, Mouse mouse) {
		MouseReservation mr = mouse.getMouseReservation();
		if (mr != null) {
			if (mr.getReservationId() == reservation.getId()) {
				return true;
			}
		}
		return false;
	}


	/**
	 * Calls the mouse reservation services delete method for each mouse in collection. The delete method wil delete the
	 * reservation if the deletion of a mouse reservation results in the reservation becoming empty.
	 *
	 * @param mouseIds -
	 *            collection of long mouse ids to delete.
	 * @param user -
	 *            the logged in user.
	 */
	@Transactional
	public void deleteReservationForMice(Collection<Long> mouseIds, Person user) {
		for (Long id : mouseIds) {
			MouseReservation mr = mouseReservationService.fetchForMouse(id, null);
			if (mr != null) {
				mouseReservationService.delete(mr, user);
			}
		}
	}


	/**
	 * Retrieve reservation that the mouse with mouse id is a part of, if any. Attatch the required components (see
	 * attachComponents()).
	 *
	 * @param mouseId -
	 *            the id of the mouse to fetch.
	 * @param requiredComponents -
	 *            map of components to attatch.
	 * @return r = the reservation that mouse is a part of.
	 * @see uk.ac.sanger.mig.reservations.services.ReservationService#fetchByMouseId(java.lang.Long,
	 *      uk.ac.sanger.mig.common.ComponentMap)
	 */
	public Reservation fetchByMouseId(Long mouseId, ComponentMap requiredComponents) {
		Reservation r = getPrimaryDao().fetchForMouse(mouseId);
		if (r != null)
			attachComponents(r, requiredComponents);
		return r;

	}


	/**
	 * Deletes the reservation with id reservationId. Won't delete a non-empty reservation i.e. if the reservation still
	 * contains mouse reservations then this method will not delete the reservation.
	 *
	 * @param reservationId -
	 *            the id of the reservation to delete.
	 * @param user -
	 *            the logged in user.
	 * @throws IllegalArgumentException -
	 *             reservationId is null.
	 * @see uk.ac.sanger.mig.reservations.services.ReservationService#deleteReservationIfEmpty(java.lang.Long,
	 *      uk.ac.sanger.mig.personnel.Person)
	 */
	@Transactional
	public void deleteReservationIfEmpty(Long reservationId, Person user) {
		Reservation r = fetchById(reservationId, ComponentMap.of(Reservation.Component.MOUSE_RESERVATIONS));

		if (r == null)
			throw new IllegalArgumentException("Not a valid reservation id: " + reservationId);

		if (r.getMouseReservations().isEmpty()) {
			getPrimaryDao().delete(r, user);
		}

	}
}
