/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.reservations.validators;

import org.springframework.validation.Errors;
import uk.ac.sanger.mig.common.AbstractBindingValidator;
import uk.ac.sanger.mig.common.AllowedFormField;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.reservations.Reservation;
import uk.ac.sanger.mig.reservations.forms.ReservationForm;

import java.util.HashSet;
import java.util.Set;


public class ReservationFormValidator extends AbstractBindingValidator {

	private PersonService personService;


	/**
	 * The person service used to validate the reserver exists and has an email address set if the 'alert on death'
	 * checkbox was selected.
	 *
	 * @param personService - the person service interface to PersonServiceImpl.
	 */
	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}


	/**
	 * Defines the form fields and their status, required or not. The fields are then validated using the
	 * ValidationUtils.rejectIfEmptyOrWhitespace() method (if they have no bind errors). The id fields are set as required,
	 * whilst the string fields are not - we definitely need the ids.
	 *
	 * @return formFields - a set of AllowedFormField objects.
	 */
	@Override
	public Set<AllowedFormField> defineAllowedFields() {

		Set<AllowedFormField> formFields = new HashSet<AllowedFormField>();

		// Required
		formFields.add(new AllowedFormField("reservation.reserverId", "reserver name", true));
		formFields.add(new AllowedFormField("reservation.teamId", "reserver team", true));
		formFields.add(new AllowedFormField("reservation.categoryId", "reservation category", true));
		formFields.add(new AllowedFormField("reservation.purposeId", "reservation purpose", true));

		// Optional.
		formFields.add(new AllowedFormField("reservation.reserverName", "reserver name", false));
		formFields.add(new AllowedFormField("reservation.teamName", "reserver team", false));
		formFields.add(new AllowedFormField("reservation.purposeName", "reservation purpose", false));
		formFields.add(new AllowedFormField("reservation.notify", "alert on death", false));

		return formFields;
	}


	/**
	 * If no bind errors have taken place and the reserver id is set then fetch the person, check a person was found and,
	 * if 'alert on death' was checked, verify that the person has an email address set.
	 *
	 * @param object		   - the form backing object.
	 * @param errors		   - the Errors
	 * @param hasBindingErrors - true if bding between the form and the backing object has failed, perhaps due to type
	 *                         casting errors, otherwise false.
	 */
	@Override
	protected void validate(Object object, Errors errors, boolean hasBindingErrors) {

		ReservationForm reservationForm = (ReservationForm) object;
		Reservation reservation = reservationForm.getReservation();

		if (!hasBindingErrors && reservation.getReserverId() != null) {

			Person person = personService.fetchById(reservation.getReserverId(), null);

			if (person == null) {
				// Shouldn't happen, but if it does then this catches the reserverId pointing to no person.
				errors.reject(null, "failed to find reserver");
			} else if (reservation.isNotify() && person.getEmail() == null) {
				// Reject the submission if the reserver has no email set.
				errors.reject(null, "reserver does not have an email set for notification");
			}
		}
	}


	public boolean supports(Class clazz) {
		return ReservationForm.class.equals(clazz);
	}

}