/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.tecniplast.services;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.tecniplast.AhuMeasurement;
import uk.ac.sanger.mig.tecniplast.AirHandlingUnit;


public class AirHandlingUnitDataImportServiceImpl implements
		AirHandlingUnitDataImportService {
	
	public static final Log logger = LogFactory.getLog(AirHandlingUnitDataImportServiceImpl.class);
	
	private static final Long LOAD_USER_PERSON_ID = 1L; // MIG user id number for the user that is listed as having
	                                                    // loaded each of the measurements into the Oracle DB.
	
	private static final int EXPECTED_COLUMN_COUNT = 6;
	private static final int DATE_COLUMN = 0;
	private static final int TEMPERATURE_COLUMN = 1;
	private static final int RELATIVE_HUMIDITY_COLUMN = 2;
	private static final int AIR_CHANGES_PER_HOUR_COLUMN = 3;
	private static final int RELATIVE_PRESSURE_COLUMN = 4;
	private static final int ERROR_CODE_COLUMN = 5;
	
	private DateFormat formatter = new SimpleDateFormat("dd/MM/yy HH:mm");

	private String ahuDataDirectory;
	private String backupDirectory;
	
	private AirHandlingUnitService airHandlingUnitService;
	private AhuMeasurementService ahuMeasurementService;
	
	private PersonService personService;
	
	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setAhuMeasurementService(AhuMeasurementService ahuMeasurementService) {
		this.ahuMeasurementService = ahuMeasurementService;
	}

	public void setAirHandlingUnitService(
			AirHandlingUnitService airHandlingUnitService) {
		this.airHandlingUnitService = airHandlingUnitService;
	}

	public void setAhuDataDirectory(String ahuDataDirectory) {
		this.ahuDataDirectory = ahuDataDirectory;
	}

	public void setBackupDirectory(String backupDirectory) {
		this.backupDirectory = backupDirectory;
	}

	public AirHandlingUnitDataImportServiceImpl() {
		logger.debug("Air Handling Unit Data Import Service Initalised");
	}
	
	
	
	/**
	 * Do the main service work. Import AHU data (currently Tecniplast) into the
	 * database at regular intervals. This is called bu a quartz scheduling task.
	 */
	public void importMeasurementData() {
		
		logger.debug("Starting Data Collection");
		
		Person loadUser = personService.fetchById(LOAD_USER_PERSON_ID, null);
		
		Collection<String> files = getCsvFilesInDir(ahuDataDirectory);
		
		for (String file : files) {
			
			/*
			 * Make a backup of the CSV data prior to doing the import.
			 */
			String backupFile = backupDirectory + File.separatorChar + file;
			String originalFile = ahuDataDirectory + File.separatorChar + file;
			copy(originalFile, backupFile);
			
			importAirHandlingMeasurments(new File(backupFile), loadUser);
			
		}
	}
	
	/*
	 * Get the collection of files with a file name ending '.csv' that are
	 * located in the supplied directory. If the directory does not contain
	 * any .csv files then it will return an empty Collection.
	 */
	private Collection<String> getCsvFilesInDir(String path){
		
		Pattern pattern = Pattern.compile("([a-zA-Z0-9]+).csv");
		String[] list = new File(path).list();
		
		Collection<String> res = new ArrayList<String>();
		
		if(list == null){
			return res;
		}
		
		//Go through each file in the directory
		int len = list.length;
		for (int i = 0; i < len; i++) {
			
			Matcher matcher = pattern.matcher(list[i]);
			if(matcher.find()){
				res.add(list[i]);
			}
		}
		return res;
	}
	
	/*
	 * Do the guts of the import processing. Read the file (wich is CSV formatted),
	 * skip any rows that dont have a date > than the last date we captured data for this AHU
	 * and any duplicate rows (Tecniplast put these in under certain, unclear, error
	 * conditions).
	 */
	private void importAirHandlingMeasurments(File src, Person loadUser){
		
		String ahuRadioAddress = getRadioAddress(src);
		
		AirHandlingUnit airHandlingUnit = airHandlingUnitService.fetchByRadioAddress(ahuRadioAddress, ComponentMap.of(AirHandlingUnit.Component.ALL_COMPONENTS));
		if(airHandlingUnit == null){
			// We will ignore any AHU serial number that we don't already have registered in the system.
			// This avoids pulling data for unknown AHU's we log it and move on
			
			logger.warn("Unknown Air Handling Unit Radio Address (" + ahuRadioAddress + ") found" );
			return;
			
		}

		try{
			BufferedReader in = new BufferedReader(new FileReader(src));
			String dataRow = null;

			Date lastStoredDate = airHandlingUnitService.getLastCollectionDateByAhuId(airHandlingUnit.getId());
			
			while((dataRow = in.readLine()) != null){
				
				String[] row = dataRow.split(",");
				
				if(row.length == EXPECTED_COLUMN_COUNT){

					Date rowDate = formatter.parse(row[DATE_COLUMN]);

					// Check that the rowDate is not null (i.e. no previous entries) and is more recent
					// than the last entry made for the AHU
					if((lastStoredDate == null) || (rowDate.compareTo(lastStoredDate) > 0)){
						
						logger.debug(ahuRadioAddress + " - Storing");
						
						AhuMeasurement measurement = new AhuMeasurement();
						
						measurement.setAirHandlingUnitId(airHandlingUnit.getId());
						measurement.setCollectionDate(rowDate);
						measurement.setTemperature(Integer.parseInt(row[TEMPERATURE_COLUMN]));
						measurement.setRelativeHumidity(Integer.parseInt(row[RELATIVE_HUMIDITY_COLUMN]));
						measurement.setAirChangesPerHour(Integer.parseInt(row[AIR_CHANGES_PER_HOUR_COLUMN]));
						measurement.setRelativePressure(Integer.parseInt(row[RELATIVE_PRESSURE_COLUMN]));
						measurement.setErrorCode(Integer.parseInt(row[ERROR_CODE_COLUMN]));
						
						ahuMeasurementService.store(measurement, loadUser);
						
						lastStoredDate = rowDate;
						
					} else if((lastStoredDate == null) || (rowDate.compareTo(lastStoredDate) == 0)) {
						// This case covers a duplicate row in the input file. Skip these rows until 
						// we find one with a date > than the currently last stored one.
						
						logger.debug(ahuRadioAddress + " - Skipping");
						continue;
					}
				}
			}
		}catch(FileNotFoundException fnfe){
			fnfe.printStackTrace();
		}catch(IOException ioe){
			ioe.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/*
	 * Copy a file using the underlying OS copy command. This is a speed optimisation
	 * that makes the class Unix specific.
	 */
	private void copy(String sourceFile, String destinationFile){
		
		try{
			String[] cmd = { "/bin/cp", sourceFile, destinationFile };
			Runtime.getRuntime().exec(cmd);

		}catch(FileNotFoundException fnfe){
			fnfe.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/*
	 * Compute the RadioAddress of the unit based on the filename supplied. Basically its the base filename
	 * (i.e. without the full path) upper cased and with the .CSV stripped off.
	 */
	private String getRadioAddress(File src){
		
		String ahuRadioAddress;
		
		//Oops no meanigfull filename to work from.
		if(src.getName() == null)
			return null;
		
		// File names are always of the form 'FEF1C40A0000.csv' where the first
		// part of the name is the serial number of the AHU.
		ahuRadioAddress = src.getName().toUpperCase();
		int dotPos = ahuRadioAddress.indexOf('.');
		
		//Oops... No '.CSV' ending
		if(dotPos < 0)
			return null;
		
		ahuRadioAddress = ahuRadioAddress.substring(0, dotPos);
		
		return ahuRadioAddress;
	}

}
