/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.tools;
import java.io.File;
import java.io.FileWriter;
import java.io.Writer;

import javax.sql.DataSource;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import uk.ac.sanger.mig.tools.classgenerator.Table;
import uk.ac.sanger.mig.tools.classgenerator.Utils;

public class ClassGenerator {
	
	private Table table;
	private String name;
	private String pkg;
	private static String tableName;
	private static String className;
	private static String packageName;
	private static String sourceDirectory;

	public ClassGenerator(Table table, String className, String packageName, String sourceDirectory) {
		this.name = className;
		this.table = table;
		this.pkg = packageName;
	}
	
	public String getName() {
		return Utils.toCamelCase(name, true);
	}
	
	public String getPackageName() {
		return pkg;
	}
	
	public String getInstance() {
		return Utils.toCamelCase(name, false);
	}
	
	public String getUnderscoreClassName() {
		return name.toLowerCase();
	}
	
	public Table getTable() {
		return table;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		
		if(args.length < 2 || args.length > 4) {
			System.out.println("Usage: java ClassGenerator <ORACLE_TABLE_NAME> <POJO_NAME (lowercase)> [<PACKAGE_NAME>] [<SOURCE_DIR> defaults to src]\n"+
					           "e.g. java ClassGenerator GNM_LOCUS locus uk.ac.sanger.mig.gene src");
			System.exit(1);
		}
		
		tableName = args[0];
		className = args[1];
		
		if(args.length == 2) {
			packageName = "uk.ac.sanger.mig.tools.classgenerator.sandbox";
		} else {
			packageName = args[2];
		}
		

		String srcPath = packageName.replace('.', File.separatorChar);

		
		if(args.length == 2 || args.length == 3) {
			sourceDirectory = "src" + File.separator + srcPath;
		} else {
			sourceDirectory = args[3] + File.separator + srcPath;
		}
		
		System.out.println("Creating classes for: " + tableName + 
				          "\nClass: " + className +
				          "\nPackage: " + packageName +
				          "\nDestination: " + new File(sourceDirectory).getAbsolutePath());
		
		ClassPathXmlApplicationContext appContext = 
			new ClassPathXmlApplicationContext(new String[] {"uk/ac/sanger/mig/tools/classgenerator/ClassGeneratorContext.xml"});
		
		BeanFactory factory = (BeanFactory)appContext;
		
		Utils utils = new Utils((DataSource)factory.getBean("dataSource")); 
		
		ClassGenerator newClass = new ClassGenerator(utils.fetchTable(tableName), className, packageName, sourceDirectory);
		
		Velocity.init();		
		VelocityContext context = new VelocityContext();
		context.put("class", newClass);	
		
		new File(sourceDirectory).mkdir();
		new File(sourceDirectory+"/dao").mkdir();
		new File(sourceDirectory+"/dao/rowmappers").mkdir();
		new File(sourceDirectory+"/controllers").mkdir();
		new File(sourceDirectory+"/views").mkdir();
		new File(sourceDirectory+"/forms").mkdir();
		new File(sourceDirectory+"/services").mkdir();
		new File(sourceDirectory+"/validators").mkdir();
		new File(sourceDirectory+"/jsp").mkdir();
    
		writeRowMapper(context, newClass);
		writeDao(context, newClass);
		writeDaoSpringImpl(context, newClass);
		writePojo(context, newClass);
		writeViewOneController(context, newClass);
		writeListController(context, newClass);
		writeFormController(context, newClass);
		writeService(context, newClass);
		writeServiceImpl(context, newClass);
    writeServiceTest(context, newClass);
		writeForm(context, newClass);
		writeFormValidator(context, newClass);
		writeViewOneJSP(context, newClass);
		writeListJSP(context, newClass);
		writeEditJSP(context, newClass);
		writeSearchBean(context, newClass);
	}
	
	private static void writeSearchBean(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/" + newClass.getName() + "SearchBean.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/SearchBean.vm", "UTF-8", context, writer);
		writer.close();	
	}
	
	private static void writeRowMapper(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/dao/rowmappers/" + newClass.getName() + "RowMapper.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/RowMapper.vm", "UTF-8", context, writer);
		writer.close();	
	}

	private static void writeDao(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/dao/" + newClass.getName() + "Dao.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/Dao.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writeDaoSpringImpl(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/dao/" + newClass.getName() + "DaoSpringImpl.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/DaoSpringImpl.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writePojo(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/" + newClass.getName() + ".java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/Pojo.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writeViewOneController(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/controllers/ViewOne" + newClass.getName() + "Controller.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/ViewOneController.vm", "UTF-8", context, writer);
		writer.close();	
	}
	
	private static void writeListController(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/controllers/" + newClass.getName() + "ListController.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/ListController.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writeFormController(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/controllers/" + newClass.getName() + "FormController.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/FormController.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writeService(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/services/" + newClass.getName() + "Service.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/Service.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writeServiceImpl(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/services/" + newClass.getName() + "ServiceImpl.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/ServiceImpl.vm", "UTF-8", context, writer);
		writer.close();	
	}
  
  private static void writeServiceTest(VelocityContext context, ClassGenerator newClass) throws Exception {
    File rowMapper = new File(sourceDirectory + "/services/" + newClass.getName() + "ServiceTest.java");
    Writer writer = new FileWriter(rowMapper);    
    Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/ServiceImplTest.vm", "UTF-8", context, writer);
    writer.close(); 
  }
	
	private static void writeForm(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/forms/" + newClass.getName() + "Form.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/Form.vm", "UTF-8", context, writer);
		writer.close();	
	}	

	private static void writeFormValidator(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/validators/" + newClass.getName() + "FormValidator.java");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/FormValidator.vm", "UTF-8", context, writer);
		writer.close();	
	}
	
	private static void writeViewOneJSP(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/jsp/view_one_" + newClass.getUnderscoreClassName() + ".jsp");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/ViewOneJSP.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writeListJSP(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/jsp/list_" + newClass.getUnderscoreClassName() + "s.jsp");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/ListJSP.vm", "UTF-8", context, writer);
		writer.close();	
	}	
	
	private static void writeEditJSP(VelocityContext context, ClassGenerator newClass) throws Exception {
		File rowMapper = new File(sourceDirectory + "/jsp/edit_" + newClass.getUnderscoreClassName() + ".jsp");
		Writer writer = new FileWriter(rowMapper);		
		Velocity.mergeTemplate("src/uk/ac/sanger/mig/tools/classgenerator/EditJSP.vm", "UTF-8", context, writer);
		writer.close();	
	}		
}
