/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.tools.datafix;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.database.TableMetadataEnum;
import uk.ac.sanger.mig.mousetracking.services.MouseProcedureService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSet;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSetSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ExperimentSetService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.ObservationService;
import uk.ac.sanger.mig.phenotyping.datacapture.services.RequiredObservationService;
import uk.ac.sanger.mig.phenotyping.datacapture.views.ExperimentSetListView;

public class PhlebotomyRepairService {
  
  // services
  private ExperimentSetService experimentSetService;
  private ExperimentService experimentService;
  private ObservationService observationService;
  private MouseProcedureService mouseProcedureService;
  private PersonService personService;
  private RequiredObservationService requiredObservationService;
  
  private Log logger = LogFactory.getLog(this.getClass());
  
  // user to take responsibility for changes 
  private Person user;
  
  
  private static final Long SOP_ID = 120L;
  
  @Transactional
  public void resolveDuplicateRecords() {
    
    Collection<Collection<Experiment>> problems = findProblematicRecords();
    
    logger.info("Found problems: "+problems.size());
    
    for(Collection<Experiment> expGroup : problems) {
      fixForOneMouse(expGroup);
    }
    
    updateExperimentSets(); 
  }
  
  private void updateExperimentSets() {
    int deleteCount = 0;
    int completeCount = 0;
    int incompleteCount = 0;
    
    ExperimentSetSearchBean criteria = new ExperimentSetSearchBean();
    
    criteria.setExperimentSet(new ExperimentSet());
    criteria.getExperimentSet().setSopId(SOP_ID);
    criteria.setPaginated(false);
    
    Collection<ExperimentSetListView> views = experimentSetService.search(criteria);
    
    for (ExperimentSetListView view : views) {
      ExperimentSet set = experimentSetService.fetchById(view.getId(), null);
      
      // if empty, remove
      if (view.getExperimentCount() < 1) {
        experimentSetService.delete(set, getUser());
        deleteCount++;
      }
      // complete
      else if (view.getCompletedExperimentCount() == view.getExperimentCount()) {
        set.setComplete(true);
        experimentSetService.store(set, getUser());
        completeCount++;
      }
      // incomplete
      else if (view.getCompletedExperimentCount() < view.getExperimentCount()) {
        set.setComplete(false);
        experimentSetService.store(set, getUser());
        incompleteCount++;
      }
      else {
        throw new RuntimeException("Went to update experiment set but cannot work out what to do: "+set.getId());
      }
    }
    
    logger.debug("Deleted experiment sets: "+deleteCount);
    logger.debug("Updated experiment sets to complete: "+completeCount);
    logger.debug("Updated experiment sets to incomplete: "+incompleteCount);
  }

  private void fixForOneMouse(Collection<Experiment> expGroup) {
    Experiment firstRecord = null;
    Collection<Observation> removableObservations = new LinkedList<Observation>();
    Map<RequiredObservation, Observation> observationsToUse = new HashMap<RequiredObservation, Observation>(); 
    
    // loop through the records, identify the experiment to keep and the observations to attach to it
    for (Experiment e : expGroup) {
      checkMouseConsistency(firstRecord, e);
      firstRecord = useEarliest(firstRecord, e);
      
      chooseObservations(removableObservations, observationsToUse, e);
    }
    
    boolean noMissingValues = true;
    
    
    // point the observations we're using towards the experiment we're keeping
    for (Observation o : observationsToUse.values()) {
      o.setExperimentId(firstRecord.getId());
      observationService.storeAuthenticated(o, getUser());
      
      if (empty(o) && o.getRequiredObservation().isMandatoryObservation()) noMissingValues = false;
      
      if (!empty(o) && !o.isValid()) throw new RuntimeException("Trying to store invalid data in observation "+o.getId());
    }
    
    // store the experiment we are keeping
    firstRecord.setComplete(noMissingValues);
    experimentService.storeAuthenticated(firstRecord, getUser());
    
    // get rid of the surplus
    for (Observation o : removableObservations) {
      observationService.delete(o, getUser());
    }
    
    // remove the unnecessary duplicates and their procedure records
    for (Experiment e : expGroup) {
      if (e != firstRecord) {
        mouseProcedureService.deleteByTableRef(
            TableMetadataEnum.PHN_EXPERIMENT,
            e.getId(),
            getUser()
        );
        experimentService.delete(e, getUser());
      }
    }
    
  }

  private Person getUser() {
    if (user == null) {
      String userName = System.getProperty("user.name");
      
      if (userName == null) throw new RuntimeException("Could not find logged in user");
      
      user = personService.fetchByUserName(userName,null);
      
      if (user == null) throw new RuntimeException("Could not find user for user name: "+userName);
    }
    
    return user;
  }

  private void chooseObservations(Collection<Observation> removableObservations, Map<RequiredObservation, Observation> observationsToUse,
      Experiment e) {
    for (Observation o : e.getObservations()) {
      boolean useNewRecord = false;
      boolean haveExistingRecord = observationsToUse.containsKey(o.getRequiredObservation());
      
      if (haveExistingRecord) {
        Observation existingRecord = observationsToUse.get(o.getRequiredObservation());
        useNewRecord = useNewRecord(existingRecord, e, o);
      }
      else {
        useNewRecord = true;
      }
      
      if (useNewRecord) {
        if (haveExistingRecord) {
          Observation oldRecord = observationsToUse.get(o.getRequiredObservation());
          removableObservations.add(oldRecord);
        }
        
        observationsToUse.put(o.getRequiredObservation(),o);
      }
      else {
        removableObservations.add(o);
      }
    }
  }

  private boolean useNewRecord(Observation current, Experiment experiment, Observation candidate) {
    boolean useNewRecord = false;
    
    // use the observation with a value
    if (empty(current) && !empty(candidate)) {
      useNewRecord = true;
    }
    
    // use the oldest if they are both empty
    if (empty(current) && empty(candidate)) {
      useNewRecord = candidate.getCreatedDate().before(current.getCreatedDate());
    }
    // records not empty, use the oldest if they are equal
    if (!empty(current) && !empty(candidate)){
      // if the data is equivalent, or it's a barcode, use the first one
      if (candidate.getValue().equals(current.getValue()) || candidate.getRequiredObservation().getObservationType().isBloodSampleBarcode()) {
        useNewRecord = candidate.getCreatedDate().before(current.getCreatedDate());  
      }
      else {
        throw new RuntimeException("Observations are not empty and not equal, panic! For mouse "+experiment.getMouseId());
      }
    }
    return useNewRecord;
  }

  private boolean empty(Observation o) {
    return (o.getValue() == null || o.getValue().length() < 1);
  }
  
  private void checkMouseConsistency(Experiment firstRecord, Experiment e) {
    if (firstRecord != null && !e.getMouseId().equals(firstRecord.getMouseId())) {
      throw new RuntimeException("Abort! Abort! All experiments should be for the same mouse");
    }
  }

  private Experiment useEarliest(Experiment firstRecord, Experiment e) {
    
    if (firstRecord == null ||
        e.getCreatedDate().before(firstRecord.getCreatedDate())) {
      firstRecord = e;
    }
    return firstRecord;
  }

  private Collection<Collection<Experiment>> findProblematicRecords() {
  
    Collection<Experiment> relevantRecords = findRelevantRecords();
   
    Map<Long, Collection<Experiment>> expsByMouseId = groupExperimentsByMouse(relevantRecords);
   
    removeGoodRecords(expsByMouseId);
    
    attachRecords(expsByMouseId.values());
    
    return expsByMouseId.values();
  }

  // normal attachment is too slow, can be improved by caching the reqObv entries
  private void attachRecords(Collection<Collection<Experiment>> groupedExperiments) {
    Map<Long,RequiredObservation> localCache = new HashMap<Long,RequiredObservation>();
    
    
    for(Collection<Experiment> exps : groupedExperiments) {
      
      for(Experiment e : exps) {
        Collection<Observation> observations = observationService.fetchByExperimentId(e.getId(), null);
        e.setObservations(observations);
        
        for (Observation o : observations) {
          if (!localCache.containsKey(o.getRequiredObservationId())) {
            RequiredObservation ro = requiredObservationService.fetchById(
                o.getRequiredObservationId(),
                new ComponentMap(
                    RequiredObservation.Component.OBSERVATION_TYPE,
                    ComponentMap.of(ObservationType.Component.ACTIVE_RESTRICTED_OPTIONS)
                )
            );
            
            localCache.put(o.getRequiredObservationId(),ro);
          }
          
          o.setRequiredObservation(localCache.get(o.getRequiredObservationId()));
          
          
          if (!o.getRequiredObservationId().equals(o.getRequiredObservation().getId())) {
            throw new RuntimeException("Ids don't match up on required observations");
          }
        }
      }
    }
    
  }

  private Map<Long, Collection<Experiment>> groupExperimentsByMouse(Collection<Experiment> relevantRecords) {
    Map<Long,Collection<Experiment>> expsByMouseId = new HashMap<Long,Collection<Experiment>>();

    for (Experiment experiment : relevantRecords) {
      
      if (!expsByMouseId.containsKey(experiment.getMouseId())) {
        expsByMouseId.put(experiment.getMouseId(), new LinkedList<Experiment>());
      }
      
      Collection<Experiment> experimentsForMouse = expsByMouseId.get(experiment.getMouseId());
      experimentsForMouse.add(experiment);
    }
    return expsByMouseId;
  }

  private Collection<Experiment> findRelevantRecords() {
    ExperimentSearchBean criteria = new ExperimentSearchBean();
    criteria.setPaginated(false);
    criteria.setExperiment(new Experiment());
    criteria.getExperiment().setSopId(SOP_ID);
    
    Collection<Experiment> relevantRecords = experimentService.search(
        criteria,
        null
    );
    return relevantRecords;
  }

  private void removeGoodRecords(Map<Long, Collection<Experiment>> expsByMouseId) {
    // It's OK if there aren't any duplicates
    Iterator<Map.Entry<Long, Collection<Experiment>>> i = expsByMouseId.entrySet().iterator();
    while (i.hasNext()) {
      Map.Entry<Long, Collection<Experiment>> entry = i.next();
      Collection<Experiment> experiments = entry.getValue();
      
      if (experiments.size() < 2) {
        i.remove();
      }
    }
  }

  public void setRequiredObservationService(RequiredObservationService requiredObservationService) {
    this.requiredObservationService = requiredObservationService;
  }

  public void setExperimentSetService(ExperimentSetService experimentSetService) {
    this.experimentSetService = experimentSetService;
  }

  public void setExperimentService(ExperimentService experimentService) {
    this.experimentService = experimentService;
  }

  public void setObservationService(ObservationService observationService) {
    this.observationService = observationService;
  }

  public void setMouseProcedureService(MouseProcedureService mouseProcedureService) {
    this.mouseProcedureService = mouseProcedureService;
  }

  public void setPersonService(PersonService personService) {
    this.personService = personService;
  }

}
