package uk.ac.sanger.mig.abr.parser;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import org.junit.Before;
import org.junit.Test;

/*
 *****************************************************
 * Class name: FileParserTest
 * User: holroyd
 * Date: Aug 5, 2008
 * Time: 10:52:42 AM
 *****************************************************
 */
public class FileParserTest extends FileParserTestSetUp {

	/**
	 * Sets up the test. Creates a new FileParser object and asks it to pass the project test file.
	 * Catches any exceptions thrown.
	 */
	@Before
	public void setUp() {
		super.setUp();
	}


	/**
	 * Tests that the <code>fileParser.getAllTraceBlocks()</code> method returns Block objects
	 * that contain trace data.
	 */
	@Test
	public void getTraceBlocks() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> traceBlocks = fp.getTraceBlocks();
			for (Block b : traceBlocks) {
				assertTrue(b.isTraceBlock());
			}
		}
	}


	/**
	 * Tests that the getAbrTraceBlocks() method returns only ABR trace blocks. This method is used
	 * to feed other methods with blocks so it's important it is supplying the correct types of
	 * blocks.
	 * 
	 */
	@Test
	public void getAbrTraceBlocks() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> blocks = fp.getAbrTraceBlocks();
			for (Block b : blocks) {
				assertTrue(b.isAbrTraceBlock());
				assertTrue(!b.isCalibrationTraceBlock());
				assertTrue(!b.isClickBlock());
				assertTrue(b.getBlockTraceInfo().getNumberPoints() == FileParser.DEFAULT_NUMBER_DATAPOINTS);
			}
		}
	}


	/**
	 * Tests that the getFreqAbrTraceBlocks() method returns only ABR trace blocks and excludes
	 * calibration and click blocks.
	 * 
	 */
	@Test
	public void getFreqAbrTraceBlocks() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			int frequency = 100;
			Collection<Block> blocks = fp.getFreqAbrTraceBlocks(frequency);
			for (Block b : blocks) {
				assertTrue(b.isAbrTraceBlock());
				assertTrue(!b.isCalibrationTraceBlock());
				assertTrue(!b.isClickBlock());
			}
		}
	}


	/**
	 * Tests that the getAllCalibrationBlocks() method returns blocks that are calibration trace
	 * blocks only.
	 */
	@Test
	public void testCalibrationBlockFound() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> traceBlocks = fp.getCalibrationBlocks();
			boolean isCalibrationBlock = true;
			for (Block block : traceBlocks) {
				if (!block.isCalibrationTraceBlock()) {
					isCalibrationBlock = false;
				}
			}
			assertTrue("a collection contining at least one block was returned", traceBlocks.size() > 0);
			assertTrue("returned blocks are calibration blocks", isCalibrationBlock);
		}
	}


	/**
	 * Tests that the getCalibrationTraceBlock() method, used to return the one and only calibration
	 * trace block, is returning a Block object. Tests that that blocks boolean method, which is
	 * used to define if it is a calibration block, is returning true and that the expected block is
	 * returned (using the test file).
	 */
	@Test
	public void getCalibrationTraceBlock() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Block calibrationBlock = fp.getCalibrationTraceBlock();
			assertNotNull("Block object returned", calibrationBlock);
			assertTrue("is a calibration block", calibrationBlock.isCalibrationTraceBlock());
			assertTrue(!calibrationBlock.isClickBlock());
		}
	}


	/**
	 * Tests that the getAbrTraceBlocks() returns blocks that contain data points at that a
	 * frequency and level are available.
	 * 
	 */
	@Test
	public void getDataPoints() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> traceBlocks = fp.getAbrTraceBlocks();
			for (Block block : traceBlocks) {
				assertTrue(block.isAbrTraceBlock());
				int expectedNumberDataPoints = block.getBlockTraceInfo().getNumberPoints();
				double frequency = block.getTraceName().getFrequency();
				double level = block.getTraceName().getLevel();
				assertNotNull(frequency);
				assertNotNull(level);
				assertTrue("frequency is non zero", frequency > 0);
				assertTrue("level is non zero", level > 0);
				double[] dataPoints = block.getDataPoints();
				assertTrue("one or more data points returned", dataPoints.length > 0);
				assertTrue("number of expected data points equals actual number found",
						expectedNumberDataPoints == dataPoints.length);
			}
		}
	}


	/**
	 * Tests that the getAbrTraceBlocksByFrequencyLevel() method returns frequencies with levels
	 * that have blocks - a frequency with no level should not be possible, as well as a level
	 * without at least one block should not be possible.
	 * 
	 */
	@Test
	public void getAbrTraceBlocksByFrequencyLevelAll() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Frequency> frequencies = fp.getAbrTraceBlocksByFrequencyLevel();
			for (Frequency f : frequencies) {
				Collection<Level> levels = f.getLevels();
				assertTrue(!levels.isEmpty());
				for (Level l : levels) {
					List<Block> blocks = new ArrayList<Block>(l.getBlocks());
					assertTrue(!blocks.isEmpty());
					Block lastListBlock = blocks.get(blocks.size() - 1);
					Block block = l.getBlock();
					assertEquals(lastListBlock, block);
					for (Block b : blocks) {
						assertTrue(!b.isClickBlock());
						assertTrue(b.getBlockTraceInfo().getNumberPoints() > 0);
						double[] dataPoints = b.getDataPoints();
						assertTrue(dataPoints.length > 0);
						for (int i = 0; i < dataPoints.length; i++) {
							double dp = dataPoints[i];
							assertTrue("data point value should not be 0.0: " + dp, dp != 0.0);
						}
					}
				}
			}
		}
	}


	/**
	 * Tests that the getAbrTraceBlocksByFrequencyLevel() method that returns an ordered list of
	 * frequencies, which contain one or more levels, which contain blocks orders the frequencies
	 * and levels in ascending order.
	 * 
	 */
	@Test
	public void getAbrTraceBlocksByFrequencyLevelOrdering() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Frequency> frequencies = fp.getAbrTraceBlocksByFrequencyLevel();
			Iterator fi = frequencies.iterator();
			Frequency testFrequency = (Frequency) fi.next();
			while (fi.hasNext()) {
				Frequency f = (Frequency) fi.next();
				assertTrue(testFrequency.getFrequency() <= f.getFrequency());
				testFrequency = f;
				Collection<Level> levels = f.getLevels();
				Iterator li = levels.iterator();
				Level testLevel = (Level) li.next();
				while (li.hasNext()) {
					Level l = (Level) li.next();
					assertTrue(testLevel.getLevel() <= l.getLevel());
					testLevel = l;
				}
			}
		}
	}


	/**
	 * Tests the getAbrTraceBlocksByFrequencyLevel() method that returns an ordered list of
	 * frequencies, which contain one or more levels, which contain blocks. If more than one block
	 * has been associated with a frequency and level then the last block should be the one used.
	 * This tests that Block.getBlock() returns the last block.
	 * 
	 */
	@Test
	public void getAbrTraceBlocksByFrequencyLevel() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Frequency> frequencies = fp.getAbrTraceBlocksByFrequencyLevel();
			for (Frequency f : frequencies) {
				Collection<Level> levels = f.getLevels();
				for (Level l : levels) {
					List<Block> blocks = new ArrayList<Block>(l.getBlocks());
					if (blocks.size() > 1) {
						// Get a list of the blocks from the original list straight from the file.
						List<Block> testBlocks = new ArrayList<Block>(findTestBlocksByFrequencyLevel(fp, f, l));
						assertEquals(blocks.size(), testBlocks.size());
						for (int i = 0; i < blocks.size(); i++) {
							Block actualBlock = blocks.get(i);
							Block testBlock = testBlocks.get(i);
							assertEquals(actualBlock, testBlock);
							assertEquals(actualBlock.getBlockTraceInfo().getNumberPoints(),
									FileParser.DEFAULT_NUMBER_DATAPOINTS);
						}
						assertEquals(l.getBlock(), testBlocks.get(testBlocks.size() - 1));
					}
				}
			}
		}
	}


	/**
	 * Tests that the getStartClickBlocks() returns a list of blocks that are flagged as click
	 * blocks.
	 * 
	 */
	@Test
	public void getStartClickBlocks() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> clickBlocks = fp.getStartClickBlocks();
			assertTrue(clickBlocks.size() > 0);
			for (Block b : clickBlocks) {
				assertTrue(b.isClickBlock());
			}
		}
	}


	/**
	 * Tests that the getEndClickBlocks() returns a list of blocks that are flagged as click blocks.
	 * 
	 */
	@Test
	public void getEndClickBlocks() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> clickBlocks = fp.getEndClickBlocks();
			assertTrue(clickBlocks.size() > 0);
			for (Block b : clickBlocks) {
				assertTrue(b.isClickBlock());
			}
		}
	}


	/**
	 * Tests that the getStartClickBlock() returns the last block from the list of start click
	 * blocks and that it is set as a click block.
	 * 
	 */
	@Test
	public void getStartClickBlock() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			List<Block> clickBlocks = fp.getStartClickBlocks();
			Block startClickBlock = fp.getStartClickBlock();
			assertEquals(clickBlocks.get(clickBlocks.size() - 1), startClickBlock);
			assertTrue(startClickBlock.isClickBlock());
			assertEquals("CLICK_BLOCK-TOP", startClickBlock.getTraceName().getMouseBarcode());
		}
	}


	/**
	 * Tests that the getEndClickBlock() returns the last block from the list of end click blocks
	 * and that it is set as a click block.
	 * 
	 */
	@Test
	public void getEndClickBlock() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			List<Block> clickBlocks = fp.getEndClickBlocks();
			Block endClickBlock = fp.getEndClickBlock();
			assertEquals(clickBlocks.get(clickBlocks.size() - 1), endClickBlock);
			assertTrue(endClickBlock.isClickBlock());
			assertEquals("CLICK_BLOCK-BTTM", endClickBlock.getTraceName().getMouseBarcode());
		}
	}


	/**
	 * Tests that the trace block attributes are stored in order. If a block attribute is suffixed
	 * with "_1" where 1 may be any number, then the BlockAttribute object will use the prefix
	 * attribute name as the name and the number after the underscore as th index. The Block stores
	 * the block attributes using their attribute name as the key and all of the associated block
	 * attributes in a collection as the value.
	 * 
	 */
	@Test
	public void getDataPointsBlockAttributes() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> blocks = fp.getAbrTraceBlocks();
			for (Block b : blocks) {
				List<BlockAttribute> blockAttributes = new ArrayList<BlockAttribute>(b.getBlockAttributes().get(
						Block.TRACE_DATA));
				assertTrue(blockAttributes.size() > 0);
				for (int i = 0, index = 1; i < blockAttributes.size(); i++, index++) {
					BlockAttribute ba = blockAttributes.get(i);
					assertEquals(ba.getIndex(), index);
				}
			}
		}
	}


	/**
	 * Tests that the header block can extract the experiments data and time correctly. This method
	 * uses a pattern to match the experiment date and the induction time elements. It then removes
	 * the elements and puts them together to be parsed into a Date object, so the elements format
	 * in the file is very important.
	 * 
	 */
	@Test
	public void extractExperimentDateTime() {
		Pattern p = Pattern
				.compile("^.*\\s*(\\d\\d)\\s*,\\s*(\\d\\d)\\s*,\\s*(\\d\\d\\d\\d)\\s*,\\s*(\\d\\d)\\s*,\\s*(\\d\\d).*$");
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			HeaderBlock headerBlock = fp.getHeaderBlock();
			String experimentTime = headerBlock.getDateOfExperiment();
			String inductionTime = headerBlock.getInductionTime();
			String dateToMatch = experimentTime + ", " + inductionTime;
			assertTrue(p.matcher(dateToMatch).matches());
		}
	}


	/**
	 * Verifies that there are no click blocks in the list of ABR trace blocks. Click blocks have a
	 * frequency of 100 and a level of 70.
	 * 
	 */
	@Test
	public void extractClickBlocks() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<Block> blocks = fp.getAbrTraceBlocks();
			for (Block block : blocks) {
				assertTrue(!block.isClickBlock());
			}
		}
	}


	/**
	 * Tests that user comments never return null.
	 * 
	 */
	@Test
	public void getUserComments() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			Collection<ExperimentComment> experimentComments = fp.getUserComments();
			assertNotNull(experimentComments);
		}
	}


	/**
	 * Tests that the isHarmfulMutant method returns the right thing depending on the contents of
	 * the file.
	 * 
	 */
	@Test
	public void isHarmfulMutant() {
		for (File file : this.getFiles()) {
			FileParser fp = this.getFileParser(file);
			HeaderBlock headerBlock = fp.getHeaderBlock();
			String optionalLicence = headerBlock.getOptionalLicence();
			if (optionalLicence == null) {
				assertTrue(!headerBlock.isHarmfulMutant());
			} else if (optionalLicence.toLowerCase().equals(HeaderBlock.HARMFUL_PROTOCOL_CODE)) {
				assertTrue(headerBlock.isHarmfulMutant());
			} else {
				assertTrue(!headerBlock.isHarmfulMutant());
			}
		}
	}


	/**
	 * Finds all blocks with matching frequency and levels from the list taken directly from the
	 * file. Returns the list of blocks for comparison with, hopefully, the same list of blocks that
	 * has been manipulated in some way during the parsing of the file.
	 * 
	 * @param fp -
	 *            the FileParser.
	 * @param f -
	 *            the frequency.
	 * @param l -
	 *            the level
	 * @return testBlocks - collections of blocks with matching frequency and level attributes.
	 */
	public Collection<Block> findTestBlocksByFrequencyLevel(FileParser fp, Frequency f, Level l) {
		Collection<Block> testBlocks = new ArrayList<Block>();
		int frequency = f.getFrequency();
		int level = l.getLevel();
		List<Block> blocks = fp.getAbrTraceBlocks();
		for (Block b : blocks) {
			if (!b.isAbrTraceBlock())
				continue;
			if (b.getTraceName().getFrequency() == frequency && b.getTraceName().getLevel() == level) {
				testBlocks.add(b);
			}
		}
		return testBlocks;
	}

}
