/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony.dao;

import static org.junit.Assert.assertNull;

import java.util.Collection;
import java.util.Date;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.dbunit.annotation.DataSet;
import org.unitils.reflectionassert.ReflectionAssert;
import org.unitils.reflectionassert.ReflectionComparatorMode;
import org.unitils.spring.annotation.SpringApplicationContext;
import org.unitils.spring.annotation.SpringBean;

import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.personnel.Person;

/*
 *
 * @DataSet annotation causes unitils to load data from
 * personDaoTest.xml into the unit test database before each test
 *
 * @SpringApplicationContext loads the specified xml files,
 * these should be the minimum dependencies to make the dao work
 *
 * the unitils-test-database.xml sets the migDataSource to
 * use the db configured in unitils. It should be the last
 * thing in the config list, to override any other datasource
 * definition.
 */
@SpringApplicationContext({"colony-dao.xml","test-config.xml","unitils-test-database.xml"})
@DataSet
public class ColonyInfoDaoTest extends UnitilsJUnit4 {


	@SpringBean("colonyInfoDao")
	private ColonyInfoDao colonyInfoDao;

	private Person testUser;
	private ColonyInfo colony;

	private void setupTestUser() {
		testUser = new Person();
		testUser.setId(10L);
		testUser.setUserName("test_user");
	}

	private void setupColony() {
		colony = new ColonyInfo();

		colony.setAssessingPhenotype("assessingPhenotype");
		colony.setAverageLitterSize(1);
		colony.setBreedingLifespanWeeks(1.2f);
		colony.setColonyName("Test Colony");
		colony.setColonyPrefix("TEST");
		colony.setColonyManagement("colonyManagement");
		colony.setColonyOrigin("colonyOrigin");
		colony.setDateOfImport(new Date());
		colony.setDateOfRederivation(new Date());
		colony.setDiet("diet");
		colony.setEnvironmentalEnrichment("environmentalEnrichment");
		colony.setHousingSystem("housingSystem");
		colony.setHusbandryRecommendations("husbandryRecommendations");
		colony.setImmuneStatus("immuneStatus");
		colony.setMiscNotes("miscNotes");
		colony.setModeOfInheritance("modeOfInheritance");
		colony.setOtherAbnormalities("otherAbnormalities");
		colony.setPlaceOfOrigin("placeOfOrigin");
		colony.setProjectLicenceId(1L);
		colony.setRederivationMethod("rederivationMethod");
		colony.setSubjectToMta(true);
		colony.setWelfareAssessment("welfareAssessment");
		colony.setWelfareAssessmentPerformed(true);
		colony.setWtsiOrigin(true);
		colony.setPoolable(true);
	}

	@Before
	public void setUp() {
		setupTestUser();
		setupColony();

		colonyInfoDao.store(colony, testUser);
	}

	@Test
	public void testFetchById_validId() {
		ColonyInfo testColony = colonyInfoDao.fetchById(colony.getId());
		ReflectionAssert.assertRefEquals(colony, testColony, ReflectionComparatorMode.LENIENT_DATES);
	}

	@Test
	public void testFetchById_invalidId() {
		ColonyInfo testColony = colonyInfoDao.fetchById(Long.MAX_VALUE);
		assertNull(testColony);
	}

	@Test @DataSet("colonyInfoDaoTest.exists_true.xml")
	public void testColonyNameExists_true() {
		Assert.assertTrue(colonyInfoDao.colonyNameExists(colony.getId(), colony.getColonyName()));
	}

	@Test @DataSet("colonyInfoDaoTest.exists_false.xml")
	public void testColonyNameExists_false() {
		Assert.assertFalse(colonyInfoDao.colonyNameExists(colony.getId(), colony.getColonyName()));
	}

	@Test @DataSet("colonyInfoDaoTest.exists_true.xml")
	public void testColonyPrefixExists_true() {
		Assert.assertTrue(colonyInfoDao.colonyPrefixExists(colony.getId(), colony.getColonyPrefix()));
	}

	@Test @DataSet("colonyInfoDaoTest.exists_false.xml")
	public void testColonyPrefixExists_false() {
		Assert.assertFalse(colonyInfoDao.colonyPrefixExists(colony.getId(), colony.getColonyPrefix()));
	}

	@Test @DataSet("colonyInfoDaoTest.fetchAll.xml")
	public void testFetchAll() {
		Collection<ColonyInfo> allRecords = colonyInfoDao.fetchForLookup(false);

		//3 records from XML file + 1 record from setup method = 4 records
		Assert.assertEquals(allRecords.size(), 4);
	}
}
