/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.colony.services;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.assertEquals;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.util.ArrayList;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.colony.dao.ColonyInfoDao;
import uk.ac.sanger.mig.colony.services.ColonyInfoServiceImpl;
import uk.ac.sanger.mig.colony.services.PrefixExclusionPredicate;

public class ColonyInfoServiceMockTest extends UnitilsJUnit4 {

	@TestedObject
	private ColonyInfoServiceImpl colonyInfoService;

	@Mock @InjectIntoByType private ColonyInfoDao colonyInfoDaoMock;

	private PrefixExclusionPredicate prefixExclusionPredicate;

	@Before
	public void setUp() {

		final char[] exclusionChars = new char[] {'I', 'O'};

		prefixExclusionPredicate = new PrefixExclusionPredicate() {
			public boolean exclude(String prefix) {

				for(char c : exclusionChars) {

					if(prefix.contains( Character.toString(c).toUpperCase() ) ) {

						return true;
					}
				}

				return false;
			}
		};

	}

	 @Test(expected=IllegalArgumentException.class)
	 public void getFirstUnusedPrefix_from_arg_greater_than_to_arg() {

		 colonyInfoService.getFirstUnusedPrefix("ZZZ", "AAA",null);
	 }

	 @Test(expected=IllegalArgumentException.class)
	 public void getFirstUnusedPrefix_string_args_different_width() {

		 colonyInfoService.getFirstUnusedPrefix("AA", "AAA",null);
	 }


	@Test
	public void getFirstUnusedPrefix_full_capacity() {

		String startPrefix   = "MAAA";
		String endPrefix     = "MZZZ";
		String expectedValue = null;

		ArrayList<String> fullList = new ArrayList<String>( (int)Math.pow(26.0, 3.0) );

		String prefix = startPrefix;

		while( prefix != null ) {//fill up the list

			fullList.add( prefix );
			prefix = colonyInfoService.increment(  prefix  );

		}
		expect(colonyInfoDaoMock.fetchUsedPrefixes(startPrefix, endPrefix)).andReturn(fullList);

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix(startPrefix, endPrefix,null);

		verify();

		assertEquals(expectedValue, actualValue);
	}

	@Test
	public void getFirstUnusedPrefix_last_prefix_available() {

		String startPrefix   = "MAAA";
		String endPrefix     = "MZZZ";
		String expectedValue = "MZZZ";

		ArrayList<String> oneUnusedPrefixList = new ArrayList<String>();

		String prefix = startPrefix;

		while( prefix.compareTo( endPrefix ) < 0 ){

			oneUnusedPrefixList.add( prefix );
			prefix = colonyInfoService.increment(  prefix  );

		}

		expect(colonyInfoDaoMock.fetchUsedPrefixes(startPrefix, endPrefix)).andReturn(oneUnusedPrefixList);

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix(startPrefix, endPrefix,null);

		verify();

		assertEquals(expectedValue, actualValue);
	}

	@Test
	public void getFirstUnusedPrefix_skips_I_place4() {

		String startPrefix   = "MAAH";
		String endPrefix     = "MZZZ";
		String expectedValue = "MAAJ";

		ArrayList<String> usedPrefixList = new ArrayList<String>();

		usedPrefixList.add(startPrefix);

		expect(colonyInfoDaoMock.fetchUsedPrefixes(startPrefix, endPrefix)).andReturn(usedPrefixList);

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix(startPrefix, endPrefix,prefixExclusionPredicate);

		verify();

		assertEquals(expectedValue, actualValue);
	}

	@Test
	public void getFirstUnusedPrefix_skips_I_place3() {

		String startPrefix   = "MAHZ";
		String endPrefix     = "MZZZ";
		String expectedValue = "MAJA";

		ArrayList<String> usedPrefixList = new ArrayList<String>();

		usedPrefixList.add( startPrefix );

		expect(colonyInfoDaoMock.fetchUsedPrefixes( startPrefix, endPrefix)).andReturn(usedPrefixList);

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix(startPrefix, endPrefix, prefixExclusionPredicate);

		verify();

		assertEquals(expectedValue, actualValue);
	}

	@Test
	public void getFirstUnusedPrefix_skips_O_place4() {

		String startPrefix   = "MAAN";
		String endPrefix     = "MZZZ";
		String expectedValue = "MAAP";

		ArrayList<String> usedPrefixList = new ArrayList<String>();

		usedPrefixList.add(startPrefix);

		expect(colonyInfoDaoMock.fetchUsedPrefixes(startPrefix, endPrefix)).andReturn(usedPrefixList);

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix(startPrefix, endPrefix,
				prefixExclusionPredicate);

		verify();

		assertEquals(expectedValue, actualValue);
	}

	@Test
	public void getFirstUnusedPrefix_skips_O_place3() {

		String startPrefix   = "MANZ";
		String endPrefix     = "MZZZ";
		String expectedValue = "MAPA";

		ArrayList<String> usedPrefixList = new ArrayList<String>();

		usedPrefixList.add( startPrefix );

		expect(colonyInfoDaoMock.fetchUsedPrefixes(startPrefix, endPrefix)).andReturn(usedPrefixList);

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix(startPrefix, endPrefix, prefixExclusionPredicate);

		verify();

		assertEquals(expectedValue, actualValue);
	}


	@Test
	public void getFirstUnusedPrefix_no_used_prefixes() {

		String expectedValue = "MAAA";

		expect(colonyInfoDaoMock.fetchUsedPrefixes("MAAA", "MZZZ")).andReturn(new ArrayList<String>());

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix("MAAA", "MZZZ", null);

		verify();

		assertEquals(expectedValue, actualValue);
	}

	@Test
	public void getFirstUnusedPrefix_one_used_prefix() {

		ArrayList<String> oneUsedPrefixList = new ArrayList<String>();

		oneUsedPrefixList.add("MAAA");

		String expectedValue = "MAAB";

		expect(colonyInfoDaoMock.fetchUsedPrefixes("MAAA", "MZZZ")).andReturn(oneUsedPrefixList);

		replay();

		String actualValue = colonyInfoService.getFirstUnusedPrefix("MAAA", "MZZZ", null);

		verify();

		assertEquals(expectedValue, actualValue);
	}

	 @Test
	 public void increment_less_than_max_1() {

		 String expectedValue = "BAAA";

		 String actualValue = colonyInfoService.increment("AZZZ");

		 assertEquals(expectedValue, actualValue);
	 }


	 @Test
	 public void increment_less_than_max_2() {

		 String expectedValue = "AACA";

		 String actualValue = colonyInfoService.increment("AABZ");

		 assertEquals(expectedValue, actualValue);
	 }

	 @Test
	 public void increment_less_than_max_3() {

		 String expectedValue = "AACZ";

		 String actualValue = colonyInfoService.increment("AACY");

		 assertEquals(expectedValue, actualValue);
	 }

	 @Test
	 public void increment_at_max() {

		 String expectedValue = null;

		 String actualValue = colonyInfoService.increment("ZZZZ");

		 assertEquals(expectedValue, actualValue);
	 }

	 @Test
	 public void increment_empty_str() {

		 String expectedValue = null;

		 String actualValue = colonyInfoService.increment("");

		 assertEquals(expectedValue, actualValue);
	 }

	 @Test(expected=IllegalArgumentException.class)
	 public void increment_non_letter_arg() {

		 colonyInfoService.increment( "AB7C" );
	 }

	 @Test(expected=IllegalArgumentException.class)
	 public void increment_null_arg() {

		 colonyInfoService.increment( null );
	 }



}
