/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.database;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.reflectionassert.ReflectionComparatorMode;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.personnel.Person;

/*
 *
 *
 * @SpringApplicationContext loads the specified xml files,
 * these should be the minimum dependencies to make the dao work
 *
 * the unitils-test-database.xml sets the migDataSource to
 * use the db configured in unitils. It should be the last
 * thing in the config list, to override any other datasource
 * definition.
 */
public abstract class TypedDaoTest<D extends TypedDao<O>,O extends AuthenticatedDataObject> extends UnitilsJUnit4 {

	protected final Long VALID_ID = -2L;
	protected final Long INVALID_ID = -99999999L;
	protected final Long DEFAULT_CHECK_NUMBER = 123L;
	protected final Long DEFAULT_CREATOR_ID = 234L;
	protected final String DEFAULT_EDITED_BY = "test_user";
	protected final String DEFAULT_CREATED_DATE = "2008-01-01";
	protected final String DEFAULT_EDIT_DATE = "2008-01-02";

	protected SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

	protected D dao;

	protected Person testUser;
	protected O testObject = null;
	protected O dbObject = null;

	/**
	 * Override this method to use the specific DAO to be tested.
	 * Use the SpringBean("") annotation
	 * @param dao DAO to be tested
	 */
	protected abstract void setDao(D dao);

	/**
	 * Instantiate the object and set the fields for 'testObject' to be compared with a database record.
	 * The following fields are set automatically:
	 * creatorId,
	 * createdDate,
	 * checknumber,
	 * editedBy,
	 * editDate,
	 * id
	 */
	protected abstract void setUpTestObject() throws ParseException;

	@Before
	public final void before() throws ParseException {
		testUser = new Person();
		testUser.setId(1L);
		testUser.setUserName("test_user");

		setUpTestObject();

		if(testObject != null) {
			testObject.setId(VALID_ID);
			testObject.setCheckNumber(DEFAULT_CHECK_NUMBER);
			testObject.setCreatorId(DEFAULT_CREATOR_ID);
			testObject.setCreatedDate(sdf.parse(DEFAULT_CREATED_DATE));
			testObject.setEditDate(sdf.parse(DEFAULT_EDIT_DATE));
			testObject.setEditedBy(DEFAULT_EDITED_BY);
		}
	}

	@Test
	public void fetchByIdWithValidId() {
		dbObject = dao.fetchById(VALID_ID);
		assertRefEquals(testObject, dbObject, ReflectionComparatorMode.LENIENT_DATES);
	}

	@Test
	public void fetchByIdWithInvalidId() {
		dbObject = dao.fetchById(INVALID_ID);
		assertNull(dbObject);
	}

	@Test
	public void fetchByIdNull() {
		dbObject = dao.fetchById(null);
		assertNull(dbObject);
	}

	@Test
	public void storeInsert() {
		testObject.setId(null);
		Long id = dao.store(testObject, testUser);
		assertNotNull(id);

		dbObject = dao.fetchById(id);
		assertRefEquals(testObject, dbObject);
	}

	@Test
	public void storeUpdate() {
		Long insertPk = dao.store(testObject, testUser);
		assertNotNull(insertPk);

		dbObject = dao.fetchById(insertPk);
		assertRefEquals(testObject, dbObject);
	}

}
