package uk.ac.sanger.mig.europhenome.services;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.*;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Vector;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.europhenome.dao.CohortExportDao;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.services.CellLineService;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.GenotypeDictionary;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.personnel.Person;

public class CohortExportServiceImplMockTest extends UnitilsJUnit4 {
	@TestedObject private CohortExportServiceImpl cohortExportService;

	@Mock @InjectIntoByType private CellLineService mockCellLineService;
	@Mock @InjectIntoByType private CohortExportDao mockCohortExportDao;
	@Mock @InjectIntoByType private ColonyInfoService mockColonyInfoService;
	@Mock @InjectIntoByType private MouseExportedService mockMouseExportedService;
	@Mock @InjectIntoByType private MouseService mockMouseService;

	private Cohort cohort;
	private Cohort filteredCohort;
	private Collection<Cohort> cohortWrapper;
	private Collection<Mouse> groupA; // wildtype
	private Collection<Mouse> groupB; // hets
	private Collection<Mouse> groupC; // homs
	private Collection<Mouse> groupD; // hets
	private Collection<Mouse> groupE; // failed - should not be present
	private Collection<Mouse> miceThatShouldBeExported;
	private Person person;
	private DictionaryEntry strainAB;
	private DictionaryEntry strainCDE;
	private Allele allele;
	private ColonyInfo colony;
	private CellLine cellLine;
	private Collection<CellLine> cellLineWrapper;
	private int mouseIdCount;

	@Before
	public void setUp(){
		mouseIdCount = 101;
		int micePerGroup = 3;

		colony = new ColonyInfo();
		colony.setId(5L);
		colony.setColonyPrefix("TEST");

		allele = new Allele();
		allele.setId(3L);
		allele.setOfficialNameSuperscript("tm1awtsi");
		allele.setMigGene(new MigGene());
		allele.getMigGene().setId(4L);
		allele.getMigGene().setPrimaryName("Braf");


		strainAB = new DictionaryEntry();
		strainAB.setId(10L);
		strainAB.setName("Strain A & B (prefix meh)");
		strainAB.setDescription("Strain A & B");
		strainCDE = new DictionaryEntry();
		strainCDE.setId(11L);
		strainCDE.setName("Strain C & D (prefix mehhhh)");
		strainCDE.setDescription("Strain C & D");

		groupA = new ArrayList<Mouse>(micePerGroup);
		groupB = new ArrayList<Mouse>(micePerGroup);
		groupC = new ArrayList<Mouse>(micePerGroup);
		groupD = new ArrayList<Mouse>(micePerGroup);
		groupE = new ArrayList<Mouse>(micePerGroup);

		for (int i = 0; i < micePerGroup; i++ ) {
			groupA.add(makeMouse(GenotypeDictionary.WILDTYPE, strainAB));
			groupB.add(makeMouse(GenotypeDictionary.HETEROZYGOUS, strainAB));
			groupC.add(makeMouse(GenotypeDictionary.HOMOZYGOUS, strainCDE));
			groupD.add(makeMouse(GenotypeDictionary.HETEROZYGOUS, strainCDE));
			groupE.add(makeMouse(GenotypeDictionary.FAILED, strainCDE));
		}

		person = new Person();
		person.setId(1L);

		cohort = new Cohort();
		cohort.setId(2L);
		cohort.setCohortName("Test cohort 1");
		cohort.setCohortPipelineDictId(2L);
		cohort.setMice(new ArrayList<Mouse>());
		cohort.getMice().addAll(groupA);
		cohort.getMice().addAll(groupB);
		cohort.getMice().addAll(groupC);
		cohort.getMice().addAll(groupD);
		cohort.getMice().addAll(groupE);

		cohortWrapper = new ArrayList<Cohort>();
		cohortWrapper.add(cohort);

		miceThatShouldBeExported = new ArrayList<Mouse>();
		miceThatShouldBeExported.addAll(groupA);
		miceThatShouldBeExported.addAll(groupB);
		miceThatShouldBeExported.addAll(groupC);
		miceThatShouldBeExported.addAll(groupD);

		filteredCohort = new Cohort();
		filteredCohort.setId(cohort.getId());
		filteredCohort.setCohortName(cohort.getCohortName());
		filteredCohort.setMice(miceThatShouldBeExported);
		filteredCohort.setCohortPipelineDictId(cohort.getCohortPipelineDictId());

		cellLine = new CellLine();
		cellLine.setId(4L);
		cellLine.setCloneName("EPD_001_001");
		cellLine.setCellLineSource(new DictionaryEntry());
		cellLine.getCellLineSource().setName("Team 87");
		cellLine.setModifiedAllele(allele);
		cellLine.setModifiedAlleleId(allele.getId());
		cellLine.setFundingDictId(1L);
		cellLine.setFundingDict(new DictionaryEntry());
		cellLine.getFundingDict().setName("EUCOMM");

		cellLineWrapper = new ArrayList<CellLine>(1);
		cellLineWrapper.add(cellLine);
	}

	private Mouse makeMouse(Long genotypeId, DictionaryEntry strain) {
		Mouse m = new Mouse();
		int id = mouseIdCount++;

		m.setId(Long.valueOf(id));
		m.setBarcode("M"+id);
		m.setName(colony.getColonyPrefix()+id);
		m.setColonyInfo(colony);
		m.setBirthDate(new Date());
		m.setStrain(strain);
		m.setColonyInfoId(colony.getId());

		if (id % 2 == 0) {
			m.setGenderDictId(MouseService.MOUSE_GENDER_MALE);
		}
		else {
			m.setGenderDictId(MouseService.MOUSE_GENDER_FEMALE);
		}

		MouseAllele ma = new MouseAllele();
		ma.setAllele(allele);
		ma.setGenotypeDictId(genotypeId);
		m.setAlleles(new ArrayList<MouseAllele>());
		m.getAlleles().add(ma);

		return m;
	}

	@SuppressWarnings("unchecked")
	@Test
	public void testCohortExport() {
		//set up
		expect(mockCohortExportDao.fetchCohortsToExport()).andReturn(cohortWrapper);
		expect(mockMouseService.fetchByCohortId(cohort.getId(), null)).andReturn(cohort.getMice());
		expect(mockColonyInfoService.fetchById(colony.getId(), null)).andReturn(colony);
		expect(mockCellLineService.fetchCellLinesForColony(colony.getId(), null)).andReturn(cellLineWrapper).atLeastOnce();
		mockMouseExportedService.recordCohortExport(filteredCohort, person);

		//run
		replay();
		Vector<org.europhenome.Cohort> exportCohorts = cohortExportService.exportCohorts(person);
		//check
		verify();
		assertEquals(4, exportCohorts.size());

		boolean gotA = false;
		boolean gotB = false;
		boolean gotC = false;
		boolean gotD = false;

		for (org.europhenome.Cohort euCohort : exportCohorts) {
			assertTrue(euCohort.getCohortId().startsWith(cohort.getId()+"_"));
			assertEquals(cohort.getCohortPipelineDictId().intValue(), euCohort.getPipelineId());
			Collection<Mouse> expectedMice = null;

			if (euCohort.getCloneName() != null
					&& euCohort.getCloneName().equals("background")
					&& euCohort.getStrain().equals(strainAB.getDescription())) {
				gotA = true;
				expectedMice = groupA;

				checkWtDetails(euCohort);
			}
			else if (euCohort.getZygosity() == org.europhenome.Cohort.HETROZYGOUS
					&& euCohort.getStrain().equals(strainAB.getDescription())) {
				gotB = true;
				expectedMice = groupB;
				checkAlleleAndCellLine(euCohort);
			}
			else if (euCohort.getZygosity() == org.europhenome.Cohort.HOMOZYGOUS
					&& euCohort.getStrain().equals(strainCDE.getDescription())) {
				gotC = true;
				expectedMice = groupC;
				checkAlleleAndCellLine(euCohort);
			}
			else if (euCohort.getZygosity() == org.europhenome.Cohort.HETROZYGOUS
					&& euCohort.getStrain().equals(strainCDE.getDescription())) {
				gotD = true;
				expectedMice = groupD;
				checkAlleleAndCellLine(euCohort);
			}

			assertEquals(false, (expectedMice == null));

			Collection<org.europhenome.Mouse> euMice = (Vector<org.europhenome.Mouse>)euCohort.getMice();
			for (org.europhenome.Mouse euMouse : euMice){
				assertTrue(isEuMousePresent(euMouse, expectedMice));
			}
			for (Mouse m : expectedMice) {
				assertTrue(isMousePresent(m, euMice));
			}
			for (Mouse m : groupE) {
				assertFalse(isMousePresent(m, euMice));
			}
		}

		assertTrue(gotA);
		assertTrue(gotB);
		assertTrue(gotC);
		assertTrue(gotD);
	}

	private void checkAlleleAndCellLine(org.europhenome.Cohort euCohort) {
		assertEquals(allele.getOfficialNameSuperscript(), euCohort.getAlleleName());
		assertEquals(allele.getMigGene().getPrimaryName(), euCohort.getGeneName());
		assertEquals(cellLine.getCloneName(), euCohort.getCloneName());
		assertEquals("EUCOMM", euCohort.getSource());
	}
	private void checkWtDetails(org.europhenome.Cohort euCohort) {
		assertNull(euCohort.getAlleleName());
		assertNull(euCohort.getGeneName());
		assertTrue(euCohort.getSource() == null || euCohort.getSource().equals(""));
		assertEquals("background", euCohort.getCloneName());
	}

	private boolean isEuMousePresent(org.europhenome.Mouse em, Collection<Mouse> mouseGroup){
		boolean isPresent = false;

		for (Mouse m : mouseGroup) {
			if (m.getBarcode().equals(em.getLocalName())){
				isPresent = true;
				break;
			}
		}
		return isPresent;
	}

	private boolean isMousePresent(Mouse m, Collection<org.europhenome.Mouse> euMice) {
		boolean isPresent = false;

		for (org.europhenome.Mouse em : euMice) {
			if (em.getLocalName().equals(m.getBarcode())){
				isPresent = true;
				break;
			}
		}
		return isPresent;
	}
}
