/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.europhenome.services;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.apache.commons.lang.NotImplementedException;
import org.europhenome.Increment;
import org.europhenome.Parameter;
import org.europhenome.Procedure;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.europhenome.OptionTranslation;
import uk.ac.sanger.mig.europhenome.ParameterDefinition;
import uk.ac.sanger.mig.europhenome.ParameterMapping;
import uk.ac.sanger.mig.europhenome.ProcedureDefinition;
import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.ObservationType;
import uk.ac.sanger.mig.phenotyping.datacapture.RequiredObservation;
import uk.ac.sanger.mig.phenotyping.datacapture.RestrictedOption;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.UploadMapping;
import uk.ac.sanger.mig.phenotyping.datacapture.services.UploadMappingService;

public class ExperimentExportServiceImplTest extends UnitilsJUnit4 {

  private static final String empressID = "EUR1";
  private static final String empressParamOne = "EUR1_1";
  private static final String empressParamTwo = "EUR1_2";
  private static final String empressParamThree = "EUR1_3";
  private static final Date startDate = new Date();
  private static final String mouseBarcode = "M0000TEST";
  private static final String grumpyText = "Bah, humbug" ;
  private static final String translationPrefix = "EU version: ";
  private static final String firstOptionText = "An option";
  private static final String secondOptionText = "Another option";
  private static final Long firstReqObvId = 1L;
  private static final Long secondReqObvId = 2L;
  private static final Long thirdReqObvId = 3L;
  private static final Long fourthReqObvId = 4L;
  private static final String firstSeriesText = "I am the first value";
  private static final String secondSeriesText = "I am the second value";
  private static final Long sopId = 101L;


  @TestedObject private ExperimentExportServiceImpl service;
  @Mock @InjectIntoByType private UploadMappingService mockUploadMappingService;


  @Test
  @SuppressWarnings("unchecked")
  public void testExperimentConversion(){
  	expect(mockUploadMappingService.fetchBySopId(sopId)).andReturn(new ArrayList<UploadMapping>());
  	replay();

    Experiment e = getTestExperiment();
    ProcedureDefinition def = getTestProcedureDefinition();

    Procedure procedure = service.convertExperimentToProcedure(e, def);

    verify();

    // assertions about the basic records - what is it, who did it, what did they do it to & when
    assertNotNull("Where's my procedure?", procedure);
    assertEquals("Empress procedure ID",empressID, procedure.getEMPReSSlimID());
    assertEquals("Experiment start point", startDate, procedure.getStartTime());
    assertNotNull("Where are my mice?", procedure.getMice());
    assertEquals("I only have one mouse", 1, procedure.getMice().size());
    assertEquals("I want this mouse", mouseBarcode, procedure.getMice().iterator().next());

    // assertions about the data recorded
    List<Parameter> parameters = (Vector<Parameter>)procedure.getParameters();
    assertNotNull("Where's my data?", parameters);
    assertEquals("I want this much data",3, parameters.size());

    // text param
    Parameter textParameter = parameters.get(0);
    assertEquals("I want that parameter", empressParamOne, textParameter.getParameterId());
    assertEquals("I want my grumpy text", grumpyText, textParameter.getValue());

    // option parameter
    Parameter optionParameter = parameters.get(1);
    assertEquals("I want that parameter", empressParamTwo, optionParameter.getParameterId());
    assertEquals("Translated option value",translationPrefix+firstOptionText, optionParameter.getValue());

    // text increment parameter
    Parameter textIncrementParameter = parameters.get(2);
    assertNotNull("Expecting increments",textIncrementParameter.getIncrements());
    assertEquals("I want two increments", 2, textIncrementParameter.getIncrements().size());
    Increment firstIncrement = (Increment)textIncrementParameter.getIncrements().get(0);
    assertEquals("I should see this",firstSeriesText,firstIncrement.getValue());
    Increment secondIncrement = (Increment)textIncrementParameter.getIncrements().get(1);
    assertEquals("I should see that", secondSeriesText, secondIncrement.getValue());
  }

  private ProcedureDefinition getTestProcedureDefinition() {
    ProcedureDefinition procDef = new ProcedureDefinition();

    procDef.setEmpressId(empressID);
    procDef.setName("Test def 1");
    procDef.setParameterDefinitions(new LinkedList<ParameterDefinition>());

    List<RequiredObservation> reqs = getRequiredObservations();


    // basic text definiton
    ParameterDefinition textParamDef = new ParameterDefinition();
    procDef.getParameterDefinitions().add(textParamDef);
    textParamDef.setEmpressId(empressParamOne);
    textParamDef.setId(1L);
    textParamDef.setDataSeries(false);
    textParamDef.setParameterTypeId(ParameterDefinition.ParameterType.SIMPLE_PARAMETER.getId());

    textParamDef.setParameterMappings(new LinkedList<ParameterMapping>());

    ParameterMapping textParamMapping = new ParameterMapping();
    textParamDef.getParameterMappings().add(textParamMapping);
    textParamMapping.setParameterDefinition(textParamDef);
    textParamMapping.setRequiredObservationId(reqs.get(0).getId());
    textParamMapping.setParameterDefinitionId(1L);
    textParamMapping.setRequiredObservation(reqs.get(0));

    // option/restricted vocabulary definition
    ParameterDefinition optParamDef = new ParameterDefinition();
    procDef.getParameterDefinitions().add(optParamDef);
    optParamDef.setEmpressId(empressParamTwo);
    optParamDef.setId(2L);
    optParamDef.setDataSeries(false);
    optParamDef.setParameterTypeId(ParameterDefinition.ParameterType.SIMPLE_PARAMETER.getId());

    optParamDef.setParameterMappings(new LinkedList<ParameterMapping>());

    ParameterMapping optParamMapping = new ParameterMapping();
    optParamDef.getParameterMappings().add(optParamMapping);
    optParamMapping.setParameterDefinition(optParamDef);
    optParamMapping.setRequiredObservationId(reqs.get(1).getId());
    optParamMapping.setParameterDefinitionId(2L);
    optParamMapping.setRequiredObservation(reqs.get(1));

    Collection<OptionTranslation> translations = new LinkedList<OptionTranslation>();
    optParamMapping.setOptionTranslations(translations);

    for (RestrictedOption option : getOptions()) {
      OptionTranslation translation = new OptionTranslation();
      translation.setOption(option);
      translation.setOptionId(option.getId());
      translation.setEuValue(translationPrefix+option.getName());

      translations.add(translation);
    }

    // text data series definition
    ParameterDefinition incrementParamDef = new ParameterDefinition();
    procDef.getParameterDefinitions().add(incrementParamDef);
    incrementParamDef.setEmpressId(empressParamThree);
    incrementParamDef.setId(3L);
    incrementParamDef.setDataSeries(true);
    incrementParamDef.setParameterTypeId(ParameterDefinition.ParameterType.SERIES_PARAMETER.getId());

    incrementParamDef.setParameterMappings(new LinkedList<ParameterMapping>());

    ParameterMapping firstIncrementMapping = new ParameterMapping();
    incrementParamDef.getParameterMappings().add(firstIncrementMapping);

    firstIncrementMapping.setRequiredObservationId(reqs.get(2).getId());
    firstIncrementMapping.setParameterDefinition(incrementParamDef);
    firstIncrementMapping.setParameterDefinitionId(3L);
    firstIncrementMapping.setRequiredObservation(reqs.get(2));
    firstIncrementMapping.setSeriesMapping("0");

    ParameterMapping secondIncrementMapping = new ParameterMapping();
    incrementParamDef.getParameterMappings().add(secondIncrementMapping);

    secondIncrementMapping.setRequiredObservationId(reqs.get(3).getId());
    secondIncrementMapping.setParameterDefinition(incrementParamDef);
    secondIncrementMapping.setParameterDefinitionId(3L);
    secondIncrementMapping.setRequiredObservation(reqs.get(3));
    secondIncrementMapping.setSeriesMapping("1");

    return procDef;
  }



  private Experiment getTestExperiment() {
    Experiment e = new Experiment();

    e.setId(1L);
    e.setMouse(getTestMouse());
    e.setProcedure(getMouseProcedure());
    e.setComplete(true);
    e.setSop(getSop());
    e.setSopId(sopId);

    e.setObservations(getObservations(e.getSop().getRequiredObservations()));

    return e;
  }

  private Collection<Observation> getObservations(Collection<RequiredObservation> requiredObservations) {
    Collection<Observation> observations = new LinkedList<Observation>();

    for (RequiredObservation ro : requiredObservations) {
      Observation o = new Observation();
      observations.add(o);
      o.setRequiredObservationId(ro.getId());

      if (ro.getId().equals(firstReqObvId)) {
        o.setValue(grumpyText);
      }
      else if (ro.getId().equals(secondReqObvId)) {
        RestrictedOption opt = ro.getObservationType().getRestrictedOptions().iterator().next();
        o.setValue(opt.getId().toString());
      }
      else if (ro.getId().equals(thirdReqObvId)) {
        o.setValue(firstSeriesText);
      }
      else if (ro.getId().equals(fourthReqObvId)) {
        o.setValue(secondSeriesText);
      }
      else {
        throw new NotImplementedException("No data for type "+ro.getObservationType().getObservationMetaTypeId());
      }
    }

    return observations;
  }



  private MouseProcedure getMouseProcedure() {
    MouseProcedure mp = new MouseProcedure();
    mp.setStartDate(startDate);

    PersonalLicence pl = new PersonalLicence();
    pl.setPersonId(1L);
    mp.setPersonalLicence(pl);

    return mp;
  }

  private StdOperatingProcedure getSop() {
    StdOperatingProcedure sop = new StdOperatingProcedure();
    sop.setId(sopId);
    sop.setName("Test SOP");
    sop.setRequiredObservations(getRequiredObservations());

    return sop;
  }

  private List<RequiredObservation> getRequiredObservations() {
    List<RequiredObservation> reqs = new LinkedList<RequiredObservation>();

    // add a requirement for some text
    ObservationType textType = new ObservationType();
    textType.setObservationMetaTypeId(ObservationType.MetaType.FREE_TEXT.getId());
    textType.setName("Text Test type");

    RequiredObservation textRequired = new RequiredObservation();
    textRequired.setObservationType(textType);
    textRequired.setDisplayName("Test comment");
    textRequired.setId(firstReqObvId);

    reqs.add(textRequired);

    // add a requirement for an option
    ObservationType optionType = new ObservationType();
    optionType.setObservationMetaTypeId(ObservationType.MetaType.RADIO_BUTTON.getId());
    optionType.setName("Option Test Type");

    optionType.setRestrictedOptions(getOptions());

    RequiredObservation optionRequired = new RequiredObservation();
    optionRequired.setObservationType(optionType);
    optionRequired.setDisplayName("Test select");
    optionRequired.setId(secondReqObvId);

    reqs.add(optionRequired);

    RequiredObservation incrementOneRequired = new RequiredObservation();
    incrementOneRequired.setObservationType(textType);
    incrementOneRequired.setDisplayName("Increment Text One");
    incrementOneRequired.setId(thirdReqObvId);

    reqs.add(incrementOneRequired);

    RequiredObservation incrementTwoRequired = new RequiredObservation();
    incrementTwoRequired.setObservationType(textType);
    incrementTwoRequired.setDisplayName("Increment Text Two");
    incrementTwoRequired.setId(fourthReqObvId);

    reqs.add(incrementTwoRequired);

    return reqs;
  }

  private Collection<RestrictedOption> getOptions() {
    Collection<RestrictedOption> options = new LinkedList<RestrictedOption>();
    RestrictedOption firstOpt = new RestrictedOption();
    firstOpt.setId(10L);
    firstOpt.setName(firstOptionText);
    options.add(firstOpt);

    RestrictedOption secondOpt = new RestrictedOption();
    secondOpt.setId(11L);
    secondOpt.setName(secondOptionText);
    options.add(secondOpt);

    return options;
  }


  private Mouse getTestMouse() {
    Mouse m = new Mouse();
    m.setBarcode(mouseBarcode);
    return m;
  }

}
