package uk.ac.sanger.mig.genotyping;

import static junit.framework.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.reflectionassert.ReflectionAssert;
import org.unitils.reflectionassert.ReflectionComparatorMode;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.genotyping.AssayPlate;
import uk.ac.sanger.mig.genotyping.AssayPlateWell;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.MouseAllele;

public class AssayPlateTest extends UnitilsJUnit4 {

	private AssayPlate assayPlate;
	private final int maxRows = 8;
	private final int maxCols = 12;
	private final Long TPM1_ID = 1L;
	private final Long BRD7_ID = 2L;
	private final Long RARS_ID = 3L;
	private Allele tpm1;
	private Allele brd7;
	private Allele rars;

	/**
	 * set up a 8 row x 12 column plate with tpm1 allele in column 11 of all rows
	 * and brd7 in columns 0-10 rows 0,2,4,6 and rars in rows 1,3,5,7
	 *
	 */
	@Before
	public void setup() {
		assayPlate = new AssayPlate(8,12);

		tpm1 = new Allele();
		tpm1.setId(TPM1_ID);
		tpm1.setShortAlleleSymbol("tpm1");

		brd7 = new Allele();
		brd7.setId(BRD7_ID);
		brd7.setShortAlleleSymbol("brd7");

		rars = new Allele();
		rars.setId(RARS_ID);
		rars.setShortAlleleSymbol("rars");

		for(int r=0; r < maxRows; r++) {
			for(int c=0; c < maxCols; c++) {
				AssayPlateWell well = new AssayPlateWell();
				well.setPlateRow(r+1);
				well.setPlateColumn(c+1);

				Mouse mouse = new Mouse();
				mouse.setId(new Long((r*12)+c));

				Collection<MouseAllele> mouseAlleles = new ArrayList<MouseAllele>();
				MouseAllele mouseAllele = new MouseAllele();
				// put tpm1 in last column
				if(c == 11) {
					mouseAllele.setAlleleId(tpm1.getId());
					mouseAllele.setAllele(tpm1);
				}
				// put brd7 in even rows
				else if(r%2==0) {
					mouseAllele.setAlleleId(brd7.getId());
					mouseAllele.setAllele(brd7);
				}
				// put rars in odd rows
				else {
					mouseAllele.setAlleleId(rars.getId());
					mouseAllele.setAllele(rars);
				}
				mouseAlleles.add(mouseAllele);
				mouse.setAlleles(mouseAlleles);

				well.setMouse(mouse);

				assayPlate.getWellArray()[r][c] = well;
			}
		}
	}

	@Test
	public void getWellsByAlleleIdWithValidId() {
		Collection<AssayPlateWell> wells = assayPlate.getWellsByAlleleId(TPM1_ID);
		// there should be 8 wells with TPM1 allele
		assertTrue(wells.size() == 8);
	}

	@Test
	public void getWellsByAlleleWithInvalidId() {
		Collection<AssayPlateWell> wells = assayPlate.getWellsByAlleleId(0L);
		// there should be an empty collection
		assertTrue(wells.isEmpty());
	}

	/**
	 * This should return a set of 2 alleles brd7 and tpm1
	 *
	 */
	@Test
	public void getAllelesByRow0() {
		Set<Allele> plateAlleles = assayPlate.getAllelesInRow(0);
		Set<Allele> expectedAlleles = new HashSet<Allele>();
		expectedAlleles.add(brd7);
		expectedAlleles.add(tpm1);

		ReflectionAssert.assertRefEquals(expectedAlleles, plateAlleles, ReflectionComparatorMode.LENIENT_ORDER);
	}

	/**
	 * This should return a set of 2 alleles rars and tpm1
	 *
	 */
	@Test
	public void getAllelesByRow1() {
		Set<Allele> plateAlleles = assayPlate.getAllelesInRow(1);
		Set<Allele> expectedAlleles = new HashSet<Allele>();
		expectedAlleles.add(rars);
		expectedAlleles.add(tpm1);

		ReflectionAssert.assertRefEquals(expectedAlleles, plateAlleles, ReflectionComparatorMode.LENIENT_ORDER);
	}

	/**
	 * This should return a set of 2 alleles rars, brd7 and tpm1
	 *
	 */
	@Test
	public void getAllelesInPlate() {
		Set<Allele> plateAlleles = assayPlate.getAllelesInPlate();
		Set<Allele> expectedAlleles = new HashSet<Allele>();
		expectedAlleles.add(rars);
		expectedAlleles.add(tpm1);
		expectedAlleles.add(brd7);

		ReflectionAssert.assertRefEquals(expectedAlleles, plateAlleles, ReflectionComparatorMode.LENIENT_ORDER);
	}

	@Test
	public void populateWell() {
		assayPlate = new AssayPlate(8,12);
		AssayPlateWell well = new AssayPlateWell();
		assayPlate.populateWell(well, 0, 0);
		junit.framework.Assert.assertTrue(well.equals(assayPlate.getWell(0, 0)));
	}
}
