/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.*;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.histology.Crop;


public class CropMockTest extends UnitilsJUnit4 {

	@TestedObject
	private Crop crop;
	private final int WIDTH = 25000, HEIGHT = 60000;

	@Before
	public void setUp() {
		crop = new Crop();
		crop.setX(30);
		crop.setY(50);
		crop.setCropWidth(WIDTH);
		crop.setCropHeight(HEIGHT);
		crop.setTileEdgeExponent( 8 );
		crop.setZoomRadix( 2 );
	}

	@Test
	public void getMaxZoomDepth_25000x60000() {
		int expectedResult = 8;//where 8 = ciel( log2( 60000 / 256 ) )
		int actualResult = crop.getMaxZoomDepth();
		assertEquals(expectedResult, actualResult);
	}

	@Test(expected=IllegalStateException.class)
	public void getMaxZoomDepth_invalid_height() {
		Crop _crop = (Crop)crop.clone();
		_crop.setCropHeight( 0 );
		_crop.getMaxZoomDepth();
	}

	@Test(expected=IllegalStateException.class)
	public void getMaxZoomDepth_invalid_width() {
		Crop _crop = (Crop)crop.clone();
		_crop.setCropWidth( 0 );
		_crop.getMaxZoomDepth();
	}

	@Test(expected=IllegalStateException.class)
	public void getMaxZoomDepth_invalid_tile_edge_exponent() {
		Crop _crop = (Crop)crop.clone();
		_crop.setTileEdgeExponent( 0 );
		_crop.getMaxZoomDepth();
	}

	@Test(expected=IllegalStateException.class)
	public void getMaxZoomDepth_invalid_zoom_radix() {
		Crop _crop = (Crop)crop.clone();
		_crop.setZoomRadix( 0 );
		_crop.getMaxZoomDepth();
	}

	@Test
	public void getTileEdge() {
		int expectedTileEdge = 256;
		assertEquals( crop.getTileEdge(), expectedTileEdge );
	}

	@Test(expected=IllegalStateException.class)
	public void getTileEdge_bad_exponent() {

		Crop badCrop = new Crop();
		badCrop.setTileEdgeExponent( 0 );
		badCrop.setZoomRadix( 2 );
		badCrop.getTileEdge();
	}

	@Test(expected=IllegalStateException.class)
	public void getTileEdge_bad_radix() {

		Crop badCrop = new Crop();
		badCrop.setTileEdgeExponent( 8 );
		badCrop.setZoomRadix( -1 );
		badCrop.getTileEdge();
	}

	@Test
	public void equals_true() {
		Crop testCrop = new Crop();
		testCrop.setX(30);
		testCrop.setY(50);
		testCrop.setCropWidth(WIDTH);
		testCrop.setCropHeight(HEIGHT);
		assertTrue( crop.equals( testCrop ) );
	}

	@Test
	public void equals_false_different_X() {
		Crop testCrop = new Crop();
		testCrop.setX( -1 );
		testCrop.setY(50);
		testCrop.setCropWidth(WIDTH);
		testCrop.setCropHeight(HEIGHT);
		assertFalse( crop.equals( testCrop ) );
	}

	@Test
	public void equals_false_different_Y() {
		Crop testCrop = new Crop();
		testCrop.setX( 30 );
		testCrop.setY( -1 );
		testCrop.setCropWidth(WIDTH);
		testCrop.setCropHeight(HEIGHT);
		assertFalse( crop.equals( testCrop ) );
	}

	@Test
	public void equals_false_different_Width() {
		Crop testCrop = new Crop();
		testCrop.setX( 30 );
		testCrop.setY( 50 );
		testCrop.setCropWidth( 123 );
		testCrop.setCropHeight(HEIGHT);
		assertFalse( crop.equals( testCrop ) );
	}

	@Test
	public void equals_false_different_Height() {
		Crop testCrop = new Crop();
		testCrop.setX( 30 );
		testCrop.setY( 50 );
		testCrop.setCropWidth(WIDTH);
		testCrop.setCropHeight( 123 );
		assertFalse( crop.equals( testCrop ) );
	}

	@Test
	public void equals_false_null_arg() {

		assertFalse( crop.equals( null ) );
	}

	@Test
	public void equals_false_not_instanceof() {

		Object testCrop = new Object();;
		assertFalse( crop.equals( testCrop ) );
	}

}
