/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.services;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.assertEquals;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;
import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropJob;
import uk.ac.sanger.mig.histology.Scan;
import uk.ac.sanger.mig.histology.dao.CropDao;
import uk.ac.sanger.mig.histology.services.CropService.QcFailAction;
import uk.ac.sanger.mig.personnel.Person;

public class CropServiceMockTest extends UnitilsJUnit4 {

	@TestedObject
	private CropServiceImpl cropServiceImpl;

	@Mock @InjectIntoByType private SlideService slideServiceMock;
	@Mock @InjectIntoByType private ScanService scanServiceMock;
	@Mock @InjectIntoByType private CropDao cropDaoMock;

	private Crop crop;
	private Person user;
	@Before
	public void setUp() {
		//set up default Crop
		crop = new Crop();
		user = new Person();

		//build hierarchy up to scan
		CropJob cropJob = new CropJob();
		cropJob.setScanId( 10L );
		Scan scan = new Scan();
		scan.setSlideId( 100L );
		cropJob.setScan( scan );
		crop.setCropJob(cropJob);
	}

	@Test
	public void qcPass() {

		Crop actualCrop = new Crop();

		Crop expectedCrop = new Crop();
		expectedCrop.setQcPass( Boolean.TRUE );
		expectedCrop.setQcFailReason( null );
		expectedCrop.setFlaggedForDelete( false );

		expect( cropDaoMock.store(actualCrop, user) ).andReturn( new Long(1L) ).once();

		replay();
		cropServiceImpl.qcPass(actualCrop, user);
		verify();

		assertRefEquals(expectedCrop, actualCrop);

	}

	@Test(expected=IllegalArgumentException.class)
	public void qcPass_null_Crop_arg() {

		cropServiceImpl.qcPass(null , user);
	}

	@Test
	public void qcLock() {


		Crop actualCrop = new Crop();

		Crop expectedCrop = new Crop();
		expectedCrop.setQcLocked( true );

		expect( cropDaoMock.store(actualCrop, user) ).andReturn( new Long(1L) ).once();

		replay();
		cropServiceImpl.qcLock(actualCrop, user);
		verify();

		assertRefEquals(expectedCrop, actualCrop);

	}

	@Test(expected=IllegalArgumentException.class)
	public void qcLock_null_Crop_arg() {

		cropServiceImpl.qcLock(null , user);
	}

	@Test
	public void qcFail() {

		String qcFailureReason = "Bad Scan/Slide";

		Crop actualCrop = new Crop();

		Crop expectedCrop = new Crop();
		expectedCrop.setExternalApproved( false );
		expectedCrop.setQcFailReason( qcFailureReason );
		expectedCrop.setQcPass( Boolean.FALSE );
		expectedCrop.setFlaggedForTiling( false );

		expect( cropDaoMock.store(actualCrop, user) ).andReturn( new Long(1L) ).once();

		replay();
		cropServiceImpl.qcFail(actualCrop, qcFailureReason , user);
		verify();

		assertRefEquals(expectedCrop, actualCrop);

	}

	@Test(expected=IllegalArgumentException.class)
	public void qcFail_null_Crop_arg() {

		cropServiceImpl.qcFail(null, "" , user);
	}

	@Test
	public void initiateQcFailAction_TRY_FAILING_AND_REORDER_SLIDE() {

		crop.setQcLocked( Boolean.FALSE );
		crop.setQcPass(  Boolean.FALSE );

		QcFailAction qcFailAction = QcFailAction.TRY_FAILING_AND_REORDER_SLIDE;

		//we're simply testing that this method gets called called 1x
		expect( slideServiceMock.gentlyFailAndReorder( null, null ) ).andReturn( true ).once();
		replay();
		cropServiceImpl.initiateQcFailAction(crop, qcFailAction, null);
		verify();
	}

	@Test
	public void initiateQcFailAction_TRY_FAILING_AND_REORDER_SCAN() {

		crop.setQcLocked( Boolean.FALSE );
		crop.setQcPass(  Boolean.FALSE );

		QcFailAction qcFailAction = QcFailAction.TRY_FAILING_AND_REORDER_SCAN;

		//we're simply testing that this method gets called called 1x
		expect( scanServiceMock.gentlyFailAndReorder( null, null)  ).andReturn( true ).once();
		replay();
		cropServiceImpl.initiateQcFailAction( crop , qcFailAction, null);
		verify();
	}

	@Test
	public void initiateQcFailAction_RESELECT_REGIONS() {

		crop.setQcLocked( Boolean.FALSE );
		crop.setQcPass(  Boolean.FALSE );

		QcFailAction qcFailAction = QcFailAction.RESELECT_REGIONS;

		//we're simply testing that this method gets called called 1x
		expect( scanServiceMock.addToCropSelectionBin(null, null, null)  ).andReturn( null ).once();
		replay();
		cropServiceImpl.initiateQcFailAction(crop, qcFailAction, null);
		verify();
	}

	@Test
	public void initiateQcFailAction_MARK_REDUNDANT() {

		crop.setQcLocked( Boolean.FALSE );
		crop.setQcPass(  Boolean.FALSE );

		String startComments = "start comments";

		String appendString = "***"+QcFailAction.MARK_REDUNDANT.getLabel();
		String expectedComments = startComments + appendString;

		crop.setComments( startComments );
		QcFailAction qcFailAction = QcFailAction.MARK_REDUNDANT;

		//test this method gets called called 1x, delegated by markRedundant()
		expect(cropDaoMock.store(crop, null)  ).andReturn( null ).once();
		replay();
		cropServiceImpl.initiateQcFailAction(crop, qcFailAction, null);
		verify();
		String actualComments = crop.getComments();
		assertEquals(expectedComments, actualComments);
	}


	@Test(expected=IllegalArgumentException.class)
	public void initiateQcFailAction_null_CropId_arg() {

		cropServiceImpl.initiateQcFailAction(null,
				QcFailAction.RESELECT_REGIONS, null);
	}

	@Test(expected=IllegalArgumentException.class)
	public void initiateQcFailAction_null_QcFailAction_arg() {

		cropServiceImpl.initiateQcFailAction(crop,  null, null);
	}

	@Test(expected=RuntimeException.class)
	public void initiateQcFailAction_passed_crop() {

		crop.setQcLocked( Boolean.FALSE );
		crop.setQcPass(  Boolean.TRUE );

		cropServiceImpl.initiateQcFailAction(crop,
				QcFailAction.RESELECT_REGIONS, null);
	}

	@Test(expected=RuntimeException.class)
	public void initiateQcFailAction_locked_crop() {

		crop.setQcLocked( Boolean.TRUE  );
		crop.setQcPass( Boolean.FALSE  );

		cropServiceImpl.initiateQcFailAction(crop,
				QcFailAction.RESELECT_REGIONS, null);

	}

}
