/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.histology.services;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.assertEquals;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;
import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.histology.Crop;
import uk.ac.sanger.mig.histology.CropJob;
import uk.ac.sanger.mig.histology.Scan;
import uk.ac.sanger.mig.histology.Slide;
import uk.ac.sanger.mig.histology.dao.ScanDao;
import uk.ac.sanger.mig.personnel.Person;

public class ScanServiceMockTest extends UnitilsJUnit4 {

	@TestedObject
	private ScanServiceImpl scanServiceImpl;

	@Mock @InjectIntoByType private ScanDao scanDaoMock;
	@Mock @InjectIntoByType private CropService cropServiceMock;
	@Mock @InjectIntoByType private CropJobService cropJobServiceMock;
	@Mock @InjectIntoByType private SlideService slideServiceMock;

	private Scan testScan;
	private Crop crop1, crop2;
	private final Long SCAN_ID = 1L;
	private Person user;
	private final int EXPECTED_CROP_COUNT = 2;
	private List<CropJob> cropJobs;

	@Before
	public void setUp() {

		CropJob cropJob1 = new CropJob();
		crop1 = new Crop();
	    crop2 = new Crop();
		List<Crop> crops = new ArrayList<Crop>();
		crops.add( crop1 ); 	crops.add( crop2 );

		cropJobs = new ArrayList<CropJob>();
		cropJob1.setCrops( crops );
		cropJobs.add( cropJob1 );

		testScan = new Scan();
		testScan.setId( SCAN_ID );

		user = new Person();

	}

	@Test
	public void gentlyFailAndReorder() {

		Slide actualSlide = new Slide();
		Slide expectedSlide = new Slide();
		expectedSlide.setStatusDictId( Slide.StatusDictEnum.AWAITING_SCAN.getId() );

		expect( scanDaoMock.fetchById( SCAN_ID ) ).andReturn(testScan).once();
		expect( slideServiceMock.fetchById(null, null) ).andReturn( actualSlide ).once();
		expect( slideServiceMock.store(actualSlide, user) ).andReturn( null ).once();
		expect( slideServiceMock.addToScanningBin( null ) ).andReturn( null ).once();

		//validate call to gentlyFail()
		expect( scanDaoMock.fetchById(SCAN_ID) ).andReturn( testScan ).once();
		expect( cropJobServiceMock.fetchByScanId(null, null) ).andReturn( cropJobs );
		expect( cropServiceMock.gentlyFail(null, null, null)).andReturn(true).times( EXPECTED_CROP_COUNT );
		expect( scanDaoMock.removeFromCropSelectionBin( null ) ).andReturn(testScan).once();
		expect( scanDaoMock.store(null, null) ).andReturn( testScan.getId() ).once();
		expect( cropServiceMock.qcLock( null, null ) ).andReturn( null ).times( EXPECTED_CROP_COUNT );

		replay();

		scanServiceImpl.gentlyFailAndReorder(SCAN_ID, user);

		verify();

		//validate slide status set to AWAITING_SCAN
		assertRefEquals(expectedSlide, actualSlide);
	}


	@Test(expected=IllegalArgumentException.class)
	public void gentlyFailAndReorder_null_scanId() {

		scanServiceImpl.gentlyFailAndReorder(null, user);

	}

	@Test
	public void gentlyFail_fail_all_children_succeeds() {

		boolean expected_is_failure_success = true;

		//fetch Scan with attached components
		expect( scanDaoMock.fetchById(SCAN_ID) ).andReturn( testScan ).once();
		expect( cropJobServiceMock.fetchByScanId(null, null) ).andReturn( cropJobs );

		//failing crops succeeds each time returning true x crop count
		expect( cropServiceMock.gentlyFail(null, null, null)).andReturn(true).times( EXPECTED_CROP_COUNT );

		//called  x 1 scan
		expect( scanDaoMock.removeFromCropSelectionBin( null ) ).andReturn(testScan).once();

		//called  x 1 scan
		expect( scanDaoMock.store(null, null) ).andReturn( testScan.getId() ).once();

		//called  x crop count
		expect( cropServiceMock.qcLock( null, null ) ).andReturn( null ).times( EXPECTED_CROP_COUNT );

		replay();

		boolean actual_is_failure_success
			= scanServiceImpl.gentlyFail(SCAN_ID,
									     Scan.StatusDictEnum.THUMBNAIL_QC_FAIL_BAD_SCAN,
									     user);

		verify();

		assertEquals(expected_is_failure_success, actual_is_failure_success);


	}


	@Test(expected=IllegalArgumentException.class)
	public void gentlyFail_null_cause_arg() {

		scanServiceImpl.gentlyFail(SCAN_ID,
			     null,
			     user);

	}

	@Test(expected=IllegalArgumentException.class)
	public void gentlyFail_null_user_arg() {

		scanServiceImpl.gentlyFail(SCAN_ID,
			     Scan.StatusDictEnum.THUMBNAIL_QC_FAIL_BAD_SCAN,
			     null);

	}

}
