/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.hamcrest.core.IsInstanceOf;
import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.miprocessing.workflow.Task;

public class ScheduledMicroInjectionJobTest extends UnitilsJUnit4 {

	@TestedObject
	private ScheduledMicroInjectionJob scheduledMicroInjectionJob;

	private List<Task> unordredTasks = new ArrayList<Task>();
	private List<Task> expectedSortedTasks = new ArrayList<Task>();
	
	private List<Task> allTasksMilestoneTest = new ArrayList<Task>();//milestones and non-milestones
	private List<Task> expectedMilestoneSet = new ArrayList<Task>();//just milestones
	
	private Task expectedMilestoneTask2, expectedMilestoneTask4;
	
	@Before
	public void setUp() {

		//unsorted input list
		Task t1 = new ScheduledMiTask();
		t1.setSequenceNumber(5);
		Task t2 = new ScheduledMiTask();
		t2.setSequenceNumber(4);
		Task t3 = new ScheduledMiTask();
		t3.setSequenceNumber(7);
		Task t4 = new ScheduledMiTask();
		t4.setSequenceNumber(11);
		Task t5 = new ScheduledMiTask();
		t5.setSequenceNumber(10);
		
		unordredTasks.add( t1 );
		unordredTasks.add( t2 );
		unordredTasks.add( t3 );
		unordredTasks.add( t4 );
		unordredTasks.add( t5 );
		
		//sorted expected list
		Task ts1 = new ScheduledMiTask();
		ts1.setSequenceNumber(4);
		Task ts2 = new ScheduledMiTask();
		ts2.setSequenceNumber(5);
		Task ts3 = new ScheduledMiTask();
		ts3.setSequenceNumber(7);
		Task ts4 = new ScheduledMiTask();
		ts4.setSequenceNumber(10);
		Task ts5 = new ScheduledMiTask();
		ts5.setSequenceNumber(11);
		
		expectedSortedTasks.add( ts1 );
		expectedSortedTasks.add( ts2 );
		expectedSortedTasks.add( ts3 );
		expectedSortedTasks.add( ts4 );
		expectedSortedTasks.add( ts5 );	
		
	
		//milestone tests
		Task nonMilestineTask1 = new ScheduledMiTask();
		nonMilestineTask1.setMilestone( false );
		nonMilestineTask1.setName("Microinjection");
		
		expectedMilestoneTask2 = new ScheduledMiTask();
		expectedMilestoneTask2.setMilestone( true );
		expectedMilestoneTask2.setName( ScheduledMicroInjectionJob
				.MilestoneEnum.EMBRYO_TRANSFER_TASK_COMPLETED.getTaskName() );
		
		Task nonMilestineTask3 = new ScheduledMiTask();
		nonMilestineTask3.setMilestone( false );
		
		expectedMilestoneTask4 = new ScheduledMiTask();
		expectedMilestoneTask4.setMilestone( true );
		expectedMilestoneTask4.setName( ScheduledMicroInjectionJob
				.MilestoneEnum.F1_GENOTYPING_TASK_COMPLETED.getTaskName());
		
		allTasksMilestoneTest.add(nonMilestineTask1);
		allTasksMilestoneTest.add(expectedMilestoneTask2);
		allTasksMilestoneTest.add(nonMilestineTask3);
		allTasksMilestoneTest.add(expectedMilestoneTask4);
		
		expectedMilestoneSet.add( expectedMilestoneTask2 );
		expectedMilestoneSet.add( expectedMilestoneTask4 );
	}

	@Test
	public void getTasks_testSorted() {
		
		//set unordered list
		scheduledMicroInjectionJob.setTasks( unordredTasks );
		List<Task> actualReturnedList = scheduledMicroInjectionJob.getTasks();	
		assertRefEquals( expectedSortedTasks, actualReturnedList );	
	}
		

	@Test
	public void getMilestones() {
		
		scheduledMicroInjectionJob.setTasks( allTasksMilestoneTest );
		List<Task> actualMilestones = scheduledMicroInjectionJob.getMilestones();		
		assertEquals(expectedMilestoneSet, actualMilestones);
	}	

	@Test
	public void getMilestoneByEnum() {
	
		scheduledMicroInjectionJob.setTasks( allTasksMilestoneTest );	
		Task actualTask = scheduledMicroInjectionJob.getMilestoneByEnum(
				ScheduledMicroInjectionJob.MilestoneEnum.F1_GENOTYPING_TASK_COMPLETED);	
		assertEquals(expectedMilestoneTask4, actualTask);
	}
	
	@Test
	public void getMilestoneByEnum_no_match_found() {
	
		scheduledMicroInjectionJob.setTasks( allTasksMilestoneTest );	
		Task actualTask = scheduledMicroInjectionJob.getMilestoneByEnum( null );
		assertThat( actualTask, new IsInstanceOf( NullScheduledMiTask.class ) );
	}	
	
	@Test(expected=UnattachedComponentException.class) 
	public void testSchedule_smiJob_null() {
		
		scheduledMicroInjectionJob.setTasks( null );
		scheduledMicroInjectionJob.getMilestones();
	}
	
	@Test
	public void getUnusedMicroInjectedEmbryos() {
		final int EXPECTED_RESULT = 15;
		MicroInjection microInjection = new MicroInjection();
		microInjection.setInjectedEmbryoCount( 30 );
		EmbryoTransferProcedure et1 = new EmbryoTransferProcedure();
		EmbryoTransferProcedure et2 = new EmbryoTransferProcedure();
		et1.setNumberLeft( 3 ); et1.setNumberRight( 7 );
		et2.setNumberLeft( 2 ); et2.setNumberRight( 3 );
		Collection<EmbryoTransferProcedure> embryoTransfers
				= new ArrayList<EmbryoTransferProcedure>();
		embryoTransfers.add( et1 ); embryoTransfers.add( et2 );
		scheduledMicroInjectionJob.setMicroInjection( microInjection );
		scheduledMicroInjectionJob.setEmbryoTransfers( embryoTransfers );
		int actualResult = scheduledMicroInjectionJob.getUnusedMicroInjectedEmbryos();
		assertEquals(EXPECTED_RESULT, actualResult);
	}
	
	
	@Test(expected=UnattachedComponentException.class) 
	public void getUnusedMicroInjectedEmbryos_null_embryo_transfers() {

		MicroInjection microInjection = new MicroInjection();
		microInjection.setInjectedEmbryoCount( 30 );
		scheduledMicroInjectionJob.setMicroInjection( microInjection );
		scheduledMicroInjectionJob.setEmbryoTransfers( null );
		scheduledMicroInjectionJob.getUnusedMicroInjectedEmbryos();
		
	}
	
	
	@Test(expected=UnattachedComponentException.class) 
	public void getUnusedMicroInjectedEmbryos_null_microinjection() {
		EmbryoTransferProcedure et1 = new EmbryoTransferProcedure();
		EmbryoTransferProcedure et2 = new EmbryoTransferProcedure();
		et1.setNumberLeft( 3 ); et1.setNumberRight( 7 );
		et2.setNumberLeft( 2 ); et2.setNumberRight( 3 );
		Collection<EmbryoTransferProcedure> embryoTransfers
				= new ArrayList<EmbryoTransferProcedure>();
		embryoTransfers.add( et1 ); embryoTransfers.add( et2 );
		scheduledMicroInjectionJob.setMicroInjection( null );
		scheduledMicroInjectionJob.setEmbryoTransfers( embryoTransfers );
		scheduledMicroInjectionJob.getUnusedMicroInjectedEmbryos();
	}	
	
	
	@Test
	public void getInjectedEmbryoCount() {
		final int EXPECTED_RESULT = 500;
		MicroInjection microInjection = new MicroInjection();
		microInjection.setInjectedEmbryoCount( EXPECTED_RESULT );
		scheduledMicroInjectionJob.setMicroInjection( microInjection );
		int actualResult = scheduledMicroInjectionJob.getInjectedEmbryoCount();
		assertEquals(EXPECTED_RESULT, actualResult);
	}
	
	@Test
	public void getInjectedEmbryoCount_null_count() {
		final int EXPECTED_RESULT = 0;
		MicroInjection microInjection = new MicroInjection();
		microInjection.setInjectedEmbryoCount( null );
		scheduledMicroInjectionJob.setMicroInjection( microInjection );
		int actualResult = scheduledMicroInjectionJob.getInjectedEmbryoCount();
		assertEquals(EXPECTED_RESULT, actualResult);
	}	
	
	@Test(expected=UnattachedComponentException.class) 
	public void getInjectedEmbryoCount_null_microInjection() {

		scheduledMicroInjectionJob.setMicroInjection( null );
		scheduledMicroInjectionJob.getUnusedMicroInjectedEmbryos();
	}	
	
	@Test
	public void getMicroInjectionDateCompleted() {
		final Date EXPECTED_RESULT = new Date(); 
		MicroInjection microInjection = new MicroInjection();
		microInjection.setDateCompleted( EXPECTED_RESULT );
		scheduledMicroInjectionJob.setMicroInjection( microInjection );
		Date actualResult = scheduledMicroInjectionJob.getMicroInjectionDateCompleted();
		assertEquals(EXPECTED_RESULT, actualResult);
	}	
	
	@Test(expected=UnattachedComponentException.class) 
	public void getMicroInjectionDateCompleted_null_microInjection() {

		scheduledMicroInjectionJob.setMicroInjection( null );
		scheduledMicroInjectionJob.getMicroInjectionDateCompleted();
	}	
	

	
	
}
