/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;
import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.animalrequest.AnimalRequest;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.miprocessing.ScheduledMiTask;
import uk.ac.sanger.mig.miprocessing.ScheduledMicroInjectionJob;
import uk.ac.sanger.mig.miprocessing.workflow.Task;
import uk.ac.sanger.mig.miprocessing.workflow.TaskService;
import uk.ac.sanger.mig.mirequest.MiRequest;
import uk.ac.sanger.mig.mirequest.services.MiRequestService;
import uk.ac.sanger.mig.personnel.Person;

public class SchedulerServiceMockTest extends UnitilsJUnit4 {

	@TestedObject
	private SchedulerServiceImpl schedulerServiceImpl;

	@Mock @InjectIntoByType private ScheduledMicroInjectionService scheduledMicroInjectionService;
	@Mock @InjectIntoByType private MiRequestService miRequestService;
	@Mock @InjectIntoByType private ScheduledMiTaskService scheduledMiTaskService;
	@Mock @InjectIntoByType private TaskService timelineService;
	@Mock @InjectIntoByType private MirMouseTrackingBridgeService mirMouseTrackingBridgeService;
		
	private MiRequest miRequest = new MiRequest(); 
	private ScheduledMicroInjectionJob smiJob = new ScheduledMicroInjectionJob();
	private List<Task> tasks = new ArrayList<Task>();
	private Person mrPeebles = new Person();
	private final DictionaryEntry SCHEDULED_STATUS = new DictionaryEntry();
	private final int DEFAULT_TASK_COUNT = 3;
	
	@Before
	public void setUp() {

		miRequest.setId( 10L );
		
		smiJob.setId( 100L );
		smiJob.setMicroInjectionRequestId( miRequest.getId() );
		smiJob.setAnchorDate( new Date() );
		
		//set to 'scheduled' status 
		SCHEDULED_STATUS.setId( ScheduledMicroInjectionJob.InternalStatus.SCHEDULED.getId() );	
		
		//create 3 orphan tasks t be assigned
		for(int i = 0; i < DEFAULT_TASK_COUNT; i++) {
			tasks.add(new ScheduledMiTask());
		}
	}

	@Test
	public void testSchedule_noWtReq() {
		
		expect(miRequestService.fetchById( smiJob.getMiRequestId(), 
				ComponentMap.of(MiRequest.Component.CELL_LINE)) ).andReturn( miRequest ).once();
		
		expect( scheduledMicroInjectionService.getNewStatusDict(
				ScheduledMicroInjectionJob.InternalStatus.SCHEDULED ) ).andReturn( SCHEDULED_STATUS ).once();
		
		expect( scheduledMicroInjectionService.store(smiJob, mrPeebles) ).andReturn( smiJob.getId() ).once();
		
	
		expect( timelineService.generateTaskList(smiJob.getAnchorDate(),
				miRequest) ).andReturn( tasks ).once();
		
		expect( scheduledMiTaskService.store(new ScheduledMiTask(), mrPeebles) ).andReturn( null ).times(DEFAULT_TASK_COUNT);
		
		replay();

		int numDonors = 0, numRecips = 0;
		
		schedulerServiceImpl.schedule(smiJob, mrPeebles, numDonors, numRecips);

		verify();

		//check status properly set
		assertNotNull( smiJob.getScheduledMiStatusDict() );	
		assertRefEquals( smiJob.getScheduledMiStatusDict(), SCHEDULED_STATUS );
		
		//check *all* tasks have smi id assigned
		for(Task task : tasks) {
			assertNotNull(task.getParentJobId());
			assertEquals( smiJob.getId(), task.getParentJobId());
		}
		
	}
	
	
	@Test
	public void testSchedule_with_Donor_and_Recip_Req() {
		
		final Long expectedDonorRequestId = 1L, expectedRecipRequestId = 2L;
		
		expect(miRequestService.fetchById( smiJob.getMiRequestId(), 
				ComponentMap.of(MiRequest.Component.CELL_LINE)) ).andReturn( miRequest );
		
		expect( scheduledMicroInjectionService.getNewStatusDict(
				ScheduledMicroInjectionJob.InternalStatus.SCHEDULED ) ).andReturn( SCHEDULED_STATUS );
		
		expect( scheduledMicroInjectionService.store(smiJob, mrPeebles) ).andReturn( smiJob.getId() );
		
		expect( timelineService.generateTaskList(smiJob.getAnchorDate(),miRequest) 
				).andReturn( tasks );
		
		//called once for each task 
		expect( scheduledMiTaskService.store(new ScheduledMiTask(), mrPeebles) )
			.andReturn( null ).times(DEFAULT_TASK_COUNT);
				
		expect(scheduledMicroInjectionService.fetchRequiredDonorStrain( 
				miRequest.getCellLineStrainId() )).andReturn( new DictionaryEntry() );
		
		//called once for each of 2 requests
		expect( mirMouseTrackingBridgeService.storeAnimalRequest( 
				new AnimalRequest(), mrPeebles )).
				andReturn( expectedDonorRequestId ).once().andReturn( expectedRecipRequestId ).once();
		
		replay();

		int numDonors = 5, numRecips = 5;
		
		schedulerServiceImpl.schedule(smiJob, mrPeebles, numDonors, numRecips);

		verify();

		//verify both donor and recip ids set in smi
		assertEquals( expectedDonorRequestId,  smiJob.getDonorAnimalRequestId() );
		assertEquals( expectedRecipRequestId, smiJob.getRecipientAnimalRequestId() );
		
		//check status properly set
		assertNotNull( smiJob.getScheduledMiStatusDict() );	
		assertRefEquals( smiJob.getScheduledMiStatusDict(), SCHEDULED_STATUS );
		
		//check *all* tasks have smi id assigned
		for(Task task : tasks) {
			assertNotNull(task.getParentJobId());
			assertEquals( smiJob.getId(), task.getParentJobId());
		}
		
	}
	
	@Test
	public void testSchedule_with_Just_Just_Recip_Req() {
		
		final Long recipRequestId = 2L;
		
		expect(miRequestService.fetchById( smiJob.getMiRequestId(), 
				ComponentMap.of(MiRequest.Component.CELL_LINE)) ).andReturn( miRequest ).once();
		
		expect( scheduledMicroInjectionService.getNewStatusDict(
				ScheduledMicroInjectionJob.InternalStatus.SCHEDULED ) ).andReturn( SCHEDULED_STATUS ).once();
		
		expect( scheduledMicroInjectionService.store(smiJob, mrPeebles) ).andReturn( smiJob.getId() ).once();
		
		expect( timelineService.generateTaskList(smiJob.getAnchorDate(), miRequest) ).andReturn( tasks ).once();
		
		expect( scheduledMiTaskService.store(new ScheduledMiTask(), mrPeebles) ).andReturn( null ).times(DEFAULT_TASK_COUNT);
		
		//only one call this time!
		expect( mirMouseTrackingBridgeService.storeAnimalRequest( 
				new AnimalRequest(), mrPeebles )).andReturn( recipRequestId ).once();
		
		replay();

		int numDonors = 0, numRecips = 5;
		
		schedulerServiceImpl.schedule(smiJob, mrPeebles, numDonors, numRecips);

		verify();

		//verify just recip id set in smi, 
		assertEquals( smiJob.getRecipientAnimalRequestId(), recipRequestId ) ;
		assertNull( smiJob.getDonorAnimalRequestId() );
		
		//check status properly set
		assertNotNull( smiJob.getScheduledMiStatusDict() );	
		assertRefEquals( smiJob.getScheduledMiStatusDict(), SCHEDULED_STATUS );
		
		//check *all* tasks have smi id assigned
		for(Task task : tasks) {
			assertNotNull(task.getParentJobId());
			assertEquals(task.getParentJobId(), smiJob.getId() );
		}
		
	}
	
	
	@Test(expected=IllegalArgumentException.class) 
	public void testSchedule_smiJob_null() {
		
		schedulerServiceImpl.schedule(null, mrPeebles, 0, 0);
	}	
	
	@Test(expected=IllegalArgumentException.class) 
	public void testSchedule_user_null() {
		
		schedulerServiceImpl.schedule(smiJob, null, 0, 0);
	}	
	
	@Test(expected=IllegalArgumentException.class) 
	public void testSchedule_numberDonors_negative() {
		
		schedulerServiceImpl.schedule(smiJob, mrPeebles, -1, 0);
	}

	@Test(expected=IllegalArgumentException.class) 
	public void testSchedule_numberRecips_negative() {
		
		schedulerServiceImpl.schedule(smiJob, mrPeebles, 0, -1);
	}	

	@Test(expected=IllegalStateException.class) 
	public void testSchedule_smiJob_no_anchor_date() {
		
		smiJob.setAnchorDate( null );
		schedulerServiceImpl.schedule(smiJob, mrPeebles, 0, 0);
	}	

	@Test(expected=IllegalStateException.class) 
	public void testSchedule_smiJob_no_mi_req_id() {
		
		smiJob.setMicroInjectionRequestId( null );
		schedulerServiceImpl.schedule(smiJob, mrPeebles, 0, 0);
	}	
	
	@Test(expected=RuntimeException.class) 
	public void testSchedule_smiJob_mi_req_non_existent() {

		//reset MiRequest ID
		smiJob.setMicroInjectionRequestId( 1L );

		expect(miRequestService.fetchById( smiJob.getMiRequestId(), 
				ComponentMap.of(MiRequest.Component.CELL_LINE)) ).andStubReturn( null );
		
		replay();
		
		schedulerServiceImpl.schedule(smiJob, mrPeebles, 0, 0);
		
		verify();
	}	
	
}
