/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.miprocessing.services;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.miprocessing.workflow.Task;
import uk.ac.sanger.mig.miprocessing.workflow.TaskDao;
import uk.ac.sanger.mig.miprocessing.workflow.TaskServiceImpl;
import uk.ac.sanger.mig.miprocessing.workflow.TaskTemplate;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.MiRequest;

public class TaskServiceMockTest extends UnitilsJUnit4 {

	@TestedObject
	private TaskServiceImpl taskServiceImpl;

	@Mock @InjectIntoByType private TaskDao taskDao;

	private List<TaskTemplate> taskTemplates;
	private Date anchorDate;
	private MiRequest miRequest;
	private Long EXPECTED_TASK_GROUP_ID = 100L;

	
	@Before
	public void setUp() {
		
		taskTemplates = new ArrayList<TaskTemplate>();
		
		TaskTemplate taskTemplate1 = new TaskTemplate();				
		taskTemplate1.setName("first task");
		taskTemplate1.setSequenceNumber( 1 );	
		taskTemplate1.setTaskGroupDictId( 100L );
		taskTemplate1.setDaysOffset( new Double(0.0) );
		
		TaskTemplate taskTemplate2 = new TaskTemplate();
		taskTemplate2.setName("second task");
		taskTemplate2.setSequenceNumber( 2 );	
		taskTemplate2.setTaskGroupDictId( 100L );
		taskTemplate2.setDaysOffset( new Double(7.0) );
		
		taskTemplates.add(taskTemplate1);
		taskTemplates.add(taskTemplate2);
		
		miRequest = new MiRequest();
		miRequest.setCellLine( new CellLine() );

		anchorDate = new Date();
	}

	@Test
	public void generateTaskList_good_args() {
	
		miRequest.getCellLine().setCellLineStrainDictId( 1L );
		miRequest.setEndpointDictId( 1L );
		miRequest.setChimeraMateStrainDictId( 1L);
		
		expect( taskDao.fetchTaskTemplates( miRequest ) ).andReturn( taskTemplates );
		
		replay();

		List<Task> tasks = taskServiceImpl.generateTaskList(anchorDate, miRequest);
		
		verify();

		assertNotNull( tasks ); //has non-null tasks
		assertEquals( taskTemplates.size(), tasks.size() ); //as many tasks as taskTemplates
		

		for(Task task : tasks) {
			
			assertNotNull( task.getEstimatedCompletionDate() ); //has completion date
			assertNotNull( task.getName() ); //has a name
			assertEquals( EXPECTED_TASK_GROUP_ID, task.getTaskGroupDictId() );//belongs to same group	
		}
		
	}	
	
	@Test(expected=IllegalArgumentException.class) 
	public void  generateTaskList_null_miRequest() {

		taskServiceImpl.generateTaskList(anchorDate, null);
	}	

	@Test(expected=IllegalArgumentException.class) 
	public void  generateTaskList_null_anchorDate() {

		taskServiceImpl.generateTaskList(null, miRequest);
	}	
	
	@Test(expected=RuntimeException.class) 
	public void  generateTaskList_null_endpoint_id() {

		miRequest.getCellLine().setCellLineStrainDictId( 1L );
		miRequest.setEndpointDictId( null );
		miRequest.setChimeraMateStrainDictId( 1L);
		taskServiceImpl.generateTaskList(anchorDate, miRequest);
	}	

	@Test(expected=RuntimeException.class) 
	public void  generateTaskList_null_f1_and_null_cell_line_strain() {

		miRequest.getCellLine().setCellLineStrainDictId( null );
		miRequest.setEndpointDictId( MiRequest.EndpointEnum.F1.getId() );
		miRequest.setChimeraMateStrainDictId( 1L);
		taskServiceImpl.generateTaskList(anchorDate, miRequest);
	}	

	@Test(expected=RuntimeException.class) 
	public void  generateTaskList_null_f1_and_null_chimera_strain() {

		miRequest.getCellLine().setCellLineStrainDictId(1L );
		miRequest.setEndpointDictId( MiRequest.EndpointEnum.F1.getId() );
		miRequest.setChimeraMateStrainDictId( null );
		taskServiceImpl.generateTaskList(anchorDate, miRequest);
	}
	
}
