/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.services;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.unitils.easymock.EasyMockUnitils.replay;

import java.io.IOException;

import javax.xml.transform.stream.StreamSource;

import org.junit.Before;
import org.junit.Test;
import org.springframework.oxm.Unmarshaller;
import org.springframework.oxm.UnmarshallingFailureException;
import org.springframework.oxm.XmlMappingException;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.alleles.services.AlleleService;
import uk.ac.sanger.mig.alleles.services.AlleleServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.common.UnattachedComponentException;
import uk.ac.sanger.mig.common.services.ComponentMapService;
import uk.ac.sanger.mig.database.DictionaryService;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.mirequest.dao.CellLineDao;

public class CellLineServiceMockTest extends UnitilsJUnit4 {

	@TestedObject
	private CellLineServiceImpl cellLineServiceImpl;


	@Mock @InjectIntoByType private WebServiceTemplate webServiceTemplateMock;
	@Mock @InjectIntoByType private CellLineDao cellLineDaoMock;
	@Mock @InjectIntoByType private AlleleService alleleServiceMock;
	@Mock @InjectIntoByType private DictionaryService dictionaryServiceMock;
	@Mock @InjectIntoByType private ComponentMapService componentMapServiceMock;
	@Mock private Unmarshaller unmarshallerMock;

	private CellLine testCellLine;
	private Allele testAllele;

	private CellLine webServiceCellLine;
	private final String EXPECTED_DESIGN_INSTANCE = "2";
	private final String ALLELE_SUPERSCRIPT = "tm"+EXPECTED_DESIGN_INSTANCE+"aWtsi";
	private final String SHORT_ALLELE_SYMBOL = "Art4";
	private final String TEAM87_GENE_SYMBOL = "Art4";

	@Before
	public void setUp() {

		testCellLine = new CellLine();
		testAllele = new Allele();
		testCellLine.setModifiedAllele( testAllele );

		//minimum expected object hierarchy expected from
		//Team 87 web service
		webServiceCellLine = new CellLine();
		Allele allele = new Allele();
		allele.setOfficialNameSuperscript( ALLELE_SUPERSCRIPT );
		allele.setShortAlleleSymbol( SHORT_ALLELE_SYMBOL );
		allele.setTeam87GeneSymbol(TEAM87_GENE_SYMBOL);
		webServiceCellLine.setModifiedAllele( allele );
		DictionaryEntry fundingDict = new DictionaryEntry();
		fundingDict.setName("KOMP");
		webServiceCellLine.setFundingDict( fundingDict );
		DictionaryEntry strainDict = new DictionaryEntry();
		strainDict.setName("JM8.N4");
		webServiceCellLine.setCellLineStrain(strainDict);
		DictionaryEntry coatDict = new DictionaryEntry();
		coatDict.setName("Black");
		webServiceCellLine.setExpectedCoatColor(coatDict);
	}

	//@Test
	public void deriveEpdCellLine_service_reponse_OK()
				throws IOException, XmlMappingException, InvalidTeam87InfoException {

		expect( cellLineDaoMock.fetchByEpdId("") ).andReturn( null ).once();

		expect( webServiceTemplateMock.getDefaultUri() )
			.andReturn( "http://gtrap1a.internal.sanger.ac.uk:8000/htgt/report/get_mig_microinjection_info" ).once();

		expect( webServiceTemplateMock.getUnmarshaller() )
			.andReturn( unmarshallerMock ).once();

		expect( unmarshallerMock.unmarshal( new StreamSource() ))
			.andReturn( webServiceCellLine ).once();

		expect( componentMapServiceMock.getComponentMap("editCellLine")).andReturn(new ComponentMap()).once();

		expect(  alleleServiceMock.deriveEpdAllele( null ) )
			.andReturn( webServiceCellLine.getModifiedAllele() ).once();

		//expect( AlleleServiceImpl.SUPERSCRIPT_PATTERN ).andReturn("(tm\\d+[a-z]).*").times(2);

		expect( dictionaryServiceMock.fetchEntry(null,null, true))
			.andReturn( new DictionaryEntry() ).times( 3 );

		replay();

		cellLineServiceImpl.deriveEpdCellLine( "" );

		verify();

	    assertTrue(webServiceCellLine.getCellLineName().startsWith( SHORT_ALLELE_SYMBOL ) );
	    assertTrue(webServiceCellLine.getCellLineName().endsWith( "2" ) );
	}

	@Test(expected=IOException.class)
	public void deriveEpdCellLine_service_bad_connection()
				throws IOException, XmlMappingException, InvalidTeam87InfoException {
		//inject proper URL to avoid malformed url except
		expect( webServiceTemplateMock.getDefaultUri() )
			.andReturn( "http://gtrap1a.internal.sanger.ac.uk:8000/htgt/report/get_mig_microinjection_info" ).once();

		expect( webServiceTemplateMock.getUnmarshaller() )
			.andReturn( unmarshallerMock ).once();

		expect( unmarshallerMock.unmarshal( new StreamSource() )).andThrow( new IOException("test") );

		replay();

		cellLineServiceImpl.deriveEpdCellLine( "" );
	}

	@Test(expected=XmlMappingException.class)
	public void deriveEpdCellLine_service_bad_xml()
				throws IOException, XmlMappingException, InvalidTeam87InfoException {
		//inject proper URL to avoid malformed url except
		expect( webServiceTemplateMock.getDefaultUri() )
			.andReturn( "http://gtrap1a.internal.sanger.ac.uk:8000/htgt/report/get_mig_microinjection_info" ).once();

		expect( webServiceTemplateMock.getUnmarshaller() )
			.andReturn( unmarshallerMock ).once();

		expect( unmarshallerMock.unmarshal( new StreamSource() )).andThrow( new UnmarshallingFailureException("test") );

		replay();

		cellLineServiceImpl.deriveEpdCellLine( "" );
	}

	@Test(expected=InvalidTeam87InfoException.class)
	public void deriveEpdCellLine_no_allele_superscript()
				throws IOException, XmlMappingException, InvalidTeam87InfoException {

		CellLine webServiceCellLine = new CellLine();
		Allele allele = new Allele();
		allele.setOfficialNameSuperscript( null );
		allele.setShortAlleleSymbol( SHORT_ALLELE_SYMBOL );
		allele.setTeam87GeneSymbol( TEAM87_GENE_SYMBOL );
		webServiceCellLine.setModifiedAllele( allele );
		DictionaryEntry fundingDict = new DictionaryEntry();
		fundingDict.setName("KOMP");
		webServiceCellLine.setFundingDict( fundingDict );
		DictionaryEntry strainDict = new DictionaryEntry();
		strainDict.setName("JM8.N4");
		webServiceCellLine.setCellLineStrain(strainDict);
		DictionaryEntry coatDict = new DictionaryEntry();
		coatDict.setName("Black");
		webServiceCellLine.setExpectedCoatColor(coatDict);

		expect( webServiceTemplateMock.getDefaultUri() )
			.andReturn( "http://gtrap1a.internal.sanger.ac.uk:8000/htgt/report/get_mig_microinjection_info" ).once();

		expect( webServiceTemplateMock.getUnmarshaller() )
			.andReturn( unmarshallerMock ).once();

		expect( unmarshallerMock.unmarshal( new StreamSource() )).andReturn( webServiceCellLine );

		replay();

		cellLineServiceImpl.deriveEpdCellLine( "" );
	}

	@Test(expected=InvalidTeam87InfoException.class)
	public void deriveEpdCellLine_no_allele()
				throws IOException, XmlMappingException, InvalidTeam87InfoException {

		CellLine webServiceCellLine = new CellLine();
		webServiceCellLine.setModifiedAllele( null );
		DictionaryEntry fundingDict = new DictionaryEntry();
		fundingDict.setName("KOMP");
		webServiceCellLine.setFundingDict( fundingDict );
		DictionaryEntry strainDict = new DictionaryEntry();
		strainDict.setName("JM8.N4");
		webServiceCellLine.setCellLineStrain(strainDict);
		DictionaryEntry coatDict = new DictionaryEntry();
		coatDict.setName("Black");
		webServiceCellLine.setExpectedCoatColor(coatDict);

		expect( webServiceTemplateMock.getDefaultUri() )
			.andReturn( "http://gtrap1a.internal.sanger.ac.uk:8000/htgt/report/get_mig_microinjection_info" ).once();

		expect( webServiceTemplateMock.getUnmarshaller() )
			.andReturn( unmarshallerMock ).once();

		expect( unmarshallerMock.unmarshal( new StreamSource() )).andReturn( webServiceCellLine );

		replay();

		cellLineServiceImpl.deriveEpdCellLine( "" );
	}

	@Test(expected=InvalidTeam87InfoException.class)
	public void deriveEpdCellLine_no_cellline()
				throws IOException, XmlMappingException, InvalidTeam87InfoException {

		CellLine webServiceCellLine = null;

		expect( webServiceTemplateMock.getDefaultUri() )
			.andReturn( "http://gtrap1a.internal.sanger.ac.uk:8000/htgt/report/get_mig_microinjection_info" ).once();

		expect( webServiceTemplateMock.getUnmarshaller() )
			.andReturn( unmarshallerMock ).once();

		expect( unmarshallerMock.unmarshal( new StreamSource() )).andReturn( webServiceCellLine );

		replay();

		cellLineServiceImpl.deriveEpdCellLine( "" );
	}

	@Test(expected=InvalidTeam87InfoException.class)
	public void deriveEpdCellLine_no_gene_symbol()
				throws IOException, XmlMappingException, InvalidTeam87InfoException {

		CellLine webServiceCellLine = new CellLine();
		Allele allele = new Allele();
		allele.setOfficialNameSuperscript( ALLELE_SUPERSCRIPT );
		allele.setShortAlleleSymbol( SHORT_ALLELE_SYMBOL );
		allele.setTeam87GeneSymbol( null );
		webServiceCellLine.setModifiedAllele( allele );
		DictionaryEntry fundingDict = new DictionaryEntry();
		fundingDict.setName("KOMP");
		webServiceCellLine.setFundingDict( fundingDict );
		DictionaryEntry strainDict = new DictionaryEntry();
		strainDict.setName("JM8.N4");
		webServiceCellLine.setCellLineStrain(strainDict);
		DictionaryEntry coatDict = new DictionaryEntry();
		coatDict.setName("Black");
		webServiceCellLine.setExpectedCoatColor(coatDict);

		expect( webServiceTemplateMock.getDefaultUri() )
			.andReturn( "http://gtrap1a.internal.sanger.ac.uk:8000/htgt/report/get_mig_microinjection_info" ).once();

		expect( webServiceTemplateMock.getUnmarshaller() )
			.andReturn( unmarshallerMock ).once();

		expect( unmarshallerMock.unmarshal( new StreamSource() )).andReturn( webServiceCellLine );

		replay();

		cellLineServiceImpl.deriveEpdCellLine( "" );
	}

	/**
import static org.easymock.EasyMock.expect;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;
import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

*/

	@Test
	public void storeComplex_OK() {

		Long expectedAlleleId = 1L;
		Long expectedCellLineId = 10L;

		expect( alleleServiceMock.store( testAllele, null ) ).andReturn( expectedAlleleId );
		expect( cellLineDaoMock.store( testCellLine, null ) ).andReturn( expectedCellLineId );

		replay();

		Long actualCellLineId = cellLineServiceImpl.storeComplex(testCellLine, null);

		verify();

		//ensure allele id gets set
		assertEquals(expectedAlleleId, testCellLine.getModifiedAlleleId());
		//ensure rv is what we expect
		assertEquals(expectedCellLineId, actualCellLineId);

	}

	@Test(expected=IllegalArgumentException.class)
	public void storeComplex_null_cellLine() {

		cellLineServiceImpl.storeComplex(null, null);
	}

	@Test(expected=UnattachedComponentException.class)
	public void storeComplex_null_allele() {

		cellLineServiceImpl.storeComplex(new CellLine(), null);
	}

}
