/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.mirequest.services;
import static org.easymock.EasyMock.expect;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;
import static org.unitils.reflectionassert.ReflectionAssert.assertRefEquals;

import java.io.IOException;

import org.junit.Before;
import org.junit.Test;
import org.springframework.oxm.UnmarshallingFailureException;
import org.springframework.oxm.XmlMappingException;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.alleles.Allele;
import uk.ac.sanger.mig.colony.ColonyInfo;
import uk.ac.sanger.mig.colony.services.ColonyInfoService;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.mirequest.CellLine;
import uk.ac.sanger.mig.mirequest.InvalidTeam87InfoException;
import uk.ac.sanger.mig.mirequest.MiRequest;

public class MiRequestServiceMockTest extends UnitilsJUnit4 {

	@TestedObject
	private MiRequestServiceImpl miRequestServiceImpl;

	@Mock @InjectIntoByType private CellLineService cellLineServiceMock;
	@Mock @InjectIntoByType private ColonyInfoService colonyInfoServiceMock;

	private final String epdId = "EPD0025_1_H03";
	private CellLine testCellLine;
	private ColonyInfo testColonyInfo;
	private MiRequest testMiRequest;
	@Before
	public void setUp() {

		testMiRequest = new MiRequest();
		testCellLine = new CellLine();
		Allele testAllele = new Allele();
		testAllele.setOfficialNameSuperscript("nonNullOfficialAlleleName");
		testAllele.setMigGene( new MigGene() );
		testCellLine.setModifiedAllele( testAllele );
		testCellLine.setCellLineStrainDictId( 1L );
		testMiRequest.setCellLine( testCellLine );
	}

	@Test
	public void buildComplex_OK()
		throws IOException, XmlMappingException, InvalidTeam87InfoException {

		MiRequest expectedMiRequest = new MiRequest();
		CellLine expectedCellLine = new CellLine();
		Allele expectedAllele = new Allele();
		expectedAllele.setOfficialNameSuperscript("nonNullOfficialAlleleName");
		expectedAllele.setMigGene( new MigGene() );
		expectedCellLine.setCellLineStrainDictId( 1L );
		expectedCellLine.setModifiedAllele( expectedAllele );

		expectedMiRequest.setCellLine( expectedCellLine );

		expect( cellLineServiceMock.deriveEpdCellLine( epdId ) ).andReturn( testCellLine );
		expect(  colonyInfoServiceMock.deriveDefaultColonyInfo( testCellLine ) ).andReturn( testColonyInfo );

		replay();

		miRequestServiceImpl.buildComplex(testMiRequest, epdId);

		verify();

		assertRefEquals( expectedMiRequest, testMiRequest );

	}


	@Test(expected=XmlMappingException.class)
	public void buildComplex_XmlMappingException()
		throws IOException, XmlMappingException, InvalidTeam87InfoException {

		expect( cellLineServiceMock.deriveEpdCellLine( epdId ) ).andThrow(
				new UnmarshallingFailureException("bad xml") );

		replay();

		miRequestServiceImpl.buildComplex(testMiRequest, epdId);

	}

	@Test(expected=IOException.class)
	public void buildComplex_IOException()
		throws IOException, XmlMappingException, InvalidTeam87InfoException {

		expect( cellLineServiceMock.deriveEpdCellLine( epdId ) ).andThrow( new IOException("bad IO") );

		replay();

		miRequestServiceImpl.buildComplex(testMiRequest, epdId);

	}


}
