package uk.ac.sanger.mig.mousetracking.dao;

import static org.junit.Assert.*;
import static org.unitils.reflectionassert.ReflectionAssert.*;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.dbunit.annotation.DataSet;
import org.unitils.dbunit.annotation.ExpectedDataSet;
import org.unitils.spring.annotation.SpringApplicationContext;
import org.unitils.spring.annotation.SpringBean;

import uk.ac.sanger.mig.common.DictionaryEntry;
import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.CohortSearchBean;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.views.CohortListView;
import uk.ac.sanger.mig.personnel.Person;

@SpringApplicationContext({"mouse-tracking-dao.xml","common.xml","test-config.xml","unitils-test-database.xml"})
@DataSet
public class CohortDaoTest extends UnitilsJUnit4 {
	@SpringBean("cohortDao")
	private CohortDao cohortDao;
	private Cohort referenceCohort;
	private Mouse referenceMouse;
	private Log logger = LogFactory.getLog(this.getClass());
	private DateFormat df = new SimpleDateFormat("dd-MM-yyyy");
	private DictionaryEntry cohortPipeline;
	private Person testUser;

	@Before
	public void setup() throws ParseException {
		cohortPipeline = new DictionaryEntry();
		cohortPipeline.setId(1L);
		cohortPipeline.setName("Pipeline 1");

		referenceCohort = new Cohort();
		referenceCohort.setId(100L);
		referenceCohort.setCohortName("Test cohort");
		referenceCohort.setReason("hah");
		referenceCohort.setCohortPipelineDictId(1L);
		referenceCohort.setCohortTypeDictId(1L);
		referenceCohort.setCreatorId(14L);
		referenceCohort.setEndDate(df.parse("13-08-1980"));
		referenceCohort.setCohortPipeline(cohortPipeline);

		referenceMouse = new Mouse();
		referenceMouse.setId(99L);
		referenceMouse.setStrainDictId(1L);
		referenceMouse.setName("TestMoose1.1a");
		referenceMouse.setBarcode("M00000099");
		referenceMouse.setCageId(1L);
		referenceMouse.setColonyInfoId(77L);

		testUser = new Person();
		testUser.setId(14L);
	}

	@Test @ExpectedDataSet
	public void testStore() {
		Cohort c = cohortDao.fetchById(referenceCohort.getId());
		assertRefEquals(referenceCohort, c);
		cohortDao.store(c, testUser);
	}

	@Test
	public void testFetchByName() {
		singleAssert(cohortDao.fetchByName(referenceCohort.getCohortName()));
	}
	@Test
	public void testDontFetchByName() {
		Cohort c = cohortDao.fetchByName("gibberish");
		assertNull(c);
	}

	@Test
	public void testFetchForLookup() {
		collectionAsserts(cohortDao.fetchForLookup(referenceCohort.getCohortTypeDictId()));
	}

	@Test
	public void testDontFetchForLookup() {
		Collection<Cohort> cohorts = cohortDao.fetchForLookup(-1L);
		assertTrue(cohorts.isEmpty());
	}

	@Test
	public void testFetchByMouseId() {
		collectionAsserts(cohortDao.fetchByMouseId(referenceMouse.getId()));

	}

	@Test
	public void testSearch() throws IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		for(Map.Entry<String, CohortSearchBean> entry : getSearchCriteria().entrySet()) {
				CohortSearchBean csb = entry.getValue();
				String name = entry.getKey();
				logger.debug("Searching with search criteria from "+name);

				Collection<CohortListView> views = (cohortDao.search(csb));
				assertNotNull(views);
				assertEquals(1, views.size());
		}
	}

	private Map<String,CohortSearchBean> getSearchCriteria() throws IllegalArgumentException, IllegalAccessException, InvocationTargetException{
		Map<String,CohortSearchBean> searchBeans = new HashMap<String,CohortSearchBean>();

		for(Method m :this.getClass().getDeclaredMethods()) {
			if (m.getReturnType().equals(CohortSearchBean.class)) {
				CohortSearchBean csb = (CohortSearchBean)m.invoke(this, new Object[0]);
				searchBeans.put(m.getName(),csb);
			}
		}
		return searchBeans;
	}

	// The following methods provide search beans to feed into the search & fetch queries

	@SuppressWarnings("unused")
	private CohortSearchBean getTypeSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setCohortTypeDictId(referenceCohort.getCohortTypeDictId());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getMouseAliveSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setMouseStatus("Alive");
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getMouseStrainSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.getMouse().setStrainDictId(1L);
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getMouseNameSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.getMouse().setName(referenceMouse.getName());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getMouseBarcodeSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.getMouse().setBarcode(referenceMouse.getBarcode());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getMouseIdentifierSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setMouseIdentifier(referenceMouse.getBarcode());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getCageIdSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.getMouse().setCageId(referenceMouse.getCageId());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getColonyIdSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.getMouse().setColonyInfoId(referenceMouse.getColonyInfoId());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getMouseIdSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.getMouse().setId(referenceMouse.getId());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getCreatorIdSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setCreatorId(referenceCohort.getCreatorId());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getEndDateSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setFilterPeriodStart(referenceCohort.getEndDate());
		csb.setFilterPeriodEnd(referenceCohort.getEndDate());
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getOrderBySearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setOrderBy("ID");
		csb.setOrderDesc(true);
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getOrderByPipelineSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setOrderBy("pipeline");
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getOrderByTotalSearchBean() {
		CohortSearchBean csb = getSearchBean();
		csb.setOrderBy("total_mice");
		return csb;
	}

	@SuppressWarnings("unused")
	private CohortSearchBean getSearchBean() {
		CohortSearchBean csb = new CohortSearchBean();
		csb.setPaginated(false);
		csb.setMouse(new Mouse());
		return csb;
	}

	private void collectionAsserts(Collection<Cohort> cohorts) {
		assertNotNull(cohorts);
		assertEquals(1,cohorts.size());
		assertRefEquals(referenceCohort, cohorts.iterator().next());
	}

	private void singleAssert(Cohort c) {
		assertNotNull(c);
		assertRefEquals(referenceCohort, c);
	}


}
