/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.personnel.dao;

import static org.junit.Assert.*;
import static org.unitils.reflectionassert.ReflectionAssert.*;

import java.util.Collection;
import java.util.Iterator;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.dbunit.annotation.DataSet;
import org.unitils.dbunit.annotation.ExpectedDataSet;
import org.unitils.spring.annotation.SpringApplicationContext;
import org.unitils.spring.annotation.SpringBean;

import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.PersonSearchBean;

/*
 *
 * @DataSet annotation causes unitils to load data from
 * personDaoTest.xml into the unit test database before each test
 *
 * @SpringApplicationContext loads the specified xml files,
 * these should be the minimum dependencies to make the dao work
 *
 * the unitils-test-database.xml sets the migDataSource to
 * use the db configured in unitils. It should be the last
 * thing in the config list, to override any other datasource
 * definition.
 */
@SpringApplicationContext({"personnel-dao.xml","common.xml","test-config.xml","unitils-test-database.xml"})
@DataSet
public class PersonDaoTest extends UnitilsJUnit4 {


  /* @SpringBean loads the personDao from the application
   * context configured with @SpringApplicationContext
   */
  @SpringBean("personDao")
  private PersonDao personDao;

  private Person testUser;
  private String password;

  @Before
  public void setUp() {
    testUser = new Person();
    testUser.setId(10L);
    testUser.setUserName("test_user");
    testUser.setPasswordHash("a94a8fe5ccb19ba61c4c0873d391e987982fbbd3");
    testUser.setFirstName("First");
    testUser.setLastName("Last");
    password = "test";
    testUser.setActive(true);
    testUser.setCheckNumber(4L);
  }
  @Test
  public void fetchByUserNameGood() {
    Person person = personDao.fetchByUserName(testUser.getUserName());
    assertNotNull(person);
    assertRefEquals(testUser, person);
  }

  @Test
  public void fetchByByUserNameBad() {
    Person person = personDao.fetchByUserName("not a user name");
    assertNull(person);
  }

  /* Expected data set will compare data from
   * personDaoTest.removeActivation-result.xml
   * at the end of the test.
   *
   * Everything in the xml file should be in the database
   * once the method has completed, or the test will fail
   *
   * Columns omitted in the xml file are not checked   *
   */
  @Test @ExpectedDataSet
  public void removeActivation() {
    Person person = personDao.fetchById(testUser.getId());
    person.setActive(false);
    personDao.store(person, person);
  }

  @Test
  public void authenticateGood() {
    boolean authorised = personDao.authoriseAgainstDatabase(
        testUser.getUserName(), password);
    assertTrue(authorised);
  }

  @Test
  public void authenticateBad() {
    boolean authorised = personDao.authoriseAgainstDatabase(
        testUser.getUserName(), "not the password");
    assertFalse(authorised);
  }

  @Test
  public void fetchByNameGood() {
    Person person = personDao.fetchPersonByFirstAndLastName(
        testUser.getFirstName(), testUser.getLastName());
    assertNotNull(person);
    assertRefEquals(testUser, person);
  }

  @Test
  public void fetchByNameBad() {
    Person person = personDao.fetchPersonByFirstAndLastName(
        "not a name", "still not a name");
    assertNull(person);
  }

  @Test @DataSet("personDaoTest.search.xml")
  public void searchAll() {
    PersonSearchBean psb = new PersonSearchBean();

    Collection<Person> people = personDao.fetch(psb);
    assertEquals(2, people.size());
  }

  @Test @DataSet("personDaoTest.search.xml")
  public void searchOrderAsc() {
    PersonSearchBean psb = new PersonSearchBean();
    psb.setOrderBy("person_id");
    psb.setOrderDesc(false);
    Collection<Person> people = personDao.fetch(psb);
    assertEquals(2, people.size());

    Iterator<Person> i = people.iterator();
    Person first = i.next();
    Person second = i.next();

    assertTrue(first.getId() < second.getId());
  }

  @Test @DataSet("personDaoTest.search.xml")
  public void searchOrderDesc() {
    PersonSearchBean psb = new PersonSearchBean();
    psb.setOrderBy("person_id");
    psb.setOrderDesc(true);
    Collection<Person> people = personDao.fetch(psb);
    assertEquals(2, people.size());

    Iterator<Person> i = people.iterator();
    Person first = i.next();
    Person second = i.next();

    assertTrue(first.getId() > second.getId());
  }

  @Test @DataSet("personDaoTest.search.xml")
  public void searchFindNone() {
    PersonSearchBean psb = new PersonSearchBean();

    psb.setActive(true);
    psb.setHidden(false);

    Collection<Person> people = personDao.fetch(psb);
    assertTrue(people.isEmpty());
  }

  /*
   *  Use a slightly larger data set when testing all the search methods
   *  Specify the data set for each method instead of relying on convention
   *
   */
  @Test @DataSet("personDaoTest.search.xml")
  public void searchHidden() {
    PersonSearchBean psb = new PersonSearchBean();
    psb.setHidden(true);
    Collection<Person> people = personDao.fetch(psb);
    assertEquals(1,people.size());
    Person person = people.iterator().next();
    assertEquals("active_hidden", person.getUserName());
  }

  @Test @DataSet("personDaoTest.search.xml")
  public void searchVisible() {
    PersonSearchBean psb = new PersonSearchBean();
    psb.setHidden(false);
    Collection<Person> people = personDao.fetch(psb);
    assertEquals(1,people.size());
    Person person = people.iterator().next();
    assertEquals("inactive_visible", person.getUserName());
  }

  @Test @DataSet("personDaoTest.search.xml")
  public void searchActive() {
    PersonSearchBean psb = new PersonSearchBean();
    psb.setActive(true);
    Collection<Person> people = personDao.fetch(psb);
    assertEquals(1,people.size());
    Person person = people.iterator().next();
    assertEquals("active_hidden", person.getUserName());
  }

  @Test @DataSet("personDaoTest.search.xml")
  public void searchInactive() {
    PersonSearchBean psb = new PersonSearchBean();
    psb.setActive(false);
    Collection<Person> people = personDao.fetch(psb);
    assertEquals(1,people.size());
    Person person = people.iterator().next();
    assertEquals("inactive_visible", person.getUserName());
  }





}
