/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.personnel.services;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.util.HashMap;
import java.util.Map;

import org.easymock.IAnswer;
import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectInto;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.licensing.PersonalLicence;
import uk.ac.sanger.mig.licensing.services.PersonalLicenceService;
import uk.ac.sanger.mig.login.LdapAuthenticator;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.Role;
import uk.ac.sanger.mig.personnel.Team;
import uk.ac.sanger.mig.personnel.dao.PersonDao;

public class PersonServiceImplMockTest extends UnitilsJUnit4  {
  @TestedObject private PersonServiceImpl personService;

  @Mock @InjectIntoByType private LdapAuthenticator mockLdapAuthenticator;
  @Mock @InjectIntoByType private PersonalLicenceService mockPersonalLicenceService;
  @Mock @InjectIntoByType private PersonDao mockPersonDao;
  @Mock @InjectIntoByType private RoleService mockRoleService;
  @Mock @InjectIntoByType private TeamService mockTeamService;

  @SuppressWarnings("unused")
  @InjectInto(property="ldapEnabled")
  private static final boolean ldapEnabled = true;

  private static final String userName = "testuser";
  private static final String password = "testpassword";
  private static final Long personId = 1L;
  private static final Long teamId = 2L;
  private static final Long licenceId = 3L;

  private Person testPerson;
  private Team testTeam;
  private Map<String,Role> testRoles;
  private PersonalLicence testLicence;

  @Before
  public void setUp() {
    testPerson = new Person();
    testPerson.setId(personId);
    testPerson.setTeamId(teamId);
    testPerson.setUserName(userName);
    testPerson.setActive(true);

    testTeam = new Team();
    testTeam.setId(teamId);

    testRoles = new HashMap<String,Role>();
    testRoles.put("test role",new Role());

    testLicence = new PersonalLicence();
    testLicence.setId(licenceId);
  }

  @Test
  public void testAuthenticateGoodDb() {
    expect(mockPersonDao.fetchByUserName(userName)).andReturn(testPerson);
    expect(mockLdapAuthenticator.authenticate(userName, password)).andReturn(false);
    expect(mockPersonDao.authoriseAgainstDatabase(userName, password)).andReturn(true);
    expect(mockTeamService.fetchById(teamId, null)).andReturn(testTeam);
    expect(mockRoleService.fetchByPersonId(personId, null)).andReturn(testRoles);
    expect(mockPersonalLicenceService.fetchByPersonId(personId, null)).andReturn(testLicence);

    replay();

    final Person person = personService.authenticate(userName, password);

    verify();

    assertNotNull(person);
    assertEquals(testPerson, person);
    assertEquals(testTeam, person.getTeam());
    assertEquals(testRoles, person.getRoles());
    assertEquals(testLicence, person.getPersonalLicence());
  }

  @Test
  public void testAuthenticateNoLdap() {
    personService.setLdapEnabled(false);

    expect(mockPersonDao.fetchByUserName(userName)).andReturn(testPerson);
    expect(mockPersonDao.authoriseAgainstDatabase(userName, password)).andReturn(true);
    expect(mockTeamService.fetchById(teamId, null)).andReturn(testTeam);
    expect(mockRoleService.fetchByPersonId(personId, null)).andReturn(testRoles);
    expect(mockPersonalLicenceService.fetchByPersonId(personId, null)).andReturn(testLicence);

    replay();

    final Person person = personService.authenticate(userName, password);

    verify();

    assertNotNull(person);
  }


  @Test
  public void testAuthenticateGoodLdap() {
    expect(mockPersonDao.fetchByUserName(userName)).andReturn(testPerson);
    expect(mockLdapAuthenticator.authenticate(userName, password)).andReturn(true);
    expect(mockTeamService.fetchById(teamId, null)).andReturn(testTeam);
    expect(mockRoleService.fetchByPersonId(personId, null)).andReturn(testRoles);
    expect(mockPersonalLicenceService.fetchByPersonId(personId, null)).andReturn(testLicence);

    replay();

    final Person person = personService.authenticate(userName, password);

    verify();

    assertNotNull(person);
    assertEquals(testPerson, person);
    assertEquals(testTeam, person.getTeam());
    assertEquals(testRoles, person.getRoles());
    assertEquals(testLicence, person.getPersonalLicence());
  }

  @Test
  public void testAuthenticateBadPassword() {
    expect(mockPersonDao.fetchByUserName(userName)).andReturn(testPerson);
    expect(mockLdapAuthenticator.authenticate(userName, password)).andReturn(false);
    expect(mockPersonDao.authoriseAgainstDatabase(userName, password)).andReturn(false);

    replay();

    Person person = personService.authenticate(userName, password);

    verify();

    assertNull(person);
  }

  @Test
  public void testAuthenticateBadUsername() {
    expect(mockPersonDao.fetchByUserName(userName)).andReturn(null);

    replay();

    Person person = personService.authenticate(userName, password);

    verify();

    assertNull(person);
  }

  @Test(expected=IllegalArgumentException.class)
  public void testAuthenticateNullUsername() {
    personService.authenticate(null, password);
  }

  @Test(expected=IllegalArgumentException.class)
  public void testAuthenticateNullPassword() {
    personService.authenticate(userName, null);
  }


  @Test(expected=RuntimeException.class)
  public void testMissingFromLdap() {
    expect(mockPersonDao.fetchByUserName(userName)).andReturn(testPerson);
    expect(mockLdapAuthenticator.authenticate(userName, password));
    expectLastCall().andAnswer(new IAnswer<Object>() {
      public Object answer() throws Throwable {
        throw new RuntimeException();
      }
    });
    expect(mockPersonDao.authoriseAgainstDatabase(userName, password)).andReturn(false);


    replay();

    personService.authenticate(userName, password);
    // and now it should fall over
  }

  @Test
  public void testMissingFromLdapButInDb() {
    expect(mockPersonDao.fetchByUserName(userName)).andReturn(testPerson);
    expect(mockLdapAuthenticator.authenticate(userName, password));
    expectLastCall().andAnswer(new IAnswer<Object>() {
      public Object answer() throws Throwable {
        throw new RuntimeException();
      }
    });
    expect(mockPersonDao.authoriseAgainstDatabase(userName, password)).andReturn(true);
    expect(mockTeamService.fetchById(teamId, null)).andReturn(testTeam);
    expect(mockRoleService.fetchByPersonId(personId, null)).andReturn(testRoles);
    expect(mockPersonalLicenceService.fetchByPersonId(personId, null)).andReturn(testLicence);

    replay();

    final Person person = personService.authenticate(userName, password);

    verify();

    assertNotNull(person);
  }

  @Test
  public void testMissingFromLdapButInDbWrongPassword() {
    testPerson.setPasswordHash("whatever");

    expect(mockPersonDao.fetchByUserName(userName)).andReturn(testPerson);
    expect(mockLdapAuthenticator.authenticate(userName, password));
    expectLastCall().andAnswer(new IAnswer<Object>() {
      public Object answer() throws Throwable {
        throw new RuntimeException();
      }
    });
    expect(mockPersonDao.authoriseAgainstDatabase(userName, password)).andReturn(false);

    replay();

    Person person = personService.authenticate(userName, password);

    verify();

    assertNull(person);
  }

}
