/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture;

import java.util.ArrayList;
import java.util.Collection;

import junit.framework.TestCase;
import uk.ac.sanger.mig.common.OperationNotSupported;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;

public class ObservationTypeTest extends TestCase {
	
  private Observation observation;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
    observation = new Observation();
    observation.setRequiredObservation(new RequiredObservation());
    observation.getRequiredObservation().setObservationType(new ObservationType());
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		observation = null;
	}
  
  private void setMetaType(ObservationType.MetaType metaType) {
    observation.getRequiredObservation().getObservationType().setObservationMetaTypeId(metaType.getId());
  }
  private void setMandatory(boolean mandatory) {
    observation.getRequiredObservation().setMandatoryObservation(mandatory);
  }
  private void setRange(Double min, Double max) {
    observation.getRequiredObservation().getObservationType().setRangeMin(min);
    observation.getRequiredObservation().getObservationType().setRangeMax(max);
  }
  private void setupOptions() {
    Collection<RestrictedOption>  options = new ArrayList<RestrictedOption>();    
    observation.getRequiredObservation().getObservationType().setRestrictedOptions(options);

    RestrictedOption option = new RestrictedOption();
    option.setName("Test Option");
    option.setId(1L);

    RestrictedOption secondOption = new RestrictedOption();
    secondOption.setName("Another Test Option");
    secondOption.setId(2L);

    options.add(option);
    options.add(secondOption);
  }


	public void testEmptyString() {
    setMetaType(ObservationType.MetaType.FREE_TEXT);
    setMandatory(false);
    observation.setValue("");
		assertTrue("Empty string should validate", observation.isValid());
	}
  
  public void testRequiredEmptyString() {
    setMetaType(ObservationType.MetaType.FREE_TEXT);
    setMandatory(true);
    observation.setValue("");
    assertFalse("Empty string should not validate", observation.isValid());
  }

  public void testNullString() {
    setMetaType(ObservationType.MetaType.FREE_TEXT);
    setMandatory(false);
    observation.setValue(null);
    assertTrue("Null string should validate", observation.isValid());
  }

  public void testNullRequiredString() {
    setMetaType(ObservationType.MetaType.FREE_TEXT);
    setMandatory(true);
    observation.setValue(null);
    assertFalse("Null string should not validate", observation.isValid());
  }

  public void testRequiredGoodString() {
    setMetaType(ObservationType.MetaType.FREE_TEXT);
    setMandatory(true);
    observation.setValue("This is a comment");
    assertTrue("Comment string should validate", observation.isValid());
  }

  public void testGoodString() {
    setMetaType(ObservationType.MetaType.FREE_TEXT);
    setMandatory(false);
    observation.setValue("This is a comment");
    assertTrue("Comment string should validate", observation.isValid());
  }
  
	public void testPositiveInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(false);
    observation.setValue("42");
    assertTrue("Number should validate", observation.isValid());
	}
  
  public void testNegativeInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(false);
    observation.setValue("-42");
    assertTrue("Number should validate", observation.isValid());
  }

  public void testRequiredInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(true);
    observation.setValue("42");
    assertTrue("Number should validate", observation.isValid());
  }
  
  public void testEmptyInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(false);
    observation.setValue("");
    assertTrue("Empty number should validate", observation.isValid());
  }
  
  public void testInvalidInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(false);
    observation.setValue("I am not a number");
    assertFalse("Bad number should validate", observation.isValid());
  }
  

  public void testEmptyRequiredInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(true);
    observation.setValue("");
    assertFalse("Empty number should not validate", observation.isValid());
  }
  
  public void testGoodRangedInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(true);
    setRange(41.5,42.5);
    observation.setValue("42");
    assertTrue("Number should validate", observation.isValid());
  }
  
  public void testHighRangedInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(true);
    setRange(41.5,42.5);
    observation.setValue("43");
    assertFalse("Number should not validate", observation.isValid());
  }
  
  public void testLowRangedInteger() {
    setMetaType(ObservationType.MetaType.INTEGER_NUMBER);
    setMandatory(true);
    setRange(41.5,42.5);
    observation.setValue("41");
    assertFalse("Number should not validate", observation.isValid());
  }
  
	public void testGoodOption() {
    setMetaType(ObservationType.MetaType.OPTION_LIST);
		setupOptions();
    setMandatory(false);
    observation.setValue("1");
		assertTrue("Option should validate", observation.isValid());
	}
  
  public void testGoodRadioButton() {
    setMetaType(ObservationType.MetaType.RADIO_BUTTON);
    setupOptions();
    setMandatory(true);
    observation.setValue("1");
    assertTrue("Option should validate", observation.isValid());
  }
  
  public void testBadOption() {
    setMetaType(ObservationType.MetaType.OPTION_LIST);
    setupOptions();
    observation.setValue("-1");
    assertFalse("Option should not validate", observation.isValid());
  }

  public void testNullOption() {
    setMetaType(ObservationType.MetaType.OPTION_LIST);
    setupOptions();
    setMandatory(false);
    observation.setValue(null);
    assertTrue("Option should validate", observation.isValid());
  }
  
  public void testNullRequiredOption() {
    setMetaType(ObservationType.MetaType.OPTION_LIST);
    setupOptions();
    setMandatory(true);
    observation.setValue(null);
    assertFalse("Option should not validate", observation.isValid());
  }

  public void testNotNumberOption() {
    setMetaType(ObservationType.MetaType.OPTION_LIST);
    setupOptions();
    setMandatory(true);
    observation.setValue("This makes no sense");
    assertFalse("Option should not validate", observation.isValid());
  }

  public void testPositiveFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(false);
    observation.setValue("4.2");
    assertTrue("Number should validate", observation.isValid());
  }
  
  public void testInvalidFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(false);
    observation.setValue("I am still not a number");
    assertFalse("Number should validate", observation.isValid());
  }
  
  public void testNegativeFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(false);
    observation.setValue("-4.2");
    assertTrue("Number should validate", observation.isValid());
  }

  public void testRequiredFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(true);
    observation.setValue("4.2");
    assertTrue("Number should validate", observation.isValid());
  }
  
  public void testEmptyFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(false);
    observation.setValue("");
    assertTrue("Empty number should validate", observation.isValid());
  }

  public void testEmptyRequiredFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(true);
    observation.setValue("");
    assertFalse("Empty number should not validate", observation.isValid());
  }
  
  public void testGoodRangedFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(true);
    setRange(4.0,5.0);
    observation.setValue("4.2");
    assertTrue("Number should validate", observation.isValid());
  }
  
  public void testLowRangedFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(true);
    setRange(4.1,4.2);
    observation.setValue("4");
    assertFalse("Number should not validate", observation.isValid());
  }


  public void testHighRangedFloat() {
    setMetaType(ObservationType.MetaType.FLOATING_POINT_NUMBER);
    setMandatory(true);
    setRange(4.1,4.2);
    observation.setValue("4.3");
    assertFalse("Number should not validate", observation.isValid());
  }
  
  public void testGoodDate() {
    setMetaType(ObservationType.MetaType.DATE);
    setMandatory(true);
    observation.setValue("01-Jan-2007");
    assertTrue("Date should validate", observation.isValid());
  }
  
  public void testBadDate() {
    setMetaType(ObservationType.MetaType.DATE);
    setMandatory(true);
    observation.setValue("01 Jan 2007");
    assertFalse("Date should not validate", observation.isValid());
  }
  
  public void testGoodClock() {
    setMetaType(ObservationType.MetaType.CLOCK_TIME);
    setMandatory(true);
    observation.setValue("13:13");
    assertTrue("Clock time should validate", observation.isValid());
  }
  
  public void testBadClock() {
    setMetaType(ObservationType.MetaType.CLOCK_TIME);
    setMandatory(true);
    observation.setValue("13 13");
    assertFalse("Clock time should not validate", observation.isValid());
  }
  
  public void testGoodDateTime() {
    setMetaType(ObservationType.MetaType.DATE_TIME);
    setMandatory(true);
    observation.setValue("01-Jan-2007 13:13");
    assertTrue("Date & time should validate", observation.isValid());
  }
  
  public void testBadDateTime() {
    setMetaType(ObservationType.MetaType.DATE_TIME);
    setMandatory(true);
    observation.setValue("01 Jan 2007");
    assertFalse("Date & time should not validate", observation.isValid());
  }
  
  public void testGoodBloodSampleBarcode() {
    setMetaType(ObservationType.MetaType.EDTA_BLOOD_SAMPLE_BARCODE);
    setMandatory(true);
    observation.setValue("00000491");
    assertTrue("Barcode should validate", observation.isValid());
  }
  
  public void testBadLengthBloodSampleBarcode() {
    setMetaType(ObservationType.MetaType.EDTA_BLOOD_SAMPLE_BARCODE);
    setMandatory(true);
    observation.setValue("blah");
    assertFalse("Barcode should not validate", observation.isValid());
  }
  
  public void testMalformedBloodSampleBarcode() {
    setMetaType(ObservationType.MetaType.EDTA_BLOOD_SAMPLE_BARCODE);
    setMandatory(true);
    observation.setValue("0000049l"); // last char is not a number
    assertFalse("Barcode should not validate", observation.isValid());
  }

  
	public void testUnsupportedMetaType() {
		observation.getRequiredObservation().getObservationType().setObservationMetaTypeId(Long.MIN_VALUE);
		observation.setValue("a value");
    
		Exception error = null;
		try {
      observation.isValid();
		} catch (OperationNotSupported e) {
			error = e;
		}

		assertTrue("OperationNotSupported should be thrown",
				(error != null && error.getClass().equals(
						OperationNotSupported.class)));
	}

}
