/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.springframework.util.StopWatch;

import uk.ac.sanger.mig.phenotyping.datacapture.Experiment;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSet;
import uk.ac.sanger.mig.phenotyping.datacapture.ExperimentSetSearchBean;
import uk.ac.sanger.mig.phenotyping.datacapture.Observation;
import uk.ac.sanger.mig.phenotyping.datacapture.views.ExperimentSetListView;
import uk.ac.sanger.mig.testsupport.AbstractGenericServiceTest;

public class ExperimentSetServiceTest extends AbstractGenericServiceTest {
  public void setExperimentSetService(ExperimentSetService service) {
    this.service = service;
  }

  private ExperimentSetService getExperimentSetService() {
    return (ExperimentSetService)service;
  }

  public void testDelete() {
    Collection<ExperimentSetListView> viewList = getView();

    boolean triedDeletableSet = false;
    boolean triedNonDeleteableSet = false;

    for (ExperimentSetListView view : viewList) {

      if (!triedDeletableSet && view.isDeletable()) {
        triedDeletableSet = true;
        assertTrue(
            "Should be able to delete experiment set",
            getExperimentSetService().fullDelete(view.getId(), getFakeUser())
        );
      }

      if (!triedNonDeleteableSet && !view.isDeletable()) {
        triedNonDeleteableSet = true;
        assertFalse(
            "Should not be able to delete experiment set",
            getExperimentSetService().fullDelete(view.getId(), getFakeUser())
        );
      }

      if (triedDeletableSet && triedNonDeleteableSet) {
        break;
      }

    }

    if (!triedDeletableSet || !triedNonDeleteableSet) {
      fail("Could not fully test experiment set deletion.");
    }

  }

  private Collection<ExperimentSetListView> getView() {
    ExperimentSetSearchBean esb = new ExperimentSetSearchBean();
    esb.setPaginated(false);
    Collection<ExperimentSetListView> viewList = getExperimentSetService().search(esb);
    return viewList;
  }

  public void testPhenoFetch() {
    Collection<ExperimentSetListView> viewList = getView();

    Map<String, Long> sample = new HashMap<String,Long>();
    for (ExperimentSetListView esv : viewList) {
      sample.put(esv.getSopName(), esv.getId());
    }

    StopWatch stopWatch = new StopWatch();

    for (Map.Entry<String, Long> entry : sample.entrySet()) {
      stopWatch.start(entry.getKey());
      ExperimentSet es = getExperimentSetService().fetchForPhenotyping(entry.getValue());
      stopWatch.stop();

      double time = (double)stopWatch.getLastTaskTimeMillis()/(double)1000;

      int totalObsReads = 0;

      int experimentCount = es.getExperiments().size();
      if (experimentCount > 0) {
        int observationsPerExperiment = es.getExperiments().iterator().next().getObservations().size();
        totalObsReads = experimentCount * observationsPerExperiment;
      }

      double timePerObs = time/totalObsReads;
      assertTrue("Maximim acceptable load time per observation read is 0.05, this was "+timePerObs, (timePerObs < 0.05) );

      assertNotNull("Should have retrieved ExperimentSet", es);
      assertNotNull("Experiments should be attached", es.getExperiments());
      assertFalse("Should have some experiments", es.getExperiments().isEmpty());

      for (Experiment e : es.getExperiments()) {
        assertNotNull("Observations should be attached",e.getObservations());
        assertFalse("Should have some observations",e.getObservations().isEmpty());

        for (Observation o : e.getObservations()) {
          assertNotNull("Need required observation", o.getRequiredObservation());
          assertNotNull("Need observation type", o.getRequiredObservation().getObservationType());

          if (o.getRequiredObservation().getObservationType().isRestrictedOption()) {
            assertNotNull("Options should be attached", o.getRequiredObservation().getObservationType().getRestrictedOptions());
            assertFalse("Should have some options", o.getRequiredObservation().getObservationType().getRestrictedOptions().isEmpty());
          }
        }

      }
    }

    double averageTime = stopWatch.getTotalTimeSeconds() / (double)stopWatch.getTaskCount();
    // this method was created for performance reasons, so check that performance is acceptable
    assertTrue("Average load time should be below 1s", (averageTime < 1));




  }



}
