/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.services;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;
import static org.easymock.EasyMock.expect;

import static org.junit.Assert.*;
import static org.unitils.easymock.EasyMockUnitils.*;
import static org.unitils.reflectionassert.ReflectionAssert.*;

import uk.ac.sanger.mig.mousetracking.Cohort;
import uk.ac.sanger.mig.mousetracking.services.CohortService;


/**
 *
 */
public class UploadMappingServiceCohortDivisionTest extends UnitilsJUnit4 {
    @TestedObject UploadMappingServiceImpl uploadMappingService;
    @Mock @InjectIntoByType CohortService mockCohortService;

    private Long[] mice = {100L, 200L, 300L};
    private Long[] cohortIds = {1L,2L,3L};

    private List<Long> oneMouse, twoMice, threeMice;
    private List<Cohort> oneCohort, twoCohorts, threeCohorts;

    @Before
    public void setUp() {
      threeMice = Arrays.asList(mice);;
      twoMice = threeMice.subList(0, 2);
      oneMouse = threeMice.subList(0, 1);

      threeCohorts = new Stack<Cohort>();
      for(Long id : cohortIds) {
        Cohort c = new Cohort();
        threeCohorts.add(c);
        c.setId(id);
        c.setCohortTypeDictId(CohortService.COHORT_TYPE_PHENOTYPING);
      }
      twoCohorts = threeCohorts.subList(0, 2);
      oneCohort = threeCohorts.subList(0, 1);
    }

    @Test(expected=IllegalArgumentException.class)
    public void oneMouseNoCohort() {
      expect(mockCohortService.fetchByMouseId(mice[0], null)).andReturn(new LinkedList<Cohort>());
      replay();
      uploadMappingService.groupMiceByCohorts(oneMouse);
    }

    @Test
    public void oneMouseOneCohort() {
      expect(mockCohortService.fetchByMouseId(mice[0], null)).andReturn(oneCohort);

      replay();
      Map<Cohort, Collection<Long>> grouped = uploadMappingService.groupMiceByCohorts(oneMouse);
      verify();

      Map<Cohort, Collection<Long>> expected = new HashMap<Cohort, Collection<Long>>();
      expected.put(oneCohort.get(0),oneMouse);

      assertNotNull(grouped);
      assertRefEquals(expected,grouped);
    }

    @Test
    public void twoMiceOneCohort() {
      expect(mockCohortService.fetchByMouseId(mice[0], null)).andReturn(oneCohort);
      expect(mockCohortService.fetchByMouseId(mice[1], null)).andReturn(oneCohort);

      replay();
      Map<Cohort, Collection<Long>> grouped = uploadMappingService.groupMiceByCohorts(twoMice);
      verify();

      Map<Cohort, Collection<Long>> expected = new HashMap<Cohort, Collection<Long>>();
      expected.put(oneCohort.get(0),twoMice);

      assertNotNull(grouped);
      assertRefEquals(expected,grouped);
    }

    @Test
    public void twoMiceTwoCohortsOneRequired() {
      expect(mockCohortService.fetchByMouseId(mice[0], null)).andReturn(oneCohort);
      expect(mockCohortService.fetchByMouseId(mice[1], null)).andReturn(twoCohorts);

      replay();
      Map<Cohort, Collection<Long>> grouped = uploadMappingService.groupMiceByCohorts(twoMice);
      verify();

      Map<Cohort, Collection<Long>> expected = new HashMap<Cohort, Collection<Long>>();
      expected.put(oneCohort.get(0),twoMice);

      assertNotNull(grouped);
      assertRefEquals(expected,grouped);
    }

    @Test
    public void twoMiceTwoCohorts() {
      List<Cohort> anotherCohort = threeCohorts.subList(1, 2);
      Collection<Long> anotherMouse = threeMice.subList(1, 2);

      expect(mockCohortService.fetchByMouseId(mice[0], null)).andReturn(oneCohort);
      expect(mockCohortService.fetchByMouseId(mice[1], null)).andReturn(anotherCohort);

      replay();
      Map<Cohort, Collection<Long>> grouped = uploadMappingService.groupMiceByCohorts(twoMice);
      verify();

      Map<Cohort, Collection<Long>> expected = new HashMap<Cohort, Collection<Long>>();
      expected.put(oneCohort.get(0),oneMouse);
      expected.put(anotherCohort.get(0),anotherMouse);

      assertNotNull(grouped);
      assertRefEquals(expected,grouped);
    }
    @Test
    public void threeMiceThreeCohortsOneRequired() {
      expect(mockCohortService.fetchByMouseId(mice[0], null)).andReturn(oneCohort);
      expect(mockCohortService.fetchByMouseId(mice[1], null)).andReturn(twoCohorts);
      expect(mockCohortService.fetchByMouseId(mice[2], null)).andReturn(threeCohorts);

      replay();
      Map<Cohort, Collection<Long>> grouped = uploadMappingService.groupMiceByCohorts(threeMice);
      verify();

      Map<Cohort, Collection<Long>> expected = new HashMap<Cohort, Collection<Long>>();
      expected.put(oneCohort.get(0),threeMice);

      assertNotNull(grouped);
      assertRefEquals(expected,grouped);
    }
    @Test
    public void threeMiceTwoCohortsRequired() {
      expect(mockCohortService.fetchByMouseId(mice[0], null)).andReturn(oneCohort);
      expect(mockCohortService.fetchByMouseId(mice[1], null)).andReturn(twoCohorts);
      expect(mockCohortService.fetchByMouseId(mice[2], null)).andReturn(threeCohorts.subList(2, 3));

      replay();
      Map<Cohort, Collection<Long>> grouped = uploadMappingService.groupMiceByCohorts(threeMice);
      verify();

      Map<Cohort, Collection<Long>> expected = new HashMap<Cohort, Collection<Long>>();
      expected.put(oneCohort.get(0),twoMice);
      expected.put(threeCohorts.get(2),threeMice.subList(2, 3));

      assertNotNull(grouped);
      assertRefEquals(expected,grouped);
    }





}
