/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.phenotyping.datacapture.validators;

import static org.easymock.EasyMock.expect;
import static org.junit.Assert.fail;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.text.ParseException;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;

import org.junit.Before;
import org.junit.Test;
import org.springframework.validation.Errors;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.filesupport.BinaryFile;
import uk.ac.sanger.mig.licensing.ProjectLicence;
import uk.ac.sanger.mig.licensing.services.ProjectLicenceService;
import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.mousetracking.services.CohortService;
import uk.ac.sanger.mig.mousetracking.services.MouseService;
import uk.ac.sanger.mig.mousetracking.views.CohortListView;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.BoxData;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.CalorimetryData;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.MalformedCalorimetryFileException;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.Parser;
import uk.ac.sanger.mig.phenotyping.calorimetry.parser.ParserTestUtil;
import uk.ac.sanger.mig.phenotyping.datacapture.StdOperatingProcedure;
import uk.ac.sanger.mig.phenotyping.datacapture.forms.CalorimetryUploadForm;
import uk.ac.sanger.mig.phenotyping.datacapture.services.UploadMappingService;


public class CalorimetryUploadFormValidatorTest extends UnitilsJUnit4{

  @TestedObject private CalorimetryUploadFormValidator validator;
  @Mock private Errors mockErrors;
  @Mock private CalorimetryUploadForm mockForm;
  @Mock @InjectIntoByType private MouseService mockMouseService;
  @Mock @InjectIntoByType private ProjectLicenceService mockProjectLicenceService;
  @Mock @InjectIntoByType private CohortService mockCohortService;
  @Mock @InjectIntoByType private UploadMappingService mockUploadMappingService;

  private CalorimetryData data;
  private Mouse mouse;
  private CohortListView cohort;
  private Collection<CohortListView> cohorts;
  private StdOperatingProcedure sop;

  private final Long animalNumber = 101L;
  private final Long cohortId = 72L;
  private final Long pilId = 20L;
  private final Long pplId = 10L;
  private final Long procedureId = 15L;
  private final Long sopId = 42L;

  private static final String uploadFile = "uploadFile";

  @Before
  public void setUp() {
    data = new CalorimetryData();
    data.setBoxes(new LinkedList<BoxData>());

    BoxData box = new BoxData();
    box.setAnimalNumber(animalNumber);
    data.getBoxes().add(box);


    mouse = new Mouse();
    mouse.setBirthDate(new Date());
    mouse.setId(animalNumber);

    cohort = new CohortListView();
    cohort.setId(cohortId);

    cohorts = new LinkedList<CohortListView>();
    cohorts.add(cohort);

    sop = new StdOperatingProcedure();
    sop.setProcedureId(procedureId);
    sop.setId(sopId);
  }



  @Test
  public void validateMiceGood() {
    expect(mockForm.getCalorimetryData()).andReturn(data);
    expect(mockMouseService.fetchById(animalNumber, null)).andReturn(mouse);
    expect(mockCohortService.search(null)).andReturn(cohorts);

    replay();

    validator.validateMice(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateMiceMissing() {
    expect(mockForm.getCalorimetryData()).andReturn(data);
    expect(mockMouseService.fetchById(animalNumber, null)).andReturn(null);

    // must reject the file, we cannot find the mouse
    mockErrors.rejectValue("uploadFile", null, null);

    replay();

    validator.validateMice(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateMiceNoCohort() {
    expect(mockForm.getCalorimetryData()).andReturn(data);
    expect(mockMouseService.fetchById(animalNumber, null)).andReturn(mouse);
    expect(mockCohortService.search(null)).andReturn(new LinkedList<CohortListView>());


    // must reject the file, mouse isn't in a cohort
    mockErrors.rejectValue("uploadFile", null, null);

    replay();

    validator.validateMice(mockForm, mockErrors);

    verify();
  }

  @Test
  /**
   * At one point, a group of mice that weren't in the same cohort
   * would have been rejected, but this is now permitted
   */
  public void validateMiceMultipleCohorts() {
    Long secondMouseId = 102L;
    BoxData box = new BoxData();
    box.setAnimalNumber(secondMouseId.intValue());
    data.getBoxes().add(box);
    Mouse secondMouse = new Mouse();
    secondMouse.setId(secondMouseId);

    expect(mockForm.getCalorimetryData()).andReturn(data);
    expect(mockCohortService.search(null)).andReturn(cohorts).times(2);
    expect(mockMouseService.fetchById(animalNumber, null)).andReturn(secondMouse);
    expect(mockMouseService.fetchById(secondMouseId, null)).andReturn(secondMouse);

    // should not give an error

    replay();

    validator.validateMice(mockForm, mockErrors);

    verify();
  }


  @Test
  public void validateFileMissing() {
    expect(mockForm.getBinaryFile()).andReturn(null);
    //must reject the file
    mockErrors.rejectValue("uploadFile", null,null);

    replay();

    validator.validateFile(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateFileBad() {
    //setup an empty file
    BinaryFile binaryFile = new BinaryFile();
    binaryFile.setBytes(new byte[10]);

    expect(mockForm.getBinaryFile()).andReturn(binaryFile).anyTimes();

    //  must reject the file
    mockErrors.rejectValue("uploadFile", null,null);

    replay();

    validator.validateFile(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateLicenceGood() {
    Collection<ProjectLicence> ppls = new LinkedList<ProjectLicence>();
    ProjectLicence ppl = new ProjectLicence();
    ppl.setId(pplId);
    ppls.add(ppl);

    expect(mockUploadMappingService.fetchActiveSopForUploadType(UploadMappingService.Type.MGP_CALORIMETRY, null)).andReturn(sop);
    expect(mockForm.getPplId()).andReturn(pplId);
    expect(mockForm.getPilId()).andReturn(pilId);
    mockForm.setSopId(sopId);

    expect(
        mockProjectLicenceService.fetchByPersonalLicenceAndProcedureId(
            pilId, procedureId, null)
    ).andReturn(ppls);

    // should not call an error

    replay();

    validator.validateLicence(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateLicenceNoPpls() {
    Collection<ProjectLicence> ppls = Collections.emptyList();

    expect(mockUploadMappingService.fetchActiveSopForUploadType(UploadMappingService.Type.MGP_CALORIMETRY, null)).andReturn(sop);
    expect(mockForm.getPplId()).andReturn(pplId);
    expect(mockForm.getPilId()).andReturn(pilId);
    mockForm.setSopId(sopId);
    expect(
        mockProjectLicenceService.fetchByPersonalLicenceAndProcedureId(
            pilId, procedureId, null)
    ).andReturn(ppls);

    // should call an error
    mockErrors.reject("pilId",null,null);

    replay();

    validator.validateLicence(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateLicenceWrongPpl() {
    Collection<ProjectLicence> ppls = new LinkedList<ProjectLicence>();
    ProjectLicence ppl = new ProjectLicence();
    ppl.setId(pplId+1);
    ppls.add(ppl);

    expect(mockUploadMappingService.fetchActiveSopForUploadType(UploadMappingService.Type.MGP_CALORIMETRY, null)).andReturn(sop);
    expect(mockForm.getPplId()).andReturn(pplId);
    expect(mockForm.getPilId()).andReturn(pilId);
    mockForm.setSopId(sopId);

    expect(
        mockProjectLicenceService.fetchByPersonalLicenceAndProcedureId(
            pilId, 15L, null)
    ).andReturn(ppls);

    // should call an error
    mockErrors.reject("pplId",null,null);

    replay();

    validator.validateLicence(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateEmpty() {
    expect(mockErrors.getFieldValue("pplId")).andReturn(null);
    expect(mockErrors.getFieldValue("pilId")).andReturn(null);
    expect(mockErrors.getFieldValue(uploadFile)).andReturn(null);


    mockErrors.rejectValue("pplId", null, null, null);
    mockErrors.rejectValue("pilId", null, null, null);
    mockErrors.rejectValue("uploadFile", null, null, null);

    expect(mockForm.getUploadFile()).andReturn(null).anyTimes();
    expect(mockForm.getCalorimetryData()).andReturn(null).anyTimes();
    expect(mockForm.getPilId()).andReturn(null).anyTimes();
    expect(mockForm.getPplId()).andReturn(null).anyTimes();


    replay();

    validator.validate(mockForm, mockErrors);

    verify();
  }

  @Test
  public void validateFileGood() {
    Parser parser = new Parser();

    byte[] testData = ParserTestUtil.getTestData();
    CalorimetryData data = null;
    try {
      parser.loadFromBytes(testData);
    }
    catch (MalformedCalorimetryFileException e) {
      fail("Threw MalformedCalorimetryFileException on valid file");
    }
    catch (ParseException e) {
      fail("Threw ParseException on valid file");
    }

    BinaryFile binaryFile = new BinaryFile();
    binaryFile.setBytes(testData);

    expect(mockForm.getBinaryFile()).andReturn(binaryFile).atLeastOnce();
    mockForm.setCalorimetryData(data);

    replay();

    validator.validateFile(mockForm, mockErrors);

    verify();
  }




}
