package uk.ac.sanger.mig.reservations.dao;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.LinkedList;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.dbunit.annotation.DataSet;
import org.unitils.spring.annotation.SpringApplicationContext;
import org.unitils.spring.annotation.SpringBean;

import uk.ac.sanger.mig.reservations.MouseReservation;

@SpringApplicationContext( { "reservations-dao.xml", "common.xml", "test-config.xml", "unitils-test-database.xml" })
@DataSet
public class MouseReservationDaoSpringImplTest extends UnitilsJUnit4 {

	@SpringBean("mouseReservationDao")
	private MouseReservationDao	mouseReservationDao;
	private Long				reservationId;
	private Long				nullReservationId;
	private Long				mouseId;
	private Long				nullMouseId;


	@Before
	public void setUp() {
		reservationId = 1L;
		nullReservationId = 100L;
		mouseId = 1L;
		nullMouseId = 100L;
	}


	/**
	 * Tests that the fetchByReservation() method returns a collection and not null. Also tests that the collection is
	 * not of zero length and that all of the mouse reservations are associated with the test reservation id.
	 *
	 */
	@Test
	public void fetchByReservation() {
		Collection<MouseReservation> mouseReservations = mouseReservationDao.fetchByReservation(reservationId);
		assertNotNull("fetch by reservation id returned a non null collection", mouseReservations);
		assertEquals(3L, mouseReservations.size());
		boolean areFromSameReservation = true;
		for (MouseReservation mr : mouseReservations) {
			if (mr.getReservationId() != reservationId) {
				areFromSameReservation = false;
			}
		}
		assertTrue("mouse reservations are from the same reservation", areFromSameReservation);
	}


	/**
	 * Tests that the fetchByReservation() method returns an empty collection and not null when an unknown reservation
	 * id is passed to it.
	 *
	 */
	@Test
	public void fetchByReservationNull() {
		Collection<MouseReservation> mouseReservations = mouseReservationDao.fetchByReservation(nullReservationId);
		assertNotNull("fetch by reservation id returned a non null collection", mouseReservations);
		assertEquals(0L, mouseReservations.size());
	}


	/**
	 * Tests that the fetchForMouse() method returns a single MouseReservation and that its mouse id matches the test
	 * mouse id.
	 *
	 */
	@Test
	public void fetchForMouse() {
		MouseReservation mouseReservation = mouseReservationDao.fetchForMouse(mouseId);
		assertNotNull(mouseReservation);
		assertEquals(mouseId, mouseReservation.getMouseId());
	}


	/**
	 * Tests that the fetchForMouse() method returns a null MouseReservation when an unknown reservation id is passed to
	 * it.
	 *
	 */
	@Test
	public void fetchForMouseNull() {
		MouseReservation mouseReservation = mouseReservationDao.fetchForMouse(nullMouseId);
		assertNull(mouseReservation);
	}


	/**
	 * Tests that the fetchForDeathNotification() method finds only two mouse reservations as specified in the test data
	 * set. The data set also contains a mouse reservations that has already been notified and one that is associated
	 * with a terminal mouse procedure - both of these should always be ignored by this method.
	 *
	 */
	@Test
	@DataSet("MouseReservationDaoSpringImplTest.fetchForDeathNotification.xml")
	public void fetchForDeathNotification() {
		LinkedList<MouseReservation> mouseReservations = new LinkedList<MouseReservation>();
		mouseReservations.addAll(mouseReservationDao.fetchForDeathNotification());
		assertNotNull(mouseReservations);
		assertEquals(2, mouseReservations.size());
		MouseReservation m1 = (MouseReservation) mouseReservations.get(0);
		MouseReservation m2 = (MouseReservation) mouseReservations.get(1);
		assertEquals(m1.getMouseId().longValue(), 1l);
		assertEquals(m2.getMouseId().longValue(), 2l);
	}

}
