package uk.ac.sanger.mig.reservations.services;

import static org.easymock.EasyMock.expect;
import static org.unitils.easymock.EasyMockUnitils.replay;
import static org.unitils.easymock.EasyMockUnitils.verify;

import java.util.LinkedList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.easymock.annotation.Mock;
import org.unitils.inject.annotation.InjectIntoByType;
import org.unitils.inject.annotation.TestedObject;

import uk.ac.sanger.mig.mousetracking.Mouse;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.reservations.MouseReservation;
import uk.ac.sanger.mig.reservations.Reservation;
import uk.ac.sanger.mig.reservations.dao.ReservationDao;
import uk.ac.sanger.mig.reservations.forms.ReservationForm;

public class ReservationServiceImplTest extends UnitilsJUnit4 {

	@TestedObject
	private ReservationServiceImpl	reservationService;
	@Mock
	@InjectIntoByType
	private ReservationDao			mockReservationDao;
	@Mock
	@InjectIntoByType
	private MouseReservationService	mockMouseReservationService;
	private Person					person;
	private Mouse					mouse;
	private List<Mouse>				mice;
	private Reservation				reservation;
	private ReservationForm			reservationForm;
	private Long					mouseId	= 101L;
	private Long					resId	= 10L;
	private MouseReservation		mouseReservation;


	@Before
	public void setUp() {
		person = new Person();
		mouse = new Mouse();
		mouse.setId(mouseId);
		mouse.setMouseReservation(new MouseReservation());
		mouse.getMouseReservation().setMouseId(mouseId);
		mice = new LinkedList<Mouse>();
		mice.add(mouse);
		reservationForm = new ReservationForm();
		reservationForm.setReservation(new Reservation());
		reservationForm.getReservation().setId(resId);
		mouseReservation = new MouseReservation();
		mouseReservation.setMouseId(mouse.getId());
	}


	/**
	 * Tests the reserveMouse() method by passing it a single mouse that has a current reservation and has been
	 * selected. The mouses current reservation has the same id as the reservation being stored so it should be ignored.
	 */
	@Test
	public void reserveOneMouseThatIsPartOfThisReservation() {
		mouse.getMouseReservation().setReservationId(resId);
		reservationForm.setMouseList(mice);
		reservationForm.setSelectedMouseIds(initSelectedMiceAllChecked(mice));
		expect(mockReservationDao.store(reservation, person)).andReturn(resId);
		replay();
		reservationService.reserveMice(reservationForm.getReservation(), reservationForm, person);
		verify();
	}


	/**
	 * Tests the reserveMouse() method by passing it a single mouse that has no current reservation and has been
	 * selected. In this instance the reservation should be stored and the mouse reservation should be deleted.
	 */
	@Test
	public void reserveOneMouseThatIsPartOfThisReservationButNotSelected() {
		mouse.getMouseReservation().setReservationId(resId);
		reservationForm.setMouseList(mice);
		reservationForm.setSelectedMouseIds(initSelectedMiceOneUnselected(mice, mouseId));
		expect(mockReservationDao.store(reservation, person)).andReturn(resId);
		mockMouseReservationService.delete(mouseReservation, person);
		replay();
		reservationService.reserveMice(reservationForm.getReservation(), reservationForm, person);
		verify();
	}


	/**
	 * Tests the reserveMice() method with a single mouse that already has a mouse reservation. The old reservation
	 * should be deleted before the new one is added.
	 *
	 */
	@Test
	public void reserveOneMouseThatAlreadyHasCurrentReservation() {
		mouse.getMouseReservation().setReservationId(1L);
		reservationForm.setMouseList(mice);
		reservationForm.setSelectedMouseIds(initSelectedMiceAllChecked(mice));
		expect(mockReservationDao.store(reservation, person)).andReturn(resId);
		mockMouseReservationService.delete(mouse.getMouseReservation(), person);
		expect(mockMouseReservationService.store(mouseReservation, person)).andReturn(1L);
		replay();
		reservationService.reserveMice(reservationForm.getReservation(), reservationForm, person);
		verify();
	}


	/**
	 * Tests the reserveMice() method with a single mouse that already has a mouse reservation. The old reservation
	 * should be deleted before the new one is added.
	 *
	 */
	@Test
	public void reserveOneMouseThatAlreadyMemberCurrentReservationNotSelected() {
		mouse.getMouseReservation().setReservationId(resId);
		reservationForm.setMouseList(mice);
		reservationForm.setSelectedMouseIds(initSelectedMiceOneUnselected(mice, mouseId));
		expect(mockReservationDao.store(reservation, person)).andReturn(resId);
		mockMouseReservationService.delete(mouse.getMouseReservation(), person);
		replay();
		reservationService.reserveMice(reservationForm.getReservation(), reservationForm, person);
		verify();
	}


	/**
	 * Initialises a list of mouse ids mimicking what the form will be passed by the jsp on submission. All of these
	 * mice will mimick being selected.
	 *
	 * @param mice -
	 *            list of Mouse objects representing the mice selected for a reservation to be added to them.
	 * @return mouseIds - list of mice ids to set into the forms selectedMouseIds attribute.
	 */
	private List<Long> initSelectedMiceAllChecked(List<Mouse> mice) {
		List<Long> mouseIds = new LinkedList<Long>();
		for (Mouse m : mice) {
			mouseIds.add(m.getId());
		}
		return mouseIds;
	}


	/**
	 * Initialises a list of mouse ids mimicking what the form will be passed by the jsp on submission. If mouseId is
	 * found then it will be left unchecked.
	 *
	 * @param mice -
	 *            list of Mouse objects.
	 * @param mouseId
	 * @return
	 */
	private List<Long> initSelectedMiceOneUnselected(List<Mouse> mice, Long mouseId) {
		List<Long> mouseIds = new LinkedList<Long>();
		for (Mouse m : mice) {
			mouseIds.add(m.getId() == mouseId ? null : m.getId());
		}
		return mouseIds;
	}
}
