/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.testsupport;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.test.AbstractTransactionalSpringContextTests;


import uk.ac.sanger.mig.charting.ChartData;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.common.DataObject;
import uk.ac.sanger.mig.common.PaginatedSearchBean;
import uk.ac.sanger.mig.common.SortableSearchBean;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public abstract class AbstractGenericServiceTest extends AbstractTransactionalSpringContextTests {

  private PersonService personService;

  private int maxNumberOfObjectsToRetrieve = 100;
  private int maxNumerOfObjectsToStore = 1;



  protected Object service;
  protected Log logger = LogFactory.getLog(this.getClass());

  /**
   * To be overridden where fetch methods should be removed from automatic testing
   *    *
   * @param method
   * @return
   */
  protected boolean testFetchMethod(Method method){
    return Arrays.asList(service.getClass().getMethods()).contains(method);
  }

  public void testSearchRetrievalAndStore() {
    logger.info("Testing service "+service.getClass().getName());

    Collection<Object> searchResults = attemptSearch();
    Collection<Object> retrieved = attemptRetrieval(searchResults);

    if (!searchResults.isEmpty()) {
      logger.info("Search successful");
    }

    if (!retrieved.isEmpty()) {
      logger.info("Retrieval successful");
    }

    Collection<Object> stored = attemptStore(retrieved);

    if (!stored.isEmpty()) {
      logger.info("Store successful");
    }
  }

  @SuppressWarnings("unchecked")
  public void testFetch() {
    Map<Method, Object[]> fetchMethods = new HashMap<Method, Object[]>();

    for (Method m : service.getClass().getMethods()) {
      Class[] args = m.getParameterTypes();

      boolean isPublic = Modifier.isPublic(m.getModifiers());
      boolean isFetch = m.getName().startsWith("fetch");

      if (isPublic && isFetch && testFetchMethod(m)) {
        boolean testable = true;
        Object[] params = new Object[args.length];

        for(int i = 0; i < args.length; i++) {

          if (args[i].equals(Long.class)) {
            params[i] = Long.valueOf(-1);
          }
          else if (args[i].equals(String.class)) {
            params[i] = "";
          }
          else if (args[i].equals(ComponentMap.class)) {
            params[i] = new ComponentMap();
          }
          else {
            testable = false;
            break;
          }

        }

        if (testable) {
          fetchMethods.put(m,params);
        }
        else {
          logger.warn("Could not test fetch method "+m.getName());
        }

      }
    }

    if (fetchMethods.isEmpty()) {
      logger.warn("No usable fetch* methods found for testing");
    }

    for (Map.Entry<Method, Object[]> entry : fetchMethods.entrySet()) {
      Method fetchMethod = entry.getKey();
      Object[] params = entry.getValue();
      boolean acceptableOutcome = false;
      Object fetchResult = null;

      assertFalse("fetch methods should not return void", Void.TYPE == fetchMethod.getReturnType());

      boolean probableFetchAll = (params.length < 1  || (params.length == 1 && params[0].getClass().equals(ComponentMap.class)));

      // dictionary services have fetch for edit, this will normally return a non-empty collection
      boolean fetchForEdit = fetchMethod.getName().contains("fetchDictionaryEntryForEdit") || fetchMethod.getName().contains("fetchForEdit");


      try {
        fetchResult = fetchMethod.invoke(service, params);


        if (fetchResult == null) {
          acceptableOutcome = true;
        }
        else if (Collection.class.isAssignableFrom(fetchMethod.getReturnType())){
          acceptableOutcome = (
             ((Collection)fetchResult).isEmpty() ||
             probableFetchAll ||
             fetchForEdit
          );
        }
        else if (Map.class.isAssignableFrom(fetchMethod.getReturnType())) {
          acceptableOutcome = (
              ((Map)fetchResult).isEmpty() || probableFetchAll
           );
        }
        else if (fetchMethod.getReturnType().isArray()) {
          acceptableOutcome = (
            ((Object[])fetchResult).length < 1 || probableFetchAll
          );
        }
        else if (ChartData.class.isAssignableFrom(fetchMethod.getReturnType())) {
          acceptableOutcome = true;
        }
        else if(fetchMethod.getReturnType().isPrimitive()){
        	acceptableOutcome = true;
        }


      } catch (IllegalArgumentException e) {
        reflectionError(e);
      } catch (IllegalAccessException e) {
        reflectionError(e);
      } catch (InvocationTargetException e) {
        if (EmptyResultDataAccessException.class.isAssignableFrom(e.getCause().getClass())) {
          acceptableOutcome = true;
        } else {
          throw new RuntimeException(fetchMethod.getName()+" threw an exception during testing",e.getCause());
        }


      }
      if (!acceptableOutcome) {
        logger.warn("Method "+fetchMethod.getName()+" returned "+fetchResult);
      }

      assertTrue("fetch method "+fetchMethod.getName()+" returned "+fetchResult+", expected  null, an empty collection, or an EmptyDataAccess exception when called with a bad id. ",acceptableOutcome);

    }

  }

  private Collection<Object> attemptStore(Collection<Object> retrieved) {
    Collection<Object> storedObjects = new LinkedList<Object>();

    Method storeMethod = null;

    storeMethod = findMatchingMethod(service.getClass().getMethods(), "storeAuthenticated",
        new Class[]{DataObject.class, Person.class}, Long.class);

    if (storeMethod == null) {
      storeMethod = findMatchingMethod(service.getClass().getMethods(), "store",
          new Class[]{DataObject.class, Person.class}, Long.class);
    }

    if (storeMethod != null) {

      Person fakeUser = getFakeUser();

      for (Object data : sampleFromCollection(retrieved, maxNumerOfObjectsToStore)) {

        try {
          Long id = (Long)storeMethod.invoke(service, new Object[]{data, fakeUser});

          assertTrue("DataObject id should not be null after store", (id != null));
          assertTrue("DataObject id should be greater than 0 after store", (id > 0));
          storedObjects.add(data);

        } catch (IllegalArgumentException e) {
          reflectionError(e);
        } catch (IllegalAccessException e) {
          reflectionError(e);
        } catch (InvocationTargetException e) {
          throw new RuntimeException(storeMethod.getName()+" threw an exception during testing",e.getCause());
        }
      }
    }

    return storedObjects;
  }

  protected Person getFakeUser() {
    Person fake = personService.fetchByUserName("test", ComponentMap.of(Person.Component.ALL_COMPONENTS));
    return fake;
  }

  protected Collection<Object> attemptSearch() {
    int searchesDone = 0;

    Collection<Object> searchResults = new LinkedList<Object>();

    if (searchWithPaginatedSearchBean(searchResults)) searchesDone++;
    if (searchWithSortableSearchBean(searchResults)) searchesDone++;
    if (searchWithPagainatedSearchBeanAndComponentMap(searchResults)) searchesDone++;
    if (fetchAll(searchResults)) searchesDone++;
    if (fetchAllWithComponentMap(searchResults)) searchesDone++;


    if (searchesDone < 1) {
      logger.warn("No 'search' or 'fetchAll' method was found to invoke");
    }

    return searchResults;
  }

  @SuppressWarnings("unchecked")
  private boolean fetchAllWithComponentMap(Collection<Object> resultsHolder) {
    boolean searched = false;
    Method searchMethod = findMatchingMethod(service.getClass().getMethods(), "fetchAll",
        new Class[]{ComponentMap.class}, Collection.class);

    if (searchMethod != null) {
      try {
        ComponentMap cm = new ComponentMap();

        resultsHolder.addAll((Collection<Object>)searchMethod.invoke(service, new Object[]{cm}));
        searched = true;
      } catch (IllegalAccessException e) {
        reflectionError(e);
      } catch (InvocationTargetException e) {
        throw new RuntimeException(searchMethod.getName()+" threw an exception during testing",e.getCause());
      }
    }

    return searched;
  }

  @SuppressWarnings("unchecked")
  private boolean fetchAll(Collection<Object> resultsHolder) {
    boolean searched = false;

    Method searchMethod = findMatchingMethod(service.getClass().getMethods(), "fetchAll",
        new Class[]{}, Collection.class);

    if (searchMethod != null) {
      try {
        resultsHolder.addAll((Collection<Object>)searchMethod.invoke(service, new Object[]{}));
        searched = true;
      } catch (IllegalAccessException e) {
        reflectionError(e);
      } catch (InvocationTargetException e) {
        throw new RuntimeException(searchMethod.getName()+" threw an exception during testing",e.getCause());
      }
    }

    return searched;
  }

  @SuppressWarnings("unchecked")
  private boolean searchWithPagainatedSearchBeanAndComponentMap(Collection<Object> resultsHolder) {
    boolean searched = false;

    Method searchMethod = findMatchingMethod(service.getClass().getMethods(), "search",
        new Class[]{PaginatedSearchBean.class, ComponentMap.class}, Collection.class);

    if (searchMethod != null) {
      try {
        PaginatedSearchBean searchBean = createPaginatedSearchBean(searchMethod);

        ComponentMap cm = new ComponentMap();

        resultsHolder.addAll((Collection<Object>)searchMethod.invoke(service, new Object[]{searchBean,cm}));
        searched = true;
      } catch (InstantiationException e) {
        reflectionError(e);
      } catch (IllegalAccessException e) {
        reflectionError(e);
      } catch (InvocationTargetException e) {
        throw new RuntimeException(searchMethod.getName()+" threw an exception during testing",e.getCause());
      }
    }

    return searched;
  }

  private PaginatedSearchBean createPaginatedSearchBean(Method searchMethod) throws InstantiationException, IllegalAccessException {
    PaginatedSearchBean searchBean = (PaginatedSearchBean)searchMethod.getParameterTypes()[0].newInstance();
    searchBean.setPaginated(true);
    searchBean.setQueryName("unit test");
    searchBean.setSessionId("unit test "+Math.random());
    modifyPaginatedSearchBean(searchBean);
    return searchBean;
  }

  @SuppressWarnings("unchecked")
  private boolean searchWithSortableSearchBean(Collection<Object> resultsHolder) {
    boolean searched = false;

    Method searchMethod = findMatchingMethod(service.getClass().getMethods(), "search",
        new Class[]{SortableSearchBean.class}, Collection.class);

    // PaginatedSearchBeans inherit from sortable and we don't want to consider them here
    if (searchMethod != null &&
        PaginatedSearchBean.class.isAssignableFrom(searchMethod.getParameterTypes()[0])) {
      searchMethod = null;
    }

    if (searchMethod != null) {
      try {
        SortableSearchBean searchBean = (SortableSearchBean)searchMethod.getParameterTypes()[0].newInstance();
        searched = true;

        resultsHolder.addAll((Collection<Object>)searchMethod.invoke(service, new Object[]{searchBean}));
      } catch (InstantiationException e) {
        reflectionError(e);
      } catch (IllegalAccessException e) {
        reflectionError(e);
      } catch (InvocationTargetException e) {
        throw new RuntimeException(searchMethod.getName()+" threw an exception during testing",e.getCause());
      }
    }

    return searched;
  }

  @SuppressWarnings("unchecked")
  private boolean searchWithPaginatedSearchBean(Collection<Object> resultsHolder) {
    boolean searched = false;

    Method searchMethod = findMatchingMethod(service.getClass().getMethods(), "search",
        new Class[]{PaginatedSearchBean.class}, Collection.class);

    if (searchMethod != null) {
      try {
        PaginatedSearchBean searchBean = createPaginatedSearchBean(searchMethod);
        searched = true;

        resultsHolder.addAll((Collection<Object>)searchMethod.invoke(service, new Object[]{searchBean}));
      } catch (InstantiationException e) {
        reflectionError(e);
      } catch (IllegalAccessException e) {
        reflectionError(e);
      } catch (InvocationTargetException e) {
        throw new RuntimeException(searchMethod.getName()+" threw an exception during testing",e.getCause());
      }
    }

    return searched;
  }

  protected Collection<Object> attemptRetrieval(Collection<Object> searchResults) {
    Collection<Object> retrievedObjects = new LinkedList<Object>();

    Method fetchMethod = getFetchByIdMethod();

    if (fetchMethod == null){
    	logger.warn("No 'fetchById' method found");
    }
    else {
      Collection<Object> filteredResults = getObjectsWithIdMethod(searchResults);

      if (filteredResults.isEmpty()) {
        logger.warn("No 'getId' method found for any search result");
      }

      for (Object searchResult : sampleFromCollection(searchResults, maxNumberOfObjectsToRetrieve)) {
        Method idMethod = getIdMethod(searchResult);

        try {
          Long id = (Long) idMethod.invoke(searchResult, new Object[]{});
          ComponentMap cm = createComponentMap(fetchMethod);

          DataObject data = (DataObject)fetchMethod.invoke(service, new Object[]{id,cm});

          retrievedObjects.add(data);

        } catch (IllegalArgumentException e) {
          reflectionError(e);
        } catch (IllegalAccessException e) {
          reflectionError(e);
        } catch (InvocationTargetException e) {
          throw new RuntimeException(fetchMethod.getName()+" threw an exception during testing",e.getCause());
        }
      }
    }

    return retrievedObjects;
  }

  private Collection<Object> getObjectsWithIdMethod(Collection<Object> searchResults) {
    Collection<Object> filteredResults = new LinkedList<Object>();

    for (Object o : searchResults) {
      if (getIdMethod(o) != null) {
        filteredResults.add(o);
      }
    }

    return filteredResults;
  }

  private Method getIdMethod(Object searchResult) {
    Method idMethod = findMatchingMethod(searchResult.getClass().getMethods(), "getId",
        new Class[]{}, Long.class);
    return idMethod;
  }

  private Method getFetchByIdMethod() {
    Method fetchMethod = findMatchingMethod(service.getClass().getMethods(), "fetchById",
        new Class[]{Long.class, ComponentMap.class}, DataObject.class);
    return fetchMethod;
  }

  private Collection<Object> sampleFromCollection(Collection<Object> srcCollection, int amountToSample) {
    Collection<Object> sample;
    Set<Object> uniqueObjects = new LinkedHashSet<Object>(srcCollection);

    if (uniqueObjects.size() <= amountToSample) {
      sample = uniqueObjects;
    }
    else {
      LinkedList<Object> sr = new LinkedList<Object>(uniqueObjects);
      sample = sr.subList(0, amountToSample);
    }

    return sample;
  }

  @SuppressWarnings("unchecked")
  private ComponentMap createComponentMap(Method fetchMethod) {
    ComponentMap cm = new ComponentMap();

    for (Class c : fetchMethod.getReturnType().getClasses()) {

      boolean isEnum = c.isEnum();
      boolean isAccessible = Modifier.isPublic(c.getModifiers());
      boolean isRightClass = ComponentEnum.class.isAssignableFrom(c);

      if (isEnum && isAccessible && isRightClass) {
        for (Object en : c.getEnumConstants()) {
          if (en.toString().contains("ALL_COMPONENTS")) {
            cm.put((ComponentEnum)en,null);
          }
        }
      }
    }

    return cm;
  }

  @SuppressWarnings("unchecked")
  private Method findMatchingMethod(Method[] methods, String name, Class[] paramTypes, Class returnType) {
    Method match = null;

    for (Method m : methods) {
      if (isMatchingMethod(m, name, paramTypes, returnType)) {
        match = m;
      }
    }

    return match;
  }


  @SuppressWarnings("unchecked")
  private boolean isMatchingMethod(Method m, String name, Class[] paramTypes, Class returnType) {
    boolean match = Modifier.isPublic(m.getModifiers());

    if (match) {
      match = m.getName().equals(name);
    }

    if (match) {
      match = (paramTypes.length == m.getParameterTypes().length);
    }

    for (int i = 0; match && i < paramTypes.length; i++) {
      match = (paramTypes[i].isAssignableFrom(m.getParameterTypes()[i]));
    }

    if (match) {
      match = (returnType.isAssignableFrom(m.getReturnType()));
    }

    return match;
  }


  private void reflectionError(Exception e) {
    logger.fatal("Reflection error", e);
    throw new RuntimeException("Reflection error",e);
  }

  protected String[] getConfigLocations() {
    setAutowireMode(AUTOWIRE_BY_NAME);

    String baseLoc = "file:WebRoot"+File.separator+"WEB-INF"+File.separator+"conf"+File.separator;

    return new String[] {
        baseLoc+"application-context.xml"
    };
  }

  /**
   * some searches will exceed memory available if they aren't given a parameter. override
   * this method to add parameters
   */
  public void modifyPaginatedSearchBean(PaginatedSearchBean psb) { // NOPMD by on 20/08/08 17:19
    // to be overridden
  }

  public void setPersonService(PersonService personService) {
    this.personService = personService;
  }

}
