/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.testsupport;

import static org.unitils.util.AnnotationUtils.getMethodOrClassLevelAnnotation;

import java.lang.reflect.Method;

import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.filter.DefaultColumnFilter;
import org.dbunit.dataset.filter.IColumnFilter;
import org.unitils.dbunit.DbUnitModule;
import org.unitils.dbunit.util.MultiSchemaDataSet;

public class MigDbUnitModule extends DbUnitModule {

  /**
   * Compares the contents of the expected DbUnitDataSet with the contents of the database. Only the tables and columns
   * that occur in the expected DbUnitDataSet are compared with the database contents.
   *
   * @param testMethod The test method, not null
   * @param testObject The test object, not null
   */
  @Override
  public void assertDbContentAsExpected(Method testMethod, Object testObject) {
      try {
          // get the expected dataset
          MultiSchemaDataSet multiSchemaExpectedDataSet = getExpectedDataSet(testMethod, testObject);
          if (multiSchemaExpectedDataSet == null) {
              // no data set should be compared
              return;
          }
          // first make sure every database update is flushed to the database
          getDatabaseModule().flushDatabaseUpdates(testObject);


          // get filter criteria
          IColumnFilter columnFilter = getColumnFilter(testMethod,testObject);

          for (String schemaName : multiSchemaExpectedDataSet.getSchemaNames()) {
              IDataSet compositeDataSet = multiSchemaExpectedDataSet.getDataSetForSchema(schemaName);
              MigUnitTestAssert.assertDbContentAsExpected(compositeDataSet, getDbUnitDatabaseConnection(schemaName), columnFilter);
          }
      } finally {
          closeJdbcConnection();
      }
  }

  private IColumnFilter getColumnFilter(Method testMethod, Object testObject) {
    DefaultColumnFilter columnFilter = null;

    Class<?> testClass = testObject.getClass();
    MigColumnFilter columnFilterAnnotation = getMethodOrClassLevelAnnotation(MigColumnFilter.class, testMethod, testClass);

    if (columnFilterAnnotation != null) {
      // annotation found
      columnFilter = new DefaultColumnFilter();

      for (String filterCriteria : columnFilterAnnotation.value()) {
        columnFilter.excludeColumn(filterCriteria);
      }
    }

    return columnFilter;
  }



}
