/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;

/**
 * This class represents the underlying sequence assembly on which a Build is based. 
 * 
 * @author dgm
 *
 */
public class Assembly extends AuthenticatedDataObject  {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS
	};
	
    private int ncbiTaxon;        
    private String speciesName;        
    private String name;              

    /**
     * Get the NCBI Taxonomy number that this Assembly is associated with.
     * See http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Taxonomy for the meaning
     * of this numerical value.
     * 
     * @return the numerical NCBI Taxonomy code, or null if this has not been assigned.
     */
    public int getNcbiTaxon() {
        return ncbiTaxon;
    }
    
    /**
     * Set the NCBI Taxonomy number that this Assembly is associated with.
     * See http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Taxonomy for suitable values
     * for this attribute.
     * 
     * @param ncbiTaxon
     */
    public void setNcbiTaxon(int ncbiTaxon) {
        this.ncbiTaxon = ncbiTaxon;
    }
    
    /**
     * Get the textual name of the species, for example 'Mus musculus', that this assembly 
     * is associated with.
     * 
     * @return the species name associated with the Assembly, or null if this has not been 
     *     assigned.
     */
    public String getSpeciesName() {
        return speciesName;
    }
    
    /**
     * Set the textual name of the species, for example 'Mus musculus', that this assembly 
     * is associated with.
     * 
     * @param speciesName is the String representing the name of the species.
     */
    public void setSpeciesName(String speciesName) {
        this.speciesName = speciesName;
    }
    
    /**
     * Get the name of the assembly. This is typically a string of the form 'NCBI34' or 'Vega v19' and
     * typically represents the underlying data used in the Assembly process or the group that carried
     * out the assembly. 
     *   
     * @return the String which describes the source of the assembly data.
     */
    public String getName() {
        return name;
    }
    
    /**
     * Set the name of the assembly. This is typically a string of the form 'NCBI34' or 'Vega v19' and
     * typically represents the underlying data used in the Assembly process or the group that carried
     * out the assembly. 
     * 
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }
}