/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;
import uk.ac.sanger.mig.personnel.Person;

/**
 * A class used to carry information about the source of the data relating to a Build. A (Gene) Build
 * referrs to a collection of predicted or manually curated genes sourced either from Ensembl or Vega (or
 * occasionaly otterlace). This object describes all of the details required to uniquely identify the
 * source.
 * 
 * @author dgm
 *
 */
public class Build extends AuthenticatedDataObject implements LookupListElement {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		ASSEMBLY,
		CREATOR
	}
	
	private Long assemblyId;
	private Assembly assembly;
	
	private Long creatorId;
	private Person creator;
	
	private String name;
	private String version;
	private String source;
	private int orderBy;
	
	/**
	 * Return the ranking of this Build (in terms of recency). This is, typically, used to indicate
	 * the most recent build when selecting out a gene to associate with a GNM_GENE. This may not have
	 * much in the way of purpose at the Object layer but is included for completeness.
	 * 
	 * @return and int containin g the rank of this build.
	 */
	public int getOrderBy() {
		return orderBy;
	}

	/**
	 * Set the rankin of the Build  (in terms of recency).  This is, typically, used to indicate
	 * the most recent build when selecting out a gene to associate with a GNM_GENE. This may not have
	 * much in the way of purpose at the Object layer but is included for completeness.
	 * 
	 * @param orderBy is an int indicating this Build's rank.
	 */
	public void setOrderBy(int orderBy) {
		this.orderBy = orderBy;
	}

	/**
	 * Get the primary key of the Assembly associated with this build.
	 * 
	 * @return the primary key value for the Assembly associated with this build.
	 */
	public Long getAssemblyId() {
		return assemblyId;
	}
	
	/**
	 * Set the primary key of the Assembly associated with this build.
	 * 
	 * @param is the primary key value for the Assembly associated with this build.
	 */
	public void setAssemblyId(Long assemblyId) {
		this.assemblyId = assemblyId;
	}
	
	/**
	 * Get the Assembly object associated with the assemblyId. This will only be valid if the
	 * service layer has been instructed to load this data. This reqirement should be indicated
	 * by adding Build.Component.ASSEMBLY to the Component map.
	 * 
	 * @return the Assembly object associated with this build, or null if this has not been set
	 *     by the service layer method.
	 */
	public Assembly getAssembly() {
		return assembly;
	}
	
	/**
	 * Set the Assembly object associated with the this Build. This will carried out by the
	 * service layer if it has been instructed to load this data. This reqirement should be indicated
	 * by adding Build.Component.ASSEMBLY to the Component map passed to the service layer.
	 * 
	 * @param assembly is the Assembly object associated tobe associated with this build.
	 */	
	public void setAssembly(Assembly assembly) {
		this.assembly = assembly;
	} 
	
	/**
	 * Get the Mouse Informatics name associated with this build. This is a String, typically of the
	 * form '0906Emsembl' or '0906Vega' which represents the date of import and the source of the data.
	 * 
	 * @return the MIG short-hand name for the Build.
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Set Mouse Informatics name associated with this build. This is a String, typically of the
	 * form '0906Emsembl' or '0906Vega' which represents the date of import and the source of the data.
	 * This should only be called by the rowmapper (in the DAO).
	 * 
	 * @param name is the MIG short-hand name for the Build.
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Get the source of the build data. Currently this includes: otter, Vega or Ensembl. It indicates the
	 * primary source for the Build data.
	 * 
	 * @return the String indicating where the build data was collected from.
	 */
	public String getSource() {
		return source;
	}
	
	/**
	 * Set the source of the build data. Currently this includes: otter, Vega or Ensembl. It indicates the
	 * primary source for the Build data. This should only be called by the rowmapper (in the DAO).
	 *  
	 * @param source is the String representing the source of the build data.
	 */
	public void setSource(String source) {
		this.source = source;
	}
	
	/**
	 * Get the version of the Build. This is typically a string of the form '36.34d' or 'v16' which
	 * represents the ensembl or vega build that is represented by this Build. 
	 * 
	 * @return The version string for this build (as assigned by ensembl or vega, or otterlace).
	 */
	public String getVersion() {
		return version;
	}
	
	/**
	 * Set the version of the Build. This is typically a string of the form '36.34d' or 'v16' which
	 * represents the ensembl or vega build that is represented by this Build. This should only be
	 * called by the rowmapper (in the DAO). 
	 * 
	 * @param version is the String containing the unique descriptor string for the Build version.
	 */
	public void setVersion(String version) {
		this.version = version;
	}

	/**
	 * Set the foreign key value that refers to the user that added this record to the GNM_GENE_BUILD table.
	 * 
	 * @param is the foreign key value referencing a record in the PER_PERSON table.
	 */
	public void setCreatorId(Long personId) {
		creatorId = personId;
	}
	
	/**
	 * Get the foreign key value that refers to the user that added this record to the GNM_GENE_BUILD table.
	 * 
	 * @return the foreign key value that references a record in the PER_PERSON table.
	 */
	public Long getCreatorId() {
		return creatorId;
	}
	
	/**
	 * Set the Person object that is associated with this Build object. This will carried out by the
	 * service layer if it has been instructed to load this data. This reqirement should be indicated
	 * by adding Build.Component.CREATOR to the Component map passed to the service layer.
	 * 
	 * @param person is the Person object to be associated with this build.
	 */
	public void setCreator(Person person){
		creator = person;
	}
	
	/**
	 * Get the Person object associated with the creatorId. This will only be valid if the
	 * service layer has been instructed to load this data. This reqirement should be indicated
	 * by adding Build.Component.CREATOR to the Component map.
	 * 
	 * @return the Person object associated with this build, or null if this has not been set
	 *     by the service layer method.
	 */
	public Person getCreator() {
		return creator;
	}
	  
	public String getGeneBuildDescription() {
	    return source + " " + version;
	}
	
	public String getDisplayText() {
		return getName();
	}

	public String getElementData() {
		return 
			LookupListUtil.format("geneBuildId", getId()) +
			LookupListUtil.format("name", getName());
	}

	public void setDisplayText(String displayText) {
		// TODO Auto-generated method stub
	}

	public void setElementData(String elementData) {
		// TODO Auto-generated method stub
	}	
}
