/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import java.util.Collection;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;

/**
 * This class represents a predicted or manually curated Gene that lives within one of the Ensembl or Vega
 * gene builds.
 * 
 * @author dgm
 *
 */
public class BuildGene extends AuthenticatedDataObject implements LookupListElement {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		BUILD,
		LOCUS,
		TRANSCRIPTS
	}

	private Long buildId;
	private Build build;
	
	private Long locusId;
	private Locus locus;
	
	private String isFinished;
	private int transcriptCount;
	private int totalExonCount;
	private int uniqueExonCount;
	private String primaryName;
	
	private Collection<Transcript> transcripts;

	/**
	 * Get the foreign key associated with the Build record for this BuildGene.
	 * 
	 * @return the Long value containing the foreign key associated with this BuildGene
	 */
	public Long getBuildId() {
		return buildId;
	}

	/**
	 * Set the foreign key for the Build record that should be associated with this BuildGene.
	 * 
	 * @param buildId is the Long value containing the foreign key which refers to the Build record
	 *     that is associated with this BuildGene.
	 */
	public void setBuildId(Long buildId) {
		this.buildId = buildId;
	}

	/**
	 * Get the Build object containing administrative information about the gene build that this Gene is
	 * part of.
	 * 
	 * @return a Build object containing information about the Build that contains this Gene. If the service layer
	 *     has not been requested to load this data then this will be null.
	 */
	public Build getBuild() {
		return build;
	}

	/**
	 * Set the Build object containing administrative information about the gene build that this Gene is
	 * part of. This should only be called by the service layer in response to the BuildGene.Component.BUILD
	 * being specified in the required components map.
	 * 
	 * @param build is a Build object containing information about the Build that contains this Gene.
	 */
	public void setBuild(Build build) {
		this.build = build;
	}

	/**
	 * Get the foreign key that links this BuildGene to its genomic location. This references a row in the
	 * LOCUS table.
	 * 
	 * @return a Long value that contains the foreign key which references a row in the LOCUS table. 
	 */
	public Long getLocusId() {
		return locusId;
	}

	/**
	 * Set the foreign key that links this BuildGene to its genomic location. This is used by the rowmapper
	 * to store the value from the DB table to an instance of this object.
	 * 
	 * @param locusId is a Long value that contains the foreign key which references a row in the LOCUS table. 
	 */
	public void setLocusId(Long locusId) {
		this.locusId = locusId;
	}

	/**
	 * Get the exact location of the Gene within an assembly and chromosome.
	 * 
	 * @return the Locus which describes, the assembly, chromosome, start, end and strand of the gene.
	 */
	public Locus getLocus() {
		return locus;
	}

	/**
	 * Set the exact location of the Gene within an assembly and chromosome.
	 * 
	 * @param locus describes, the assembly, chromosome, start, end and strand of the gene.
	 */
	public void setLocus(Locus locus) {
		this.locus = locus;
	}

	/**
	 * Get the clone finishing status for this Gene. This is a tri-state String value that can be "Yes" meaning that
	 * the status has been determined (using the perl loader script) and the clone has been finished (Based on avc
	 * criteria: ie. not in HTG, WGS or KW) then the clone covering this Gene is considered to be finished. If it 
	 * falls into HTG or WGS then the clone is not finished (KW is ignored) and if it has not been determined for this
	 * gene then the value will be null.
	 * 
	 * @return a string contining "Yes", "No" or null depending upon the clone finishing status and the presence of
	 *     data in our database.
	 */
	public String getFinishingStatus() {
		return isFinished;
	}

	/**
	 * Set the 'finished' status of the clone that covers this gene. This should only be called by the rowmapper
	 * when its processing a table row.
	 * 
	 * @param isFinished is a String containing "Yes", "No" or null depending on our state of knowledge about the
	 *      clone that covers this gene.
	 */
	public void setFinishingStatus(String isFinished) {
		this.isFinished = isFinished;
	}

	/**
	 * Get the number of transcripts that this gene has.
	 * 
	 * @return the number of transcripts that are known or predicted for this gene.
	 */
	public int getTranscriptCount() {
		return transcriptCount;
	}

	/**
	 * Set the number of transcripts that are known for this gene. This should only be called by the rowmapper
	 * when it is processing a row from the database table.
	 * 
	 * @param transcriptCount is the count of transcripts for this gene.
	 */
	public void setTranscriptCount(int transcriptCount) {
		this.transcriptCount = transcriptCount;
	}

	/**
	 * Get the 'primary' name for this gene. This is typically an Ensembl or Vega 'stable' identifier of the
	 * form 'ENSMUSG00000065650' (from ensembl) or 'OTTMUSG00000000650' from Vega. This is the external name
	 * that, together with the Build can be used to search the Ensembl/Vega website for this gene.
	 * 
	 * @return a String containing a 'stable' identifier (in some external naming system e.g. Ensembl/Vega) 
	 */
	public String getPrimaryName() {
		return primaryName;
	}

	/**
	 * Set the 'primary' name for this gene. This is typically an Ensembl or Vega 'stable' identifier of the
	 * form 'ENSMUSG00000065650' (from ensembl) or 'OTTMUSG00000000650' from Vega. This is the external name
	 * that, together with the Build can be used to search the Ensembl/Vega website for this gene. This should
	 * only be called by a rowmapper while processing a database table row.
	 * 
	 * @param primaryName is a String containing a 'stable' identifier (in some external naming system e.g. Ensembl/Vega) 
	 */
	public void setPrimaryName(String primaryName) {
		this.primaryName = primaryName;
	}

	/**
	 * Get the Transcripts of this gene. These may be predicted (Ensembl) or manually curated (Vega) depending
	 * on the source of the Gene data.
	 * 
	 * @return a Transcript object containing information about each of the alternate splicings known (or
	 *     predicted) for this Gene or null if this has not been set by the service layer (as a consequence
	 *     of the user specifying BuildGene.Components.TRANSCRIPTS in the component map).
	 */
	public Collection<Transcript> getTranscripts() {
		return transcripts;
	}

	/**
	 * Set the Transcripts for this gene. This will only be called by the service layer when the user has
	 * specified BuildGene.Components.TRANSCRIPTS in the component map passed to the service layer. 
	 * 
	 * @param transcripts is a Collection of Transcript objects containing information about each of the
	 * alternate splicings known (or predicted) for this Gene.
	 */
	public void setTranscripts(Collection<Transcript> transcripts) {
		this.transcripts = transcripts;
	}
	
	/**
	 * Get the count of exons in this gene. This is the total (analogous to getting the length of the array of
	 * all exons, using the Ensembl API for a given Gene) number of exons in all transcripts of the gene where
	 * exons that exactly match (between transcripts) are only counted once.
	 * 
	 * @return the number of exons in this gene.
	 */
	public int getTotalExonCount() {
		return totalExonCount;
	}

	/**
	 * Set the count of exons in this gene. This is the total (analogous to getting the length of the array of
	 * all exons, using the Ensembl API for a given Gene) number of exons in all transcripts of the gene where
	 * exons that exactly match (between transcripts) are only counted once. This should only be called by the 
	 * rowmapper when processing a database record.
	 * 
	 * @param exonCount the number of exons in this gene.
	 */
	public void setTotalExonCount(int totalExonCount) {
		this.totalExonCount = totalExonCount;
	}

	public int getUniqueExonCount() {
		return uniqueExonCount;
	}

	public void setUniqueExonCount(int uniqueExonCount) {
		this.uniqueExonCount = uniqueExonCount;
	}
	
	
	public String getDisplayText() {
		return getPrimaryName();
	}

	public String getElementData() {
		return 
	    LookupListUtil.format("geneBuildGeneId", getId()) +
	    LookupListUtil.format("primaryname", getPrimaryName());
	}

	public void setDisplayText(String displayText) {
		// TODO Auto-generated method stub
	}

	public void setElementData(String elementData) {
		// TODO Auto-generated method stub
	}
}