/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;


/**
 * A class used to represent a link between a MigGene and a BuildGene. This represents the 1:m linkage to the
 * many BuildGenes (loaded from Ensembl and Vega) that is intended to define the same gene in each of these
 * sources
 * 
 * @author dgm
 *
 */
public class BuildGeneLink extends AuthenticatedDataObject  {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS
	}

    private Long migGeneId;
    private Long migGeneBuildGeneId;
    private String reasoning;
    private boolean isValid;       

    /**
     * Get the primary key of the MigGene that is at one end of the link.
     * 
     * @return a Long containing the primary key of the MigGene that is at one end of this link.
     */
    public Long getMigGeneId() {
        return migGeneId;
    }
    
    /**
     * Set the primary key of the MigGene that is being linked to. This will typically only be called by the
     * Dao/Rowmapper when loading a database record.
     * 
     * @param geneId is a Long containing the primary key of the MigGene that is being linked to.
     */
    public void setMigGeneId(Long migGeneId) {
        this.migGeneId = migGeneId;
    }
    
    /**
     * Get the primary key of the BuildGene that is being linked to.
     * 
     * @return a Long containing the primary key of the BuildGene that is being linked to.
     */
    public Long getMigGeneBuildGeneId() {
        return migGeneBuildGeneId;
    }
    
    /**
     * Set the primary key of the BuildGene that is being linked to. This will typically only be called by the
     * Dao/Rowmapper when loading a database record.
     * 
     * @param geneBuildGeneId is a Long containing the primary key of the BuildGene that is being linked to.
     */
    public void setMigGeneBuildGeneId(Long migGeneBuildGeneId) {
        this.migGeneBuildGeneId = migGeneBuildGeneId;
    }
    
    /**
     * Get the description of the reasoning for this link existing. Specifically, the statement of why the
     * person who added the link thinks that the link is valid.
     * 
     * @return a String containing the description of why the link exists.
     */
    public String getReasoning() {
        return reasoning;
    }
   
    /**
     * Set the description of the reasoning for this link existing. Specifically, the statement of why the
     * person who added the link thinks that the link is valid. This will typically only be called by the
     * Dao/Rowmapper when loading a database record.
     * 
     * @return a String containing the description of why the link exists.
     */
    public void setReasoning(String reasoning) {
        this.reasoning = reasoning;
    }
    
    /**
     * Get a validity flag for this link. If the validity flag is set to false then this record should be 
     * considered to be deprecated.
     * 
     * @return true if the link is thought to be valid and false otherwise.
     */
    public boolean isValid() {
        return isValid;
    }
    
    /**
     * Set a validity flag for this link. If the validity flag is set to false then this record should be 
     * considered to be deprecated. This will typically only be called by the Dao/Rowmapper when loading a
     * database record.
     * 
     * @param isValid should be set to true if the link is considered to be correct or false if not.
     */
    public void setValid(boolean isValid) {
        this.isValid = isValid;
    }
}