/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import java.util.Collection;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;

public class Exon extends AuthenticatedDataObject implements LookupListElement {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		LOCUS,
		TRANSCRIPTS
	}

    private Long locusId;
    private Locus locus;
    
    private Collection<Transcript> transcripts;
    
    private int phase;        
    private String primaryName;        

    /**
     * Get the foreign key value for the Locus that is associated with this Exon.
     * 
     * @return the foreign key that references the Locus of this Exon.
     */
    public Long getLocusId() {
        return locusId;
    }
    
    /**
     * Set the foreign key value for the Locus that is associated with this Exon.
     * 
     * @param locusId is the value for the foreign key that references the Locus of this Exon.
     */
    public void setLocusId(Long locusId) {
        this.locusId = locusId;
    } 
    
    /**
     * Get the 'phase' of the exon. Phase 0 means that the exon begins with a complete codon, whereas
     * phase 1 means that the first two bases (5-prime) of the exon constitute the second and third
     * bases of an interrupted codon.  Phase 2 means the first base of the exon consists of the last base
     * of an interrupted codon.
     * 
     * @return the phase of the exon encoded as an integer value 0, 1, 2. See the full description for
     *     details of the meaning.
     */
    public int getPhase() {
        return phase;
    }
    
    /**
     * Set the 'phase' of the exon. Phase 0 means that the exon begins with a complete codon, whereas
     * phase 1 means that the first two bases (5-prime) of the exon constitute the second and third
     * bases of an interrupted codon.  Phase 2 means the first base of the exon consists of the last base
     * of an interrupted codon. This should only be called by 
     * 
     * @return the phase of the exon encoded as an integer value 0, 1, 2. See the full description for
     *     details of the meaning.
     */
    public void setPhase(int phase) {
        this.phase = phase;
    }
    
    /**
     * Get the primary name for this exon. For example, exons loaded from ensembl will have a primary name of the
     * form 'ENSMUSE00000582523' and those loaded from Vega will be of the form 'OTTMUSE00000582523'. This is an
     * identifier that can be used to search in either of these systems for this exon.
     * 
     * @return the String containing the name for this entity (although the original source of this name can only
     *     be inferred).
     */
    public String getPrimaryName() {
        return primaryName;
    }
    
    /**
     * Set the primary name for this exon. For example, exons loaded from ensembl should have a primary name of the
     * form 'ENSMUSE00000582523' and those loaded from Vega should be of the form 'OTTMUSE00000582523'. This is an
     * identifier that can be used to search in either of these systems for this exon.
     * 
     * @param primaryName
     */
    public void setPrimaryName(String primaryName) {
        this.primaryName = primaryName;
    }                                                                                                                                                                                                                                                                             

    /**
     * Get the Locus object for this Exon. This defines its location (in terms of chromosome, strand, start and end
     * coordinates). This may be null if the service layer has not been requested to populate this object. 
     * @see Locus
     * @return a Locus object or null if this has not been set by the service layer.
     */
	public Locus getLocus() {
		return locus;
	}

	/**
	 * Set the location of this Exon. This should only be called by the service layer or DAO to link this
	 * exon to a Locus object. 
	 * 
	 * @param locus is a Locus object that identifies the position of this exon on a Gene Build.
	 */
	public void setLocus(Locus locus) {
		this.locus = locus;
	}

	/**
	 * Get the collection of Transcript objects that contain this Exon. This should always contain, at least
	 * one Transcript if it has been populated by the service layer. This can be sepcified by adding
	 * Exon.Component.TRANSCRIPTS to the component map when requesting it from the service layer.
	 * 
	 * @return The collection of Transcripts or null if this has not been populated by the service layer.
	 */
	public Collection<Transcript> getTranscripts() {
		return transcripts;
	}

	/**
	 * Set the collection of Transcript objects that contain this Exon. This should always contain, at least
	 * one Transcript if it has been populated by the service layer. This should only be called by the service
	 * layer.
	 * 
	 * @param transcripts is the Collection of Transcript objects that contain this exon.
	 */
	public void setTranscripts(Collection<Transcript> transcripts) {
		this.transcripts = transcripts;
	}
	
	public String getDisplayText() {
		return primaryName +", " + getChromosomeStart() + ", "+ getChromosomeEnd();
	}

	public String getElementData() {
		return 
		LookupListUtil.format("primaryname", getPrimaryName())+
		LookupListUtil.format("exonId", getId())+
		LookupListUtil.format("chromosomestart", getChromosomeStart())+
		LookupListUtil.format("chromosomeend", getChromosomeEnd());
	}

	public void setDisplayText(String displayText) {
		// TODO Auto-generated method stub
	}

	public void setElementData(String elementData) {
		// TODO Auto-generated method stub
	}

	public int getChromosomeStart() {
		return locus.getChromosomeStart();
	}
	
	public int getChromosomeEnd() {
		return locus.getChromosomeEnd();
	}
}