/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;

/**
 * This class represents a genomic range. Specifically, this is used to represent either points or ranges
 * of specific genomic coordinates (assembly, chromosome, start_coord, end_coord, strand). These are used to
 * uniquely describe the location of genomic features (e.g. Genes, Exons etc.) but are also used to store the
 * locations of entire genes (as defined in a build gene) and knockout designs.
 *
 * @author dgm
 *
 */

public class Locus extends AuthenticatedDataObject  {

	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		BUILD,
		ASSEMBLY
	}

    private String chromosomeName;
    private Integer chromosomeStart;
    private Integer chromosomeEnd;
    private Integer chromosomeStrand;

    private Long assemblyId;
    private Assembly assembly;

    private Long buildId;
    private Build build;

    private String type;

    /**
     * Get the name of the chromosome. This may contain a value that doesn't correspond to the normal 1, 2...19, X, Y
     * (for mouse) since this also includes strings such as 'NT_101323' which correspond to blocks of assembled
     * sequence for which the actual chromosome location is yet to be identified.
     *
     * @return The name of the Chromosome or contig that contains the location (and provides the origin for the
     *     start and end coordinates of this Locus).
     */
    public String getChromosomeName() {
        return chromosomeName;
    }

    /**
     * Set the name of the chromosome. This may contain a value that doesn't correspond to the normal 1, 2...19, X, Y
     * (for mouse) since this may also include strings such as 'NT_101323' which correspond to blocks of assembled
     * sequence for which the actual chromosome location is yet to be identified. These provide the origin for the
     * start and end coordinates.
     *
     * @param chromosomeName is the string representing the assembled block of sequence to which the start and end coordinates
     *     refer.
     */
    public void setChromosomeName(String chromosomeName) {
        this.chromosomeName = chromosomeName;
    }

    /**
     * Get the starting coordinate of this locus (relative to the Chromosome/Contig). The first base in the contig
     * is referenced as position 1.
     *
     * @return The starting position, relative to the Chromosome/Contig.
     */
    public Integer getChromosomeStart() {
        return chromosomeStart;
    }

    /**
     * Set the starting coordinate of this locus (relative to the Chromosome/Contig). The first base in the contig
     * is referenced as position 1.
     *
     * @param chromosomeStart is the starting position, relative to the Chromosome/Contig.
     */
    public void setChromosomeStart(Integer chromosomeStart) {
        this.chromosomeStart = chromosomeStart;
    }

    /**
     * Get the end coordinate of this locus (relative to the Chromosome/Contig). The first base in the contig
     * is referenced as position 1.
     *
     * @return The end position, relative to the Chromosome/Contig.
     */
    public Integer getChromosomeEnd() {
        return chromosomeEnd;
    }

    /**
     * Set the end coordinate of this locus (relative to the Chromosome/Contig). The first base in the contig
     * is referenced as position 1.
     *
     * @param chromosomeEnd is the end position, relative to the Chromosome/Contig.
     */
    public void setChromosomeEnd(Integer chromosomeEnd) {
        this.chromosomeEnd = chromosomeEnd;
    }

    /**
     * Get the strand that this Locus refers to. This will either be +1 = forward or -1 = reverse strand. Examination
     * of the database suggests that null (effective value 0) is also an option although this is likely to be and error
     * in the data.
     *
     * @return +1 if the locus is on the forward strand, -1 if it is on the reverse or (0 error case).
     */
    public Integer getChromosomeStrand() {
        return chromosomeStrand;
    }

    /**
     * Set the strand that this Locus lies on. This will either be +1 = forward or -1 = reverse strand.
     *
     * @param chromosomeStrand should be +1 if the locus is on the forward strand, -1 if it is on the reverse.
     */
    public void setChromosomeStrand(Integer chromosomeStrand) {
        this.chromosomeStrand = chromosomeStrand;
    }

    /**
     * Get the primary key of the Assembly object that this Locus is associated with.
     *
     * @return the Long value containing the primary key of the Assembly that this Locus is part of.
     */
    public Long getAssemblyId() {
        return assemblyId;
    }

    /**
     * Set the primary key of the Assembly object that this Locus is associated with.
     *
     * @param assemblyId is the Long value containing the primary key of the Assembly that this Locus is
     *     part of.
     */
    public void setAssemblyId(Long assemblyId) {
        this.assemblyId = assemblyId;
    }

    /**
     * Get the type of the Locus. This is used to indicate the purpose of the locus. This currently contains
     * values such as: EXON, TRANSCRIPT, DESIGN, GENE_BUILD_GENE.
     *
     * @return a String containing an indicative type for the entity described by this Locus.
     */
    public String getType() {
        return type;
    }

    /**
     * Set the type of the Locus. This is used to indicate the purpose of the locus. This currently contains
     * values such as: EXON, TRANSCRIPT, DESIGN, GENE_BUILD_GENE but may be extended, in future, to include
     * other types.
     *
     * @param type is a String containing an indicative type for the entity described by this Locus.
     */
    public void setType(String type) {
        this.type = type;
    }

	/**
	 * Get the Assembly object associated with the assemblyId. This will only be valid if the
	 * service layer has been instructed to load this data. This reqirement should be indicated
	 * by adding Locus.Component.ASSEMBLY to the Component map.
	 *
	 * @return the Assembly object associated with this locus, or null if this has not been set
	 *     by the service layer method.
	 */
	public Assembly getAssembly() {
		return assembly;
	}

	/**
	 * Set the Assembly object associated with the this Locus. This will carried out by the
	 * service layer if it has been instructed to load this data. This reqirement should be indicated
	 * by adding Locus.Component.ASSEMBLY to the Component map passed to the service layer.
	 *
	 * @param assembly is the Assembly object associated tobe associated with this locus.
	 */
	public void setAssembly(Assembly assembly) {
		this.assembly = assembly;
	}

	public Long getBuildId() {
		return buildId;
	}

	public void setBuildId(Long buildId) {
		this.buildId = buildId;
	}

	public Build getBuild() {
		return build;
	}

	public void setBuild(Build build) {
		this.build = build;
	}

}