/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;
import uk.ac.sanger.mig.geneevidence.MigCuratedStatus;
import uk.ac.sanger.mig.geneevidence.MigGeneTargetingHits;

/**
 * This class represents a gene object. This acts as the primary reference object
 * for linking information to a gene. This points to a current Ensembl (and where available a Vega
 * gene). In addition, this object also supports the concept of a primary Build gene. This points
 * this to a BuildGene which is considered to be the best available knowledge. Typically this is
 * expected to start as an Ensembl BuildGene and migrate to a Vega when one becomes available.
 * 
 * @author dgm
 *
 */
public class MigGene extends AuthenticatedDataObject implements LookupListElement {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		PRIMARY_BUILD_GENE,
		PRIMARY_VEGA_BUILD_GENE,
		PRIMARY_ENSEMBL_BUILD_GENE,
		BUILD_GENE_LINKS,
		CREATOR,
		NAMES,
		CURATED_STATUS,
		SUMMARY_SCORE,
		TARGETING_HITS
	}
	
	private Integer ncbiTaxon; 
    
    private Long primaryBuildGeneId;
    private BuildGene primaryBuildGene;
    
    private boolean isValid;
    
    private String primaryName;
    private String primaryNameSource;
    
    private Long primaryVegaBuildGeneId;
    private BuildGene primaryVegaBuildGene;
    private boolean inCurrentVega;
    
    private Long primaryEnsemblBuildGeneId;
    private BuildGene primaryEnsemblBuildGene;
    private boolean inCurrentEnsembl;
    
    private Collection<BuildGeneLink> buildGeneLinks;
    
    private MigGeneTargetingHits migGeneTargetingHits;
	
	private Map<Long, BuildGene> buildGenes = new HashMap<Long, BuildGene>();
	
	private Map<String, Set<MigGeneName>> sources = new HashMap<String, Set<MigGeneName>>();
	
	private MigCuratedStatus curatedStatus;
	private int summaryScore;	
           
    /**
     * Get the NCBI Taxonomy id number for this Gene. For our (MIG) purposes this will most likely
     * be 10090 (Mus musculus) see, for example:
     *     http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?mode=Info&id=10090
     * for the meaning of the Taxonomy numbers.
     * 
     * @return the integer value which codifies the species of the Build that contained this Gene.
     */
    public Integer getNcbiTaxon() {
        return ncbiTaxon;
    }
    
    /**
     * Set the NCBI taxonomy id for this Gene. This should only be called by rowmappers when it is
     * processing a database record.
     * 
     * @param ncbiTaxon is the integer value codifying the species to which this Gene belongs.
     */
    public void setNcbiTaxon(Integer ncbiTaxon) {
        this.ncbiTaxon = ncbiTaxon;
    }
    
    /**
     * Get the foreign key value for the 'primary' BuildGene. This should be considered to be the best
     * available gene structure for this gene. Its will typically point to an Ensembl gene during the early
     * life of this object and will be switched to a Vega gene when one becomes available.
     *  
     * @return the Long value containing the foreign key for a record in the GNM_GENE_BUILD_GENE table.
     */
    public Long getPrimaryBuildGeneId() {
        return primaryBuildGeneId;
    }
    
    /**
     * Set the foreign key value for the 'primary' BuildGene. This should be considered to be the best
     * available gene structure for this gene. Its will typically point to an Ensembl gene during the early
     * life of this object and will be switched to a Vega gene when one becomes available. This should only
     * be called by rowmappers when it is processing a database record.
     *  
     * @param primaryGeneBuildGeneId is the Long value containing the foreign+ key for a record in
     *     the GNM_GENE_BUILD_GENE table.
     */
    public void setPrimaryBuildGeneId(Long primaryBuildGeneId) {
        this.primaryBuildGeneId = primaryBuildGeneId;
    }                                                                                                                                            
    
    /**
     * Determine if this gene is still valid. The typical scenario for this being set to false is that
     * this gene has been found to be identical to another.
     * 
     * @return a boolean indicating whether or not this gene is still valid.
     */
	public boolean isValid() {
		return isValid;
	}

	/**
     * Set the validity flag for this Gene. The typical scenario for this being set to false is that
     * this gene has been found to be identical to another and this one should no longer be used.
     * 
     * @param isValid is a boolean indicating whether or not this gene is still valid.
     */
	public void setValid(boolean isValid) {
		this.isValid = isValid;
	}       
    
	/**
	 * Get the primary name for this gene. This is denormalised data and is taken directly from a column
	 * in the GNM_GENE table. It should correspond to either the primary name of the associated Ensembl
	 * BuildGene or the Vega BuildGene (if available). 
	 * 
	 * @return a String, typically in the form 'ENSMUSG00000071549' (Ensembl) or 'OTTMUSG00000000549' (Vega)
	 *      representing the name (or more correctly stable_id) of the gene in the respective system. 
	 */
    public String getPrimaryName() {
        return primaryName;
    }
    
	/**
	 * Set the primary name for this gene. This is denormalised data and is assigned directly to a column
	 * in the GNM_GENE table. It should correspond to either the primary name of the associated Ensembl
	 * BuildGene or the Vega BuildGene (if available). This should only be called by a rowmapper while 
	 * processing a database record. 
	 * 
	 * @param primaryName is a String, typically in the form 'ENSMUSG00000071549' (Ensembl) or
	 *      'OTTMUSG00000000549' (Vega) representing the name (or more correctly stable_id) of the gene
	 *      in the respective system. 
	 */
    public void setPrimaryName(String primaryName) {
        this.primaryName = primaryName;
    }
    
    /**
     * Get the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Vega BuildGenes.
     *  
     * @return a Long containing the foreign key value linking to the Vega BuildGene.
     */
    public Long getPrimaryVegaBuildGeneId() {
        return primaryVegaBuildGeneId;
    }
    
    /**
     * Set the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Vega BuildGenes. This should only be called by a rowmapper while processing a
     * database record. 
     *  
     * @param primaryVegaBuildGeneId is a Long containing the foreign key value linking to the Vega BuildGene.
     */
    public void setPrimaryVegaBuildGeneId(Long primaryVegaBuildGeneId) {
        this.primaryVegaBuildGeneId = primaryVegaBuildGeneId;
    }
    
    /**
     * Get the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Ensembl BuildGenes.
     *  
     * @return a Long containing the foreign key value linking to the Ensembl BuildGene.
     */
    public Long getPrimaryEnsemblBuildGeneId() {
        return primaryEnsemblBuildGeneId;
    }
    
    /**
     * Set the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Ensembl BuildGenes. This should only be called by a rowmapper while processing a
     * database record. 
     *  
     * @param primaryEnsemblBuildGeneId is a Long containing the foreign key value linking to the Ensembl BuildGene.
     */
    public void setPrimaryEnsemblBuildGeneId(Long primaryEnsemblBuildGeneId) {
        this.primaryEnsemblBuildGeneId = primaryEnsemblBuildGeneId;
    }                                                                                                                                            

    /**
     * Get the most current Ensembl BuildGene that is associated with this Gene.
     * 
     * @return a BuildGene containing the details of the most current Ensembl BuildGene associated with
     *     this Gene.
     */
	public BuildGene getPrimaryEnsemblBuildGene() {
		return primaryEnsemblBuildGene;
	}

    /**
     * Set the most current Ensembl BuildGene that is associated with this Gene. This should be carried out
     * by the service layer in response to the component map containing a MigGene.Components.PRIMARY_ENSEMBL_BUILD_GENE.
     * 
     * @param primaryEnsemblBuildGene is a BuildGene containing the details of the most current Ensembl
     *     BuildGene associated with this Gene.
     */
	public void setPrimaryEnsemblBuildGene(BuildGene primaryEnsemblBuildGene) {
		this.primaryEnsemblBuildGene = primaryEnsemblBuildGene;
	}

	/**
     * Get the most current Vega BuildGene that is associated with this Gene.
     * 
     * @return a BuildGene containing the details of the most current Vega BuildGene associated with
     *     this Gene.
     */
	public BuildGene getPrimaryVegaBuildGene() {
		return primaryVegaBuildGene;
	}

	/**
     * Set the most current Vega BuildGene that should be associated with this Gene. This should be carried out
     * by the service layer in response to the component map containing a MigGene.Components.PRIMARY_VEGA_BUILD_GENE.
     * 
     * @param primaryVegaBuildGene is a BuildGene containing the details of the most current Vega
     *     BuildGene associated with this Gene.
     */
	public void setPrimaryVegaBuildGene(BuildGene primaryVegaBuildGene) {
		this.primaryVegaBuildGene = primaryVegaBuildGene;
	}
	
	/**
	 * Get the primary BuildGene associated with this Gene. This will typically start off being equal to the
	 * getPrimaryEnsemblBuildGene() and will, when it is available, become equal to getPrimaryVegaBuildGene().
	 * 
	 * @return the primary BuildGene that corresponds to this Gene.
	 */
	public BuildGene getPrimaryBuildGene() {
		return primaryBuildGene;
	}

	/**
	 * Set the primary BuildGene to be associated with this Gene. This will typically start off being equal to the
	 * getPrimaryEnsemblBuildGene() and will, when it is available, become equal to getPrimaryVegaBuildGene().
	 * 
	 * @param primaryBuildGene is the primary BuildGene that is accosiated with this Gene.
	 */
	public void setPrimaryBuildGene(BuildGene primaryBuildGene) {
		this.primaryBuildGene = primaryBuildGene;
	}

	/**
	 * Get the collection of Links to BuildGenes. This effectively provides a link to all of the BuildGenes
	 * that are related to this Gene (including thise that are no longer considered primary). This will be
	 * available if the user requests this item to be filled by the service layer using the ComponentMap
	 * (by specifying MigGene.Components.BUILD_GENE_LINKS)
	 * 
	 * @return the set of BuildGeneLinks or null if this has not been populated by the service layer.
	 */
	public Collection<BuildGeneLink> getBuildGeneLinks() {
		return buildGeneLinks;
	}

	/**
	 * Set the collection of Links to BuildGenes. This will be called by the service layer to populate
	 * this field if the user requests it from the service layer using the ComponentMap
	 * (by specifying MigGene.Components.BUILD_GENE_LINKS)
	 * 
	 * @param buildGeneLinks is the set of BuildGeneLinks that should be associated with this gene.
	 */
	public void setBuildGeneLinks(Collection<BuildGeneLink> buildGeneLinks) {
		this.buildGeneLinks = buildGeneLinks;
	}

	/**
	 * Return a flag that indicates that the Vega identifier associated with this gene is available
	 * in the current public release of Vega. This will typically be false for genes that have been
	 * annotated and are available in otterlace but which have not yet been made public.
	 * 
	 * @return true if the Vega identifier is known in the current public Vega release.
	 */
	public boolean isInCurrentVega() {
		return inCurrentVega;
	}

	/**
	 * Set the flag to indicate that the Vega Id associated with this gene is known in the current
	 * public Vega release/web-site. This is typically only used by the DAO when mapping a database
	 * row.
	 * 
	 * @param inCurrentVega should be set to true if the Vega id associated with this gene is known in
	 *     the public Vega release.
	 */
	public void setInCurrentVega(boolean inCurrentVega) {
		this.inCurrentVega = inCurrentVega;
	}

	/**
	 * Return a flag that indicates that the Ensembl identifier associated with this gene is available
	 * in the current public release of Ensembl. This will typically be false for genes that have been
	 * silently deprecated from Ensembl.
	 * 
	 * @return true if the Ensembl identifier is known in the current public Ensembl release.
	 */
	public boolean isInCurrentEnsembl() {
		return inCurrentEnsembl;
	}

	/**
	 * Set the flag to indicate that the Ensembl Id associated with this gene is known in the current
	 * public Ensembl release/web-site. This is typically only used by the DAO when mapping a database
	 * row.
	 * 
	 * @param inCurrentEnsembl should be set to true if the Ensembl id associated with this gene is known in
	 *     the public Ensembl release.
	 */
	public void setInCurrentEnsembl(boolean inCurrentEnsembl) {
		this.inCurrentEnsembl = inCurrentEnsembl;
	}

	/**
	 * Get a string that indicates where the primaryName parameter was derived from. The current database
	 * contains, for example: mgi-2-ensembl, ottmus from vega build, mgi from vega build,  mgi from ensembl build
	 * mgi-2-vega, ensmus from ensembl build etc.
	 * 
	 * @return a String containing a representation of the source of the primary name data.
	 */
	public String getPrimaryNameSource() {
		return primaryNameSource;
	}

	/**
	 * Set the string that indicates the source of the primaryName of this Gene. This will typically
	 * only be called by a rowmapper in the DAO layer.
	 * 
	 * @param primaryNameSource is a String that can be used to identify the source of the primaryName
	 *     for this gene.
	 */
	public void setPrimaryNameSource(String primaryNameSource) {
		this.primaryNameSource = primaryNameSource;
	}

	public void setBuildGenes(HashMap<Long, BuildGene> buildGenes) {
		this.buildGenes = buildGenes;
	}
	
	/**
	 * @return Returns the buildGenes.
	 */
	public Collection<BuildGene> getBuildGenes() {
		return buildGenes.values();
	}
	
	public void addBuildGene(BuildGene buildGene) {
		buildGenes.put(buildGene.getId(), buildGene);
		if(primaryBuildGene != null) {
			buildGenes.put(primaryBuildGene.getId(), primaryBuildGene);
		}
	}
	
	public Map<String, Set<MigGeneName>> getSources() {
		return sources;
	}

	public void setSources(Map<String, Set<MigGeneName>> sources) {
		this.sources = sources;
	}
	
	/**
	 * Get one of the many names that are associated with this gene. The source name
	 * is that used by Ensembl when referring to external names. It contains such
	 * 'peaches' as RefSeq_dna when referring to a mouse accession number. Please use
	 * the other accessor functions where possible in case these ensembl'isms ever 
	 * change.
	 * 
	 * @param sourceName is the string representing an ensembl term that corresponds
	 *      to a naming scheme for genes.
	 *      
	 * @return the string containing the requested name type for this gene. If no name
	 *      of this type is accociated then an empty string is returned. Iff more than
	 *      one name of this type is available the first will be returned.
	 */
	private String getNameBySource(String sourceName){
		String name = "";
		
		Set<MigGeneName> source = sources.get(sourceName);
		
		if(source != null && !source.isEmpty()) {
			name = source.iterator().next().getName();
		}
		
		return name;
	}
	
	/**
	 * Return the list of names associated with a named source. For example 'Interpro' will
	 * yield a set of strings of the form 'IP00001'.
	 * 
	 * @param sourceName is the source of the names
	 * @return an ArrayList containing all of the names associated with this gene originating
	 *     from the supplied source. This will be an empty array (NB. not null) if no entries
	 *     exist for that source.
	 */
	private Collection<String> getNamesBySource(String sourceName){
		
		Collection<String> names = new ArrayList<String>();
		
		Set<MigGeneName> source = sources.get(sourceName);
		if(source != null) {
			for(MigGeneName geneName : source) {
				names.add(geneName.getName());
			}
		}
		
		return names;
	}
	
	/**
	 * Return an ArrayList of any additional MGI Marker Symbols that are associated with this Gene.
	 * 
	 * @return an ArrayList contating all of the names (encoded as String's) found in the MGI 
	 *     Additional Names list.
	 */
	public Collection<String> getMgiAdditionalSymbols() {
		return getNamesBySource("MgiMarkerSymbol");
	}
	
	/**
	 * Return an ArrayList of Interpro names that are associated with this Gene.
	 * 
	 * @return an ArrayList containing all of the Interpro symbols associated with this
	 *     gene. Each element is a String object.
	 */
	public Collection<String> getInterproSymbols() {
		return getNamesBySource("Interpro");
	}
	
	/**
	 * Return an ArrayList of GO term that are associated with this Gene.
	 * 
	 * @return an ArrayList containing all of the GO terms associated with this
	 *     gene. Each element is a String object.
	 */
	public Collection<String> getGoTerms() {
		return getNamesBySource("GO");
	}
	
	/**
	 * Get the MGI identifier associated with this gene.
	 * 
	 * @return a string representing the MGI identifier for this gene or an empty string
	 *     if this name doesn't exist. The string is typically of the form: MGI:1914297
	 */
	public String getMgiId() {
		String id = getNameBySource("MGI_ID");
		return id;
	}
	
	public Collection<String> getMigGeneAliases() {
		HashSet<String> nameSet = new HashSet<String>();
		nameSet.addAll(getNamesBySource("MgiMarkerSymbol"));
		nameSet.addAll(getNamesBySource("MGI"));
		nameSet.addAll(getNamesBySource("MarkerSymbol"));
		nameSet.addAll(getNamesBySource("Ensembl_ID"));
		nameSet.addAll(getNamesBySource("Vega_ID"));
		nameSet.addAll(getNamesBySource("Havana_ID"));
		return nameSet;
	}

	public Integer getSummaryScore() {
		return summaryScore;
	}

	public void setSummaryScore(Integer summaryScore) {
		this.summaryScore = summaryScore;
	}

	public MigCuratedStatus getCuratedStatus() {
		return curatedStatus;
	}

	public void setCuratedStatus(MigCuratedStatus curatedStatus) {
		this.curatedStatus = curatedStatus;
	}
	

	public String getDisplayText() {
		return getPrimaryName();
	}

	public String getElementData() {
		return 
			LookupListUtil.format("geneId", getId()) +
			LookupListUtil.format("locusId", getLocusId()) +
			LookupListUtil.format("chromosome", getChromosomeName()) +
			LookupListUtil.format("start", getChromosomeStart()) +
			LookupListUtil.format("end", getChromosomeEnd()) +
			LookupListUtil.format("buildId", getBuildId())+
			LookupListUtil.format("buildGenePrimaryName", getBuildGenePrimaryName())+
			LookupListUtil.format("primaryName", getPrimaryName())
		;
	}

	public void setDisplayText(String displayText) {
		// TODO Auto-generated method stub	
	}

	public void setElementData(String elementData) {
		// TODO Auto-generated method stub	
	}
	

	public String getBuildGenePrimaryName() {
		return primaryBuildGene.getPrimaryName();
	}
	
	/**
	 * Get the foreign key value for the link to the Build that contained the original data for this gene (this is a 
	 * build in the sense of and Ensembl/Vega/OtterLace build).
	 * 
	 * @return a Long containing the foreign key value that can be used to link this Gene to its associiated Build.
	 */
	public Long getBuildId() {
		return primaryBuildGene.getBuildId();
	}
	
	public int getChromosomeEnd() {
		return primaryBuildGene.getLocus().getChromosomeEnd();
	}

	public String getChromosomeName() {
		return primaryBuildGene.getLocus().getChromosomeName();
	}

	public int getChromosomeStart() {
		return primaryBuildGene.getLocus().getChromosomeStart();
	}

	public MigGeneTargetingHits getMigGeneTargetingHits() {
		return migGeneTargetingHits;
	}

	public void setMigGeneTargetingHits(MigGeneTargetingHits migGeneTargetingHits) {
		this.migGeneTargetingHits = migGeneTargetingHits;
	}
	
	public Long getLocusId() {
		return primaryBuildGene.getLocusId();
	}
}