/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene;

import java.util.Collection;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;

/**
 * This class encodes a transcript for a gene. This is used to encode the 1 or more alternate
 * splicings of a gene.
 * 
 * @author dgm
 *
 */
public class Transcript extends AuthenticatedDataObject  {
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		EXONS,
		BUILD_GENE,
		LOCUS
	}

    private Long locusId;
    private Locus locus;
    
    private int exonCount;
    private String primaryName; 
    
    private Long buildGeneId;
    private BuildGene buildGene;
    
    private Collection<Exon> exons;

    /**
     * Get the collection of Exons associated with this transcript. This may be null if this component of 
     * the object has not been populated by the service layer. (This should be requested by specifiying 
     * Transcript.Component.EXONS.
     * 
     * @return the collection of exons that are associated with this Transcript.
     */
    public Collection<Exon> getExons() {
		return exons;
	}

    /**
     * Set the collection of Exon objects that should be associated with this Transcript. This should only be
     * called by the service or DAO layers.
     *   
     * @param exons is the Collection holding the set of Exons associated with this Transcript.
     */
	public void setExons(Collection<Exon> exons) {
		this.exons = exons;
	}

	/**
     * Get the foreign key value for the Locus that is associated with this Transcript. (This covers the
     * genomic region of the entire transcript (most 5' to most 3' exon's).
     * 
     * @return the foreign key that references the Locus of this Transcript.
     */
	public Long getLocusId() {
        return locusId;
    }
    
	/**
     * Set the foreign key value for the Locus that is associated with this Transcript. (This covers the
     * genomic region of the entire transcript (most 5' to most 3' exon's).
     * 
     * @param locusId is the value for the foreign key that references the Locus of this Transcript.
     */
    public void setLocusId(Long locusId) {
        this.locusId = locusId;
    }
    
    /**
     * Get the exon count for this transcript. This is taken from a pre-computed field in the database to 
     * avoid the overhead of fetching all of the exons to compute this count (without actually needing the
     * details of the exons).
     * 
     * @return the number of exons in this transcript.
     */
    public int getExonCount() {
        return exonCount;
    }
    
    /**
     * Set the exon count for this transcript. This should only be called by the service or DAO layer.
     * 
     * @param exonCount is the number of exons in this transcript.
     */
    public void setExonCount(int exonCount) {
        this.exonCount = exonCount;
    }
    
    /**
     * Get the 'primary' name for this transcript. This is a String, typically of the form 'ENSMUST00000083517'
     * if the transcript was sourced from Ensembl or 'OTTMUST00000083517' if the data was sourced from Vega. It can
     * be used to search for the transcript in either of those systems.
     * 
     * @return a String containing an identifier for the transcript that can be considered a unique idenntifier in
     *      some external system (such as Vega or Ensembl). There is currently no way of determining (other than 
     *      interpreting the string) which system this identifier belongs to.
     */
    public String getPrimaryName() {
        return primaryName;
    }
    
    /**
     * Set the 'primary' name for this transcript. This is a String, typically of the form 'ENSMUST00000083517'
     * if the transcript was sourced from Ensembl or 'OTTMUST00000083517' if the data was sourced from Vega. 
     * 
     * @param primaryName is a String containing an identifier for the transcript that can be considered a
     *      unique idenntifier in some external system (such as Vega or Ensembl).
     */
    public void setPrimaryName(String primaryName) {
        this.primaryName = primaryName;
    }                                                                                                                                            
 
    /**
     * Get the Locus object for this Transcript. This defines its location (in terms of chromosome, strand, start and end
     * coordinates). This may be null if the service layer has not been requested to populate this object. 
     * @see Locus
     * @return a Locus object or null if this has not been set by the service layer.
     */
	public Locus getLocus() {
		return locus;
	}

	/**
	 * Set the location of this Transcript. This should only be called by the service layer or DAO to link this
	 * exon to a Locus object. 
	 * 
	 * @param locus is a Locus object that identifies the position of this transcript on a Gene Build.
	 */
	public void setLocus(Locus locus) {
		this.locus = locus;
	}

	/**
	 * Get the primary key for the BuildGene that 'owns' this Transcript.
	 * 
	 * @return a Long value that contains the primary key for the BuildGene that 'owns' the transcript.
	 */
	public Long getBuildGeneId() {
		return buildGeneId;
	}
	
	/**
	 * Set the primary key for the BuildGene that 'owns' this Transcript. This should only be called by the
	 * DAO or service layers.
	 * 
	 * @param buildGeneId is a Long value that contains the primary key for the BuildGene that 'owns' the transcript.
	 */
	public void setBuildGeneId(Long buildGeneId) {
		this.buildGeneId = buildGeneId;
	}

	/**
	 * Get the BuildGene object that 'owns' this Transcript. The population of this data field should be requested
	 * by setting Transcript.Component.BUILD_GENE when requesting this data from the service layer.
	 *  
	 * @return The BuildGene object that 'owns' this Transcript or null if the service layer has not populated
	 *     this element.
	 */
	public BuildGene getBuildGene() {
		return buildGene;
	}

	/**
	 * Set the BuildGene object that 'owns' this Transcript. This will be called by the service layer if the user has
	 * added a Transcript.Component.BUILD_GENE to the required component map.
	 *  
	 * @return The BuildGene object that 'owns' this Transcript or null if the service layer has not populated
	 *     this element.
	 */
	public void setBuildGene(BuildGene buildGene) {
		this.buildGene = buildGene;
	}
}