/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *  3. The name of the author may not be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.dao;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.biojava.dasobert.das.FeatureThread;
import org.biojava.dasobert.dasregistry.Das1Source;
import org.biojava.dasobert.eventmodel.FeatureEvent;
import org.biojava.dasobert.eventmodel.FeatureListener;

import uk.ac.sanger.mig.gene.BacImport;


public class BacImportDaoDasImpl implements BacImportDao {
  private String srcUrl;
  private Collection<String> requiredTypes;
  private Double timeLimit;
  private long sleepPeriod = 1000;
  private Long assemblyId;
  private Long bacTypeId;

  protected Log logger = LogFactory.getLog(this.getClass());

  /* (non-Javadoc)
   * @see uk.ac.sanger.mig.gene.dao.BacImportDao#fetchByChromosome(java.lang.String)
   */
  public Collection<BacImport> fetchByChromosome(String query) {
    // hack around the low number of features that Dasobert will pull back
    FeatureThread.MAX_NR_FEATURES = Integer.MAX_VALUE;

    Collection<BacImport> bacs = new LinkedList<BacImport>();

    Das1Source dasSource = new Das1Source();
    dasSource.setUrl(srcUrl);

    BacFeatureListener listener = new BacFeatureListener(bacs,requiredTypes,query);
    FeatureThread requestThread = new FeatureThread(query, dasSource);

    requestThread.addFeatureListener(listener);
    requestThread.start();

    long startTime = System.currentTimeMillis();

    while (true){
      double timeElapsed = (System.currentTimeMillis() - startTime)/1000d;
      logger.info(timeElapsed  + " seconds have passed, "+bacs.size()+" retrieved");

      if (listener.isResponseReceived()) {
        logger.info("Response received");
        break;
      }

      if ( timeLimit != null && timeElapsed > timeLimit) {
        requestThread.stop();
        throw new RuntimeException("DAS timeout after "+timeElapsed+" seconds. Source: "+srcUrl+" Query: "+query);
      }

      try {
        Thread.sleep(sleepPeriod);
      } catch (InterruptedException e) {
        throw new RuntimeException(e);
      }
    }

    requestThread.stop();
    return bacs;
  }

  private class BacFeatureListener implements FeatureListener {
    private Collection<BacImport> bacs;
    private String chromosome;
    private Collection<String> requiredTypes;
    private boolean responseReceived = false;
    public boolean isResponseReceived() {return responseReceived;}

    public BacFeatureListener (Collection<BacImport> bacs, Collection<String> types, String chromosome) {
      this.bacs = bacs;
      this.requiredTypes = types;
      this.chromosome = chromosome;
    }

    public void comeBackLater(FeatureEvent event) {
      // ignore this event
    }

    public void newFeatures(FeatureEvent event) {

      Map<String,String>[] features = event.getFeatures();

      for (int i = 0;i < features.length; i++) {
        Map<String,String> feature = features[i];
        String type  = feature.get("TYPE");

        if (requiredTypes.contains(type)) {
          BacImport bac = mapFeature(feature);
          bacs.add(bac);
        }
      }
      responseReceived = true;
    }
    private BacImport mapFeature(Map<String,String> feature) {
      BacImport bac = new BacImport();

      bac.setChromosome(chromosome);
      bac.setAssemblyId(assemblyId);
      bac.setBacTypeId(bacTypeId);

      bac.setLabel(feature.get("id"));
      bac.setStart(Integer.parseInt(feature.get("START")));
      bac.setStop(Integer.parseInt(feature.get("END")));

      String ori = feature.get("ORIENTATION");

      if (ori == null || ori.trim().equals("")) {
        bac.setChromosomeStrand(null);
      }
      else if (ori.equals("+")) {
        bac.setChromosomeStrand(1);
      }
      else if (ori.equals("-")) {
        bac.setChromosomeStrand(-1);
      }
      else {
        throw new RuntimeException("Could not parse orientation for BAC "+bac.getLabel()+ ", recieved '"+ori+"' from "+srcUrl);
      }

      return bac;
    }
  }

  public void setSrcUrl(String srcUrl) {
    this.srcUrl = srcUrl;
  }

  public void setRequiredTypes(Collection<String> requiredTypes) {
    this.requiredTypes = requiredTypes;
  }

  public void setTimeLimit(Double timeLimit) {
    this.timeLimit = timeLimit;
  }

  public void setSleepPeriod(long sleepPeriod) {
    this.sleepPeriod = sleepPeriod;
  }

  public void setAssemblyId(Long assemblyId) {
    this.assemblyId = assemblyId;
  }

  public void setBacTypeId(Long bacTypeId) {
    this.bacTypeId = bacTypeId;
  }

  public Long getAssemblyId() {
    return assemblyId;
  }

  public Long getBacTypeId() {
    return bacTypeId;
  }


}
