package uk.ac.sanger.mig.gene.dao;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collection;
import java.util.LinkedList;

import org.biojava.bio.BioException;
import org.biojava.bio.program.gff.GFFDocumentHandler;
import org.biojava.bio.program.gff.GFFParser;
import org.biojava.bio.program.gff.GFFRecord;
import org.biojava.utils.ParserException;

import uk.ac.sanger.mig.gene.BacImport;

public class BacImportDaoFileImpl implements BacImportDao {
  private String fileName;
  private Long assemblyId;
  private Long bacTypeId;

  public String getFileName() {
    return fileName;
  }

  public void setFileName(String file) {
    this.fileName = file;
  }

  public Long getAssemblyId() {
    return assemblyId;
  }

  public void setAssemblyId(Long assemblyId) {
    this.assemblyId = assemblyId;
  }

  public Long getBacTypeId() {
    return bacTypeId;
  }

  public void setBacTypeId(Long bacTypeId) {
    this.bacTypeId = bacTypeId;
  }

  public Collection<BacImport> fetchByChromosome(String chromosome) {
    Collection<BacImport> bacs = new LinkedList<BacImport>();
    
    BufferedReader in = null;
    
    File file = new File(fileName);
    if (!file.canRead()) {
      throw new IllegalArgumentException("File cannot be read: "+fileName);
    }
    
    
    try {
      in = new BufferedReader(new FileReader(file));
      GFFDocumentHandler handler = new BacDocumentHandler(chromosome,bacs);
      GFFParser parser = new GFFParser();
      parser.parse(in, handler);
    }
    catch (FileNotFoundException e) {
      throw new RuntimeException(e);
    }
    catch (IOException e) {
      throw new RuntimeException(e);
    }
    catch(ParserException e) {
      throw new RuntimeException(e);
    }
    catch(BioException e) {
      throw new RuntimeException(e);
    }
    finally {
      if (in != null) {
        try {
          in.close();
        } catch (IOException e) {
          throw new RuntimeException(e);
        }
      }
    }
    
    return bacs;
  }
  /**
   * Handler for GFF documents provided by our colleagues in t87.
   * Almost follows the <a href="http://www.sanger.ac.uk/Software/formats/GFF/GFF_Spec.shtml">GFF spec</a>
   * 
   * They seem to have used the seqname field to encode chromosome information as 6 colon seperated fields:
   * 
   * chromosome:NCBIM37:X:1:166650296:1
   * 
   * We only care about the 3rd field, which is the chromosome 
   *
   */
  private class BacDocumentHandler implements GFFDocumentHandler{
    private String chromosome;
    private Collection<BacImport> bacs;
    
    public BacDocumentHandler(String chromosome, Collection<BacImport> bacs) {
      this.chromosome = chromosome;
      this.bacs = bacs;
    }
    
    //discard these events
    public void commentLine(String comment) {}
    public void endDocument() {}
    public void startDocument(String locator) {}
    
    @SuppressWarnings("unchecked")
    public void recordLine(GFFRecord gffRecord) {
      
      String[] splitSeqName = gffRecord.getSeqName().split(":");
      if (splitSeqName.length != 6 || splitSeqName[2].length() < 1 ) {
        throw new RuntimeException("Could not parse seq name field to find chromosome: "+gffRecord.getSeqName());
      }
      
      if (!splitSeqName[2].equals(chromosome)) return;
      
      Collection<Object> id = (Collection<Object>)gffRecord.getGroupAttributes().get("ID");
      if (id == null || id.size() != 1) {
        throw new RuntimeException("Could not parse bac name from gff record: "+gffRecord.toString());
      }
      
      BacImport bac = new BacImport();
      
      bac.setBacTypeId(bacTypeId);
      bac.setAssemblyId(assemblyId);
      bac.setChromosome(chromosome);
      
      bac.setLabel(id.iterator().next().toString());
      bac.setStart(gffRecord.getStart());
      bac.setStop(gffRecord.getEnd());
      
      bac.setChromosomeStrand(gffRecord.getStrand().getValue());
      
      bacs.add(bac);
    }
  }
    
    

}
