/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.dao;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.gene.BuildGene;
import uk.ac.sanger.mig.gene.BuildGeneSearchBean;
import uk.ac.sanger.mig.gene.MigGene;

public class BuildGeneDaoSpringImpl extends AbstractDaoSpringImpl<BuildGene> implements BuildGeneDao {

	private PaginatedSearchUtils<BuildGene> paginator;

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, BuildGene buildGene) {
		sql.addLongParam(buildGene.getBuildId(), "BUILD_ID");
		sql.addLongParam(buildGene.getLocusId(), "LOCUS_ID");
		sql.addStringParam(buildGene.getFinishingStatus(), "IS_FINISHED");
		sql.addIntParam(buildGene.getTranscriptCount(), "TRANSCRIPT_COUNT");
		sql.addIntParam(buildGene.getTotalExonCount(), "TOTAL_EXON_COUNT");
		sql.addIntParam(buildGene.getUniqueExonCount(), "UNIQUE_EXON_COUNT");
		sql.addStringParam(buildGene.getPrimaryName(), "PRIMARY_NAME");
		sql.addLongParam(buildGene.getCheckNumber(), "CHECK_NUMBER");
		sql.addStringParam(buildGene.getEditedBy(), "EDITED_BY");
		sql.addDateParam(buildGene.getEditDate(), "EDIT_DATE");
		sql.addDateParam(buildGene.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(buildGene.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(buildGene.getId(), "ID");
	}

	public Collection<BuildGene> fetchAll(){
		String sql = "SELECT * FROM gnm_gene_build_gene";		

		Collection<BuildGene> geneBuilds = getSimpleJdbcTemplate().query(sql, getRowMapper());

		return (geneBuilds == null) ? new ArrayList<BuildGene>() : geneBuilds;
	}

	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.gene.dao.BuildGeneDao#search(uk.ac.sanger.mig.gene.BuildGeneSearchBean)
	 */
	public Collection<BuildGene> search(BuildGeneSearchBean buildGeneSearchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		String selectSql  = "select a.* ";
		String fromSql = " from " + getTableName() + " a ";

		String whereSql = " where 1=1 ";

		if(buildGeneSearchBean.getCreatedDateFrom() != null) {
			whereSql += " and a.created_date >= ? ";
			bindVariables.add(defaultDateFormat.format(buildGeneSearchBean.getCreatedDateFrom()));
		}

		if(buildGeneSearchBean.getCreatedDateTo() != null) {
			whereSql += " and a.created_date <= ? ";
			bindVariables.add(defaultDateFormat.format(buildGeneSearchBean.getCreatedDateTo()));
		}

		if(buildGeneSearchBean.getPrimaryName() != null) {
			whereSql += " and a.primary_name = ? ";
			bindVariables.add(buildGeneSearchBean.getPrimaryName());
		}

		String orderBySql = "";

		if(buildGeneSearchBean.getOrderBy() != null && buildGeneSearchBean.getOrderBy().length() > 0) {
			orderBySql += " order by " + buildGeneSearchBean.getOrderBy();
			if(buildGeneSearchBean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}

		String sql = selectSql + fromSql + whereSql + orderBySql;	

		return paginator.fetchSearchResults(sql, bindVariables, buildGeneSearchBean, getRowMapper());
	}

	public void setPaginator(PaginatedSearchUtils<BuildGene> paginator) {
		this.paginator = paginator;
	}

	public Collection<BuildGene> fetchAllForMigGene(MigGene migGene) {
		
		String sql = "SELECT gg.id  gene_build_gene_id, gg.*, l.* " +
		"FROM gnm_gene_2_gene_build_gene ggg " +
		"JOIN gnm_gene_build_gene gg on(gg.id=ggg.gene_build_gene_id) " +
		"JOIN gnm_gene g on(g.id=ggg.gene_id) " +
		"LEFT JOIN gnm_locus l on(gg.locus_id=l.id) " +
		"WHERE g.id = ? and ggg.is_valid=1";
		
		Collection<BuildGene> buildGenes = getSimpleJdbcTemplate().query(sql, getRowMapper(), migGene.getId());

		if (buildGenes != null) {
			for(BuildGene buildGene : buildGenes) {
				migGene.addBuildGene(buildGene);
			}
		}

		return buildGenes;
	}
	

	public Collection<BuildGene> fetchByBuildIdAndDesignRequestId(Long buildId, Long designRequestId) {
		
		Object[] args = new Long[2];
		args[0]= buildId;
		args[1]= designRequestId;
		String sql = 
			"select gnm_gene_build_gene.*, gnm_locus.* from gnm_gene, arq_allele_request, arq_design_request, " +
			" gnm_gene_2_gene_build_gene, gnm_gene_build_gene, gnm_locus where arq_design_request.allele_request_id = arq_allele_request.id and " +
			" arq_allele_request.gene_id = gnm_gene.id and gnm_gene.id = gnm_gene_2_gene_build_gene.gene_id and" +
			" gnm_gene_2_gene_build_gene.gene_build_gene_id = gnm_gene_build_gene.id and gnm_gene_build_gene.locus_id = gnm_locus.id and gnm_gene_build_gene.build_id = ? and arq_design_request.id = ?";

		Collection<BuildGene> geneBuildGenes = 
			getSimpleJdbcTemplate().query(sql, getRowMapper(), args);

		return geneBuildGenes == null ? new ArrayList<BuildGene>() : geneBuildGenes;

	}
}