/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.dao;

import java.util.ArrayList;
import java.util.Collection;

import org.springframework.dao.IncorrectResultSizeDataAccessException;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.gene.Locus;
import uk.ac.sanger.mig.gene.LocusSearchBean;

public class LocusDaoSpringImpl extends AbstractDaoSpringImpl<Locus> implements LocusDao {

	private PaginatedSearchUtils<Locus> paginator;

	public void setPaginator(PaginatedSearchUtils<Locus> paginator) {
		this.paginator = paginator;
	}

	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, Locus locus) {
		sql.addStringParam(locus.getChromosomeName(), "CHR_NAME");
		sql.addIntParam(locus.getChromosomeStart(), "CHR_START");
		sql.addIntParam(locus.getChromosomeEnd(), "CHR_END");
		sql.addIntParam(locus.getChromosomeStrand(), "CHR_STRAND");
		sql.addLongParam(locus.getAssemblyId(), "ASSEMBLY_ID");
		sql.addStringParam(locus.getType(), "TYPE");
		sql.addLongParam(locus.getCheckNumber(), "CHECK_NUMBER");
		sql.addStringParam(locus.getEditedBy(), "EDITED_BY");
		sql.addDateParam(locus.getEditDate(), "EDIT_DATE");
		sql.addDateParam(locus.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(locus.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(locus.getBuildId(), "BUILD_ID");
		sql.addLongParam(locus.getId(), "ID");
	}

  public Locus fetchByPositionAndAssembly(String chromosome, Integer start, Integer stop, Integer strand, Long assemblyId) {
    String sql = "select * from "+getTableName()+
      " where CHR_NAME = ? "+
      " and CHR_START = ?"+
      " and CHR_END = ?"+
      " and CHR_STRAND = ?"+
      " and ASSEMBLY_ID = ?";
    
    Locus locus = null; 
    
    try {
      locus = getSimpleJdbcTemplate().queryForObject(sql, getRowMapper(), chromosome,start,stop,strand,assemblyId);
    }
    catch (IncorrectResultSizeDataAccessException e) {
      if (e.getActualSize() > 1) {
        throw e;
      }
    }
    
    return locus;
  }
	
	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.gene.dao.LocusDao#search(uk.ac.sanger.mig.gene.LocusSearchBean)
	 */
	public Collection<Locus> search(LocusSearchBean locusSearchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();
			
		String selectSql  = "select l.* ";
		String fromSql = " from " + getTableName() + " l ";
			
		String whereSql = " where 1=1 ";
			
		if(locusSearchBean.getCreatedDateFrom() != null) {
			whereSql += " and l.created_date >= ? ";
			bindVariables.add(defaultDateFormat.format(locusSearchBean.getCreatedDateFrom()));
		}
			
		if(locusSearchBean.getCreatedDateTo() != null) {
			whereSql += " and l.created_date <= ? ";
			bindVariables.add(defaultDateFormat.format(locusSearchBean.getCreatedDateTo()));
		}
			
		if(locusSearchBean.getChromosomeName() != null) {
			whereSql += " and l.chr_name = ? ";
			bindVariables.add(locusSearchBean.getChromosomeName());
		}
			
		if(locusSearchBean.getChromosomeStart() != null) {
			whereSql += " and l.chr_start = ? ";
			bindVariables.add(locusSearchBean.getChromosomeStart());
		}
			
		if(locusSearchBean.getChromosomeEnd() != null) {
			whereSql += " and l.chr_end = ? ";
			bindVariables.add(locusSearchBean.getChromosomeEnd());
		}
			
		if(locusSearchBean.getChromosomeStrand() != null) {
			whereSql += " and l.chr_strand = ? ";
			bindVariables.add(locusSearchBean.getChromosomeStrand());
		}
			
		if(locusSearchBean.getType() != null) {
			whereSql += " and l.type = ? ";
			bindVariables.add(locusSearchBean.getType());
		}
		
		if (locusSearchBean.getAssemblyId() != null) {
		  whereSql += " and l.assembly_id = ?";
		  bindVariables.add(locusSearchBean.getAssemblyId());
		}
			
		String orderBySql = "";
			
		if(locusSearchBean.getOrderBy() != null && locusSearchBean.getOrderBy().length() > 0) {
			orderBySql += " order by " + locusSearchBean.getOrderBy();
			if(locusSearchBean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}
			
		String sql = selectSql + fromSql + whereSql + orderBySql;	
			
		return paginator.fetchSearchResults(sql, bindVariables, locusSearchBean, getRowMapper());
	}

  public Collection<Locus> fetchByBacId(Long bacId) {
    String sql = "select * from GNM_LOCUS l " +
    "join GNM_BAC_LOCUS bl on bl.locus_id = l.id " +
    "where bl.bac_id = ?";
    return getSimpleJdbcTemplate().query(sql, getRowMapper(), bacId);
  }


}