/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import uk.ac.sanger.mig.common.AbstractDaoSpringImpl;
import uk.ac.sanger.mig.common.PaginatedSearchUtils;
import uk.ac.sanger.mig.database.ParameterizedSqlUpdate;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.MigGeneSearchBean;
import uk.ac.sanger.mig.gene.dao.rowmappers.MigGeneListViewRowMapper;
import uk.ac.sanger.mig.gene.views.MigGeneListView;

public class MigGeneDaoSpringImpl extends AbstractDaoSpringImpl<MigGene> implements MigGeneDao {

	private MigGeneListViewRowMapper migGeneListViewRowMapper;
	private PaginatedSearchUtils<MigGene> pojoPaginator;
	private PaginatedSearchUtils<MigGeneListView> listPaginator;
	
	public MigGeneListViewRowMapper getMigGeneListViewRowMapper() {
		return migGeneListViewRowMapper;
	}

	public void setMigGeneListViewRowMapper(
			MigGeneListViewRowMapper migGeneListViewRowMapper) {
		this.migGeneListViewRowMapper = migGeneListViewRowMapper;
	}
	
	public void setListPaginator(PaginatedSearchUtils<MigGeneListView> listPaginator) {
		this.listPaginator = listPaginator;
	}

	public void setPojoPaginator(PaginatedSearchUtils<MigGene> pojoPaginator) {
		this.pojoPaginator = pojoPaginator;
	}
	
	@Override
	protected void bindVariables(ParameterizedSqlUpdate sql, MigGene migGene) {
		sql.addIntParam(migGene.getNcbiTaxon(), "NCBI_TAXON");
		sql.addLongParam(migGene.getPrimaryBuildGeneId(), "PRIMARY_BUILD_GENE_ID");
		sql.addBooleanParam(migGene.isValid(), "IS_VALID");
		sql.addStringParam(migGene.getPrimaryName(), "PRIMARY_NAME");
		sql.addStringParam(migGene.getPrimaryNameSource(), "PRIMARY_NAME_SOURCE");
		sql.addLongParam(migGene.getPrimaryVegaBuildGeneId(), "PRIMARY_VEGA_BUILD_GENE_ID");
		sql.addBooleanParam(migGene.isInCurrentVega(), "IN_CURRENT_VEGA");
		sql.addLongParam(migGene.getPrimaryEnsemblBuildGeneId(), "PRIMARY_ENSEMBL_BUILD_GENE_ID");
		sql.addBooleanParam(migGene.isInCurrentEnsembl(), "IN_CURRENT_ENSEMBL");
		sql.addLongParam(migGene.getCheckNumber(), "CHECK_NUMBER");
		sql.addStringParam(migGene.getEditedBy(), "EDITED_BY");
		sql.addDateParam(migGene.getEditDate(), "EDIT_DATE");
		sql.addDateParam(migGene.getCreatedDate(), "CREATED_DATE");
		sql.addLongParam(migGene.getCreatorId(), "CREATOR_ID");
		sql.addLongParam(migGene.getId(), "ID");
	}

	public Collection<MigGeneListView> search(MigGeneSearchBean migGeneSearchBean) {
		Collection<Object> bindVariables = new ArrayList<Object>();

		String sql = queryBuilder(migGeneSearchBean, bindVariables, "GNM_GENE_VW_NEW");

		return listPaginator.fetchSearchResults(sql, bindVariables, migGeneSearchBean, migGeneListViewRowMapper);
	}

	public Collection<MigGene> fetch(MigGeneSearchBean migGeneSearchBean) {

		Collection<Object> bindVariables = new ArrayList<Object>();

		String sql = queryBuilder(migGeneSearchBean, bindVariables, "GNM_GENE");

		return pojoPaginator.fetchSearchResults(sql, bindVariables, migGeneSearchBean, getRowMapper());
	}

	public Collection<MigGene> fetchMigGenesForGeneSetId(Long id) {

		String sql = " select g.* from gnm_gene g, gnm_gene_set_2_gene gs where g.id = gs.gene_id and gs.gene_set_id=? ";

		Collection<MigGene> genes = getSimpleJdbcTemplate().query(sql, getRowMapper(), id);

		return (genes == null) ? new ArrayList<MigGene>() : genes;
	}
	
	public int linkToMigGeneSet(Long migGeneId, Long migGeneSetId) {
		String sql = " insert into gnm_gene_set_2_gene(gene_set_id, gene_id) select ?, ? from dual where not exists (select null from gnm_gene_set_2_gene where gene_set_id = ? and gene_id = ?) ";
		return getSimpleJdbcTemplate().update(sql, migGeneSetId, migGeneId, migGeneSetId, migGeneId);
	}

	public Collection<MigGene> fetchByName(String geneName) {
		
		geneName = geneName.replace('*','%');
		geneName = geneName.toUpperCase();
		
		String sql = " select g.* from GNM_GENE g ";
		sql += " where g.uc_primary_name like ? ";
		sql += " or exists (select null from gnm_gene_alias_mv m where m.gene_id = g.id and ";
		sql += " m.uc_name like ? ) ";

		Collection<MigGene> genes = getSimpleJdbcTemplate().query(sql, getRowMapper(), geneName, geneName);

		return (genes == null) ? new ArrayList<MigGene>() : genes;

	}
	
	private String queryBuilder(MigGeneSearchBean migGeneSearchBean, Collection<Object> bindVariables, String tableName) {

		String selectSql = "";
		String fromSql = "";
		String whereSql = "";
		String existsSql = "";
		String dateSql = "";
		String geneSetSql = "";

		if(migGeneSearchBean.getQueryString() == null && migGeneSearchBean.getMigGeneSetId() == null){
			// Simple case. Just selecting from the table based on simple values for the fields of the search bean.		
			selectSql  = " select g.* ";
			fromSql = " from " + tableName + " g ";
			whereSql = " where 1=1 ";
		} else if(migGeneSearchBean.getMigGeneSetId() != null){
			selectSql = " select g.* ";
			fromSql += " from " + tableName + " g, gnm_gene_set_2_gene g2g ";
			whereSql = " where 1=1 ";
			geneSetSql += " and (g.id = g2g.gene_id and g2g.gene_set_id = ?) ";
			bindVariables.add(migGeneSearchBean.getMigGeneSetId());
		} else {
			selectSql = " select g.* ";
			fromSql = " from " + tableName + " g ";
			whereSql = " where ( ";
			existsSql = " or exists (select null from gnm_gene_alias_mv m where m.gene_id = g.id and ( ";

			Collection<String> geneNames = new ArrayList<String>(); 	
			StringTokenizer stringTokenizer = new StringTokenizer(migGeneSearchBean.getQueryString());

			while (stringTokenizer.hasMoreTokens()){
				geneNames.add(stringTokenizer.nextToken());
			}
			int nameCount = 0;
			if(geneNames.size() > 1){
				for(String geneName : geneNames){
					nameCount++;
					geneName = geneName.replace('*','%');
					geneName = geneName.toUpperCase();
					whereSql += " g.uc_primary_name like ? ";
					existsSql += " m.uc_name like ? ";
					if(nameCount < geneNames.size()){
						whereSql += " or ";
						existsSql += " or ";
					}
					bindVariables.add(geneName);
				}
				if(true){
					Object[] duplicatedBindVariables = null;
					duplicatedBindVariables = new Object[bindVariables.toArray().length];
					System.arraycopy(bindVariables.toArray(), 0, duplicatedBindVariables, 0, bindVariables.toArray().length);
					int count = 0;
					while(count < duplicatedBindVariables.length){
						bindVariables.add(duplicatedBindVariables[count]);
						count++;
					}
				}
			}else{
				String geneName = geneNames.iterator().next();
				geneName = geneName.replace('*','%');
				geneName = geneName.toUpperCase();
				whereSql += " g.uc_primary_name like ? ";
				existsSql += " m.uc_name like ? ";
				bindVariables.add(geneName);
				bindVariables.add(geneName);
			}
			whereSql += " ) ";
			existsSql += " )) ";
		} 

		if(migGeneSearchBean.getCreatedDateFrom() != null) {
			dateSql += " and g.created_date >= ? ";
			bindVariables.add(defaultDateFormat.format(migGeneSearchBean.getCreatedDateFrom()));
		}

		if(migGeneSearchBean.getCreatedDateTo() != null) {
			dateSql += " and g.created_date <= ? ";
			bindVariables.add(defaultDateFormat.format(migGeneSearchBean.getCreatedDateTo()));
		}

		String orderBySql = "";

		if(migGeneSearchBean.getOrderBy() != null && migGeneSearchBean.getOrderBy().length() > 0) {
			orderBySql += " order by g." + migGeneSearchBean.getOrderBy();
			if(migGeneSearchBean.isOrderDesc()) {
				orderBySql += " desc ";
			}
		}

		return selectSql + fromSql + whereSql + existsSql + geneSetSql + dateSql + orderBySql;
	}

	public List<Map<String, Object>> fetchPubmedKeywords(String ensemblId) {

		String sql = "select n2.name from gnm_gene_name n1 "+
		"left join gnm_gene_name n2 on(n1.gene_id=n2.gene_id) "+
		"where n1.name = ? and n2.source in ('MarkerSymbol','MgiMarkerSymbol')";

		return getSimpleJdbcTemplate().queryForList(sql, ensemblId);

	}
}