/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package uk.ac.sanger.mig.gene.services;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.Bac;
import uk.ac.sanger.mig.gene.BacImport;
import uk.ac.sanger.mig.gene.BacLocus;
import uk.ac.sanger.mig.gene.BacSearchBean;
import uk.ac.sanger.mig.gene.Locus;
import uk.ac.sanger.mig.gene.LocusSearchBean;
import uk.ac.sanger.mig.gene.dao.BacDao;
import uk.ac.sanger.mig.gene.dao.BacImportDao;
import uk.ac.sanger.mig.gene.dao.BacLocusDao;
import uk.ac.sanger.mig.personnel.Person;

public class BacServiceImpl extends AbstractServiceImpl<Bac> implements BacService {

	private BacDao bacDao;
	private BacLocusDao bacLocusDao;
	private LocusService locusService;
	private Set<BacImportDao> importDaos;

	public void setImportDaos(Set<BacImportDao> importDaos) {
    this.importDaos = importDaos;
  }
	public void setBacDao(BacDao bacDao) {
		this.bacDao = bacDao;
	}
	public void setBacLocusDao(BacLocusDao bacLocusDao) {
    this.bacLocusDao = bacLocusDao;
  }
  public void setLocusService(LocusService locusService) {
    this.locusService = locusService;
  }

  public Long store(Bac bac, Person user) {
		if(bac == null) {
			throw new RuntimeException("Cannot save a null bac");
		}

		return bacDao.store(bac, user);
	}

	public Bac fetchById(Long bacId, ComponentMap requiredComponents) {
		Bac bac = bacDao.fetchById(bacId);

		if(bac != null)
			attachComponents(bac, requiredComponents);

		return bac;
	}

	@Override
	protected void attachComponents(Bac bac, ComponentMap requiredComponents) {
		if(bac == null) {
			throw new RuntimeException("Cannot attach components to null bac");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Bac.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Bac.Component.LOCI)) {
			  bac.setLoci(
			      locusService.fetchByBacId(bac.getId(), requiredComponents.get(Bac.Component.LOCI))
			  );
			}
		}
	}

  public Collection<Bac> search(BacSearchBean bacSearchBean) {
    return bacDao.search(bacSearchBean);
  }

  @Transactional
  public void addLocusToBac(Locus locus, Bac bac, Person person) {
    if (locus == null) throw new IllegalArgumentException("Locus is a required argument");
    if (locus.getId() == null) throw new IllegalArgumentException("Locus must have an ID");
    if (bac == null) throw new IllegalArgumentException("BAC is a required argument");
    if (bac.getId() == null) throw new IllegalArgumentException("Bac must have an ID");

    if (!isBacLinkedToLocus(bac,locus) ) {
      BacLocus bacLocus = new BacLocus();
      bacLocus.setLocusId(locus.getId());
      bacLocus.setBacId(bac.getId());
      bacLocusDao.store(bacLocus,person);
    }

  }
  @Transactional
  public void removeLocusFromBac(Locus locus, Bac bac, Person person) {
    if (locus == null) throw new IllegalArgumentException("Locus is a required argument");
    if (locus.getId() == null) throw new IllegalArgumentException("Locus must have an ID");
    if (bac == null) throw new IllegalArgumentException("BAC is a required argument");
    if (bac.getId() == null) throw new IllegalArgumentException("Bac must have an ID");

    BacLocus bacLocus = bacLocusDao.fetchByBacAndLocus(bac.getId(), locus.getId());
    bacLocusDao.delete(bacLocus, person);
  }

  private boolean isBacLinkedToLocus(Bac bac, Locus locus) {
    return (bacLocusDao.fetchByBacAndLocus(bac.getId(), locus.getId()) != null);
  }

  @Transactional
  public void importBacs(Person person) {
    Collection<String>  chromosomes = new HashSet<String>();
    for (int i = 1;i < 20;i++){chromosomes.add(Integer.toString(i));}
    chromosomes.add("X");
    chromosomes.add("Y");

    for (BacImportDao importSrc : importDaos) {

      Map<String,Bac> knownBacs = getBacsByNameForType(importSrc.getBacTypeId());

      for (String chr : chromosomes) {

        Map<String,Locus> knownLoci = getLociByPositionForAssemblyAndChromosome(chr,importSrc.getAssemblyId());


        Collection<BacImport> bacs = importSrc.fetchByChromosome(chr);
        for (BacImport bacInfo : bacs) {
          importBac(bacInfo,knownBacs,knownLoci,person);
        }
      }
    }
  }

  private Map<String,Locus> getLociByPositionForAssemblyAndChromosome(String chromosome, Long assemblyId) {
    Map<String,Locus> hashedLoci = new HashMap<String, Locus>();

    LocusSearchBean lsb = new LocusSearchBean();
    lsb.setChromosomeName(chromosome);
    lsb.setAssemblyId(assemblyId);

    lsb.setPaginated(false);

    Collection<Locus> loci = locusService.search(lsb);
    for (Locus locus : loci){
      String locusKey = positionString(locus);
      hashedLoci.put(locusKey,locus);
    }

    return hashedLoci;
  }

  private Map<String,Bac> getBacsByNameForType(Long bacType) {
    Map<String,Bac> hashedBacs = new HashMap<String, Bac>();

    BacSearchBean bsb = new BacSearchBean();
    bsb.setBacType(bacType);

    bsb.setPaginated(false);

    Collection<Bac> bacs = search(bsb);
    for (Bac bac : bacs) {
      hashedBacs.put(bac.getName(), bac);
    }

    return hashedBacs;
  }

  private String positionString(Locus l){
    return positionString(l.getChromosomeStart(),l.getChromosomeEnd(),l.getChromosomeStrand());
  }

  private String positionString(Integer start, Integer stop, Integer strand) {
    StringBuilder sb = new StringBuilder();
    sb.append(start).append('#');
    sb.append(stop).append('#');
    sb.append(strand);
    return sb.toString();
  }

  private void importBac(BacImport bacInfo, Map<String,Bac> knownBacs, Map<String,Locus> knownLoci, Person person) {
    Bac bac = getOrCreateBac(bacInfo.getLabel(),bacInfo.getBacTypeId(),knownBacs,person);
    Locus locus = getOrCreateLocus(bacInfo.getChromosome(),bacInfo.getStart(),bacInfo.getStop(),bacInfo.getChromosomeStrand(),bacInfo.getAssemblyId(),knownLoci,person);

    addLocusToBac(locus, bac, person);
  }

  private Bac getOrCreateBac(String name, Long bacTypeId, Map<String,Bac> knownBacs, Person person) {
    Bac bac = knownBacs.get(name);

    if (bac == null) {
      bac = new Bac();
      bac.setName(name);
      bac.setTypeId(bacTypeId);
      store(bac,person);
      knownBacs.put(name,bac);
    }

    return bac;
  }

  private Locus getOrCreateLocus(String chromosome, Integer start, Integer stop, Integer strand, Long assemblyId, Map<String,Locus> knownLoci, Person person) {
    String locusKey = positionString(start, stop, strand);
    Locus locus = knownLoci.get(locusKey);

    if (locus == null) {
      locus = new Locus();

      locus.setChromosomeName(chromosome);
      locus.setChromosomeStart(start);
      locus.setChromosomeEnd(stop);
      locus.setChromosomeStrand(strand);

      locus.setAssemblyId(assemblyId);

      locusService.storeAuthenticated(locus,person);

      knownLoci.put(locusKey,locus);
    }

    return locus;
  }
}
