/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.services;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.Build;
import uk.ac.sanger.mig.gene.BuildGene;
import uk.ac.sanger.mig.gene.BuildGeneSearchBean;
import uk.ac.sanger.mig.gene.Locus;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.Transcript;
import uk.ac.sanger.mig.gene.dao.BuildGeneDao;
import uk.ac.sanger.mig.personnel.Person;

public class BuildGeneServiceImpl extends AbstractServiceImpl<BuildGene> implements BuildGeneService {

	private BuildGeneDao buildGeneDao;

	private BuildService buildService;
	private TranscriptService transcriptService;
	private LocusService locusService;

	public void setBuildGeneDao(BuildGeneDao buildGeneDao) {
		this.buildGeneDao = buildGeneDao;
	}

	public void setBuildService(BuildService buildService) {
		this.buildService = buildService;
	}

	public void setLocusService(LocusService locusService) {
		this.locusService = locusService;
	}

	public void setTranscriptService(TranscriptService transcriptService) {
		this.transcriptService = transcriptService;
	}

	public Long storeAuthenticated(BuildGene buildGene, Person user) {
		if(buildGene == null) {
			throw new RuntimeException("Cannot save a null buildGene");
		}

		return buildGeneDao.store(buildGene, user);
	}

	public BuildGene fetchById(Long buildGeneId, ComponentMap requiredComponents) {
		BuildGene buildGene = buildGeneDao.fetchById(buildGeneId);

		if(buildGene != null)
			attachComponents(buildGene, requiredComponents);

		return buildGene;
	}

	@Override
	protected void attachComponents(BuildGene buildGene, ComponentMap requiredComponents) {
		if(buildGene == null) {
			throw new RuntimeException("Cannot attach components to null buildGene");
		}		

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(BuildGene.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(BuildGene.Component.BUILD)) {
				buildGene.setBuild(fetchBuild(buildGene, requiredComponents.get(BuildGene.Component.BUILD)));
			}

			if(fetchAll || requiredComponents.containsKey(BuildGene.Component.LOCUS)) {
				buildGene.setLocus(fetchLocus(buildGene, requiredComponents.get(BuildGene.Component.LOCUS)));
			}

			if(fetchAll || requiredComponents.containsKey(BuildGene.Component.TRANSCRIPTS)) {
				buildGene.setTranscripts(fetchTranscripts(buildGene, requiredComponents.get(BuildGene.Component.TRANSCRIPTS)));
			}
		}
	}

	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.gene.services.BuildGeneService#search(uk.ac.sanger.mig.gene.BuildGeneSearchBean)
	 */
	public Collection<BuildGene> search(BuildGeneSearchBean buildGeneSearchBean) {
		return buildGeneDao.search(buildGeneSearchBean);
	}

	private Collection<Transcript> fetchTranscripts(BuildGene buildGene, ComponentMap map) {
		return transcriptService.fetchAllTranscriptsForBuildGeneId(buildGene.getId(), map);
	}

	private Locus fetchLocus(BuildGene buildGene, ComponentMap map) {
		return locusService.fetchById(buildGene.getLocusId(), map);
	}

	private Build fetchBuild(BuildGene buildGene, ComponentMap map) {
		return buildService.fetchById(buildGene.getBuildId(), map);
	}

	public void inflateBuildGeneComponents(BuildGene buildGene) {
		if(buildGene == null) {
			throw new RuntimeException("BuildGene cannot be null when inflating components.");
		}

		// set build
		if(buildGene.getBuild() == null) {
			Build build = new Build();
			buildService.inflateBuildComponents(build);
			buildGene.setBuild(build);
		}

		// set locus
		if(buildGene.getLocus() == null){
			Locus locus = new Locus();
			locusService.inflateLocusComponents(locus);
			buildGene.setLocus(locus);
		}

		// set transcripts
		if(buildGene.getTranscripts() == null){
			Collection<Transcript> transcripts = new ArrayList<Transcript>();
			buildGene.setTranscripts(transcripts);
		}
	}
	
	public Collection<BuildGene> fetchAllForMigGene(MigGene migGene, ComponentMap requiredComponents) {
		
		Collection<BuildGene> buildGenes = buildGeneDao.fetchAllForMigGene(migGene);

		if(buildGenes != null)
			for(BuildGene buildGene : buildGenes){
				attachComponents(buildGene, requiredComponents);
			}

		return buildGenes;
	}
	
	public Collection<BuildGene> fetchByBuildIdAndDesignRequestId(Long buildId, Long designRequestId, ComponentMap requiredComponents) {
		
		Collection<BuildGene> buildGenes = buildGeneDao.fetchByBuildIdAndDesignRequestId(buildId, designRequestId);

		if(buildGenes != null)
			for(BuildGene buildGene : buildGenes){
				attachComponents(buildGene, requiredComponents);
			}

		return buildGenes;
	}
}
