/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.services;

import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.Assembly;
import uk.ac.sanger.mig.gene.Build;
import uk.ac.sanger.mig.gene.BuildSearchBean;
import uk.ac.sanger.mig.gene.dao.BuildDao;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class BuildServiceImpl extends AbstractServiceImpl<Build> implements BuildService {

	private BuildDao buildDao;

	private PersonService personService;
	private AssemblyService assemblyService;

	public void setAssemblyService(AssemblyService assemblyService) {
		this.assemblyService = assemblyService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setBuildDao(BuildDao buildDao) {
		this.buildDao = buildDao;
	}

	public Long storeAuthenticated(Build build, Person user) {
		if(build == null) {
			throw new RuntimeException("Cannot save a null build");
		}

		return buildDao.store(build, user);
	}

	public Build fetchById(Long buildId, ComponentMap requiredComponents) {
		Build build = buildDao.fetchById(buildId);

		if(build != null)
			attachComponents(build, requiredComponents);

		return build;
	}

	public Collection<Build> fetchAll(ComponentMap requiredComponents) {
		Collection<Build> builds = buildDao.fetchAll( );

		if(builds != null){
			for(Build build:builds)
				attachComponents(build, requiredComponents);
		}		
		return builds;
	}

	@Override
	protected void attachComponents(Build build, ComponentMap requiredComponents) {
		if(build == null) {
			throw new RuntimeException("Cannot attach components to null build");
		}		

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Build.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Build.Component.ASSEMBLY)) {
				build.setAssembly(fetchAssembly(build, requiredComponents.get(Build.Component.ASSEMBLY)));
			}

			if(fetchAll || requiredComponents.containsKey(Build.Component.CREATOR)) {
				build.setCreator(fetchCreator(build, requiredComponents.get(Build.Component.CREATOR)));
			}
		}
	}

	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.gene.services.BuildService#search(uk.ac.sanger.mig.gene.BuildSearchBean)
	 */
	public Collection<Build> search(BuildSearchBean buildSearchBean) {
		return buildDao.search(buildSearchBean);
	}

	/**
	 * @param build
	 * @param map
	 * @return
	 */
	private Person fetchCreator(Build build, ComponentMap map) {
		return personService.fetchById(build.getCreatorId(), map);
	}


	/**
	 * @param build
	 * @param map
	 * @return
	 */
	private Assembly fetchAssembly(Build build, ComponentMap map) {
		return assemblyService.fetchById(build.getAssemblyId(), map);
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateBuildComponents(Build build) {
		if(build == null) {
			throw new RuntimeException("Build cannot be null when inflating components.");
		}

		// set assembly
		if(build.getAssembly() == null)
			build.setAssembly(new Assembly());
		
		// set creator
		if(build.getCreator() == null) {
			Person creator = new Person();
			personService.inflatePersonComponents(creator);
			build.setCreator(creator);
		}
	}
}
