/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.services;

import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.Assembly;
import uk.ac.sanger.mig.gene.Build;
import uk.ac.sanger.mig.gene.Locus;
import uk.ac.sanger.mig.gene.LocusSearchBean;
import uk.ac.sanger.mig.gene.dao.LocusDao;
import uk.ac.sanger.mig.personnel.Person;

public class LocusServiceImpl extends AbstractServiceImpl<Locus> implements LocusService {

	private LocusDao locusDao;

	private AssemblyService assemblyService;
	private BuildService buildService;

	public void setLocusDao(LocusDao locusDao) {
		this.locusDao = locusDao;
	}

	public void setAssemblyService(AssemblyService assemblyService) {
		this.assemblyService = assemblyService;
	}

	public Long storeAuthenticated(Locus locus, Person user) {
		if(locus == null) {
			throw new RuntimeException("Cannot save a null locus");
		}
		return locusDao.store(locus, user);
	}

	public Locus fetchById(Long locusId, ComponentMap requiredComponents) {
		Locus locus = locusDao.fetchById(locusId);

		if(locus != null)
			attachComponents(locus, requiredComponents);

		return locus;
	}

	@Override
	protected void attachComponents(Locus locus, ComponentMap requiredComponents) {
		if(locus == null) {
			throw new RuntimeException("Cannot attach components to null locus");
		}		

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(Locus.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(Locus.Component.BUILD)) {
				locus.setBuild(fetchBuild(locus, requiredComponents.get(Locus.Component.BUILD)));
			}

			if(fetchAll || requiredComponents.containsKey(Locus.Component.ASSEMBLY)) {
				locus.setAssembly(fetchAssembly(locus, requiredComponents.get(Locus.Component.ASSEMBLY)));
			}

		}
	}

	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.gene.services.LocusService#search(uk.ac.sanger.mig.gene.LocusSearchBean)
	 */
	public Collection<Locus> search(LocusSearchBean locusSearchBean) {
		return locusDao.search(locusSearchBean);
	}

	/**
	 * @param locus
	 * @param map
	 * @return
	 */
	private Build fetchBuild(Locus locus, ComponentMap map) {
		return buildService.fetchById(locus.getBuildId(), map);
	}

	private Assembly fetchAssembly(Locus locus, ComponentMap map) {
		return assemblyService.fetchById(locus.getAssemblyId(), map);
	}

	public BuildService getBuildService() {
		return buildService;
	}

	public void setBuildService(BuildService buildService) {
		this.buildService = buildService;
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateLocusComponents(Locus locus) {
		if(locus == null) {
			throw new RuntimeException("Locus cannot be null when inflating components.");
		}

		// set build
		if(locus.getBuild() == null){
			Build build = new Build();
			buildService.inflateBuildComponents(build);
			locus.setBuild(build);
		}

		// set assembly
		if(locus.getAssembly() == null)
			locus.setAssembly(new Assembly());
	}

  public Collection<Locus> fetchByBacId(Long bacId, ComponentMap componentMap) {
    Collection<Locus> loci = locusDao.fetchByBacId(bacId);
    attachComponents(loci, componentMap);
    return loci;
  }

  public Locus fetchByPositionAndAssembly(String chromosome, Integer start, Integer stop, Integer strand, Long assemblyId, ComponentMap componentMap) {
    Locus locus = locusDao.fetchByPositionAndAssembly(chromosome, start, stop, strand, assemblyId);
    if (locus != null){
      attachComponents(locus, componentMap);
    }
    return locus;
  }	
}
