/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.transaction.annotation.Transactional;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.BuildGene;
import uk.ac.sanger.mig.gene.BuildGeneLink;
import uk.ac.sanger.mig.gene.MigGene;
import uk.ac.sanger.mig.gene.MigGeneName;
import uk.ac.sanger.mig.gene.MigGeneSearchBean;
import uk.ac.sanger.mig.gene.dao.MigGeneDao;
import uk.ac.sanger.mig.gene.views.MigGeneListView;
import uk.ac.sanger.mig.geneevidence.MigCuratedStatus;
import uk.ac.sanger.mig.geneevidence.MigGeneTargetingHits;
import uk.ac.sanger.mig.geneevidence.services.MigCuratedStatusService;
import uk.ac.sanger.mig.geneevidence.services.MigGeneSummaryScoreService;
import uk.ac.sanger.mig.geneevidence.services.MigGeneTargetingHitsService;
import uk.ac.sanger.mig.personnel.Person;
import uk.ac.sanger.mig.personnel.services.PersonService;

public class MigGeneServiceImpl extends AbstractServiceImpl<MigGene> implements MigGeneService {

	private MigGeneDao migGeneDao;
	private BuildGeneService buildGeneService;
	private BuildGeneLinkService buildGeneLinkService;
	private PersonService personService;
	private MigGeneNameService migGeneNameService;
	private MigCuratedStatusService migCuratedStatusService;
	private MigGeneSummaryScoreService migGeneSummaryScoreService;
	private MigGeneTargetingHitsService migGeneTargetingHitsService;

	public void setMigGeneDao(MigGeneDao migGeneDao) {
		this.migGeneDao = migGeneDao;
	}

	public void setBuildGeneService(BuildGeneService buildGeneService) {
		this.buildGeneService = buildGeneService;
	}

	public void setBuildGeneLinkService(BuildGeneLinkService buildGeneLinkService) {
		this.buildGeneLinkService = buildGeneLinkService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public void setMigGeneNameService(MigGeneNameService migGeneNameService) {
		this.migGeneNameService = migGeneNameService;
	}

	public Collection<MigGene> fetchByName(String geneName, ComponentMap componentMap) {

		if(geneName == null) {
			throw new RuntimeException("geneName cannot be bull!");
		}

		Collection<MigGene> migGenes = migGeneDao.fetchByName(geneName);

		attachComponents(migGenes, componentMap);

		return migGenes;
	}

	public Long store(MigGene migGene, Person user) {
		if(migGene == null) {
			throw new RuntimeException("Cannot save a null migGene");
		}

		return migGeneDao.store(migGene, user);
	}

	public MigGene fetchById(Long migGeneId, ComponentMap requiredComponents) {
		MigGene migGene = migGeneDao.fetchById(migGeneId);

		if(migGene != null)
			attachComponents(migGene, requiredComponents);

		return migGene;
	}

	public Collection<MigGene> fetchMigGenesForGeneSetId(Long id, ComponentMap componentMap) {
		Collection<MigGene> migGenes = migGeneDao.fetchMigGenesForGeneSetId(id);

		attachComponents(migGenes, componentMap);

		return migGenes;
	}

	@Transactional
	public int linkToMigGeneSet(Long migGeneId, Long migGeneSetId) {
		return migGeneDao.linkToMigGeneSet(migGeneId, migGeneSetId);
	}

	@Override
	protected void attachComponents(MigGene migGene, ComponentMap requiredComponents) {
		if(migGene == null) {
			throw new RuntimeException("Cannot attach components to null migGene");
		}

		boolean fetchAll = false;

		if(requiredComponents != null) {

			if(requiredComponents.containsKey(MigGene.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.BUILD_GENE_LINKS)) {
				migGene.setBuildGeneLinks(fetchBuildGeneLinks(migGene, requiredComponents.get(MigGene.Component.BUILD_GENE_LINKS)));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.PRIMARY_BUILD_GENE)) {
				migGene.setPrimaryBuildGene(fetchPrimaryBuildGene(migGene, requiredComponents.get(MigGene.Component.PRIMARY_BUILD_GENE)));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.PRIMARY_ENSEMBL_BUILD_GENE)) {
				migGene.setPrimaryEnsemblBuildGene(fetchPrimaryEnsemblBuildGene(migGene, requiredComponents.get(MigGene.Component.PRIMARY_ENSEMBL_BUILD_GENE)));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.PRIMARY_VEGA_BUILD_GENE)) {
				migGene.setPrimaryVegaBuildGene(fetchPrimaryVegaBuild(migGene, requiredComponents.get(MigGene.Component.PRIMARY_VEGA_BUILD_GENE)));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.CREATOR)) {
				migGene.setCreator(fetchCreator(migGene, requiredComponents.get(MigGene.Component.CREATOR)));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.NAMES)) {
				migGene.setSources(fetchMigGeneNames(migGene.getId(), requiredComponents.get(MigGene.Component.NAMES)));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.CURATED_STATUS)) {
				migGene.setCuratedStatus(fetchMigGeneCuratedStatus(migGene.getId(), requiredComponents.get(MigGene.Component.CURATED_STATUS)));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.SUMMARY_SCORE)) {
				migGene.setSummaryScore(fetchMigGeneSummaryScore(migGene.getId()));
			}

			if(fetchAll || requiredComponents.containsKey(MigGene.Component.TARGETING_HITS)) {
				migGene.setMigGeneTargetingHits(fetchTargetingHits(migGene.getId()));
			}
		}
	}

	public Collection<MigGeneListView> search(MigGeneSearchBean migGeneSearchBean) {
		return migGeneDao.search(migGeneSearchBean);
	}

	public Collection<MigGene> fetch(MigGeneSearchBean migGeneSearchBean, ComponentMap requiredComponents) {

		Collection<MigGene> migGenes = migGeneDao.fetch(migGeneSearchBean);

		attachComponents(migGenes, requiredComponents);

		return migGenes;
	}

	private BuildGene fetchPrimaryVegaBuild(MigGene migGene, ComponentMap components) {
		return buildGeneService.fetchById(migGene.getPrimaryVegaBuildGeneId(), components);
	}


	private BuildGene fetchPrimaryEnsemblBuildGene(MigGene migGene, ComponentMap components) {
		return buildGeneService.fetchById(migGene.getPrimaryEnsemblBuildGeneId(), components);
	}


	private BuildGene fetchPrimaryBuildGene(MigGene migGene, ComponentMap components) {
		return buildGeneService.fetchById(migGene.getPrimaryBuildGeneId(), components);
	}


	private Collection<BuildGeneLink> fetchBuildGeneLinks(MigGene migGene, ComponentMap components) {
		return buildGeneLinkService.fetchAllBuildGeneLinksByMigGeneId(migGene.getId(), components);
	}

	private Person fetchCreator(MigGene migGene, ComponentMap components) {
		return personService.fetchById(migGene.getCreatorId(), components);
	}

	private Map<String, Set<MigGeneName>> fetchMigGeneNames(Long migGeneId, ComponentMap components) {
		return migGeneNameService.fetchNamesAndSourcesByMigGeneId(migGeneId, components);
	}

	private MigCuratedStatus fetchMigGeneCuratedStatus(Long migGeneId, ComponentMap components) {
		return migCuratedStatusService.fetchCurrentCuratedStatusByMigGeneId(migGeneId, components);
	}

	private Integer fetchMigGeneSummaryScore(Long migGeneId) {
		return migGeneSummaryScoreService.fetchCurrentScore(migGeneId);
	}

	private MigGeneTargetingHits fetchTargetingHits(Long migGeneId) {
		return migGeneTargetingHitsService.fetchTargetingHitsByMigGeneId(migGeneId);
	}

	/**
	 * Expands null child objects to contain empty new objects
	 *
	 */
	public void inflateMigGeneComponents(MigGene migGene) {
		if(migGene == null) {
			throw new RuntimeException("MigGene cannot be null when inflating components.");
		}

		// set primary gene build gene
		if(migGene.getPrimaryBuildGene() == null) {
			BuildGene primaryGeneBuildGene = new BuildGene();
			buildGeneService.inflateBuildGeneComponents(primaryGeneBuildGene);
			migGene.setPrimaryBuildGene(primaryGeneBuildGene);
		}

		// set primary vega build gene
		if(migGene.getPrimaryVegaBuildGene() == null) {
			BuildGene primaryVegaBuildGene = new BuildGene();
			buildGeneService.inflateBuildGeneComponents(primaryVegaBuildGene);
			migGene.setPrimaryVegaBuildGene(primaryVegaBuildGene);
		}

		// set primary ensembl build gene
		if(migGene.getPrimaryEnsemblBuildGene() == null) {
			BuildGene primaryEnsemblBuildGene = new BuildGene();
			buildGeneService.inflateBuildGeneComponents(primaryEnsemblBuildGene);
			migGene.setPrimaryEnsemblBuildGene(primaryEnsemblBuildGene);
		}

		// set build gene links
		if(migGene.getBuildGeneLinks() == null) {
			Collection<BuildGeneLink> buildGeneLinks = new ArrayList<BuildGeneLink>();
			migGene.setBuildGeneLinks(buildGeneLinks);
		}

		// set mutation nature
		if(migGene.getCreator() == null){
			Person creator = new Person();
			personService.inflatePersonComponents(creator);
			migGene.setCreator(creator);
		}


	}

	public List<String> fetchPubmedKeywords(String ensemblId) {
		List<String> keywordList = new ArrayList<String>();
		List <Map<String, Object>> keywordMapList = migGeneDao.fetchPubmedKeywords(ensemblId);
		Iterator iter = keywordMapList.iterator();
		while(iter.hasNext()){
			Map keywordMap = (Map)iter.next();
			keywordList.add((String)keywordMap.get("NAME"));
		}

		return keywordList;
	}

	public void setMigCuratedStatusService(
			MigCuratedStatusService migCuratedStatusService) {
		this.migCuratedStatusService = migCuratedStatusService;
	}

	public void setMigGeneSummaryScoreService(
			MigGeneSummaryScoreService migGeneSummaryScoreService) {
		this.migGeneSummaryScoreService = migGeneSummaryScoreService;
	}

	public void setMigGeneTargetingHitsService(
			MigGeneTargetingHitsService migGeneTargetingHitsService) {
		this.migGeneTargetingHitsService = migGeneTargetingHitsService;
	}

}
