/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.services;

import java.util.Collection;

import uk.ac.sanger.mig.common.AbstractServiceImpl;
import uk.ac.sanger.mig.common.ComponentMap;
import uk.ac.sanger.mig.gene.BuildGene;
import uk.ac.sanger.mig.gene.Exon;
import uk.ac.sanger.mig.gene.Locus;
import uk.ac.sanger.mig.gene.Transcript;
import uk.ac.sanger.mig.gene.dao.TranscriptDao;
import uk.ac.sanger.mig.personnel.Person;

public class TranscriptServiceImpl extends AbstractServiceImpl<Transcript> implements TranscriptService {
	
	private TranscriptDao transcriptDao;
	
	private LocusService locusService;
	private ExonService exonService;
	private BuildGeneService buildGeneService;
	
	public void setBuildGeneService(BuildGeneService buildGeneService) {
		this.buildGeneService = buildGeneService;
	}

	public void setExonService(ExonService exonService) {
		this.exonService = exonService;
	}

	public void setLocusService(LocusService locusService) {
		this.locusService = locusService;
	}
	
	public void setTranscriptDao(TranscriptDao transcriptDao) {
		this.transcriptDao = transcriptDao;
	}


	public Long storeAuthenticated(Transcript transcript, Person user) {
		if(transcript == null) {
			throw new RuntimeException("Cannot save a null transcript");
		}
		
		return transcriptDao.store(transcript, user);
	}
	
	public Transcript fetchById(Long transcriptId, ComponentMap requiredComponents) {
		Transcript transcript = transcriptDao.fetchById(transcriptId);
		
		if(transcript != null)
			attachComponents(transcript, requiredComponents);
		
		return transcript;
	}

	@Override
	protected void attachComponents(Transcript transcript, ComponentMap requiredComponents) {
		if(transcript == null) {
			throw new RuntimeException("Cannot attach components to null transcript");
		}		
		
		boolean fetchAll = false;
		
		if(requiredComponents != null) {
			
			if(requiredComponents.containsKey(Transcript.Component.ALL_COMPONENTS)) {
				fetchAll = true;
			}
			
			if(fetchAll || requiredComponents.containsKey(Transcript.Component.BUILD_GENE)) {
				transcript.setBuildGene(fetchBuildGene(transcript, requiredComponents.get(Transcript.Component.BUILD_GENE)));
			}
			
			if(fetchAll || requiredComponents.containsKey(Transcript.Component.EXONS)) {
				transcript.setExons(fetchExons(transcript, requiredComponents.get(Transcript.Component.EXONS)));
			}
			
			if(fetchAll || requiredComponents.containsKey(Transcript.Component.LOCUS)) {
				transcript.setLocus(fetchLocus(transcript, requiredComponents.get(Transcript.Component.LOCUS)));
			}
		}
	}

	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.gene.services.TranscriptService#fetchAllTranscriptsForBuildGeneId(java.lang.Long, uk.ac.sanger.mig.common.ComponentMap)
	 */
	public Collection<Transcript> fetchAllTranscriptsForBuildGeneId(Long id, ComponentMap map) {
		Collection<Transcript> transcripts = transcriptDao.fetchAllTranscriptsForBuildGeneId(id);
		
		attachComponents(transcripts, map);
		
		return transcripts;
	}
	
	/**
	 * @param transcript
	 * @param map
	 * @return
	 */
	private Locus fetchLocus(Transcript transcript, ComponentMap map) {
		return locusService.fetchById(transcript.getLocusId(), map);
	}


	/**
	 * @param transcript
	 * @param map
	 * @return
	 */
	private Collection<Exon> fetchExons(Transcript transcript, ComponentMap map) {
		return exonService.fetchAllExonsByTranscriptId(transcript.getId(), map);
	}


	/**
	 * @param transcript
	 * @param map
	 * @return
	 */
	private BuildGene fetchBuildGene(Transcript transcript, ComponentMap map) {
		return buildGeneService.fetchById(transcript.getBuildGeneId(), map);
	}


	/* (non-Javadoc)
	 * @see uk.ac.sanger.mig.gene.services.TranscriptService#fetchAllTranscriptsForExonId(java.lang.Long, uk.ac.sanger.mig.common.ComponentMap)
	 */
	public Collection<Transcript> fetchAllTranscriptsForExonId(Long id, ComponentMap map) {
		Collection<Transcript> transcripts = transcriptDao.fetchAllTranscriptsForExonId(id);
		
		attachComponents(transcripts, map);
		
		return transcripts;
	}
}
								