/*
 * Copyright (c) 2007 Genome Research Ltd.
 *
 * Author: Mouse Informatics Group <team110g@sanger.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package uk.ac.sanger.mig.gene.views;

import java.util.Collection;

import uk.ac.sanger.mig.common.AuthenticatedDataObject;
import uk.ac.sanger.mig.common.ComponentEnum;
import uk.ac.sanger.mig.common.LookupListElement;
import uk.ac.sanger.mig.common.LookupListUtil;
import uk.ac.sanger.mig.gene.BuildGene;
import uk.ac.sanger.mig.gene.BuildGeneLink;

/**
 * This class represents a gene object when it is expected to be displayed in a list view. It is used to store the 
 * results of a rowmapper that is interpreting the results of a query against a view. It should, for this reason, be
 * regarded as read-only.
 * 
 * @author dgm
 *
 */
public class MigGeneListView extends AuthenticatedDataObject implements LookupListElement  {
	
	public enum Component implements ComponentEnum {
		ALL_COMPONENTS,
		PRIMARY_BUILD_GENE,
		PRIMARY_VEGA_BUILD_GENE,
		PRIMARY_ENSEMBL_BUILD_GENE,
		BUILD_GENE_LINKS
	}

    private int ncbiTaxon; 
    
    private Long primaryGeneBuildGeneId;
    private BuildGene primaryGeneBuildGene;
    
    private boolean isValid;
    
    private String primaryName;
	private String primaryNameSource;
	    
    private Long primaryVegaBuildGeneId;
    private BuildGene primaryVegaBuildGene;
    private boolean inCurrentVega;
    
    private Long primaryEnsemblBuildGeneId;
    private BuildGene primaryEnsemblBuildGene;
    private boolean inCurrentEnsembl;
    
    private Collection<BuildGeneLink> buildGeneLinks;

	private Long buildId;
	private Long locusId;
	
	private String finished;
	private int transcriptCount;
	private int totalExonCount;
	private int uniqueExonCount;
	
	private String buildGenePrimaryName;
	
	private String chromosomeName;
	private int chromosomeStart;
	private int chromosomeEnd;
	private int chromosomeStrand;
	
	private String buildName;
	private String buildVersion;
	
	private String ensemblIdString;
	private String vegaIdString;
	
	private String curatedStatus;
	
	private int summaryScore;
	
	private int igtcHits;
	private int omnibankHits;


	/**
     * Get the NCBI Taxonomy id number for this Gene. For our (MIG) purposes this will most likely
     * be 10090 (Mus musculus) see, for example:
     *     http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?mode=Info&id=10090
     * for the meaning of the Taxonomy numbers.
     * 
     * @return the integer value which codifies the species of the Build that contained this Gene.
     */
    public int getNcbiTaxon() {
        return ncbiTaxon;
    }
    
    /**
     * Set the NCBI taxonomy id for this Gene. This should only be called by rowmappers when it is
     * processing a database record.
     * 
     * @param ncbiTaxon is the integer value codifying the species to which this Gene belongs.
     */
    public void setNcbiTaxon(int ncbiTaxon) {
        this.ncbiTaxon = ncbiTaxon;
    }
    
    /**
     * Get the foreign key value for the 'primary' BuildGene. This should be considered to be the best
     * available gene structure for this gene. Its will typically point to an Ensembl gene during the early
     * life of this object and will be switched to a Vega gene when one becomes available.
     *  
     * @return the Long value containing the foreign key for a record in the GNM_GENE_BUILD_GENE table.
     */
    public Long getPrimaryGeneBuildGeneId() {
        return primaryGeneBuildGeneId;
    }
    
    /**
     * Set the foreign key value for the 'primary' BuildGene. This should be considered to be the best
     * available gene structure for this gene. Its will typically point to an Ensembl gene during the early
     * life of this object and will be switched to a Vega gene when one becomes available. This should only
     * be called by rowmappers when it is processing a database record.
     *  
     * @param primaryGeneBuildGeneId is the Long value containing the foreign+ key for a record in
     *     the GNM_GENE_BUILD_GENE table.
     */
    public void setPrimaryGeneBuildGeneId(Long primaryGeneBuildGeneId) {
        this.primaryGeneBuildGeneId = primaryGeneBuildGeneId;
    }                                                                                                                                            
    
    /**
     * Determine if this gene is still valid. The typical scenario for this being set to false is that
     * this gene has been found to be identical to another.
     * 
     * @return a boolean indicating whether or not this gene is still valid.
     */
	public boolean isValid() {
		return isValid;
	}

	/**
     * Set the validity flag for this Gene. The typical scenario for this being set to false is that
     * this gene has been found to be identical to another and this one should no longer be used.
     * 
     * @param isValid is a boolean indicating whether or not this gene is still valid.
     */
	public void setValid(boolean isValid) {
		this.isValid = isValid;
	}       
    
	/**
	 * Get the primary name for this gene. This is denormalised data and is taken directly from a column
	 * in the GNM_GENE table. It should correspond to either the primary name of the associated Ensembl
	 * BuildGene or the Vega BuildGene (if available). 
	 * 
	 * @return a String, typically in the form 'ENSMUSG00000071549' (Ensembl) or 'OTTMUSG00000000549' (Vega)
	 *      representing the name (or more correctly stable_id) of the gene in the respective system. 
	 */
    public String getPrimaryName() {
        return primaryName;
    }
    
	/**
	 * Set the primary name for this gene. This is denormalised data and is assigned directly to a column
	 * in the GNM_GENE table. It should correspond to either the primary name of the associated Ensembl
	 * BuildGene or the Vega BuildGene (if available). This should only be called by a rowmapper while 
	 * processing a database record. 
	 * 
	 * @param primaryName is a String, typically in the form 'ENSMUSG00000071549' (Ensembl) or
	 *      'OTTMUSG00000000549' (Vega) representing the name (or more correctly stable_id) of the gene
	 *      in the respective system. 
	 */
    public void setPrimaryName(String primaryName) {
        this.primaryName = primaryName;
    }
    
    /**
     * Get the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Vega BuildGenes.
     *  
     * @return a Long containing the foreign key value linking to the Vega BuildGene.
     */
    public Long getPrimaryVegaBuildGeneId() {
        return primaryVegaBuildGeneId;
    }
    
    /**
     * Set the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Vega BuildGenes. This should only be called by a rowmapper while processing a
     * database record. 
     *  
     * @param primaryVegaBuildGeneId is a Long containing the foreign key value linking to the Vega BuildGene.
     */
    public void setPrimaryVegaBuildGeneId(Long primaryVegaBuildGeneId) {
        this.primaryVegaBuildGeneId = primaryVegaBuildGeneId;
    }
    
    /**
     * Get the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Ensembl BuildGenes.
     *  
     * @return a Long containing the foreign key value linking to the Ensembl BuildGene.
     */
    public Long getPrimaryEnsemblBuildGeneId() {
        return primaryEnsemblBuildGeneId;
    }
    
    /**
     * Set the foreign key which links this gene to the appropriate (and most current) BuildGene in the
     * set of available Ensembl BuildGenes. This should only be called by a rowmapper while processing a
     * database record. 
     *  
     * @param primaryEnsemblBuildGeneId is a Long containing the foreign key value linking to the Ensembl BuildGene.
     */
    public void setPrimaryEnsemblBuildGeneId(Long primaryEnsemblBuildGeneId) {
        this.primaryEnsemblBuildGeneId = primaryEnsemblBuildGeneId;
    }                                                                                                                                            

    /**
     * Get the most current Ensembl BuildGene that is associated with this Gene.
     * 
     * @return a BuildGene containing the details of the most current Ensembl BuildGene associated with
     *     this Gene.
     */
	public BuildGene getPrimaryEnsemblBuildGene() {
		return primaryEnsemblBuildGene;
	}

    /**
     * Set the most current Ensembl BuildGene that is associated with this Gene. This should be carried out
     * by the service layer in response to the component map containing a MigGene.Components.PRIMARY_ENSEMBL_BUILD_GENE.
     * 
     * @param primaryEnsemblBuildGene is a BuildGene containing the details of the most current Ensembl
     *     BuildGene associated with this Gene.
     */
	public void setPrimaryEnsemblBuildGene(BuildGene primaryEnsemblBuildGene) {
		this.primaryEnsemblBuildGene = primaryEnsemblBuildGene;
	}

	/**
     * Get the most current Vega BuildGene that is associated with this Gene.
     * 
     * @return a BuildGene containing the details of the most current Vega BuildGene associated with
     *     this Gene.
     */
	public BuildGene getPrimaryVegaBuildGene() {
		return primaryVegaBuildGene;
	}

	/**
     * Set the most current Vega BuildGene that should be associated with this Gene. This should be carried out
     * by the service layer in response to the component map containing a MigGene.Components.PRIMARY_VEGA_BUILD_GENE.
     * 
     * @param primaryVegaBuildGene is a BuildGene containing the details of the most current Vega
     *     BuildGene associated with this Gene.
     */
	public void setPrimaryVegaBuildGene(BuildGene primaryVegaBuildGene) {
		this.primaryVegaBuildGene = primaryVegaBuildGene;
	}
	
	/**
	 * Get the primary BuildGene associated with this Gene. This will typically start off being equal to the
	 * getPrimaryEnsemblBuildGene() and will, when it is available, become equal to getPrimaryVegaBuildGene().
	 * 
	 * @return the primary BuildGene that corresponds to this Gene.
	 */
	public BuildGene getPrimaryGeneBuildGene() {
		return primaryGeneBuildGene;
	}

	/**
	 * Set the primary BuildGene to be associated with this Gene. This will typically start off being equal to the
	 * getPrimaryEnsemblBuildGene() and will, when it is available, become equal to getPrimaryVegaBuildGene().
	 * 
	 * @param primaryGeneBuildGene is the primary BuildGene that is accosiated with this Gene.
	 */
	public void setPrimaryGeneBuildGene(BuildGene primaryGeneBuildGene) {
		this.primaryGeneBuildGene = primaryGeneBuildGene;
	}

	/**
	 * Get the collection of Links to BuildGenes. This effectively provides a link to all of the BuildGenes
	 * that are related to this Gene (including thise that are no longer considered primary). This will be
	 * available if the user requests this item to be filled by the service layer using the ComponentMap
	 * (by specifying MigGene.Components.BUILD_GENE_LINKS)
	 * 
	 * @return the set of BuildGeneLinks or null if this has not been populated by the service layer.
	 */
	public Collection<BuildGeneLink> getBuildGeneLinks() {
		return buildGeneLinks;
	}

	/**
	 * Set the collection of Links to BuildGenes. This will be called by the service layer to populate
	 * this field if the user requests it from the service layer using the ComponentMap
	 * (by specifying MigGene.Components.BUILD_GENE_LINKS)
	 * 
	 * @param buildGeneLinks is the set of BuildGeneLinks that should be associated with this gene.
	 */
	public void setBuildGeneLinks(Collection<BuildGeneLink> buildGeneLinks) {
		this.buildGeneLinks = buildGeneLinks;
	}

	/**
	 * Set the foreign key value for the Build that contained the original data for this gene. 
	 * 
	 * @param buildId is the foreign key that links this Gene record to the Ensemble/Vega/etc. build that was the
	 * original source of the data.
	 */
	public void setBuildId(Long buildId) {
		this.buildId = buildId;	
	}
	
	/**
	 * Get the foreign key value for the link to the Build that contained the original data for this gene (this is a 
	 * build in the sense of and Ensembl/Vega/OtterLace build).
	 * 
	 * @return a Long containing the foreign key value that can be used to link this Gene to its associiated Build.
	 */
	public Long getBuildId() {
		return buildId;
	}
	
	/**
	 * Set the foreign key value for the Locus that identifies the position of this Gene in the Genome. 
	 * 
	 * @param locusId is the foreign key that links this Gene record to the Ensemble/Vega/etc. Locus.
	 */
	public void setLocusId(Long locusId){
		this.locusId = locusId;
	}
	
	/**
	 * Get the foreign key value that links this Gene to its associated Locus as defined by the Ensembl/Vega/OtterLace 
	 * build data that it is asscoiated with.
	 * 
	 * @return a Long containing the foreign key value that can be used to link this Gene to its associated Locus.
	 */
	public Long getLocusId() {
		return locusId;
	}
	
	/**
	 * Determine if the Gene is considered to be finished. This is based on a slightly dubious algorithim by (avc) that 
	 * looks to see if the (or perhaps any) clone covering the genomic span covered by this gene is considered finished
	 * (in a sequencing sense).
	 * 
	 * @return yes if the Gene is considered 'finished' by the above criteria.
	 */
	public String isFinished() {
		return finished;
	}

	/**
	 * Set the String value which indicates whether WTSI considers the underlying sequence to be 'finished'
	 * 
	 * @param finished should be set to yes if the sequence underlying the Gene is considered to be finished.
	 */
	public void setFinished(String finished) {
		this.finished = finished;
	}
	
	/**
	 * Return a flag that indicates that the Vega identifier associated with this gene is available
	 * in the current public release of Vega. This will typically be false for genes that have been
	 * annotated and are available in otterlace but which have not yet been made public.
	 * 
	 * @return true if the Vega identifier is known in the current public Vega release.
	 */
	public boolean isInCurrentVega() {
		return inCurrentVega;
	}

	/**
	 * Set the flag to indicate that the Vega Id associated with this gene is known in the current
	 * public Vega release/web-site. This is typically only used by the DAO when mapping a database
	 * row.
	 * 
	 * @param inCurrentVega should be set to true if the Vega id associated with this gene is known in
	 *     the public Vega release.
	 */
	public void setInCurrentVega(boolean inCurrentVega) {
		this.inCurrentVega = inCurrentVega;
	}

	/**
	 * Return a flag that indicates that the Ensembl identifier associated with this gene is available
	 * in the current public release of Ensembl. This will typically be false for genes that have been
	 * silently deprecated from Ensembl.
	 * 
	 * @return true if the Ensembl identifier is known in the current public Ensembl release.
	 */
	public boolean isInCurrentEnsembl() {
		return inCurrentEnsembl;
	}

	/**
	 * Set the flag to indicate that the Ensembl Id associated with this gene is known in the current
	 * public Ensembl release/web-site. This is typically only used by the DAO when mapping a database
	 * row.
	 * 
	 * @param inCurrentEnsembl should be set to true if the Ensembl id associated with this gene is known in
	 *     the public Ensembl release.
	 */
	public void setInCurrentEnsembl(boolean inCurrentEnsembl) {
		this.inCurrentEnsembl = inCurrentEnsembl;
	}

	/**
	 * Get a string that indicates where the primaryName parameter was derived from. The current database
	 * contains, for example: mgi-2-ensembl, ottmus from vega build, mgi from vega build,  mgi from ensembl build
	 * mgi-2-vega, ensmus from ensembl build etc.
	 * 
	 * @return a String containing a representation of the source of the primary name data.
	 */
	public String getPrimaryNameSource() {
		return primaryNameSource;
	}

	/**
	 * Set the string that indicates the source of the primaryName of this Gene. This will typically
	 * only be called by a rowmapper in the DAO layer.
	 * 
	 * @param primaryNameSource is a String that can be used to identify the source of the primaryName
	 *     for this gene.
	 */
	public void setPrimaryNameSource(String primaryNameSource) {
		this.primaryNameSource = primaryNameSource;
	}

	public String getBuildGenePrimaryName() {
		return buildGenePrimaryName;
	}

	public void setBuildGenePrimaryName(String buildGenePrimaryName) {
		this.buildGenePrimaryName = buildGenePrimaryName;
	}

	public String getBuildName() {
		return buildName;
	}

	public void setBuildName(String buildName) {
		this.buildName = buildName;
	}

	public String getBuildVersion() {
		return buildVersion;
	}

	public void setBuildVersion(String buildVersion) {
		this.buildVersion = buildVersion;
	}

	public int getChromosomeEnd() {
		return chromosomeEnd;
	}

	public void setChromosomeEnd(int chromosomeEnd) {
		this.chromosomeEnd = chromosomeEnd;
	}

	public String getChromosomeName() {
		return chromosomeName;
	}

	public void setChromosomeName(String chromosomeName) {
		this.chromosomeName = chromosomeName;
	}

	public int getChromosomeStart() {
		return chromosomeStart;
	}

	public void setChromosomeStart(int chromosomeStart) {
		this.chromosomeStart = chromosomeStart;
	}

	public int getChromosomeStrand() {
		return chromosomeStrand;
	}

	public void setChromosomeStrand(int chromosomeStrand) {
		this.chromosomeStrand = chromosomeStrand;
	}

	public int getTranscriptCount() {
		return transcriptCount;
	}

	public void setTranscriptCount(int transcriptCount) {
		this.transcriptCount = transcriptCount;
	}

	public String getDisplayText() {
		return getPrimaryName();
	}

	public String getElementData() {
		return 
			LookupListUtil.format("geneId", getId()) +
			LookupListUtil.format("chromosome", getChromosomeName()) +
			LookupListUtil.format("start", getChromosomeStart()) +
			LookupListUtil.format("end", getChromosomeEnd()) +
			LookupListUtil.format("buildId", getBuildId())+
			LookupListUtil.format("buildGenePrimaryName", getBuildGenePrimaryName())+
			LookupListUtil.format("primaryName", getPrimaryName())
		;
	}

	public void setDisplayText(String displayText) {
		// TODO Auto-generated method stub
		
	}

	public void setElementData(String elementData) {
		// TODO Auto-generated method stub
		
	}

	public String getEnsemblIdString() {
		return ensemblIdString;
	}

	public void setEnsemblIdString(String ensemblIdString) {
		this.ensemblIdString = ensemblIdString;
	}

	public String getVegaIdString() {
		return vegaIdString;
	}

	public void setVegaIdString(String vegaIdString) {
		this.vegaIdString = vegaIdString;
	}
	
	public int getIgtcHits() {
		return igtcHits;
	}

	public void setIgtcHits(int igtcHits) {
		this.igtcHits = igtcHits;
	}

	public int getOmnibankHits() {
		return omnibankHits;
	}

	public void setOmnibankHits(int omnibankHits) {
		this.omnibankHits = omnibankHits;
	}

	public int getSummaryScore() {
		return summaryScore;
	}

	public void setSummaryScore(int summaryScore) {
		this.summaryScore = summaryScore;
	}

	public String getCuratedStatus() {
		return curatedStatus;
	}

	public void setCuratedStatus(String curatedStatus) {
		this.curatedStatus = curatedStatus;
	}

	public int getTotalExonCount() {
		return totalExonCount;
	}

	public void setTotalExonCount(int totalExonCount) {
		this.totalExonCount = totalExonCount;
	}

	public int getUniqueExonCount() {
		return uniqueExonCount;
	}

	public void setUniqueExonCount(int uniqueExonCount) {
		this.uniqueExonCount = uniqueExonCount;
	}
}